/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSCameraThumbnailView.h"
#import "FSCameraThumbnailCollectionViewCell.h"

@interface FSCameraThumbnailView()<UICollectionViewDelegate, UICollectionViewDataSource, UICollectionViewDelegateFlowLayout>
@property (nonatomic, strong) UICollectionView* collectionView;
@property (nonatomic, strong) UILabel* labelNoData;

@end

@implementation FSCameraThumbnailView

- (instancetype)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        _arrayImages = [[NSMutableArray alloc] init];
        self.backgroundColor = self.fs_backgroundColor_B5; // self.fs_textColor_T4;
        [self initSubView];
    }
    return self;
}

- (UILabel *)labelNoData
{
    if (!_labelNoData) {
        _labelNoData = [[UILabel alloc] init];
        [self addSubview:_labelNoData];
        [_labelNoData mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(self.fs_mas_left).offset(16);
            make.right.equalTo(self.fs_mas_right).offset(-16);
            make.centerY.mas_equalTo(self.labelNoData.superview);
        }];
        _labelNoData.font = self.fs_standardGeneralFont_GF3;
        _labelNoData.textColor = self.fs_textColor_T8;
        _labelNoData.textAlignment = NSTextAlignmentLeft;
        _labelNoData.numberOfLines = 0;
        _labelNoData.text = FSLocalizedForKey(@"kNoPhotos");
    }
    return _labelNoData;
}

- (UICollectionView *)collectionView
{
    if (!_collectionView) {
        CGFloat sectionInset = 3;
        UICollectionViewFlowLayout *layout = [[UICollectionViewFlowLayout alloc] init];
        layout.minimumLineSpacing = 2;
        layout.sectionInset = UIEdgeInsetsMake(0, sectionInset, 0, sectionInset);
        CGFloat itemW = DEVICE_iPHONE ? 78 : 178;
        layout.itemSize = CGSizeMake(itemW, self.frame.size.height);
        layout.scrollDirection = UICollectionViewScrollDirectionHorizontal;
        _collectionView = [[UICollectionView alloc] initWithFrame:self.bounds collectionViewLayout:layout];
        _collectionView.backgroundColor = self.fs_backgroundColor_B5; // self.fs_textColor_T4;
        _collectionView.dataSource = self;
        _collectionView.delegate = self;
        
        UILongPressGestureRecognizer *longPressGesture = [[UILongPressGestureRecognizer alloc] initWithTarget:self action:@selector(moveAction:)];
         _collectionView.userInteractionEnabled = YES;
         [_collectionView addGestureRecognizer:longPressGesture];
    }
    
    return _collectionView;
}

- (void)initSubView{
    [self addSubview:self.collectionView];
    [self.collectionView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.fs_mas_left);
        make.right.equalTo(self.fs_mas_right);
        make.top.bottom.mas_equalTo(0);
    }];
    [self.collectionView registerClass:[FSCameraThumbnailCollectionViewCell class] forCellWithReuseIdentifier:@"cameraThumbnailCell"];
    [self.labelNoData setHidden:YES];
}

- (void)setArrayImages:(NSMutableArray *)arrayImages
{
    _arrayImages = arrayImages;
    if (arrayImages.count == 0) {
        [self.labelNoData setHidden:NO];
        [self.collectionView setHidden:YES];
    }else {
        [self.labelNoData setHidden:YES];
        [self.collectionView setHidden:NO];
        [self.collectionView reloadData];
    }
}

#pragma mark -- collectionView moveAction
- (void)moveAction:(UILongPressGestureRecognizer *)longGes
{
    if (longGes.state == UIGestureRecognizerStateBegan) {
        _selectPath = [self.collectionView indexPathForItemAtPoint:[longGes locationInView:longGes.view]];
        [self.collectionView reloadData];
       /* FSCameraThumbnailCollectionViewCell *cell = (FSCameraThumbnailCollectionViewCell *)[self.collectionView cellForItemAtIndexPath:selectPath];
        [self.collectionView reloadData];
        cell.deleteBtn.hidden = NO;
        
        [cell.deleteBtn addTarget:self action:@selector(deleteItemAction:) forControlEvents:UIControlEventTouchUpInside];
        cell.deleteBtn.tag = selectPath.item;*/
    }
}

- (void)deleteItemAction:(UIButton *)btn {
    [self.arrayImages removeObjectAtIndex:btn.tag];
    _selectPath = nil;
    [self.collectionView reloadData];
    if ([self.delegate respondsToSelector:@selector(deleteItemOfCollectionView:)])
    {
        [self.delegate deleteItemOfCollectionView:btn.tag];
    }
}

#pragma mark --UICollectionView

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView
{
    return 1;
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section
{
    if (_arrayImages.count == 0) {
        [self.labelNoData setHidden:NO];
        [self.collectionView setHidden:YES];
    }
    return self.arrayImages.count;
}

- (__kindof UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath
{
    FSCameraThumbnailCollectionViewCell *cell = (FSCameraThumbnailCollectionViewCell *)[collectionView dequeueReusableCellWithReuseIdentifier:@"cameraThumbnailCell" forIndexPath:indexPath];
    cell.deleteBtn.hidden = YES;
    cell.imgView.image = self.arrayImages[indexPath.row];
    
    if (indexPath == _selectPath) {
        cell.deleteBtn.hidden = NO;
        [cell.deleteBtn addTarget:self action:@selector(deleteItemAction:) forControlEvents:UIControlEventTouchUpInside];
        cell.deleteBtn.tag = _selectPath.item;
    }
    
    return cell;
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath
{
    FSCameraThumbnailCollectionViewCell *cell = (FSCameraThumbnailCollectionViewCell *)[self.collectionView cellForItemAtIndexPath:indexPath];
    cell.deleteBtn.hidden = YES;
    if ([self.delegate respondsToSelector:@selector(didSelectItemOfCollectionView:)])
    {
        [self.delegate didSelectItemOfCollectionView:indexPath];
    }
}

- (void)resetSelect
{
    _selectPath = nil;
    [self.collectionView reloadData];
}
@end
