/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSScanSaveAsViewController.h"
#import "PDFScanManager+private.h"
#import "PDFScanMenuViewManager+Private.h"
#import "PasswordModule.h"
#import "QualityViewController.h"
#import "FileSelectDestinationViewController.h"
#import "FSFileSelectDestinationViewController.h"
//#import "AlertView.h"

@interface FSScanSaveAsViewController ()<UITableViewDelegate, UITableViewDataSource, IDocEventListener>

@property (nonatomic, strong) UITableView *tableView;
@property (nonatomic, weak) FSPDFViewCtrl *pdfViewCtrl;
@property (nonatomic, copy) NSString *pdfFilePath;
@property (nonatomic, copy) NSString *pdfFileName;
@property (nonatomic, copy) NSString *saveAsPdfFilePath;
@property (nonatomic, assign) BOOL canFlattenCopy;
@property (nonatomic, assign) BOOL canOptimizeDoc;
@property (nonatomic, assign) FSImageSettingsImageCompressQuality colorCompressQuality;
@property (nonatomic, assign) FSMonoImageSettingsMonoImageCompressQuality monoCompressQuality;
@property (nonatomic, assign) BOOL isCompress;
@property (nonatomic, assign) unsigned long long beforeCompressSize;
@property (nonatomic, assign) unsigned long long afterCompressSize;
@property (nonatomic, assign) BOOL isScanTrial;
@property (nonatomic, strong) NSMutableArray <PDFScanMenuItemGroup *> *itemGroups;
@end

@implementation FSScanSaveAsViewController
- (instancetype)init
{
    self = [super init];
    if (self) {
        
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];

    NSDictionary *attributesLeftRight = [NSDictionary dictionaryWithObjectsAndKeys:
                                        self.fs_primaryColor,
                                        NSForegroundColorAttributeName,
                                        self.fs_standardGeneralFont_GF2,
                                        NSFontAttributeName,
                                        nil];
    
    
    // Do any additional setup after loading the view.
    self.navigationController.navigationBar.barTintColor = self.fs_backgroundColor_B2;
    UIBarButtonItem *leftButton = [[UIBarButtonItem alloc] initWithTitle:FSLocalizedForKey(@"kCancel") style:UIBarButtonItemStylePlain target:self action:@selector(clickedCancel)];
    [leftButton setTitleTextAttributes:attributesLeftRight forState:UIControlStateNormal];

    [BindThemeKeySel(leftButton, setTintColor:) fs_updateThemeCompleted:^(UIBarButtonItem*  _Nonnull target, SEL  _Nonnull sel) {
        [target setTintColor:target.fs_primaryColor];
    }];
    
    self.navigationItem.leftBarButtonItem = leftButton;
    UIBarButtonItem *rightButton = [[UIBarButtonItem alloc] initWithTitle:FSLocalizedForKey(@"kSave") style:UIBarButtonItemStylePlain target:self action:@selector(clickedOK)];
    [rightButton setTitleTextAttributes:attributesLeftRight forState:UIControlStateNormal];
    self.navigationItem.rightBarButtonItem = rightButton;
    [BindThemeKeySel(rightButton, setTintColor:) fs_updateThemeCompleted:^(UIBarButtonItem*  _Nonnull target, SEL  _Nonnull sel) {
        [target setTintColor:target.fs_primaryColor ];
    }];
    
    NSDictionary *attributes=[NSDictionary dictionaryWithObjectsAndKeys:
                                        self.fs_textColor_T4,
                                        NSForegroundColorAttributeName,
                                        self.fs_standardGeneralFont_GF1,
                                        NSFontAttributeName,
                                        nil];
        
    [self.navigationController.navigationBar setTitleTextAttributes:attributes];
    self.title = FSLocalizedForKey(@"kSaveAs");

    self.tableView = [[UITableView alloc] initWithFrame:self.view.bounds style:UITableViewStyleGrouped];

    [self.view addSubview:self.tableView];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    [self.tableView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.top.right.bottom.mas_equalTo(0);
    }];
    self.tableView.tableFooterView = [[UIView alloc] init];
    self.tableView.separatorStyle = UITableViewCellSeparatorStyleSingleLine;
    
    self.view.backgroundColor = self.fs_backgroundColor_B9;
    self.tableView.backgroundColor = self.fs_backgroundColor_B9;
    self.tableView.separatorColor = DividingLineColor;
    
    _colorCompressQuality = FSImageSettingsImageCompressQualityMedium;
    _monoCompressQuality = FSMonoImageSettingsImageCompressQualityLossless;
    _isCompress = NO;
    _canOptimizeDoc = YES;
}

- (void)tableView:(UITableView *)tableView selectRowAtIndexPath:(NSIndexPath *)indexPath{
    
    [tableView selectRowAtIndexPath:indexPath animated:NO scrollPosition:0];
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

- (void)setTableViewItemGroups
{    
    self.itemGroups = [[[PDFScanManager shareManager].menuViewManager getMenuItemGroupsForMenuViewType:PDFScanMenuViewTypeSaveAs] mutableCopy];
    for (PDFScanMenuItemGroup* object in self.itemGroups) {
        if (object.items) {
            BOOL bIsItem = NO;
            for (PDFScanMenuItem* item in object.items) {
                if (item.actionTag == 0) {
                    item.title = self.pdfFileName;
                    bIsItem = YES;
                    break;
                }
            }
            if (bIsItem) {
                break;
            }
        }
    }
    
    [self.tableView reloadData];
}

- (void)setPdfFilePath:(NSString *)pdfFilePath{
    _pdfFilePath = pdfFilePath;
    if (pdfFilePath.length) {
        self.pdfFileName = [[pdfFilePath lastPathComponent] stringByDeletingPathExtension];
        self.saveAsPdfFilePath = [NSString stringWithFormat:@"%@/%@",DOCUMENT_PATH,self.pdfFileName];
    }
    
    [self setTableViewItemGroups];
}

- (void)clickedCancel {
    [self dismissViewControllerAnimated:YES completion:nil];
}
- (void)clickedOK {
    FSFileSelectDestinationViewController *selectDestination = [[FSFileSelectDestinationViewController alloc] init];
        selectDestination.isRootFileDirectory = YES;
        selectDestination.shouldBindTemeForStatusBar = NO;
        selectDestination.fileSelectDestinationType = FSFileSelectDestinationTypeSelectFolder;
        selectDestination.expectFileType = @[@"*"];
        [selectDestination loadFilesWithPath:DOCUMENT_PATH];
        
        selectDestination.operatingHandler = ^(FSFileSelectDestinationViewController *controller, NSArray *destinationFolder) {
            
            void (^saveAs)(NSString *) = ^(NSString *pdfFilePath){
                [controller dismissViewControllerAnimated:NO completion:nil];
                self.saveAsPdfFilePath = pdfFilePath;
                if (self.saveAsPdfFilePath.length) {
                        dispatch_semaphore_t wait = dispatch_semaphore_create(0);
                        void (^dismissViewAndOpenDoc)(void) = ^(){
                            dispatch_async(dispatch_get_main_queue(), ^{
                                [self dismissViewControllerAnimated:YES completion:^{
                                    !self.saveAsCallBack ? : self.saveAsCallBack(nil, pdfFilePath);
                                    dispatch_semaphore_signal(wait);
                                }];
                            });
                            dispatch_semaphore_wait(wait, DISPATCH_TIME_FOREVER);
                        };
                        {
                            void (^block) (void) = ^(){
                                if (self.pdfViewCtrl) {
                                    if ([[self.pdfViewCtrl currentDoc] getSignatureCount] > 0) {
                                        [self.pdfViewCtrl saveDoc:self.saveAsPdfFilePath flag:FSPDFDocSaveFlagIncremental];
                                    }else{
                                        [self.pdfViewCtrl saveDoc:self.saveAsPdfFilePath flag:FSPDFDocSaveFlagNormal];
                                    }                            }
                                dismissViewAndOpenDoc();
                            };

                           {
                                [self.view fs_showHUDLoading:FSLocalizedForKey(@"kSaving") process:^{
                                    block();
                                }];
                            }
                        }
                }
            };
            
            NSString *pdfFilePath = [[destinationFolder[0] stringByAppendingPathComponent:self.pdfFileName] stringByAppendingPathExtension:[self.pdfFilePath pathExtension]];
            NSFileManager *fileManager = [[NSFileManager alloc] init];
            if ([fileManager fileExistsAtPath:pdfFilePath]) {
                
                dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.3 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kFileAlreadyExists") preferredStyle:UIAlertControllerStyleAlert];
                    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel")
                                                                           style:UIAlertActionStyleCancel
                                                                         handler:^(UIAlertAction *action) {
                                                                             [self tableView:self.tableView didSelectRowAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0]];
                                                                             [controller.doneItem setEnabled:YES];
                                                                         }];
                    UIAlertAction *replaceAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                                            style:UIAlertActionStyleDefault
                                                                          handler:^(UIAlertAction *action) {
                                                                            [fileManager removeItemAtPath:pdfFilePath error:nil];
                                                                            saveAs(pdfFilePath);
                                                                          }];
                    [alertController addAction:cancelAction];
                    [alertController addAction:replaceAction];
                    [controller presentViewController:alertController animated:YES completion:nil];
                });
            } else {
                saveAs(pdfFilePath);
            }
        };
    FSNavigationController *selectDestinationNavController = [[FSNavigationController alloc] initWithRootViewController:selectDestination];
    if (DEVICE_iPHONE)
    {
        selectDestinationNavController.modalPresentationStyle = UIModalPresentationFullScreen;
    }
    else
    {
        selectDestinationNavController.modalPresentationStyle = UIModalPresentationFormSheet;
    }
    [self presentViewController:selectDestinationNavController
                                     animated:YES
                                   completion:nil];
}

- (unsigned long long)pdfFileSize{
    unsigned long long fileSize = 0;
    NSFileManager *manager = [NSFileManager defaultManager];
    if ([manager fileExistsAtPath:self.pdfViewCtrl.filePath]) {
        fileSize = [[manager attributesOfItemAtPath:self.pdfViewCtrl.filePath error:nil] fileSize];
    }
    return fileSize;
}

#pragma mak UITableViewDataSource UITableViewDelegate

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return self.itemGroups.count;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {

    return self.itemGroups[section].items.count;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    return self.itemGroups[section].title;
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath{
    static NSString *cellIdentifier = @"UITableViewCellID";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier];
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellIdentifier];
        cell.selectedBackgroundView = [UIView new];
        cell.selectedBackgroundView.backgroundColor = self.fs_dividerColor_P1;
        cell.contentView.backgroundColor = nil;
    }
    cell.textLabel.font = self.fs_standardGeneralFont_GF2;
    cell.textLabel.textColor = self.fs_textColor_T4;
    cell.textLabel.text = self.itemGroups[indexPath.section].items[indexPath.row].title;
    cell.imageView.image = self.itemGroups[indexPath.section].items[indexPath.row].image;
    cell.backgroundColor = self.fs_backgroundColor_B8;
    return cell;
}

- (BOOL)fileWithNameDoesExist:(NSString*) filename
{
    return [[PDFScanManager shareManager].documentManager documentByName:filename] != nil;
}

- (void)renameScaneFile:(NSString *)fileName
{
    NSString *msg = FSLocalizedForKey(@"kInputNewName");
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kRename") message:msg preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *renameaction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        NSString *newfileName = alertController.textFields.firstObject.text;
        NSString *newFile = [newfileName stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
        
        if (newFile == nil || newFile.length == 0)
        {
            FSAlertViewCtrlShow(@"kWarning", @"kInputNewFileName");
            return;
        }
        
        if ([newFile rangeOfCharacterFromSet:[NSCharacterSet characterSetWithCharactersInString:@"\\/:*?\\\\<>|"]].location != NSNotFound)
        {
            [UIAlertController fs_alert_OK_WithTitle:@"kWarning" message:@"kIllegalNameWarning" actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                [self renameScaneFile:newFile];
            } onCurrentSceneResponder:self];
            return;
        }
        self.pdfFileName = newFile;
        [self setTableViewItemGroups];
    }];
    renameaction.enabled = NO;
    UIAlertAction *cancel = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel") style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {
    }];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
        textField.text = [fileName stringByDeletingPathExtension];
        textField.clearButtonMode = UITextFieldViewModeWhileEditing;
        [textField setBlockForControlEvents:UIControlEventEditingChanged block:^(UITextField * _Nonnull sender) {
            renameaction.enabled = ![sender.text isEqualToString:self.pdfFileName];
            renameaction.enabled = ![sender.text isEqualToString:@""];
        }];
    }];
    [alertController addAction:cancel];
    [alertController addAction:renameaction];
    [self presentViewController:alertController animated:YES completion:nil];
}


- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    PDFScanMenuItem* item = self.itemGroups[indexPath.section].items[indexPath.row];
    if (item) {
        if (item.action) {
            item.action(item);
        }
        else
        {
            if (item.actionTag == 0) {
                [self renameScaneFile:self.pdfFileName];
            }
        }
    }
    
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section
{
    return 32;
}

- (CGFloat)tableView:(UITableView *)tableView heightForFooterInSection:(NSInteger)section
{
    return 0;
}

- (UIView *)tableView:(UITableView *)tableView viewForFooterInSection:(NSInteger)section {
    return [UIView new];
}

@end

