/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "PDFScanToolbarManager.h"
#import "FSMainToolbar+PDFScanPrivate.h"

@interface PDFScanToolbarManager()
@property (nonatomic, strong) NSMutableDictionary <NSString *, FSMainToolbar *>*toolbars;
@end

@implementation PDFScanToolbarManager

- (instancetype)init
{
    self = [super init];
    if (self) {
        _toolbars = @{}.mutableCopy;
        [self setDefaultToolbar];
    }
    return self;
}

- (void)setDefaultToolbar
{
    [self setDefaultToolbarWithTypeBottombar];
}

- (void)setDefaultToolbarWithTypeBottombar
{
    FSMainToolbar* bottomBar = [[FSMainToolbar alloc] initWithStyle:FSToolbarStyleBottom];
    [self setToolbar:bottomBar forToolbarType:FSPDFScanToolbarTypeBottombar];
    FSItem *fsAddPageItem = [FSItem createItemWithImage:ImageNamedAndNormalColor(@"scan_edit_add") imageSelected:ImageNamedAndNormalColor(@"scan_edit_add") imageDisable:ImageNamedAndNormalColor(@"scan_edit_add")];
    FSMainToolbarItem* addPageItem = [[FSMainToolbarItem alloc] initWithCustomView:fsAddPageItem];
    addPageItem.itemType = FSMainToolbarScanItemTypeAddFile;
    addPageItem.spacing = 50;
    FSItem *fsSaveItem = [FSItem createItemWithImage:ImageNamedAndNormalColor(@"scan_edit_save_as") imageSelected:ImageNamedAndNormalColor(@"scan_edit_save_as") imageDisable:ImageNamedAndDisabledColor(@"scan_edit_save_as")];
    FSMainToolbarItem* saveItem = [[FSMainToolbarItem alloc] initWithCustomView:fsSaveItem];
    saveItem.itemType = FSMainToolbarScanItemTypeSaveFile;
    saveItem.spacing = 50;
    [bottomBar addItem:addPageItem atPosition:FSMainToolbarItemPositionCenter];
    [bottomBar addItem:saveItem atPosition:FSMainToolbarItemPositionCenter];
}

- (void)setToolbar:(FSMainToolbar*)toolbar forToolbarType:(FSPDFScanToolbarType)toolbarType
{
    NSAssert(toolbar, @"The toolbar is not nil");
    NSAssert(self.toolbars[@(toolbarType).stringValue] == nil, @"The addedPDFScanToolbar has been added and is not allowed to be added repeatedly");
    NSAssert(toolbarType >= FSPDFScanToolbarTypeBottombar && toolbarType <= FSPDFScanToolbarTypeBottombar, @"The pdfScanToolbarType is not supported");

    [self.toolbars setValue:toolbar forKey:@(toolbarType).stringValue];
}

- (FSMainToolbar*)getToolbar:(FSPDFScanToolbarType)toolbarType
{
    return self.toolbars[@(toolbarType).stringValue];
}

@end
