//
//  LTCanmeraViewControllerNew.m
//  FoxitApp
//
//  Created by Apple on 16/11/30.
//
//

#import "LTCanmeraViewControllerNew.h"
#import "LTDetectionLayer.h"
#import "LTStatusLabelController.h"
#import "LTDocumentEditViewControllerNew.h"
#import "LTPageFormatHelper.h"
#import "LTDefaultFilenameProvider.h"
#import "LTPageFormat.h"
#import "FXUIPickViewController.h"
#import "PDFScanManager+private.h"
#import "Preference.h"
#import "FSCameraThumbnailView.h"

@interface LTCanmeraViewControllerNew ()<LTImageProcessViewControllerDelegate,IFXUIPickViewControllerDelegate,LTDocumentSessionDelegate,LTImageProcessorDelegate,FSCameraThumbnailViewDalegate>{
    BOOL isWaiting;
}

@property (nonatomic, strong) LTCamera* camera;
@property (nonatomic, strong) LTCaptureService* captureService;
@property (nonatomic, strong) LTPageFormat* pageFormat;
@property (nonatomic, strong) FSCameraThumbnailView* cameraThumbnailView;
@property (nonatomic, strong) UIView* cameraThumbnailContentView;
@property (nonatomic, strong) UIImageView *animationImageView;

@property (nonatomic, strong) FSToolbar *bottomBar;
@property (nonatomic, strong) FSToolbar *topBar;
@property (nonatomic, strong) UIButton *albumBtn;
@property (nonatomic, strong) UIButton *shootBtn;
@property (nonatomic, strong) UIButton *completeBtn;
@property (nonatomic, strong) UIButton *cancelBtn;
@property (nonatomic, strong) UIButton *flashlightBtn;
@property (nonatomic, strong) UIButton *singleFrametBtn;
@property (nonatomic, strong) UIButton *detectionBtn;
@property (nonatomic, strong) UIButton *formatSizeBtn;
@property (nonatomic, strong) UIButton *dragBtn;
@property (nonatomic, strong) UIView *videoPreviewView;
@property (nonatomic, strong) UIView *maskView;
@property (nonatomic, strong) UILabel *countLabel;
@property (nonatomic, strong) LTImageProcessor *imageProcess;
@property (nonatomic, assign) BOOL cameraTriggered;
@property (nonatomic, strong) LTStatusLabelController* statusLabelController;
@property (nonatomic, strong) UILabel *detectionLabel;
@property (nonatomic) int pageInProcessor;
@property (nonatomic, strong) NSMutableArray *photos;
@property (nonatomic) int imageNilCounts;
@end

@implementation LTCanmeraViewControllerNew

- (instancetype)init
{
    if (self = [super init]) {
        
    }
    return self;
}

- (UIInterfaceOrientationMask)supportedInterfaceOrientations {
    return  UIInterfaceOrientationMaskPortrait;
}

- (UIInterfaceOrientation)preferredInterfaceOrientationForPresentation{
    return UIInterfaceOrientationPortrait;
}

- (BOOL)shouldAutorotate{
    return NO;
}

- (void)dealloc{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    [self initSubViews];
    
    _photos = [NSMutableArray array];
    _camera = [[LTCamera alloc] initWithDevicePosition:AVCaptureDevicePositionBack];
    _camera.lightEnabled = NO;
    
    isWaiting = NO;
    if (_camera)
    {
        _captureService = [[LTCaptureService alloc] initWithCamera:_camera];
        _captureService.delegate = self;
    }
    _statusLabelController = [[LTStatusLabelController alloc] initWithLabel:_detectionLabel];
    _pageFormat = [LTPageFormatHelper getStoredFormat];
    
    [self setupPreviewLayer];
    [self setupDetectionLayer];
    
    if (DEVICE_iPHONE) {
        [self orientChange:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(orientChange:)  name:UIDeviceOrientationDidChangeNotification object:nil];
    }
}

- (UIStatusBarStyle)preferredStatusBarStyle {
    return UIStatusBarStyleLightContent;
}

//- (BOOL)prefersStatusBarHidden
//{
//    return YES;
//}

- (void)initSubViews
{
    FSToolbar *bottomBar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleBottom];
    _bottomBar = bottomBar;
    bottomBar.hasDivide = NO;
    [self.view addSubview:bottomBar];
    bottomBar.backgroundColor = [UIColor colorWithHexString:@"#000000FF"];
    [bottomBar mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.leading.trailing.bottom.equalTo(bottomBar.superview);
    }];
    bottomBar.contentBarHeight = 58;

    self.cameraThumbnailView.delegate = self;
    [self.view addSubview:self.cameraThumbnailContentView];
    [self.cameraThumbnailContentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.mas_equalTo(0);
        make.bottom.equalTo(bottomBar.mas_top);
        make.height.mas_equalTo(self.cameraThumbnailContentView.frame.size.height);
    }];
    
    self.shootBtn = [[UIButton alloc] init];
    self.shootBtn.accessibilityLabel = FSLocalizedForKey(@"kScan");
    [self.shootBtn setBackgroundImage:ImageNamed(@"scan_take_photo") forState:UIControlStateNormal];
    [bottomBar addSubview:self.shootBtn];
    [self.shootBtn addTarget:self action:@selector(shootTap:) forControlEvents:UIControlEventTouchUpInside];
    [self.shootBtn mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.centerX.mas_equalTo(self.shootBtn.superview.mas_centerX);
        make.bottom.mas_equalTo(self.shootBtn.superview.mas_bottom);
        make.width.mas_equalTo(58);
        make.height.mas_equalTo(58);
    }];
    
    self.cancelBtn = [[UIButton alloc] init];
    self.cancelBtn.accessibilityIdentifier = @"scan_cancel_btn";
    [self.cancelBtn setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
    [self.cancelBtn setTitleColor:[[UIColor whiteColor] colorWithAlphaComponent:LABEL_HIGHLIGHTED_ALPHA] forState:UIControlStateHighlighted];
    [self.cancelBtn setTitle:FSLocalizedForKey(@"kCancel") forState:UIControlStateNormal];
    self.cancelBtn.titleLabel.textAlignment = NSTextAlignmentCenter;
    self.cancelBtn.titleLabel.font = self.fs_standardGeneralFont_GF2;
    [bottomBar addSubview:self.cancelBtn];
    [self.cancelBtn addTarget:self action:@selector(closeCanmeraView) forControlEvents:UIControlEventTouchUpInside];
    CGSize cancelBtnSize = [Utility getTextSize:FSLocalizedForKey(@"kCancel") fontSize:self.cancelBtn.titleLabel.font.pointSize maxSize:CGSizeMake(150, 36)];
    
    [self.cancelBtn mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(16);
        make.centerY.mas_equalTo(self.shootBtn.mas_centerY);
        make.width.mas_equalTo(cancelBtnSize.width);
        make.height.mas_equalTo(36);
    }];
    
    
    self.completeBtn = [[UIButton alloc] init];
    if (self.replaceCallBack) self.completeBtn.hidden = YES;
    self.completeBtn.accessibilityIdentifier = @"scan_done_btn";
    self.completeBtn.enabled = NO;
    [self.completeBtn setTitleColor:[UIColor grayColor] forState:UIControlStateDisabled];
    [self.completeBtn setTitleColor:[UIColor whiteColor] forState:UIControlStateNormal];
    [self.completeBtn setTitleColor:[[UIColor whiteColor] colorWithAlphaComponent:LABEL_HIGHLIGHTED_ALPHA] forState:UIControlStateHighlighted];
    [self.completeBtn setTitle:FSLocalizedForKey(@"kDone") forState:UIControlStateNormal];
    self.completeBtn.titleLabel.textAlignment = NSTextAlignmentCenter;
    self.completeBtn.titleLabel.font = self.fs_standardGeneralFont_GF2;
    [bottomBar addSubview:self.completeBtn];
    [self.completeBtn addTarget:self action:@selector(capturedImagecountBtnTap) forControlEvents:UIControlEventTouchUpInside];
    CGSize size = [Utility getTextSize:FSLocalizedForKey(@"kDone") fontSize:self.completeBtn.titleLabel.font.pointSize maxSize:CGSizeMake(150, 36)];
    
    [self.completeBtn mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(-16);
        make.centerY.mas_equalTo(self.shootBtn.mas_centerY);
        make.width.mas_equalTo(size.width);
        make.height.mas_equalTo(36);
    }];
    
    CGFloat btnWidth = 40;
    CGFloat sidesLength = DEVICE_iPHONE ? 24 : 78;
    
    FSToolbar *topToolBar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleTop];
    _topBar = topToolBar;
    topToolBar.hasDivide = NO;
    topToolBar.backgroundColor = [UIColor colorWithHexString:@"#000000FF"];
    [self.view addSubview:topToolBar];
    [topToolBar mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(topToolBar.superview.mas_left);
        make.right.mas_equalTo(topToolBar.superview.mas_right);
        make.top.mas_equalTo(topToolBar.superview.mas_top);
    }];
    
    UIView *videoPreview = [[UIView alloc] init];
    videoPreview.backgroundColor = [UIColor blackColor];
    self.videoPreviewView = videoPreview;
    [self.view addSubview:videoPreview];
    [videoPreview mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(videoPreview.superview.mas_left);
        make.right.mas_equalTo(videoPreview.superview.mas_right);
        make.top.mas_equalTo(topToolBar.mas_bottom);
        make.bottom.mas_equalTo(bottomBar.mas_top);
    }];
    
    
    UIImageView *leftTop = [[UIImageView alloc] initWithImage:ImageNamed(@"scan_left_top")];
    [videoPreview addSubview:leftTop];
    [leftTop mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.top.mas_equalTo(videoPreview);
    }];
    UIImageView *leftBottom = [[UIImageView alloc] initWithImage:ImageNamed(@"scan_left_bottom")];
    [videoPreview addSubview:leftBottom];
    [leftBottom mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.bottom.mas_equalTo(videoPreview);
    }];
    UIImageView *rightTop = [[UIImageView alloc] initWithImage:ImageNamed(@"scan_right_top")];
    [videoPreview addSubview:rightTop];
    [rightTop mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.right.top.mas_equalTo(videoPreview);
    }];
    UIImageView *rightBottom = [[UIImageView alloc] initWithImage:ImageNamed(@"scan_right_bottom")];
    [videoPreview addSubview:rightBottom];
    [rightBottom mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.right.bottom.mas_equalTo(videoPreview);
    }];
    
    self.albumBtn = [[UIButton alloc] init];
    self.albumBtn.accessibilityLabel = FSLocalizedForKey(@"kAddPhotos");
//    [self.albumBtn setBackgroundImage:ImageNamed(@"scan_pictures_btn_bg") forState:UIControlStateNormal];
    [self.albumBtn setImage:ImageNamed(@"scan_pictures_btn_icon") forState:UIControlStateNormal];
//    self.albumBtn.contentHorizontalAlignment = UIControlContentHorizontalAlignmentLeft;
//    self.albumBtn.titleLabel.lineBreakMode = NSLineBreakByTruncatingTail;
//    [self.albumBtn setTitle: FSLocalizedForKey(@"kPhotos") forState:UIControlStateNormal];
//    self.albumBtn.contentEdgeInsets = UIEdgeInsetsMake(0,0, 0, 5);
//    self.albumBtn.titleLabel.font = self.fs_standardWeakFont_WF1;
    
    [topToolBar addSubview:self.albumBtn];
    [self.albumBtn addTarget:self action:@selector(openAlbumLibraryTap) forControlEvents:UIControlEventTouchUpInside];
    [self.albumBtn mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.center.equalTo(self.albumBtn.superview);
        make.width.height.mas_equalTo(btnWidth);
    }];
    
    self.singleFrametBtn = [[UIButton alloc] init];
    self.singleFrametBtn.selected = [Preference getIntValue:@"PhotoToPDF" type:@"singleFrame" defaultValue:1];
    if (self.singleFrametBtn.selected) {
        self.singleFrametBtn.accessibilityLabel = FSLocalizedForKey(@"kContinueScanVoiceover");
    }else{
        self.singleFrametBtn.accessibilityLabel = FSLocalizedForKey(@"kSingleScanVoiceover");
    }
    [self.singleFrametBtn setImage:ImageNamed(@"scan_continuous_shooting_close") forState:UIControlStateNormal];
    [self.singleFrametBtn setImage:ImageNamed(@"scan_continuous_shooting_open") forState:UIControlStateSelected];
    [topToolBar addSubview:self.singleFrametBtn];
    [self.singleFrametBtn addTarget:self action:@selector(singleFrametBtnTap:) forControlEvents:UIControlEventTouchUpInside];
    [self.singleFrametBtn mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(self.albumBtn.mas_left).offset(-sidesLength);
        make.width.height.mas_equalTo(btnWidth);
        make.centerY.mas_equalTo(self.singleFrametBtn.superview.mas_centerY);
    }];
    self.singleFrametBtn.accessibilityTraits = UIAccessibilityTraitButton;
    
    self.flashlightBtn = [[UIButton alloc] init];
    self.flashlightBtn.accessibilityLabel = FSLocalizedForKey(@"kFlashOffVoiceover");
    [self.flashlightBtn setImage:ImageNamed(@"scan_flash_close") forState:UIControlStateNormal];
    [self.flashlightBtn setImage:ImageNamed(@"scan_flash_open")  forState:UIControlStateSelected];
    [topToolBar addSubview:self.flashlightBtn];
    [self.flashlightBtn addTarget:self action:@selector(flashlightBtnTap:) forControlEvents:UIControlEventTouchUpInside];
    [self.flashlightBtn mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(self.singleFrametBtn.mas_left).offset(-sidesLength);
        make.width.height.mas_equalTo(btnWidth);
        make.centerY.mas_equalTo(self.flashlightBtn.superview.mas_centerY);
    }];
    self.flashlightBtn.accessibilityTraits = UIAccessibilityTraitButton;
    
    self.detectionBtn = [[UIButton alloc] init];
    self.detectionBtn.selected = [Preference getIntValue:@"PhotoToPDF" type:@"detectAuto" defaultValue:0];
    if (self.detectionBtn) {
        self.detectionBtn.accessibilityLabel = FSLocalizedForKey(@"kOpenDetectVoiceover");
    }else{
        self.detectionBtn.accessibilityLabel = FSLocalizedForKey(@"kCloseDetectVoiceover");
    }
    [self.detectionBtn setImage:ImageNamed(@"scan_automatic_detection_close") forState:UIControlStateNormal];
    [self.detectionBtn setImage:ImageNamed(@"scan_automatic_detection_open") forState:UIControlStateSelected];
    [topToolBar addSubview:self.detectionBtn];
    [self.detectionBtn addTarget:self action:@selector(detectionBtnTap:) forControlEvents:UIControlEventTouchUpInside];
    [self.detectionBtn mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.albumBtn.mas_right).offset(sidesLength);
        make.width.height.mas_equalTo(btnWidth);
        make.centerY.mas_equalTo(self.detectionBtn.superview.mas_centerY);
    }];
    self.detectionBtn.accessibilityTraits = UIAccessibilityTraitButton;
    
    self.formatSizeBtn = [[UIButton alloc] init];
    self.formatSizeBtn.accessibilityLabel = FSLocalizedForKey(@"kScaneFrameSizeTip");
    [self.formatSizeBtn setImage:ImageNamed(@"scan_iamge_page_size") forState:UIControlStateNormal];
    [topToolBar addSubview:self.formatSizeBtn];
    [self.formatSizeBtn addTarget:self action:@selector(formatSizeBtnTap:) forControlEvents:UIControlEventTouchUpInside];
    [self.formatSizeBtn mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.detectionBtn.mas_right).offset(sidesLength);
        make.width.height.mas_equalTo(btnWidth);
        make.centerY.mas_equalTo(self.formatSizeBtn.superview.mas_centerY);
    }];
    self.formatSizeBtn.enabled = self.detectionBtn.selected;
    
    UILabel *detectionLabel = [[UILabel alloc] init];
    detectionLabel.hidden = YES;
    _detectionLabel = detectionLabel;
    [self.view addSubview:detectionLabel];
    [detectionLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.centerX.mas_equalTo(detectionLabel.superview.mas_centerX);
        make.centerY.mas_equalTo(detectionLabel.superview.mas_centerY);
    }];
    
    UIView *maskView = [[UIView alloc] init];
    _maskView = maskView;
    maskView.backgroundColor = self.fs_dividerColor_P3;
    maskView.hidden = YES;
    [self.view addSubview:maskView];
    [maskView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.edges.mas_equalTo(self.view);
    }];
    
    UIButton *closeButton = [UIButton buttonWithType:UIButtonTypeCustom];
    [closeButton addTarget:self action:@selector(closeCanmeraView) forControlEvents:UIControlEventTouchUpInside];
    [closeButton setImage:ImageNamed(@"scan_split_close") forState:UIControlStateNormal];
    [maskView addSubview:closeButton];
    [closeButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.size.mas_equalTo(CGSizeMake(40, 40));
        make.top.mas_equalTo(10);
        make.left.mas_equalTo(10);
    }];
    
    
    UILabel *tips = [[UILabel alloc] init];
    tips.backgroundColor = [UIColor clearColor];
    tips.font = self.fs_standardImportantFont_IF3;
    tips.numberOfLines = 0;
    tips.textAlignment = NSTextAlignmentCenter;
    tips.text = FSLocalizedForKey(@"kNeedFullScreen");
    tips.textColor = [UIColor whiteColor];
    [maskView addSubview:tips];
    [tips mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.mas_equalTo(tips.superview.mas_centerY).offset(-104);
        make.left.mas_equalTo(tips.superview.mas_left).offset(50);
        make.right.mas_equalTo(tips.superview.mas_right).offset(-50);
    }];
    
    UIView *tipsBgView = [UIView new];
    tipsBgView.backgroundColor = [UIColor colorWithWhite:0 alpha:0.6];
    [maskView insertSubview:tipsBgView belowSubview:tips];
    [tipsBgView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(tips).offset(-15);
        make.bottom.equalTo(tips).offset(15);
        make.left.equalTo(tips).offset(-20);
        make.right.equalTo(tips).offset(20);
    }];
    tipsBgView.layer.cornerRadius = 10;
    tipsBgView.clipsToBounds = YES;
    
    [self.view bringSubviewToFront:self.cameraThumbnailContentView];
    [self.view bringSubviewToFront:bottomBar];
    [self.view bringSubviewToFront:topToolBar];
    
    for (id key in [PDFScanManager shareManager].topButtonsHidden) {
        FSScanCameraControllerItemType type = (FSScanCameraControllerItemType)((NSString*)key).intValue;
        NSNumber* hidden = (NSNumber*)[[PDFScanManager shareManager].topButtonsHidden valueForKey:key];
        switch (type) {
            case FSScanCameraControllerItemTypeFlashLight:
                [_flashlightBtn setHidden:hidden.boolValue];
                break;
            case FSScanCameraControllerItemTypeSingleShooting:
                [_singleFrametBtn setHidden:hidden.boolValue];
                break;
            case FSScanCameraControllerItemTypePhotos:
                [_albumBtn setHidden:hidden.boolValue];
                break;
            case FSScanCameraControllerItemTypeAutoDection:
                [_detectionBtn setHidden:hidden.boolValue];
                break;
            case FSScanCameraControllerItemTypeFormatPageSize:
                [_formatSizeBtn setHidden:hidden.boolValue];
                break;
            default:
                break;
        }
    }
    [self updateThumbnailViewConstains];
}

- (void)dragBtnClick:(UIButton *)btn
{
    btn.selected = !btn.selected;
    [self.cameraThumbnailView resetSelect];
    [self updateThumbnailViewConstains];
}

- (void)updateThumbnailViewConstains
{
    CGFloat height = self.cameraThumbnailContentView.frame.size.height;
    if (self.dragBtn.selected){
        [self.cameraThumbnailContentView mas_updateConstraints:^(MASConstraintMaker *make) {
            make.bottom.mas_equalTo(self.bottomBar.mas_top);
        }];
    }else{
        [self.cameraThumbnailContentView mas_updateConstraints:^(MASConstraintMaker *make) {
            make.bottom.mas_equalTo(self.bottomBar.mas_top).offset(height - 27);
        }];
    }
}

-(FSCameraThumbnailView *)cameraThumbnailView
{
    if (!_cameraThumbnailView) {
        UIView *contentView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, SCREEN_WIDTH, DEVICE_iPHONE ? 127 : 157)];
        contentView.backgroundColor = [UIColor clearColor];
        _cameraThumbnailContentView = contentView;
        _cameraThumbnailView = [[FSCameraThumbnailView alloc] initWithFrame:CGRectMake(0, 0, SCREEN_WIDTH, DEVICE_iPHONE ? 100 : 130)];
        _cameraThumbnailView.backgroundColor = [[UIColor blackColor] colorWithAlphaComponent:0.6];
        UIButton *dragBtn = [[UIButton alloc] init];
        _dragBtn = dragBtn;
        [dragBtn addTarget:self action:@selector(dragBtnClick:) forControlEvents:UIControlEventTouchUpInside];
        [dragBtn setImage:ImageNamed(@"scan_thumb_up") forState:UIControlStateNormal];
        [dragBtn setImage:ImageNamed(@"scan_thumb_down") forState:UIControlStateSelected];
        [contentView addSubview:dragBtn];
        [dragBtn mas_makeConstraints:^(MASConstraintMaker *make) {
            make.centerX.mas_equalTo(contentView.mas_centerX);
            make.top.mas_equalTo(contentView.mas_top);
        }];
        
        _countLabel = [[UILabel alloc] init];
        _countLabel.textAlignment = NSTextAlignmentCenter;
        _countLabel.backgroundColor = [UIColor redColor];
        _countLabel.layer.cornerRadius = 10;
        _countLabel.layer.masksToBounds = YES;
        _countLabel.font = self.fs_standardWeakFont_WF2;
        _countLabel.adjustsFontSizeToFitWidth = YES;
        _countLabel.textColor = [UIColor whiteColor];
        [dragBtn addSubview:self.countLabel];
        [_countLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.right.mas_equalTo(dragBtn.mas_right).mas_offset(7);
            make.top.mas_equalTo(dragBtn.mas_top).mas_offset(-7);
            make.width.mas_equalTo(20);
            make.height.mas_equalTo(20);
        }];
        
        [contentView addSubview:_cameraThumbnailView];
        [_cameraThumbnailView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.right.mas_equalTo(0);
            make.top.mas_equalTo(dragBtn.mas_bottom);
            make.height.mas_equalTo(_cameraThumbnailView.frame.size.height);
        }];
    }
    return _cameraThumbnailView;
}

- (UIImageView *)animationImageView
{
    if (!_animationImageView){
        _animationImageView = [[UIImageView alloc] initWithFrame:self.videoPreviewView.frame];
        _animationImageView.contentMode = UIViewContentModeScaleAspectFit;
    }
    return _animationImageView;
}

- (void)animationWithImg:(UIImage *)img
{
    if (self.replaceCallBack) return;
    [self.animationImageView setImage:img];
    [self.view insertSubview:self.animationImageView belowSubview:self.cameraThumbnailContentView];
    self.animationImageView.frame = self.videoPreviewView.frame;
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(2.0 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        CGPoint point1 = self.animationImageView.center;
        CGPoint point2 = [self.view convertPoint:self.dragBtn.center fromView:self.cameraThumbnailContentView];
        
        CAAnimationGroup *animationGroup = [CAAnimationGroup animation];
        animationGroup.duration = 0.8;
        
        CAKeyframeAnimation *animation1 = [CAKeyframeAnimation animationWithKeyPath:@"transform.scale"];
        animation1.values = @[[NSNumber numberWithFloat:1.0],[NSNumber numberWithFloat:0.2]];
        animation1.beginTime = 0.f;
        
        CAKeyframeAnimation *animation2 = [CAKeyframeAnimation animationWithKeyPath:@"position"];
        UIBezierPath *bezierPath = [UIBezierPath bezierPath];
        [bezierPath moveToPoint:point1];
        [bezierPath addLineToPoint:point2];
        animation2.path = bezierPath.CGPath;
        animation2.beginTime = 0.f;
        
        CABasicAnimation *animation3 = [CABasicAnimation animationWithKeyPath:@"opacity"];
        animation3.fromValue = [NSNumber numberWithDouble:1.0];
        animation3.toValue = [NSNumber numberWithDouble:0.4];
        animation3.beginTime = 0.f;
        
        animationGroup.animations = @[animation1, animation2,animation3];
        [self.animationImageView.layer addAnimation:animationGroup forKey:nil];
        
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.75 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
            [self.animationImageView removeFromSuperview];
        });
    });
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:YES];
 
    [_detectionLayer updateResultDisplay:nil];
#if TARGET_IPHONE_SIMULATOR
    NSLog(@"run on simulator can not call canmera.");
#else
    [self startCameraSession];
#endif
    _cameraTriggered = NO;
    [_statusLabelController updateMessage:nil];
    [self configureAndSetLayoutOfImageCount];
    [self setCameraThumbnailViewImages];
    
    if (DEVICE_iPAD) {
        NSLog(@"%@,%@",NSStringFromCGSize(self.fs_currentSceneKeyWindow.fs_size),NSStringFromCGSize([UIScreen mainScreen].bounds.size));
        if (self.fs_currentSceneKeyWindow.fs_width != SCREEN_WIDTH) {
            _maskView.hidden = NO;
            [self.view bringSubviewToFront:_maskView];
        }else{
            _maskView.hidden = YES;
        }
    }
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(appHasGoneInBackground:) name:UIApplicationDidEnterBackgroundNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(appHasGoneInForeground:) name:UIApplicationWillEnterForegroundNotification object:nil];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:YES];
    
    [_detectionLayer updateResultDisplay:nil];
    [_statusLabelController updateMessage:nil];
    [self stopCameraSession];
    
    // Remove app goes to foreground/background delegates
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIApplicationDidEnterBackgroundNotification object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIApplicationWillEnterForegroundNotification object:nil];
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
    if (self.documentSession) {
        self.documentSession.delegate = self;
    }
}

- (void)viewWillTransitionToSize:(CGSize)size withTransitionCoordinator:(id<UIViewControllerTransitionCoordinator>)coordinator
{
    [super viewWillTransitionToSize:size withTransitionCoordinator:coordinator];
    [coordinator animateAlongsideTransition:^(id<UIViewControllerTransitionCoordinatorContext>  _Nonnull context) {
        for (UIView *subview in self.view.subviews) {
            if (subview.tag == 2118) {
                [subview removeFromSuperview];
            }
        }
    } completion:^(id<UIViewControllerTransitionCoordinatorContext>  _Nonnull context) {
        
        if (DEVICE_iPAD) {
            if (size.width != SCREEN_WIDTH) {
                self.maskView.hidden = NO;
                [self.view bringSubviewToFront:self.maskView];
            }else{
                self.maskView.hidden = YES;
            }
        }
    }];
    
}

#define DEGREES_TO_RADIANS(x) (M_PI * x / 180.0)
- (void)orientChange:(NSNotification *)noti {
    UIDeviceOrientation interfaceOrientation = [UIDevice currentDevice].orientation;
    
    if (interfaceOrientation == UIDeviceOrientationPortrait) {
        [UIView animateWithDuration:0.3 animations:^{
            self.shootBtn.transform = CGAffineTransformRotate(CGAffineTransformIdentity, 0);
            self.flashlightBtn.transform = CGAffineTransformRotate(CGAffineTransformIdentity, 0);
            self.singleFrametBtn.transform = CGAffineTransformRotate(CGAffineTransformIdentity, 0);
            self.detectionBtn.transform = CGAffineTransformRotate(CGAffineTransformIdentity, 0);
            self.formatSizeBtn.transform = CGAffineTransformRotate(CGAffineTransformIdentity, 0);
        }];
    }
    else if (interfaceOrientation == UIDeviceOrientationPortraitUpsideDown) {
        [UIView animateWithDuration:0.3 animations:^{
            self.shootBtn.transform = CGAffineTransformRotate(CGAffineTransformIdentity, DEGREES_TO_RADIANS(180));
            self.flashlightBtn.transform = CGAffineTransformRotate(CGAffineTransformIdentity, DEGREES_TO_RADIANS(180));
            self.singleFrametBtn.transform = CGAffineTransformRotate(CGAffineTransformIdentity, DEGREES_TO_RADIANS(180));
            self.detectionBtn.transform = CGAffineTransformRotate(CGAffineTransformIdentity, DEGREES_TO_RADIANS(180));
            self.formatSizeBtn.transform = CGAffineTransformRotate(CGAffineTransformIdentity, DEGREES_TO_RADIANS(180));
        }];
    }
    else if (interfaceOrientation == UIDeviceOrientationLandscapeLeft) {
        [UIView animateWithDuration:0.3 animations:^{
            self.shootBtn.transform = CGAffineTransformRotate(CGAffineTransformIdentity, DEGREES_TO_RADIANS(90));
            self.flashlightBtn.transform = CGAffineTransformRotate(CGAffineTransformIdentity, DEGREES_TO_RADIANS(90));
            self.singleFrametBtn.transform = CGAffineTransformRotate(CGAffineTransformIdentity, DEGREES_TO_RADIANS(90));
            self.detectionBtn.transform = CGAffineTransformRotate(CGAffineTransformIdentity, DEGREES_TO_RADIANS(90));
            self.formatSizeBtn.transform = CGAffineTransformRotate(CGAffineTransformIdentity, DEGREES_TO_RADIANS(90));
        }];
    }
    else if (interfaceOrientation == UIDeviceOrientationLandscapeRight) {
        [UIView animateWithDuration:0.3 animations:^{
            self.shootBtn.transform = CGAffineTransformRotate(CGAffineTransformIdentity, DEGREES_TO_RADIANS(-90));
            self.flashlightBtn.transform = CGAffineTransformRotate(CGAffineTransformIdentity, DEGREES_TO_RADIANS(-90));
            self.singleFrametBtn.transform = CGAffineTransformRotate(CGAffineTransformIdentity, DEGREES_TO_RADIANS(-90));
            self.detectionBtn.transform = CGAffineTransformRotate(CGAffineTransformIdentity, DEGREES_TO_RADIANS(-90));
            self.formatSizeBtn.transform = CGAffineTransformRotate(CGAffineTransformIdentity, DEGREES_TO_RADIANS(-90));
        }];
    }
}

+ (int)fromUIInterfaceOrientation:(UIInterfaceOrientation) orientation
{
    switch (orientation)
    {
        case UIInterfaceOrientationPortrait:
            return AVCaptureVideoOrientationPortrait;
            
        case UIInterfaceOrientationPortraitUpsideDown:
            return AVCaptureVideoOrientationPortraitUpsideDown;
            
        case UIInterfaceOrientationLandscapeLeft:
            return AVCaptureVideoOrientationLandscapeLeft;
            
        case UIInterfaceOrientationLandscapeRight:
            return AVCaptureVideoOrientationLandscapeRight;
            
        default:
            return 0;
    }
}

- (void) updateVideoOrientation
{
    UIInterfaceOrientation interfaceOrientation = [[UIApplication sharedApplication] statusBarOrientation];
    
    int videoOrientation = [LTCanmeraViewControllerNew fromUIInterfaceOrientation:interfaceOrientation];
    if ( videoOrientation != 0 ) {
        self.previewLayer.connection.videoOrientation = (AVCaptureVideoOrientation)videoOrientation;
    }
}
- (void) viewDidLayoutSubviews
{
    [super viewDidLayoutSubviews];
    [self updateVideoOrientation];
    [self.previewLayer setFrame:self.videoPreviewView.bounds];
    self.detectionLayer.rect = self.videoPreviewView.bounds;
}

- (void)appHasGoneInBackground: (UIApplication *)application {

    NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
    [defaults setBool:_flashlightBtn.selected forKey:@"camera.lightEnabled"];
    _camera.lightEnabled = NO;
}

- (void)appHasGoneInForeground: (UIApplication *)application {

    NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
    bool lightEnabled = (bool)[defaults boolForKey:@"camera.lightEnabled"];
    _flashlightBtn.selected = lightEnabled;
    _camera.lightEnabled = lightEnabled;
}

- (void) setupPreviewLayer
{
    _previewLayer = [_camera createPreviewLayer];
    
    CALayer *viewLayer = [self.videoPreviewView layer];
    [viewLayer setMasksToBounds:YES];
    
    [_previewLayer setVideoGravity:AVLayerVideoGravityResizeAspect];
    [viewLayer insertSublayer:_previewLayer below:[viewLayer sublayers][0]];
}

- (void) setupDetectionLayer
{
    self.detectionLayer = [[LTDetectionLayer alloc] initWithPreviewLayer:self.previewLayer];
    
    [BindThemeKeySel(self.detectionLayer, setLineColor:) fs_updateThemeCompleted:^(LTDetectionLayer*  _Nonnull target, SEL  _Nonnull sel) {
        [target setLineColor:target.fs_primaryColor ];
    }];
    
    CALayer *viewLayer = [self.videoPreviewView layer];
    [viewLayer addSublayer:self.detectionLayer];
}

- (void) startCameraSession
{
    [_captureService start];
    [_camera start];
    _captureService.useAutoTrigger = self.detectionBtn.selected;
    _detectionLabel.hidden = !self.detectionBtn.selected;
}

- (void) stopCameraSession
{
    _camera.lightEnabled = NO;
    [_captureService stop];
    [_camera stop];
    _flashlightBtn.selected = NO;
}

#pragma mark - btn action
- (void)closeCanmeraView
{
    if (self.documentSession) {
        if (self.documentSession.pageCount > 0) {
            [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kScanUnSaveTitle") message:FSLocalizedForKey(@"kScanUnSaveMsg") actions:@[FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kDiscard")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                if (index == 0) {
                    return;
                }else if (index == 1){
                    [self dismissViewControllerAnimated:YES completion:nil];
                }
            } onCtrl:[self fs_currentSceneTopMostViewController]];
            return;
        }
    }
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (void)openAlbumLibraryTap
{
    _imageNilCounts = 0;
    PHAuthorizationStatus statusA = [PHPhotoLibrary authorizationStatus];
    if (statusA == PHAuthorizationStatusRestricted || statusA == PHAuthorizationStatusDenied) {
        [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kALBUM") actions:@[FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kGoSetting")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
            if (index == 0) {
                return;
            }else if (index == 1){
                NSURL*url =[NSURL URLWithString:UIApplicationOpenSettingsURLString];
                if([[UIApplication sharedApplication] canOpenURL:url]) {
                    [[UIApplication sharedApplication] openURL:url options:@{} completionHandler:nil];
                }
            }
        } onCtrl:[self fs_currentSceneTopMostViewController]];
        return;
    }else if (statusA == PHAuthorizationStatusNotDetermined){
        [PHPhotoLibrary  requestAuthorization:^(PHAuthorizationStatus status) {
            if (status == PHAuthorizationStatusRestricted || status == PHAuthorizationStatusDenied) {
                return;
            }
            
            dispatch_async(dispatch_get_main_queue(), ^{
                [self->_photos removeAllObjects];
                FXUIPickViewController *pickVc = [[FXUIPickViewController alloc] init];
                if (self.replaceCallBack) pickVc.allowSingleSelect = YES;
                pickVc.delegate = self;
                
                FSNavigationController *imagePickerNav = [[FSNavigationController alloc] initWithRootViewController:pickVc];
                self->_cameraTriggered = YES;
                imagePickerNav.modalPresentationStyle = UIModalPresentationFullScreen;
                [self presentViewController:imagePickerNav animated:YES completion:^{
                    
                }];
            });
        }];
    }else{
        [_photos removeAllObjects];
        FXUIPickViewController *pickVc = [[FXUIPickViewController alloc] init];
        if (self.replaceCallBack) pickVc.allowSingleSelect = YES;
        pickVc.delegate = self;
        
        FSNavigationController *imagePickerNav = [[FSNavigationController alloc] initWithRootViewController:pickVc];
        _cameraTriggered = YES;
        imagePickerNav.modalPresentationStyle = UIModalPresentationFullScreen;
        [self presentViewController:imagePickerNav animated:YES completion:nil];
    }
}

- (void)shootTap:(UIButton *)btn
{
    [self takePhoto];
}

- (void)takePhoto
{
    if (!_cameraTriggered) {
        _cameraTriggered = YES;
        [self.view fs_showHUDLoading:nil];
        [_camera takePicture];
    }
}

- (void)capturedImagecountBtnTap
{
    if(self.cameraThumbnailView.selectPath){
        [self.cameraThumbnailView resetSelect];
    }
    if (PDFScanManager.doneCallBack) {

        NSString* documentName = [@"Scan-Done " stringByAppendingString:[LTDefaultFilenameProvider timeBasedFilename]];
        [self.documentSession saveAsPDF:documentName];
        
        return;
    }
    
    if (_isOpenFromEditor) {
        [self dismissViewControllerAnimated:YES completion:nil];
    }else{
            LTDocumentEditViewControllerNew* editorView = [[LTDocumentEditViewControllerNew alloc] init];
            editorView.documentSession = self.documentSession;
            editorView.documentSession.delegate = editorView;
            editorView.normalModelSavedDocument = ^(){
            };
            editorView.discardCompelet = ^(){
    
            };
            editorView.addPageDocumentReload = ^(LTDocumentSession *session){
                self->_documentSession = session;
                self->_documentSession.delegate = self;
            };
        [self.navigationController pushViewController:editorView animated:YES];
    }
}

- (void)flashlightBtnTap:(UIButton *)btn
{
    if (!_camera.hasTorch) {
        FSAlertViewCtrlShow(@"kWarning", @"kNoTorch");
        btn.selected = NO;
        return;
    }
    if (_camera.lightEnabled) {
        self.flashlightBtn.accessibilityLabel = FSLocalizedForKey(@"kFlashOffVoiceover");
    }else{
        self.flashlightBtn.accessibilityLabel = FSLocalizedForKey(@"kFlashOnVoiceover");
    }
    btn.accessibilityTraits = UIAccessibilityTraitButton;
    btn.selected = !btn.selected;
    _camera.lightEnabled = !_camera.lightEnabled;
    
    if (_camera.lightEnabled) {
        [self.view fs_showHUDMessage:FSLocalizedForKey(@"kLightOpen") afterHideDelay:1.0];
    }else{
        [self.view fs_showHUDMessage:FSLocalizedForKey(@"kLightClose") afterHideDelay:1.0];
    }
}

- (void)singleFrametBtnTap:(UIButton *)btn
{
    if (btn.selected) {
        btn.accessibilityLabel = FSLocalizedForKey(@"kSingleScanVoiceover");
    }else{
        btn.accessibilityLabel = FSLocalizedForKey(@"kContinueScanVoiceover");
    }
    btn.accessibilityTraits = UIAccessibilityTraitButton;
    btn.selected = !btn.selected;
    [Preference setIntValue:@"PhotoToPDF" type:@"singleFrame" value:(int)btn.selected];
    if (btn.selected) {
        [self.view fs_showHUDMessage:FSLocalizedForKey(@"kContinueMode") afterHideDelay:1.0];
    }else{
        [self.view fs_showHUDMessage:FSLocalizedForKey(@"kSigleMode") afterHideDelay:1.0];
    }
}

- (void)detectionBtnTap:(UIButton *)btn
{
    btn.selected = !btn.selected;
    if (btn.selected) {
        btn.accessibilityLabel = FSLocalizedForKey(@"kCloseDetectVoiceover");
    }else{
        btn.accessibilityLabel = FSLocalizedForKey(@"kOpenDetectVoiceover");
    }
    btn.accessibilityTraits = UIAccessibilityTraitButton;
    [Preference setIntValue:@"PhotoToPDF" type:@"detectAuto" value:(int)btn.selected];
    _captureService.useAutoTrigger = btn.selected;
    _detectionLabel.hidden = !btn.selected;
    _formatSizeBtn.enabled = btn.selected;
    if (!btn.selected) {
        [_detectionLayer updateResultDisplay:nil];
    }

    if (btn.selected) {
        [self.view fs_showHUDMessage:FSLocalizedForKey(@"kAutoDetectOpen") afterHideDelay:1.0];
    }else{
        [self.view fs_showHUDMessage:FSLocalizedForKey(@"kAutoDetectClose") afterHideDelay:1.0];
    }
}

- (void)formatSizeBtnTap:(UIButton *)btn
{
    LTPageFormatViewControllerNew* formatView = [[LTPageFormatViewControllerNew alloc] init];
    
    formatView.delegate = self;
    
    FSNavigationController *formatPageNav = [[FSNavigationController alloc] initWithRootViewController:formatView];
    formatPageNav.modalPresentationStyle = DEVICE_iPHONE ? UIModalPresentationFullScreen : UIModalPresentationFormSheet;
    formatPageNav.modalTransitionStyle = UIModalTransitionStyleCoverVertical;
    _cameraTriggered = YES;
    [self presentViewController:formatPageNav animated:YES completion:nil];
}

- (void)FXUIPickViewController:(FXUIPickViewController *)pickViewController Image:(UIImage *)image counts:(int)counts currentIndex:(int)currentIndex
{
    if (counts > 0) {
        if (image == nil) {
            _imageNilCounts += 1;
            if (currentIndex == counts - 1) {
                [self addImageToDocumentSession];
            }
        }else{
            if (currentIndex != counts - 1) {
                [self.photos addObject:image];
            }else{
                [self.photos addObject:image];
                [self addImageToDocumentSession];
            }
        }
        if (counts == _imageNilCounts && (currentIndex == counts - 1)) {
            dispatch_async(dispatch_get_main_queue(), ^{
                FSAlertViewCtrlShow(@"kWarning", @"kOK");
            });
            _cameraTriggered = NO;
            return;
        }
    }else{
        _cameraTriggered = NO;
    }
}

- (void)addImageToDocumentSession
{
    [self.view fs_showHUDLoading:nil process:^{
        for (int index = 0; index < self->_photos.count; index++) {
            UIImage *image = self->_photos[index];
            self->_imageProcess = [[LTImageProcessor alloc] initWithImage:image];
            const unsigned int LowResThreshold = 600 * 1000 * 1000;    // 600 MB
            bool useLowRes = [NSProcessInfo processInfo].physicalMemory < LowResThreshold;
            [self->_imageProcess setComputeLowResPreview: useLowRes];
            [self->_imageProcess setPageFormatWidth:self->_pageFormat.width height:self->_pageFormat.height receipt:self->_pageFormat.receipt];
            [self->_imageProcess setPageFormatThreshold:10.0f];
            self->_imageProcess.delegate = self;
            [self->_imageProcess computeResult];
            self->isWaiting = YES;
            while (self->isWaiting) {
                [NSThread sleepForTimeInterval:0.1];
            }
        }
        self->_cameraTriggered = NO;
    }];
}

- (void) pageFomatViewController:(LTPageFormatViewControllerNew *)controller didFinishWithFormat:(LTPageFormat *)pageFormat
{
    [controller dismissViewControllerAnimated:YES completion:^{
        if (pageFormat)
        {
            self->_pageFormat = pageFormat;
            [LTPageFormatHelper storeFormat:pageFormat];
            self->_cameraTriggered = NO;
        }
    }];
}

#pragma mark - LTCaptureService delegate

- (void) captureService:(LTCaptureService *)service didCaptureImage:(UIImage *)image withDetection:(LTDetectionResult*) detection
{
    _imageProcess = [[LTImageProcessor alloc] initWithImage:image];
   const unsigned int LowResThreshold = 600 * 1000 * 1000;    // 600 MB
    bool useLowRes = [NSProcessInfo processInfo].physicalMemory < LowResThreshold;
    [_imageProcess setComputeLowResPreview: useLowRes];
    LTPageFormat *format =_pageFormat;
    NSArray *points = [NSMutableArray arrayWithObjects:
                        [NSValue valueWithCGPoint:CGPointMake(0, 0)],
                        [NSValue valueWithCGPoint:CGPointMake(image.size.width, 0)],
                        [NSValue valueWithCGPoint:CGPointMake(image.size.width, image.size.height)],
                        [NSValue valueWithCGPoint:CGPointMake(0, image.size.height)],
                        nil];
     detection = [[LTDetectionResult alloc]
                                initWithPoints:points
                                imageWidth:image.size.width
                                imageHeight:image.size.height];
    [_imageProcess setDetectionResult:detection];

    if (!self.detectionBtn.selected || (self.detectionBtn.selected && !detection.valid)) {
        int horDpi = [_imageProcess horizontalDpi];
        int verDpi = [_imageProcess verticalDpi];
        CGSize size = image.size;
        format = [LTPageFormatHelper evaluateFormat:size.width height:size.height horDpi:horDpi verDpi:verDpi];
    }
    [_imageProcess setPageFormatWidth:format.width height:format.height receipt:format.receipt];
   
    [_imageProcess setPageFormatThreshold:10.0f];
     _imageProcess.delegate = self;
    
    [_imageProcess computeResult];
    [self.view fs_showHUDLoading:nil];
    [_detectionLayer updateResultDisplay:nil];
    [self animationWithImg:image];
}

- (void) captureService:(LTCaptureService *)service detected:(LTDetectionResult *)result
{
    if (!_cameraTriggered && service.useAutoTrigger){
        [_statusLabelController updateMessage:result];
        [_detectionLayer updateResultDisplay:result];
    }
}

- (void) captureServiceDidAutoTrigger:(LTCaptureService *)service
{
    _cameraTriggered = YES;
}

- (void) captureServiceAutoTriggerCanceled:(LTCaptureService *)service
{
    _cameraTriggered = NO;
}

- (void) captureService:(LTCaptureService*) service proposesGUIUpdate:(LTDetectionResult*) result
{
    
    if (!_cameraTriggered && service.useAutoTrigger) {
        [_detectionLayer updateResultDisplay:result];
    }
}

- (void) imageProcessorStartedComputing:(LTImageProcessor*) processor
{
}

- (void) imageProcessor:(LTImageProcessor*) processor didFinishComputingPreviewImage:(UIImage*) image
{
    [self.view fs_hideHUDLoading];
}

- (void) imageProcessor:(LTImageProcessor*) processor didFinishComputingResultImage:(UIImage*) image
{
    if (_cameraTriggered) {
        [self.view fs_hideHUDLoading];
    }
    if (_documentSession != nil)
    {
        [_documentSession addPage:image];
    }
    else
    {
        if (self.replaceCallBack){
            [self dismissViewControllerAnimated:YES completion:^{
                self.replaceCallBack(image);
            }];
            return;
        }
        _documentSession = [LTDocumentSession createNewDocument];
        _documentSession.delegate = self;
        _documentSession.rename = nil;
        [_documentSession addPage:image];
    }
    [self configureAndSetLayoutOfImageCount];
    [self setCameraThumbnailViewImages];
    if (!self.singleFrametBtn.selected && !isWaiting) {
        LTImageProcessViewControllerNew *imageProcessViewControllerNew = [[LTImageProcessViewControllerNew alloc] initWithImage:[_documentSession imageForPage:(_documentSession.pageCount - 1)] isPresent:YES];
        imageProcessViewControllerNew.delegate = self;
        
        FSNavigationController *nav = [[FSNavigationController alloc] initWithRootViewController:imageProcessViewControllerNew];
        nav.modalPresentationStyle = UIModalPresentationFullScreen;
        [self presentViewController:nav animated:YES completion:nil];
        _pageInProcessor = _documentSession.pageCount - 1;
    }
    
    if (!isWaiting && self.singleFrametBtn.selected) {
        [_captureService start];
        if (self.flashlightBtn.selected) {
            _camera.lightEnabled = self.flashlightBtn.selected;
        }
    }
    
    [self setCameraThumbnailViewImages];
    _cameraTriggered = NO;
    isWaiting = NO;
    
}

- (void) processImageView:(LTImageProcessViewControllerNew*) ctrl
      didFinishWithResult:(UIImage*) image
{
    if (image) {
        [_documentSession replacePage:_pageInProcessor withImage:image];
    }
    [self setCameraThumbnailViewImages];
}

- (void)configureAndSetLayoutOfImageCount
{
    NSString *countText;
    if (self.documentSession) {
        if (self.documentSession.pageCount > 0) {
            countText = [NSString stringWithFormat:@"%d",self.documentSession.pageCount];
        }else{
            countText = @"0";
        }
    }else{
        countText = @"0";
    }
    if ([countText isEqualToString:@"0"]) {
        [self.cameraThumbnailContentView setHidden:YES];
        self.completeBtn.enabled = NO;
    }
    else
    {
        self.completeBtn.enabled = YES;
        [self.cameraThumbnailContentView setHidden:NO];
        self.countLabel.text = countText;
        [self.countLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.right.mas_equalTo(self.dragBtn.mas_right).mas_offset(7);
            make.top.mas_equalTo(self.dragBtn.mas_top).mas_offset(-7);
            make.width.mas_equalTo(20);
            make.height.mas_equalTo(20);
        }];
    }
    self.shootBtn.accessibilityValue = [NSString stringWithFormat:FSLocalizedForKey(@"kScansCountVoiceover"),self.documentSession.pageCount];
}

- (void) documentSession:(LTDocumentSession*) session startedSavingDocumentToFile:(NSString*) documentName
{
    [self.view fs_showHUDLoading:nil];
}

- (void) documentSession:(LTDocumentSession*) session didFinishSavingDocument:(LTDocument*) document
{
    [self.view fs_hideHUDLoading];
    [[PDFScanManager shareManager].documentManager addDocument:document];
    
    if (PDFScanManager.doneCallBack)
    {
        NSString *filePath = [[PDFScanManager shareManager].documentPath stringByAppendingPathComponent:_documentSession.documentName];
        filePath = [filePath stringByAppendingString:@".pdf"];

        NSString *realSaveAsPath = document.urlForDocument.path;
        NSError *error = nil;
        if (![filePath isEqualToString:realSaveAsPath]) {
            [[NSFileManager defaultManager] copyItemAtPath:realSaveAsPath toPath:filePath error:&error];
        }
        PDFScanManager.doneCallBack(error, filePath);
    }
    
    if ([[PDFScanManager shareManager] savePDFFlieWithRealFilePath:document.urlForDocument.path]) {
        return;
    }
    
    [self dismissViewControllerAnimated:YES completion:nil];
}

#pragma mark --set images for cameraThumbnailView
- (void)setCameraThumbnailViewImages
{
    NSMutableArray *imagesArray = [NSMutableArray array];
    for (int i = 0; i < _documentSession.pageCount; i++) {
        UIImage *image = [_documentSession imageForPage:i];
        if (image) {
            [imagesArray addObject:image];
        }
    }

    self.cameraThumbnailView.arrayImages = [imagesArray mutableCopy];
}

#pragma mark --LTCameraThumbnailViewDalegate
- (void)didSelectItemOfCollectionView:(NSIndexPath *)indexPath
{
    if (indexPath.row >= _documentSession.pageCount) {
        return;
    }
    int itemIndex = (int)indexPath.row;
    LTImageProcessViewControllerNew *imageProcessViewControllerNew = [[LTImageProcessViewControllerNew alloc] initWithImage:[_documentSession imageForPage:itemIndex] isPresent:YES];
    imageProcessViewControllerNew.delegate = self;
    _pageInProcessor = itemIndex;
    
    FSNavigationController *nav = [[FSNavigationController alloc] initWithRootViewController:imageProcessViewControllerNew];
    nav.modalPresentationStyle = UIModalPresentationFullScreen;
    [self presentViewController:nav animated:YES completion:nil];
}

- (void)deleteItemOfCollectionView:(NSInteger) itemIndex
{
    [_documentSession removePage:(int)itemIndex];
    [self configureAndSetLayoutOfImageCount];
}

@end
