//
//  LTImageProcessViewControllerNew.m
//  FoxitApp
//
//  Created by Apple on 16/12/2.
//
//

#import "LTImageProcessViewControllerNew.h"
#import "LTImageContentView.h"
#import "LTModeList.h"
#import "LTPageFormat.h"
#import "LTPageFormatHelper.h"
#import "LTImageCropProcessViewController.h"
#import "LTSliderBar.h"
#import "LTCanmeraViewControllerNew.h"

@interface LTImageProcessViewControllerNew ()<UIScrollViewDelegate,LTImageCropProcessViewDelegate,UIGestureRecognizerDelegate,LTImageProcessorDelegate,LTSliderBarDelegate>
{
    int _selectedPoint;
    bool _didMovePoint;
}
@property (nonatomic, strong) LTImageContentView *contentView;
@property (nonatomic, strong) FSToolbar *bottomBar;
@property (nonatomic, strong) FSToolbar *topBar;
@property (nonatomic, strong) LTImageProcessor* imageProcessor;
@property (nonatomic, strong) LTPageFormat* pageFormat;
@property (nonatomic, assign) BOOL isCroping;
@property (nonatomic, strong) UITapGestureRecognizer *tapGesture;
@property (nonatomic, strong) UIButton *modelBtn;
@property (nonatomic, strong) UIButton *saveBtn;
@property (nonatomic, strong) UIButton *cancelBtn;
@property (nonatomic, strong) UIButton *doneBtn;
@property (nonatomic, strong) UIButton *cancelPropertyBtn;
@property (nonatomic, strong) UIButton *rotationBtn;
@property (nonatomic, strong) UIButton *cropBtn;
@property (nonatomic, strong) UIButton *replaceBtn;
@property (nonatomic, strong) FSToolbar *colorSettingBottomBar;
@property (nonatomic, strong) FSToolbar *colorBrightContrastBottomBar;
@property (nonatomic, strong) FSItem* changeColorItem;
@property (nonatomic, strong) FSItem* changeBrightnessItem;
@property (nonatomic, strong) FSItem* changeContrastItem;
@property (nonatomic, strong) FSItem* colorNoneItem;
@property (nonatomic, strong) FSItem* colorfulItem;
@property (nonatomic, strong) FSItem* colorWhiteBlackItem;
@property (nonatomic, strong) FSItem* colorGrayItem;

@property (nonatomic, strong) UIImageView* colorNoneSelected;
@property (nonatomic, strong) UIImageView* colorfulSelected;
@property (nonatomic, strong) UIImageView* colorWhiteBlackSelected;
@property (nonatomic, strong) UIImageView* colorGraySelected;

@property (nonatomic, strong) LTSliderBar* brightnessSliderBar;
@property (nonatomic, strong) LTSliderBar* constrastSliderBar;
@property (nonatomic, assign) BOOL isChanged;
@property NSMutableArray* points;
@property (nonatomic, strong) UIImage* orignalImage;
@property (nonatomic, assign) float orignalContrastLevel;
@property (nonatomic, assign) float orignalBrightnessLevel;
@property (nonatomic, assign) LTColorMode orignalColorMode;
@property (nonatomic, assign) BOOL isPresent;
@end

@implementation LTImageProcessViewControllerNew

- (id) initWithImage:(UIImage*)image isPresent:(BOOL)isPresent
{
    if (self = [super init])
    {
        _orignalImage = image;
        [self initImageProcessor:image isPresent:isPresent];
    }
    return self;
}

- (void)initImageProcessor:(UIImage*)image isPresent:(BOOL)isPresent
{
    _isPresent = isPresent;
    [self resetImageProcessorWithImg:image];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    self.navigationController.navigationBarHidden = YES;
    self.navigationController.navigationBar.translucent = YES;
    [self initSubViews];
    self.isChanged = NO;
    // Do any additional setup after loading the view.
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)resetImageProcessorWithImg:(UIImage*)image
{
    _isCroping = NO;
    _imageProcessor = [[LTImageProcessor alloc] initWithImage:image];
    [_imageProcessor setDelegate:self];
    
    int horDpi = [_imageProcessor horizontalDpi];
    int verDpi = [_imageProcessor verticalDpi];
    CGSize size = image.size;
    LTPageFormat *format = [LTPageFormatHelper evaluateFormat:size.width height:size.height horDpi:horDpi verDpi:verDpi];
    _pageFormat = format;
    
    [_imageProcessor setPageFormatWidth:_pageFormat.width height:_pageFormat.height];
    //[_imageProcessor rotateImage:LTRotationDirectionCW];
    const unsigned int LowResThreshold = 600 * 1000 * 1000;    // 600 MB
    bool useLowRes = [NSProcessInfo processInfo].physicalMemory < LowResThreshold;
    [_imageProcessor setComputeLowResPreview: useLowRes];
    if (self.brightnessSliderBar) [self.brightnessSliderBar setSliderValue:_imageProcessor.brightnessLevel minimumValue:-5 maximumValue:5 minimumValueForShow:-1 maximumValueForShow:1];
    if (self.constrastSliderBar) [self.constrastSliderBar setSliderValue:_imageProcessor.contrastLevel minimumValue:-5 maximumValue:5 minimumValueForShow:0.5 maximumValueForShow:1.5];
    [self saveLastTimeValueWithColorSetting];
}
    
- (void)initSubViews
{
    [self initTopBar];
    
    self.view.backgroundColor = self.fs_backgroundColor_B5;
    
    FSToolbar *bottomBar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleBottom];
    bottomBar.hasDivide = NO;
    self.bottomBar = bottomBar;
    bottomBar.backgroundColor = [UIColor colorWithWhite:0.f alpha:0.8];
    [self.view addSubview:bottomBar];
    [bottomBar mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(bottomBar.superview.mas_left);
        make.right.mas_equalTo(bottomBar.superview.mas_right);
        make.bottom.mas_equalTo(bottomBar.superview.mas_bottom);
    }];
    [self initBottomBar:bottomBar];
    
    FSToolbar* colorBrightContrastBottomBarBottomBar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleBottom];
    [colorBrightContrastBottomBarBottomBar setHasDivide:NO];
    colorBrightContrastBottomBarBottomBar.barTintColor = UIColor.blackColor;
    _colorBrightContrastBottomBar = colorBrightContrastBottomBarBottomBar;
    [self.view addSubview:colorBrightContrastBottomBarBottomBar];
    [colorBrightContrastBottomBarBottomBar mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(colorBrightContrastBottomBarBottomBar.superview.mas_left);
        make.right.mas_equalTo(colorBrightContrastBottomBarBottomBar.superview.mas_right);
        make.bottom.mas_equalTo(colorBrightContrastBottomBarBottomBar.superview.mas_bottom);
    }];
    [colorBrightContrastBottomBarBottomBar setHidden:YES];
    [self initColorBrightContrastBottomBarBottomBar:colorBrightContrastBottomBarBottomBar];
    
    FSToolbar* colorSettingBottomBar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleBottom];
    [colorSettingBottomBar setHasDivide:NO];
    colorSettingBottomBar.backgroundColor = [UIColor colorWithWhite:0.f alpha:0.4];
    _colorSettingBottomBar = colorSettingBottomBar;
    [self.view addSubview:colorSettingBottomBar];
    [colorSettingBottomBar mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(colorSettingBottomBar.superview.mas_left);
        make.right.mas_equalTo(colorSettingBottomBar.superview.mas_right);
        make.bottom.mas_equalTo(colorBrightContrastBottomBarBottomBar.mas_top);
        make.height.mas_equalTo(58).priorityHigh();
    }];
    [colorSettingBottomBar setHidden:YES];
    [self initColorSettingBottomBar:colorSettingBottomBar];
    
    //contentView.contentOffset = CGPointMake([NSObject fs_getForegroundActiveWindow].bounds.size.width, 0);
    [self.view addSubview:self.brightnessSliderBar];
    [self.brightnessSliderBar mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.brightnessSliderBar.superview.mas_left);
        make.right.mas_equalTo(self.brightnessSliderBar.superview.mas_right);
        make.bottom.mas_equalTo(colorBrightContrastBottomBarBottomBar.mas_top);
        make.height.mas_equalTo(58).priorityHigh();
    }];
    [self.brightnessSliderBar layoutIfNeeded];
    [self.brightnessSliderBar setHidden:YES];
    [self.brightnessSliderBar setSliderValue:_imageProcessor.brightnessLevel minimumValue:-5 maximumValue:5 minimumValueForShow:-1 maximumValueForShow:1];
    
    [self.view addSubview:self.constrastSliderBar];
    [self.constrastSliderBar mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.constrastSliderBar.superview.mas_left);
        make.right.mas_equalTo(self.constrastSliderBar.superview.mas_right);
        make.bottom.mas_equalTo(colorBrightContrastBottomBarBottomBar.mas_top);
        make.height.mas_equalTo(58).priorityHigh();
    }];
    [self.constrastSliderBar layoutIfNeeded];
    [self.constrastSliderBar setHidden:YES];
    [self.constrastSliderBar setSliderValue:_imageProcessor.contrastLevel minimumValue:-5 maximumValue:5 minimumValueForShow:0.5 maximumValueForShow:1.5];
    
    UIView *contentViewAutoLayout = [[UIView alloc] init];
    contentViewAutoLayout.backgroundColor = self.fs_backgroundColor_B5; // self.fs_textColor_T4;
    [self.view addSubview:contentViewAutoLayout];
    [contentViewAutoLayout mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(contentViewAutoLayout.superview.mas_left);
        make.right.mas_equalTo(contentViewAutoLayout.superview.mas_right);
        make.top.mas_equalTo(self.topBar.mas_bottom);
        make.bottom.mas_equalTo(bottomBar.mas_top).offset(-63);
    }];
    
    LTImageContentView *contentView = [[LTImageContentView alloc] init];
    self.contentView = contentView;
    contentView.delegate = self;
    contentView.backgroundColor = self.fs_backgroundColor_B5; 

    contentView.minimumZoomScale=1.0f;
    contentView.maximumZoomScale=3.0f;
    [contentViewAutoLayout addSubview:contentView];
    [contentView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(contentView.superview.mas_left).offset(10);
        make.right.mas_equalTo(contentView.superview.mas_right).offset(-10);
        make.top.mas_equalTo(contentView.superview.mas_top).offset(20);
        make.bottom.mas_equalTo(0);
    }];
    [contentView addImageView];
    
    [self.contentView.editImageView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.center.equalTo(self.contentView);
    }];
    
    contentView.editImage = _imageProcessor.inputImage;
    
    [self.view bringSubviewToFront:colorSettingBottomBar];
    [self.view bringSubviewToFront:_constrastSliderBar];
    [self.view bringSubviewToFront:_brightnessSliderBar];
 
    [self saveLastTimeValueWithColorSetting];
}

- (void)viewDidLayoutSubviews
{
    [super viewDidLayoutSubviews];
    [self.contentView.editImageView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.center.equalTo(self.contentView);
        make.size.mas_equalTo(self.contentView.frame.size);
    }];
}

- (void)initTopBar
{
    _topBar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleTop];
    [_topBar setBackgroundColor:self.fs_backgroundColor_B5]; // self.fs_textColor_T4
    _topBar.hasDivide = NO;
    [self.view addSubview:_topBar];
    [_topBar mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(_topBar.superview.fs_mas_top);
        make.left.right.mas_equalTo(_topBar.superview);
    }];
    UIButton *cancelBtn = [[UIButton alloc] init];
    _cancelBtn = cancelBtn;
    [_cancelBtn setTitleColor:UIColor.whiteColor forState:UIControlStateNormal];
    [cancelBtn setTitleColor:[[UIColor whiteColor] colorWithAlphaComponent:LABEL_HIGHLIGHTED_ALPHA] forState:UIControlStateHighlighted];
    [cancelBtn addTarget:self action:@selector(cancelBtnClick:) forControlEvents:UIControlEventTouchUpInside];
    [cancelBtn setTitle:FSLocalizedForKey(@"kCancel") forState:UIControlStateNormal];
    cancelBtn.titleLabel.font = self.fs_standardGeneralFont_GF2;
    [_topBar addSubview:cancelBtn];
    [cancelBtn mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(cancelBtn.superview.mas_left).offset(14);
        make.centerY.mas_equalTo(cancelBtn.superview.mas_centerY);
        make.height.mas_equalTo(49);
    }];
    
    UIButton *cancelBtn1 = [[UIButton alloc] init];
    cancelBtn1.hidden = YES;
    _cancelPropertyBtn = cancelBtn1;
    [_cancelPropertyBtn setTitleColor:UIColor.whiteColor forState:UIControlStateNormal];
    [cancelBtn1 setTitleColor:[[UIColor whiteColor] colorWithAlphaComponent:LABEL_HIGHLIGHTED_ALPHA] forState:UIControlStateHighlighted];
    [cancelBtn1 addTarget:self action:@selector(cancelBtnClick:) forControlEvents:UIControlEventTouchUpInside];
    [cancelBtn1 setTitle:FSLocalizedForKey(@"kCancel") forState:UIControlStateNormal];
    cancelBtn1.titleLabel.font = self.fs_standardGeneralFont_GF2;
    [_topBar addSubview:cancelBtn1];
    [cancelBtn1 mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(cancelBtn1.superview.mas_left).offset(14);
        make.centerY.mas_equalTo(cancelBtn1.superview.mas_centerY);
        make.height.mas_equalTo(49);
    }];
    
    UIButton *saveBtn = [[UIButton alloc] init];
    saveBtn.enabled = NO;
    _saveBtn = saveBtn;
    [_saveBtn setTitleColor:UIColor.whiteColor forState:UIControlStateNormal];
    [_saveBtn setTitleColor:[[UIColor whiteColor] colorWithAlphaComponent:LABEL_HIGHLIGHTED_ALPHA] forState:UIControlStateHighlighted];
    [_saveBtn setTitleColor:[[UIColor whiteColor] colorWithAlphaComponent:LABEL_DISABLED_ALPHA] forState:UIControlStateDisabled];
    saveBtn.titleLabel.textAlignment = NSTextAlignmentRight;
    [saveBtn addTarget:self action:@selector(saveBtnClick:) forControlEvents:UIControlEventTouchUpInside];
    [saveBtn setTitle:FSLocalizedForKey(@"kSave") forState:UIControlStateNormal];
    saveBtn.titleLabel.font = self.fs_standardGeneralFont_GF2;
    [_topBar addSubview:saveBtn];
    [saveBtn mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(saveBtn.superview.mas_right).offset(-14);
        make.centerY.mas_equalTo(saveBtn.superview.mas_centerY);
        make.height.mas_equalTo(49);
    }];
    
    UIButton *doneBtn = [[UIButton alloc] init];
    doneBtn.hidden = YES;
    _doneBtn = doneBtn;
    [_doneBtn setTitleColor:UIColor.whiteColor forState:UIControlStateNormal];
    [_doneBtn setTitleColor:[[UIColor whiteColor] colorWithAlphaComponent:LABEL_HIGHLIGHTED_ALPHA] forState:UIControlStateHighlighted];
    _doneBtn.titleLabel.textAlignment = NSTextAlignmentRight;
    [_doneBtn addTarget:self action:@selector(saveBtnClick:) forControlEvents:UIControlEventTouchUpInside];
    [_doneBtn setTitle:FSLocalizedForKey(@"kDone") forState:UIControlStateNormal];
    _doneBtn.titleLabel.font = self.fs_standardGeneralFont_GF2;
    [_topBar addSubview:_doneBtn];
    [doneBtn mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(doneBtn.superview.mas_right).offset(-14);
        make.centerY.mas_equalTo(doneBtn.superview.mas_centerY);
        make.height.mas_equalTo(49);
    }];
}

- (void)initBottomBar:(UIView *)bottomBar
{
    UIButton *cropBtn = [[UIButton alloc] init];
    _cropBtn = cropBtn;
    _cropBtn.accessibilityLabel = FSLocalizedForKey(@"kCropDone");
    [cropBtn setImage:ImageNamed(@"scan_image_crop") forState:UIControlStateNormal];
    [cropBtn setImage:[Utility unableImage:ImageNamed(@"scan_image_crop")] forState:UIControlStateHighlighted];
    [cropBtn addTarget:self action:@selector(cropBtnClick:) forControlEvents:UIControlEventTouchUpInside];
    [bottomBar addSubview:cropBtn];
    [cropBtn mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.centerX.mas_equalTo(cropBtn.superview.mas_centerX).offset([self fs_currentSceneIsCompact] ? 30 : 40);
        make.centerY.mas_equalTo(cropBtn.superview.mas_centerY);
        make.height.mas_equalTo(40);
        make.width.mas_equalTo(40);
    }];
    
    UIButton *rotateBtn = [[UIButton alloc] init];
    rotateBtn.accessibilityLabel = FSLocalizedForKey(@"kCounterClockWiseVoiceover");
    _rotationBtn = rotateBtn;
    [rotateBtn setImage:ImageNamed(@"scan_image_rotate") forState:UIControlStateNormal];
    [rotateBtn setImage:[Utility unableImage:ImageNamed(@"scan_image_rotate")] forState:UIControlStateHighlighted];
    [rotateBtn addTarget:self action:@selector(rotateBtnClick:) forControlEvents:UIControlEventTouchUpInside];
    [bottomBar addSubview:rotateBtn];
    [rotateBtn mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.centerX.mas_equalTo(rotateBtn.superview.mas_centerX).offset([self fs_currentSceneIsCompact] ? -30 : -40);
        make.centerY.mas_equalTo(rotateBtn.superview.mas_centerY);
        make.height.mas_equalTo(40);
        make.width.mas_equalTo(40);
    }];
    
    UIButton *modelBtn = [[UIButton alloc] init];
    modelBtn.accessibilityLabel = FSLocalizedForKey(@"kAdjustBtn");
    _modelBtn = modelBtn;
    [modelBtn setImage:ImageNamed(@"scan_image_color") forState:UIControlStateNormal];
    [modelBtn setImage:ImageNamed(@"scan_image_color") forState:UIControlStateSelected];
    [modelBtn addTarget:self action:@selector(modelBtnClick:) forControlEvents:UIControlEventTouchUpInside];
    [bottomBar addSubview:modelBtn];
    [modelBtn mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(_cropBtn.mas_right).offset([self fs_currentSceneIsCompact] ? 20 : 40);
        make.centerY.mas_equalTo(modelBtn.superview.mas_centerY);
        make.height.mas_equalTo(40);
        make.width.mas_equalTo(40);
    }];
    
    UIButton *replaceBtn = [[UIButton alloc] init];
    _replaceBtn = replaceBtn;
    _replaceBtn.accessibilityLabel = FSLocalizedForKey(@"kReplace");
    [_replaceBtn setImage:ImageNamed(@"scan_image_replace") forState:UIControlStateNormal];
    [_replaceBtn setImage:[Utility unableImage:ImageNamed(@"scan_image_replace")] forState:UIControlStateHighlighted];
    [_replaceBtn addTarget:self action:@selector(replaceBtnClick:) forControlEvents:UIControlEventTouchUpInside];
    [bottomBar addSubview:_replaceBtn];
    [replaceBtn mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(_rotationBtn.mas_left).offset([self fs_currentSceneIsCompact] ? -20 : -40);
        make.centerY.mas_equalTo(replaceBtn.superview.mas_centerY);
        make.height.mas_equalTo(40);
        make.width.mas_equalTo(40);
    }];
}

- (void)initColorBrightContrastBottomBarBottomBar:(FSToolbar *)bottomBar
{
    FSItem *changeColorItem = [FSItem createItemWithImage:ImageNamed(@"scan_change_color_normal") imageSelected:ImageNamed(@"scan_change_color_selected") imageDisable:ImageNamed(@"scan_change_color_normal")];
    changeColorItem.accessibilityLabel = FSLocalizedForKey(@"kColorVoiceover");
    _changeColorItem = changeColorItem;
    
    [BindThemeKeySel(changeColorItem, setImage:forState:) fs_updateThemeCompleted:^(FSItem*  _Nonnull target, SEL  _Nonnull sel) {
        [target setImage:[ImageNamed(@"scan_change_color_selected") fs_imageWithTintColor:target.fs_primaryColor] forState:UIControlStateSelected];
    }];
    
    @weakify(self)
    _changeColorItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [_item setSelected:!_item.isSelected];
        [self.changeContrastItem setSelected:NO];
        [self.changeBrightnessItem setSelected:NO];
        [self.colorSettingBottomBar setHidden:![self.colorSettingBottomBar isHidden]];
        [self.brightnessSliderBar setHidden:YES];
        [self.constrastSliderBar setHidden:YES];
    };
    [bottomBar addItem:changeColorItem spacing:(DEVICE_iPHONE?20 :60) atPosition:FSItemPositionCenter];
    
    FSItem *changeBrightnessItem = [FSItem createItemWithImage:ImageNamed(@"scan_change_brightness_normal") imageSelected:ImageNamed(@"scan_change_brightness_selected") imageDisable:ImageNamed(@"scan_change_brightness_normal")];
    changeBrightnessItem.accessibilityLabel = FSLocalizedForKey(@"kBrightnessVoiceover");
    _changeBrightnessItem = changeBrightnessItem;
    changeBrightnessItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [_item setSelected:!_item.isSelected];
        [self.changeContrastItem setSelected:NO];
        [self.changeColorItem setSelected:NO];
        [self.colorSettingBottomBar setHidden:YES];
        [self.brightnessSliderBar setHidden:![self.brightnessSliderBar isHidden]];
        [self.constrastSliderBar setHidden:YES];
    };
    [bottomBar addItem:changeBrightnessItem spacing:(DEVICE_iPHONE?20 :60) atPosition:FSItemPositionCenter];
    
    [BindThemeKeySel(changeBrightnessItem, setImage:forState:) fs_updateThemeCompleted:^(FSItem*  _Nonnull target, SEL  _Nonnull sel) {
        [target setImage:[ImageNamed(@"scan_change_brightness_selected") fs_imageWithTintColor:target.fs_primaryColor] forState:UIControlStateSelected];
    }];
    
    FSItem *changeContrastItem = [FSItem createItemWithImage:ImageNamed(@"scan_change_contrast_normal") imageSelected:ImageNamed(@"scan_change_contrast_selected") imageDisable:ImageNamed(@"scan_change_contrast_normal")];
    changeContrastItem.accessibilityLabel = FSLocalizedForKey(@"kContrastVoiceover");
    _changeContrastItem = changeContrastItem;
    changeContrastItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [_item setSelected:!_item.isSelected];
        [self.changeBrightnessItem setSelected:NO];
        [self.changeColorItem setSelected:NO];
        [self.colorSettingBottomBar setHidden:YES];
        [self.constrastSliderBar setHidden:![self.constrastSliderBar isHidden]];
        [self.brightnessSliderBar setHidden:YES];
    };
    [bottomBar addItem:changeContrastItem spacing:(DEVICE_iPHONE?20 :60) atPosition:FSItemPositionCenter];
    
    [BindThemeKeySel(changeContrastItem, setImage:forState:) fs_updateThemeCompleted:^(FSItem*  _Nonnull target, SEL  _Nonnull sel) {
        [target setImage:[ImageNamed(@"scan_change_contrast_selected") fs_imageWithTintColor:target.fs_primaryColor] forState:UIControlStateSelected];
    }];
}

- (void)initColorSettingBottomBar:(FSToolbar *)bottomBar
{
    @weakify(self)
    FSItem *colorNoneItem = [FSItem createItemWithImage:ImageNamed(@"scan_color_none") imageSelected:ImageNamed(@"scan_color_none") imageDisable:ImageNamed(@"scan_color_none")];
    colorNoneItem.accessibilityLabel = FSLocalizedForKey(@"kPhotoModelVoiceover");
    _colorNoneItem = colorNoneItem;
    [bottomBar addItem:colorNoneItem spacing:20 atPosition:FSItemPositionCenter];
    [colorNoneItem setNeedsLayout];
    UIImage* colorSelectedImg = ImageNamed(@"scan_color_selected");
    UIImageView* colorNoneSelected = [[UIImageView alloc] init];
    [BindThemeKeySel(colorNoneSelected, setImage:) fs_updateThemeCompleted:^(UIImageView*  _Nonnull target, SEL  _Nonnull sel) {
        [target setImage:[colorSelectedImg fs_imageWithTintColor:target.fs_primaryColor] ];
    }];
    [bottomBar addSubview:colorNoneSelected];
    _colorNoneSelected = colorNoneSelected;
    CGFloat offsetImgLength = -1.5;
    [colorNoneSelected mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(colorNoneItem.mas_left).offset(offsetImgLength);
        make.top.equalTo(colorNoneItem.mas_top).offset(offsetImgLength);
        make.size.mas_equalTo(colorSelectedImg.size);
    }];
    
    colorNoneItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [self.imageProcessor setColorMode:LTColorModeRGBWhiteBalance];
        [self.view fs_showHUDLoading:nil];
        [self.imageProcessor computePreview];
        
        [self.colorNoneSelected setHidden:NO];
        [self.colorfulSelected setHidden:YES];
        [self.colorWhiteBlackSelected setHidden:YES];
        [self.colorGraySelected setHidden:YES];
    };
    
    FSItem *colorfulItem = [FSItem createItemWithImage:ImageNamed(@"scan_color_colorful") imageSelected:ImageNamed(@"scan_color_colorful") imageDisable:ImageNamed(@"scan_color_colorful")];
    colorfulItem.accessibilityLabel = FSLocalizedForKey(@"kColorModelVoiceover");
    _colorfulItem = colorfulItem;
    [bottomBar addItem:colorfulItem spacing:20 atPosition:FSItemPositionCenter];
    
    [colorfulItem setNeedsLayout];
    UIImageView* colorfulSelected = [[UIImageView alloc] init];
    [BindThemeKeySel(colorfulSelected, setImage:) fs_updateThemeCompleted:^(UIImageView*  _Nonnull target, SEL  _Nonnull sel) {
        [target setImage:[colorSelectedImg fs_imageWithTintColor:target.fs_primaryColor] ];
    }];
    [colorfulSelected setHidden:YES];
    [bottomBar addSubview:colorfulSelected];
    _colorfulSelected = colorfulSelected;
    [colorfulSelected mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(colorfulItem.mas_left).offset(offsetImgLength);
        make.top.equalTo(colorfulItem.mas_top).offset(offsetImgLength);
        make.size.mas_equalTo(colorSelectedImg.size);
    }];
    
    colorfulItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [self.imageProcessor setColorMode:LTColorModeRGB];
        [self.view fs_showHUDLoading:nil];
        [self.imageProcessor computePreview];
        
        [self.colorNoneSelected setHidden:YES];
        [self.colorfulSelected setHidden:NO];
        [self.colorWhiteBlackSelected setHidden:YES];
        [self.colorGraySelected setHidden:YES];
    };
    
    FSItem *colorWhiteBlackItem = [FSItem createItemWithImage:ImageNamed(@"scan_color_white_black") imageSelected:ImageNamed(@"scan_color_white_black") imageDisable:ImageNamed(@"scan_color_white_black")];
    colorWhiteBlackItem.accessibilityLabel = FSLocalizedForKey(@"kBackWhiteModelVoiceover");
    _colorWhiteBlackItem = colorWhiteBlackItem;
    [bottomBar addItem:colorWhiteBlackItem spacing:20 atPosition:FSItemPositionCenter];
    
    [colorWhiteBlackItem setNeedsLayout];
    UIImageView* colorWhiteBlackSelected = [[UIImageView alloc] init];
    [colorWhiteBlackSelected setHidden:YES];
    [BindThemeKeySel(colorWhiteBlackSelected, setImage:) fs_updateThemeCompleted:^(UIImageView*  _Nonnull target, SEL  _Nonnull sel) {
        [target setImage:[colorSelectedImg fs_imageWithTintColor:target.fs_primaryColor] ];
    }];
    [bottomBar addSubview:colorWhiteBlackSelected];
    _colorWhiteBlackSelected = colorWhiteBlackSelected;
    [colorWhiteBlackSelected mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(colorWhiteBlackItem.mas_left).offset(offsetImgLength);
        make.top.equalTo(colorWhiteBlackItem.mas_top).offset(offsetImgLength);
        make.size.mas_equalTo(colorSelectedImg.size);
    }];
    
    colorWhiteBlackItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [self.imageProcessor setColorMode:LTColorModeBitonal];
        [self.view fs_showHUDLoading:nil];
        [self.imageProcessor computePreview];
        
        [self.colorNoneSelected setHidden:YES];
        [self.colorfulSelected setHidden:YES];
        [self.colorWhiteBlackSelected setHidden:NO];
        [self.colorGraySelected setHidden:YES];
    };
    
    FSItem *colorGrayItem = [FSItem createItemWithImage:ImageNamed(@"scan_color_gray") imageSelected:ImageNamed(@"scan_color_gray") imageDisable:ImageNamed(@"scan_color_gray")];
    colorGrayItem.accessibilityLabel = FSLocalizedForKey(@"kGrayscaleModelVoiceover");
    _colorGrayItem = colorGrayItem;
    [bottomBar addItem:colorGrayItem spacing:20 atPosition:FSItemPositionCenter];
    
    [colorGrayItem setNeedsLayout];
    UIImageView* colorGraySelected = [[UIImageView alloc] init];
    [colorGraySelected setHidden:YES];
    [BindThemeKeySel(colorGraySelected, setImage:) fs_updateThemeCompleted:^(UIImageView*  _Nonnull target, SEL  _Nonnull sel) {
        [target setImage:[colorSelectedImg fs_imageWithTintColor:target.fs_primaryColor] ];
    }];
    [bottomBar addSubview:colorGraySelected];
    _colorGraySelected = colorGraySelected;
    [colorGraySelected mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(colorGrayItem.mas_left).offset(offsetImgLength);
        make.top.equalTo(colorGrayItem.mas_top).offset(offsetImgLength);
        make.size.mas_equalTo(colorSelectedImg.size);
    }];
    
    colorGrayItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [self.imageProcessor setColorMode:LTColorModeGrayscale];
        [self.view fs_showHUDLoading:nil];
        [self.imageProcessor computePreview];
        
        [self.colorNoneSelected setHidden:YES];
        [self.colorfulSelected setHidden:YES];
        [self.colorWhiteBlackSelected setHidden:YES];
        [self.colorGraySelected setHidden:NO];
    };
}

- (void)updateTopBarItems
{
    if (![self.colorBrightContrastBottomBar isHidden]){
        self.cancelBtn.hidden = YES;
        self.saveBtn.hidden = YES;
        self.cancelPropertyBtn.hidden = NO;
        self.doneBtn.hidden = NO;
    }else{
        self.cancelBtn.hidden = NO;
        self.saveBtn.hidden = NO;
        self.cancelPropertyBtn.hidden = YES;
        self.doneBtn.hidden = YES;
    }
    
    if (self.isChanged){
        [self.cancelBtn setTitle:FSLocalizedForKey(@"kCancel") forState:UIControlStateNormal];
        self.saveBtn.enabled = YES;
    }else{
        self.saveBtn.enabled = NO;
        [self.cancelBtn setTitle:FSLocalizedForKey(@"kBack") forState:UIControlStateNormal];
    }
}

- (void)setIsChanged:(BOOL)isChanged
{
    _isChanged = isChanged;
    [self updateTopBarItems];
}

- (UIColor *)bottomBarIconColor {
    return UIColorHex_DarkMode(#FFFFFF, #A2A2A2);
}

- (UIColor *)sliderBarColor {
    return UIColorHex(#E2E2E2);
}

- (LTSliderBar *)brightnessSliderBar
{
    if (_brightnessSliderBar == nil) {
        _brightnessSliderBar = [[LTSliderBar alloc] initWithSliderData:[self sliderBarColor] thumbTintColor:[self sliderBarColor] sliderValueBGImgName:@"scan_slider_value_bg"];
        _brightnessSliderBar.backgroundColor = [UIColor colorWithWhite:0.f alpha:0.4];
        _brightnessSliderBar.delegate = self;
    }
    return _brightnessSliderBar;
}

- (LTSliderBar *)constrastSliderBar
{
    if (_constrastSliderBar == nil) {
        _constrastSliderBar = [[LTSliderBar alloc] initWithSliderData:[self sliderBarColor] thumbTintColor:[self sliderBarColor] sliderValueBGImgName:@"scan_slider_value_bg"];
        _constrastSliderBar.backgroundColor = [UIColor colorWithWhite:0.f alpha:0.4];
        _constrastSliderBar.delegate = self;
    }
    return _constrastSliderBar;
}

- (UIView *)viewForZoomingInScrollView:(UIScrollView *)scrollView
{
    return self.contentView.editImageView;
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    [self.contentView refreshImage];
}

- (void)viewDidAppear:(BOOL)animated
{
}

- (BOOL)prefersStatusBarHidden
{
    return YES;
}

- (void)scrollViewWillBeginDragging:(UIScrollView *)scrollView
{
    
}

- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration
{
    NSLog(@"------will rotation-------");
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation
{
    NSLog(@"------did rotation-------");

    [self.contentView.editImageView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.center.equalTo(self.contentView);
        make.size.mas_equalTo(self.contentView.frame.size);
    }];
    
}

#pragma mark ------bottom btn click event-------

- (void)cancelBtnClick:(UIButton *)btn
{
    if ([self setColorBrightContrastBottomBarHidden])
    {
        self.contentView.editImage = _orignalImage;
        [self.contentView refreshImage];
        [self.imageProcessor setContrastLevel:_orignalContrastLevel];
        [self.imageProcessor setBrightnessLevel:_orignalBrightnessLevel];
        [self.imageProcessor setColorMode:_orignalColorMode];
        [self updateTopBarItems];
        return;
    }
    if (!self.isPresent) {
        [self.navigationController popViewControllerAnimated:YES];
    }else{
        [self dismissViewControllerAnimated:YES completion:nil];
    }
}

- (void)replaceBtnClick:(UIButton *)btn
{
    @weakify(self)
    NSString *mediaType = AVMediaTypeVideo;
    AVAuthorizationStatus authStatus = [AVCaptureDevice authorizationStatusForMediaType:mediaType];
    if (authStatus == AVAuthorizationStatusRestricted || authStatus == AVAuthorizationStatusDenied)
    {
        
        UIAlertController *alert = [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kNotAllowTakePhoto") message:FSLocalizedForKey(@"kCAMERADenied") actions:@[ FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kGoSetting") ] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
            if (index == 1){
                NSURL*url =[NSURL URLWithString:UIApplicationOpenSettingsURLString];
                if([[UIApplication sharedApplication] canOpenURL:url]) {
                    [[UIApplication sharedApplication] openURL:url options:@{} completionHandler:nil];
                }
            }
        }];
        [self presentViewController:alert animated:YES completion:nil];
        return;
    }else if (authStatus == AVAuthorizationStatusNotDetermined){
        [AVCaptureDevice requestAccessForMediaType:AVMediaTypeVideo completionHandler:^(BOOL granted) {
            dispatch_async(dispatch_get_main_queue(), ^{
                if (granted) {
                    LTCanmeraViewControllerNew* cameraView = [[LTCanmeraViewControllerNew alloc] init];
                    cameraView.replaceCallBack = ^(UIImage *replaceImage) {
                        @strongify(self)
                        self.isChanged = YES;
                        self.orignalImage = replaceImage;
                        self.contentView.editImage = replaceImage;
                        [self.contentView refreshImage];
                        [self resetImageProcessorWithImg:replaceImage];
                    };
                    //cameraView.documentSession = self->_documentSession;
                    cameraView.isOpenFromEditor = YES;
                    [self presentViewController:cameraView animated:YES completion:nil];
                }else{
                    UIAlertController *alert = [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kNotAllowTakePhoto") message:FSLocalizedForKey(@"kCAMERADenied") actions:@[ FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kGoSetting")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                        if (index == 1){
                            NSURL *url =[NSURL URLWithString:UIApplicationOpenSettingsURLString];
                            if([[UIApplication sharedApplication] canOpenURL:url]) {
                                [[UIApplication sharedApplication] openURL:url options:@{} completionHandler:nil];
                            }
                        }
                    }];
                    [self presentViewController:alert animated:YES completion:nil];
                    return;
                }
            });
        }];
    }else{
        LTCanmeraViewControllerNew* cameraView = [[LTCanmeraViewControllerNew alloc] init];
        cameraView.replaceCallBack = ^(UIImage *replaceImage) {
            @strongify(self)
            self.isChanged = YES;
            self.orignalImage = replaceImage;
            self.contentView.editImage = replaceImage;
            [self.contentView refreshImage];
            [self resetImageProcessorWithImg:replaceImage];
        };
        //cameraView.documentSession = self->_documentSession;
        cameraView.isOpenFromEditor = YES;
        cameraView.modalPresentationStyle = UIModalPresentationFullScreen;
        [self presentViewController:cameraView animated:YES completion:nil];
    }
}

- (void)rotateBtnClick:(UIButton *)btn
{
    [_imageProcessor rotateImage:LTRotationDirectionCCW];
    [self.view fs_showHUDLoading:nil];
    [_imageProcessor computePreview];
}

- (void)cropBtnClick:(UIButton *)btn
{
    if (!_isCroping) {
        _isCroping = YES;
        LTDetectionResult* result = _imageProcessor.detectionResult;
        NSMutableArray* points = [NSMutableArray arrayWithCapacity:4];
        for (NSValue* v in result.points)
        {
            CGPoint p;
            [v getValue:&p];
            
            p.x /= result.imageWidth;
            p.y /= result.imageHeight;
            p.x = MIN(MAX(0, p.x), 1);
            p.y = MIN(MAX(0, p.y), 1);
            [points addObject:[NSValue valueWithCGPoint:p]];
        }
        
        LTImageCropProcessViewController *cropVc = [[LTImageCropProcessViewController alloc] initWithImage:_imageProcessor.inputImage andPoints:points];
        cropVc.delegate = self;
        cropVc.modalPresentationStyle = UIModalPresentationFullScreen;
        [self presentViewController:cropVc animated:YES completion:nil];
    }
}

- (void)modelBtnClick:(UIButton *)btn
{
    [_bottomBar setHidden:YES];
    [_colorBrightContrastBottomBar setHidden:NO];
    [self setDefaultItemStateWithColorBarShow];
    [self updateTopBarItems];
}

- (void)saveBtnClick:(UIButton *)btn
{
    if ([self setColorBrightContrastBottomBarHidden]) {
        _orignalImage = self.contentView.editImage;
        [self saveLastTimeValueWithColorSetting];
        [self updateTopBarItems];
        return;
    }
    if (!_isChanged)
    {
        if (!self.isPresent) {
            [self.navigationController popViewControllerAnimated:YES];
            [self.delegate processImageView:self didFinishWithResult:nil];
        }else{
            [self dismissViewControllerAnimated:YES completion:^{
                [self.delegate processImageView:self didFinishWithResult:nil];
            }];
        }
        return;
    }
     [self.view fs_showHUDLoading:nil];
    [_imageProcessor computeResult];
}

- (void)cancelCrop
{
    if (_isCroping) {
        _isCroping = NO;
    }
}

- (void) cropViewChangedDetection:(NSArray*) points imageWidth:(int)width imageHeight:(int) height
{
    if (_isCroping) {
        _isCroping = NO;
        LTDetectionResult* result = [[LTDetectionResult alloc]
                                     initWithPoints:points
                                     imageWidth:width
                                     imageHeight:height];
        
        [_imageProcessor setDetectionResult:result];
         [self.view fs_showHUDLoading:nil];
        [_imageProcessor computePreview];
    }
}

- (BOOL)gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer shouldReceiveTouch:(UITouch *)touch
{
    return NO;
}

#pragma mark -imageProcess Delegate

- (void) imageProcessor:(LTImageProcessor*) processor didFinishComputingPreviewImage:(UIImage*) image
{
    [self.view fs_hideHUDLoading];
    self.isChanged = YES;
    self.contentView.editImage = image;
    [self.contentView refreshImage];
    if (_isCroping) {
        _isCroping = NO;
    }
}

- (void) imageProcessor:(LTImageProcessor*) processor didFinishComputingResultImage:(UIImage*) image
{
    [self.view fs_hideHUDLoading];
    if ([self.delegate respondsToSelector:@selector(processImageView:didFinishWithResult:)]) {
        NSData *imageDate = UIImagePNGRepresentation(image);
        image = nil;
       
        if (!self.isPresent) {
            [self.navigationController popViewControllerAnimated:YES];
            [self.delegate processImageView:self didFinishWithResult:[UIImage imageWithData:imageDate]];
        }else{
            [self dismissViewControllerAnimated:YES completion:^{
                 [self.delegate processImageView:self didFinishWithResult:[UIImage imageWithData:imageDate]];
            }];
        }
    }
}

- (BOOL)setColorBrightContrastBottomBarHidden
{
    if (![self.colorBrightContrastBottomBar isHidden])
    {
        [_saveBtn setTitle:FSLocalizedForKey(@"kSave") forState:UIControlStateNormal];
        [self.colorBrightContrastBottomBar setHidden:YES];
        [self.colorSettingBottomBar setHidden:YES];
        [self.brightnessSliderBar setHidden:YES];
        [self.constrastSliderBar setHidden:YES];
        [_bottomBar setHidden:NO];
        return YES;
    }
    return NO;
}

- (void)setDefaultItemStateWithColorBarShow
{
    self.changeColorItem.selected = NO;
    self.changeBrightnessItem.selected = NO;
    self.changeContrastItem.selected = NO;

    [self.brightnessSliderBar setSliderCurrentValue:_orignalBrightnessLevel];
    [self.constrastSliderBar setSliderCurrentValue:_orignalContrastLevel];
    LTColorMode colorMode = _orignalColorMode;
    switch (colorMode) {
        case LTColorModeRGB:
            self.colorNoneItem.selected = NO;
            self.colorfulItem.selected = YES;
            self.colorWhiteBlackItem.selected = NO;
            self.colorGrayItem.selected = NO;
            
            [self.colorNoneSelected setHidden:YES];
            [self.colorfulSelected setHidden:NO];
            [self.colorWhiteBlackSelected setHidden:YES];
            [self.colorGraySelected setHidden:YES];
            break;
        case LTColorModeRGBWhiteBalance:
            self.colorNoneItem.selected = YES;
            self.colorfulItem.selected = NO;
            self.colorWhiteBlackItem.selected = NO;
            self.colorGrayItem.selected = NO;
            
            [self.colorNoneSelected setHidden:NO];
            [self.colorfulSelected setHidden:YES];
            [self.colorWhiteBlackSelected setHidden:YES];
            [self.colorGraySelected setHidden:YES];
            break;
        case LTColorModeGrayscale:
            self.colorNoneItem.selected = NO;
            self.colorfulItem.selected = NO;
            self.colorWhiteBlackItem.selected = NO;
            self.colorGrayItem.selected = YES;
            
            [self.colorNoneSelected setHidden:YES];
            [self.colorfulSelected setHidden:YES];
            [self.colorWhiteBlackSelected setHidden:YES];
            [self.colorGraySelected setHidden:NO];
            break;
        case LTColorModeBitonal:
            self.colorNoneItem.selected = NO;
            self.colorfulItem.selected = NO;
            self.colorWhiteBlackItem.selected = YES;
            self.colorGrayItem.selected = NO;
            
            [self.colorNoneSelected setHidden:YES];
            [self.colorfulSelected setHidden:YES];
            [self.colorWhiteBlackSelected setHidden:NO];
            [self.colorGraySelected setHidden:YES];
            break;
        default:
            break;
    }
}

- (void)saveLastTimeValueWithColorSetting
{
    _orignalContrastLevel = _constrastSliderBar.currentValue;
    _orignalBrightnessLevel = _brightnessSliderBar.currentValue;
    _orignalColorMode = _imageProcessor.colorMode;
}

#pragma mark -- LTSliderBarDelegate
- (void)sliderValueChange:(LTSliderBar*)sliderBar value:(float)value
{
    if (sliderBar == self.brightnessSliderBar) {
        [_imageProcessor setBrightnessLevel:value];
    }
    else if (sliderBar == self.constrastSliderBar)
    {
        [_imageProcessor setContrastLevel:value];
    }
    [_imageProcessor computePreview];
}
@end
