//
//  LTSliderBar.m
//  FoxitPDFScanUI
//
//  Created by Wei Zheng on 2021/1/21.
//  Copyright © 2021 foxitcorporation. All rights reserved.
//

#import "LTSliderBar.h"

@interface LTSliderBar ()
@property (nonatomic, strong) UISlider *slider;
@property (nonatomic, strong) UIColor* maximumTrackTintColor;
@property (nonatomic, strong) UIColor* thumbTintColor;
@property (nonatomic, strong) NSString* sliderValueBGImgName;
@property (nonatomic, strong) UIView *sliderCoverView;
@property (nonatomic, strong) UIView *sliderMiddleDotView;
@property (nonatomic, strong) UIButton *sliderValueView;
@property (nonatomic, assign) float minimumValueForShow;
@property (nonatomic, assign) float maximumValueForShow;
@property (nonatomic, assign) BOOL isFirstSettingLeft;
@property (nonatomic, assign) BOOL isFirstSettingRight;
@property (nonatomic, assign) BOOL isInit;
@end

@implementation LTSliderBar

/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
    // Drawing code
}
*/

- (instancetype)initWithSliderData:(UIColor*)maximumTrackTintColor thumbTintColor:(UIColor*)thumbTintColor sliderValueBGImgName:(NSString*)sliderValueBGImgName
{
    self = [super init];
    if (self) {
        _maximumTrackTintColor = maximumTrackTintColor;
        _thumbTintColor = thumbTintColor;
        _sliderValueBGImgName = sliderValueBGImgName;
        _isFirstSettingLeft = YES;
        _isFirstSettingRight = YES;
        [self initView];
    }
    return self;
}

- (void)setSliderValue:(float)value minimumValue:(float)minimumValue maximumValue:(float)maximumValue minimumValueForShow:(float)minimumValueForShow maximumValueForShow:(float)maximumValueForShow
{
    self.slider.minimumValue = minimumValue;
    self.slider.maximumValue = maximumValue;
    [self.slider setValue:value animated:NO];
    self.minimumValueForShow = minimumValueForShow;
    self.maximumValueForShow = maximumValueForShow;
    _isInit = YES;
    [self changeCurrentProgress:self.slider];
    _isInit = NO;
}

- (void)initView
{
    [self addSubview:self.slider];
    [self.slider mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.slider.superview);
        make.centerX.equalTo(self.slider.superview);
        make.width.mas_equalTo(300);
        make.height.offset(12);
    }];
    [self.slider layoutIfNeeded];
    
    [self addSubview:self.sliderCoverView];
    
    [self addSubview:self.sliderValueView];
    
    [self.slider addSubview:self.sliderMiddleDotView];
    [self.sliderMiddleDotView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.mas_equalTo(self.slider.mas_centerX);
        make.centerY.mas_equalTo(self.slider.mas_centerY).offset(1);
        make.size.mas_equalTo(self.sliderMiddleDotView.frame.size);
    }];
}

- (UIButton *)sliderValueView
{
    if (_sliderValueView == nil) {
        UIImage* img = ImageNamed(_sliderValueBGImgName);
        _sliderValueView = [[UIButton alloc] initWithFrame:CGRectMake(0, 0, img.size.width, img.size.height)];
        [_sliderValueView setBackgroundImage:img forState:UIControlStateNormal];
        [_sliderValueView setTitleColor:self.fs_backgroundColor_B5 forState:UIControlStateNormal]; // self.fs_textColor_T4
        _sliderValueView.titleLabel.font = self.fs_standardWeakFont_WF2;
        _sliderValueView.titleLabel.numberOfLines = 0;
    }
    return _sliderValueView;
}

- (UIView *)sliderCoverView
{
    if (_sliderCoverView == nil) {
        _sliderCoverView = [[UIView alloc] init];
        _sliderCoverView.backgroundColor = _maximumTrackTintColor;
    }
    return _sliderCoverView;
}

- (UIView *)sliderMiddleDotView
{
    if (_sliderMiddleDotView == nil) {
        CGFloat width = 6;
        CGFloat height = 6;
        _sliderMiddleDotView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, width, height)];
        _sliderMiddleDotView.layer.cornerRadius = width / 2;
        [_sliderMiddleDotView setBackgroundColor:self.fs_dividerColor_P2];
    }
    return _sliderMiddleDotView;
}

- (UISlider *)slider
{
    if (_slider == nil) {
        _slider = [[UISlider alloc] init];
        
        _slider.continuous = YES;
        _slider.thumbTintColor = _thumbTintColor;
        [_slider addTarget:self action:@selector(changeCurrentProgress:)
         forControlEvents:UIControlEventValueChanged];
    }
    return _slider;
}

-(void)changeCurrentProgress:(UISlider *)slider{
    CGRect trackRect = [slider trackRectForBounds:slider.bounds];
    CGFloat trackHeight = trackRect.size.height;
    CGRect thumbRect = [slider thumbRectForBounds:slider.bounds trackRect:trackRect value:slider.value];
    [_sliderMiddleDotView setHidden:CGRectContainsRect(thumbRect, _sliderMiddleDotView.frame) ? YES :NO];
    [self.sliderValueView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.centerX.mas_equalTo(thumbRect.origin.x - slider.frame.size.width/2 + thumbRect.size.width/2);
        make.bottom.equalTo(slider.mas_top).offset(-8);
        make.size.mas_equalTo(self.sliderValueView.frame.size);
    }];
    if (slider.value < (slider.minimumValue+slider.maximumValue)/2) {
        if (slider.maximumTrackTintColor == _maximumTrackTintColor) {
            slider.minimumTrackTintColor = _maximumTrackTintColor;
          
            [BindThemeKeySel(slider, setMaximumTrackTintColor:) fs_updateThemeCompleted:^(UISlider*  _Nonnull target, SEL  _Nonnull sel) {
                [target setMaximumTrackTintColor:target.fs_primaryColor ];
            }];
           
            [self.sliderCoverView mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.left.equalTo(slider.mas_centerX);
                make.right.equalTo(slider.mas_right).offset(-1);
                make.centerY.equalTo(slider).offset(1);
                make.height.offset(trackHeight);
            }];
            
        }
        else
        {
            if (_isFirstSettingLeft) {
                [self setNeedsDisplay];
                _isFirstSettingLeft = NO;
                slider.minimumTrackTintColor = _maximumTrackTintColor;
                
                [BindThemeKeySel(slider, setMaximumTrackTintColor:) fs_updateThemeCompleted:^(UISlider*  _Nonnull target, SEL  _Nonnull sel) {
                    [target setMaximumTrackTintColor:target.fs_primaryColor ];
                }];

                [self.sliderCoverView mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.left.equalTo(slider.mas_centerX);
                    make.right.equalTo(slider.mas_right).offset(-1);
                    make.centerY.equalTo(slider).offset(1);
                    make.height.offset(trackHeight);
                }];
            }
        }
    }
    else
    {
        if (slider.minimumTrackTintColor == _maximumTrackTintColor) {
            [BindThemeKeySel(slider, setMinimumTrackTintColor:) fs_updateThemeCompleted:^(UISlider*  _Nonnull target, SEL  _Nonnull sel) {
                [target setMinimumTrackTintColor:target.fs_primaryColor ];
            }];
            slider.maximumTrackTintColor = _maximumTrackTintColor;
            [self.sliderCoverView mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.left.equalTo(slider.mas_left).offset(1);
                make.right.equalTo(slider.mas_centerX);
                make.centerY.equalTo(slider).offset(1);
                make.height.offset(trackHeight);
            }];
        }
        else
        {
            if (_isFirstSettingRight) {
                _isFirstSettingRight = NO;
                [self setNeedsDisplay];
                [BindThemeKeySel(slider, setMinimumTrackTintColor:) fs_updateThemeCompleted:^(UISlider*  _Nonnull target, SEL  _Nonnull sel) {
                    [target setMinimumTrackTintColor:target.fs_primaryColor ];
                }];
                slider.maximumTrackTintColor = _maximumTrackTintColor;
                [self.sliderCoverView mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.left.equalTo(slider.mas_left).offset(1);
                    make.right.equalTo(slider.mas_centerX);
                    make.centerY.equalTo(slider).offset(1);
                    make.height.offset(trackHeight);
                }];
            }
        }
    }
    
    //show value
    float midValue = (self.maximumValueForShow + self.minimumValueForShow)/2;
    float num = (self.maximumValueForShow - self.minimumValueForShow) / (slider.maximumValue-slider.minimumValue);
    
    float valueForShow = midValue + slider.value * num;
    NSString *strValueForShow = [NSString stringWithFormat:@"%0.2f\r",valueForShow];
    [_sliderValueView setTitle:strValueForShow forState:UIControlStateNormal];
    
    if (!_isInit && [self.delegate respondsToSelector:@selector(sliderValueChange:value:)]) {
        [self.delegate sliderValueChange:self value:self.slider.value];
    }
}

- (void)setSliderCurrentValue:(float)value
{
    [self.slider setValue:value animated:NO];
    _isInit = YES;
    [self changeCurrentProgress:self.slider];
    _isInit = NO;
}
- (float)currentValue
{
    return self.slider.value;
}

@end
