/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/
#import <Foundation/Foundation.h>
#import <FoxitRDK/FSPDFObjC.h>
#import <uiextensionsDynamic/FSMainToolbar.h>
#import "PDFScanToolbarManager.h"
#import "PDFScanMenuViewManager.h"

NS_ASSUME_NONNULL_BEGIN

typedef NS_ENUM(NSUInteger, FSScanCameraControllerItemType) {
    ///< The Flash of light item in the top bar of camera viewcontroller.
    FSScanCameraControllerItemTypeFlashLight,
    ///< The Single shooting item in the top bar of camera viewcontroller.
    FSScanCameraControllerItemTypeSingleShooting,
    ///< The photos item in the top bar of camera viewcontroller.
    FSScanCameraControllerItemTypePhotos,
    ///< The auto dection item in the top bar of camera viewcontroller.
    FSScanCameraControllerItemTypeAutoDection,
    ///< The format pagesize item in the top bar of camera viewcontroller.
    FSScanCameraControllerItemTypeFormatPageSize,
};

typedef void (^ScanPDFSaveAsCallBack) (NSError * _Nullable error, NSString * _Nullable savePath);
typedef void (^ScanPDFDoneCallBack) (NSError * _Nullable error, NSString * _Nullable savePath);
typedef void (^ScanPDFSessionPageCountChangeCallBack)(int pageCount);

@interface PDFScanManager : NSObject

/** @brief Singleton mode of PDFScanManager.
 *
 *  @details Initialize the PDFScanManager module  return the object of PDFScanManager.
 
 *  @return The object of PDFScanManager.
*/
+ (PDFScanManager *)shareManager;

+ (instancetype)new NS_UNAVAILABLE;
- (instancetype)init NS_UNAVAILABLE;

/** @brief Callback after the file is saved.*/
@property (class, nonatomic, copy) ScanPDFSaveAsCallBack saveAsCallBack;

/** @brief Callback after user click 'done' button in the camera viewcontroller of PDFScan. The callback will return pdffilePath by saveasPd.f*/
@property (class, nonatomic, copy) ScanPDFDoneCallBack doneCallBack;

@property (class, nonatomic, copy) ScanPDFSessionPageCountChangeCallBack pageCountCallBack;
/** @brief User custom toolbar manager .*/
@property (nonatomic, strong, readonly) PDFScanToolbarManager* toolbarManager;

/** @brief User custom view manager.*/
@property (nonatomic, strong, readonly) PDFScanMenuViewManager* menuViewManager;

/** @brief Initialize the FoxitMobileRDKScanning module with additional parameters.
 *
 *  @details Successful initialization of the SDK requires a valid serial number.
 *
 *  @param[in]    serial1        First part of the serial number.
 *  @param[in]    serial2        Second part of the serial number.
 *  @return       FSErrSuccess when the serial number is valid and initialization is successful.
 *               FSErrInvalidLicense when the serial number is invalidand initialization fails.
 *               FSErrRightsExpired when serial number is valid but expired.
*/
+ (FSErrorCode)initializeScanner:(unsigned long)serial1 serial2:(unsigned long)serial2;
/** @brief Initialize the FoxitMobileRDKScanning Compression module with additional parameters.
 *
 *  @details Successful initialization of the SDK requires a valid serial number.
 *
 *  @param[in]    serial1        First part of the serial number.
 *  @param[in]    serial2        Second part of the serial number.
 *  @return       FSErrSuccess when the serial number is valid and initialization is successful.
 *               FSErrInvalidLicense when the serial number is invalidand initialization fails.
 *               FSErrRightsExpired when serial number is valid but expired.
*/
+ (FSErrorCode)initializeCompression:(unsigned long)serial1 serial2:(unsigned long)serial2;
/** @brief      Get the PDFScan controller.
 *
 *  @details      Scan module start interface.
 *  @return       return the pdcscan viewcontroller.
*/
- (UIViewController*)getPDFScanView;
/** @brief      Set the item of photos hidden  in the camera viewcontroller of PDFScan.
 *
 *  @details      hidden is YES otherwise is NO, default is NO.
 *  @param[in]    hidden        hidden  is  YES otherwise is NO.
*/
- (void)setItemHiddenWithType:(FSScanCameraControllerItemType)itemType hidden:(BOOL)hidden;
/** @brief      Save as PDF with file path.
 *
 *  @details      Save as PDF with file path, after saving will callback the function of ScanPDFSaveAsCallBack.
 *  @param[in]    pdfFilePath        The pdf file path with saveas.
*/
- (void)saveAsPDF:(NSString*)pdfFilePath;
@end

NS_ASSUME_NONNULL_END
