/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

@class FSReadToolbarItem;
/**@brief The item is in the bar*/
typedef NS_ENUM(NSInteger, FSMainToolbarItemPosition) {
    ///The item is on the left
    FSMainToolbarItemPositionLeft = 0,
    ///The item is on the center
    FSMainToolbarItemPositionCenter,
    ///The item is on the right
    FSMainToolbarItemPositionRight,
};
/**@brief The tool tag*/
typedef NS_ENUM(NSUInteger, FSMainTopbarToolTag) {
    ///The home tag
    FSMainTopbarToolTagHome,
    ///The edit tag
    FSMainTopbarToolTagEdit,
    ///The comment tag
    FSMainTopbarToolTagComment,
    ///The drawing tag
    FSMainTopbarToolTagDrawing,
    ///The view tag
    FSMainTopbarToolTagView,
    ///The form tag
    FSMainTopbarToolTagForm,
    ///The sign tag
    FSMainTopbarToolTagSign,
    ///The protect tag
    FSMainTopbarToolTagProtect,
    ///The custom tag
    FSMainTopbarToolTagCustom,
};

/**@brief Defines existing-supplied images  for main tool bar items..*/
typedef NS_ENUM(NSUInteger, FSMainToolbarItemType) {
    ///The unknown item
    FSMainToolbarItemTypeUnknown = 0,
    ///The bark item
    FSMainToolbarItemTypeBack,
    ///The more item
    FSMainToolbarItemTypeMore,
    ///The search item
    FSMainToolbarItemTypeSearch,
    ///The pan item
    FSMainToolbarItemTypePan,
    ///The add tool item
    FSMainToolbarItemTypeAddTool,
    ///The redo item
    FSMainToolbarItemTypeRedo,
    ///The undo item
    FSMainToolbarItemTypeUndo,
    ///The property item
    FSMainToolbarItemTypeProperty,
    ///The multi-select item
    FSMainToolbarItemTypeMultiSelect,
    ///The divide item
    FSMainToolbarItemTypeDivide,
    
    ///The panel item
    FSMainToolbarItemTypePanel,
    ///The view settings item
    FSMainToolbarItemTypeViewSettings,
    ///The Thumbnai item
    FSMainToolbarItemTypeThumbnail,
    ///The bookmark item
    FSMainToolbarItemTypeReadingBookmark,
    
    ///The sort tool item
    FSMainToolbarItemTypeSortTool,
};

/**@brief The subbar is in the bar*/
typedef NS_ENUM(NSUInteger, FSTopbarSubitemContentViewPosition) {
    ///The subbar is on the top
    FSTopbarSubitemContentViewPositionTop,
    ///The subbar is on the left
    FSTopbarSubitemContentViewPositionLeft,
    ///The subbar is on the right
    FSTopbarSubitemContentViewPositionRight,
};

@interface FSMainToolbarItem : NSObject
/**@brief Disable the default initialization.*/
+ (instancetype)new NS_UNAVAILABLE;
- (instancetype)init NS_UNAVAILABLE;

/**@brief Get the  existing-supplied item. Please refer to {@link FSMainToolbarItemType FSMainToolbarItemTypeXXX} values and it would be one of these values..*/
@property (nonatomic, assign, readonly) FSMainToolbarItemType itemType;
/**@brief Get/Set the  add spacing to the toolbar.*/
@property (nonatomic, assign) CGFloat spacing;
/**@brief Get/Set the  add  width to the toolbar.*/
@property (nonatomic, assign) CGFloat width;
/**@brief Gets a custom view.*/
@property (nonatomic, strong, readonly, nullable) UIView *customView;

/** @brief Initialization method
 *
 *  @param[in]    customView         The custom view .
 *  @return       The main toolbar Item instance.
 */
- (instancetype)initWithCustomView:(UIView *)customView;
@end

@interface FSMainToolbar : NSObject
/**@brief Disable the default initialization.*/
+ (instancetype)new NS_UNAVAILABLE;
- (instancetype)init NS_UNAVAILABLE;

/**@brief Get/Set the items displayed on the toolbar left.*/
@property (nonatomic, copy, nullable) NSArray<FSMainToolbarItem *> *itemsForPositionLeft;
/**@brief Get/Set the items displayed on the toolbar center.*/
@property (nonatomic, copy, nullable) NSArray<FSMainToolbarItem *> *itemsForPositionCenter;
/**@brief Get/Set the items displayed on the toolbar right.*/
@property (nonatomic, copy, nullable) NSArray<FSMainToolbarItem *> *itemsForPositionRight;
/**@brief Set content bar height*/
@property (nonatomic, assign) CGFloat contentHeight;

/**@brief Get the  the toolbar content view.
 *
 *  @return  The toolbar content view.    It's a subclass of UIStackView,  the axis of the default is UILayoutConstraintAxisVertical
 */
- (__kindof UIStackView *)getToolbarContentView;

/**@brief Add item at position, Please refer to {@linkFSMainToolbarItemPositionXXX} and {@link FSMainToolbarItemPosition} values and it would be one of these values..*/
- (void)addItem:(FSMainToolbarItem *)item atPosition:(FSMainToolbarItemPosition)position;
/**@brief Removes the index item at position, Please refer to {@linkFSMainToolbarItemPositionXXX} and {@link FSMainToolbarItemPosition} values and it would be one of these values..*/
- (void)removeItemAtPosition:(FSMainToolbarItemPosition)position index:(NSUInteger)index;
/**@brief Removes  item at position, Please refer to {@linkFSMainToolbarItemPositionXXX} and {@link FSMainToolbarItemPosition} values and it would be one of these values..*/
- (void)removeItem:(FSMainToolbarItem *)item atPosition:(FSMainToolbarItemPosition)position;
/**@brief Set the background color.*/
- (void)setBackgroundColor:(UIColor *)color;
@end

@interface FSMainTopbarToolTagItem : NSObject
/**@brief Disable the default initialization.*/
+ (instancetype)new NS_UNAVAILABLE;
- (instancetype)init NS_UNAVAILABLE;

/**@brief Get/Set the item title.*/
@property (nonatomic, copy) NSString *title;
/**@brief Get/Set the item image.*/
@property (nonatomic, strong) UIImage *image;

/**@brief Default iphone UIEdgeInsets*/
@property (nonatomic) UIEdgeInsets phoneTitleEdgeInsets;
@property (nonatomic) UIEdgeInsets phoneImageEdgeInsets;
/**@brief Default ipad UIEdgeInsets*/
@property (nonatomic) UIEdgeInsets padTitleEdgeInsets;
@property (nonatomic) UIEdgeInsets padImageEdgeInsets;

/**@brief Get/Set the read Items displayed on the toolbar.*/
@property (nonatomic, copy, nullable) NSArray<FSReadToolbarItem *> *readToolbarItems;
/**@brief Get/Set the main Items displayed on the toolbar.*/
@property (nonatomic, copy, nullable) NSArray<FSMainToolbarItem *> *mainToolbarItems;

/**@brief Get the  existing-supplied tool tag. Please refer to {@link FSMainTopbarToolTag FSMainTopbarToolTagXXX} values and it would be one of these values..*/
@property (nonatomic, assign, readonly) FSMainTopbarToolTag toolTag;
/**@brief Whether to display on the toolbar.*/
@property (nonatomic, assign, readonly) BOOL isOnToolbar;

/** @brief Initialization method
 *
 *  @param[in]    toolTag         The tool tag.
 *
 *  @param[in]    readToolbarItems        The read Items displayed on the toolbar
 *
 *  @return       The main topbar tool tag Item.
 */
+ (instancetype)ItemWithToolTag:(FSMainTopbarToolTag)toolTag readToolbarItems:(nullable NSArray<FSReadToolbarItem *> *)readToolbarItems;
@end

@protocol IFSMainTopbarEventListener <NSObject>

@optional
/**
 * @brief Did show tag tab popuper.
 */
- (void)didShowTagTabPopuper;
/**
 * @brief Did dismiss tag tab popuper.
 */
- (void)didDismissTagTabPopuper;
/** @brief Triggered when the tag tab is changed.
 *
 * @param[in]    curTagItem    Current tag item.
 *
 * @param[in]    lastTagItem   Last tag item.
 */
- (void)didSetCurrentTagItem:(FSMainTopbarToolTagItem *)curTagItem lastTagItem:(FSMainTopbarToolTagItem *)lastTagItem;
/** @brief Triggered when the position of the topbar`s` subbar changes.
 *
 * @param[in]    position    The position of subbar.
 *
 */
- (void)onSubitemViewChanged:(FSTopbarSubitemContentViewPosition)position;
@end

@interface FSMainTopbar : FSMainToolbar
/**@brief Disable this method.*/
@property (nonatomic, copy, nullable) NSArray<FSMainToolbarItem *> *itemsForPositionCenter NS_UNAVAILABLE;
/**@brief Get/Set the main tag Items displayed on the toolbar.*/
@property (nonatomic, copy, nullable) NSArray<FSMainTopbarToolTagItem *> *tagItems;
/**@brief Get/Set the main tag Items to display a new icon in the popup window.*/
@property (nonatomic, copy, nullable) NSArray<FSMainTopbarToolTagItem *> *currentNewTagItems;
/**@brief Get the current tag Item */
@property (nonatomic, strong, readonly) FSMainTopbarToolTagItem *currentTagItem;
/**@brief Get the subtoolbar position. Please refer to {@link FSTopbarSubitemContentViewPosition FSTopbarSubitemContentViewPositionXXX} values and it would be one of these values..*/
@property (nonatomic, assign) FSTopbarSubitemContentViewPosition subitemContentPosition;
/**@brief Set horizontal content height*/
@property (nonatomic, assign) CGFloat horizontalContentHeight;
/**@brief Set vertical content width*/
@property (nonatomic, assign) CGFloat verticalContentWidth;
/**@brief Get/Set the subview to contview.*/
@property (nonatomic, strong, nullable) __kindof UIView *attachView;
/**@brief Reset the tag Item in order to update on the view.Calling this method causes the current toolbar to be updated immediately*/
- (void)resetCurrentTagItem;

/** @brief Set the color of the state bar.
 *
 *  @param[in]    color         The background color used to specify the status.
 */
- (void)setStateBarColor:(nullable UIColor *)color;

/** @brief Set the background color of the tagItem.
 *
 *  @param[in]    color         The background color used to specify the status.
 */
- (void)setTagItemBackgroudColor:(nullable UIColor *)color;

/**
 * @brief Register an event listener.
 *
 * @param[in]    listener    The Topbar  Listener.
 *
 */
- (void)registerEventListener:(id<IFSMainTopbarEventListener>)listener;

/**
 * @brief Unregister  an event listener.
 *
 * @param[in]    listener    The Topbar Listener.
 *
 */
- (void)unregisterEventListener:(id<IFSMainTopbarEventListener>)listener;
@end

@interface FSMainBottombar : FSMainToolbar
/**@brief Disable these methods.*/
@property (nonatomic, copy, nullable) NSArray<FSMainToolbarItem *> *itemsForPositionLeft NS_UNAVAILABLE;
@property (nonatomic, copy, nullable) NSArray<FSMainToolbarItem *> *itemsForPositionRight NS_UNAVAILABLE;

/**@brief Whether to evenly distribute items across the toolbar.*/
@property (nonatomic, assign, getter=isAverage) BOOL average;
@end

NS_ASSUME_NONNULL_END
