/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSCollectionViewCell.h"

@interface FSCollectionViewCell ()
@property (nonatomic, strong) FSItem *editItem;

@end
@implementation FSCollectionViewCell

- (instancetype)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:frame];
    if (self) {
    }
    return self;
}

- (void)prepareForReuse {
    [super prepareForReuse];
    self.customEditing = NO;
}



- (void)setCustomEditing:(BOOL)editing{
    if (_customEditing == editing) {
        return;
    }
    _customEditing = editing;
    if (editing) {
        [self.contentView addSubview:self.editItem];
        [_editItem mas_makeConstraints:^(MASConstraintMaker *make) {
            switch (self.checkPosition) {
                case FSCollectionCellCheckItemPositionTopLeft:
                case FSCollectionCellCheckItemPositionTopRight:
                    make.right.top.fs_equalToSuperview();
                    break;
                default:
                    make.left.top.fs_equalToSuperview();
                    break;
            }
        }];
    }else{
        _editItem.selected = NO;
        [_editItem removeFromSuperview];
    }
}

- (void)setSelected:(BOOL)selected{
    [super setSelected:selected];
    if (_customEditing) {
        _editItem.selected = selected;
        if (_editItem.highlighted) {
            _editItem.highlighted = NO;
        }
    }
}

- (void)didClicked:(FSItem *)item{
    item.selected = !item.selected;
    [self setSelected:item.selected];
}

- (FSItem *)editItem{
    if(!_editItem){
        _editItem = [FSItem new];
        UIBezierPath *path = [UIBezierPath bezierPathWithRoundedRect:CGRectMake(9, 9, 24, 24) cornerRadius:12];
        CAShapeLayer *shapeLayer = [CAShapeLayer layer];
        shapeLayer.path = path.CGPath;
        _editItem.layer.mask = shapeLayer;
        [_editItem setImageEdgeInsets:UIEdgeInsetsMake(9, 9, 9, 9)];
        UIColor *color = UIColorHex_DarkMode(#FFFFFF, #36404A);
        [_editItem setBackgroundImage:[UIImage fs_imageWithColor:color size:CGSizeMake(24, 24)] forState:UIControlStateNormal];
        [_editItem addTarget:self action:@selector(didClicked:) forControlEvents:UIControlEventTouchUpInside];
    }
    return _editItem;
}

@end
