/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/

#import <UIKit/UIKit.h>

NS_ASSUME_NONNULL_BEGIN

///The relative position of the title and picture
typedef NS_ENUM(NSInteger, FSImageTextRelation){
    FSImageTextRelationTop,    //title top
    FSImageTextRelationLeft,   //title left
    FSImageTextRelationRight,  //title right
    FSImageTextRelationBottom, //title bottom
};

@interface FSItem : UIControl<NSCopying>

typedef void (^InteractionCallBack)(FSItem *_item);

+ (FSItem *)createHeadlineItemWithTitle:(nullable NSString *)title;

+ (FSItem *)createItemWithTitle:(nullable NSString *)title;

+ (FSItem *)createItemWithImage:(nullable UIImage *)image;

+ (FSItem *)createItemWithImage:(nullable UIImage *)image background:(nullable UIImage *)background;

+ (FSItem *)createItemWithTitle:(nullable NSString *)title image:(nullable UIImage *)image imageTextRelation:(FSImageTextRelation)imageTextRelation;

+ (FSItem *)createItemWithImage:(nullable UIImage *)imageNormal imageSelected:(nullable UIImage *)imageSelected;

+ (FSItem *)createItemWithImage:(nullable UIImage *)imageNormal
                      imageSelected:(nullable UIImage *)imageSelected
                       imageDisable:(nullable UIImage *)imageDisabled;

+ (FSItem *)createItemWithImage:(nullable UIImage *)imageNormal
                  imageSelected:(nullable UIImage *)imageSelected
                   imageDisable:(nullable UIImage *)imageDisabled
                     background:(nullable UIImage *)background;

+ (FSItem *)createItemWithTitle:(nullable NSString *)title
                                imageNormal:(nullable UIImage *)imageNormal
                              imageSelected:(nullable UIImage *)imageSelected
                               imageDisable:(nullable UIImage *)imageDisabled
                          imageTextRelation:(FSImageTextRelation)imageTextRelation;

+ (FSItem *)createItemWithTitle:(nullable NSString *)title
                                imageNormal:(nullable UIImage *)imageNormal
                              imageSelected:(nullable UIImage *)imageSelected
                               imageDisable:(nullable UIImage *)imageDisabled
                                 background:(nullable UIImage *)background
                          imageTextRelation:(FSImageTextRelation)imageTextRelation;

+ (FSItem *)createItemWithTitle:(nullable NSString *)title
                                imageNormal:(nullable UIImage *)imageNormal
                          imageHighlighted:(nullable UIImage *)imageHighlighted
                              imageSelected:(nullable UIImage *)imageSelected
                               imageDisable:(nullable UIImage *)imageDisabled
                                 background:(nullable UIImage *)background
                          imageTextRelation:(FSImageTextRelation)imageTextRelation;

- (instancetype)init; //relation default is FSImageTextRelationTop

- (instancetype)initWithFrame:(CGRect)frame relation:(FSImageTextRelation)relation;
- (instancetype)initWithRrelation:(FSImageTextRelation)relation;

@property (class, nonatomic, strong)  UIColor *appearanceDisableTitleColor; // default is nil, the transparency of the color used in normal mode will be LABEL_DISABLED_ALPHA

@property (nonatomic)          UIEdgeInsets titleEdgeInsets;                // default is UIEdgeInsetsZero engrave and emboss appearance
@property (nonatomic)          UIEdgeInsets imageEdgeInsets;                // default is UIEdgeInsetsZero
@property (nullable, nonatomic, readonly, strong) NSString *currentTitle;   // normal/highlighted/selected/disabled. can return nil
@property (nonatomic, readonly, strong) UIColor  *currentTitleColor;        // normal/highlighted/selected/disabled. always returns non-nil. default is white(1,1)
@property (nullable, nonatomic, readonly, strong) UIImage  *currentImage;   // normal/highlighted/selected/disabled. can return nil
@property (nullable, nonatomic, readonly, strong) UIImage  *currentBackgroundImage;   // normal/highlighted/selected/disabled. can return nil
@property (nullable, nonatomic, readonly, strong) NSAttributedString *currentAttributedTitle;  // normal/highlighted/selected/disabled. can return nil

@property (null_resettable, nonatomic,strong) UIFont      *font; // default is nil (system font 17 plain)
// return title and image views. will always create them if necessary.
@property (nullable, nonatomic, readonly, strong) UILabel     *titleLabel;
@property (nullable, nonatomic, readonly, strong) UIImageView *imageView;

@property (nonatomic)          UIAccessibilityTraits customAccessibilityTraits;
/* default is superview.
- (id)owner{
    if (_owner) return _owner;
    return self.superview;
}*/
@property (nullable, nonatomic, weak) id owner;

@property (nonatomic, copy, nullable) InteractionCallBack onTapClick;
@property (nonatomic, copy, nullable) InteractionCallBack onLongPress;

// UIControlStateNormal | UIControlStateHighlighted | UIControlStateDisabled | UIControlStateSelected
UIKIT_EXTERN const UIControlState FSItemControlStateAll;
- (void)setTitle:(nullable NSString *)title forState:(UIControlState)state;
- (void)setTitleColor:(nullable UIColor *)color forState:(UIControlState)state; // default is nil. use opaque white
- (void)setBackgroudViewColor:(nullable UIColor *)color forState:(UIControlState)state; // default is clearColor.
- (void)setImage:(nullable UIImage *)image forState:(UIControlState)state; // default is nil. should be same size if different for different states
- (void)setBackgroundImage:(nullable UIImage *)image forState:(UIControlState)state; // default is nil
- (void)setImageViewBackgroundImage:(nullable UIImage *)image forState:(UIControlState)state; // default is nil
- (void)setAttributedTitle:(NSAttributedString *)title forState:(UIControlState)state; // default is nil. title is assumed to
//
- (void)rebuildImageTextRelation:(FSImageTextRelation)relation;
@end

NS_ASSUME_NONNULL_END
