/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/

#import "FSConst.h"
#import "FSMacro.h"
#import "FSPreferences.h"
#import "UIImage+FSAdd.h"
#import "Masonry+FSAdd.h"
#import "UIColor+FSAdd.h"
#import "FSItem.h"

@interface FSItem ()
{
    NSString *_accessibilityLabel;
}
@property (nonatomic, assign) BOOL currentTouch;
@property (nonatomic, assign) BOOL touchNoChange;
@property (nonatomic, assign) FSImageTextRelation relation;
@property (nonatomic, strong) NSMutableDictionary *stateConfig;
@property (nonatomic, assign) BOOL backgroundImage;
@property (nonatomic, strong) UIImageView *backgroundImageView;
@property (nonatomic, assign) BOOL imageViewBackgroundImage;
@property (nonatomic, strong) UIImageView *imageViewBackgroundImageView;
@property (nonatomic, assign) BOOL backgroundViewColor;
@property (nonatomic, strong) UIView *backgroundView;
@property (nullable, nonatomic, readwrite, strong) UILabel     *titleLabel;
@property (nullable, nonatomic, readwrite, strong) UIImageView *imageView;
@property (nullable, nonatomic, copy) NSString *currentRecordKey;
@end

@interface FSItemAttributes : NSObject<NSCopying>

@property (nonatomic, copy) NSString *titleLabelTitle;
@property (nonatomic, strong) UIColor *titleLabelTitleColor;
@property (nonatomic, copy) NSAttributedString *titleLabelAttributedText;
@property (nonatomic, strong) UIImage *imageViewImage;
@property (nonatomic, strong) UIImage *backgroundImageViewImage;
@property (nonatomic, strong) UIImage *imageViewBackgroundImageViewImage;
@property (nonatomic, strong) UIColor *backgroundViewColor;
@property (nonatomic, assign) FSImageTextRelation imageTextRelation;
@end

@implementation FSItem

#define TitleLabelHeightGreaterThanImageViewHeight (_titleLabel.intrinsicContentSize.height + _titleEdgeInsets.top + _titleEdgeInsets.bottom ) > (_imageView.intrinsicContentSize.height + _imageEdgeInsets.top + _imageEdgeInsets.bottom)

#define TitleLabelWidthGreaterThanImageViewWidth (_titleLabel.intrinsicContentSize.width + _titleEdgeInsets.left + _titleEdgeInsets.right ) > (_imageView.intrinsicContentSize.width + _imageEdgeInsets.left + _imageEdgeInsets.right)

+ (FSItem *)createHeadlineItemWithTitle:(nullable NSString *)title{
    FSItem *titleItem = [self createItemWithTitle:title
                                       background:nil];
    [titleItem setFont:self.fs_standardGeneralFont_GF1];
    titleItem.touchNoChange = YES;
    return titleItem;
}

+ (FSItem *)createItemWithTitle:(nullable NSString *)title{
    return [self createItemWithTitle:title
                          background:nil];
}

+ (FSItem *)createItemWithImage:(nullable UIImage *)image{
    return [self createItemWithImage:image
                          background:nil];
}

+ (FSItem *)createItemWithImage:(nullable UIImage *)image background:(nullable UIImage *)background{
    return [self createItemWithTitle:nil
                                 imageNormal:image
                               imageSelected:image
                                imageDisable:nil
                                  background:background
                           imageTextRelation:FSImageTextRelationTop];
}

+ (FSItem *)createItemWithTitle:(nullable NSString *)title
                          image:(nullable UIImage *)image
              imageTextRelation:(FSImageTextRelation)imageTextRelation{
    return [self createItemWithTitle:title
                             imageNormal:image
                           imageSelected:image
                            imageDisable:nil
                              background:nil
                       imageTextRelation:imageTextRelation];
}


+ (FSItem *)createItemWithTitle:(nullable NSString *)title
                         background:(nullable UIImage *)background{
    return [self createItemWithTitle:title
                             imageNormal:nil
                           imageSelected:nil
                            imageDisable:nil
                              background:background
                       imageTextRelation:FSImageTextRelationTop];
}

+ (FSItem *)createItemWithImage:(nullable UIImage *)imageNormal
                  imageSelected:(nullable UIImage *)imageSelected{
    return [self createItemWithImage:imageNormal imageSelected:imageSelected imageDisable:nil];
}

+ (FSItem *)createItemWithImage:(nullable UIImage *)imageNormal
                      imageSelected:(nullable UIImage *)imageSelected
                       imageDisable:(nullable UIImage *)imageDisabled{
        return [self createItemWithImage:imageNormal
                           imageSelected:imageSelected
                            imageDisable:imageDisabled
                              background:nil];
}

+ (FSItem *)createItemWithImage:(nullable UIImage *)imageNormal
                  imageSelected:(nullable UIImage *)imageSelected
                   imageDisable:(nullable UIImage *)imageDisabled
                     background:(nullable UIImage *)background{
    return [self createItemWithTitle:nil
                             imageNormal:imageNormal
                           imageSelected:imageSelected
                            imageDisable:imageDisabled
                              background:background
                       imageTextRelation:FSImageTextRelationTop];
}

+ (FSItem *)createItemWithTitle:(nullable NSString *)title
                                imageNormal:(nullable UIImage *)imageNormal
                              imageSelected:(nullable UIImage *)imageSelected
                               imageDisable:(nullable UIImage *)imageDisabled
                          imageTextRelation:(FSImageTextRelation)imageTextRelation{
    return [self createItemWithTitle:title
                                 imageNormal:imageNormal
                               imageSelected:imageSelected
                                imageDisable:imageDisabled
                                  background:nil
                           imageTextRelation:imageTextRelation];
}

+ (FSItem *)createItemWithTitle:(nullable NSString *)title
                                imageNormal:(nullable UIImage *)imageNormal
                              imageSelected:(nullable UIImage *)imageSelected
                               imageDisable:(nullable UIImage *)imageDisabled
                                 background:(nullable UIImage *)background
                          imageTextRelation:(FSImageTextRelation)imageTextRelation{
    return [self createItemWithTitle:title
                                 imageNormal:imageNormal
                            imageHighlighted:nil
                               imageSelected:imageSelected
                                imageDisable:imageDisabled
                                  background:background
                           imageTextRelation:imageTextRelation];
}

+ (FSItem *)createItemWithTitle:(nullable NSString *)title
                                imageNormal:(nullable UIImage *)imageNormal
                          imageHighlighted:(nullable UIImage *)imageHighlighted
                              imageSelected:(nullable UIImage *)imageSelected
                               imageDisable:(nullable UIImage *)imageDisabled
                                 background:(nullable UIImage *)background
                          imageTextRelation:(FSImageTextRelation)imageTextRelation{
    FSItem *item = [[FSItem alloc] initWithRrelation:imageTextRelation];
    [item setTitle:title forState:UIControlStateNormal];
    if (imageNormal) [item setImage:imageNormal forState:UIControlStateNormal];
    if (imageHighlighted)[item setImage:imageHighlighted forState:UIControlStateHighlighted];
    if (imageSelected) [item setImage:imageSelected forState:UIControlStateSelected];
    if (imageDisabled)[item setImage:imageDisabled forState:UIControlStateDisabled];
    if (background) [item setBackgroundImage:background forState: UIControlStateNormal | UIControlStateHighlighted | UIControlStateDisabled | UIControlStateSelected];
    return item;
}

static UIColor *_appearanceDisableTitleColor;
+ (void)setAppearanceDisableTitleColor:(UIColor *)color{
    _appearanceDisableTitleColor = color;
}

+ (UIColor *) appearanceDisableTitleColor{
    return _appearanceDisableTitleColor;
}

- (id)copyWithZone:(nullable NSZone *)zone{
    FSItem *item = [[FSItem allocWithZone:zone] initWithRrelation:self.relation];
    item.stateConfig = @{}.mutableCopy;
    for (NSString *key in [self.stateConfig allKeys]) {
        FSItemAttributes *attributes = [self.stateConfig valueForKey:key];
        [item.stateConfig setValue:[attributes copy] forKey:key];
    }
    @weakify(self)
    item.onTapClick = ^(FSItem * _Nonnull item) {
        @strongify(self)
        !self.onTapClick ? : self.onTapClick(item);
    };
    item.onLongPress = ^(FSItem * _Nonnull item) {
        @strongify(self)
        !self.onLongPress ? : self.onLongPress(item);
    };
    item.titleEdgeInsets = self.titleEdgeInsets;
    item.imageEdgeInsets = self.imageEdgeInsets;
    item.font = self.font;
    [item setConfigToItem];
    
    return item;
}

- (instancetype)init{
    return [self initWithRrelation:FSImageTextRelationTop];
}

- (instancetype)initWithFrame:(CGRect)frame{
    return [self initWithFrame:frame relation:FSImageTextRelationTop];;
}

- (instancetype)initWithFrame:(CGRect)frame relation:(FSImageTextRelation)relation{
    self = [super initWithFrame:frame];
    if (self) {
        self.relation = relation;
        [self setup];
    }
    return self;
}
- (instancetype)initWithRrelation:(FSImageTextRelation)relation {
    self = [super init];
    if (self) {
        self.relation = relation;
        [self setup];
    }
    return self;
}

- (id)owner{
    if (_owner) return _owner;
    return self.superview;
}

- (UIImageView *)imageView{
    if (!_imageView) {
        _imageView = [[UIImageView alloc] init];
        [self addSubview:_imageView];
        [self makeSubviewsConstraints];
    }
    return _imageView;
}

- (UILabel *)titleLabel{
    if (!_titleLabel) {
        _titleLabel = [[UILabel alloc] init];
        _titleLabel.font = _font;
        [self addSubview:_titleLabel];
        [self makeSubviewsConstraints];
    }
   return _titleLabel;
}

- (UIImageView *)backgroundImageView{
    if (!_backgroundImageView) {
        _backgroundImageView = [[UIImageView alloc] init];
        [self insertSubview:_backgroundImageView atIndex:0];
        [_backgroundImageView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.right.top.bottom.mas_equalTo(self).priorityMedium();
        }];
    }
    return _backgroundImageView;
}

- (UIImageView *)imageViewBackgroundImageView{
    if (!_imageViewBackgroundImageView) {
        _imageViewBackgroundImageView = [[UIImageView alloc] init];
        [self insertSubview:_imageViewBackgroundImageView belowSubview:self.imageView];
        [_imageViewBackgroundImageView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.right.top.bottom.mas_equalTo(self.imageView).priorityMedium();
        }];
    }
    return _imageViewBackgroundImageView;
}

- (UIView *)backgroundView
{
    if (!_backgroundView) {
        _backgroundView = [[UIView alloc] init];
        _backgroundView.layer.cornerRadius = 4.0f;
        _backgroundView.userInteractionEnabled = NO;
        _backgroundView.backgroundColor = [UIColor clearColor];
        [self insertSubview:_backgroundView atIndex:0];
        [_backgroundView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.leftMargin.topMargin.mas_equalTo(4).priorityMedium();
            make.rightMargin.bottomMargin.mas_equalTo(-4).priorityMedium();
        }];
    }
    return _backgroundView;
}

- (void) makeSubviewsConstraints{
    if (_backgroundView && [self.subviews firstObject] != _backgroundView) {
        [self sendSubviewToBack:_backgroundView];
    }
//    if (_backgroundImageView) {
//        if (_backgroundView) {
//            [self sendSubviewToBack:_backgroundImageView];
//            [self sendSubviewToBack:_backgroundView];
//        }else{
//            if ([self.subviews firstObject] != _backgroundImageView) [self sendSubviewToBack:_backgroundImageView];
//        }
//    }
    if (!_titleLabel.superview && !_imageView.superview) return;
    else if (_titleLabel.superview && !_imageView.superview) {
        [_titleLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(self).offset(_titleEdgeInsets.left);
            make.right.mas_equalTo(self).offset(-_titleEdgeInsets.right);
            make.top.mas_equalTo(self).offset(_titleEdgeInsets.top);
            make.bottom.mas_equalTo(self).offset(-_titleEdgeInsets.bottom);
        }];
    }
    else if (!_titleLabel.superview && _imageView.superview) {
        [_imageView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(self).offset(_imageEdgeInsets.left);
            make.right.mas_equalTo(self).offset(-_imageEdgeInsets.right);
            make.top.mas_equalTo(self).offset(_imageEdgeInsets.top);
            make.bottom.mas_equalTo(self).offset(-_imageEdgeInsets.bottom);
        }];
    }
    else{
        switch (self.relation) {
            case FSImageTextRelationLeft:{
                    [_titleLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
                        make.left.mas_equalTo(self).offset(_titleEdgeInsets.left);
                    }];
                    [_imageView mas_remakeConstraints:^(MASConstraintMaker *make) {
                        make.left.mas_equalTo(_titleLabel.mas_right).offset(_titleEdgeInsets.right + _imageEdgeInsets.left);
                        make.right.mas_equalTo(self).offset(-_imageEdgeInsets.right);
                    }];
                if (TitleLabelHeightGreaterThanImageViewHeight) {
                    [_titleLabel mas_updateConstraints:^(MASConstraintMaker *make) {
                        make.centerY.mas_equalTo(self).priorityHigh();
                        make.centerY.mas_equalTo(_imageView).priorityHigh();
                        make.top.mas_equalTo(self).offset(_titleEdgeInsets.top);
                        make.bottom.mas_equalTo(self).offset(-_titleEdgeInsets.bottom);
                    }];
                }else{
                    [_imageView mas_updateConstraints:^(MASConstraintMaker *make) {
                        make.centerY.mas_equalTo(self).priorityHigh();
                        make.centerY.mas_equalTo(_titleLabel).priorityHigh();
                        make.top.mas_equalTo(self).offset(_imageEdgeInsets.top);
                        make.bottom.mas_equalTo(self).offset(-_imageEdgeInsets.bottom);
                    }];
                }
            }
                break;
            case FSImageTextRelationRight:{
                    [_titleLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
                        make.right.mas_equalTo(self).offset(-_titleEdgeInsets.right);;
                    }];
                    [_imageView mas_remakeConstraints:^(MASConstraintMaker *make) {
                        make.right.mas_equalTo(_titleLabel.mas_left).offset(-_imageEdgeInsets.right - _titleEdgeInsets.left);
                        make.left.mas_equalTo(self).offset(_imageEdgeInsets.left);
                    }];
                if (TitleLabelHeightGreaterThanImageViewHeight) {
                    [_titleLabel mas_updateConstraints:^(MASConstraintMaker *make) {
                        make.centerY.mas_equalTo(self).priorityHigh();
                        make.centerY.mas_equalTo(_imageView).priorityHigh();
                        make.top.mas_equalTo(self).offset(_titleEdgeInsets.top);
                        make.bottom.mas_equalTo(self).offset(-_titleEdgeInsets.bottom);
                    }];
                }else{
                    [_imageView mas_updateConstraints:^(MASConstraintMaker *make) {
                        make.centerY.mas_equalTo(self).priorityHigh();
                        make.centerY.mas_equalTo(_titleLabel).priorityHigh();
                        make.top.mas_equalTo(self).offset(_imageEdgeInsets.top);
                        make.bottom.mas_equalTo(self).offset(-_imageEdgeInsets.bottom);
                    }];
                }
            }
                break;
            case FSImageTextRelationBottom:{
                    [_titleLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
                        make.bottom.mas_equalTo(self).offset(-_titleEdgeInsets.bottom);;
                    }];
                    [_imageView mas_remakeConstraints:^(MASConstraintMaker *make) {
                        make.bottom.mas_equalTo(_titleLabel.mas_top).offset(-_imageEdgeInsets.bottom - _titleEdgeInsets.top);
                        make.top.mas_equalTo(self).offset(_imageEdgeInsets.top);
                    }];
                if (TitleLabelWidthGreaterThanImageViewWidth) {
                    [_titleLabel mas_updateConstraints:^(MASConstraintMaker *make) {
                        make.centerX.mas_equalTo(self).priorityHigh();
                        make.centerX.mas_equalTo(_imageView).priorityHigh();
                        make.left.mas_equalTo(self).offset(_titleEdgeInsets.left);
                        make.right.mas_equalTo(self).offset(-_titleEdgeInsets.right);
                    }];
                }else{
                    [_imageView mas_updateConstraints:^(MASConstraintMaker *make) {
                        make.centerX.mas_equalTo(self).priorityHigh();
                        make.centerX.mas_equalTo(_titleLabel).priorityHigh();
                        make.left.mas_equalTo(self).offset(_imageEdgeInsets.left);
                        make.right.mas_equalTo(self).offset(-_imageEdgeInsets.right);
                    }];
                }
            }
                break;
            default:{
                    [_titleLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
                        make.top.mas_equalTo(self).offset(_titleEdgeInsets.top);
                    }];
                    [_imageView mas_remakeConstraints:^(MASConstraintMaker *make) {
                        make.top.mas_equalTo(_titleLabel.mas_bottom).offset(_imageEdgeInsets.top + _titleEdgeInsets.bottom);
                        make.bottom.mas_equalTo(self).offset(-_imageEdgeInsets.bottom);
                    }];
                if (TitleLabelWidthGreaterThanImageViewWidth) {
                    [_titleLabel mas_updateConstraints:^(MASConstraintMaker *make) {
                        make.centerX.mas_equalTo(self).priorityHigh();
                        make.centerX.mas_equalTo(_imageView).priorityHigh();
                        make.left.mas_equalTo(self).offset(_titleEdgeInsets.left);
                        make.right.mas_equalTo(self).offset(-_titleEdgeInsets.right);
                    }];
                }else{
                    [_imageView mas_updateConstraints:^(MASConstraintMaker *make) {
                        make.centerX.mas_equalTo(self).priorityHigh();
                        make.centerX.mas_equalTo(_titleLabel).priorityHigh();
                        make.left.mas_equalTo(self).offset(_imageEdgeInsets.left);
                        make.right.mas_equalTo(self).offset(-_imageEdgeInsets.right);
                    }];
                }
            }
                break;
        }
    }
}

- (void)setup{
    self.customAccessibilityTraits = UIAccessibilityTraitButton;
    self.stateConfig = @{}.mutableCopy;
    NSArray *tmp = @[@(UIControlStateNormal),@(UIControlStateHighlighted),@(UIControlStateDisabled),@(UIControlStateSelected),@(UIControlStateSelected|UIControlStateDisabled)];
    for (int i = 0; i < tmp.count; i++) {
        NSNumber *state = tmp[i];
        FSItemAttributes *itemAttributes = [FSItemAttributes new];
        [self.stateConfig setValue:itemAttributes forKey:[NSString stringWithFormat:@"%d",state.intValue]];
    }
    _titleEdgeInsets = UIEdgeInsetsZero;
    _imageEdgeInsets = UIEdgeInsetsZero;
    self.isAccessibilityElement = YES;
    self.backgroundImage = NO;
}

- (void)setOnTapClick:(InteractionCallBack)onTapClick{
    _onTapClick = [onTapClick copy];
    [self addTarget:self action:@selector(tapClick:) forControlEvents:UIControlEventTouchUpInside];
}

- (void)setOnLongPress:(InteractionCallBack)onLongPress{
    _onLongPress = [onLongPress copy];
    UILongPressGestureRecognizer *longPress = [[UILongPressGestureRecognizer alloc] initWithTarget:self action:@selector(onLongPressClicked:)];
    longPress.minimumPressDuration = 0.8; //press time
    [self addGestureRecognizer:longPress];
}

- (void)setTitleEdgeInsets:(UIEdgeInsets)titleEdgeInsets{
    _titleEdgeInsets = titleEdgeInsets;
    [self updateConstraints];
}

- (void)setImageEdgeInsets:(UIEdgeInsets)imageEdgeInsets{
    _imageEdgeInsets = imageEdgeInsets;
    [self updateConstraints];
}

const UIControlState FSItemControlStateAll = UIControlStateNormal | UIControlStateHighlighted | UIControlStateDisabled | UIControlStateSelected;
- (void)setTitle:(NSString *)title forState:(UIControlState)state{
    if (![title isKindOfClass:[NSString class]] && title) return;
    self.currentRecordKey = titleLabel_title;
    [self setStateConfigValue:title forState:state];
}

- (void)setTitleColor:(nullable UIColor *)color forState:(UIControlState)state{
    if (!color) color = [UIColor whiteColor];
    if (![color isKindOfClass:[UIColor class]]) return;
    _backgroundViewColor = NO;
    self.currentRecordKey = titleLabel_title_color;
    [self setStateConfigValue:color forState:state];
}

- (void)setBackgroudViewColor:(nullable UIColor *)color forState:(UIControlState)state
{
    if (!color) color = [UIColor clearColor];
    if (![color isKindOfClass:[UIColor class]]) return;
    _backgroundViewColor = YES;
    self.currentRecordKey = background_view_color;
    [self setStateConfigValue:color forState:state];
    _backgroundViewColor = NO;
}

- (void)setImage:(UIImage *)image forState:(UIControlState)state{
    if (![image isKindOfClass:[UIImage class]] && image) return;
    _backgroundImage = NO;
    _imageViewBackgroundImage = NO;
    self.currentRecordKey = imageView_image;
    [self setStateConfigValue:image forState:state];
}

- (void)setBackgroundImage:(nullable UIImage *)image forState:(UIControlState)state{
    if (![image isKindOfClass:[UIImage class]] && image) return;
    _imageViewBackgroundImage = NO;
    _backgroundImage = YES;
    self.currentRecordKey = background_imageView_image;
    [self setStateConfigValue:image forState:state];
    _backgroundImage = NO;
}

- (void)setImageViewBackgroundImage:(nullable UIImage *)image forState:(UIControlState)state{
    if (![image isKindOfClass:[UIImage class]] && image) return;
    _imageViewBackgroundImage = YES;
    self.currentRecordKey = imageViewBackground_imageView_image;
    [self setStateConfigValue:image forState:state];
    _imageViewBackgroundImage = NO;
}

- (void)setAttributedTitle:(NSAttributedString *)title forState:(UIControlState)state{
    if (![title isKindOfClass:[NSAttributedString class]]) return;
    self.currentRecordKey = titleLabel_attributed_text;
    [self setStateConfigValue:title forState:state];
}

- (NSString *)currentTitle{
    if (!_titleLabel.superview) return nil;
    return _titleLabel.text;
}

- (NSAttributedString *)currentAttributedTitle{
    if (!_titleLabel.superview) return nil;
    return _titleLabel.attributedText;
}

- (UIColor *)currentTitleColor{
    if (!_titleLabel.superview) return nil;
    return _titleLabel.textColor;
}

- (UIImage *)currentImage{
    if (!_imageView.superview) return nil;
    return _imageView.image;
}

- (UIImage *)currentBackgroundImage{
    return _backgroundImageView.image;
}

- (void)tapClick:(FSItem *)item{
    !self.onTapClick ? : self.onTapClick(self);
}

- (void)onLongPressClicked:(UILongPressGestureRecognizer *)gest{
    if (gest.state == UIGestureRecognizerStateBegan) {
        !self.onLongPress ? : self.onLongPress(self);
    }
}

static NSString *const titleLabel_title = @"titleLabel_title";
static NSString *const titleLabel_title_color = @"titleLabel_title_color";
static NSString *const titleLabel_attributed_text = @"titleLabel_attributed_text";
static NSString *const imageView_image = @"imageView_image";
static NSString *const background_imageView_image = @"background_imageView_image";
static NSString *const imageViewBackground_imageView_image = @"imageViewBackground_imageView_image";
static NSString *const background_view_color = @"background_color";

- (BOOL)isExistValueInAllAttributesForKey:(NSString *)attributeKey{
    BOOL isExist = NO;
    for (NSString *key in [self.stateConfig allKeys]) {
        FSItemAttributes *attributes = [self.stateConfig valueForKey:key];
        if ([attributeKey isEqualToString:titleLabel_title] && attributes.titleLabelTitle) {
            isExist = YES;
            break;
        }else if ([attributeKey isEqualToString:titleLabel_title_color] && attributes.titleLabelTitleColor) {
            isExist = YES;
            break;
        }else if ([attributeKey isEqualToString:titleLabel_attributed_text] && attributes.titleLabelAttributedText) {
            isExist = YES;
            break;
        }else if ([attributeKey isEqualToString:imageView_image] &&  attributes.imageViewImage) {
            isExist = YES;
            break;
        }else if ([attributeKey isEqualToString:background_imageView_image] && attributes.backgroundImageViewImage) {
            isExist = YES;
            break;
        }else if ([attributeKey isEqualToString:imageViewBackground_imageView_image] && attributes.imageViewBackgroundImageViewImage) {
            isExist = YES;
            break;
        }else if ([attributeKey isEqualToString:background_view_color] && attributes.backgroundViewColor) {
            isExist = YES;
            break;
        }
    }
    return isExist;
}

- (FSItemAttributes *)getItemAttributesWithState:(UIControlState)state{
    return [_stateConfig valueForKey:[NSString stringWithFormat:@"%lu",(unsigned long)state]];
}

- (void)setItemAttribute:(id)value forState:(UIControlState)state{
    FSItemAttributes *itemAttributes = [self getItemAttributesWithState:state];
    if ([self.currentRecordKey isEqualToString:titleLabel_title]) {
        itemAttributes.titleLabelTitle = value;
    }else if ([self.currentRecordKey isEqualToString:titleLabel_title_color]) {
        itemAttributes.titleLabelTitleColor = value;
    }else if ([self.currentRecordKey isEqualToString:titleLabel_attributed_text]) {
        itemAttributes.titleLabelAttributedText = value;
    }else if ([self.currentRecordKey isEqualToString:imageView_image]) {
        itemAttributes.imageViewImage = value;
    }else if ([self.currentRecordKey isEqualToString:background_imageView_image]) {
        itemAttributes.backgroundImageViewImage = value;
    }else if ([self.currentRecordKey isEqualToString:imageViewBackground_imageView_image]) {
        itemAttributes.imageViewBackgroundImageViewImage = value;
    }else if ([self.currentRecordKey isEqualToString:background_view_color]) {
        itemAttributes.backgroundViewColor = value;
    }
}

- (void)setAccessibilityLabel:(NSString *)accessibilityLabel{
    [super setAccessibilityLabel:accessibilityLabel];
    _accessibilityLabel = accessibilityLabel;
}

- (id)getItemAttribute:(FSItemAttributes *)attributes kindOfClass:(id)value{
    if (!value) return nil;
    if ([value isKindOfClass:[NSString class]]) return attributes.titleLabelTitle;
    else if ([value isKindOfClass:[UIColor class]] && !_backgroundViewColor) return attributes.titleLabelTitleColor;
    else if ([value isKindOfClass:[UIColor class]] && _backgroundViewColor) return attributes.backgroundViewColor;
    else if ([value isKindOfClass:[NSAttributedString class]]) return attributes.titleLabelAttributedText;
    else if ([value isKindOfClass:[UIImage class]] && !_backgroundImage && !_imageViewBackgroundImage) return attributes.imageViewImage;
    else if ([value isKindOfClass:[UIImage class]] && _backgroundImage && !_imageViewBackgroundImage) return attributes.backgroundImageViewImage;
    else if ([value isKindOfClass:[UIImage class]] && _imageViewBackgroundImage) return attributes.backgroundImageViewImage;
    return nil;
}

- (void)setStateConfigValue:(id)value forState:(UIControlState)state{
    if (state == UIControlStateNormal || ((state & UIControlStateNormal) == UIControlStateNormal && ![self getItemAttribute:[self getItemAttributesWithState:UIControlStateNormal] kindOfClass:value])) {
        [self setItemAttribute:value forState:UIControlStateNormal];
    }
    if ((state & UIControlStateHighlighted) == UIControlStateHighlighted) {
        [self setItemAttribute:value forState:UIControlStateHighlighted];
    }
    if ((state & UIControlStateDisabled) == UIControlStateDisabled){
        if ((state & UIControlStateSelected) == UIControlStateSelected) {
            [self setItemAttribute:value forState:UIControlStateDisabled | UIControlStateSelected];
        }else{
            [self setItemAttribute:value forState:UIControlStateDisabled];
        }
    }
    if ((state & UIControlStateSelected) == UIControlStateSelected) {
        [self setItemAttribute:value forState:UIControlStateSelected];
    }
    [self setConfigToItem];
}

- (void)setConfigToItem{
    
    if (self.touchNoChange && self.currentTouch) return;
    if (!self.enabled) {
        if (self.selected) {
            [self updateAttributeToItemWithState:UIControlStateDisabled | UIControlStateSelected];
        }else{
            [self updateAttributeToItemWithState:UIControlStateDisabled];
        }
        self.accessibilityTraits |= UIAccessibilityTraitNotEnabled;
        self.accessibilityTraits &= ~UIAccessibilityTraitSelected;
    }else if (self.highlighted) {
        if (self.selected) {
            [self updateAttributeToItemWithState:UIControlStateHighlighted | UIControlStateSelected];
        }else{
            [self updateAttributeToItemWithState:UIControlStateHighlighted];
        }
        self.accessibilityTraits = self.customAccessibilityTraits;
    }else if (self.selected) {
        [self updateAttributeToItemWithState:UIControlStateSelected];
        self.accessibilityTraits |= UIAccessibilityTraitSelected;
        self.accessibilityTraits &= ~UIAccessibilityTraitNotEnabled;
    }else{
        [self updateAttributeToItemWithState:UIControlStateNormal];
        self.accessibilityTraits = self.customAccessibilityTraits;
    }
}

- (void)setCustomAccessibilityTraits:(UIAccessibilityTraits)customAccessibilityTraits{
    _customAccessibilityTraits = customAccessibilityTraits;
    self.accessibilityTraits = customAccessibilityTraits;
}

- (void)updateAttributeToItemWithState:(UIControlState)state{
    FSItemAttributes *itemAttributes = [self getItemAttributesWithState:state];
    [self setLabelTitle:itemAttributes.titleLabelTitle];
    if (self.currentTitle.length > 0) {
        if (!_accessibilityLabel) {
            [super setAccessibilityLabel:self.currentTitle];
        }
    }
    [self setLabelTitleColor:itemAttributes.titleLabelTitleColor];
    [self setLabelAttributed:itemAttributes.titleLabelAttributedText];
    [self setBackGroundViewColor:itemAttributes.backgroundViewColor];
    [self setImage:itemAttributes.imageViewImage isBackground:NO];
    [self setImage:itemAttributes.backgroundImageViewImage isBackground:YES];
    [self setImageViewBackgroundImageContent:itemAttributes.imageViewBackgroundImageViewImage];
}

- (void)setLabelTitle:(NSString *)title{
    if (!title && ![self isExistValueInAllAttributesForKey:titleLabel_title] && ![self isExistValueInAllAttributesForKey:titleLabel_attributed_text]) {
        if (_titleLabel) {
            _titleLabel.text = nil;
            [_titleLabel removeFromSuperview];
            [self makeSubviewsConstraints];
        }
        return;
    }
    if (self.state != UIControlStateNormal && !title){
        FSItemAttributes *attributes = [self getItemAttributesWithState:UIControlStateNormal];
        if (!attributes.titleLabelTitle) return;
        if ((self.state & UIControlStateSelected) == UIControlStateSelected) {
            FSItemAttributes *selectedAttributes = [self getItemAttributesWithState:UIControlStateSelected];
            if (selectedAttributes.titleLabelTitle) attributes = selectedAttributes;
        }
        title = attributes.titleLabelTitle;
    }
    BOOL res = [self currntImageViewSizeIsGreaterThanTitleLabel];
    self.titleLabel.text = title;
    if (!_titleLabel.superview) {
        [self addSubview:_titleLabel];
        [self makeSubviewsConstraints];
    }
    if (res != [self currntImageViewSizeIsGreaterThanTitleLabel]) {
        [self makeSubviewsConstraints];
    }
}

- (void)setLabelTitleColor:(UIColor *)color{
    if (!color && !_titleLabel) return;
    if (self.state != UIControlStateNormal && !color){
        FSItemAttributes *attributes = [self getItemAttributesWithState:UIControlStateNormal];
        if (!attributes.titleLabelTitleColor) attributes.titleLabelTitleColor = self.titleLabel.textColor;
        if ((self.state & UIControlStateDisabled) == UIControlStateDisabled) {
            if ((self.state & UIControlStateSelected) == UIControlStateSelected) {
                FSItemAttributes *selectedAttributes = [self getItemAttributesWithState:UIControlStateSelected];
                if (selectedAttributes.titleLabelTitleColor) attributes = selectedAttributes;
            }
            if (!FSItem.appearanceDisableTitleColor) {
                color =  [attributes.titleLabelTitleColor colorWithAlphaComponent:LABEL_DISABLED_ALPHA];
            }else{
                color =  FSItem.appearanceDisableTitleColor;
            }

        }else if ((self.state & UIControlStateHighlighted) == UIControlStateHighlighted){
            if ((self.state & UIControlStateSelected) == UIControlStateSelected) {
                FSItemAttributes *selectedAttributes = [self getItemAttributesWithState:UIControlStateSelected];
                if (selectedAttributes.titleLabelTitleColor) attributes = selectedAttributes;
            }
            color = [attributes.titleLabelTitleColor colorWithAlphaComponent:LABEL_HIGHLIGHTED_ALPHA];
        }else{
            color = attributes.titleLabelTitleColor;
            if (color == _titleLabel.textColor) return;
        }
    }
    BOOL res = [self currntImageViewSizeIsGreaterThanTitleLabel];
    self.titleLabel.textColor = color;
    if (![self currentTitle] || [self currentTitle].length < 1) return;
    if (!_titleLabel.superview) {
        [self addSubview:_titleLabel];
        [self makeSubviewsConstraints];
    }
    if (res != [self currntImageViewSizeIsGreaterThanTitleLabel]) {
        [self makeSubviewsConstraints];
    }
}

- (void)setBackGroundViewColor:(UIColor *)color{
    if (!color) return;
    if (self.state != UIControlStateNormal && !color){
        FSItemAttributes *attributes = [self getItemAttributesWithState:UIControlStateNormal];
        if (!attributes.backgroundViewColor) attributes.titleLabelTitleColor = self.backgroundView.backgroundColor;
        color = attributes.titleLabelTitleColor;
        if (color == _backgroundView.backgroundColor) return;
    }
    self.backgroundView.backgroundColor = color;
}

- (void)setLabelAttributed:(NSAttributedString *)title{
    if (!title) return;
    BOOL res = [self currntImageViewSizeIsGreaterThanTitleLabel];
    self.titleLabel.attributedText = title;
    if (!self.titleLabel.superview) {
        [self addSubview:_titleLabel];
        [self makeSubviewsConstraints];
    }
    if (res != [self currntImageViewSizeIsGreaterThanTitleLabel]) {
        [self makeSubviewsConstraints];
    }
}

- (void)setImage:(UIImage *)image isBackground:(BOOL)isBackground{
    if (isBackground){
        if (!image && !_backgroundImageView.image) return;
        if (self.state != UIControlStateNormal && !image){
            FSItemAttributes *attributes = [self getItemAttributesWithState:UIControlStateNormal];
            if (!attributes.backgroundImageViewImage) return;
            if ((self.state & UIControlStateDisabled) == UIControlStateDisabled) {
                if ((self.state & UIControlStateSelected) == UIControlStateSelected) {
                    FSItemAttributes *selectedAttributes = [self getItemAttributesWithState:UIControlStateSelected];
                    if (selectedAttributes.backgroundImageViewImage) attributes = selectedAttributes;
                }
                image = [attributes.backgroundImageViewImage fs_applyingAlpha:IMAGE_DISABLED_ALPHA];
            }else if((self.state & UIControlStateHighlighted) == UIControlStateHighlighted){
                if ((self.state & UIControlStateSelected) == UIControlStateSelected) {
                    FSItemAttributes *selectedAttributes = [self getItemAttributesWithState:UIControlStateSelected];
                    if (selectedAttributes.backgroundImageViewImage) attributes = selectedAttributes;
                }
                image = [attributes.backgroundImageViewImage fs_applyingAlpha:IMAGE_HIGHLIGHTED_ALPHA];
            }else{
                image = attributes.backgroundImageViewImage;
            }
        }
        self.backgroundImageView.image = image;
    }
    else{
        if (!image && ![self isExistValueInAllAttributesForKey:imageView_image]) {
            if (_imageView) {
                _imageView.image = nil;
                [_imageView removeFromSuperview];
                [self makeSubviewsConstraints];
            }
            return;
        }
        if (!image.CGImage && image && self.state != UIControlStateNormal) return;
        if (self.state != UIControlStateNormal && !image){
            FSItemAttributes *attributes = [self getItemAttributesWithState:UIControlStateNormal];
            if (!attributes.imageViewImage) return;
            if ((self.state & UIControlStateDisabled) == UIControlStateDisabled) {
                if ((self.state & UIControlStateSelected) == UIControlStateSelected) {
                    FSItemAttributes *selectedAttributes = [self getItemAttributesWithState:UIControlStateSelected];
                    if (selectedAttributes.imageViewImage) attributes = selectedAttributes;
                }
                image = [attributes.imageViewImage fs_applyingAlpha:IMAGE_DISABLED_ALPHA];
            }else if((self.state & UIControlStateHighlighted) == UIControlStateHighlighted){
                if ((self.state & UIControlStateSelected) == UIControlStateSelected) {
                    FSItemAttributes *selectedAttributes = [self getItemAttributesWithState:UIControlStateSelected];
                    if (selectedAttributes.imageViewImage) attributes = selectedAttributes;
                }
                image = [attributes.imageViewImage fs_applyingAlpha:IMAGE_HIGHLIGHTED_ALPHA];
            }else{
                image = attributes.imageViewImage;
            }
        }
        
        if (image == _imageView.image){
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
            // fix iOS 13 dark mode can not refresh
            _imageView.image = nil;
            _imageView.image = image;
#endif
            return;
        }
        BOOL res = [self currntTitleLabelSizeIsGreaterThanImageView];
        self.imageView.image = image;
        if (!_imageView.superview) {
            [self addSubview:_imageView];
            [self makeSubviewsConstraints];
        }
        if (res != [self currntTitleLabelSizeIsGreaterThanImageView]) {
            [self makeSubviewsConstraints];
        }
    }
}

- (void)setImageViewBackgroundImageContent:(UIImage *)image
{
    if (!image && !_imageViewBackgroundImageView.image) return;
    if (self.state != UIControlStateNormal && !image){
        FSItemAttributes *attributes = [self getItemAttributesWithState:UIControlStateNormal];
        if (!attributes.imageViewBackgroundImageViewImage) return;
        if ((self.state & UIControlStateDisabled) == UIControlStateDisabled) {
            if ((self.state & UIControlStateSelected) == UIControlStateSelected) {
                FSItemAttributes *selectedAttributes = [self getItemAttributesWithState:UIControlStateSelected];
                if (selectedAttributes.imageViewBackgroundImageViewImage) attributes = selectedAttributes;
            }
            image = [attributes.imageViewBackgroundImageViewImage fs_applyingAlpha:IMAGE_DISABLED_ALPHA];
        }else if((self.state & UIControlStateHighlighted) == UIControlStateHighlighted){
            if ((self.state & UIControlStateSelected) == UIControlStateSelected) {
                FSItemAttributes *selectedAttributes = [self getItemAttributesWithState:UIControlStateSelected];
                if (selectedAttributes.imageViewBackgroundImageViewImage) attributes = selectedAttributes;
            }
            image = [attributes.imageViewBackgroundImageViewImage fs_applyingAlpha:IMAGE_HIGHLIGHTED_ALPHA];
        }else{
            image = attributes.imageViewBackgroundImageViewImage;
        }
    }
    self.imageViewBackgroundImageView.image = image;
}

- (void)rebuildImageTextRelation:(FSImageTextRelation)relation{
    if (_relation == relation) return;
    _relation = relation;
    [self makeSubviewsConstraints];
}

- (BOOL)currntTitleLabelSizeIsGreaterThanImageView{
    BOOL isGreaterThan = NO;
    if (_titleLabel) {
        switch (self.relation) {
            case FSImageTextRelationLeft:
            case FSImageTextRelationRight:{
                if (TitleLabelHeightGreaterThanImageViewHeight) {
                    isGreaterThan = YES;
                }else{
                    isGreaterThan = NO;
                }
            }break;
            case FSImageTextRelationBottom:
            default:{
                if (TitleLabelWidthGreaterThanImageViewWidth) {
                    isGreaterThan = YES;
                }else{
                    isGreaterThan = NO;
                }
            }break;
        }
    }
    return isGreaterThan;
}

- (BOOL)currntImageViewSizeIsGreaterThanTitleLabel{
    BOOL isGreaterThan = NO;
    if (_imageView) {
        switch (self.relation) {
            case FSImageTextRelationLeft:
            case FSImageTextRelationRight:{
                if (TitleLabelHeightGreaterThanImageViewHeight) {
                    isGreaterThan = YES;
                }else{
                    isGreaterThan = NO;
                }
            }break;
            case FSImageTextRelationBottom:
            default:{
                if (TitleLabelWidthGreaterThanImageViewWidth) {
                    isGreaterThan = YES;
                }else{
                    isGreaterThan = NO;
                }
            }break;
        }
    }
    return isGreaterThan;
}

- (void)setFont:(UIFont *)font{
    _font = font;
    _titleLabel.font = font;
}

- (void)setEnabled:(BOOL)enabled{
    [super setEnabled:enabled];
    [self setConfigToItem];
}

- (void)setSelected:(BOOL)selected{
    [super setSelected:selected];
    [self setConfigToItem];
}

- (void)setHighlighted:(BOOL)highlighted{
    [super setHighlighted:highlighted];
    [self setConfigToItem];
}

- (void)touchesBegan:(NSSet*)touches withEvent:(UIEvent*)event{
    self.currentTouch = YES;
    [super touchesBegan:touches withEvent:event];
    [self setConfigToItem];
}

- (void)touchesMoved:(NSSet*)touches withEvent:(UIEvent*)event{
    self.currentTouch = YES;
    [super touchesMoved:touches withEvent:event];
    [self setConfigToItem];
}

- (void)touchesEnded:(NSSet*)touches withEvent:(UIEvent*)event{
    [super touchesEnded:touches withEvent:event];
    self.currentTouch = NO;
    [self setConfigToItem];
}

- (void)touchesCancelled:(NSSet*)touches withEvent:(UIEvent*)event{
    [super touchesCancelled:touches withEvent:event];
    self.currentTouch = NO;
    [self setConfigToItem];
}

- (void)updateConstraints{
    [self makeSubviewsConstraints];
    [super updateConstraints];
}

#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
- (void)traitCollectionDidChange:(UITraitCollection *)previousTraitCollection{
    [super traitCollectionDidChange:previousTraitCollection];
    if (@available(iOS 13.0, *)) {
        if ([self.traitCollection hasDifferentColorAppearanceComparedToTraitCollection:previousTraitCollection]) {
            [self updateAttributeToItemWithState:self.state];
        }
    }
}
#endif
@end

@implementation FSItemAttributes
- (id)copyWithZone:(nullable NSZone *)zone{
    FSItemAttributes *attributes = [FSItemAttributes new];
    attributes.titleLabelTitle = [self.titleLabelTitle copy];
    attributes.titleLabelAttributedText = [self.titleLabelAttributedText copy];
    attributes.titleLabelTitleColor = [self.titleLabelTitleColor fs_colorFromCopy];
    attributes.imageViewImage = [self.imageViewImage fs_imageFromCopy];
    attributes.backgroundImageViewImage = [self.backgroundImageViewImage fs_imageFromCopy];
    attributes.backgroundViewColor = [self.backgroundViewColor fs_colorFromCopy];
    return attributes;
}
@end
