/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "Masonry+FSAdd.h"
#import "UIView+FSAdd.h"
#import "FSDescriptionEstimatedTableViewCell.h"
#import "FSConst.h"

@interface FSDescriptionEstimatedTableViewCell ()
@property (nonatomic, assign, readwrite) BOOL installed;
@property (nonatomic, strong, readwrite) UILabel *estimatedDescriptionLabel;
@end
@implementation FSDescriptionEstimatedTableViewCell

@synthesize installed = _installed;

- (UILabel *)estimatedDescriptionLabel{
    if(!_estimatedDescriptionLabel){
        _estimatedDescriptionLabel = [UILabel new];
        _estimatedDescriptionLabel.numberOfLines = 0;
        _estimatedDescriptionLabel.textAlignment = NSTextAlignmentLeft;
        _estimatedDescriptionLabel.textColor = self.descriptionDefaultTextColor;
        _estimatedDescriptionLabel.font = self.descriptionDefaultFont;
        [_estimatedDescriptionLabel setContentHuggingPriority:ESTIMATED_HORIZONTAL_HEUGGING_PRIORITY forAxis:UILayoutConstraintAxisHorizontal];
        [_estimatedDescriptionLabel setContentHuggingPriority:UILayoutPriorityDefaultLow+1 forAxis:UILayoutConstraintAxisVertical];
        [self.contentView addSubview:_estimatedDescriptionLabel];
    }
    return _estimatedDescriptionLabel;
}

- (void)initialize{
    [super initialize];
    
    _descriptionBottomMargin = DESCRIPTION_LABEL_MIN_HEIGHT;
    
    self.topMargin = _descriptionBottomMargin;
}

- (void)installConstraints{
    if (_installed) return;
    !self.willInstallConstraints ? : self.willInstallConstraints(self);
    if (self.leftView) {
        [self.leftView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.fs_equalToSuperview().offset(self.leftMargin).priorityHigh();
            make.top.fs_equalToSuperview().offset(self.leftViewTopMargin).priorityHigh();
            if (!CGSizeEqualToSize(CGSizeZero, self.leftViewSzie)) {
                make.size.mas_lessThanOrEqualTo(self.leftViewSzie);
            }else if(!CGSizeEqualToSize(CGSizeZero, self.leftView.fs_size)){
                make.width.mas_lessThanOrEqualTo(self.leftView.fs_width);
                make.height.mas_lessThanOrEqualTo(self.leftView.fs_height);
            }else{
                make.size.mas_equalTo(self.leftView).priority(MASLayoutPriorityFittingSizeLevel);
            }
        }];
    }
    
    [self.estimatedTitleLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
        if (self.leftView) {
            make.left.mas_equalTo(self.leftView.mas_right).offset(self.mediumSpace).priorityHigh();
        }else{
            make.left.fs_equalToSuperview().offset(self.leftMargin).priorityHigh();
        }
        if (self.rightView) {
            make.right.mas_equalTo(self.rightView.mas_left).offset(-self.rightMargin).priorityHigh();
        }else{
            make.right.fs_equalToSuperview().offset(-self.rightMargin).priorityHigh();
        }
        make.top.fs_equalToSuperview().offset(self.topMargin).priorityHigh();
        make.bottom.mas_equalTo(self.estimatedSubjectLabel.mas_top).offset(-self.titleBottomMargin).priorityHigh();
        make.height.mas_greaterThanOrEqualTo(TITLE_LABEL_MIN_HEIGHT).priorityHigh();
    }];
    
    [self.estimatedSubjectLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.right.mas_equalTo(self.estimatedTitleLabel).priorityHigh();
        make.bottom.mas_equalTo(self.estimatedDescriptionLabel.mas_top).offset(-self.subjectBottomMargin).priorityHigh();
        make.height.mas_greaterThanOrEqualTo(SUBJECT_LABEL_MIN_HEIGHT).priorityHigh();
    }];
    
    [self.estimatedDescriptionLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.fs_equalToSuperview().offset(ITEM_MARGIN_NORMAL);
        make.right.fs_equalToSuperview().offset(-ITEM_MARGIN_NORMAL);
        make.bottom.fs_equalToSuperview().offset(-self.descriptionBottomMargin);
        make.height.mas_greaterThanOrEqualTo(DESCRIPTION_LABEL_MIN_HEIGHT).priorityHigh();
    }];
    
    if (self.rightView) {
        [self.rightView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.right.fs_equalToSuperview().offset(-self.rightMargin).priorityHigh();
            make.top.fs_equalToSuperview().offset(self.rightViewTopMargin).priorityHigh();
            make.size.mas_equalTo(self.rightView).priority(MASLayoutPriorityFittingSizeLevel);
        }];
    }
    _installed = YES;
    !self.didInstallConstraints ? : self.didInstallConstraints(self);
    
}

- (void)prepareForReuse{
    [super prepareForReuse];
    LabelAttributeAssign(self.estimatedDescriptionLabel, self.descriptionDefaultTextColor, self.descriptionDefaultFont);
}

@end
