/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#if __has_include(<FSFoundation/FSFoundation.h>)
#import <FSFoundation/FSTableViewCell.h>
#import <FSFoundation/FSConst.h>
#else
#import "FSTableViewCell.h"
#import "FSConst.h"
#endif

typedef NS_ENUM(NSUInteger, FSEstimatedTableViewCellStyle) {
    FSEstimatedTableViewCellStyleNone,
    FSEstimatedTableViewCellStyleTitle,   // 2 views and 1 estimated label
    FSEstimatedTableViewCellStyleSubject,  // 2 views and 2 estimated label
    FSEstimatedTableViewCellStyleDescription, // 2 views and estimated 3 label
};
NS_ASSUME_NONNULL_BEGIN

#define LabelAttributeAssign(label, _color, _font)  if (label.textColor != _color) { \
    label.textColor = _color;\
}\
    if (label.font != _font) { \
        label.font = _font;\
    }

@interface FSEstimatedTableViewCell : FSTableViewCell

//After set, call reinstall
@property (nonatomic, assign) CGFloat topMargin;
@property (nonatomic, assign) CGFloat bottomMargin;
@property (nonatomic, assign) CGFloat leftMargin;
@property (nonatomic, assign) CGFloat mediumSpace;
@property (nonatomic, assign) CGFloat rightMargin;
@property (nonatomic, assign) CGSize leftViewSzie;

@property (nonatomic, copy) VoidBlock_id willInstallConstraints;
@property (nonatomic, copy) VoidBlock_id didInstallConstraints;
@property (nonatomic, assign, readonly) BOOL installed;

//Avoid restoring when attributedText characters are matched and font and color changes
@property (nonatomic, strong, nullable) UIColor *titleDefaultTextColor;
@property (nonatomic, strong, nullable) UIFont *titleDefaultFont;
@property (nonatomic, strong, nullable) UIColor *subjectDefaultTextColor;
@property (nonatomic, strong, nullable) UIFont *subjectDefaultFont;
@property (nonatomic, strong, nullable) UIColor *descriptionDefaultTextColor;
@property (nonatomic, strong, nullable) UIFont *descriptionDefaultFont;

- (void)installConstraints;

- (void)reinstallConstraints;
@end

NS_ASSUME_NONNULL_END
