/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "UIView+FSAdd.h"
#import "FSConst.h"
#import "Masonry+FSAdd.h"
#import "FSPreferences.h"
#import "FSSubjectEstimatedTableViewCell.h"

@interface FSSubjectEstimatedTableViewCell ()
@property (nonatomic, assign, readwrite) BOOL installed;
@property (nonatomic, strong, readwrite) UILabel *estimatedSubjectLabel;
@end
@implementation FSSubjectEstimatedTableViewCell

@synthesize installed = _installed;

- (UILabel *)estimatedSubjectLabel{
    if(!_estimatedSubjectLabel){
        _estimatedSubjectLabel = [UILabel new];
        _estimatedSubjectLabel.numberOfLines = 0;
        _estimatedSubjectLabel.textAlignment = NSTextAlignmentLeft;
        _estimatedSubjectLabel.textColor = self.subjectDefaultTextColor;
        _estimatedSubjectLabel.font = self.subjectDefaultFont;
        [_estimatedSubjectLabel setContentHuggingPriority:ESTIMATED_HORIZONTAL_HEUGGING_PRIORITY forAxis:UILayoutConstraintAxisHorizontal];
        [_estimatedSubjectLabel setContentHuggingPriority:UILayoutPriorityDefaultLow+1 forAxis:UILayoutConstraintAxisVertical];
        [self.contentView addSubview:_estimatedSubjectLabel];
    }
    return _estimatedSubjectLabel;
}

- (void)initialize{
    [super initialize];
    
    _leftViewTopMargin = ITEM_IMAGE_TOP_MARGIN_NORMAL;
    _rightViewTopMargin = _leftViewTopMargin;
    _titleBottomMargin = TITLE_LABEL_BOTTOM_MARGINT;
    _subjectBottomMargin = SUBJECT_LABEL_BOTTOM_MARGINT;
}

- (void)installConstraints{
    if (_installed) return;
    !self.willInstallConstraints ? : self.willInstallConstraints(self);
    if (self.leftView) {
        [self.leftView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.fs_equalToSuperview().offset(self.leftMargin).priorityHigh();
            make.top.fs_equalToSuperview().offset(self.leftViewTopMargin).priorityHigh();
            if (!CGSizeEqualToSize(CGSizeZero, self.leftViewSzie)) {
                make.size.mas_lessThanOrEqualTo(self.leftViewSzie);
            }else if(!CGSizeEqualToSize(CGSizeZero, self.leftView.fs_size)){
                make.width.mas_lessThanOrEqualTo(self.leftView.fs_width);
                make.height.mas_lessThanOrEqualTo(self.leftView.fs_height);
            }else{
                make.size.mas_equalTo(self.leftView).priority(MASLayoutPriorityFittingSizeLevel);
            }
        }];
    }
    
    [self.estimatedTitleLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
        if (self.leftView) {
            make.left.mas_equalTo(self.leftView.mas_right).offset(self.mediumSpace).priorityHigh();
        }else{
            make.left.fs_equalToSuperview().offset(self.leftMargin).priorityHigh();
        }
        if (self.rightView) {
            make.right.mas_equalTo(self.rightView.mas_left).offset(-self.rightMargin).priorityHigh();
        }else{
            make.right.fs_equalToSuperview().offset(-self.rightMargin).priorityHigh();
        }
        make.top.fs_equalToSuperview().offset(self.topMargin).priorityHigh();
        make.bottom.mas_equalTo(self.estimatedSubjectLabel.mas_top).offset(-self.titleBottomMargin).priorityHigh();
        make.height.mas_greaterThanOrEqualTo(TITLE_LABEL_MIN_HEIGHT).priorityHigh();
    }];
    
    [self.estimatedSubjectLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.right.mas_equalTo(self.estimatedTitleLabel).priorityHigh();
        make.bottom.fs_equalToSuperview().offset(-self.subjectBottomMargin);
        make.height.mas_greaterThanOrEqualTo(SUBJECT_LABEL_MIN_HEIGHT).priorityHigh();
    }];
    
    if (self.rightView) {
        [self.rightView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.right.fs_equalToSuperview().offset(-self.rightMargin).priorityHigh();
            make.top.fs_equalToSuperview().offset(self.rightViewTopMargin).priorityHigh();
            make.size.mas_equalTo(self.rightView).priority(MASLayoutPriorityFittingSizeLevel);
        }];
    }
    _installed = YES;
    [self.estimatedTitleLabel layoutIfNeeded];
    if (self.estimatedTitleLabel.fs_width) self.estimatedTitleLabel.preferredMaxLayoutWidth = self.estimatedTitleLabel.fs_width;
    !self.didInstallConstraints ? : self.didInstallConstraints(self);
}

- (void)prepareForReuse{
    [super prepareForReuse];

    LabelAttributeAssign(self.estimatedSubjectLabel, self.subjectDefaultTextColor, self.subjectDefaultFont);
}

@end
