/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "UIView+FSAdd.h"
#import "FSConst.h"
#import "Masonry+FSAdd.h"
#import "FSItem.h"
#import "FSPreferences.h"
#import "FSTableViewCell.h"

static NSString *const kFillHorName = @"kFillHorName";

@interface FSTableViewCell ()
@property (nonatomic, strong, nullable) UIView *moreItem;
@property (nonatomic, strong) FSItem *editItem;
@property (nonatomic, weak) UIView *customSeparator;
@property (nonatomic, assign) BOOL moreItemOrigHidden;
@property (nonatomic, strong) CALayer *highlightedLayer;
@property (nonatomic, strong) UIColor *tmpBackgroundColor;
@end
@implementation FSTableViewCell

- (instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier{
    self = [super initWithStyle:style reuseIdentifier:reuseIdentifier];
    if (self) {
        [self initialize];
    }
    return self;
}

- (void)initialize{
    self.backgroundColor = self.fs_backgroundColor_B1;
    self.contentView.backgroundColor = self.backgroundColor;
    self.textLabel.textColor = self.fs_textColor_T4;
    
    self.selectedBackgroundView = [UIView new];
    self.selectedBackgroundView.backgroundColor = self.selectedBackgroundColor ? self.selectedBackgroundColor : self.fs_backgroundColor_B1;
}

- (void)awakeFromNib{
    [super awakeFromNib];
    
    [self initialize];
}

- (CALayer *)highlightedLayer{
    if (!_highlightedLayer) {
        _highlightedLayer = [CALayer layer];
    }
    return _highlightedLayer;
}

- (FSItem *)editItem{
    if (!_editItem) {
        _editItem = [FSItem createItemWithImage:nil];
        _editItem.userInteractionEnabled = NO;
    }
    return _editItem;
}

- (NSIndexPath *)indexPath{
    for (NSIndexPath *indexPath in self.supTableView.indexPathsForVisibleRows) {
        if ([self.supTableView cellForRowAtIndexPath:indexPath] == self) {
            return indexPath;
        }
    }
    return nil;
}

- (UIColor *)selectedBackgroundColor{
    return self.selectedBackgroundView.backgroundColor;
}

- (void)setSelectedBackgroundColor:(UIColor *)selectedBackgroundColor{
    self.selectedBackgroundView.backgroundColor = selectedBackgroundColor;
}

- (void)setUseCustomSeparator:(BOOL)useCustomSeparator{
    if (useCustomSeparator == _useCustomSeparator) return;
    _useCustomSeparator = useCustomSeparator;
    if (useCustomSeparator) {
        UIView *customSeparator = [[UIView alloc] init];
        customSeparator.backgroundColor = self.fs_dividerColor_P1;
        [self addSubview:customSeparator];
        [customSeparator mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.right.bottom.fs_equalToSuperview();
            make.height.mas_equalTo(DIVIDE_VIEW_THICKNES);
        }];
        self.customSeparator = customSeparator;
    }else{
        [self.customSeparator removeFromSuperview];
    }
}

- (void)setEditing:(BOOL)editing animated:(BOOL)animated{
    [super setEditing:editing animated:animated];
    if (self.editing) {
        [self addSubview:self.editItem];
        [self.editItem mas_makeConstraints:^(MASConstraintMaker *make) {
            make.right.equalTo(self.contentView.mas_left);
            if (self.editTopOffset) make.top.fs_equalToSuperview().offset(self.editTopOffset);
            else make.centerY.fs_equalToSuperview();
        }];
        if (self.editingHiddenRightView){
            if (!self.moreItem.hidden) {
                _moreItemOrigHidden = self.moreItem.hidden;
                self.moreItem.hidden = YES;
            }
        }
    }else{
        if (_editItem && _editItem.superview) {
            [_editItem removeFromSuperview];
            if (self.editingHiddenRightView) {
                self.moreItem.hidden = _moreItemOrigHidden;
            }
        }
    }
}

- (void)setSelected:(BOOL)selected animated:(BOOL)animated{
    [super setSelected:selected animated:animated];
    [_editItem setSelected:selected];
}

- (void)layoutSubviews{
    [super layoutSubviews];
    UIView *view = [self fs_getSubviewWithClassName:@"UITableViewCellEditControl"];
    view.hidden = YES;
    
    CGRect frame = self.bounds;
    if ([_highlightedLayer.name isEqualToString:kFillHorName]) {
        UIView *tabSuperview = self.supTableView.superview;
        frame.origin.x -= tabSuperview.fs_safeAreaInsets.left;
        frame.size.width += tabSuperview.fs_safeAreaInsets.left + tabSuperview.fs_safeAreaInsets.right;
        
        frame = [self convertRect:frame toView:tabSuperview];
    }
    _highlightedLayer.frame = frame;
    
    if (self.editDragItemImage) {
        for (UIView * view in self.subviews) {
            if ([NSStringFromClass([view class]) rangeOfString: @"Reorder"].location != NSNotFound) {
                for (UIView * subview in view.subviews) {
                    if ([subview isKindOfClass: [UIImageView class]]) {
                        ((UIImageView *)subview).image = self.editDragItemImage;
                        ((UIImageView *)subview).contentMode = UIViewContentModeCenter;
                        break;
                    }
                }
                break;
            }
        }
    }
}

- (void)touchesBegan:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event{
    [super touchesBegan:touches withEvent:event];
    [self setHighlighted:YES custom:YES];
}

- (void)touchesEnded:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event{
    [super touchesEnded:touches withEvent:event];
    [self setHighlighted:NO custom:YES];
}

- (void)touchesCancelled:(NSSet<UITouch *> *)touches withEvent:(UIEvent *)event{
    [super touchesCancelled:touches withEvent:event];
    [self setHighlighted:NO custom:YES];
}

- (void)setHighlighted:(BOOL)highlighted custom:(BOOL)custom{
    [self setHighlighted:highlighted color:self.hightlightBackgroundColor custom:custom];
}

- (void)setHighlighted:(BOOL)highlighted color:(UIColor *)color custom:(BOOL)custom{
    [self setHighlighted:highlighted color:color custom:custom fillHor:self.hightlightBackgroundColorFillHor];
}

- (void)setHighlighted:(BOOL)highlighted color:(UIColor *)color custom:(BOOL)custom fillHor:(BOOL)fillHor{
    if (custom) {
        if (color) {
            if (highlighted) {
                self.highlightedLayer.name = fillHor ? kFillHorName : nil;
                CGRect frame = self.bounds;
                if (fillHor) {
                    UIView *tabSuperview = self.supTableView.superview;
                    frame.origin.x -= tabSuperview.fs_safeAreaInsets.left;
                    frame.size.width += tabSuperview.fs_safeAreaInsets.left + tabSuperview.fs_safeAreaInsets.right;
                    
                    frame = [self convertRect:frame toView:tabSuperview];
                    [tabSuperview.layer insertSublayer:self.highlightedLayer atIndex:0];
                    if (!self.tmpBackgroundColor) self.tmpBackgroundColor = self.backgroundColor;
                    self.backgroundColor = color;
                }else{
                    [self.layer insertSublayer:self.highlightedLayer atIndex:0];
                }
                self.highlightedLayer.frame = frame;
                self.highlightedLayer.backgroundColor = color.CGColor;
            }else{
                dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.06 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                    if (fillHor){
                        self.backgroundColor = self.tmpBackgroundColor;
                        self.tmpBackgroundColor = nil;
                    }
                    [self->_highlightedLayer removeFromSuperlayer];
                });
            }
        }
    }else{
        [self setHighlighted:highlighted];
    }
}


@end
