/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "UIView+FSAdd.h"
#import "FSConst.h"
#import "Masonry+FSAdd.h"
#import "FSPreferences.h"
#import "FSTitleEstimatedTableViewCell.h"

@interface FSTitleEstimatedTableViewCell ()
@property (nonatomic, strong, readwrite) UILabel *estimatedTitleLabel;
@property (nonatomic, assign, readwrite) BOOL installed;
@end
@implementation FSTitleEstimatedTableViewCell
@synthesize installed = _installed;


- (UILabel *)estimatedTitleLabel{
    if (!_estimatedTitleLabel) {
        _estimatedTitleLabel = [UILabel new];
        _estimatedTitleLabel.numberOfLines = 0;
        _estimatedTitleLabel.textAlignment = NSTextAlignmentLeft;
        _estimatedTitleLabel.textColor = self.titleDefaultTextColor;
        _estimatedTitleLabel.font = self.titleDefaultFont;
        
        [_estimatedTitleLabel setContentHuggingPriority:ESTIMATED_HORIZONTAL_HEUGGING_PRIORITY forAxis:UILayoutConstraintAxisHorizontal];
        [_estimatedTitleLabel setContentCompressionResistancePriority:ESTIMATED_HORIZONTAL_COMPRESSION_RESISTANCE_PRIORITY forAxis:UILayoutConstraintAxisHorizontal];
        [self.contentView addSubview:_estimatedTitleLabel];
    }
    return _estimatedTitleLabel;
}

- (void)setLeftView:(__kindof UIView *)leftView{
    
    if (_leftView == leftView) {
        return;
    }
    [_leftView removeFromSuperview];
    _leftView = leftView;
    [self.contentView addSubview:leftView];
}

- (void)setRightView:(__kindof UIView *)rightView{
    if (_rightView == rightView) {
        return;
    }
    [_rightView removeFromSuperview];
    _rightView = rightView;
    [self.contentView addSubview:rightView];
}

- (void)installConstraints{
    if (_installed) return;
    !self.willInstallConstraints ? : self.willInstallConstraints(self);
    if (self.leftView) {
        [self.leftView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.fs_equalToSuperview().offset(self.leftMargin).priorityHigh();
            make.centerY.fs_equalToSuperview();
            if (!CGSizeEqualToSize(CGSizeZero, self.leftViewSzie)) {
                make.size.mas_lessThanOrEqualTo(self.leftViewSzie);
            }else if(!CGSizeEqualToSize(CGSizeZero, self.leftView.fs_size)){
                make.width.mas_lessThanOrEqualTo(self.leftView.fs_width);
                make.height.mas_lessThanOrEqualTo(self.leftView.fs_height);
            }else{
                make.size.mas_equalTo(self.leftView).priority(MASLayoutPriorityFittingSizeLevel);
            }
        }];
    }
    
    [self.estimatedTitleLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
        if (self.leftView) {
            make.left.mas_equalTo(self.leftView.mas_right).offset(self.mediumSpace).priorityHigh();
        }else{
            make.left.fs_equalToSuperview().offset(self.leftMargin).priorityHigh();
        }
        if (self.rightView) {
            make.right.mas_equalTo(self.rightView.mas_left).offset(-self.rightMargin).priorityHigh();
        }else{
            make.right.fs_equalToSuperview().offset(-self.rightMargin).priorityHigh();
        }
        make.top.fs_equalToSuperview().offset(self.topMargin).priorityHigh();
        make.bottom.fs_equalToSuperview().offset(-self.bottomMargin).priorityHigh();
        make.height.mas_greaterThanOrEqualTo(TITLE_LABEL_MIN_HEIGHT).priorityHigh();
    }];
    
    if (self.rightView) {
        [self.rightView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.right.fs_equalToSuperview().offset(-self.rightMargin).priorityHigh();
            make.centerY.fs_equalToSuperview();
            make.size.mas_equalTo(self.rightView).priority(MASLayoutPriorityFittingSizeLevel);
        }];
    }
    _installed = YES;
    [self.estimatedTitleLabel layoutIfNeeded];
    if (self.estimatedTitleLabel.fs_width) self.estimatedTitleLabel.preferredMaxLayoutWidth = self.estimatedTitleLabel.fs_width;
    !self.didInstallConstraints ? : self.didInstallConstraints(self);
}

- (void)reinstallConstraints{
    _installed = NO;
    [self installConstraints];
}

- (void)awakeFromNib {
    [super awakeFromNib];
    // Initialization code
    
    [self initialize];
}

- (void)prepareForReuse{
    [super prepareForReuse];

    LabelAttributeAssign(self.estimatedTitleLabel, self.titleDefaultTextColor, self.titleDefaultFont);
}

- (UIView *)moreItem{
    return self.rightView;
}

@end
