/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSNavigationBar.h"
#import "FSItem.h"
#import "UIColor+FSAdd.h"
#import "UIView+FSAdd.h"
#import "UIImage+FSAdd.h"
#import "FSConst.h"
#import "FSPreferences.h"

@interface FSNavigationBar ()
@property (nonatomic, strong, nullable) FSItem *leftItem;
@property (nonatomic, strong, nullable) FSItem *rightItem;

@property (nonatomic, strong, readwrite) FSItem *titleItem;
@end
@implementation FSNavigationBar

- (instancetype) initWithShowStyle:(FSNavigationBarShowStyle)showStyle{
    self = [super initWithStyle:FSToolbarStyleTop];
    if (self) {
        _showStyle = showStyle;
        
        if (self.leftItem.currentImage.size.width > 26) {
            [self addItem:self.leftItem spacing:ITEM_TOP_ICON_MARGIN atPosition:FSItemPositionLeft];
        }else{
            [self addItem:self.leftItem atPosition:FSItemPositionLeft];
        }
    }
    return self;
}

- (FSItem *)leftItem{
    if(!_leftItem){
        _leftItem = [FSItem createItemWithTitle:self.fs_navBarBackTitle];
        _leftItem.font = self.fs_standardGeneralFont_GF2;
        _leftItem.accessibilityLabel = self.fs_navBarBackTitle;
        switch (_showStyle) {
            case FSNavigationBarShowStyleUnknow:
            case FSNavigationBarShowStylePush:
            {
                if (self.fs_navBarBackIcon) {
                    [_leftItem setImage:self.fs_navBarBackIcon forState:UIControlStateNormal];
                    _leftItem.accessibilityLabel = FSLocalizedForKey(@"kBack");
                    if (self.fs_navBarBackTitle) [_leftItem setTitle:nil forState:UIControlStateNormal];
                }else{
                    [_leftItem setTitle:self.fs_navBarBackTitle forState:UIControlStateNormal];
                }
            }
                break;
            case FSNavigationBarShowStylePresent:
            {
                if (self.fs_navBarCloseIcon) {
                    [_leftItem setImage:self.fs_navBarCloseIcon forState:UIControlStateNormal];
                    _leftItem.accessibilityLabel = FSLocalizedForKey(@"kClose");
                    if (self.fs_navBarBackTitle) [_leftItem setTitle:nil forState:UIControlStateNormal];
                }
                else{
                    [_leftItem setTitle:self.fs_navBarBackTitle forState:UIControlStateNormal];
                }
            }
        }
        @weakify(self)
        _leftItem.onTapClick = ^(FSItem * _Nonnull item) {
            @strongify(self)
            if (self.delegate !=nil && [self.delegate respondsToSelector:@selector(navigationBarBackTouch)]) {
                [self.delegate navigationBarBackTouch];
            }
        };
        [BindThemeKeySel(_leftItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem *  _Nonnull target, SEL  _Nonnull sel) {
            [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
            if (target.currentImage && target.state == UIControlStateNormal) {
                [target setImage:[target.currentImage fs_imageWithTintColor:target.fs_primaryColor] forState:target.state];
            }
        }];
    }
    return _leftItem;
}

- (FSItem *)rightItem{
    if (!_rightItem) {
        _rightItem = [FSItem createItemWithTitle:nil];
        _rightItem.font = self.fs_standardGeneralFont_GF2;
        [self addItem:_rightItem atPosition:FSItemPositionRight];
        
        [BindThemeKeySel(_rightItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem *  _Nonnull target, SEL  _Nonnull sel) {
            [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
        }];
    }
    return _rightItem;
}

- (FSItem *)titleItem{
    if(!_titleItem){
        _titleItem = [FSItem createHeadlineItemWithTitle:self.title];
        _titleItem.accessibilityTraits = UIAccessibilityTraitHeader;
        _titleItem.customAccessibilityTraits = UIAccessibilityTraitHeader;
        _titleItem.accessibilityLabel = self.title;
        _titleItem.titleLabel.textAlignment = NSTextAlignmentCenter;
        [_titleItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal | UIControlStateHighlighted | UIControlStateDisabled];
        [self addItem:_titleItem atPosition:FSItemPositionCenter];
    }
    return _titleItem;
}

-(void)setTitle:(NSString *)title{
    _title = title;
    [self.titleItem setTitle:title forState:UIControlStateNormal];
    self.titleItem.accessibilityIdentifier = title;
    self.titleItem.accessibilityLabel = title;
}

- (void)updateItem{
    if (_leftItem.superview) {
        [self removeItem:_leftItem];
    }
    if (_leftItem.currentImage.size.width > ITEM_ICON_WIDTH_NORMAL) {
        [self addItem:_leftItem spacing:ITEM_TOP_ICON_MARGIN atPosition:FSItemPositionLeft];
    }else{
        [self addItem:_leftItem atPosition:FSItemPositionLeft];
    }
}

- (void)layoutSubviews{
    [super layoutSubviews];
    CGFloat LW = [self getWidthWithPosition:FSItemPositionLeft];
    CGFloat RW = [self getWidthWithPosition:FSItemPositionRight];
    
    CGFloat marginMax = LW;
    if (marginMax < RW) {
        marginMax = RW;
    }
    [self.layoutGuideHiddenBlankViewCenter mas_updateConstraints:^(MASConstraintMaker *make) {
        make.width.fs_equalToSuperview().offset(-(marginMax * 2 + ITEM_MARGIN_NORMAL * 2));
    }];
}

@end
