/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/

#import "UIColor+FSAdd.h"
#import "UIView+FSAdd.h"
#import "FSConst.h"
#import "Masonry+FSAdd.h"
#import "FSPreferences.h"
#import "FSToolbar+Private.h"

@interface FSToolbar ()
@property (nonatomic, strong) UIView *contentView;
@property (nonatomic, strong) UIView *stateBar;
@property (nonatomic, strong) UIView *contentBar;
@property (nonatomic, strong) UIView *divideView;
@property (nonatomic, assign) FSToolbarStyle style;

@property (nonatomic, assign) CGFloat spacing;
@property (nonatomic, assign) CGFloat margin;
@property (nonatomic, assign) BOOL even;
@end

@implementation FSToolbar

- (instancetype)init{
    return [self initWithFrame:CGRectZero style:FSToolbarStyleDefault];;
}

- (instancetype)initWithFrame:(CGRect)frame{
    return [self initWithFrame:frame style:FSToolbarStyleDefault];
}

- (instancetype)initWithStyle:(FSToolbarStyle)style{
    return [self initWithFrame:CGRectZero style:style];
}

- (instancetype)initWithFrame:(CGRect)frame style:(FSToolbarStyle)style{
    self = [super initWithFrame:frame];
    if (self) {
        self.style = style;
        [self setup];
    }
    return self;
}

- (void)setup{
    //fix ios11
    [self layoutIfNeeded];
    self.spacing = ITEM_MARGIN_LITTLE_NORMAL;
    self.margin = ITEM_MARGIN_NORMAL ;
    self.entiretyInterval = TOOLBAR_ITEM_INVALID_MARGIN_VALUE;
    self.stateBar.backgroundColor = self.fs_primaryColor;
    self.barTintColor = self.fs_backgroundColor_B2;
    _axis = UILayoutConstraintAxisHorizontal;
    self.even = NO;
    
    if (self.style != FSToolbarStyleDefault){
        [self setContentHuggingPriority:UILayoutPriorityFittingSizeLevel forAxis:UILayoutConstraintAxisVertical];
        
        self.contentView = [[UIView alloc] init];
        self.contentView.backgroundColor = [UIColor fs_clearColor];
        
        self.contentBar = [[UIView alloc] init];
        self.contentBar.backgroundColor = [UIColor fs_clearColor];

        [super addSubview:self.contentView];
        [self.contentView addSubview:self.contentBar];
        
        if (self.style == FSToolbarStyleTop) {
            self.stateBar = [[UIView alloc] init];
            self.stateBar.backgroundColor = [UIColor fs_clearColor];
            [self.contentView addSubview:self.stateBar];
            
            CGSize statusBarSize = [NSObject fs_getStatusBarFrame].size;
            [self.stateBar mas_makeConstraints:^(MASConstraintMaker *make) {
                make.left.right.top.mas_equalTo(self.contentView);
                if (@available(iOS 11.0, *)) {} else { make.height.mas_equalTo(MIN(statusBarSize.width, statusBarSize.height)); }
                make.bottom.mas_equalTo(self.contentBar.mas_top);
            }];
        }
        
        [self.contentBar mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.right.mas_equalTo(self.contentView);
            make.bottom.mas_equalTo(self.contentView.fs_mas_bottom);
            if (self.style == FSToolbarStyleTop){
                make.height.offset(NAVIGATION_BAR_HEIGHT_NORMAL()).priorityHigh();
                if (@available(iOS 11.0, *)) {make.top.mas_equalTo(self.fs_mas_top);}
            }else if (self.style == FSToolbarStyleBottom) {
                make.top.mas_equalTo(self.contentView);
                make.height.mas_equalTo(TAB_BAR_HEIGHT_NORMAL).priorityHigh();
            }
        }];
        
        [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.bottom.top.mas_equalTo(self);
            if (@available(iOS 11.0, *)){
                 make.left.mas_equalTo(self.mas_safeAreaLayoutGuideLeft);
                 make.right.mas_equalTo(self.mas_safeAreaLayoutGuideRight);
            }
            else make.left.right.mas_equalTo(self);
        }];
        self.hasDivide = YES;
        self.clipsToBounds = YES;
    }else{
        self.hasDivide = NO;
    }
}

- (void)setSubviewContentViewLayoutMargins:(UIEdgeInsets)subviewContentViewLayoutMargins{
    if (UIEdgeInsetsEqualToEdgeInsets(_subviewContentViewLayoutMargins, subviewContentViewLayoutMargins)) return;
    _subviewContentViewLayoutMargins = subviewContentViewLayoutMargins;
    [self.contentView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.bottom.top.mas_equalTo(self);
        if (UIEdgeInsetsEqualToEdgeInsets(_subviewContentViewLayoutMargins, UIEdgeInsetsZero)) {
            if (@available(iOS 11.0, *)){
                 make.left.mas_equalTo(self.mas_safeAreaLayoutGuideLeft);
                 make.right.mas_equalTo(self.mas_safeAreaLayoutGuideRight);
            }
            else make.left.right.mas_equalTo(self);
        }else{
            make.left.equalTo(self.contentView.superview).offset(subviewContentViewLayoutMargins.left);
            make.right.equalTo(self.contentView.superview).offset(-subviewContentViewLayoutMargins.right);
        }
    }];

}

- (void)setHasDivide:(BOOL)hasDivide{
    if (_hasDivide == hasDivide) return;
    _hasDivide = hasDivide;
    if (hasDivide) {
        UIView *divideView = [[UIView alloc] init];
        divideView.layer.zPosition = 0.01;
        self.divideView = divideView;
        if (self.style != FSToolbarStyleDefault){
            divideView.backgroundColor = self.fs_dividerColor_P2;
            [self.contentView addSubview:divideView];
            
            [divideView mas_makeConstraints:^(MASConstraintMaker *make) {
                make.left.right.mas_equalTo(divideView.superview);
                make.height.equalTo(@(DIVIDE_TOOL_BAR_THICKNES));
                if (self.style == FSToolbarStyleTop){ make.bottom.mas_equalTo(divideView.superview); }
                else if (self.style == FSToolbarStyleBottom) { make.top.mas_equalTo(divideView.superview); }
            }];
        }else{
            divideView.backgroundColor = self.fs_dividerColor_P2;
            [self addSubview:divideView];
            
            [divideView mas_makeConstraints:^(MASConstraintMaker *make) {
                make.left.right.mas_equalTo(divideView.superview);
                make.height.equalTo(@(DIVIDE_TOOL_BAR_THICKNES));
                make.bottom.mas_equalTo(divideView.superview);
            }];
        }
    }else{
        [self.divideView removeFromSuperview];
    }
}

- (void)setContentBarHeight:(CGFloat)contentBarHeight{
    _contentBarHeight = contentBarHeight;
    if (self.style != FSToolbarStyleDefault ){
        [self.contentBar mas_updateConstraints:^(MASConstraintMaker *make) {
            make.height.mas_equalTo(contentBarHeight).priorityHigh();
        }];
    }else{
        FSLog(@"contentBarHeight only applies to FSToolbarStyleTop or FSToolbarStyleBottom");
    }
}

- (BOOL)containsItem:(__kindof UIView *)item{
    return [item isDescendantOfView:self];
}

- (__kindof UIView *)getItemAtPosition:(FSItemPosition)position index:(NSUInteger)index{
    switch (position) {
        case FSItemPositionLeft:
            return [self.leftItems getItemAtIndex:index];
        case FSItemPositionCenter:
            return [self.centerItems getItemAtIndex:index];
        case FSItemPositionRight:
            return [self.rightItems getItemAtIndex:index];
    }
    return nil;
}

- (CGFloat)getWidthWithPosition:(FSItemPosition)position{
    switch (position) {
        case FSItemPositionLeft:
            if (self.layoutGuideHiddenBlankViewLeft.fs_width == 0 && self.leftItems.items.count) {
                [self.layoutGuideHiddenBlankViewLeft layoutIfNeeded];
            }
            return self.layoutGuideHiddenBlankViewLeft.fs_width;;
        case FSItemPositionCenter:
            if (self.layoutGuideHiddenBlankViewCenter.fs_width == 0 && self.centerItems.items.count) {
                [self.layoutGuideHiddenBlankViewCenter layoutIfNeeded];
            }
            return self.layoutGuideHiddenBlankViewCenter.fs_width;
        case FSItemPositionRight:
            if (self.layoutGuideHiddenBlankViewRight.fs_width == 0 && self.rightItems.items.count) {
                [self.layoutGuideHiddenBlankViewRight layoutIfNeeded];
            }
            return self.layoutGuideHiddenBlankViewRight.fs_width;;
    }
    return 0;
}

- (CGFloat)getWidthWithPosition:(FSItemPosition)position layoutIfNeeded:(BOOL)layoutIfNeeded{
    switch (position) {
        case FSItemPositionLeft:
            if (layoutIfNeeded) [self.layoutGuideHiddenBlankViewLeft layoutIfNeeded];
            break;
        case FSItemPositionCenter:
            if (layoutIfNeeded) {
                [self.layoutGuideHiddenBlankViewCenter setNeedsLayout];
                [self.layoutGuideHiddenBlankViewCenter layoutIfNeeded];
            }
            break;
        case FSItemPositionRight:
            if (layoutIfNeeded) [self.layoutGuideHiddenBlankViewRight layoutIfNeeded];
            break;
    }
    return [self getWidthWithPosition:position];
}


- (NSUInteger)getItemCountWithPosition:(FSItemPosition)position{
    switch (position) {
        case FSItemPositionLeft:
            return self.leftItems.items.count;
        case FSItemPositionCenter:
            return self.centerItems.items.count;
        case FSItemPositionRight:
            return self.rightItems.items.count;
    }
    return 0;
}

- (void)addEvenItem:(__kindof UIView *)item{
    self.even = YES;
    [self addItem:item margin:self.spacing spacing:self.spacing atPosition:FSItemPositionCenter];
    self.even = NO;
}

- (void)addEvenItem:(__kindof UIView *)item margin:(CGFloat)margin{
    [self addItem:item margin:self.margin spacing:self.spacing atPosition:FSItemPositionCenter];
}
- (void)addItem:(__kindof UIView *)item atPosition:(FSItemPosition)position{
    if (self.entiretyInterval < 0) return [self addItem:item margin:self.margin spacing:self.spacing atPosition:position];
    [self addItem:item spacing:self.entiretyInterval atPosition:position];
}

- (void)addItem:(__kindof UIView *)item spacing:(CGFloat)spacing atPosition:(FSItemPosition)position{
    [self addItem:item margin:position == FSItemPositionCenter ? 0 : spacing  spacing:spacing atPosition:position];
}

- (void)addItem:(__kindof UIView *)item margin:(CGFloat)margin spacing:(CGFloat)spacing atPosition:(FSItemPosition)position{
    if (!item) @throw [NSException exceptionWithName:NSGenericException reason:@"item is nil" userInfo:nil];
    if (![item isDescendantOfView:self]) {
        [self addSubview:item];
    }
    switch (position) {
        case FSItemPositionLeft:{
            FSToolbarItem *lastItem = [self.leftItems lastItem];
            if (!self.leftItems) self.leftItems = [[FSToolbarItems alloc] init];
            if (!self.layoutGuideHiddenBlankViewLeft){
                self.layoutGuideHiddenBlankViewLeft = [FSLayoutGuideHiddenBlankView new];
                self.layoutGuideHiddenBlankViewLeft.hidden = YES;
                [self insertSubview:self.layoutGuideHiddenBlankViewLeft atIndex:0];
                [self.layoutGuideHiddenBlankViewLeft mas_remakeConstraints:^(MASConstraintMaker *make) {
                    if (_axis == UILayoutConstraintAxisHorizontal) {
                        make.top.bottom.fs_equalToSuperview();
                        make.left.equalTo(self.layoutGuideHiddenBlankViewLeft.superview.fs_mas_left);
                    }else{
                        make.left.right.fs_equalToSuperview();
                        make.top.equalTo(self.layoutGuideHiddenBlankViewLeft.superview.fs_mas_top);
                    }
                }];
            }
            FSToolbarItem *barItem = [[FSToolbarItem alloc] initWithItem:item marginCconstraint:nil];
            if (lastItem) barItem.margin = spacing;
            else barItem.margin = margin;
            
            [item mas_remakeConstraints:^(MASConstraintMaker *make) {
                [lastItem.marginCconstraint deactivate];
                if (_axis == UILayoutConstraintAxisHorizontal) {
                    make.left.mas_equalTo(lastItem ? lastItem.item.mas_right : self.layoutGuideHiddenBlankViewLeft).offset(barItem.margin);
                    make.centerY.mas_equalTo(lastItem ? lastItem.item : item.superview);
                    barItem.marginCconstraint = make.right.mas_equalTo(self.layoutGuideHiddenBlankViewLeft).offset(0);
                }else{
                    make.top.mas_equalTo(lastItem ? lastItem.item.mas_bottom : self.layoutGuideHiddenBlankViewLeft).offset(barItem.margin);
                    make.centerX.mas_equalTo(lastItem ? lastItem.item : item.superview);
                    barItem.marginCconstraint = make.bottom.mas_equalTo(self.layoutGuideHiddenBlankViewLeft).offset(0);
                }
            }];
            [self.leftItems addItem:barItem];
        }
            break;
        case FSItemPositionCenter:{
            if (!self.centerItems) self.centerItems = [[FSToolbarItems alloc] init];
            if (!self.layoutGuideHiddenBlankViewCenter){
                self.layoutGuideHiddenBlankViewCenter = [FSLayoutGuideHiddenBlankView new];
                self.layoutGuideHiddenBlankViewCenter.hidden = YES;
                [self insertSubview:self.layoutGuideHiddenBlankViewCenter atIndex:0];
                [self.layoutGuideHiddenBlankViewCenter mas_makeConstraints:^(MASConstraintMaker *make) {
                    if (_axis == UILayoutConstraintAxisHorizontal) {
                        make.centerX.top.bottom.fs_equalToSuperview();
                    }else{
                        make.centerY.left.right.fs_equalToSuperview();
                    }
                }];
            }
            FSToolbarItem *lastItem = [self.centerItems lastItem];
            FSToolbarItem *barItem = [[FSToolbarItem alloc] initWithItem:item marginCconstraint:nil];
            if (lastItem) barItem.margin = spacing;
            else barItem.margin = margin;
            
            [item mas_remakeConstraints:^(MASConstraintMaker *make) {
                [lastItem.marginCconstraint deactivate];
                if (_axis == UILayoutConstraintAxisHorizontal) {
                    make.left.mas_equalTo(lastItem ? lastItem.item.mas_right : self.layoutGuideHiddenBlankViewCenter).offset(lastItem ? barItem.margin : 0);
                    barItem.marginCconstraint = make.right.mas_equalTo(self.layoutGuideHiddenBlankViewCenter).offset(0);
                    make.centerY.mas_equalTo(lastItem ? lastItem.item : item.superview);
                }else{
                    make.top.mas_equalTo(lastItem ? lastItem.item.mas_bottom : self.layoutGuideHiddenBlankViewCenter).offset(lastItem ? barItem.margin : 0);
                    barItem.marginCconstraint = make.bottom.mas_equalTo(self.layoutGuideHiddenBlankViewCenter).offset(0);
                    make.centerX.mas_equalTo(lastItem ? lastItem.item : item.superview);
                }
            }];

            [self.centerItems addItem:barItem];
        }
            break;
        case FSItemPositionRight:{
            if (!self.rightItems) self.rightItems = [[FSToolbarItems alloc] init];
            if (!self.layoutGuideHiddenBlankViewRight){
                self.layoutGuideHiddenBlankViewRight = [FSLayoutGuideHiddenBlankView new];
                self.layoutGuideHiddenBlankViewRight.hidden = YES;
                [self insertSubview:self.layoutGuideHiddenBlankViewRight atIndex:0];
                [self.layoutGuideHiddenBlankViewRight mas_remakeConstraints:^(MASConstraintMaker *make) {
                    if (_axis == UILayoutConstraintAxisHorizontal) {
                        make.top.bottom.fs_equalToSuperview();
                        make.right.equalTo(self.layoutGuideHiddenBlankViewRight.superview.fs_mas_right);
                    }else{
                        make.left.right.fs_equalToSuperview();
                        make.bottom.equalTo(self.layoutGuideHiddenBlankViewRight.superview.fs_mas_bottom);
                    }
                }];
            }
            FSToolbarItem *lastItem = [self.rightItems lastItem];
            FSToolbarItem *barItem = [[FSToolbarItem alloc] initWithItem:item marginCconstraint:nil];
            if (lastItem) barItem.margin = spacing;
            else barItem.margin = margin;
            
            [item mas_remakeConstraints:^(MASConstraintMaker *make) {
                [lastItem.marginCconstraint deactivate];
                if (_axis == UILayoutConstraintAxisHorizontal) {
                    make.right.mas_equalTo(lastItem ? lastItem.item.mas_left : self.layoutGuideHiddenBlankViewRight).offset(-barItem.margin);
                    make.centerY.mas_equalTo(lastItem ? lastItem.item : item.superview);
                    barItem.marginCconstraint = make.left.mas_equalTo(self.layoutGuideHiddenBlankViewRight).offset(0);
                }else{
                    make.bottom.mas_equalTo(lastItem ? lastItem.item.mas_top : self.layoutGuideHiddenBlankViewRight).offset(-barItem.margin);
                    make.centerX.mas_equalTo(lastItem ? lastItem.item : item.superview);
                    barItem.marginCconstraint = make.top.mas_equalTo(self.layoutGuideHiddenBlankViewRight).offset(0);
                }
            }];
            [self.rightItems addItem:barItem];
        }
            break;
    }
    
    if (self.layoutGuideHiddenBlankViewCenter && self.layoutGuideHiddenBlankViewRight && self.layoutGuideHiddenBlankViewLeft){
        [self.layoutGuideHiddenBlankViewCenter mas_updateConstraints:^(MASConstraintMaker *make) {
            make.left.greaterThanOrEqualTo(self.layoutGuideHiddenBlankViewLeft.mas_right).offset(10);
            make.right.lessThanOrEqualTo(self.layoutGuideHiddenBlankViewRight.mas_left).offset(-10);
        }];
        
        [self.layoutGuideHiddenBlankViewCenter setContentCompressionResistancePriority:UILayoutPriorityDefaultLow forAxis:UILayoutConstraintAxisHorizontal];
        [self.layoutGuideHiddenBlankViewLeft setContentCompressionResistancePriority:UILayoutPriorityDefaultHigh forAxis:UILayoutConstraintAxisHorizontal];
        [self.layoutGuideHiddenBlankViewRight setContentCompressionResistancePriority:UILayoutPriorityDefaultHigh forAxis:UILayoutConstraintAxisHorizontal];
    }
}

- (void)removeItem:(__kindof UIView *)item atPosition:(FSItemPosition)position{
    [item removeFromSuperview];
    switch (position) {
        case FSItemPositionLeft:
            if ([self.leftItems removeItem:item]) {
                if (!self.leftItems.items.count) {
                    [self cleanAuxiliaryViewForPosition:position];
                }else{
                    [self readdItemsForPosition:position];
                }
            }
        case FSItemPositionCenter:
            if ([self.centerItems removeItem:item]) {
                if (!self.centerItems.items.count) {
                    [self cleanAuxiliaryViewForPosition:position];
                }else{
                    [self readdItemsForPosition:position];
                }
            }

        case FSItemPositionRight:
            if ([self.rightItems removeItem:item]) {
                if (!self.rightItems.items.count) {
                    [self cleanAuxiliaryViewForPosition:position];
                }else{
                    [self readdItemsForPosition:position];
                }
            }
    }
}

- (void)setAxis:(UILayoutConstraintAxis)axis{
    if (self.style != FSToolbarStyleDefault ){
        FSLog(@"axis only only applies to FSToolbarStyleDefault");
        return;
    }
    if (_axis == axis) return;
    _axis = axis;
    [self readdItemsForPosition:FSItemPositionLeft];
    [self readdItemsForPosition:FSItemPositionCenter];
    [self readdItemsForPosition:FSItemPositionRight];
}

- (void)readdItemsForPosition:(FSItemPosition)position{
    switch (position) {
        case FSItemPositionLeft:{
            NSArray *tmp = self.leftItems.items.copy;
            [self removeItemAtPosition:position];
            for (FSToolbarItem *barItem in tmp) {
                [self addItem:barItem.item margin:barItem.margin spacing:barItem.margin atPosition:position];
            }
        }
            break;
        case FSItemPositionCenter:{
            NSArray *tmp = self.centerItems.items.copy;
            [self removeItemAtPosition:position];
            for (FSToolbarItem *barItem in tmp) {
                [self addItem:barItem.item margin:barItem.margin spacing:barItem.margin atPosition:position];
            }
        }
            break;
        case FSItemPositionRight:{
            NSArray *tmp = self.rightItems.items.copy;
            [self removeItemAtPosition:position];
            for (FSToolbarItem *barItem in tmp) {
                [self addItem:barItem.item margin:barItem.margin spacing:barItem.margin atPosition:position];
            }
        }
            break;
    }
}

- (void)removeItemAtPosition:(FSItemPosition)position index:(NSUInteger)index{
    __kindof UIView *item = nil;
    switch (position) {
        case FSItemPositionLeft:
            item = [self.leftItems getItemAtIndex:index];
            break;
        case FSItemPositionCenter:
            item = [self.centerItems getItemAtIndex:index];
            break;
        case FSItemPositionRight:
            item = [self.rightItems getItemAtIndex:index];
            break;
    }
    [self removeItem:item atPosition:position];
}

- (void)removeItem:(__kindof UIView *)item{
    FSItemPosition position = -1;
    if ([self.leftItems containsItem:item]) {
        position = FSItemPositionLeft;
    }
    if ([self.centerItems containsItem:item]) {
        position = FSItemPositionCenter;
    }
    if ([self.rightItems containsItem:item]) {
        position = FSItemPositionRight;
    }
    if (position == -1) FSLog(@"item did not add");
    [self removeItem:item atPosition:position];
}

- (void)removeAllItems{
    [self removeItemAtPosition:FSItemPositionLeft];
    [self removeItemAtPosition:FSItemPositionCenter];
    [self removeItemAtPosition:FSItemPositionRight];
}

- (void)removeItemAtPosition:(FSItemPosition)position{
    switch (position) {
        case FSItemPositionLeft:
            if (self.leftItems) {
                for (FSToolbarItem *item in self.leftItems.items) {
                    [item.item removeFromSuperview];
                }
                self.leftItems = nil;
            }
            break;
        case FSItemPositionCenter:
            if (self.centerItems) {
                for (FSToolbarItem *item in self.centerItems.items) {
                    [item.item removeFromSuperview];
                }
                self.centerItems = nil;
            }
            break;
        case FSItemPositionRight:
            if (self.rightItems) {
                for (FSToolbarItem *item in self.rightItems.items) {
                    [item.item removeFromSuperview];
                }
                self.rightItems = nil;
            }
            break;
    }
    [self cleanAuxiliaryViewForPosition:position];
}

- (void)cleanAuxiliaryViewForPosition:(FSItemPosition)position{
    switch (position) {
        case FSItemPositionLeft:
            [self.layoutGuideHiddenBlankViewLeft removeFromSuperview];
            self.layoutGuideHiddenBlankViewLeft = nil;
            break;
        case FSItemPositionCenter:
            [self.layoutGuideHiddenBlankViewCenter removeFromSuperview];
            self.layoutGuideHiddenBlankViewCenter = nil;
            break;
        case FSItemPositionRight:
            [self.layoutGuideHiddenBlankViewRight removeFromSuperview];
            self.layoutGuideHiddenBlankViewRight = nil;
            break;
    }
}

- (void)adjustStateBarHeightForStyleTop{
    if (self.style == FSToolbarStyleTop) {
        
        if (@available(iOS 11.0, *)) {} else {
            CGSize statusBarSize = [[UIApplication sharedApplication] statusBarFrame].size;
            CGRect rect = [self.stateBar convertRect:self.stateBar.frame toView:self.window];
            
            CGFloat height = MIN(statusBarSize.width, statusBarSize.height);
            if (rect.origin.y > height) height = 0;
            [self.stateBar mas_updateConstraints:^(MASConstraintMaker *make) {
                make.height.offset(height);
            }];
        }
    }
}

- (UIColor *)barTintColor{
    return self.backgroundColor;
}

- (void)setBarTintColor:(UIColor *)barTintColor{
    self.backgroundColor = barTintColor;
}

- (__kindof UIView *)viewWithTag:(NSInteger)tag{
    if (self.style != FSToolbarStyleDefault){
        return [self.contentBar viewWithTag:tag];
    }
    return [super viewWithTag:tag];
}

- (void)insertSubview:(UIView *)view atIndex:(NSInteger)index{
    if (self.style != FSToolbarStyleDefault){
        [self.contentBar insertSubview:view atIndex:index];
        return;
    }
    [super insertSubview:view atIndex:index];
}

- (void)addSubview:(UIView *)view{
    if (self.style != FSToolbarStyleDefault){
        [self.contentBar addSubview:view];
        return;
    }
    [super addSubview:view];
}

#pragma mark - UIViewAdapter
- (void)layoutSubviews{
    [super layoutSubviews];
    [self adjustStateBarHeightForStyleTop];
    if (self.adapter && [self.adapter respondsToSelector:@selector(layoutSubviewsFromView:)]) {
        [self.adapter layoutSubviewsFromView:self];
    }
}

- (void)traitCollectionDidChange:(UITraitCollection *)previousTraitCollection{
    [super traitCollectionDidChange:previousTraitCollection];
    if (self.adapter && [self.adapter respondsToSelector:@selector(traitCollectionDidChange:fromView:)]) {
        [self.adapter traitCollectionDidChange:previousTraitCollection fromView:self];
    }
}
#pragma mark - API DEPRECATED

- (__kindof UIView *)getItemForIndex:(NSUInteger)index position:(FSItemPosition)position {
    return [self getItemAtPosition:position index:index];
}
- (CGFloat)getWidthForPosition:(FSItemPosition)position{
    return [self getWidthWithPosition:position];
}
- (NSUInteger)getItemCountForPosition:(FSItemPosition)position {
    return [self getItemCountWithPosition:position];
}
- (void)addItem:(__kindof UIView *)item position:(FSItemPosition)position{
    [self addItem:item atPosition:position];
}
- (void)addItem:(__kindof UIView *)item interval:(CGFloat)interval position:(FSItemPosition)position {
    [self addItem:item spacing:interval atPosition:position];
}
- (void)removeItem:(__kindof UIView *)item position:(FSItemPosition)position{
    [self removeItem:item atPosition:position];
}
- (void)removeItemByIndex:(NSUInteger)index position:(FSItemPosition)position  {
    [self removeItemAtPosition:position index:index];
}
- (void)removeItemsByPosition:(FSItemPosition)position {
    [self removeItemAtPosition:position];
}
@end

@implementation FSLayoutGuideHiddenBlankView @end

@implementation FSToolbarItem

- (instancetype)initWithItem:(__kindof UIView *)item marginCconstraint:(MASConstraint *)marginCconstraint{
    self = [super init];
    if (self) {
        _item = item;
        _marginCconstraint = marginCconstraint;
    }
    return self;
}

@end

@implementation FSToolbarItems

- (instancetype)init
{
    self = [super init];
    if (self) {
        _items = [NSArray array];
    }
    return self;
}

- (BOOL)containsItem:(__kindof UIView *)item{
    for (FSToolbarItem *barItem in _items) {
        if (barItem.item == item) {
            return YES;
        }
    }
    return NO;
}

- (__kindof UIView *)getItemAtIndex:(NSUInteger)index{
    if (index > _items.count - 1) return nil;
    return _items[index].item;
}

- (void)addItem:(FSToolbarItem *)item{
    if (!item || !item.item || [self containsItem:item.item])return;
    NSMutableArray *tmp = [NSMutableArray arrayWithArray:_items];
    [tmp addObject:item];
    _items = tmp.copy;
}

- (BOOL)removeItem:(__kindof UIView *)item{
    NSMutableArray *tmp = [NSMutableArray arrayWithArray:_items];
    for (FSToolbarItem *barItem in tmp) {
        if (barItem.item == item) {
            [tmp removeObject:barItem];
            _items = tmp.copy;
            return YES;
        }
    }
    return NO;
}

- (FSToolbarItem *)getItemForItem:(__kindof UIView *)item{
    for (FSToolbarItem *barItem in _items) {
        if (barItem.item == item) {
            return barItem;
        }
    }
    return nil;
}

- (FSToolbarItem *)firstItem{
    return [_items firstObject];
}

- (FSToolbarItem *)lastItem{
    return [_items lastObject];
}

@end
