/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSPresentedViewControllerAnimator.h"
#import "FSTableViewController.h"
#import "FSPreferences.h"
#import "FSMacro.h"
#import "Masonry+FSAdd.h"
#import "NSObject+FSAdd.h"
#import "UIView+FSAdd.h"

static NSString *const kTransitionPanGestureKey = @"kTransitionPanGestureKey";

@interface FSPresentationController : UIPresentationController <UIGestureRecognizerDelegate,UINavigationControllerDelegate>{
    BOOL _viewDidLayoutSubviews;
}
@property (nonatomic, assign) FSModalPresentationStyle modalPresentationStyle;
@property (nonatomic, assign) CGFloat maxVerticalHeigh;
@property (nonatomic, assign) CGSize size;
@property (nonatomic, weak) __kindof UIView *viewCtrl;
@property (nonatomic, weak) UIViewController *sourceViewController;
@property (nonatomic, assign) BOOL shouldAutorotate;
@property (nonatomic, assign) double currentRotation;
@property (nonatomic, assign) UIInterfaceOrientation lastPreferredInterfaceOrientation;

@property (nonatomic, strong) UIView *dimmingView;
@property (nonatomic, strong) UIView *safeFillView;
@property (nonatomic, assign) CGSize srcSize;
@property (nonatomic, assign) CGPoint srcPoint;
@property (nonatomic, assign) CGPoint lastPoint;
@property (nonatomic, assign) BOOL isChangeSize;
@property (nonatomic, assign) BOOL isStopPanUp;
@property (nonatomic, assign) BOOL isStopPanDown;
@property (nonatomic, assign) BOOL isDismissLayoutSubviews;
@property (nonatomic, copy) dispatch_block_t dismissViewControllerCompletion;
@property (nonatomic, copy) dispatch_block_t dismissSplitViewControllerCompletion;

- (UIInterfaceOrientation)preferredInterfaceOrientationForPresentation;
- (UIInterfaceOrientation)currentPreferredInterfaceOrientation;
@end

@implementation FSPresentationController

- (instancetype)initWithPresentedViewController:(UIViewController *)presentedViewController presentingViewController:(UIViewController *)presentingViewController{
    self = [super initWithPresentedViewController:presentedViewController presentingViewController:presentingViewController];
    if (self) {
        self.shouldAutorotate = [self.presentedViewController shouldAutorotate];
        self.currentRotation = 0;
        if ([self.presentedViewController isKindOfClass:[FSNavigationController class]]) {
            FSNavigationController *nav = (FSNavigationController *)self.presentedViewController;
            nav.delegate = self;
        }
    }
    return self;
}

- (nullable id <UIViewControllerAnimatedTransitioning>)navigationController:(UINavigationController *)navigationController
                                            animationControllerForOperation:(UINavigationControllerOperation)operation
                                                         fromViewController:(UIViewController *)fromVC
                                                           toViewController:(UIViewController *)toVC  NS_AVAILABLE_IOS(7_0){
    switch (operation) {
        case UINavigationControllerOperationNone:
        {
        }
            break;
        case UINavigationControllerOperationPush:{
            UIScrollView *observerScrollView = [self getScrollViewPresentedView];
            if (observerScrollView) {
                [((UIScrollView *)observerScrollView) addObserver:self forKeyPath:@keypath(observerScrollView, contentOffset) options:NSKeyValueObservingOptionNew context:(__bridge void * _Nullable)(observerScrollView)];
                for (int i = 0; i < observerScrollView.gestureRecognizers.count; i++) {
                    UIGestureRecognizer *gestureRecognizer = observerScrollView.gestureRecognizers[i];
                    if ([gestureRecognizer  isKindOfClass:NSClassFromString(@"UIScrollViewPanGestureRecognizer")])
                    {
                        [gestureRecognizer addTarget:self action:@selector(handleGesture:)];
                    }
                }
            }
        }
            break;
        case UINavigationControllerOperationPop:{
            if ([[self getScrollViewPresentedView] isKindOfClass:[UIScrollView class]]) {
                @try {
                    [[self getScrollViewPresentedView] removeObserver:self forKeyPath:@"contentOffset"];
                } @catch (NSException *exception) {
                }
            }
        }
            break;
        default:
            break;
    }
    
    return nil;

}

- (UIInterfaceOrientation)preferredInterfaceOrientationForPresentation{
    return [self.presentedViewController preferredInterfaceOrientationForPresentation];
}

- (UIInterfaceOrientation)currentPreferredInterfaceOrientation{
    #if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
    if (@available(iOS 13.0, *)) {
        if (self.containerView.window.windowScene) return self.containerView.window.windowScene.interfaceOrientation;
    }
    #endif
    return [UIApplication sharedApplication].statusBarOrientation;
}

- (UIScrollView *)getScrollViewPresentedView{
    UIView *presentedView = self.presentedView;
    if ([presentedView isKindOfClass:[UIScrollView class]]) {
        return (UIScrollView *)presentedView;
   }else  if ([self.presentedViewController isKindOfClass:[FSTableViewController class]]) {
       FSTableViewController *vc = (FSTableViewController *)self.presentedViewController;
       return vc.tableView;
   }else if ([self.presentedViewController isKindOfClass:[FSNavigationController class]]) {
       FSNavigationController *navCtrl = (FSNavigationController *)self.presentedViewController;
       if ([navCtrl.visibleViewController isKindOfClass:[FSTableViewController class]]) {
           FSTableViewController *vc = (FSTableViewController *)navCtrl.visibleViewController;
           return vc.tableView;
       }else if ([navCtrl.visibleViewController isKindOfClass:[UIViewController class]]) {
           if ([navCtrl.visibleViewController.view isKindOfClass:[UIScrollView class]]) {
               return (UIScrollView *)navCtrl.visibleViewController.view;
               
           }
       }
   }
    return nil;
}

- (BOOL)shouldPresentInFullscreen{
    if (self.modalPresentationStyle == FSModalPresentationFullViewCtrl) {return CGRectEqualToRect([UIScreen mainScreen].bounds, self.viewCtrl.bounds); }
    return YES;
}

- (void)presentationTransitionWillBegin{
    [super presentationTransitionWillBegin];
    _isDismissLayoutSubviews = NO;
    if (self.modalPresentationStyle == FSModalPresentationFullViewCtrl){
        self.presentedViewController.view.hidden = NO;
        self.presentedView.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleTopMargin | UIViewAutoresizingFlexibleRightMargin | UIViewAutoresizingFlexibleBottomMargin | UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleWidth;
        UIInterfaceOrientation preferredInterfaceOrientationForPresentation = [self preferredInterfaceOrientationForPresentation];
        UIInterfaceOrientation currentPreferredInterfaceOrientation = [self currentPreferredInterfaceOrientation];
        if (self.preferredInterfaceOrientationForPresentation != currentPreferredInterfaceOrientation) {
            int rotation = ([self convertInterfaceOrientation:preferredInterfaceOrientationForPresentation] + [self convertInterfaceOrientation:currentPreferredInterfaceOrientation]) % 4;
            if (rotation != 0) {
                self.currentRotation = M_PI * rotation * 0.5;
                self.presentedView.transform = CGAffineTransformMakeRotation(self.currentRotation);
            }
        }
        self.lastPreferredInterfaceOrientation = currentPreferredInterfaceOrientation;
    }else{
        [self.containerView addSubview:self.dimmingView];
        self.dimmingView.backgroundColor = [UIColor colorWithWhite:0.0 alpha:0.4f];
        self.dimmingView.center = self.containerView.center;
        self.dimmingView.bounds = self.containerView.bounds;
        self.safeFillView.fs_top = self.dimmingView.fs_height;
        self.safeFillView.fs_size = self.presentedView.fs_size;
        
        if ( @available(iOS 11.0, *) ){
            [self.containerView addSubview:self.safeFillView];
            self.safeFillView.backgroundColor = [self getScrollViewPresentedView] ? [self getScrollViewPresentedView].backgroundColor : self.presentedView.backgroundColor;
            
            [self.presentedViewController.transitionCoordinator animateAlongsideTransition:^(id<UIViewControllerTransitionCoordinatorContext>  _Nonnull context) {
                self.safeFillView.fs_bottom = self.dimmingView.fs_height;
            } completion:^(id<UIViewControllerTransitionCoordinatorContext>  _Nonnull context) {
            }];
        }
    }
    

}

- (void)presentationTransitionDidEnd:(BOOL)completed
{
    [super presentationTransitionDidEnd:completed];
    if (self.modalPresentationStyle == FSModalPresentationBottom) {
        UIScrollView *observerScrollView = [self getScrollViewPresentedView];
        if (observerScrollView) {
            [((UIScrollView *)observerScrollView) addObserver:self forKeyPath:@keypath(observerScrollView, contentOffset) options:NSKeyValueObservingOptionNew context:(__bridge void * _Nullable)(observerScrollView)];
            for (int i = 0; i < observerScrollView.gestureRecognizers.count; i++) {
                UIGestureRecognizer *gestureRecognizer = observerScrollView.gestureRecognizers[i];
                if ([gestureRecognizer  isKindOfClass:NSClassFromString(@"UIScrollViewPanGestureRecognizer")])
                {
                    [gestureRecognizer addTarget:self action:@selector(handleGesture:)];
                }
            }
        }
    }else if (self.modalPresentationStyle == FSModalPresentationCenter){
        UIPanGestureRecognizer *gesture = [self.presentedView fs_getAssociatedForKey:kTransitionPanGestureKey];
        if (gesture) {
            [self.presentedView removeGestureRecognizer:gesture];
        }
        gesture = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(handleGesture:)];
        gesture.delegate = self;
        [self.presentedView addGestureRecognizer:gesture];
        [self.presentedView fs_addAssociatedObject:gesture forKey:kTransitionPanGestureKey];
    }

}

- (CGRect)frameOfPresentedViewInContainerView{
    if (self.modalPresentationStyle == FSModalPresentationFullViewCtrl) return [super frameOfPresentedViewInContainerView];
    CGRect rect = self.presentedView.bounds;
    return rect;
}

- (void)dismissalTransitionWillBegin{
    [super dismissalTransitionWillBegin];
    _isDismissLayoutSubviews = YES;
    if (self.modalPresentationStyle == FSModalPresentationFullViewCtrl){
        self.presentedViewController.view.hidden = YES;
        if (self.currentRotation != 0) {
            self.presentedView.transform = CGAffineTransformIdentity;
        }
    }else{
        if (self.modalPresentationStyle == FSModalPresentationBottom){
            if ([[self getScrollViewPresentedView] isKindOfClass:[UIScrollView class]]) {
                @try {
                    if ([self.presentedViewController isKindOfClass:[FSNavigationController class]]) {
                        FSNavigationController *nav = (FSNavigationController *)self.presentedViewController;
                        for (UIViewController *vc in nav.viewControllers) {
                            if ([vc isKindOfClass:[FSTableViewController class]]) {
                                FSTableViewController *vc1 = (FSTableViewController *)vc;
                                [vc1.tableView removeObserver:self forKeyPath:@"contentOffset"];
                            }else if ([vc isKindOfClass:[UIViewController class]]) {
                                if ([vc.view isKindOfClass:[UIScrollView class]]) {
                                    [vc.view removeObserver:self forKeyPath:@"contentOffset"];
                                }
                            }
                        }
                    }else{
                        [[self getScrollViewPresentedView] removeObserver:self forKeyPath:@"contentOffset"];
                    }
                } @catch (NSException *exception) {
                }
                for (int i = 0; i < self.presentedView.gestureRecognizers.count; i++) {
                    UIGestureRecognizer *gestureRecognizer = [self getScrollViewPresentedView].gestureRecognizers[i];
                    if ([gestureRecognizer  isKindOfClass:NSClassFromString(@"UIScrollViewPanGestureRecognizer")])
                    {
                        [gestureRecognizer removeTarget:self action:@selector(handleGesture:)];
                    }
                }
            }else{
                UIPanGestureRecognizer *gesture = [[self getScrollViewPresentedView] fs_getAssociatedForKey:kTransitionPanGestureKey];
                [gesture removeTarget:self action:@selector(handleGesture:)];
                if (gesture) {
                    [self.presentedView removeGestureRecognizer:gesture];
                }
            }
        }
        [self.presentedViewController.transitionCoordinator animateAlongsideTransition:^(id<UIViewControllerTransitionCoordinatorContext>  _Nonnull context) {
            self.dimmingView.alpha = 0.0;
            if ( @available(iOS 11.0, *) ) self.safeFillView.hidden = YES;
        } completion:^(id<UIViewControllerTransitionCoordinatorContext>  _Nonnull context) {
            
        }];
    }
}

- (void)containerViewWillLayoutSubviews{
    [super containerViewWillLayoutSubviews];
    if (self.modalPresentationStyle == FSModalPresentationFullViewCtrl){
        #if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
        if (@available(iOS 13.0, *)) {
            if (self.containerView.window.windowScene && self.containerView.window.windowScene.activationState == UISceneActivationStateBackground) return;
        }
        #endif
        UIInterfaceOrientation preferredInterfaceOrientationForPresentation = [self preferredInterfaceOrientationForPresentation];
        UIInterfaceOrientation currentPreferredInterfaceOrientation = [self currentPreferredInterfaceOrientation];
        if (self.lastPreferredInterfaceOrientation != preferredInterfaceOrientationForPresentation) {
            if (self.preferredInterfaceOrientationForPresentation != currentPreferredInterfaceOrientation) {
                if (self.currentRotation != 0) {
                    self.presentedView.transform = CGAffineTransformIdentity;
                }
                int rotation = ([self convertInterfaceOrientation:preferredInterfaceOrientationForPresentation] + [self convertInterfaceOrientation:currentPreferredInterfaceOrientation]) % 4;
                if (rotation != 0) {
                    self.currentRotation = M_PI * rotation * 0.5;
                    self.presentedView.transform = CGAffineTransformMakeRotation(self.currentRotation);
                }
            }
        }
        self.lastPreferredInterfaceOrientation = currentPreferredInterfaceOrientation;
    }
}

- (void)containerViewDidLayoutSubviews{
    [super containerViewDidLayoutSubviews];
    if (_isDismissLayoutSubviews) {
        if (self.modalPresentationStyle == FSModalPresentationCenter) {
            [self.presentedView mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.top.equalTo(self.presentedView.superview.mas_bottom);
                make.center.equalTo(self.presentedView.superview).priorityHigh();
                make.size.mas_equalTo(self.presentedView.fs_size).priorityHigh();
            }];
        }else if (self.modalPresentationStyle == FSModalPresentationBottom) {
            self.presentedView.fs_top =  self.presentedView.superview.fs_height;
        }
        return;
    }
    if (self.modalPresentationStyle == FSModalPresentationFullViewCtrl){
    }else{
        if (_isChangeSize) return;
        self.dimmingView.center = self.containerView.center;
        self.dimmingView.bounds = self.containerView.bounds;
        if (self.modalPresentationStyle == FSModalPresentationBottom) {
            if ([self.presentedView isKindOfClass:[UIScrollView class]]) {
                ((UIScrollView *)self.presentedView).contentOffset = CGPointZero;
            }
        }
        
        if (self.presentedView.superview == self.containerView && self.containerView) {
            
            if ( @available(iOS 11.0, *) ){
                [_safeFillView mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.size.mas_equalTo(self.presentedView.fs_size).priorityMedium();
                    if (self.modalPresentationStyle == FSModalPresentationCenter){
                        make.center.mas_equalTo(self.presentedView).priorityHigh();
                    }else{
                        make.top.mas_equalTo(self.presentedView).priorityHigh();
                        make.bottom.mas_equalTo(self.safeFillView.superview).priorityHigh();
                        make.left.right.mas_equalTo(self.presentedView);
                    }
                }];
            }
            
            if (self.modalPresentationStyle == FSModalPresentationCenter) {
                [self.presentedView mas_remakeConstraints:^(MASConstraintMaker *make) {
                    if (!CGSizeEqualToSize(self.size, CGSizeZero)) {
                        make.center.mas_equalTo(self.presentedView.superview).priorityHigh();
                        make.size.mas_equalTo(self.size).priorityHigh();
                    }else{
                        make.center.mas_equalTo(self.presentedView.superview).priorityHigh();
                        make.size.mas_equalTo(self.presentedView.fs_size).priorityHigh();
                    }
                }];
            }else{
                if (self.presentedView.traitCollection.horizontalSizeClass && DEVICE_iPAD) {
                    [self.presentedView mas_remakeConstraints:^(MASConstraintMaker *make) {
                        make.centerX.mas_equalTo(self.presentedView.superview);
                        make.bottom.mas_equalTo(self.presentedView.superview.fs_mas_bottom).priorityHigh();
                        make.width.mas_equalTo(self.presentedView.superview);
                        if (!CGSizeEqualToSize(self.size, CGSizeZero)) {
                            make.height.mas_equalTo(self.size.height).priorityHigh();
                        }else{
                            make.height.mas_equalTo(self.presentedView.fs_size.height).priorityHigh();
                        }
                        make.top.greaterThanOrEqualTo(self.presentedView.superview.fs_mas_top).offset(NAVIGATION_BAR_HEIGHT_NORMAL());
                    }];
                }else{
                    [self.presentedView mas_remakeConstraints:^(MASConstraintMaker *make) {
                        make.centerX.mas_equalTo(self.presentedView.superview);
                        make.bottom.mas_equalTo(self.presentedView.superview.fs_mas_bottom).priorityHigh();
                        if (!CGSizeEqualToSize(self.size, CGSizeZero)) {
                            make.size.mas_equalTo(self.size).priorityHigh();
                        }else{
                            make.size.mas_equalTo(self.presentedView.fs_size).priorityHigh();
                        }
                        make.top.greaterThanOrEqualTo(self.presentedView.superview.fs_mas_top).offset(NAVIGATION_BAR_HEIGHT_NORMAL());
                    }];
                }
            }
        }else{
            if (self.modalPresentationStyle == FSModalPresentationCenter) {
                self.presentedView.center = self.containerView.center;
            }else{
                if (_maxVerticalHeigh < self.presentedView.bounds.size.height) {
                    _maxVerticalHeigh = self.presentedView.bounds.size.height;
                }
                CGFloat width = MIN(self.containerView.frame.size.height, self.containerView.frame.size.width);
                CGFloat heigh = (self.presentedView.bounds.size.height > (self.containerView.bounds.size.height - 20) ? (self.containerView.bounds.size.height - 20) : self.presentedView.bounds.size.height);
                if ( @available(iOS 11.0, *) ) {
                    heigh = (self.presentedView.bounds.size.height > (self.containerView.bounds.size.height - 20 - self.containerView.safeAreaInsets.top) ? (self.containerView.bounds.size.height - 20 - self.containerView.safeAreaInsets.top) : self.presentedView.bounds.size.height);
                }
                self.presentedView.bounds = CGRectMake(0, 0, width, heigh);
                self.presentedView.center = CGPointMake(self.containerView.center.x, self.containerView.frame.size.height - heigh * 0.5);
            }
        }
    }
    _viewDidLayoutSubviews = YES;
}

- (void)setSize:(CGSize)size{
    if (!CGSizeEqualToSize(size, _size)) {
        _size = size;
        if (_viewDidLayoutSubviews) [self containerViewDidLayoutSubviews];
    }
}

- (void)callDismissViewControllerCompletion{
    !self.dismissViewControllerCompletion ? : self.dismissViewControllerCompletion();
    !self.dismissSplitViewControllerCompletion ? : self.dismissSplitViewControllerCompletion();
}

- (void)handleBackgroundTap:(UIGestureRecognizer *)gestureRecognizer
{
    self.presentedViewController.transitioningDelegate = nil;
    [self.presentedViewController dismissViewControllerAnimated:YES completion:^{
        [self callDismissViewControllerCompletion];
    }];
}

- (void)handleGesture:(UIPanGestureRecognizer*)gestureRecognizer {
    CGPoint translation = [gestureRecognizer translationInView:gestureRecognizer.view.superview];
    //NSLog(@"%s",__func__);
    if ([[self getScrollViewPresentedView] isKindOfClass:[UIScrollView class]]) {
        UIScrollView *touchView = ((UIScrollView *)[self getScrollViewPresentedView]);
        CGFloat maxHeigh = (_maxVerticalHeigh  > (self.containerView.bounds.size.height - 20) ? (self.containerView.bounds.size.height - 20) : _maxVerticalHeigh);
        if (self.presentedView.superview != self.containerView) {
            if ( @available(iOS 11.0, *) ) {
                maxHeigh = (_maxVerticalHeigh  > (self.containerView.bounds.size.height - 20 - self.containerView.safeAreaInsets.top) ? (self.containerView.bounds.size.height - 20 - self.containerView.safeAreaInsets.top) : _maxVerticalHeigh);
            }
        }
        
        maxHeigh = (maxHeigh > touchView.contentSize.height ? touchView.contentSize.height : maxHeigh);
        
        switch (gestureRecognizer.state) {
            case UIGestureRecognizerStateBegan: {
                //[touchView performSelector:@selector(handlePan:) withObject:gestureRecognizer]
                self.srcSize = self.presentedView.bounds.size;
                self.srcPoint = self.presentedView.center;
                _isChangeSize = YES;
                _isStopPanUp = NO;
                _isStopPanDown = NO;
                self.lastPoint = translation;
                break;
            }
            case UIGestureRecognizerStateChanged: {
                CGFloat dy = translation.y - self.lastPoint.y;
                if (self.presentedView.bounds.size.height >= maxHeigh) {
                    if (touchView.contentOffset.y <= 0 && dy < 0) {
                        _isChangeSize = NO;
                        break;
                    }
                    
                    if (touchView.contentOffset.y > 0) {
                        _isChangeSize = NO;
                        _isStopPanDown = YES;
                        break;
                    }
                }
                
                CGRect bounds = self.presentedView.bounds;
                if (bounds.size.height < maxHeigh && bounds.size.height - dy >= maxHeigh) {
                    bounds.size.height -= dy;
                    bounds.size.height = (bounds.size.height > maxHeigh ? maxHeigh : bounds.size.height);
                    if (self.presentedView.superview == self.containerView) {
                        [self.presentedView mas_updateConstraints:^(MASConstraintMaker *make) {
                            make.size.mas_equalTo(bounds.size).priorityHigh();
                        }];
                    }else{
                        self.presentedView.bounds = bounds;
                        self.presentedView.center = CGPointMake(self.presentedView.center.x, self.containerView.frame.size.height - bounds.size.height * 0.5);
                    }
                    
                    self.lastPoint = translation;
                    _isStopPanUp = YES;
                }
                if (_isStopPanDown || _isStopPanUp) {
                    break;
                }
                bounds.size.height -= dy;
                bounds.size.height = (bounds.size.height > maxHeigh ? maxHeigh : bounds.size.height);
                if (self.presentedView.superview == self.containerView && touchView.contentOffset.y <= 0) {
                    [self.presentedView mas_updateConstraints:^(MASConstraintMaker *make) {
                        make.size.mas_equalTo(bounds.size).priorityHigh();
                    }];
                }else{
                    self.presentedView.bounds = bounds;
                    self.presentedView.center = CGPointMake(self.presentedView.center.x, self.containerView.frame.size.height - bounds.size.height * 0.5);
                }
                self.lastPoint = translation;
                //[self.presentedView.superview layoutSubviews];
                break;
            }
            case UIGestureRecognizerStateEnded:
            case UIGestureRecognizerStateCancelled:
            {
                _isChangeSize = NO;
                CGRect bounds = self.presentedView.bounds;
                if (bounds.size.height > self.srcSize.height) {
                    _isStopPanUp = YES;
                    if (maxHeigh - self.srcSize.height > 0) {
                        if ((bounds.size.height - self.srcSize.height) / (maxHeigh - self.srcSize.height) >= 0.5) {
                            bounds.size.height = maxHeigh;
                            [UIView animateWithDuration:0.3 animations:^{
                                if (self.presentedView.superview == self.containerView) {
                                    [self.presentedView mas_updateConstraints:^(MASConstraintMaker *make) {
                                        make.size.mas_equalTo(bounds.size).priorityHigh();
                                    }];
                                }else{
                                    self.presentedView.bounds = bounds;
                                    self.presentedView.center = CGPointMake(self.presentedView.center.x, self.containerView.frame.size.height - bounds.size.height * 0.5);
                                }

                            }];
                        }else{
                            [UIView animateWithDuration:0.3 animations:^{
                                if (self.presentedView.superview == self.containerView) {
                                    [self.presentedView mas_updateConstraints:^(MASConstraintMaker *make) {
                                        make.size.mas_equalTo(self.srcSize).priorityHigh();
                                    }];
                                }else{
                                    self.presentedView.bounds = CGRectMake(0, 0, self.srcSize.width, self.srcSize.height);
                                    self.presentedView.center = self.srcPoint;
                                }
                            }];
                        }
                    }
                }
                
                if (bounds.size.height < self.srcSize.height) {
                    if (self.srcSize.height > 0) {
                        if ((self.srcSize.height - bounds.size.height) / self.srcSize.height >= 0.5) {
                            self.presentedViewController.transitioningDelegate = nil;
                            [self.presentedViewController dismissViewControllerAnimated:YES completion:^{
                                [self callDismissViewControllerCompletion];
                            }];
                        }else{
                            [UIView animateWithDuration:0.3 animations:^{
                                if (self.presentedView.superview == self.containerView) {
                                    [self.presentedView mas_updateConstraints:^(MASConstraintMaker *make) {
                                        make.size.mas_equalTo(self.srcSize).priorityHigh();
                                    }];
                                }else{
                                    self.presentedView.bounds = CGRectMake(0, 0, self.srcSize.width, self.srcSize.height);
                                    self.presentedView.center = self.srcPoint;
                                }
                            }];
                        }
                    }
                }
                //[self.presentedView.superview layoutSubviews];
                break;
            }
            default:
                _isChangeSize = NO;
                break;
        }
    }else{
        
        CGFloat maxHeigh = (_maxVerticalHeigh  > (self.containerView.bounds.size.height - 20) ? (self.containerView.bounds.size.height - 20) : _maxVerticalHeigh);
        if ( @available(iOS 11.0, *) ) {
            maxHeigh = (_maxVerticalHeigh  > (self.containerView.bounds.size.height - 20 - self.containerView.safeAreaInsets.top) ? (self.containerView.bounds.size.height - 20 - self.containerView.safeAreaInsets.top) : _maxVerticalHeigh);
        }
        
        switch (gestureRecognizer.state) {
            case UIGestureRecognizerStateBegan: {
                //[touchView performSelector:@selector(handlePan:) withObject:gestureRecognizer]
                self.srcPoint = self.presentedView.center;
                _isChangeSize = YES;
                self.lastPoint = translation;
                break;
            }
            case UIGestureRecognizerStateChanged: {
                CGFloat dy = translation.y - self.lastPoint.y;
                if (self.presentedView.center.y + dy < self.srcPoint.y) {
                    break;
                }
                self.presentedView.center = CGPointMake(self.srcPoint.x, self.presentedView.center.y + dy);
                self.lastPoint = translation;
                //[self.presentedView.superview layoutSubviews];
                break;
            }
            case UIGestureRecognizerStateEnded:
            case UIGestureRecognizerStateCancelled:
            {
                _isChangeSize = NO;
                CGPoint curPoint = self.presentedView.center;
                if (curPoint.y - self.srcPoint.y > maxHeigh * 0.5) {
                    self.presentedViewController.transitioningDelegate = nil;
                    [self.presentedViewController dismissViewControllerAnimated:YES completion:^{
                        [self callDismissViewControllerCompletion];
                    }];
                }else{
                    [UIView animateWithDuration:0.3 animations:^{
                        self.presentedView.center = self.srcPoint;
                    }];
                }
                //[self.presentedView.superview layoutSubviews];
                break;
            }
            default:
                _isChangeSize = NO;
                break;
        }
    }
}

- (void)observeValueForKeyPath:(nullable NSString *)keyPath ofObject:(nullable id)object change:(nullable NSDictionary<NSString*, id> *)change context:(nullable void *)context{

    if(context && (context == (__bridge void * _Nullable)([self getScrollViewPresentedView])) && [keyPath isEqualToString:@"contentOffset"])
    {
        CGPoint offset = [[change objectForKey:@"new"] CGPointValue];
        
        if (offset.y < 0) {
            [self getScrollViewPresentedView].contentOffset = CGPointZero;
        }
    }
}

- (UIView *)dimmingView{
    if (!_dimmingView) {
        _dimmingView = [[UIView alloc] init];
        _dimmingView.isAccessibilityElement = YES;
        _dimmingView.accessibilityTraits = UIAccessibilityTraitNone;
        UITapGestureRecognizer *backgroundTapRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handleBackgroundTap:)];
        [_dimmingView addGestureRecognizer:backgroundTapRecognizer];
        
        if ( @available(iOS 11.0, *) ) {
            if (self.modalPresentationStyle != FSModalPresentationCenter) {
                _safeFillView = [[UIView alloc] init];
                _safeFillView.isAccessibilityElement = YES;
                _safeFillView.accessibilityTraits = UIAccessibilityTraitNone;
                UITapGestureRecognizer *backgroundTapRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handleBackgroundTap:)];
                [_safeFillView addGestureRecognizer:backgroundTapRecognizer];
            }
        }
    }
    return _dimmingView;
}

- (int)convertInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation{
    switch (interfaceOrientation) {
        case UIInterfaceOrientationPortrait:
            return 0;
        case UIInterfaceOrientationLandscapeRight:
            return 1;
        case UIInterfaceOrientationPortraitUpsideDown:
            return 2;
        case UIInterfaceOrientationLandscapeLeft:
            return 3;
        default:
            return 0;
    }
}

@end

@interface FSPresentedViewControllerAnimator ()
@property (nonatomic, weak, readonly) FSPresentationController *presentationController;
@property (nonatomic, weak) __kindof  UIView *viewCtrl;
@end
@implementation FSPresentedViewControllerAnimator

- (instancetype)init{
    return [[FSPresentedViewControllerAnimator alloc] initWithViewCtrl:nil];
}

- (instancetype)initWithViewCtrl:(nullable __kindof UIView *)viewCtrl;{
    self = [super init];
    if (self) {
        _presentationStyle = FSModalPresentationFullScreen;
        if (viewCtrl) {
            _presentationStyle = FSModalPresentationFullViewCtrl;
            _viewCtrl = viewCtrl;
        }
    }
    return self;
}

- (instancetype)initWithVerticalHeigh:(CGFloat)verticalHeigh presentationStyle:(FSModalPresentationStyle)modalPresentationStyle{
    self = [super init];
    if (self) {
        _presentationStyle = modalPresentationStyle;
        _maxVerticalHeigh = verticalHeigh;
    }
    return self;
}
- (instancetype)initWithSize:(CGSize)size presentationStyle:(FSModalPresentationStyle)presentationStyle{
    self = [super init];
    if (self) {
        _presentationStyle = presentationStyle;
        _size = size;
    }
    return self;
}

- (void)setSize:(CGSize)size{
    _size = size;
    if (_presentationController) _presentationController.size = size;
}

- (void)setMaxVerticalHeigh:(CGFloat)maxVerticalHeigh{
    _maxVerticalHeigh = maxVerticalHeigh;
    if (_presentationController)  _presentationController.maxVerticalHeigh = maxVerticalHeigh;
}

- (void)setDismissViewControllerCompletion:(dispatch_block_t)dismissViewControllerCompletion{
    _dismissViewControllerCompletion = dismissViewControllerCompletion;
    if (_presentationController)  _presentationController.dismissViewControllerCompletion = [dismissViewControllerCompletion copy];
}

- (void)setDismissSplitViewControllerCompletion:(dispatch_block_t)dismissSplitViewControllerCompletion{
    _dismissSplitViewControllerCompletion = dismissSplitViewControllerCompletion;
    if (_presentationController)  _presentationController.dismissSplitViewControllerCompletion = [dismissSplitViewControllerCompletion copy];
}

- (UIPresentationController *)presentationControllerForPresentedViewController:(UIViewController *)presented presentingViewController:(UIViewController *)presenting sourceViewController:(UIViewController *)source{
    FSPresentationController *presentation = [[FSPresentationController alloc] initWithPresentedViewController:presented presentingViewController:presenting];
    presentation.dismissViewControllerCompletion = [self.dismissViewControllerCompletion copy];
    if (!CGSizeEqualToSize(self.size, CGSizeZero)) {
        presented.view.bounds = CGRectMake(0, 0, _size.width, _size.height);
    }
    _presentationController = presentation;
    
    presentation.modalPresentationStyle = self.presentationStyle;
    switch (self.presentationStyle) {
        case FSModalPresentationFullViewCtrl:
            presented.view.bounds = CGRectMake(0, 0, self.viewCtrl.fs_width, self.viewCtrl.fs_height);
            presentation.viewCtrl = self.viewCtrl;
            break;
        case FSModalPresentationCenter:
            presentation.size = self.size;
            break;
        case FSModalPresentationBottom:
            presentation.maxVerticalHeigh = self.maxVerticalHeigh;
            break;
        default:
            return nil;
            break;
    }
    return presentation;
}

@end

