/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSEmptyDataSetView.h"
#import "FSItem.h"
#import "Masonry+FSAdd.h"

@interface FSEmptyDataSetView ()<UIGestureRecognizerDelegate>
@property (nonatomic, weak) FSItem *item;
@property (nonatomic, weak) UIPanGestureRecognizer *conflictSuperviewPan;
@end
@implementation FSEmptyDataSetView

- (instancetype)init
{
    self = [super init];
    if (self) {
        FSItem *item = [FSItem createItemWithTitle:nil image:nil imageTextRelation:FSImageTextRelationBottom];
        item.enabled = NO;
        item.titleLabel.numberOfLines = 0;
        item.titleLabel.textAlignment = NSTextAlignmentCenter;
        [item setTitleEdgeInsets:UIEdgeInsetsMake(10, 0, 0, 0)];
        [self addSubview:item];
        [item mas_makeConstraints:^(MASConstraintMaker *make) {
            make.centerX.fs_equalToSuperview();
            make.centerY.fs_equalToSuperview();
            make.width.lessThanOrEqualTo(item.superview.mas_width);
        }];
        
        UIPanGestureRecognizer *conflictSuperviewPan = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(conflictSuperviewPanHandle:)];
        conflictSuperviewPan.delegate = self;
        [self addGestureRecognizer:conflictSuperviewPan];
        _item = item;
    }
    return self;
}

- (void)conflictSuperviewPanHandle:(UIPanGestureRecognizer *)recognizer{}

- (void)setVerticalOffset:(CGFloat)verticalOffset{
    _verticalOffset = verticalOffset;
    [_item mas_updateConstraints:^(MASConstraintMaker *make) {
        make.centerY.fs_equalToSuperview().offset(verticalOffset);
    }];
}

- (void)setTitle:(NSAttributedString *)title{
    _title = title;
    self.accessibilityLabel = title.string;
    [self.item setAttributedTitle:title forState:UIControlStateNormal | UIControlStateDisabled];
}

- (void)setImage:(UIImage *)image{
    _image = image;
    [self.item setImage:image forState:UIControlStateNormal | UIControlStateDisabled];
}

- (void)setScrollEnabledForSupView:(BOOL)scrollEnabledForSupView{
    _conflictSuperviewPan.enabled = scrollEnabledForSupView;
}

- (BOOL)canClick{
    return self.item.enabled;
}

- (void)setCanClick:(BOOL)canClick{
    [self.item setEnabled:canClick];
}

- (void)setTarget:(id)target action:(SEL)action{
    [self.item setTarget:target action:action forControlEvents:UIControlEventTouchUpInside];
}

- (BOOL)gestureRecognizerShouldBegin:(UIGestureRecognizer *)gestureRecognizer{
    return [super gestureRecognizerShouldBegin:gestureRecognizer];
}

- (BOOL)gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer shouldRecognizeSimultaneouslyWithGestureRecognizer:(UIGestureRecognizer *)otherGestureRecognizer{
    if (otherGestureRecognizer.view == self.superview) {
        return NO;
    }
    return YES;
}

- (BOOL)isAccessibilityElement
{
    return YES;
}
@end
