/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import <UIKit/UIKit.h>

NS_ASSUME_NONNULL_BEGIN

@class FSPagerView;
@protocol FSPagerViewDataSource <NSObject>

- (NSInteger)numberOfViewsInPagerView:(FSPagerView *)pagerView;
- (UIView *)pagerView:(FSPagerView *)pagerView viewForIndex:(NSInteger)index;

@end

@protocol FSPagerViewDelegate <NSObject>
@optional
// scrollView delegate
- (void)pagerViewDidScroll:(FSPagerView *)pageView;
- (void)pagerViewWillBeginScrolling:(FSPagerView *)pageView animate:(BOOL)animate;
- (void)pagerViewDidEndScrolling:(FSPagerView *)pageView animate:(BOOL)animate;
- (void)pagerView:(FSPagerView *)pagerView transitionFromIndex:(NSInteger)fromIndex toIndex:(NSInteger)toIndex progress:(CGFloat)progress;

@end

@interface FSPagerView : UIView
@property (nonatomic, weak, nullable) id<FSPagerViewDataSource> dataSource;
@property (nonatomic, weak, nullable) id<FSPagerViewDelegate> delegate;

@property (nonatomic, assign) BOOL scrollEnabled;
@property (nonatomic, assign, readonly) NSInteger countOfPagerViews;
@property (nonatomic, assign, readonly) NSInteger curIndex;// default -1

@property (nonatomic, assign, nullable, readonly) NSArray<UIView *> *visibleViews;

@property (nonatomic, assign) UIEdgeInsets contentInset;

- (UIView *_Nullable)viewForIndex:(NSInteger)index;

- (void)scrollToViewAtIndex:(NSInteger)index animate:(BOOL)animate;

- (void)reloadData;

@end

@class FSTabPagerBar;
@protocol FSTabPagerBarDataSource <NSObject>

- (NSInteger)numberOfItemsInPagerTabBar:(FSTabPagerBar *)pagerTabBar;
- (__kindof UIView *)pagerTabBar:(FSTabPagerBar *)pagerTabBar cellForItemAtIndex:(NSInteger)index;

@end

@protocol FSTabPagerBarDelegate <NSObject>

@optional

- (CGFloat)pagerTabBar:(FSTabPagerBar *)pagerTabBar widthForItemAtIndex:(NSInteger)index;
- (void)pagerTabBar:(FSTabPagerBar *)pagerTabBar didSelectItemAtIndex:(NSInteger)index;
- (void)pagerTabBar:(FSTabPagerBar *)pagerTabBar transitionFromItem:(__kindof UIView *)fromItem toItem:(__kindof UIView *)toItem animated:(BOOL)animated;

@end

typedef NS_ENUM(NSInteger, FSTabPagerBarItemContentMode) {
    FSTabPagerBarItemContentModeScaleToFill = 0,
    FSTabPagerBarItemContentModeScaleAspectFit = 1
};

@interface FSTabPagerBar : UIView

@property (nonatomic, assign) CGFloat itemWidth;

@property (nonatomic, assign) CGFloat itemEdging;

@property (nonatomic, strong) UIColor *progressColor;

@property (nonatomic, weak, nullable) id<FSTabPagerBarDataSource> dataSource;

@property (nonatomic, weak, nullable) id<FSTabPagerBarDelegate> delegate;

@property (nonatomic, assign, readonly) NSInteger countOfItems;

@property (nonatomic, assign, readonly) NSInteger curIndex;

@property (nonatomic, assign) UIEdgeInsets contentInset;

@property (nonatomic, assign) FSTabPagerBarItemContentMode itemContentMode;

- (void)reloadData;

- (void)scrollToItemFromIndex:(NSInteger)fromIndex toIndex:(NSInteger)toIndex progress:(CGFloat)progress;
- (void)scrollToItemAtIndex:(NSInteger)index animated:(BOOL)animated;

- (nullable __kindof UIView *)itemForIndex:(NSInteger)index;

@end


NS_ASSUME_NONNULL_END
