/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#if __has_include(<FSFoundation/FSFoundation.h>)
#import <FSFoundation/FSViewController.h>
#else
#import "FSViewController.h"
#endif

NS_ASSUME_NONNULL_BEGIN

@interface FSCollectionViewController : FSViewController<UICollectionViewDelegate, UICollectionViewDataSource>
@property (nonatomic, strong, readonly) __kindof UICollectionView *collectionView;
@property (nonatomic, copy) NSArray *sectionIndexTitles1;
@property (nonatomic, assign) BOOL needMakeUpSectionHeaderInset;
@property (nonatomic, assign) BOOL disableTitleUppercase;

@property (nonatomic, copy, nullable) NSArray *selectedDataSource;
@property (nonatomic, copy, nullable) NSArray<NSIndexPath *> *selectedIndexPaths;
@property (nonatomic, strong) UIColor *cellSelectedColor;

@property (nonatomic, copy, nullable) NSArray *dataSource;
@property (nonatomic, assign, readonly) UIEdgeInsets contentInset;
@property (nonatomic, assign) UIEdgeInsets subviewCollectionViewLayoutMargins;
@property (nonatomic, assign, readonly) UICollectionViewLayout *collectionViewLayout;

@property (nonatomic, assign) BOOL customEditing;
@property (nonatomic, strong) UIImage *editingImageNormal;
@property (nonatomic, strong) UIImage *editingImageSelected;

@property (nonatomic, assign, readonly) BOOL emptyDataSetDisplayed;

@property (nonatomic, strong, readonly) FSToolbar *editBar;
@property (nonatomic, strong, readonly) FSItem *editLeftItem;
@property (nonatomic, strong, readonly) FSItem *editRightItem;

- (void)reloadData;
- (void)reloadItemsAtIndexPaths:(id)indexPaths completion:(void (^ _Nullable)(BOOL finished))completion;
- (void)insertItemsAtIndexPaths:(id)indexPaths completion:(void (^ _Nullable)(BOOL finished))completion;
- (void)deleteItemsAtIndexPaths:(id)indexPaths completion:(void (^ _Nullable)(BOOL finished))completion;
- (void)selectItem:(BOOL)select atIndexPath:(NSIndexPath *)indexPath;
- (__kindof UICollectionViewCell *)collectionView:(UICollectionView *)collectionView dequeueReusableCellWithReuseIdentifier:(NSString *)identifier forIndexPath:(NSIndexPath *)indexPath withDataItem:(id)dataItem;
- (id)dataForAtIndexPath:(NSIndexPath *)indexPath;
- (NSString *)titles1ForAtSection:(NSUInteger)section;
- (void)configureCell:(UICollectionViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(id)dataItem;

//EmptyDataSet
- (void)reloadEmptyDataSet;
- (NSString *)titleForEmptyDataSet;
- (NSAttributedString *)titleForEmptyDataSet:(UICollectionView *)collectionView;
- (UIImage *)imageForEmptyDataSet:(UICollectionView *)collectionView;
- (CGFloat)verticalOffsetForEmptyDataSet:(UICollectionView *)collectionView;
- (BOOL)emptyDataSetShouldDisplay:(UICollectionView *)collectionView;
- (BOOL)emptyDataSetShouldAllowScroll:(UICollectionView *)collectionView;

- (void)clickedEditLeftItem;
- (void)clickedEditRightItem;

- (void)selectAll;
- (void)deselectAll;

@end

NS_ASSUME_NONNULL_END
