/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSCollectionViewController.h"
#if __has_include(<YZHookHandler/YZHookHandler.h>)
#import <YZHookHandler/YZHookHandler.h>
#else
#import "YZHookHandler.h"
#endif
#import "NSObject+FSAddForKVO.h"
#import "FSCollectionView.h"
#import "FSEmptyDataSetView.h"
#import "FSPreferences.h"
#import "Masonry+FSAdd.h"
#import "FSDataItem.h"
#import "FSCollectionViewCell.h"

@interface FSCollectionTitleView : UICollectionReusableView
@property (nonatomic, copy) NSString *title;
@property (nonatomic, strong) UILabel *titleLab;
@property (nonatomic, assign) BOOL disableTitleUppercase;
@property (nonatomic, assign) BOOL needMakeUpSectionHeaderInset;
@end

@interface FSCollectionViewController ()
@property (nonatomic, strong) FSEmptyDataSetView *emptyDataSetView;
@property (nonatomic, strong, readwrite) __kindof UICollectionView *collectionView;
@property (nonatomic, strong, readwrite) FSToolbar *editBar;
@property (nonatomic, strong, readwrite) FSItem *editLeftItem;
@property (nonatomic, strong, readwrite) FSItem *editRightItem;
@property (nonatomic, assign) BOOL didLoadData;
@property (nonatomic, weak) MASConstraint *collectionViewBottomConstraint;

@end

@implementation FSCollectionViewController

+ (instancetype)allocWithZone:(struct _NSZone *)zone {
    FSCollectionViewController *viewController = [super allocWithZone:zone];
    
    @weakify(viewController)
    [[viewController yz_hookForSelector:@selector(numberOfSectionsInCollectionView:) fromProtocol:@protocol(UICollectionViewDataSource)]
     after:^(NSArray * _Nonnull args) {
        @strongify(viewController)
        [viewController reloadEmptyDataSet];
    }];
    
    [[[viewController yz_hookForSelector:@selector(loadData)]
      befor:^(NSArray * _Nonnull args) {
        @strongify(viewController)
        viewController.didLoadData = NO;
    }] after:^(NSArray * _Nonnull args) {
        @strongify(viewController)
        viewController.didLoadData = YES;
    }];
    return viewController;
}

- (void)initialize {
    [super initialize];
    _cellSelectedColor = [UIColor clearColor];
    [self fs_addObserverBlockForKeyPath:@keypath(self, dataSource)
                                  block:^(FSCollectionViewController * _Nonnull _self, id  _Nullable oldVal, id  _Nullable newVal) {
        dispatch_main_async_safe(^{
            [_self reloadData];
        });
        
    }];
    
    [BindThemeKeySel(self, reloadEmptyDataSet) fs_updateThemeCompleted:^(FSCollectionViewController * _Nonnull target, SEL  _Nonnull sel) {
        dispatch_main_async_safe(^{
            if (target.emptyDataSetView.superview) {
                [target reloadEmptyDataSet];
            }
        });
    }];
}

- (void)dealloc{
    [self fs_removeObserverBlocks];
}

- (UIEdgeInsets)contentInset {
    return UIEdgeInsetsZero;
}

- (UICollectionViewLayout *)collectionViewLayout{
    return [UICollectionViewLayout new];
}

- (FSEmptyDataSetView *)emptyDataSetView{
    if (!_emptyDataSetView) {
        _emptyDataSetView = [FSEmptyDataSetView new];
    }
    return  _emptyDataSetView;
}

- (BOOL)emptyDataSetDisplayed{
    return  _emptyDataSetView.superview;
}

- (FSCollectionView *)collectionView{
    if (!_collectionView) {
        _collectionView = [[FSCollectionView alloc] initWithFrame:CGRectZero collectionViewLayout:self.collectionViewLayout];
        _collectionView.contentInset  = self.contentInset;
        _collectionView.scrollIndicatorInsets = self.contentInset;
        
        _collectionView.backgroundColor = self.fs_backgroundColor_B1;
        
        _collectionView.delegate = self;
        _collectionView.dataSource = self;
    }
    return _collectionView;
}

- (FSToolbar *)editBar{
    if (!_editBar) {
        _editBar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleBottom];
        [_editBar addItem:self.editLeftItem atPosition:FSItemPositionLeft];
        [_editBar addItem:self.editRightItem atPosition:FSItemPositionRight];
    }
    return _editBar;
}

- (FSItem *)editLeftItem{
    if (!_editLeftItem) {
        _editLeftItem = [FSItem createItemWithTitle:@"Clear All"];
        [_editLeftItem addTarget:self action:@selector(clickedEditLeftItem) forControlEvents:UIControlEventTouchUpInside];
    }
    return _editLeftItem;
}

- (FSItem *)editRightItem{
    if (!_editRightItem) {
        _editRightItem = [FSItem createItemWithTitle:@"Delete"];
        _editRightItem.enabled = NO;
        [_editRightItem addTarget:self action:@selector(clickedEditRightItem) forControlEvents:UIControlEventTouchUpInside];
    }
    return _editRightItem;
}

- (void)makeConstraints{
    [super makeConstraints];
    [self makeCollectionViewConstraints];

}

- (void)makeCollectionViewConstraints{
    if (self.view != self.collectionView) {
        if (UIEdgeInsetsEqualToEdgeInsets(self.subviewCollectionViewLayoutMargins, UIEdgeInsetsZero)) {
            [self.collectionView mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.left.equalTo(self.collectionView.superview);
                make.right.equalTo(self.collectionView.superview);
                self.collectionViewBottomConstraint = make.bottom.equalTo(self.shouldShowBottomBar ? self.bottomBar.mas_top : self.collectionView.superview.fs_mas_bottom);
                make.top.equalTo(self.shouldShowNavigationBar ? self.navBar.mas_bottom : self.collectionView.superview.fs_mas_top);
            }];
        }else{
            [self.collectionView mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.left.equalTo(self.collectionView.superview).offset(self.subviewCollectionViewLayoutMargins.left);
                make.right.equalTo(self.collectionView.superview).offset(-self.subviewCollectionViewLayoutMargins.right);
                self.collectionViewBottomConstraint = make.bottom.equalTo(self.shouldShowBottomBar ? self.bottomBar.mas_top : self.collectionView.superview).offset(-self.subviewCollectionViewLayoutMargins.bottom);
                make.top.equalTo(self.shouldShowNavigationBar ? self.navBar.mas_bottom : self.collectionView.superview).offset(self.subviewCollectionViewLayoutMargins.top);
            }];
        }
    }
}

- (void)setSelectedDataSource:(NSArray *)selectedDataSource{
    _selectedDataSource = selectedDataSource;
    self.editRightItem.enabled = selectedDataSource.count;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    if (self.view != self.collectionView) [self.view addSubview:self.collectionView];
    
    [self.collectionView registerClass:[UICollectionViewCell class] forCellWithReuseIdentifier:@"UICollectionViewCell"];
    [self.collectionView registerClass:[FSCollectionTitleView class] forSupplementaryViewOfKind:UICollectionElementKindSectionHeader withReuseIdentifier:@"FSCollectionTitleView"];
}

- (void)viewSafeAreaInsetsDidChange{
    [super viewSafeAreaInsetsDidChange];
    for (UICollectionReusableView *reusableView in [self.collectionView visibleSupplementaryViewsOfKind:UICollectionElementKindSectionHeader]) {
        if ([reusableView isKindOfClass:[FSCollectionTitleView class]]) {
            FSCollectionTitleView *titleHeader = (FSCollectionTitleView *)reusableView;
            titleHeader.needMakeUpSectionHeaderInset = self.needMakeUpSectionHeaderInset;
        }
    }
}

- (void)setSubviewCollectionViewLayoutMargins:(UIEdgeInsets)subviewCollectionViewLayoutMargins{
    if (UIEdgeInsetsEqualToEdgeInsets(_subviewCollectionViewLayoutMargins, subviewCollectionViewLayoutMargins)) return;
    _subviewCollectionViewLayoutMargins = subviewCollectionViewLayoutMargins;
    [self makeCollectionViewConstraints];

}

- (void)setCustomEditing:(BOOL)editing{
    if ([self emptyDataSetDisplayed] && !_customEditing && _didLoadData) return;
    if (_customEditing == editing) {
        return;
    }
    _customEditing = editing;
    if (editing) {
        self.selectedDataSource = @[];
        self.selectedIndexPaths = @[];
        [self.view addSubview:self.editBar];
        [self.editBar mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.right.bottom.fs_equalToSuperview();
        }];
        
        if (self.collectionViewBottomConstraint) {
            [self.collectionViewBottomConstraint uninstall];
        }
        [self.collectionView mas_updateConstraints:^(MASConstraintMaker *make) {
            make.bottom.mas_equalTo(self.editBar.mas_top);
        }];
    }else{
        if (_editBar) {
            [self.editBar removeFromSuperview];
            [self makeCollectionViewConstraints];
        }
        for (NSIndexPath *indexPath in self.collectionView.indexPathsForSelectedItems) {
            [self.collectionView deselectItemAtIndexPath:indexPath animated:NO];
        }
    }
    
    for (UICollectionViewCell *cell in self.collectionView.visibleCells) {
        if ([cell isKindOfClass:[FSCollectionViewCell class]]) {
            FSCollectionViewCell *fs_cell = (FSCollectionViewCell *)cell;
            fs_cell.customEditing = editing;
            [fs_cell.editItem setImage:self.editingImageNormal forState:UIControlStateNormal];
            @weakify(self)
            [BindThemeKeySel(fs_cell.editItem, setImage:forState:) fs_updateThemeCompleted:^(FSItem * _Nonnull target, SEL  _Nonnull sel) {
                @strongify(self)
                [target setImage:[self.editingImageSelected fs_imageWithTintColor:target.fs_primaryColor] forState:UIControlStateSelected];
            }];
        }
        cell.selectedBackgroundView.backgroundColor = self.cellSelectedColor ? self.cellSelectedColor : cell.contentView.backgroundColor;
    }
}

- (void)loadData{}

- (void)reloadData{
    [self.collectionView reloadData];
}

- (void)reloadItemsAtIndexPaths:(id)indexPaths completion:(void (^ _Nullable)(BOOL finished))completion{
    if (!indexPaths) return;
    [self.collectionView performBatchUpdates:^{
        [self.collectionView reloadItemsAtIndexPaths:[indexPaths isKindOfClass:[NSArray class]] ? indexPaths : @[indexPaths]];
    } completion:completion];

}

- (void)insertItemsAtIndexPaths:(id)indexPaths completion:(void (^ _Nullable)(BOOL finished))completion{
    if (!indexPaths) return;
    [self.collectionView performBatchUpdates:^{
        [self.collectionView insertItemsAtIndexPaths:[indexPaths isKindOfClass:[NSArray class]] ? indexPaths : @[indexPaths]];
    } completion:completion];
}

- (void)deleteItemsAtIndexPaths:(id)indexPaths completion:(void (^ _Nullable)(BOOL finished))completion{
    if (!indexPaths) return;
    [self.collectionView performBatchUpdates:^{
        [self.collectionView deleteItemsAtIndexPaths:[indexPaths isKindOfClass:[NSArray class]] ? indexPaths : @[indexPaths]];
    } completion:completion];
}

- (void)selectItem:(BOOL)select atIndexPath:(NSIndexPath *)indexPath{
    if (select) {
        [self.collectionView selectItemAtIndexPath:indexPath animated:NO scrollPosition:UICollectionViewScrollPositionNone];
    }else{
        [self.collectionView deselectItemAtIndexPath:indexPath animated:NO];
    }
}

- (void)selectDataSourceItem:(BOOL)select atIndexPath:(NSIndexPath *)indexPath{
    NSMutableArray *tmpDataSource = self.selectedDataSource.mutableCopy;
    NSMutableArray *tmpDataIndexPaths = self.selectedIndexPaths.mutableCopy;
    id data = [self dataForAtIndexPath:indexPath];
    if ([data isKindOfClass:[FSDataItem class]]) {
        FSDataItem *dataItem = (FSDataItem *)data;
        dataItem.editingSelected = select;
    }
    if (select) {
        if (![tmpDataSource containsObject:data]) {
            if (!data) data = [NSNull null];
            [tmpDataSource addObject:data];
        }
        
        if (![tmpDataIndexPaths containsObject:indexPath]) {
            [tmpDataIndexPaths addObject:indexPath];
        }
    }else{
        if ([tmpDataSource containsObject:data]) {
            [tmpDataSource removeObject:data];
        }
        
        if ([tmpDataIndexPaths containsObject:indexPath]) {
            [tmpDataIndexPaths removeObject:indexPath];
        }
    }
    self.selectedDataSource = tmpDataSource.copy;
    self.selectedIndexPaths = tmpDataIndexPaths.copy;
    [self selectItem:select atIndexPath:indexPath];
    UICollectionViewCell *cell = [self.collectionView cellForItemAtIndexPath:indexPath];
    cell.selectedBackgroundView.backgroundColor = select ? cell.contentView.backgroundColor : self.cellSelectedColor ? self.cellSelectedColor : cell.contentView.backgroundColor;
}

- (__kindof UICollectionViewCell *)collectionView:(UICollectionView *)collectionView dequeueReusableCellWithReuseIdentifier:(NSString *)identifier forIndexPath:(NSIndexPath *)indexPath withDataItem:(id)dataItem{
    return [collectionView dequeueReusableCellWithReuseIdentifier:identifier forIndexPath:indexPath];
}

- (id)dataForAtIndexPath:(NSIndexPath *)indexPath{
    @try {
        if ([self.dataSource count]) {
            id sectionData = [self.dataSource firstObject];
            if ([sectionData isKindOfClass:[NSArray class]]) {
                if ([sectionData count]) return self.dataSource[indexPath.section][indexPath.row];
            }else{
                return self.dataSource[indexPath.row];
            }
            return nil;
        }
    } @catch (NSException *exception) {
        return nil;
    }
    return nil;
}

- (NSString *)titles1ForAtSection:(NSUInteger)section{
    if (self.sectionIndexTitles1.count > section) {
        return self.sectionIndexTitles1[section];
    }
    return nil;
}

- (void)configureCell:(UICollectionViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(id)dataItem {}

- (void)reloadEmptyDataSet{
    if ([self emptyDataSetShouldDisplay:self.collectionView]) {
        [self.collectionView addSubview:self.emptyDataSetView];
        [self.emptyDataSetView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.center.left.right.bottom.top.fs_equalToSuperview();
        }];
        self.emptyDataSetView.title = [self titleForEmptyDataSet:self.collectionView];
        self.emptyDataSetView.image = [self imageForEmptyDataSet:self.collectionView];
        self.emptyDataSetView.verticalOffset = [self verticalOffsetForEmptyDataSet:self.collectionView];
        self.emptyDataSetView.scrollEnabledForSupView = ![self emptyDataSetShouldAllowScroll:self.collectionView];
        self.emptyDataSetView.canClick = [self emptyDataSetShouldAllowClick:self.collectionView];
    }else{
        [self.emptyDataSetView removeFromSuperview];
    }
}

- (NSString *)titleForEmptyDataSet{
    return @"NO Data";
}

- (NSAttributedString *)titleForEmptyDataSet:(UICollectionView *)collectionView {
    return [[NSAttributedString alloc] initWithString:[self titleForEmptyDataSet] attributes:@{
        NSFontAttributeName:self.fs_standardGeneralFont_GF3,
        NSForegroundColorAttributeName:self.fs_textColor_T3
        }];
}

- (UIImage *)imageForEmptyDataSet:(UICollectionView *)collectionView {
    return nil;
}

- (CGFloat)verticalOffsetForEmptyDataSet:(UICollectionView *)collectionView {
    return 0;
}

- (BOOL)emptyDataSetShouldDisplay:(UICollectionView *)collectionView {
    @try {
        return ![self dataForAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0]];;
    } @catch (NSException *exception) {
        return YES;
    }
    return YES;
}

- (BOOL)emptyDataSetShouldAllowScroll:(UICollectionView *)collectionView{
    return NO;
}

- (BOOL)emptyDataSetShouldAllowClick:(UICollectionView *)collectionView{
    return NO;
}

- (void)clickedEditLeftItem {}
- (void)clickedEditRightItem {self.selectedIndexPaths = @[]; self.selectedDataSource = @[];}
- (void)selectAll{
    if (!self.customEditing) {
        return;
    }
    NSInteger section = [self numberOfSectionsInCollectionView:self.collectionView];
    for (int s = 0 ; s < section; s++ ) {
        NSInteger row = [self collectionView:self.collectionView numberOfItemsInSection:s];
        for (int r = 0; r < row; r++) {
            NSIndexPath *indexPath = [NSIndexPath indexPathForRow:r inSection:s];
            [self selectItem:YES atIndexPath:indexPath];
            [self selectDataSourceItem:YES atIndexPath:indexPath];
        }
    }
}

- (void)deselectAll{
    if (!self.customEditing) {
        return;
    }
    for (NSIndexPath *selectedIndexPath in self.selectedIndexPaths) {
        id data = [self dataForAtIndexPath:selectedIndexPath];
        if ([data isKindOfClass:[FSDataItem class]]) {
            FSDataItem *dataItem = (FSDataItem *)data;
            if (!dataItem.editingSelected) dataItem.editingSelected = NO;
        }
        [self selectItem:NO atIndexPath:selectedIndexPath];
    }
    self.selectedDataSource = @[];
    self.selectedIndexPaths = @[];
}


#pragma mark -UICollectionViewDataSource

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView{
    return self.dataSource ? self.dataSource.count : 1;
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section{
    return [self.dataSource[section] count];
}

- (__kindof UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath{
    id data = [self dataForAtIndexPath:indexPath];
    UICollectionViewCell *cell = [self collectionView:collectionView dequeueReusableCellWithReuseIdentifier:@"UICollectionViewCell" forIndexPath:indexPath withDataItem:data];
    cell.selectedBackgroundView = [UIView new];
    if ([cell isKindOfClass:[FSCollectionViewCell class]]) {
        FSCollectionViewCell *fs_cell = (FSCollectionViewCell *)cell;
        fs_cell.customEditing = self.customEditing;
        if (self.customEditing) {
            [fs_cell.editItem setImage:self.editingImageNormal forState:UIControlStateNormal];
            @weakify(self)
            [BindThemeKeySel(fs_cell.editItem, setImage:forState:) fs_updateThemeCompleted:^(FSItem * _Nonnull target, SEL  _Nonnull sel) {
                @strongify(self)
                [target setImage:[self.editingImageSelected fs_imageWithTintColor:target.fs_primaryColor] forState:UIControlStateSelected];
            }];
        }
        if (self.cellSelectedColor) {
            cell.contentView.backgroundColor = nil;
        }
    }
    cell.selectedBackgroundView.backgroundColor = self.cellSelectedColor ? self.cellSelectedColor : cell.contentView.backgroundColor;
    
    if ([data isKindOfClass:[FSDataItem class]]) {
        FSDataItem *dataItem = (FSDataItem *)data;
        if (self.customEditing) {
            [self selectItem:dataItem.editingSelected atIndexPath:indexPath];
        }
        else {
            if (dataItem.editingSelected) dataItem.editingSelected = NO;
        }
    }else{
        if (self.customEditing) {
            if ([self.selectedIndexPaths containsObject:indexPath]) {
                [self selectItem:YES atIndexPath:indexPath];
            }
        }
    }
    
    BOOL selected = [[self.collectionView indexPathsForSelectedItems] containsObject:indexPath];
    cell.selected = selected;
    [self configureCell:cell atIndexPath:indexPath withDataItem:data];
    return cell;
}

- (UICollectionReusableView *)collectionView:(UICollectionView *)collectionView viewForSupplementaryElementOfKind:(NSString *)kind atIndexPath:(NSIndexPath *)indexPath{
    NSString *title =  [self titles1ForAtSection:indexPath.section];
    NSString *reuseIdentifier = @"FSCollectionTitleView";
    if ([kind isEqualToString:UICollectionElementKindSectionHeader]) {
        FSCollectionTitleView *titleHeader = (FSCollectionTitleView *)[collectionView dequeueReusableSupplementaryViewOfKind:UICollectionElementKindSectionHeader withReuseIdentifier:reuseIdentifier forIndexPath:indexPath];
        titleHeader.disableTitleUppercase = self.disableTitleUppercase;
        [titleHeader.titleLab mas_updateConstraints:^(MASConstraintMaker *make) {
            make.centerY.fs_equalToSuperview().offset(indexPath.section == 0 ? 4 : 0);
        }];
        titleHeader.title = title;
        titleHeader.needMakeUpSectionHeaderInset = self.needMakeUpSectionHeaderInset;
        return titleHeader;
    }
    return nil;
}


#pragma mark -UICollectionViewDelegate

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath{
    if (self.customEditing) {
        [self selectDataSourceItem:YES atIndexPath:indexPath];
        return;
    }
    [collectionView deselectItemAtIndexPath:indexPath animated:YES];
}

- (void)collectionView:(UICollectionView *)collectionView didDeselectItemAtIndexPath:(NSIndexPath *)indexPath{
    if (self.customEditing) {
        [self selectDataSourceItem:NO atIndexPath:indexPath];
        return;
    }
}

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout referenceSizeForHeaderInSection:(NSInteger)section{
    NSString *title =  [self titles1ForAtSection:section];
    if (!title) return CGSizeMake(0, 0);
    if (section == 0) {
        return CGSizeMake(collectionView.fs_width, 32.f);
    }
    return CGSizeMake(collectionView.fs_width, 24.f);
}
@end

@interface FSCollectionTitleView ()
@property (nonatomic, strong) CALayer *makeUpLayer;
@end
@implementation FSCollectionTitleView

- (instancetype)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:frame];
    if (self) {
        self.backgroundColor = self.fs_backgroundColor_B2;
    }
    return self;
}

- (void)setNeedMakeUpSectionHeaderInset:(BOOL)needMakeUpSectionHeaderInset{
    _needMakeUpSectionHeaderInset = needMakeUpSectionHeaderInset;
}

#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
- (void)traitCollectionDidChange:(UITraitCollection *)previousTraitCollection{
    [super traitCollectionDidChange:previousTraitCollection];
    if (@available(iOS 13.0, *)) {
        if ([self.traitCollection hasDifferentColorAppearanceComparedToTraitCollection:previousTraitCollection]) {
            _makeUpLayer.backgroundColor = self.backgroundColor.CGColor;
        }
    }
}
#endif

- (void)layoutSubviews{
    [super layoutSubviews];
    
    if (self.needMakeUpSectionHeaderInset) {
        if (!_makeUpLayer) {
            _makeUpLayer = [CALayer new];
            _makeUpLayer.backgroundColor = self.backgroundColor.CGColor;
        }
        [self.layer insertSublayer:_makeUpLayer atIndex:0];
        UICollectionView *superview = (UICollectionView *)self.superview;
        _makeUpLayer.frame = CGRectMake(-superview.adjustedContentInset.left, 0, self.superview.fs_width + superview.adjustedContentInset.left + superview.adjustedContentInset.right , self.fs_height);
    }else{
        [_makeUpLayer removeFromSuperlayer];
    }
}

- (UILabel *)titleLab{
    if(!_titleLab){
        UILabel *titleLab = [UILabel new];
        titleLab.font = self.fs_standardWeakFont_WF1;
        titleLab.backgroundColor = [UIColor clearColor];
        titleLab.textColor = self.fs_textColor_T3;
        [self addSubview:titleLab];
        
        [titleLab mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(titleLab.superview.fs_mas_left).offset(ITEM_MARGIN_NORMAL);
            make.centerY.fs_equalToSuperview().offset(0);
        }];
        
        _titleLab = titleLab;
    }
    return _titleLab;
}

- (void)setTitle:(NSString *)title{
    _title = title;
    _titleLab.text = self.disableTitleUppercase ? title : [title uppercaseString];
}
@end
