/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/

#import "FSNavigationController.h"
#import "FSViewController.h"
#import "FSPreferences.h"

@interface FSNavigationController ()
@end

@implementation FSNavigationController

- (instancetype)initWithRootViewController:(UIViewController *)rootViewController{
    self = [super initWithRootViewController:rootViewController];
    if (self) {
        self.fs_presentByTopViewController = rootViewController.fs_presentByTopViewController;
        if ([rootViewController isKindOfClass:[FSViewController class]]) {
            self.navigationBarHidden = YES;
            FSViewController *viewController = (FSViewController *)rootViewController;
            if (![[viewController.services navigationControllerStack] topNavigationController]) {
                [viewController.services.navigationControllerStack pushNavigationController:self];
            }
        }
    }
    return self;;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    BindThemeKeyPath(self.navigationBar, barTintColor);
    self.navigationBar.titleTextAttributes = @{NSForegroundColorAttributeName : [FSPreferences sharePreferences].textColor_T2};
}

- (void)dismissViewControllerAnimated:(BOOL)flag completion:(void (^)(void))completion{
    [super dismissViewControllerAnimated:flag completion:^{
        !completion ? : completion();
        for (UIViewController *viewController in self.viewControllers) {
            if (!viewController.view.window && !self.presentingViewController) {
                if ([viewController isKindOfClass:[FSViewController class]]) {
                    FSViewController *fs_viewController = (FSViewController *)viewController;
                    if (fs_viewController.services.navigationControllerStack.rootNavigationController == self) {
                        [fs_viewController.services.navigationControllerStack forcedPopNavigationController];
                    }else if ([fs_viewController.services.navigationControllerStack containsNavigationController:self]) {
                        [fs_viewController.services.navigationControllerStack popNavigationController];
                    }
                    break;
                }
            }
        }
    }];
}

- (UIInterfaceOrientationMask)supportedInterfaceOrientations {
    if ([[self topViewController] respondsToSelector:@selector(supportedInterfaceOrientations)]) {
        return [[self topViewController] supportedInterfaceOrientations];
    }
    return  [super supportedInterfaceOrientations];
}

- (UIInterfaceOrientation)preferredInterfaceOrientationForPresentation{
    if ([[self topViewController] respondsToSelector:@selector(preferredInterfaceOrientationForPresentation)]) {
        return  [[self topViewController] preferredInterfaceOrientationForPresentation];
    }
    return  [super preferredInterfaceOrientationForPresentation];
}

- (BOOL)shouldAutorotate{
    if ([[self topViewController] respondsToSelector:@selector(shouldAutorotate)]) {
        return  [[self topViewController] shouldAutorotate];
    }
    return  [super shouldAutorotate];
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
