/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/

#if __has_include(<FSFoundation/FSFoundation.h>)
#import <FSFoundation/FSViewController.h>
#import <FSFoundation/FSStyleValue1TableViewCell.h>
#import <FSFoundation/FSTitleEstimatedTableViewCell.h>
#import <FSFoundation/FSSubjectEstimatedTableViewCell.h>
#import <FSFoundation/FSDescriptionEstimatedTableViewCell.h>
#else
#import "FSViewController.h"
#import "FSStyleValue1TableViewCell.h"
#import "FSTitleEstimatedTableViewCell.h"
#import "FSSubjectEstimatedTableViewCell.h"
#import "FSDescriptionEstimatedTableViewCell.h"
#endif

NS_ASSUME_NONNULL_BEGIN

//only tableview.style = UITableViewStylePlain to take effect
typedef NS_ENUM(NSUInteger, FSTableViewSectionStyle) {
    FSTableViewSectionStyleNone,
    FSTableViewSectionStyleFirstNone,
    FSTableViewSectionStyleTitle,
    FSTableViewSectionStyleLikeGrouped,
};

@interface FSTableViewController : FSViewController<UITableViewDelegate, UITableViewDataSource>
@property (nonatomic, strong, readonly) IBOutlet __kindof UITableView *tableView;
@property (nonatomic, copy) NSArray *sectionIndexTitles1;
@property (nonatomic, copy) NSArray *sectionIndexTitles2;
@property (nonatomic, assign) CGFloat sectionHeaderHeight;
@property (nonatomic, assign) FSTableViewSectionStyle sectionStyle;
@property (nonatomic, assign) BOOL needMakeUpSectionHeaderInset;
@property (nonatomic, assign) BOOL disableTitleUppercase;

@property (nonatomic, strong, nullable) NSArray *selectedDataSource;
@property (nonatomic, copy, nullable) NSArray<NSIndexPath *> *selectedIndexPaths;
@property (nonatomic, strong) UIColor *cellSelectedColor;

@property (nonatomic, copy, nullable) NSArray *dataSource;
@property (nonatomic, assign, readonly) UIEdgeInsets contentInset;
@property (nonatomic, assign) UIEdgeInsets subviewTableViewLayoutMargins;
@property (nonatomic, assign, readonly) UITableViewStyle style;

@property (nonatomic, assign) BOOL editing;
@property (nonatomic, strong) UIImage *editingImageNormal;
@property (nonatomic, strong) UIImage *editingImageSelected;

@property (nonatomic, assign, readonly) BOOL emptyDataSetDisplayed;

//
@property (nonatomic, assign) FSEstimatedTableViewCellStyle defaultEstimatedCellStyle;

@property (nonatomic, strong, readonly) FSToolbar *editBar;
@property (nonatomic, strong, readonly) FSItem *editLeftItem;
@property (nonatomic, strong, readonly) FSItem *editRightItem;

- (void)reloadData;
- (void)reloadRowsAtIndexPaths:(id)indexPaths;
- (void)insertRowsAtIndexPaths:(id)indexPaths;
- (void)insertRowsAtIndexPath:(NSIndexPath *)indexPath;
- (void)deleteRowsAtIndexPaths:(id)indexPaths;
- (void)selectRow:(BOOL)select atIndexPath:(NSIndexPath *)indexPath;
- (void)selectDataSourceRow:(BOOL)select atIndexPath:(NSIndexPath *)indexPath;
- (NSString *)dequeueReusableIdentifierWithIndexPath:(NSIndexPath *)indexPath;
- (__kindof UITableViewCell *)tableView:(UITableView *)tableView dequeueReusableCellWithIdentifier:(NSString *)identifier forIndexPath:(NSIndexPath *)indexPath withDataItem:(nullable id)dataItem;
- (id)dataForAtIndexPath:(NSIndexPath *)indexPath;
- (NSString *)titles1ForAtSection:(NSUInteger)section;
- (NSString *)titles2ForAtSection:(NSUInteger)section;
- (void)configureCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(id)dataItem;
- (void)setCell:(UITableViewCell *)cell separatorInset:(UIEdgeInsets)separatorInset;

//EmptyDataSet
- (void)reloadEmptyDataSet;
- (NSString *)titleForEmptyDataSet;
- (NSAttributedString *)titleForEmptyDataSet:(UITableView *)tableView;
- (UIImage *)imageForEmptyDataSet:(UITableView *)tableView;
- (CGFloat)verticalOffsetForEmptyDataSet:(UITableView *)tableView;
- (BOOL)emptyDataSetShouldDisplay:(UITableView *)tableView;
- (BOOL)emptyDataSetShouldAllowScroll:(UITableView *)tableView;

- (void)clickedEditLeftItem;
- (void)clickedEditRightItem;

- (void)selectAll;
- (void)deselectAll;
@end

NS_ASSUME_NONNULL_END
