/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/

#import "FSTableViewController.h"
#if __has_include(<YZHookHandler/YZHookHandler.h>)
#import <YZHookHandler/YZHookHandler.h>
#else
#import "YZHookHandler.h"
#endif
#import "NSObject+FSAddForKVO.h"
#import "FSPreferences.h"
#import "Masonry+FSAdd.h"
#import "UITableView+FSAdd.h"
#import "UIImage+FSAdd.h"
#import "UIView+FSAdd.h"
#import "FSDataItem.h"
#import "FSItem.h"
#import "FSemptyDataSetView.h"
#import "FSTableView.h"

@interface FSTableViewController ()
@property (nonatomic, strong) FSEmptyDataSetView *emptyDataSetView;
@property (nonatomic, strong, readwrite) FSTableView *tableView;
@property (nonatomic, strong, readwrite) FSToolbar *editBar;
@property (nonatomic, strong, readwrite) FSItem *editLeftItem;
@property (nonatomic, strong, readwrite) FSItem *editRightItem;
@property (nonatomic, assign) BOOL didLoadData;
@property (nonatomic, weak) MASConstraint *tableViewBottomConstraint;

@end

@implementation FSTableViewController
@synthesize tableView = _tableView;

+ (instancetype)allocWithZone:(struct _NSZone *)zone {
    FSTableViewController *viewController = [super allocWithZone:zone];
    
    @weakify(viewController)
    [[viewController yz_hookForSelector:@selector(numberOfSectionsInTableView:) fromProtocol:@protocol(UITableViewDataSource)]
     after:^(NSArray * _Nonnull args) {
        @strongify(viewController)
        [viewController reloadEmptyDataSet];
    }];
    
    [[[viewController yz_hookForSelector:@selector(loadData)]
      befor:^(NSArray * _Nonnull args) {
        @strongify(viewController)
        viewController.didLoadData = NO;
    }] after:^(NSArray * _Nonnull args) {
        @strongify(viewController)
        viewController.didLoadData = YES;
    }];
    
    return viewController;
}

- (void)initialize {
    [super initialize];
    _cellSelectedColor = [UIColor clearColor];
    if (self.style == UITableViewStyleGrouped) _sectionHeaderHeight = 0;
    else _sectionHeaderHeight = DIVIDE_SECTION_HEADER_HEIGHT;
    
    _defaultEstimatedCellStyle = FSEstimatedTableViewCellStyleNone;
    _sectionStyle = FSTableViewSectionStyleNone;
    
    [self fs_addObserverBlockForKeyPath:@keypath(self, dataSource)
                                  block:^(FSTableViewController * _Nonnull _self, id  _Nullable oldVal, id  _Nullable newVal) {
        dispatch_main_async_safe(^{
            [_self reloadData];
        });
        
    }];
    
    [BindThemeKeySel(self, reloadEmptyDataSet) fs_updateThemeCompleted:^(FSTableViewController * _Nonnull target, SEL  _Nonnull sel) {
        dispatch_main_async_safe(^{
            if (target.emptyDataSetView.superview) {
                [target reloadEmptyDataSet];
            }
        });
    }];
}

- (void)dealloc{
    [self fs_removeObserverBlocks];
}

- (FSEmptyDataSetView *)emptyDataSetView{
    if (!_emptyDataSetView) {
        _emptyDataSetView = [FSEmptyDataSetView new];
    }
    return  _emptyDataSetView;
}

- (BOOL)emptyDataSetDisplayed{
    return _emptyDataSetView.superview;
}

#pragma mark - life cycle
- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    if (self.view != self.tableView) [self.view addSubview:self.tableView];
    
}

- (UITableViewStyle)style{
    return UITableViewStylePlain;
}

- (UIEdgeInsets)contentInset {
    return UIEdgeInsetsZero;
}

- (FSTableView *)tableView{
    if (!_tableView) {
        _tableView = [[FSTableView alloc] initWithFrame:CGRectZero style:self.style];
        _tableView.contentInset  = self.contentInset;
        _tableView.scrollIndicatorInsets = self.contentInset;
        
        _tableView.sectionIndexColor = [UIColor darkGrayColor];
        _tableView.sectionIndexBackgroundColor = [UIColor clearColor];
        _tableView.sectionIndexMinimumDisplayRowCount = 20;
        _tableView.separatorStyle = UITableViewCellSeparatorStyleNone;
        _tableView.allowsMultipleSelectionDuringEditing = YES;
        
        [self unifiedTableView:_tableView];
    }
    return _tableView;
}

- (void)setTableView:(FSTableView *)tableView{
    _tableView = tableView;
    [self unifiedTableView:tableView];
}

- (FSToolbar *)editBar{
    if (!_editBar) {
        _editBar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleBottom];
        [_editBar addItem:self.editLeftItem atPosition:FSItemPositionLeft];
        [_editBar addItem:self.editRightItem atPosition:FSItemPositionRight];
    }
    return _editBar;
}

- (FSItem *)editLeftItem{
    if (!_editLeftItem) {
        _editLeftItem = [FSItem createItemWithTitle:@"Clear All"];
        [_editLeftItem addTarget:self action:@selector(clickedEditLeftItem) forControlEvents:UIControlEventTouchUpInside];
    }
    return _editLeftItem;
}

- (FSItem *)editRightItem{
    if (!_editRightItem) {
        _editRightItem = [FSItem createItemWithTitle:@"Delete"];
        _editRightItem.enabled = NO;
        [_editRightItem addTarget:self action:@selector(clickedEditRightItem) forControlEvents:UIControlEventTouchUpInside];
    }
    return _editRightItem;
}

- (void)makeConstraints{
    [super makeConstraints];
    [self makeTableViewConstraints];
}

- (void)makeTableViewConstraints{
    if (self.view != self.tableView) {
        if (UIEdgeInsetsEqualToEdgeInsets(self.subviewTableViewLayoutMargins, UIEdgeInsetsZero)) {
            [self.tableView mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.left.equalTo(self.tableView.superview);
                make.right.equalTo(self.tableView.superview);
                self.tableViewBottomConstraint = make.bottom.equalTo(self.shouldShowBottomBar ? self.bottomBar.mas_top : self.tableView.superview.fs_mas_bottom);
                make.top.equalTo(self.shouldShowNavigationBar ? self.navBar.mas_bottom : self.tableView.superview.fs_mas_top);
            }];
        }else{
            [self.tableView mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.left.equalTo(self.tableView.superview).offset(self.subviewTableViewLayoutMargins.left);
                make.right.equalTo(self.tableView.superview).offset(-self.subviewTableViewLayoutMargins.right);
                self.tableViewBottomConstraint = make.bottom.equalTo(self.shouldShowBottomBar ? self.bottomBar.mas_top : self.tableView.superview).offset(-self.subviewTableViewLayoutMargins.bottom);
                make.top.equalTo(self.shouldShowNavigationBar ? self.navBar.mas_bottom : self.tableView.superview).offset(self.subviewTableViewLayoutMargins.top);
            }];
        }
    }
}

- (void)setSelectedDataSource:(NSArray *)selectedDataSource{
    _selectedDataSource = selectedDataSource;
    self.editRightItem.enabled = selectedDataSource.count;
}

- (BOOL)editing{
    return self.tableView.editing;
}

- (void)unifiedTableView:(UITableView *)tableView{
    tableView.delegate = self;
    tableView.dataSource = self;
    if (tableView.style == UITableViewStyleGrouped || _sectionStyle == FSTableViewSectionStyleLikeGrouped) tableView.backgroundColor = self.fs_backgroundColor_B2;
    else tableView.backgroundColor = self.fs_backgroundColor_B1;
    
    switch (self.defaultEstimatedCellStyle) {
        case FSEstimatedTableViewCellStyleNone:
            tableView.estimatedRowHeight = UITableViewAutomaticDimension;
            tableView.rowHeight = PREFERENCE_CELL_ROW_HEIGHT_NORMAL;

            break;
        case FSEstimatedTableViewCellStyleTitle:
            tableView.estimatedRowHeight = PREFERENCE_CELL_ROW_HEIGHT_V_V_LAGER;
            tableView.rowHeight = UITableViewAutomaticDimension;

            break;
        case FSEstimatedTableViewCellStyleSubject:
            tableView.estimatedRowHeight = PREFERENCE_CELL_ROW_HEIGHT_V_V_LAGER;
            tableView.rowHeight = UITableViewAutomaticDimension;

            break;
        case FSEstimatedTableViewCellStyleDescription:
            tableView.estimatedRowHeight = PREFERENCE_CELL_ROW_HEIGHT_V_V_V_GREAT;
            tableView.rowHeight = UITableViewAutomaticDimension;
            
            break;
    }
    [tableView fs_registerClass:[FSStyleValue1TableViewCell class]];
    [tableView fs_registerClass:[FSTitleEstimatedTableViewCell class]];
    [tableView fs_registerClass:[FSSubjectEstimatedTableViewCell class]];
    [tableView fs_registerClass:[FSDescriptionEstimatedTableViewCell class]];
    
    @weakify(self)
    [[tableView yz_hookForSelector:@selector(setEditing:)] after:^(NSArray * _Nonnull args) {
        NSNumber *editing = args[0];
        @strongify(self)
        if ([editing boolValue]) {
            self.selectedDataSource = @[];
            self.selectedIndexPaths = @[];
        }
    }];
}


- (void)setNeedMakeUpSectionHeaderInset:(BOOL)needMakeUpSectionHeaderInset{
    _needMakeUpSectionHeaderInset = needMakeUpSectionHeaderInset;
    for (NSIndexPath *indexPath in self.tableView.indexPathsForVisibleRows) {
        UITableViewCell *cell = [self.tableView cellForRowAtIndexPath:indexPath];
        if ([cell isKindOfClass:[FSTableViewCell class]]) {
            FSTableViewCell *fs_cell = (FSTableViewCell *)cell;
            fs_cell.hightlightBackgroundColorFillHor = needMakeUpSectionHeaderInset;
        }
    }
}

- (void)setSubviewTableViewLayoutMargins:(UIEdgeInsets)subviewTableViewLayoutMargins{
    if (UIEdgeInsetsEqualToEdgeInsets(_subviewTableViewLayoutMargins, subviewTableViewLayoutMargins)) return;
    _subviewTableViewLayoutMargins = subviewTableViewLayoutMargins;
    [self makeTableViewConstraints];
}

- (void)setEditing:(BOOL)editing{
    if ([self emptyDataSetDisplayed] && !self.editing && _didLoadData) return;
    self.tableView.editing = editing;
    if (editing) {
        [self.view addSubview:self.editBar];
        [self.editBar mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.right.bottom.fs_equalToSuperview();
        }];
        
        if (self.tableViewBottomConstraint) {
            [self.tableViewBottomConstraint uninstall];
        }
        [self.tableView mas_updateConstraints:^(MASConstraintMaker *make) {
            make.bottom.mas_equalTo(self.editBar.mas_top);
        }];
    }else{
        if (_editBar) {
            [self.editBar removeFromSuperview];
            [self makeTableViewConstraints];
        }
    }
    for (NSIndexPath *indexPath in self.tableView.indexPathsForVisibleRows) {
        UITableViewCell *cell = [self.tableView cellForRowAtIndexPath:indexPath];
        if (editing) {
            if ([cell isKindOfClass:[FSTableViewCell class]]) {
                FSTableViewCell *fs_cell = (FSTableViewCell *)cell;
                [fs_cell.editItem setImage:self.editingImageNormal forState:UIControlStateNormal];
                @weakify(self)
                [BindThemeKeySel(fs_cell.editItem, setImage:forState:) fs_updateThemeCompleted:^(FSItem * _Nonnull target, SEL  _Nonnull sel) {
                    @strongify(self)
                    [target setImage:[self.editingImageSelected fs_imageWithTintColor:target.fs_primaryColor] forState:UIControlStateSelected];
                }];
            }
        }
        cell.selectedBackgroundView.backgroundColor = self.cellSelectedColor ? self.cellSelectedColor : cell.contentView.backgroundColor;
    }
}

- (void)setDefaultEstimatedCellStyle:(FSEstimatedTableViewCellStyle)defaultEstimatedCellStyle{
    _defaultEstimatedCellStyle = defaultEstimatedCellStyle;
}

- (void)loadData{}

- (void)reloadData {
    [self.tableView reloadData];
}

- (void)reloadRowsAtIndexPaths:(id)indexPaths{
    if (!indexPaths) return;
    [self.tableView reloadRowsAtIndexPaths:[indexPaths isKindOfClass:[NSArray class]] ? indexPaths : @[indexPaths] withRowAnimation:UITableViewRowAnimationNone];
}

- (void)insertRowsAtIndexPath:(NSIndexPath *)indexPath{
    NSMutableArray *tmpDataIndexPaths = self.selectedIndexPaths.mutableCopy;
    for (int i = 0; i < self.selectedIndexPaths.count; i++) {
        NSIndexPath *selectedIndexPath = self.selectedIndexPaths[i];
        if (selectedIndexPath.row >= indexPath.row) {
            [tmpDataIndexPaths replaceObjectAtIndex:i withObject:[NSIndexPath indexPathForRow:selectedIndexPath.row+1 inSection:selectedIndexPath.section]];
        }
    }
    self.selectedIndexPaths = tmpDataIndexPaths.copy;
    [self insertRowsAtIndexPaths:indexPath];
}

- (void)insertRowsAtIndexPaths:(id)indexPaths{
    if (!indexPaths) return;
    [self.tableView insertRowsAtIndexPaths:[indexPaths isKindOfClass:[NSArray class]] ? indexPaths : @[indexPaths] withRowAnimation:UITableViewRowAnimationNone];
}

- (void)deleteRowsAtIndexPaths:(id)indexPaths{
    if (!indexPaths) return;
    [self.tableView deleteRowsAtIndexPaths:[indexPaths isKindOfClass:[NSArray class]] ? indexPaths : @[indexPaths] withRowAnimation:UITableViewRowAnimationBottom];
}

- (void)selectRow:(BOOL)select atIndexPath:(NSIndexPath *)indexPath{
    if (select) {
        [self.tableView selectRowAtIndexPath:indexPath animated:NO scrollPosition:UITableViewScrollPositionNone];
    }else{
        [self.tableView deselectRowAtIndexPath:indexPath animated:NO];
    }
}

- (void)selectDataSourceRow:(BOOL)select atIndexPath:(NSIndexPath *)indexPath{
    NSMutableArray *tmpDataSource = self.selectedDataSource.mutableCopy;
    NSMutableArray *tmpDataIndexPaths = self.selectedIndexPaths.mutableCopy;
    id data = [self dataForAtIndexPath:indexPath];
    if ([data isKindOfClass:[FSDataItem class]]) {
        FSDataItem *dataItem = (FSDataItem *)data;
        dataItem.editingSelected = select;
    }
    if (select) {
        if (![tmpDataSource containsObject:data]) {
            [tmpDataSource addObject:data];
        }
        
        if (![tmpDataIndexPaths containsObject:indexPath]) {
            [tmpDataIndexPaths addObject:indexPath];
        }
    }else{
        if ([tmpDataSource containsObject:data]) {
            [tmpDataSource removeObject:data];
        }
        
        if ([tmpDataIndexPaths containsObject:indexPath]) {
            [tmpDataIndexPaths removeObject:indexPath];
        }
    }
    self.selectedDataSource = tmpDataSource.copy;
    self.selectedIndexPaths = tmpDataIndexPaths.copy;
    [self selectRow:select atIndexPath:indexPath];
    UITableViewCell *cell = [self.tableView cellForRowAtIndexPath:indexPath];
    cell.selectedBackgroundView.backgroundColor = select ? cell.contentView.backgroundColor : self.cellSelectedColor ? self.cellSelectedColor : cell.contentView.backgroundColor;
}

- (NSString *)dequeueReusableIdentifierWithIndexPath:(NSIndexPath *)indexPath{
    NSString *identifier = NSStringFromClass([FSStyleValue1TableViewCell class]);
    switch (self.defaultEstimatedCellStyle) {
        case FSEstimatedTableViewCellStyleNone:
        case FSEstimatedTableViewCellStyleTitle:
            identifier = NSStringFromClass([FSTitleEstimatedTableViewCell class]);
            break;
        case FSEstimatedTableViewCellStyleSubject:
            identifier = NSStringFromClass([FSSubjectEstimatedTableViewCell class]);
            break;
        case FSEstimatedTableViewCellStyleDescription:
            identifier = NSStringFromClass([FSDescriptionEstimatedTableViewCell class]);
            break;
    }
    return identifier;
}

- (__kindof UITableViewCell *)tableView:(UITableView *)tableView dequeueReusableCellWithIdentifier:(NSString *)identifier forIndexPath:(NSIndexPath *)indexPath withDataItem:(id)dataItem{
    return [tableView dequeueReusableCellWithIdentifier:identifier forIndexPath:indexPath];
}

- (id)dataForAtIndexPath:(NSIndexPath *)indexPath{
    @try {
        if ([self.dataSource count] > indexPath.section) {
            id sectionData = self.dataSource[indexPath.section];
            if ([sectionData isKindOfClass:[NSArray class]]) {
                if ([sectionData count] > indexPath.row) return sectionData[indexPath.row];
            }else{
                if ([self.dataSource count] > indexPath.row) {
                    return self.dataSource[indexPath.row];
                }
            }
            return nil;
        }
    } @catch (NSException *exception) {
        return nil;
    }
    return nil;
}

- (NSString *)titles1ForAtSection:(NSUInteger)section{
    if (self.sectionIndexTitles1.count > section) {
        return self.sectionIndexTitles1[section];
    }
    return nil;
}
- (NSString *)titles2ForAtSection:(NSUInteger)section{
    if (self.sectionIndexTitles2.count > section) {
        return self.sectionIndexTitles2[section];
    }
    return nil;
}

- (void)configureCell:(UITableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(id)dataItem {}

- (void)setCell:(UITableViewCell *)cell separatorInset:(UIEdgeInsets)separatorInset{
    if ([cell respondsToSelector:@selector(setPreservesSuperviewLayoutMargins:)]) {
        [cell setPreservesSuperviewLayoutMargins:NO];
    }
    if ([cell respondsToSelector:@selector(setSeparatorInset:)]) {
        [cell setSeparatorInset:separatorInset];
    }
    if ([cell respondsToSelector:@selector(setLayoutMargins:)]) {
        [cell setLayoutMargins:separatorInset];
    }
}

- (void)reloadEmptyDataSet{
    if ([self emptyDataSetShouldDisplay:self.tableView]) {
        if (self.emptyDataSetView.superview != self.tableView) {
            [self.tableView insertSubview:self.emptyDataSetView atIndex:0];
            [self.emptyDataSetView mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.center.left.right.bottom.top.fs_equalToSuperview();
            }];
        }
        self.emptyDataSetView.title = [self titleForEmptyDataSet:self.tableView];
        self.emptyDataSetView.image = [self imageForEmptyDataSet:self.tableView];
        self.emptyDataSetView.verticalOffset = [self verticalOffsetForEmptyDataSet:self.tableView];
        self.emptyDataSetView.scrollEnabledForSupView = ![self emptyDataSetShouldAllowScroll:self.tableView];
        self.emptyDataSetView.canClick = [self emptyDataSetShouldAllowClick:self.tableView];
    }else{
        if (self.emptyDataSetView.superview) {
            [self.emptyDataSetView removeFromSuperview];
        }
    }
}

- (NSString *)titleForEmptyDataSet{
    return @"NO Data";
}

- (NSAttributedString *)titleForEmptyDataSet:(UITableView *)tableView {
    return [[NSAttributedString alloc] initWithString:[self titleForEmptyDataSet] attributes:@{
        NSFontAttributeName:self.fs_standardGeneralFont_GF3,
        NSForegroundColorAttributeName:self.fs_textColor_T3
        }];
}

- (UIImage *)imageForEmptyDataSet:(UITableView *)tableView {
    return nil;
}

- (CGFloat)verticalOffsetForEmptyDataSet:(UITableView *)tableView {
    return 0;
}

- (BOOL)emptyDataSetShouldDisplay:(UITableView *)tableView {
    @try {
        BOOL isEmpty = ![self dataForAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0]];
        if (isEmpty) {
            for (id dataLv1 in self.dataSource) {
                if ([dataLv1 isKindOfClass:[NSArray class]]) {
                    NSArray *datas = (NSArray *)dataLv1;
                    isEmpty = !datas.count;
                }
            }
        }
        return isEmpty;
    } @catch (NSException *exception) {
        return YES;
    }
    return YES;
}

- (BOOL)emptyDataSetShouldAllowScroll:(UITableView *)tableView{
    return NO;
}

- (BOOL)emptyDataSetShouldAllowClick:(UITableView *)tableView{
    return NO;
}

- (void)clickedEditLeftItem {}
- (void)clickedEditRightItem {self.selectedIndexPaths = @[]; self.selectedDataSource = @[];}
- (void)selectAll{
    if (!self.editing) {
        return;
    }
    NSInteger section = [self numberOfSectionsInTableView:self.tableView];
    for (int s = 0 ; s < section; s++ ) {
        NSInteger row = [self tableView:self.tableView numberOfRowsInSection:s];
        for (int r = 0; r < row; r++) {
            NSIndexPath *indexPath = [NSIndexPath indexPathForRow:r inSection:s];
            [self selectRow:YES atIndexPath:indexPath];
            [self selectDataSourceRow:YES atIndexPath:indexPath];
        }
    }
}

- (void)deselectAll{
    if (!self.editing) {
        return;
    }
    for (NSIndexPath *selectedIndexPath in self.selectedIndexPaths) {
        id data = [self dataForAtIndexPath:selectedIndexPath];
        if ([data isKindOfClass:[FSDataItem class]]) {
            FSDataItem *dataItem = (FSDataItem *)data;
            if (!dataItem.editingSelected) dataItem.editingSelected = NO;
        }
        [self selectRow:NO atIndexPath:selectedIndexPath];
    }
    self.selectedDataSource = @[];
    self.selectedIndexPaths = @[];
}

#pragma mark - UITableViewDataSource

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return self.dataSource ? self.dataSource.count : 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    if (self.dataSource.count > section) {
        id sectionData = self.dataSource[section];
        if ([sectionData isKindOfClass:[NSArray class]]) {
            return [sectionData count];
        }
        return self.dataSource.count;
    }
    return 0;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {

    id data = [self dataForAtIndexPath:indexPath];
    
    FSTableViewCell *cell = [self tableView:tableView dequeueReusableCellWithIdentifier:[self dequeueReusableIdentifierWithIndexPath:indexPath] forIndexPath:indexPath withDataItem:data];
    
    cell.selectedBackgroundView = [UIView new];
    if ([cell isKindOfClass:[FSTableViewCell class]]) {
        cell.hightlightBackgroundColorFillHor = self.needMakeUpSectionHeaderInset;
        cell.supTableView = tableView;
        cell.data = data;
        
        cell.editingHiddenRightView = YES;
        if (self.editing) {
            [cell.editItem setImage:self.editingImageNormal forState:UIControlStateNormal];
            @weakify(self)
            [BindThemeKeySel(cell.editItem, setImage:forState:) fs_updateThemeCompleted:^(FSItem * _Nonnull target, SEL  _Nonnull sel) {
                @strongify(self)
                [target setImage:[self.editingImageSelected fs_imageWithTintColor:target.fs_primaryColor] forState:UIControlStateSelected];
            }];
        }
        if (self.cellSelectedColor) {
            cell.contentView.backgroundColor = nil;
        }
    }
    cell.selectedBackgroundView.backgroundColor = self.cellSelectedColor ? self.cellSelectedColor : cell.contentView.backgroundColor;
    
    if ([data isKindOfClass:[FSDataItem class]]) {
        FSDataItem *dataItem = (FSDataItem *)data;
        if (self.editing) {
            [self selectRow:dataItem.editingSelected atIndexPath:indexPath];
        }
        else {
            if (dataItem.editingSelected) dataItem.editingSelected = NO;
        }
    }else{
        if (self.editing) {
            if ([self.selectedIndexPaths containsObject:indexPath]) {
                [self selectRow:YES atIndexPath:indexPath];
            }
        }
    }
    
    [self configureCell:cell atIndexPath:indexPath withDataItem:data];
    
    if ([cell isKindOfClass:[FSEstimatedTableViewCell class]]) {
        [(FSEstimatedTableViewCell *)cell installConstraints];
    }
    
    return cell;
}

    
#pragma mark - UITableViewDelegate

- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section{
    if (self.style == UITableViewStyleGrouped || self.sectionStyle == FSTableViewSectionStyleLikeGrouped){
        NSString *title = nil;
        if ([self respondsToSelector:@selector(tableView:titleForHeaderInSection:)]) {
            title = [self tableView:tableView titleForHeaderInSection:section];
        }else{
            title = [self titles1ForAtSection:section];
        }
        if (title) {
            UIView *sectionView = [UIView new];
            sectionView.backgroundColor = self.fs_backgroundColor_B2;
            UILabel *titleLabel = [UILabel new];
            titleLabel.font = self.fs_standardWeakFont_WF1;
            titleLabel.backgroundColor = [UIColor clearColor];
            titleLabel.textColor = self.fs_textColor_T3;
            titleLabel.text = self.disableTitleUppercase ? title : [title uppercaseString];
            [sectionView addSubview:titleLabel];
            
            [titleLabel mas_makeConstraints:^(MASConstraintMaker *make) {
                make.left.equalTo(sectionView.fs_mas_left).offset(ITEM_MARGIN_NORMAL);
                if (section == 0 && [self tableView:tableView heightForHeaderInSection:section] == PREFERENCE_GROUPED_PRIMARY_SECTION_HEADER_HEIGHT) {
                    make.centerY.fs_equalToSuperview().offset((PREFERENCE_GROUPED_PRIMARY_SECTION_HEADER_HEIGHT - PREFERENCE_GROUPED_SECONDARY_SECTION_HEADER_HEIGHT)/2);
                }else{
                    make.centerY.fs_equalToSuperview();
                }
            }];
            return sectionView;
        }
    }
    if (self.sectionStyle == FSTableViewSectionStyleNone) {
        return nil;
    }
    if (self.sectionStyle == FSTableViewSectionStyleFirstNone) {
        if (section == 0) return nil;
        UIView *view = [UIView new];
        view.backgroundColor = self.fs_backgroundColor_B2;
        view.fs_height = self.sectionHeaderHeight;
        return view;
    }

    if (self.sectionStyle == FSTableViewSectionStyleTitle) {
        NSString *title1 = [self titles1ForAtSection:section];
        NSString *title2 = [self titles2ForAtSection:section];
        if (title1 || title2) {
            UIView *sectionView = [UIView new];
            sectionView.backgroundColor = self.fs_backgroundColor_B1;
            {
                UIView *line = [UIView new];
                line.backgroundColor = self.fs_dividerColor_P2;
                [sectionView addSubview:line];
                [line mas_makeConstraints:^(MASConstraintMaker *make) {
                    make.right.mas_equalTo(sectionView.fs_mas_right);
                    make.left.mas_equalTo(sectionView.fs_mas_left);
                    make.bottom.mas_equalTo(sectionView);
                    make.height.mas_offset(DIVIDE_TOOL_BAR_THICKNES);
                }];
            }

            if (title1) {
                UILabel *titleLabel = [UILabel new];
                titleLabel.font = self.fs_standardWeakFont_WF1;
                titleLabel.backgroundColor = [UIColor clearColor];
                titleLabel.textColor = self.fs_textColor_T3;
                titleLabel.text = title1;
                
                [sectionView addSubview:titleLabel];
                
                [titleLabel mas_makeConstraints:^(MASConstraintMaker *make) {
                    make.left.equalTo(sectionView.fs_mas_left).offset(ITEM_MARGIN_NORMAL);
                    make.centerY.fs_equalToSuperview();
                }];
            }
            
            if (title2) {
                UILabel *titleLabel = [UILabel new];
                titleLabel.font = self.fs_standardWeakFont_WF1;
                titleLabel.backgroundColor = [UIColor clearColor];
                titleLabel.textColor = self.fs_textColor_T3;
                titleLabel.text = title2;
                
                [sectionView addSubview:titleLabel];
                
                [titleLabel mas_makeConstraints:^(MASConstraintMaker *make) {
                    make.right.equalTo(sectionView.fs_mas_right).offset(-ITEM_MARGIN_NORMAL);
                    make.centerY.fs_equalToSuperview();
                }];
            }
            return sectionView;
        }
    }
    return nil;
}

- (void)tableView:(UITableView *)tableView willDisplayHeaderView:(UIView *)view forSection:(NSInteger)section{
    @fs_weakify(self)
    [[view yz_hookForSelector:@selector(didMoveToSuperview)] after:^(NSArray * _Nonnull args) {
        @fs_strongify(self)
        UIView *tabSuperview = tableView.superview;
        NSString *uuidName = @"FSMakeUpLayerName";
        UIView *makeUpView;
        for (UIView *view in tabSuperview.subviews) {
            if ([view.layer.name isEqualToString:uuidName]) {
                makeUpView = view;
                break;
            }
        }
        if (self.needMakeUpSectionHeaderInset) {
            if (view.superview) {
                if (!makeUpView) {
                    makeUpView = [UIView new];
                    makeUpView.backgroundColor = view.backgroundColor;
                    makeUpView.layer.name = uuidName;
                }
            
                [tabSuperview insertSubview:makeUpView atIndex:0];
                [makeUpView mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.left.right.fs_equalToSuperview();
                    make.top.height.equalTo(view);
                }];
            }
        }else{
            [makeUpView removeFromSuperview];
        }
    }];
}

- (nullable UIView *)tableView:(UITableView *)tableView viewForFooterInSection:(NSInteger)section{
    return nil;
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section{
    if (self.sectionStyle == FSTableViewSectionStyleLikeGrouped) {
        if (section == 0) return PREFERENCE_GROUPED_PRIMARY_SECTION_HEADER_HEIGHT;
        return PREFERENCE_GROUPED_SECONDARY_SECTION_HEADER_HEIGHT;
    }
    if (self.sectionHeaderHeight && self.sectionStyle != FSTableViewSectionStyleNone){
        if (self.sectionStyle == FSTableViewSectionStyleFirstNone && section == 0 && self.style != UITableViewStyleGrouped) {
            return 0.00f;
        }
        return self.sectionHeaderHeight;
    }
    if (self.style == UITableViewStyleGrouped) return PREFERENCE_GROUPED_PRIMARY_SECTION_HEADER_HEIGHT;
    return 0.00f;
}

- (CGFloat)tableView:(UITableView *)tableView heightForFooterInSection:(NSInteger)section{
    return 0;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    if (self.editing) {
        [self selectDataSourceRow:YES atIndexPath:indexPath];
        return;
    }
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

- (void)tableView:(UITableView *)tableView didDeselectRowAtIndexPath:(NSIndexPath *)indexPath{
    if (self.editing) {
        [self selectDataSourceRow:NO atIndexPath:indexPath];
        return;
    }
}

-(void)tableView:(UITableView *)tableView willDisplayCell:(UITableViewCell *)cell forRowAtIndexPath:(NSIndexPath *)indexPat{
    [self setCell:cell separatorInset:UIEdgeInsetsZero];
}
    
- (UITableViewCellEditingStyle)tableView:(UITableView *)tableView editingStyleForRowAtIndexPath:(NSIndexPath *)indexPath{
    return UITableViewCellEditingStyleDelete | UITableViewCellEditingStyleInsert;
}

@end
