/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#if __has_include(<FSFoundation/FSFoundation.h>)
#import <FSFoundation/FSTableViewController.h>
#else
#import "FSTableViewController.h"
#endif

NS_ASSUME_NONNULL_BEGIN

typedef NS_ENUM(NSUInteger, FSTableViewPopoverShowStyle) {
    FSTableViewPopoverShowStyleDefault = 0,
    FSTableViewPopoverShowStyleBubble,
};
@class FSPopoverItem;
typedef void(^FSPopoverItemAction) (FSPopoverItem *popoverItem, int index);
@interface FSPopoverItem : NSObject
@property (nonatomic, strong) UIImage *image;
@property (nonatomic, copy) NSString *title;
@property (nonatomic, copy) FSPopoverItemAction action;
- (instancetype)initWithTitle:(NSString *)title image:(UIImage *)image;
- (instancetype)initWithTitle :(NSString *)title image:(UIImage *)image action:(FSPopoverItemAction)action;
@end

@interface FSPopoverItemGroup : NSObject
@property (nonatomic, copy) NSString *title;
@property (nonatomic, copy) NSArray<FSPopoverItem *> *items;

- (instancetype)initWithTitle:(nullable NSString *)title items:(NSArray <FSPopoverItem *> *)items;
- (instancetype)initWithTitle:(nullable NSString *)title titlesAndImageNames:(NSArray<NSArray<NSString *> *> *)titlesAndImageNames;
@end

@interface FSTableViewPopoverController : FSTableViewController
@property (nonatomic, copy) NSArray <FSPopoverItemGroup *> *groups;
@property (nonatomic, assign, readonly) FSTableViewPopoverShowStyle popoverShowStyle;

///The default is CGSizeZero, popoverSize has a higher priority than popoverWidth
@property (nonatomic, assign) CGSize popoverSize;
///The default is 0.f, popoverSize has a higher priority than popoverWidth
@property (nonatomic, assign) CGFloat popoverWidth;
@property (nonatomic, copy) FSPopoverItemAction action;

- (instancetype)initWithServices:(id<FSViewControllerServices>)services showStyle:(FSTableViewPopoverShowStyle)showStyle groups:(NSArray <FSPopoverItemGroup *> *)groups;
- (void)configureModalPresentationStyle;
- (void)autoCalculatePopoverHeight;
- (CGFloat)getAutoCalculatePopoverHeight;
@end


NS_ASSUME_NONNULL_END
