/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/

#if __has_include(<FSFoundation/FSFoundation.h>)
#import <FSFoundation/FSViewControllerServices.h>
#import <FSFoundation/FSNavigationBar.h>
#else
#import "FSViewControllerServices.h"
#import "FSNavigationBar.h"
#endif

typedef NS_ENUM(NSUInteger, FSViewControllerShowStyle){
    FSViewControllerShowStyleUnknow = 0,
    FSViewControllerShowStylePush,
    FSViewControllerShowStylePresent,
};

NS_ASSUME_NONNULL_BEGIN
@interface FSViewController : UIViewController<FSNavigationBarDelegate>
@property (nonatomic, strong, readonly) id<FSViewControllerServices> services;
@property (nonatomic, strong, readonly) id params;

@property (nonatomic, strong, readonly, nullable) FSNavigationBar *navBar;
@property (nonatomic, strong, readonly, nullable) FSToolbar *bottomBar;
@property (nonatomic, copy, nullable) NSString *navBarLeftTitle;
@property (nonatomic, copy, nullable) NSString *navBarRightTitle;

@property (nonatomic, copy, nullable) VoidBlock commonBlock;
@property (nonatomic, copy, nullable) VoidBlock_int commonBlock_int;
@property (nonatomic, copy, nullable) VoidBlock_BOOL commonBlock_BOOL;
@property (nonatomic, copy, nullable) VoidBlock_id commonBlock_id;

@property (nonatomic, copy, nullable) VoidBlock dismissCompletion;
@property (nonatomic, copy, nullable) VoidBlock splitScrrenDismissCompletion;

//initialize set
@property (nonatomic, assign) BOOL shouldShowNavigationBar;
@property (nonatomic, assign) BOOL shouldShowBottomBar;
@property (nonatomic, assign) BOOL shouldLoadDataOnViewDidLoad;
@property (nonatomic, assign) BOOL shouldBindTemeForStatusBar;


@property (nonatomic, assign, readonly) FSViewControllerShowStyle showStyle;

- (instancetype)init;
- (instancetype)initWithParams:(nullable id)params;
- (instancetype)initWithServices:(nullable id<FSViewControllerServices>)services;
- (instancetype)initWithServices:(nullable id<FSViewControllerServices>)services params:(nullable id)params;

- (void)initialize;
- (void)makeConstraints;
- (void)loadData;
- (void)setViewBackgroundColor:(UIColor *)backgroundColor;
@end

NS_ASSUME_NONNULL_END
