/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSViewPopoverController.h"
#import "FSPresentedViewControllerAnimator.h"
#import "FSPreferences.h"
#import "UIView+FSAdd.h"
#import "Masonry+FSAdd.h"

@interface FSViewPopoverController ()<UIPopoverPresentationControllerDelegate, UIScrollViewDelegate>
@property (nonatomic, assign) FSPopoverShowStyle popoverShowStyle;

@property (nonatomic, weak) UIView *targetView;
@property (nonatomic, strong) UIScrollView *contentView;
@property (nonatomic, weak) FSPresentedViewControllerAnimator *animator;
@property (nonatomic, strong) UIViewController *targetViewCtrl;
@property (nonatomic, weak) UIView *handleView;
@end

@implementation FSViewPopoverController

- (void)initialize{
    [super initialize];
    self.shouldShowNavigationBar = NO;
}

- (instancetype)initWithServices:(id<FSViewControllerServices>)services showStyle:(FSPopoverShowStyle)showStyle targetViewController:(UIViewController *)targetViewController{
    self = [self initWithServices:services showStyle:showStyle targetView:targetViewController.view];
    if (self) {
        self.targetViewCtrl = targetViewController;
        
    }
    return self;
}

- (instancetype)initWithServices:(id<FSViewControllerServices>)services showStyle:(FSPopoverShowStyle)showStyle targetView:(UIView *)view{
    self = [super initWithServices:services];
    if (self) {
        _popoverShowStyle = showStyle;
        _popoverSize = CGSizeZero;
        _targetView = view;
        [self configureModalPresentationStyle];
        [self.contentView addSubview:self.targetView];

    }
    return self;
}

- (void)viewDidLoad{
    [super viewDidLoad];
    if (self.targetView.backgroundColor != [UIColor clearColor]) {
        self.contentView.backgroundColor = self.targetView.backgroundColor;
        self.contentView.delegate = self;
    }
}

- (void)setPopoverSize:(CGSize)popoverSize{
    _popoverSize = popoverSize;
    if (_popoverShowStyle == FSTableViewPopoverShowStyleDefault) {
        FSPresentedViewControllerAnimator *animator = self.animator;
        animator.size = popoverSize;
        animator.maxVerticalHeigh = popoverSize.height;
    }else{
        CGSize size = CGSizeMake(popoverSize.width + self.view.fs_safeAreaInsets.left + self.view.fs_safeAreaInsets.right, popoverSize.height + self.view.fs_safeAreaInsets.top + self.view.fs_safeAreaInsets.bottom);
        self.preferredContentSize = size;
    }
}

- (void)setPopoverWidth:(CGFloat)popoverWidth{
    _popoverWidth = popoverWidth;
    CGFloat height = self.popoverSize.height;
    if (_popoverShowStyle == FSTableViewPopoverShowStyleDefault) {
        FSPresentedViewControllerAnimator *animator = self.animator;
        animator.size = CGSizeMake(popoverWidth, height);
        animator.maxVerticalHeigh = height;
    }else{
        self.preferredContentSize = CGSizeMake(popoverWidth + self.view.fs_safeAreaInsets.left + self.view.fs_safeAreaInsets.right, height);
    }
}

- (void)setAutoresizesFollowContnetView:(BOOL)autoresizesFollowContnetView{
    _autoresizesFollowContnetView = autoresizesFollowContnetView;
    if (autoresizesFollowContnetView && self.targetView.superview) {
        [self.targetView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.centerX.width.fs_equalToSuperview().priorityHigh();
            make.left.equalTo(self.targetView.superview.fs_mas_left);
            make.right.equalTo(self.targetView.superview.fs_mas_right);
            make.bottom.equalTo(self.targetView.superview.fs_mas_bottom);
            make.top.equalTo(self.targetView.superview.fs_mas_top);
        }];
    }else{
        [self.targetView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.fs_equalToSuperview().offset(self.targetView.fs_left);
            make.top.fs_equalToSuperview().offset(self.targetView.fs_top);
            make.size.equalTo(@(self.targetView.fs_size));
        }];
    }
}

- (void)configureModalPresentationStyle{
    self.contentView = [[UIScrollView alloc] initWithFrame:self.targetView.bounds];
    CGFloat height = self.popoverSize.height;
    if (_popoverShowStyle == FSTableViewPopoverShowStyleDefault) {
        FSPresentedViewControllerAnimator *animator = [[FSPresentedViewControllerAnimator alloc] init];
        animator.size = CGSizeMake(self.popoverSize.width, height + ITEM_MARGIN_LITTLE_NORMAL);
        animator.presentationStyle = FSModalPresentationBottom;
        animator.maxVerticalHeigh = height + ITEM_MARGIN_LITTLE_NORMAL;
        animator.dismissViewControllerCompletion = self.dismissCompletion;
        [self fs_setCustomTransitionAnimator:animator];
        self.animator = animator;
    }else{
        self.contentView.scrollEnabled = NO;
        self.modalPresentationStyle = UIModalPresentationPopover;
        UIPopoverPresentationController *popover = [self popoverPresentationController];
        popover.backgroundColor = self.fs_backgroundColor_B1;
        popover.delegate = self;
        self.preferredContentSize = CGSizeMake(self.popoverSize.width, height);
        popover.permittedArrowDirections = UIPopoverArrowDirectionAny;
    }
}

- (void)setDismissCompletion:(VoidBlock)dismissCompletion{
    [super setDismissCompletion:dismissCompletion];
    self.animator.dismissViewControllerCompletion = [dismissCompletion copy];
}

- (void)setPopOverDismissCompletion:(VoidBlock)dismissCompletion
{
    self.dismissCompletion = dismissCompletion;
    self.animator.dismissViewControllerCompletion = self.dismissCompletion;
}

- (void)setView:(UIView *)view{
    [self.contentView addSubview:self.targetView];
    if (self.autoresizesFollowContnetView) {
        [self.targetView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.centerX.width.fs_equalToSuperview().priorityHigh();
            make.left.equalTo(self.targetView.superview.fs_mas_left);
            make.right.equalTo(self.targetView.superview.fs_mas_right);
            make.bottom.equalTo(self.targetView.superview.fs_mas_bottom);
            make.top.equalTo(self.targetView.superview.fs_mas_top);
        }];
    }
    if (_popoverShowStyle == FSTableViewPopoverShowStyleDefault) {
        self.contentView.fs_height += ITEM_MARGIN_LITTLE_NORMAL;
        UIView *handleView = [UIView new];
        handleView.fs_size = CGSizeMake(31.f, 4.f);
        handleView.fs_top = ITEM_TOP_ICON_MARGIN;
        handleView.fs_centerX = self.popoverSize.width/2;
        [handleView fs_setLayerCornerRadius:LAYER_BORDER_RADIUS_3];
        handleView.backgroundColor = self.fs_dividerColor_P2;
        [self.contentView addSubview:handleView];
        self.handleView = handleView;
        
        self.targetView.fs_top = ITEM_MARGIN_LITTLE_NORMAL;
    }
    self.contentView.contentSize = CGSizeMake(_popoverWidth ? _popoverWidth : self.popoverSize.width, self.contentView.fs_height);
    [super setView:self.contentView];
}

- (void)viewDidLayoutSubviews{
    [super viewDidLayoutSubviews];
    if (_popoverShowStyle == FSPopoverShowStyleStyleBubble) {
        [self.view mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.right.bottom.top.fs_equalToSuperview();
            make.size.mas_equalTo(self.preferredContentSize).priorityHigh();
        }];
    }else{
        self.handleView.fs_centerX = self.handleView.superview.fs_width/2;
    }
}

- (void)makeConstraints{}

- (UIModalPresentationStyle)adaptivePresentationStyleForPresentationController:(UIPresentationController *)controller {
    return UIModalPresentationNone;
}

- (UIModalPresentationStyle)adaptivePresentationStyleForPresentationController:(UIPresentationController *)controller traitCollection:(UITraitCollection *)traitCollection {
    return UIModalPresentationNone;
}

- (void)popoverPresentationControllerDidDismissPopover:(UIPopoverPresentationController *)popoverPresentationController{
    !self.dismissCompletion ?: self.dismissCompletion();
    !self.splitScrrenDismissCompletion ?: self.splitScrrenDismissCompletion();
}

- (void)popoverPresentationController:(UIPopoverPresentationController *)popoverPresentationController willRepositionPopoverToRect:(inout CGRect *)rect inView:(inout UIView  * __nonnull * __nonnull)view{
    if (self.willRepositionPopoverToRectInView) {
        self.willRepositionPopoverToRectInView(rect, view);
    }
}

- (void)scrollViewDidScroll:(UIScrollView *)scrollView{
    self.handleView.fs_top = scrollView.fs_offsetY + ITEM_TOP_ICON_MARGIN;
}


@end
