/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSNavigationControllerStack.h"
#import "FSWeakPointerArray.h"

@interface FSNavigationControllerStack ()
@property (nonatomic, strong) FSWeakPointerArray *weakNavigationControllers;
@property (nonatomic, weak) id<FSViewControllerServices> services;
@end
@implementation FSNavigationControllerStack

- (instancetype)initWithServices:(id<FSViewControllerServices>)services{
    self = [super init];
    if (self) {
        _services = services;
        _weakNavigationControllers = [FSWeakPointerArray array];
    }
    return self;
}

- (void)pushNavigationController:(UINavigationController *)navigationController {
    if ([self.navigationControllers containsObject:navigationController]) return;
    [_weakNavigationControllers addObject:navigationController];
}

- (UINavigationController *)popNavigationController {
    if (self.navigationControllers.count == 1) return nil;
    UINavigationController *navigationController = [self topNavigationController];
    [_weakNavigationControllers removeLastObject];
    return navigationController;
}

- (__kindof UINavigationController *)topVisibleNavigationController{
    NSMutableArray<UINavigationController *> *navigationControllers = self.navigationControllers.mutableCopy;
    while (!navigationControllers.lastObject.view.window && navigationControllers.count) {
        [navigationControllers removeLastObject];
    }
    return navigationControllers.lastObject;
}

- (__kindof UINavigationController *)topNavigationController {
    return self.navigationControllers.lastObject;
}

- (__kindof UINavigationController *)rootNavigationController {
    return self.navigationControllers.firstObject;
}

- (NSArray<UINavigationController *> *)navigationControllers{
    return _weakNavigationControllers.allObjects;
}

- (BOOL)containsNavigationController:(__kindof UINavigationController *)navigationController{
    return [self.navigationControllers containsObject:navigationController];
}

- (void)forcedPopNavigationController{
    [_weakNavigationControllers removeLastObject];
}

- (void)removeAllNavigationControllers{
    [_weakNavigationControllers removeAllObjects];
}

- (void)removeNavigationController:(__kindof UINavigationController *)navigationController{
    [_weakNavigationControllers removeObject:navigationController];
}

- (NSString *)description{
    return self.navigationControllers.description;
}

@end
