/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/

#import "FSViewControllerServicesImpl.h"
#import "FSNavigationController.h"
#import "FSViewController.h"

@interface FSNavigationControllerStack ()

- (void)removeAllNavigationControllers;
- (void)removeNavigationController:(__kindof UINavigationController *)navigationController;
@end

@interface FSViewController (){
    @public
    id<FSViewControllerServices> _services;
}
@end

@interface FSViewControllerServicesImpl (){
    FSNavigationControllerStack *_navigationControllerStack;
}
@end
@implementation FSViewControllerServicesImpl

- (instancetype)init{
    self = [super init];
    if (self) {
        _navigationControllerStack = [[FSNavigationControllerStack alloc] initWithServices:self];
    }
    return self;
}

- (void)pushViewController:(UIViewController *)viewController animated:(BOOL)animated{
    viewController.hidesBottomBarWhenPushed = YES;
    if ([viewController isKindOfClass:[FSViewController class]] ) {
        UIViewController *topViewController = [[_navigationControllerStack topNavigationController] topViewController];
        if ([topViewController isKindOfClass:[FSViewController class]]) {
            ((FSViewController *)viewController).dismissCompletion = ((FSViewController *)topViewController).dismissCompletion;
        }
    }
    [[_navigationControllerStack topNavigationController] pushViewController:viewController animated:YES];
}

- (void)popViewControllerAnimated:(BOOL)animated{
    [(UINavigationController *)[_navigationControllerStack topNavigationController] popViewControllerAnimated:animated];
}

- (void)popToViewControllerClass:(Class)cls animated:(BOOL)animated{
    for (UIViewController *viewController in _navigationControllerStack.topNavigationController.viewControllers) {
        if ([viewController isKindOfClass:cls]) {
            [[_navigationControllerStack topNavigationController] popToViewController:viewController animated:animated];
        }
    }
}

- (void)popToRootViewControllerAnimated:(BOOL)animated{
    [(UINavigationController *)[_navigationControllerStack topNavigationController] popToRootViewControllerAnimated:animated];
}

- (void)presentViewController:(UIViewController *)viewController animated:(BOOL)animated completion:(VoidBlock)completion{
    UIViewController *presentingViewController = [self.navigationControllerStack topVisibleNavigationController];

    BOOL needPushStack = YES;
    if (![viewController isKindOfClass:UINavigationController.class]) {
        if ([viewController isKindOfClass:[FSViewController class]] ) {
            FSViewController *fs_viewController = (FSViewController *)viewController;
            if (!fs_viewController.services) {
                fs_viewController->_services = self;
            }else if (fs_viewController.services != self){
                needPushStack = NO;
            }
        }
        if ([viewController isMemberOfClass:[UIAlertController class]] && ![viewController isKindOfClass:[UIViewController class]]) {
            if (viewController.transitioningDelegate) {
                id <UIViewControllerTransitioningDelegate> transitioningDelegate = viewController.transitioningDelegate;
                UIModalPresentationStyle modalPresentationStyle = viewController.modalPresentationStyle;
                viewController = [[FSNavigationController alloc] initWithRootViewController:viewController];
                viewController.transitioningDelegate = transitioningDelegate;
                viewController.modalPresentationStyle = modalPresentationStyle;
            }else if (!viewController.transitioningDelegate && DEVICE_iPHONE && viewController.modalPresentationStyle != UIModalPresentationFullScreen) {
                viewController = [[FSNavigationController alloc] initWithRootViewController:viewController];
            }
        }
    }else{
        FSViewController *fs_viewController = [((UINavigationController *)viewController).viewControllers firstObject];
        if ([fs_viewController isKindOfClass:[FSViewController class]] ) {
            if (!fs_viewController.services) {
                fs_viewController->_services = self;
            }else if (fs_viewController.services != self){
                needPushStack = NO;
            }
            
            if (fs_viewController.transitioningDelegate && !viewController.transitioningDelegate) {
                id <UIViewControllerTransitioningDelegate> transitioningDelegate = fs_viewController.transitioningDelegate;
                UIModalPresentationStyle modalPresentationStyle = fs_viewController.modalPresentationStyle;
                viewController.transitioningDelegate = transitioningDelegate;
                viewController.modalPresentationStyle = modalPresentationStyle;
            }
        }
    }
    if ([viewController isKindOfClass:UINavigationController.class] && needPushStack) {
        [_navigationControllerStack pushNavigationController:(UINavigationController *)viewController];
    }
    
    if (!presentingViewController.view.window || viewController.fs_presentByTopViewController || !needPushStack) {
        presentingViewController = [_navigationControllerStack.rootNavigationController fs_currentSceneTopMostViewController];
    }
    [presentingViewController presentViewController:viewController animated:animated completion:completion];
}

- (void)dismissViewControllerAnimated:(BOOL)animated completion:(VoidBlock)completion{
    if (self.priorityController) {
        if ([self.navigationControllerStack containsNavigationController:self.priorityController]) {
            [self.navigationControllerStack removeNavigationController:self.priorityController];
        }
        if ([self.navigationControllerStack containsNavigationController:self.priorityController.navigationController]) {
            [self.navigationControllerStack removeNavigationController:self.priorityController.navigationController];
        }
        UIViewController *priorityController = self.priorityController;
        self.priorityController = nil;
        [priorityController dismissViewControllerAnimated:animated completion:completion];
        return;
    }
    UINavigationController *nav = [_navigationControllerStack popNavigationController];
    [nav ? nav : [_navigationControllerStack topNavigationController] dismissViewControllerAnimated:animated completion:completion];
}

- (void)resetRootViewController:(UIViewController *)viewController{
    NSAssert(viewController, @"rootViewController cannot be nil");

    [_navigationControllerStack removeAllNavigationControllers];
    
    if (![viewController isKindOfClass:[UINavigationController class]] &&
        ![viewController isKindOfClass:[UITabBarController class]]) {
        viewController = [[FSNavigationController alloc] initWithRootViewController:viewController];
        [self.navigationControllerStack pushNavigationController:(UINavigationController *)viewController];
    }
    
    FSSharedAppDelegate.window.rootViewController = viewController;
}

- (FSNavigationControllerStack *)navigationControllerStack{
    return _navigationControllerStack;
}

@synthesize priorityController;

@end

