/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "NSDate+FSAdd.h"

#define HAS_YYKIT_NSDATE_YYADD __has_include(<YYKit/NSDate+YYAdd.h>) || __has_include("NSDate+YYAdd.h")

@implementation NSDate (FSAdd)

#if HAS_YYKIT_NSDATE_YYADD
//Reference YYKit
- (NSString *)fs_stringWithFormat:(NSString *)format {
    return [self stringWithFormat:self];
}

- (NSString *)fs_stringWithFormat:(NSString *)format timeZone:(NSTimeZone *)timeZone locale:(NSLocale *)locale {
    return [self stringWithFormat:format timeZone:timeZone locale:locale];
}

+ (NSDate *)fs_dateWithString:(NSString *)dateString format:(NSString *)format {
    return [NSDate dateWithString:dateString format:format];
}
#else

- (NSString *)fs_stringWithFormat:(NSString *)format {
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
    [formatter setDateFormat:format];
    [formatter setLocale:[NSLocale currentLocale]];
    return [formatter stringFromDate:self];
}

- (NSString *)fs_stringWithFormat:(NSString *)format timeZone:(NSTimeZone *)timeZone locale:(NSLocale *)locale {
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
    [formatter setDateFormat:format];
    if (timeZone) [formatter setTimeZone:timeZone];
    if (locale) [formatter setLocale:locale];
    return [formatter stringFromDate:self];
}

+ (NSDate *)fs_dateWithString:(NSString *)dateString format:(NSString *)format {
    NSDateFormatter *formatter = [[NSDateFormatter alloc] init];
    [formatter setDateFormat:format];
    return [formatter dateFromString:dateString];
}

#endif

@end
