/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/

#import "NSObject+FSAdd.h"
#import "UIView+FSAdd.h"
#import <objc/runtime.h>


NSString *FSGetObjectPropertyDescription (id obj, NSString *descriptionString){
    unsigned int count = 0 ;
    objc_property_t *propertyList = class_copyPropertyList([obj class], &count);
    for (int i = 0; i < count; i ++) {
        @try {
            objc_property_t property = propertyList[i];
            NSString *propertyName = [NSString stringWithUTF8String: property_getName(property)];
            NSString *propertyValue = [[obj valueForKey:propertyName] description];
            descriptionString = [NSString stringWithFormat:@"%@\n %@ = %@; ",descriptionString, propertyName, propertyValue];
        } @catch (NSException *exception) {
            continue;
        }
    }
    free(propertyList);
    return descriptionString;
};


@implementation NSObject (Extensions)

+ (UIWindow *)fs_getForegroundActiveWindowInSceneView:(UIView *)view{

    UIWindow *originalKeyWindow = nil;
    #if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
    if (@available(iOS 13.0, *)) {
        NSSet<UIScene *> *connectedScenes = [UIApplication sharedApplication].connectedScenes;
        for (UIScene *scene in connectedScenes) {
            if ([scene isKindOfClass:[UIWindowScene class]]) {
                UIWindowScene *windowScene = (UIWindowScene *)scene;
                if (windowScene.activationState == UISceneActivationStateUnattached){
                    originalKeyWindow = windowScene.windows.firstObject;
                }
                if (scene.activationState == UISceneActivationStateForegroundActive) {
                    for (UIWindow *window in windowScene.windows) {
                        if (window.isKeyWindow) {
                            originalKeyWindow = window;
                            if ([windowScene.windows containsObject:view.window]) {
                                break;
                            }
                        }
                    }
                    if (_MAC_CATALYST_ && !originalKeyWindow && windowScene.windows.firstObject) {
                        return windowScene.windows.firstObject;
                    }
                }
            }

        }
        if (originalKeyWindow) {
            return originalKeyWindow;
        }
    }
    #endif
    return [[[UIApplication sharedApplication] delegate] window];
}


+ (UIWindow *)fs_getForegroundAnyKeyWindow{
    
    UIWindow *originalKeyWindow = nil;
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
    if (@available(iOS 13.0, *)) {
        NSSet<UIScene *> *connectedScenes = [UIApplication sharedApplication].connectedScenes;
        for (UIScene *scene in connectedScenes) {
            if ([scene isKindOfClass:[UIWindowScene class]]) {
                UIWindowScene *windowScene = (UIWindowScene *)scene;
                
                for (UIWindow *window in windowScene.windows) {
                    if (window) {
                        originalKeyWindow = window;
                        if (window.hidden) continue;
                        break;
                    }
                }
            }
        }
        if (originalKeyWindow) {
            return originalKeyWindow;
        }
    }
#endif
    return [[[UIApplication sharedApplication] delegate] window];
}

+ (CGRect)fs_getStatusBarFrame{
    CGRect statusBarFrame = [[UIApplication sharedApplication] statusBarFrame];
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
    if (@available(iOS 13.0, *)) statusBarFrame = [[[NSObject fs_getForegroundAnyKeyWindow].windowScene statusBarManager] statusBarFrame];
#endif
    return statusBarFrame;
}

- (void)fs_addWeakAssociatedObject:(id)object forKey:(NSString *)key{
    __weak id weakObject = object;
    objc_setAssociatedObject(self, (const void *)key.hash, ^{
        return weakObject;
    }, OBJC_ASSOCIATION_COPY);
}

- (id)fs_getWeakAssociatedForKey:(NSString *)key{
    id(^getter)() = objc_getAssociatedObject(self, (const void *)key.hash);
    if (getter) return getter();
    return nil;
}

- (void)fs_addCopyAssociatedObject:(id)object forKey:(NSString *)key{
    objc_setAssociatedObject(self, (const void *)key.hash, object, OBJC_ASSOCIATION_COPY_NONATOMIC);
}

- (void)fs_addAssociatedObject:(id)object forKey:(NSString *)key{
    objc_setAssociatedObject(self, (const void *)key.hash, object, OBJC_ASSOCIATION_RETAIN_NONATOMIC);
}

- (id)fs_getAssociatedForKey:(NSString *)key{
    return objc_getAssociatedObject(self, (const void *)key.hash);
}

- (BOOL)fs_existKey:(NSString *)key{
    unsigned int count = 0;
    BOOL exist = NO;
    Ivar *ivars = class_copyIvarList([self class], &count);
    for (int i = 0; i < count; i++) {
        Ivar ivar = ivars[i];
        NSString *keyName = [NSString stringWithCString:ivar_getName(ivar)
                                               encoding:NSUTF8StringEncoding];
        if ([keyName rangeOfString:key].length>0) {
            exist = YES;
        }
    }
    return exist;
}


- (__kindof UIViewController *)fs_storyBoardViewControllerWithName:(NSString *)name identifier:(NSString *)identifier{
    if ([NSStringFromClass([self class]) isEqualToString:@"NSObject"]) {
        @throw [NSException exceptionWithName:@"Foxit Exception" reason:@"The class must not be NSObject" userInfo:nil];
    }
    return [[UIStoryboard storyboardWithName:name bundle:[NSBundle bundleForClass:[self class]]] instantiateViewControllerWithIdentifier:identifier];
}

@end

@implementation NSObject(MultipleScenes)

#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
- (UIWindowScene *)fs_windowScene API_AVAILABLE(ios(13.0)){
    if ([self isKindOfClass:[UIWindowScene class]]) return (UIWindowScene *)self;
    UIWindowScene *windowScene;
    if ([self isKindOfClass:[UIResponder class]]) {
        UIResponder *responder = (UIResponder *)self;
        while (responder) {
            if ([responder isKindOfClass:[UIWindowScene class]]) {
                windowScene = (UIWindowScene *)responder;
                break;
            }else if([responder isKindOfClass:[UIViewController class]]){
                UIViewController *vc = (UIViewController *)responder;
                windowScene = vc.parentViewController.fs_windowScene;
                if (!windowScene) {
                    windowScene = vc.presentingViewController.fs_windowScene;
                }
                if (windowScene) {
                    break;
                }
            }
            responder = [responder nextResponder];
        }
    }
    UIWindowScene *fs_windowScene = [self fs_getWeakAssociatedForKey:@keypath(self, fs_windowScene)];
    if (!fs_windowScene && windowScene) {
        [self fs_addWeakAssociatedObject:windowScene forKey:@keypath(self, fs_windowScene)];
    }
    if (!windowScene && fs_windowScene) windowScene = fs_windowScene;
    @try {
        //!!!!: Check that the code 'WindowScene' is necessary
        if (!windowScene) {
            @throw [NSException exceptionWithName:@"Check WindowScene" reason:@"WindowScene is currently null, check whether a repair is needed" userInfo:nil];
        }
    } @catch (NSException *exception) {
    }
    return windowScene;
}

- (void)setFs_windowScene:(UIWindowScene *)fs_windowScene API_AVAILABLE(ios(13.0)){
    [self fs_addWeakAssociatedObject:fs_windowScene forKey:@keypath(self, fs_windowScene)];
}
#endif

- (UIWindow *)fs_currentSceneKeyWindow{
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
    if (@available(iOS 13.0, *)) {
        for (UIWindow *window in self.fs_windowScene.windows) {
            if (window.isKeyWindow) {
                return window;
            }
        }
        UIWindow *window = [(id<UIWindowSceneDelegate>)self.fs_windowScene.delegate window];
        if (window) return window;
    }
#endif
    return [NSObject fs_getForegroundAnyKeyWindow];
}

- (UIViewController *)fs_currentSceneTopMostViewController{
    UIViewController *presentingViewController = self.fs_currentSceneKeyWindow.rootViewController;
    while (presentingViewController.presentedViewController != nil) {
        presentingViewController = presentingViewController.presentedViewController;
    }
    return presentingViewController;
}


- (UIUserInterfaceSizeClass)fs_currentSceneSizeClass{
    return [self fs_currentSceneKeyWindow].traitCollection.horizontalSizeClass;
}

- (BOOL)fs_currentSceneHorizontalCompact{
    return [self fs_currentSceneSizeClass] == UIUserInterfaceSizeClassCompact;
}

- (BOOL)fs_currentSceneiPadHorizontalCompact{
    return [self fs_currentSceneHorizontalCompact] && [NSObject fs_currentDeviceIsPad];
}

- (BOOL)fs_currentSceneIsCompact{
    return [self fs_currentSceneiPadHorizontalCompact] || [NSObject fs_currentDeviceIsPhone];
}

+ (BOOL)fs_currentDeviceIsPhone{
    return [[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone;
}

+ (BOOL)fs_currentDeviceIsPad{
    return [[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad;
}

#if __has_include(<MBProgressHUD/MBProgressHUD.h>) || __has_include("MBProgressHUD.h")
#if __has_include(<MBProgressHUD/MBProgressHUD.h>)
    #import <MBProgressHUD/MBProgressHUD.h>
#else
    #import "MBProgressHUD.h"
#endif
static NSString *const kfs_toastHUD = @"kfs_toastHUD";

- (void)fs_currentSceneToastWithMsg:(NSString *)msg{
    UIWindow *win = [self fs_currentSceneKeyWindow];
    
    MBProgressHUD *hud = [win fs_getWeakAssociatedForKey:kfs_toastHUD];
    if (!hud) {
        hud = [win fs_showHUDMessage:msg];
        hud.transform = CGAffineTransformIdentity;
        hud.detailsLabelFont = self.fs_standardGeneralFont_GF3;
        hud.userInteractionEnabled = NO;
        [win fs_addWeakAssociatedObject:hud forKey:kfs_toastHUD];
    }else{
        hud.detailsLabelText = msg;
        [NSObject cancelPreviousPerformRequestsWithTarget:hud];
        [hud hide:YES afterDelay:2.f];
    }
    
    CGSize maxSize = CGSizeMake(hud.fs_width - 4 * hud.margin, hud.fs_height);
    CGSize detailsLabelSize = [msg boundingRectWithSize:maxSize options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName : hud.detailsLabelFont} context:nil].size;
    
    CGFloat bottomH = TAB_BAR_HEIGHT_NORMAL;
    CGFloat safeAreaInsetBottom = 0;
    if (@available(iOS 11.0, *)) safeAreaInsetBottom = win.safeAreaInsets.bottom;
    CGFloat margin = 16;
    CGFloat visibleH = detailsLabelSize.height + 2 * hud.margin;

    hud.yOffset = win.fs_height / 2 - bottomH - safeAreaInsetBottom - margin - visibleH/2;
}

- (void)fs_toastWithMsg:(NSString *)msg inView:(UIView *)view{
    
    MBProgressHUD *hud = [view fs_getWeakAssociatedForKey:kfs_toastHUD];
    if (!hud) {
        hud = [view fs_showHUDMessage:msg];
        hud.detailsLabelFont = self.fs_standardGeneralFont_GF3;
        hud.userInteractionEnabled = NO;
        [view fs_addWeakAssociatedObject:hud forKey:kfs_toastHUD];
    }else{
        hud.detailsLabelText = msg;
        [NSObject cancelPreviousPerformRequestsWithTarget:hud];
        [hud hide:YES afterDelay:2.f];
    }
    
    CGSize maxSize = CGSizeMake(hud.fs_width - 4 * hud.margin, hud.fs_height);
    CGSize detailsLabelSize = [msg boundingRectWithSize:maxSize options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName : hud.detailsLabelFont} context:nil].size;
    
    CGFloat bottomH = TAB_BAR_HEIGHT_NORMAL;
    CGFloat safeAreaInsetBottom = 0;
    if (@available(iOS 11.0, *)) safeAreaInsetBottom = view.safeAreaInsets.bottom;
    CGFloat margin = 16;
    CGFloat visibleH = detailsLabelSize.height + 2 * hud.margin;

    hud.yOffset = view.fs_height / 2 - bottomH - safeAreaInsetBottom - margin - visibleH/2;
}
#endif

@end


@implementation NSObject(MethodSwizzling)

+ (void)fs_exchangeImpWithOriginalSel:(SEL)originalSel swizzledSel:(SEL)swizzledSel{
    [self fs_exchangeImpWithClass:[self class] originalSel:originalSel swizzledSel:swizzledSel];
}

+ (void)fs_exchangeImpWithClass:(Class)cls originalSel:(SEL)originalSel swizzledSel:(SEL)swizzledSel
{
    Method originalMethod = class_getInstanceMethod(cls, originalSel);
    Method swizzledMethod = class_getInstanceMethod(cls, swizzledSel);
    
    BOOL didAddMethod = class_addMethod(cls,originalSel,
                                        method_getImplementation(swizzledMethod),
                                        method_getTypeEncoding(swizzledMethod));
    if (didAddMethod) {
        class_replaceMethod(cls,swizzledSel,
                            method_getImplementation(originalMethod),
                            method_getTypeEncoding(originalMethod));
    } else {
        method_exchangeImplementations(originalMethod, swizzledMethod);
    }
    
}

+ (void)fs_exchangeImpWithOriginalClass:(Class)oriCls swizzledClass:(Class)swiCls originalSel:(SEL)oriSel swizzledSel:(SEL)swiSel tmpSel:(SEL)tmpSel{
    
    SpanClassExchangeImp(oriCls, oriSel, swiCls, swiSel, tmpSel);
}

static void SpanClassExchangeImp(Class originalClass, SEL originalSel, Class swizzledClass, SEL swizzledSel, SEL noneSel){
    Method originalMethod = class_getInstanceMethod(originalClass, originalSel);
    Method swizzledMethod = class_getInstanceMethod(swizzledClass, swizzledSel);
    if (!originalMethod) {
        Method noneMethod = class_getInstanceMethod(swizzledClass, noneSel);
        class_addMethod(originalClass, originalSel, method_getImplementation(noneMethod), method_getTypeEncoding(noneMethod));
        originalMethod = class_getInstanceMethod(originalClass, originalSel);
    }
    BOOL didAddMethod = class_addMethod(originalClass, swizzledSel, method_getImplementation(swizzledMethod), method_getTypeEncoding(swizzledMethod));
    if (didAddMethod) {
        Method newMethod = class_getInstanceMethod(originalClass, swizzledSel);
        method_exchangeImplementations(originalMethod, newMethod);
    }
}

@end

IMP ReplaceIMPWithBlock(Class cls, SEL origSEL, id block) {
    NSCParameterAssert(block);

    Method origMethod = class_getInstanceMethod(cls, origSEL);
    NSCParameterAssert(origMethod);

    IMP newIMP = imp_implementationWithBlock(block);

    if (!class_addMethod(cls, origSEL, newIMP, method_getTypeEncoding(origMethod))) {
        return method_setImplementation(origMethod, newIMP);
    }else {
        return method_getImplementation(origMethod);
    }
}

@interface NSObject (FixAppleBug)
@end

@implementation NSObject(FixAppleBug)

+ (void)load{
    if (@available(iOS 13.2, *)){
        #if DEBUG
        const char *className = "_UIScrollerImpContainerView";
        Class cls = objc_getClass(className);
        if (cls == nil) {
            cls = objc_allocateClassPair([UIView class], className, 0);
            objc_registerClassPair(cls);
            printf("added %s dynamically\n", className);
        }
        #endif
    }else{
        const char *className = "_UITextLayoutView";
        Class cls = objc_getClass(className);
        if (cls == nil) {
            cls = objc_allocateClassPair([UIView class], className, 0);
            objc_registerClassPair(cls);
        #if DEBUG
            printf("added %s dynamically\n", className);
        #endif
        }
    }
    
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{

        SEL selectors[] = {
            @selector(becomeFirstResponder),
        };
        
        Class cls = NSClassFromString(@"UIView");
        for (NSUInteger index = 0; index < sizeof(selectors) / sizeof(SEL); ++index) {
            SEL originalSel = selectors[index];
            SEL swizzledSel = NSSelectorFromString([@"fs_swizzled_" stringByAppendingString:NSStringFromSelector(originalSel)]);
            
            [self fs_exchangeImpWithClass:cls originalSel:originalSel swizzledSel:swizzledSel];
        }
    });
}

- (void)fs_swizzled_becomeFirstResponder{
    if ([self isKindOfClass:[UIView class]]) {
        UIView *_self = (UIView *)self;
        if ([_self conformsToProtocol:@protocol(UIKeyInput)]) {
            UIWindow *window = _self.fs_currentSceneKeyWindow;
            if (![window isKeyWindow]) {
                [window makeKeyWindow];
            }
            [_self fs_resignOtherInputFirstResponderCurrent:window];
        }
    }
    [self fs_swizzled_becomeFirstResponder];
}

@end
