/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "NSObject+FSAddCoder.h"
#import <objc/runtime.h>

#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_11_0 && __IPHONE_OS_VERSION_MAX_ALLOWED <= __IPHONE_13_0
@interface NSKeyedArchiver (FSAddCoder)
+ (nullable NSData *)archivedDataWithRootObject:(id)object requiringSecureCoding:(BOOL)requiresSecureCoding error:(NSError **)error;
@end

@interface NSKeyedUnarchiver (FSAddCoder)
+ (nullable id)unarchivedObjectOfClass:(Class)cls fromData:(NSData *)data error:(NSError **)error;
@end
#endif

@implementation NSObject (FSAddCoder)
+ (BOOL)fs_archivedWithRootObject:(id)rootObject toPath:(NSString *)path{
    NSError *error;
    NSData *data = [NSKeyedArchiver archivedDataWithRootObject:rootObject requiringSecureCoding:YES error:&error];
    if (error) {
        return NO;
    }
    return [data writeToFile:path atomically:YES];
}

+ (nullable id)fs_unarchiveObjectWithFile:(NSString *)path{
    NSError *error;
    NSData *data = [[NSData alloc] initWithContentsOfFile:path];
    id obj = [NSKeyedUnarchiver unarchivedObjectOfClass:[self class] fromData:data error:&error];
    return obj;
}

- (void)fs_encodeWithCoder:(NSCoder *)coder{
    unsigned int count = 0;
    objc_property_t *propertes = class_copyPropertyList([self class], &count);
    for (int i = 0; i < count; i++) {
        const char *propertyName = property_getName(propertes[i]);
        NSString *name = [NSString stringWithUTF8String:propertyName];
        id value = [self valueForKey:name];
        if (value) [coder encodeObject:value forKey:name];
    }
    free(propertes);
}

- (void)fs_decoderWithCoder:(NSCoder *)decoder{
    unsigned int count = 0;
    objc_property_t *propertes = class_copyPropertyList([self class], &count);
    for (int i = 0; i < count; i++) {
        const char *propertyName = property_getName(propertes[i]);
        NSString *name = [NSString stringWithUTF8String:propertyName];
        id value = [decoder decodeObjectForKey:name];
        if (value) [self setValue:value forKey:name];
    }
    free(propertes);
}

@end
