/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/


#import "NSString+FSAdd.h"

@implementation NSString (FSAdd)

- (BOOL)fs_isExist {
    return self && ![self isEqualToString:@""];
}

- (BOOL)fs_isImage {
    if (!self.fs_isExist) return NO;
    
    NSArray *imageExtensions = @[ @".png", @".gif", @".jpg", @".jpeg" ];
    for (NSString *extension in imageExtensions) {
        if ([self.lowercaseString hasSuffix:extension]) return YES;
    }
    
    return NO;
}

- (BOOL)fs_isMarkdown {
    if (!self.fs_isExist) return NO;
    
    NSArray *markdownExtensions = @[ @".md", @".mkdn", @".mdwn", @".mdown", @".markdown", @".mkd", @".mkdown", @".ron" ];
    for (NSString *extension in markdownExtensions) {
        if ([self.lowercaseString hasSuffix:extension]) return YES;
    }
    
    return NO;
}

- (BOOL)fs_isNumber{
    NSCharacterSet *charSet = [[NSCharacterSet characterSetWithCharactersInString:@"0123456789"] invertedSet];
    NSString *filteredStr = [[self componentsSeparatedByCharactersInSet:charSet] componentsJoinedByString:@""];
    return [self isEqualToString:filteredStr];
}

- (NSString *)fs_fileName{
    NSArray *urlCom = [[NSArray alloc]initWithArray:[self pathComponents]];
    return [urlCom lastObject];
}

- (NSString *)fs_firstLetter {
    return [[self substringToIndex:1] uppercaseString];
}

- (NSString *)fs_stringByTrim {
    NSCharacterSet *set = [NSCharacterSet whitespaceAndNewlineCharacterSet];
    return [self stringByTrimmingCharactersInSet:set];
}

- (NSString *)fs_stringByTrimOccurrences:(NSString *)occurrences{
    return [self stringByReplacingOccurrencesOfString:occurrences withString:@""];
}


@end
