/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/
#import "Masonry+FSAdd.h"

@implementation UIView (MasonrySafeAreaLayout)

- (MASViewAttribute *)fs_mas_left {
    if (@available(iOS 11.0, *))return self.mas_safeAreaLayoutGuideLeft;
    return self.mas_left;
}

- (MASViewAttribute *)fs_mas_right {
    if (@available(iOS 11.0, *))return self.mas_safeAreaLayoutGuideRight;
    return self.mas_right;
}

- (MASViewAttribute *)fs_mas_top {
    if (@available(iOS 11.0, *))return self.mas_safeAreaLayoutGuideTop;
    return self.mas_top;
}

- (MASViewAttribute *)fs_mas_bottom {
    if (@available(iOS 11.0, *))return self.mas_safeAreaLayoutGuideBottom;
    return self.mas_bottom;
}


@end

@implementation MASConstraint (FSAdd)
- (MASConstraint * (^)(void))fs_equalToSuperview{
    return ^id (void){
        return self.equalTo(self.fs_superview);
    };
}
- (MASConstraint * (^)(void))fs_greaterThanOrEqualToSuperview{
    return ^id (void){
        return self.greaterThanOrEqualTo(self.fs_superview);
    };
}
- (MASConstraint * (^)(void))fs_lessThanOrEqualToSuperview{
    return ^id (void){
        return self.lessThanOrEqualTo(self.fs_superview);
    };
}

- (MAS_VIEW *)fs_superview{
    if ([self isKindOfClass:[MASViewConstraint class]]) {
        MASViewConstraint *viewConstraint = (MASViewConstraint *)self;
        MAS_VIEW *superView = viewConstraint.firstViewAttribute.view.superview;
        if (superView) {
            return superView;
        }
    }
    if ([self respondsToSelector:@selector(delegate)]) {
        id delegate = [self performSelector:@selector(delegate)];
        if (delegate) {
            if ([delegate performSelector:@selector(view)]) {
                MAS_VIEW *selfView = [delegate performSelector:@selector(view)];
                if (selfView.superview) {
                    return selfView.superview;
                }
            }
        }
    }
    return nil;
}
@end

