/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "UIAlertController+FSAdd.h"
#import "FSMacro.h"
#import "UIViewController+FSAdd.h"
#import <objc/runtime.h>

@interface UIAlertController (FSAdd)
@property (nonatomic, strong, setter=fs_setEnablesActionsAutomaticallyIndexs:) NSArray<NSNumber *> *fs_enablesActionsAutomaticallyIndexs;
@property (nonatomic, copy, setter=fs_setDefaultText:) NSString *fs_defaultText;
@property (nonatomic, copy, setter=fs_setActionEnables:) FSAlertActionEnables fs_actionEnables;
@end
@implementation UIAlertController (FSAdd)

YYSYNTH_DYNAMIC_PROPERTY_OBJECT(fs_enablesActionsAutomaticallyIndexs, fs_setEnablesActionsAutomaticallyIndexs, RETAIN, NSArray *);
YYSYNTH_DYNAMIC_PROPERTY_OBJECT(fs_defaultText, fs_setDefaultText, COPY, NSString *);
YYSYNTH_DYNAMIC_PROPERTY_OBJECT(fs_actionEnables, fs_setActionEnables, COPY, FSAlertActionEnables);

+ (UIAlertController *)fs_alertWithTitle:(NSString *)title message:(NSString *)message actions:(NSArray <NSString *>*)actions actionBack:(FSAlertActionCallBack)actionBack {
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:title message:message preferredStyle:UIAlertControllerStyleAlert];
    for (int i = 0; i < actions.count; i ++) {
        UIAlertAction *action = [UIAlertAction actionWithTitle:actions[i] style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
            !actionBack?:actionBack(i, action);
        }];
        [alert addAction:action];
    }
    return alert;
}

+ (UIAlertController *)fs_alertWithTitle:(NSString *)title message:( NSString *)message textFieldText:(NSString *)text actions:(NSArray <NSString *>*)actions actionBack:(FSAlertActionWithTextCallBack)actionBack {
    return [self fs_alertWithTitle:title message:message textFieldText:text actions:actions enablesActionsAutomaticallyIndexs:nil actionBack:actionBack];
}

+ (UIAlertController *)fs_enablesActionAutomaticallyAlertWithTitle:(NSString *)title message:(NSString *)message textFieldText:(NSString *)text actions:(NSArray <NSString *>*)actions actionBack:(FSAlertActionWithTextCallBack)actionBack{
    return [self fs_alertWithTitle:title message:message textFieldText:text actions:actions enablesActionsAutomaticallyIndexs:@[@1] actionBack:actionBack];
}

+ (UIAlertController *)fs_alertWithTitle:(NSString *)title message:(NSString *)message textFieldText:(NSString *)text actions:(NSArray <NSString *>*)actions enablesActionsAutomaticallyIndexs:(nullable NSArray<NSNumber *> *)indexs actionBack:(FSAlertActionWithTextCallBack)actionBack {
    
    //Save a UITextField.Can also be:
    /*__block __weak UIAlertController *alert = [self fs_alertWithTitle:title message:message actions:actions actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
        UITextField *textField = [alert.textFields firstObject];
        !actionBack?:actionBack(index,action,textField);
    }]*/
    NSMutableArray *tmpStorage = @[].mutableCopy;
    UIAlertController *alert = [self fs_alertWithTitle:title message:message actions:actions actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
        UITextField *textField = [tmpStorage firstObject];
        !actionBack?:actionBack(index,action,textField);
    }];
    if (text) {
        [alert addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
            textField.accessibilityIdentifier = @"input_text_id";
            [tmpStorage addObject:textField];
            textField.text = text;
            textField.clearButtonMode = UITextFieldViewModeWhileEditing;
        }];
        alert.fs_defaultText = text;
        UITextField *textField = [alert.textFields firstObject];
        [textField addTarget:alert action:@selector(fs_textFieldDidChange:) forControlEvents:UIControlEventEditingChanged];
        alert.fs_enablesActionsAutomaticallyIndexs = indexs;
        for (NSNumber *index in indexs){
            UIAlertAction *action = alert.actions[index.intValue];
            if (action.enabled && alert.fs_defaultText.hash == textField.text.hash) action.enabled = NO;
            NSAssert(index.intValue <= actions.count - 1, @"Index cannot be larger than actions");
        }
    }
    return alert;
}

+ (void)fs_alertWithTitle:(NSString *)title message:(NSString *)message actions:(NSArray <NSString *>*)actions actionBack:(FSAlertActionCallBack)actionBack onCtrl:(UIViewController *)ctrl {
    [[self fs_alertWithTitle:title message:message actions:actions actionBack:actionBack] fs_showByViewController:ctrl];
}

- (UIAlertController *)fs_actionEnables:(FSAlertActionEnables)actionEnables{
    self.fs_actionEnables = [actionEnables copy];
    
    void (^TextFieldsAddTarget)(UIAlertController *) = ^(UIAlertController *_self){
        for (UITextField *textField in _self.textFields) {
            [textField addTarget:_self action:@selector(fs_textFieldDidChange:) forControlEvents:UIControlEventEditingChanged];
            [_self fs_textFieldDidChange:textField];
        }
    };
    if (actionEnables) {
        TextFieldsAddTarget(self);
        [self fs_addObserverBlockForKeyPath:@keypath(self, textFields) block:^(UIAlertController *_Nonnull _self, id  _Nullable oldVal, id newVal) {
            TextFieldsAddTarget(_self);
        }];
    }else{
        [self fs_removeObserverBlocksForKeyPath:@keypath(self, textFields)];
    }

    return self;
}

- (void)fs_showByViewController:(UIViewController *)viewController{
    NSAssert(viewController, @"viewController can't be nil");
    [viewController presentViewController:self animated:YES completion:nil];
}

- (void)fs_textFieldDidChange:(UITextField *)textField{
    if (self.fs_actionEnables) {
        for (int i = 0;  i < self.actions.count; i++) {
            UIAlertAction *action = self.actions[i];
            action.enabled = self.fs_actionEnables(i, action, textField);
        }
    }else{
        for (NSNumber *index in self.fs_enablesActionsAutomaticallyIndexs) {
            UIAlertAction *action = self.actions[index.intValue];
            action.enabled = textField.text.length;
            if (action.enabled && self.fs_defaultText.hash == textField.text.hash) action.enabled = NO;
        }
    }
}

@end

