/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/


#import <UIKit/UIKit.h>

NS_ASSUME_NONNULL_BEGIN
struct RGB {
    float red;
    float green;
    float blue;
    float alpha;
};

struct HSV {
    float hue;
    float saturation;
    float brightness;
    float alpha;
};

typedef struct RGB RGB;
typedef struct HSV HSV;

RGB hsv2rgb(HSV hsv);

HSV rgb2hsv(RGB rgb);

/*
Create UIColor with a hex string.
Example: UIColorHex_DarkMode(0xF0F, 66ccff), UIColorHex_DarkMode(#66CCFF88, 66ccff), UIColorHex_DarkMode(66ccff, #66CCFF88)

Valid format: #RGB #RGBA #RRGGBB #RRGGBBAA 0xRGB ...
The `#` or "0x" sign is not required.
*/

#ifndef UIColorHex
#define UIColorHex(_hex_)   [UIColor colorWithHexString:((__bridge NSString *)CFSTR(#_hex_))]
#endif

#ifndef UIColorHex_DarkMode
#define UIColorHex_DarkMode(_lightHex_, _darkHex_)   [UIColor fs_colorWithLightHex:((__bridge NSString *)CFSTR(#_lightHex_)) darkHex:((__bridge NSString *)CFSTR(#_darkHex_))]
#endif

#ifndef UIColor_DarkMode
    #define UIColor_DarkMode(lightColor, darkColor)   [UIColor fs_colorWithLight:lightColor dark:darkColor]
#endif

#ifndef UIColor_Owner_DarkMode
    #define UIColor_Owner_DarkMode(owner, lightColor, darkColor)   [UIColor fs_colorWithOwner:owner light:lightColor dark:darkColor]
#endif

#define HAS_YYKIT_COLOR_YYADD __has_include(<YYKit/UIColor+YYAdd.h>) || __has_include("UIColor+YYAdd.h")

@interface UIColor (FSAdd)

#if !HAS_YYKIT_COLOR_YYADD
+ (nullable UIColor *)colorWithHexString:(NSString *)hexStr;
+ (UIColor *)colorWithRGB:(uint32_t)rgbValue;
#endif

+ (UIColor *)fs_blackColor;
+ (UIColor *)fs_darkGrayColor;
+ (UIColor *)fs_lightGrayColor;
+ (UIColor *)fs_whiteColor;
+ (UIColor *)fs_grayColor;
+ (UIColor *)fs_redColor;
+ (UIColor *)fs_greenColor;
+ (UIColor *)fs_blueColor;
+ (UIColor *)fs_cyanColor;
+ (UIColor *)fs_yellowColor;
+ (UIColor *)fs_magentaColor;
+ (UIColor *)fs_orangeColor;
+ (UIColor *)fs_purpleColor;
+ (UIColor *)fs_brownColor;
+ (UIColor *)fs_clearColor;


+ (UIColor *)fs_colorWithARGBHexString:(NSString *)hexStr;
- (uint32_t)fs_rgbValue;
- (uint32_t)fs_argbValue;
- (UIColor *)fs_colorFromCopy;
@end


@interface UIColor (DarkMode_iOS13)

/**
Creates and returns a color object for dark mode.

@discussion:
Valid format: #RGB #RGBA #RRGGBB #RRGGBBAA 0xRGB ...
The `#` or "0x" sign is not required.
The alpha will be set to 1.0 if there is no alpha component.
It will return nil when an error occurs in parsing.

Example: @"0xF0F", @"66ccff", @"#66CCFF88"
 */
+ (UIColor *)fs_colorWithLightHex:(NSString *)lightHex darkHex:(NSString *)darkHex;
+ (UIColor *)fs_colorWithLight:(UIColor *)light dark:(UIColor *)dark;
+ (UIColor *)fs_colorWithOwner:(nullable id<UITraitEnvironment>)owner light:(UIColor *)light dark:(UIColor *)dark;
+ (UIColor *)fs_colorWithDynamicProvider:(UIColor * (^)(UITraitCollection *traitCollection))dynamicProvider;
- (UIColor *)fs_initWithDynamicProvider:(UIColor * (^)(UITraitCollection *traitCollection))dynamicProvider;
- (UIColor *)fs_lightColor;
- (UIColor *)fs_darkColor;
- (BOOL)fs_isDynamicColor;
@end


NS_ASSUME_NONNULL_END
