/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/


#import "UIColor+FSAdd.h"
#import "NSString+FSAdd.h"

RGB hsv2rgb(HSV hsv) {
    RGB rgb = {0.0, 0.0, 0.0, 0.0};
    float r, g, b;

    int i = (int)(hsv.hue * 6);
    float f = hsv.hue * 6 - (float)(i);
    float p = hsv.brightness * (1 - hsv.saturation);
    float q = hsv.brightness * (1 - f * hsv.saturation);
    float t = hsv.brightness * (1 - (1 - f) * hsv.saturation);
    
    switch (i % 6) {
        case 0: r = hsv.brightness; g = t; b = p; break;
            
        case 1: r = q; g = hsv.brightness; b = p; break;
            
        case 2: r = p; g = hsv.brightness; b = t; break;
            
        case 3: r = p; g = q; b = hsv.brightness; break;
            
        case 4: r = t; g = p; b = hsv.brightness; break;
            
        case 5: r = hsv.brightness; g = p; b = q; break;
            
        default: r = hsv.brightness; g = t; b = p;
    }
    rgb.red = r;
    rgb.green = g;
    rgb.blue = b;
    rgb.alpha = hsv.alpha;
    return rgb;
}

HSV rgb2hsv(RGB rgb) {
    HSV hsb = {0.0, 0.0, 0.0, 0.0};
    float rd = rgb.red, gd = rgb.green, bd = rgb.blue;
    
    float maxV = MAX(rd, MAX(gd, bd));
    float minV = MIN(rd, MIN(gd, bd));
    float h = 0, s = 0, b = maxV;
    
    float d = maxV - minV;
    
    s = maxV == 0 ? 0 : d / minV;
    
    if (maxV == minV) {
        h = 0;
    } else {
        if (maxV == rd) {
            h = (gd - bd) / d + (gd < bd ? 6 : 0);
        } else if (maxV == gd) {
            h = (bd - rd) / d + 2;
        } else if (maxV == bd) {
            h = (rd - gd) / d + 4;
        }
        h /= 6;
    }
    hsb.hue = h;
    hsb.saturation = s;
    hsb.brightness = b;
    hsb.alpha = rgb.alpha;
    return hsb;
}

@implementation UIColor (Extensions)

#if !HAS_YYKIT_COLOR_YYADD

static inline NSUInteger hexStrToInt(NSString *str) {
    uint32_t result = 0;
    sscanf([str UTF8String], "%X", &result);
    return result;
}

static BOOL hexStrToRGBA(NSString *str,
                         CGFloat *r, CGFloat *g, CGFloat *b, CGFloat *a) {
    str = [[str fs_stringByTrim] uppercaseString];
    if ([str hasPrefix:@"#"]) {
        str = [str substringFromIndex:1];
    } else if ([str hasPrefix:@"0X"]) {
        str = [str substringFromIndex:2];
    }
    
    NSUInteger length = [str length];
    //         RGB            RGBA          RRGGBB        RRGGBBAA
    if (length != 3 && length != 4 && length != 6 && length != 8) {
        return NO;
    }
    
    //RGB,RGBA,RRGGBB,RRGGBBAA
    if (length < 5) {
        *r = hexStrToInt([str substringWithRange:NSMakeRange(0, 1)]) / 255.0f;
        *g = hexStrToInt([str substringWithRange:NSMakeRange(1, 1)]) / 255.0f;
        *b = hexStrToInt([str substringWithRange:NSMakeRange(2, 1)]) / 255.0f;
        if (length == 4)  *a = hexStrToInt([str substringWithRange:NSMakeRange(3, 1)]) / 255.0f;
        else *a = 1;
    } else {
        *r = hexStrToInt([str substringWithRange:NSMakeRange(0, 2)]) / 255.0f;
        *g = hexStrToInt([str substringWithRange:NSMakeRange(2, 2)]) / 255.0f;
        *b = hexStrToInt([str substringWithRange:NSMakeRange(4, 2)]) / 255.0f;
        if (length == 8) *a = hexStrToInt([str substringWithRange:NSMakeRange(6, 2)]) / 255.0f;
        else *a = 1;
    }
    return YES;
}

+ (instancetype)colorWithHexString:(NSString *)hexStr {
    CGFloat r, g, b, a;
    if (hexStrToRGBA(hexStr, &r, &g, &b, &a)) {
        return [UIColor colorWithRed:r green:g blue:b alpha:a];
    }
    return nil;
}

+ (UIColor *)colorWithRGB:(uint32_t)rgbValue {
    return [UIColor colorWithRed:((rgbValue & 0xFF0000) >> 16) / 255.0f
                           green:((rgbValue & 0xFF00) >> 8) / 255.0f
                            blue:(rgbValue & 0xFF) / 255.0f
                           alpha:1];
}


- (CGFloat)red {
    CGFloat r = 0, g, b, a;
    [self getRed:&r green:&g blue:&b alpha:&a];
    return r;
}

- (CGFloat)green {
    CGFloat r, g = 0, b, a;
    [self getRed:&r green:&g blue:&b alpha:&a];
    return g;
}

- (CGFloat)blue {
    CGFloat r, g, b = 0, a;
    [self getRed:&r green:&g blue:&b alpha:&a];
    return b;
}

- (CGFloat)alpha {
    return CGColorGetAlpha(self.CGColor);
}

#endif

+ (UIColor *)fs_blackColor{ return [self blackColor]; }
+ (UIColor *)fs_darkGrayColor { return [self darkGrayColor]; }
+ (UIColor *)fs_lightGrayColor { return [self lightGrayColor]; }
+ (UIColor *)fs_whiteColor { return [self whiteColor]; }
+ (UIColor *)fs_grayColor { return [self grayColor]; }
+ (UIColor *)fs_redColor { return [self redColor]; }
+ (UIColor *)fs_greenColor { return [self greenColor]; }
+ (UIColor *)fs_blueColor { return [self blueColor]; }
+ (UIColor *)fs_cyanColor { return [self cyanColor]; }
+ (UIColor *)fs_yellowColor { return [self yellowColor]; }
+ (UIColor *)fs_magentaColor { return [self magentaColor]; }
+ (UIColor *)fs_orangeColor { return [self orangeColor]; }
+ (UIColor *)fs_purpleColor { return [self purpleColor]; }
+ (UIColor *)fs_brownColor { return [self brownColor]; }
+ (UIColor *)fs_clearColor { return [self clearColor]; }

+ (UIColor *)fs_colorWithARGBHexString:(NSString *)hexStr {
    NSScanner *scanner = [NSScanner scannerWithString:hexStr];
    unsigned hexNum;
    if (![scanner scanHexInt:&hexNum]) return nil;
    if (hexStr.length == 8) {
        uint32_t result = 0;
        sscanf([[hexStr substringWithRange:NSMakeRange(0, 2)] UTF8String], "%X", &result);
        float a = result/255.0f;
        return [[UIColor colorWithRGB:hexNum] colorWithAlphaComponent:a];
    }
    return [UIColor colorWithRGB:hexNum];
}

- (uint32_t)fs_rgbValue {
    return [self rgbValue];
}

- (uint32_t)fs_argbValue {
    CGFloat r,g,b,a;
    if (![self getRed:&r green:&g blue:&b alpha:&a]) return 0;
    
    a = MIN(MAX(self.alpha, 0.0f), 1.0f);
    r = MIN(MAX(self.red, 0.0f), 1.0f);
    g = MIN(MAX(self.green, 0.0f), 1.0f);
    b = MIN(MAX(self.blue, 0.0f), 1.0f);
    
    return (((int)roundf(a * 255)) << 24)
    | (((int)roundf(r * 255)) << 16)
    | (((int)roundf(g * 255)) << 8)
    | (((int)roundf(b * 255)));
}

- (UIColor *)fs_colorFromCopy{
    CGColorRef colorRef = CGColorCreateCopy(self.CGColor);
    UIColor *newColor = [UIColor colorWithCGColor:colorRef];
    CGColorRelease(colorRef);
    return newColor;
}

@end

@implementation UIColor (DarkMode_iOS13)

+ (UIColor *)fs_colorWithLightHex:(NSString *)lightHex darkHex:(NSString *)darkHex{
    return [self fs_colorWithLight:[UIColor colorWithHexString:lightHex] dark:[UIColor colorWithHexString:darkHex]];
}

+ (UIColor *)fs_colorWithLight:(UIColor *)light dark:(UIColor *)dark{
   return [self fs_colorWithOwner:nil light:light dark:dark];
}

+ (UIColor *)fs_colorWithOwner:(nullable id<UITraitEnvironment>)owner light:(UIColor *)light dark:(UIColor *)dark {
    if (!light && !dark) return nil;
    if (!light) light = [self fs_whiteColor];
    if (!dark) return light;
   return [UIColor fs_colorWithDynamicProvider:^UIColor * _Nonnull(UITraitCollection * _Nonnull traitCollection) {
       #if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_12_0
       if (@available(iOS 12.0, *)) {
           if (owner) {
               traitCollection = owner.traitCollection;
           }
           if (traitCollection.userInterfaceStyle == UIUserInterfaceStyleDark) {
               return dark;
           }
       }
       #endif
       return light;
    }];
}

+ (UIColor *)fs_colorWithDynamicProvider:(UIColor * (^)(UITraitCollection *traitCollection))dynamicProvider{
    return [[self alloc] fs_initWithDynamicProvider:dynamicProvider];
}

- (UIColor *)fs_initWithDynamicProvider:(UIColor * (^)(UITraitCollection *traitCollection))dynamicProvider{
    
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
    if (@available(iOS 13.0, *)) {
        return [[UIColor alloc] initWithDynamicProvider:dynamicProvider];
    }
#elif __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_12_0
    if (@available(iOS 12.0, *)) {
        return dynamicProvider([UITraitCollection traitCollectionWithUserInterfaceStyle:UIUserInterfaceStyleLight]);
    }
#endif
    return dynamicProvider([UITraitCollection new]);
}

- (UIColor *)fs_lightColor{
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
    if (@available(iOS 13.0, *)) {
        return [self resolvedColorWithTraitCollection:[UITraitCollection traitCollectionWithUserInterfaceStyle:UIUserInterfaceStyleLight]] ? : self;
    }
#endif
    return self;
}

- (UIColor *)fs_darkColor{
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
    if (@available(iOS 13.0, *)) {
        return [self resolvedColorWithTraitCollection:[UITraitCollection traitCollectionWithUserInterfaceStyle:UIUserInterfaceStyleDark]] ? : self;
    }
#endif
    return self;
}

- (BOOL)fs_isDynamicColor{
    if (@available(iOS 13.0, *)) {
        return [self fs_lightColor] != [self fs_darkColor];
    }
    return NO;
}

@end
