/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "UIImage+FSAdd.h"
#import "UIColor+FSAdd.h"
#import "FSMacro.h"
#import <objc/runtime.h>

static Class _fs_bundleForClass = nil;
static NSString *_fs_customImageColorSuffix = @"_color_";
@implementation UIImage (FSAdd)

+ (Class)fs_bundleForClass{
    return _fs_bundleForClass ? _fs_bundleForClass : [self class];
}

+ (void)setFs_bundleForClass:(Class)fs_bundleForClass{
    _fs_bundleForClass = fs_bundleForClass;
}

+ (NSString *)fs_customImageColorSuffix{
    return _fs_customImageColorSuffix;
}

+ (void)setFs_customImageColorSuffix:(NSString *)fs_customImageColorSuffix{
    _fs_customImageColorSuffix = fs_customImageColorSuffix;
}

+ (UIImage *)fs_imageNamed:(NSString *)name{
    return [self fs_imageNamed:name tintColor:nil];
}

+ (UIImage *)fs_imageNamed:(NSString *)name tintColor:(UIColor *)tintColor{
    if (!name.length) return nil;
    return [[UIImage imageNamed:name inBundle:[NSBundle bundleForClass:self.fs_bundleForClass] compatibleWithTraitCollection:nil] fs_imageWithTintColor:tintColor];
}

+ (UIImage *)fs_imageNamed:(NSString *)name tintColor:(UIColor *)tintColor customColor:(UIColor *)color{
    return [self fs_imageNamed:name tintColor:tintColor customColors:@[color ? color : [UIColor fs_clearColor]]];
}

+ (UIImage *)fs_imageNamed:(NSString *)name tintColor:(UIColor *)tintColor customColors:(NSArray<UIColor *> *)colors{
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
    if (@available(iOS 13.0, *)) {
        do {
            NSString *customImageName = [name stringByAppendingFormat:@"%@%d",
                                         self.fs_customImageColorSuffix, 0];
            bool (^hasCustom)() = ^bool() {
                if (colors.count == 0) {
                    return false;
                }
                if ([self fs_imageNamed:customImageName] == nil) {
                    return false;
                }
                return true;
            };
            if (!hasCustom()) {
                break;
            }
            typedef UIColor *(^ResolveColor)(UIColor *);
            ResolveColor resolveLightColor = ^UIColor *(UIColor *color) {
                UITraitCollection *traitCollection = [UITraitCollection traitCollectionWithUserInterfaceStyle:UIUserInterfaceStyleLight];
                return [color resolvedColorWithTraitCollection:traitCollection] ?: color;
            };
            ResolveColor resolveDarkColor = ^UIColor *(UIColor *color) {
                UITraitCollection *traitCollection = [UITraitCollection traitCollectionWithUserInterfaceStyle:UIUserInterfaceStyleDark];
                return [color resolvedColorWithTraitCollection:traitCollection] ?: color;
            };
            
            UIImage *(^getImage)(ResolveColor) = ^UIImage *(ResolveColor resolveColor) {
                UIImage *image = [self fs_imageNamed:name tintColor:resolveColor(self.fs_iconColor_I3)];
                if (!image) {
                    return nil;
                }
                for (int i = 0; i < colors.count; i++) {
                    UIImage *colorImage = [self fs_imageNamed:customImageName tintColor:resolveColor(colors[i])];
                    if (colorImage) {
                        image = [image fs_addImage:colorImage];
                    }
                }
                return image;
            };
            UIImage *normalImage = getImage(resolveLightColor);
            UIImage *darkImage = getImage(resolveDarkColor);
            return [UIImage fs_dynamicImageWithNormalImage:normalImage darkImage:darkImage];
        } while (false);
    } // @available(iOS 13.0, *)
#endif
    UIImage *image = [self fs_imageNamed:name tintColor:tintColor];
    if (!image) {
        return nil;
    }
    for (int i = 0; i < colors.count; i++) {
        name = [name stringByAppendingFormat:@"%@%d",self.fs_customImageColorSuffix,0];
        UIImage *colorImage = [self fs_imageNamed:name tintColor:colors[i]];
        if (colorImage) {
            image = [image fs_addImage:colorImage];
        }
    }
    return image;
}

+ (UIImage *)fs_imageWithColor:(UIColor *)color{
    return [self fs_imageWithColor:color size:CGSizeMake(1, 1)];
}

+ (UIImage *)fs_imageWithColor:(UIColor *)color size:(CGSize)size {
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
    if (@available(iOS 13.0, *)) {
        UIColor *lightColor = [color resolvedColorWithTraitCollection:[UITraitCollection traitCollectionWithUserInterfaceStyle:UIUserInterfaceStyleLight]];
        UIColor *darkColor = [color resolvedColorWithTraitCollection:[UITraitCollection traitCollectionWithUserInterfaceStyle:UIUserInterfaceStyleDark]];
        if (![lightColor isEqual:darkColor]) {
            UIImage *(^getImage)(UIColor *) = ^UIImage *(UIColor *color) {
                UIGraphicsBeginImageContext(size);
                CGContextRef context = UIGraphicsGetCurrentContext();
                
                CGContextSetFillColorWithColor(context, [color CGColor]);
                CGContextFillRect(context, CGRectMake(0, 0, size.width, size.height));
                
                UIImage *image = UIGraphicsGetImageFromCurrentImageContext();
                UIGraphicsEndImageContext();
                
                return image;
            };
            UIImage *lightImage = getImage(lightColor);
            UIImage *darkImage = getImage(darkColor);
            return [UIImage fs_dynamicImageWithNormalImage:lightImage darkImage:darkImage];
        }
    }
#endif
    UIGraphicsBeginImageContext(size);
    CGContextRef context = UIGraphicsGetCurrentContext();
    
    CGContextSetFillColorWithColor(context, [color CGColor]);
    CGContextFillRect(context, CGRectMake(0, 0, size.width, size.height));
    
    UIImage *image = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    
    return image;
}

+ (UIImage *)fs_imageWithColor:(UIColor *)color font:(UIFont *)font text:(NSString *)text size:(CGSize)size{
    
    if (!color ) return nil;
    NSDictionary *textAttributes = @{
        NSFontAttributeName:font,
        NSForegroundColorAttributeName:color
    };
    CGSize textSize = [text sizeWithAttributes:textAttributes];
    if (size.height <= 0 || size.width <= 0) {
        size = textSize;
    }
    CGRect rect = CGRectMake(0, 0, size.width, size.height);
    if (rect.size.width <= 0 || rect.size.height <= 0) return nil;
    UIGraphicsBeginImageContextWithOptions(rect.size, NO, [UIScreen mainScreen].scale);
    CGContextRef context = UIGraphicsGetCurrentContext();
    
    CGContextSetFillColorWithColor(context, [UIColor clearColor].CGColor);
    CGContextFillRect(context, rect);
    
    [text drawInRect:CGRectMake((size.width - textSize.width) / 2, (size.height - textSize.height) / 2, textSize.width, textSize.height) withAttributes:textAttributes];
    
    UIImage *image = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return image;
}

+ (UIImage *)fs_dynamicImageWithNormalImage:(UIImage *)normalImage darkImage:(UIImage *)darkImage {
    if (normalImage == nil || darkImage == nil) {
        return normalImage ?: darkImage;
    }
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
    if (@available(iOS 13.0, *)) {
        UIImageAsset* imageAseset = [[UIImageAsset alloc] init];
        UITraitCollection* lightImageTraitCollection = [UITraitCollection traitCollectionWithTraitsFromCollections:
                                                        @[[UITraitCollection traitCollectionWithUserInterfaceStyle:UIUserInterfaceStyleLight],
                                                          [UITraitCollection traitCollectionWithDisplayScale:normalImage.scale]]];
        [imageAseset registerImage:normalImage withTraitCollection:lightImageTraitCollection];
        
        UITraitCollection* darkImageTraitCollection = [UITraitCollection traitCollectionWithTraitsFromCollections:
                                                       @[[UITraitCollection traitCollectionWithUserInterfaceStyle:UIUserInterfaceStyleDark],
                                                         [UITraitCollection traitCollectionWithDisplayScale:darkImage.scale]]];
        [imageAseset registerImage:darkImage withTraitCollection:darkImageTraitCollection];
        
        return [imageAseset imageWithTraitCollection:[UITraitCollection currentTraitCollection]];
    }
#endif
    return normalImage;
}

+ (UIImage *)fs_imageDrawRingWithRingW:(CGFloat)ringW borderColor:(UIColor *)color size:(CGSize)size{
    if (size.width <= 0 || size.height <= 0) return nil;
    UIGraphicsBeginImageContextWithOptions(size, NO, 0);
    UIBezierPath *path = [UIBezierPath bezierPathWithRoundedRect:CGRectMake(0, 0, size.width, size.height) cornerRadius:size.width/2];
    [[UIColor fs_clearColor] set];
    [path fill];
    CGSize innerRingSize = CGSizeMake(size.width - ringW * 2, size.height - ringW * 2);
    path = [UIBezierPath bezierPathWithRoundedRect:CGRectMake(ringW, ringW, innerRingSize.width, innerRingSize.height) cornerRadius:innerRingSize.width/2];
    [color set];
    [path stroke];
    UIImage *newImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return newImage;
}

- (UIImage *)fs_borderWidth:(CGFloat)borderW borderColor:(UIColor *)color{
    return [self fs_borderWidth:borderW borderColor:color cornerRadius:0];
}

- (UIImage *)fs_borderWidth:(CGFloat)borderW borderColor:(UIColor *)color cornerRadius:(CGFloat)radius{
    
    CGSize size = CGSizeMake(self.size.width + 2 * borderW, self.size.height + 2 * borderW);
    if (size.width <= 0 || size.height <= 0) return nil;
    UIGraphicsBeginImageContextWithOptions(size, NO, 0);
    
    UIBezierPath *path = nil;
    if (radius) {
        path = [UIBezierPath bezierPathWithRoundedRect:CGRectMake(0, 0, size.width, size.height) cornerRadius:radius];
    }else {
        path = [UIBezierPath bezierPathWithRect:CGRectMake(0, 0, size.width, size.height)];
    }
    [color set];
    [path fill];

    if (radius) {
        path = [UIBezierPath bezierPathWithRoundedRect:CGRectMake(borderW, borderW, self.size.width, self.size.height) cornerRadius:radius];
    }else{
        path = [UIBezierPath bezierPathWithRect:CGRectMake(borderW, borderW, self.size.width, self.size.height)];
    }
    [path addClip];
    [self drawInRect:CGRectMake(borderW, borderW, self.size.width, self.size.height)];
    UIImage *newImage = UIGraphicsGetImageFromCurrentImageContext();

    UIGraphicsEndImageContext();
    return newImage;
}

- (UIImage *)fs_addImage:(UIImage *)image {
    if (!image) return self;
    BOOL baseIsDynamicImage = self.fs_isDynamicImage;
    BOOL coverIsDynamicImage = image.fs_isDynamicImage;
    if (baseIsDynamicImage && coverIsDynamicImage) {
        UIImage *lightImage = [self.fs_lightImage fs_addImage:image.fs_lightImage];
        UIImage *darkImage = [self.fs_darkImage fs_addImage:image.fs_darkImage];
        return [UIImage fs_dynamicImageWithNormalImage:lightImage darkImage:darkImage];
    }else if (baseIsDynamicImage && !coverIsDynamicImage){
        UIImage *lightImage = [self.fs_lightImage fs_addImage:image];
        UIImage *darkImage = [self.fs_darkImage fs_addImage:image];
        return [UIImage fs_dynamicImageWithNormalImage:lightImage darkImage:darkImage];
    }else if (!baseIsDynamicImage && coverIsDynamicImage){
        UIImage *lightImage = [self fs_addImage:image.fs_lightImage];
        UIImage *darkImage = [self fs_addImage:image.fs_darkImage];
        return [UIImage fs_dynamicImageWithNormalImage:lightImage darkImage:darkImage];
    }
    if (self.size.width <= 0 || self.size.height <= 0) return nil;
    UIGraphicsBeginImageContextWithOptions(self.size, NO, self.scale);
    CGContextRef ctx = UIGraphicsGetCurrentContext();
    if (!ctx) return nil;
    CGRect area = CGRectMake(0, 0, self.size.width, self.size.height);
    [self drawInRect:area];
    
    area = CGRectMake(CGRectGetMidX(area) - image.size.width/2, CGRectGetMidY(area) - image.size.height/2, image.size.width, image.size.height);
    [image drawInRect:area];
    
    UIImage *newImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return newImage;
}

- (UIImage *)fs_applyingAlpha:(CGFloat)alpha {
    if (self.fs_isDynamicImage) {
        UIImage *lightImage = [self.fs_lightImage fs_applyingAlpha:alpha];
        UIImage *darkImage =  [self.fs_darkImage fs_applyingAlpha:alpha];
        return [UIImage fs_dynamicImageWithNormalImage:lightImage darkImage:darkImage];
    }
    if (self.size.width <= 0 || self.size.height <= 0) return nil;
    UIGraphicsBeginImageContextWithOptions(self.size, NO, self.scale);
    CGContextRef ctx = UIGraphicsGetCurrentContext();
    if (!ctx)
        return nil;
    CGRect area = CGRectMake(0, 0, self.size.width, self.size.height);
    [self drawInRect:area blendMode:kCGBlendModeNormal alpha:alpha];

    UIImage *newImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return newImage;
}

- (UIImage *) fs_imageWithTintColor:(UIColor *)tintColor{
    if (!tintColor) return self;
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
    if (@available(iOS 13.0, *)) {
        if (![tintColor fs_isDynamicColor] && ![self fs_isDynamicImage]) return [self imageWithTintColor:tintColor];
    }
#endif
    if ([tintColor fs_isDynamicColor] || [self fs_isDynamicImage]){
        UIImage *lightImage = [self.fs_lightImage fs_imageWithTintColor:tintColor.fs_lightColor blendMode:kCGBlendModeDestinationIn];
        UIImage *darkImage =  [self.fs_darkImage fs_imageWithTintColor:tintColor.fs_darkColor blendMode:kCGBlendModeDestinationIn];
        return [UIImage fs_dynamicImageWithNormalImage:lightImage darkImage:darkImage];
    }
    return [self fs_imageWithTintColor:tintColor blendMode:kCGBlendModeDestinationIn];
}

- (UIImage *) fs_imageWithGradientTintColor:(UIColor *)tintColor{
    return [self fs_imageWithTintColor:tintColor blendMode:kCGBlendModeOverlay];
}

- (UIImage *) fs_imageWithTintColor:(UIColor *)tintColor blendMode:(CGBlendMode)blendMode{
    UIImage *img = self;
    if (tintColor) {
        CGSize size = img.size;
        if (size.width <= 0 || size.height <= 0) return nil;
        UIGraphicsBeginImageContextWithOptions(size, NO, img.scale);
        [tintColor setFill];
        CGRect bounds = CGRectMake(0, 0, size.width, size.height);
        UIRectFill(bounds);
        
        [img drawInRect:bounds blendMode:blendMode alpha:1.0f];
        
        if (blendMode != kCGBlendModeDestinationIn) {
            [img drawInRect:bounds blendMode:kCGBlendModeDestinationIn alpha:1.0f];
        }
        
        img = UIGraphicsGetImageFromCurrentImageContext();
        UIGraphicsEndImageContext();
    }
    return img;
}

- (UIImage *)fs_cornerRadius:(CGFloat)radius{
    if (self.size.width <= 0 || self.size.height <= 0) return nil;
    UIGraphicsBeginImageContextWithOptions(self.size, NO, self.scale);
    CGContextRef ctx = UIGraphicsGetCurrentContext();
    CGRect rect = CGRectMake(0, 0, self.size.width, self.size.height);
    
    CGContextAddPath(ctx, [UIBezierPath bezierPathWithRoundedRect:rect cornerRadius:radius].CGPath);
    CGContextClip(ctx);
    
    [self drawInRect:rect];
    
    UIImage *newImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return newImage;
}

- (UIImage *)fs_adjustScaledBySize:(CGSize)newSize {
    if (newSize.width <= 0 || newSize.height <= 0) return nil;
    UIGraphicsBeginImageContextWithOptions(newSize, NO, 0.0);
    [self drawInRect:CGRectMake(0, 0, newSize.width, newSize.height)];
    UIImage *newImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return newImage;
}

- (UIImage *)fs_compressWithMaxLength:(NSUInteger)maxLength{
    CGFloat compression = 1;
    NSData *data = UIImageJPEGRepresentation(self, compression);
    if (data.length < maxLength) return self;
    
    CGFloat max = 1;
    CGFloat min = 0;
    for (int i = 0; i < 6; ++i) {
        compression = (max + min) / 2;
        data = UIImageJPEGRepresentation(self, compression);
        if (data.length < maxLength * 0.9) {
            min = compression;
        } else if (data.length > maxLength) {
            max = compression;
        } else {
            break;
        }
    }
    if (data.length < maxLength) return [UIImage imageWithData:data];
    UIImage *resultImage = [UIImage imageWithData:data];
    NSUInteger lastDataLength = 0;
    while (data.length > maxLength && data.length != lastDataLength) {
        lastDataLength = data.length;
        CGFloat ratio = (CGFloat)maxLength / data.length;
        CGSize size = CGSizeMake((NSUInteger)(resultImage.size.width * sqrtf(ratio)),
                                 (NSUInteger)(resultImage.size.height * sqrtf(ratio)));
        UIGraphicsBeginImageContext(size);
        [resultImage drawInRect:CGRectMake(0, 0, size.width, size.height)];
        resultImage = UIGraphicsGetImageFromCurrentImageContext();
        UIGraphicsEndImageContext();
        data = UIImageJPEGRepresentation(resultImage, compression);
    }
    return [UIImage imageWithData:data];
}

- (UIImage*)fs_tailoringRect:(CGRect)rect{
    if (self.fs_isDynamicImage) {
        UIImage *lightImage = [self.fs_lightImage fs_tailoringRect:rect];
        UIImage *darkImage = [self.fs_darkImage fs_tailoringRect:rect];
        return [UIImage fs_dynamicImageWithNormalImage:lightImage darkImage:darkImage];
    }
    CGImageRef newImageRef = CGImageCreateWithImageInRect(self.CGImage, rect);
    UIImage *newImage = [UIImage imageWithCGImage:newImageRef];
    CGImageRelease(newImageRef);
    return newImage;
}

- (UIImage *)fs_imageFromCopy {
    CGImageRef imageRef = CGImageCreateCopy(self.CGImage);
    UIImage *newImage = [UIImage imageWithCGImage:imageRef scale:self.scale orientation:self.imageOrientation];
    CGImageRelease(imageRef);
    return newImage;
}

- (UIImage *)fs_lightImage{
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
    if (@available(iOS 13.0, *)) {
        if (self.imageAsset) {
            UIImageAsset* imageAseset = self.imageAsset;
            UITraitCollection *lightImageTraitCollection = [UITraitCollection traitCollectionWithTraitsFromCollections:
                                                            @[[UITraitCollection traitCollectionWithUserInterfaceStyle:UIUserInterfaceStyleLight]]];
            UIImage *lightImage = [imageAseset imageWithTraitCollection:lightImageTraitCollection];
            return [lightImage fs_imageFromCopy] ? : self;
        }
    }
#endif
    return self;
}

- (UIImage *)fs_darkImage{
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
    if (@available(iOS 13.0, *)) {
        if (self.imageAsset) {
            UIImageAsset *imageAseset = self.imageAsset;
            UITraitCollection *darkImageTraitCollection = [UITraitCollection traitCollectionWithTraitsFromCollections:
                                                           @[[UITraitCollection traitCollectionWithUserInterfaceStyle:UIUserInterfaceStyleDark]]];
            UIImage *darkImage = [imageAseset imageWithTraitCollection:darkImageTraitCollection];
            return [darkImage fs_imageFromCopy] ? : self;
        }
    }
#endif
    return self;
}

- (BOOL)fs_isDynamicImage{
    if (@available(iOS 13.0, *)) {
        if (self.imageAsset) {
            NSData *lightImageData = UIImagePNGRepresentation(self.fs_lightImage);
            NSData *darkImageData = UIImagePNGRepresentation(self.fs_darkImage);
            if (!lightImageData || !darkImageData) return NO;
            return ![lightImageData isEqualToData:darkImageData];
        }
    }
    return NO;
}

- (UIImage *)fs_fixOrientation {
    
    UIImage *aImage = self;
    if (aImage.imageOrientation == UIImageOrientationUp)
        return aImage;
    CGAffineTransform transform = CGAffineTransformIdentity;
    
    switch (aImage.imageOrientation) {
        case UIImageOrientationDown:
        case UIImageOrientationDownMirrored:
            transform = CGAffineTransformTranslate(transform, aImage.size.width, aImage.size.height);
            transform = CGAffineTransformRotate(transform, M_PI);
            break;
            
        case UIImageOrientationLeft:
        case UIImageOrientationLeftMirrored:
            transform = CGAffineTransformTranslate(transform, aImage.size.width, 0);
            transform = CGAffineTransformRotate(transform, M_PI_2);
            break;
            
        case UIImageOrientationRight:
        case UIImageOrientationRightMirrored:
            transform = CGAffineTransformTranslate(transform, 0, aImage.size.height);
            transform = CGAffineTransformRotate(transform, -M_PI_2);
            break;
        default:
            break;
    }
    
    switch (aImage.imageOrientation) {
        case UIImageOrientationUpMirrored:
        case UIImageOrientationDownMirrored:
            transform = CGAffineTransformTranslate(transform, aImage.size.width, 0);
            transform = CGAffineTransformScale(transform, -1, 1);
            break;
            
        case UIImageOrientationLeftMirrored:
        case UIImageOrientationRightMirrored:
            transform = CGAffineTransformTranslate(transform, aImage.size.height, 0);
            transform = CGAffineTransformScale(transform, -1, 1);
            break;
        default:
            break;
    }
    
    // Now we draw the underlying CGImage into a new context, applying the transform
    // calculated above.
    CGContextRef ctx = CGBitmapContextCreate(NULL, aImage.size.width, aImage.size.height,
                                             CGImageGetBitsPerComponent(aImage.CGImage), 0,
                                             CGImageGetColorSpace(aImage.CGImage),
                                             CGImageGetBitmapInfo(aImage.CGImage));
    CGContextConcatCTM(ctx, transform);
    switch (aImage.imageOrientation) {
        case UIImageOrientationLeft:
        case UIImageOrientationLeftMirrored:
        case UIImageOrientationRight:
        case UIImageOrientationRightMirrored:
            // Grr...
            CGContextDrawImage(ctx, CGRectMake(0,0,aImage.size.height,aImage.size.width), aImage.CGImage);
            break;
            
        default:
            CGContextDrawImage(ctx, CGRectMake(0,0,aImage.size.width,aImage.size.height), aImage.CGImage);
            break;
    }
    
    // And now we just create a new UIImage from the drawing context
    CGImageRef cgimg = CGBitmapContextCreateImage(ctx);
    UIImage *img = [UIImage imageWithCGImage:cgimg];
    CGContextRelease(ctx);
    CGImageRelease(cgimg);
    return img;
}

- (UIImage*)fs_replaceColor:(UIColor*)color withTolerance:(float)tolerance {
    CGImageRef imageRef = [self CGImage];
    
    NSUInteger width = CGImageGetWidth(imageRef);
    NSUInteger height = CGImageGetHeight(imageRef);
    CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
    
    NSUInteger bytesPerPixel = 4;
    NSUInteger bytesPerRow = bytesPerPixel * width;
    NSUInteger bitsPerComponent = 8;
    NSUInteger bitmapByteCount = bytesPerRow * height;
    
    unsigned char *rawData = (unsigned char*) calloc(bitmapByteCount, sizeof(unsigned char));
    
    CGContextRef context = CGBitmapContextCreate(rawData, width, height,
                                                 bitsPerComponent, bytesPerRow, colorSpace,
                                                 kCGImageAlphaPremultipliedLast | kCGBitmapByteOrder32Big);
    CGColorSpaceRelease(colorSpace);
    
    CGContextDrawImage(context, CGRectMake(0, 0, width, height), imageRef);
    
    CGColorRef cgColor = [color CGColor];
    const CGFloat *components = CGColorGetComponents(cgColor);
    float r = components[0];
    float g = components[1];
    float b = components[2];
    //float a = components[3]; // not needed
    
    r = r * 255.0;
    g = g * 255.0;
    b = b * 255.0;
    
    const float redRange[2] = {
        MAX(r - (tolerance / 2.0), 0.0),
        MIN(r + (tolerance / 2.0), 255.0)
    };
    
    const float greenRange[2] = {
        MAX(g - (tolerance / 2.0), 0.0),
        MIN(g + (tolerance / 2.0), 255.0)
    };
    
    const float blueRange[2] = {
        MAX(b - (tolerance / 2.0), 0.0),
        MIN(b + (tolerance / 2.0), 255.0)
    };
    
    int byteIndex = 0;
    
    while (byteIndex < bitmapByteCount) {
        unsigned char red   = rawData[byteIndex];
        unsigned char green = rawData[byteIndex + 1];
        unsigned char blue  = rawData[byteIndex + 2];
        
        if (((red >= redRange[0]) && (red <= redRange[1])) &&
            ((green >= greenRange[0]) && (green <= greenRange[1])) &&
            ((blue >= blueRange[0]) && (blue <= blueRange[1]))) {
            rawData[byteIndex] = 0;
            rawData[byteIndex + 1] = 0;
            rawData[byteIndex + 2] = 0;
            rawData[byteIndex + 3] = 0;
        }
        byteIndex += 4;
    }
    
    CGImageRef imgref = CGBitmapContextCreateImage(context);
    UIImage *result = [UIImage imageWithCGImage:imgref];
    
    CGImageRelease(imgref);
    CGContextRelease(context);
    free(rawData);
    
    return result;
}

- (UIImage *)fs_smoothWithRadius:(CGFloat)radius {
    CIImage *ciImage = [[CIImage alloc] initWithImage:self];
    
    CIImage *clampedImage = [ciImage imageByClampingToExtent];
    
    CIFilter *blurFilter = [CIFilter filterWithName:@"CIBoxBlur"];
    [blurFilter setValue:clampedImage forKey:kCIInputImageKey];
    [blurFilter setValue:@(radius) forKey:kCIInputRadiusKey];
    
    CIImage *outputImage = blurFilter.outputImage;
    CIImage *croppedImage = [outputImage imageByCroppingToRect:ciImage.extent];
    
    CIContext *context = [CIContext contextWithOptions:nil];
    CGRect rect = [ciImage extent];
    CGImageRef cgImage = [context createCGImage:croppedImage fromRect:rect];
    
    UIImage *smoothedImage = [UIImage imageWithCGImage:cgImage];
    CGImageRelease(cgImage);
    return smoothedImage;
}

- (UIImage *)fs_sharpenWithSharpness:(CGFloat)sharpness {
    CIImage *ciImage = [[CIImage alloc] initWithImage:self];
    CIFilter *sharpenFilter = [CIFilter filterWithName:@"CISharpenLuminance"];
    [sharpenFilter setValue:ciImage forKey:kCIInputImageKey];
    [sharpenFilter setValue:@(sharpness) forKey:kCIInputSharpnessKey];
    
    CIImage *outputImage = sharpenFilter.outputImage;
    CIContext *context = [CIContext contextWithOptions:nil];
    CGImageRef cgImage = [context createCGImage:outputImage fromRect:[outputImage extent]];
    
    UIImage *sharpenedImage = [UIImage imageWithCGImage:cgImage];
    CGImageRelease(cgImage);
    return sharpenedImage;
}

+ (UIImage *)fs_imageDrawByBlackColor{
    static UIImage *drawImage;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        drawImage = [UIImage fs_imageWithColor:[UIColor fs_blackColor]];
    });
    return drawImage;
}

+ (UIImage *)fs_imageDrawByWhiteColor{
    static UIImage *drawImage;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        drawImage = [UIImage fs_imageWithColor:[UIColor fs_whiteColor]];
    });
    return drawImage;
}

+ (UIImage *)fs_imageDrawByRedColor{
    static UIImage *drawImage;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        drawImage = [UIImage fs_imageWithColor:[UIColor fs_redColor]];
    });
    return drawImage;
}

+ (UIImage *)fs_imageDrawByGreenColor{
    static UIImage *drawImage;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        drawImage = [UIImage fs_imageWithColor:[UIColor fs_greenColor]];
    });
    return drawImage;
}

+ (UIImage *)fs_imageDrawByBlueColor{
    static UIImage *drawImage;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        drawImage = [UIImage fs_imageWithColor:[UIColor fs_blueColor]];
    });
    return drawImage;
}

+ (UIImage *)fs_imageDrawByClearColor{
    static UIImage *drawImage;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        drawImage = [UIImage fs_imageWithColor:[UIColor fs_clearColor]];
    });
    return drawImage;
}

+ (UIImage *)fs_imageDrawByCenterColor{
    static UIImage *drawImage;

    drawImage = [UIImage fs_imageWithColor:[UIColor fs_backgroundColor_B13]];
    return drawImage;
}
@end

