/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "UISlider+FSAdd.h"

static NSString *const kFS_allowTapAction = @"kFS_allowTapAction";
@interface UISlider (FSAdd)<UIGestureRecognizerDelegate>
@property (nonatomic, readonly) UITapGestureRecognizer *tapGesture;
@end
@implementation UISlider (FSAdd)

- (UITapGestureRecognizer *)tapGesture{
    for (UIGestureRecognizer *recognizer in self.gestureRecognizers) {
        if ([recognizer.name isEqualToString:kFS_allowTapAction]  && [recognizer isKindOfClass:[UITapGestureRecognizer class]]) {
            return (UITapGestureRecognizer *)recognizer;
        }
    }
    return nil;
}

- (BOOL)fs_allowTapAction{
    return self.tapGesture;
}

- (void)setFs_allowTapAction:(BOOL)fs_allowTapAction{
    UITapGestureRecognizer *tapGesture = self.tapGesture;
    if (fs_allowTapAction) {
        if (!tapGesture) {
            [self addTarget:self action:@selector(touchDown:) forControlEvents:UIControlEventTouchDown];
            [self addTarget:self action:@selector(touchUp:) forControlEvents:UIControlEventTouchUpInside | UIControlEventTouchUpOutside];
            tapGesture = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(actionTapGesture:)];
            tapGesture.delegate = self;
            [self addGestureRecognizer:tapGesture];
        }
    }else{
        if (tapGesture) {
            [self removeTarget:self action:@selector(touchDown:) forControlEvents:UIControlEventTouchDown];
            [self removeTarget:self action:@selector(touchUp:) forControlEvents:UIControlEventTouchUpInside | UIControlEventTouchUpOutside];
            [self removeGestureRecognizer:tapGesture];
        }
    }
}

- (void)actionTapGesture:(UITapGestureRecognizer *)sender {
    CGPoint touchPoint = [sender locationInView:self];
    CGFloat value = (self.maximumValue - self.minimumValue) * (touchPoint.x / self.frame.size.width);
    [self setValue:value animated:NO];
    [self sendActionsForControlEvents:UIControlEventValueChanged];
}

#pragma mark - UIGestureRecognizerDelegate
- (void)touchDown:(UISlider *)sender {
    self.tapGesture.enabled = NO;
}

- (void)touchUp:(UISlider *)sender {
    self.tapGesture.enabled = YES;
}
@end
