/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import <UIKit/UIKit.h>

#ifndef dispatch_async_global
#define dispatch_async_global(block)\
    dispatch_async(dispatch_get_global_queue(0, 0), block);
#endif

NS_ASSUME_NONNULL_BEGIN

@protocol UIViewAdapter <NSObject>
@optional
- (void)traitCollectionDidChange:(UITraitCollection *)previousTraitCollection fromView:(__kindof UIView *)view;
- (void)layoutSubviewsFromView:(__kindof UIView *)view;
@end

@interface FSAdapterUIView : UIView
@property (nonatomic, weak) id<UIViewAdapter> adapter;
@end

@interface FSAdapterUIScrollView: UIScrollView
@property (nonatomic, weak) id<UIViewAdapter> adapter;
@end

#if __has_include(<YZHookHandler/YZHookHandler.h>) || __has_include("YZHookHandler.h")
@interface UIView (FSAdapter)
@property (nonatomic, weak, setter=fs_setViewAdapter:)   id<UIViewAdapter> fs_adapter;
@end
#endif

@interface UIView (FSAdd)

@property (nonatomic, setter=fs_setLeft:)    CGFloat fs_left;
@property (nonatomic, setter=fs_setTop:)     CGFloat fs_top;
@property (nonatomic, setter=fs_setRight:)   CGFloat fs_right;
@property (nonatomic, setter=fs_setBottom:)  CGFloat fs_bottom;
@property (nonatomic, setter=fs_setWidth:)   CGFloat fs_width;
@property (nonatomic, setter=fs_setHeight:)  CGFloat fs_height;
@property (nonatomic, setter=fs_setCenterX:) CGFloat fs_centerX;
@property (nonatomic, setter=fs_setCenterY:) CGFloat fs_centerY;
@property (nonatomic, setter=fs_setOrigin:)  CGPoint fs_origin;
@property (nonatomic, setter=fs_setSize:)    CGSize  fs_size;
@property (nonatomic, readonly)    UIEdgeInsets  fs_safeAreaInsets;

- (void)fs_setLayerCornerRadius:(CGFloat)cornerRadius;
- (void)fs_setLayerBorderWidth:(CGFloat)borderWidth borderColor:(nullable UIColor *)borderColor;
- (void)fs_setLayerBorderWidth:(CGFloat)borderWidth borderColor:(nullable UIColor *)borderColor lineDashPattern:(nullable NSArray *)lineDashPattern;

- (void)fs_setShadowColor:(nullable UIColor *)color;

- (UITableView *)fs_tableView;

- (UIViewController *)fs_windowRootViewController;

- (UIViewController *)fs_viewController;

- (void)fs_logSubviewWithLevel:(int)level;

- (UIView *)fs_getSubviewWithClassName:(NSString *)className;
- (__kindof UIView *)fs_getSuperviewOfClassName:(NSString *)className;
- (void)fs_removeSubviewWithClassName:(NSString *)className;

- (UIImage *)fs_snapshoot;

- (id)fs_findFirstResponder;

- (void)fs_resignOtherInputFirstResponderCurrent:(UIWindow *)window;

@end

#if __has_include(<MBProgressHUD/MBProgressHUD.h>) || __has_include("MBProgressHUD.h")
@class MBProgressHUD;
@interface UIView (MBProgressHUD)
- (MBProgressHUD *)fs_MBProgressHUD;

- (MBProgressHUD *)fs_showHUDCustomView:(UIView * _Nullable)customView;

- (MBProgressHUD *)fs_showHUDCustomView:(UIView * _Nullable)customView msg:(NSString * _Nullable)msg;

- (void)fs_scheduledMainQueueShowHUDLoading:(NSString *)msg process:(dispatch_block_t)process;
//queue dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0)
- (void)fs_showHUDLoading:(NSString *_Nullable)msg process:(_Nullable dispatch_block_t)process;

- (void)fs_showHUDLoading:(NSString *)msg process:(dispatch_block_t)process synch:(BOOL)synch;

- (void)fs_showHUDLoading:(NSString *)msg process:(dispatch_block_t)process synch:(BOOL)synch onQueue:(dispatch_queue_t)queue;

- (MBProgressHUD *)fs_showHUDLoading:(NSString *_Nullable)msg;

- (void)fs_hideHUDLoading;

- (MBProgressHUD *)fs_showHUDMessage:(NSString *)msg;

- (MBProgressHUD *)fs_showHUDMessage:(NSString *)msg afterHideDelay:(NSTimeInterval)delay;
@end
#endif

NS_ASSUME_NONNULL_END
