/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "UIView+FSAdd.h"
#import "FSPreferences.h"
#import "FSMacro.h"

#if __has_include(<YZHookHandler/YZHookHandler.h>) || __has_include("YZHookHandler.h")
#if __has_include(<YZHookHandler/YZHookHandler.h>)
    #import <YZHookHandler/YZHookHandler.h>
#else
    #import "YZHookHandler.h"
#endif
@implementation UIView (FSAdapter)
- (id<UIViewAdapter>)fs_adapter{
    return [self fs_getWeakAssociatedForKey:@keypath(self, fs_adapter)];
}

- (void)fs_setViewAdapter:(id<UIViewAdapter>)adapter{
    [self fs_addWeakAssociatedObject:adapter forKey:@keypath(self, fs_adapter)];
    if (adapter) {
        @fs_weakify(self)
        [[self yz_hookForSelector:@selector(layoutSubviews)] after:^(NSArray * _Nonnull args) {
            @fs_strongify(self)
            if (self.fs_adapter && [self.fs_adapter respondsToSelector:@selector(layoutSubviewsFromView:)]) {
                [self.fs_adapter layoutSubviewsFromView:self];
            }
        }];
        
        [[self yz_hookForSelector:@selector(traitCollectionDidChange:)] after:^(NSArray * _Nonnull args) {
            @fs_strongify(self)
            if (self.fs_adapter && [self.fs_adapter respondsToSelector:@selector(traitCollectionDidChange:fromView:)]) {
                UITraitCollection *previousTraitCollection = args[0];
                [self.fs_adapter traitCollectionDidChange:previousTraitCollection fromView:self];
            }
        }];
    }else{
        [self yz_removeAllHooks];
    }
}
@end
#endif

@implementation UIView (FSAdd)

- (CGFloat)fs_left {
    return self.frame.origin.x;
}

- (void)fs_setLeft:(CGFloat)x {
    CGRect frame = self.frame;
    frame.origin.x = x;
    self.frame = frame;
}

- (CGFloat)fs_top {
    return self.frame.origin.y;
}

- (void)fs_setTop:(CGFloat)y {
    CGRect frame = self.frame;
    frame.origin.y = y;
    self.frame = frame;
}

- (CGFloat)fs_right {
    return self.frame.origin.x + self.frame.size.width;
}

- (void)fs_setRight:(CGFloat)right {
    CGRect frame = self.frame;
    frame.origin.x = right - frame.size.width;
    self.frame = frame;
}

- (CGFloat)fs_bottom {
    return self.frame.origin.y + self.frame.size.height;
}

- (void)fs_setBottom:(CGFloat)bottom {
    CGRect frame = self.frame;
    frame.origin.y = bottom - frame.size.height;
    self.frame = frame;
}

- (CGFloat)fs_width {
    return self.frame.size.width;
}

- (void)fs_setWidth:(CGFloat)width {
    CGRect frame = self.frame;
    frame.size.width = width;
    self.frame = frame;
}

- (CGFloat)fs_height {
    return self.frame.size.height;
}

- (void)fs_setHeight:(CGFloat)height {
    CGRect frame = self.frame;
    frame.size.height = height;
    self.frame = frame;
}

- (CGFloat)fs_centerX {
    return self.center.x;
}

- (void)fs_setCenterX:(CGFloat)centerX {
    self.center = CGPointMake(centerX, self.center.y);
}

- (CGFloat)fs_centerY {
    return self.center.y;
}

- (void)fs_setCenterY:(CGFloat)centerY {
    self.center = CGPointMake(self.center.x, centerY);
}

- (CGPoint)fs_origin {
    return self.frame.origin;
}

- (void)fs_setOrigin:(CGPoint)origin {
    CGRect frame = self.frame;
    frame.origin = origin;
    self.frame = frame;
}

- (CGSize)fs_size {
    return self.frame.size;
}

- (void)fs_setSize:(CGSize)size {
    CGRect frame = self.frame;
    frame.size = size;
    self.frame = frame;
}

- (UIEdgeInsets)fs_safeAreaInsets{
    if (@available(iOS 11.0, *))return self.safeAreaInsets;
    return UIEdgeInsetsZero;
}

- (void)fs_setLayerCornerRadius:(CGFloat)cornerRadius{
    self.layer.cornerRadius = cornerRadius;
    self.clipsToBounds = YES;
}

- (void)fs_setLayerBorderWidth:(CGFloat)borderWidth borderColor:(UIColor *)borderColor{
    [self fs_setLayerBorderWidth:borderWidth borderColor:borderColor lineDashPattern:nil];
}

- (void)fs_setLayerBorderWidth:(CGFloat)borderWidth borderColor:(nullable UIColor *)borderColor lineDashPattern:(nullable NSArray *)lineDashPattern{
    CALayer *border;
    for (CALayer *_border in self.layer.sublayers) {
        if ([_border.name isEqualToString:@"fs_dashlayer"] && [_border isKindOfClass:[CAShapeLayer class]]) {
            border = _border;
            break;
        }
    }
    if (lineDashPattern) {
        CAShapeLayer *shapeLayer;
        if (border) shapeLayer = (CAShapeLayer *)border;
        else shapeLayer = [CAShapeLayer layer];
        shapeLayer.strokeColor = borderColor.CGColor;
        shapeLayer.fillColor = [UIColor clearColor].CGColor;
        shapeLayer.path = [UIBezierPath bezierPathWithRect:self.bounds].CGPath;
        shapeLayer.frame = self.bounds;
        shapeLayer.lineWidth = borderWidth;
        shapeLayer.lineDashPattern = lineDashPattern;
        shapeLayer.name = @"fs_dashlayer";
        [self.layer addSublayer:shapeLayer];
        self.layer.borderWidth = 0;
        self.layer.borderColor = [UIColor clearColor].CGColor;
    }else{
        if (border) [border removeFromSuperlayer];
        self.layer.borderWidth = borderWidth;
        self.layer.borderColor = borderColor.CGColor;
    }
}

- (void)fs_setShadowColor:(UIColor *)color{
    if (color) {
        [self fs_setShadowOffset:CGSizeMake(0, 0) opacity:0.5 radius:3 color:color];
    }else{
        [self fs_setShadowOffset:CGSizeMake(0, 0) opacity:0 radius:0 color:nil];
    }

}

- (void)fs_setShadowOffset:(CGSize)offset opacity:(CGFloat)opacity radius:(CGFloat)radius color:(UIColor *)color{
    self.layer.masksToBounds = NO;
    self.layer.shadowColor = color.CGColor;
    self.layer.shadowOffset = offset;
    self.layer.shadowOpacity = opacity;
    self.layer.shadowRadius = radius;
}


- (UITableView *)fs_tableView {
    for (UIView *view = self; view; view = view.superview) {
        UIResponder *nextResponder = [view nextResponder];
        if ([nextResponder isKindOfClass:[UITableView class]]) {
            return (UITableView *)nextResponder;
        }
    }
    return nil;
}

- (UIViewController *)fs_windowRootViewController{
    return self.window.rootViewController;
}

- (UIViewController *)fs_viewController {
    for (UIView *view = self; view; view = view.superview) {
        UIResponder *nextResponder = [view nextResponder];
        if ([nextResponder isKindOfClass:[UIViewController class]]) {
            return (UIViewController *)nextResponder;
        }
    }
    return nil;
}

- (void)fs_logSubviewWithLevel:(int)level {
    for (UIView *subview in self.subviews) {
        NSString *blank = @"";
        for (int i = 1; i < level; i++) {
            blank = [NSString stringWithFormat:@"  %@", blank];
        }
        FSCLog(@"%@%d: %@", blank, level, subview.class);
        [subview fs_logSubviewWithLevel:(level+1)];
    }
}


- (UIView *)fs_getSubviewWithClassName:(NSString *)className{
   UIView *res = nil;
   for (UIView *subview in self.subviews) {
        NSString *name = NSStringFromClass([subview class]);
        if ([className hasPrefix:name]) {
            return subview;
        }else{
            res = [subview fs_getSubviewWithClassName:className];
            if (res)  return res;
        }
    }
    return res;
}


- (__kindof UIView *)fs_getSuperviewOfClassName:(NSString *)className{
    UIView *view = self;
    while (view) {
        if ([view isKindOfClass:NSClassFromString(className)]) {
            break;
        }
        view = view.superview;
    }
    return view;
}

- (void)fs_removeSubviewWithClassName:(NSString *)className{
    [[self fs_getSubviewWithClassName:className] removeFromSuperview];
}

- (UIImage *)fs_snapshoot{
    CGSize shotSize = CGSizeMake(self.fs_width , self.fs_height);
    if (shotSize.width <= 0 || shotSize.height <= 0) return nil;
    CGFloat scale = [UIScreen mainScreen].scale;
    UIGraphicsBeginImageContextWithOptions(shotSize, NO, scale);
    
    if ([self respondsToSelector:@selector(drawViewHierarchyInRect:afterScreenUpdates:)]) {
        if (![self drawViewHierarchyInRect:self.bounds afterScreenUpdates:NO]) {
            goto layerRenderInContext;
        }
    } else {
        layerRenderInContext: {
            CGContextRef contextRef = UIGraphicsGetCurrentContext();
            [self.layer renderInContext:contextRef];
        }
    }

    UIImage *snapshoot  = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    
    return snapshoot;
}

- (id)fs_findFirstResponder{
    if (self.isFirstResponder) {
        return self;
    }
    for (UIView *subView in self.subviews) {
        id responder = [subView fs_findFirstResponder];
        if (responder) return responder;
    }
    return nil;
}

- (void)fs_resignOtherInputFirstResponderCurrent:(UIWindow *)window{
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
    if (@available(iOS 13.0, *)) {
        for (UIScene *scene in [[UIApplication sharedApplication] connectedScenes]) {
            UIWindow *keyWindow = [scene fs_currentSceneKeyWindow];
            UIResponder *responder = [keyWindow fs_findFirstResponder];
            
            if (responder == self || scene == [window windowScene]) {
                continue;
            }
            if ([responder conformsToProtocol:@protocol(UIKeyInput)]) {
                [responder resignFirstResponder];
            }
        }
    } else {
        // Fallback on earlier versions
    }
#endif
}



@end

#if __has_include(<MBProgressHUD/MBProgressHUD.h>) || __has_include("MBProgressHUD.h")
#if __has_include(<MBProgressHUD/MBProgressHUD.h>)
    #import <MBProgressHUD/MBProgressHUD.h>
#else
    #import "MBProgressHUD.h"
#endif

@interface _FSMBProgressHUD : MBProgressHUD @end

const NSUInteger kMBProgressHUDTag = 8686;
@implementation UIView (MBProgressHUD)

- (MBProgressHUD *)fs_MBProgressHUD{
    return [self viewWithTag:kMBProgressHUDTag];
}

- (MBProgressHUD *)fs_createMBProgressHUD{
    __block MBProgressHUD *progressHUD;
    if (dispatch_current_queue_equal_to(dispatch_get_main_queue())) {
        progressHUD = [_FSMBProgressHUD showHUDAddedTo:self animated:YES];
    }else{
        dispatch_semaphore_t wait = dispatch_semaphore_create(0);
        dispatch_async(dispatch_get_main_queue(), ^{
            progressHUD = [_FSMBProgressHUD showHUDAddedTo:self animated:YES];
            dispatch_semaphore_signal(wait);
        });
        dispatch_semaphore_wait(wait, DISPATCH_TIME_FOREVER);
    }
    return progressHUD;
}

- (MBProgressHUD *)fs_showHUDCustomView:(UIView * _Nullable)customView{
    return [self fs_showHUDCustomView:customView msg:nil];
}

- (MBProgressHUD *)fs_showHUDCustomView:(UIView * _Nullable)customView msg:(NSString * _Nullable)msg{
    return [self fs_showHUDCustomView:customView msg:msg process:nil];
}

- (MBProgressHUD *)fs_showHUDCustomView:(UIView * _Nullable)customView msg:(NSString * _Nullable)msg process:(_Nullable dispatch_block_t)process{
    [self fs_hideHUDLoading];
    MBProgressHUD *hud = [self fs_createMBProgressHUD];
    hud.tag = kMBProgressHUDTag;
    hud.customView = customView;
    hud.mode = MBProgressHUDModeCustomView;
    hud.detailsLabelText = msg;
    hud.removeFromSuperViewOnHide = YES;
    
    UITapGestureRecognizer *tap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(hideHUD)];
    [hud addGestureRecognizer:tap];
    return hud;
}

- (void)hideHUD{
    [self fs_hideHUDLoading];
}

- (void)fs_scheduledMainQueueShowHUDLoading:(NSString *)msg process:(dispatch_block_t)process {
    [self fs_showHUDLoading:msg process:process synch:NO onQueue:dispatch_get_main_queue()];
}

- (void)fs_showHUDLoading:(NSString *)msg process:(dispatch_block_t)process {
    [self fs_showHUDLoading:msg process:process synch:NO];
}

- (void)fs_showHUDLoading:(NSString *)msg process:(dispatch_block_t)process synch:(BOOL)synch{
    [self fs_showHUDLoading:msg process:process synch:synch onQueue:dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0)];
}

- (void)fs_showHUDLoading:(NSString *)msg process:(dispatch_block_t)process synch:(BOOL)synch onQueue:(dispatch_queue_t)queue {
    if (dispatch_current_queue_equal_to(queue)) {
        [self fs_showHUDLoading:msg];
        if (dispatch_current_queue_equal_to(dispatch_get_main_queue())) {
            dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.01 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                process();
                [self fs_hideHUDLoading];
            });
        }else{
            process();
            [self fs_hideHUDLoading];
        }
    } else {
        dispatch_semaphore_t wait;
        if (synch) wait = dispatch_semaphore_create(0);
        @try {
            [self fs_showHUDLoading:msg];
            dispatch_async(queue, ^{
                process();
                if (synch) dispatch_semaphore_signal(wait);
                [self fs_hideHUDLoading];
            });
            if (synch) dispatch_semaphore_wait(wait, DISPATCH_TIME_FOREVER);
        } @catch (NSException *exception) {
            if (synch) dispatch_semaphore_signal(wait);
            [self fs_hideHUDLoading];
            @throw exception;
        }
    }
}


- (MBProgressHUD *)fs_showHUDLoading:(NSString *)msg {
    __block MBProgressHUD *hud = [self viewWithTag:kMBProgressHUDTag];
    if (hud) {
        if (hud.mode != MBProgressHUDModeIndeterminate) [self fs_hideHUDLoading];
    }else{
        hud = [self fs_createMBProgressHUD];
        hud.mode = MBProgressHUDModeIndeterminate;
        hud.tag = kMBProgressHUDTag;
        hud.backgroundColor = [UIColor colorWithWhite:0 alpha:0.4];
        hud.removeFromSuperViewOnHide = YES;
    }
    hud.detailsLabelText = msg;
    return hud;
}

- (void)fs_hideHUDLoading {
    dispatch_main_async_safe(^{
        MBProgressHUD *hud = [self viewWithTag:kMBProgressHUDTag];
        if (hud) {
            [MBProgressHUD hideHUDForView:self animated:YES];
            [hud removeFromSuperview];
        }
    });
}

- (MBProgressHUD *)fs_showHUDMessage:(NSString *)msg{
    return [self fs_showHUDMessage:msg afterHideDelay:2];
}

- (MBProgressHUD *)fs_showHUDMessage:(NSString *)msg afterHideDelay:(NSTimeInterval)delay{
    [MBProgressHUD hideHUDForView:self animated:YES];
    MBProgressHUD *hud = [self fs_createMBProgressHUD];
    hud.color = [[UIColor blackColor] colorWithAlphaComponent:0.6];
    hud.mode = MBProgressHUDModeText;
    hud.detailsLabelText = msg;
    hud.detailsLabelColor = UIColor.whiteColor;
    hud.detailsLabelFont = [UIFont systemFontOfSize:15];
    hud.cornerRadius = 10;
    hud.removeFromSuperViewOnHide = YES;
    [hud hide:YES afterDelay:delay];
    return hud;
}

@end


@implementation _FSMBProgressHUD
#if _MAC_CATALYST_
- (void)hide:(BOOL)animated afterDelay:(NSTimeInterval)delay {
    FS_CLANG_IGNORED_SELECTOR_WARNING(
    if ([self respondsToSelector:@selector(hideDelayed:)]) {
        [self performSelector:@selector(hideDelayed:) withObject:[NSNumber numberWithBool:animated] afterDelay:delay inModes:@[NSRunLoopCommonModes]];
    })
}
#endif
@end

#endif


@implementation FSAdapterUIView
#pragma mark - UIViewAdapter
- (void)layoutSubviews{
    [super layoutSubviews];
    if (self.adapter && [self.adapter respondsToSelector:@selector(layoutSubviewsFromView:)]) {
        [self.adapter layoutSubviewsFromView:self];
    }
}

- (void)traitCollectionDidChange:(UITraitCollection *)previousTraitCollection{
    [super traitCollectionDidChange:previousTraitCollection];
    if (self.adapter && [self.adapter respondsToSelector:@selector(traitCollectionDidChange:fromView:)]) {
        [self.adapter traitCollectionDidChange:previousTraitCollection fromView:self];
    }
}
@end

@implementation FSAdapterUIScrollView
#pragma mark - UIViewAdapter
- (void)layoutSubviews{
    [super layoutSubviews];
    if (self.adapter && [self.adapter respondsToSelector:@selector(layoutSubviewsFromView:)]) {
        [self.adapter layoutSubviewsFromView:self];
    }
}

- (void)traitCollectionDidChange:(UITraitCollection *)previousTraitCollection{
    [super traitCollectionDidChange:previousTraitCollection];
    if (self.adapter && [self.adapter respondsToSelector:@selector(traitCollectionDidChange:fromView:)]) {
        [self.adapter traitCollectionDidChange:previousTraitCollection fromView:self];
    }
}
@end
