/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSCacheForImage.h"

@interface  FSCacheForImage ()
@property (nonatomic, strong) NSMutableDictionary <NSString *, UIImage *>*linkedImageMap;
@property (nonatomic, strong) NSMutableArray <NSString *>*linkedCacheKey;
@property (nonatomic, assign) NSUInteger maxCount;
@end
@implementation FSCacheForImage

- (instancetype)initWithMaxImageCount:(NSUInteger)count{
    self = [super init];
    if (self) {
        _maxCount = count;
        _linkedImageMap = [NSMutableDictionary dictionaryWithCapacity:count+1];
        _linkedCacheKey = [NSMutableArray arrayWithCapacity:count+1];
    }
    return self;
}

- (void)setImage:(UIImage *)image forKey:(NSString *)key{
    if (!image || !key) {
        FSCLog(@"image or key is nil");
        return;
    }
    if ([_linkedCacheKey containsObject:key]) {
        [_linkedCacheKey removeObject:key];
    }
    [_linkedCacheKey insertObject:key atIndex:0];
    [_linkedImageMap setValue:image forKey:key];
    
    if (_linkedCacheKey.count > _maxCount) [_linkedCacheKey removeLastObject];
}

- (UIImage *)getImageForKey:(NSString *)key{
    UIImage *image = [_linkedImageMap objectForKey:key];
    if (image) {
        [_linkedCacheKey removeObject:key];
        [_linkedCacheKey insertObject:key atIndex:0];
    }
    return image;
}

@end
