/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSFPSMonitor.h"
#import "FSConst.h"
#import "Masonry+FSAdd.h"
#import "NSObject+FSAdd.h"

#define kSize CGSizeMake(55, 20)

@interface FSFPSLabel : UILabel @end

@implementation FSFPSMonitor
static BOOL _hidden = NO;
static FSFPSLabel *_fpsLabel = nil;

+ (FSFPSLabel *)fpsLabel{
    if (!_fpsLabel) {
        FSFPSLabel *fpsLabel = [FSFPSLabel new];
        fpsLabel.layer.zPosition = 1.f;
        [[self fs_getForegroundAnyKeyWindow] addSubview:fpsLabel];
        if (fpsLabel.superview) {
            [fpsLabel  mas_makeConstraints:^(MASConstraintMaker *make) {
                make.right.mas_equalTo(fpsLabel.superview.fs_mas_right).offset(-ITEM_MARGIN_NORMAL);
                make.bottom.mas_equalTo(fpsLabel.superview.fs_mas_bottom).offset(-(ITEM_MARGIN_NORMAL + TAB_BAR_HEIGHT_NORMAL));
            }];
            _fpsLabel = fpsLabel;
        }
    }
    return _fpsLabel;
}

+ (void)setHidden:(BOOL)hidden{
    _hidden = hidden;
#if DEBUG
    self.fpsLabel.hidden = hidden;
#endif
}

@end

@implementation FSFPSLabel {
    CADisplayLink *_link;
    NSUInteger _count;
    NSTimeInterval _lastTime;
    UIFont *_font;
    UIFont *_subFont;
}

- (instancetype)initWithFrame:(CGRect)frame {
    if (frame.size.width == 0 && frame.size.height == 0) {
        frame.size = kSize;
    }
    self = [super initWithFrame:frame];
    
    self.layer.cornerRadius = 5;
    self.clipsToBounds = YES;
    self.textAlignment = NSTextAlignmentCenter;
    self.userInteractionEnabled = NO;
    self.backgroundColor = [UIColor colorWithWhite:0.000 alpha:0.700];
    
    _font = [UIFont fontWithName:@"Menlo" size:14];
    if (_font) {
        _subFont = [UIFont fontWithName:@"Menlo" size:4];
    } else {
        _font = [UIFont fontWithName:@"Courier" size:14];
        _subFont = [UIFont fontWithName:@"Courier" size:4];
    }
    
    _link = [CADisplayLink displayLinkWithTarget:self selector:@selector(tick:)];
    [_link addToRunLoop:[NSRunLoop mainRunLoop] forMode:NSRunLoopCommonModes];
    return self;
}

- (void)dealloc {
    [_link invalidate];
}

- (CGSize)sizeThatFits:(CGSize)size {
    return kSize;
}

- (void)tick:(CADisplayLink *)link {
    if (_lastTime == 0) {
        _lastTime = link.timestamp;
        return;
    }
    
    _count++;
    NSTimeInterval delta = link.timestamp - _lastTime;
    if (delta < 1) return;
    _lastTime = link.timestamp;
    float fps = _count / delta;
    _count = 0;
    
    CGFloat progress = fps / 60.0;
    UIColor *color = [UIColor colorWithHue:0.27 * (progress - 0.2) saturation:1 brightness:0.9 alpha:1];
    
    NSMutableAttributedString *text = [[NSMutableAttributedString alloc] initWithString:[NSString stringWithFormat:@"%d FPS",(int)round(fps)]];
    
    [text addAttributes:@{
        NSForegroundColorAttributeName:color
    } range:NSMakeRange(0, text.length - 3)];
    
    [text addAttributes:@{
        NSForegroundColorAttributeName:[UIColor whiteColor]
    } range:NSMakeRange(text.length - 3, 3)];
    
    [text addAttributes:@{
            NSFontAttributeName:_font,
    } range:NSMakeRange(0, text.length)];
    
    [text addAttributes:@{
            NSFontAttributeName:_subFont,
    } range:NSMakeRange(text.length - 4, 1)];
    
    self.attributedText = text;
}

@end
