/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/

#import <UIKit/UIKit.h>
#import <CoreGraphics/CGBase.h>


#ifndef FSMacro_h
#define FSMacro_h

#if defined(__cplusplus)
    #define FS_EXTERN extern "C"
#else
    #define FS_EXTERN extern
#endif

typedef CGFLOAT_TYPE FSFLOAT_TYPE;


#if DEBUG
#define FSCLog(format, ...) if ([FSPreferences sharePreferences].deubgDescriptionLog) {\
    fprintf(stderr, "%s\n",[[NSString stringWithFormat:format, ##__VA_ARGS__] UTF8String]);            \
}
#else
#define FSCLog(format, ...)
#endif

#if DEBUG
#define FSLog(format, ...) if ([FSPreferences sharePreferences].deubgDescriptionLog) {\
    fprintf(stderr, "-----------------------------------------\n");             \
    fprintf(stderr, "File: %s\nFunc: %s\nLine: %d / Time: [%s]\n",         \
    [[[NSString stringWithUTF8String:__FILE__] lastPathComponent] UTF8String],  \
    __func__, __LINE__, __TIME__);                                              \
    FSCLog(@"Format:\n %@",[NSString stringWithFormat:format, ##__VA_ARGS__])\
    fprintf(stderr, "-----------------------------------------\n\n");           \
}
#else
#define FSLog(format, ...)
#endif

#define _po(o) FSLog(@"%@", (o))
#define _pn(o) FSLog(@"%d", (o))
#define _pf(o) FSLog(@"%f", (o))
#define _ps(o) FSLog(@"CGSize: {%.0f, %.0f}", (o).width, (o).height)
#define _pr(o) FSLog(@"CGRect: {{%.0f, %.0f}, {%.0f, %.0f}}", (o).origin.x, (o).origin.y, (o).size.width, (o).size.height)
#define _mark FSLog(@"MARK: %s, %d", __PRETTY_FUNCTION__, __LINE__)

#define FSOBJ(obj)  FSLog(@"%s: %@", #obj, [(obj) description])

#define MARK FSCLog(@"MARK: %s %d", __PRETTY_FUNCTION__ , __LINE__)

#define NSStringFormat(format,...) [NSString stringWithFormat:format,##__VA_ARGS__]


///--------------
/// SystemVersion
///--------------

#ifndef SystemVersion
#define SystemVersion [[UIDevice currentDevice] systemVersion].floatValue
#endif

#ifndef iOS10Later
#define iOS10Later (SystemVersion >= 10)
#endif

#ifndef iOS11Later
#define iOS11Later (SystemVersion >= 11)
#endif

#ifndef iOS12Later
#define iOS12Later (SystemVersion >= 12)
#endif

#ifndef iOS13Later
#define iOS13Later (SystemVersion >= 13)
#endif

#ifndef iOS14Later
#define iOS14Later (SystemVersion >= 14)
#endif

//compile macro
#ifndef __IPHONE_10_0
#define __IPHONE_10_0 100000
#endif

#ifndef __IPHONE_11_0
#define __IPHONE_11_0 110000
#endif

#ifndef __IPHONE_12_0
#define __IPHONE_12_0 120000
#endif

#ifndef __IPHONE_12_1
#define __IPHONE_12_1 120100
#endif

#ifndef __IPHONE_13_0
#define __IPHONE_13_0 130000
#endif

#ifndef __IPHONE_14_0
#define __IPHONE_14_0 140000
#endif

#ifndef __IPHONE_15_0
#define __IPHONE_15_0 150000
#endif

#ifndef __IPHONE_16_0
#define __IPHONE_16_0 160000
#endif

#import <TargetConditionals.h>
#if defined(TARGET_OS_IPHONE) && defined(TARGET_OS_UIKITFORMAC)
    #if !TARGET_OS_UIKITFORMAC && TARGET_OS_IPHONE
        #define _MAC_CATALYST_ 0
    #else
        #define _MAC_CATALYST_ 1
    #endif
#else
    #define _MAC_CATALYST_ 0
#endif

#ifndef DEVICE_iPHONE
#define DEVICE_iPHONE ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone)
#endif
#ifndef DEVICE_iPAD
#define DEVICE_iPAD   ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad)
#endif
#ifndef DEVICE_MAC_CATALYST
#define DEVICE_MAC_CATALYST  _MAC_CATALYST_
#endif

#if __IPHONE_OS_VERSION_MAX_ALLOWED < __IPHONE_13_0
#ifndef FSDEVICE_SIZECLASS
    #define FSDEVICE_SIZECLASS ([self fs_currentSceneKeyWindow].traitCollection.horizontalSizeClass)
    #define FSHORIZONTAL_COMPACT (FSDEVICE_SIZECLASS == UIUserInterfaceSizeClassCompact)
    #define FSDEVICE_iPAD_HORIZONTAL_COMPACT (FSDEVICE_SIZECLASS == UIUserInterfaceSizeClassCompact && DEVICE_iPAD)

    #ifndef FSSCREEN_IS_COMPACT
        #define FSSCREEN_IS_COMPACT (DEVICE_iPHONE || FSDEVICE_iPAD_HORIZONTAL_COMPACT)
    #endif
#endif

#define FS_TRANSFER_WINDOW_SCENE(a, b)
#else
#define FS_TRANSFER_WINDOW_SCENE(a, b) \
        if (@available(iOS 13.0, *)) { \
            a.fs_windowScene = b.fs_windowScene; \
        }
#endif

///----------------------
/// Persistence Directory
///----------------------
#define DOCUMENT_PATH NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES).firstObject
#define CACHES_PATH NSSearchPathForDirectoriesInDomains(NSCachesDirectory, NSUserDomainMask, YES).firstObject
#define TEMP_PATH NSTemporaryDirectory()
#define LIBRARY_PATH [NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES) objectAtIndex:0]


///-----------------
/// Screen, Width and Height
///-----------------

#define SCREEN_WIDTH  ([UIScreen mainScreen].bounds.size.width)
#define SCREEN_HEIGHT ([UIScreen mainScreen].bounds.size.height)

#define STYLE_CELLWIDTH_IPHONE (SCREEN_WIDTH < SCREEN_HEIGHT ? SCREEN_WIDTH : SCREEN_HEIGHT)
#define STYLE_CELLHEIHGT_IPHONE (SCREEN_WIDTH > SCREEN_HEIGHT ? SCREEN_WIDTH : SCREEN_HEIGHT)

#ifndef iPhone5
#define iPhone5 ([UIScreen instancesRespondToSelector:@selector(currentMode)] ? CGSizeEqualToSize(CGSizeMake(640, 1136), [[UIScreen mainScreen] currentMode].size) : NO)
#endif

#ifndef iPhone6 //equal to iPhone7,8
#define iPhone6 ([UIScreen instancesRespondToSelector:@selector(currentMode)] ? (CGSizeEqualToSize(CGSizeMake(750, 1334), [[UIScreen mainScreen] currentMode].size)) : NO)
#endif

#ifndef iPhone6plus //equal to iPhone7,8plus
#define iPhone6plus ([UIScreen instancesRespondToSelector:@selector(currentMode)] ? (CGSizeEqualToSize(CGSizeMake(1125, 2001), [[UIScreen mainScreen] currentMode].size) || CGSizeEqualToSize(CGSizeMake(1242, 2208), [[UIScreen mainScreen] currentMode].size)) : NO)
#endif

#ifndef iPhone6plusLarge
#define iPhone6plusLarge ([UIScreen instancesRespondToSelector:@selector(currentMode)] ? CGSizeEqualToSize(CGSizeMake(1125, 2001), [[UIScreen mainScreen] currentMode].size) : NO)
#endif

#ifndef iPhoneX //equal to iPhoneXS
#define iPhoneX ([UIScreen instancesRespondToSelector:@selector(currentMode)] ? (CGSizeEqualToSize(CGSizeMake(1125, 2436), [[UIScreen mainScreen] currentMode].size) && !DEVICE_iPAD) : NO)
#endif

#ifndef iPhoneXR
#define iPhoneXR ([UIScreen instancesRespondToSelector:@selector(currentMode)] ? (CGSizeEqualToSize(CGSizeMake(828, 1792), [[UIScreen mainScreen] currentMode].size) && !DEVICE_iPAD) : NO)
#endif

#ifndef iPhoneXMax
#define iPhoneXMax ([UIScreen instancesRespondToSelector:@selector(currentMode)] ? (CGSizeEqualToSize(CGSizeMake(1242, 2688), [[UIScreen mainScreen] currentMode].size) && !DEVICE_iPAD) : NO)
#endif

#ifndef isiPhoneXSet
#define isiPhoneXSet (iPhoneX == YES || iPhoneXR == YES || iPhoneXMax == YES)
#endif

#ifndef STATUS_BAR_HEIGHT //44.f or 20.f
#define STATUS_BAR_HEIGHT (isiPhoneXSet ? 44.f : STATUS_BAR_HEIGHT_NORMAL)
#endif

#ifndef NAVGATION_BAR_HEIGHT //44.f or 44.f
#define NAVGATION_BAR_HEIGHT (isiPhoneXSet ? 44.f : NAVIGATION_BAR_HEIGHT_NORMAL())
#endif

#ifndef TAB_BAR_SAFE_BOTTOM_MARGIN //
#define TAB_BAR_SAFE_BOTTOM_MARGIN (isiPhoneXSet ? 34.f  : 0)
#endif

#ifndef NAVIGATION_HEIGHT //88.f or 64.f
#define NAVIGATION_HEIGHT (STATUS_BAR_HEIGHT + NAVGATION_BAR_HEIGHT)
#endif

#ifndef TAB_BAR_HEIGHT //83.f or 49.f
#define TAB_BAR_HEIGHT (TAB_BAR_SAFE_BOTTOM_MARGIN + TAB_BAR_HEIGHT_NORMAL)
#endif

#ifndef FSImageNamed
    #define FSImageNamed(name) ({\
    UIImage.fs_bundleForClass = [self class];\
    [UIImage fs_imageNamed:name];})
#endif

#ifndef FSImageNamedAndColor
    #define FSImageNamedAndColor(name, color) ({\
    UIImage.fs_bundleForClass = [self class];\
    NSString *key = [NSString stringWithFormat:@"%p_%@_%d_%d",UIImage.fs_bundleForClass,name,color.fs_lightColor.fs_rgbValue,color.fs_darkColor.fs_rgbValue];\
    UIImage *image = [[FSPreferences sharePreferences].imageCache getImageForKey:key];\
    if (!image){\
        image = [UIImage fs_imageNamed:name tintColor:color];\
        [[FSPreferences sharePreferences].imageCache setImage:image forKey:key];\
    }\
    image;})
#endif

#ifndef FSImageNamedAndCustomColor
    #define FSImageNamedAndCustomColor(name, color) ({\
    UIImage.fs_bundleForClass = [self class];\
    UIImage.fs_customImageColorSuffix = @"_color_";\
    NSString *key = [NSString stringWithFormat:@"%p_%@_%@_%lu",UIImage.fs_bundleForClass,name,UIImage.fs_customImageColorSuffix, color ? color.fs_rgbValue : NSUIntegerMax];\
    UIImage *image = [[FSPreferences sharePreferences].imageCache getImageForKey:key];\
    if (!image){\
        NSObject *weakObj = [NSObject new];\
        image = [UIImage fs_imageNamed:name tintColor:weakObj.fs_iconColor_I3 customColor:color];\
        [[FSPreferences sharePreferences].imageCache setImage:image forKey:key];\
    }\
    image;})
#endif

#ifndef FSImageNamedAndPrimaryColor
    #define FSImageNamedAndPrimaryColor(name) ({\
        NSObject *weakObj = [NSObject new];\
        FSImageNamedAndColor(name, weakObj.fs_primaryColor);})
#endif

#ifndef FSImageNamedAndNormalColor
    #define FSImageNamedAndNormalColor(name) ({\
        NSObject *weakObj = [NSObject new];\
        FSImageNamedAndColor(name, weakObj.fs_iconColor_I3);})
#endif



///------------------
/// Speedy Definitive
///------------------
#ifndef FSSYNTH_INTERFACE_IMP_CLASS
#define FSSYNTH_INTERFACE_IMP_CLASS(_name_, _class_) \
@interface _name_ : _class_ @end \
@implementation _name_ @end
#endif

///------------
/// static inline
///------------


NS_INLINE UIResponder<UIApplicationDelegate> * fs_appDelegate() {
    id<UIApplicationDelegate> appDelegate = [UIApplication sharedApplication].delegate;
    return (UIResponder<UIApplicationDelegate> *)appDelegate;
}

NS_INLINE BOOL isLandscape() {
    return [UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationMaskLandscape;
}

NS_INLINE CGFloat MIN_WIDTH_IN_VIEW(UIView *view) {
    return MIN(view.frame.size.width, view.frame.size.height);
}

NS_INLINE BOOL dispatch_current_queue_equal_to(dispatch_queue_t queue) {
    return strcmp(dispatch_queue_get_label(DISPATCH_CURRENT_QUEUE_LABEL), dispatch_queue_get_label(queue))==0;
}

NS_INLINE void dispatch_main_async_safe(dispatch_block_t block) {
    if (dispatch_current_queue_equal_to(dispatch_get_main_queue())) {
        block();
    } else {
        dispatch_async(dispatch_get_main_queue(), block);
    }
}

NS_INLINE NSString *CREATE_DIR_IF_NOT_PATH_AND_APPENDING_PATH(NSString *scrPath, NSString *appendingPath) {
    NSFileManager *defaultManager = [NSFileManager defaultManager];
    NSString *dstPath = [scrPath stringByAppendingPathComponent:appendingPath];
    if (![defaultManager fileExistsAtPath:dstPath]) {
        NSError *error;
        [defaultManager createDirectoryAtPath:dstPath withIntermediateDirectories:YES attributes:nil error:&error];
        if (error) NSLog(@"%@", error);
    }
    return dstPath;
}

NS_INLINE BOOL CGRectInvalid(CGRect rect) {
    return CGRectIsInfinite(rect) || CGRectIsNull(rect) || CGRectIsEmpty(rect);
}

NS_INLINE CGRect CGRectMake1(CGPoint origin, CGSize size) {
    return CGRectMake(origin.x, origin.y, size.width, size.height);
}

NS_INLINE CGRect CGRectMake2(CGPoint center, CGSize size) {
    return CGRectMake(center.x - size.width/2, center.y - size.height/2, size.width, size.height);
}

#ifndef FS_APP_VERSION
#define FS_APP_VERSION [[[NSBundle mainBundle] infoDictionary] objectForKey:@"CFBundleShortVersionString"];
#endif

#define FSSharedAppDelegate fs_appDelegate()

#define HAS_RAC_CMETA_MACROS __has_include(<ReactiveObjC/RACmetamacros.h>) || __has_include("RACmetamacros.h")
#if !HAS_RAC_CMETA_MACROS

#define keypath(OBJ, PATH) \
    (((void)(NO && ((void)OBJ.PATH, NO)), # PATH))

#endif

#define keyselector(OBJ, sel) \
    (((void)(NO && ((void)[OBJ respondsToSelector:@selector(sel)], NO)), # sel))

#define HAS_YYKIT_MACROS __has_include(<YYKit/YYKitMacro.h>) || __has_include("YYKitMacro.h")
#if !HAS_YYKIT_MACROS
#ifndef YYSYNTH_DYNAMIC_PROPERTY_OBJECT
#define YYSYNTH_DYNAMIC_PROPERTY_OBJECT(_getter_, _setter_, _association_, _type_) \
- (void)_setter_ : (_type_)object { \
    [self willChangeValueForKey:@#_getter_]; \
    objc_setAssociatedObject(self, _cmd, object, OBJC_ASSOCIATION_ ## _association_); \
    [self didChangeValueForKey:@#_getter_]; \
} \
- (_type_)_getter_ { \
    return objc_getAssociatedObject(self, @selector(_setter_:)); \
}
#endif
#endif

#if !HAS_YYKIT_MACROS
#ifndef fs_weakify
    #if DEBUG
        #if __has_feature(objc_arc)
        #define fs_weakify(object) autoreleasepool{} __weak __typeof__(object) weak##_##object = object;
        #else
        #define fs_weakify(object) autoreleasepool{} __block __typeof__(object) block##_##object = object;
        #endif
    #else
        #if __has_feature(objc_arc)
        #define fs_weakify(object) try{} @finally{} {} __weak __typeof__(object) weak##_##object = object;
        #else
        #define fs_weakify(object) try{} @finally{} {} __block __typeof__(object) block##_##object = object;
        #endif
    #endif
#endif

#ifndef fs_strongify
    #if DEBUG
        #if __has_feature(objc_arc)
        #define fs_strongify(object) autoreleasepool{} __typeof__(object) object = weak##_##object;
        #else
        #define fs_strongify(object) autoreleasepool{} __typeof__(object) object = block##_##object;
        #endif
    #else
        #if __has_feature(objc_arc)
        #define fs_strongify(object) try{} @finally{} __typeof__(object) object = weak##_##object;
        #else
        #define fs_strongify(object) try{} @finally{} __typeof__(object) object = block##_##object;
        #endif
    #endif
#endif
#else
#define fs_weakify weakify
#define fs_strongify strongify
#endif


#define FS_CLANG_IGNORED_WARNING(ignored , ...) \
_Pragma("clang diagnostic push") \
_Pragma(#ignored) \
__VA_ARGS__; \
_Pragma("clang diagnostic pop") \

#define FS_CLANG_IGNORED_SELECTOR_WARNING(...) FS_CLANG_IGNORED_WARNING(clang diagnostic ignored "-Wundeclared-selector",##__VA_ARGS__)
#endif /* YZMacro_h */

