/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/

#import "FSPreferences.h"
#if __has_include(<YZHookHandler/YZHookHandler.h>)
#import <YZHookHandler/YZHookHandler.h>
#else
#import "YZHookHandler.h"
#endif
#import "FSMacro.h"
#import "NSObject+FSAdd.h"
#import "UIColor+FSAdd.h"
#import "FSCacheForImage.h"

@interface FSPreferences ()
- (void)_setPrimaryColor:(UIColor *)primaryColor;
- (void)_setStateBarColor:(UIColor *)stateBarColor;

@end
@implementation FSPreferences

+ (void)load{
    [super load];
    
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        YZHookHandler.yz_hookHandlerForSelectorAliasPrefix = @"foxit_rdk_alias_";
        YZHookHandler.yz_subclassSuffix = @"_FoxitRDK_FSFoundation";
    });
}

static FSPreferences *_preferences = nil;
+ (FSPreferences *)sharePreferences{
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        _preferences = [[self alloc] init];
    });
    return _preferences;
}

#define UIFontSystemOfSize(size) [UIFont systemFontOfSize:size]
#define UIFontBoldSystemOfSize(size) [UIFont boldSystemFontOfSize:size]

- (instancetype)init
{
    self = [super init];
    if (self) {
        _deubgDescriptionLog = NO;
    #ifdef DEBUG
        _deubgDescriptionLog = YES;
    #endif
        
        _primaryColor = UIColorHex_DarkMode(#A236B2, #C977D6);
        _secondaryColor = UIColorHex(#FEC0FF);
        
        _stateBarColor = UIColorHex_DarkMode(#A236B2, #2D343D);

        _backgroundColor_B1 = UIColorHex_DarkMode(#FFFFFF, #21282D);
        _backgroundColor_B2 = UIColorHex_DarkMode(#F4F4F4, #2D343D);
        _backgroundColor_B3 = UIColorHex(#D8D8D8);
        _backgroundColor_B4 = UIColorHex_DarkMode(#D8D8D8, #36404A);
        _backgroundColor_B5 = UIColorHex(#2E2E2E);
        _backgroundColor_B6 = UIColorHex_DarkMode(#D8D8D8, #21282D);
        _backgroundColor_B7 = UIColorHex_DarkMode(#D8D8D8, #2D343D);
        _backgroundColor_B8 = UIColorHex_DarkMode(#FFFFFF, #2D343D);
        _backgroundColor_B9 = UIColorHex_DarkMode(#F4F4F4, #21282D);
        _backgroundColor_B10 = UIColorHex_DarkMode(#F4F4F4, #36404A);
        _backgroundColor_B11 = UIColorHex_DarkMode(#FFFFFF, #8C8C8C);
        _backgroundColor_B12 = UIColorHex_DarkMode(#A236B2, #21282D);
        _backgroundColor_B13 = UIColorHex_DarkMode(#F4F4F4, #C977D6);
        _backgroundColor_B14 = UIColorHex_DarkMode(#186DE2, #84B8FF);
        _backgroundColor_B15 = UIColorHex_DarkMode(#FFFFFF, #2D343D);

        _dividerColor_P1 = UIColorHex_DarkMode(#E8E8E8, #36404A);
        _dividerColor_P2 = UIColorHex_DarkMode(#E2E2E2, #545454);
        _dividerColor_P3 = UIColorHex_DarkMode(#A9A9A9, #808080);
        _dividerColor_P4 = UIColorHex(#179cd8);

        _textColor_T1 = UIColorHex(#FFFFFF);
        _textColor_T2 = UIColorHex_DarkMode(#A9A9A9, #808080);
        _textColor_T3 = UIColorHex_DarkMode(#6E6E6E, #9C9C9C);
        _textColor_T4 = UIColorHex_DarkMode(#2E2E2E, #EAEAEA);
        _textColor_T5 = UIColorHex_DarkMode(#E10200, #FF706F);
        _textColor_T6 = UIColorHex(#6084D0);
        _textColor_T7 = UIColorHex(#A7A7A7);
        _textColor_T8 = UIColorHex_DarkMode(#FFFFFF, #EAEAEA);
        _textColor_T9 = UIColorHex_DarkMode(#A7A7A7, #777777);
        _textColor_T10 = UIColorHex_DarkMode(#A236B2, #21282D);
        _textColor_T11 = UIColorHex_DarkMode(#FFFFFF, #EAEAEA);
        _textColor_T12 = UIColorHex_DarkMode(#186DE2, #84B8FF);
        _textColor_T13 = UIColorHex_DarkMode(#777777, #8C8C8C);
        
        _iconColor_I1 = UIColorHex(#FFFFFF);
        _iconColor_I2 = UIColorHex_DarkMode(#A7A7A7, #777777);
        _iconColor_I3 = UIColorHex_DarkMode(#6E6E6E, #A2A2A2);
        _iconDarkColor_I3 = UIColorHex(#A2A2A2);
        _iconColor_I4 = UIColorHex(#2CA7E4);
        _iconColor_I5 = UIColorHex_DarkMode(#A9A9A9, #808080);
        _iconColor_I6 = UIColorHex_DarkMode(#FFFFFF, #A2A2A2);
        _iconColor_I7 = _backgroundColor_B13;
        
        _stamp_signautre_color = UIColorHex(#DD5350);
        
        _linkColor_L1 = UIColorHex_DarkMode(#1962ED, #99EBFF);

        _standardImportantFont_IF1 = UIFontBoldSystemOfSize(34.f);
        _standardImportantFont_IF2 = UIFontBoldSystemOfSize(20.f);
        _standardImportantFont_IF3 = UIFontBoldSystemOfSize(18.f);
        
        _standardGeneralFont_GF1 = UIFontBoldSystemOfSize(16.f);
        _standardGeneralFont_GF2 = UIFontSystemOfSize(15.f);
        _standardGeneralFont_GF3 = UIFontSystemOfSize(14.f);

        _standardWeakFont_WF1 = UIFontSystemOfSize(12.f);
        _standardWeakFont_WF2 = UIFontSystemOfSize(10.f);
        _standardWeakFont_WF3 = UIFontSystemOfSize(7.f);
        
        _navBarBackTitle = @"< Back";
        
        _imageCache = [[FSCacheForImage alloc] initWithMaxImageCount:10];
    }
    return self;
}

- (void)setPrimaryColor:(UIColor *)primaryColor{
    self.backgroundColor_B12 =  UIColor_DarkMode(primaryColor.fs_lightColor,self.backgroundColor_B12.fs_darkColor);
    self.iconColor_I7 =  UIColor_DarkMode(self.iconColor_I7.fs_lightColor, primaryColor.fs_darkColor);
    self.backgroundColor_B13 =  UIColor_DarkMode(self.backgroundColor_B13.fs_lightColor, primaryColor.fs_darkColor);
    self.textColor_T10 =  UIColor_DarkMode(primaryColor.fs_lightColor, self.textColor_T10.fs_darkColor);
    if(self.stateBarColor){
        self.stateBarColor = UIColor_DarkMode(primaryColor.fs_lightColor,self.stateBarColor.fs_darkColor);
    }
    self.fs_primaryColor = primaryColor;
}

- (void)_setPrimaryColor:(UIColor *)primaryColor{
    _primaryColor = primaryColor;
}

- (void)setStateBarColor:(UIColor *)stateBarColor{
    self.fs_stateBarColor = stateBarColor;
}

- (void)_setStateBarColor:(UIColor *)stateBarColor{
    _stateBarColor = stateBarColor;
}

- (void)setPreferredStatusBarStyle:(UIStatusBarStyle)preferredStatusBarStyle{
    _preferredStatusBarStyle = preferredStatusBarStyle;
    for (UIWindow *window in [UIApplication sharedApplication].windows) {
        [window.rootViewController setNeedsStatusBarAppearanceUpdate];
    }
}

@end

static NSMapTable *_bindedObjs;

@interface NSObject (Theme)

@property (nonatomic, copy) UpdateTheme fs_completed;

@property (nonatomic, copy, setter=fs_setLastBindedKey:) NSString *fs_lastBindedKey;

@property (nonatomic, strong, setter=fs_setBindedKeys:) NSMutableSet<NSString *> *fs_bindedKeys;

@end

@implementation NSObject(Theme)

#ifndef FSSYNTH_THEME_GETTER
#define FSSYNTH_THEME_GETTER(_getter_, _type_) \
- (_type_)fs_##_getter_ { \
    return [FSPreferences sharePreferences]._getter_;\
}
#endif

FSSYNTH_THEME_GETTER(primaryColor, UIColor *);
FSSYNTH_THEME_GETTER(secondaryColor, UIColor *);
FSSYNTH_THEME_GETTER(stateBarColor, UIColor *);

FSSYNTH_THEME_GETTER(backgroundColor_B1, UIColor *);
FSSYNTH_THEME_GETTER(backgroundColor_B2, UIColor *);
FSSYNTH_THEME_GETTER(backgroundColor_B3, UIColor *);
FSSYNTH_THEME_GETTER(backgroundColor_B4, UIColor *);
FSSYNTH_THEME_GETTER(backgroundColor_B5, UIColor *);
FSSYNTH_THEME_GETTER(backgroundColor_B6, UIColor *);
FSSYNTH_THEME_GETTER(backgroundColor_B7, UIColor *);
FSSYNTH_THEME_GETTER(backgroundColor_B8, UIColor *);
FSSYNTH_THEME_GETTER(backgroundColor_B9, UIColor *);
FSSYNTH_THEME_GETTER(backgroundColor_B10, UIColor *);
FSSYNTH_THEME_GETTER(backgroundColor_B11, UIColor *);
FSSYNTH_THEME_GETTER(backgroundColor_B12, UIColor *);
FSSYNTH_THEME_GETTER(backgroundColor_B13, UIColor *);
FSSYNTH_THEME_GETTER(backgroundColor_B14, UIColor *);
FSSYNTH_THEME_GETTER(backgroundColor_B15, UIColor *);

FSSYNTH_THEME_GETTER(dividerColor_P1, UIColor *);
FSSYNTH_THEME_GETTER(dividerColor_P2, UIColor *);
FSSYNTH_THEME_GETTER(dividerColor_P3, UIColor *);
FSSYNTH_THEME_GETTER(dividerColor_P4, UIColor *);

FSSYNTH_THEME_GETTER(textColor_T1, UIColor *);
FSSYNTH_THEME_GETTER(textColor_T2, UIColor *);
FSSYNTH_THEME_GETTER(textColor_T3, UIColor *);
FSSYNTH_THEME_GETTER(textColor_T4, UIColor *);
FSSYNTH_THEME_GETTER(textColor_T5, UIColor *);
FSSYNTH_THEME_GETTER(textColor_T6, UIColor *);
FSSYNTH_THEME_GETTER(textColor_T7, UIColor *);
FSSYNTH_THEME_GETTER(textColor_T8, UIColor *);
FSSYNTH_THEME_GETTER(textColor_T9, UIColor *);
FSSYNTH_THEME_GETTER(textColor_T10, UIColor *);
FSSYNTH_THEME_GETTER(textColor_T11, UIColor *);
FSSYNTH_THEME_GETTER(textColor_T12, UIColor *);
FSSYNTH_THEME_GETTER(textColor_T13, UIColor *);

FSSYNTH_THEME_GETTER(iconColor_I1, UIColor *);
FSSYNTH_THEME_GETTER(iconColor_I2, UIColor *);
FSSYNTH_THEME_GETTER(iconColor_I3, UIColor *);
FSSYNTH_THEME_GETTER(iconDarkColor_I3, UIColor *);
FSSYNTH_THEME_GETTER(iconColor_I4, UIColor *);
FSSYNTH_THEME_GETTER(iconColor_I5, UIColor *);
FSSYNTH_THEME_GETTER(iconColor_I6, UIColor *);
FSSYNTH_THEME_GETTER(iconColor_I7, UIColor *);

FSSYNTH_THEME_GETTER(linkColor_L1, UIColor *);

FSSYNTH_THEME_GETTER(standardImportantFont_IF1, UIFont *);
FSSYNTH_THEME_GETTER(standardImportantFont_IF2, UIFont *);
FSSYNTH_THEME_GETTER(standardImportantFont_IF3, UIFont *);

FSSYNTH_THEME_GETTER(standardGeneralFont_GF1, UIFont *);
FSSYNTH_THEME_GETTER(standardGeneralFont_GF2, UIFont *);
FSSYNTH_THEME_GETTER(standardGeneralFont_GF3, UIFont *);

FSSYNTH_THEME_GETTER(standardWeakFont_WF1, UIFont *);
FSSYNTH_THEME_GETTER(standardWeakFont_WF2, UIFont *);
FSSYNTH_THEME_GETTER(standardWeakFont_WF3, UIFont *);

FSSYNTH_THEME_GETTER(navBarBackIcon, UIImage *);
FSSYNTH_THEME_GETTER(navBarCloseIcon, UIImage *);
FSSYNTH_THEME_GETTER(navBarBackTitle, NSString *);

#undef FSSYNTH_THEME_GETTER
static NSString *const kfs_bind_theme_esel_prefix = @"kfs_bind_theme_esel_prefix";
- (NSMapTable *)bindedObjs{
    if (!_bindedObjs) {
        _bindedObjs = [NSMapTable mapTableWithKeyOptions:NSMapTableCopyIn valueOptions:NSMapTableWeakMemory];
    }
    return _bindedObjs;
}

- (NSMutableSet *)fs_bindedKeys{
    NSMutableSet *fs_bindedKeys = [self fs_getAssociatedForKey:@keypath(self, fs_bindedKeys)];
    if(!fs_bindedKeys){
        fs_bindedKeys = [NSMutableSet set];
        [self fs_addAssociatedObject:fs_bindedKeys forKey:@keypath(self,fs_bindedKeys)];
    }
    return fs_bindedKeys;
}

- (UpdateTheme)fs_completed{
    return [self fs_getAssociatedForKey:@keypath(self, fs_completed)];
}

- (NSString *)fs_lastBindedKey{
    return [self fs_getAssociatedForKey:@keypath(self, fs_lastBindedKey)];
}

- (void)fs_setLastBindedKey:(NSString *)fs_lastBindedKey{
    [self fs_addCopyAssociatedObject:fs_lastBindedKey forKey:@keypath(self, fs_lastBindedKey)];
}

- (void)fs_setPrimaryColor:(UIColor *)fs_primaryColor{
    [[FSPreferences sharePreferences] _setPrimaryColor:fs_primaryColor];
    @synchronized (self) {
        for ( NSObject *bindedObj in [[self.bindedObjs objectEnumerator] allObjects]) {
            for (NSString *key in bindedObj.fs_bindedKeys) {
                NSString *selString = key;
                if(![key hasPrefix:kfs_bind_theme_esel_prefix]){
                    selString = [key stringByReplacingOccurrencesOfString:kfs_bind_theme_esel_prefix withString:@""];
                    [bindedObj setValue:fs_primaryColor forKey:selString];
                }
                !bindedObj.fs_completed ? : bindedObj.fs_completed(bindedObj, NSSelectorFromString(selString));
            }
        }
    }
}
- (void)fs_setStateBarColor:(UIColor *)fs_stateBarColor{
    [[FSPreferences sharePreferences] _setStateBarColor:fs_stateBarColor];
    
}

- (void)fs_setNavBarBackIcon:(UIImage *)fs_navBarBackIcon{
    [FSPreferences sharePreferences].navBarBackIcon = fs_navBarBackIcon;
}

- (void)fs_setNavBarCloseIcon:(UIImage *)fs_navBarCloseIcon{
    [FSPreferences sharePreferences].navBarCloseIcon = fs_navBarCloseIcon;
}

- (id)fs_bindThemeWithKey:(NSString *)key {
    [self.bindedObjs setObject:self forKey:[NSString stringWithFormat:@"%p",self]];
    [self.fs_bindedKeys addObject:key];
    self.fs_lastBindedKey = key;
    return self;
}

- (id)fs_bindThemeWithKeyPath:(NSString *)keyPath{
    [self setValue:self.fs_primaryColor forKey:keyPath];
    return [self fs_bindThemeWithKey:keyPath];
}

- (void)fs_stateBarThemeWithKeyPath:(NSString *)keyPath{
    UIColor * stateColor = self.fs_stateBarColor ? self.fs_stateBarColor : self.fs_primaryColor;
    [self setValue:stateColor forKey:keyPath];
}
- (id)fs_bindThemeWithKeySel:(NSString *)keySel{
    keySel = [NSString stringWithFormat:@"%@%@",kfs_bind_theme_esel_prefix,keySel];
    return [self fs_bindThemeWithKey:keySel];
}

- (void)fs_cancelBindThemeWithKey:(NSString *)key{
   
    if ([self.fs_bindedKeys containsObject:key]){
        [self.fs_bindedKeys removeObject:key];
    }else{
        key = [NSString stringWithFormat:@"%@%@",kfs_bind_theme_esel_prefix,key];
        if ([self.fs_bindedKeys containsObject:key]) {
            [self.fs_bindedKeys removeObject:key];
        }
    }
    if (!self.fs_bindedKeys.count) {
        [self.bindedObjs removeObjectForKey:[NSString stringWithFormat:@"%p",self]];
    }
}

- (void)fs_updateThemeCompleted:(UpdateTheme)completed{
    NSAssert(completed, @"completed cannot be null");
    [self fs_addAssociatedObject:completed forKey:@keypath(self,fs_completed)];
    if (self.fs_lastBindedKey && self.fs_completed) {
        NSString *selString =  [self.fs_lastBindedKey stringByReplacingOccurrencesOfString:kfs_bind_theme_esel_prefix withString:@""];
        self.fs_completed(self, NSSelectorFromString(selString));
    }
}

@end
