/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

/** @brief History data sort by key.*/
typedef NS_ENUM(NSUInteger, FSHistoryDataSortKey) {
    ///According to the default key
    FSHistoryDataSortKeyDefault,
    ///According to the date
    FSHistoryDataSortKeyDate,
    ///According to the char
    FSHistoryDataSortKeyChar
};

/** @brief History data  match rules*/
typedef NS_ENUM(NSUInteger, FSHistoryDataSortRule) {
    ///Contains chars
    FSHistoryDataMatchNormal = 0x00,
    ///Case sensitive
    FSHistoryDataMatchCase = 0x01,
    ///consecutive chars
    FSHistoryDataMatcConsecutive = 0x02
};

/** @brief History data model object*/
@interface FSHistoryDataItem : NSObject<NSCoding>
///Data creation date
@property (nonatomic, strong) NSDate *createDate;
///Data last use date
@property (nonatomic, strong) NSDate *lastUseDate;
///Data content
@property (nonatomic, copy) NSString *content;
@end

/** @brief History data config .*/
@interface FSHistoryDataConfig : NSObject<NSCoding>
///Set/Get enable
@property (nonatomic, assign) BOOL enable;
///Set/Get the scene
@property (nonatomic, assign) FSInteractiveScene scene;
///Set/Get sort key
@property (nonatomic, assign) FSHistoryDataSortKey sortKey;
///Set/Get sort order
@property (nonatomic, assign) FSDataSortComparison sortOrder;
///Set/Get sort rule
@property (nonatomic, assign) FSHistoryDataSortRule sortRule;
///Set/Get maximum count of items
@property (nonatomic, assign) NSUInteger maxCount;
///Set/Get current items
@property (nonatomic, copy) NSArray<FSHistoryDataItem *> *items;
@end

/** @brief History data module*/
@interface FSHistoryDataModule : NSObject<IModule>

/**@brief Disable the default initialization.*/
+ (instancetype)new NS_UNAVAILABLE;
- (instancetype)init NS_UNAVAILABLE;

/** @brief Configure the scene
 *
 * @param[in] config   Scene configuration
 *
 */
- (void)configScene:(FSHistoryDataConfig *)config;

/** @brief Get configure for the scene
 *
 * @param[in] scene   For the scene.
 *
 * @return    History data config.
 */
- (FSHistoryDataConfig *)configForScene:(FSInteractiveScene)scene;

/** @brief Current history data from the scene.
 *
 * @param[in] scene   From the scene
 *
 * @return    Current history data.
 */
- (NSArray<FSHistoryDataItem *> *)currentItemsFromScene:(FSInteractiveScene)scene;

/** @brief Sort by keywords to get history data from the scene.
 *
 * @param[in] keyword   From the scene
 *
 * @param[in] scene   From the scene
 *
 * @return    Sort results.
 */
- (NSArray<FSHistoryDataItem *> *)sortItemsByKeyword:(NSString *)keyword
                                            fromScene:(FSInteractiveScene)scene;
@end

NS_ASSUME_NONNULL_END
