/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "AnnotationListCell.h"
#import "AttachmentViewController.h"

#import "ReplyTableViewController.h"

@interface AnnotationListCell ()

@property (nonatomic, assign) BOOL isMenu;

@property (nonatomic, readonly) CGFloat rightMargin; // for hiding edit view

@end

@implementation AnnotationListCell

- (instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier isMenu:(BOOL)isMenu {
    if (self = [super initWithStyle:style reuseIdentifier:reuseIdentifier]) {
        self.isInputText = NO;
        self.currentlevel = 1;
        self.item = nil;
        self.cellDelegate = nil;
        self.isMenu = isMenu;
        AnnotationButton *buttonViewLevel = [AnnotationButton buttonWithType:UIButtonTypeCustom];
        buttonViewLevel.frame = CELL_ANNOTATIONBUTTON;
        buttonViewLevel.tag = 100;
        [buttonViewLevel setImage:FSImageNamedAndNormalColor(@"panel_annotation_close") forState:UIControlStateSelected];
        [buttonViewLevel setImage:FSImageNamedAndNormalColor(@"panel_annotation_open") forState:UIControlStateNormal];
        [self.contentView addSubview:buttonViewLevel];

        UIImageView *imageViewAnnotation = [UIImageView new];
        imageViewAnnotation.fs_left = CELL_ANNOTATION_IMAGE_LEFT;
        imageViewAnnotation.fs_top = ITEM_IMAGE_TOP_MARGIN_NORMAL;
        imageViewAnnotation.fs_width = 40.f;
        imageViewAnnotation.fs_height = 40.f;
        imageViewAnnotation.tag = 99;
        [self.contentView addSubview:imageViewAnnotation];

        UIImageView *annoupdatetip = [[UIImageView alloc] initWithFrame:CELL_ANNOTATIONUPDATEVIEW];
        annoupdatetip.tag = 108;
        annoupdatetip.image = FSImageNamedAndNormalColor(@"annoupdatetip");
        [self.contentView addSubview:annoupdatetip];

        UIImageView *annouprepltip = [[UIImageView alloc] initWithFrame:CELL_ANNOTATIONREPLYTIP];
        annouprepltip.tag = 109;
        annouprepltip.image = FSImageNamedAndNormalColor(@"panel_annotation_reply");
        [self.contentView addSubview:annouprepltip];

        UILabel *labelAuthor = [[UILabel alloc] init];
        labelAuthor.tag = 102;
        labelAuthor.textColor = self.fs_textColor_T4;
        labelAuthor.font = self.fs_standardGeneralFont_GF2;
        [labelAuthor setTextAlignment:NSTextAlignmentLeft];
        labelAuthor.lineBreakMode = NSLineBreakByTruncatingTail;
        labelAuthor.fs_left =  imageViewAnnotation.fs_right + ITEM_MEDIUM_SPACE_NORMAL;
        labelAuthor.fs_top =  16;
        labelAuthor.fs_width = self.contentView.fs_width - labelAuthor.fs_left;
        labelAuthor.fs_height = 20.f;
        labelAuthor.backgroundColor = [UIColor clearColor];
        [self.contentView addSubview:labelAuthor];

        UILabel *labelDate = [[UILabel alloc] init];
        labelDate.tag = 103;
        labelDate.textColor = self.fs_textColor_T4;
        labelDate.font = self.fs_standardWeakFont_WF2;
        labelDate.textAlignment = NSTextAlignmentLeft;
        labelDate.frame = labelAuthor.frame;
        labelDate.fs_top = labelAuthor.fs_bottom;
        labelDate.backgroundColor = [UIColor clearColor];
        [self.contentView addSubview:labelDate];

        UILabel *labelSize = [[UILabel alloc] init];
        labelSize.tag = 110;
        labelSize.textColor = self.fs_textColor_T4;
        labelSize.font = self.fs_standardGeneralFont_GF2;
        labelSize.textAlignment = NSTextAlignmentLeft;
        labelSize.frame = CELL_ATTACHMENTSIZE;
        labelSize.backgroundColor = [UIColor clearColor];
        labelSize.hidden = YES;
        [self.contentView addSubview:labelSize];

        UILabel *labelContents = [[UILabel alloc] init];
        labelContents.lineBreakMode = NSLineBreakByWordWrapping;
        labelContents.textAlignment = NSTextAlignmentLeft;
        labelContents.tag = 104;
        labelContents.textColor = self.fs_textColor_T2;
        labelContents.font = self.fs_standardWeakFont_WF1;
        if (isMenu) {
            labelContents.fs_left = imageViewAnnotation.fs_left;
            labelContents.fs_top = labelDate.fs_bottom + 18.f;
            labelContents.fs_width = self.contentView.fs_width - labelContents.fs_left - 44.f;
            labelContents.fs_height = 20.f;
            
        } else {
            labelContents.frame = CELL_ANNOTATIONCONTENTS;
        }

        labelContents.backgroundColor = [UIColor clearColor];
        [self.contentView addSubview:labelContents];

        UITextView *edititextview = [[UITextView alloc] init];
        edititextview.autoresizingMask = UIViewAutoresizingNone;
        edititextview.hidden = YES;
        edititextview.backgroundColor = [UIColor clearColor];
        edititextview.textContainerInset = UIEdgeInsetsMake(0, 10, 0, 0);

        edititextview.returnKeyType = UIReturnKeyDefault;
        edititextview.font = self.fs_standardWeakFont_WF1;
        edititextview.textColor = self.fs_textColor_T2;
        [edititextview setTextAlignment:NSTextAlignmentLeft];
        edititextview.tag = 107;
        if (isMenu) {
            edititextview.frame = labelContents.frame;
        } else {
            edititextview.frame = CELL_ANNOTATIONEDITVIEW;
        }
        [self.contentView addSubview:edititextview];
        
        UIButton *bottomClickBtn = [[UIButton alloc] init];
        bottomClickBtn.backgroundColor = [UIColor fs_clearColor];
        bottomClickBtn.hidden =YES;
        [self.contentView addSubview:bottomClickBtn];
        _bottomClickBtn = bottomClickBtn;

        self.detailButton = [UIButton buttonWithType:UIButtonTypeCustom];
        _detailButton.tag = 200;
        [_detailButton setImage:FSImageNamedAndNormalColor(@"document_cellmore_more") forState:UIControlStateNormal];
        [_detailButton addTarget:self action:@selector(handleClickDetailButton) forControlEvents:UIControlEventTouchUpInside];
        _detailButton.frame = CGRectMake(self.contentView.frame.size.width - 50, 0, 50, 50);
        _detailButton.center = CGPointMake(_detailButton.center.x, 34);
        _detailButton.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin;
        [self.contentView addSubview:_detailButton];
        if (isMenu) {
            if ((DEVICE_iPHONE && ([UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationPortraitUpsideDown || [UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationPortrait)) || (DEVICE_iPHONE && ((STYLE_CELLWIDTH_IPHONE * STYLE_CELLHEIHGT_IPHONE) < (375 * 667)) && ([UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationLandscapeLeft || [UIApplication sharedApplication].statusBarOrientation == UIInterfaceOrientationLandscapeRight))) {
                UIView *doneView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 0, 40)];
                doneView.backgroundColor = ReplyThemeBackgroundColor ;
                UIButton *doneBT = [UIButton buttonWithType:UIButtonTypeCustom];
                [doneBT setBackgroundImage:FSImageNamedAndNormalColor(@"common_keyboard_done") forState:UIControlStateNormal];
                [doneBT addTarget:self action:@selector(dismissKeyboard) forControlEvents:UIControlEventTouchUpInside];
                [doneView addSubview:doneBT];
                [doneBT mas_makeConstraints:^(MASConstraintMaker *make) {
                    make.right.equalTo(doneView.mas_right).offset(0);
                    make.top.equalTo(doneView.mas_top).offset(0);
                    make.size.mas_equalTo(CGSizeMake(40, 40));
                }];
                edititextview.inputAccessoryView = doneView;
            }
        }
    }
    return self;
}

- (void)setReadOnly:(BOOL)readOnly{
    _readOnly = readOnly;
    _detailButton.hidden = readOnly;
}

- (void)setCellDelegate:(id<AnnotationListCellDelegate>)cellDelegate {
    if (_cellDelegate != cellDelegate) {
        _cellDelegate = cellDelegate;
        [self setupEditView];
    }
}

- (void)setItem:(AnnotationItem *)item {
    if (_item != item) {
        _editViewHidden = YES;
        _item = item;
        if (item.highlighted) {
            self.backgroundColor = GroupCellBackgroundColor;
        }else{
            self.backgroundColor = [UIColor fs_clearColor];
        }
        [self setupEditView];
    }
}

- (void)setupEditView {

}

- (CGFloat)rightMargin {
    BOOL landscape = UIInterfaceOrientationIsLandscape([UIApplication sharedApplication].statusBarOrientation);
    if (iOS11Later && landscape) {
        return 44;
    }
    return 0;
}

#pragma mark <AnnotationListMoreDelegate>

- (void)annotationListMoreReply:(AnnotationListMore *)annotationListMore {
    if ([self.cellDelegate respondsToSelector:@selector(annotationListCellReply:)]) {
        [self.cellDelegate annotationListCellReply:self];
    }
}

- (void)annotationListMoreEdit:(AnnotationListMore *)annotationListMore {
    if ([self.cellDelegate respondsToSelector:@selector(annotationListCellEdit:)]) {
        [self.cellDelegate annotationListCellEdit:self];
    }
}

- (void)annotationListMoreDescript:(AnnotationListMore *)annotationListMore {
    if ([self.cellDelegate respondsToSelector:@selector(annotationListCellDescript:)]) {
        [self.cellDelegate annotationListCellDescript:self];
    }
}

- (void)annotationListMoreDelete:(AnnotationListMore *)annotationListMore {
    if ([self.cellDelegate respondsToSelector:@selector(annotationListCellDelete:)]) {
        [self.cellDelegate annotationListCellDelete:self];
    }
}

- (void)annotationListMoreSave:(AnnotationListMore *)annotationListMore {
    if ([self.cellDelegate respondsToSelector:@selector(annotationListCellSave:)]) {
        [self.cellDelegate annotationListCellSave:self];
    }
}

- (void)annotationListMoreFlatten:(AnnotationListMore *)annotationListMore {
    if ([self.cellDelegate respondsToSelector:@selector(annotationListCellFlatten:)]) {
        [self.cellDelegate annotationListCellFlatten:self];
    }
}

- (void)annotationListMoreApplyRedaction:(AnnotationListMore *)annotationListMore{
    if ([self.cellDelegate respondsToSelector:@selector(annotationListCellApplyRedaction:)]) {
        [self.cellDelegate annotationListCellApplyRedaction:self];
    }
}

#pragma mark

- (void)dismissKeyboard {
    if (self.cellDelegate && [self.cellDelegate respondsToSelector:@selector(dismissKeyboard)]) {
        [self.cellDelegate dismissKeyboard];
    }
}

- (void)prepareForReuse {
    [super prepareForReuse];
}

- (void)handleClickDetailButton {
    if ([self.cellDelegate respondsToSelector:@selector(annotationListCellWillShowEditView:)]) {
        [self.cellDelegate annotationListCellWillShowEditView:self];
    }
    [self setEditViewHidden:NO];
    if ([self.cellDelegate respondsToSelector:@selector(annotationListCellDidShowEditView:)]) {
        [self.cellDelegate annotationListCellDidShowEditView:self];
    }
}

- (void)setEditViewHidden:(BOOL)isHidden {
    [self setEditViewHidden:isHidden animated:YES];
}

- (void)setEditViewHidden:(BOOL)isHidden animated:(BOOL)animated {
    if (isHidden) {
        if ([self.cellDelegate respondsToSelector:@selector(annotationListCellWillDisappearEditView:)]) {
            [self.cellDelegate annotationListCellWillDisappearEditView:self];
        }
        [self hideEditViewAnimated:animated];
        if ([self.cellDelegate respondsToSelector:@selector(annotationListCellDidDisappearEditView:)]) {
            [self.cellDelegate annotationListCellDidDisappearEditView:self];
        }
    } else {
        [self showEditViewAnimated:animated];
    }
}

- (void)showEditViewAnimated:(BOOL)animated {
   
}

- (void)hideEditViewAnimated:(BOOL)animated {
    
}

@end
