/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "AnnotationStruct.h"
#import "FSAnnot+Extensions.h"

@interface AnnotationStruct ()
+ (void)recursionAnnostruct:(AnnotationItem *)currentnode childnodes:(NSArray *)childs collectarray:(NSMutableArray *)collectarray AnnoStruct:(NSDictionary *)annostruct;
+ (void)recursionAnnostruct:(AnnotationItem *)deleteanno childnodes:(NSArray *)childs currentAnnotation:(AnnotationItem *)currentanno AnnoStruct:(NSDictionary *)annostruct superAnnotation:(AnnotationItem **)superannotation;
@end

@implementation AnnotationStruct

+ (NSMutableDictionary *)getAnnotationStructWithAnnos:(NSArray *)annosarray {
    if (annosarray == nil) {
        return [NSMutableDictionary dictionary];
    }

    NSMutableDictionary *tempdic = [NSMutableDictionary dictionary];

    for (int i = 0; i < annosarray.count; i++) {
        AnnotationItem *searchanno = [annosarray objectAtIndex:i];

        NSString *annotionuuid = searchanno.annot.uuidWithPageIndex;

        NSMutableArray *onenoteannotempary = [NSMutableArray array];

        if ([searchanno.annot isMarkup]) {
            FSMarkup *markup = [[FSMarkup alloc] initWithAnnot:searchanno.annot];
            int countOfReplies = [markup getReplyCount];
            for (int r = 0; r < countOfReplies; r++) {
                FSNote *note = [markup getReply:r];
                if (!note)
                    continue;
                for (int a = 0; a < annosarray.count; a++) {
                    AnnotationItem *tempanno = [annosarray objectAtIndex:a];
                    if ([note isEqualToAnnot:tempanno.annot]) {
                        [onenoteannotempary addObject:tempanno];
                        break;
                    }
                }
            }
        }

        [tempdic setObject:onenoteannotempary forKey:annotionuuid];
    }
    return tempdic;
}

+ (NSArray *)getAllChildNodesWithSuperAnnotation:(AnnotationItem *)superanno annoStruct:(NSDictionary *)annostruct {
    if (superanno == nil || annostruct == nil) {
        return [NSArray array];
    }
    NSMutableArray *childsarray = [NSMutableArray array];
    [self recursionAnnostruct:superanno childnodes:[annostruct objectForKey:superanno.annot.uuidWithPageIndex] collectarray:childsarray AnnoStruct:annostruct];

    return childsarray;
}

+ (void)recursionAnnostruct:(AnnotationItem *)currentnode childnodes:(NSArray *)childs collectarray:(NSMutableArray *)collectarray AnnoStruct:(NSDictionary *)annostruct {
    if (childs.count == 0) {
        return;
    }

    [collectarray addObjectsFromArray:childs];

    for (AnnotationItem *annotation in childs) {
        [self recursionAnnostruct:annotation childnodes:[annostruct objectForKey:annotation.annot.uuidWithPageIndex] collectarray:collectarray AnnoStruct:annostruct];
    }
}

+ (NSInteger)getAnnotationLevel:(AnnotationItem *)annotation AnnoStruct:(NSDictionary *)annostruct rootAnnotation:(AnnotationItem *)rootanno {
    NSUInteger annlevel = 0;

    AnnotationItem *superannotation = nil;

    [self recursionAnnostruct:annotation childnodes:[annostruct objectForKey:rootanno.annot.uuidWithPageIndex] currentAnnotation:rootanno AnnoStruct:annostruct superAnnotation:&superannotation];

    while (superannotation) {
        AnnotationItem *tempanno = superannotation;
        superannotation = nil;
        annlevel++;
        [self recursionAnnostruct:tempanno childnodes:[annostruct objectForKey:rootanno.annot.uuidWithPageIndex] currentAnnotation:rootanno AnnoStruct:annostruct superAnnotation:&superannotation];
    }
    return annlevel;
}

+ (void)getRootAnnotation:(AnnotationItem *)annotation TargetAnnotation:(AnnotationItem **)targetanno AnnoArray:(NSArray *)annoarrays {
    for (AnnotationItem *checkanno in annoarrays) {
        if ([annotation.annot isReplyToAnnot:checkanno.annot]) {
            *targetanno = checkanno;
            [self getRootAnnotation:checkanno TargetAnnotation:targetanno AnnoArray:annoarrays];
        }
    }
}

+ (BOOL)deleteAnnotationFromAnnoStruct:(NSMutableDictionary *)annostruct deleteAnnotation:(AnnotationItem *)deletenode rootAnnotation:(AnnotationItem *)rootanno {
    if (deletenode == nil || annostruct == nil || rootanno == nil) {
        return NO;
    }

    AnnotationItem *superannotation = nil;

    [self recursionAnnostruct:deletenode childnodes:[annostruct objectForKey:rootanno.annot.uuidWithPageIndex] currentAnnotation:rootanno AnnoStruct:annostruct superAnnotation:&superannotation];

    if (superannotation && [annostruct objectForKey:superannotation.annot.uuidWithPageIndex]) {
        [(NSMutableArray *)[annostruct objectForKey:superannotation.annot.uuidWithPageIndex] removeObject:deletenode];
        [annostruct removeObjectForKey:deletenode.annot.uuidWithPageIndex];

        return YES;
    }

    return NO;
}

+ (void)recursionAnnostruct:(AnnotationItem *)deleteanno childnodes:(NSArray *)childs currentAnnotation:(AnnotationItem *)currentanno AnnoStruct:(NSDictionary *)annostruct superAnnotation:(AnnotationItem **)superannotation {
    if (childs.count == 0) {
        return;
    }
    for (AnnotationItem *anno in childs) {
        if ([anno.annot.uuidWithPageIndex isEqualToString:deleteanno.annot.uuidWithPageIndex]) {
            *superannotation = currentanno;

            break;

        } else {
            [self recursionAnnostruct:deleteanno childnodes:[annostruct objectForKey:anno.annot.uuidWithPageIndex] currentAnnotation:anno AnnoStruct:annostruct superAnnotation:superannotation];
        }
    }
}

+ (BOOL)insertAnnotationToAnnoStruct:(NSMutableDictionary *)annostruct insertAnnotation:(AnnotationItem *)insertnode SuperAnnotation:(AnnotationItem *)superanno {
    if (annostruct == nil || insertnode == nil || superanno == nil) {
        return NO;
    }

    NSMutableArray *nodes = [NSMutableArray array];
    [annostruct setObject:nodes forKey:insertnode.annot.uuidWithPageIndex];

    if ([annostruct objectForKey:superanno.annot.uuidWithPageIndex]) {
        [[annostruct objectForKey:superanno.annot.uuidWithPageIndex] addObject:insertnode];
    }

    return NO;
}

+ (NSString *)annotationImageName:(AnnotationItem *)annotation {
    NSString *resultString = nil;

    FSAnnotType annotType = [annotation.annot getFilterAnnotType];
    if (annotType == FSAnnotTextbox) resultString = @"comment_textbox";
    else if (annotType == FSAnnotCallout) resultString = @"comment_callout";
    else if (annotType == FSAnnotArrowLine) resultString = @"comment_arrow";
    else if (annotType == FSAnnotDistance) resultString = @"comment_measure";
    else if (annotType == FSAnnotReplace) resultString = @"comment_replace_text";
    else if (annotType == FSAnnotInsert) resultString = @"comment_insert_text";
    else if (annotType == FSAnnotImage) resultString = @"comment_image";
    else if (annotType == FSAnnotCloud) resultString = @"comment_cloud";
    else{
        switch (annotType) {
        case FSAnnotHighlight:
            resultString = @"comment_highlight";
            break;
        case FSAnnotUnderline:
            resultString = @"comment_underline";
            break;
        case FSAnnotStrikeOut:
            resultString = @"comment_strikeout";
            break;
        case FSAnnotSquiggly:
            resultString = @"comment_squiggly";
            break;
        case FSAnnotNote:
            resultString = @"comment_note";
            break;
        case FSAnnotSquare:
            resultString = @"comment_rectangle";
            break;
        case FSAnnotCircle:
            resultString = @"comment_oval";
            break;
        case FSAnnotInk:
            resultString = @"comment_pencil";
            break;
        case FSAnnotFileAttachment:
            resultString = @"comment_attachment";
            break;
        case FSAnnotPolyLine:
            resultString = @"comment_polyLine";
            break;
        case FSAnnotRedact:
            resultString = @"comment_redaction";
            break;
        case FSAnnotSound:
            resultString = @"comment_sound";
            break;
        case FSAnnotFreeText:
            resultString = @"comment_typewriter";
            break;
        case FSAnnotStamp:
            resultString = @"comment_stamp";
            break;
        case FSAnnotLine:
            resultString = @"comment_line";
            break;
        case FSAnnotPolygon:
            resultString = @"comment_polygon";
            break;
        default:
            resultString = @"comment_unknow";
            break;
        }
    }
    return resultString;
}

@end
