/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSAnnotFilterViewController.h"
#import "NSUserDefaults+Extensions.h"

static NSString *const kImageName = @"kImageName";
static NSString *const kTitle = @"kTitle";
static NSString *const kIsSelect = @"kIsSelect";
static NSString *const kAnnotType = @"kAnnotType";

@interface FSAnnotFilterItem : NSObject
@property (nonatomic, copy) NSString *imageName;
@property (nonatomic, copy) NSString *title;
@property (nonatomic, assign) BOOL isSelect;
@property (nonatomic, assign) FSAnnotType annotType;
@property (nonatomic, assign) BOOL enabled;
@end

@implementation FSAnnotFilterItem

+ (NSDictionary *)modelCustomPropertyMapper {
    return @{@"imageName"  : kImageName,
             @"title"  : kTitle,
             @"isSelect"  : kIsSelect,
             @"annotType" : kAnnotType,
    };
}

@end

@interface FSAnnotFilterViewController ()
@property (nonatomic, strong) NSData *oldAnnotFilterData;

@end

@implementation FSAnnotFilterViewController

- (void)initialize{
    [super initialize];
    self.defaultEstimatedCellStyle = FSEstimatedTableViewCellStyleTitle;
    self.shouldLoadDataOnViewDidLoad = YES;
    self.sectionStyle = FSTableViewSectionStyleFirstNone;
    self.sectionHeaderHeight = 8.f;
    self.cellSelectedColor = self.fs_dividerColor_P1;
}

- (void)loadData{
    NSArray *prefetchData;
    if (self.params) {
        prefetchData = self.params;
    }else{
        FSPDFViewCtrl *pdfViewCtrl = ((FSViewControllerManager *)self.services).extensionsManager.pdfViewCtrl;
             if([pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_REFLOW){
                 prefetchData = @[@[
                          @{kImageName:@"comment_highlight",kTitle:@"kHighlight",kIsSelect:@NO,kAnnotType:@(FSAnnotHighlight)},
                          @{kImageName:@"comment_underline",kTitle:@"kUnderline",kIsSelect:@NO,kAnnotType:@(FSAnnotUnderline)},
                          @{kImageName:@"comment_squiggly",kTitle:@"kSquiggly",kIsSelect:@NO,kAnnotType:@(FSAnnotSquiggly)},
                          @{kImageName:@"comment_strikeout",kTitle:@"kStrikeout",kIsSelect:@NO,kAnnotType:@(FSAnnotStrikeOut)},
                 ]];
             }else{
                 prefetchData = @[@[
                          @{kImageName:@"comment_note",kTitle:@"kNote",kIsSelect:@NO,kAnnotType:@(FSAnnotNote)},
                          @{kImageName:@"comment_attachment",kTitle:@"kAttachment",kIsSelect:@NO,kAnnotType:@(FSAnnotFileAttachment)},
                          @{kImageName:@"comment_stamp",kTitle:@"kStamp",kIsSelect:@NO,kAnnotType:@(FSAnnotStamp)},
                          @{kImageName:@"comment_typewriter",kTitle:@"kTypewriter",kIsSelect:@NO,kAnnotType:@(FSAnnotFreeText)},
                          @{kImageName:@"comment_callout",kTitle:@"kCallout",kIsSelect:@NO,kAnnotType:@(FSAnnotCallout)},
                          @{kImageName:@"comment_textbox",kTitle:@"kTextbox",kIsSelect:@NO,kAnnotType:@(FSAnnotTextbox)},
                          @{kImageName:@"comment_highlight",kTitle:@"kHighlight",kIsSelect:@NO,kAnnotType:@(FSAnnotHighlight)},
                          @{kImageName:@"comment_underline",kTitle:@"kUnderline",kIsSelect:@NO,kAnnotType:@(FSAnnotUnderline)},
                          @{kImageName:@"comment_squiggly",kTitle:@"kSquiggly",kIsSelect:@NO,kAnnotType:@(FSAnnotSquiggly)},
                          @{kImageName:@"comment_strikeout",kTitle:@"kStrikeout",kIsSelect:@NO,kAnnotType:@(FSAnnotStrikeOut)},
                          @{kImageName:@"comment_replace_text",kTitle:@"kReplaceText",kIsSelect:@NO,kAnnotType:@(FSAnnotReplace)},
                          @{kImageName:@"comment_insert_text",kTitle:@"kInsertText",kIsSelect:@NO,kAnnotType:@(FSAnnotInsert)},
                          @{kImageName:@"comment_area_highlight",kTitle:@"kAreaHighlight",kIsSelect:@NO,kAnnotType:@(FSAnnotAreaHighlight)},
                          @{kImageName:@"comment_measure",kTitle:@"kMeasure",kIsSelect:@NO,kAnnotType:@(FSAnnotDistance)},
                          @{kImageName:@"comment_image",kTitle:@"kImageAnnotation",kIsSelect:@NO,kAnnotType:@(FSAnnotImage)},
                       ],@[
                           @{kImageName:@"comment_pencil",kTitle:@"kPencil",kIsSelect:@NO,kAnnotType:@(FSAnnotInk)},
                           @{kImageName:@"comment_rectangle",kTitle:@"kRectangle",kIsSelect:@NO,kAnnotType:@(FSAnnotSquare)},
                           @{kImageName:@"comment_oval",kTitle:@"kCircle",kIsSelect:@NO,kAnnotType:@(FSAnnotCircle)},
                           @{kImageName:@"comment_line",kTitle:@"kLine",kIsSelect:@NO,kAnnotType:@(FSAnnotLine)},
                           @{kImageName:@"comment_arrow",kTitle:@"kArrow",kIsSelect:@NO,kAnnotType:@(FSAnnotArrowLine)},
                           @{kImageName:@"comment_polyLine",kTitle:@"kPolyLine",kIsSelect:@NO,kAnnotType:@(FSAnnotPolyLine)},
                           @{kImageName:@"comment_polygon",kTitle:@"kPolygon",kIsSelect:@NO,kAnnotType:@(FSAnnotPolygon)},
                           @{kImageName:@"comment_cloud",kTitle:@"kCloud",kIsSelect:@NO,kAnnotType:@(FSAnnotCloud)},
                       ]];
             }
        NSMutableArray *tmp = @[].mutableCopy;
        for (NSArray *data in prefetchData) {
            NSMutableArray *items = [[NSArray modelArrayWithClass:[FSAnnotFilterItem class] json:data] mutableCopy];
            [tmp addObject:items];
            for (FSAnnotFilterItem *item in items.copy) {
                if (![((FSViewControllerManager *)self.services).extensionsManager.config canInteractWithAnnotType:item.annotType]) {
                    [items removeObject:item];
                }
            }
        }
        prefetchData = tmp.copy;
    }
    self.dataSource = prefetchData.copy;
    self.oldAnnotFilterData = [self.dataSource modelToJSONData];
}

- (void)configureCell:(FSTitleEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(FSAnnotFilterItem *)annotFilterItem{
    if (!cell.installed) {
        cell.leftMargin = ITEM_MARGIN_NORMAL;
    }
    if (!cell.leftView) {
        UIImageView *leftImageView = [UIImageView new];
        cell.leftView = leftImageView;
    }
    ((UIImageView *)cell.leftView).image = FSImageNamedAndNormalColor(annotFilterItem.imageName);
    
    cell.estimatedTitleLabel.text = FSLocalizedForKey(annotFilterItem.title);
    
    if (!cell.rightView) {
        UIImageView *imageView = [UIImageView new];
        [BindThemeKeySel(imageView, setImage:) fs_updateThemeCompleted:^(UIImageView * _Nonnull target, SEL  _Nonnull sel) {
            target.image = FSImageNamedAndPrimaryColor(@"common_select_check");
        }];
        imageView.hidden = YES;
        cell.rightView = imageView;
    }
    cell.rightView.hidden = !annotFilterItem.isSelect;
}

- (void)navigationBarBackTouch{
    NSMutableSet *needFilterType = [NSMutableSet set];
    for (NSArray *data in self.dataSource) {
        for (FSAnnotFilterItem *item in data) {
            if (item.isSelect) {
                [needFilterType addObject:@(item.annotType)];
            }
        }
    }
    NSData *tmp = [self.dataSource modelToJSONData];
    if (![tmp isEqualToData:self.oldAnnotFilterData]) !self.didClickedCloseItem ? : self.didClickedCloseItem(needFilterType, self.dataSource);
    [super navigationBarBackTouch];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    self.title = FSLocalizedForKey(@"kFilterComments");
    self.navBarLeftTitle = FSLocalizedForKey(@"kClose");
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{
    [super tableView:tableView didSelectRowAtIndexPath:indexPath];
    FSTitleEstimatedTableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
    cell.rightView.hidden = !cell.rightView.hidden;
    
    FSAnnotFilterItem *annotFilterItem = [self dataForAtIndexPath:indexPath];
    annotFilterItem.isSelect = !cell.rightView.hidden;
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
