/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSAnnotationListViewController.h"
#import "FSAnnotationItem.h"
#import "AnnotationPanel.h"
#import "PanelController.h"
#import "FSAnnot+Extensions.h"
#import "MultipleSelectionToolHandler.h"
#import "FSViewControllerManager.h"
#import "FSGCDServer.h"
#import "FSPermissionUtilty.h"
#import "ReflowModule.h"
#import <objc/runtime.h>

@interface FSAnnoationWithouAuthorSubjectEstimatedTableViewCell : FSSubjectEstimatedTableViewCell
@property (nonatomic, assign, readwrite) BOOL installed;
@end

@implementation FSAnnoationWithouAuthorSubjectEstimatedTableViewCell
@synthesize installed = _installed;
- (void)installConstraints{
    if (_installed) return;
    !self.willInstallConstraints ? : self.willInstallConstraints(self);
    if (self.leftView) {
        [self.leftView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.fs_equalToSuperview().offset(self.leftMargin).priorityHigh();
            make.top.fs_equalToSuperview().offset(self.leftViewTopMargin).priorityHigh();
            if (!CGSizeEqualToSize(CGSizeZero, self.leftViewSzie)) {
                make.size.mas_lessThanOrEqualTo(self.leftViewSzie);
            }else if(!CGSizeEqualToSize(CGSizeZero, self.leftView.fs_size)){
                make.width.mas_lessThanOrEqualTo(self.leftView.fs_width);
                make.height.mas_lessThanOrEqualTo(self.leftView.fs_height);
            }else{
                make.size.mas_equalTo(self.leftView).priority(MASLayoutPriorityFittingSizeLevel);
            }
        }];
    }
    
    self.estimatedTitleLabel.hidden = YES;
    [self.estimatedSubjectLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
        if (self.leftView) {
            make.left.mas_equalTo(self.leftView.mas_right).offset(self.mediumSpace).priorityHigh();
        }else{
            make.left.fs_equalToSuperview().offset(self.leftMargin).priorityHigh();
        }
        if (self.rightView) {
            make.right.mas_equalTo(self.rightView.mas_left).offset(-self.rightMargin).priorityHigh();
        }else{
            make.right.fs_equalToSuperview().offset(-self.rightMargin).priorityHigh();
        }
        if (self.leftView) {
            make.centerY.mas_equalTo(self.leftView.mas_centerY);
            make.bottom.fs_equalToSuperview().offset(-ITEM_LABEL_BOTTOM_MARGIN_LARGE).priorityHigh();
        }else{
            make.top.fs_equalToSuperview().offset(self.topMargin).priorityHigh();
            make.height.mas_greaterThanOrEqualTo(TITLE_LABEL_MIN_HEIGHT).priorityHigh();
            make.bottom.fs_equalToSuperview().offset(-self.subjectBottomMargin).priorityHigh();
        }
    }];
    
    if (self.rightView) {
        [self.rightView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.right.fs_equalToSuperview().offset(-self.rightMargin).priorityHigh();
            make.top.fs_equalToSuperview().offset(self.rightViewTopMargin).priorityHigh();
            make.size.mas_equalTo(self.rightView).priority(MASLayoutPriorityFittingSizeLevel);
        }];
    }
    _installed = YES;
    [self.estimatedTitleLabel layoutIfNeeded];
    if (self.estimatedTitleLabel.fs_width) self.estimatedTitleLabel.preferredMaxLayoutWidth = self.estimatedTitleLabel.fs_width;
    !self.didInstallConstraints ? : self.didInstallConstraints(self);
}

@end


@interface FSAnnoationWithouAuthorDescriptionEstimatedTableViewCell : FSDescriptionEstimatedTableViewCell
@property (nonatomic, assign, readwrite) BOOL installed;
@end

@implementation FSAnnoationWithouAuthorDescriptionEstimatedTableViewCell
@synthesize installed = _installed;

- (void)installConstraints{
    if (_installed) return;
    !self.willInstallConstraints ? : self.willInstallConstraints(self);
    if (self.leftView) {
        [self.leftView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.fs_equalToSuperview().offset(self.leftMargin).priorityHigh();
            make.top.fs_equalToSuperview().offset(self.leftViewTopMargin).priorityHigh();
            if (!CGSizeEqualToSize(CGSizeZero, self.leftViewSzie)) {
                make.size.mas_lessThanOrEqualTo(self.leftViewSzie);
            }else if(!CGSizeEqualToSize(CGSizeZero, self.leftView.fs_size)){
                make.width.mas_lessThanOrEqualTo(self.leftView.fs_width);
                make.height.mas_lessThanOrEqualTo(self.leftView.fs_height);
            }else{
                make.size.mas_equalTo(self.leftView).priority(MASLayoutPriorityFittingSizeLevel);
            }
        }];
    }
    
    self.estimatedTitleLabel.hidden = YES;
    [self.estimatedSubjectLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
        if (self.leftView) {
            make.left.mas_equalTo(self.leftView.mas_right).offset(self.mediumSpace).priorityHigh();
        }else{
            make.left.fs_equalToSuperview().offset(self.leftMargin).priorityHigh();
        }
        if (self.rightView) {
            make.right.mas_equalTo(self.rightView.mas_left).offset(-self.rightMargin).priorityHigh();
        }else{
            make.right.fs_equalToSuperview().offset(-self.rightMargin).priorityHigh();
        }
        
        if (self.leftView) {
            make.centerY.mas_equalTo(self.leftView.mas_centerY);
        }else{
            make.top.fs_equalToSuperview().offset(self.topMargin).priorityHigh();
            make.height.mas_greaterThanOrEqualTo(TITLE_LABEL_MIN_HEIGHT).priorityHigh();
        }
    }];
    
    [self.estimatedDescriptionLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.fs_equalToSuperview().offset(ITEM_MARGIN_NORMAL);
        make.right.fs_equalToSuperview().offset(-ITEM_MARGIN_NORMAL);
        if (self.leftView) {
            make.top.mas_equalTo(self.leftView.mas_bottom).offset(ITEM_MEDIUM_SPACE_NORMAL).priorityHigh();
        }else{
            if (self.rightView) {
                make.top.mas_equalTo(self.rightView.mas_bottom).offset(ITEM_MEDIUM_SPACE_NORMAL).priorityHigh();
            }
        }
        make.bottom.fs_equalToSuperview().offset(-self.descriptionBottomMargin);
        make.height.mas_greaterThanOrEqualTo(DESCRIPTION_LABEL_MIN_HEIGHT).priorityHigh();
    }];
    
    if (self.rightView) {
        [self.rightView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.right.fs_equalToSuperview().offset(-self.rightMargin).priorityHigh();
            make.top.fs_equalToSuperview().offset(self.rightViewTopMargin).priorityHigh();
            make.size.mas_equalTo(self.rightView).priority(MASLayoutPriorityFittingSizeLevel);
        }];
    }
    _installed = YES;
    !self.didInstallConstraints ? : self.didInstallConstraints(self);
    
}
@end


@interface NSMutableArray (FSPDFPageAnnotItems)
@property (nonatomic, assign) NSUInteger fs_notReplyItemCount;
@end

@implementation NSMutableArray (FSPDFPageAnnotItems) YYSYNTH_DYNAMIC_PROPERTY_CTYPE(fs_notReplyItemCount, setFs_notReplyItemCount, NSUInteger); @end

@interface FSAnnotationListViewController () <IAnnotEventListener>

@property (nonatomic, strong) NSOperationQueue *loadAnnotsQueue;
@property (nonatomic, strong) NSOperationQueue *calculateAnnotsQueue;

@property (nonatomic, copy) NSArray *annotFilterItems;
@property (nonatomic, strong) NSMutableArray *allFilterAnnots;
@property (nonatomic, strong) NSMutableArray *allPageAnnots;
@property (nonatomic, strong) NSMutableArray *allSearchPageAnnots;
@property (nonatomic, strong) NSMutableDictionary *allFilterAnnotsMapToAllPageAnnots;
@property (nonatomic, strong) NSMutableDictionary *pageIndexMapToPageAnnots;
@property (nonatomic, strong) NSMutableDictionary *pageIndexMapToPageFilterAnnots;
@property (nonatomic, copy) NSSet *needFilterType;

@property (nonatomic, assign) BOOL needAnnotEvent;

@property (nonatomic, assign) BOOL isGroupList;
@property (nonatomic, copy) NSArray *groupListAnnots;

@property (nonatomic, strong) FSItem *filterItem;

@property (nonatomic, assign) NSInteger flattenAnnotBeforeCount;
@property (nonatomic, assign) NSInteger canDeleteAnnotCounts;
@end

@implementation FSAnnotationListViewController {
    __weak FSPDFViewCtrl *_pdfViewCtrl;
    __weak UIExtensionsManager *_extensionsManager;
    __weak FSPanelController *_panelController;
    __weak AnnotationPanel *_annotPanel;
}

- (void)initialize{
    [super initialize];
    self.shouldShowNavigationBar = NO;
    self.shouldShowBottomBar = YES;
    self.defaultEstimatedCellStyle = FSEstimatedTableViewCellStyleDescription;
    self.sectionStyle = FSTableViewSectionStyleTitle;
    self.editingImageNormal = FSImageNamedAndNormalColor(@"common_unselect");
    self.editingImageSelected = FSImageNamed(@"common_select");
}

- (id)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager module:(AnnotationPanel *)annotPanel{
    self = [super initWithServices:extensionsManager.viewCtrlManager];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _panelController = annotPanel.panelController;
        _annotPanel = annotPanel;

        self.allPageAnnots = @[].mutableCopy;
        self.needAnnotEvent = YES;

        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(reloadDataList:) name:NSNotificationNameAnnotLsitUpdate object:nil];
        
    }
    return self;
}

- (FSItem *)filterItem{
    if(!_filterItem){
        _filterItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kFilter") imageNormal:FSImageNamedAndNormalColor(@"panel_filter_gray") imageSelected:FSImageNamedAndNormalColor(@"panel_filter_select_gray") imageDisable:nil imageTextRelation:FSImageTextRelationRight];
        [_filterItem setTitle:FSLocalizedForKey(@"kFilterResult") forState:UIControlStateSelected];
        [_filterItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
        _filterItem.font = self.fs_standardGeneralFont_GF2;
        @weakify(self)
        _filterItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            FSAnnotFilterViewController *viewController = [self->_extensionsManager.viewCtrlManager presentViewCtrlType:FSViewCtrlTypeAnnotFilter params:self.annotFilterItems];
            [viewController setDidClickedCloseItem:^(NSSet * _Nonnull needFilterType, NSArray * _Nonnull annotFilterItems) {
                self.filterItem.selected = needFilterType.count;
                self.annotFilterItems = annotFilterItems;
                self.needFilterType = needFilterType.copy;
                [self filterAllAnnotWithTypes:needFilterType];
                [self reloadData];
            }];
        };
    }
    return _filterItem;
}

- (BOOL)searchState{
    return self.enteredSearch && self.allSearchPageAnnots;
}

- (BOOL)filterState{
    return self.filterItem.selected;;
}

- (__kindof NSArray *)dataSource{
    if (self.isGroupList) return self.groupListAnnots;
    if ([self searchState]) return self.allSearchPageAnnots;
    if (self.filterState) return self.allFilterAnnots;
    return self.allPageAnnots;
}

- (void)loadGroupListWithGroupAnnotItems:(NSArray<FSAnnotationItem *> *)groupAnnotItems{
    NSMutableArray *groupListAnnots = @[].mutableCopy;
    for (FSAnnotationItem *annotItem in groupAnnotItems) {
        FSAnnot *annot = annotItem.annot;
        if (!annot.replyTo) {
            FSAnnotationItem *annoItem = [[FSAnnotationItem alloc] initWithAnnot:annot isReply:NO extensionsManager:_extensionsManager];
            [groupListAnnots addObject:annoItem];
            groupListAnnots.fs_notReplyItemCount++;
            annoItem.locPageItmes = groupListAnnots;
            if ([annot isMarkup]) {
                FSMarkup *markup = [[FSMarkup alloc] initWithAnnot:annot];
                int replyCount = [markup getReplyCount];
                for (int i = 0 ; i < replyCount; i ++) {
                    FSNote *note = [markup getReply:i];
                    FSTopReplyAnnotationItem *replyItem = [[FSTopReplyAnnotationItem alloc] initWithAnnot:note extensionsManager:_extensionsManager];
                    [annoItem addReplyItem:replyItem];
                }
            }
        }
    }
    self.groupListAnnots = @[groupListAnnots];
}

- (void)loadData{
    if (!_pdfViewCtrl.currentDoc || self.isGroupList) return;
    @synchronized(self) {

        if (!self.loadAnnotsQueue) {
            self.loadAnnotsQueue = [[NSOperationQueue alloc] init];
            self.loadAnnotsQueue.name = @"load annotations";
            self.loadAnnotsQueue.maxConcurrentOperationCount = 1;
        } else {
            [self cancelLoadAnnotations];
        }
        [self resetAnnotationArray];
        NSBlockOperation *op = [[NSBlockOperation alloc] init];
        NSBlockOperation *__weak weakOp = op;
        typeof(self) __weak weakSelf = self;
        int pageCount = [self->_pdfViewCtrl.currentDoc getPageCount];
        [self.view fs_showHUDLoading:@"loading"];
        [op addExecutionBlock:^{
            @autoreleasepool {
            for (int i = 0; i < pageCount; i++) {
                if (weakOp.isCancelled) {
                    dispatch_main_async_safe(^{
                        [weakSelf.view fs_hideHUDLoading];
                    });
                    return;
                }
                [weakSelf loadAnnotationsForPageAtIndex:i];
            }
            }
            if (weakOp.isCancelled) {
                dispatch_main_async_safe(^{
                    [weakSelf.view fs_hideHUDLoading];
                });
                return;
            }
            dispatch_main_async_safe(^{
                [weakSelf.view fs_hideHUDLoading];
                [weakSelf reloadData];
            });
        }];
        [self.loadAnnotsQueue addOperation:op];
    }
}

- (void)clickedSearchCancelItem{
    self.allSearchPageAnnots = nil;
    [super clickedSearchCancelItem];
    [_panelController.panel.pagerContentView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(_panelController.panel.contentView.mas_left);
        make.right.mas_equalTo(_panelController.panel.contentView.mas_right);
        make.bottom.fs_equalToSuperview();
        make.top.mas_equalTo(_panelController.panel.topBar.mas_bottom);
    }];
    [_panelController.panel.contentView layoutIfNeeded];
    _panelController.panel.pagerContentView.scrollEnabled = YES;
}

- (void)filterAllAnnotWithTypes:(NSSet *)types{
    if (!types.count) {
        self.pageIndexMapToPageFilterAnnots = nil;
        self.allFilterAnnotsMapToAllPageAnnots = nil;
        self.allFilterAnnots = nil;
        [self updateInfoInAdvance];
        return;
    }
    NSMutableArray *allFilterAnnots = @[].mutableCopy;
    NSMutableDictionary *allFilterAnnotsMapToAllPageAnnots = @{}.mutableCopy;
    NSMutableDictionary *pageIndexMapToPageFilterAnnots = @{}.mutableCopy;
    for (NSArray *nodearray in self.allPageAnnots) {
        NSMutableArray *filterAPageAnnot = @[].mutableCopy;
        for (FSAnnotationItem *item in nodearray) {
            if ([types containsObject:@([item.annot getFilterAnnotType])] || [filterAPageAnnot containsObject:item.replyToAnnotItem]) {
                if (item.isTopReply && ![types containsObject:@([item.replyToAnnotItem.annot getFilterAnnotType])]) continue;
                if (!item.isTopReply && item.isReply) continue;
                NSString *key = [NSString stringWithFormat:@"%p", filterAPageAnnot];
                if (![allFilterAnnotsMapToAllPageAnnots valueForKey:key]) {
                    [allFilterAnnotsMapToAllPageAnnots setValue:nodearray forKey:key];
                }
        
                key = [NSString stringWithFormat:@"%d",item.annot.pageIndex];
                if (![pageIndexMapToPageFilterAnnots valueForKey:key]) {
                    [pageIndexMapToPageFilterAnnots setValue:filterAPageAnnot forKey:key];
                }
                
                item.filterLocPageItmes = filterAPageAnnot;
                if ([filterAPageAnnot containsObject:item]) continue;
                [filterAPageAnnot addObject:item];
                if (item.isTopReply) {
                    FSTopReplyAnnotationItem *topRelyItem = (FSTopReplyAnnotationItem *)item;
                    if (topRelyItem.unfold) [filterAPageAnnot addObjectsFromArray:topRelyItem.allReplies];
                }
                if (!item.isReply) {
                    filterAPageAnnot.fs_notReplyItemCount++;
                }
            }
        }
        if (filterAPageAnnot.count)  [allFilterAnnots addObject:filterAPageAnnot];
    }
    self.allFilterAnnots = allFilterAnnots;
    self.allFilterAnnotsMapToAllPageAnnots = allFilterAnnotsMapToAllPageAnnots;
    self.pageIndexMapToPageFilterAnnots = pageIndexMapToPageFilterAnnots;
    [self updateInfoInAdvance];
}

#pragma mark - IPanelChangedListener
- (void)onPanelChanged:(BOOL)isHidden {
    if (isHidden) {
        self.editing = NO;
    }
}

#pragma mark - rewrite

- (void)viewDidLoad {
    [super viewDidLoad];

    if (!self.isGroupList) {
        self.placeholder = FSLocalizedForKey(@"kSearch");
        self.cancelTitle = FSLocalizedForKey(@"kCancel");
        [self.editLeftItem setTitle:FSLocalizedForKey(@"kSelectAll") forState:UIControlStateNormal];
        [self.editLeftItem setTitle:FSLocalizedForKey(@"kDeselectAll") forState:UIControlStateSelected];
        [self.editRightItem setTitle:@"" forState:UIControlStateNormal];
        [self.editRightItem setImage:FSImageNamedAndNormalColor(@"common_delete_gray") forState:UIControlStateNormal];
        self.editRightItem.accessibilityLabel = FSLocalizedForKey(@"kDelete");
        [self.bottomBar addItem:self.filterItem atPosition:FSItemPositionCenter];
    }else{
        self.navBarLeftTitle = FSLocalizedForKey(@"kClose");
        self.title = FSLocalizedForKey(@"kGroupList");
    }
    
    if (!self.isGroupList) {
        [_extensionsManager registerAnnotEventListener:self];
        [_annotPanel.panelController registerPanelChangedListener:self];
        self.loadAnnotsQueue = nil;
        self.flattenAnnotBeforeCount = -1;
    }
    self.cellSelectedColor = UIColor.clearColor;
    
    [self.tableView fs_registerClass:[FSAnnoationWithouAuthorSubjectEstimatedTableViewCell class]];
    [self.tableView fs_registerClass:[FSAnnoationWithouAuthorDescriptionEstimatedTableViewCell class]];
}

- (BOOL)searchBarShouldBeginEditing:(UISearchBar *)searchBar{
    if (self.editing) {
        self.editing = NO;
    }
    [_panelController.panel.pagerContentView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(_panelController.panel.contentView.fs_mas_left);
        make.right.mas_equalTo(_panelController.panel.contentView.fs_mas_right);
        make.bottom.top.fs_equalToSuperview();
    }];
    [_panelController.panel.contentView layoutIfNeeded];
    _panelController.panel.pagerContentView.scrollEnabled = NO;
    return YES;
}

- (void)setSearchBarHidden:(BOOL)hidden{
    if (!self.isGroupList) {
        if (hidden && self.allSearchPageAnnots) {
            return;
        }
        [super setSearchBarHidden:hidden];
    }
}

- (void)enteredSearchState{
    [super enteredSearchState];
    NSMutableArray *allSearchPageAnnots = @[].mutableCopy;
    for (NSArray *nodearray in self.dataSource) {
        NSMutableArray *searchAPageAnnot = @[].mutableCopy;
        for (FSAnnotationItem *item in nodearray) {
            if (item.isReply) {
                if (item.isTopReply) {
                    [searchAPageAnnot addObject:item];
                    FSTopReplyAnnotationItem *topReply = (FSTopReplyAnnotationItem *)item;
                    [searchAPageAnnot addObjectsFromArray:topReply.allReplies];
                }else{
                    continue;
                }
            }else{
                [searchAPageAnnot addObject:item];
                searchAPageAnnot.fs_notReplyItemCount++;
            }
        }
        if (searchAPageAnnot.count)  [allSearchPageAnnots addObject:searchAPageAnnot];
    }
    self.allSearchPageAnnots = allSearchPageAnnots.copy;
    [self reloadData];
}

- (void)setEditing:(BOOL)editing{
    [super setEditing:editing];
    _annotPanel.rightItem.selected = editing;
}

- (NSOperationQueue *)calculateAnnotsQueue{
    if(!_calculateAnnotsQueue){
        _calculateAnnotsQueue = [[NSOperationQueue alloc] init];
        _calculateAnnotsQueue.maxConcurrentOperationCount = 5;
        _calculateAnnotsQueue.name = @"foxit.calculate.annot.can.delete";
    }
    return _calculateAnnotsQueue;
}

- (void)updateInfoInAdvance{
    [self.calculateAnnotsQueue cancelAllOperations];
    [self.calculateAnnotsQueue waitUntilAllOperationsAreFinished];
    
    __block NSUInteger canDeleteAnnotCounts = 0;
    for (NSMutableArray *pageAnnont in self.dataSource) {
        pageAnnont.fs_notReplyItemCount = 0;
        NSBlockOperation *op = [[NSBlockOperation alloc] init];
        @weakify(self)
        @weakify(op)
        [op addExecutionBlock:^{
            @strongify(self)
            @strongify(op)
            for (FSAnnotationItem *annotItem in pageAnnont) {
                if (op.isCancelled) break;
                if (!annotItem.isReply) {
                    pageAnnont.fs_notReplyItemCount ++;
                }
                if ([self annotationListItemCanDelete:annotItem]) {
                    canDeleteAnnotCounts++;
                }
            }
        }];
        [self.calculateAnnotsQueue addOperation:op];
    }
    [self.calculateAnnotsQueue waitUntilAllOperationsAreFinished];
    self.canDeleteAnnotCounts = canDeleteAnnotCounts;
}

- (void)setSelectedIndexPaths:(NSArray<NSIndexPath *> *)selectedIndexPaths{
    [super setSelectedIndexPaths:selectedIndexPaths];
    self.editLeftItem.selected = self.selectedIndexPaths.count == self.canDeleteAnnotCounts;
}

- (void)clickedEditLeftItem{
    if ([self allPageAnnots].count > 0) {
        self.editLeftItem.selected = !self.editLeftItem.selected;
        if (self.editLeftItem.selected) {
            NSInteger section = [self numberOfSectionsInTableView:self.tableView];
            for (int s = 0 ; s < section; s++ ) {
                NSInteger row = [self tableView:self.tableView numberOfRowsInSection:s];
                for (int r = 0; r < row; r++) {
                    NSIndexPath *indexPath = [NSIndexPath indexPathForRow:r inSection:s];
                    FSAnnotationItem *item = [self dataForAtIndexPath:indexPath];
                    if ([self annotationListItemCanDelete:item]) {
                        [self selectRow:YES atIndexPath:indexPath];
                        [self selectDataSourceRow:YES atIndexPath:indexPath];
                    }
                }
            }
        }else{
            [self deselectAll];
        }
    }
}

- (void)clickedEditRightItem{
    if ([self selectedIndexPaths].count > 0 && [self selectedDataSource].count > 0) {
        void(^block)(void) = ^{
            [self->_extensionsManager.viewCtrlManager.navigationControllerStack.topNavigationController.view fs_scheduledMainQueueShowHUDLoading:FSLocalizedForKey(@"kDeleting") process:^{
                [self deleteAnnotations:self.selectedDataSource];
                if ([self emptyDataSetDisplayed] || !self.canDeleteAnnotCounts) {
                    self.editing = NO;
                    self->_annotPanel.rightItem.enabled = NO;
                }
                self->_extensionsManager.isDocModified = YES;
                [super clickedEditRightItem];
            }];
        };
        if (self.selectedDataSource.count != self.canDeleteAnnotCounts) {
            block();
        }else{
            [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kConfirm") message:FSLocalizedForKey(@"kClearAnnotations") actions:@[ FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                if (index == 1) {
                    block();
                }
            } onCtrl:[self->_pdfViewCtrl fs_currentSceneTopMostViewController]];
        }
    }
}

- (void)resetAnnotationArray {
    [self.allPageAnnots removeAllObjects];
    [self.allFilterAnnotsMapToAllPageAnnots removeAllObjects];
    [self.pageIndexMapToPageAnnots removeAllObjects];
    [self.pageIndexMapToPageFilterAnnots removeAllObjects];
    
    [self filterAllAnnotWithTypes:nil];
    self.annotFilterItems = nil;
}

- (void)rearrangePageIndexMapToPageAnnots{
    self.pageIndexMapToPageAnnots = @{}.mutableCopy;
    for (NSMutableArray *pageAnnots in self.allPageAnnots) {
        for (FSAnnotationItem *item in pageAnnots) {
            if (item.annot && ![item.annot isEmpty]) {
                [self.pageIndexMapToPageAnnots setValue:pageAnnots forKey:[NSString stringWithFormat:@"%d",item.annot.pageIndex]];
                break;
            }
            
        }
    }
}

- (void)cleanInvalidAnnot{
    for (NSMutableArray *pageAnnots in self.allPageAnnots.copy) {
        for (FSAnnotationItem *item in pageAnnots.copy) {
            if (!item.annot || [item.annot isEmpty]) {
                [pageAnnots removeObject:item];
            }
        }
        if (!pageAnnots.count) {
            [self.allPageAnnots removeObject:pageAnnots];
        }
    }
    [self rearrangePageIndexMapToPageAnnots];
    [self reloadData];
}

#pragma mark IPageEventListener
- (void)onPagesWillRemove:(NSArray<NSNumber *> *)indexes {
}

- (void)onPagesWillMove:(NSArray<NSNumber *> *)indexes dstIndex:(int)dstIndex {
}

- (void)onPagesWillRotate:(NSArray<NSNumber *> *)indexes rotation:(int)rotation {
}

- (void)onPagesRemoved:(NSArray<NSNumber *> *)indexes {
    for (NSNumber *index in indexes) {
        NSString *pageIndexStr = index.stringValue;
        NSMutableArray *pageAnnots = [self.pageIndexMapToPageAnnots valueForKey:pageIndexStr];
        [self.allPageAnnots removeObject:pageAnnots];
    }
    if ([self filterState]) [self filterAllAnnotWithTypes:self.needFilterType];
    [self rearrangePageIndexMapToPageAnnots];
    [self updateInfoInAdvance];
    [self reloadData];
}

- (void)onPagesMoved:(NSArray<NSNumber *> *)indexes dstIndex:(int)dstIndex {
    NSMutableArray *tmp = @[].mutableCopy;
    for (NSNumber *index in indexes) {
        NSString *pageIndexStr = index.stringValue;
        NSMutableArray *pageAnnots = [self.pageIndexMapToPageAnnots valueForKey:pageIndexStr];
        if (pageAnnots) {
            [self.allPageAnnots removeObject:pageAnnots];
            [tmp addObject:pageAnnots];
        }
    }
//    if (!tmp.count) {
//        return;
//    }
    ///-[FSReorderableCollectionView handleLongPressGesture:] line:124
    /// I don't know why
    if (tmp.count) {
        if (dstIndex == 0) {
            [self.allPageAnnots fs_insertObjects:tmp atIndex:0];
        }else{
            [self.allPageAnnots fs_insertObjects:tmp atIndex:dstIndex+1];
        }
    }

    if ([self filterState]) [self filterAllAnnotWithTypes:self.needFilterType];
    [self rearrangePageIndexMapToPageAnnots];
    [self updateInfoInAdvance];
    [self reloadData];
}

- (void)onPagesInsertedAtRange:(NSRange)range {
    if (range.location == NSNotFound || range.length == 0) {
        return;
    }
    NSMutableArray *tmp = @[].mutableCopy;
    for (int i = (int)range.location; i < range.location + range.length; i++) {
        NSArray *items = [self getAnnotationItemsForPageAtIndex:i];
        if (items.count) {
            [tmp addObject:items];
        }
    }
    if (tmp.count) {
        
        NSUInteger index = [self findIndexInAllPageAnnotsForPageIndex:range.location];
        
        [self.allPageAnnots fs_insertObjects:tmp atIndex:index];
        
        if ([self filterState]) [self filterAllAnnotWithTypes:self.needFilterType];
        [self rearrangePageIndexMapToPageAnnots];
        [self updateInfoInAdvance];
        [self reloadData];
    }
}

- (NSUInteger)findIndexInAllPageAnnotsForPageIndex:(NSUInteger)pageIndex{
    
    NSUInteger index = 0;
    for (int i = 0; i <  self.allPageAnnots.count; i++) {
        NSMutableArray *_pageAnnots = self.allPageAnnots[i];
        FSAnnotationItem *_annotItem = [_pageAnnots firstObject];
        if (_annotItem) {
            int _index = [_annotItem.annot pageIndex];
            if ((index < pageIndex && pageIndex < _index) || pageIndex == _index) {
                return  i;
            }
            index = _index;
        }else{
            continue;
        }
    }
    return self.allPageAnnots.count;
}

#pragma mark - IAnnotEventListener

- (void)onAnnotAdded:(FSPDFPage *)page annot:(FSAnnot *)annot {
    FSAnnotType fileType = [annot getFilterAnnotType];
    if(self.isReflow){
        if(annot.type != FSAnnotHighlight && annot.type != FSAnnotUnderline && annot.type != FSAnnotSquiggly && annot.type != FSAnnotStrikeOut){
            return;
        }
    }
    if (!self.needAnnotEvent ||
        annot.type == FSAnnotWidget ||annot.type == FSAnnotMovie || annot.type == FSAnnotRichMedia ||annot.type == FSAnnotLink || fileType == FSAnnotVideo || fileType == FSAnnotAudio ||
        (annot.type == FSAnnotStrikeOut && [Utility isReplaceText:[[FSStrikeOut alloc] initWithAnnot:annot]]) ||
        (annot.type == FSAnnotScreen && fileType == FSAnnotURI)) {
        return;
    }
    
    int pageIndex = page.getIndex;
    NSString *key = [NSString stringWithFormat:@"%d",pageIndex];
    NSMutableArray *pageAnnots = [self.pageIndexMapToPageAnnots valueForKey:key];
    NSMutableArray *filterPageAnnots = [self.pageIndexMapToPageFilterAnnots valueForKey:key];
    
    if (!pageAnnots) {
        NSUInteger index = [self findIndexInAllPageAnnotsForPageIndex:pageIndex];
        pageAnnots = @[].mutableCopy;
        [self.allPageAnnots insertObject:pageAnnots atIndex:index];
        
        if (!self.pageIndexMapToPageAnnots) self.pageIndexMapToPageAnnots = @{}.mutableCopy;
        [self.pageIndexMapToPageAnnots setValue:pageAnnots forKey:key];
    }
    
    FSAnnotationItem *annoItem;
    if (annot.getType == FSAnnotNote) {
        FSNote *note = [[FSNote alloc] initWithAnnot:annot];
        FSMarkup *replyTo = [note getReplyTo];
        if (replyTo && ![replyTo isEmpty]) {
            
            if (!replyTo.replyTo) {
                annoItem = [[FSTopReplyAnnotationItem alloc] initWithAnnot:annot extensionsManager:_extensionsManager];
                for (FSAnnotationItem *replyToAnnotItem in pageAnnots) {
                    if ([replyToAnnotItem.annot isEqualToAnnot:replyTo]) {
                        annoItem.locPageItmes = pageAnnots;
                        [replyToAnnotItem addReplyItem:annoItem];
                        
                        if (filterPageAnnots) annoItem.filterLocPageItmes = filterPageAnnots;
                        
                        break;
                    }
                }
            }else{
                annoItem = [[FSAnnotationItem alloc] initWithAnnot:annot isReply:YES extensionsManager:_extensionsManager];
                
                for (FSAnnotationItem *replyToAnnotItem in pageAnnots) {
                    if ([replyToAnnotItem.annot isEqualToAnnot:replyTo]) {
                        [replyToAnnotItem addReplyItem:annoItem];
                        break;
                    }
                    if (replyToAnnotItem.isTopReply) {
                        FSTopReplyAnnotationItem *topReplyItem = (FSTopReplyAnnotationItem *)replyToAnnotItem;
                        for (FSAnnotationItem *subReplyToAnnotItem in topReplyItem.allReplies) {
                            if ([subReplyToAnnotItem.annot isEqualToAnnot:replyTo]) {
                                [subReplyToAnnotItem addReplyItem:annoItem];
                                break;
                            }
                        }
                        if ([topReplyItem.allReplies containsObject:annoItem]) {
                            break;
                        }
                    }
                }
            }
        }else{
            goto createAnnotItem;
        }
    }else{
        createAnnotItem:{annoItem = [[FSAnnotationItem alloc] initWithAnnot:annot isReply:NO extensionsManager:_extensionsManager];}
    }
    
    if (!annoItem.isReply && annoItem) {
        [pageAnnots addObject:annoItem];
        annoItem.locPageItmes = pageAnnots;
        if ([self filterState]) [self filterAllAnnotWithTypes:self.needFilterType];
    }
    [self updateInfoInAdvance];
    [self reloadData];
}

- (void)onAnnotWillDelete:(FSPDFPage *)page annot:(FSAnnot *)annot {
    //If currently is clearing all the annotations, then we don't reload the data here for better performance.
    if (!self.needAnnotEvent) return;
    [self allPageAnnotsDeleteAnnot:annot];
    if ([self filterState]) [self filterAllAnnotWithTypes:self.needFilterType];
    [self updateInfoInAdvance];
    [self reloadData];
}

- (void)onAnnotWillFlatten:(FSPDFPage *)page annot:(FSAnnot *)annot{
    //If currently is clearing all the annotations, then we don't reload the data here for better performance.
    if (!self.needAnnotEvent) return;
    [self allPageAnnotsDeleteAnnot:annot];
    if ([self filterState]) [self filterAllAnnotWithTypes:self.needFilterType];
    [self updateInfoInAdvance];
    [self reloadData];

}

- (void)onAnnotModified:(FSPDFPage *)page annot:(FSAnnot *)annot {
    if (annot.type == FSAnnotStrikeOut && [Utility isReplaceText:[[FSStrikeOut alloc] initWithAnnot:annot]]) {
        return;
    }
    [self reloadData];
}

- (void)onAnnotsWillDelete:(NSArray <FSAnnot *> *)annots{
    if (self.allPageAnnots.count > 100 || annots.count > 300) {
        [self cancelLoadAnnotations];
        [self resetAnnotationArray];
    }else{
        int allAnnotCount = 0;
        for (NSArray *pageAnnots in self.allPageAnnots) {
            allAnnotCount += pageAnnots.count;
            if (allAnnotCount > 600) {
                [self cancelLoadAnnotations];
                [self resetAnnotationArray];
                break;
            }
        }
        if (allAnnotCount <= 600) {
            for (FSAnnot *annot in annots) {
                [self allPageAnnotsDeleteAnnot:annot];
            }
            if ([self filterState]) [self filterAllAnnotWithTypes:self.needFilterType];
            [self updateInfoInAdvance];
            [self reloadData];
        }
    }
}

- (void)allPageAnnotsDeleteAnnot:(FSAnnot *)annot{
    if (annot.type == FSAnnotStrikeOut && [Utility isReplaceText:[[FSStrikeOut alloc] initWithAnnot:annot]]) {
        return;
    }
    if(self.isReflow){
        if(annot.type != FSAnnotHighlight && annot.type != FSAnnotUnderline && annot.type != FSAnnotSquiggly && annot.type != FSAnnotStrikeOut){
            return;
        }
    }
    int pageIndex = annot.getPage.getIndex;
    NSString *key = [NSString stringWithFormat:@"%d",pageIndex];
    NSMutableArray *pageAnnots = [self.pageIndexMapToPageAnnots valueForKey:key];
    
    BOOL foundReply = NO;
    for (FSAnnotationItem *annotItem in pageAnnots.copy) {
        if (annotItem.isTopReply) {
            FSTopReplyAnnotationItem *topReply = (FSTopReplyAnnotationItem *)annotItem;
            if (!topReply.unfold) {
                for (FSAnnotationItem *replyItem in topReply.allReplies.copy) {
                    if ([annot isEqualToAnnot:replyItem.annot]) {
                        [topReply removeReplyItem:replyItem];
                        foundReply = YES;
                        break;
                    }
                }
            }
        }
        if (foundReply) break;
        if ([annot isEqualToAnnot:annotItem.annot]) {
            if (annotItem.isReply) {
                if (annotItem.isTopReply) {
                    FSTopReplyAnnotationItem *topReply = (FSTopReplyAnnotationItem *)annotItem;
                    if (topReply.unfold) {
                        [pageAnnots removeObjectsInArray:topReply.allReplies];
                    }
                    [annotItem.replyToAnnotItem removeReplyItem:annotItem];
                }else{
                    FSTopReplyAnnotationItem *topReply = annotItem.topReplyAnnotItem;
                    [topReply removeReplyItem:annotItem];
                    if (topReply.unfold) {
                        FSTopReplyAnnotationItem *subTopReply = [[FSTopReplyAnnotationItem alloc] initWithAnnot:annotItem.annot extensionsManager:_extensionsManager];
                        NSMutableArray *subRepliesNM = subTopReply.repliesNM.mutableCopy;
                        if (subRepliesNM.count) {
                            for (FSAnnotationItem *replyItem in topReply.allReplies.copy) {
                                if (!subRepliesNM.count) break;
                                if ([subRepliesNM containsObject:replyItem.annot.NM]) {
                                    [pageAnnots removeObject:replyItem];
                                    [topReply removeReplyItem:replyItem];
                                    [subRepliesNM removeObject:replyItem.annot.NM];
                                }
                            }
                        }
                    }
                    [annotItem.replyToAnnotItem removeReplyItem:annotItem];
                }
            }else{
                if (annotItem.allReplies.count) {
                    [pageAnnots removeObjectsInArray:annotItem.allReplies];
                }
            }
            if ([pageAnnots containsObject:annotItem]) {
                [pageAnnots removeObject:annotItem];
            }
            break;
        }
    }
    
    if (!pageAnnots.count) {
        [self.pageIndexMapToPageAnnots removeObjectForKey:key];
        [self.allPageAnnots removeObject:pageAnnots];
    }
}

- (void)onAnnotsAdded:(NSArray<FSAnnot *> *)annots{
    NSInteger tmptag = 0;
    for (FSAnnot *annot in annots) {
        if ( annot.type == FSAnnotWidget ||
            (annot.type == FSAnnotStrikeOut && [Utility isReplaceText:[[FSStrikeOut alloc] initWithAnnot:annot]])) {
            continue;
        }
        if(self.isReflow){
            if(annot.type != FSAnnotHighlight && annot.type != FSAnnotUnderline && annot.type != FSAnnotSquiggly && annot.type != FSAnnotStrikeOut){
                return;
            }
        }
        FSPDFPage *page = annot.getPage;
        int pageIndex = page.getIndex;
        NSString *key = [NSString stringWithFormat:@"%d",pageIndex];
        NSMutableArray *pageAnnots = [self.pageIndexMapToPageAnnots valueForKey:key];
        NSMutableArray *filterPageAnnots = [self.pageIndexMapToPageFilterAnnots valueForKey:key];
        
        if (!pageAnnots) {
            NSUInteger index = [self findIndexInAllPageAnnotsForPageIndex:pageIndex];
            pageAnnots = @[].mutableCopy;
            [self.allPageAnnots insertObject:pageAnnots atIndex:index];
            
            if (!self.pageIndexMapToPageAnnots) self.pageIndexMapToPageAnnots = @{}.mutableCopy;
            [self.pageIndexMapToPageAnnots setValue:pageAnnots forKey:key];
        }
        
        FSAnnotationItem *annoItem = [[FSAnnotationItem alloc] initWithAnnot:annot isReply:NO extensionsManager:_extensionsManager];
        annoItem.locPageItmes = pageAnnots;
        [pageAnnots addObject:annoItem];
        
        tmptag++;
    }
    if (tmptag) {
        
        dispatch_main_async_safe(^{
            if ([self filterState]) [self filterAllAnnotWithTypes:self.needFilterType];
            [self updateInfoInAdvance];
            [self reloadData];
        });
    }
}


- (void)onCurrentAnnotChanged:(FSAnnot *)lastAnnot currentAnnot:(FSAnnot *)currentAnnot {
}


- (void)reloadDataList:(NSNotification *)notf{
    FSMessageDebouncePerformDefault(^{
        if (self.isGroupList) {
            NSMutableArray *groupAnnotItems = [self.groupListAnnots firstObject];
            for (FSAnnotationItem *items in groupAnnotItems.copy) {
                FSAnnot *annot = [[FSAnnot alloc] initWithAnnot:items.annot];
                if ([annot isEmpty] || (self.flattenAnnotBeforeCount != [[annot getPage] getAnnotCount] && self.flattenAnnotBeforeCount != -1)) {
                    [self navigationBarBackTouch];
                    self.groupListAnnots = nil;
                    break;
                }
            }
        }
        [self loadData];
    });
}

- (BOOL)emptyDataSetShouldDisplay:(UITableView *)tableView{
    BOOL emptyDataSetShouldDisplay = [super emptyDataSetShouldDisplay:tableView];
    if (!self.isGroupList) {
        self.bottomBar.hidden = !self.allPageAnnots.count;
        if (!self.view.hidden){
            _annotPanel.rightItem.enabled = !emptyDataSetShouldDisplay && self.canDeleteAnnotCounts;
        }
    }
    if (!self.searchBarContentView.hidden && emptyDataSetShouldDisplay){
        [self setSearchBarHidden:YES];
    }
    return emptyDataSetShouldDisplay;
}


- (NSString *)titleForEmptyDataSet{
    if (self.keywords.length) return FSLocalizedForKey(@"kNoResultsTip");
    return FSLocalizedForKey(@"kNoComments");
}

- (UIImage *)imageForEmptyDataSet:(UITableView *)tableView {
    if (self.keywords.length) return FSImageNamedAndPrimaryColor(@"search_result_no_data");
    return FSImageNamedAndPrimaryColor(@"panel_comment_no_data");
}

- (BOOL)evaluateWithObject:(FSAnnotationItem *)annotItem{
    @try {
        return [super evaluateWithObject:annotItem.annot.contents] || [super evaluateWithObject:annotItem.annot.author];
    } @catch (NSException *exception) {
        return NO;
    }
    return NO;
}

- (NSArray<UILabel *> *)keywordsHighlightLabels:(FSDescriptionEstimatedTableViewCell *)cell{
    return [cell isKindOfClass:[FSSubjectEstimatedTableViewCell class]] ? @[cell.estimatedTitleLabel] :  @[cell.estimatedTitleLabel, cell.estimatedDescriptionLabel];
}

- (NSString *)titles1ForAtSection:(NSUInteger)section{
    if (self.keywords.length) {
        return [NSString stringWithFormat:@"%@", FSLocalizedForKey(@"kTotalFound")];
    }
    FSAnnotationItem *item = [self dataForAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:section]];
    return  item ? [NSString stringWithFormat:FSLocalizedForKey(@"kPage"), item.annot.pageIndex + 1] : nil;
}

- (NSString *)titles2ForAtSection:(NSUInteger)section{
    if (!self.dataSource.count) return nil;
    if (self.keywords.length) return [NSString stringWithFormat:@"%lu",(unsigned long)self.searchResultSoruce.count];
    if (self.dataSource.count > section) {
        NSMutableArray *annotItems = [self.dataSource objectAtIndex:section];
        return annotItems ? [NSString stringWithFormat:@"%lu", (unsigned long)annotItems.fs_notReplyItemCount] : nil;
    }
    return nil;
}

- (__kindof UITableViewCell *)tableView:(UITableView *)tableView dequeueReusableCellWithIdentifier:(NSString *)identifier forIndexPath:(NSIndexPath *)indexPath withDataItem:(FSAnnotationItem *)annoItem{
    if (!annoItem.annot.contents.length) {
        if (annoItem.annot.author.length) {
            identifier = NSStringFromClass([FSSubjectEstimatedTableViewCell class]);
        }else{
            identifier = NSStringFromClass([FSAnnoationWithouAuthorSubjectEstimatedTableViewCell class]);
        }
    }else{
        if (annoItem.annot.author.length) {
            identifier = NSStringFromClass([FSDescriptionEstimatedTableViewCell class]);
        }else{
            identifier = NSStringFromClass([FSAnnoationWithouAuthorDescriptionEstimatedTableViewCell class]);
        }
    }
    return  [super tableView:tableView dequeueReusableCellWithIdentifier:identifier forIndexPath:indexPath withDataItem:annoItem];
}

- (void)configureCell:(FSDescriptionEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(FSAnnotationItem *)annoItem{
    if (!cell.installed) {
        cell.editTopOffset = ITEM_IMAGE_TOP_MARGIN_NORMAL + 7;
        cell.leftMargin = 9.f;
    }
    cell.hightlightBackgroundColor = self.fs_dividerColor_P1;
    if (!cell.leftView) {
        UIView *leftView = [UIView new];
        cell.leftView = leftView;
    }
    
    if (!cell.rightView) {
        FSItem *moreItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"common_more_gray")];
        moreItem.accessibilityLabel = FSLocalizedForKey(@"kMore");
        cell.rightView = moreItem;
    }
    
    cell.titleDefaultFont = self.fs_standardWeakFont_WF2;
    cell.rightView.hidden = [self searchState];
    cell.estimatedTitleLabel.text = annoItem.annot.author;
    cell.estimatedSubjectLabel.text = [Utility displayDateInMDYHM:annoItem.annot.modifiedDate];
    cell.contentView.backgroundColor = [UIColor clearColor];
    [cell.leftView.subviews makeObjectsPerformSelector:@selector(removeFromSuperview)];
    if (annoItem.isReply && ![self searchState]) {
        FSItem *unfold = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"panel_unfold_+_gray") imageSelected:FSImageNamedAndNormalColor(@"panel_unfold_-_gray")];
        unfold.accessibilityLabel = FSLocalizedForKey(@"kUnfoldVoiceOver");
        if (annoItem.isTopReply) {
            FSTopReplyAnnotationItem *replyItem = (FSTopReplyAnnotationItem *)annoItem;
            ///Unfold and fold
            @weakify(self)
            unfold.onTapClick = ^(FSItem * _Nonnull item) {
                @strongify(self)
                [self clickedUnfoldItem:item replyItem:replyItem inSection:indexPath.section];
            };
            
            if (replyItem.allReplies.count < 1) {
                replyItem.unfold = NO;
                unfold.hidden = YES;
            }
            unfold.selected = replyItem.unfold;
        }else{
            unfold.hidden = YES;
        }
        
        UIImageView *turnImageView = [[UIImageView alloc] initWithImage:FSImageNamedAndNormalColor(@"panel_reply_arrow_gray")];
        [cell.leftView addSubview:unfold];
        [cell.leftView addSubview:turnImageView];
        
        [unfold mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.top.bottom.fs_equalToSuperview();
        }];

        
        [turnImageView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(unfold.mas_right).offset(0);
            make.size.mas_equalTo(turnImageView.image.size);
            make.centerY.mas_equalTo(unfold);
            make.right.fs_equalToSuperview();
        }];
        
        cell.estimatedTitleLabel.text = [NSString stringWithFormat:@"%@ to %@", annoItem.annot.author ? annoItem.annot.author : @"", annoItem.replyToAnnotItem.annot.author ? annoItem.replyToAnnotItem.annot.author : @""];
    }else{
        UIImageView *annotImage = [UIImageView new];
        if (!annoItem.isReply) {
            annotImage.image = FSImageNamedAndNormalColor([annoItem getAnnotImageName]);
            annotImage.accessibilityIdentifier = [NSString stringWithFormat:@"%@_id",[annoItem getAnnotImageName]];
        }else{
            annotImage.image = FSImageNamedAndNormalColor(@"panel_reply_arrow_gray_40");
        }
        [cell.leftView addSubview:annotImage];
        
        [annotImage mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.top.right.bottom.fs_equalToSuperview();
        }];
    }
    
    BOOL canEdit = [self annotationListItemCanEdit:annoItem];
    NSMutableArray *items = @[].mutableCopy;
    
    BOOL isGroup = NO;
    if (annoItem.annot.canGetGroupHeader && !([annoItem.annot isMarkup] && [Utility isReplaceText:[[FSMarkup alloc] initWithAnnot:annoItem.annot]]) && !self.isGroupList) {
        isGroup = YES;
        [items addObject:[[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kGroupList") image:FSImageNamedAndNormalColor(@"panel_group_list_gray") action:^(FSPopoverItem * _Nonnull item, int index) {
            FSAnnotationListViewController *VC = [[FSAnnotationListViewController alloc] initWithExtensionsManager:self->_extensionsManager module:self->_annotPanel];
            [VC loadGroupListWithGroupAnnotItems:[self getAnnotationGroupElements:annoItem]];
            VC.isGroupList = YES;
            VC.shouldShowBottomBar = NO;
            VC.shouldShowNavigationBar = YES;
            [self.services presentViewController:VC animated:YES completion:nil];
        }]];
    }
    if ([self annotationListItemCanReply:annoItem]) {
        [items addObject:[[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kReply") image:FSImageNamedAndNormalColor(@"panel_reply_gray") action:^(FSPopoverItem * _Nonnull item, int index) {
            [self replyToAnnotation:[self getAnnotationHeaderGroupItem:annoItem] atIndexPath:indexPath];
        }]];
    }
    if (canEdit) {
        [items addObject:[[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kNote") image:FSImageNamedAndNormalColor(@"panel_note_gray") action:^(FSPopoverItem * _Nonnull item, int index) {
            [self addNoteToAnnotation:annoItem atIndexPath:indexPath];
        }]];
    }
    if ([self annotationListItemCanFlatten:annoItem]) {
        [items addObject:[[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kFlatten") image:FSImageNamedAndNormalColor(@"panel_flatten_gray") action:^(FSPopoverItem * _Nonnull item, int index) {
            [self->_extensionsManager.permissionProvider checkPermission:FSFunctionFlatten checkPermissionState:^(FSPermissionState state) {
                if (state == FSPermissionStateShow) {
                    [self annotationListItemFlatten:annoItem atIndexPath:indexPath];
                }
            }];
        }]];
    }
    if ([self annotationListItemCanDelete:annoItem]) {
        [items addObject:[[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kDelete") image:FSImageNamedAndNormalColor(@"common_delete_gray") action:^(FSPopoverItem * _Nonnull item, int index) {
            [self annotationListItemDelete:annoItem atIndexPath:indexPath];
        }]];
    }
    
    ((FSItem *)cell.rightView).enabled = items.count;
    @fs_weakify(self)
    @fs_weakify(cell)
    ((FSItem *)cell.rightView).onTapClick = ^(FSItem * _Nonnull _item) {
        @fs_strongify(self)
        @fs_strongify(cell)
        if (!annoItem.annot.nonempty){
            [self cleanInvalidAnnot];
            return;
        }
        annoItem.isShowMore = YES;
        if ([_item fs_currentSceneIsCompact]) [cell setHighlighted:YES color:self.fs_backgroundColor_B2 custom:YES];
        if (isGroup) {
            NSArray *curGroutAnnotItems = [self getAnnotationGroupElements:annoItem];
            for (NSIndexPath *indexPath in self.tableView.indexPathsForVisibleRows) {
                FSAnnotationItem *annotItem = [self dataForAtIndexPath:indexPath];
                if (annotItem.isReply) annotItem = annotItem.topReplyAnnotItem.replyToAnnotItem;
                if ([curGroutAnnotItems containsObject:annotItem] && annotItem.annot.canGetGroupHeader) {
                    FSTableViewCell *cell = [self.tableView cellForRowAtIndexPath:indexPath];
                    [cell setHighlighted:YES color:self.fs_backgroundColor_B2 custom:YES];
                    annotItem.isShowMore = YES;
                }
            }
        }
        [self->_extensionsManager.viewCtrlManager showMoreMenuWithItems:items.copy dismissCompletion:^{
            if (isGroup) {
                for (FSTableViewCell *cell in self.tableView.visibleCells) {
                    [cell setHighlighted:NO color:self.fs_backgroundColor_B2 custom:YES];
                }

                NSArray *curGroutAnnotItems = [self getAnnotationGroupElements:annoItem];
                [curGroutAnnotItems enumerateObjectsUsingBlock:^(FSAnnotationItem * _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
                    obj.isShowMore = NO;
                }];
            }else{
                [cell setHighlighted:NO color:self.fs_backgroundColor_B2 custom:YES];
                annoItem.isShowMore = NO;
            }
        } sourceView:![_item fs_currentSceneIsCompact] ? ^UIView * _Nonnull(UIPopoverPresentationController * _Nonnull popver) {
            popver.permittedArrowDirections = UIPopoverArrowDirectionRight;
            return _item;
        } : nil];
        
    };
    
    cell.editItem.hidden = ![self annotationListItemCanDelete:annoItem];
    if ([cell respondsToSelector:@selector(estimatedDescriptionLabel)]) {
        NSString *contents = annoItem.annot.contents;
        
        if ((contents == nil || contents.length == 0) && annoItem.annot.screenActiontype != FSScreenActionTypeUnScreen) {
            contents = annoItem.annot.ha_fileSpec.fileName;
        }
        cell.descriptionDefaultFont = self.fs_standardGeneralFont_GF3;
        cell.descriptionDefaultTextColor = self.fs_textColor_T4;
        cell.estimatedDescriptionLabel.text = contents;
        
        UIButton *editNoteClicked = [cell.contentView viewWithTag:100];
        if (canEdit) {
            if (!editNoteClicked) {
                editNoteClicked = [UIButton new];
                editNoteClicked.tag = 100;
                [cell.contentView addSubview:editNoteClicked];
                
                [editNoteClicked mas_makeConstraints:^(MASConstraintMaker *make) {
                    make.left.right.bottom.fs_equalToSuperview();
                    make.top.equalTo(cell.estimatedDescriptionLabel);
                }];
            }
            
            editNoteClicked.hidden = YES;
            
            if ([FSPermissionUtilty canAddAnnot:_extensionsManager] ) {
                editNoteClicked.hidden = NO;
                @weakify(cell)
                [editNoteClicked setBlockForControlEvents:UIControlEventTouchUpInside block:^(id  _Nonnull sender) {
                    @strongify(self)
                    @strongify(cell)
                    if (!cell.editing) [self addNoteToAnnotation:annoItem atIndexPath:indexPath];
                }];
            }
        }else{
            editNoteClicked.hidden = YES;
        }
    }
    [super configureCell:cell atIndexPath:indexPath withDataItem:annoItem];
    
    if ([self->_pdfViewCtrl fs_currentSceneIsCompact]) [cell setHighlighted:annoItem.isShowMore color:self.fs_backgroundColor_B2 custom:YES];
    else [cell setHighlighted:NO color:self.fs_backgroundColor_B2 custom:YES];
}

- (void)clickedUnfoldItem:(FSItem *)item replyItem:(FSTopReplyAnnotationItem *)replyItem inSection:(NSUInteger)section{
    
    NSMutableArray<FSAnnotationItem *> *currentPageAnnots = [self.dataSource objectAtIndex:section];
    NSMutableArray<FSAnnotationItem *> *pageAnnots = nil;
    NSMutableArray<FSAnnotationItem *> *filterPageAnnots = nil;
    if ([self filterState]) {
        NSString *key = [NSString stringWithFormat:@"%p",currentPageAnnots];
        pageAnnots = [self.allFilterAnnotsMapToAllPageAnnots valueForKey:key];
    
        filterPageAnnots = currentPageAnnots;
    }else{
        pageAnnots = currentPageAnnots;
    }
    
    NSUInteger row = [currentPageAnnots indexOfObject:replyItem];
    
    replyItem.unfold = !replyItem.unfold;
    item.selected = replyItem.unfold;
    if (item.selected) {
        item.accessibilityLabel = FSLocalizedForKey(@"kUnfoldVoiceOver");
    }else{
        item.accessibilityLabel = FSLocalizedForKey(@"kFoldVoiceOver");
    }
    NSMutableIndexSet *tmpSet = [NSMutableIndexSet indexSet];
    NSMutableArray *tmpIndexPaths = @[].mutableCopy;
    for (int i = 1; i < replyItem.allReplies.count+1; i ++ ) {
        NSUInteger loc =  i + row;
        [tmpSet addIndex:loc];
        NSIndexPath *_indexPath =  [NSIndexPath indexPathForRow:loc inSection:section];
        [tmpIndexPaths addObject:_indexPath];
    }
    if (replyItem.unfold) {
        if (filterPageAnnots) {
            NSRange range = NSMakeRange(row, 1);
            NSMutableArray *tmp = @[replyItem].mutableCopy;
            [tmp addObjectsFromArray:replyItem.allReplies];
            [pageAnnots replaceObjectsInRange:range withObjectsFromArray:tmp];
        }
        [currentPageAnnots insertObjects:replyItem.allReplies atIndexes:tmpSet];
        [self insertRowsAtIndexPaths:tmpIndexPaths.copy];
    }else{
        if (filterPageAnnots) {
            NSRange range = NSMakeRange(row, 1+replyItem.allReplies.count);
            [pageAnnots replaceObjectsInRange:range withObjectsFromArray:@[replyItem]];
        }
        [currentPageAnnots removeObjectsAtIndexes:tmpSet];
        [self deleteRowsAtIndexPaths:tmpIndexPaths];
    }
    
    [self updateInfoInAdvance];
}

#pragma mark - Table view data source
- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section{
    if (self.keywords.length && self.searchResultSoruce && !self.searchResultSoruce.count) {
        return nil;
    }
    UIView *headerView = [super tableView:tableView viewForHeaderInSection:section];
    headerView.backgroundColor = self.fs_backgroundColor_B2;
    return headerView;
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section {
    if ((self.keywords.length && self.searchResultSoruce && !self.searchResultSoruce.count) || self.dataSource.count == 0) {
        return 0.01;
    }
    return 25;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    if (self.editing) {
        FSAnnotationItem *annotItem = [self dataForAtIndexPath:indexPath];
        if (![self annotationListItemCanDelete:annotItem]) {
            return;
        }
        [super tableView:tableView didSelectRowAtIndexPath:indexPath];
        return;
    }
    [tableView deselectRowAtIndexPath:indexPath animated:NO];

    if (self.isGroupList) {
        return;
    }
    if ([[_extensionsManager.currentToolHandler getName] isEqualToString:Tool_Pencil]) {
        _extensionsManager.currentToolHandler = nil;
    }
    FSAnnotationItem *annotItem = [self dataForAtIndexPath:indexPath];
    if (annotItem.annot.nonempty) {
        [self performSelector:@selector(afterDelayGotoPage:) withObject:annotItem afterDelay:0.2];
        
    }else{
        [self cleanInvalidAnnot];
    }
}

- (void)afterDelayGotoPage:(FSAnnotationItem *)annotationItem {
    [self clickedSearchCancelItem];
    _panelController.isHidden = YES;
    if (annotationItem.isReply) {
        if (annotationItem.isTopReply) {
            annotationItem = annotationItem.replyToAnnotItem;
        }else{
            annotationItem = annotationItem.topReplyAnnotItem.replyToAnnotItem;
        }
    }
    FSPointF *point = [[FSPointF alloc] init];
    [point set:annotationItem.annot.fsrect.left y:annotationItem.annot.fsrect.top];
    point.x -= 20;
    CGFloat topH = [Utility convertWidth:_extensionsManager.topHeight fromPageViewToPDF:_pdfViewCtrl pageIndex:[_pdfViewCtrl getCurrentPage]];
    [point setY:[point getY] + topH + 20];
    [_pdfViewCtrl gotoPage:annotationItem.annot.pageIndex withDocPoint:point animated:NO];
    [self performSelector:@selector(setCurrentAnnotionItem:) withObject:annotationItem afterDelay:0.8];
}

- (void)setCurrentAnnotionItem:(FSAnnotationItem *)annotationItem {
    if ([_pdfViewCtrl getPageLayoutMode] != PDF_LAYOUT_MODE_REFLOW) {
        
        if ([[_extensionsManager.currentToolHandler getName] isEqualToString:Tool_Eraser]) {
            _extensionsManager.currentToolHandler = nil;
        }
        
        if (annotationItem.annot.canGetGroupHeader && [_extensionsManager.config.tools containsObject:Tool_Multiple_Selection] && [FSPermissionUtilty canAddAnnot:_extensionsManager]) {
            MultipleSelectionToolHandler *toolHandler = [_extensionsManager getToolHandlerByName:Tool_Multiple_Selection];
            [_extensionsManager setCurrentToolHandler:toolHandler];
            _extensionsManager.shouldShowMenu = YES;
            [toolHandler selectedGroupAnnot:annotationItem.annot];
        }else{
            [_extensionsManager setCurrentAnnot:annotationItem.annot];
        }
    }
}

- (BOOL)annotationListItemCanEdit:(FSAnnotationItem *)item {
    if (!item) return NO;
    if (![_extensionsManager annotsPermissionCanModify:item.annot]) {
        return NO;
    }
    BOOL canEdit = item.annot.canEdit && item.annot.type != FSAnnotFreeText && item.annot.type != FSAnnotScreen;
    if ([self getAnnotationGroupElements:item].count > 1) canEdit = canEdit && !item.annot.canGetGroupHeader;
    if (canEdit) {
        return [FSPermissionUtilty canAddAnnot:_extensionsManager];
    }
    return canEdit;
}

- (BOOL)annotationListItemCanReply:(FSAnnotationItem *)item {
    if (![_extensionsManager annotsPermissionCanReply:item.annot]) {
        return NO;
    }
    if (!item) return NO;
    BOOL canReply = item.annot.canReply;
    if ([self getAnnotationGroupElements:item].count > 1)  canReply = item.annot.canReplyGroup;
    if (canReply) {
        return [FSPermissionUtilty canAddAnnot:_extensionsManager];
    }
    return canReply;
}

- (BOOL)annotationListItemCanDelete:(FSAnnotationItem *)item {
    if (![_extensionsManager annotsPermissionCanDelete:item.annot]) {
        return NO;
    }
    if (!item) return NO;
    return item.annotCanDelete;
}

- (BOOL)annotationListItemCanFlatten:(FSAnnotationItem *)item {
    if (![_extensionsManager annotPermissionCanFlatten:item.annot]) {
        return NO;
    }
    if (![FSPermissionUtilty canAddAnnot:_extensionsManager]) return NO;
    return !item.annot.replyTo;
}

- (BOOL)annotationListCellCanApplyRedaction:(FSAnnotationItem *)item{
    return _extensionsManager.enableModifyDocument && item.annot.canApplyRedaction && _extensionsManager.isSupportRedaction;
}

- (void)annotationListItemDelete:(FSAnnotationItem *)item atIndexPath:(NSIndexPath *)indexPath{
    if ([_extensionsManager.config.tools containsObject:Tool_Multiple_Selection] && item.annot.canGetGroupHeader){
        MultipleSelectionToolHandler *toolHandler = [_extensionsManager getToolHandlerByName:Tool_Multiple_Selection];
        if (self.isGroupList) {
            [self navigationBarBackTouch];
            self.groupListAnnots = nil;
        }
        [toolHandler selectedGroupAnnotForDelete:item.annot];
        return;
    }
    [self deleteAnnotation:item atIndexPath:indexPath];
    if (self.isGroupList) {
        [item.replyToAnnotItem removeReplyItem:item];
        [self reloadData];
    }
}

- (void)annotationListItemFlatten:(FSAnnotationItem *)item atIndexPath:(NSIndexPath *)indexPath {
    if ([_extensionsManager.config.tools containsObject:Tool_Multiple_Selection] && item.annot.canGetGroupHeader){
        MultipleSelectionToolHandler *toolHandler = [_extensionsManager getToolHandlerByName:Tool_Multiple_Selection];
        self.flattenAnnotBeforeCount = [[item.annot getPage] getAnnotCount];
        [toolHandler selectedGroupAnnotForFlatten:item.annot];
        return;
    }
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kFlatten") message:FSLocalizedForKey(@"kFlattenTip") preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel") style:UIAlertActionStyleCancel handler:nil];
    UIAlertAction *flattenAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kFlatten") style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        
        id<IAnnotHandler> annotHandler = [self->_extensionsManager getAnnotHandlerByAnnot:item.annot];
        [annotHandler flattenAnnot:item.annot];

        if (self.isGroupList) {
            [item.replyToAnnotItem removeReplyItem:item];
            [self reloadData];
        }
        
    }];
    [alertController addAction:cancelAction];
    [alertController addAction:flattenAction];
    [[self fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
}

#pragma mark

- (NSArray<FSAnnotationItem *> *)getAnnotationGroupElements:(FSAnnotationItem *)item{
    if (![_extensionsManager.config.tools containsObject:Tool_Multiple_Selection]) return @[item];
    FSMarkupArray *markupArray = [item.annot canGetGroupElements];
    if (markupArray) {
        NSMutableArray *groupElements = [NSMutableArray array];
        [self.dataSource enumerateObjectsUsingBlock:^(NSArray *pageAnnots, NSUInteger idx, BOOL * _Nonnull stop) {
            FSAnnotationItem *firstItem = [pageAnnots firstObject];
            if (firstItem.annot.pageIndex == item.annot.pageIndex) {
                NSMutableArray *pageAnnotsMutableCopy = pageAnnots.mutableCopy;
                int index = 0;
                for (FSAnnotationItem *_item in pageAnnotsMutableCopy.copy) {
                    for (int i = index; i < markupArray.getSize; i++) {
                        FSMarkup *markuop = [markupArray getAt:i];
                        if ([_item.annot.uuidWithPageIndex isEqualToString:markuop.uuidWithPageIndex]) {
                            [pageAnnotsMutableCopy removeObject:_item];
                            [groupElements addObject:_item];
                            index = i + 1;
                            break;
                        }
                    }
                    
                    if (groupElements.count == markupArray.getSize) {
                         break;
                    }
                }

                *stop = YES;
            }
        }];
        return groupElements.copy;
    }
    return @[item];
}

- (FSAnnotationItem *)getAnnotationHeaderGroupItem:(FSAnnotationItem *)item{
    if (![_extensionsManager.config.tools containsObject:Tool_Multiple_Selection]) return item;
    FSMarkup *groupHeader = [item.annot canGetGroupHeader];
    if (groupHeader) {
        if ([groupHeader.uuidWithPageIndex isEqualToString:item.annot.uuidWithPageIndex]) {
            return item;
        }
        NSArray *dataSource = self.allPageAnnots;
        if (self.isGroupList) {
            dataSource = self.groupListAnnots;
        }
        for (NSArray *pageAnnots in dataSource) {
            for (FSAnnotationItem *groupElementsItem in pageAnnots) {
                if (groupElementsItem.annot.pageIndex != item.annot.pageIndex) {
                    continue;
                }
                if ([groupElementsItem.annot.uuidWithPageIndex isEqualToString:groupHeader.uuidWithPageIndex]) {
                    return groupElementsItem;
                }
            }
        }
    }
    return item;
}

#pragma mark - methods

- (NSArray<FSAnnotationItem *> *)getAnnotationItemsForPageAtIndex:(int)pageIndex {
    if ([_pdfViewCtrl isDynamicXFA]) {
        return nil;
    }
//    if ([_pdfViewCtrl isRMSV2]) {
//        return nil;
//    }
    FSPDFPage *page = nil;
    @try {
        page = [_pdfViewCtrl.currentDoc getPage:pageIndex];
    }
    @catch (NSException *exception) {
        return nil;
    }
    NSMutableArray<FSAnnotationItem *> *itemsArray = [NSMutableArray<FSAnnotationItem *> arrayWithCapacity:1];
    [Utility getAnnotsInPage:page
    predicateBlock:^BOOL(FSAnnot *_Nonnull annot) {
        if (![self->_extensionsManager.config canInteractWithAnnot:annot]) {
            return NO;
        }
        FSAnnotType type = [annot getType];
        if (type == FSAnnot3D) // MOBRD-8135
            return NO;
        if(self->_isReflow){
            if(type != FSAnnotHighlight && type != FSAnnotUnderline && type != FSAnnotSquiggly && type != FSAnnotStrikeOut){
                return NO;
            }
        }
        FSAnnotType filterAnnotType = [annot getFilterAnnotType];
        if (type == FSAnnotWidget || type == FSAnnotMovie || type == FSAnnotRichMedia || type == FSAnnotLink || filterAnnotType == FSAnnotVideo || filterAnnotType == FSAnnotAudio || (type == FSAnnotScreen && filterAnnotType == FSAnnotURI)) {
            return NO;
        }
        // 'replace text' consist of caret and strikeout annotation, add former only
        if ((type == FSAnnotStrikeOut && [Utility isReplaceText:[[FSStrikeOut alloc] initWithAnnot:annot]])) {
            return NO;
        }
        //State annot is not supported.
        if (type == FSAnnotNote && [[[FSNote alloc] initWithAnnot:annot] getState] != 0)
            return NO;

        //                                   if (type == FSAnnotFreeText) {
        //                                       NSString *intent = [((FSMarkup *) annot) getIntent];
        //                                       if (intent && [intent caseInsensitiveCompare:@"FreeTextCallout"] == NSOrderedSame) {
        //                                           return NO;
        //                                       }
        //                                   }
        if ((annot.flags & FSAnnotFlagHidden) != 0)
            return NO;

        if (!annot.replyTo) {
            FSAnnotationItem *annoItem = [[FSAnnotationItem alloc] initWithAnnot:annot isReply:NO extensionsManager:self->_extensionsManager];
            [itemsArray addObject:annoItem];
            itemsArray.fs_notReplyItemCount++;
            annoItem.locPageItmes = itemsArray;
            if ([annot isMarkup]) {
                FSMarkup *markup = [[FSMarkup alloc] initWithAnnot:annot];
                int replyCount = [markup getReplyCount];
                for (int i = 0 ; i < replyCount; i ++) {
                    FSNote *note = [markup getReply:i];
                    FSTopReplyAnnotationItem *replyItem = [[FSTopReplyAnnotationItem alloc] initWithAnnot:note extensionsManager:self->_extensionsManager];
                    [annoItem addReplyItem:replyItem];
                    if ([self annotationListItemCanDelete:replyItem]) {
                        self.canDeleteAnnotCounts ++;
                    }
                }
            }
            if ([self annotationListItemCanDelete:annoItem]) {
                self.canDeleteAnnotCounts ++;
            }
        }
        return YES;
    }];
    return itemsArray;
}

- (void)loadAnnotationsForPageAtIndex:(int)pageIndex {
    NSArray<FSAnnotationItem *> *itemsArray = [self getAnnotationItemsForPageAtIndex:pageIndex];
    if (itemsArray.count == 0) {
        return;
    }
    [self.allPageAnnots addObject:itemsArray];
    if (!self.pageIndexMapToPageAnnots) {
        self.pageIndexMapToPageAnnots = @{}.mutableCopy;
    }
    [self.pageIndexMapToPageAnnots setValue:itemsArray forKey:[NSString stringWithFormat:@"%d",pageIndex]];
}

- (void)cancelLoadAnnotations {
    [self.loadAnnotsQueue cancelAllOperations];
    [self.loadAnnotsQueue waitUntilAllOperationsAreFinished];
}

- (void)clearData {
    [self.loadAnnotsQueue cancelAllOperations];
    [self.loadAnnotsQueue waitUntilAllOperationsAreFinished];
    [self resetAnnotationArray];
    self.canDeleteAnnotCounts = 0;
    
    if (!_panelController.isHidden) {
        [self clickedSearchCancelItem];
    }
    dispatch_main_async_safe(^{
        [self reloadData];
        self.filterItem.selected = NO;
    });
}

- (void)deleteAnnotations:(NSArray *)annotations {

    self.needAnnotEvent = NO;
    
    NSMutableArray *dataSource = (NSMutableArray *)self.dataSource;
    
    NSMutableArray *mutabAnnotations = annotations.mutableCopy;
    for (NSMutableArray *currentPageAnnot in dataSource.copy) {
        NSMutableArray *pageAnnot = nil;
        if ([self filterState]) {
            pageAnnot = [self.allFilterAnnotsMapToAllPageAnnots valueForKey:[NSString stringWithFormat:@"%p",currentPageAnnot]];
        }
        int pageIndex = -1;
        ///delete some annots
        if (annotations != dataSource) {
            if (!mutabAnnotations.count) break;
            for (FSAnnotationItem *opItem in mutabAnnotations.copy) {
                if (![mutabAnnotations containsObject:opItem]) continue;
                if ([currentPageAnnot containsObject:opItem]) {
                    
                    pageIndex = opItem.annot.pageIndex;
                    id<IAnnotHandler> annotHandler = [self->_extensionsManager getAnnotHandlerByAnnot:opItem.annot];
                    BOOL ret = [annotHandler removeAnnot:opItem.annot]; // don't add undo for child annotation when
                    if (ret) {
                        [currentPageAnnot removeObject:opItem];
                        if (pageAnnot) [pageAnnot removeObject:opItem];
                        [mutabAnnotations removeObject:opItem];
                        
                        if (opItem.isReply) {
                            FSTopReplyAnnotationItem *topReplyItem = nil;
                            if (opItem.isTopReply) {
                                topReplyItem = (FSTopReplyAnnotationItem *)opItem;
                                if (topReplyItem.allReplies.count && topReplyItem.unfold) {
                                    [currentPageAnnot removeObjectsInArray:topReplyItem.allReplies];
                                    if (pageAnnot) [pageAnnot removeObjectsInArray:topReplyItem.allReplies];
                                    [mutabAnnotations removeObjectsInArray:topReplyItem.allReplies];
                                }
                                [topReplyItem.replyToAnnotItem removeReplyItem:topReplyItem];
                            }else{
                                if (opItem.allReplies.count) {
                                    [mutabAnnotations removeObjectsInArray:opItem.allReplies];
                                }
                                [opItem.topReplyAnnotItem removeReplyItem:opItem];
                            }
                        }else{
                            if (opItem.replies.count) {
                                for (FSTopReplyAnnotationItem *subTopReplyItem in opItem.replies) {
                                    NSMutableArray *tmp = @[subTopReplyItem].mutableCopy;
                                    if (subTopReplyItem.unfold) {
                                        [tmp addObjectsFromArray:subTopReplyItem.allReplies];
                                    }

                                    [currentPageAnnot removeObjectsInArray:tmp];
                                    if (pageAnnot) [pageAnnot removeObjectsInArray:tmp];
                                    [mutabAnnotations removeObjectsInArray:tmp];
                                }
                                [currentPageAnnot removeObjectsInArray:opItem.replies];
                                if (pageAnnot) [pageAnnot removeObjectsInArray:opItem.replies];
                                [mutabAnnotations removeObjectsInArray:opItem.replies];
                            }
                        }
                    }
                }
            }
            
        }else{ ///clear all annots
            for (FSAnnotationItem *item in currentPageAnnot.copy) {
                if (![self annotationListItemCanDelete:item]) continue;
                if (item.isReply) continue;
                pageIndex = item.annot.pageIndex;
                id<IAnnotHandler> annotHandler = [self->_extensionsManager getAnnotHandlerByAnnot:item.annot];
                BOOL ret = [annotHandler removeAnnot:item.annot]; // don't add undo for child annotation when clearing
                if (ret) {
                    [currentPageAnnot removeObject:item];
                    if (pageAnnot) [pageAnnot removeObject:item];
                    if (item.replies.count) {
                        for (FSTopReplyAnnotationItem *subTopReplyItem in item.replies) {
                            NSMutableArray *tmp = @[subTopReplyItem].mutableCopy;
                            if (subTopReplyItem.unfold) {
                                [tmp addObjectsFromArray:subTopReplyItem.allReplies];
                            }
                            [currentPageAnnot removeObjectsInArray:tmp];
                            if (pageAnnot) [pageAnnot removeObjectsInArray:tmp];
                        }
                        [currentPageAnnot removeObjectsInArray:item.replies];
                        if (pageAnnot) [pageAnnot removeObjectsInArray:item.replies];
                    }
                }
            }
        }
        if (pageIndex > -1) {
            NSString *key = [NSString stringWithFormat:@"%d",pageIndex];
            if (!currentPageAnnot.count) {
                [dataSource removeObject:currentPageAnnot];
                [self.pageIndexMapToPageAnnots setValue:nil forKey:key];
            }
            if ([self filterState]) {
                if (pageAnnot && !pageAnnot.count){
                    [self.allPageAnnots removeObject:pageAnnot];
                    [self.pageIndexMapToPageAnnots setValue:nil forKey:key];
                    [self.pageIndexMapToPageFilterAnnots setValue:nil forKey:key];
                }
            }
        }
    }
    
    self.needAnnotEvent = YES;
    [self updateInfoInAdvance];
    [self reloadData];

    if (self->_extensionsManager.currentAnnot) {
        [self->_extensionsManager setCurrentAnnot:nil];
    }
}

- (void)deleteAnnotation:(FSAnnotationItem *)item atIndexPath:(NSIndexPath *)indexPath{
    
    if (_extensionsManager.currentAnnot == item.annot) {
        [_extensionsManager setCurrentAnnot:nil];
    }
    
    id<IAnnotHandler> annotHandler = [_extensionsManager getAnnotHandlerByAnnot:item.annot];
    [annotHandler removeAnnot:item.annot];
}

//reply
- (void)replyToAnnotation:(FSAnnotationItem *)item atIndexPath:(NSIndexPath *)indexPath{
    
    FSTextViewPopoverViewController *textViewVC = [_extensionsManager.viewCtrlManager presentViewCtrlType:FSViewCtrlTypeTextViewPopover params:_panelController.panel.topBar];
    textViewVC.title = FSLocalizedForKey(@"kReply");
    textViewVC.okItemEnabled = ^BOOL(NSString *text) {
        return YES;
    };
    [textViewVC setDidClickedOK:^(NSString *text) {

        FSPDFPage *page = [self->_pdfViewCtrl.currentDoc getPage:item.annot.pageIndex];
        if (!page || [page isEmpty])
            return;
        FSNote *note = [[[FSMarkup alloc] initWithAnnot:item.annot] addReply];
        note.NM = [Utility getUUID];
        note.fsrect = item.annot.fsrect;
        note.author = self->_extensionsManager.annotAuthor;
        note.contents = @"";
        note.color = 0;
        note.opacity = 1.0;
        note.lineWidth = 2;
        note.icon = 0;
        note.contents = text;
        NSDate *now = [NSDate date];
        note.modifiedDate = now;
        note.createDate = now;
        
        self.needAnnotEvent = NO;
        id<IAnnotHandler> annotHandler = [self->_extensionsManager getAnnotHandlerByAnnot:note];
        [annotHandler addAnnot:note addUndo:YES];
        self.needAnnotEvent = YES;
        
        NSMutableArray<FSAnnotationItem *> *currentPageAnnots = [self.dataSource objectAtIndex:indexPath.section];
        NSMutableArray<FSAnnotationItem *> *pageAnnots = nil;
        NSMutableArray<FSAnnotationItem *> *filterPageAnnots = nil;
        if ([self filterState]) {
            NSString *key = [NSString stringWithFormat:@"%p",currentPageAnnots];
            pageAnnots = [self.allFilterAnnotsMapToAllPageAnnots valueForKey:key];
        
            filterPageAnnots = currentPageAnnots;
        }else{
            pageAnnots = currentPageAnnots;
        }
        
        NSUInteger index = 0;
        FSTopReplyAnnotationItem *replyItem;
        FSAnnotationItem *noteItem;
        if (!item.isReply) {
            noteItem = [[FSTopReplyAnnotationItem alloc] initWithAnnot:note extensionsManager:self->_extensionsManager];
            [item addReplyItem:noteItem];
            
            replyItem = (FSTopReplyAnnotationItem *)noteItem;
            
        }else{
            noteItem = [[FSAnnotationItem alloc] initWithAnnot:note isReply:YES extensionsManager:self->_extensionsManager];

            if (item.isTopReply) {
                
                replyItem = (FSTopReplyAnnotationItem *)item;
                
                void (^unfoldTopReplyItem)(FSTopReplyAnnotationItem *topReplyItem) = ^(FSTopReplyAnnotationItem *topReplyItem){
                    [self clickedUnfoldItem:nil replyItem:topReplyItem inSection:indexPath.section];
                    [self reloadRowsAtIndexPaths:indexPath];
                };
                if (!replyItem.unfold && replyItem.allReplies.count) {
                    unfoldTopReplyItem(replyItem);
                }
                
                index = item.annot.replyAnnots.count - 1;
                [replyItem addReplyItem:noteItem];
                if (replyItem.allReplies.count && !replyItem.unfold) {
                    unfoldTopReplyItem(replyItem);
                    return;
                }
            }else{
                replyItem = item.topReplyAnnotItem;
                index = indexPath.row - [currentPageAnnots indexOfObject:replyItem] + item.annot.replyAnnots.count - 1;
                [item addReplyItem:noteItem];
            }
        }
        
        if (self.needFilterType.count && ![self.needFilterType containsObject:@([item.annot getFilterAnnotType])]) {
            NSArray *pageFilterAnnots = [self.pageIndexMapToPageFilterAnnots objectForKey:[NSString stringWithFormat:@"%d",item.annot.pageIndex]];
            if (![pageFilterAnnots containsObject:item.replyToAnnotItem]) {
                [self fs_currentSceneToastWithMsg:FSLocalizedForKey(@"kReplyHasBeenAdded")];
                return;
            }
        }
        
        index = [currentPageAnnots indexOfObject:noteItem];
        if (index == NSNotFound) return;
        
        NSIndexPath *_indexPath = [NSIndexPath indexPathForRow:index inSection:indexPath.section];
        [self insertRowsAtIndexPaths:@[_indexPath]];
        self.canDeleteAnnotCounts++;
    }];
}

- (void)addNoteToAnnotation:(FSAnnotationItem *)item atIndexPath:(NSIndexPath *)indexPath {
    if (![_extensionsManager annotsPermissionCanModify:item.annot]) {
        return;
    }
    FSTextViewPopoverViewController *textViewVC = [_extensionsManager.viewCtrlManager presentViewCtrlType:FSViewCtrlTypeTextViewPopover params:_panelController.panel.topBar];
    textViewVC.text = item.annot.contents;
    [textViewVC setDidClickedOK:^(NSString *contents) {
        
        FSAnnot *annot = item.annot;
        
        NSString *oldContents = annot.contents;
        NSDate *oldDate = annot.modifiedDate;
        NSDate *now = [NSDate date];
        NSString *NM = annot.NM;
        FSPDFPage *page = [self->_pdfViewCtrl.currentDoc getPage:annot.pageIndex];
        id<IAnnotHandler> annotHandler = [self->_extensionsManager getAnnotHandlerByAnnot:annot];
        
        UndoItem *undoItem = [UndoItem itemWithUndo:^(UndoItem *item) {
            FSAnnot *annot = [Utility getAnnotByNM:NM inPage:page];
            annot.contents = oldContents;
            annot.modifiedDate = oldDate;
            if ([annotHandler respondsToSelector:@selector(modifyAnnot:addUndo:)]) {
                [annotHandler modifyAnnot:annot addUndo:NO];
            } else {
                [annotHandler modifyAnnot:annot];
            }
        }
                        redo:^(UndoItem *item) {
                            FSAnnot *annot = [Utility getAnnotByNM:NM inPage:page];
                            annot.contents = contents;
                            annot.modifiedDate = now;
                            if ([annotHandler respondsToSelector:@selector(modifyAnnot:addUndo:)]) {
                                [annotHandler modifyAnnot:annot addUndo:NO];
                            } else {
                                [annotHandler modifyAnnot:annot];
                            }
                        }
                                          pageIndex:annot.pageIndex];
        [self->_extensionsManager addUndoItem:undoItem];
        
        undoItem.redo(undoItem);
        [self reloadRowsAtIndexPaths:indexPath];
        self->_extensionsManager.isDocModified = YES;
    }];
}

@end
