/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "AttachmentController.h"
#import "TbBaseBar.h"
#import <WebKit/WebKit.h> //There are some bugs in ios13.2 https://bugs.webkit.org/show_bug.cgi?id=202173
#define UX_BG_COLOR_TOOLBAR_LIGHT 0xF5F5F5

@interface AttachmentController ()
@property (nonatomic, strong) WKWebView *webView;

@end

@implementation AttachmentController

- (WKWebView *)webView{
    if(!_webView){
        _webView = [[WKWebView alloc] init];
        _webView.accessibilityIdentifier = @"commentContentId";
        _webView.allowsLinkPreview = YES;
    }
    return _webView;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    [self.view insertSubview:self.webView atIndex:0];
    self.webView.backgroundColor = self.view.backgroundColor;
    self.navBarLeftTitle = FSLocalizedForKey(@"kClose");
    self.isShowing = YES;
    
    @weakify(self)
    self.dismissCompletion = ^{
        @strongify(self)
        self.isShowing = NO;
        [self.webView removeFromSuperview];
        self.webView = nil;
    };
}

- (void)makeConstraints{
    [super makeConstraints];
    
    [self.webView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.view.mas_left).offset(0);
        make.top.equalTo(self.navBar.mas_bottom);
        make.right.equalTo(self.view.mas_right).offset(0);
        make.bottom.equalTo(self.view.mas_bottom).offset(0);
    }];
}

- (BOOL)openDocument:(AttachmentItem *)attachmentItem {
    NSString *filePath = attachmentItem.filePath;
    
    
    NSString *title = attachmentItem.fileName;
    NSUInteger prefixLocation = [title rangeOfString:@"."].location;
    title = [title substringToIndex:prefixLocation];
    
    if (title.length > 20) {
        self.title = [NSString stringWithFormat:@"%@..%@", [title substringToIndex:10], [title substringFromIndex:title.length - 10]];
    }else{
        self.title = title;
    }

    self.navBar.titleItem.accessibilityIdentifier = @"commentTitleId";
    BOOL willLoadRequest = YES;
    NSURL *url = [NSURL fileURLWithPath:filePath isDirectory:NO];
    if ([Utility isGivenPath:filePath type:@"pdf"]) {
        NSURLRequest *request = [NSURLRequest requestWithURL:url];
        [_webView loadRequest:request];
        willLoadRequest = NO;
    } else if ([Utility isGivenPath:filePath type:@"txt"] || [Utility isGivenPath:filePath type:@"htm"] || [Utility isGivenPath:filePath type:@"html"]) {
        NSStringEncoding encoding;
        NSError *error = nil;
        NSString *temp = [NSString stringWithContentsOfURL:url usedEncoding:&encoding error:&error];
        if (!error) {
            if (encoding != NSUTF8StringEncoding) {
                willLoadRequest = NO;
                [self.webView loadHTMLString:temp baseURL:[url URLByDeletingLastPathComponent]];
            }
        } else {
            willLoadRequest = NO;
            NSData *data = [NSData dataWithContentsOfFile:filePath options:NSDataReadingMappedIfSafe error:nil];
            [self.webView loadData:data MIMEType:@"text/html" characterEncodingName:@"GBK" baseURL:[url URLByDeletingLastPathComponent]];
        }
    } else if ([Utility isGivenPath:filePath type:@"jpg"] || [Utility isGivenPath:filePath type:@"jpeg"] || [Utility isGivenPath:filePath type:@"png"] || [Utility isGivenPath:filePath type:@"bmp"] || [Utility isGivenPath:filePath type:@"gif"] || [Utility isGivenPath:filePath type:@"tiff"] || [Utility isGivenPath:filePath type:@"tif"]) {
        willLoadRequest = NO;
        
        NSURL *baseURL = [[NSBundle bundleForClass:[self class]] bundleURL];//adapter WKWebView
        if ([filePath rangeOfString:@"file://"].location != NSNotFound) {
            //iCloud
            NSString *htmString = [NSString stringWithFormat:@"<!DOCTYPE html><html><body><img src=\"%@\" width=\"100%%\" height=\"100%%\"/></body></html>", filePath];
            [self.webView loadHTMLString:htmString baseURL:baseURL];

        } else {
            //Document
            NSString *htmString = [NSString stringWithFormat:@"<!DOCTYPE html><html><body><img src=\"%@\" width=\"100%%\" height=\"100%%\"/></body></html>", [url absoluteString]];
            [self.webView loadHTMLString:htmString baseURL:baseURL];
        }
    }

    if (willLoadRequest) {
        NSURLRequest *request = [NSURLRequest requestWithURL:url];
        [self.webView loadRequest:request];
    }

    return 0;
}
@end
