/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "AttachmentModule.h"
#import "AttachmentAnnotHandler.h"
#import "AttachmentToolHandler.h"
#import "Utility.h"

@interface AttachmentModule ()

@property (nonatomic, weak) FSMainToolbarItem *propertyItem;
@property (nonatomic, weak) FSReadToolbarItem *readToolbarItem;
@property (nonatomic, strong) FSReadToolbarItem *attachmentItem;
@end

@implementation AttachmentModule {
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;
}

- (NSString *)getName {
    return @"Attachment";
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        [self loadModule];
        AttachmentAnnotHandler* annotHandler = [[AttachmentAnnotHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerAnnotHandler:annotHandler];
        
        AttachmentToolHandler* toolHandler = [[AttachmentToolHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerToolHandler:toolHandler];
    }
    return self;
}

- (void)loadModule {
    _attachmentItem = [[FSReadToolbarItem alloc] initWithToolSettings:[_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeAttachments]];
    [_extensionsManager.toolbarManager addReadItem:_attachmentItem atToolTag:FSMainTopbarToolTagComment];
    [_extensionsManager registerAnnotPropertyListener:self];
}

- (void)didClickedReadToolItem:(FSReadToolbarItem *)readToolbarItem {
    [_extensionsManager changeState:FSUIManagerStateAnnotTool];
    id<IToolHandler> toolHandler = [_extensionsManager getToolHandlerByName:Tool_Attachment];
    toolHandler.toolSettings = readToolbarItem.toolSettings;
    [_extensionsManager setCurrentToolHandler:toolHandler];
    
    self.readToolbarItem = readToolbarItem;
    FSMainToolbarItem *propertyItem =  [_extensionsManager.toolbarManager getMainItemWithType:FSMainToolbarItemTypeProperty];
    propertyItem.item.onTapClick = ^(FSItem * _Nonnull _item) {
        CGRect rect = [_item convertRect:_item.bounds toView:self->_extensionsManager.pdfViewCtrl];
        if (DEVICE_iPHONE) {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:rect inView:self->_extensionsManager.pdfViewCtrl];
        } else {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:_item.bounds inView:_item];
        }
    };
    self.propertyItem = propertyItem;

    [_extensionsManager onUIInteractElementClicked:FSUIElementReadingCommentBarAttachment];
}

- (void)dismissAnnotationContinue {
    [Utility dismissAnnotationContinue:_pdfViewCtrl];
}

#pragma mark - IAnnotPropertyListener

- (void)onToolSettingsChanged:(FSReadToolSettings *)toolSettings {
    if (self.readToolbarItem && self.readToolbarItem.toolSettings == toolSettings) {
        self.readToolbarItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }else if (_attachmentItem.toolSettings == toolSettings){
        _attachmentItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }
}

@synthesize annotType;

@end
