/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "AudioPlayerControl.h"
#import <AVKit/AVKit.h>
#import <MediaPlayer/MediaPlayer.h>
#if !_MAC_CATALYST_
#import <CoreTelephony/CTCallCenter.h>
#import <CoreTelephony/CTCall.h>
#endif
@interface AudioPlayerControl () <AVAudioPlayerDelegate, IViewAppleEventListener>

{
    CADisplayLink *_link;
}
@property (nonatomic, strong) AVAudioPlayer *audioPlayer;
@property (nonatomic, strong) UILabel *currentTime;
@property (nonatomic, strong) UILabel *totalTime;
@property (nonatomic, strong) UISlider *slider;
@property (nonatomic, strong) UIView *containerView;
@property (nonatomic, strong) UIButton *pauseOrPlay;
#if !_MAC_CATALYST_
@property (nonatomic, strong) CTCallCenter *callCenter;
#endif
@property (nonatomic, assign) BOOL isPlay;
@property (nonatomic, assign) CGPoint startPoint;
@property (nonatomic, assign) CGPoint startOrigin;
@property (nonatomic, assign) NSTimeInterval lastCurrentTime;
@property (nonatomic, weak) UIExtensionsManager *extensionsMgr;
@end

@implementation AudioPlayerControl

#pragma mark - lifecycle

static AudioPlayerControl *playerControl = nil;
+ (AudioPlayerControl *)sharedControl{
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        playerControl = [[self alloc] init];
    });
    return playerControl;
}

- (instancetype)init{
    self = [super init];
    if (self) {
        [self commonInit];
    }
    return self;
}

- (void)commonInit{
    
    _isPlay = NO;
    #if !_MAC_CATALYST_
    _callCenter = [[CTCallCenter alloc] init];
    _callCenter.callEventHandler=^(CTCall* call){
        if (!playerControl.audioPlayer.isPlaying) {
            return;
        }
        [playerControl pause];
        if([call.callState isEqualToString:CTCallStateDisconnected]){
            [playerControl play];
        }
    };
    #endif
    
    UIView *containerView = [[UIView alloc] init];
    containerView.layer.masksToBounds = YES;
    containerView.layer.cornerRadius = 17.f;
    containerView.backgroundColor =  [UIColor colorWithRed:0 green:0 blue:0 alpha:0.5];
    self.containerView = containerView;
    
    UIView *handleView = [[UIView alloc] init];
    handleView.backgroundColor = self.fs_backgroundColor_B3;
    handleView.layer.cornerRadius = 2;
    handleView.clipsToBounds = YES;
    
    UILabel *currentTime = [[UILabel alloc] init];
    currentTime.text = @"00:00";
    currentTime.textAlignment = NSTextAlignmentLeft;
    currentTime.textColor = self.fs_textColor_T1;
    currentTime.font = [UIFont systemFontOfSize:10.f];
    self.currentTime = currentTime;
    
    UILabel *totalTime = [[UILabel alloc] init];
    totalTime.text = @"00:00";
    totalTime.textAlignment = NSTextAlignmentRight;
    totalTime.textColor = currentTime.textColor;
    totalTime.font = currentTime.font;
    self.totalTime = totalTime;
    
    UISlider *slider = [[UISlider alloc] init];
    slider.continuous = YES;
    slider.minimumTrackTintColor = self.fs_primaryColor;
    slider.maximumTrackTintColor = self.fs_iconColor_I2;
    BindThemeKeyPath(slider, minimumTrackTintColor);
    [slider setThumbImage:FSImageNamed(@"audio_slider_thumb") forState:UIControlStateNormal];
    [slider addTarget:self action:@selector(changeCurrentProgress:) forControlEvents:UIControlEventTouchUpInside];
    self.slider = slider;
    
    UIButton *backward = [[UIButton alloc] init];
    [backward setImage:FSImageNamed(@"edit_audio_back") forState:UIControlStateNormal];
    [backward addTarget:self action:@selector(backward:) forControlEvents:UIControlEventTouchUpInside];
    
    UIButton *speed = [[UIButton alloc] init];
    [speed setImage:FSImageNamed(@"edit_audio_speed") forState:UIControlStateNormal];
    [speed addTarget:self action:@selector(speed:) forControlEvents:UIControlEventTouchUpInside];
    
    UIButton *pauseOrPlay = [[UIButton alloc] init];
    [pauseOrPlay setImage:FSImageNamed(@"edit_audio_pause") forState:UIControlStateSelected];
    [pauseOrPlay setImage:FSImageNamed(@"edit_audio_continue") forState:UIControlStateNormal];
    [pauseOrPlay addTarget:self action:@selector(pauseOrPlay:) forControlEvents:UIControlEventTouchUpInside];
    self.pauseOrPlay = pauseOrPlay;
    
    UIButton *stop = [[UIButton alloc] init];
    [stop setImage:FSImageNamed(@"edit_audio_stop") forState:UIControlStateNormal];
    [stop addTarget:self action:@selector(stop:) forControlEvents:UIControlEventTouchUpInside];
    
    [containerView addSubview:currentTime];
    [containerView addSubview:totalTime];
    [containerView addSubview:slider];
    [containerView addSubview:backward];
    [containerView addSubview:speed];
    [containerView addSubview:pauseOrPlay];
    [containerView addSubview:stop];
    [containerView addSubview:handleView];
    
    [handleView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.size.mas_equalTo(CGSizeMake(31, 4));
        make.top.equalTo(containerView).offset(3);
        make.centerX.equalTo(containerView);
    }];
    
    [currentTime mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(containerView).offset(15);
        make.top.equalTo(containerView).offset(18);
        make.width.mas_equalTo(32);
    }];
    
    [totalTime mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(containerView).offset(-15);
        make.centerY.equalTo(currentTime);
    }];
    
    [slider mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(currentTime.mas_right).offset(10);
        make.right.equalTo(totalTime.mas_left).offset(-10);
        make.centerY.equalTo(currentTime);
        make.height.offset(30);
    }];
    
    [pauseOrPlay mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(containerView.mas_centerX).offset(-9);
        make.bottom.offset(-10);
    }];
    
    [stop mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(containerView.mas_centerX).offset(9);
        make.centerY.equalTo(pauseOrPlay);
    }];
    
    [backward mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(pauseOrPlay.mas_left).offset(-14);
        make.centerY.equalTo(pauseOrPlay);
    }];
    
    [speed mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(stop.mas_right).offset(14);
        make.centerY.equalTo(pauseOrPlay);
    }];
    
    UIPanGestureRecognizer * pan = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(panAudioView:)];
    [self.containerView addGestureRecognizer:pan];
    
}

- (void)registerEventListens:(UIExtensionsManager*)extensionsMgr{
    self.extensionsMgr = extensionsMgr;
    [self.extensionsMgr.pdfViewCtrl registerViewAppleEventListener:self];
}

- (void)dealloc
{
    [self.extensionsMgr.pdfViewCtrl unregisterViewAppleEventListener:self];
}

-(void)resetPlayerPosition {
    
    if (!self.containerView.superview) {
        return;
    }
    
    CGFloat topBarHeight = self.extensionsMgr.toolbarManager.mainTopbar.getToolbarContentView.bounds.size.height;
    CGFloat bottomBarHeight = self.extensionsMgr.toolbarManager.mainBottombar.getToolbarContentView.bounds.size.height;
 
    BOOL isLandscape = UIInterfaceOrientationIsLandscape([[UIApplication sharedApplication] statusBarOrientation]);
    CGFloat supViewWidth = self.containerView.superview.frame.size.width;
    CGFloat supViewHeight = self.containerView.superview.frame.size.height;
    
    CGPoint point = self.containerView.fs_origin;
    float viewLeft = point.x;
    float viewTop = point.y;
    if (viewLeft < 0) {
        viewLeft = 0;
    }else if (viewLeft + 232 > supViewWidth)
    {
        viewLeft = supViewWidth - 232;
    }
    
    if (viewTop < topBarHeight) {
        viewTop = topBarHeight;
    }else if (viewTop + 95  + bottomBarHeight > supViewHeight)
    {
        viewTop = supViewHeight - 95 - bottomBarHeight;
    }
    
    [self.containerView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(viewLeft);
        make.top.mas_equalTo(viewTop);
        make.width.offset(232);
        make.height.offset(95);
    }];
}

- (void)panAudioView:(UIPanGestureRecognizer *)pan {
    
    if (!self.containerView.superview) {
        return;
    }
    if (pan.state == UIGestureRecognizerStateBegan) {
        self.startPoint = [pan locationInView:self.containerView.superview];
        self.startOrigin = self.containerView.frame.origin;
        return;
    }
    CGPoint curPoint = [pan locationInView:self.containerView.superview];
    
    float x = curPoint.x - self.startPoint.x;
    float y = curPoint.y - self.startPoint.y;

    int topBarHeight = self.extensionsMgr.toolbarManager.mainTopbar.getToolbarContentView.bounds.size.height;
    int bottomBarHeight = self.extensionsMgr.toolbarManager.mainBottombar.getToolbarContentView.bounds.size.height;

    float viewLeft = self.startOrigin.x+x;
    float viewTop = self.startOrigin.y+y;
    if (viewLeft < self.containerView.superview.safeAreaInsets.left) {
        viewLeft = self.containerView.superview.safeAreaInsets.left;
    }else if (viewLeft + 232 > self.containerView.superview.frame.size.width - self.containerView.superview.safeAreaInsets.right)
    {
        viewLeft = self.containerView.superview.frame.size.width - self.containerView.superview.safeAreaInsets.right - 232;
    }
    
    if (viewTop < topBarHeight) {
        viewTop = topBarHeight;
    }else if (viewTop + 95  + bottomBarHeight > self.containerView.superview.frame.size.height)
    {
        viewTop = self.containerView.superview.frame.size.height - 95 - bottomBarHeight;
    }
    
    [self.containerView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(viewLeft);
        make.top.mas_equalTo(viewTop);
        make.width.offset(232);
        make.height.offset(95);
    }];

}

#pragma mark - custom accessors

- (AVAudioPlayer *)audioPlayer{
    if (!_audioPlayer) {
        NSError *error = nil;
            _audioPlayer = [[AVAudioPlayer alloc] initWithContentsOfURL:_filePath error:&error];
        _audioPlayer.numberOfLoops = 0;
        _audioPlayer.delegate = self;
        [_audioPlayer prepareToPlay];
        if (error) {
            return nil;
        }
        _slider.minimumValue = 0;
        _slider.maximumValue = _audioPlayer.duration;
        _totalTime.text = [NSString stringWithFormat:@"%02d:%02d",(int)_audioPlayer.duration / 60, (int)_audioPlayer.duration % 60];
        _link = [CADisplayLink displayLinkWithTarget:self selector:@selector(tick)];
        [_link addToRunLoop:[NSRunLoop mainRunLoop] forMode:NSRunLoopCommonModes];
        _link.preferredFramesPerSecond = 10;
    }
    return _audioPlayer;
}

- (void)setFilePath:(NSURL *)filePath{
    self.audioPlayer = nil;
    _filePath = filePath;
}

#pragma mark - events
static NSTimeInterval skipInterval = 10;

- (void)backward:(UIButton *)btn{
    [self setAudioPlayTime:(self.audioPlayer.currentTime - skipInterval)?:0];
}
- (void)speed:(UIButton *)btn{
    
    [self setAudioPlayTime:self.audioPlayer.currentTime + skipInterval];
}

- (void)pauseOrPlay:(UIButton *)btn{
    btn.selected = !btn.selected;
    if (btn.selected) {
        [self play];
    }else{
        [self pause];
    }
}

- (void)stop:(UIButton *)btn{
    [self.audioPlayer stop];
    [self dismiss];
}

-(void)changeCurrentProgress:(UISlider *)slider{
    NSTimeInterval difference = self.audioPlayer.currentTime - self.slider.value;
    if (difference > -1 && difference < 1) {
        return;
    }
    [self setAudioPlayTime:slider.value];
}

#pragma mark- public
- (void)addSuperview:(UIView *)view{
    [self dismiss];
    [view insertSubview:self.containerView atIndex:1];
    [self.containerView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(view);
        make.centerY.equalTo(view).multipliedBy(1.5);
        make.width.offset(232);
        make.height.offset(95);
    }];
    
    [self.extensionsMgr cancelAutoFullScreen];
}
- (void)play{
    [self setNowPlayingInfo];
    _isPlay = YES;
    if (![self.audioPlayer isPlaying]) {
        [self.audioPlayer play];
    }
    self.pauseOrPlay.selected = YES;
}

- (void)pause{
    [self resetNowPlayingInfo];
    if ([self.audioPlayer isPlaying]) {
        [self.audioPlayer pause];
    }
    self.pauseOrPlay.selected = NO;
}

- (void)setNumberOfLoops:(NSInteger)numberOfLoops
{
    self.audioPlayer.numberOfLoops = numberOfLoops;
}

#pragma mark- private
- (void)dismiss{
    if (!_isPlay) return;
    _isPlay = NO;
    [_link invalidate];
    _link = nil;
    [self.audioPlayer stop];
    [self.containerView removeFromSuperview];
    [self.extensionsMgr delayAutoFullScreen];
}

- (void)setAudioPlayTime:(NSTimeInterval)time{
    BOOL isPlaying = [self.audioPlayer isPlaying];
    [self.audioPlayer pause];
    self.audioPlayer.currentTime = time;
    if (isPlaying) {
        [self.audioPlayer play];
    }else{
        self.currentTime.text = [NSString stringWithFormat:@"%02d:%02d",(int)self.audioPlayer.currentTime / 60, (int)self.audioPlayer.currentTime % 60];
    }
}

- (void)tick{
    

    if (self.audioPlayer.currentTime != self.lastCurrentTime) {
        self.currentTime.text = [NSString stringWithFormat:@"%02d:%02d",(int)self.audioPlayer.currentTime / 60, (int)self.audioPlayer.currentTime % 60];
        if (self.slider.state == UIControlStateHighlighted) {
            return;
        }

        if (self.audioPlayer.currentTime <= 0.0f) {
            [self.slider setValue:self.audioPlayer.currentTime animated:NO];
        }else{
            [self.slider setValue:self.audioPlayer.currentTime animated:YES];
        }
        self.lastCurrentTime = self.audioPlayer.currentTime;
    }
}

- (void)resetNowPlayingInfo
{
    if ([MPNowPlayingInfoCenter class])
    {
        [MPNowPlayingInfoCenter defaultCenter].nowPlayingInfo = nil;
    }
}

- (void)setNowPlayingInfo
{
    NSMutableDictionary *currentlyPlayingTrackInfo = [NSMutableDictionary dictionary];
    NSString *name = _filePath.absoluteString;
    if (!name.length) return;
    NSArray *array = [name componentsSeparatedByString:@"/"];
    NSString *fileName = [array lastObject];
    [currentlyPlayingTrackInfo setObject:fileName forKey:MPMediaItemPropertyTitle];
    [currentlyPlayingTrackInfo setObject:@"FoxitSpeech" forKey:MPNowPlayingInfoPropertyExternalUserProfileIdentifier];
    [MPNowPlayingInfoCenter defaultCenter].nowPlayingInfo = currentlyPlayingTrackInfo;
    
}

#pragma mark- <AVAudioPlayerDelegate>
- (void)audioPlayerDidFinishPlaying:(AVAudioPlayer *)player successfully:(BOOL)flag{
    if (flag) {
//        [self dismiss];
        if (![self.audioPlayer isPlaying]) {
             self.pauseOrPlay.selected = NO;
        }
        
    }
    
    [self resetNowPlayingInfo];
}

- (void)audioPlayerDecodeErrorDidOccur:(AVAudioPlayer *)player error:(NSError * __nullable)error{
    if (error) {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kMediaFileValid") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
            [self dismiss];
        }];
        [alertController addAction:action];
        [[self.extensionsMgr fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];

    }
}

#pragma mark -- IViewAppleEventListener
- (void)layoutSubviewsFromPDFViewCtrl:(FSPDFViewCtrl *)pdfViewCtrl{
    
    [self resetPlayerPosition];
    
}

@end
