/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "CalloutModule.h"
#import "FtAnnotHandler.h"
#import "SelectToolHandler.h"
#import "CalloutToolHandler.h"


@interface CalloutModule ()

@property (nonatomic, weak) UIExtensionsManager *extensionsManager;

@property (nonatomic, weak) FSMainToolbarItem *propertyItem;
@property (nonatomic, weak) FSReadToolbarItem *readToolbarItem;
@property (nonatomic, strong) FSReadToolbarItem *calloutItem;
@end

@implementation CalloutModule

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        self.extensionsManager = extensionsManager;
        [_extensionsManager registerAnnotPropertyListener:self];
        [self loadModule];
    }
    return self;
}

- (NSString *)getName {
    return @"Callout";
}

- (void)loadModule {
    CalloutToolHandler *toolHandler = [[CalloutToolHandler alloc] initWithUIExtensionsManager:_extensionsManager];
    [_extensionsManager registerToolHandler:toolHandler];
    [_extensionsManager registerUndoEventListener:toolHandler];

    FtAnnotHandler *annotHandler = [[FtAnnotHandler alloc] initWithUIExtensionsManager:_extensionsManager];
    [_extensionsManager registerAnnotHandler:annotHandler];
    //    [_extensionsManager.propertyBar registerPropertyBarListener:annotHandler];

//    _extensionsManager.annotationToolsBar.calloutClicked = ^(){
//        [self annotItemClicked];
//    };
    _calloutItem = [[FSReadToolbarItem alloc] initWithToolSettings:[_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeCallout]];
    [_extensionsManager.toolbarManager addReadItem:_calloutItem atToolTag:FSMainTopbarToolTagComment];
}

- (void)didClickedReadToolItem:(FSReadToolbarItem *)readToolbarItem
{
    SelectToolHandler *select = (SelectToolHandler *) [_extensionsManager getToolHandlerByName:Tool_Select];
    [select clearSelection];
    [_extensionsManager changeState:FSUIManagerStateAnnotTool];
    id<IToolHandler> toolHandler = [_extensionsManager getToolHandlerByName:Tool_Callout];
    toolHandler.toolSettings = readToolbarItem.toolSettings;
    [_extensionsManager setCurrentToolHandler:toolHandler];
    
    self.readToolbarItem = readToolbarItem;
    FSMainToolbarItem *propertyItem =  [_extensionsManager.toolbarManager getMainItemWithType:FSMainToolbarItemTypeProperty];
    propertyItem.item.onTapClick = ^(FSItem * _Nonnull _item) {
        [[UIApplication sharedApplication] sendAction:@selector(resignFirstResponder) to:nil from:nil forEvent:nil];
        
        CGRect rect = [_item convertRect:_item.bounds toView:self->_extensionsManager.pdfViewCtrl];
        if (DEVICE_iPHONE) {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:rect inView:self->_extensionsManager.pdfViewCtrl];
        } else {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:_item.bounds inView:_item];
        }
    };
    self.propertyItem = propertyItem;

    [_extensionsManager onUIInteractElementClicked:FSUIElementReadingCommentBarCallout];
}

#pragma mark <IAnnotPropertyListener>

- (void)onToolSettingsChanged:(FSReadToolSettings *)toolSettings {
    if (self.readToolbarItem && self.readToolbarItem.toolSettings == toolSettings) {
        self.readToolbarItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }else if (_calloutItem.toolSettings == toolSettings){
        _calloutItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }
}


@end
