/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "InsertToolHandler.h"

@interface InsertToolHandler ()

@property (nonatomic, weak) FSPDFViewCtrl *pdfViewCtrl;
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
@property (nonatomic, weak) FSTaskServer *taskServer;
@property (nonatomic, assign) BOOL isTextSelect;
@property (nonatomic, assign) int currentPageIndex;
@property (nonatomic, strong) FSPointF *currentPoint;
@property (nonatomic, assign) int startPosIndex;
@property (nonatomic, assign) int endPosIndex;
@property (nonatomic, assign) int wordStartPosIndex;
@property (nonatomic, assign) int wordEndPosIndex;
@property (nonatomic, strong) NSArray *arraySelectedRect;
@property (nonatomic, strong) FSRectF *currentEditPdfRect;
@property (nonatomic, strong) NSArray *colors;
@end

@implementation InsertToolHandler {
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        self.extensionsManager = extensionsManager;
        self.pdfViewCtrl = extensionsManager.pdfViewCtrl;
        self.taskServer = self.extensionsManager.taskServer;
        self.type = FSAnnotCaret;
    }
    return self;
}

- (int)getCharIndexAtPos:(int)pageIndex point:(CGPoint)point {
    FSPointF *dibPoint = [self.pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
    FSTextPage *textPage = [Utility getTextSelect:self.pdfViewCtrl.currentDoc pageIndex:pageIndex];
    if ([textPage isEmpty]) return -1;
    return [textPage getIndexAtPos:dibPoint.x y:dibPoint.y tolerance:0];
}

- (NSArray *)getCurrentSelectRects:(int)pageIndex {
    if (self.startPosIndex == -1 || self.endPosIndex == -1) {
        return nil;
    }
    UIView *view = [_pdfViewCtrl getPageView:pageIndex];
    CGSize size = view.frame.size;
    int offsetY = 0;
    int offsetX = 0;
    __block CGRect unionRect = CGRectZero;
    NSMutableArray *ret = [NSMutableArray array];

    FSTextPage *textPage = [Utility getTextSelect:_pdfViewCtrl.currentDoc pageIndex:pageIndex];
    NSArray *array = [Utility getTextRects:textPage start:MIN(_startPosIndex, _endPosIndex) count:ABS(_endPosIndex - _startPosIndex) + 1];
    [array enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        CGRect rect = [[obj objectAtIndex:0] CGRectValue];
        FSRectF *dibRect = [Utility CGRect2FSRectF:rect];
        CGRect selfRect = [self getRealRectWithOptions:pageIndex dibRect:dibRect size:size offsetY:offsetY offsetX:offsetX];
        [ret addObject:[NSValue valueWithCGRect:selfRect]];
        if (CGRectEqualToRect(unionRect, CGRectZero)) {
            unionRect = selfRect;
        } else {
            unionRect = CGRectUnion(unionRect, selfRect);
        }
    }];
    self.currentEditPdfRect = [Utility normalizeFSRect:[Utility CGRect2FSRectF:unionRect]];
    return ret;
}

- (CGRect)getRealRectWithOptions:(int)pageIndex dibRect:(FSRectF *)dibRect size:(CGSize)size offsetY:(int)offsetY offsetX:(int)offsetX {
    CGRect rect = [_pdfViewCtrl convertPdfRectToPageViewRect:dibRect pageIndex:pageIndex];
    return rect;
}

- (void)getTheCurrentRowPdfRectWithCurrentSelectedIndex:(int)index pageIndex:(int)pageIndex {
    FSRectF *selectCharpdfRect = self.currentEditPdfRect;
    int firstIndex = index;
    FSTextPage *textPage = [Utility getTextSelect:self.pdfViewCtrl.currentDoc pageIndex:pageIndex];
    for (int i = index; i > -1; i--) {
        NSArray *array = [Utility getTextRects:textPage start:i count:1];
        __block CGRect unionRect = CGRectZero;
        [array enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
            CGRect rect = [[obj objectAtIndex:0] CGRectValue];
            if (CGRectEqualToRect(unionRect, CGRectZero)) {
                unionRect = rect;
            } else {
                unionRect = CGRectUnion(unionRect, rect);
            }
        }];
        self.currentEditPdfRect = [Utility normalizeFSRect:[Utility CGRect2FSRectF:unionRect]];
        if (selectCharpdfRect.top < self.currentEditPdfRect.bottom) {
            firstIndex = i;
            break;
        }
        if (i == 0) {
            firstIndex = i;
            break;
        }
    }

    int lastIndex = index;
    int pageCharCount = [textPage getCharCount];
    for (int i = index; i < pageCharCount; i++) {
        NSArray *array = [Utility getTextRects:textPage start:i count:1];
        if (array.count == 0 || !array) {
            break;
        }
        __block CGRect unionRect = CGRectZero;
        [array enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
            CGRect rect = [[obj objectAtIndex:0] CGRectValue];
            if (CGRectEqualToRect(unionRect, CGRectZero)) {
                unionRect = rect;
            } else {
                unionRect = CGRectUnion(unionRect, rect);
            }
        }];
        self.currentEditPdfRect = [Utility normalizeFSRect:[Utility CGRect2FSRectF:unionRect]];
        if (selectCharpdfRect.bottom > self.currentEditPdfRect.top) {
            lastIndex = i;
            break;
        }
    }

    __block CGRect unionRect = CGRectZero;

    int startIndex = firstIndex == index ? index : firstIndex + 1;
    int endIndex = lastIndex == index ? index : lastIndex - 1;
    NSArray *array = [Utility getTextRects:textPage start:MIN(startIndex, endIndex) count:ABS(endIndex - startIndex) + 1];

    [array enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        CGRect rect = [[obj objectAtIndex:0] CGRectValue];
        unionRect = CGRectZero;
        if (CGRectEqualToRect(unionRect, CGRectZero)) {
            unionRect = rect;
        } else {
            unionRect = CGRectUnion(unionRect, rect);
        }
    }];
    self.currentEditPdfRect = [Utility normalizeFSRect:[Utility CGRect2FSRectF:unionRect]];
}

- (void)clearSelection {
    self.isEdit = NO;
    self.isTextSelect = NO;
    self.startPosIndex = -1;
    self.endPosIndex = -1;
    self.wordStartPosIndex = -1;
    self.wordEndPosIndex = -1;
    self.arraySelectedRect = nil;
    if ([self.extensionsManager.menuControl isMenuVisible]) {
        [self.extensionsManager.menuControl hideMenu];
    }
    [self.pdfViewCtrl refresh:self.currentPageIndex needRender:NO];
}

- (NSString *)getName {
    return Tool_Insert;
}

- (BOOL)isEnabled {
    return YES;
}

- (void)onActivate {
}

- (void)onDeactivate {
}

- (BOOL)onPageViewLongAndPan:(int)pageIndex recognizer:(UIGestureRecognizer *)recognizer {
    if (_extensionsManager.currentToolHandler != self) {
        return NO;
    }
    self.currentPageIndex = pageIndex;
    FSAnnot *annot = _extensionsManager.currentAnnot;
    id<IAnnotHandler> annotHandler = annot ? [_extensionsManager getAnnotHandlerByAnnot:annot] : nil;
    if (annot != nil && ![annot isEmpty]) {
        if (recognizer.state == UIGestureRecognizerStateBegan) {
            self.currentPageIndex = pageIndex;

            if ([annotHandler onPageViewShouldBegin:pageIndex recognizer:recognizer annot:annot]) {
                [annotHandler onPageViewPan:pageIndex recognizer:(UIPanGestureRecognizer *) recognizer annot:annot];
                return YES;
            } else {
                [_extensionsManager setCurrentAnnot:nil];
            }
        } else {
            [annotHandler onPageViewPan:pageIndex recognizer:(UIPanGestureRecognizer *) recognizer annot:annot];
            return YES;
        }
    }
    
    CGPoint point = [recognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];
    if (recognizer.state == UIGestureRecognizerStateBegan) {
        self.currentPageIndex = pageIndex;
        [self clearSelection];
        int index = [self getCharIndexAtPos:pageIndex point:point];
        if (index > -1) {
            if ([recognizer isKindOfClass:[UILongPressGestureRecognizer class]]) {
                FSTextPage *textPage = [Utility getTextSelect:_pdfViewCtrl.currentDoc pageIndex:pageIndex];
                NSRange range = [Utility getWordByTextIndex:index textPage:textPage];
                self.startPosIndex = (int) range.location;
                self.endPosIndex = (int) (range.location + range.length - 1);
                self.wordStartPosIndex = self.startPosIndex;
                self.wordEndPosIndex = self.endPosIndex;
                [self getCurrentSelectRects:pageIndex];
            } else {
                self.startPosIndex = index;
            }
            self.isEdit = YES;
            self.isTextSelect = YES;
            [self showMagnifier:pageIndex index:index point:point];
        }

    } else if (recognizer.state == UIGestureRecognizerStateChanged) {
        if (pageIndex != self.currentPageIndex) {
            return NO;
        }
        CGPoint point = [recognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];
        int index = [self getCharIndexAtPos:pageIndex point:point];
        if (index > -1) {
            if (self.startPosIndex == -1) {
                self.startPosIndex = index;
            } else {
                if (index < self.wordStartPosIndex) {
                    self.startPosIndex = self.wordEndPosIndex;
                }else if (index > self.wordStartPosIndex){
                    self.startPosIndex = self.wordStartPosIndex;
                }
                self.endPosIndex = index;
            }
            [self getCurrentSelectRects:pageIndex];
            [_pdfViewCtrl refresh:pageIndex needRender:NO];
            [self moveMagnifier:pageIndex index:index point:point];
        }
    } else if (recognizer.state == UIGestureRecognizerStateEnded || recognizer.state == UIGestureRecognizerStateCancelled) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self closeMagnifier];
        });
        if (self.startPosIndex == -1 || self.endPosIndex == -1) {
            return YES;
        }
        if (pageIndex != self.currentPageIndex) {
            pageIndex = self.currentPageIndex;
        }
        FSTextPage *textPage = [Utility getTextSelect:_pdfViewCtrl.currentDoc pageIndex:pageIndex];

        BOOL isHorizontal = YES;
        BOOL isLeftToRight = YES;
        BOOL isTopToBottom = YES;
        FSRotation textRotation = FSRotationUnknown;

        for (int i = 0; i < [textPage getTextRectCount:self.endPosIndex count:1]; i++) {
            textRotation = [textPage getBaselineRotation:i];
            if (textRotation == FSRotationUnknown)
                continue;
            isLeftToRight = (textRotation == FSRotation0 || textRotation == FSRotation270);
            isTopToBottom = (textRotation == FSRotation0 || textRotation == FSRotation90);
            isHorizontal = (textRotation == FSRotation0 || textRotation == FSRotation180);
            break;
        }

        __block CGRect unionRect = CGRectZero;
        for (int i = 0; i < 10; i++) {
            int startIndex = self.endPosIndex > self.startPosIndex ? self.endPosIndex - i : self.startPosIndex - i;
            int endIndex = self.endPosIndex > self.startPosIndex ? self.endPosIndex - i : self.startPosIndex - i;
            NSArray *array = [Utility getTextRects:textPage start:MIN(startIndex, endIndex) count:ABS(endIndex - startIndex) + 1];
            [array enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
                CGRect rect = [[obj objectAtIndex:0] CGRectValue];
                if (CGRectEqualToRect(unionRect, CGRectZero)) {
                    unionRect = rect;
                } else {
                    unionRect = CGRectUnion(unionRect, rect);
                }
            }];
            self.currentEditPdfRect = [Utility normalizeFSRect:[Utility CGRect2FSRectF:unionRect]];
            if (unionRect.origin.x != 0 && unionRect.origin.y != 0) {
                break;
            }
        }

        FSRectF *rectChar = self.currentEditPdfRect;
        if (ABS(rectChar.left - rectChar.right) < 1E-5 || ABS(rectChar.top - rectChar.bottom) < 1E-5) {
            return YES;
        }

        int index = self.endPosIndex > self.startPosIndex ? self.endPosIndex : self.startPosIndex;

        [self getTheCurrentRowPdfRectWithCurrentSelectedIndex:index pageIndex:pageIndex];
        FSRectF *rectWord = self.currentEditPdfRect;
        if (ABS(rectWord.left - rectWord.right) < 1E-5 || ABS(rectWord.top - rectWord.bottom) < 1E-5) {
            return YES;
        }

        float width, height, left, top;
        if (isHorizontal) {
            height = (rectWord.top - rectWord.bottom) * (8.5 / 10.0);
            width = height * (2.0 / 3.0);
            if (isLeftToRight) {
                left = rectChar.right - (width / 2.0);
            } else {
                left = rectChar.left - (width / 2.0);
            }
            if (isTopToBottom) {
                top = rectWord.top - (height * (7.5 / 10.0));
            } else {
                top = rectWord.bottom + (height * (7.5 / 10.0)) + height;
            }
        } else {
            width = (rectWord.right - rectWord.left) * (8.5 / 10.0);
            height = width * (2.0 / 3.0);
            if (isTopToBottom) {
                top = rectChar.bottom + (height / 2.0);
            } else {
                top = rectChar.top + (height / 2.0);
            }
            if (isLeftToRight) {
                left = rectWord.left + width * (7.5 / 10.0);
            } else {
                left = rectWord.right - width * (7.5 / 10.0) - width;
            }
        }
       
        [self showTextPopupView:pageIndex textRotation:textRotation top:top left:left  width:width height:height];
    }
    return YES;
}

// PageView Gesture+Touch
- (BOOL)onPageViewLongPress:(int)pageIndex recognizer:(UILongPressGestureRecognizer *)recognizer {

    return [self onPageViewLongAndPan:pageIndex recognizer:recognizer];
}

- (void)showTextPopupView:(int)pageIndex textRotation:(FSRotation)textRotation top:(float)top left:(float)left width:(float)width height:(float)height{
    FSTextViewPopoverViewController *insertTextViewC = [_extensionsManager.viewCtrlManager presentViewCtrlType:FSViewCtrlTypeTextViewPopover params:_extensionsManager.topToolbar.getToolbarContentView];
    insertTextViewC.title = FSLocalizedForKey(@"kInsertText");
    self.insertTextViewC = insertTextViewC;
    insertTextViewC.okItemEnabled = ^BOOL(NSString *text) {
        return YES;
    };
    @fs_weakify(self)
    insertTextViewC.didClickedOK = ^(NSString *text) {
        @fs_strongify(self)
        FSPDFPage *page = [self.pdfViewCtrl.currentDoc getPage:pageIndex];
        if (!page || [page isEmpty])
            return;
        FSRectF *dibRect = [Utility makeFSRectWithLeft:left top:top right:(left + width) bottom:(top - height)];
        FSCaret *annot = [[FSCaret alloc] initWithAnnot:[page addAnnot:FSAnnotCaret rect:dibRect]];
        annot.NM = [Utility getUUID];
        annot.author = self->_extensionsManager.annotAuthor;
        annot.color = self.toolSettings.settingsColor;
        annot.opacity = self.toolSettings.opacity;
        annot.subject = @"Insert Text";
        annot.contents = text;
        annot.intent = @"Insert Text";
        {
            FSPDFDictionary *dict = [annot getDict];
            int iRotation = 360 - (int)textRotation * 90;
            [dict setAt:@"Rotate" pdf_object:[FSPDFObject createFromInteger:iRotation]];
            [annot resetAppearanceStream];
        }

        NSDate *now = [NSDate date];
        FSDateTime *time = [Utility convert2FSDateTime:now];
        [annot setCreationDateTime:time];
        [annot setModifiedDateTime:time];

        if (annot && ![annot isEmpty]) {
            [[self.extensionsManager getAnnotHandlerByAnnot:annot] addAnnot:annot addUndo:YES];
        }
        [self clearSelection];
    };
    [insertTextViewC setDismissCompletion:^{
        @fs_strongify(self)
        [self clearSelection];
    }];
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer {
    CGPoint point = [recognizer locationInView:[self.pdfViewCtrl getPageView:pageIndex]];
    FSTextPage *textPage = [Utility getTextSelect:self.pdfViewCtrl.currentDoc pageIndex:pageIndex];
    self.currentPageIndex = pageIndex;
    BOOL isHorizontal = YES;
    BOOL isLeftToRight = YES;
    BOOL isTopToBottom = YES;
    FSRotation textRotation = FSRotationUnknown;

    int index = [self getCharIndexAtPos:pageIndex point:point];
    [self clearSelection];
    if (index > -1) {
        self.isEdit = YES;
        self.isTextSelect = YES;
        self.startPosIndex = index;
        self.endPosIndex = index;
        [self.pdfViewCtrl refresh:pageIndex needRender:NO];

        for (int i = 0; i < [textPage getTextRectCount:index count:1]; i++) {
            textRotation = [textPage getBaselineRotation:i];
            if (textRotation == FSRotationUnknown)
                continue;
            isLeftToRight = (textRotation == FSRotation0 || textRotation == FSRotation270);
            isTopToBottom = (textRotation == FSRotation0 || textRotation == FSRotation90);
            isHorizontal = (textRotation == FSRotation0 || textRotation == FSRotation180);
            break;
        }
    } else {
        self.isTextSelect = NO;
        self.isEdit = YES;
        self.startPosIndex = -1;
        self.endPosIndex = -1;
        self.currentEditPdfRect = [Utility makeFSRectWithLeft:0 top:0 right:0 bottom:0];
        return YES;
    }

    for (int i = 0; i < 4; i++) {
        __block CGRect unionRect = CGRectZero;
        int startIndex = self.startPosIndex - i;
        int endIndex = self.endPosIndex - i;
        NSArray *array = [Utility getTextRects:textPage start:MIN(startIndex, endIndex) count:ABS(endIndex - startIndex) + 1];

        [array enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
            CGRect rect = [[obj objectAtIndex:0] CGRectValue];
            if (CGRectEqualToRect(unionRect, CGRectZero)) {
                unionRect = rect;
            } else {
                unionRect = CGRectUnion(unionRect, rect);
            }
        }];
        self.currentEditPdfRect = [Utility normalizeFSRect:[Utility CGRect2FSRectF:unionRect]];
        if (unionRect.origin.x != 0 && unionRect.origin.y != 0) {
            self.startPosIndex = self.startPosIndex - i;
            self.endPosIndex = self.endPosIndex - i;
            break;
        }
    }

    FSRectF *rectChar = self.currentEditPdfRect;
    if (ABS(rectChar.left - rectChar.right) < 1E-5 || ABS(rectChar.top - rectChar.bottom) < 1E-5) {
        return YES;
    }

    [self getTheCurrentRowPdfRectWithCurrentSelectedIndex:index pageIndex:pageIndex];
    FSRectF *rectWord = self.currentEditPdfRect;
    if (ABS(rectWord.left - rectWord.right) < 1E-5 || ABS(rectWord.top - rectWord.bottom) < 1E-5) {
        return YES;
    }
    
    CGRect pvRect = [_pdfViewCtrl convertPdfRectToPageViewRect:rectChar pageIndex:pageIndex];
    CGPoint pvRBPt = CGPointMake(CGRectGetMaxX(pvRect), CGRectGetMaxY(pvRect));
    FSPointF *fspoint = [self.pdfViewCtrl convertPageViewPtToPdfPt:pvRBPt pageIndex:pageIndex];
    float width, height, left, top;
    if (isHorizontal) {
        height = (rectWord.top - rectWord.bottom) * (8.5 / 10.0);
        width = height * (2.0 / 3.0);
        left = fspoint.x > ((rectChar.left + rectChar.right) / 2.0) ? rectChar.right - (width / 2.0) : rectChar.left - (width / 2.0);
        if (isTopToBottom) {
            top = rectWord.top - (height * (7.5 / 10.0));
        } else {
            top = rectWord.bottom + (height * (7.5 / 10.0)) + height;
        }
    } else {
        width = (rectWord.right - rectWord.left) * (8.5 / 10.0);
        height = width * (2.0 / 3.0);
        top = fspoint.y > ((rectChar.top + rectChar.bottom) / 2.0) ? rectChar.top + (height / 2.0) : rectChar.bottom + (height / 2.0);
        if (isLeftToRight) {
            left = rectWord.left + width * (7.5 / 10.0);
        } else {
            left = rectWord.right - width * (7.5 / 10.0) - width;
        }
    }

    [self showTextPopupView:pageIndex textRotation:textRotation top:top left:left width:width height:height];
    return YES;
}

- (BOOL)onPageViewPan:(int)pageIndex recognizer:(UIPanGestureRecognizer *)recognizer {
    return NO;
}

- (BOOL)onPageViewShouldBegin:(int)pageIndex recognizer:(UIGestureRecognizer *)gestureRecognizer {
    return NO;
}

- (BOOL)onPageViewTouchesBegan:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesMoved:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesEnded:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (BOOL)onPageViewTouchesCancelled:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    return NO;
}

- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context {
    if (pageIndex != self.currentPageIndex) {
        return;
    }

    if (self.isEdit && self.isTextSelect) {
        if (self.startPosIndex == -1 || self.endPosIndex == -1) {
            return;
        }
        self.arraySelectedRect = [self getCurrentSelectRects:pageIndex];
        [self.arraySelectedRect enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
            CGRect selfRect = [obj CGRectValue];
            CGContextSetRGBFillColor(context, 0, 0, 1, 0.3);
            CGContextFillRect(context, selfRect);
        }];
    }
}

#pragma mark - Magnifier

- (void)showMagnifier:(int)pageIndex index:(int)index point:(CGPoint)point {
    if (_magnifierView == nil) {
        FSTextPage *textPage = [Utility getTextSelect:_pdfViewCtrl.currentDoc pageIndex:pageIndex];
        NSArray *array = [Utility getTextRects:textPage start:index count:2];
        if (array.count > 0) {
            FSRectF *dibRect = [Utility CGRect2FSRectF:[[[array objectAtIndex:0] objectAtIndex:0] CGRectValue]];
            CGRect rect = [_pdfViewCtrl convertPdfRectToPageViewRect:dibRect pageIndex:pageIndex];
            point = CGPointMake(point.x, CGRectGetMidY(rect));
        }
        _magnifierView = [[MagnifierView alloc] init];
        _magnifierView.viewToMagnify = [_pdfViewCtrl getDisplayView];
        _magnifierView.touchPoint = point;
        _magnifierView.magnifyPoint = [[_pdfViewCtrl getPageView:pageIndex] convertPoint:point toView:[_pdfViewCtrl getDisplayView]];
        [[_pdfViewCtrl getPageView:pageIndex] addSubview:_magnifierView];
    }
}

- (void)moveMagnifier:(int)pageIndex index:(int)index point:(CGPoint)point {
    FSTextPage *textPage = [Utility getTextSelect:_pdfViewCtrl.currentDoc pageIndex:pageIndex];
    NSArray *array = [Utility getTextRects:textPage start:index count:2];
    if (array.count > 0) {
        FSRectF *dibRect = [Utility CGRect2FSRectF:[[[array objectAtIndex:0] objectAtIndex:0] CGRectValue]];
        CGRect rect = [_pdfViewCtrl convertPdfRectToPageViewRect:dibRect pageIndex:pageIndex];
        point = CGPointMake(point.x, CGRectGetMidY(rect));
    }
    _magnifierView.touchPoint = point;
    _magnifierView.magnifyPoint = [_pdfViewCtrl convertPageViewPtToDisplayViewPt:point pageIndex:pageIndex];
    [_magnifierView setNeedsDisplay];
}

- (void)closeMagnifier {
    [_magnifierView removeFromSuperview];
    _magnifierView = nil;
}
@synthesize toolSettings;

@end
