/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "ReplaceModule.h"
#import "CaretAnnotHandler.h"
#import "ReplaceToolHandler.h"
#import "SelectToolHandler.h"
#import "Utility.h"
#import <FoxitRDK/FSPDFViewControl.h>

@interface ReplaceModule ()

@property (nonatomic, weak) FSMainToolbarItem *propertyItem;
@property (nonatomic, weak) FSReadToolbarItem *readToolbarItem;
@property (nonatomic, strong) FSReadToolbarItem *replaceTextItem;
@end

@implementation ReplaceModule {
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;
    FSAnnotType _annotType;
}

- (NSString *)getName {
    return @"Replace";
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        CaretAnnotHandler* annotHandler = [[CaretAnnotHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerAnnotHandler:annotHandler];

        ReplaceToolHandler* toolHandler = [[ReplaceToolHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerToolHandler:toolHandler];
        [self loadModule];
    }
    return self;
}

- (void)loadModule {
    _replaceTextItem = [[FSReadToolbarItem alloc] initWithToolSettings:[_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeReplaceText]];
    [_extensionsManager.toolbarManager addReadItem:_replaceTextItem atToolTag:FSMainTopbarToolTagComment];
    [_extensionsManager registerAnnotPropertyListener:self];
}

- (void)didClickedReadToolItem:(FSReadToolbarItem *)readToolbarItem {
    SelectToolHandler *select = (SelectToolHandler *) [_extensionsManager getToolHandlerByName:Tool_Select];
    [select clearSelection];

    id<IToolHandler> toolHandler = [_extensionsManager getToolHandlerByName:Tool_Replace];
    toolHandler.toolSettings = readToolbarItem.toolSettings;
    [_extensionsManager setCurrentToolHandler:toolHandler];
    [_extensionsManager changeState:FSUIManagerStateAnnotTool];

    self.readToolbarItem = readToolbarItem;
    FSMainToolbarItem *propertyItem =  [_extensionsManager.toolbarManager getMainItemWithType:FSMainToolbarItemTypeProperty];
    propertyItem.item.onTapClick = ^(FSItem * _Nonnull _item) {
        CGRect rect = [_item convertRect:_item.bounds toView:self->_extensionsManager.pdfViewCtrl];
        if (DEVICE_iPHONE) {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:rect inView:self->_extensionsManager.pdfViewCtrl];
        } else {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:_item.bounds inView:_item];
        }
    };
    self.propertyItem = propertyItem;

    [_extensionsManager onUIInteractElementClicked:FSUIElementReadingCommentBarReplaceText];
}

#pragma mark - IAnnotPropertyListener

- (void)onToolSettingsChanged:(FSReadToolSettings *)toolSettings {
    if (self.readToolbarItem && self.readToolbarItem.toolSettings == toolSettings) {
        self.readToolbarItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }else if (_replaceTextItem.toolSettings == toolSettings){
        _replaceTextItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }
}

@end
