/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSBookmark+Extensions.h"
#import <objc/runtime.h>

@interface FSBookmark (Extensions)
@property (nonatomic, copy) NSArray *childBookmarks;
@end
@implementation FSBookmark (Extensions)

YYSYNTH_DYNAMIC_PROPERTY_OBJECT(childBookmarks, setChildBookmarks, RETAIN, NSArray *);

- (NSArray *)getChildBookmarks{
    NSArray *childBookmarks = self.childBookmarks;
    if ([self isEmpty]) {
        if (childBookmarks) self.childBookmarks = nil;
        return nil;
    }
    if (!childBookmarks) {
        NSMutableArray *array = [NSMutableArray array];
        FSBookmark *bmChild = [self getFirstChild];
        while (true) {
            if (!bmChild || [bmChild isEmpty]) break;
            if (![bmChild.title isEqualToString:BookmarkRootFolder]) [array addObject:bmChild];
            if ([bmChild isLastChild]) break;
            bmChild = [bmChild getNextSibling];
        }
        childBookmarks = array.copy;
        self.childBookmarks = childBookmarks.count ? self.childBookmarks : nil;
    }
    return childBookmarks;
}

- (NSArray *)reloadChildBookmarks{
    NSArray *childBookmarks = nil;
    if ([self isEmpty]) {
        if (childBookmarks) self.childBookmarks = nil;
        return nil;
    }

    NSMutableArray *array = [NSMutableArray array];
    FSBookmark *bmChild = [self getFirstChild];
    while (true) {
        if (!bmChild || [bmChild isEmpty]) break;
        if (![bmChild.title isEqualToString:BookmarkRootFolder]) [array addObject:bmChild];
        if ([bmChild isLastChild]) break;
        bmChild = [bmChild getNextSibling];
    }
    childBookmarks = array.copy;
    self.childBookmarks = childBookmarks;
    
    return childBookmarks;
}

- (void)removeChildBookmark:(FSBookmark *)bookmark{
    NSArray *childBookmarks = self.childBookmarks;
    if (childBookmarks) {
        NSMutableArray *tmp = childBookmarks.mutableCopy;
        if ([tmp containsObject:bookmark]) {
            [tmp removeObject:bookmark];
            childBookmarks = tmp.copy;
            self.childBookmarks = childBookmarks.count ? self.childBookmarks : nil;
        }
    }
}
@end
