/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import <Foundation/Foundation.h>

NS_ASSUME_NONNULL_BEGIN

typedef NS_ENUM(NSUInteger, FSMenuControllerArrowDirection) {
    FSMenuControllerArrowDirectionDown,
    FSMenuControllerArrowDirectionUp,
};

@protocol FSMenuControllerDelegate <NSObject>

@optional
- (BOOL)menuControllerEnableAtIndex:(NSUInteger)index;
- (void)menuControllerDidClickedItem:(FSMenuItem *)item atIndex:(NSUInteger)index;
- (void)menuControllerDidDisappear;

@end

@class FSMenuItem;
@interface FSMenuController : NSObject

@property (nonatomic, copy, nullable) NSArray<FSMenuItem *> *menuItems;
@property (nonatomic, readonly) CGRect menuFrame;
@property (nonatomic) FSMenuControllerArrowDirection arrowDirection;
@property (nonatomic, getter=isMenuVisible, readonly) BOOL menuVisible;
@property (nonatomic, weak) id<FSMenuControllerDelegate> delegate;

+ (FSMenuController *)sharedMenuController;
- (void)setMenuVisible:(BOOL)menuVisible animated:(BOOL)animated;
- (void)setTargetRect:(CGRect)targetRect inView:(UIView *)targetView;
- (void)updateTargetRect:(CGRect)targetRect inView:(UIView *)targetView;
- (void)update;

@end

NS_ASSUME_NONNULL_END
