/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSViewControllerServicesImpl.h"
#import "UIExtensionsManager+Private.h"
#import "FSAnnotFilterViewController.h"
#import "FSFileSelectDestinationViewController.h"
#import "FSTableViewPopoverController.h"
#import "FSMoreMenuViewController.h"
#import "CheckCerViewCtr.h"
#import "CerInfoViewController.h"
#import "CerTrustedListViewCtr.h"
#import "FSToolsPopoverViewController.h"
#import "FSTextViewPopoverViewController.h"
#import "FSInsertPageConfigVCtrl.h"
#import "FSInsertPageSizeOrOriVCtrl.h"
#import "FSInsertPageColorViewController.h"
#import "FSSaveAsViewController.h"
#import "FSViewPopoverController.h"
#import "FSCustomViewContainerController.h"
#import "FSAddToolsViewController.h"
#import "SignatureListViewController.h"
#import "FSToolbarSettingsPopover.h"

NS_ASSUME_NONNULL_BEGIN

typedef NS_ENUM(NSUInteger, FSViewCtrlType) {
    FSViewCtrlTypeViewBase = 0,
    FSViewCtrlTypeTabViewBase = 1,
    FSViewCtrlTypeSearchBase = 2,
    FSViewCtrlTypeTablePopoverBase = 3,
    FSViewCtrlTypeMoreTablePopover,
    FSViewCtrlTypeAnnotFilter,
    FSViewCtrlTypeFileSelect,
    FSViewCtrlTypeCheckCer,
    FSViewCtrlTypeCerInfo,
    FSViewCtrlTypeCerTrustedList,
    FSViewCtrlTypeTextViewPopover,
    FSViewCtrlTypeInsertPageConfig,
    FSViewCtrlTypeInsertPageSizeOrOri,
    FSViewCtrlTypeInsertPageColor,
    FSViewCtrlTypeSaveAs,
    FSViewCtrlTypeCustomViewContainer,
    FSViewCtrlTypeAddTools,
    FSViewCtrlTypeSignatureList,
    FSViewCtrlTypeSortTools,
};

@interface FSViewControllerManager : FSViewControllerServicesImpl

@property (nonatomic, weak) UIExtensionsManager *extensionsManager;

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager;

- (__kindof FSViewController *)createViewCtrlType:(FSViewCtrlType)type;
- (__kindof FSViewController *)createViewCtrlType:(FSViewCtrlType)type params:(nullable id)params;

- (__kindof FSViewController *)pushViewCtrlType:(FSViewCtrlType)type;
- (__kindof FSViewController *)pushViewCtrlType:(FSViewCtrlType)type params:(nullable id)params;

- (__kindof FSViewController *)presentViewCtrlType:(FSViewCtrlType)type;
- (__kindof FSViewController *)presentViewCtrlType:(FSViewCtrlType)type params:(nullable id)params;
- (__kindof FSViewController *)presentViewCtrlType:(FSViewCtrlType)type completion:(nullable VoidBlock)completion;
- (__kindof FSViewController *)presentViewCtrlType:(FSViewCtrlType)type params:(nullable id)params completion:(nullable VoidBlock)completion;

- (FSViewPopoverController *)popPopoverCustomView:(UIView *)view dismissCompletion:(VoidBlock)dismissCompletion sourceView:(nullable UIView * (^)(UIPopoverPresentationController *popover))sourceView;
- (FSViewPopoverController *)popPopoverCustomView:(UIView *)view sourceView:(nullable UIView * (^)(UIPopoverPresentationController *popover))sourceView;
- (FSViewPopoverController *)popPopoverCustomView:(UIView *)view sourceView:(UIView * (^)(UIPopoverPresentationController *popover))sourceView sourceRect:(CGRect)sourceRect;
- (FSViewPopoverController *)popPopoverCustomViewCtrl:(UIViewController *)viewCtrl sourceView:(nullable UIView * (^)(UIPopoverPresentationController *popover))sourceView;

- (__kindof FSTableViewPopoverController *)popPopoverCtrlClass:(Class)ctrlClass items:(NSArray<FSPopoverItem *> *)items sourceView:(UIView * (^)(UIPopoverPresentationController *popover))sourceView;
- (__kindof FSTableViewPopoverController *)popPopoverCtrlClass:(Class)ctrlClass groups:(nullable NSArray<FSPopoverItemGroup *> *)groups sourceView:(UIView * (^)(UIPopoverPresentationController *popover))sourceView;
- (__kindof FSTableViewPopoverController *)popPopoverCtrlClass:(Class)ctrlClass groups:(nullable NSArray<FSPopoverItemGroup *> *)groups sourceView:(UIView * (^)(UIPopoverPresentationController *popover))sourceView autoAdapt:(BOOL)autoAdapt;

//Class FSTableViewPopoverController
- (void)popPopoverWithGroups:(NSArray<FSPopoverItemGroup *> *)groups sourceView:(UIView * (^)(UIPopoverPresentationController *popover))sourceView;
- (void)popPopoverWithItems:(NSArray<FSPopoverItem *> *)items sourceView:(UIView * (^)(UIPopoverPresentationController *popover))sourceView;

//Class FSMoreMenuViewController
- (void)showMoreMenuWithItems:(NSArray<FSPopoverItem *> *)items sourceView:(UIView * (^)(UIPopoverPresentationController *popover))sourceView;
- (void)showMoreMenuWithTitles:(NSArray *)titles images:(NSArray *)images action:(FSPopoverItemAction)action sourceView:(UIView * (^)(UIPopoverPresentationController *popover))sourceView;

- (void)showMoreMenuWithItems:(NSArray<FSPopoverItem *> *)items dismissCompletion:(nullable VoidBlock)dismissCompletion sourceView:(UIView * (^)(UIPopoverPresentationController *popover))sourceView;
- (void)showMoreMenuWithTitles:(NSArray *)titles images:(NSArray *)images action:(FSPopoverItemAction)action dismissCompletion:(nullable VoidBlock)dismissCompletion sourceView:(UIView * (^)(UIPopoverPresentationController *popover))sourceView;

- (void)registerEventListens;
@end

@interface UIViewController (FSViewControllerManager)
- (void)fs_presentViewController:(UIViewController *)viewControllerToPresent animated:(BOOL)flag completion:(void (^__nullable)(void))completion;
@end

NS_ASSUME_NONNULL_END
