/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>

@class FSMenuItem;
/** @brief The item of menu. */
@interface MenuItem : NSObject
/** @brief The title of menuItem.  */
@property (nonatomic, strong) NSString *title;
/** @brief The object of menuItem.  */
@property (nonatomic, strong) id object;
/** @brief The action of menuItem.  TYPE: (void)action:(id)object;*/
@property (nonatomic, assign) SEL action;
/** @brief The image of menuItem.  */
@property (nonatomic, strong) UIImage *image;
/** @brief The tag of menuItem.  */
@property (nonatomic, assign) int tag;

- (instancetype)initWithMenuItem:(FSMenuItem *)menuItem;
/** @brief Intialize custom menuItem.
 *
 * @param[in]    image        The image of menuItem.
 * @param[in]    object        The object of menuItem.
 * @param[in]    action        The action of menuItem.
 *
 * @return    The custom menuItem instance.
 */
- (instancetype)initWithImage:(UIImage *)image object:(id)object action:(SEL)action;

/** @brief Intialize custom menuItem.
 *
 * @param[in]    title        The title of menuItem.
 * @param[in]    object        The object of menuItem.
 * @param[in]    action        The action of menuItem.
 *
 * @return    The custom menuItem instance.
 */
- (instancetype)initWithTitle:(NSString *)title object:(id)object action:(SEL)action;

/** @brief Intialize custom menuItem.
 *
 * @param[in]    title        The title of menuItem.
 * @param[in]    image        The image of menuItem.
 * @param[in]    object        The object of menuItem.
 * @param[in]    action        The action of menuItem.
 *
 * @return    The custom menuItem instance.
 */
- (instancetype)initWithTitle:(NSString *)title image:(UIImage *)image object:(id)object action:(SEL)action;
@end
