/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "PaintUtility.h"
#import "PrivateDefine.h"

@interface FSDrawCaretTimerWeakObj : NSObject

- (instancetype)initWithTarget:(id)target;
@end

@interface PaintUtility ()
@property (nonatomic, strong) NSTimer *drawCaretTimer;
@property (nonatomic, assign) BOOL showCaretIntervel;
@end
@implementation PaintUtility

+(CGPoint)rotateVec:(float)px py:(float)py ang:(float)ang isChlen:(BOOL)isChlen newLine:(float)newLen
{
    CGPoint point = CGPointMake(0, 0);
    float vx = px * cosf(ang) - py * sinf(ang);
    float vy = px * sinf(ang) + py * cosf(ang);
    if (isChlen) {
        float d = sqrtf(vx * vx + vy * vy);
        vx = vx / d * newLen;
        vy = vy / d * newLen;
        point.x = vx;
        point.y = vy;
    }
    return point;
}

-(void)drawLineFromStartPoint:(CGPoint)startPoint toEndPoint:(CGPoint)endPoint lineWidth: (CGFloat)lineWidth inContext:(CGContextRef)context isArrowLine:(BOOL) isArrowLine{
    CGContextSetLineWidth(context, lineWidth);
    
    UIColor *color = [UIColor colorWithRGB:self.annotColor];
    CGContextSetRGBStrokeColor(context,[color red], [color green], [color blue],self.annotOpacity);
    
    CGContextBeginPath(context);
    CGContextMoveToPoint(context, startPoint.x, startPoint.y);
    CGContextAddLineToPoint(context, endPoint.x, endPoint.y);
    CGContextSetBlendMode(context, kCGBlendModeDestinationAtop);
    CGContextStrokePath(context);
    
    if (isArrowLine)
    {
        float x1 = startPoint.x;
        float y1 = startPoint.y;
        float x2 = endPoint.x;
        float y2 = endPoint.y;
        
        float arctanglent = M_PI / 6;
        float arrow_len = lineWidth*4.0;
        
        CGPoint endPoint1 = [PaintUtility rotateVec:(x2 - x1) py:(y2 - y1) ang:arctanglent isChlen:YES newLine:arrow_len];
        CGPoint endPoint2 = [PaintUtility rotateVec:(x2 - x1) py:(y2 - y1) ang:-arctanglent isChlen:YES newLine:arrow_len];
        
        float x3 = x2 - endPoint1.x;
        float y3 = y2 - endPoint1.y;
        float x4 = x2 - endPoint2.x;
        float y4 = y2 - endPoint2.y;
        
        if (isnan(x3) || isnan(y3) || isnan(x2) || isnan(y2)|| isnan(x4) || isnan(y4)) {
            return;
        }
        CGMutablePathRef path = CGPathCreateMutable();
        CGPathMoveToPoint(path, NULL, x3, y3);
        CGPathAddLineToPoint(path, NULL, x2, y2);
        CGPathAddLineToPoint(path, NULL, x4, y4);
        CGPathRef pathToDraw = CGPathCreateCopyByStrokingPath(path, NULL, 0.1, kCGLineCapRound, kCGLineJoinRound, 0);
        
        CGContextAddPath(context, pathToDraw);
        CGContextStrokePath(context);
        CGPathRelease(path);
        CGPathRelease(pathToDraw);
    }
}

-(void)drawPencilWithPath:(UIBezierPath *)path inContext:(CGContextRef)context {

    
    float lineWidth = self.annotLineWidth;
    CGContextSetLineWidth(context, lineWidth);
    
    UIColor *color = [UIColor colorWithRGB:self.annotColor];
    CGContextSetRGBStrokeColor(context,[color red], [color green], [color blue],self.annotOpacity);
    
    CGContextBeginPath(context);
    CGContextSetLineCap(context, kCGLineCapRound);
    CGContextSetLineJoin(context, kCGLineJoinRound);
    CGContextSetBlendMode(context, kCGBlendModeNormal);
    
    CGContextAddPath(context, path.CGPath);
    
    CGContextStrokePath(context);
}

-(void)drawPencilWith:(NSArray *)arrayLines inContext:(CGContextRef)context {

    float lineWidth = self.annotLineWidth;
    CGContextSetLineWidth(context, lineWidth);
    
    UIColor *color = [UIColor colorWithRGB:self.annotColor];
    CGContextSetRGBStrokeColor(context,[color red], [color green], [color blue],self.annotOpacity);
    
    CGContextSetLineCap(context, kCGLineCapRound);
    CGContextSetLineJoin(context, kCGLineJoinRound);
    CGContextSetBlendMode(context, kCGBlendModeNormal);


    NSArray *inkList = arrayLines;
    if (!inkList || !context)
           return;
    NSMutableDictionary *dic = [inkList lastObject];
//     for (NSMutableDictionary *dic in inkList)
        {
           NSArray <NSValue *> *line = dic[@"pointArray"];
           BOOL isEnd = NO;

           if (dic[@"isEnd"]) isEnd = YES;
           UIBezierPath *path = dic[@"path"];
           if (isEnd && path) {
               CGContextAddPath(context, path.CGPath);
               CGContextStrokePath(context);
               return;
           }
           NSInteger ptnum = line.count;
           if (ptnum < 1)
               return;

           if (ptnum == 1) {
               CGPoint point = [line[0] CGPointValue];
               if (isEnd) {
                   if (!path){
                       path = [UIBezierPath bezierPath];
                       dic[@"path"] = path;
                       [path moveToPoint:point];
                       [path addLineToPoint:CGPointMake(point.x + 0.001f, point.y + 0.001f)];
                   }
                   CGContextAddPath(context, path.CGPath);
                   CGContextStrokePath(context);
               }else{
                   CGContextMoveToPoint(context, point.x, point.y);
                   CGContextAddLineToPoint(context, point.x + 0.001f, point.y + 0.001f);
                   CGContextStrokePath(context);
               }
               return ;
           }

           CGPoint vertice[4];
           BOOL isClosed = CGPointEqualToPoint([line[0] CGPointValue], [line[ptnum - 1] CGPointValue]);

           vertice[0] = isClosed ? [line[ptnum - 2] CGPointValue] : [line[0] CGPointValue];
           CGPoint startPoint = [line[0] CGPointValue];
//           if (isEnd) {
//               if (!path){
//                   path = [UIBezierPath bezierPath];
//                   dic[@"path"] = path;
//                   [path moveToPoint:startPoint];
//
//                   for (NSInteger j = 0; j < ptnum - 1; j++) {
//                       vertice[1] = [line[j] CGPointValue];
//                       vertice[2] = [line[j + 1] CGPointValue];
//
//                       if (j == ptnum - 2) {
//                           vertice[3] = isClosed ? [line[1] CGPointValue] : vertice[2];
//                       } else {
//                           vertice[3] = [line[j + 2] CGPointValue];
//                       }
//
//                       NSArray *cp = [self.class calcBesselControlPoints:vertice rate:0.8f];
//
//                       [path addCurveToPoint:vertice[2] controlPoint1:[cp[0] CGPointValue] controlPoint2:[cp[1] CGPointValue]];
//                       vertice[0] = vertice[1];
//                       startPoint = vertice[2];
//                   }
//               }
//               CGContextAddPath(context, path.CGPath);
//               CGContextStrokePath(context);
//           }else{
               
             CGContextMoveToPoint(context, startPoint.x, startPoint.y);
               for (NSInteger j = 0; j < ptnum - 1; j++) {
                   vertice[1] = [line[j] CGPointValue];
                   vertice[2] = [line[j + 1] CGPointValue];

                   if (j == ptnum - 2) {
                       vertice[3] = isClosed ? [line[1] CGPointValue] : vertice[2];
                   } else {
                       vertice[3] = [line[j + 2] CGPointValue];
                   }

                   NSArray *cp = [self.class calcBesselControlPoints:vertice rate:0.8f];
                   
                   CGContextAddCurveToPoint(context,
                                            [cp[0] CGPointValue].x, [cp[0] CGPointValue].y
                                            , [cp[1] CGPointValue].x, [cp[1] CGPointValue].y
                                            , vertice[2].x, vertice[2].y);
                   
                   vertice[0] = vertice[1];
                   startPoint = vertice[2];
               }
             CGContextStrokePath(context);
               
//        }
    }
}

+ (NSArray *)calcBesselControlPoints:(CGPoint *)pLinePoints rate:(float)k
{
    // calculate the control points between vertice[1] and vertice[2].
    // vertice[0] is the previous vertex, and vertice[3] is the next one.
    
    if (CGPointEqualToPoint(pLinePoints[0], pLinePoints[1]) &&
        CGPointEqualToPoint(pLinePoints[1], pLinePoints[2])) {
        return nil;
    }
    
    float xc1 = (pLinePoints[0].x + pLinePoints[1].x) / 2.0f;
    float yc1 = (pLinePoints[0].y + pLinePoints[1].y) / 2.0f;
    float xc2 = (pLinePoints[1].x + pLinePoints[2].x) / 2.0f;
    float yc2 = (pLinePoints[1].y + pLinePoints[2].y) / 2.0f;
    float xc3 = (pLinePoints[2].x + pLinePoints[3].x) / 2.0f;
    float yc3 = (pLinePoints[2].y + pLinePoints[3].y) / 2.0f;
    
    float xDiff, yDiff;
    xDiff = pLinePoints[1].x - pLinePoints[0].x;
    yDiff = pLinePoints[1].y - pLinePoints[0].y;
    float len1 = sqrtf(xDiff * xDiff + yDiff * yDiff);
    xDiff = pLinePoints[2].x - pLinePoints[1].x;
    yDiff = pLinePoints[2].y - pLinePoints[1].y;
    float len2 = sqrtf(xDiff * xDiff + yDiff * yDiff);
    xDiff = pLinePoints[3].x - pLinePoints[2].x;
    yDiff = pLinePoints[3].y - pLinePoints[2].y;
    float len3 = sqrtf(xDiff * xDiff + yDiff * yDiff);
    
    //FXSYS_assert((len1 + len2) > 0.001f);
    //FXSYS_assert((len2 + len3) > 0.001f);
    float k1 = 0.5f;
    float k2 = 0.5f;
    if (len1 + len2 > 0.0001f && len2 + len3 > 0.0001f)
    {
        k1 = len1 / (len1 + len2);
        k2 = len2 / (len2 + len3);
    }
    
    float xm1 = xc1 + (xc2 - xc1) * k1;
    float ym1 = yc1 + (yc2 - yc1) * k1;
    float xm2 = xc2 + (xc3 - xc2) * k2;
    float ym2 = yc2 + (yc3 - yc2) * k2;
    
    CGPoint point1,point2;
    
    point1.x = xm1 + (xc2 - xm1) * k + pLinePoints[1].x - xm1;
    point1.y = ym1 + (yc2 - ym1) * k + pLinePoints[1].y - ym1;
    point2.x = xm2 + (xc2 - xm2) * k + pLinePoints[2].x - xm2;
    point2.y = ym2 + (yc2 - ym2) * k + pLinePoints[2].y - ym2;
    
    return [NSArray arrayWithObjects:[NSValue valueWithCGPoint:point1], [NSValue valueWithCGPoint:point2],nil];
}

-(void)drawShapWithRect:(CGRect)rect type:(FSAnnotType)type inContext:(CGContextRef)context {
    float lineWidth = self.annotLineWidth;
    CGContextSetLineWidth(context, lineWidth);
    
    UIColor *color = [UIColor colorWithRGB:self.annotColor];
    CGContextSetRGBStrokeColor(context,[color red], [color green], [color blue],self.annotOpacity);
    UIColor *fillcolor = [UIColor colorWithRGB:self.annotFillColor];
    BOOL setFillColor = (self.annotFillColor != 0xFFFFFF00);
    CGContextSetRGBFillColor(context,[fillcolor red], [fillcolor green], [fillcolor blue],self.annotOpacity);
    
    if (type == FSAnnotSquare) {
        if (setFillColor) CGContextFillRect(context, rect);
        CGContextStrokeRect(context, rect);
    }else{
        if (setFillColor) CGContextFillEllipseInRect(context, rect);
        CGContextStrokeEllipseInRect(context, rect);
    }
    
}

-(void)drawMultipleSelectWithRect:(CGRect)rect inContext:(CGContextRef)context {
    float lineWidth = self.annotLineWidth;
    CGContextSetLineWidth(context, lineWidth);
    
    UIColor *color = [UIColor colorWithRGB:self.annotColor];
    
    CGContextSetRGBStrokeColor(context,[color red], [color green], [color blue],1);
    CGContextStrokeRect(context, rect);
    
    CGContextSetRGBFillColor(context,[color red], [color green], [color blue],self.annotOpacity);
    CGContextFillRect(context, rect);
}

-(void)drawMultipleSelectedWithRect:(CGRect)rect inContext:(CGContextRef)context {
    float lineWidth = self.annotLineWidth;
    CGContextSetLineWidth(context, lineWidth);
    
    UIColor *color = [UIColor colorWithRGB:self.annotColor];
    
    CGContextSetRGBStrokeColor(context,[color red], [color green], [color blue],self.annotOpacity);
    CGContextStrokeRect(context, rect);
}


- (void)setDrawCaretBlock:(FSPaintUtilityDrawCaretBlock)drawCaretBlock{
    if (_drawCaretTimer) {
        [_drawCaretTimer invalidate];
        _drawCaretTimer = nil;
    }
    _drawCaretBlock = drawCaretBlock;
    if (drawCaretBlock) {
        _drawCaretTimer = [NSTimer scheduledTimerWithTimeInterval:1.f target:[[FSDrawCaretTimerWeakObj alloc] initWithTarget:self] selector:@selector(drawCaret) userInfo:nil repeats:YES];
    }
}

- (void)drawCaret{
    ! _drawCaretBlock ? : _drawCaretBlock(self);
}

-(void)drawCaretWithRect:(CGRect)rect inContext:(CGContextRef)context{
    if (_showCaretIntervel) {
        UIColor *color = _caretColor ? _caretColor : UIColorHex(#179CD8);
        CGContextSetLineDash(context, 0, NULL, 0);
        CGContextMoveToPoint(context, CGRectGetMinX(rect), CGRectGetMinY(rect));
        CGContextAddLineToPoint(context, CGRectGetMinX(rect), CGRectGetMaxY(rect));
        CGContextSetLineWidth(context, 2.0);
        CGContextSetStrokeColorWithColor(context, color.CGColor);
        CGContextStrokePath(context);
    }
    _showCaretIntervel = !_showCaretIntervel;
}

+ (void)drawCallout:(FSFreeText *)callout annotImage:(UIImage *)annotImage inContext:(CGContextRef)context pdfViewCtrl:(FSPDFViewCtrl *)pdfViewCtrl isLineDash:(BOOL)isLineDash{
    if (!callout.nonempty) return;
    int pageIndex = callout.pageIndex;
    CGRect drawRect = [pdfViewCtrl convertPdfRectToPageViewRect:[callout getInnerRect] pageIndex:pageIndex];
    FSPointFArray *points = [callout getCalloutLinePoints];
    if ([points getSize]) {
        float lineWidth = callout.lineWidth;
        CGContextSetLineWidth(context, lineWidth);
        
        UIColor *color = [UIColor colorWithRGB:[callout getBorderColor]];
        CGContextSetRGBStrokeColor(context,[color red], [color green], [color blue],callout.opacity);
        
        if (isLineDash) {
            CGFloat dashArray[] = {3, 3, 3, 3};
            CGContextSetLineDash(context, 3, dashArray, 4);
            
            color = [UIColor colorWithRGB:callout.color];
            CGContextSetRGBStrokeColor(context,[color red], [color green], [color blue],callout.opacity);
        }
        
        CGContextBeginPath(context);
        CGPoint lastPt;
        for (int i = [points getSize] - 1; i >=0; i--) {
            CGPoint pt = [pdfViewCtrl convertPdfPtToPageViewPt:[points getAt:i] pageIndex:pageIndex];
            if (i == [points getSize] - 1) {
                CGContextMoveToPoint(context, pt.x, pt.y);
            }else{
                CGContextAddLineToPoint(context, pt.x, pt.y);
            }
            
            if (i == 0) {
                CGContextSetBlendMode(context, kCGBlendModeDestinationAtop);
                CGContextStrokePath(context);
                
                float x1 = lastPt.x;
                float y1 = lastPt.y;
                float x2 = pt.x;
                float y2 = pt.y;

                float arctanglent = M_PI / 6;
                float arrow_len = lineWidth*4.0;

                CGPoint endPoint1 = [PaintUtility rotateVec:(x2 - x1) py:(y2 - y1) ang:arctanglent isChlen:YES newLine:arrow_len];
                CGPoint endPoint2 = [PaintUtility rotateVec:(x2 - x1) py:(y2 - y1) ang:-arctanglent isChlen:YES newLine:arrow_len];

                float x3 = x2 - endPoint1.x;
                float y3 = y2 - endPoint1.y;
                float x4 = x2 - endPoint2.x;
                float y4 = y2 - endPoint2.y;

                if (isnan(x3) || isnan(y3) || isnan(x2) || isnan(y2)|| isnan(x4) || isnan(y4)) {
                    return;
                }
                CGMutablePathRef path = CGPathCreateMutable();
                CGPathMoveToPoint(path, NULL, x3, y3);
                CGPathAddLineToPoint(path, NULL, x2, y2);
                CGPathAddLineToPoint(path, NULL, x4, y4);
                CGPathRef pathToDraw = CGPathCreateCopyByStrokingPath(path, NULL, 0.1, kCGLineCapRound, kCGLineJoinRound, 0);

                CGContextAddPath(context, pathToDraw);
                CGContextStrokePath(context);
                CGPathRelease(path);
                CGPathRelease(pathToDraw);
            }
            lastPt = pt;
        }
        if (annotImage) {
//            CGContextSaveGState(context);
//            CGRect rect = CGRectMake(ceilf(drawRect.origin.x), ceilf(drawRect.origin.y), ceilf(annotImage.size.width), ceilf(annotImage.size.height));
//            CGContextTranslateCTM(context, rect.origin.x, rect.origin.y);
//            CGContextTranslateCTM(context, 0, rect.size.height);
//            CGContextScaleCTM(context, 1.0, -1.0);
//            CGContextTranslateCTM(context, -rect.origin.x, -rect.origin.y);
//            CGContextDrawImage(context, rect, [annotImage CGImage]);
//            CGContextRestoreGState(context);
        }else{
            color = [UIColor colorWithRGB:[callout getBorderColor]];
            CGContextSetRGBStrokeColor(context,[color red], [color green], [color blue],callout.opacity);
            CGContextStrokeRect(context, drawRect);
        }
    }
}

@end


@interface FSDrawCaretTimerWeakObj ()
@property (nonatomic, weak) id target;
@end


@implementation FSDrawCaretTimerWeakObj

- (instancetype)initWithTarget:(id)target {
    _target = target;
    return self;
}

- (id)forwardingTargetForSelector:(SEL)selector {
    return _target;
}

- (void)forwardInvocation:(NSInvocation *)invocation {
    void *null = NULL;
    [invocation setReturnValue:&null];
}

- (NSMethodSignature *)methodSignatureForSelector:(SEL)selector {
    return [NSObject instanceMethodSignatureForSelector:@selector(init)];
}

- (BOOL)respondsToSelector:(SEL)aSelector {
    return [_target respondsToSelector:aSelector];
}

- (BOOL)isEqual:(id)object {
    return [_target isEqual:object];
}

- (NSUInteger)hash {
    return [_target hash];
}

- (Class)superclass {
    return [_target superclass];
}

- (Class)class {
    return [_target class];
}

- (BOOL)isKindOfClass:(Class)aClass {
    return [_target isKindOfClass:aClass];
}

- (BOOL)isMemberOfClass:(Class)aClass {
    return [_target isMemberOfClass:aClass];
}

- (BOOL)conformsToProtocol:(Protocol *)aProtocol {
    return [_target conformsToProtocol:aProtocol];
}

- (NSString *)description {
    return [_target description];
}

- (NSString *)debugDescription {
    return [_target debugDescription];
}

@end
