/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import <uiextensionsDynamic/uiextensionsDynamic.h>

NS_ASSUME_NONNULL_BEGIN

#ifndef FSAnnotPermissionDelegateProtocol
#define FSAnnotPermissionDelegateProtocol
/** @brief Annotation permission delegate. */
@protocol FSAnnotPermissionDelegate <NSObject>
@optional
/** @brief Judge whether the annotation can be modified.
 *
 *  @param[in]    annot    The annotation object.
 *
 *  @return       If the annotation can be modified, return YES otherwise return NO.
 */
- (BOOL)canModifyAnnot:(FSAnnot*)annot;
/** @brief Judge whether the annotation can be deleted.
 *
 *  @param[in]    annot   The annotation object.
 *
 *  @return       If the annotation can be delete, return YES otherwise return NO.
 */
- (BOOL)canDeleteAnnot:(FSAnnot*)annot;
/** @brief Judge whether the annotation can add reply.
 *
 *  @param[in]    annot    The annotation object.
 *
 *  @return       If the annotation can add reply, return YES otherwise return NO.
 */
- (BOOL)canReplyAnnot:(FSAnnot*)annot;

/** @brief Judge whether the annotation can add an popup.
 *
 *  @param[in]    annot    The annotation object.
 *
 *  @return       If the annotation can add popup, return YES otherwise return NO.
 */
- (BOOL)canAddPopupAnnot:(FSAnnot*)annot;
@end
#endif

@interface UIExtensionsManager ()
@property (nonatomic, weak) id<FSAnnotPermissionDelegate> annotPermissionDelegate;
@end

@interface UIExtensionsManager (FSAnnotPermission)
- (BOOL)annotsPermissionCanAdd;
- (BOOL)annotsPermissionCanModify;
- (BOOL)annotsPermissionCanDelete;
- (BOOL)annotsPermissionCanFlatten;
- (BOOL)annotsPermissionCanModify:(FSAnnot *)annot;
- (BOOL)annotsPermissionCanDelete:(FSAnnot *)annot;
- (BOOL)annotsPermissionCanReply:(FSAnnot *)annot;
- (BOOL)annotPermissionCanFlatten:(FSAnnot *)annot;
- (BOOL)allAnnotPermission;
- (BOOL)allowFormPermission;
@end

NS_ASSUME_NONNULL_END
