/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "UIExtensionsManager+FSAnnotPermission.h"
#import "FSPermissionUtilty.h"

@implementation UIExtensionsManager (FSAnnotPermission)

- (BOOL)annotsPermissionCanAdd{
    if (!self.enableModifyDocument) {
        return NO;
    }
    BOOL res = [Utility canAddAnnot:self.pdfViewCtrl];
    if ([self.annotsPermissionDelegate respondsToSelector:@selector(canAddAnnot)]) {
        res &= [self.annotsPermissionDelegate canAddAnnot];
    }
    return res;
}

- (BOOL)annotsPermissionCanModify{
    if (!self.enableModifyDocument) {
        return NO;
    }
    if ([self.annotsPermissionDelegate respondsToSelector:@selector(canModifyAnnot)]) {
        if (![self.annotsPermissionDelegate canModifyAnnot]) {
            return NO;
        }
    }
    return YES;
}

- (BOOL)annotsPermissionCanDelete{
    if (!self.enableModifyDocument) {
        return NO;
    }
    if ([self.annotsPermissionDelegate respondsToSelector:@selector(canDeleteAnnot)]) {
        if (![self.annotsPermissionDelegate canDeleteAnnot]) {
            return NO;
        }
    }
    return YES;
}

- (BOOL)annotsPermissionCanFlatten{
    BOOL docHasSignature = [Utility isDocumentSigned:[self.pdfViewCtrl currentDoc]];
    BOOL allowEdit = docHasSignature ? NO : [FSPermissionUtilty canModifyContents:self];
    return !(!allowEdit || ![FSPermissionUtilty canAddAnnot:self]) && [self annotsPermissionCanDelete];
}

- (BOOL)annotPermissionCanModify:(FSAnnot *)annot
{
    if (!self.enableModifyDocument) {
        return NO;
    }
    if ([self.annotPermissionDelegate respondsToSelector:@selector(canModifyAnnot:)]) {
        return [self.annotPermissionDelegate canModifyAnnot:annot];
    }
    return YES;
}

- (BOOL)annotPermissionCanDelete:(FSAnnot *)annot
{
    if (!self.enableModifyDocument) {
        return NO;
    }
    if ([self.annotPermissionDelegate respondsToSelector:@selector(canDeleteAnnot:)]) {
        return [self.annotPermissionDelegate canDeleteAnnot:annot];
    }
    return YES;
}

- (BOOL)annotPermissionCanReply:(FSAnnot *)annot
{
    if (!self.enableModifyDocument) {
        return NO;
    }
    if ([self.annotPermissionDelegate respondsToSelector:@selector(canReplyAnnot:)]) {
        return [self.annotPermissionDelegate canReplyAnnot:annot];
    }
    return YES;
}

- (BOOL)annotPermissionCanFlatten:(FSAnnot *)annot{
    return [self annotsPermissionCanModify:annot] && [self annotsPermissionCanFlatten];
}

- (BOOL)annotsPermissionCanModify:(FSAnnot *)annot
{
    return [self annotsPermissionCanModify] && [self annotPermissionCanModify:annot];
}

- (BOOL)annotsPermissionCanDelete:(FSAnnot *)annot
{
    return [self annotsPermissionCanDelete] && [self annotPermissionCanDelete:annot];
}

- (BOOL)annotsPermissionCanReply:(FSAnnot *)annot
{
    return [self annotsPermissionCanAdd] && [self annotPermissionCanReply:annot];
}

- (BOOL)allowFormPermission{
    return [self annotsPermissionCanAdd] && [self allAnnotPermission];
}

- (BOOL)allAnnotPermission{
    return [self allowFormPermissionAnnot:nil];
}

- (BOOL)allowFormPermissionAnnot:(nullable FSAnnot *)annot
{
    return !self.annotPermissionDelegate || ([self annotPermissionCanModify:annot] && [self annotPermissionCanDelete:annot] && [self annotPermissionCanReply:annot]);
}

@end
