/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "ComparisonViewController.h"
#import "FileSelectDestinationViewController.h"
#import "MBProgressHUD.h"
#import "FSNavigationController.h"
#import "FSPropertyMainView.h"
#import "FSFileSelectDestinationViewController.h"
#import "UIExtensionsManager+Private.h"

#define CompareModeOverlay 10000000
#define CompareModeAbreast 10000001
#define CompareModeBtnWidth 154
#define CompareModeBtnHeight 106
#define CompareColorOld 20000000
#define CompareColorNew 20000001
#define CompareResultInfoBase 30000000
#define CompareResultInfoCompared 30000001

@interface ComparisonViewController ()<FSPropertyMainViewDelegate>

@property (nonatomic, strong) UIBarButtonItem *buttonCancel;
@property (nonatomic, strong) UIBarButtonItem *buttonDone;
@property (nonatomic,strong) UIButton *selectedBtn;
@property (nonatomic, strong) FSPropertyMainView *propertyMainView;
@property (nonatomic, strong) NSMutableDictionary *selectedColorDict;
@property (nonatomic, assign) int selectedColorBtnTag;

@property (nonatomic, strong) NSMutableDictionary *tempFilesDict;
@property (nonatomic, strong) UIView *containterView;

@property (nonatomic, strong) NSString *oldFilePwd;
@property (nonatomic, strong) NSString *cnewFilePwd;
@property (nonatomic, strong) FSToolbar *topToolbar;
@end

@implementation ComparisonViewController

#pragma mark - Initialization
- (instancetype)init {
    if (self = [super init]) {
        self.selectedFileArr = [NSMutableArray array];
        self.selectedFilePwdDict = [NSMutableDictionary dictionary];
        
        self.propertyMainView = [[FSPropertyMainView alloc] initWithUIExtensionsManager:nil];
        self.propertyMainView.delegate = self;
        self.selectedColorDict = @{ @(CompareColorOld):@(0xB9534E),@(CompareColorNew):@(0x82C84F)}.mutableCopy;
        
        self.tempFilesDict = @{}.mutableCopy;
        self.oldFilePwd = nil;
        self.cnewFilePwd = nil;
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    self.topToolbar.hidden = NO;
    // Do any additional setup after loading the view.
    NSArray *result = [self.selectedFileArr sortedArrayUsingComparator:^NSComparisonResult(NSString *p1, NSString *p2){
        return [[self getModifyTime:p1] compare:[self getModifyTime:p2]];
    }];
    self.selectedFileArr = [[NSMutableArray alloc] initWithArray:result];
    
    self.view.backgroundColor = self.fs_backgroundColor_B2;
    
    UIScrollView *verticalScrollView = [[UIScrollView alloc] init];
    [self.view addSubview:verticalScrollView];
    [verticalScrollView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.bottom.mas_equalTo(self.view);
        make.top.mas_equalTo(self.topToolbar.mas_bottom);
    }];

    self.containterView = [[UIView alloc] init];
    [verticalScrollView addSubview:self.containterView];
    
    [self.containterView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.left.bottom.and.right.equalTo(verticalScrollView);
        make.width.equalTo(verticalScrollView);
    }];
    
    
    //[self initNavigationBar];
    [self initContentView];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    self.navigationController.navigationBarHidden = YES;
}

-(NSDate *)getModifyTime:(NSString *)filePath {
    NSError *error = nil;
    NSDictionary *fileAttributes = [[NSFileManager defaultManager] attributesOfItemAtPath:filePath error:&error];
    NSDate *fileModDate = [fileAttributes objectForKey:NSFileModificationDate];
    return fileModDate;
}

- (FSToolbar *)topToolbar
{
    if (_topToolbar == nil) {
        FSToolbar* topToolbar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleTop];
        [topToolbar setHidden:YES];
        _topToolbar = topToolbar;
        topToolbar.backgroundColor = self.fs_backgroundColor_B2;
        [self.view  addSubview:topToolbar];
        FSItem* cancelItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kCancel")];
        [cancelItem setFont:self.fs_standardGeneralFont_GF2];
        [BindThemeKeySel(cancelItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem*  _Nonnull target, SEL  _Nonnull sel) {
            [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
        }];
        @fs_weakify(self)
        cancelItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @fs_strongify(self)
            [self cancelAction];
        };
        [topToolbar addItem:cancelItem atPosition:FSItemPositionLeft];
        
        FSItem* doneItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kOK")];
        [doneItem setFont:self.fs_standardGeneralFont_GF2];
        [BindThemeKeySel(doneItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem*  _Nonnull target, SEL  _Nonnull sel) {
            [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
        }];
        doneItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @fs_strongify(self)
            [self doneAction];
        };
        [topToolbar addItem:doneItem atPosition:FSItemPositionRight];
        
        FSItem* titleItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kComparisonVCTitle")];
        titleItem.accessibilityTraits = UIAccessibilityTraitHeader;
        titleItem.customAccessibilityTraits = UIAccessibilityTraitHeader;
        [titleItem setFont:self.fs_standardGeneralFont_GF1];
        [titleItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
        [topToolbar addItem:titleItem atPosition:FSItemPositionCenter];

        [topToolbar mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.right.top.mas_equalTo(topToolbar.superview);
        }];
    }
    return _topToolbar;
}

#pragma mark - Configure NavigationBar
- (void)initNavigationBar {
    UIButton *buttonLeft = [UIButton buttonWithType:UIButtonTypeCustom];
    buttonLeft.frame = CGRectMake(0.0, 0.0, 65.0, 32);
    buttonLeft.titleLabel.font = self.fs_standardGeneralFont_GF2;
    [buttonLeft setTitle:FSLocalizedForKey(@"kCancel") forState:UIControlStateNormal];
    [BindThemeKeySel(buttonLeft, setTitleColor:forState:) fs_updateThemeCompleted:^(UIButton*  _Nonnull target, SEL  _Nonnull sel) {
        [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
    }];
    [buttonLeft addTarget:self action:@selector(cancelAction) forControlEvents:UIControlEventTouchUpInside];
    
    self.buttonCancel = [[UIBarButtonItem alloc] initWithCustomView:buttonLeft];
    [self.navigationItem setLeftBarButtonItem:self.buttonCancel];
    [self.buttonCancel setEnabled:YES];

    self.buttonDone = [[UIBarButtonItem alloc] initWithTitle:@"" style:UIBarButtonItemStylePlain target:self action:@selector(doneAction)];
    [self.buttonDone setEnabled:YES];
    
    [BindThemeKeySel(self.buttonDone, setTintColor:) fs_updateThemeCompleted:^(UIBarButtonItem*  _Nonnull target, SEL  _Nonnull sel) {
        NSDictionary *attributesRight = [NSDictionary dictionaryWithObjectsAndKeys:
                                         target.fs_primaryColor,
                                            NSForegroundColorAttributeName,
                                         target.fs_standardGeneralFont_GF2,
                                            NSFontAttributeName,
                                            nil];
        [target setTitleTextAttributes:attributesRight forState:UIControlStateNormal];
    }];
    
    [self.navigationItem setRightBarButtonItem:self.buttonDone];
    self.navigationController.navigationBar.barTintColor = self.fs_backgroundColor_B2;
    self.navigationController.navigationBar.titleTextAttributes = @{NSForegroundColorAttributeName : self.fs_textColor_T4};
    
    [self.buttonDone setTitle:FSLocalizedForKey(@"kOK")];
    
    self.title = FSLocalizedForKey(@"kComparisonVCTitle");
}

#pragma mark - Configure content view
- (void)initContentView {
    self.containterView.backgroundColor = self.fs_backgroundColor_B2;
    // compare mode
    UILabel *comapreItemTitle = [[UILabel alloc] init];
    comapreItemTitle.numberOfLines = 0;
    comapreItemTitle.text = [FSLocalizedForKey(@"kCompareItemTitle") uppercaseString];
    comapreItemTitle.font = self.fs_standardWeakFont_WF1;
    comapreItemTitle.textColor = self.fs_textColor_T3;
    comapreItemTitle.backgroundColor = self.fs_backgroundColor_B2;
    UIView* titleBackgroundView = [[UIView alloc] init];
    titleBackgroundView.backgroundColor = self.fs_backgroundColor_B2;
    [titleBackgroundView addSubview:comapreItemTitle];
    [self.containterView addSubview:titleBackgroundView];
    
    [titleBackgroundView mas_makeConstraints:^(MASConstraintMaker *make) {
        if (@available(iOS 11.0, *)) {
            make.top.mas_equalTo(self.containterView.mas_safeAreaLayoutGuideTop);
        } else {
            make.top.mas_equalTo(self.containterView.mas_top);
        }
        make.left.mas_equalTo(titleBackgroundView.superview.mas_safeAreaLayoutGuideLeft);
        make.right.mas_equalTo(titleBackgroundView.superview.mas_safeAreaLayoutGuideRight);
        make.height.mas_equalTo(32);
    }];
    
    [comapreItemTitle mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(comapreItemTitle.superview);
        make.left.mas_equalTo(comapreItemTitle.superview).offset(16);
        make.right.mas_equalTo(comapreItemTitle.superview);
        make.height.mas_equalTo(32);
    }];

    UIImage *frameBGImg = FSImageNamedAndPrimaryColor(@"comparison_overlay_bg");
    UIImage* bgImg = [frameBGImg fs_addImage:FSImageNamed(@"comparison_overlay")];
    
    UIButton *abreastButton = [[UIButton alloc] init];
    abreastButton.accessibilityLabel = FSLocalizedForKey(@"kCompareButtonAbreast");
    [abreastButton setBackgroundImage:bgImg forState:UIControlStateNormal];
    abreastButton.tag = CompareModeAbreast;
    [abreastButton addTarget:self action:@selector(onTapCompareModebtn:) forControlEvents:UIControlEventTouchUpInside];
    
    UIView* abreastView = [[UIView alloc] init];
    abreastView.backgroundColor = self.fs_backgroundColor_B1;
    [self.containterView addSubview:abreastView];
    [abreastView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(comapreItemTitle.mas_bottom);
        make.left.right.mas_equalTo(abreastView.superview);
        make.height.mas_equalTo(168);
    }];
    [self.containterView addSubview:abreastButton];
    
    [abreastButton mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(comapreItemTitle.mas_bottom).offset(15);
        make.left.mas_equalTo(self.containterView.mas_centerX).offset(-CompareModeBtnWidth / 2);
        make.width.mas_equalTo(CompareModeBtnWidth);
        make.height.mas_equalTo(CompareModeBtnHeight);
    }];
    
    UILabel* label = [[UILabel alloc] init];
    label.font = self.fs_standardWeakFont_WF1;
    label.textColor = self.fs_textColor_T4;
    label.textAlignment = NSTextAlignmentCenter;
    label.text = FSLocalizedForKey(@"kCompareButtonAbreast");
    [self.containterView addSubview:label];
    [label mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.mas_equalTo(label.superview);
        make.top.mas_equalTo(abreastButton.mas_bottom).offset(8);
        make.size.mas_equalTo(CGSizeMake(80, 20));
    }];

    self.selectedBtn = abreastButton;
    self.selectedBtn.selected = YES;
    
    NSString *oldFileName = [[self.selectedFileArr[0] lastPathComponent] stringByDeletingPathExtension];
    UIView *lastview = [self createFileItemInfo:[FSLocalizedForKey(@"kCompareFileInfoOld") uppercaseString] filename:oldFileName defaultColor:(UInt32)[self.selectedColorDict[@(CompareColorOld)] unsignedIntegerValue] lastView:abreastView btnTag:CompareColorOld];
    
    NSString *newFileName = [[self.selectedFileArr[1] lastPathComponent] stringByDeletingPathExtension];
    UIView *newFileView = [self createFileItemInfo:[FSLocalizedForKey(@"kCompareFileInfoNew") uppercaseString] filename:newFileName defaultColor:(UInt32)[self.selectedColorDict[@(CompareColorNew)] unsignedIntegerValue] lastView:lastview btnTag:CompareColorNew];
    
    [self.containterView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.mas_equalTo(newFileView.mas_bottom).offset(20);
    }];
}

-(UIView *)createFileItemInfo:(NSString *)title filename:(NSString *)fileName defaultColor:(UInt32)defaultColor lastView:(UIView *)lastview btnTag:(int)btnTag {
    UILabel *fileItemTitle = [[UILabel alloc] init];
    fileItemTitle.numberOfLines = 0;
    fileItemTitle.text = FSLocalizedForKey(title);
    
    fileItemTitle.font = self.fs_standardWeakFont_WF1;
    fileItemTitle.textColor = self.fs_textColor_T3;
    fileItemTitle.backgroundColor = self.fs_backgroundColor_B2;
    
    UIView* titleBackgroundView = [[UIView alloc] init];
    titleBackgroundView.backgroundColor = self.fs_backgroundColor_B2;
    [self.containterView addSubview:titleBackgroundView];
    [titleBackgroundView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(lastview.mas_bottom);
        make.left.right.mas_equalTo(titleBackgroundView.superview.mas_safeAreaLayoutGuide);
        make.height.mas_equalTo(24);
    }];
    
    [self.containterView addSubview:fileItemTitle];
    [fileItemTitle mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(lastview.mas_bottom);
        make.left.mas_equalTo(self.view.mas_safeAreaLayoutGuideLeft).offset(16);
        make.height.mas_equalTo(24);
    }];
    
    UIButton *filenameBtn = [[UIButton alloc] init];
    filenameBtn.backgroundColor = self.fs_backgroundColor_B1;
    UIFont *textFont = self.fs_standardGeneralFont_GF2;
    filenameBtn.titleLabel.font = textFont;
    filenameBtn.titleLabel.textAlignment = NSTextAlignmentLeft;
    filenameBtn.contentHorizontalAlignment = UIControlContentHorizontalAlignmentLeft;
    [filenameBtn setTitle:fileName forState:UIControlStateNormal];
    [filenameBtn setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
    [filenameBtn addTarget:self action:@selector(onTapCompareFileNameBtn:) forControlEvents:UIControlEventTouchUpInside];
    filenameBtn.tag = btnTag;
    filenameBtn.titleEdgeInsets = UIEdgeInsetsMake(0, 16, 0, 0);
    [filenameBtn setBackgroundImage:[UIImage fs_imageWithColor:self.fs_dividerColor_P1] forState:UIControlStateHighlighted];
    
    UIView * filenameBtnView = [[UIView alloc] init];
    filenameBtnView.backgroundColor = self.fs_backgroundColor_B1;
    
    [self.containterView addSubview:filenameBtnView];
    [self.containterView addSubview:filenameBtn];
    
    
    [filenameBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(filenameBtn.superview.mas_safeAreaLayoutGuideLeft);
        make.right.mas_equalTo(filenameBtn.superview.mas_safeAreaLayoutGuideRight);
        make.top.mas_equalTo(fileItemTitle.mas_bottom);
        make.height.mas_equalTo(50);
    }];
    
    [filenameBtnView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(filenameBtnView.superview);
        make.top.bottom.equalTo(filenameBtn);
    }];
    
    UIImageView* gotoImgView = [[UIImageView alloc] init];
    gotoImgView.image = FSImageNamed(@"comparison_goto");
    [filenameBtn addSubview:gotoImgView];
    [gotoImgView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.mas_equalTo(gotoImgView.superview);
        make.right.mas_equalTo(-3);
        make.size.mas_equalTo(CGSizeMake(40, 40));
    }];
    
    return filenameBtn;
}

#pragma mark - clear tmp file
- (BOOL)clearCacheWithPrefix:(NSString *)prefix {
    NSString *cacheFilePath = NSTemporaryDirectory();
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSArray *subChildPath = [fileManager subpathsOfDirectoryAtPath:cacheFilePath error:nil];
    NSError *error = nil;
    for (NSString *subPath in subChildPath) {
        NSString *path = [cacheFilePath stringByAppendingFormat:@"%@", subPath];
        NSString *filename = [path lastPathComponent];
        if ([filename hasPrefix:prefix]) {
            [fileManager removeItemAtPath:path error:&error];
            if (error) {
                NSLog(@"delete file faild");
            }
        }
    }
    return YES;
}

#pragma mark - Action methods
- (BOOL)removeEncrypt:(FSPDFDoc *)pdfDoc filePath:(NSString *)filePath{
    BOOL isOK = NO;
    NSString *decryptedPDFPath = nil;
    NSFileManager *fileManager = [NSFileManager defaultManager];
    
    @autoreleasepool {
        isOK = [pdfDoc removeSecurity];
        if (isOK) {
            NSString *fileName = [filePath lastPathComponent];
            decryptedPDFPath = [NSTemporaryDirectory() stringByAppendingString:[NSString stringWithFormat:@"_decryped_%@", fileName]];
            if ([fileManager fileExistsAtPath:decryptedPDFPath isDirectory:nil]) {
                [fileManager removeItemAtPath:decryptedPDFPath error:nil];
            }
            isOK = [pdfDoc saveAs:decryptedPDFPath save_flags:FSPDFDocSaveFlagNormal];
        }
        if (isOK) {
            isOK = [fileManager removeItemAtPath:filePath error:nil];
            
            if (isOK) {
                isOK = [fileManager moveItemAtPath:decryptedPDFPath toPath:filePath error:nil];
            }
        }
    }
    
    return isOK;
}

- (void)doneAction {
    if (![FSLibrary hasModuleLicenseRight:FSModuleNameComparison]) {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kNoModuleInvalidLicense") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
        [alertController addAction:action];
        [[self fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
        return;
    }
    
    [self.buttonCancel setEnabled:NO];
    [self.buttonDone setEnabled:NO];
    
    NSString *oldFilepath = [self.selectedFileArr objectAtIndex:0];
    NSString *newFilepath = [self.selectedFileArr objectAtIndex:1];
    if ([oldFilepath isEqualToString:newFilepath]) {
        [self.view fs_showHUDMessage:FSLocalizedForKey(@"kCompareNotSomeFile") afterHideDelay:1.5];
        [self.buttonCancel setEnabled:YES];
        [self.buttonDone setEnabled:YES];
        return ;
    }
    
    __block NSMutableDictionary *comapreResultInfos = @{}.mutableCopy;
    comapreResultInfos[@"compareMode"] = @(self.selectedBtn.tag);
    comapreResultInfos[@"compareColor"] = self.selectedColorDict;
    comapreResultInfos[@"compareFiles"] = self.selectedFileArr;
    
    //remove files
    [self clearCacheWithPrefix:@"new"];
    [self clearCacheWithPrefix:@"old"];
    self.oldFilePwd = nil;
    self.cnewFilePwd = nil;
    
    // load file with password
    FSPDFDoc *oldDoc = [[FSPDFDoc alloc] initWithPath:oldFilepath];
    self.oldFilePwd = [self.selectedFilePwdDict objectForKey:oldFilepath];
    [oldDoc load:self.oldFilePwd];
//    int oldPageCount = [oldDoc getPageCount];
    
    FSPDFDoc *newDoc = [[FSPDFDoc alloc] initWithPath:newFilepath];
    self.cnewFilePwd = [self.selectedFilePwdDict objectForKey:newFilepath];
    [newDoc load:self.cnewFilePwd];
//    int newPageCount = [newDoc getPageCount];
    
    MBProgressHUD *progressHUD = [MBProgressHUD showHUDAddedTo:self.view animated:YES];
    progressHUD.detailsLabelText =  FSLocalizedForKey(@"kCompareing");
    
    // check compare module key and generate ComparedDoc
    @try {
        FSComparison *comparison = [[FSComparison alloc] initWithBase_doc:oldDoc compared_doc:newDoc];
        if ([comparison isEmpty]) return;
    } @catch (NSException *exception) {
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        if ([exception.name isEqualToString:@"FSErrInvalidLicense"]) {
            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kNoModuleInvalidLicense") preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
            [alertController addAction:action];
            [[self fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
        }
        [self.buttonCancel setEnabled:YES];
        [self.buttonDone setEnabled:YES];
        return;
    }

    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, 0.01 * NSEC_PER_SEC), dispatch_get_main_queue(), ^(void){
        BOOL hasError = NO;
        
        FSPDFDoc *poldDoc = [[FSPDFDoc alloc] initWithPath:oldFilepath];
        [poldDoc load:self.oldFilePwd];
        
        FSPDFDoc *pnewDoc = [[FSPDFDoc alloc] initWithPath:newFilepath];
        [pnewDoc load:self.cnewFilePwd];
        
        @try {
            FSComparison *comparison = [[FSComparison alloc] initWithBase_doc:poldDoc compared_doc:pnewDoc];
            
            FSPDFDoc *doc = [comparison generateComparedDoc:FSComparisonCompareTypeAll];
            
            NSString *tempDirectory = NSTemporaryDirectory();
            NSString *compareResultpdf = [tempDirectory stringByAppendingString:@"compareResult.pdf"];
            
            [doc saveAs:compareResultpdf save_flags:FSPDFDocSaveFlagNormal];
            
            comapreResultInfos[@"compareResultFilePath"] = compareResultpdf;
        } @catch (NSException *exception) {
            hasError = YES;
        }
        
        if (hasError) {
            [MBProgressHUD hideHUDForView:self.view animated:YES];
            
            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kUnknownError") preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
            [alertController addAction:action];
            [[self fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
            
            [self.buttonCancel setEnabled:YES];
            [self.buttonDone setEnabled:YES];
            
            return ;
        }
        
        [self.buttonDone setEnabled:YES];
        
        [MBProgressHUD hideHUDForView:self.view animated:YES];
        
        [[self.navigationController.viewControllers objectAtIndex:0] dismissViewControllerAnimated:YES
                                                                                        completion:^{
                                                                                            if (self.doneHandler) {
                                                                                                self.doneHandler(self,comapreResultInfos);
                                                                                            }
                                                                                        }];
        
        
    });
}

-(NSString *)drawAnnotToComareResultPdf:(NSString *)compareResultPath resultDicts:(NSMutableDictionary *)resultDicts {
    FSPDFDoc *compareResultPdf = [[FSPDFDoc alloc] initWithPath:compareResultPath];
    
    FSErrorCode errorcode = [compareResultPdf load:self.oldFilePwd];
    if (errorcode != FSErrSuccess) {
        errorcode = [compareResultPdf load:self.cnewFilePwd];
    }
    
    int pageCount = [compareResultPdf getPageCount] / 2;
    
    for (int m = 0 ; m < pageCount; m++) {
        @autoreleasepool{
            
            int leftIndex = 2 * m;
            int rightIndex = 2 * m + 1;
            
            FSCompareResults *compareResult = [resultDicts objectForKey:@(m)];
            
            FSPDFPage *leftPage = [compareResultPdf getPage:leftIndex];
            [leftPage flatten:NO options:FSPDFPageFlattenAll];
            unsigned long baseResultInfoSize = [compareResult.base_doc_results getSize];
            
            for (int i = 0; i < baseResultInfoSize; i++)
            {
                FSCompareResultInfo *item = [compareResult.base_doc_results getAt:i];
                FSCompareResultInfoCompareResultType type = item.type;
                
                if ([[self getDeleteCompareResultTypeArr] containsObject:@(type)])
                {
                    NSString* res_string = [NSString stringWithFormat:@"\"%@\"", item.diff_contents];
                    [self createTextMarkup:leftPage rects:item.rect_array color:0xE23224 type:FSAnnotStrikeOut compareContents:res_string compareType:[NSString stringWithFormat:@"Compare : %@",FSLocalizedForKey(@"kComapreContainerLegendDelete")]];
                }
                else if ([[self getInsertCompareResultTypeArr] containsObject:@(type)] )
                {
                    NSString* res_string = [NSString stringWithFormat:@"\"%@\"", item.diff_contents];
                    [self createNote:leftPage rects:item.rect_array color:0x4F8DEE type:CompareResultInfoBase compareContents:res_string compareType:[NSString stringWithFormat:@"Compare : %@",FSLocalizedForKey(@"kComapreContainerLegendInsert")]];
                }
                else if ([[self getReplaceCompareResultTypeArr] containsObject:@(type)] )
                {
                    NSString* res_string = [NSString stringWithFormat:@"[%@]: \"%@\"\r\n[%@]: \"%@\"",FSLocalizedForKey(@"kCompareFileInfoOld"),item.diff_contents,FSLocalizedForKey(@"kCompareFileInfoNew"),[compareResult.compared_doc_results getAt:i].diff_contents];
                    [self createTextMarkup:leftPage rects:item.rect_array color:0xEE8F00 type:FSAnnotSquiggly compareContents:res_string compareType:[NSString stringWithFormat:@"Compare : %@",FSLocalizedForKey(@"kComapreContainerLegendReplace")]];
                }
            }
            
            FSPDFPage *rightPage = [compareResultPdf getPage:rightIndex];
            [rightPage flatten:NO options:FSPDFPageFlattenAll];
            unsigned long comparedResultInfoSize = [compareResult.compared_doc_results getSize];
            
            for (int i = 0; i < comparedResultInfoSize; i++)
            {
                FSCompareResultInfo *item = [compareResult.compared_doc_results getAt:i];
                FSCompareResultInfoCompareResultType type = item.type;
                
                if ([[self getDeleteCompareResultTypeArr] containsObject:@(type)])
                {
                    NSString* res_string = [NSString stringWithFormat:@"\"%@\"", item.diff_contents];
                    [self createNote:rightPage rects:item.rect_array color:0xE23224 type:CompareResultInfoCompared compareContents:res_string compareType:[NSString stringWithFormat:@"Compare : %@",FSLocalizedForKey(@"kComapreContainerLegendDelete")]];
                }
                else if ([[self getInsertCompareResultTypeArr] containsObject:@(type)] )
                {
                    NSString* res_string = [NSString stringWithFormat:@"\"%@\"", item.diff_contents];
                    [self createTextMarkup:rightPage rects:item.rect_array color:0x4F8DEE type:FSAnnotUnderline compareContents:res_string compareType:[NSString stringWithFormat:@"Compare : %@",FSLocalizedForKey(@"kComapreContainerLegendInsert")]];
                }
                else if ([[self getReplaceCompareResultTypeArr] containsObject:@(type)] )
                {
                    NSString* res_string = [NSString stringWithFormat:@"[%@]: \"%@\"\r\n[%@]: \"%@\"",FSLocalizedForKey(@"kCompareFileInfoOld"),[compareResult.base_doc_results getAt:i].diff_contents,FSLocalizedForKey(@"kCompareFileInfoNew"),item.diff_contents];
                    [self createTextMarkup:rightPage rects:item.rect_array color:0xEE8F00 type:FSAnnotSquiggly compareContents:res_string compareType:[NSString stringWithFormat:@"Compare : %@",FSLocalizedForKey(@"kComapreContainerLegendReplace")]];
                }
            }
            
        }
    }
    
    NSString *tempDirectory = NSTemporaryDirectory();
    NSString *tmpcompareResultpdf = [tempDirectory stringByAppendingString:@"compareResult_compared.pdf"];
    
    [compareResultPdf saveAs:tmpcompareResultpdf save_flags:FSPDFDocSaveFlagIncremental];
    
    return tmpcompareResultpdf;
}

-(void)fillTwoDocument:(FSPDFDoc *)oldDoc newDocument:(FSPDFDoc *)newDoc completion:(void (^ __nullable)(void))completion{
    int oldPageCount = [oldDoc getPageCount];
    int newPageCount = [newDoc getPageCount];
    
    NSString *tempDirectory = NSTemporaryDirectory();
    NSString *oldTmpPath = [tempDirectory stringByAppendingString:@"old.pdf"];
    NSString *newTmpPath = [tempDirectory stringByAppendingString:@"new.pdf"];
    if (oldPageCount > newPageCount) {
        @try {
            for (int i = newPageCount  ; i < oldPageCount; i ++) {
                @autoreleasepool{
                    FSPDFPage *page = nil;
                    page = [newDoc insertPageSize:i size:FSPDFPageSizeA3];
                    
                    [page setRotation:FSRotation0];
                    
                    if (!page.isParsed) {
                        FSProgressive *progressive = [page startParse:FSPDFPageParsePageNormal pause:nil is_reparse:NO];
                        FSProgressiveState proState = FSProgressiveToBeContinued;
                        while (proState == FSProgressiveToBeContinued) {
                            proState = [progressive resume];
                        }
                        
                        if (proState != FSProgressiveFinished){
                            return;
                        }
                    }
                }
                
            }
            
        } @catch (NSException *exception) {
            return;
        }
    }
    [newDoc saveAs:newTmpPath save_flags:FSPDFDocSaveFlagNormal];
    
    if (oldPageCount < newPageCount) {
        @try {
            for (int i = oldPageCount ; i < newPageCount; i ++) {
                @autoreleasepool{
                    FSPDFPage *page = nil;
                    page = [oldDoc insertPageSize:i size:FSPDFPageSizeA3];
                    
                    [page setRotation:FSRotation0];
                    
                    if (!page.isParsed) {
                        FSProgressive *progressive = [page startParse:FSPDFPageParsePageNormal pause:nil is_reparse:NO];
                        FSProgressiveState proState = FSProgressiveToBeContinued;
                        while (proState == FSProgressiveToBeContinued) {
                            proState = [progressive resume];
                        }
                        
                        if (proState != FSProgressiveFinished){
                            return;
                        }
                    }
                }
                
            }
            
        } @catch (NSException *exception) {
            return;
        }
    }
    [oldDoc saveAs:oldTmpPath save_flags:FSPDFDocSaveFlagNormal];
    
    completion();
}

- (FSRectF *)normalizePDFRect:(FSRectF *)rectF {
    if (rectF.left > rectF.right) {
        float tmp = rectF.left;
        rectF.left = rectF.right;
        rectF.right = tmp;
    }
    
    if (rectF.top < rectF.bottom) {
        float tmp = rectF.top;
        rectF.top = rectF.bottom;
        rectF.bottom = tmp;
    }
    
    if (rectF.left == rectF.right) rectF.right += 1;
    if (rectF.top == rectF.bottom) rectF.top += 1;
    return rectF;
}

-(void)createNote:(FSPDFPage *)page rects:(FSRectFArray *)rects color:(UInt32)color type:(int)type compareContents:(NSString *)compareContents compareType:(NSString *)compareType {
    FSRectF *rcNote = [[FSRectF alloc] init];
    int rectSize = [rects getSize];
    if (rectSize > 0) {
        FSRectF *item = [rects getAt:0];
        rcNote.left = item.left;
        rcNote.top = item.top ;
        rcNote.right = item.right ;
        rcNote.bottom = item.bottom ;
        
        rcNote = [self normalizePDFRect:rcNote];
    }
    
    FSNote *note = [[FSNote alloc] initWithAnnot:[page addAnnot:FSAnnotNote rect:rcNote]];
    [note setContent:compareContents];
    [note setBorderColor:color];
    [note setSubject:@"Note"];
    [note setTitle:compareType];
    note.flags = FSAnnotFlagPrint | FSAnnotFlagNoZoom | FSAnnotFlagNoRotate;
    
    NSDate *now = [NSDate date];
    FSDateTime *time = [Utility convert2FSDateTime:now];
    [note setCreationDateTime:time];
    [note setModifiedDateTime:time];
    [note setUniqueID:[Utility getUUID]];

    if (type == CompareResultInfoBase) {
        [note setIconName:@"Insert"];
    }else{
        [note setIconName:@"Cross"];
    }
    
    [note resetAppearanceStream];
}

-(void)createTextMarkup:(FSPDFPage *)page rects:(FSRectFArray *)rects color:(UInt32)color type:(FSAnnotType)type compareContents:(NSString *)compareContents compareType:(NSString *)compareType{
    FSRectF *annotRect = [[FSRectF alloc] init];
    FSTextMarkup *textMarkup = [[FSTextMarkup alloc] initWithAnnot:[page addAnnot:type rect:annotRect]];
    [textMarkup setContent:compareContents];
    
    FSQuadPointsArray *quadPointsArr = [FSQuadPointsArray new];
    int rectSize = [rects getSize];
    for (int i=0; i < rectSize; i++) {

        FSRectF *item = [rects getAt:i];
        FSQuadPoints *quadPoints = [FSQuadPoints new];

        FSPointF* point = [FSPointF new];
        [point set:item.left y:item.top];
        quadPoints.first = point;
        [point set:item.right y:item.top];
        quadPoints.second = point;
        [point set:item.left y:item.bottom];
        quadPoints.third = point;
        [point set:item.right y:item.bottom];
        quadPoints.fourth = point;

        [quadPointsArr add:quadPoints];
    }
    [textMarkup setQuadPoints:quadPointsArr];
    
    [textMarkup setBorderColor:color];
    
    [textMarkup setTitle:compareType];
    
    NSDate *now = [NSDate date];
    FSDateTime *time = [Utility convert2FSDateTime:now];
    [textMarkup setCreationDateTime:time];
    [textMarkup setModifiedDateTime:time];
    [textMarkup setUniqueID:[Utility getUUID]];
    textMarkup.flags = FSAnnotFlagPrint;
    
    if (type == FSAnnotSquiggly) {
        textMarkup.subject = @"Squiggly";
    } else if (type == FSAnnotStrikeOut) {
        textMarkup.subject = @"Strikeout";
    } else if (type == FSAnnotUnderline) {
        textMarkup.subject = @"Underline";
    }
    
    [textMarkup resetAppearanceStream];
}

- (void)cancelAction {
    [self.selectedFileArr removeAllObjects];
    self.selectedFileArr = nil;
    
    if (self.cancelHandler) {
        self.cancelHandler(self);
    }
    [[self.navigationController.viewControllers objectAtIndex:0] dismissViewControllerAnimated:YES
                                                                                    completion:^{
                                                                                        
                                                                                    }];
}

- (void)onTapCompareModebtn:(UIButton *)btn {
    if (btn != self.selectedBtn) {
        self.selectedBtn.selected = NO;
        btn.selected = YES;
        self.selectedBtn = btn;
    }else{
        self.selectedBtn.selected = YES;
    }
}

-(void)onTapCompareFileNameBtn:(UIButton *)btn {
    
    FSFileSelectDestinationViewController *selectDestination = [[FSFileSelectDestinationViewController alloc] init];
    selectDestination.fileSelectDestinationType = FSFileSelectDestinationTypeSelectFile;
    selectDestination.fileSelectSpecialActionType = FSFileSelectSpecialActionTypeSingle;
    selectDestination.isRootFileDirectory = YES;
    selectDestination.expectFileType = @[@"pdf"];
    [selectDestination loadFilesWithPath:DOCUMENT_PATH];


    selectDestination.operatingHandler = ^(FSFileSelectDestinationViewController *controller, NSArray *destinationFolder) {
        
        if (destinationFolder.count <= 0) {
            return;
        }
        int replaceIndex = btn.tag == CompareColorOld ? 0 : 1;
        
        [controller.services dismissViewControllerAnimated:YES completion:^{
            NSString *filePath = destinationFolder[0];
            NSString *pathExtension = [filePath pathExtension];
            if( ! ([pathExtension isEqualToString:@"pdf"] || pathExtension.length == 0) ){
                [self showTips:[NSString stringWithFormat:FSLocalizedForKey(@"kCompareNotSupportFileFormat"), [filePath lastPathComponent]]];
                return ;
            }
            
            NSString *pwdStr = [self.selectedFilePwdDict objectForKey:filePath];
            if (!pwdStr) {
                pwdStr = @"";
            }
            FSPDFDoc *srcDoc = [[FSPDFDoc alloc] initWithPath:filePath];
            [Utility tryLoadDocument:srcDoc withPassword:pwdStr success:^(NSString * _Nonnull password) {
                [self.selectedFilePwdDict setObject:password forKey:filePath];
                
                FSPDFDoc *doc = [[FSPDFDoc alloc] initWithPath:filePath];
                [doc load:password];
                if ([doc isXFA] || [doc isWrapper] || [Utility checkFileisRMS:filePath password:@""]) {
                    [self showTips:[NSString stringWithFormat:FSLocalizedForKey(@"kCompareNotSupportFileFormat"), [filePath lastPathComponent]]];
                    return ;
                }
                
                NSString *fileName = [[destinationFolder[0] lastPathComponent] stringByDeletingPathExtension];
                [btn setTitle:fileName forState:UIControlStateNormal];
                [self.selectedFileArr replaceObjectAtIndex:replaceIndex withObject:destinationFolder[0]];
                
            } error:^(NSString * _Nonnull description) {
                [self showTips:description];
            } abort:nil sceneProduct:self];
        }];
    };
    selectDestination.cancelHandler = ^(FSFileSelectDestinationViewController *controller) {
        [controller dismissViewControllerAnimated:YES completion:nil];
    };
    FSNavigationController *selectDestinationNavController = [[FSNavigationController alloc] initWithRootViewController:selectDestination];
    if (DEVICE_iPAD) {
        selectDestinationNavController.modalPresentationStyle = UIModalPresentationFormSheet;
        selectDestinationNavController.modalTransitionStyle = UIModalTransitionStyleCoverVertical;
    }else {
        selectDestinationNavController.modalPresentationStyle = UIModalPresentationFullScreen;
    }
    [[self fs_currentSceneTopMostViewController] presentViewController:selectDestinationNavController animated:YES completion:nil];

}

-(void)showTips:(NSString *)message {
    MBProgressHUD *progressHUD = [MBProgressHUD showHUDAddedTo:self.view animated:YES];
    progressHUD.mode = MBProgressHUDModeText;
    progressHUD.detailsLabelText =  message;
    dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, 1.5 * NSEC_PER_SEC);
    dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
        [MBProgressHUD hideHUDForView:self.view animated:YES];
    });
}

- (void)onTapCompareColorBtn:(UIButton *)btn {
    self.selectedColorBtnTag = (int)btn.tag;
    [self.propertyMainView setPropertyColor:[self.selectedColorDict[@(self.selectedColorBtnTag)] intValue]];
    [self.propertyMainView resetProperties:@[@(FSPropertyTypePropertyColor)]];
    
    CGRect rect = [btn convertRect:btn.bounds toView:self.view];
    DevicePosition *devicePos = [[DevicePosition alloc] init];
    devicePos.phoneRect = rect;
    devicePos.inPhoneView = self.view;
    devicePos.padRect = btn.bounds;
    devicePos.inPadView = btn;
    [self.propertyMainView showPropertyMainView:devicePos];

}

#pragma mark -FSPropertyMainViewDelegate
- (void)propertyMainView:(FSPropertyMainView *)propertyMainView changedProperty:(FSPropertyType)property fromOldValue:(NSValue *)oldValue toValue:(NSValue *)newValue{
    if (property == FSPropertyTypePropertyColor) {
        int color = ((NSNumber *) newValue).intValue;
        self.selectedColorDict[@(self.selectedColorBtnTag)] = @(color);
        
        UIButton *colorbtn = (UIButton *)[self.view viewWithTag:self.selectedColorBtnTag];
        [colorbtn setBackgroundColor:[UIColor colorWithRGB:color]];
    }
}

#pragma mark - compareType array
-(NSArray *)getDeleteCompareResultTypeArr {
    NSArray *deleteArr = @[
                           @(FSCompareResultInfoCompareResultTypeDeleteText),
                           @(FSCompareResultInfoCompareResultTypeDeleteImage),
                           @(FSCompareResultInfoCompareResultTypeDeletePath),
                           @(FSCompareResultInfoCompareResultTypeDeleteShading),
                           @(FSCompareResultInfoCompareResultTypeDeleteAnnot)
                           ];
    return deleteArr;
}

-(NSArray *)getInsertCompareResultTypeArr {
    NSArray *insertArr = @[
                           @(FSCompareResultInfoCompareResultTypeInsertText),
                           @(FSCompareResultInfoCompareResultTypeInsertImage),
                           @(FSCompareResultInfoCompareResultTypeInsertPath),
                           @(FSCompareResultInfoCompareResultTypeInsertShading),
                           @(FSCompareResultInfoCompareResultTypeInsertAnnot)
                           ];
    return insertArr;
}

-(NSArray *)getReplaceCompareResultTypeArr {
    NSArray *replaceArr = @[
                            @(FSCompareResultInfoCompareResultTypeReplaceText),
                            @(FSCompareResultInfoCompareResultTypeReplaceImage),
                            @(FSCompareResultInfoCompareResultTypeReplacePath),
                            @(FSCompareResultInfoCompareResultTypeReplaceShading),
                            @(FSCompareResultInfoCompareResultTypeReplaceAnnot)
                            ];
    return replaceArr;
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

@end
