/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "CropViewController.h"
#import "MBProgressHUD.h"
#import "FSSettingBar+private.h"
#import "FSSettingBar.h"
#import "Utility.h"
#import "FSCropManager.h"

@class SettingBar;

@interface CropPDFView : UIView
@property (nonatomic, weak) FSPDFViewCtrl *pdfViewCtrl;
@property (nonatomic, assign) int pageIndex;
@property (nonatomic, assign) BOOL needPauseNow;
- (id)initWithFrame:(CGRect)frame;
@end

@interface CropView : UIView

@property (nonatomic, assign) CGRect originalRect;
@property (nonatomic, assign) CGSize originalSize;
@property (nonatomic, assign) CGRect currentRect;
@property (nonatomic, assign) CGRect currentRealRect;
@property (nonatomic, assign) CGRect maxCropRect;
@property (nonatomic, assign) int currentEditPointIndex;
@property (nonatomic, strong) CropPDFView *pdfView;
@property (nonatomic, strong) UIColor* primaryColor;

- (instancetype)initWithCropPDFView:(CropPDFView *)pdfView;
- (void)resetDefaultCrop;
- (void)resetNoCrop;

@end

@interface CropViewController ()<UIToolbarDelegate,IDocEventListener> {
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;
}

@property (nonatomic, strong) CropPDFView *pdfView;
@property (nonatomic, strong) CropView *cropView;
@property (assign, nonatomic) BOOL isApplyToAllOddPages;
@property (assign, nonatomic) BOOL isApplyToAllEvenPages;
@property (nonatomic, assign) CGRect currentRect;
@property (strong, nonatomic) FSToolbar *bottomToolbar;
@property (strong, nonatomic) FSItem *cancelItem;
@property (strong, nonatomic) FSItem *doneItem;
@property (strong, nonatomic) FSItem *fullCropItem;
@property (strong, nonatomic) FSItem *settingItem;
@property (nonatomic, assign) BOOL bMultipleCropStop;
@end

@implementation CropViewController

- (instancetype)init
{
    self = [super init];
    if (self) {
        self.isApplyToAllOddPages = NO;
        self.isApplyToAllEvenPages = NO;
        self.bMultipleCropStop = NO;
    }
    return self;
}

- (FSToolbar *)bottomToolbar
{
    if (_bottomToolbar == nil) {
        @weakify(self)
        FSToolbar* bottomToolbar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleBottom];
        bottomToolbar.hasDivide = NO;
        _bottomToolbar = bottomToolbar;
        bottomToolbar.backgroundColor = self.fs_backgroundColor_B6;
        [self.view addSubview:bottomToolbar];
        [bottomToolbar mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(bottomToolbar.superview.fs_mas_left);
            make.bottom.equalTo(bottomToolbar.superview.fs_mas_bottom);
            make.right.equalTo(bottomToolbar.superview.fs_mas_right);
        }];
        
        _cancelItem = [FSItem createItemWithImage:FSImageNamed(@"crop_mode_cancel")];
        _cancelItem.accessibilityLabel = FSLocalizedForKey(@"kClose");
        [bottomToolbar addItem:_cancelItem atPosition:FSItemPositionLeft];
        _cancelItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            [self cancelItemClick];
        };
        
        _doneItem = [FSItem createItemWithImage:FSImageNamed(@"crop_mode_done")];
        _doneItem.accessibilityLabel = FSLocalizedForKey(@"kDone");
        [bottomToolbar addItem:_doneItem atPosition:FSItemPositionRight];
        _doneItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            [self doneItemClick];
        };
        
        _fullCropItem = [FSItem createItemWithImage:FSImageNamed(@"crop_mode_fullscreen")];
        _fullCropItem.accessibilityLabel = FSLocalizedForKey(@"kCropDeafultAreaAccessibilityLabel");
        [bottomToolbar addItem:_fullCropItem spacing:10 atPosition:FSItemPositionCenter];
        _fullCropItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            [self fullCropItemClick];
        };
        
        _settingItem = [FSItem createItemWithImage:FSImageNamed(@"crop_mode_setting")];
        _settingItem.accessibilityLabel = FSLocalizedForKey(@"kSettings");
        [bottomToolbar addItem:_settingItem spacing:10 atPosition:FSItemPositionCenter];
        _settingItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            [self settingItemClick];
        };
        
    }
    return _bottomToolbar;
}

- (void)cancelItemClick
{
    [self close];
}

- (void)onDocWillClose:(FSPDFDoc *)document {
    self.bMultipleCropStop = YES;
}

- (void)doneItemClick
{
    BOOL isSettingAllPages = [_extensionsManager.cropManager isSettingWithAllPages];
    int currentPageIndex = self.cropView.pdfView.pageIndex;
    CGRect cropRect = self.cropView.currentRect;
    CGRect cropSymmetricallyRect = CGRectMake(self.cropView.originalSize.width - (self.cropView.currentRect.origin.x+self.cropView.currentRect.size.width), self.cropView.currentRect.origin.y, self.cropView.currentRect.size.width, self.cropView.currentRect.size.height);
    void (^setPageCropRect)(int pageIndex, CGRect rect) = ^(int pageIndex, CGRect rect ) {
        if (self.bMultipleCropStop) {
            return;
        }
        [self->_pdfViewCtrl setCropPageRect:pageIndex pdfRect:[self convertCGRect2PDFRect:rect pageIndex:pageIndex]];
    };
    BOOL bFlag = (currentPageIndex % 2 == 0) ? YES : NO;
    int pageCount = [_pdfViewCtrl getPageCount];
    
    if (pageCount > 10000) {
        
        MBProgressHUD *progressView = [MBProgressHUD showHUDAddedTo:self.view animated:YES];
//        progressView.labelText = [NSString stringWithFormat:@"%d/%d", 1, pageCount];
        progressView.detailsLabelText = FSLocalizedForKey(@"kCropSmartCalculating");
        dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
            
            for (int i = 0; i < 100; i++) {
                
                
//                progressView.labelText = [NSString stringWithFormat:@"%d/%d", i+1, pageCount];
                if (isSettingAllPages) {
                    setPageCropRect(i, cropRect);
                }else{
                    if (((i % 2 == 0) ? YES : NO) == bFlag) {
                        setPageCropRect(i, cropRect);
                    }else{
                        setPageCropRect(i, cropSymmetricallyRect);
                    }
                }
            }
            
            dispatch_async(dispatch_get_main_queue(), ^{
                [MBProgressHUD hideHUDForView:self.view animated:YES];
                
                [self->_extensionsManager.settingBar.viewSettingManager setItemSelectedWithType:CROPPAGE selected:YES];
                [self->_pdfViewCtrl setCropMode:PDF_CROP_MODE_CUSTOMIZED];
                [self close];
            });
        });
        
        
        
        int queueCount = (int)[NSProcessInfo processInfo].activeProcessorCount;
        queueCount = queueCount < 1 ? 1 : queueCount > 16 ? 16 : queueCount;
        
        int preChunkSize = pageCount/queueCount;
        
        int chunk = pageCount / preChunkSize;
        int remain = pageCount % preChunkSize;
        
        dispatch_group_t group =  dispatch_group_create();
        dispatch_queue_attr_t attr = dispatch_queue_attr_make_with_qos_class(DISPATCH_QUEUE_CONCURRENT, QOS_CLASS_BACKGROUND, 0);
        dispatch_queue_t pagesizeQueue = dispatch_queue_create("com.foxit.queueCreate.pageCropQueue", attr);
        
        for (int m = 0 ; m < chunk; m++) {
            dispatch_group_async(group, pagesizeQueue, ^{
                  // 并行执行的线程
                int start = m * preChunkSize;
                if (m == 0) {
                    start = 100;
                }
                int end = m * preChunkSize + preChunkSize;
                for (int i = start; i < end; i++) {
                    if (self.bMultipleCropStop) {
                        break;
                    }
                    
                    if (isSettingAllPages) {
                        setPageCropRect(i, cropRect);
                    }else{
                        if (((i % 2 == 0) ? YES : NO) == bFlag) {
                            setPageCropRect(i, cropRect);
                        }else{
                            setPageCropRect(i, cropSymmetricallyRect);
                        }
                    }
                }
            });
        }
        
        dispatch_group_async(group, pagesizeQueue, ^{
            
            for (int i = pageCount -1; i > pageCount - remain; i--) {
                if (self.bMultipleCropStop) {
                    break;
                }
                if (isSettingAllPages) {
                    setPageCropRect(i, cropRect);
                }else{
                    if (((i % 2 == 0) ? YES : NO) == bFlag) {
                        setPageCropRect(i, cropRect);
                    }else{
                        setPageCropRect(i, cropSymmetricallyRect);
                    }
                }
            }
        });
        
    }else{
        for (int i = 0; i < pageCount; i++) {
            if (isSettingAllPages) {
                setPageCropRect(i, cropRect);
            }else{
                if (((i % 2 == 0) ? YES : NO) == bFlag) {
                    setPageCropRect(i, cropRect);
                }else{
                    setPageCropRect(i, cropSymmetricallyRect);
                }
            }
        }

        [_extensionsManager.settingBar.viewSettingManager setItemSelectedWithType:CROPPAGE selected:YES];
        [_pdfViewCtrl setCropMode:PDF_CROP_MODE_CUSTOMIZED];
        [self close];
    }
}

- (void)fullCropItemClick
{
    [self.cropView resetNoCrop];
}

- (void)settingItemClick
{
    [_extensionsManager.cropManager showCropSettingAlertController];
}

- (void)setExtension:(UIExtensionsManager *)extensionsManager {
    _extensionsManager = extensionsManager;
    _pdfViewCtrl = extensionsManager.pdfViewCtrl;
    
    [_pdfViewCtrl registerDocEventListener:self];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    self.view.backgroundColor = self.fs_backgroundColor_B6;
    [self prefersStatusBarHidden];
    [self performSelector:@selector(setNeedsStatusBarAppearanceUpdate)];
}

- (BOOL)prefersStatusBarHidden
{
    return YES;
}

- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    self.pdfView.needPauseNow = YES;
}

- (CGRect)getPageContentCGRect:(FSPDFPage *)page isWholePage:(BOOL)isWholePage {
    CGRect rect = CGRectZero;

    float pageWidth = [page getWidth];
    float pageHeight = [page getHeight];
    
    FSRectF *rectBBox = nil;
    if (isWholePage) {
        return CGRectMake(0, 0, pageWidth, pageHeight);
    } else {
        if ([page isParsed] == NO) {
            BOOL parseSuccess = [Utility parsePage:page];
            if (!parseSuccess) {
                return rect;
            }
        }

        rectBBox = [page calcContentBBox:FSPDFPageCalcContentsBox];
        if ([rectBBox isEmpty]) {
            rectBBox = [Utility getPageRealRect:page];
        }
    }
    
    if (rectBBox.left != 0 || rectBBox.right != 0 || rectBBox.top != 0 || rectBBox.bottom != 0) {
        FSRectF *newRectBBox = [[FSRectF alloc] init];
        newRectBBox.left = rectBBox.left;
        newRectBBox.right = rectBBox.right;
        newRectBBox.top = rectBBox.top;
        newRectBBox.bottom = rectBBox.bottom;

        CGRect newRect = CGRectZero;
        FSRotation rotation = [page getRotation];

        FSRectF *realRect = [Utility getPageRealRect:page];
        
        switch (rotation) {
        case FSRotation0:
            newRect = CGRectMake(ABS(MIN(realRect.left, realRect.right) -  MIN(newRectBBox.left, newRectBBox.right)),
                                 MAX(realRect.top, realRect.bottom) - MAX(newRectBBox.top, newRectBBox.bottom),
                                 ABS(newRectBBox.right - newRectBBox.left),
                                 ABS(newRectBBox.top - newRectBBox.bottom));
            break;
        case FSRotation90:
            newRect = CGRectMake(ABS(realRect.bottom - newRectBBox.bottom),
                                 ABS(realRect.left - newRectBBox.left),
                                 ABS(newRectBBox.top - newRectBBox.bottom),
                                 ABS(newRectBBox.right - newRectBBox.left));
            break;
        case FSRotation180:
            newRect = CGRectMake(ABS(realRect.right - newRectBBox.right),
                                 ABS(realRect.bottom - newRectBBox.bottom),
                                 ABS(newRectBBox.right - newRectBBox.left),
                                 ABS(newRectBBox.top - newRectBBox.bottom));
            break;
        case FSRotation270:
            newRect = CGRectMake(
                ABS(realRect.top - newRectBBox.top),
                ABS(realRect.right - newRectBBox.right),
                ABS(newRectBBox.top - newRectBBox.bottom),
                ABS(newRectBBox.right - newRectBBox.left));
            break;
        default:
            break;
        }

        rect = newRect;
    }

    return rect;
}

- (FSRectF *)convertCGRect2PDFRect:(CGRect)cgrect {
    FSRectF *pdfrect = [[FSRectF alloc] init];
    [pdfrect setLeft:0];
    [pdfrect setBottom:0];
    [pdfrect setRight:0];
    [pdfrect setTop:0];
    CGRect temp = cgrect;
    FSPDFPage *page = [_pdfViewCtrl.currentDoc getPage:self.pdfView.pageIndex];
    FSRotation rotation = [page getRotation];
    
    FSRotation viewrotation = (FSRotation)[_pdfViewCtrl getViewRotation];
    int endrotate = (rotation + viewrotation) % 4;
    rotation = (FSRotation)endrotate;
    
    if (rotation == FSRotation90 || rotation == FSRotation270) {
        //to do
    }

    pdfrect.left = temp.origin.x;
    pdfrect.right = temp.origin.x + temp.size.width;
    pdfrect.top = [page getHeight] - temp.origin.y;
    pdfrect.bottom = pdfrect.top - temp.size.height;
    return pdfrect;
}


- (FSRectF *)convertCGRect2PDFRect:(CGRect)cgrect pageIndex:(int)pageIndex{
    
    /**
     * Same page size, different rotation angle, using PDF FSRectF
     */
    FSPDFPage *currentCropPage = [_pdfViewCtrl.currentDoc getPage:self.pdfView.pageIndex];
    if ([currentCropPage isEmpty]) return [[FSRectF alloc] init];
    float currentCropPageWidth = [currentCropPage getWidth];
    float currentCropPageHeight = [currentCropPage getHeight];

    CGSize currentCropPageSize = CGSizeMake(currentCropPageWidth, currentCropPageHeight);
    UIEdgeInsets currentCropInsets = [Utility convertCGRect2Insets:cgrect size:currentCropPageSize];
    FSRotation currentCropPageRotation = [currentCropPage getRotation];
    
    FSRotation viewrotation = (FSRotation)[_pdfViewCtrl getViewRotation];
    int endrotate = (currentCropPageRotation + viewrotation) % 4;
    currentCropPageRotation = (FSRotation)endrotate;
    
    if (currentCropPageRotation == FSRotation90) {
        currentCropInsets = UIEdgeInsetsMake(currentCropInsets.right, currentCropInsets.top, currentCropInsets.left, currentCropInsets.bottom);
    }else if (currentCropPageRotation == FSRotation180) {
        currentCropInsets = UIEdgeInsetsMake(currentCropInsets.bottom, currentCropInsets.right, currentCropInsets.top, currentCropInsets.left);
    }else if (currentCropPageRotation == FSRotation270) {
        currentCropInsets = UIEdgeInsetsMake(currentCropInsets.left, currentCropInsets.bottom, currentCropInsets.right, currentCropInsets.top);
    }
    
    FSPDFPage *page = [_pdfViewCtrl.currentDoc getPage:pageIndex];
    if ([page isEmpty]) return [[FSRectF alloc] init];
    float pageWidth = [page getWidth];
    float pageHeight = [page getHeight];
    
    CGSize pageSize = CGSizeMake(pageWidth, pageHeight);
    UIEdgeInsets insets = currentCropInsets;
    FSRotation rotation = [page getRotation];
    
    int tmpEndrotate = (rotation + viewrotation) % 4;
    rotation = (FSRotation)tmpEndrotate;

    if (rotation == FSRotation90) {
        insets = UIEdgeInsetsMake(insets.left, insets.bottom, insets.right, insets.top);
    }else if (rotation == FSRotation180) {
        insets = UIEdgeInsetsMake(insets.bottom, insets.right, insets.top, insets.left);
    }else if (rotation == FSRotation270) {
        insets = UIEdgeInsetsMake(insets.right, insets.top, insets.left, insets.bottom);
    }
    /**
     * When top, left, bottom, right, any item is larger than the page size, crop will not be applied to the page
     */
    
    if (insets.top >= pageHeight ||
        insets.bottom >= pageHeight ||
        insets.left >= pageWidth ||
        insets.right >= pageWidth) {
        insets.top = 0;
        insets.bottom = 0;
        insets.left = 0;
        insets.right = 0;
    }
    
    FSRectF *pdfrect = [[FSRectF alloc] init];
    pdfrect.top = insets.top;
    pdfrect.left = insets.left;
    pdfrect.bottom = pageSize.height - insets.bottom;
    pdfrect.right = pageSize.width - insets.right;
    return pdfrect;
}

- (void)calcCurrentPreviewedCropRects {
    FSPDFPage *page = nil;
    @try {
        page = [_pdfViewCtrl.currentDoc getPage:self.pdfView.pageIndex];
    } @catch (NSException *_) {
        self.cropView.currentRect = CGRectMake(0, 0, 0, 0);
        return;
    }
    if ([page isEmpty]) {
        self.cropView.currentRect = CGRectMake(0, 0, 0, 0);
        return;
    }
    float pageWidth = [page getWidth];
    float pageHeight = [page getHeight];
    float h1 = MIN(_pdfViewCtrl.fs_height, _pdfViewCtrl.fs_width)-100;
    
    float h2 = MAX(pageHeight,pageWidth);
    float scale = h1/h2;
   
    if ((_pdfViewCtrl.fs_height >  _pdfViewCtrl.fs_width) && [self checkIsIphoneDevice]) {
        scale =  _pdfViewCtrl.fs_width/pageWidth;
    }
    CGSize size = CGSizeMake( pageWidth * scale, pageHeight * scale);
    
    [self.pdfView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.centerX.mas_equalTo(self.pdfView.superview);
        make.centerY.mas_equalTo(self.pdfView.superview).offset(-30);
        make.size.mas_equalTo(size);
    }];
    
    [self.cropView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.right.top.bottom.mas_equalTo(self.pdfView);
    }];
    
    [self.cropView layoutIfNeeded];
    [self.pdfView layoutIfNeeded];
    
    self.cropView.originalSize = CGSizeMake(pageWidth, pageHeight);
    self.cropView.originalRect = [self getPageContentCGRect:[_pdfViewCtrl.currentDoc getPage:self.pdfView.pageIndex] isWholePage:NO];
    int margin = 15;
    self.cropView.originalRect = [Utility convertCGRectWithMargin:self.cropView.originalRect size:self.cropView.originalSize margin:margin];
    self.cropView.currentRect = ({
        CGSize pageSize = CGSizeMake(pageWidth, pageHeight);
        CGRect pageContentRect = [self getPageContentCGRect:page isWholePage:NO];
        int margin = 15;
        CGRect pageCropRect = [Utility convertCGRectWithMargin:pageContentRect size:pageSize margin:margin];
        pageCropRect;
        
    });
    self.cropView.originalRect = self.cropView.currentRect;
    self.cropView.maxCropRect = ({
        CGRect pageContentRect = [self getPageContentCGRect:page isWholePage:YES];
        pageContentRect;
    });
    [self.cropView setNeedsDisplay];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    //[[UIApplication sharedApplication] setStatusBarStyle:UIStatusBarStyleDefault animated:NO];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(orientationDidChange:) name:UIDeviceOrientationDidChangeNotification object:nil];
    
    [self creatPdfView];
}

- (void)viewDidAppear:(BOOL)animated{
    [super viewDidAppear:animated];
    if ( ![self checkIsIphoneDevice]){
        self.pdfView.needPauseNow = YES;
        [self.pdfView removeFromSuperview];
        self.pdfView = nil;
        [self creatPdfView];
    }
}

- (void)creatPdfView{
    if (!self.pdfView) {
        self.pdfView = [[CropPDFView alloc] init];
        self.pdfView.pdfViewCtrl = _pdfViewCtrl;
        self.pdfView.pageIndex = [_pdfViewCtrl getCurrentPage];

        self.cropView = [[CropView alloc] initWithCropPDFView:self.pdfView];
        self.cropView.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin | UIViewAutoresizingFlexibleTopMargin | UIViewAutoresizingFlexibleBottomMargin | UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
        self.cropView.opaque = NO;
        self.cropView.userInteractionEnabled = YES;

        [self.pdfView addSubview:self.cropView];
        [self.view addSubview:self.pdfView];
        
        [self.bottomToolbar setHidden:NO];
        [self.pdfView bringSubviewToFront:self.bottomToolbar];
        [self.view layoutIfNeeded];
        
        [self calcCurrentPreviewedCropRects];
       // [self setPreviousAndNextBtnEnable];
    }
}

- (void)viewWillTransitionToSize:(CGSize)size withTransitionCoordinator:(id<UIViewControllerTransitionCoordinator>)coordinator{
    [super viewWillTransitionToSize:size withTransitionCoordinator:coordinator];
    [coordinator animateAlongsideTransition:nil
                                 completion:^(id<UIViewControllerTransitionCoordinatorContext>  _Nonnull context) {
                                    self.pdfView.needPauseNow = YES;
                                    [self.pdfView removeFromSuperview];
                                    self.currentRect = self.cropView.currentRect;
                                    self.pdfView = nil;
                                    [self creatPdfView];
                                    self.cropView.currentRect = self.currentRect;
    }];
}


- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (UIStatusBarStyle)preferredStatusBarStyle {
    return UIStatusBarStyleDefault;
}

- (BOOL)shouldAutorotate {
    return YES;
}

- (UIInterfaceOrientationMask)supportedInterfaceOrientations {
    return UIInterfaceOrientationMaskAll;
}

- (void)orientationDidChange:(NSNotification *)aNotification {
    if ([self checkIsIphoneDevice]) {
        [self calcCurrentPreviewedCropRects];
    }
}

- (void)close {
    self.isApplyToAllOddPages = NO;
    self.isApplyToAllEvenPages = NO;
    if (self.cropViewClosedHandler) {
        self.cropViewClosedHandler();
    }
    [self dismissViewControllerAnimated:YES
                             completion:^{
                             }];
}

- (BOOL)checkIsIphoneDevice
{
    return DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact];
}

@end

@implementation CropPDFView

- (id)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code
        self.pageIndex = 0;
    }
    return self;
}

- (void)drawRect:(CGRect)rect {
    [super drawRect:rect];
}

// Draw into the layer
- (void)drawLayer:(CALayer *)layer inContext:(CGContextRef)context {
    CGRect clipRect = CGContextGetClipBoundingBox(context);
    
    if (clipRect.size.width == 0 || clipRect.size.height == 0) {
        return;
    }

    dispatch_async(dispatch_get_global_queue(0, 0), ^{
        FSPDFPage *page = [self.pdfViewCtrl.currentDoc getPage:self.pageIndex];
        [Utility drawPage:page targetSize:clipRect.size shouldDrawAnnotation:YES progressive:^(UIImage * _Nonnull image, BOOL finish) {
            dispatch_async(dispatch_get_main_queue(), ^{
                layer.contents = (__bridge id)image.CGImage;
            });
        } needPause:^BOOL{
            return  self.needPauseNow;
        } rotation:FSRotation0 enableForPrint:NO pdfViewCtrl:self.pdfViewCtrl];
    });
}
@end

@implementation CropView

- (instancetype)initWithCropPDFView:(CropPDFView *)pdfView;
{
    self = [super init];
    if (self) {
        // Initialization code
        UIPanGestureRecognizer *panGesture = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(handlePan:)];
        [self addGestureRecognizer:panGesture];

        self.pdfView = pdfView;
        _currentRealRect = CGRectZero;
        
        [BindThemeKeySel(self, setNeedsDisplay) fs_updateThemeCompleted:^(CropView*  _Nonnull target, SEL  _Nonnull sel) {
            target.primaryColor = target.fs_primaryColor;
            [target setNeedsDisplay];
        }];
    }
    return self;
}

- (void)setMaxCropRect:(CGRect)maxCropRect{
    float scale = self.bounds.size.width / self.originalSize.width;
    maxCropRect = CGRectMake(maxCropRect.origin.x * scale, maxCropRect.origin.y * scale, maxCropRect.size.width * scale, maxCropRect.size.height * scale);
    _maxCropRect =  maxCropRect;
}

- (void)resetDefaultCrop {
    self.currentRect = self.originalRect;
    [self setNeedsDisplay];
}

- (void)resetNoCrop {
    self.currentRect = self.originalRect; 
    [self setNeedsDisplay];
}

- (NSArray *)getMovePointInRect:(CGRect)rect {
    if (rect.size.width == 0 || rect.size.height == 0) {
        return nil;
    }
    float iconRadius = 7.5f;
    NSMutableArray *array = [NSMutableArray array];
    [array addObject:[NSValue valueWithCGRect:CGRectMake(rect.origin.x - iconRadius, rect.origin.y - iconRadius, iconRadius * 2, iconRadius * 2)]];
    [array addObject:[NSValue valueWithCGRect:CGRectMake(rect.origin.x + rect.size.width - iconRadius, rect.origin.y - iconRadius, iconRadius * 2, iconRadius * 2)]];
    [array addObject:[NSValue valueWithCGRect:CGRectMake(rect.origin.x - iconRadius, rect.origin.y + rect.size.height - iconRadius, iconRadius * 2, iconRadius * 2)]];
    [array addObject:[NSValue valueWithCGRect:CGRectMake(rect.origin.x + rect.size.width - iconRadius, rect.origin.y + rect.size.height - iconRadius, iconRadius * 2, iconRadius * 2)]];
    return array;
}

- (CGRect)getCurrentRealRect {
    float scale = self.bounds.size.width / self.originalSize.width;
    CGRect croppedRect = CGRectMake(self.currentRect.origin.x * scale, self.currentRect.origin.y * scale, self.currentRect.size.width * scale, self.currentRect.size.height * scale);
    return croppedRect;
}

- (CGRect)getCurrentRect:(CGRect)rect {
    float scale = self.originalSize.width / self.bounds.size.width;
    CGRect pageRect = CGRectMake(rect.origin.x * scale, rect.origin.y * scale, rect.size.width * scale, rect.size.height * scale);
    return pageRect;
}

- (void)handlePan:(UIPanGestureRecognizer *)recognizer {
    CGPoint point = [recognizer locationInView:self];
    if (recognizer.state == UIGestureRecognizerStateBegan) {
        _currentEditPointIndex = -1;
        _currentRealRect = [self getCurrentRealRect];
        NSArray *movePointArray = [self getMovePointInRect:_currentRealRect];
        [movePointArray enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
            CGRect dotRect = [obj CGRectValue];
            dotRect = CGRectInset(dotRect, -20, -20);
            if (CGRectContainsPoint(dotRect, point)) {
                self->_currentEditPointIndex = (int) idx;
                *stop = YES;
            }
        }];
        if (_currentEditPointIndex == -1) {
            if (CGRectContainsPoint(_currentRealRect, point)) {
                _currentEditPointIndex = 4;
            }
        }
    } else if (recognizer.state == UIGestureRecognizerStateChanged) {
        if (_currentEditPointIndex == -1) {
            return;
        }

        CGPoint translationPoint = [recognizer translationInView:self];
        [recognizer setTranslation:CGPointZero inView:self];
        float tw = translationPoint.x;
        float th = translationPoint.y;
        CGRect rect = _currentRealRect;

        if (_currentEditPointIndex == 0) {
            rect.origin.x += tw;
            rect.origin.y += th;
            rect.size.width -= tw;
            rect.size.height -= th;
        } else if (_currentEditPointIndex == 1) {
            rect.size.width += tw;
            rect.origin.y += th;
            rect.size.height -= th;
        } else if (_currentEditPointIndex == 2) {
            rect.origin.x += tw;
            rect.size.width -= tw;
            rect.size.height += th;
        } else if (_currentEditPointIndex == 3) {
            rect.size.width += tw;
            rect.size.height += th;
        } else if (_currentEditPointIndex == 4) {
            rect.origin.x += tw;
            rect.origin.y += th;
        }
        
        if (rect.size.height  == self.maxCropRect.size.height && rect.size.width == self.maxCropRect.size.width) {
            return;
        }
        if (rect.origin.x < 0) {
            rect.origin.x = 0;
        }
        if (rect.origin.y < 0) {
            rect.origin.y = 0;
        }
        if (rect.size.width > self.maxCropRect.size.width) {
            rect.size.width = self.maxCropRect.size.width;
        }
        if (rect.size.height > self.maxCropRect.size.height) {
            rect.size.height = self.maxCropRect.size.height;
        }
        
        if (rect.origin.x + rect.size.width > self.maxCropRect.size.width  && _currentEditPointIndex!= 0 && _currentEditPointIndex!= 2) {
            rect.size.width = rect.size.width;
            rect.origin.x = self.maxCropRect.size.width - rect.size.width;
        }
        if (rect.origin.y + rect.size.height > self.maxCropRect.size.height && _currentEditPointIndex!= 1 && _currentEditPointIndex!= 3) {
            rect.size.height = rect.size.height;
            rect.origin.y = self.maxCropRect.size.height - rect.size.height;
        }
        _currentRealRect = rect;
        [self setNeedsDisplay];
    } else if (recognizer.state == UIGestureRecognizerStateEnded || recognizer.state == UIGestureRecognizerStateCancelled) {
        _currentEditPointIndex = -1;
        self.currentRect = [self getCurrentRect:CGRectStandardize(_currentRealRect)];
        _currentRealRect = CGRectZero;
        [self setNeedsDisplay];
    }
}

// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
    // Drawing code
    CGContextRef context = UIGraphicsGetCurrentContext();
    CGContextSetRGBFillColor(context, 255, 255, 255, .2);
    CGContextFillRect(context, rect);

    CGRect croppedRect = _currentRealRect;
    if (CGRectEqualToRect(croppedRect, CGRectZero)) {
        croppedRect = [self getCurrentRealRect];
    }

    CGContextClearRect(context, croppedRect);
    CGContextSetLineWidth(context, 1.0);
    CGContextSetStrokeColorWithColor(context, [self.fs_iconColor_I4 CGColor]);
    CGContextStrokeRect(context, croppedRect);

    UIImage *dragDot = FSImageNamed(@"crop_mode_drag");
    NSArray *movePointArray = [self getMovePointInRect:croppedRect];
    [movePointArray enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        CGRect dotRect = [obj CGRectValue];
        [dragDot drawAtPoint:dotRect.origin];
    }];
}

@end
