/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
*/


#import "FSCropManager.h"
#import "FSCropSelectAlertController.h"
#import "FSSettingBar.h"
#import "CropViewController.h"
#import "FSSettingBar+private.h"

@interface FSCropManager()
@property(nonatomic, weak) UIExtensionsManager* extensionsManager;
@property(nonatomic, strong) NSMutableArray* cropSettingData;
@property(nonatomic, strong) NSMutableArray* cropOperationData;
@end

@implementation FSCropManager

- (instancetype)initWithExtensionsManager:(UIExtensionsManager*)extensionsManager
{
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
    }
    return self;
}

- (BOOL)isSettingWithAllPages
{
    for (NSMutableDictionary* dic in self.cropSettingData) {
        int tag = ((NSString*)[dic valueForKey:@"tag"]).intValue ;
        if (tag == FSCropSelectItemTypeSettingAllPage) {
            return [[dic valueForKey:@"selected"] isEqual:@"YES"] ? YES : NO;
        }
    }
    return NO;
}

- (void)showCropSettingAlertController
{
    FSCropSelectAlertController* view = [[FSCropSelectAlertController alloc] initWithFrameSize:CGSizeMake(272, 101) isShowSelected:YES];
    if (_cropSettingData == nil) {
        _cropSettingData = [NSMutableArray array];
        NSMutableDictionary* dic = [[NSMutableDictionary alloc] init];
        [dic setValue:FSLocalizedForKey(@"kCropAllPages") forKey:@"title"];
        [dic setValue:@"YES" forKey:@"selected"];
        [dic setValue:@"YES" forKey:@"enalbed"];
        [dic setValue:[NSString stringWithFormat:@"%d",FSCropSelectItemTypeSettingAllPage] forKey:@"tag"];
        [_cropSettingData addObject:dic];
         
        NSMutableDictionary* dic1 = [[NSMutableDictionary alloc] init];
        [dic1 setValue:FSLocalizedForKey(@"kCropPagesSymmetrically") forKey:@"title"];
        [dic1 setValue:@"NO" forKey:@"selected"];
        [dic1 setValue:@"YES" forKey:@"enalbed"];
        [dic1 setValue:[NSString stringWithFormat:@"%d",FSCropSelectItemTypeSettingSymmetrically] forKey:@"tag"];
        [_cropSettingData addObject:dic1];
    }
   
    view.dataSource = [_cropSettingData mutableCopy];
    [[self.extensionsManager.pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:view animated:YES completion:nil];
    @weakify(self)
    [FSCropSelectAlertController setDidSelectItemCallBack:^(int tag) {
        @strongify(self)
        self.cropSettingData = [view.dataSource mutableCopy];
        [[self.extensionsManager.pdfViewCtrl fs_currentSceneTopMostViewController] dismissViewControllerAnimated:YES completion:^{
            [self didClickedWithTag:tag];
        }];
    }];
}

- (void)didClickedWithTag:(int)tag
{
    switch (tag) {
        case FSCropSelectItemTypeSettingAllPage:
            
            break;
        case FSCropSelectItemTypeSettingSymmetrically:
            
            break;
        case FSCropSelectItemTypeAuto:
        {
            [_extensionsManager.settingBar.viewSettingManager setItemSelectedWithType:CROPPAGE selected:YES];
            MBProgressHUD *progressView = [MBProgressHUD showHUDAddedTo:self.extensionsManager.pdfViewCtrl.getDisplayView animated:YES];
            progressView.labelText = FSLocalizedForKey(@"kCropSmartCalculating");
            dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
                dispatch_async(dispatch_get_main_queue(), ^{
                    [MBProgressHUD hideHUDForView:self.extensionsManager.pdfViewCtrl.getDisplayView animated:YES];
                    [self.extensionsManager.pdfViewCtrl setCropMode:PDF_CROP_MODE_CONTENTSBOX];
                });
            });
        }
            break;
        case FSCropSelectItemTypeManually:
            [self showCropViewController];
            break;
        case FSCropSelectItemTypeRemoveAll:
        {
            [_extensionsManager.settingBar.viewSettingManager setItemSelectedWithType:CROPPAGE selected:NO];
            [self.extensionsManager.pdfViewCtrl setCropMode:PDF_CROP_MODE_NONE];
        }
            break;
        default:
            break;
    }
}

- (void)showCropOperationAlertController
{
    if (_cropSettingData == nil) {
        _cropSettingData = [NSMutableArray array];
        NSMutableDictionary* dic = [[NSMutableDictionary alloc] init];
        [dic setValue:FSLocalizedForKey(@"kCropAllPages") forKey:@"title"];
        [dic setValue:@"YES" forKey:@"selected"];
        [dic setValue:@"YES" forKey:@"enalbed"];
        [dic setValue:[NSString stringWithFormat:@"%d",FSCropSelectItemTypeSettingAllPage] forKey:@"tag"];
        [_cropSettingData addObject:dic];
         
        NSMutableDictionary* dic1 = [[NSMutableDictionary alloc] init];
        [dic1 setValue:FSLocalizedForKey(@"kCropPagesSymmetrically") forKey:@"title"];
        [dic1 setValue:@"NO" forKey:@"selected"];
        [dic1 setValue:@"YES" forKey:@"enalbed"];
        [dic1 setValue:[NSString stringWithFormat:@"%d",FSCropSelectItemTypeSettingSymmetrically] forKey:@"tag"];
        [_cropSettingData addObject:dic1];
    }
    
    BOOL isNOCropMode = ([self.extensionsManager.pdfViewCtrl getCropMode] == PDF_CROP_MODE_NONE);
    FSCropSelectAlertController* view = [[FSCropSelectAlertController alloc] initWithFrameSize:CGSizeMake(272, 151) isShowSelected:NO];
    view.extensionsManager = self.extensionsManager;
    if (_cropOperationData == nil) {
        _cropOperationData = [NSMutableArray array];

        NSMutableDictionary* dic = [[NSMutableDictionary alloc] init];
        [dic setValue:FSLocalizedForKey(@"kAutomaticallyCropPages") forKey:@"title"];
        [dic setValue:@"YES" forKey:@"enalbed"];
        [dic setValue:[NSString stringWithFormat:@"%d",FSCropSelectItemTypeAuto] forKey:@"tag"];
        [_cropOperationData addObject:dic];
         
        NSMutableDictionary* dic1 = [[NSMutableDictionary alloc] init];
        [dic1 setValue:FSLocalizedForKey(@"kManuallyCropPages") forKey:@"title"];
        [dic1 setValue:@"YES" forKey:@"enalbed"];
        [dic1 setValue:[NSString stringWithFormat:@"%d",FSCropSelectItemTypeManually] forKey:@"tag"];
        [_cropOperationData addObject:dic1];
        
        NSMutableDictionary* dic2 = [[NSMutableDictionary alloc] init];
        [dic2 setValue:FSLocalizedForKey(@"kRemoveAllPageCropping") forKey:@"title"];
        [dic2 setValue:(isNOCropMode ? @"NO" : @"YES") forKey:@"enalbed"];
        [dic2 setValue:[NSString stringWithFormat:@"%d",FSCropSelectItemTypeRemoveAll] forKey:@"tag"];
        [_cropOperationData addObject:dic2];
    }
    else
    {
        NSMutableDictionary* dic = _cropOperationData[2];
        [dic setValue:(isNOCropMode ? @"NO" : @"YES") forKey:@"enalbed"];
        [_cropOperationData replaceObjectAtIndex:2 withObject:dic];
    }
    
   
    view.dataSource = [_cropOperationData mutableCopy];
    [[self.extensionsManager.pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:view animated:YES completion:nil];
    @weakify(self)
    [FSCropSelectAlertController setDidSelectItemCallBack:^(int tag) {
        @strongify(self)
        self.cropOperationData = [view.dataSource mutableCopy];
        
        [[self.extensionsManager.pdfViewCtrl fs_currentSceneTopMostViewController] dismissViewControllerAnimated:YES completion:^{
            [self didClickedWithTag:tag];
        }];
    }];
}

- (void)showCropMode
{
    [self showCropOperationAlertController];
}

- (void)showCropViewController
{
    CropViewController *cropViewController = [[CropViewController alloc] init];
    [cropViewController setExtension:self.extensionsManager];
    cropViewController.cropViewClosedHandler = ^() {
        [self.extensionsManager setFullScreen:NO];
    };
    FSNavigationController *navController = [[FSNavigationController alloc] initWithRootViewController:cropViewController];
    navController.navigationBarHidden = YES;
    [navController fs_setCustomTransitionAnimator:self.extensionsManager.presentedVCAnimator];
    
    [self.extensionsManager.pdfViewCtrl.fs_viewController presentViewController:navController animated:YES completion:nil];
}
@end
