/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
*/


#import "FSCropSelectAlertController.h"

@interface FSCropSelectAlertCell()
@property(nonatomic, strong) FSItem* titleItem;
@property(nonatomic, strong) FSItem* selecteImgItem;
@end

@implementation FSCropSelectAlertCell

- (instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier
{
    self = [super initWithStyle:style reuseIdentifier:reuseIdentifier];
    if (self) {
        [self initSubView];
    }
    return self;
}

- (void)initSubView
{
    self.selectedBackgroundView = [UIView new];
    self.selectedBackgroundView.backgroundColor = self.fs_dividerColor_P1;
    
    FSItem* titleItem = [FSItem createItemWithTitle:@""];
    _titleItem = titleItem;
    titleItem.userInteractionEnabled = NO;
    [titleItem.titleLabel setFont:self.fs_standardGeneralFont_GF2];

    [titleItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
   [titleItem setTitleColor:self.fs_iconColor_I2 forState:UIControlStateDisabled];
    titleItem.titleLabel.textAlignment = NSTextAlignmentLeft;
    [self.contentView addSubview:titleItem];
    
    FSItem* selecteImgItem = [FSItem createItemWithImage:FSImageNamed(@"common_select_check")];
    _selecteImgItem = selecteImgItem;
    selecteImgItem.userInteractionEnabled = NO;
    [self.contentView addSubview:selecteImgItem];
    [selecteImgItem mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(-16);
        make.centerY.equalTo(selecteImgItem.superview);
        make.height.width.mas_equalTo(40);
    }];
    
    [titleItem mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(titleItem.superview.mas_left).offset(16);
        make.right.equalTo(titleItem.superview.mas_right).offset(-16);
        make.centerY.equalTo(titleItem.superview);
    }];
    
    [BindThemeKeySel(selecteImgItem, setImage:forState:) fs_updateThemeCompleted:^(FSItem*  _Nonnull target, SEL  _Nonnull sel) {
        [target setImage:[FSImageNamed(@"common_select_check") fs_imageWithTintColor:target.fs_primaryColor] forState:UIControlStateNormal];
    }];
    [selecteImgItem setHidden:YES];
}

- (void)setCellInfo:(NSString*)itemTitleName selected:(BOOL)selected enabled:(BOOL)enabled
{
    [_titleItem setTitle:itemTitleName forState:UIControlStateNormal];
    [_selecteImgItem setHidden:!selected];
    [_titleItem setEnabled:enabled];
    if (selected) {
        [_titleItem mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(_titleItem.superview.mas_left).offset(16);
            make.right.equalTo(_selecteImgItem.mas_left).offset(-16);
            make.centerY.equalTo(_titleItem.superview);
        }];
    }else {
        [_titleItem mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(_titleItem.superview.mas_left).offset(16);
            make.right.equalTo(_titleItem.superview.mas_right).offset(-16);
            make.centerY.equalTo(_titleItem.superview);
        }];
    }
}

@end




@interface FSCropSelectAlertController ()
@property(nonatomic, strong) UITableView* tableView;
@property(nonatomic, assign) CGSize frameSize;
@property(nonatomic, assign) BOOL isShowSelected;
@end

@implementation FSCropSelectAlertController

- (void)viewDidLoad
{
    [super viewDidLoad];
    [self initTableView];
}

- (instancetype)initWithFrameSize:(CGSize)frameSize isShowSelected:(BOOL)isShowSelected
{
    self = [super init];
    if (self) {
        self.isShowSelected = isShowSelected;
        self.frameSize = frameSize;
        self.dataSource = [[NSMutableArray alloc] init];
        [self initCustomTransitionAnimator];
    }
    return self;
}

- (void)initCustomTransitionAnimator
{
    FSPresentedViewControllerAnimator *animator = [[FSPresentedViewControllerAnimator alloc] init];
    animator.size = self.frameSize;
    animator.presentationStyle = FSModalPresentationCenter;
    animator.maxVerticalHeigh = self.frameSize.height;
    [self fs_setCustomTransitionAnimator:animator];
}

- (void)initTableView
{
    _tableView = [[UITableView alloc] init];
    _tableView.layer.cornerRadius = 10;
    _tableView.delegate = self;
    _tableView.dataSource = self;
    _tableView.backgroundColor = self.fs_backgroundColor_B2;
    _tableView.separatorStyle = UITableViewCellSeparatorStyleNone;
    _tableView.allowsMultipleSelection = NO;
    _tableView.allowsSelection = YES;
    [self.view addSubview:_tableView];
    [_tableView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.top.bottom.mas_equalTo(0);
    }];
}

static DidSelectItemCallBack _didSelectItemCallBack;

+ (void)setDidSelectItemCallBack:(DidSelectItemCallBack)didSelectItemCallBack
{
    _didSelectItemCallBack = didSelectItemCallBack;
}

+ (DidSelectItemCallBack)didSelectItemCallBack
{
    return _didSelectItemCallBack;
}

#pragma mark -- UITableViewDelegate

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return 50.f;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return self.dataSource.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    static NSString *cellidentifer = @"cropSelectTableCell";
    FSCropSelectAlertCell *cell = (FSCropSelectAlertCell*)[tableView dequeueReusableCellWithIdentifier:cellidentifer];
    if (cell == nil) {
        cell = [[FSCropSelectAlertCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellidentifer];
    }
    
    NSDictionary* dic = self.dataSource[indexPath.row];
    NSString* title = [dic valueForKey:@"title"];
    BOOL selected = [[dic valueForKey:@"selected"] isEqual:@"YES"] ? YES : NO;
    BOOL enabled = [[dic valueForKey:@"enalbed"] isEqual:@"YES"] ? YES : NO;
    if (!self.isShowSelected) {
        selected = NO;
    }
    
    [cell setCellInfo:title selected:selected enabled:enabled];

    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    for (int i = 0; i < self.dataSource.count; i++) {
        NSMutableDictionary* dic = self.dataSource[i];
        if (i == indexPath.row) {
            [dic setValue:@"YES" forKey:@"selected"];
        }else{
            [dic setValue:@"NO" forKey:@"selected"];
        }
        [self.dataSource replaceObjectAtIndex:i withObject:dic];
    }
    
    NSMutableDictionary* dicCurrent = self.dataSource[indexPath.row];
    int tag = ((NSString*)[dicCurrent valueForKey:@"tag"]).intValue ;

    [self.tableView reloadData];
    if (_didSelectItemCallBack) {
        _didSelectItemCallBack(tag);
    }
}

- (void)traitCollectionDidChange:(UITraitCollection *)previousTraitCollection{
    [super traitCollectionDidChange:previousTraitCollection];
    if ([self fs_currentSceneiPadHorizontalCompact]) {
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.4 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
            if (self.extensionsManager && !self.extensionsManager.hiddenPanel) {
                [self.extensionsManager setHiddenPanel:YES];
            }
        });
    }
}

@end
