/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "CerInfoViewController.h"
#import "FSBaseViews.h"

@interface InfoTableViewCell : FSTableViewCell
@property (nonatomic, weak) IBOutlet UILabel *titleLab;
@property (nonatomic, weak) IBOutlet UILabel *infoLab;
@end

@implementation InfoTableViewCell


@end

static NSString *const InfoTableViewCellID = @"InfoTableViewCell";

@interface CerInfoViewController ()
@end

@implementation CerInfoViewController
@dynamic tableView;

- (void)initialize{
    [super initialize];
    self.sectionStyle = FSTableViewSectionStyleLikeGrouped;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    self.tableView.rowHeight = UITableViewAutomaticDimension;
    self.tableView.estimatedRowHeight = 100.f;
    self.tableView.backgroundColor = self.fs_backgroundColor_B9;
    
    self.title = FSLocalizedForKey(@"kCertInfo");
    if (self.showStyle == FSViewControllerShowStylePresent) {
        self.navBarLeftTitle = FSLocalizedForKey(@"kClose");
    }
}

- (void)setCerInfoDic:(NSDictionary *)cerInfoDic{
    NSString *title = @"titile";
    NSString *info = @"info";
    
    NSString * (^keyForValue) (NSString *) = ^(NSString *key){
        NSString *value = cerInfoDic[key];
        return value ? value : @"";
    };
    NSArray *tmpKeyArr = @[ FSLocalizedForKey(@"kCertSerialNum"), FSLocalizedForKey(@"kTrustedCertName"), FSLocalizedForKey(@"kCertIssuer"), FSLocalizedForKey(@"kUsage") , FSLocalizedForKey(@"kTrustedCertValidFrom"), FSLocalizedForKey(@"kTrustedCertValidTo"), FSLocalizedForKey(@"kCertEmail")];
    NSArray *tmpValueArr = @[ keyForValue(@"serialNumber"), keyForValue(@"name"), keyForValue(@"issuerInfo"), keyForValue(@"origUsage"), keyForValue(@"startDateStr") , keyForValue(@"endDateStr"), keyForValue(@"email"),];
    NSMutableArray *tmpMuArr = [NSMutableArray array];
    for (int i = 0; i < tmpKeyArr.count; i ++ ) {
        NSString *tmpTitle = tmpKeyArr[i];
        tmpTitle = [tmpTitle stringByReplacingOccurrencesOfString:@":" withString:@""];
        tmpTitle = [tmpTitle stringByReplacingOccurrencesOfString:@"：" withString:@""];
        
        NSString *tmpValue = tmpValueArr[i];
        NSDictionary *dic = @{ title : tmpTitle, info : tmpValue };
        if ([tmpValue isEqualToString:keyForValue(@"origUsage")]) {
            NSArray *locs = [keyForValue(@"origUsage") componentsSeparatedByString:@","];
            NSMutableString *locStrs = [NSMutableString string];
            NSString *locValue = nil;
            if (locs.count > 0) {
                locValue = FSLocalizedForKey(locs[0]);
                locStrs = [NSMutableString stringWithString:locValue];
            }
            
            for (int i = 1; i < locs.count; i++) {
                locValue = FSLocalizedForKey(locs[i]);
                [locStrs appendString:[NSString stringWithFormat:@", %@",locValue]];
            }
            dic = @{ title : tmpTitle, info : locStrs };
        }
        if ([tmpValue isEqualToString:keyForValue(@"startDateStr")] || [tmpValue isEqualToString:keyForValue(@"endDateStr")]) {
            
//            NSString *dataFormat = @"yyyy-MM-dd HH:mm:ss";
//            NSDate *date = [NSDate fs_dateWithString:tmpValue format:dataFormat];
            
            dic = @{ title : tmpTitle, info : tmpValue };
        }
        
        [tmpMuArr addObject:dic];
    }
    self.dataSource = tmpMuArr.copy;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section{
    return self.dataSource.count;
}

- (NSString *)titles1ForAtSection:(NSUInteger)section{
    return FSLocalizedForKey(@"kHeaderCertInfo");
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath{
    InfoTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:InfoTableViewCellID];
    cell.contentView.backgroundColor = self.fs_backgroundColor_B8;
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    NSDictionary *dic = self.dataSource[indexPath.row];
    NSString *title = @"titile";
    NSString *info = @"info";
    cell.titleLab.text = dic[title];
    cell.titleLab.textColor = self.fs_textColor_T4;
    cell.titleLab.font = self.fs_standardGeneralFont_GF2;
    cell.infoLab.text = [dic[info] stringByReplacingOccurrencesOfString:@"\n" withString:@""];
    cell.infoLab.textColor = self.fs_textColor_T4;
    cell.infoLab.font = self.fs_standardGeneralFont_GF2;
    return cell;
}

 - (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section
{
    UIView *headerView = [super tableView:tableView viewForHeaderInSection:section];
    headerView.backgroundColor = self.fs_backgroundColor_B9;
    return headerView;
}
@end
