/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "CerTrustedListViewCtr.h"
#import "DigitalCertSelectCtr.h"
#import "CerInfoViewController.h"
#import "FSBaseViews.h"
#import "NSDate+FSAdd.h"


static NSString *const CerTrustedInfos = @"CerTrustedInfos";

@interface CerTrustedListViewCtr ()<IRotationEventListener>
@property (nonatomic, weak) IBOutlet UITableView *tableView;
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
@end

@implementation CerTrustedListViewCtr
@dynamic tableView;

- (void)initialize{
    [super initialize];
    self.shouldShowNavigationBar = YES;
    self.defaultEstimatedCellStyle = FSEstimatedTableViewCellStyleSubject;

}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    self.extensionsManager = ((FSViewControllerManager *)self.services).extensionsManager;

    self.tableView.backgroundColor = self.fs_backgroundColor_B2;
    self.dataSource = [NSUserDefaults fs_getTrustedCerInfos];
    self.navBarRightTitle = FSLocalizedForKey(@"kAdd");
    self.navBarLeftTitle = FSLocalizedForKey(@"kClose");
    self.title = FSLocalizedForKey(@"kTrustedCert");
    @weakify(self)
    self.navBar.rightItem.onTapClick = ^(FSItem * _Nonnull item) {
        @strongify(self)
        [self touchedTrustCerItem:item];
    };
    
    self.cellSelectedColor = self.fs_dividerColor_P1;
}


- (void)touchedTrustCerItem:(FSItem *)sender {
    DigitalCertSelectCtr *digitalCertSelectCtr = [[DigitalCertSelectCtr alloc] initWithServices:self.extensionsManager.viewCtrlManager];
    digitalCertSelectCtr.selectOperator = FSCertSelectOperatorAddTrustCer;
    digitalCertSelectCtr.title = FSLocalizedForKey(@"kSelectAddCert");
    [self.navigationController pushViewController:digitalCertSelectCtr animated:YES];
    
    digitalCertSelectCtr.doneOperator = ^(NSString *path, NSString *passwd, NSString *md5) {
        self.dataSource = [NSUserDefaults fs_getTrustedCerInfos];
        [self reloadData];
    };
}

-(CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath{
    return 68;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section{
    return self.dataSource.count;
}

- (void)configureCell:(FSSubjectEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(NSDictionary *)cerInfo{
    if (!cell.leftView) {
        UIImageView *imageView = [UIImageView new];
        cell.leftView = imageView;
    }
    
    if (!cell.rightView) {
        FSItem *rightView = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"common_more_gray")];
        rightView.accessibilityLabel = FSLocalizedForKey(@"kMore");
        cell.rightView = rightView;
    }

    @weakify(self)
    ((FSItem *)cell.rightView).onTapClick = ^(FSItem * _Nonnull item) {
        @strongify(self)
        [(FSViewControllerManager *)self.services showMoreMenuWithTitles:@[FSLocalizedForKey(@"kInfo"), FSLocalizedForKey(@"kDelete")] images:@[FSImageNamedAndNormalColor(@"sign_information_gray"), FSImageNamedAndNormalColor(@"common_delete_gray")] action:^(FSPopoverItem *item, int index) {
            if (index == 0) {
                UIViewController *cerInfoVC = [self.extensionsManager.viewCtrlManager createViewCtrlType:FSViewCtrlTypeCerInfo params:cerInfo];
                [self.navigationController pushViewController:cerInfoVC animated:YES];
            }else{
                UIAlertController *alertView = [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kDeleteTrustedCer") actions:@[ FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kOK") ] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                    if (index == 1) {
                        [NSUserDefaults fs_removeTrustedCerInfo:cerInfo];
                        self.dataSource = [NSUserDefaults fs_getTrustedCerInfos];
                        [self reloadData];
                    }
                }];
                [self presentViewController:alertView animated:YES completion:nil];
            }
        }sourceView:!DEVICE_iPHONE ? ^UIView * _Nonnull(UIPopoverPresentationController * _Nonnull popver) {
            popver.permittedArrowDirections = UIPopoverArrowDirectionRight;
            return item;
        } : nil];
    };
    
    NSString *dateString = cerInfo[@"endDateStr"];
    if (dateString && dateString.length > 0) {
        NSLocale *currentLocale = [NSLocale currentLocale];
        NSString *languageCode = [currentLocale objectForKey:NSLocaleLanguageCode];
        NSString  *dataFormat = @"";
        if ([languageCode isEqualToString:@"zh"] || [languageCode isEqualToString:@"ja"]){
            dataFormat =  @"yyyy年MM月dd日 HH:mm:ss";
        }else{
            
            dataFormat =  @"MMM dd,yyyy · HH:mm:ss";
        }
        NSDate *date = [NSDate fs_dateWithString:dateString format:dataFormat];
        dateString = [Utility displayDateInMDYHMS:date];
    }else{
        dateString = @"";
    }

    ((UIImageView *)cell.leftView).image = FSImageNamedAndNormalColor(@"file_list_other");
    cell.estimatedTitleLabel.text = cerInfo[@"subject"];
    cell.estimatedSubjectLabel.text = dateString;
}

- (NSString *)titleForEmptyDataSet{
    return FSLocalizedForKey(@"kNoCertificates");
}

- (NSAttributedString *)titleForEmptyDataSet:(UITableView *)tableView {
    return [[NSAttributedString alloc] initWithString:[self titleForEmptyDataSet] attributes:@{
        NSFontAttributeName:self.fs_standardGeneralFont_GF3,
        NSForegroundColorAttributeName:self.fs_textColor_T2
        }];
}

- (UIImage *)imageForEmptyDataSet:(UITableView *)tableView {
    return FSImageNamedAndPrimaryColor(@"more_cert_no_certificate");
}

- (CGFloat)verticalOffsetForEmptyDataSet:(UITableView *)tableView {
    return 0;
}

- (BOOL)emptyDataSetShouldDisplay:(UITableView *)tableView {
    @try {
        return ![self dataForAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:0]];;
    } @catch (NSException *exception) {
        return YES;
    }
    return YES;
}

@end


