/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "DigitalCertSelectCtr.h"
#import "CerInfoViewController.h"

@interface DigitalCertSelectCtr ()<IRotationEventListener>
@property (nonatomic, strong) NSMutableArray *dataArray;
@property (nonatomic, assign) NSInteger currentSelectIndexPathRow;
@property (nonatomic, assign) NSInteger showPromptTimes;
@property (nonatomic, strong) NSMutableDictionary *currentCertPasswordDic;
@property (nonatomic, copy) void (^finishInputPassword)(NSString *);

@property (nonatomic, strong) UIViewController *originalPresentedViewController;
@property (nonatomic, strong) CERT_INFO *cerInfo;

@property (nonatomic, readonly) FSViewControllerManager *viewControllerManager;
@property (nonatomic, strong) FSItem* titleItem;
@end

@implementation DigitalCertSelectCtr

- (void)initialize{
    [super initialize];
    self.defaultEstimatedCellStyle = FSEstimatedTableViewCellStyleTitle;
    self.editingImageNormal = FSImageNamedAndNormalColor(@"common_unselect");
    self.editingImageSelected = FSImageNamed(@"common_select");
}

- (FSViewControllerManager *)viewControllerManager{
    return self.services;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    self.dataArray = [NSMutableArray array];
    self.currentCertPasswordDic = [[NSMutableDictionary alloc] init];
    self.currentSelectIndexPathRow = -1;
    [self loadCertInfo];
        
    self.originalPresentedViewController = nil;
    
    self.navBarLeftTitle = FSLocalizedForKey(@"kCancel");
    self.navBarRightTitle = FSLocalizedForKey(@"kOK");
    [self.navBar.leftItem addTarget:self action:@selector(didCancel) forControlEvents:UIControlEventTouchUpInside];
    [self.navBar.rightItem addTarget:self action:@selector(didOk) forControlEvents:UIControlEventTouchUpInside];
    self.navBar.rightItem.enabled = NO;
    self.tableView.editing = YES;
}

- (void)loadCertInfo {
    NSString *path = DOCUMENT_PATH;
    NSFileManager *myFileManager = [NSFileManager defaultManager];

    NSDirectoryEnumerator *myDirectoryEnumerator;

    myDirectoryEnumerator = [myFileManager enumeratorAtPath:path];

    while ((path = [myDirectoryEnumerator nextObject]) != nil) {
        if ([[[path pathExtension] lowercaseString] isEqualToString:@"p12"] || [[[path pathExtension] lowercaseString] isEqualToString:@"pfx"]) {
            [self.dataArray addObject:path.copy];
        }
    }
    
    [self.tableView reloadData];
    [self reloadEmptyDataSet];
}

- (void)infoBtnPress:(NSIndexPath *)indexPath{
    [self tableView:self.tableView accessoryButtonTappedForRowWithIndexPath:indexPath];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.dataArray.count;
}

- (void)configureCell:(FSTitleEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(id)dataItem{
    cell.editingHiddenRightView = NO;
    if (!cell.rightView) {
        FSItem *infoItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"more_cert_information")];
        infoItem.accessibilityLabel = FSLocalizedForKey(@"kCertInfo");
        cell.rightView = infoItem;
    }
    cell.selectedBackgroundColor = self.fs_dividerColor_P1;
    cell.contentView.backgroundColor = nil;
    
    cell.estimatedTitleLabel.text = [(NSString *) [self.dataArray objectAtIndex:indexPath.row] lastPathComponent];
    cell.estimatedTitleLabel.accessibilityIdentifier = @"cert_name_id";

    @weakify(self)
    ((FSItem *)cell.rightView).onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [self infoBtnPress:indexPath];
    };
    
    if (self.currentSelectIndexPathRow == indexPath.row) {
        self.navBar.rightItem.enabled = YES;
        [self selectRow:YES atIndexPath:indexPath];
        cell.editItem.accessibilityLabel = FSLocalizedForKey(@"kSelected");
    }else{
        [self selectRow:NO atIndexPath:indexPath];
        cell.editItem.accessibilityLabel = FSLocalizedForKey(@"kDeselectAccessibilityLabel");
    }

}

- (void)tableView:(UITableView *)tableView didDeselectRowAtIndexPath:(NSIndexPath *)indexPath{
    [super tableView:tableView didDeselectRowAtIndexPath:indexPath];
    FSTitleEstimatedTableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
    cell.editItem.accessibilityLabel = FSLocalizedForKey(@"kDeselectAccessibilityLabel");
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    NSMutableDictionary *certs = [NSUserDefaults fs_objectForKey:@"CERT_INFO_PASSWORDS"];
    certs = certs ? certs.mutableCopy : @{}.mutableCopy;
    CERT_INFO *info = [[CERT_INFO alloc] init];
    self.cerInfo = info;
    NSString *path = [DOCUMENT_PATH stringByAppendingPathComponent:self.dataArray[indexPath.row]];
    NSString *md5 = [NSString fs_getFileMD5WithPath:path];
    if ([self.currentCertPasswordDic objectForKey:md5]) {
        NSString *passwordStr = [self.currentCertPasswordDic objectForKey:md5];
        [FSPDFCertUtil getCertInfo:path password:passwordStr certInfo:info];
        self.currentSelectIndexPathRow = indexPath.row;
        [self.tableView reloadData];
    } else {
        __weak typeof(self) weakSelf = self;
        void(^promptForPassword)(NSString *password) = ^(NSString *password){
            if (password == nil) {
                weakSelf.showPromptTimes = 0;
                return;
            }
            [certs removeObjectForKey:path];
            PDF_CERT_ERROR_CODE result =[FSPDFCertUtil getCertInfo:path password:password certInfo:info];
            if (result == PDF_CERT_ERROR_FILE) {
                return;
            } else if (result == PDF_CERT_ERROR_PASSWORD) {
                [weakSelf tableView:tableView didSelectRowAtIndexPath:indexPath];
                return;
            }else{
                [certs setValue:password forKey:path];
            }
            NSLocale *currentLocale = [NSLocale currentLocale];
            NSString *languageCode = [currentLocale objectForKey:NSLocaleLanguageCode];
            NSDateFormatter* formatter = [[NSDateFormatter alloc]init];
            if ([languageCode isEqualToString:@"zh"] || [languageCode isEqualToString:@"ja"]){
                formatter.dateFormat = @"yyyy年MM月dd日 HH:mm:ss";
            }else{
                formatter.dateFormat = @"MMM dd,yyyy · HH:mm:ss";
            }
            NSDate *endDate = [formatter dateFromString:info.certEndDate];

            weakSelf.showPromptTimes = 0;
            if (endDate.timeIntervalSince1970 - [NSDate date].timeIntervalSince1970 < 0) {
                UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kError") message:FSLocalizedForKey(@"kCertExpired") preferredStyle:UIAlertControllerStyleAlert];
                UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                                 style:UIAlertActionStyleCancel
                                                               handler:^(UIAlertAction *_Nonnull action) {
                                                                   [self popoverDidCancel];
                                                               }];
                [alertController addAction:action];
                [self presentAlertController:alertController];
            } else {
                [weakSelf.currentCertPasswordDic setObject:password forKey:md5];
                weakSelf.currentSelectIndexPathRow = indexPath.row;
            }
            [weakSelf.tableView reloadData];
        };
        NSString *psw = [certs valueForKey:path];
        if (psw) {
            promptForPassword(psw);
            [NSUserDefaults fs_setObject:certs forKey:@"CERT_INFO_PASSWORDS"];
        }else{
            [self promptForPassword:^(NSString *password) {
                promptForPassword(password);
                [NSUserDefaults fs_setObject:certs forKey:@"CERT_INFO_PASSWORDS"];
            }];
        }
    }
}

- (void)tableView:(UITableView *)tableView accessoryButtonTappedForRowWithIndexPath:(nonnull NSIndexPath *)indexPath {
    CERT_INFO *info = [[CERT_INFO alloc] init];
    NSString *path = [DOCUMENT_PATH stringByAppendingPathComponent:self.dataArray[indexPath.row]];
    NSString *md5 = [NSString fs_getFileMD5WithPath:path];
    if ([self.currentCertPasswordDic objectForKey:md5]) {
        PDF_CERT_ERROR_CODE result =[FSPDFCertUtil getCertInfo:path password:[self.currentCertPasswordDic objectForKey:md5] certInfo:info];
        if (result == PDF_CERT_ERROR_FILE) {
            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kCertFileError") preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                             style:UIAlertActionStyleCancel
                                                           handler:^(UIAlertAction *_Nonnull action) {
                                                               [self popoverDidCancel];
                                                           }];
            [alertController addAction:action];
            [self presentAlertController:alertController];
            return;
        } else if (result == PDF_CERT_ERROR_PASSWORD) {
            [self.currentCertPasswordDic removeObjectForKey:md5];
            [self tableView:tableView accessoryButtonTappedForRowWithIndexPath:indexPath];
            return;
        }
        UIStoryboard *storyboard = [UIStoryboard storyboardWithName:@"CerOperationView" bundle:[NSBundle bundleForClass:[self class]]];
        CerInfoViewController *VC = [storyboard instantiateViewControllerWithIdentifier:@"CerInfoViewController"];
        [VC setCerInfoDic:info.cerInfoDic];
        [self.navigationController pushViewController:VC animated:YES];
    } else {
        [self promptForPassword:^(NSString *password) {
            if (password == nil) {
                self.showPromptTimes = 0;
                return;
            }

            PDF_CERT_ERROR_CODE result = [FSPDFCertUtil getCertInfo:path password:password certInfo:info];
            if (result == PDF_CERT_ERROR_FILE) {
                UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kCertFileError") preferredStyle:UIAlertControllerStyleAlert];
                UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                                 style:UIAlertActionStyleCancel
                                                               handler:^(UIAlertAction *_Nonnull action) {
                                                                   [self popoverDidCancel];
                                                               }];
                [alertController addAction:action];
                [self presentAlertController:alertController];
                return;
            } else if (result == PDF_CERT_ERROR_PASSWORD) {
                [self tableView:tableView accessoryButtonTappedForRowWithIndexPath:indexPath];
                return;
            }

            self.showPromptTimes = 0;
            NSLocale *currentLocale = [NSLocale currentLocale];
            NSString *languageCode = [currentLocale objectForKey:NSLocaleLanguageCode];
            NSDateFormatter* formatter = [[NSDateFormatter alloc]init];
            if ([languageCode isEqualToString:@"zh"] || [languageCode isEqualToString:@"ja"]){
                formatter.dateFormat = @"yyyy年MM月dd日 HH:mm:ss";
            }else{
                formatter.dateFormat = @"MMM dd,yyyy · HH:mm:ss";
            }
            NSDate *endDate = [formatter dateFromString:info.certEndDate];

            if (endDate.timeIntervalSince1970 - [NSDate date].timeIntervalSince1970 < 0) {
                UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kError") message:FSLocalizedForKey(@"kCertExpired") preferredStyle:UIAlertControllerStyleAlert];
                UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                                 style:UIAlertActionStyleCancel
                                                               handler:^(UIAlertAction *_Nonnull action) {
                                                                   [self popoverDidCancel];
                                                               }];
                [alertController addAction:action];
                [self presentAlertController:alertController];
            } else {
                [self.currentCertPasswordDic setObject:password forKey:md5];
                self.currentSelectIndexPathRow = indexPath.row;
            }
            CerInfoViewController *cerInfo = [self.viewControllerManager createViewCtrlType:FSViewCtrlTypeCerInfo params:info.cerInfoDic];
            [self.navigationController pushViewController:cerInfo animated:YES];
        }];
    }
}

- (void)didOk {
    void (^doneOperator)() = ^{
        if (self.currentSelectIndexPathRow != -1 && self.currentSelectIndexPathRow < self.dataArray.count) {
            NSString *path = [DOCUMENT_PATH stringByAppendingPathComponent:self.dataArray[self.currentSelectIndexPathRow]];
            NSString *md5 = [NSString fs_getFileMD5WithPath:path];
            if ([self.currentCertPasswordDic objectForKey:md5]) {
                if (self.doneOperator) {
                    self.doneOperator(path, [self.currentCertPasswordDic objectForKey:md5], md5);
                }
            }
        }
    };
    if (self.selectOperator == FSCertSelectOperatorSignature) {
        [self dismissViewControllerAnimated:YES
                                 completion:^{
                                     doneOperator();
                                 }];
    }else{
        [NSUserDefaults fs_setTrustedCerInfo:self.cerInfo.cerInfoDic];
        [self.navigationController popViewControllerAnimated:YES];
        doneOperator();
    }
}

- (void)didCancel {
    void (^cancelOperator)() = ^{
        if (self.cancelOperator) {
            self.cancelOperator(nil, nil, nil);
        }
    };
    if (self.selectOperator == FSCertSelectOperatorSignature) {
        [self dismissViewControllerAnimated:YES
                                 completion:^{
                                     cancelOperator();
                                 }];
    }else{
        [self.navigationController popViewControllerAnimated:YES];
        cancelOperator();
    }
    
}

- (void)promptForPassword:(void (^)(NSString *))complete {
    self.finishInputPassword = complete;
    NSString *msg;
    if (self.showPromptTimes == 0) {
        msg = FSLocalizedForKey(@"kOfflineCopyAlterTitle");
    } else {
        msg = FSLocalizedForKey(@"kPassWordErrorAlterTitle");
    }

    UIAlertController *alert = [UIAlertController fs_enablesActionAutomaticallyAlertWithTitle:nil message:msg textFieldText:@"" actions:@[FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action, UITextField * _Nonnull textField) {
        if (index == 1) {
            NSString *password = textField.text;;
            self.finishInputPassword(password);
        } else {
            self.finishInputPassword(nil);
        }
    }];
    [[alert textFields] firstObject].accessibilityIdentifier = @"passwd_input_id";
    [[alert textFields] firstObject].secureTextEntry = YES;
    [alert fs_showByViewController:self];

    self.showPromptTimes++;
}

#pragma mark pop over

- (void)presentAlertController:(UIAlertController *)alertController {
    self.originalPresentedViewController = self.presentedViewController;
    if (self.originalPresentedViewController) {
        [self.originalPresentedViewController dismissViewControllerAnimated:true
                                                                 completion:^{
                                                                     [[self fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
                                                                 }];
    } else {
        [[self fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
    }
}

- (void)popoverDidCancel {
    if (self.originalPresentedViewController) {
        [[self fs_currentSceneTopMostViewController] presentViewController:self.originalPresentedViewController animated:true completion:nil];
        self.originalPresentedViewController = nil;
    }
}

#pragma mark Empty Data

- (NSString *)titleForEmptyDataSet{
    return FSLocalizedForKey(@"kNoCertAndImportTip");
}

- (NSAttributedString *)titleForEmptyDataSet:(UITableView *)tableView {
    return [[NSAttributedString alloc] initWithString:[self titleForEmptyDataSet] attributes:@{
        NSFontAttributeName:self.fs_standardGeneralFont_GF3,
        NSForegroundColorAttributeName:self.fs_textColor_T2
        }];
}

- (UIImage *)imageForEmptyDataSet:(UITableView *)tableView {
    return FSImageNamedAndPrimaryColor(@"more_cert_no_certificate");
}

- (CGFloat)verticalOffsetForEmptyDataSet:(UITableView *)tableView {
    return 0;
}

- (BOOL)emptyDataSetShouldDisplay:(UITableView *)tableView {
    return self.dataArray.count <= 0;
}

@end
