/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "DistanceModule.h"
#import <FoxitRDK/FSPDFViewControl.h>

#import "LineAnnotHandler.h"
#import "DistanceToolHandler.h"
#import "Utility.h"
#import "LineToolHandler.h"

@interface DistanceModule ()

@property (nonatomic, weak) FSMainToolbarItem *propertyItem;
@property (nonatomic, weak) FSReadToolbarItem *readToolbarItem;
@property (nonatomic, strong) FSReadToolbarItem *measureItem;
@end

@implementation DistanceModule {
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;
    FSAnnotType _annotType;
    BOOL _isArrLine;
}

- (NSString *)getName {
    return @"Distance";
}

#pragma mark init
- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;

        [self loadModule];
        
        LineAnnotHandler* annotHandler = [[LineAnnotHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerAnnotHandler:annotHandler];

        DistanceToolHandler* toolHandler = [[DistanceToolHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerToolHandler:toolHandler];
    }
    return self;
}

- (void)loadModule {
    _measureItem = [[FSReadToolbarItem alloc] initWithToolSettings:[_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeMeasure]];
    [_extensionsManager.toolbarManager addReadItem:_measureItem atToolTag:FSMainTopbarToolTagComment];
    [_extensionsManager registerAnnotPropertyListener:self];
}

- (void)didClickedReadToolItem:(FSReadToolbarItem *)readToolbarItem {
    [_extensionsManager changeState:FSUIManagerStateAnnotTool];
    DistanceToolHandler *toolHandler = (DistanceToolHandler*)[_extensionsManager getToolHandlerByName:Tool_Distance];
    toolHandler.type = FSAnnotLine;
    toolHandler.toolSettings = readToolbarItem.toolSettings;
    [_extensionsManager setCurrentToolHandler:toolHandler];
    
    self.readToolbarItem = readToolbarItem;
    FSMainToolbarItem *propertyItem =  [_extensionsManager.toolbarManager getMainItemWithType:FSMainToolbarItemTypeProperty];
    propertyItem.item.onTapClick = ^(FSItem * _Nonnull _item) {
        CGRect rect = [_item convertRect:_item.bounds toView:self->_extensionsManager.pdfViewCtrl];
        if (DEVICE_iPHONE) {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:rect inView:self->_extensionsManager.pdfViewCtrl];
        } else {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:_item.bounds inView:_item];
        }
    };
    self.propertyItem = propertyItem;
    [_extensionsManager onUIInteractElementClicked:FSUIElementReadingCommentBarDistance];
}

- (void)dismissAnnotationContinue {
    [Utility dismissAnnotationContinue:_extensionsManager.pdfViewCtrl];
}

- (void)onToolSettingsChanged:(FSReadToolSettings *)toolSettings {
    if (self.readToolbarItem && self.readToolbarItem.toolSettings == toolSettings) {
        self.readToolbarItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }else if (_measureItem.toolSettings == toolSettings){
        _measureItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }
}

@end
