/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSFileSelectDestinationViewController.h"
#import "FileBrowser.h"
#import "FileItem.h"
#import <AVFoundation/AVFoundation.h>
#import "MBProgressHUD.h"
#import "FSMacro.h"

#define kThumbnailSize CGSizeMake(32.f, 40.f);


@interface FSFileSelectDestinationViewController () {
    UILabel *lblInstructionMsg;
    dispatch_queue_t _drawThumbnailQueue;
}
@property (nonatomic, strong) NSMutableArray *selectedArray;
@property (nonatomic, strong) NSString *currentDirectoryPath;

@property (nonatomic, assign) int MultiSelectLimit;
@property (nonatomic, strong) NSMutableArray *multiSelectedFilesArr;
@property (nonatomic, strong) NSMutableDictionary *multiSelectedIndexPathsDict;
@property (nonatomic, strong) NSMutableArray *multiSelectedIndexPathsArr;

@property (nonatomic, strong) NSIndexPath *lastPath;

@property (nonatomic, strong) NSString* currentSelectedFolderName;

- (void)loadFilesWithPath:(NSString *)filePath;
@end

@implementation FSFileSelectDestinationViewController

#pragma mark - Initialization

- (void)initialize{
    [super initialize];
    self.defaultEstimatedCellStyle = FSEstimatedTableViewCellStyleSubject;
    self.editingImageNormal = FSImageNamedAndNormalColor(@"common_unselect");
    self.editingImageSelected = FSImageNamed(@"common_select");
    self.isClosedWhenDone = YES;
    self.fileItemsArray = [NSMutableArray array];
    self.selectedArray = [NSMutableArray array];
    self.multiSelectedFilesArr = [NSMutableArray array];
    self.multiSelectedIndexPathsDict = [NSMutableDictionary dictionary];
    self.multiSelectedIndexPathsArr = [NSMutableArray array];
    dispatch_queue_attr_t attr = dispatch_queue_attr_make_with_qos_class(DISPATCH_QUEUE_SERIAL, QOS_CLASS_USER_INITIATED, 0);
    _drawThumbnailQueue = dispatch_queue_create("com.foxit.rdk.file.select.thumbnail", attr);
    self.fileSelectSpecialActionType = FSFileSelectSpecialActionTypeNone;
}

- (UIExtensionsManager *)extensionsManager{
    if ([self.services isKindOfClass:[FSViewControllerManager class]]) {
        return ((FSViewControllerManager *)self.services).extensionsManager;
    }
    return nil;
}

#pragma mark - View lifeCycle
- (void)viewDidLoad {
    [super viewDidLoad];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(applicationHandleOpenURL:) name:FSApplicationHandleOpenURLNotification object:nil];
    
   
    if (self.fileSelectDestinationType == FSFileSelectDestinationTypeSelectFile) {
        self.tableView.editing = YES;
    }

    [self initNavigationBar];
}

- (void)makeConstraints{
    [super makeConstraints];
    
    if (self.view != self.tableView && self.editBar.superview != self.view) {
        [self.tableView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(self.tableView.superview.fs_mas_left);
            make.right.equalTo(self.tableView.superview.fs_mas_right);
            make.bottom.equalTo(self.shouldShowBottomBar ? self.bottomBar.mas_top : self.tableView.superview);
            make.top.equalTo(self.shouldShowNavigationBar ? self.navBar.mas_bottom : self.tableView.superview);
        }];
    }
}

- (void)viewWillAppear:(BOOL)animated {
    if (self.fileItemsArray.count == 0)
        [lblInstructionMsg setHidden:NO];
    else if (!self.isRootFileDirectory && self.fileItemsArray.count == 1)
        [lblInstructionMsg setHidden:NO];
    else
        [lblInstructionMsg setHidden:YES];
    
    [self.multiSelectedIndexPathsDict enumerateKeysAndObjectsUsingBlock:^(NSString * _Nonnull key, NSMutableArray *indexPathArr, BOOL * _Nonnull stop) {
        NSString *currentDirectory = [self getDirectoryPath:self.currentDirectoryPath];
        if ([currentDirectory isEqualToString: key]) {
            [indexPathArr enumerateObjectsUsingBlock:^(NSIndexPath *indexPath, NSUInteger idx, BOOL * _Nonnull stop) {
                [self.tableView selectRowAtIndexPath:indexPath animated:YES scrollPosition:UITableViewScrollPositionNone];
            }];
        }
    }];
}

- (NSString *)currentTitle{

    NSString * title = FSLocalizedForKey(@"kDocuments");
    switch (self.fileSelectDestinationType) {
        case FSFileSelectDestinationTypeSelectFile:
            title = FSLocalizedForKey(@"kSelectFile");
            break;
        case FSFileSelectDestinationTypeSelectFolder:
            if (self.titleForTypeSelectFolder && self.titleForTypeSelectFolder.length > 0) {
                title = self.titleForTypeSelectFolder;
            }
            else{
                title = FSLocalizedForKey(@"kSelectFolder");
            }
            
            break;
    }
    return title;
}

#pragma mark - Configure NavigationBar
- (void)initNavigationBar {
    
    self.title = [self currentTitle];
    
    self.navBarLeftTitle = FSLocalizedForKey(@"kCancel");
    
    FSItem *doneItem = self.navBar.rightItem;
    doneItem.accessibilityIdentifier = @"Done_item_id";
    switch (self.fileSelectDestinationType) {
        case FSFileSelectDestinationTypeSelectFile:
            doneItem.enabled = NO;
            break;
        case FSFileSelectDestinationTypeSelectFolder:
            doneItem.enabled = YES;
            break;
    }
    [doneItem setTitle:FSLocalizedForKey(@"kDone") forState:UIControlStateNormal];
    doneItem.accessibilityLabel = FSLocalizedForKey(@"kDone");
    self.doneItem = doneItem;
    
    [doneItem setTarget:self action:@selector(doneAction) forControlEvents:UIControlEventTouchUpInside];
}

- (void)navigationBarBackTouch{
    
    if (self.cancelHandler) {
        self.cancelHandler(self);
    }
    
    [self.services dismissViewControllerAnimated:YES completion:self.dismissCompletion];
}

#pragma mark - Action methods
-(void)showTips:(NSString *)message {
    MBProgressHUD *progressHUD = [MBProgressHUD showHUDAddedTo:self.view animated:YES];
    progressHUD.mode = MBProgressHUDModeText;
    progressHUD.detailsLabelText =  message;
    dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, 1.5 * NSEC_PER_SEC);
    dispatch_after(popTime, dispatch_get_main_queue(), ^(void){
        [MBProgressHUD hideHUDForView:self.view animated:YES];
    });
}
- (void)doneAction {
    
    if (self.fileSelectDestinationType == FSFileSelectDestinationTypeSelectFolder) {
        self.operatingHandler(self, [NSArray arrayWithObject:self.currentDirectoryPath]);
        return;
    }
    
    if (self.fileSelectDestinationType == FSFileSelectDestinationTypeSelectFile) {
        NSMutableArray *tmp = @[].mutableCopy;
        for (int i =0 ; i < self.selectedDataSource.count; i++) {
            FileItem *fileItem = [self.selectedDataSource objectAtIndex:i];
            NSString *filePath = fileItem.filekey;
            [tmp addObject:filePath];
        }
        
        self.operatingHandler(self, tmp);
        if (self.isClosedWhenDone) {
            [self dismissViewControllerAnimated:YES completion:self.dismissCompletion];
        }
    }
}


#pragma mark - Other Methods
- (NSInteger)_getFilesCountWithPath:(NSString *)filePath {
    __block NSMutableArray *fileListWithExcept = [[NSMutableArray alloc] init];
    [FileBrowser getFileListWithFolderPath:filePath
                         withSearchKeyword:nil
                                  userData:nil
                               withHandler:^(NSMutableArray *fileList) {
        fileListWithExcept = [fileList mutableCopy];
        }];
    NSInteger fileCount = 0;
    for (FileItem *fileObj in fileListWithExcept) {
        if (fileObj.isFolder) {
            fileCount ++;
        }else{
            NSString *file = fileObj.path;
            NSString *fileDisplayName = [file lastPathComponent];
            BOOL willAdd = NO;
            NSString *extName = [fileDisplayName pathExtension];
            for (NSString *ext in _expectFileType) {
                if ([Utility isGivenExtension:extName type:ext]) {
                    willAdd = YES;
                    break;
                }
            }
            if (willAdd) {
                fileCount ++;
            }
        }
    }
    return fileCount;
}


//Avi - new method addded to add the cloudname in copy page to fix the issue 0071868
- (void)_loadFilesWithPath:(NSString *)filePath {
    __block NSMutableArray *folderList = [[NSMutableArray alloc] init];
    [FileBrowser getFileListWithFolderPath:filePath
                         withSearchKeyword:nil
                                  userData:nil
                               withHandler:^(NSMutableArray *fileList) {
                                   NSArray *favFileListArr = [fileList sortedArrayUsingComparator:^NSComparisonResult(id obj1, id obj2) {
                                       FileItem *fileObj1 = (FileItem *) obj1;
                                       FileItem *fileObj2 = (FileItem *) obj2;
                                       if (fileObj1.isFolder != fileObj2.isFolder) {
                                           return fileObj1.isFolder ? NSOrderedAscending : NSOrderedDescending;
                                       } else {
                                           NSComparisonResult compareResult = NSOrderedSame;
                                           return compareResult;
                                       }
                                   }];

                                   [folderList addObjectsFromArray:[NSArray arrayWithArray:favFileListArr]];
                               }];

    
    for (FileItem *fileitem in folderList) {
        if (fileitem.isFolder) {
            BOOL willAdd = YES;
            for (NSString *expFolder in self.exceptFolderList) {
                if ([[expFolder lastPathComponent] isEqualToString:[fileitem.path lastPathComponent]]) {
                    willAdd = NO;
                    break;
                }
            }
            if (willAdd) {
                
                fileitem.filekey = fileitem.path;
                fileitem.title = fileitem.fileName;
                fileitem.fileSize = [self _getFilesCountWithPath:fileitem.path];
                [self.fileItemsArray addObject:fileitem];
            }
        }else{
            if (self.fileSelectDestinationType == FSFileSelectDestinationTypeSelectFolder || self.fileSelectDestinationType == FSFileSelectDestinationTypeSelectFile) {
                NSString *file = fileitem.path;
                NSString *fileDisplayName = [file lastPathComponent];
                BOOL willAdd = NO;
                NSString *extName = [fileDisplayName pathExtension];
                for (NSString *ext in _expectFileType) {
                    if ([Utility isGivenExtension:extName type:ext]) {
                        willAdd = YES;
                        break;
                    }
                }
                if (willAdd) {
                    fileitem.filekey = file;
                    fileitem.title = fileDisplayName;
                    [self.fileItemsArray addObject:fileitem];
                }
            }
        }
    }
}

- (NSString *)titleForFileSelected{
    return self.selectedDataSource.count == 0 ? [self currentTitle] : [NSString stringWithFormat:FSLocalizedForKey(@"kFormatSelected"),(int)self.selectedDataSource.count];
}

- (void)setSelectedDataSource:(NSArray *)selectedDataSource{
    [super setSelectedDataSource:selectedDataSource];
    self.title = [self titleForFileSelected];
    self.doneItem.enabled = selectedDataSource.count;
}

- (void)loadFilesWithPath:(NSString *)filePath {
    //Avi - added
    [self _loadFilesWithPath:filePath];
    self.currentDirectoryPath = filePath;
}

- (void)setFileMultiSelectLimit:(int)limit{
    if (self.fileSelectDestinationType == FSFileSelectDestinationTypeSelectFile) {
        self.MultiSelectLimit = limit;
    }else{
        self.MultiSelectLimit = -1;
    }
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIApplicationDidChangeStatusBarOrientationNotification object:nil];
}

#pragma mark - UITableViewDelegate Methods


- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [self.fileItemsArray count];
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section{
    if (!self.isRootFileDirectory) return 24*2.5;
    return 0;
}

- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section{
    UIView *view = [UIView new];
    view.backgroundColor = self.fs_backgroundColor_B1;
    
    if (!self.isRootFileDirectory) {
        FSItem *backItem = [FSItem createItemWithTitle:_currentSelectedFolderName image:FSImageNamedAndNormalColor(@"panel_outline_list_back_gray") imageTextRelation:FSImageTextRelationRight];
        backItem.accessibilityLabel = [NSString stringWithFormat:@"%@ %@",FSLocalizedForKey(@"kBack"), _currentSelectedFolderName];
        backItem.font = self.fs_standardGeneralFont_GF1;
        [backItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
        [backItem setTitleEdgeInsets:UIEdgeInsetsMake(0, 20, 0, 0)];
        @weakify(self)
        backItem.onTapClick = ^(FSItem * _Nonnull item) {
            @strongify(self)
            [self.services popViewControllerAnimated:YES];
        };
        [view addSubview:backItem];
        
        [backItem mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.fs_equalToSuperview().offset(TITLE_LEFT_ICON_MARGINT);
            make.top.fs_equalToSuperview().offset(6.f);
        }];
    }
    return view;
}

- (NSString *)titleForEmptyDataSet{
    return FSLocalizedForKey(@"kNoFolderFiles");
}

- (UIImage *)imageForEmptyDataSet:(UITableView *)tableView {
    return FSImageNamedAndPrimaryColor(@"more_file_no_file");
}

- (id)dataForAtIndexPath:(NSIndexPath *)indexPath{

    id data = nil;
    if (indexPath.row < self.fileItemsArray.count) {
        data = [self.fileItemsArray objectAtIndex:indexPath.row];
    }
    return data;
}

- (NSString *)dequeueReusableIdentifierWithIndexPath:(NSIndexPath *)indexPath{
    NSString *identifier = NSStringFromClass([FSSubjectEstimatedTableViewCell class]);
    return identifier;
}


- (void)configureCell:(FSSubjectEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(FileItem *)fileItem{
    cell.editingHiddenRightView = NO;
    cell.hightlightBackgroundColor = self.fs_dividerColor_P1;
    cell.hightlightBackgroundColorFillHor = YES;
    if (!cell.installed) {
        cell.leftMargin = ITEM_MARGIN_NORMAL;
        cell.mediumSpace = ITEM_MARGIN_NORMAL;
        cell.rightMargin = ITEM_RIGHT_MARGIN_NORMAL;
        cell.editTopOffset = ITEM_IMAGE_TOP_MARGIN_NORMAL;
    }
    if (!cell.leftView) {
        UIImageView *imageView = [UIImageView new];
        imageView.fs_size = kThumbnailSize;
        imageView.contentMode = UIViewContentModeScaleAspectFit;
        cell.leftView = imageView;
    }

    cell.estimatedTitleLabel.text = DEVICE_iPHONE ? fileItem.title :  [fileItem.title stringByDeletingPathExtension];
    cell.estimatedTitleLabel.numberOfLines = 2;
    
    if (!cell.rightView) {
        FSItem *numLabel = [FSItem new];
        [numLabel fs_setLayerCornerRadius:LAYER_BORDER_RADIUS_3];
        [numLabel setTitleColor:self.fs_textColor_T3 forState:UIControlStateNormal | UIControlStateHighlighted | UIControlStateSelected];
        numLabel.font = self.fs_standardWeakFont_WF1;
        [numLabel setTitleEdgeInsets:UIEdgeInsetsMake(3, 10, 3, 10)];
        cell.rightView = numLabel;
        if (cell.installed) [cell reinstallConstraints];
    }
    
    NSString *subjectStr = [NSString stringWithFormat:@"%@%@ · %@",DEVICE_iPHONE ? @"" : [NSString stringWithFormat:@"%@ · ",[[fileItem.title pathExtension] uppercaseString]], [Utility displayDateInMDY:fileItem.modifiedDate],[Utility displayFileSize:fileItem.fileSize]];
    if (fileItem.isFolder) {
        if (cell.rightView) {
            FSItem *numLabel = (FSItem *)cell.rightView;
            numLabel.backgroundColor = self.fs_dividerColor_P2;
            [numLabel setTitle:[NSString stringWithFormat:@"%llu",fileItem.fileSize] forState:UIControlStateNormal];
            ((UIImageView *)cell.leftView).image = FSImageNamed(@"file_list_folder");
            subjectStr = [NSString stringWithFormat:@"%@",[Utility displayDateInMDY:fileItem.modifiedDate]];
        }
    } else {
        if (cell.rightView)
        {
            FSItem *numLabel = (FSItem *)cell.rightView;
            [numLabel setTitle:@"" forState:UIControlStateNormal];
            ((UIImageView *)cell.leftView).image = Nil;
            numLabel.backgroundColor = UIColor.clearColor;
            [cell reinstallConstraints];
        }
        UIImage *image = FSImageNamed([Utility getIconName:fileItem.filekey]);
        ((UIImageView *)cell.leftView).image = image;
        [cell.leftView fs_setLayerBorderWidth:0 borderColor:nil];
        if ([Utility isPDFPath:fileItem.filekey]) {
            [cell.leftView fs_setLayerBorderWidth:1.f borderColor:self.fs_dividerColor_P1];
            fileItem.path = fileItem.filekey;
            
            FileItem *_fileItem = fileItem;
            @weakify(fileItem)
            [fileItem getThumbnailForPageIndex:0 dispatchQueue:_drawThumbnailQueue WithHandler:^(UIImage *imageThumbnail, int pageIndex, NSString *pdfPath) {
                @strongify(fileItem)
                if (imageThumbnail && !_fileItem.isFolder) {
                    if ([[fileItem.filekey lastPathComponent] isEqualToString: fileItem.title]) {
                        dispatch_async(dispatch_get_main_queue(), ^{
                            ((UIImageView *)cell.leftView).image =  imageThumbnail;
                        });
                    }
                }
            }];
        }
    }
    cell.estimatedSubjectLabel.numberOfLines = 1;
    cell.estimatedSubjectLabel.text = subjectStr;
    [self selectRow:fileItem.selected atIndexPath:indexPath];
    
    cell.isAccessibilityElement = YES;
    if (fileItem.isFolder) {
        cell.accessibilityLabel = [NSString stringWithFormat:@"%@ %@ %@",cell.estimatedTitleLabel.text,cell.estimatedSubjectLabel.text,FSLocalizedForKey(@"kFolderVoiceover")];
    }else {
        cell.accessibilityLabel = [NSString stringWithFormat:@"%@ %@",cell.estimatedTitleLabel.text,cell.estimatedSubjectLabel.text];
    }
    
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    FileItem *fileItem = [self dataForAtIndexPath:indexPath];
    if (self.editing && !fileItem.isFolder) {

        if (self.fileSelectSpecialActionType == FSFileSelectSpecialActionTypeSingle) {

            for (NSIndexPath *selectedIndexPath  in self.selectedIndexPaths) {
                FileItem *selectedFileItem = [self dataForAtIndexPath:selectedIndexPath];
                selectedFileItem.selected = NO;
                [self selectRow:NO atIndexPath:selectedIndexPath];
            }
            self.selectedDataSource = @[].copy;
            self.selectedIndexPaths = @[].copy;
        }

        if (self.fileSelectSpecialActionType == FSFileSelectSpecialActionTypeMultiple) {

//            if (self.extensionsManager.pdfViewCtrl) {
//                FSPDFNameTree *nameTree = [[FSPDFNameTree alloc] initWithDocument:[self.extensionsManager.pdfViewCtrl currentDoc] type:FSPDFNameTreeEmbeddedFiles];
//                if ([nameTree hasName:fileItem.title]) {
//                    [UIAlertController fs_alert_OK_WithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kFailedAddAttachmentForExistedName") onCtrl:self];
//                    [tableView deselectRowAtIndexPath:indexPath animated:NO];
//                    return;
//                }
//            }
            
            if (fileItem.fileSize > 50 * 1024 * 1024) {
                [UIAlertController fs_alert_OK_WithTitle:FSLocalizedForKey(@"kWarning") message:[NSString stringWithFormat:FSLocalizedForKey(@"kAttachmentMaxSize"), 50] onCtrl:self];
                [tableView deselectRowAtIndexPath:indexPath animated:NO];
                return;
            }
            
            if (self.MultiSelectLimit > 0 && self.selectedDataSource.count >= self.MultiSelectLimit) {
                [tableView deselectRowAtIndexPath:indexPath animated:NO];
                return;
            }
        }
        [super tableView:tableView didSelectRowAtIndexPath:indexPath];
        fileItem.selected = YES;
        return;
    }else if (fileItem.isFolder){
        [tableView deselectRowAtIndexPath:indexPath animated:YES];
        
        FSFileSelectDestinationViewController *selectDestination = [[FSFileSelectDestinationViewController alloc] initWithServices:self.services];
        selectDestination.fileSelectDestinationType = self.fileSelectDestinationType;
        selectDestination.fileSelectSpecialActionType = self.fileSelectSpecialActionType;
        selectDestination.operatingHandler = self.operatingHandler;
        selectDestination.isRootFileDirectory = NO;
        selectDestination.exceptFolderList = self.exceptFolderList;
        selectDestination.expectFileType = self.expectFileType;
        selectDestination.currentSelectedFolderName = fileItem.title;
        selectDestination.titleForTypeSelectFolder = self.titleForTypeSelectFolder;

        if (self.fileSelectSpecialActionType == FSFileSelectSpecialActionTypeMultiple) {

            selectDestination.multiSelectDoneHandler = self.multiSelectDoneHandler;
            [selectDestination setMultiSelectLimit:self.MultiSelectLimit];
            selectDestination.multiSelectedFilesArr = self.multiSelectedFilesArr;
            selectDestination.multiSelectedIndexPathsDict = self.multiSelectedIndexPathsDict;
        }
        [selectDestination loadFilesWithPath:fileItem.filekey];
        [self.services pushViewController:selectDestination animated:YES];
    }
    
}

-(NSString *)getDirectoryPath:(NSString *)filePath {
    BOOL isDirectory = NO;
    [[NSFileManager defaultManager] fileExistsAtPath:filePath isDirectory:&isDirectory];
    if (!isDirectory) {
        filePath = [filePath stringByDeletingLastPathComponent];
    }
    
    NSRange range = [filePath rangeOfString:@"Documents"];
    NSString *directoryName = [filePath substringFromIndex:range.location];

    return directoryName;
}

- (void)tableView:(UITableView *)tableView didDeselectRowAtIndexPath:(NSIndexPath *)indexPath {
    FileItem *fileItem = [self dataForAtIndexPath:indexPath];
    if (self.editing) {
        [super tableView:tableView didDeselectRowAtIndexPath:indexPath];
        fileItem.selected = NO;
        return;
    }
}

- (BOOL)tableView:(UITableView*)tableView canEditRowAtIndexPath:(NSIndexPath*)indexPath {
    if (self.fileSelectDestinationType == FSFileSelectDestinationTypeSelectFile) {
        FileItem *fileItem = [self.fileItemsArray objectAtIndex:indexPath.row];
        if (fileItem.isFolder){
            return NO;
        }
        return YES;
    }

    return YES;
}

#pragma mark - NSNotification handler Method
- (void)applicationHandleOpenURL:(NSNotification *)object {
    [self navigationBarBackTouch];
}

@end
