/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FileManageBaseViewController.h"
#import "EGORefreshTableHeaderView.h"
#import "FileItem.h"
#import "FileManageListViewController.h"
#import "ThumbnailScrollViewCell.h"
#import "PrivateDefine.h"
#import "Utility.h"
#import "FSEmptyDataSetView.h"

#define FILE_CHANGED @"FileChangedNotification"

@interface FileManageBaseViewController () <EGORefreshTableHeaderDelegate>

@property (nonatomic, strong) FSEmptyDataSetView *emptyDataSetView;
@property (nonatomic, assign) BOOL hasLoad;

- (BOOL)checkDiskFile:(int)currentSelectRow;
- (void)setNavigationBarTitleForFolderName;
- (NSArray *)getFileObjectFromFileNames:(NSArray *)fileNames;

@end

@implementation FileManageBaseViewController {
    EGORefreshTableHeaderView *_refreshHeaderView;
}

@synthesize tableFileList = _tableFileList;
@synthesize thumbnailFileList = _thumbnailFileList;
@synthesize activityIndicator = _activityIndicator;

@synthesize currentFolder = _currentFolder;
@synthesize isEnableThumbnail = _isEnableThumbnail;
@synthesize viewMode = _viewMode;
@synthesize selectedItemIndexForRename = _selectedItemIndexForRename;
@synthesize fileItem = _fileItem;

#pragma mark - View lifecycle

- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
        arrayFile = [[NSMutableArray alloc] init];
        arraySelectedFilepaths = [[NSMutableArray alloc] init];
        searchKeyword = nil;
        //monitor file change
        queue = dispatch_queue_create("FileManageQueue", NULL);
        _isTouchModeFileOperation = NO;
        _isEnableThumbnail = YES;
        // File view mode initialization
        _viewMode = -1;
        _hasLoad = NO;
    }
    return self;
}

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];

    // Release any cached data, images, etc that aren't in use.
}

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
    if (_isEnableThumbnail) {
        _tableFileList.hidden = YES;
        CGRect viewFrame = self.view.bounds;
        self.thumbnailFileList.frame = viewFrame;
        [self.thumbnailFileList setBackgroundColor:[UIColor whiteColor]];
        self.thumbnailFileList.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
        self.thumbnailFileList.scrollEnabled = YES;
        self.thumbnailFileList.mainSuperView = self.navigationController.view.superview.superview;
        self.thumbnailFileList.isSwapMode = NO;
        self.thumbnailFileList.isNeedShake = NO;
        self.thumbnailFileList.cellArrangement = ThumbnailScrollViewCellArrangementRight;
        [self.view addSubview:self.thumbnailFileList];
    }
    //pull to refresh
    if (_refreshHeaderView == nil) {
        CGRect headerViewFrame = CGRectMake(0.0f, 0.0f - _tableFileList.bounds.size.height, self.view.frame.size.width, _tableFileList.bounds.size.height);
        if (_isEnableThumbnail) {
            headerViewFrame = CGRectMake(0.0f, 0.0f - _thumbnailFileList.bounds.size.height, self.view.frame.size.width, _thumbnailFileList.bounds.size.height);
        }
        EGORefreshTableHeaderView *view = [[EGORefreshTableHeaderView alloc] initWithFrame:headerViewFrame];
        view.delegate = self;
        [_tableFileList addSubview:view];
        if (_isEnableThumbnail) {
            [view removeFromSuperview];
            [_thumbnailFileList addSubview:view];
        }
        _refreshHeaderView = view;
        _refreshHeaderView.backgroundColor = ThemeViewBackgroundColor;
    }
    //  update the last update date
    [_refreshHeaderView refreshLastUpdatedDate];
    [self refreshInterface];
    [self setUIAccordingToData];
    _selectedItemIndexForRename = -1;
}

//- (void)viewDidUnload {
//    [self setTableFileList:nil];
//    [self setActivityIndicator:nil];
//    [self setThumbnailFileList:nil];
//    _refreshHeaderView = nil; //_refreshHeaderView is retained by _tableFileList, so don't need to release it
//    [super viewDidUnload];
//    // Release any retained subviews of the main view.
//    // e.g. self.myOutlet = nil;
//}

- (ThumbnailScrollView *)thumbnailFileList {
    if (!_thumbnailFileList) {
        ThumbnailScrollView *scrollView = [[ThumbnailScrollView alloc] init];
        _thumbnailFileList = scrollView;
        _thumbnailFileList.delegate = self;
        _thumbnailFileList.actionDelegate = self;
        _thumbnailFileList.dataSource = self;
        _thumbnailFileList.sortDelegate = self;
    }
    return _thumbnailFileList;
}

/*
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return YES;
}
*/

- (void)dealloc {
    //this fix the loadData crash problem. in navigation the viewController get released, so delay perform fail.
    _refreshHeaderView.delegate = nil;
    _refreshHeaderView = nil; //_refreshHeaderView is retained by _tableFileList, so don't need to release it, and it must be called before
    _thumbnailFileList.delegate = nil;
    _thumbnailFileList.dataSource = nil;
    _thumbnailFileList.sortDelegate = nil;
    _thumbnailFileList.actionDelegate = nil;
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    if (self.navigationController) {
        self.navigationController.interactivePopGestureRecognizer.enabled = NO;
        if (!self.navigationItem.titleView) {
            UILabel *titleLabel = [[UILabel alloc] init];
            titleLabel.frame = CGRectMake(0.0f, 0.0f, self.navigationController.navigationBar.frame.size.width, 44.0f);
            titleLabel.text = self.title;
            titleLabel.textAlignment = NSTextAlignmentCenter;
            titleLabel.autoresizesSubviews = YES;
            titleLabel.backgroundColor = [UIColor clearColor];
            titleLabel.textColor = WhiteThemeTextColor;
            titleLabel.shadowColor = [UIColor darkGrayColor];
            titleLabel.shadowOffset = CGSizeMake(0.0f, -1.0f);
            titleLabel.font = [UIFont boldSystemFontOfSize:18.0f];
            [titleLabel sizeToFit];
            self.navigationItem.titleView = titleLabel;
        }
    }
}

#pragma mark - Properties

- (NSString *)currentFolder {
    if (!_currentFolder) {
        NSString *path = DOCUMENT_PATH;
        self.currentFolder = path;
    }

    return _currentFolder;
}

#pragma mark - UINavigationController delegate handler
- (void)navigationController:(UINavigationController *)navigationController willShowViewController:(UIViewController *)viewController animated:(BOOL)animated {
    if (![viewController.navigationItem respondsToSelector:@selector(setRightBarButtonItems:)]) //iOS 4
    {
        [viewController viewWillAppear:animated];
    }
    if (navigationController.navigationBar.tag != 1) {
        navigationController.navigationBar.tag = 1;
    }
}

#pragma mark EGORefreshTableHeaderDelegate Methods

- (void)scrollViewDidScroll:(UIScrollView *)scrollView {
    [_refreshHeaderView adjustImageFrame];
    [_refreshHeaderView egoRefreshScrollViewDidScroll:scrollView];
}

- (void)scrollViewDidEndDragging:(UIScrollView *)scrollView willDecelerate:(BOOL)decelerate {
    [_refreshHeaderView egoRefreshScrollViewDidEndDragging:scrollView];
}

- (void)egoRefreshTableHeaderDidTriggerRefresh:(EGORefreshTableHeaderView *)view {
    [self loadData];
}

- (BOOL)egoRefreshTableHeaderDataSourceIsLoading:(EGORefreshTableHeaderView *)view {
    return _reloading; // should return if data source model is reloading
}

- (NSDate *)egoRefreshTableHeaderDataSourceLastUpdated:(EGORefreshTableHeaderView *)view {
    return nil;
}

//#pragma mark -  table view delegate and datasource handler
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return 0;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    return nil;
}

#pragma mark - sort methods for this class and subclass

- (void)refreshSort {
    [arrayFile sortUsingComparator:^NSComparisonResult(id obj1, id obj2) {
        FileItem *fileObj1 = (FileItem *) obj1;
        FileItem *fileObj2 = (FileItem *) obj2;
        if (fileObj1.isFolder != fileObj2.isFolder) {
            return fileObj1.isFolder ? NSOrderedAscending : NSOrderedDescending;
        } else {
            NSComparisonResult compareResult = NSOrderedSame;
            if (self.sortType == FileSortType_Name) {
                if (([fileObj1.fileName rangeOfCharacterFromSet:[NSCharacterSet decimalDigitCharacterSet]].location != NSNotFound) && ([fileObj2.fileName rangeOfCharacterFromSet:[NSCharacterSet decimalDigitCharacterSet]].location != NSNotFound)) {
                    compareResult = [fileObj1.fileName compare:fileObj2.fileName options:NSNumericSearch];
                } else {
                    compareResult = [fileObj1.fileName compare:fileObj2.fileName options:NSCaseInsensitiveSearch];
                }
            } else if (self.sortType == FileSortType_Date) {
                compareResult = [fileObj1.modifiedDate compare:fileObj2.modifiedDate];
            } else if (self.sortType == FileSortType_Size) {
                if (fileObj1.fileSize != fileObj2.fileSize) {
                    compareResult = fileObj1.fileSize < fileObj2.fileSize ? NSOrderedAscending : NSOrderedDescending;
                }
            } else if (self.sortType == FileSortType_LastView) {
                compareResult = [fileObj1.lastViewDate compare:fileObj2.lastViewDate];
            }
            if (compareResult != NSOrderedSame && self.sortMode == FileSortMode_Descending) {
                compareResult = compareResult == NSOrderedAscending ? NSOrderedDescending : NSOrderedAscending;
            }
            return compareResult;
        }
    }];
}

#pragma mark - Public API

- (void)clearData {
    [arrayFile removeAllObjects];
    if (_isEnableThumbnail) {
        [_thumbnailFileList reloadData];
    } else {
        [_tableFileList reloadData];
    }
}

- (void)loadData {
    _reloading = YES;
    //get data first
    [_activityIndicator startAnimating];
    NSString *path = self.currentFolder;
    NSMutableDictionary *dictUserData = nil;
    [FileBrowser getFileListWithFolderPath:path
                         withSearchKeyword:searchKeyword
                                  userData:dictUserData
                               withHandler:^(NSMutableArray *fileListArray) {
                                   [self->_activityIndicator stopAnimating];
                                   NSArray *fileList = [fileListArray sortedArrayUsingComparator:^NSComparisonResult(id obj1, id obj2) {
                                       FileItem *fileObj1 = (FileItem *) obj1;
                                       FileItem *fileObj2 = (FileItem *) obj2;
                                       if (fileObj1.isFolder != fileObj2.isFolder) {
                                           return fileObj1.isFolder ? NSOrderedAscending : NSOrderedDescending;
                                       } else {
                                           NSComparisonResult compareResult = NSOrderedSame;
                                           if (self.sortType == FileSortType_Name) {
                                               if (([fileObj1.fileName rangeOfCharacterFromSet:[NSCharacterSet decimalDigitCharacterSet]].location != NSNotFound) && ([fileObj2.fileName rangeOfCharacterFromSet:[NSCharacterSet decimalDigitCharacterSet]].location != NSNotFound)) {
                                                   compareResult = [fileObj1.fileName compare:fileObj2.fileName options:NSNumericSearch];
                                               } else {
                                                   compareResult = [fileObj1.fileName compare:fileObj2.fileName options:NSCaseInsensitiveSearch];
                                               }
                                           } else if (self.sortType == FileSortType_Date) {
                                               compareResult = [fileObj1.modifiedDate compare:fileObj2.modifiedDate];
                                           } else if (self.sortType == FileSortType_Size) {
                                               if (fileObj1.fileSize != fileObj2.fileSize) {
                                                   compareResult = fileObj1.fileSize < fileObj2.fileSize ? NSOrderedAscending : NSOrderedDescending;
                                               }
                                           } else if (self.sortType == FileSortType_LastView) {
                                               compareResult = [fileObj2.lastViewDate compare:fileObj1.lastViewDate];
                                           }
                                           if (compareResult != NSOrderedSame && self.sortMode == FileSortMode_Descending) {
                                               compareResult = compareResult == NSOrderedAscending ? NSOrderedDescending : NSOrderedAscending;
                                           }
                                           return compareResult;
                                       }
                                   }];
                                   self->arrayFile = [[NSMutableArray alloc] initWithArray:fileList];
                                   //refresh UI accordingly
                                   if (!self->_isEnableThumbnail) {
                                       [self->_tableFileList reloadData];
                                   } else {
                                       dispatch_async(dispatch_get_main_queue(), ^{
                                           [self->_thumbnailFileList reloadData];
                                       });
                                   }
                                   self->_reloading = NO;
                                    self.hasLoad = YES;
                                   if (self->_isEnableThumbnail) {
                                       [self->_refreshHeaderView performSelector:@selector(egoRefreshScrollViewDataSourceDidFinishedLoading:) withObject:self->_thumbnailFileList afterDelay:0.1];
                                   } else {
                                       [self->_refreshHeaderView performSelector:@selector(egoRefreshScrollViewDataSourceDidFinishedLoading:) withObject:self->_tableFileList afterDelay:0.1];
                                   }
                               }];
}

//when file change, load data after a delay, otherwise the file size is not correct
- (void)loadDataForFileChange:(NSNotification *)notification {
    //only when changed folder is current folder's sub-folder, need to load data: 1. update current list; 2. update current folder's item number
    //if the change folder is above current folder, not need to refresh as it will do when navigate back
    NSString *directory = notification.object;
    if ([directory rangeOfString:self.currentFolder].location == 0) {
        //the local file change event can happen multiple times, so that multiple notification posted.
        //but only the last one will be executed
        [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(loadData) object:nil];
        if (_thumbnailFileList.isMoving) {
            _needReloadData = YES;
        } else {
            [self performSelector:@selector(loadData) withObject:nil afterDelay:0.5];
        }
    }
}

- (BOOL)getValidFloderWithContentsOfDirectory:(NSString *)directoryPath {
    NSFileManager *fileManager = [[NSFileManager alloc] init];
    NSDirectoryEnumerator *dirEnumerator = [fileManager enumeratorAtPath:directoryPath];
    NSString *fileName = nil;
    BOOL isDir = YES;
    while ((fileName = [dirEnumerator nextObject])) {
        NSString *fullFilePath = [directoryPath stringByAppendingPathComponent:fileName];
        [fileManager fileExistsAtPath:fullFilePath isDirectory:&isDir];
        if (!isDir) {
            return YES;

        } else {
            isDir = NO;
        }
    }
    BOOL isDirectory = YES;
    [[NSFileManager defaultManager] fileExistsAtPath:directoryPath isDirectory:&isDirectory];
    if (isDir) {
        return isDirectory ? NO : YES;

    } else {
        return NO;
    }
}

#pragma mark - Protected override methods

//FileManageViewController is especially driven by data, as it will open outside file and the UI must rest according to data.
- (void)setUIAccordingToData {
    //nothing to do here. set UI in child class.
    [self setNavigationBarTitleForFolderName];
}

- (void)refreshInterface {
    _searchBar.placeholder = FSLocalizedForKey(@"kSearchPlaceholder");
}

#pragma mark - Private API

- (BOOL)checkDiskFile:(int)currentSelectRow {
    FileItem *selectedFile = [arrayFile objectAtIndex:currentSelectRow];
    NSArray *selectedRows = [NSArray arrayWithObject:[NSIndexPath indexPathForRow:currentSelectRow inSection:0]];
    //check the file exist and the modified date is same as selected one

    FileItem *diskFile = [[FileItem alloc] initWithPath:selectedFile.path modifiedDate:nil isFavorite:nil];
    if (!diskFile.isValidPDF) //cannot find the file in this path
    {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:[NSString stringWithFormat:FSLocalizedForKey(@"kNoFileInDisk"), selectedFile.fileName] preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
        [alertController addAction:action];
        [[self fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
        [arrayFile removeObject:selectedFile];
        if (_isEnableThumbnail) {
            [_thumbnailFileList reloadData];
        } else {
            [_tableFileList deleteRowsAtIndexPaths:selectedRows withRowAnimation:UITableViewRowAnimationFade];
            [_tableFileList reloadData];
        }
        return NO;
    }
    if (![diskFile.modifiedDate isEqualToDate:selectedFile.modifiedDate]) //modified date is not the same, means it's not the same file. update list.
    {
        [arrayFile replaceObjectAtIndex:currentSelectRow withObject:diskFile];
        if (_isEnableThumbnail) {
            [_thumbnailFileList reloadCellAtIndex:currentSelectRow withAnimation:ThumbnailScrollViewCellAnimationFade];
        } else {
            [_tableFileList reloadRowsAtIndexPaths:selectedRows withRowAnimation:UITableViewRowAnimationFade];
        }
        return YES;
    }
    return YES;
}

- (void)setNavigationBarTitleForFolderName {
    if ([[self.currentFolder lastPathComponent] isEqualToString:@"Documents"]) {
        self.title = FSLocalizedForKey(@"kDocuments");
    } else {
        {
            self.title = [self.currentFolder lastPathComponent];
        }
    }
}

- (NSArray *)getFileObjectFromFileNames:(NSArray *)fileNames {
    NSMutableArray *fileObjects = [[NSMutableArray alloc] init];
    for (NSString *file in fileNames) {
        FileItem *fileObject = [[FileItem alloc] initWithPath:file modifiedDate:nil isFavorite:nil];
        [fileObjects addObject:fileObject];
    }
    return (NSArray *) fileObjects;
}

#pragma mark - Check button on alertView

//- (void)addCheckButtonForAlertView:(UIAlertView *)alertView buttonTitle:(NSString *)buttonTitle {
//    UIImage *image = FSImageNamedAndNormalColor(@"common_redio_blank");
//    UIButton *buttonCheck = [[UIButton alloc] initWithFrame:CGRectMake(20.0f, 75.0f, 100.0, 30.0f)];
//    [buttonCheck setTitle:buttonTitle forState:UIControlStateNormal];
//    [buttonCheck setImage:image forState:UIControlStateNormal];
//    image = FSImageNamedAndNormalColor(@"common_redio_selected");
//    [buttonCheck setImage:image forState:UIControlStateHighlighted];
//    [buttonCheck setImage:image forState:UIControlStateSelected];
//    buttonCheck.tag = 200;
//    [buttonCheck addTarget:self action:@selector(buttonCheckOfAlertViewClick:) forControlEvents:UIControlEventTouchUpInside];
//    [alertView addSubview:buttonCheck];
//}
//
//- (void)buttonCheckOfAlertViewClick:(UIButton *)sender {
//    sender.selected = !sender.selected;
//}

#pragma mark - Check and need start or stop file monitor

- (void)needStartFileMonitor {
    [[NSNotificationCenter defaultCenter] removeObserver:self name:FILE_CHANGED object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(loadDataForFileChange:) name:FILE_CHANGED object:nil];
}

- (void)needStopFileMonitor {
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(loadData) object:nil];
}

#pragma mark - public event handler
- (IBAction)buttonNavigationBackClick:(id)sender {
    [self.navigationController popViewControllerAnimated:YES];
}

#pragma mark - ThumbnailScrollView data source

- (int)numberOfItemsInThumbnailScrollView:(ThumbnailScrollView *)scrollView {
    [self reloadEmptyDataSet];
    return (int) arrayFile.count;
}

- (CGSize)sizeForCellsInThumbnailScrollView:(ThumbnailScrollView *)scrollView {
    if (_viewMode == 0) //listview mode
    {
        return CGSizeMake(scrollView.bounds.size.width, 68.0f);
    }
    return DEVICE_iPHONE || [self fs_currentSceneiPadHorizontalCompact] ? CGSizeMake(87.0f, 135.0f) : CGSizeMake(134.f, 184.0f);
}

- (ThumbnailScrollViewCell *)thumbnailScrollView:(ThumbnailScrollView *)scrollView cellAtIndex:(NSInteger)index {
    NSAssert(false, @"cellAtIndex in base class FileManageBaseViewController should not be called");
    return nil;
}

- (int)thumbnailScrollView:(ThumbnailScrollView *)scrollView numberPagesOfOneRowInInterfaceOrientation:(UIInterfaceOrientation)orientation {
    if (_viewMode == 0) {
        return 1;
    }
    UIEdgeInsets contentMargins = [self thumbnailScrollView:scrollView contentMarginsInInterfaceOrientation:orientation];
    UIEdgeInsets pageMargins = [self thumbnailScrollView:scrollView pageMarginsInInterfaceOrientation:orientation];
    CGSize screenSize = [self sizeForCellsInThumbnailScrollView:scrollView];
    CGFloat w = self.view.fs_width - contentMargins.left - contentMargins.right;
    int count = w / (screenSize.width + pageMargins.left + pageMargins.right);
    return count;
}

- (UIEdgeInsets)thumbnailScrollView:(ThumbnailScrollView *)scrollView contentMarginsInInterfaceOrientation:(UIInterfaceOrientation)orientation {
    if (_viewMode == 0) {
        return UIEdgeInsetsMake(0, 0.0f, 44.0f, 0.0f);
    }
    if ([self fs_currentSceneiPadHorizontalCompact]) {
        return UIEdgeInsetsMake(0, 16.0f, 44, 0.0);
    }
    if (UIInterfaceOrientationIsPortrait(orientation)) {
        return DEVICE_iPHONE ? UIEdgeInsetsMake(0, 16.0f, 44, 0.0) : UIEdgeInsetsMake(0, 15.0f, 44, 0.0);
    } else {
        return DEVICE_iPHONE ? UIEdgeInsetsMake(0, ([Utility getScreenSizeMode] == ScreenSizeMode_35 ? 12.0f : 35.0f), 44, ([Utility getScreenSizeMode] == ScreenSizeMode_35 ? 5 : 0)) : UIEdgeInsetsMake(0, 0.0f, 44.0f, 0.0f);
    }
}

- (UIEdgeInsets)thumbnailScrollView:(ThumbnailScrollView *)scrollView pageMarginsInInterfaceOrientation:(UIInterfaceOrientation)orientation {
    if (_viewMode == 0) {
        return UIEdgeInsetsMake(0, 0, 0, 0);
    }
    if (UIInterfaceOrientationIsPortrait(orientation)) {
        if (DEVICE_iPHONE || [self fs_currentSceneiPadHorizontalCompact]) {
            ScreenSizeMode sizeMode = [Utility getScreenSizeMode];
            if (sizeMode == ScreenSizeMode_47) {
                return UIEdgeInsetsMake(5, 11, 5, 11);
            } else if (sizeMode == ScreenSizeMode_55) {
                return UIEdgeInsetsMake(5, 5, 5, 2);
            } else {
                return UIEdgeInsetsMake(5, 3, 5, 3);
            }
        } else {
            return UIEdgeInsetsMake(10.0f, 8.0f, 5.0f, 5.0f);
        }
    } else {
        if (DEVICE_iPHONE || [self fs_currentSceneiPadHorizontalCompact]) {
            ScreenSizeMode sizeMode = [Utility getScreenSizeMode];
            if (sizeMode == ScreenSizeMode_40) {
                return UIEdgeInsetsMake(5, 10, 5, 3);
            } else if (sizeMode == ScreenSizeMode_47) {
                return UIEdgeInsetsMake(5, 7, 5, 5);
            } else if (sizeMode == ScreenSizeMode_55) {
                return UIEdgeInsetsMake(5, 12, 5, 12);
            } else {
                return UIEdgeInsetsMake(5, 7, 5, 12);
            }
        } else {
            return UIEdgeInsetsMake(10.0f, 18.0f, 5.0f, 18.0f);
        }
    }
}

#pragma mark - ThumbnailScrollView delegate

- (BOOL)shouldSortInThumbnailScrollView:(ThumbnailScrollView *)scrollView cellIndex:(int)cellIndex {
    BOOL _needSort = YES;
    if (_needSort) {
        FoxitLog(@"stop file monitor");
        [self needStopFileMonitor];
    }
    return _needSort;
}

- (void)thumbnailScrollView:(ThumbnailScrollView *)scrollView didCancelSortCellIndexes:(NSArray *)cellIndexes blankCellIndex:(NSInteger)blankCellIndex {
    FoxitLog(@"start file monitor-didCancel");
    if (_needReloadData) {
        FoxitLog(@"need reload data");
        _needReloadData = NO;
        [self loadData];
    }
    [self needStartFileMonitor];
}

- (void)thumbnailScrollView:(ThumbnailScrollView *)scrollView didTapOnCellAtIndex:(NSInteger)index {
    self.fileItem = nil;
    FileItem *openingFile = [arrayFile objectAtIndex:index];
    if (!openingFile.isFolder) {
        if (!_thumbnailFileList.editing && !openingFile.isOpen) //open selected file if it's not opened yet
        {
            if ([self checkDiskFile:(int) index]) {
                FileItem *openingFile = [arrayFile objectAtIndex:index];
                //find if there is file ready open
                FileItem *openedFile = nil;
                for (FileItem *obj in arrayFile) {
                    if (obj.isOpen) {
                        openedFile = obj;
                        break;
                    }
                }
                if (openedFile != nil)
                    openedFile.isOpen = NO;

                //refresh UI
                if (_isEnableThumbnail)
                    [_thumbnailFileList reloadData];
                else
                    [_tableFileList reloadData];

                if ([self.delegate conformsToProtocol:@protocol(FileDelegate)] && [self.delegate respondsToSelector:@selector(onItemCliked:)]) {
                    [self.delegate onItemCliked:openingFile];
                    _fileItem = openingFile;
                }
            }
        }
    }
}

- (FSEmptyDataSetView *)emptyDataSetView{
    if (!_emptyDataSetView) {
        _emptyDataSetView = [FSEmptyDataSetView new];
    }
    return  _emptyDataSetView;
}

- (BOOL)emptyDataSetDisplayed{
    return _emptyDataSetView.superview;
}


- (void)reloadEmptyDataSet{
    if ([self emptyDataSetShouldDisplay:self.thumbnailFileList] && self.hasLoad) {
        if (self.emptyDataSetView.superview != self.thumbnailFileList) {
            [self.thumbnailFileList addSubview:self.emptyDataSetView];
            [self.emptyDataSetView mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.center.left.right.bottom.top.fs_equalToSuperview();
            }];
        }
        self.emptyDataSetView.title = [self titleForEmptyDataSet:self.thumbnailFileList];
        self.emptyDataSetView.image = [self imageForEmptyDataSet:self.thumbnailFileList];
        self.emptyDataSetView.verticalOffset = [self verticalOffsetForEmptyDataSet:self.thumbnailFileList];
        self.emptyDataSetView.scrollEnabledForSupView = ![self emptyDataSetShouldAllowScroll:self.thumbnailFileList];
    }else{
        if (self.emptyDataSetView.superview) {
            [self.emptyDataSetView removeFromSuperview];
        }
    }
}

- (NSAttributedString *)titleForEmptyDataSet:(ThumbnailScrollView *)tableView {
    return [[NSAttributedString alloc] initWithString:[self titleForEmptyDataSet] attributes:@{
        NSFontAttributeName:self.fs_standardGeneralFont_GF3,
        NSForegroundColorAttributeName:self.fs_textColor_T3
        }];
}

- (NSString *)titleForEmptyDataSet{
    return FSLocalizedForKey(@"kNoFolderFiles");
}

- (UIImage *)imageForEmptyDataSet:(ThumbnailScrollView *)tableView {
    return FSImageNamedAndPrimaryColor(@"more_file_no_file");
}

- (CGFloat)verticalOffsetForEmptyDataSet:(ThumbnailScrollView *)tableView {
    return 0;
}

- (BOOL)emptyDataSetShouldDisplay:(ThumbnailScrollView *)tableView {
    @try {
        return !arrayFile.count;
    } @catch (NSException *exception) {
        return YES;
    }
    return YES;
}

- (BOOL)emptyDataSetShouldAllowScroll:(ThumbnailScrollView *)tableView{
    return NO;
}

@end
