/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "DynamicXFASignatureAnnotHandler.h"
#import "AnnotationSignature.h"
#import "SignToolHandler.h"
#import "SignatureListViewController.h"
#import "SignatureViewController.h"
#import <sys/stat.h>
#import "DigitalSignatureUtility.h"
#import "CheckCerViewCtr.h"
#import "FSPermissionUtilty.h"

@interface DynamicXFASignatureAnnotHandler() <UIPopoverPresentationControllerDelegate, SignatureListDelegate> {
}
@property (nonatomic, strong) FSSignature *currentSelectSign;

@property (nonatomic, strong) FSAnnot *editAnnot;
@property (nonatomic, strong) NSMutableDictionary *digitalSignDic;
@property (nonatomic, strong) SignatureListViewController *signatureListCtr;
@property (nonatomic, assign) BOOL isAdded;
@property (nonatomic, strong) UIImage *annotImage;
@property (nonatomic, strong) AnnotationSignature *signature;
@property (nonatomic, strong) SignatureViewController *signViewCtr;
@property (nonatomic, assign) BOOL isShowList;
@property (nonatomic, assign) BOOL isReview;
@property (nonatomic, assign) BOOL isShowVerifyInfo;
@property (nonatomic, strong) SignToolHandler *signToolHandler;
@property (nonatomic, assign) int currentSigState;
@property (nonatomic, assign) BOOL shouldShowMenu;

@property (nonatomic, assign) BOOL isUseLTVVerify;
//ltv
@property (nonatomic, assign) BOOL isVerifySig;
@property (nonatomic, assign) BOOL isUseExpire;
@property (nonatomic, assign) BOOL isIgnoreDocInfo;
@property (nonatomic, assign) FSLTVVerifierTimeType sigTimeType;
@property (nonatomic, assign) FSLTVVerifierVerifyMode verifyMode;
@property (nonatomic, strong) FSSignatureVerifyResult *ltvVerifyResult;
@property (nonatomic, assign) FSSignatureVerifyResultLTVState ltvState;
@property (nonatomic, assign) BOOL isFileChanged;

@property (nonatomic, assign) int certIndex;
@property (nonatomic, assign) int storeCertIndex;
@end
static unsigned long get_file_size(const char *path);

@implementation DynamicXFASignatureAnnotHandler {
    UIExtensionsManager __weak *_extensionsManager;
    FSPDFViewCtrl __weak *_pdfViewCtrl;
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _extensionsManager = extensionsManager;
        self.signToolHandler = (SignToolHandler *) [_extensionsManager getToolHandlerByName:Tool_Signature];
        
        _shouldShowMenu = NO;
        self.editAnnot = nil;
        self.editXFAWidget = nil;
        self.digitalSignDic = [[NSMutableDictionary alloc] init];
        self.isDynamicXFAFile = NO;
        self.menuRect = CGRectZero;
        
        self.isUseLTVVerify = !FSPDFViewCtrl.fipsMode;
        // FSLTVVerifierVerfiryModeAcrobat
        self.isVerifySig = YES;
        self.isUseExpire = YES;
        self.isIgnoreDocInfo = NO;
        self.sigTimeType = FSLTVVerifierSignatureCreationTime;
        self.verifyMode = FSLTVVerifierVerifyModeAcrobat;
        
        self.ltvVerifyResult = nil;
        self.ltvState = 0;
        self.isFileChanged = NO;
        self.currentSigState = 0;
        
        self.certIndex = -1;
        self.storeCertIndex = -1;
        
        [_pdfViewCtrl registerScrollViewEventListener:self];
        [_extensionsManager registerRotateChangedListener:self];
        [_extensionsManager registerAnnotHandler:self];
    }
    return self;
}

- (SignatureListViewController *)signatureListCtr {
    if (!_signatureListCtr) {
        _signatureListCtr = [_extensionsManager.viewCtrlManager createViewCtrlType:FSViewCtrlTypeSignatureList];
        _signatureListCtr.isFieldSigList = YES;
    }
    return _signatureListCtr;
}

- (FSAnnotType)getType {
    return FSAnnotWidget;
}

- (BOOL)annotCanAnswer:(FSAnnot *)annot {
    return YES;
}

- (FSRectF *)getAnnotBBox:(FSAnnot *)annot {
    return [annot getRect];
}

- (void)setShouldShowMenu:(BOOL)shouldShowMenu{
    _shouldShowMenu = shouldShowMenu;
    _extensionsManager.shouldShowMenu = shouldShowMenu;
}

- (void)onAnnotSelected:(FSAnnot *)annot{}

- (void)onAnnotDeselected:(FSAnnot *)annot{}

- (BOOL)isHitAnnot:(FSAnnot *)annot point:(FSPointF *)point {
    CGRect pvRect = [_pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:annot.pageIndex];
    pvRect = CGRectInset(pvRect, -30, -30);
    CGPoint pvPoint = [_pdfViewCtrl convertPdfPtToPageViewPt:point pageIndex:annot.pageIndex];
    if (CGRectContainsPoint(pvRect, pvPoint)) {
        return YES;
    }
    
    return NO;
}

- (void)dynamicXFASignatureSelected:(FSSignature *)signature shouldShowMenu:(BOOL)isShouldShowMenu{
    self.editAnnot = nil;

    
    FSSignature *sig = signature;
    self.currentSelectSign = sig;
    
    BOOL isSigned = [sig isSigned];
    
    NSMutableArray *array = [NSMutableArray array];
    
    _isAdded = NO;
    
    self.shouldShowMenu = isShouldShowMenu;
    if (isSigned) {
        MenuItem *vaildDigitalSign = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kVerifySign") object:self action:@selector(vaildDigitalSign)];
        MenuItem *cancel = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kCancel") object:self action:@selector(cancel)];
        [array addObject:vaildDigitalSign];
        [array addObject:cancel];
        _extensionsManager.menuControl.menuItems = array;
        [self showAnnotMenu];
    } else {
        if ([_pdfViewCtrl isRMSProtected]) {
            return;
        }
        
        BOOL canFillForm = [FSPermissionUtilty canFillForm:_extensionsManager];
        if (!canFillForm) {
            [_extensionsManager setCurrentWidget:nil];
            return;
        }
        
        MenuItem *addSign = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kAddSign") object:self action:@selector(sign)];
        MenuItem *signList = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kSignList") object:self action:@selector(signList)];
        MenuItem *cancel = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kCancel") object:self action:@selector(cancel)];
        [array addObject:addSign];
        [array addObject:signList];
        [array addObject:cancel];
        _extensionsManager.menuControl.menuItems = array;
        
        //Fill the necessary field of signature for signning if there is no such one.
        FSPDFDictionary *sigDict = [sig getSignatureDict];
        if (![sigDict hasKey:@"Filter"]) {
            FSPDFObject *value = [FSPDFObject createFromName:@"Adobe.PPKLite"];
            [sigDict setAt:@"Filter" pdf_object:value];
        }
        if (![sigDict hasKey:@"SubFilter"]) {
            FSPDFObject *value = [FSPDFObject createFromName:@"adbe.pkcs7.detached"];
            [sigDict setAt:@"SubFilter" pdf_object:value];
        }
        
        [self addFieldSign];
    }
}

#pragma mark do ltv verify
- (NSString *)getAlertViewMessageWithInfos:(NSMutableDictionary *)infos{
    
    FSSignatureStates status = (FSSignatureStates)self.currentSigState;
    BOOL isFileChange = [[infos objectForKey:@"isFileChange"] boolValue];
    
    NSString *valid = nil;
    if (status & FSSignatureStateVerifyValid || status == FSSignatureStateVerifyNoChange) {
        if (isFileChange) {
            valid = FSLocalizedForKey(@"kVerifyVaildModifyResult");
        } else {
            valid = FSLocalizedForKey(@"kVerifyVaildResult");
        }
        
    } else if (status & FSSignatureStateVerifyInvalid) {
        valid = FSLocalizedForKey(@"kVerifyInvaildResult");
    }
    else if (status & FSSignatureStateVerifyErrorByteRange){
        valid = FSLocalizedForKey(@"kVerifyResultErrorByteRange");
    } else if (status & FSSignatureStateVerifyIssueUnknown){
        valid = FSLocalizedForKey(@"kVerifyUnknownResult");
    } else if (status & FSSignatureStateVerifyIssueExpire){
        valid = FSLocalizedForKey(@"kVerifyResultIssueExpire");
    } else if (status & FSSignatureStateVerifyErrorData){
        valid = FSLocalizedForKey(@"kVerifyResultErrorData");
    } else {
        valid = FSLocalizedForKey(@"kVerifyResultInvaildAndOther");
    }
    
    NSMutableString *message = [[NSMutableString alloc] initWithFormat:@"\n%@",valid];
    
    if (self.ltvState == FSSignatureVerifyResultLTVStateEnable) {
        [message appendFormat:@"\n%@",FSLocalizedForKey(@"kLTVStateEnable")];
    }
    
    [message appendString:@"\n\n"];
    
    NSMutableArray *keyArr = @[
                               @"kCertIssuer",
                               @"kCertSerialNum",
                               @"kCertEmail",
                               @"kCertStartTime",
                               @"kCertEndTime",
                               @"kSignDate",
                               ].mutableCopy;
    NSMutableArray *valueArr = @[
                                 @"certPublisher",
                                 @"certSerialNum",
                                 @"certEmailInfo",
                                 @"certStartDate",
                                 @"certEndDate",
                                 @"signDate",
                                 ].mutableCopy;
    
    
    NSString *dateStr = [Utility displayDateInMDYHMS:[Utility convertFSDateTime2NSDate: [infos objectForKey:@"signDate"]]];
    if (dateStr == nil || dateStr.length == 0) {
        dateStr = @"";
    }
    [infos setObject:dateStr forKey:@"signDate"];
    
    for (int i = 0 ; i < keyArr.count; i++) {
        
        NSString *keyFlag = [keyArr objectAtIndex:i];
        NSString *valueFlag = [valueArr objectAtIndex:i];
        
        NSString *keyText = FSLocalizedForKey(keyFlag);
        NSString *valueText = [infos objectForKey:valueFlag];

        [message appendFormat:@"%@%@\n",keyText, valueText];
    }
    [message appendString:@"\n"];
    return message;
}

-(BOOL)hasModifiedDocument{
    BOOL isFileChanged = NO;
    NSString *fileName = @"";
    if (self.signToolHandler.getDocPath) {
        fileName = self.signToolHandler.getDocPath();
    }
    unsigned long fileLength = get_file_size([fileName UTF8String]);
    @try {
        FSInt32Array *byteRanges = [self.currentSelectSign getByteRangeArray];
        if ([byteRanges getSize] == 4) {
            unsigned int r1 = [byteRanges getAt:2];
            unsigned int r2 = [byteRanges getAt:3];
            if (fileLength != (r1 + r2)) {
                isFileChanged = YES;
                return isFileChanged;
            }
        }
    }
    @catch(NSException *exception) {
        NSLog(@"%@",exception);
    }
    
    return NO;
}

-(void)doLTVVerify {
    @try {
        [_pdfViewCtrl fs_showHUDLoading:FSLocalizedForKey(@"kVerifyingDigital") process:^{
            if (self.currentSelectSign == nil) return;
            FSLTVVerifier *ltvVerifier = [[FSLTVVerifier alloc] initWithDocument:self->_pdfViewCtrl.currentDoc is_verify_signature:self.isVerifySig use_expired_tst:self.isUseExpire ignore_doc_info:self.isIgnoreDocInfo time_type:self.sigTimeType];
            [ltvVerifier setVerifyMode:self.verifyMode];
            
            SignTrustedCertStoreCallback *signTrustedCertStoreCallback = [[SignTrustedCertStoreCallback alloc] init];
            [ltvVerifier setTrustedCertStoreCallback:signTrustedCertStoreCallback];
            
            FSSignatureVerifyResultArray *resultArr = [ltvVerifier verifySignature:self.currentSelectSign];
            
            if ([resultArr getSize] == 0) {
                NSLog(@"error");
            }
            
            self.ltvVerifyResult = [resultArr getAt:0];
            self.currentSigState = [self.ltvVerifyResult getSignatureState];
            self.ltvState = [self.ltvVerifyResult getLTVState];
            self.isFileChanged = [self hasModifiedDocument];
           
            dispatch_main_async_safe(^{
                [self showVerifyResult];
            });
        }];

        
    } @catch (NSException *exception) {
        NSLog(@"error");
    }
}

-(NSData *)getCertFromSignatureContent:(FSSignature *)signature {
    NSData *result = nil;
    NSMutableArray *certArr = [FSPDFCertUtil getCertFromSignatureContent:signature];
    
    if (self.certIndex == -1) {
        self.certIndex = 0;
    }
    else{
        if (self.certIndex >= certArr.count -1) {
            self.certIndex = 0;
        }else{
            self.certIndex++;
        }
    }
    
    result = [certArr objectAtIndex:certArr.count -1];
    return result;
}

- (UIView *)createCertContentView:(NSMutableDictionary *)infos{
    int contentViewHeight = 0;
    
    UIView *contentView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 320, 290)];
    contentView.backgroundColor = [UIColor whiteColor];
    contentView.layer.cornerRadius = 7;
    contentView.layer.masksToBounds = YES;
    
    UILabel *titleLable = [[UILabel alloc] init];
    titleLable.text = FSLocalizedForKey(@"kAddTrustedCertTitle");
    titleLable.font = [UIFont boldSystemFontOfSize:16];
    [contentView addSubview:titleLable];
    
    NSMutableParagraphStyle *paragraphStyle = [[NSMutableParagraphStyle alloc] init];
    paragraphStyle.lineBreakMode = NSLineBreakByWordWrapping;
    CGSize size = [titleLable.text boundingRectWithSize:CGSizeMake(300, 100) options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName : [UIFont boldSystemFontOfSize:16], NSParagraphStyleAttributeName : paragraphStyle} context:nil].size;
    [titleLable mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(titleLable.superview.mas_top).offset(10);
        make.centerX.mas_equalTo(titleLable.superview.mas_centerX);
        make.width.mas_equalTo(size.width + 1);
        make.height.mas_equalTo(size.height);
    }];
    contentViewHeight += 10 + size.height +15 ;
    
    UILabel *signatureViald = [[UILabel alloc] init];
    signatureViald.text = FSLocalizedForKey(@"kAddTrustedCertDesciption");
    
    signatureViald.font = [UIFont systemFontOfSize:15];
    signatureViald.numberOfLines = 0;
    [contentView addSubview:signatureViald];
    size = [signatureViald.text boundingRectWithSize:CGSizeMake(300, 100) options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName : [UIFont systemFontOfSize:15], NSParagraphStyleAttributeName : paragraphStyle} context:nil].size;
    [signatureViald mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(titleLable.mas_bottom).offset(15);
        make.left.mas_equalTo(signatureViald.superview.mas_left).offset(10);
        make.width.mas_equalTo(size.width + 1);
    }];
    contentViewHeight += size.height +15;
    
    NSMutableArray *keyArr = @[
                               @"kTrustedCertName",
                               @"kTrustedCertIssuer",
                               @"kTrustedCertValidFrom",
                               @"kTrustedCertValidTo",
                               @"kTrustedCertKeyUsage",
                               ].mutableCopy;
    
    NSMutableArray *valueArr = @[
                                 @"name",
                                 @"issuer",
                                 @"certStartDate",
                                 @"certEndDate",
                                 @"keyUsage",
                                 ].mutableCopy;
    
    UILabel *lastLabel = nil;
    
    for (int i = 0 ; i < keyArr.count; i++) {
        float maxRowHeight = 0;
        
        NSString *keyFlag = [keyArr objectAtIndex:i];
        NSString *valueFlag = [valueArr objectAtIndex:i];
        
        NSString *keyText = FSLocalizedForKey(keyFlag);
        NSString *valueText = [infos objectForKey:valueFlag];
        
        CGSize keySize = [keyText boundingRectWithSize:CGSizeMake(contentView.frame.size.width -20 , 100) options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName : [UIFont systemFontOfSize:14], NSParagraphStyleAttributeName : paragraphStyle} context:nil].size;
        
        CGSize valueSize = [valueText boundingRectWithSize:CGSizeMake(contentView.frame.size.width /2 -10 , 200) options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName : [UIFont systemFontOfSize:14], NSParagraphStyleAttributeName : paragraphStyle} context:nil].size;
        
        maxRowHeight = MAX(keySize.height, valueSize.height);
        
        UILabel *keyLabel = [[UILabel alloc] init];
        keyLabel.text = keyText;
        keyLabel.font = [UIFont systemFontOfSize:14];
        keyLabel.textColor = [UIColor colorWithRGB:0x8A8A8A];
        [contentView addSubview:keyLabel];
        
        [keyLabel mas_makeConstraints:^(MASConstraintMaker *make) {
            if (i == 0) {
                make.top.mas_equalTo(contentViewHeight);
            }else{
                make.top.mas_equalTo(lastLabel.mas_bottom).offset(5);
            }
            
            make.left.mas_equalTo(keyLabel.superview.mas_left).offset(10);
            make.width.mas_equalTo(keySize.width +1);
            make.height.mas_equalTo(maxRowHeight);
        }];
        
        UILabel *valueLabel = [[UILabel alloc] init];
        valueLabel.text = valueText;
        valueLabel.font = [UIFont systemFontOfSize:14];
        valueLabel.textColor = [UIColor colorWithRGB:0x8A8A8A];
        valueLabel.numberOfLines = 0;
        [contentView addSubview:valueLabel];
        
        [valueLabel mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(contentView.frame.size.width /2 );
            make.right.mas_equalTo(valueLabel.superview.mas_right).offset(-10);
            make.top.mas_equalTo(keyLabel.mas_top);
        }];
        
        lastLabel = keyLabel;
        
        contentViewHeight += 5 + maxRowHeight;
    }
    
    contentViewHeight += 10;
    
    CGRect frame = contentView.frame;
    frame.size.height = contentViewHeight;
    contentView.frame = frame;
    
    return contentView;
}

-(void)showVerifyResult{
    FSSignature *sig = self.currentSelectSign;
    
    int status = self.currentSigState;
    
    NSMutableDictionary *signatureVerifyInfo = @{
                                                 @"signDate":[sig getSignTime],
                                                 @"certSerialNum":[sig getCertificateInfo:@"SerialNumber"],
                                                 @"certStartDate":[sig getCertificateInfo:@"ValidPeriodFrom"],
                                                 @"certEndDate":[sig getCertificateInfo:@"ValidPeriodTo"],
                                                 @"certPublisher":@"",
                                                 @"certEmailInfo":@"",
                                                 @"status":@(status),
                                                 @"ltvState":@(self.ltvState),
                                                 }.mutableCopy;
    
    
    NSString *certIssuer = [sig getCertificateInfo:@"Issuer"];
    NSRange r = [certIssuer rangeOfString:@"CN="];
    if (0 < r.length) {
        certIssuer = [certIssuer substringFromIndex:r.location + 3];
        r = [certIssuer rangeOfString:@","];
        if (0 < r.length) {
            [signatureVerifyInfo setObject:[certIssuer substringToIndex:r.location] forKey:@"certPublisher"];
        }
    }
    
    NSString *certSubject = [sig getCertificateInfo:@"Subject"];
    r = [certSubject rangeOfString:@"E="];
    if (0 < r.length) {
        certSubject = [certSubject substringFromIndex:r.location + 2];
        r = [certSubject rangeOfString:@","];
        if (0 < r.length) {
            [signatureVerifyInfo setObject:[certSubject substringToIndex:r.location] forKey:@"certEmailInfo"];
        }
    }
    
    BOOL isFileChanged = NO;
    NSString *fileName = @"";
    if (self.signToolHandler.getDocPath) {
        fileName = self.signToolHandler.getDocPath();
    }
    unsigned long fileLength = get_file_size([fileName UTF8String]);
    FSInt32Array *byteRanges = [sig getByteRangeArray];
    if ([byteRanges getSize] == 4) {
        unsigned int r1 = [byteRanges getAt:2];
        unsigned int r2 = [byteRanges getAt:3];
        if (fileLength != (r1 + r2)) {
            isFileChanged = YES;
        }
    }
    
    [signatureVerifyInfo setObject:@(isFileChanged) forKey:@"isFileChanged"];
    
    [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kVerifyTitle") message:[self getAlertViewMessageWithInfos:signatureVerifyInfo] actions:@[FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kViewCert")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
        if (index == 1) {
            self.shouldShowMenu = NO;
            [self->_extensionsManager.viewCtrlManager presentViewCtrlType:FSViewCtrlTypeCheckCer params:sig].dismissCompletion = ^{
                self.shouldShowMenu = YES;
            };
            return ;
        }
        self.shouldShowMenu = YES;
        self->_isShowVerifyInfo = NO;
        [self->_extensionsManager setCurrentAnnot:nil];
        self.certIndex = -1;
        
    } onCtrl:[_extensionsManager.viewCtrlManager.navigationControllerStack topNavigationController]];
    self.shouldShowMenu = NO;
    _isShowVerifyInfo = YES;
}

- (void)dynamicXFASignatureSelected {
    self.currentSelectSign = nil;
    _annotImage = nil;
    if (_isAdded) {
        [self deleteSign];
        _isAdded = NO;
    }
    MenuControl *annotMenu = _extensionsManager.menuControl;
    if (annotMenu.isMenuVisible) {
        [annotMenu setMenuVisible:NO animated:YES];
    }
    self.isDynamicXFAFile = NO;
}

- (void)onReviewStateChanged:(BOOL)start {
    if (start) {
        _isReview = YES;
    } else {
        _isReview = NO;
    }
}

- (void)addFieldSign {
    NSString *selectSig = [AnnotationSignature getSignatureSelected];
    AnnotationSignature *signnature = [AnnotationSignature getSignature:selectSig];
    if (signnature.certMD5 && signnature.certPasswd && signnature.certFileName) {
        _isAdded = YES;
        self.signature = signnature;
        self.annotImage = [AnnotationSignature getSignatureImage:signnature.name];
//        CGRect newRect = [_pdfViewCtrl convertPdfRectToPageViewRect:self.editAnnot.fsrect pageIndex:self.editAnnot.pageIndex];
//        newRect = CGRectInset(newRect, -30, -30);
//        [_pdfViewCtrl refresh:newRect pageIndex:self.editAnnot.pageIndex needRender:NO];
        
        [self showAnnotMenu];
    } else {
        if ([AnnotationSignature getCertSignatureList].count <= 0) {
            SignatureViewController *signatureCtr = [[SignatureViewController alloc] initWithUIExtensionsManager:_extensionsManager];
            self.signViewCtr = signatureCtr;
            [signatureCtr fs_setCustomTransitionAnimator:self->_extensionsManager.presentedVCAnimator];
            signatureCtr.isFieldSig = YES;
            signatureCtr.currentSignature = nil;
            DynamicXFASignatureAnnotHandler *__weak weakSelf = self;
            signatureCtr.saveHandler = ^{
                NSString *selectSig = [AnnotationSignature getSignatureSelected];
                AnnotationSignature *signnature = [AnnotationSignature getSignature:selectSig];
                DynamicXFASignatureAnnotHandler *strongSelf = weakSelf;
                assert(strongSelf);
                strongSelf->_isAdded = YES;
                strongSelf.signature = signnature;
                strongSelf.annotImage = [AnnotationSignature getSignatureImage:signnature.name];
//                int pageIndex = strongSelf.editAnnot.pageIndex;
//                CGRect newRect = [strongSelf->_pdfViewCtrl convertPdfRectToPageViewRect:strongSelf.editAnnot.fsrect pageIndex:pageIndex];
//                newRect = CGRectInset(newRect, -30, -30);
//                [strongSelf->_pdfViewCtrl refresh:newRect pageIndex:pageIndex needRender:NO];
                weakSelf.shouldShowMenu = YES;
                [strongSelf showAnnotMenu];
            };
            signatureCtr.cancelHandler = ^{
                DynamicXFASignatureAnnotHandler *strongSelf = weakSelf;
                assert(strongSelf);
                [strongSelf->_extensionsManager setCurrentAnnot:nil];
            };
            weakSelf.shouldShowMenu = NO;
            [_pdfViewCtrl.fs_viewController presentViewController:signatureCtr
                                             animated:NO
                                           completion:^{
                                               
                                           }];
        } else {
            [self signList];
        }
    }
}

- (void)cancel {
    [_extensionsManager setCurrentAnnot:nil];
}

- (void)verify {
    [self vaildDigitalSign];
}

- (void)vaildDigitalSign {
    if (self.isUseLTVVerify) {
        [self doLTVVerify];
        return ;
    }
    
    FSSignature *sig = self.currentSelectSign;
    CERT_INFO *info = [[CERT_INFO alloc] init];
    
    FSSignatureStates status = [self.signToolHandler verifyDigitalSignature:nil signature:sig];
    info.signDate = [sig getSignTime];
    info.certSerialNum = [sig getCertificateInfo:@"SerialNumber"];
    info.certStartDate = [sig getCertificateInfo:@"ValidPeriodFrom"];
    info.certEndDate = [sig getCertificateInfo:@"ValidPeriodTo"];
    NSString *certIssuer = [sig getCertificateInfo:@"Issuer"];
    NSRange r = [certIssuer rangeOfString:@"CN="];
    if (0 < r.length) {
        certIssuer = [certIssuer substringFromIndex:r.location + 3];
        r = [certIssuer rangeOfString:@","];
        if (0 < r.length) {
            info.certPublisher = [certIssuer substringToIndex:r.location];
        }
    }
    
    NSString *certSubject = [sig getCertificateInfo:@"Subject"];
    r = [certSubject rangeOfString:@"E="];
    if (0 < r.length) {
        certSubject = [certSubject substringFromIndex:r.location + 2];
        r = [certSubject rangeOfString:@","];
        if (0 < r.length) {
            info.certEmailInfo = [certSubject substringToIndex:r.location];
        }
    }
    
    BOOL isFileChanged = NO;
    NSString *fileName = @"";
    if (self.signToolHandler.getDocPath) {
        fileName = self.signToolHandler.getDocPath();
    }
    unsigned long fileLength = get_file_size([fileName UTF8String]);
    FSInt32Array *byteRanges = [sig getByteRangeArray];
    if ([byteRanges getSize] == 4) {
        unsigned int r1 = [byteRanges getAt:2];
        unsigned int r2 = [byteRanges getAt:3];
        if (fileLength != (r1 + r2)) {
            isFileChanged = YES;
        }
    }
    
    [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kVerifyTitle") message:[self getAlertViewMessageWithStatus:status certInfo:info isFileChange:isFileChanged] actions:@[FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
        self->_isShowVerifyInfo = NO;
    } onCtrl:[self->_pdfViewCtrl fs_currentSceneTopMostViewController]];
    
    _isShowVerifyInfo = YES;
    [_extensionsManager setCurrentAnnot:nil];
}

static unsigned long get_file_size(const char *path) {
    unsigned long filesize = -1;
    struct stat statbuff;
    if (stat(path, &statbuff) < 0) {
        return filesize;
    } else {
        filesize = (unsigned long) statbuff.st_size;
    }
    return filesize;
}

static const char* statusMsg[] = {
    "Unknonw", //0x00000000
    "Unsigned signature.",//0x00000001
    "Signed signature.",//0x00000002
    "Verification state of a signature is valid.",//0x00000004
    "Verification state of a signature is invalid.",//0x00000008
    "Signature data is corrupted (that means the signature data cannot be parsed properly).", //0x00000010
    "Unsupported signature.",//0x00000020
    "Non expected byte range.",//0x00000040
    "The document has changed within the scope of the signature.",//0x00000080
    "Signature cannot be trusted (containing aggression). ",//0x00000100
    "Signature does not have any data for signing. ",//0x00000200
    "",
    "",
    "Verification state of the issuer is valid.",//0x00001000
    "Verification state of the issuer is unknown.",//0x00002000
    "Certificate for verifying issuer is revoked.",//0x00004000
    "Certificate for verifying issuer is expired.",//0x00008000
    "Not check the issuer.",//0x00010000
    "The verified issue is current issuer.",//0x00020000
    "No timestamp or not check timestamp.", //0x00040000
    "The signature is a timestamp signature.",//0x00080000
    "Verification state of the timestamp is valid.",  //0x00100000
    "Verification state of the timestamp is invalid.",//0x00200000
    "Verification state of the timestamp is expired.",//0x00400000
    "Verification state of the timestamp issuer is unknown.",//0x00800000
    "Verification state of the timestamp issuer is valid.",//0x01000000
    "Verification state of the timestamp time is valid, since the times is before the expiration date."//0x02000000
};
- (NSString*) getStatusErrorString:(FSSignatureStates) status
{
    NSString* result = nil;
    int index = 0;
    while (index < 32)
    {
        int length = sizeof(statusMsg)/sizeof(const char*);
        if(((status >> index)&0x1) && (index > 3 && index < length-1))
        {
            NSString* status = [[NSString alloc] initWithUTF8String:statusMsg[index+1]];
            if(!result)
                result = status;
            else
                result = [result stringByAppendingString:status];
        }
        index++;
    }
    return result;
}

- (NSString *)getAlertViewMessageWithStatus:(FSSignatureStates)status certInfo:(CERT_INFO *)certInfo isFileChange:(BOOL)isFileChange{

    NSString *valid = nil;
    if (status & FSSignatureStateVerifyValid || status == FSSignatureStateVerifyNoChange) {
        if (isFileChange) {
            valid = FSLocalizedForKey(@"kVerifyVaildModifyResult");
        } else {
            valid = FSLocalizedForKey(@"kVerifyVaildResult");
        }
        
    } else if (status & FSSignatureStateVerifyInvalid) {
        valid = FSLocalizedForKey(@"kVerifyInvaildResult");
    }
    else if (status & FSSignatureStateVerifyErrorByteRange){
        valid = FSLocalizedForKey(@"kVerifyResultErrorByteRange");
    } else if (status & FSSignatureStateVerifyIssueUnknown){
        valid = FSLocalizedForKey(@"kVerifyUnknownResult");
    } else if (status & FSSignatureStateVerifyIssueExpire){
        valid = FSLocalizedForKey(@"kVerifyResultIssueExpire");
    } else if (status & FSSignatureStateVerifyErrorData){
        valid = FSLocalizedForKey(@"kVerifyResultErrorData");
    } else {
        valid = FSLocalizedForKey(@"kVerifyResultInvaildAndOther");
    }
    
    NSMutableString *message = [[NSMutableString alloc] initWithFormat:@"\n%@",valid];
    
    [message appendString:@"\n\n"];
    [message appendFormat:@"%@%@\n",FSLocalizedForKey(@"kCertIssuer"), certInfo.certPublisher];
    [message appendFormat:@"%@%@\n",FSLocalizedForKey(@"kCertSerialNum"), certInfo.certSerialNum];
    [message appendFormat:@"%@%@\n",FSLocalizedForKey(@"kCertEmail"), certInfo.certEmailInfo];
    [message appendFormat:@"%@%@\n",FSLocalizedForKey(@"kCertStartTime"), certInfo.certStartDate];
    [message appendFormat:@"%@%@\n",FSLocalizedForKey(@"kCertEndTime"), certInfo.certEndDate];

    [message appendFormat:@"%@%@\n",FSLocalizedForKey(@"kSignDate"), [Utility displayDateInMDYHMS:[Utility convertFSDateTime2NSDate:certInfo.signDate]]];
    [message appendString:@"\n"];
    
    return message;
}

- (void)signDigitalSignature: (FSSignature *)sig signArea:(FSRectF *)rect signImagePath:(NSString *)imagePath savePath:(NSString*) pdfFilePath {
    if (pdfFilePath == nil) return;
    NSString *selectSig = [AnnotationSignature getSignatureSelected];
    AnnotationSignature *signature = [AnnotationSignature getSignature:selectSig];
    
    DIGITALSIGNATURE_PARAM *param = [[DIGITALSIGNATURE_PARAM alloc] init];
    param.certFile = [SIGNATURE_PATH stringByAppendingPathComponent:signature.certMD5];
    param.certPwd = signature.certPasswd;
    param.subfilter = @"adbe.pkcs7.detached";
    param.imagePath = imagePath;
    param.rect = rect;
    param.sigName = signature.name;
    param.signFilePath = pdfFilePath;
    [self.signToolHandler initSignature:sig withParam:param signature:signature];
    [[NSFileManager defaultManager] removeItemAtPath:param.imagePath error:nil];
    BOOL isSuccess = [self.signToolHandler signSignature:sig withParam:param];
    [_extensionsManager onDigitalSignatureSigned:isSuccess originalDoc:[sig getDocument] outputPath:pdfFilePath];
    if (isSuccess) {
        double delayInSeconds = 0.4;
        dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delayInSeconds * NSEC_PER_SEC));
        dispatch_after(popTime, dispatch_get_main_queue(), ^(void) {
            
            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kSuccess") message:FSLocalizedForKey(@"kSaveSignedDocSuccess") preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                             style:UIAlertActionStyleDefault
                                                           handler:^(UIAlertAction *action) {
                double delayInSeconds = 0.4;
                dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delayInSeconds * NSEC_PER_SEC));
                dispatch_after(popTime, dispatch_get_main_queue(), ^(void) {
                    if (self.signToolHandler.docChanging) {
                        self->_extensionsManager.isDocModified = NO;
                        self.signToolHandler.docChanging(pdfFilePath);
                    }
                });
                return;
            }];
            [alertController addAction:action];
            [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
        });
    } else {
        double delayInSeconds = 0.4;
        dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delayInSeconds * NSEC_PER_SEC));
        dispatch_after(popTime, dispatch_get_main_queue(), ^(void) {
            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kError") message:FSLocalizedForKey(@"kSaveSignedDocFailure") preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
            [alertController addAction:action];
            [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
        });
    }
}

- (void)addDigitalSign:(FSSignature *)sig signArea:(FSRectF *)rect signImagePath:(NSString *)imagePath {
    BOOL isAutoSaveSignedDoc = self->_extensionsManager.isAutoSaveSignedDoc;
    if (isAutoSaveSignedDoc) {
        NSString* userSavePath = self->_extensionsManager.signedDocSavePath;
        if (userSavePath == nil) {
            userSavePath = self->_pdfViewCtrl.filePath;
            if (userSavePath == nil) return;
            userSavePath = [userSavePath.stringByDeletingPathExtension stringByAppendingString:@"-signed.pdf"];
        }
        NSFileManager *fileManager = [NSFileManager defaultManager];
        if ([fileManager fileExistsAtPath:userSavePath]) {
            [fileManager removeItemAtPath:userSavePath error:nil];
        }
        [self signDigitalSignature:sig signArea:rect signImagePath:imagePath savePath:userSavePath];
        return;
    }
    
    FSFileSelectDestinationViewController *selectDestination = [[FSFileSelectDestinationViewController alloc] initWithServices:_extensionsManager.viewCtrlManager];
    selectDestination.isRootFileDirectory = YES;
    selectDestination.fileSelectDestinationType = FSFileSelectDestinationTypeSelectFolder;
    selectDestination.expectFileType = @[@"*"];
    [selectDestination loadFilesWithPath:DOCUMENT_PATH];
    selectDestination.operatingHandler = ^(FSFileSelectDestinationViewController *controller, NSArray *destinationFolder) {
        [controller dismissViewControllerAnimated:YES completion:^{
            [self inputFileNameWithDestinationFolder:destinationFolder sig:sig signArea:rect signImagePath:imagePath];
        }];
    };
    
    [_extensionsManager.viewCtrlManager presentViewController:selectDestination animated:YES completion:nil];
}

- (void)inputFileNameWithDestinationFolder:(NSArray *)destinationFolder sig:(FSSignature *)sig signArea:(FSRectF *)rect signImagePath:(NSString *)imagePath{
    [[UIAlertController fs_enablesActionAutomaticallyAlertWithTitle:@"" message:FSLocalizedForKey(@"kInputNewFileName") textFieldText:@"" actions:@[FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action, UITextField * _Nonnull textField) {
        if (index == 1) {
            NSString *fileName = textField.text;
            
            if ([fileName rangeOfString:@"/"].location != NSNotFound || fileName.length == 0) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kIllegalNameWarning") preferredStyle:UIAlertControllerStyleAlert];
                    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                                           style:UIAlertActionStyleCancel
                                                                         handler:^(UIAlertAction *action) {
                        dispatch_async(dispatch_get_main_queue(), ^{
                            [self inputFileNameWithDestinationFolder:destinationFolder sig:sig signArea:rect signImagePath:imagePath];
                        });
                        return;
                    }];
                    [alertController addAction:cancelAction];
                    [self->_pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
                });
                return;
            }
            
            NSString *pdfFilePath = [destinationFolder[0] stringByAppendingPathComponent:[NSString stringWithFormat:@"%@.pdf", fileName]];
            NSFileManager *fileManager = [NSFileManager defaultManager];
            if ([fileManager fileExistsAtPath:pdfFilePath]) {
                double delayInSeconds = 0.3;
                dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delayInSeconds * NSEC_PER_SEC));
                dispatch_after(popTime, dispatch_get_main_queue(), ^(void) {
                    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kFileAlreadyExists") preferredStyle:UIAlertControllerStyleAlert];
                    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel")
                                                                           style:UIAlertActionStyleCancel
                                                                         handler:^(UIAlertAction *action) {
                    }];
                    UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kReplace")
                                                                     style:UIAlertActionStyleDefault
                                                                   handler:^(UIAlertAction *action) {
                        [fileManager removeItemAtPath:pdfFilePath error:nil];
                        [self signDigitalSignature:sig signArea:rect signImagePath:imagePath savePath:pdfFilePath];
                    }];
                    [alertController addAction:cancelAction];
                    [alertController addAction:action];
                    [self->_pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
                });
                return;
            }
            [self signDigitalSignature:sig signArea:rect signImagePath:imagePath savePath:pdfFilePath];
        }
    }] fs_showByViewController:[self->_pdfViewCtrl fs_currentSceneTopMostViewController]];
}


- (void)sign {
    if (!_isAdded) {
        self.currentSelectSign = nil;
        _annotImage = nil;
        _isAdded = NO;
        
        CGRect newRect = [_pdfViewCtrl convertPdfRectToPageViewRect:self.signature.rect pageIndex:self.signature.pageIndex];
        newRect = CGRectInset(newRect, -30, -30);
        [_pdfViewCtrl refresh:newRect pageIndex:self.signature.pageIndex needRender:YES];
        return;
    }
    
    BOOL isSaveTip = [Preference getBoolValue:FSPreferenceModuleNameSignature type:@"SignSaveTip" delaultValue:NO];
    if (!isSaveTip) {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kConfirm") message:FSLocalizedForKey(@"kConfirmSign") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kNo")
                                                               style:UIAlertActionStyleCancel
                                                             handler:^(UIAlertAction *action) {
                                                                 [self deleteSign];
                                                             }];
        UIAlertAction *signAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kYes")
                                                             style:UIAlertActionStyleDefault
                                                           handler:^(UIAlertAction *action) {
                                                               [self addSign];
                                                               [Preference setBoolValue:FSPreferenceModuleNameSignature type:@"SignSaveTip" value:YES];
                                                           }];
        [alertController addAction:cancelAction];
        [alertController addAction:signAction];
        [_pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
    } else {
        [self addSign];
    }
}

- (void)addSign {
    FSSignature *sig = self.currentSelectSign;
    NSString *selectSig = [AnnotationSignature getSignatureSelected];
    AnnotationSignature *signnature = [AnnotationSignature getSignature:selectSig];
    if (signnature.certMD5 && signnature.certPasswd && signnature.certFileName) {
        NSString *imagePath = [SIGNATURE_PATH stringByAppendingPathComponent:[selectSig stringByAppendingString:@"_i"]];

        UIImage *tmpImage = [Utility scaleToSize:[UIImage imageWithContentsOfFile:imagePath] size:self.menuRect.size];
        NSString *tmpPath = [imagePath stringByAppendingString:@"_tmp"];
        [UIImagePNGRepresentation(tmpImage) writeToFile:tmpPath atomically:YES];
        
        // signArea useless
        [self addDigitalSign:sig signArea:self.editAnnot.fsrect signImagePath:tmpPath];
    }
    self.annotImage = nil;
    [self dynamicXFASignatureSelected];
}

- (void)signList {
    self.shouldShowMenu = NO;
    [_extensionsManager.viewCtrlManager presentViewController:self.signatureListCtr
                                     animated:YES
                                   completion:^{
                                       self.isShowList = YES;
                                   }];
    
    self.signatureListCtr.delegate = self;
}

#pragma mark <UIPopoverPresentationControllerDelegate>

- (void)popoverPresentationControllerDidDismissPopover:(UIPopoverPresentationController *)popoverPresentationController {
    self.isShowList = NO;
    _signatureListCtr = nil;
    if (self.signature) {
        self.shouldShowMenu = YES;
        [self showAnnotMenu];
    } else {
        [_extensionsManager setCurrentAnnot:nil];
    }
}

- (void)deleteSign {
    _isAdded = NO;
    self.annotImage = nil;
    self.signature = nil;
}

- (BOOL)onPageViewLongPress:(int)pageIndex recognizer:(UILongPressGestureRecognizer *)recognizer annot:(FSAnnot *)annot {
    return NO;
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer annot:(FSAnnot *)annot {
    if (_extensionsManager.currentAnnot == annot) {
        CGPoint point = [recognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];
        FSPointF *pdfPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
        if (pageIndex == annot.pageIndex && [self isHitAnnot:annot point:pdfPoint]) {
            if (self.shouldShowMenu)
                [self showAnnotMenu];
            return YES;
        } else {
            if (_isAdded) {
                [self deleteSign];
            } else {
                _annotImage = nil;
                _isAdded = NO;
                [_extensionsManager setCurrentAnnot:nil];
            }
            return YES;
        }
    } else {
        [_extensionsManager setCurrentAnnot:annot];
        return YES;
    }
    return NO;
}

- (BOOL)onPageViewPan:(int)pageIndex recognizer:(UIPanGestureRecognizer *)recognizer annot:(FSAnnot *)annot {
    return NO;
}

- (BOOL)onPageViewShouldBegin:(int)pageIndex recognizer:(UIGestureRecognizer *)gestureRecognizer annot:(FSAnnot *)annot {
    if ([_extensionsManager getAnnotHandlerByAnnot:annot] == self) {
        CGPoint point = [gestureRecognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];
        FSPointF *pdfPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
        if (pageIndex == annot.pageIndex && [self isHitAnnot:annot point:pdfPoint]) {
            return YES;
        }
    }
    return NO;
}

- (BOOL)onPageViewTouchesBegan:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event annot:(FSAnnot *)annot {
    return NO;
}

- (BOOL)onPageViewTouchesMoved:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event annot:(FSAnnot *)annot {
    return NO;
}

- (BOOL)onPageViewTouchesEnded:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event annot:(FSAnnot *)annot {
    return NO;
}

- (BOOL)onPageViewTouchesCancelled:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event annot:(FSAnnot *)annot {
    return NO;
}

- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context annot:(FSAnnot *)annot {
    if (_extensionsManager.currentAnnot == annot && pageIndex == annot.pageIndex && _isAdded) {
        if (self.annotImage) {
            CGRect rect = [_pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
            UIImage *image = [Utility scaleToSize:self.annotImage size:rect.size];
            CGContextSaveGState(context);
            
            CGContextTranslateCTM(context, rect.origin.x, rect.origin.y);
            CGContextTranslateCTM(context, 0, rect.size.height);
            CGContextScaleCTM(context, 1.0, -1.0);
            CGContextTranslateCTM(context, -rect.origin.x, -rect.origin.y);
            CGContextDrawImage(context, rect, [image CGImage]);
            
            CGContextRestoreGState(context);
        }
    }
}

- (void)onDocWillOpen {
    _signViewCtr = nil;
    self.signature = nil;
    _isShowList = NO;
    [self.digitalSignDic removeAllObjects];
}

#pragma mark IRotateChangedListener

- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration {
    [self dismissAnnotMenu];
    if (_isShowList) {
        [self.signatureListCtr dismissViewControllerAnimated:NO
                                                  completion:^{
                                                      self->_signatureListCtr = nil;
                                                  }];
    }
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation {
    [self showAnnotMenu];
    if (_isShowList) {
        [self signList];
    }
}

#pragma mark IScrollViewEventListener

- (void)onScrollViewWillBeginDragging:(UIScrollView *)dviewer {
    [self dismissAnnotMenu];
}

- (void)onScrollViewDidEndDragging:(UIScrollView *)dviewer willDecelerate:(BOOL)decelerate {
    if (!decelerate) {
//        [self showAnnotMenu];
    }
}

- (void)onScrollViewWillBeginDecelerating:(UIScrollView *)dviewer {
}

- (void)onScrollViewDidEndDecelerating:(UIScrollView *)dviewer {
//    [self showAnnotMenu];
}

- (void)onScrollViewWillBeginZooming:(UIScrollView *)dviewer {
    [self dismissAnnotMenu];
}

- (void)onScrollViewDidEndZooming:(UIScrollView *)dviewer {
    double delayInSeconds = .2;
    dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, (int64_t)(delayInSeconds * NSEC_PER_SEC));
    dispatch_after(popTime, dispatch_get_main_queue(), ^(void) {
        [self showAnnotMenu];
    });
}

- (void)showAnnotMenu {
    if (self.shouldShowMenu) {
        [_extensionsManager.menuControl setRect:self.menuRect];
        [_extensionsManager.menuControl showMenu];
    }
}

- (void)dismissAnnotMenu {
    if (_extensionsManager.menuControl.isMenuVisible) {
        [_extensionsManager.menuControl setMenuVisible:NO animated:YES];
    }
}

#pragma mark SignatureListDelegate

- (void)signatureListViewController:(SignatureListViewController *)signatureListViewController openSignature:(AnnotationSignature *)signature {
    self.isShowList = NO;
    void (^completion)(void) = ^{
        self->_signatureListCtr = nil;
        self.isShowList = NO;
        
        SignatureViewController *signatureCtr = [[SignatureViewController alloc] initWithUIExtensionsManager:self->_extensionsManager];
        self.signViewCtr = signatureCtr;
        [signatureCtr fs_setCustomTransitionAnimator:self->_extensionsManager.presentedVCAnimator];
        signatureCtr.currentSignature = signature;
        signatureCtr.isFieldSig = YES;
        DynamicXFASignatureAnnotHandler *__weak weakSelf = self;
        signatureCtr.saveHandler = ^{
            NSString *selectSig = [AnnotationSignature getSignatureSelected];
            AnnotationSignature *signnature = [AnnotationSignature getSignature:selectSig];
            DynamicXFASignatureAnnotHandler *strongSelf = weakSelf;
            assert(strongSelf);
            strongSelf->_isAdded = YES;
            strongSelf.signature = signnature;
            strongSelf.annotImage = [AnnotationSignature getSignatureImage:signnature.name];
            
//            CGRect newRect = [self->_pdfViewCtrl convertPdfRectToPageViewRect:strongSelf.editAnnot.fsrect pageIndex:strongSelf.editAnnot.pageIndex];
//            newRect = CGRectInset(newRect, -30, -30);
//            [strongSelf->_pdfViewCtrl refresh:newRect pageIndex:self.editAnnot.pageIndex needRender:YES];
            strongSelf.shouldShowMenu = YES;
            [strongSelf showAnnotMenu];
        };
        signatureCtr.cancelHandler = ^{
            DynamicXFASignatureAnnotHandler *strongSelf = weakSelf;
            assert(strongSelf);
            if (strongSelf.signature) {
                strongSelf.shouldShowMenu = YES;
                [strongSelf showAnnotMenu];
            } else {
                [strongSelf->_extensionsManager setCurrentAnnot:nil];
            }
        };
        UIViewController *rootViewController = [_extensionsManager fs_currentSceneTopMostViewController];
        [rootViewController presentViewController:signatureCtr
                                         animated:NO
                                       completion:^{
                                           
                                       }];
        
    };
    if (signatureListViewController.presentingViewController) {
        [signatureListViewController dismissViewControllerAnimated:YES
                                                        completion:completion];
    }else{
        completion();
    }
}

- (void)signatureListViewController:(SignatureListViewController *)signatureListViewController deleteSignature:(AnnotationSignature *)signature {
    if ([self.signature.name isEqualToString:signature.name]) {
        [self deleteSign];
        if ([AnnotationSignature getCertSignatureList].count < 1) {
            [signatureListViewController dismissViewControllerAnimated:YES
                                                            completion:^{
                                                                self->_signatureListCtr = nil;
                                                                self.isShowList = NO;
                                                            }];
        }
    }
}

- (void)signatureListViewController:(SignatureListViewController *)signatureListViewController selectSignature:(AnnotationSignature *)signature {
    NSString *selectSig = [AnnotationSignature getSignatureSelected];
    AnnotationSignature *signnature = [AnnotationSignature getSignature:selectSig];
    if (signnature.certMD5 && signnature.certPasswd && signnature.certFileName) {
        _isAdded = YES;
        self.signature = signnature;
        self.annotImage = [AnnotationSignature getSignatureImage:signnature.name];
    }
    self.isShowList = NO;
    self.shouldShowMenu = YES;
    [self showAnnotMenu];
}

- (void)cancelSignature {
    _signatureListCtr = nil;
    if (self.signature) {
        self.shouldShowMenu = YES;
        [self showAnnotMenu];
    } else {
        [_extensionsManager setCurrentAnnot:nil];
    }
    self.isShowList = NO;
}

@end
