/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "UIFSDocProviderCallback.h"
#import "FSFileAndImagePicker.h"

@interface UIFSDocProviderCallback ()<FSFileAndImagePickerDelegate>
@property (nonatomic, strong) FSFileAndImagePicker *mediaPicker;
@end
@implementation UIFSDocProviderCallback
- (id)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager {
    if (self = [super init]) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        self.mediaPicker = [[FSFileAndImagePicker alloc] init];
    }
    return self;
}

-(void)invalidateRect:(int)page_index rect:(FSRectF*)rect flag:(FSDocProviderCallbackInvalidateFlag)flag{
    CGRect newRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:rect pageIndex:page_index];
    newRect = CGRectInset(newRect, -5, -5);
    [self.pdfViewCtrl refresh:newRect pageIndex:page_index needRender:YES];
}

-(void)displayCaret:(int)page_index is_visible:(BOOL)is_visible rect:(FSRectF*)rect{
    DynamicXFAHandler *annotHandler = nil;
    annotHandler = (DynamicXFAHandler *)[_extensionsManager getDynamicAnnotHandler];
    if (is_visible) {
        annotHandler.displayCaretRect = [[FSRectF alloc] initWithLeft1:[rect getLeft] bottom1:[rect getBottom] right1:[rect getRight] top1:[rect getTop]];
    }else{
        annotHandler.displayCaretRect = nil;
    }
}

-(BOOL)getPopupPos:(int)page_index min_popup:(int)min_popup max_popup:(int)max_popup rect_widget:(FSRectF*)rect_widget inout_rect_popup:(FSRectF*)inout_rect_popup {
    CGRect pageViewRect = [_pdfViewCtrl getPageView:page_index].frame;
    FSRectF *newRect = [_pdfViewCtrl convertPageViewRectToPdfRect:pageViewRect pageIndex:page_index];
    
    BOOL ret = NO;
    inout_rect_popup.left = 0;
    if (rect_widget.right > newRect.right) {
        inout_rect_popup.left -= rect_widget.right - newRect.right;
        inout_rect_popup.right -= rect_widget.right - newRect.right;
    }else if (fabs( newRect.bottom - rect_widget.bottom) >= max_popup) {
        inout_rect_popup.top = fabs(rect_widget.bottom - rect_widget.top);
        inout_rect_popup.bottom = inout_rect_popup.top - max_popup;
        ret = YES;
    }else if (rect_widget.top - newRect.top >= max_popup) {
        inout_rect_popup.top = -max_popup;
        inout_rect_popup.bottom = inout_rect_popup.top - max_popup;
        ret = YES;
    }else if (newRect.bottom - rect_widget.bottom >= min_popup) {
        inout_rect_popup.top = fabs(rect_widget.top - rect_widget.bottom);
        inout_rect_popup.bottom = inout_rect_popup.top - (newRect.bottom - rect_widget.bottom);
        ret = YES;
    }else if (rect_widget.top - newRect.top >= min_popup) {
        inout_rect_popup.top = -(inout_rect_popup.top - newRect.top);
        inout_rect_popup.bottom = inout_rect_popup.top - (rect_widget.top - newRect.top);
        ret = YES;
    }else{
        int top_length = 0;
        int bottom_length = 0;
        int actaul_length = ABS(inout_rect_popup.top - inout_rect_popup.bottom);
        bool show_upside = YES;
        float fPoupHeight = 0;
        top_length = (int)(newRect.top - rect_widget.top);
        bottom_length = (int)(rect_widget.bottom - newRect.bottom);

        if (rect_widget.right > newRect.right)
        {
            inout_rect_popup.left -= rect_widget.right - newRect.right;
            inout_rect_popup.right -= rect_widget.right - newRect.right;
        }
        if (top_length <= 0 && bottom_length <= 0)
        {
            return NO;
        }

        if(bottom_length <= 0)
        {
            show_upside = NO;
        }
        else if (top_length > bottom_length )
        {
            show_upside = NO;
        }

        if (actaul_length<min_popup)
        {
            fPoupHeight = min_popup;
        }
        else if (actaul_length > max_popup)
        {
            fPoupHeight = max_popup;
        }
        else
        {
            fPoupHeight = (float)actaul_length;
        }

        if (!show_upside)
        {
            inout_rect_popup.top = fabs(rect_widget.bottom - rect_widget.top);
            inout_rect_popup.bottom = inout_rect_popup.top + fPoupHeight;
        }
        else
        {
            inout_rect_popup.top = -fPoupHeight;
            inout_rect_popup.bottom = inout_rect_popup.top + fPoupHeight;
        }
        ret = YES;
    }
    return ret;
}
-(BOOL)popupMenu:(int)page_index rect_popup:(FSPointF*)rect_popup{
    return YES;
}
-(int)getCurrentPage:(FSXFADoc*) doc{
    return [_pdfViewCtrl getCurrentPage];
}

-(void)setCurrentPage:(FSXFADoc*) doc current_page_index:(int)current_page_index{
    if ([NSThread isMainThread]) {
        [_pdfViewCtrl gotoPage:current_page_index animated:NO];
    }else{
        dispatch_semaphore_t wait = dispatch_semaphore_create(0);
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.pdfViewCtrl gotoPage:current_page_index animated:NO];
            dispatch_semaphore_signal(wait);
        });
        dispatch_semaphore_wait(wait, DISPATCH_TIME_FOREVER);
    }
}
-(void)setChangeMark:(FSXFADoc*) doc{
    _extensionsManager.isDocModified = YES;
}
-(NSString*)getTitle:(FSXFADoc*) doc{
    return @"";
}
-(void)setFocus:(FSXFAWidget *) xfa_widget{
    if (xfa_widget == nil || [xfa_widget isEmpty]) return;
    DynamicXFAHandler *annotHandler = (DynamicXFAHandler *)[_extensionsManager getDynamicAnnotHandler];
    
    int focusPageIndex = [[xfa_widget getXFAPage] getIndex];

    if (focusPageIndex != self.extensionsManager.currentPageIndex) {
        [_pdfViewCtrl gotoPage:focusPageIndex animated:NO];
        annotHandler.handlerCurrentWidget = nil;
        [annotHandler.formNaviBar setHidden:YES];
        
        FSRectF *rect = [xfa_widget getRect];
        FSPointF *point = [[FSPointF alloc] initWithX:rect.left Y:rect.bottom];
        @try {
            [xfa_widget onLButtonUp:point flags:0];
        } @catch (NSException *exception) {
        }
    }
    
    if (xfa_widget.getType == FSXFAWidgetWidgetTypeImageEdit) {
        [self showMenu:xfa_widget];
    }
    
    // if handlerCurrentWidget = nil , mean click done button or click space place
    // if handlerCurrentWidget != nil, mean click other widget
    
    BOOL isCurrentWidgetEmpty = (annotHandler.handlerCurrentWidget == nil && self.extensionsManager.currentWidget == nil);
    BOOL isSameWidget = (annotHandler.handlerCurrentWidget != nil && self.extensionsManager.currentWidget != nil
                         && ![annotHandler compareTwoWidgetIsEqualTo:self.extensionsManager.currentWidget compareWidget:annotHandler.handlerCurrentWidget]);
    if ((isCurrentWidgetEmpty || isSameWidget) && [GlableInstance shareInstance].xfaInputError) {
        annotHandler.isInputError = YES;
        annotHandler.isOver = NO;
        annotHandler.handlerCurrentWidget = xfa_widget;
        [self.extensionsManager setCurrentWidget:xfa_widget];
        [annotHandler gotoWidgetOnError:xfa_widget];
        [GlableInstance shareInstance].xfaInputError = NO;
    }
}
-(void)exportData:(FSXFADoc*)doc file_path:(NSString*) file_path{
    if (!([_extensionsManager.permissionProvider checkPermission:FSFunctionCanSaveToLocal] == FSPermissionStateShow)) {
        return;
    }
}
-(void)importData:(FSXFADoc*)doc file_path:(NSString*) file_path{
}
-(void)gotoURL:(FSXFADoc*)doc url:(NSString*) url{
}
-(void)print:(FSXFADoc*)doc start_page_index:(int)start_page_index end_page_index:(int)end_page_index options:(int)options{
    UIPrintInteractionCompletionHandler completion = ^(UIPrintInteractionController *_Nonnull printInteractionController, BOOL completed, NSError *_Nullable error) {
        if (error) {
            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(error.localizedDescription) preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
            [alertController addAction:action];
            [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
        }
    };
    NSString *fileName = self.pdfViewCtrl.filePath.lastPathComponent;
    if ([self.pdfViewCtrl isDynamicXFA]) {
        [Utility printXFADoc:doc animated:YES jobName:fileName delegate:nil completionHandler:completion];
    }
}
-(int)getHighlightColor:(FSXFADoc*)doc{
    return self.extensionsManager.highlightFormColor.fs_argbValue;
}
-(BOOL)submitData:(FSXFADoc*)doc target:(NSString*)target format:(FSDocProviderCallbackSubmitFormat)format text_encoding:(FSDocProviderCallbackTextEncoding)text_encoding content:(NSString*)content{
    return YES;
}
-(void)pageViewEvent:(int)page_index page_view_event_type:(FSDocProviderCallbackPageViewEventType)page_view_event_type{
    if (page_view_event_type == FSDocProviderCallbackPageViewEventTypeRemoved) {
        page_index = page_index == -1 ? [[self.pdfViewCtrl getXFADoc] getPageCount] : page_index;
        [self.pdfViewCtrl reloadXFADoc:page_view_event_type page_index:@[@(page_index)]];
    }
    
    if (page_view_event_type == FSDocProviderCallbackPageViewEventTypeAdded) {
        page_index = page_index == -1 ? [[self.pdfViewCtrl getXFADoc] getPageCount] : page_index;
        [self.pdfViewCtrl reloadXFADoc:page_view_event_type page_index:@[@(page_index)]];
    }
}
-(void)widgetEvent:(FSXFAWidget *) xfa_widget widget_event_type:(FSDocProviderCallbackWidgetEventType)widget_event_type {
    if (widget_event_type == FSDocProviderCallbackWidgetEventTypeBeforeRemoved) {
        FSXFADoc *xfadoc = [self.pdfViewCtrl getXFADoc];
        DynamicXFAHandler *annotHandler = nil;
        annotHandler = (DynamicXFAHandler *)[_extensionsManager getDynamicAnnotHandler];
        if ([annotHandler compareTwoWidgetIsEqualTo:annotHandler.handlerCurrentWidget compareWidget:xfa_widget]) {
            annotHandler.handlerCurrentWidget = nil;
            [_extensionsManager setCurrentWidget:nil];
            
            @try {
                [xfadoc killFocus];
            } @catch (NSException *exception) {
            }
        }
    }
}

- (void)showMenu:(FSXFAWidget *)xfaWidget{
    if (xfaWidget == nil || [xfaWidget isEmpty]) return;
    int pageIndex = xfaWidget.pageIndex;
    FSPDFViewCtrl *pdfViewCtrl = self.extensionsManager.pdfViewCtrl;
    CGRect dvRect = [self.extensionsManager.pdfViewCtrl convertPageViewRectToDisplayViewRect:[pdfViewCtrl convertPdfRectToPageViewRect:[xfaWidget getRect] pageIndex:pageIndex] pageIndex:pageIndex];
    
    self.mediaPicker.extensionsManager = _extensionsManager;
    self.mediaPicker.fileType = FSMediaFileTypeImage;
    self.mediaPicker.delegate = self;
    [self.mediaPicker showMenu:dvRect];
    self.focusXFAWidget = xfaWidget;
}

#pragma mark <FSFileAndImagePickerDelegate>
- (void)fileAndImagePicker:(FSFileAndImagePicker *)fileAndImagePicker didPickImage:(UIImage *)image imagePath:(NSString *)imagePath{
    
    if (self.focusXFAWidget == nil || [self.focusXFAWidget isEmpty]) return;
    FSImage *fs_image;
    if (image) {
        fs_image = [Utility uiimageToFSImage:image];
    }else{
        fs_image = [[FSImage alloc] initWithPath:imagePath];
    }
    [self.focusXFAWidget setImage:fs_image];
    FSPDFViewCtrl *pdfViewCtrl = self.extensionsManager.pdfViewCtrl;
    
    self.extensionsManager.isDocModified= YES;
    
    int pageIndex = self.focusXFAWidget.pageIndex;
    CGRect refreshRect = [pdfViewCtrl convertPdfRectToPageViewRect:[self.focusXFAWidget getRect] pageIndex:pageIndex];
    refreshRect = CGRectInset(refreshRect, -30, -30);
    [self.extensionsManager.pdfViewCtrl refresh:refreshRect pageIndex:pageIndex];
    
    [self.dynamicXFAHandler setFocus:nil isHidden:YES];
    self.dynamicXFAHandler.handlerCurrentWidget = nil;
    self.focusXFAWidget = nil;
}
- (void)fileAndImagePickerDidCancel:(FSFileAndImagePicker *)fileAndImagePicker {
    [self completed];
}

- (void)completed{
}
@end
