/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSSaveAsViewController.h"
#import "FSDataItem.h"

@interface FSSaveAsDataItem : FSDataItem
@property (nonatomic, copy) NSString *title;
@property (nonatomic, strong) id value;
@property (nonatomic, weak) NSIndexPath *indexPath;
@end

@implementation FSSaveAsDataItem @end

@interface FSSaveAsViewController ()
@property (nonatomic, weak) UITextField *fileNameTextField;
@property (nonatomic, weak) FSSaveAsDataItem *currentSelectedItem;

@property (nonatomic, strong) FSViewControllerManager *services;
@end

@implementation FSSaveAsViewController
@dynamic services;

- (void)initialize{
    [super initialize];
    self.defaultEstimatedCellStyle = FSEstimatedTableViewCellStyleTitle;
    self.sectionStyle = FSTableViewSectionStyleLikeGrouped;
    self.shouldLoadDataOnViewDidLoad = YES;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    self.title = FSLocalizedForKey(@"kSaveAs");
    
    self.navBarLeftTitle = FSLocalizedForKey(@"kCancel");
    self.navBarRightTitle = FSLocalizedForKey(@"kSave");
    
    self.tableView.backgroundColor = self.fs_backgroundColor_B9;
    
    @weakify(self)
    self.navBar.rightItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        FSFileSelectDestinationViewController *selectDestination = [[FSFileSelectDestinationViewController alloc] initWithServices:self.services];
        selectDestination.isRootFileDirectory = YES;
        selectDestination.fileSelectDestinationType = FSFileSelectDestinationTypeSelectFolder;
        selectDestination.expectFileType = @[@"*"];
        selectDestination.titleForTypeSelectFolder = FSLocalizedForKey(@"kSaveas");
        [selectDestination loadFilesWithPath:DOCUMENT_PATH];
        selectDestination.operatingHandler = ^(FSFileSelectDestinationViewController *controller, NSArray *destinationFolder) {
            [controller.services dismissViewControllerAnimated:YES completion:^{
                if (destinationFolder.count == 0)
                    return;
    
                __block void (^inputFileName)(NSString *path) = ^(NSString *path) {
                    BOOL isDir = NO;
                    NSFileManager *fileManager = [NSFileManager defaultManager];
                    if (![fileManager fileExistsAtPath:path isDirectory:&isDir]) {
                        return;
                    }
                    if (!isDir) {
                        dispatch_async(dispatch_get_main_queue(), ^{
                            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kFileAlreadyExists") preferredStyle:UIAlertControllerStyleAlert];
                            UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                                                   style:UIAlertActionStyleCancel
                                                                                 handler:^(UIAlertAction *action) {
                                                                                     dispatch_async(dispatch_get_main_queue(), ^{
                                                                                         inputFileName([path stringByDeletingLastPathComponent]);
                                                                                     });
                                                                                 }];
                            [alertController addAction:cancelAction];
                            [self presentViewController:alertController animated:YES completion:nil];
                        });
                        return;
                    }
    
                    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:nil message:FSLocalizedForKey(@"kInputNewFileName") preferredStyle:UIAlertControllerStyleAlert];
                    [alertController addTextFieldWithConfigurationHandler:^(UITextField *_Nonnull textField) {
                        textField.text = @"";
                    }];
                    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel") style:UIAlertActionStyleCancel handler:nil];
                    UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                                     style:UIAlertActionStyleDefault
                                                                   handler:^(UIAlertAction *action) {
                                                                       UITextField *textField = alertController.textFields[0];
                                                                       NSString *newName = textField.text;
                                                                       if (newName.length < 1) {
                                                                           UIAlertController *alertController = [UIAlertController alertControllerWithTitle:nil message:FSLocalizedForKey(@"kInputNewFileName") preferredStyle:UIAlertControllerStyleAlert];
                                                                           UIAlertAction *okAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleCancel handler:^(UIAlertAction *action) {
                                                                               dispatch_async(dispatch_get_main_queue(), ^{
                                                                                   inputFileName(path);
                                                                               });
                                                                           }];
                                                                           [alertController addAction:okAction];
                                                                           [self presentViewController:alertController animated:YES completion:nil];
                                                                       } else if ([fileManager fileExistsAtPath:[path stringByAppendingPathComponent:[NSString stringWithFormat:@"%@.pdf", newName]]]) {
                                                                           dispatch_async(dispatch_get_main_queue(), ^{
                                                                               UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kFileAlreadyExists") preferredStyle:UIAlertControllerStyleAlert];
                                                                               UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel")
                                                                                                                                      style:UIAlertActionStyleCancel
                                                                                                                                    handler:^(UIAlertAction *action) {
                                                                                                                                    }];
                                                                               UIAlertAction *OKAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                                                                                                      style:UIAlertActionStyleDefault
                                                                                                                                    handler:^(UIAlertAction *action) {
                                                                                                                                        dispatch_async(dispatch_get_main_queue(), ^{
                                                                                                                                            [fileManager removeItemAtPath:[path stringByAppendingPathComponent:[NSString stringWithFormat:@"%@.pdf", newName]] error:nil];
                                                                                                                                            !self.commonBlock_id ? : self.commonBlock_id([path stringByAppendingString:[NSString stringWithFormat:@"/%@.pdf", newName]]);
                                                                                                                                            inputFileName = nil;
                                                                                                                                        });
                                                                                                                                    }];
                                                                               [alertController addAction:cancelAction];
                                                                               [alertController addAction:OKAction];
                                                                               [self presentViewController:alertController animated:YES completion:nil];
                                                                           });
                                                                       } else {
                                                                           !self.commonBlock_id ? : self.commonBlock_id([path stringByAppendingString:[NSString stringWithFormat:@"/%@.pdf", newName]]);
                                                                           inputFileName = nil;
                                                                       }
                                                                   }];
                    [alertController addAction:cancelAction];
                    [alertController addAction:action];
                    [self presentViewController:alertController animated:YES completion:nil];
                };
    
                inputFileName(destinationFolder[0]);
            }];
    
        };
        [self.services presentViewController:selectDestination animated:YES completion:nil];
    };
}

- (void)loadData{
    NSArray *titleSections = @[ @[@""], @[FSLocalizedForKey(@"kDevice"), FSLocalizedForKey(@"kCloud")]];
    NSArray *defaultValueSections = @[ @[@""], @[@(YES), @(NO)]];
    NSMutableArray *tmp = @[].mutableCopy;
    for (int i = 0 ; i < titleSections.count; i++) {
        NSMutableArray *section = @[].mutableCopy;
        NSArray *titles = titleSections[i];
        NSArray *defaultValue = defaultValueSections[i];
        for (int j = 0; j < titles.count; j ++ ) {
            FSSaveAsDataItem *item = [FSSaveAsDataItem new];
            item.title = titles[j];
            item.value = defaultValue[j];
            [section addObject:item];
        }
        [tmp addObject:section];
    }
    self.dataSource = tmp.copy;
}

- (NSString *)titles1ForAtSection:(NSUInteger)section{
    if (section == 0) {
        return FSLocalizedForKey(@"kFileName");
    }
    return FSLocalizedForKey(@"kPlaces");
}

- (void)configureCell:(FSTitleEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(FSSaveAsDataItem *)dataItem{
    if (!cell.installed) {
        cell.rightMargin = ITEM_MARGIN_NORMAL;
        if (indexPath.section == 0) {
            cell.estimatedTitleLabel.hidden = YES;
            UITextField *fileNameTextField = [UITextField new];
            fileNameTextField.font = self.fs_standardGeneralFont_GF3;
            fileNameTextField.textColor = self.fs_textColor_T4;
            fileNameTextField.textAlignment = NSTextAlignmentLeft;
            fileNameTextField.borderStyle = UITextBorderStyleNone;
            [cell.contentView addSubview:fileNameTextField];
            self.fileNameTextField = fileNameTextField;
            [fileNameTextField mas_makeConstraints:^(MASConstraintMaker *make) {
                make.left.fs_equalToSuperview().offset(ITEM_MARGIN_NORMAL);
                make.right.fs_equalToSuperview().offset(-ITEM_MARGIN_NORMAL);
                make.centerY.fs_equalToSuperview();
            }];
        }
    }
    if (indexPath.section == 1) {
        if (!cell.rightView) {
            UIImageView *imageView = [UIImageView new];
            [BindThemeKeySel(imageView, setImage:) fs_updateThemeCompleted:^(UIImageView * _Nonnull target, SEL  _Nonnull sel) {
                target.image = FSImageNamedAndPrimaryColor(@"common_select_check");
            }];
            imageView.hidden = YES;
            cell.rightView = imageView;
        }
    }
    if (indexPath.section == 0) {
        self.fileNameTextField.text = dataItem.value;
    }else {
        cell.rightView.hidden = ![dataItem.value boolValue];
        if (indexPath.row == 0) self.currentSelectedItem = dataItem;
    }
    dataItem.indexPath = indexPath;
    cell.estimatedTitleLabel.text = dataItem.title;
    cell.contentView.backgroundColor = self.fs_backgroundColor_B8;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{
    if (indexPath.section == 0) {
        [self.fileNameTextField becomeFirstResponder];
    }else if (indexPath.section == 1){
        
    }else if (indexPath.section == 2){

        if (self.currentSelectedItem.indexPath != indexPath) {
            
            ((FSTitleEstimatedTableViewCell *)[tableView cellForRowAtIndexPath:self.currentSelectedItem.indexPath]).rightView.hidden = YES;
            self.currentSelectedItem.value = @(NO);
            
            FSSaveAsDataItem *dataItem = [self dataForAtIndexPath:indexPath];
            self.currentSelectedItem = dataItem;
            
            self.currentSelectedItem.value = @(YES);
            ((FSTitleEstimatedTableViewCell *)[tableView cellForRowAtIndexPath:self.currentSelectedItem.indexPath]).rightView.hidden = NO;

        }
    }
}

- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section
{
   UIView *headerView = [super tableView:tableView viewForHeaderInSection:section];
   headerView.backgroundColor = self.fs_backgroundColor_B9;
   return headerView;
}
/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
