/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSFillSignUndoItem.h"

@implementation FSFillSignObjAttributes

+ (instancetype)attributesWithFillSignObject:(FSFillSignObject *)fillSignObject{
    Class attributesClass = nil;
    switch ([fillSignObject getType]) {
    case FSFillSignFillSignObjectTypeText:
        attributesClass = [FSTextFillSignObjAttributes class];
        break;
   default:
        attributesClass = [FSFillSignObjAttributes class];
        break;
    }
    return [[attributesClass alloc] initWithFillSignObject:fillSignObject];
}
- (instancetype)initWithFillSignObject:(FSFillSignObject *)fillSignObject{
    if (self = [super init]) {
        self.type = fillSignObject.getType;
    }
    return self;
}
- (void)resetFillSignObject:(FSFillSignObject *)fillSignObject{
//    FSPointF *pointf = [[FSPointF alloc] initWithX:self.rect.left Y:self.rect.bottom];
//    [fillSignObject move:pointf width:ABS(self.rect.left - self.rect.right) height:ABS(self.rect.top - self.rect.bottom)];
}
- (BOOL)isEqualToAttributes:(FSFillSignObjAttributes *)attributes{
    return self.type == attributes.type && CGRectEqualToRect(self.rect, attributes.rect);
}

@end

@implementation FSTextFillSignObjAttributes

- (instancetype)initWithFillSignObject:(FSFillSignObject *)fillSignObject{
    if (self = [super initWithFillSignObject:fillSignObject]) {
        FSTextFillSignObject *textFillSign = [[FSTextFillSignObject alloc] initWithFillsign_object:fillSignObject];
        self.textDataArray = textFillSign.getTextDataArray;
        self.isSegText = [textFillSign isCombFieldMode];
        self.isTextFillSign = YES;
    }
    return self;
}
- (void)resetFillSignObject:(FSFillSignObject *)fillSignObject{
    [super resetFillSignObject:fillSignObject];
    if ([self.textDataArray getSize]) {
//        FSTextFillSignObject *textFillSign = [[FSTextFillSignObject alloc] initWithFillsign_object:fillSignObject];
//        textFillSign.getTextDataArray = self.textDataArray;
    }
}
- (BOOL)isEqualToAttributes:(FSTextFillSignObjAttributes *)attributes{
    return [super isEqualToAttributes:attributes] && self.textDataArray == attributes.textDataArray;
}

@end

@interface FSFillSignUndoItem ()

@property (nonatomic) void *oldFormXObjCptr;
- (void)updateFormXObjCptr:(void *)formXObj;
@end

@implementation FSFillSignUndoItem

+ (instancetype)itemForUndoModifyFillSignWithExtensionsManager:(UIExtensionsManager *)extensionsManager page:(FSPDFPage *)page{
    FSFillSignUndoItem *undoItem = [super itemWithUndo:^(UndoItem *item) {
        FSFillSignUndoItem *fillSignItem = (FSFillSignUndoItem *)item;
        [fillSignItem undoRedoCheckHasNullFormXObjCptr];
        if (!fillSignItem.formXObjCptr) return;
        [item.extensionsManager.fillSignModule.fillSignToolHandler deactiveFillSign];
        
        FSFillSign *fillSign = [[FSFillSign alloc] initWithPage:page];
        
        FSFillSignObject *fillSignObj = [fillSign getObjectByFormXObject:[[FSFormXObject alloc] initWithCptr:fillSignItem.formXObjCptr swigOwnCObject:NO]];
        [item.extensionsManager.pdfViewCtrl lockRefresh];
        [fillSign removeObject:fillSignObj];
        
        FSPointF *pointf = [[FSPointF alloc] initWithX:CGRectGetMinX(fillSignItem.oldFillSignObjAttributes.rect) Y:CGRectGetMinY(fillSignItem.oldFillSignObjAttributes.rect)];
        if (fillSignItem.oldFillSignObjAttributes.isTextFillSign) {
            FSTextFillSignObjAttributes *textFillSignAttributes = (FSTextFillSignObjAttributes *)fillSignItem.oldFillSignObjAttributes;
            fillSignObj = [fillSign addTextObject:textFillSignAttributes.textDataArray point:pointf width:CGRectGetWidth(fillSignItem.oldFillSignObjAttributes.rect) height:CGRectGetHeight(fillSignItem.oldFillSignObjAttributes.rect) rotation:[fillSignItem rotationWithPage:page] is_comb_field_mode:textFillSignAttributes.isSegText];
        }else{
            fillSignObj = [fillSign addObject:fillSignItem.oldFillSignObjAttributes.type point:pointf width:CGRectGetWidth(fillSignItem.oldFillSignObjAttributes.rect) height:CGRectGetHeight(fillSignItem.oldFillSignObjAttributes.rect) rotation:[fillSignItem rotationWithPage:page]];
        }
        
        [fillSignObj generateContent];
        [fillSignItem updateFormXObjCptr:[[fillSignObj getFormXObject] getCptr]];
        [item.extensionsManager.pdfViewCtrl unlockRefresh];
        
        [item.extensionsManager.pdfViewCtrl refresh:CGRectZero pageIndex:page.getIndex needRender:YES clearCache:YES];
    }redo:^(UndoItem *item) {
        FSFillSignUndoItem *fillSignItem = (FSFillSignUndoItem *)item;
        [fillSignItem undoRedoCheckHasNullFormXObjCptr];
        if (!fillSignItem.formXObjCptr) return;
        [item.extensionsManager.fillSignModule.fillSignToolHandler deactiveFillSign];

        FSFillSign *fillSign = [[FSFillSign alloc] initWithPage:page];
        
        FSFillSignObject *fillSignObj = [fillSign getObjectByFormXObject:[[FSFormXObject alloc] initWithCptr:fillSignItem.formXObjCptr swigOwnCObject:NO]];
        [item.extensionsManager.pdfViewCtrl lockRefresh];
        [fillSign removeObject:fillSignObj];
        
        FSPointF *pointf = [[FSPointF alloc] initWithX:CGRectGetMinX(fillSignItem.fillSignObjAttributes.rect) Y:CGRectGetMinY(fillSignItem.fillSignObjAttributes.rect)];
        if (fillSignItem.fillSignObjAttributes.isTextFillSign) {
            FSTextFillSignObjAttributes *textFillSignAttributes = (FSTextFillSignObjAttributes *)fillSignItem.fillSignObjAttributes;
            fillSignObj = [fillSign addTextObject:textFillSignAttributes.textDataArray point:pointf width:CGRectGetWidth(fillSignItem.fillSignObjAttributes.rect) height:CGRectGetHeight(fillSignItem.fillSignObjAttributes.rect) rotation:[fillSignItem rotationWithPage:page] is_comb_field_mode:textFillSignAttributes.isSegText];
        }else{
            fillSignObj = [fillSign addObject:fillSignItem.fillSignObjAttributes.type point:pointf width:CGRectGetWidth(fillSignItem.fillSignObjAttributes.rect) height:CGRectGetHeight(fillSignItem.fillSignObjAttributes.rect) rotation:[fillSignItem rotationWithPage:page]];
        }
        
        [fillSignObj generateContent];
        [fillSignItem updateFormXObjCptr:[[fillSignObj getFormXObject] getCptr]];
        [item.extensionsManager.pdfViewCtrl unlockRefresh];
        
        [item.extensionsManager.pdfViewCtrl refresh:CGRectZero pageIndex:page.getIndex needRender:YES clearCache:YES];
    } pageIndex:page.getIndex];
    undoItem.extensionsManager = extensionsManager;
    return undoItem;
}

+ (instancetype)itemForUndoAddFillSignWithExtensionsManager:(UIExtensionsManager *)extensionsManager page:(FSPDFPage *)page{
    FSFillSignUndoItem *undoItem = [super itemWithUndo:^(UndoItem *item) {
        FSFillSignUndoItem *fillSignItem = (FSFillSignUndoItem *)item;
        [fillSignItem undoRedoCheckHasNullFormXObjCptr];
        if (!fillSignItem.formXObjCptr) return;
        [item.extensionsManager.fillSignModule.fillSignToolHandler deactiveFillSign];
        
//        FSPDFPage *page = [[item.extensionsManager.pdfViewCtrl currentDoc] getParsedPage:pageIndex];
        FSFillSign *fillSign = [[FSFillSign alloc] initWithPage:page];
        
        FSFillSignObject *fillSignObj = [fillSign getObjectByFormXObject:[[FSFormXObject alloc] initWithCptr:fillSignItem.formXObjCptr swigOwnCObject:NO]];
        [item.extensionsManager.pdfViewCtrl lockRefresh];
        [fillSign removeObject:fillSignObj];
        [item.extensionsManager.pdfViewCtrl unlockRefresh];
        
        [item.extensionsManager.pdfViewCtrl refresh:CGRectZero pageIndex:page.getIndex needRender:YES clearCache:YES];
    }redo:^(UndoItem *item) {
        FSFillSignUndoItem *fillSignItem = (FSFillSignUndoItem *)item;
        [fillSignItem undoRedoCheckHasNullFormXObjCptr];
        [item.extensionsManager.fillSignModule.fillSignToolHandler deactiveFillSign];
        
//        FSPDFPage *page = [[item.extensionsManager.pdfViewCtrl currentDoc] getParsedPage:pageIndex];
        FSFillSign *fillSign = [[FSFillSign alloc] initWithPage:page];
        
        [item.extensionsManager.pdfViewCtrl lockRefresh];
        FSPointF *pointf = [[FSPointF alloc] initWithX:CGRectGetMinX(fillSignItem.fillSignObjAttributes.rect) Y:CGRectGetMinY(fillSignItem.fillSignObjAttributes.rect)];
        FSFillSignObject *fillSignObj;
        if (fillSignItem.fillSignObjAttributes.isTextFillSign) {
            FSTextFillSignObjAttributes *textFillSignAttributes = (FSTextFillSignObjAttributes *)fillSignItem.fillSignObjAttributes;
            fillSignObj = [fillSign addTextObject:textFillSignAttributes.textDataArray point:pointf width:CGRectGetWidth(fillSignItem.fillSignObjAttributes.rect) height:CGRectGetHeight(fillSignItem.fillSignObjAttributes.rect) rotation:[fillSignItem rotationWithPage:page] is_comb_field_mode:textFillSignAttributes.isSegText];
        }else{
            fillSignObj = [fillSign addObject:fillSignItem.fillSignObjAttributes.type point:pointf width:CGRectGetWidth(fillSignItem.fillSignObjAttributes.rect) height:CGRectGetHeight(fillSignItem.fillSignObjAttributes.rect) rotation:[fillSignItem rotationWithPage:page]];
        }
        
        [fillSignObj generateContent];
        [fillSignItem updateFormXObjCptr:[[fillSignObj getFormXObject] getCptr]];
        [item.extensionsManager.pdfViewCtrl unlockRefresh];
        [item.extensionsManager.pdfViewCtrl refresh:CGRectZero pageIndex:page.getIndex needRender:YES clearCache:YES];
    } pageIndex:[page getIndex]];
    undoItem.extensionsManager = extensionsManager;
    return undoItem;
}

+ (instancetype)itemForUndoDeleteFillSignWithExtensionsManager:(UIExtensionsManager *)extensionsManager page:(FSPDFPage *)page{
    FSFillSignUndoItem *undoItem = [super itemWithUndo:^(UndoItem *item) {
        FSFillSignUndoItem *fillSignItem = (FSFillSignUndoItem *)item;
        [fillSignItem undoRedoCheckHasNullFormXObjCptr];
        [item.extensionsManager.fillSignModule.fillSignToolHandler deactiveFillSign];
        
        FSFillSign *fillSign = [[FSFillSign alloc] initWithPage:page];
        
        [item.extensionsManager.pdfViewCtrl lockRefresh];
        FSPointF *pointf = [[FSPointF alloc] initWithX:CGRectGetMinX(fillSignItem.fillSignObjAttributes.rect) Y:CGRectGetMinY(fillSignItem.fillSignObjAttributes.rect)];
        FSFillSignObject *fillSignObj;
        if (fillSignItem.fillSignObjAttributes.isTextFillSign) {
            FSTextFillSignObjAttributes *textFillSignAttributes = (FSTextFillSignObjAttributes *)fillSignItem.fillSignObjAttributes;
            fillSignObj = [fillSign addTextObject:textFillSignAttributes.textDataArray point:pointf width:CGRectGetWidth(fillSignItem.fillSignObjAttributes.rect) height:CGRectGetHeight(fillSignItem.fillSignObjAttributes.rect) rotation:[fillSignItem rotationWithPage:page] is_comb_field_mode:textFillSignAttributes.isSegText];
        }else{
            fillSignObj = [fillSign addObject:fillSignItem.fillSignObjAttributes.type point:pointf width:CGRectGetWidth(fillSignItem.fillSignObjAttributes.rect) height:CGRectGetHeight(fillSignItem.fillSignObjAttributes.rect) rotation:[fillSignItem rotationWithPage:page]];
        }
        
        [fillSignObj generateContent];
        [fillSignItem updateFormXObjCptr:[[fillSignObj getFormXObject] getCptr]];
        [item.extensionsManager.pdfViewCtrl unlockRefresh];
        [item.extensionsManager.pdfViewCtrl refresh:CGRectZero pageIndex:page.getIndex needRender:YES clearCache:YES];
    }redo:^(UndoItem *item) {
        FSFillSignUndoItem *fillSignItem = (FSFillSignUndoItem *)item;
        [fillSignItem undoRedoCheckHasNullFormXObjCptr];
        if (!fillSignItem.formXObjCptr) return;
        [item.extensionsManager.fillSignModule.fillSignToolHandler deactiveFillSign];
        
        FSFillSign *fillSign = [[FSFillSign alloc] initWithPage:page];
        
        FSFillSignObject *fillSignObj = [fillSign getObjectByFormXObject:[[FSFormXObject alloc] initWithCptr:fillSignItem.formXObjCptr swigOwnCObject:NO]];
        [item.extensionsManager.pdfViewCtrl lockRefresh];
        [fillSign removeObject:fillSignObj];
        [item.extensionsManager.pdfViewCtrl unlockRefresh];
        
        [item.extensionsManager.pdfViewCtrl refresh:CGRectZero pageIndex:page.getIndex needRender:YES clearCache:YES];
    } pageIndex:page.getIndex];
    undoItem.extensionsManager = extensionsManager;
    return undoItem;
}


- (FSRotation)rotationWithPage:(FSPDFPage *)page{
    int pageRatation = (int)page.rotation;
    return (pageRatation + [self.extensionsManager.pdfViewCtrl getViewRotation]) % 4;
}

- (void)updateFormXObjCptr:(void *)formXObjCptr{
    [FillSignHandler replaceFormXObjCptr:self.formXObjCptr byNewFormXObjCptr:formXObjCptr extensionsManager:self.extensionsManager];
}

- (void)undoRedoCheckHasNullFormXObjCptr{
    [FillSignHandler restoreFormXObjCptrWithExtensionsManager:self.extensionsManager];
}

- (void)setFormXObjCptr:(void *)formXObjCptr{
    if (!formXObjCptr) {
        _oldFormXObjCptr = _formXObjCptr;
    }else{
        _oldFormXObjCptr = NULL;
    }
    _formXObjCptr = formXObjCptr;
}

- (void)restoreFormXObjCptr{
    if (!self.formXObjCptr && self.oldFormXObjCptr) {
        self.formXObjCptr = self.oldFormXObjCptr;
    }
}

@end
