/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/


#import "FillSignMenu.h"
#import "FSWeakPointerArray.h"

#define ICONWIDTH 40
#define ARROWHEIGH 10
#define ARROWWIDTH 20

@interface FillSignMenu()
@property (nonatomic,weak) UIView* targetView;
@property (nonatomic,assign) CGRect targetRect;

@property (nonatomic, strong) UIView *menuView;
@property (nonatomic, strong) UIView *contentView;
@property (nonatomic, strong) UIView *arrowViewDown;
@property (nonatomic, strong) UIView *arrowViewUp;
@property (nonatomic, assign) BOOL isMoreMenus;

@property(nonatomic,strong) NSMutableArray<UIButton *> *menuItems;
@property(nonatomic,strong) NSArray *itemTags;
@property (nonatomic, strong) FSWeakPointerArray *layoutListeners;
@property (nonatomic, strong) FSWeakPointerArray *eventListeners;
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
@end



@implementation FillSignMenu

- (instancetype)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager{
    self = [super init];
    if (self){
        self.arrowDirection = FillSignMenuArrowDirection_Default;
        self.menuItems = [NSMutableArray arrayWithCapacity:7];
        self.itemTags = @[@(FSFillSignItemtypeCheck),@(FSFillSignItemtypeX),@(FSFillSignItemtypeDot),@(FSFillSignItemtypeLine),@(FSFillSignItemtypeRoundrect),@(FSFillSignItemtypeText),@(FSFillSignItemtypeSegText)];
        self.layoutListeners = [FSWeakPointerArray array];
        self.eventListeners = [FSWeakPointerArray array];
        self.extensionsManager = extensionsManager;
    }
    return self;
}

- (void)registerFillSignMenuLayout:(id<FillSignMenuLayout>)listener{
    if (![_layoutListeners containsObject:listener]) {
        [_layoutListeners addObject:listener];
        [self launchCanSizeFillSignMenuEvent];
    }
}

- (void)unregisterFillSignMenuLayout:(id<FillSignMenuLayout>)listener{
    [_layoutListeners removeObject:listener];
}

- (void)registerFillSignMenuEvent:(id<FillSignMenuEvent>)listener{
    [_eventListeners addObject:listener];
}

- (void)unregisterFillSignMenuEvent:(id<FillSignMenuEvent>)listener{
    [_eventListeners removeObject:listener];
}

- (void)launchCanSizeFillSignMenuEvent{
    if (!_isMoreMenus && _type != FSFillSignItemtypeNone) {
        [_layoutListeners enumerateObjectsUsingBlock:^(id<FillSignMenuLayout>listener, NSUInteger idx, BOOL * _Nonnull stop) {
            if ([listener respondsToSelector:@selector(canBiggerEnable:inView:)]) {
                BOOL canBigger = [listener canBiggerEnable:self inView:self.targetView];
                UIButton *bigBtn = [self getItemByTag:FSFillSignItemtypeBigger];
                bigBtn.enabled = canBigger;
            }

            if ([listener respondsToSelector:@selector(canSmallerEnable:inView:)]) {
                BOOL canSmaller = [listener canSmallerEnable:self inView:self.targetView];
                UIButton *smallBtn = [self getItemByTag:FSFillSignItemtypeSmaller];
                smallBtn.enabled = canSmaller;
            }
        }];
    }
}

- (void)setMenuVisible:(BOOL)menuVisible animated:(BOOL)animated
{
    if ([UIMenuController sharedMenuController].isMenuVisible && menuVisible) {
        [[UIMenuController sharedMenuController] setMenuVisible:NO animated:YES];
        return;
    }
    _menuVisible = menuVisible;
    if (menuVisible) {
        self.arrowDirection = FillSignMenuArrowDirection_Default;
        [_layoutListeners enumerateObjectsUsingBlock:^(id<FillSignMenuLayout>listener, NSUInteger idx, BOOL * _Nonnull stop) {
            UIExtensionsManager *extensionsManager = self.extensionsManager;
            if (extensionsManager) {
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
                if (@available(iOS 13.0, *)) {
                    if (self.targetView.fs_windowScene != extensionsManager.windowScene){
                        return;
                    }
                } else
#endif
                if ([extensionsManager.topToolbar contentView] && ![extensionsManager.topToolbar contentView].hidden) {
                    CGRect topbarRect = [[extensionsManager.topToolbar contentView] convertRect:[extensionsManager.topToolbar contentView].frame toView:[extensionsManager.pdfViewCtrl getDisplayView]];
                    if (CGRectIntersectsRect(topbarRect, CGRectInset(self.targetRect, -20, -20))) {
                        self.arrowDirection = FillSignMenuArrowDirection_Up;
                    }
                }else if (extensionsManager.smallTopToolbar && !extensionsManager.smallTopToolbar.hidden){
                    CGRect topbarRect = [extensionsManager.smallTopToolbar convertRect:extensionsManager.smallTopToolbar.frame toView:[extensionsManager.pdfViewCtrl getDisplayView]];
                    if (CGRectIntersectsRect(topbarRect, CGRectInset(self.targetRect, -20, -20))) {
                        self.arrowDirection = FillSignMenuArrowDirection_Up;
                    }
                }

                *stop = YES;
            }
        }];
        
        UIView *menuView = self.menuView;
        [self initMenuContent];
        UIWindow *keywindow = _targetView.fs_currentSceneKeyWindow;
        [keywindow addSubview:menuView];
        [self launchCanSizeFillSignMenuEvent];
    }else{
        [_menuView removeFromSuperview];
        _menuView = nil;
    }
}

- (void)setTargetRect:(CGRect)targetRect fillSignType:(FSFillSignItemtype)type inView:(UIView *)targetView
{
    _targetRect = targetRect;
    if (_type != type) {
        _isMoreMenus = NO;
        [_menuView removeFromSuperview];
        _menuView = nil;
    }
    _type = type;
    _targetView = targetView;
    _isMoreMenus = NO;
}

- (void)updateTargetRect:(CGRect)targetRect inView:(nonnull UIView *)targetView
{
    _targetRect = targetRect;
    _targetView = targetView;
    [self caculateShowPositionAndFrame];
    [self launchCanSizeFillSignMenuEvent];
}

- (void)updateFrame
{
    [self caculateShowPositionAndFrame];
    [self launchCanSizeFillSignMenuEvent];
}

#define kMenuItemImageName(name) [UIImage fs_dynamicImageWithNormalImage:FSImageNamed(name) darkImage:[FSImageNamed(name) fs_imageWithTintColor:self.fs_iconDarkColor_I3]]
- (UIView *)menuView
{
    if (_menuView == nil) {
        _menuView = [[UIView alloc] init];
        _contentView = [[UIView alloc] init];
        _contentView.layer.cornerRadius = 8.0;
        _contentView.layer.masksToBounds = YES;
        _contentView.backgroundColor = [UIColor blackColor];
        _arrowViewDown = [[UIView alloc] init];
        _arrowViewUp = [[UIView alloc] init];
        //_arrowView.backgroundColor = [UIColor blackColor];
        CAShapeLayer *upLayer = [[CAShapeLayer alloc] init];
        UIBezierPath *path = [[UIBezierPath alloc] init];
        [path moveToPoint:CGPointMake(0, 0)];
        [path addLineToPoint:CGPointMake(ARROWWIDTH * 0.5, ARROWHEIGH)];
        [path addLineToPoint:CGPointMake(ARROWWIDTH, 0)];
        [path closePath];
        upLayer.path = path.CGPath;

        [_arrowViewDown.layer addSublayer:upLayer];
        
        CAShapeLayer *downLayer = [[CAShapeLayer alloc] init];
        UIBezierPath *path1 = [[UIBezierPath alloc] init];
        [path1 moveToPoint:CGPointMake(0, ARROWHEIGH)];
        [path1 addLineToPoint:CGPointMake(ARROWWIDTH, ARROWHEIGH)];
        [path1 addLineToPoint:CGPointMake(ARROWWIDTH * 0.5, 0)];
        [path1 closePath];
        downLayer.path = path1.CGPath;

        [_arrowViewUp.layer addSublayer:downLayer];
        
        [_menuView addSubview:_contentView];
        [_menuView addSubview:_arrowViewDown];
        
        NSArray <NSNumber *> *itemTags;
        
        if (_type == FSFillSignItemtypeNone) {
            itemTags = _itemTags;
        }else{
            NSArray *noTextTags = @[@(FSFillSignItemtypeCheck),@(FSFillSignItemtypeX),@(FSFillSignItemtypeDot),@(FSFillSignItemtypeLine),@(FSFillSignItemtypeRoundrect)];
            if (_isMoreMenus) {
                NSMutableArray *tmpMuArr = @[].mutableCopy;
                for (int i = 0; i < noTextTags.count; i++) {
                    NSNumber *tag = noTextTags[i];
                    if (tag.integerValue != _type) {
                        [tmpMuArr addObject:tag];
                    }
                }
                [tmpMuArr addObject:@(FSFillSignItemtypeMore)];
                itemTags = tmpMuArr.copy;
            }else{
                if ([noTextTags containsObject:@(_type)]) {
                    itemTags = @[@(FSFillSignItemtypeSmaller),@(FSFillSignItemtypeBigger),@(FSFillSignItemtypeMore),@(FSFillSignItemtypeDelete)];
                }else{
                    NSMutableArray *tmpMuArr = @[@(FSFillSignItemtypeSmaller),@(FSFillSignItemtypeBigger),@(FSFillSignItemtypeMore),@(FSFillSignItemtypeText),@(FSFillSignItemtypeSegText),@(FSFillSignItemtypeDelete)].mutableCopy;
                    [tmpMuArr removeObject:@(_type)];
                    itemTags = tmpMuArr.copy;
                }
            }
        }
        if (_menuItems.count > 0){
            for (UIButton *btn in _menuItems) {
                [btn removeFromSuperview];
            }
            [_menuItems removeAllObjects];
        }
        for (int i = 0; i < itemTags.count; i++) {
            NSInteger tag = [[itemTags objectAtIndex:i] integerValue];
            UIButton *btn = [[UIButton alloc] init];
            [btn addTarget:self action:@selector(itemClick:) forControlEvents:UIControlEventTouchUpInside];
            btn.frame = CGRectMake((_menuItems.count) * ICONWIDTH, 0, ICONWIDTH, ICONWIDTH);
            btn.tag = tag;
            
            if (btn.tag == FSFillSignItemtypeCheck) {
                [btn setImage:kMenuItemImageName(@"fill_sign_menu_check") forState:UIControlStateNormal];
                btn.accessibilityLabel = FSLocalizedForKey(@"kAddCheck");
            }
            if (btn.tag == FSFillSignItemtypeX) {
                [btn setImage:kMenuItemImageName(@"fill_sign_menu_X") forState:UIControlStateNormal];
                btn.accessibilityLabel = FSLocalizedForKey(@"kAddX");
            }
            if (btn.tag == FSFillSignItemtypeDot) {
                [btn setImage:kMenuItemImageName(@"fill_sign_menu_dot") forState:UIControlStateNormal];
                btn.accessibilityLabel = FSLocalizedForKey(@"kAddDot");
            }
            if (btn.tag == FSFillSignItemtypeLine) {
                [btn setImage:kMenuItemImageName(@"fill_sign_menu_line") forState:UIControlStateNormal];
                btn.accessibilityLabel = FSLocalizedForKey(@"kAddLine");
            }
            if (btn.tag == FSFillSignItemtypeRoundrect) {
                [btn setImage:kMenuItemImageName(@"fill_sign_menu_roundrect") forState:UIControlStateNormal];
                btn.accessibilityLabel = FSLocalizedForKey(@"kAddRoundedRectangle");
            }
            if (btn.tag == FSFillSignItemtypeText) {
                [btn setImage:kMenuItemImageName(@"fill_sign_menu_text") forState:UIControlStateNormal];
                btn.accessibilityLabel = FSLocalizedForKey(@"kAddText");
            }
            if (btn.tag == FSFillSignItemtypeSegText) {
                [btn setImage:kMenuItemImageName(@"fill_sign_menu_segtext") forState:UIControlStateNormal];
                btn.accessibilityLabel = FSLocalizedForKey(@"kAddCombField");
            }
            
            if (btn.tag == FSFillSignItemtypeSmaller) {
                [btn setImage:kMenuItemImageName(@"fill_sign_menu_smaller") forState:UIControlStateNormal];
                btn.accessibilityLabel = FSLocalizedForKey(@"kDecreaseVoiceOver");
            }
            if (btn.tag == FSFillSignItemtypeBigger) {
                [btn setImage:kMenuItemImageName(@"fill_sign_menu_bigger") forState:UIControlStateNormal];
                btn.accessibilityLabel = FSLocalizedForKey(@"kIncreaseVoiceOver");
            }
            if (btn.tag == FSFillSignItemtypeMore) {
                [btn setImage:kMenuItemImageName(@"fill_sign_menu_more") forState:UIControlStateNormal];
                btn.accessibilityLabel = FSLocalizedForKey(@"kMore");
            }

            if (btn.tag == FSFillSignItemtypeDelete) {
                [btn setImage:kMenuItemImageName(@"fill_sign_menu_delete") forState:UIControlStateNormal];
                btn.accessibilityLabel = FSLocalizedForKey(@"kDelete");
            }

            if (i > 0) {
                CALayer *line = [[CALayer alloc] init];
                line.frame = CGRectMake( CGRectGetMinX(btn.frame)-1, 0, 1, ICONWIDTH);
                line.backgroundColor = [UIColor whiteColor].CGColor;
                [_contentView.layer addSublayer:line];
            }
            [_contentView addSubview:btn];
            [_menuItems addObject:btn];
        }
    }
    return _menuView;
}
#undef kMenuItemImageName

- (void)caculateShowPositionAndFrame
{
    CGFloat menuContentWidth = self.menuItems.count * ICONWIDTH;
    CGSize menuContentSize = CGSizeMake(menuContentWidth, ICONWIDTH);
    
    UIWindow *keywindow = _targetView.fs_currentSceneKeyWindow;
    CGRect targetOnWindow = [_targetView convertRect:_targetRect toView:keywindow];
    CGRect targetViewOnWindow = [_targetView convertRect:_targetView.frame toView:keywindow];
    if (CGRectContainsRect(targetViewOnWindow, targetOnWindow)) targetViewOnWindow = targetOnWindow;
    if (!_targetView || targetOnWindow.origin.x + targetOnWindow.size.width < 0 || targetOnWindow.origin.y + targetOnWindow.size.height < 0 || targetOnWindow.origin.x > keywindow.frame.size.width || targetOnWindow.origin.y > keywindow.frame.size.height) {
        _menuView.hidden = YES;
        return;
    }
    _menuView.hidden = NO;
    
    if (self.arrowDirection == FillSignMenuArrowDirection_Down) {
        CGRect menuFrame = CGRectMake(CGRectGetMidX(targetOnWindow) - menuContentWidth * 0.5, CGRectGetMinY(targetOnWindow) - ARROWHEIGH - ICONWIDTH, menuContentWidth, ARROWHEIGH + ICONWIDTH);
        _menuView.frame = menuFrame;
        _contentView.frame = CGRectMake(0, 0, menuContentSize.width, menuContentSize.height);
        if (_arrowViewUp.superview) {
            [_arrowViewUp removeFromSuperview];
            [_menuView addSubview:_arrowViewDown];
        }
        _arrowViewDown.frame = CGRectMake(menuContentSize.width * 0.5 - ARROWWIDTH * 0.5, ICONWIDTH - 1, ARROWWIDTH, ARROWHEIGH);
        return;
    }else if (self.arrowDirection == FillSignMenuArrowDirection_Up) {
        CGRect menuFrame = CGRectMake(targetOnWindow.origin.x + targetOnWindow.size.width * 0.5 - menuContentWidth * 0.5, targetOnWindow.origin.y + targetOnWindow.size.height, menuContentWidth, ARROWHEIGH + ICONWIDTH);
        _menuView.frame = menuFrame;
        _contentView.frame = CGRectMake(0, ARROWHEIGH, menuContentSize.width, menuContentSize.height);
        //_arrowView.frame = CGRectMake(menuContentSize.width * 0.5 - ARROWWIDTH * 0.5, 0, ARROWWIDTH, ARROWHEIGH);
        if (_arrowViewDown.superview) {
            [_arrowViewDown removeFromSuperview];
            [_menuView addSubview:_arrowViewUp];
        }
        _arrowViewUp.frame = CGRectMake(menuContentSize.width * 0.5 - ARROWWIDTH * 0.5, 1, ARROWWIDTH, ARROWHEIGH);
        return;
    }
    CGFloat arrowx = targetOnWindow.origin.x + targetOnWindow.size.width * 0.5;
    if (arrowx - menuContentWidth * 0.5 >= 0 && arrowx + menuContentWidth * 0.5 <= keywindow.frame.size.width) {
        if (CGRectGetMinY(targetOnWindow) - ABS(CGRectGetMinY(targetOnWindow) - CGRectGetMinY(targetViewOnWindow)) > ARROWHEIGH + ICONWIDTH) {
            CGRect menuFrame = CGRectMake(CGRectGetMidX(targetOnWindow) - menuContentWidth * 0.5, CGRectGetMinY(targetOnWindow) - ARROWHEIGH - ICONWIDTH, menuContentWidth, ARROWHEIGH + ICONWIDTH);
            _menuView.frame = menuFrame;
            _contentView.frame = CGRectMake(0, 0, menuContentSize.width, menuContentSize.height);
            if (_arrowViewUp.superview) {
                [_arrowViewUp removeFromSuperview];
                [_menuView addSubview:_arrowViewDown];
            }
            _arrowViewDown.frame = CGRectMake(menuContentSize.width * 0.5 - ARROWWIDTH * 0.5, ICONWIDTH - 1, ARROWWIDTH, ARROWHEIGH);
            
        }else if (keywindow.frame.size.height - CGRectGetMaxY(targetOnWindow) > ARROWHEIGH + ICONWIDTH){
            CGRect menuFrame = CGRectMake(targetOnWindow.origin.x + targetOnWindow.size.width * 0.5 - menuContentWidth * 0.5, targetOnWindow.origin.y + targetOnWindow.size.height, menuContentWidth, ARROWHEIGH + ICONWIDTH);
            _menuView.frame = menuFrame;
            _contentView.frame = CGRectMake(0, ARROWHEIGH, menuContentSize.width, menuContentSize.height);
            //_arrowView.frame = CGRectMake(menuContentSize.width * 0.5 - ARROWWIDTH * 0.5, 0, ARROWWIDTH, ARROWHEIGH);
            if (_arrowViewDown.superview) {
                [_arrowViewDown removeFromSuperview];
                [_menuView addSubview:_arrowViewUp];
            }
            _arrowViewUp.frame = CGRectMake(menuContentSize.width * 0.5 - ARROWWIDTH * 0.5, 1, ARROWWIDTH, ARROWHEIGH);
        }else{
            CGRect menuFrame = CGRectMake(targetOnWindow.origin.x + targetOnWindow.size.width * 0.5 - menuContentWidth * 0.5, targetOnWindow.size.height * 0.5 - ARROWHEIGH - ICONWIDTH, menuContentWidth, ARROWHEIGH + ICONWIDTH);
            _menuView.frame = menuFrame;
            _contentView.frame = CGRectMake(0, 0, menuContentSize.width, menuContentSize.height);
            
            if (_arrowViewUp.superview) {
                [_arrowViewUp removeFromSuperview];
                [_menuView addSubview:_arrowViewDown];
            }
            _arrowViewDown.frame = CGRectMake(menuContentSize.width * 0.5 - ARROWWIDTH * 0.5, ICONWIDTH - 1, ARROWWIDTH, ARROWHEIGH);
        }
        return;
    }
    
    if (arrowx - menuContentWidth * 0.5 < 0 && arrowx + menuContentWidth * 0.5< keywindow.frame.size.width) {
        if (CGRectGetMinY(targetOnWindow) - ABS(CGRectGetMinY(targetOnWindow) - CGRectGetMinY(targetViewOnWindow)) > ARROWHEIGH + ICONWIDTH) {
            CGRect menuFrame = CGRectMake(0, targetOnWindow.origin.y - ARROWHEIGH - ICONWIDTH, menuContentWidth, ARROWHEIGH + ICONWIDTH);
            _menuView.frame = menuFrame;
            _contentView.frame = CGRectMake(0, 0, menuContentSize.width, menuContentSize.height);
            if (_arrowViewUp.superview) {
                [_arrowViewUp removeFromSuperview];
                [_menuView addSubview:_arrowViewDown];
            }
            _arrowViewDown.frame = CGRectMake(arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5 < 8 ? 8 : arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5, ICONWIDTH - 1, ARROWWIDTH, ARROWHEIGH);
        }else if (keywindow.frame.size.height - CGRectGetMaxY(targetOnWindow) > ARROWHEIGH + ICONWIDTH){
            CGRect menuFrame = CGRectMake(0, targetOnWindow.origin.y + targetOnWindow.size.height, menuContentWidth, ARROWHEIGH + ICONWIDTH);
            _menuView.frame = menuFrame;
            _contentView.frame = CGRectMake(0, ARROWHEIGH, menuContentSize.width, menuContentSize.height);
           
            if (_arrowViewDown.superview) {
                [_arrowViewDown removeFromSuperview];
                [_menuView addSubview:_arrowViewUp];
            }
             _arrowViewUp.frame = CGRectMake(arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5 < 8 ? 8 : arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5, 1, ARROWWIDTH, ARROWHEIGH);
        }else{
            CGRect menuFrame = CGRectMake(0, targetOnWindow.size.height * 0.5 - ARROWHEIGH - ICONWIDTH, menuContentWidth, ARROWHEIGH + ICONWIDTH);
            _menuView.frame = menuFrame;
            _contentView.frame = CGRectMake(0, 0, menuContentSize.width, menuContentSize.height);
            
            if (_arrowViewUp.superview) {
                [_arrowViewUp removeFromSuperview];
                [_menuView addSubview:_arrowViewDown];
            }
            _arrowViewDown.frame = CGRectMake(arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5 < 8 ? 8 : arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5, ICONWIDTH - 1, ARROWWIDTH, ARROWHEIGH);
        }
        return;
    }
    
    if (arrowx - menuContentWidth * 0.5 > 0 && arrowx + menuContentWidth * 0.5 > keywindow.frame.size.width) {
        if (CGRectGetMinY(targetOnWindow) - ABS(CGRectGetMinY(targetOnWindow) - CGRectGetMinY(targetViewOnWindow)) > ARROWHEIGH + ICONWIDTH) {
            CGRect menuFrame = CGRectMake(keywindow.frame.size.width - menuContentWidth, targetOnWindow.origin.y - ARROWHEIGH - ICONWIDTH, menuContentWidth, ARROWHEIGH + ICONWIDTH);
            _menuView.frame = menuFrame;
            _contentView.frame = CGRectMake(0, 0, menuContentSize.width, menuContentSize.height);
           
            if (_arrowViewUp.superview) {
                [_arrowViewUp removeFromSuperview];
                [_menuView addSubview:_arrowViewDown];
            }
            _arrowViewDown.frame = CGRectMake(arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5 > menuFrame.size.width - 28 ? menuFrame.size.width - 28 : arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5, ICONWIDTH - 1, ARROWWIDTH, ARROWHEIGH);
        }else if (keywindow.frame.size.height - CGRectGetMaxY(targetOnWindow) > ARROWHEIGH + ICONWIDTH){
            CGRect menuFrame = CGRectMake(keywindow.frame.size.width - menuContentWidth, targetOnWindow.origin.y + targetOnWindow.size.height, menuContentWidth, ARROWHEIGH + ICONWIDTH);
            _menuView.frame = menuFrame;
            _contentView.frame = CGRectMake(0, ARROWHEIGH, menuContentSize.width, menuContentSize.height);
            
            if (_arrowViewDown.superview) {
                [_arrowViewDown removeFromSuperview];
                [_menuView addSubview:_arrowViewUp];
            }
            _arrowViewUp.frame = CGRectMake(arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5 > menuFrame.size.width - 28 ? menuFrame.size.width - 28 : arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5, 1, ARROWWIDTH, ARROWHEIGH);
        }else{
            CGRect menuFrame = CGRectMake(keywindow.frame.size.width - menuContentWidth, targetOnWindow.size.height * 0.5 - ARROWHEIGH - ICONWIDTH, menuContentWidth, ARROWHEIGH + ICONWIDTH);
            _menuView.frame = menuFrame;
            _contentView.frame = CGRectMake(0, 0, menuContentSize.width, menuContentSize.height);
            
            if (_arrowViewUp.superview) {
                [_arrowViewUp removeFromSuperview];
                [_menuView addSubview:_arrowViewDown];
            }
            _arrowViewDown.frame = CGRectMake(arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5 > menuFrame.size.width - 28 ? menuFrame.size.width - 28 : arrowx - menuFrame.origin.x - ARROWWIDTH * 0.5, ICONWIDTH - 1, ARROWWIDTH, ARROWHEIGH);
        }
        return;
    }
}

- (void)initMenuContent
{
    [self caculateShowPositionAndFrame];
}


- (void)itemClick:(UIButton *)item
{
    if (_type == FSFillSignItemtypeNone) {
        _type = item.tag;
        [self setMenuVisible:NO animated:NO];
        [self setMenuVisible:YES animated:NO];
    }else{
        if (item.tag == FSFillSignItemtypeMore) {
            _isMoreMenus = !_isMoreMenus;
            [self setMenuVisible:NO animated:NO];
            [self setMenuVisible:YES animated:NO];
        }
    }
    
    if (item.tag > FSFillSignItemtypeNone && item.tag < FSFillSignItemtypeMore && _eventListeners.count) {
        [_eventListeners enumerateObjectsUsingBlock:^(id<FillSignMenuEvent>listener, NSUInteger idx, BOOL * _Nonnull stop) {
            if ([listener respondsToSelector:@selector(didSelectedMenu:type:)]) {
                [listener didSelectedMenu:self type:item.tag];
            }
        }];
        
        [_layoutListeners enumerateObjectsUsingBlock:^(id<FillSignMenuLayout>listener, NSUInteger idx, BOOL * _Nonnull stop) {
            if (item.tag == FSFillSignItemtypeBigger) {
                if ([listener respondsToSelector:@selector(canBiggerEnable:inView:)]) {
                    BOOL canBigger = [listener canBiggerEnable:self inView:self.targetView];
                    UIButton *bigBtn = [self getItemByTag:FSFillSignItemtypeBigger];
                    bigBtn.enabled = canBigger;
                    
                    UIButton *smallBtn = [self getItemByTag:FSFillSignItemtypeSmaller];
                    smallBtn.enabled = YES;
                }
            }
            
            if (item.tag == FSFillSignItemtypeSmaller) {
                if ([listener respondsToSelector:@selector(canSmallerEnable:inView:)]) {
                    BOOL canSmaller = [listener canSmallerEnable:self inView:self.targetView];
                    UIButton *smallBtn = [self getItemByTag:FSFillSignItemtypeSmaller];
                    smallBtn.enabled = canSmaller;
                    
                    UIButton *bigBtn = [self getItemByTag:FSFillSignItemtypeBigger];
                    bigBtn.enabled = YES;
                }
            }
        }];
    }
}

- (UIButton *)getItemByTag:(FSFillSignItemtype)type
{
    if (type == FSFillSignItemtypeNone) {
        return nil;
    }
    for (int i = 0; i < _menuItems.count; i++) {
        UIButton *btn = [_menuItems objectAtIndex:i];
        if (btn.tag == type) {
            return btn;
        }
    }
    return nil;
}
@end
