/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/


#import "FillSignProfileController.h"
#import "IQKeyboardManager.h"

#define FILLSIGNPROFILE [[NSSearchPathForDirectoriesInDomains(NSLibraryDirectory, NSUserDomainMask, YES) lastObject] stringByAppendingPathComponent:@"Preferences/com.foxit.fillsignProfile.plist"]

@interface FillSignProfileCell : UITableViewCell
@property (nonatomic, assign) BOOL isCustomFieldCell;
@property (nonatomic, assign) BOOL isEdit;
@property (nonatomic, strong) UILabel *keyLabel;
@property (nonatomic, strong) UITextField *keyTextField;
@property (nonatomic, strong) UITextField *valueTextField;
@property (nonatomic, strong) UIView *clearView;

@property (nonatomic, strong) NSIndexPath *indexPath;

- (instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier isCustomFieldCell:(BOOL)isCustomFieldCell isEdit:(BOOL) isEdit;
@end

@implementation FillSignProfileCell

- (instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier isCustomFieldCell:(BOOL)isCustomFieldCell isEdit:(BOOL) isEdit
{
    if (self = [super initWithStyle:style reuseIdentifier:reuseIdentifier]) {
        self.backgroundColor = self.fs_backgroundColor_B1;
        _isCustomFieldCell = isCustomFieldCell;
        _isEdit = isEdit;
        if (isCustomFieldCell) {
            _keyTextField = [[UITextField alloc] init];
            _keyTextField.layer.anchorPoint = CGPointMake(0, 0.5);
            _keyTextField.font = self.fs_standardGeneralFont_GF2;
            _keyTextField.textColor = self.fs_textColor_T4;
            [self.contentView addSubview:_keyTextField];
            FSItem *clearItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"common_clear_gray")];
            _clearView = clearItem;
                [self.contentView addSubview:_clearView];
        }else{
            _keyLabel = [[UILabel alloc] init];
            _keyLabel.layer.anchorPoint = CGPointMake(0, 0.5);
            _keyLabel.font = self.fs_standardGeneralFont_GF2;
            _keyLabel.textColor = self.fs_textColor_T4;
            [self.contentView addSubview:_keyLabel];
        }
        _valueTextField = [[UITextField alloc] init];
        _valueTextField.layer.anchorPoint = CGPointMake(0, 0.5);
        _valueTextField.font = self.fs_standardGeneralFont_GF2;
        _valueTextField.textColor = self.fs_textColor_T4;

        //self.selectionStyle = UITableViewCellSelectionStyleNone;
        [self.contentView addSubview:_valueTextField];
      
    }
    return self;
}

- (void)layoutSubviews
{
    [super layoutSubviews];
    CGSize contentViewSize = self.contentView.frame.size;
    _valueTextField.center = CGPointMake(contentViewSize.width * 0.5 - 20 + 2, contentViewSize.height * 0.5);
    
    if (_isCustomFieldCell) {
        _keyTextField.center = CGPointMake(14, contentViewSize.height * 0.5);
        _keyTextField.bounds = CGRectMake(0, 0, contentViewSize.width * 0.5 - 14 - 20, 21);
        if(_isEdit){
            _valueTextField.bounds = CGRectMake(0, 0, contentViewSize.width - (contentViewSize.width * 0.5 - 20 + 2) -30, 21);
        }else{
            _valueTextField.bounds = CGRectMake(0, 0, contentViewSize.width - (contentViewSize.width * 0.5 - 20 + 2) -14, 21);
        }
        _clearView.center = CGPointMake(contentViewSize.width - 16, contentViewSize.height * 0.5);
        _clearView.bounds = CGRectMake(0, 0, 36,36);
    }else{
        _keyLabel.center = CGPointMake(14, contentViewSize.height * 0.5);
        _keyLabel.bounds = CGRectMake(0, 0, contentViewSize.width * 0.5 - 14 - 20, 21);
        _valueTextField.bounds = CGRectMake(0, 0, contentViewSize.width - (contentViewSize.width * 0.5 - 20 + 2) - 14, 21);
    }


}

@end

@interface FillSignProfileController ()<UITableViewDelegate,UITableViewDataSource,UITextFieldDelegate>
@property (nonatomic, strong) NSMutableArray *nameArray;
@property (nonatomic, strong) NSMutableArray *addArray;
@property (nonatomic, strong) NSMutableArray *contactArray;
@property (nonatomic, strong) NSMutableArray *dateArray;
@property (nonatomic, strong) NSMutableArray *customArray;
@property (nonatomic, strong) NSArray *textTips;

@property (nonatomic, assign) BOOL isEditProfile;
@property (nonatomic, strong) NSMutableArray *contentArray;

@property (nonatomic, strong) UITextField *editingField;
@property (nonatomic, assign) BOOL isModified;

@property (nonatomic, copy) NSArray *headerTitles;
@end

@implementation FillSignProfileController

+ (NSArray *)getPreDefineConetArray
{
    FillSignProfileController *ctr = [[FillSignProfileController alloc] init];
    [ctr loadData];
    return [ctr loadPredefineValue];
}

- (void)initialize{
    [super initialize];
    self.defaultEstimatedCellStyle = FSEstimatedTableViewCellStyleTitle;
    self.sectionStyle = FSTableViewSectionStyleLikeGrouped;
    self.shouldLoadDataOnViewDidLoad = YES;
    self.shouldShowBottomBar = YES;
}

- (void)loadData{
    if ([[NSFileManager defaultManager] fileExistsAtPath:FILLSIGNPROFILE]) {
        NSDictionary *tmp = [NSDictionary dictionaryWithContentsOfFile:FILLSIGNPROFILE];
        
        _nameArray = [NSMutableArray array];
        
        NSArray *names =  tmp[@"kName"];
        NSArray *keys = @[ @"kProfileFullName", @"kProfileFirstName", @"kProfileMiddleName", @"kProfileLastName" ];
        for (NSString *key in keys) {
            BOOL isExist = NO;
            for (NSDictionary *dict in names ) {
                if ([[[dict allKeys] firstObject] isEqualToString:key]) {
                    [_nameArray addObject:@{key:dict[key]}];
                    isExist = YES;
                    break;
                }
            }
            if (!isExist) {
                [_nameArray addObject:@{key:@""}];
            }
        }
        
        _addArray = [NSMutableArray array];
        if (tmp[@"kProfileAddress"]) {
            [_addArray addObjectsFromArray:[tmp[@"kProfileAddress"] copy]];
        }
        
        _contactArray = [NSMutableArray array];
        if (tmp[@"kProfileContactInfo"]) {
            [_contactArray addObjectsFromArray:[tmp[@"kProfileContactInfo"] copy]];
        }
        
        _dateArray = [NSMutableArray array];
        if (tmp[@"kProfileDates"]) {
            [_dateArray addObjectsFromArray:[tmp[@"kProfileDates"] copy]];
        }
        
        _customArray = [NSMutableArray array];
        if (tmp[@"kProfileCustomfield"]) {
            [_customArray addObjectsFromArray:[tmp[@"kProfileCustomfield"] copy]];
        }
        _modeDatas = [NSMutableDictionary dictionary];
        [_modeDatas setDictionary: @{@"kName":_nameArray,@"kProfileAddress":_addArray,@"kProfileContactInfo":_contactArray,@"kProfileDates":_dateArray,@"kProfileCustomfield":_customArray}];
    }else{
        _nameArray = [NSMutableArray array];
        [_nameArray addObjectsFromArray:@[@{@"kProfileFullName":@""},@{@"kProfileFirstName":@""},@{@"kProfileMiddleName":@""},@{@"kProfileLastName":@""}]];
        _addArray = [NSMutableArray array];
        [_addArray addObjectsFromArray:@[@{@"kProfileStreet1":@""},@{@"kProfileStreet2":@""},@{@"kProfileCity":@""},@{@"kProfileState":@""},@{@"kProfileZip":@""},@{@"kProfileCountry":@""}]];
        _contactArray = [NSMutableArray array];
        [_contactArray addObjectsFromArray:@[@{@"kProfileEmail":@""},@{@"kProfileTel":@""}]];
        _dateArray = [NSMutableArray array];
        [_dateArray addObjectsFromArray:@[@{@"kProfileDate":@""},@{@"kProfileBirthDate":@""}]];
        _customArray = [NSMutableArray array];
        _modeDatas = [NSMutableDictionary dictionary];
        [_modeDatas setDictionary: @{@"kName":_nameArray,@"kProfileAddress":_addArray,@"kProfileContactInfo":_contactArray,@"kProfileDates":_dateArray,@"kProfileCustomfield":_customArray}];
        [_modeDatas writeToFile:FILLSIGNPROFILE atomically:YES];
    }
}


- (NSArray *)loadPredefineValue
{
    NSMutableArray *array = [NSMutableArray array];
    for (NSDictionary *dic in _nameArray) {
        NSArray *tmp = [dic allValues];
        NSString *tmpstr = [tmp lastObject];
        if (tmp.count > 0 && tmpstr.length > 0) [array addObjectsFromArray:tmp];
    }
    for (NSDictionary *dic in _addArray) {
        NSArray *tmp = [dic allValues];
        NSString *tmpstr = [tmp lastObject];
        if (tmp.count > 0 && tmpstr.length > 0) [array addObjectsFromArray:tmp];
    }
    for (NSDictionary *dic in _contactArray) {
        NSArray *tmp = [dic allValues];
        NSString *tmpstr = [tmp lastObject];
        if (tmp.count > 0 && tmpstr.length > 0) [array addObjectsFromArray:tmp];
    }
    for (NSDictionary *dic in _dateArray) {
        NSArray *tmp = [dic allValues];
        NSString *tmpstr = [tmp lastObject];
        if (tmp.count > 0 && tmpstr.length > 0) [array addObjectsFromArray:tmp];
    }
    for (NSDictionary *dic in _customArray) {
        NSArray *tmp = [dic allValues];
        NSString *tmpstr = [tmp lastObject];
        if (tmp.count > 0 && tmpstr.length > 0) [array addObjectsFromArray:tmp];
    }
    return array;
}

- (void)saveData2ModebyIndexPath:(NSIndexPath *)indexPath cell:(FillSignProfileCell*)cell
{
    switch (indexPath.section) {
        case 0:
        {
            NSDictionary *dic = _nameArray[indexPath.row];
            NSArray *tmp = [dic allKeys];
            NSString *tmpstr = [tmp lastObject];
            NSString *utlStr = [cell.valueTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
            NSDictionary *tmpdic = [NSDictionary dictionaryWithObjectsAndKeys:utlStr, tmpstr,nil];
            [_nameArray replaceObjectAtIndex:indexPath.row withObject:tmpdic];
            break;
        }
        case 1:
        {
            NSDictionary *dic = _addArray[indexPath.row];
            NSArray *tmp = [dic allKeys];
            NSString *tmpstr = [tmp lastObject];
            NSString *utlStr = [cell.valueTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
            NSDictionary *tmpdic = [NSDictionary dictionaryWithObjectsAndKeys:utlStr, tmpstr,nil];
            [_addArray replaceObjectAtIndex:indexPath.row withObject:tmpdic];
            break;
        }
        case 2:
        {
            NSDictionary *dic = _contactArray[indexPath.row];
            NSArray *tmp = [dic allKeys];
            NSString *tmpstr = [tmp lastObject];
            NSString *utlStr = [cell.valueTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
            NSDictionary *tmpdic = [NSDictionary dictionaryWithObjectsAndKeys:utlStr, tmpstr,nil];
            [_contactArray replaceObjectAtIndex:indexPath.row withObject:tmpdic];
            break;
        }
        case 3:
        {
            NSDictionary *dic = _dateArray[indexPath.row];
            NSArray *tmp = [dic allKeys];
            NSString *tmpstr = [tmp lastObject];
            if ([tmpstr isEqualToString:FSLocalizedForKey(@"kProfileDate")]) break;
            NSString *utlStr = [cell.valueTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
            NSDictionary *tmpdic = [NSDictionary dictionaryWithObjectsAndKeys:utlStr, tmpstr,nil];
            [_dateArray replaceObjectAtIndex:indexPath.row withObject:tmpdic];
            break;
        }
        case 4:
        {
            NSString *tmpstr = [cell.keyTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
            if (tmpstr.length < 1) break;
            NSString *utlStr = [cell.valueTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
            NSDictionary *tmpdic = [NSDictionary dictionaryWithObjectsAndKeys:utlStr, tmpstr,nil];
            if (indexPath.row < _customArray.count) {
                [_customArray replaceObjectAtIndex:indexPath.row withObject:tmpdic];
            }
            
            break;
        }
        default:
            break;
    }
}

- (void)saveData2Mode
{
    for (int i = 0; i < _nameArray.count; i++) {
        NSIndexPath *indexPath = [NSIndexPath indexPathForRow:i inSection:0];
        FillSignProfileCell *cell = [self.tableView cellForRowAtIndexPath:indexPath];
        if (cell == nil) continue;
        NSDictionary *dic = _nameArray[i];
        NSArray *tmp = [dic allKeys];
        NSString *tmpstr = [tmp lastObject];
        NSString *utlStr = [cell.valueTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
        NSDictionary *tmpdic = [NSDictionary dictionaryWithObjectsAndKeys:utlStr, tmpstr,nil];
        [_nameArray replaceObjectAtIndex:i withObject:tmpdic];
    }
    
    for (int i = 0; i < _addArray.count; i++) {
        NSIndexPath *indexPath = [NSIndexPath indexPathForRow:i inSection:1];
        FillSignProfileCell *cell = [self.tableView cellForRowAtIndexPath:indexPath];
        if (cell == nil) continue;
        NSDictionary *dic = _addArray[i];
        NSArray *tmp = [dic allKeys];
        NSString *tmpstr = [tmp lastObject];
        NSString *utlStr = [cell.valueTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
        NSDictionary *tmpdic = [NSDictionary dictionaryWithObjectsAndKeys:utlStr, tmpstr,nil];
        [_addArray replaceObjectAtIndex:i withObject:tmpdic];
    }
    
    for (int i = 0; i < _contactArray.count; i++) {
        NSIndexPath *indexPath = [NSIndexPath indexPathForRow:i inSection:2];
        FillSignProfileCell *cell = [self.tableView cellForRowAtIndexPath:indexPath];
        if (cell == nil) continue;
        NSDictionary *dic = _contactArray[i];
        NSArray *tmp = [dic allKeys];
        NSString *tmpstr = [tmp lastObject];
        NSString *utlStr = [cell.valueTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
        NSDictionary *tmpdic = [NSDictionary dictionaryWithObjectsAndKeys:utlStr, tmpstr,nil];
        [_contactArray replaceObjectAtIndex:i withObject:tmpdic];
    }
    
    for (int i = 0; i < _dateArray.count; i++) {
        NSIndexPath *indexPath = [NSIndexPath indexPathForRow:i inSection:3];
        FillSignProfileCell *cell = [self.tableView cellForRowAtIndexPath:indexPath];
        if (cell == nil) continue;
        NSDictionary *dic = _dateArray[i];
        NSArray *tmp = [dic allKeys];
        NSString *tmpstr = [tmp lastObject];
        if ([tmpstr isEqualToString:FSLocalizedForKey(@"kProfileDate")]) continue;
        NSString *utlStr = [cell.valueTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
        NSDictionary *tmpdic = [NSDictionary dictionaryWithObjectsAndKeys:utlStr, tmpstr,nil];
        [_dateArray replaceObjectAtIndex:i withObject:tmpdic];
    }
    
    for (int i = 0; i < _customArray.count; i++) {
        NSIndexPath *indexPath = [NSIndexPath indexPathForRow:i inSection:4];
        FillSignProfileCell *cell = [self.tableView cellForRowAtIndexPath:indexPath];
        if (cell == nil) continue;
        NSString *tmpstr = [cell.keyTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
        NSString *utlStr = [cell.valueTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
        NSDictionary *tmpdic = [NSDictionary dictionaryWithObjectsAndKeys:utlStr, tmpstr,nil];
        [_customArray replaceObjectAtIndex:i withObject:tmpdic];
    }
}

- (void)viewDidLoad {
    [super viewDidLoad];
    _textTips = @[@[@"kProfileAddFullName",@"kProfileAddFirstName",@"kProfileAddMiddleName",@"kProfileAddLastName"],@[@"kProfileAddStreet1",@"kProfileAddStreet2",@"kProfileAddCity",@"kProfileAddState",@"kProfileAddZip",@"kProfileAddCountry"],@[@"kProfileAddEmail",@"kProfileAddTel"],@[@"kProfileAddDate",@"kProfileAddBirthDate"]];
    [self initNavbar];
    _isModified = NO;
    
    FSItem *addItem = [[FSItem alloc] initWithRrelation:FSImageTextRelationRight];
    [addItem setImage:FSImageNamedAndNormalColor(@"fill_sign_profile_customfield") forState:UIControlStateNormal];
    [addItem setTitle:FSLocalizedForKey(@"kProfileAddCustom") forState:UIControlStateNormal];
    [addItem addTarget:self action:@selector(addCustomFieldClick:) forControlEvents:UIControlEventTouchUpInside];
    [self.bottomBar addItem:addItem spacing:4.f atPosition:FSItemPositionLeft];

    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(textFieldDidChange:) name:UITextFieldTextDidChangeNotification object:nil];

    self.headerTitles = @[ FSLocalizedForKey(@"kName"), FSLocalizedForKey(@"kProfileAddress"), FSLocalizedForKey(@"kProfileContactInfo"), FSLocalizedForKey(@"kProfileDates"), FSLocalizedForKey(@"kProfileCustomfield")];
    
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
        if (@available(iOS 13.0, *)) self.modalInPresentation = YES;
#endif
}

- (void)addCustomFieldClick:(FSItem *)btn{
    if (_customArray.count > 0) {
        FillSignProfileCell *cell = [self.tableView cellForRowAtIndexPath:[NSIndexPath indexPathForRow:_customArray.count - 1 inSection:4]];
        if (cell) {
            if (![cell.keyTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]].length) {
                [cell.keyTextField becomeFirstResponder];
                return;
            }
            
        }
    }
    [self saveData2Mode];
    NSDictionary *dic = [NSDictionary dictionaryWithObjectsAndKeys:@"",@"", nil];
    [_customArray addObject:dic];
    [self.tableView reloadData];
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.tableView scrollToRowAtIndexPath:[NSIndexPath indexPathForRow:self->_customArray.count - 1 inSection:4] atScrollPosition:UITableViewScrollPositionBottom animated:YES];
    });
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.3 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        FillSignProfileCell *cell = [self.tableView cellForRowAtIndexPath:[NSIndexPath indexPathForRow:self->_customArray.count - 1 inSection:4]];
        [cell.keyTextField becomeFirstResponder];
    });
}

- (void)textFieldDidChange:(NSNotification *)info
{
    UITextField *textField = [info object];
    if (textField != nil && textField == self.editingField) {
        _isModified = YES;
    }
}

- (BOOL)emptyDataSetShouldDisplay:(UITableView *)tableView{
    return NO;
}

- (void)initNavbar{
    self.navBarLeftTitle = FSLocalizedForKey(@"kCancel");
//    _contentArray = [self loadPredefineValue];
//    if (_contentArray.count > 0) {
    self.isEditProfile = NO;
//    }else{
//        self.isEditProfile = YES;
//    }
}

- (void)navigationBarBackTouch{

    if (self.selectCall) {
        self.selectCall(self, @"");
    }
    
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (void)setIsEditProfile:(BOOL)isEditProfile{
    _isEditProfile = isEditProfile;
    if (_isEditProfile) {
        [self.navBar.rightItem setTitle:FSLocalizedForKey(@"kDone") forState:UIControlStateNormal];
        @weakify(self)
        self.navBar.rightItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            [self doneClick:_item];
        };
        self.title = FSLocalizedForKey(@"kProfileAddCustomTitle");
    }else{
        [self.navBar.rightItem setTitle:FSLocalizedForKey(@"kEdit") forState:UIControlStateNormal];
        @weakify(self)
        self.navBar.rightItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            [self editClick:_item];
        };
        self.title = FSLocalizedForKey(@"kProfiletitle");
    }
//    self.navBar.leftItem.hidden = _isEditProfile;
}

- (void)doneClick:(FSItem *)item
{
    self.isEditProfile = NO;
    if (self.editingField && self.editingField.isFirstResponder) {
        [self.editingField resignFirstResponder];
        _editingField = nil;
    }
    [self saveData2Mode];
    if ([[NSFileManager defaultManager] fileExistsAtPath:FILLSIGNPROFILE]) {
        [[NSFileManager defaultManager] removeItemAtPath:FILLSIGNPROFILE error:nil];
    }
    dispatch_async(dispatch_get_global_queue(0, 0), ^{
        [self->_modeDatas writeToFile:FILLSIGNPROFILE atomically:YES];
    });
    
    NSMutableArray *deleteArray = [NSMutableArray array];
    for (int i = 0; i < _customArray.count; i++) {
        NSDictionary *tmpDic = [_customArray objectAtIndex:i];
        NSArray *tmpKeys = [tmpDic allKeys];
        NSString *tmpKeystr = [tmpKeys lastObject];

        if (tmpKeystr.length < 1 ){//|| tmpValuestr.length < 1) {
//            [_customArray removeObjectAtIndex:i];
            [deleteArray addObject:tmpDic];
        }
    }
    for (NSDictionary *dic in deleteArray) {
        [_customArray removeObject:dic];
    }
    
    [self.tableView reloadData];
}

- (void)editClick:(FSItem *)item{
    self.isEditProfile = YES;
    [self.tableView reloadData];
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 4 + (_customArray.count > 0 ? 1 : 0);
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    if (section == 0) {
        return _nameArray.count;
    }else if (section == 1){
        return _addArray.count;
    }else if (section == 2){
        return _contactArray.count;
    }else if (section == 3){
        return _dateArray.count;
    }else{
        return _customArray.count;
    }
}


- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    FillSignProfileCell *cell = [[FillSignProfileCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:@"reuseIdentifierProfileCell" isCustomFieldCell:(indexPath.section == 4) isEdit:self.isEditProfile];
    cell.indexPath = indexPath;
    NSString *tipstr;
    if ([_textTips count] > indexPath.section && [[_textTips objectAtIndex:indexPath.section] count] > indexPath.row)  tipstr = _textTips[indexPath.section][indexPath.row];
  
    switch (indexPath.section) {
        case 0:
        {
            NSDictionary *dic = _nameArray[indexPath.row];
            NSArray *tmpKeys = [dic allKeys];
            NSString *tmpKeystr = [tmpKeys lastObject];
            if (tmpKeystr.length > 0) cell.keyLabel.text = FSLocalizedForKey(tmpKeystr);
            
            NSArray *tmpValues = [dic allValues];
            NSString *tmpValuestr = [tmpValues lastObject];
            if (tmpValuestr.length > 0){
                cell.valueTextField.text = tmpValuestr;
                if (!self.isEditProfile) {
                    cell.valueTextField.userInteractionEnabled = NO;
                }
                else
                {
                    cell.valueTextField.userInteractionEnabled = YES;
                }
            }else{
                cell.valueTextField.placeholder = FSLocalizedForKey(tipstr);
                cell.valueTextField.userInteractionEnabled = YES;
            }
            cell.valueTextField.delegate = self;
            cell.valueTextField.accessibilityIdentifier = tipstr;
            break;
        }
        case 1:
        {
            NSDictionary *dic = _addArray[indexPath.row];
            NSArray *tmpKeys = [dic allKeys];
            NSString *tmpKeystr = [tmpKeys lastObject];
            if (tmpKeystr.length > 0) cell.keyLabel.text = FSLocalizedForKey(tmpKeystr);
            
            NSArray *tmpValues = [dic allValues];
            NSString *tmpValuestr = [tmpValues lastObject];
            if (tmpValuestr.length > 0){
                cell.valueTextField.text = tmpValuestr;
                cell.valueTextField.userInteractionEnabled = self.isEditProfile;
            }else{
                cell.valueTextField.placeholder = FSLocalizedForKey(tipstr);
                cell.valueTextField.userInteractionEnabled = YES;
            }
            cell.valueTextField.delegate = self;
            cell.valueTextField.accessibilityIdentifier = tipstr;
            break;
        }
        case 2:
        {
            NSDictionary *dic = _contactArray[indexPath.row];
            NSArray *tmpKeys = [dic allKeys];
            NSString *tmpKeystr = [tmpKeys lastObject];
            if (tmpKeystr.length > 0) cell.keyLabel.text = FSLocalizedForKey(tmpKeystr);
            
            NSArray *tmpValues = [dic allValues];
            NSString *tmpValuestr = [tmpValues lastObject];
            if (tmpValuestr.length > 0){
                cell.valueTextField.text = tmpValuestr;
                cell.valueTextField.userInteractionEnabled = self.isEditProfile;
            }else{
                cell.valueTextField.placeholder = FSLocalizedForKey(tipstr);
                cell.valueTextField.userInteractionEnabled = YES;
            }
            cell.valueTextField.delegate = self;
            cell.valueTextField.accessibilityIdentifier = tipstr;
            break;
        }
        case 3:
        {
            NSDictionary *dic = _dateArray[indexPath.row];
            NSArray *tmpKeys = [dic allKeys];
            NSString *tmpKeystr = [tmpKeys lastObject];
            if (tmpKeystr.length > 0) cell.keyLabel.text = FSLocalizedForKey(tmpKeystr);
            
            NSArray *tmpValues = [dic allValues];
            NSString *tmpValuestr = [tmpValues lastObject];
            if (tmpValuestr.length > 0){
                cell.valueTextField.text = tmpValuestr;
                cell.valueTextField.userInteractionEnabled = self.isEditProfile;
            }else{
                if (indexPath.row == 1) {
                    cell.valueTextField.placeholder = FSLocalizedForKey(tipstr);
                    cell.valueTextField.userInteractionEnabled = YES;
                }
            }
            
            if (indexPath.row == 0) {
                cell.valueTextField.text = [Utility displayDateInMDY:[NSDate date]];
                cell.valueTextField.userInteractionEnabled = NO;
            }
            cell.valueTextField.delegate = self;
            cell.valueTextField.accessibilityIdentifier = tipstr;
            break;
        }
        case 4:
        {
           
            if(self.isEditProfile){
                cell.clearView.hidden = NO;
            }else{
                cell.clearView.hidden = YES;
            }
            @weakify(self)
            ((FSItem *)cell.clearView).onTapClick = ^(FSItem * _Nonnull item) {
                @strongify(self)
                [self->_customArray removeObjectAtIndex:indexPath.row];
                [self reloadData];
//                if(self->_customArray.count > 0){
//                    NSIndexPath *lastIndexPath = [NSIndexPath indexPathForRow:([self.tableView numberOfRowsInSection:4] - 1) inSection:4];
//                    [self.tableView scrollToRowAtIndexPath:lastIndexPath atScrollPosition:UITableViewScrollPositionBottom animated:YES];
//                }

            };
          
            NSDictionary *dic = _customArray[indexPath.row];
            NSArray *tmpKeys = [dic allKeys];
            NSString *tmpKeystr = [tmpKeys lastObject];
            if (tmpKeystr.length > 0){
                cell.keyTextField.text = tmpKeystr;
                cell.keyTextField.userInteractionEnabled = self.isEditProfile;
            }else{
                cell.keyTextField.placeholder = FSLocalizedForKey(@"kProfileAddCustomKey");
                cell.keyTextField.userInteractionEnabled = YES;
            }
            cell.keyTextField.accessibilityIdentifier = @"kProfileAddCustomKey";
            cell.keyTextField.delegate = self;
            [cell.keyTextField addTarget:self action:@selector(changedTextField:) forControlEvents:UIControlEventEditingChanged];
            
            NSArray *tmpValues = [dic allValues];
            NSString *tmpValuestr = [tmpValues lastObject];
            if (tmpValuestr.length > 0){
                cell.valueTextField.text = tmpValuestr;
                cell.valueTextField.userInteractionEnabled = self.isEditProfile;
            }else{
                cell.valueTextField.placeholder = FSLocalizedForKey(@"kProfileAddCustomValue");
                cell.valueTextField.userInteractionEnabled = YES;
            }
            cell.valueTextField.delegate = self;
            cell.valueTextField.accessibilityIdentifier = @"kProfileAddCustomValue";
            break;
        }
        default:
            break;
    }
    
    
    return cell;
}

#pragma mark -  table view delegate handler
- (NSString *)titles1ForAtSection:(NSUInteger)section{
    return self.headerTitles[section];
}


- (void)tableView:(UITableView *)tableView didEndDisplayingCell:(UITableViewCell *)cell forRowAtIndexPath:(NSIndexPath*)indexPath{
    //[self saveData2ModebyIndexPath:indexPath];
}

- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (self.isEditProfile && indexPath.section == 4) {
        if (self.editingField && self.editingField.isFirstResponder) {
            return NO;
        }
        return YES;
    }else{
        return NO;
    }
}

-(void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath
{
    if (editingStyle == UITableViewCellEditingStyleDelete)
    {
        [_customArray removeObjectAtIndex:indexPath.row];
        [self.tableView reloadData];
    }
}

-(NSString *)tableView:(UITableView *)tableView titleForDeleteConfirmationButtonForRowAtIndexPath:(NSIndexPath *)indexPath{
    return FSLocalizedForKey(@"kDelete");
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    FillSignProfileCell *cell = [self.tableView cellForRowAtIndexPath:indexPath];
    __block BOOL  select = NO;
    if ((cell.keyLabel.text.length > 0 || cell.keyTextField.text.length > 0) && cell.valueTextField.text.length > 0) {
        
        if ((indexPath.section == 3 && indexPath.row == 0 && ![IQKeyboardManager sharedManager].isKeyboardShowing) || !self.isEditProfile) {
            [self dismissViewControllerAnimated:YES completion:^{
                if (self.selectCall) {
                    NSString *utlStr = [cell.valueTextField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
                    if (utlStr.length){
                        select = YES;
                        self.selectCall(self, utlStr);
                    }
                }
            }];
        }
    }
    if(!select){
        cell.valueTextField.userInteractionEnabled = YES;
        [cell.valueTextField becomeFirstResponder];
        return;
    }
}

#pragma mark -  textfieldDelegate
- (BOOL)textFieldShouldBeginEditing:(UITextField *)textField
{
    return ((textField.text.length < 1 && !self.isEditProfile) || self.isEditProfile);
}

- (void)textFieldDidBeginEditing:(UITextField *)textField
{
    _editingField = textField;
    self.isEditProfile = YES;
    FillSignProfileCell *cell = [self getCellBytextField:textField];
    if (cell.keyTextField == textField) {
        cell.valueTextField.userInteractionEnabled = NO;
    }
    for (int i = 0;i < _customArray.count; i++) {
        FillSignProfileCell *cell = [self.tableView cellForRowAtIndexPath:[NSIndexPath indexPathForRow:i inSection:4]];
        cell.isEdit = YES;
        cell.clearView.hidden = NO;
        [cell layoutSubviews];
      
    }
}

-(void)changedTextField:(id)textField
{
    UITextField *keyField = (UITextField*)textField;
    FillSignProfileCell *cell = [self getCellBytextField:keyField];
    if (cell.keyTextField == textField) {
        if (keyField.text.length > 0) {
            cell.valueTextField.userInteractionEnabled = YES;
        }
        else
        {
            cell.valueTextField.userInteractionEnabled = NO;
        }
    }
}


- (void)textFieldDidEndEditing:(UITextField *)textField
{
    if (_editingField == textField) {
        _editingField = nil;
    }
    FillSignProfileCell *cell = [self getCellBytextField:textField];
    if (cell.keyTextField == textField) {
        if (textField.text.length > 0) {
            cell.valueTextField.userInteractionEnabled = YES;
        }
        else
        {
            cell.valueTextField.userInteractionEnabled = NO;
        }
    }
    NSIndexPath *indexPath = cell.indexPath;
    [self saveData2ModebyIndexPath:indexPath cell:cell];
}


- (FillSignProfileCell *)getCellBytextField:(UITextField *)textField
{
    UIView *superView = textField.superview;
    while (![superView isKindOfClass:[FillSignProfileCell class]] && superView != nil) {
        superView = superView.superview;
    }
    return (FillSignProfileCell *)superView;
}

-(void) clearItemView:(NSUInteger)IndexPath
{
    [self.customArray removeObjectAtIndex:IndexPath];
    [self reloadData];
}


- (void)dismissViewControllerAnimated:(BOOL)flag completion:(void (^)(void))completion
{
    if (_isModified) {
        
        if (self.updateCall) {
            NSArray *contentArray = [self loadPredefineValue];
            self.updateCall(self, contentArray);
        }
    }
    [super dismissViewControllerAnimated:flag completion:completion];
}

@end
