/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "AnnotationSignature.h"
#import "TitleSignatureAppearanceData.h"

#define DEFAULT_COLOR 0x000000
#define DEFAULT_DIAMETER 5

@implementation AnnotationSignature

- (void)initValue {
    self.name = nil;
    self.rectSigPart = CGRectZero;
    self.color = DEFAULT_COLOR;
    self.diameter = DEFAULT_DIAMETER;
    self.isPic = NO;
    self.userName = @"";
    self.location = @"";
    self.reason = @"";
    self.version = @"";
}

- (id)init {
    self = [super init];
    if (self) {
        // Custom initialization
        [self initValue];
    }
    return self;
}

- (id)copyWithZone:(nullable NSZone *)zone{
    AnnotationSignature *sig = [[self class] allocWithZone:zone];
    sig.name = [self.name copyWithZone:zone];
    sig.contents = [self.contents copyWithZone:zone];
    sig.rectSigPart = self.rectSigPart;
    sig.diameter = self.diameter;
    sig.certFileName = [self.certFileName copyWithZone:zone];
    sig.certPasswd = [self.certPasswd copyWithZone:zone];
    sig.certMD5 = [self.certMD5 copyWithZone:zone];
    sig.pageIndex = self.pageIndex;
    sig.color = self.color;
    sig.opacity = self.opacity;
    sig.rect = self.rect;
    sig.author = [self.author copyWithZone:zone];
    sig.signature = self.signature;
    sig.dib = self.dib;
    sig.data = [self.data copyWithZone:zone];
    sig.isPic = self.isPic;
    sig.data = [self.data copyWithZone:zone];
    sig.appName = [self.appName copyWithZone:zone];
    sig.titleName = [self.titleName copyWithZone:zone];
    sig.reason = [self.reason copyWithZone:zone];
    sig.location = [self.location copyWithZone:zone];
    sig.userName = [self.userName copyWithZone:zone];
    sig.version = [self.version copyWithZone:zone];
    sig.distinguishedName = [self.distinguishedName copyWithZone:zone];
    sig.date = [self.date copyWithZone:zone];
    sig.isReasonOpen = self.isReasonOpen;
    sig.isLocationOpen = self.isLocationOpen;
    sig.isUserNameOpen = self.isUserNameOpen;
    sig.isVersionOpen =  self.isVersionOpen;
    sig.isDistinguishedNameOpen = self.isDistinguishedNameOpen;
    sig.isDateOpen = self.isDateOpen;
    sig.isLablesOpen = self.isLablesOpen;
    sig.isLogoOpen = self.isLogoOpen;
    sig.isClearBg = self.isClearBg;
    return sig;
}

+ (AnnotationSignature *)createWithDefaultOptionForPageIndex:(int)pageIndex rect:(FSRectF *)rect {
    AnnotationSignature *annot = [[AnnotationSignature alloc] init];
    annot.pageIndex = pageIndex;
    annot.rect = rect;
    annot.color = 0;
    annot.opacity = 100;
    annot.contents = @"";
    annot.name = @"FoxitMobilePDFSignature";
    return annot;
}

- (void)encodeWithCoder:(NSCoder *)coder;
{
    [coder encodeObject:self.name forKey:@"name"];
    [coder encodeCGRect:self.rectSigPart forKey:@"rectSigPart"];
    [coder encodeInt:self.color forKey:@"color"];
    [coder encodeInt:self.diameter forKey:@"diameter"];
    [coder encodeObject:self.certFileName forKey:@"certFileName"];
    [coder encodeObject:self.certPasswd forKey:@"certPasswd"];
    [coder encodeObject:self.certMD5 forKey:@"certMD5"];
    [coder encodeBool:self.isPic forKey:@"isPic"];
    
    [coder encodeObject:self.appName forKey:@"appName"];
    [coder encodeObject:self.titleName forKey:@"titleName"];
    [coder encodeObject:self.reason forKey:@"reason"];
    [coder encodeObject:self.location forKey:@"location"];
    [coder encodeObject:self.userName forKey:@"userName"];
    [coder encodeObject:self.version forKey:@"version"];
    [coder encodeObject:self.distinguishedName forKey:@"distinguishedName"];
    [coder encodeObject:self.date forKey:@"date"];
    
    [coder encodeBool:self.isReasonOpen forKey:@"isReasonOpen"];
    [coder encodeBool:self.isLocationOpen forKey:@"isLocationOpen"];
    [coder encodeBool:self.isUserNameOpen forKey:@"isUserNameOpen"];
    [coder encodeBool:self.isVersionOpen forKey:@"isVersionOpen"];
    [coder encodeBool:self.isDistinguishedNameOpen forKey:@"isDistinguishedNameOpen"];
    [coder encodeBool:self.isDateOpen forKey:@"isDateOpen"];
    [coder encodeBool:self.isLablesOpen forKey:@"isLablesOpen"];
    [coder encodeBool:self.isLogoOpen forKey:@"isLogoOpen"];
    [coder encodeBool:self.isClearBg forKey:@"isClearBg"];
}



- (id)initWithCoder:(NSCoder *)decoder {
    self = [super init];
    if (self) {
        [self initValue];
        self.name = [decoder decodeObjectForKey:@"name"];
        self.rectSigPart = [decoder decodeCGRectForKey:@"rectSigPart"];
        self.color = [decoder decodeIntForKey:@"color"];
        self.diameter = [decoder decodeIntForKey:@"diameter"];
        self.certFileName = [decoder decodeObjectForKey:@"certFileName"];
        self.certPasswd = [decoder decodeObjectForKey:@"certPasswd"];
        self.certMD5 = [decoder decodeObjectForKey:@"certMD5"];
        self.isPic = [decoder decodeBoolForKey:@"isPic"];

        self.appName = [decoder decodeObjectForKey:@"appName"];
        self.titleName = [decoder decodeObjectForKey:@"titleName"];
        self.reason = [decoder decodeObjectForKey:@"reason"];
        self.location = [decoder decodeObjectForKey:@"location"];
        self.userName = [decoder decodeObjectForKey:@"userName"];
        self.version = [decoder decodeObjectForKey:@"version"];
        self.distinguishedName = [decoder decodeObjectForKey:@"distinguishedName"];
        self.date = [decoder decodeObjectForKey:@"date"];
      
        self.isReasonOpen = [decoder decodeBoolForKey:@"isReasonOpen"];
        self.isLocationOpen = [decoder decodeBoolForKey:@"isLocationOpen"];
        self.isUserNameOpen = [decoder decodeBoolForKey:@"isUserNameOpen"];
        self.isVersionOpen = [decoder decodeBoolForKey:@"isVersionOpen"];
        self.isVersionOpen = [decoder decodeBoolForKey:@"isVersionOpen"];
        self.isDistinguishedNameOpen = [decoder decodeBoolForKey:@"isDistinguishedNameOpen"];
        self.isDateOpen = [decoder decodeBoolForKey:@"isDateOpen"];
        self.isLablesOpen = [decoder decodeBoolForKey:@"isLablesOpen"];
        self.isLogoOpen = [decoder decodeBoolForKey:@"isLogoOpen"];
        self.isClearBg = [decoder decodeBoolForKey:@"isClearBg"];

    }

    return self;
}

- (void)dealloc {
    self.name = nil;
}

+ (NSMutableArray *)getSignatureArray {
    NSMutableArray *array = nil;
    NSData *data = [[NSUserDefaults standardUserDefaults] valueForKey:SETTING_SIGNATURE];
    if (data && data.length > 0) {
        array = [NSMutableArray arrayWithArray:[NSKeyedUnarchiver unarchiveObjectWithData:data]];
    } else {
        array = [NSMutableArray array];
    }
    return array;
}

+ (void)saveSignature:(NSArray *)array {
    NSData *data = [NSKeyedArchiver archivedDataWithRootObject:array];
    [[NSUserDefaults standardUserDefaults] setObject:data forKey:SETTING_SIGNATURE];
    [[NSUserDefaults standardUserDefaults] synchronize];
}

+ (NSMutableArray *)getSignatureList {
    NSMutableArray *ret = [NSMutableArray array];
    NSArray *array = [AnnotationSignature getSignatureArray];
    [array enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        AnnotationSignature *sig = obj;
        [ret addObject:sig.name];
    }];
   
    BOOL isAmendSigKey = [Preference getBoolValue:FSPreferenceModuleNameSignature type:@"AmendSigKey" delaultValue:NO];
    if(!isAmendSigKey){
        for (NSMutableString * name in ret) {
            AnnotationSignature *sig = [AnnotationSignature getSignature:name];
            if(sig.titleName.length > 0){
                [AnnotationSignature InternationalizeErrorSig:sig];
            }
        }
        [TitleSignatureAppearanceData InternationalizeErrorTitle];
        [Preference setBoolValue:FSPreferenceModuleNameSignature type:@"AmendSigKey" value:YES];
        
    }
    return ret;
}

+ (NSMutableArray *)getCertSignatureList {
    NSMutableArray *ret = [NSMutableArray array];
    NSArray *array = [AnnotationSignature getSignatureArray];
    [array enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        AnnotationSignature *sig = obj;
        if (sig.certMD5 && sig.certPasswd && sig.certFileName) {
            [ret addObject:sig.name];
        }
    }];
    BOOL isAmendSigKey = [Preference getBoolValue:FSPreferenceModuleNameSignature type:@"AmendSigKey" delaultValue:NO];
    if(!isAmendSigKey){
        for (NSMutableString * name in ret) {
            AnnotationSignature *sig = [AnnotationSignature getSignature:name];
            if(sig.titleName.length > 0){
                [AnnotationSignature InternationalizeErrorSig:sig];
            }
        }
        [TitleSignatureAppearanceData InternationalizeErrorTitle];
        [Preference setBoolValue:FSPreferenceModuleNameSignature type:@"AmendSigKey" value:YES];
    }
    return ret;
}

+ (UIImage *)getSignatureImage:(NSString *)name {
    if (name == nil) {
        return nil;
    }
    name = [name stringByAppendingString:@"_i"];
    return [UIImage imageWithContentsOfFile:[SIGNATURE_PATH stringByAppendingPathComponent:name]];
}

+ (NSData *)getSignatureData:(NSString *)name {
    if (name == nil) {
        return nil;
    }
    name = [name stringByAppendingString:@"_i"];
    return [NSData dataWithContentsOfFile:[SIGNATURE_PATH stringByAppendingPathComponent:name]];
}

+ (void)setSignatureImage:(NSString *)name img:(UIImage *)img {
    name = [name stringByAppendingString:@"_i"];
    NSData *data = UIImagePNGRepresentation(img);
    [data writeToFile:[SIGNATURE_PATH stringByAppendingPathComponent:name] atomically:YES];
}

+ (void)setCertFileToSiganatureSpace:(NSString *)name path:(NSString *)path {
    NSFileManager *fileM = [NSFileManager defaultManager];
    NSString *newPath = [SIGNATURE_PATH stringByAppendingPathComponent:name];
    NSError *error;
    if ([fileM fileExistsAtPath:newPath]) {
        [fileM removeItemAtPath:newPath error:&error];
    }
    [fileM copyItemAtPath:path toPath:newPath error:&error];
}

+ (NSData *)getSignatureDib:(NSString *)name {
    name = [name stringByAppendingString:@"_d"];
    return [NSData dataWithContentsOfFile:[SIGNATURE_PATH stringByAppendingPathComponent:name]];
}

+ (void)setSignatureDib:(NSString *)name data:(NSData *)data {
    name = [name stringByAppendingString:@"_d"];
    [data writeToFile:[SIGNATURE_PATH stringByAppendingPathComponent:name] atomically:YES];
}

+ (void)removeSignatureResource:(NSString *)name {
    NSFileManager *file = [[NSFileManager alloc] init];
    NSString *imgPath = [SIGNATURE_PATH stringByAppendingPathComponent:[name stringByAppendingString:@"_i"]];
    NSString *dibPath = [SIGNATURE_PATH stringByAppendingPathComponent:[name stringByAppendingString:@"_d"]];
    [file removeItemAtPath:imgPath error:nil];
    [file removeItemAtPath:dibPath error:nil];
}

+ (AnnotationSignature *)getSignature:(NSString *)name {
    __block AnnotationSignature *ret = nil;
    NSArray *array = [AnnotationSignature getSignatureArray];
    [array enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        AnnotationSignature *sig = obj;
        if ([sig.name isEqualToString:name]) {
            ret = sig;
            *stop = YES;
        }
    }];

    return ret;
}

- (NSString *)add {
    self.name = [Utility getUUID];

    NSMutableArray *array = [AnnotationSignature getSignatureArray];
    [array addObject:self];

    [AnnotationSignature saveSignature:array];
    if (array.count == 1) {
        [AnnotationSignature setSignatureSelected:self.name];
    }

    return self.name;
}

- (void)update {
    NSMutableArray *array = [AnnotationSignature getSignatureArray];
    for (int i = 0; i < array.count; i++) {
        AnnotationSignature *sig = [array objectAtIndex:i];
        if ([sig.name isEqualToString:self.name]) {
            [array replaceObjectAtIndex:i withObject:self];
            break;
        }
    }

    [AnnotationSignature saveSignature:array];
}

- (void)remove {
    NSMutableArray *array = [AnnotationSignature getSignatureArray];
    for (int i = 0; i < array.count; i++) {
        AnnotationSignature *sig = [array objectAtIndex:i];
        if ([sig.name isEqualToString:self.name]) {
            [array removeObject:sig];
            break;
        }
    }

    [AnnotationSignature saveSignature:array];

    NSString *selectedName = [AnnotationSignature getSignatureSelected];
    if ([selectedName isEqualToString:self.name]) {
        if (array.count == 0) {
            [AnnotationSignature setSignatureSelected:@""];
        } else {
            AnnotationSignature *sig = [array objectAtIndex:0];
            [AnnotationSignature setSignatureSelected:sig.name];
        }
    }
}

+ (AnnotationSignature *)getSignatureOption {
    AnnotationSignature *option = nil;
    NSData *data = [[NSUserDefaults standardUserDefaults] valueForKey:SETTING_SIGNATURE_OPTION];
    if (data && data.length > 0) {
        option = [NSKeyedUnarchiver unarchiveObjectWithData:data];
    } else {
        option = [[AnnotationSignature alloc] init];
        option.color = DEFAULT_COLOR;
        option.diameter = DEFAULT_DIAMETER;
    }

    return option;
}

- (void)setOption {
    NSData *data = [NSKeyedArchiver archivedDataWithRootObject:self];
    [[NSUserDefaults standardUserDefaults] setObject:data forKey:SETTING_SIGNATURE_OPTION];
    [[NSUserDefaults standardUserDefaults] synchronize];
}

+ (NSString *)getSignatureSelected {
    return [[NSUserDefaults standardUserDefaults] valueForKey:SETTING_SIGNATURE_SELECTED];
}

+ (void)setSignatureSelected:(NSString *)name {
    [[NSUserDefaults standardUserDefaults] setObject:name forKey:SETTING_SIGNATURE_SELECTED];
    [[NSUserDefaults standardUserDefaults] synchronize];
}

- (bool) isSignInfoEmpty{
    bool isEmpty = true;
    if(self.userName.length > 0 || self.location.length > 0 || self.reason.length > 0 || self.distinguishedName.length > 0 || self.version.length > 0 || self.isDateOpen){
        isEmpty = false;
    }
    return isEmpty;
}

+ (UIImage *) renderSignature:(FSRectF *) rect signature:(AnnotationSignature *) sign bitmap:(FSBitmap *) bitmap isImage:(bool) isImage isNewTime:(bool) isNewDate{
    UIImage *img  = [self addSign:rect signature:sign bitmap:bitmap isImage:isImage isNewTime:isNewDate];
 //    UIImage *imgNew = img;
 //    if (self.clearWhiteBtn.on) imgNew = [img fs_replaceColor:[UIColor colorWithRed:1.0 green:1.0 blue:1.0 alpha:1.0] withTolerance:1.0];
     NSString* userSavePath = TEMP_PATH;
     userSavePath = [userSavePath stringByAppendingPathComponent:@"temp.png"];
     [UIImagePNGRepresentation(img) writeToFile:userSavePath atomically:YES];
    return img;
}

+ (UIImage *)addSign:(FSRectF *) viewRect signature:(AnnotationSignature *) sign bitmap:(FSBitmap *) bitmap
             isImage:(bool) isImage isNewTime:(bool) isNewDate{
    NSString *imagePath = [SIGNATURE_PATH stringByAppendingPathComponent:[@"temp" stringByAppendingString:@"_i"]];
    NSString* userSavePath = TEMP_PATH;
    userSavePath = [userSavePath stringByAppendingPathComponent:@"temp.pdf"];
    NSFileManager *fileManager = [NSFileManager defaultManager];
    if ([fileManager fileExistsAtPath:userSavePath]) {
        [fileManager removeItemAtPath:userSavePath error:nil];
    }
    [fileManager createFileAtPath:userSavePath contents:nil attributes:nil];
    
    FSPDFDoc *doc = [[FSPDFDoc alloc]  init ];
    FSPDFPage *pdfPage =  [doc insertPage:0 width:viewRect.width height:viewRect.height];
    DIGITALSIGNATURE_PARAM *param = [[DIGITALSIGNATURE_PARAM alloc] init];
    param.certFile = [SIGNATURE_PATH stringByAppendingPathComponent:sign.certMD5];
    param.certPwd = sign.certPasswd;
    param.subfilter = @"adbe.pkcs7.detached";
    param.imagePath = imagePath;
    param.rect = viewRect;
    param.sigName = sign.name;
    param.signFilePath = userSavePath;
    FSSignature *signature = [self createSignature:pdfPage withParam:param signature:sign bitmap:bitmap isNewTime:isNewDate];
    [self signSignature:signature withParam:param];
    [pdfPage startParse:1 pause:nil is_reparse:false];
    UIImage *img = nil;
//    if(!isImage){
        FSBitmap *tempBitmap = [[FSBitmap alloc] initWithWidth:viewRect.width*[UIScreen mainScreen].scale height:viewRect.height*[UIScreen mainScreen].scale format:FSBitmapDIBArgb];
        FSMatrix2D *mat = [pdfPage getDisplayMatrix:0 top:0 width:tempBitmap.getWidth height:tempBitmap.getHeight rotate:0];
        FSRenderer *render = [[FSRenderer alloc] initWithBitmap:tempBitmap is_rgb_order:YES];
        [render startRender:pdfPage matrix:mat pause:nil];
        img = [Utility  bitmapToUIImageNoColorChannel:tempBitmap];
        
//    }else{
//        const CGColorSpaceRef rgb = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);
//        const CGBitmapInfo bmi = (kCGBitmapByteOrderDefault | kCGImageAlphaNoneSkipLast);
//        CGContextRef bmc = CGBitmapContextCreate(nil, (int)(viewRect.width*[UIScreen mainScreen].scale), (int)(viewRect.height*[UIScreen mainScreen].scale), 8, (((int)(viewRect.width*[UIScreen mainScreen].scale)  * 4 + (64 - 1)) / 64) * 64, rgb, bmi); // Render thumb
//        void *data = CGBitmapContextGetData(bmc);
//        if (!data) {
//            CGContextRelease(bmc);
//            CGColorSpaceRelease(rgb);
//            return nil;
//        }
//        FSRenderer *fsrenderer = nil;
//        @try {
//            fsrenderer = [[FSRenderer alloc] initWithContext:bmc device_type:FSRendererDeviceDisplay];
//        } @catch (NSException *exception) {
//            CGContextRelease(bmc);
//            CGColorSpaceRelease(rgb);
//            return nil;
//        }
//        UIColor *backgroundColor = [UIColor whiteColor];
//        CGContextSetRGBFillColor(bmc, backgroundColor.red, backgroundColor.green, backgroundColor.blue, 1);
//        CGContextFillRect(bmc, CGRectMake(0, 0, viewRect.width*[UIScreen mainScreen].scale, viewRect.height*[UIScreen mainScreen].scale));
//        [fsrenderer setClipRect:[[FSRectI alloc] initWithLeft1:0 top1:0 right1:viewRect.width*[UIScreen mainScreen].scale bottom1:viewRect.height*[UIScreen mainScreen].scale]];
//        FSMatrix2D *mat = [pdfPage getDisplayMatrix:0 top:0 width:viewRect.width*[UIScreen mainScreen].scale height:viewRect.height*[UIScreen mainScreen].scale rotate:0];
//        [fsrenderer startRender:pdfPage matrix:mat pause:nil];
//        CGImageRef image = CGBitmapContextCreateImage(bmc);
//        img = [UIImage imageWithCGImage:image]; //todel
//        if (img) {
//            fsrenderer = nil;
//            mat = nil;
//            pdfPage = nil;
//        }
//        CGImageRelease(image);
//        CGContextRelease(bmc);
//        CGColorSpaceRelease(rgb);
//    }
//    [extensionsManager onDigitalSignatureSigned:isSuccess originalDoc:[signature getDocument] outputPath:userSavePath];
    [doc removeSignature:signature];
    return  img;
    
}


+ (FSSignature *)createSignature:(FSPDFPage *)page withParam:(DIGITALSIGNATURE_PARAM *)param signature:(AnnotationSignature *)sig bitmap:(FSBitmap *)bitmap isNewTime:(bool) isNewDate{
    FSSignature *sign = NULL;
    FSRectF *rect = param.rect;
    
    //Add an unsigned signature field with blank appearance on the specified position of page.
    sign = [page addSignature:rect];
    if (nil == sign)
        return nil;
    
    sign.filter = @"Adobe.PPKLite";
    sign.subFilter = param.subfilter;
    
    [sign setKeyValue:FSSignatureKeyNameSigner value:sig.userName];
    [sign setKeyValue:FSSignatureKeyNameLocation value:sig.location];
    [sign setKeyValue:FSSignatureKeyNameReason value:FSLocalizedForKey(sig.reason)];
    //        [sign setKeyValue:FSSignatureKeyNameContactInfo value:@""];
    [sign setKeyValue:FSSignatureKeyNameDN value:sig.distinguishedName];
    NSString *str = @"";
    if(sig.isVersionOpen){
        if (sig.isLablesOpen){
            str = sig.appName;
            str = [str stringByAppendingString:@" "];
            str = [str stringByAppendingString:FSLocalizedForKey(@"kAppearanceVersion")];
            str = [str stringByAppendingString:sig.version];
        }else{
            str = sig.version;
        }
    }
    [sign setKeyValue:FSSignatureKeyNameProducer value:str];
    [sign setKeyValue:FSSignatureKeyNameText value:@"text"];
    FSDateTime *time;
    if (!isNewDate && sig.isDateOpen && sig.date != nil){
        time = [Utility convert2FSDateTime:sig.date];
    }else{
        NSDate *now = [NSDate date];
        time = [Utility convert2FSDateTime:now];
    }
    [sign setSignTime:time];
    
    FSControl *control = [sign getControl:0];
    if (control == nil || control.isEmpty) {
        return nil;
    }
    FSWidget *widget = [control getWidget];
    if (widget == nil || widget.isEmpty) {
        return nil;
    }
   
//    FSRotation rotation = (FSRotation )(([extensionsManager.pdfViewCtrl getViewRotation] + [page getRotation]) % 4);
//    [widget setMKRotation:rotation];
//    [widget setNM:[Utility getUUID]];
//
    int flags = FSSignatureAPFlagBitmap;
    
    
    if(sig.isUserNameOpen){
        flags = flags | FSSignatureAPFlagSigner;
    }
    if(sig.isReasonOpen){
        flags =flags | FSSignatureAPFlagReason;
    }
    if(sig.isLocationOpen){
        flags = flags | FSSignatureAPFlagLocation;
    }
    if(sig.isDistinguishedNameOpen){
        flags = flags | FSSignatureAPFlagDN;
    }
    if(sig.isDateOpen){
        flags = flags | FSSignatureAPFlagSigningTime;
    }
    if(sig.isLablesOpen){
        flags = flags | FSSignatureAPFlagLabel;
    }
    if(sig.isLogoOpen){
        flags = flags | FSSignatureAPFlagFoxitEditorFlag;
    }
    if(sig.isVersionOpen){
        flags = flags | FSSignatureAPFlagProducer;
    }
    
    [sign setAppearanceFlags:flags];
    NSData *data = nil;
    if(param.sigName == nil){
        data = sig.data;
    }else{
        data = [AnnotationSignature getSignatureData:param.sigName];
    }

    [sign setBitmap:bitmap];

    return sign;
}
+ (BOOL)signSignature:(FSSignature *)sign withParam:(DIGITALSIGNATURE_PARAM *)param {
    @try {
        FSProgressive *ret = [sign startSign:param.certFile cert_password:param.certPwd digest_algorithm:FSSignatureDigestSHA1 save_path:param.signFilePath client_data:nil pause:nil];
        
        if (ret != nil) {
            FSProgressiveState state = [ret resume];
            while (FSProgressiveToBeContinued == state) {
                state = [ret resume];
            }
            return FSProgressiveFinished == state;
        } else {
            return YES;
        }
    } @catch (NSException *exception) {
        return NO;
    }
    
}

//reason Repair of internationalized storage not processed
+ (void)InternationalizeErrorSig:(AnnotationSignature *)sig {
    NSArray *localizations = [[NSBundle mainBundle] localizations];
    NSArray * reasonArray = [NSArray arrayWithObjects:@"kReasonAuthor",@"kReasonReview",@"kReasonApprove",@"kReasonApproveLegallyBinding",@"kReasonAccuracyIntegrity",@"kReasonSignTerms",@"kReasonSpecified", nil];
    for (NSString *languageCode in localizations) {
        if([@"" isEqualToString:sig.reason] && sig.isReasonOpen){
            sig.reason = @"kReasonAuthor";
            [sig update];
        }else{
            if(sig.isReasonOpen && ![reasonArray containsObject:sig.reason]){
                for (NSString * item in reasonArray) {
                    NSString *reason = FSLocalizedForKeyWithLanguage(item,languageCode);
                    if([sig.reason isEqualToString:reason]){
                        sig.reason = item;
                        [sig update];
                        break;
                    }
                }
            }
        }
        if(![sig.titleName isEqualToString:@"kAppearanceStandardStyle"]){
            NSString *titleName = FSLocalizedForKeyWithLanguage(@"kAppearanceStandardStyle",languageCode);
            if([sig.titleName isEqualToString:FSLocalizedForKey(titleName)]){
                sig.titleName = @"kAppearanceStandardStyle";
                [sig update];
                break;
            }
        }
    }
}

@end
