/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "SignatureAppearanceCellTableViewCell.h"
#import "FSTitleEstimatedTableViewCell.h"

@interface SignatureAppearanceCellTableViewCell ()
@property (nonatomic, strong, readwrite) UILabel *rightLable;
@property (nonatomic, assign, readwrite) BOOL installed;
@end
@implementation SignatureAppearanceCellTableViewCell
@synthesize installed = _installed;

- (UILabel *)rightLable{
        if (!_rightLable) {
            _rightLable = [UILabel new];
            _rightLable.numberOfLines = 0;
            _rightLable.textAlignment = NSTextAlignmentRight;
            _rightLable.textColor = self.titleDefaultTextColor;
            _rightLable.font = self.titleDefaultFont;
    
            [_rightLable setContentHuggingPriority:ESTIMATED_HORIZONTAL_HEUGGING_PRIORITY forAxis:UILayoutConstraintAxisHorizontal];
            [_rightLable setContentCompressionResistancePriority:ESTIMATED_HORIZONTAL_COMPRESSION_RESISTANCE_PRIORITY forAxis:UILayoutConstraintAxisHorizontal];
            [self.contentView addSubview:_rightLable];
        }
    return  _rightLable;
}


- (void)installConstraints{
    [super installConstraints];
    if (_installed) return;
    !self.willInstallConstraints ? : self.willInstallConstraints(self);
    if(self.rightView){
        [self.rightLable mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.right.mas_equalTo(self.rightView.mas_left).offset(-self.rightMargin).priorityHigh();
            make.top.fs_equalToSuperview().offset(self.topMargin).priorityHigh();
            make.bottom.fs_equalToSuperview().offset(-self.bottomMargin).priorityHigh();
            make.height.mas_greaterThanOrEqualTo(TITLE_LABEL_MIN_HEIGHT).priorityHigh();
            make.width.mas_equalTo(250);
        }];
    }else{
        [self.rightLable mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.right.mas_equalTo(self.rightLable.superview.mas_right).offset(-self.rightMargin).priorityHigh();
            make.top.fs_equalToSuperview().offset(self.topMargin).priorityHigh();
            make.bottom.fs_equalToSuperview().offset(-self.bottomMargin).priorityHigh();
            make.height.mas_greaterThanOrEqualTo(TITLE_LABEL_MIN_HEIGHT).priorityHigh();
            make.width.mas_equalTo(250);
        }];
    }

    _installed = YES;
    [self.rightLable layoutIfNeeded];
    if (self.rightLable.fs_width) self.rightLable.preferredMaxLayoutWidth = self.rightLable.fs_width;
    !self.didInstallConstraints ? : self.didInstallConstraints(self);

    
}
- (void)reinstallConstraints{
    
    _installed = NO;
    [self installConstraints];
}
- (void)awakeFromNib {
    [super awakeFromNib];
    // Initialization code
    
    [self initialize];
}

- (void)prepareForReuse{
    [super prepareForReuse];

    LabelAttributeAssign(self.rightLable, self.titleDefaultTextColor, self.titleDefaultFont);
}

@end
