/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "SignatureAppearanceLocationViewController.h"

@interface SignatureAppearanceLocationViewController ()
@property (nonatomic, strong) FSToolbar *topBar;

@property (nonatomic, strong) UIView *location2View;
@property (nonatomic, strong) UITextField *textLocaltion;
@property (nonatomic, strong) UISwitch *localtionSwitch;

@end

@implementation SignatureAppearanceLocationViewController

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    if (self = [super init]) {
        self.extensionsManager = extensionsManager;
    }
    return self;
}
- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager locationOpen:(BOOL)isLocationOpen location:(NSString *)location {
    if (self = [super init]) {
        self.extensionsManager = extensionsManager;
        self.isLocationOpen = isLocationOpen;
        self.location = location;
    }
    return self;
}
    

- (void)viewDidLoad {
    [super viewDidLoad];
    self.view.backgroundColor = self.fs_backgroundColor_B9;
    self.topBar = [[FSToolbar alloc] initWithStyle  :FSToolbarStyleDefault];
//    self.topBar.backgroundColor = self.fs_backgroundColor_B9;
    [self.navBar addSubview:self.topBar];
    self.title = FSLocalizedForKey(@"kAppearanceLocation");
    self.navBar.hasDivide = YES;
    @weakify(self)
    self.navBar.leftItem.onTapClick = ^(FSItem *_item) {
        @strongify(self)
        [self backClick];
    };
    UIView *locationView = [[UIView alloc] init];
    [self.view addSubview:locationView];
    [locationView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(locationView.superview.mas_left);
        make.right.mas_equalTo(locationView.superview.mas_right);
        make.top.mas_equalTo(self.navBar.mas_bottom);
        make.height.mas_equalTo(44);
    }];
    locationView.backgroundColor = self.fs_backgroundColor_B8;

    UILabel *lbLocation = [[UILabel alloc] init];
    [locationView addSubview:lbLocation];
    [lbLocation mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(locationView.fs_mas_left).offset(16);
        make.top.mas_equalTo(locationView.mas_top);
        make.bottom.mas_equalTo(locationView.mas_bottom);
    }];
    lbLocation.text = FSLocalizedForKey(@"kAppearanceLocation");
    lbLocation.font = self.fs_standardGeneralFont_GF2;
    lbLocation.textAlignment = NSTextAlignmentCenter;
    lbLocation.textColor = self.fs_textColor_T4;

    self.localtionSwitch = [[UISwitch alloc] init];
    BindThemeKeyPath(self.localtionSwitch, onTintColor);
    [self.localtionSwitch addTarget:self action:@selector(changeOption:) forControlEvents:UIControlEventTouchUpInside];
    
    [locationView addSubview:self.localtionSwitch];
    [self.localtionSwitch mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(self.localtionSwitch.superview.fs_mas_right).offset(-16);
        make.centerY.equalTo(locationView);
    }];
 
    self.location2View = [[UIView alloc] init];
    [self.view addSubview:self.location2View];
    [self.location2View mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.location2View.superview.mas_left);
        make.right.mas_equalTo(self.location2View.superview.mas_right);
        make.top.mas_equalTo(locationView.fs_mas_bottom).offset(8);
        make.height.mas_equalTo(44);
    }];
    self.location2View.backgroundColor = self.fs_backgroundColor_B8;

    UILabel *lbLocation2 = [[UILabel alloc] init];
    [self.location2View addSubview:lbLocation2];
    [lbLocation2 mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.location2View.fs_mas_left).offset(16);
        make.centerY.mas_equalTo(self.location2View);
        
    }];
    lbLocation2.text = FSLocalizedForKey(@"kAppearanceLocation");
    lbLocation2.font = self.fs_standardGeneralFont_GF2;
    lbLocation2.textColor = self.fs_textColor_T4;
    [lbLocation2 setContentHuggingPriority:UILayoutPriorityDefaultHigh forAxis:UILayoutConstraintAxisHorizontal];
    self.textLocaltion= [[UITextField alloc] init];
    self.textLocaltion.font = self.fs_standardGeneralFont_GF2;
    self.textLocaltion.textColor = self.fs_textColor_T4;
    self.textLocaltion.borderStyle = UITextBorderStyleNone;
    self.textLocaltion.placeholder = FSLocalizedForKey(@"kAppearanceLocationHint");
    self.textLocaltion.accessibilityIdentifier = @"signature_location_tv_id";
    	
    //self.selectionStyle = UITableViewCellSelectionStyleNone;
    [self.location2View addSubview:self.textLocaltion];
    [self.textLocaltion mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(lbLocation2.fs_mas_right).offset(10);
        make.right.mas_equalTo(self.textLocaltion.superview.fs_mas_right).offset(-16);
        
        make.centerY.mas_equalTo(self.location2View);
    }];
}

- (void)viewWillAppear:(BOOL)animated{
    if(self.isLocationOpen){
        [self.localtionSwitch setOn:YES];
        self.localtionSwitch.accessibilityIdentifier = @"signature_location_swtich_open_id";
        self.location2View.hidden = false;
       
    }else{
        [self.localtionSwitch setOn:NO];
        self.localtionSwitch.accessibilityIdentifier = @"signature_location_swtich_close_id";
        self.location2View.hidden = true;
    }
    if([self.location isKindOfClass:[NSString class]] || [self.location length] > 0 ){
        self.textLocaltion.text = self.location;
    }
}

-(void)backClick{
    [self dismissViewControllerAnimated:NO completion:^{
        nil;
    }];
    if (self.delegate) {
        self.location = self.textLocaltion.text;
        [self.delegate locationBack:self.isLocationOpen location:self.location];
    };
}

- (void)changeOption:(UISwitch *)sender{
    self.isLocationOpen = sender.isOn;
    if(self.isLocationOpen){
        self.location2View.hidden = false;
        self.localtionSwitch.accessibilityIdentifier = @"signature_location_swtich_open_id";
    }else{
        self.location2View.hidden = true;
        self.localtionSwitch.accessibilityIdentifier = @"signature_location_swtich_close_id";
    }
}


@end
