/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "SignatureAppearanceReasonViewController.h"
#import "SignatureAppearanceModel.h"

@interface SignatureAppearanceReasonViewController ()

@property (nonatomic, strong) FSToolbar *topBar;
@property (nonatomic, strong) UIView *reasonView;
@property (nonatomic, strong) UISwitch *reasonSwitch;
@property (nonatomic, strong) NSMutableArray *itemArray;
@property (nonatomic, assign) int oldIndex;
@property (nonatomic, strong) FSTableViewController *fsTableView;

@end

@implementation SignatureAppearanceReasonViewController 

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    if (self = [super init]) {
        self.extensionsManager = extensionsManager;
    }
    return self;
}
- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager reasonOpen:(BOOL)isReasonOpen reason:(NSString *)reason {
    if (self = [super init]) {
        self.extensionsManager = extensionsManager;
        self.isReasonOpen = isReasonOpen;
        self.reason = reason;
    }
    return self;
}


- (void)viewDidLoad {
    [super viewDidLoad];
    self.tableView.backgroundColor = self.fs_backgroundColor_B9;
    self.topBar = [[FSToolbar alloc] initWithStyle  :FSToolbarStyleDefault];
//    self.topBar.backgroundColor = self.fs_backgroundColor_B9;
    [self.navBar addSubview:self.topBar];
    self.title = FSLocalizedForKey(@"kAppearanceReason");
    self.navBar.hasDivide = YES;
    @weakify(self)
    self.navBar.leftItem.onTapClick = ^(FSItem *_item) {
        @strongify(self)
        [self backClick];
    };
    self.reasonSwitch = [[UISwitch alloc] init];
    BindThemeKeyPath(self.reasonSwitch, onTintColor);
    [self.reasonSwitch addTarget:self action:@selector(changeOption:) forControlEvents:UIControlEventTouchUpInside];
    
}

- (void)viewWillAppear:(BOOL)animated{
    if(self.isReasonOpen){
        [self loadData];
        [self.reasonSwitch setOn:YES];
        self.reasonSwitch.accessibilityIdentifier = @"signature_reason_switch_open_id";
    }else{
        [self.reasonSwitch setOn:NO];
        self.reasonSwitch.accessibilityIdentifier = @"signature_reason_switch_close_id";
    }
}
 
- (void)loadData{
    SignatureAppearanceModel *model;
    if(![self.reason isKindOfClass:[NSString class]] || self.reason.length == 0 || [self.reason isEqualToString:@"kReasonAuthor"]){
        model = [[SignatureAppearanceModel alloc ] initWithAppearance:true name:@"kReasonAuthor"];
        self.oldIndex = 0;
        self.reason = @"kReasonAuthor";
    }else{
        model = [[SignatureAppearanceModel alloc ] initWithAppearance:false name:@"kReasonAuthor"];
    }

    SignatureAppearanceModel *model2 = [[SignatureAppearanceModel alloc ] initWithAppearance:false name:@"kReasonReview"];
    SignatureAppearanceModel *model3 = [[SignatureAppearanceModel alloc ] initWithAppearance:false name:@"kReasonApprove"];
    SignatureAppearanceModel *model4 = [[SignatureAppearanceModel alloc ] initWithAppearance:false name:@"kReasonApproveLegallyBinding"];
    SignatureAppearanceModel *model5 = [[SignatureAppearanceModel alloc ] initWithAppearance:false name:@"kReasonAccuracyIntegrity"];
    SignatureAppearanceModel *model6 = [[SignatureAppearanceModel alloc ] initWithAppearance:false name:@"kReasonSignTerms"];
    SignatureAppearanceModel *model7 = [[SignatureAppearanceModel alloc ] initWithAppearance:false name:@"kReasonSpecified"];
    self.itemArray = [[NSMutableArray alloc] init];
    [self.itemArray addObject:model];
    [self.itemArray addObject:model2];
    [self.itemArray addObject:model3];
    [self.itemArray addObject:model4];
    [self.itemArray addObject:model5];
    [self.itemArray addObject:model6];
    [self.itemArray addObject:model7];
    if(self.reason.length != 0){
        for (int i = 0; i < self.itemArray.count; ++i){
            SignatureAppearanceModel *item = self.itemArray[i];
            if([self.reason isEqualToString:item.name]){
                item.select = true;
                self.oldIndex = i;
            }
        }
    }
    self.dataSource=self.itemArray;
}

- (void)backClick{
    [self dismissViewControllerAnimated:NO completion:nil];
    if (self.delegate) {
        [self.delegate  reasonBack:self.isReasonOpen reason:self.reason];
    };
}

- (void)changeOption:(UISwitch *)sender{
    self.isReasonOpen = sender.isOn;
    if(self.isReasonOpen){
        self.reasonSwitch.accessibilityIdentifier = @"signature_reason_switch_open_id";
        [self loadData];
    }else{
        self.itemArray=nil;
        self.reasonSwitch.accessibilityIdentifier = @"signature_reason_switch_close_id";
        [self reloadData];
    }
}

#pragma mark -  table view handler
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView{
    return 1;
}
- (NSString *)titleForEmptyDataSet{
    return @"";
}

- (SignatureAppearanceModel *)dataForAtIndexPath:(NSIndexPath *)indexPath{
    return self.itemArray[indexPath.row];
}

- (void)configureCell:(FSTitleEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(SignatureAppearanceModel *)dataItem{
    cell.backgroundColor = self.fs_backgroundColor_B8;
    if (!cell.rightView) {
        FSItem* selectItem = [FSItem createItemWithImage:FSImageNamed(@"common_select_check")];
        [BindThemeKeySel(selectItem, setImage:forState:) fs_updateThemeCompleted:^(FSItem*  _Nonnull target, SEL  _Nonnull sel) {
            [target setImage:[FSImageNamed(@"common_select_check") fs_imageWithTintColor:target.fs_primaryColor] forState:UIControlStateNormal];
        }];
        selectItem.accessibilityIdentifier = @"select_check_item_id";
        cell.rightView = selectItem;
    }
    if(dataItem.select){
        cell.rightView.hidden = false;
    }else{
        cell.rightView.hidden = true;
    }
    cell.estimatedTitleLabel.text = FSLocalizedForKey(dataItem.name);
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{
    [super tableView:tableView didSelectRowAtIndexPath:indexPath];
    if(self.oldIndex != -1){
        SignatureAppearanceModel *oldModel = self.itemArray[self.oldIndex];
        oldModel.select = false;
    }
    self.oldIndex = (int)indexPath.row;
    SignatureAppearanceModel *model = self.itemArray[indexPath.row];
    model.select = true;
    self.reason = model.name;
    [self reloadData];
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section{
    return 54;
}

- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section{
    UIView * headView = [[UIView alloc] init];
    headView.backgroundColor = self.fs_backgroundColor_B8;
    UIView * view = [[UIView alloc] init];
    [headView addSubview:view];
    [view mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(view.superview.mas_left);
        make.right.mas_equalTo(view.superview.mas_right);
        make.height.mas_equalTo(44);
    }];

    UILabel *lbReason = [[UILabel alloc] init];
    [view addSubview:lbReason];
    [lbReason mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(view.fs_mas_left).offset(16);
        make.top.mas_equalTo(view.mas_top);
        make.bottom.mas_equalTo(view.mas_bottom);
    }];
    lbReason.text = FSLocalizedForKey(@"kAppearanceReason");
    lbReason.font = self.fs_standardGeneralFont_GF2;
    lbReason.textColor = self.fs_textColor_T4;

    [view addSubview:self.reasonSwitch];
    [self.reasonSwitch mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(self.reasonSwitch.superview.fs_mas_right).offset(-16);
        make.centerY.equalTo(view);
    }];
    UIView * cutView = [[UIView alloc] init];
    [headView addSubview:cutView];
    cutView.backgroundColor = self.fs_backgroundColor_B9;
    [cutView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(cutView.superview.mas_left);
        make.right.mas_equalTo(cutView.superview.mas_right);
        make.height.mas_equalTo(10);
        make.bottom.mas_equalTo(headView.mas_bottom);
    }];
    return headView;
}

@end
