/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "SignatureAppearanceTitleViewController.h"
#import "SignatureAppearanceModel.h"
#import "TitleSignatureAppearanceData.h"
#import "AnnotationSignature.h"

@interface SignatureAppearanceTitleViewController ()

@property (nonatomic, strong) NSMutableArray *itemArray;
@property (nonatomic, strong) NSString *editedTitle;

@end

@implementation SignatureAppearanceTitleViewController

-(void)initialize{
    [super initialize];
    self.sectionStyle = FSTableViewSectionStyleLikeGrouped;
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    if (self = [super init]) {
        self.extensionsManager = extensionsManager;
    }
    return self;
}
- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager title:(NSString *)titleName editedTitle:editedTitle {
    if (self = [super init]) {
        self.extensionsManager = extensionsManager;
        self.titleName = titleName;
        self.editedTitle = editedTitle;
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
   
    self.tableView.backgroundColor = self.fs_backgroundColor_B9;
    self.title = FSLocalizedForKey(@"kAppearanceType");
    self.navBar.hasDivide = YES;
    @weakify(self)
    self.navBar.leftItem.onTapClick = ^(FSItem *_item) {
        @strongify(self)
        [self cancelBack];
    };

    [self loadData];
  
}

-(void)loadData{
    self.itemArray = [[NSMutableArray alloc] init];
    NSMutableArray *titleArrray = [TitleSignatureAppearanceData getTitleSignatureNames];
    for (NSString *titleName in titleArrray) {
        SignatureAppearanceModel *model = nil;
        if([self.titleName isEqualToString:titleName]){
            model = [[SignatureAppearanceModel alloc ] initWithAppearance:true name:titleName];
            self.titleName = titleName;
        }else{
            model = [[SignatureAppearanceModel alloc ] initWithAppearance:false name:titleName];
        }
        [self.itemArray addObject:model];
    }
    self.dataSource = self.itemArray;
}


-(void)cancelBack{
    [self dismissViewControllerAnimated:NO completion:^{
        nil;
    }];
    if (self.delegate) {
        [self.delegate titleBack:self.titleName];
    };
}

- (void) moreDeleteTitle:(FSItem *)fsItem indexPath:(NSIndexPath *)indexPath{
    NSMutableArray *items = @[].mutableCopy;

    [items addObject:[[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kDelete") image:FSImageNamedAndNormalColor(@"common_delete_gray") action:^(FSPopoverItem * _Nonnull item, int index) {
        SignatureAppearanceModel *model = self.itemArray[indexPath.row];
        if(self.editedTitle && [self.editedTitle isEqualToString:model.name]){
            [self deleteItemClickWarning];
        }else{
            [self deleteItemClick:fsItem indexPath:indexPath];
        }
    }]];
    
    [self.extensionsManager.viewCtrlManager showMoreMenuWithItems:items.copy dismissCompletion: nil  sourceView:!DEVICE_iPHONE ? ^UIView * _Nonnull(UIPopoverPresentationController * _Nonnull popver) {
        popver.permittedArrowDirections = UIPopoverArrowDirectionRight;
        return fsItem;
    } : nil];
        
}

- (void)deleteItemClick:(FSItem *)item indexPath:(NSIndexPath *)indexPath
{
    NSString *title = FSLocalizedForKey(@"kConfirm");
    NSString *msg = FSLocalizedForKey(@"kAppearanceDeleteStyle");
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:title message:msg preferredStyle:UIAlertControllerStyleAlert];
    @weakify(self)
    UIAlertAction *okAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        @strongify(self)
        SignatureAppearanceModel *model = self.itemArray[indexPath.row];
        NSMutableArray *sigArray =[AnnotationSignature getCertSignatureList];
        for (NSString * name in sigArray) {
            AnnotationSignature *sig = [AnnotationSignature getSignature:name];
             if([model.name isEqualToString:sig.titleName]){
                 [sig remove];
            }
        }
        NSMutableArray *array = [TitleSignatureAppearanceData getTitleSignatureData];
        for (AnnotationSignature *sig in array) {
            if([model.name isEqualToString:sig.titleName]){
                [array removeObject:sig];
                [TitleSignatureAppearanceData insertTitleSignatureData:array];
                break;
            }
        }
        [self.itemArray removeObject:model];
        if(model.select){
            NSInteger selectPosition = indexPath.row > 0 ? indexPath.row -1 : 0;
            [self.itemArray[selectPosition] setSelect:YES];
            SignatureAppearanceModel * selectModel = self.itemArray[selectPosition];
            self.titleName = selectModel.name;
            [TitleSignatureAppearanceData insertRecentTitleSignatureData:self.titleName];
        }
        self.dataSource = self.itemArray;
        [self reloadData];
        
        
    }];
    
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel") style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {
        //        @strongify(self)
        
    }];
    
   
    [alertController addAction:cancelAction];
    [alertController addAction:okAction];
    [self presentViewController:alertController animated:YES completion:nil];

}
- (void)deleteItemClickWarning
{
    NSString *title = FSLocalizedForKey(@"kConfirm");
    NSString *msg = FSLocalizedForKey(@"kAppearanceCannotDeleteStyle");
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:title message:msg preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *okAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
    }];
    [alertController addAction:okAction];
    [self presentViewController:alertController animated:YES completion:nil];
}

#pragma mark -  table view delegate handler

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView{
    return 1;
}

- (SignatureAppearanceModel *)dataForAtIndexPath:(NSIndexPath *)indexPath{
    return self.itemArray[indexPath.row];
}

- (void)configureCell:(FSTitleEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(SignatureAppearanceModel *)dataItem{
    cell.backgroundColor = self.fs_backgroundColor_B8;
    if (!cell.rightView) {
        cell.rightView = [[UIView alloc] init];
        FSItem* selectItem = [FSItem createItemWithImage:FSImageNamed(@"common_select_check")];
        [BindThemeKeySel(selectItem, setImage:forState:) fs_updateThemeCompleted:^(FSItem*  _Nonnull target, SEL  _Nonnull sel) {
            [target setImage:[FSImageNamed(@"common_select_check") fs_imageWithTintColor:target.fs_primaryColor] forState:UIControlStateNormal];
        }];

        selectItem.accessibilityIdentifier = @"select_check_item_id";
        selectItem.tag = 1;
        FSItem* moreItem = [FSItem createItemWithImage:FSImageNamed(@"common_more_gray")];
        moreItem.accessibilityIdentifier = @"select_more_item_id";
        moreItem.tag = 2;
     
    
        [cell.rightView addSubview:selectItem];
        [cell.rightView addSubview:moreItem];
       
        [selectItem mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.size.mas_equalTo(CGSizeMake(40, 40));
            make.centerY.mas_equalTo(selectItem.superview);
            make.left.mas_equalTo(moreItem.superview.mas_left);
            make.top.mas_equalTo(selectItem.superview.mas_top);
            make.bottom.mas_equalTo(selectItem.superview.mas_bottom);
        }];
        [moreItem mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.size.mas_equalTo(CGSizeMake(40, 40));
            make.centerY.mas_equalTo(moreItem.superview);
            make.left.mas_equalTo(selectItem.mas_right);
            make.right.mas_equalTo(moreItem.superview.mas_right);
            make.top.mas_equalTo(moreItem.superview.mas_top);
            make.bottom.mas_equalTo(moreItem.superview.mas_bottom);
        }];
    }
    
    for (FSItem *item in cell.rightView.subviews) {
        if(item.tag == 1){
            if(dataItem.select){
                item.hidden = false;
            }else{
                item.hidden = true;
            }
          
        }else{
            @weakify(self)
            item.onTapClick = ^(FSItem *_item) {
                @strongify(self)
                [self moreDeleteTitle:_item indexPath:indexPath];
            };
            if([dataItem.name isEqualToString:@"kAppearanceStandardStyle"]){
                item.hidden = true;
            }else{
                item.hidden = false;
            }

        }
    }
    if([dataItem.name isEqualToString:@"kAppearanceStandardStyle"]){
        cell.estimatedTitleLabel.text = FSLocalizedForKey(dataItem.name);
    }else{
        cell.estimatedTitleLabel.text = dataItem.name;
    }
   
}


- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{
    [super tableView:tableView didSelectRowAtIndexPath:indexPath];
    for (SignatureAppearanceModel *item in self.itemArray) {
        item.select = false;
    }
    SignatureAppearanceModel *model = self.itemArray[indexPath.row];
    model.select=true;
    self.titleName = model.name;
    [TitleSignatureAppearanceData insertRecentTitleSignatureData:self.titleName];
    [self reloadData];
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section{
        return 24;
}

- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section{
    UIView * headerView = [[UIView alloc] init];
    headerView.backgroundColor = self.fs_backgroundColor_B9;
    UILabel *lbText = [[UILabel alloc] init];
    [headerView addSubview:lbText];
    lbText.text = FSLocalizedForKey(@"kStyle");
    lbText.font = self.fs_standardWeakFont_WF1;
    lbText.textColor = self.fs_textColor_T4;
    [lbText mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(headerView.fs_mas_left).offset(16);
        make.centerY.mas_equalTo(headerView);
        
    }];
    return headerView;
}

@end
