/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "SignatureAppearanceViewController.h"
#import "SignatureAppearanceReasonViewController.h"
#import "SignatureAppearanceLocationViewController.h"
#import "SignatureAppearanceTitleViewController.h"
#import "SignatureAppearanceModel.h"
#import "SignatureAppearanceCellTableViewCell.h"
#import "TitleSignatureAppearanceData.h"
#import "AnnotationSignature.h"

static const float STYLE_HEADER_HEIGHT = 178;

@interface SignatureAppearanceViewController () <UITextFieldDelegate,SignatureAppearanceReasonDelegate,SignatureAppearanceTitleDelegate,SignatureAppearanceLocationDelegate>

@property (nonatomic, strong) UIView *headerView;
@property (nonatomic, strong) UIImageView *signatureIv;

@property (nonatomic, strong) UIButton *btnDone;
@property (nonatomic, assign) BOOL isCreateTitle;
@property (nonatomic, assign) BOOL isImamge;
@property (nonatomic, assign) BOOL initLoad;
@property (nonatomic, strong) CERT_INFO *cerInfo;
@property (nonatomic, strong) NSMutableArray *itemsArray;
@property (nonatomic, strong) AnnotationSignature *signature;
@property (nonatomic, strong) AnnotationSignature *createNewSignature;
@property (nonatomic, strong) NSString *editedTitle;
@end

@implementation SignatureAppearanceViewController

-(void)initialize{
    [super initialize];
    self.sectionStyle = FSTableViewSectionStyleLikeGrouped;
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager signature:(AnnotationSignature *)signature image:(UIImage *)img{
    if (self = [super init]) {
        self.extensionsManager = extensionsManager;
        self.signature = signature;
        self.image = img;
    }
    return self;
}

- (UIInterfaceOrientationMask)supportedInterfaceOrientations {
    return UIInterfaceOrientationMaskPortrait;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    [self.tableView fs_registerClass:[SignatureAppearanceCellTableViewCell class]];
    self.tableView.backgroundColor = self.fs_backgroundColor_B9;
    [self initView];
   
}

- (void)initView{
    self.navBarLeftTitle = FSLocalizedForKey(@"kCancel");
    self.navBarRightTitle = FSLocalizedForKey(@"kSave");
    self.title = FSLocalizedForKey(@"kAppearancePreview");
    self.navBar.hasDivide = NO;
    @weakify(self)
    self.navBar.leftItem.onTapClick = ^(FSItem *_item) {
        @strongify(self)
        [self cancelSign];
    };

    self.navBar.rightItem.onTapClick = ^(FSItem *_item) {
        @strongify(self)
        [self saveSign];
    };
    self.headerView = [[UIView alloc] init];
    self.headerView.backgroundColor = self.fs_backgroundColor_B11;
    self.signatureIv = [[UIImageView alloc] init];
    self.signatureIv.contentMode = UIViewContentModeScaleAspectFit;
    self.btnDone = [[UIButton alloc] init];
    [self.view addSubview:self.btnDone];
    
    [self.btnDone setTitleColor:self.fs_primaryColor forState:UIControlStateNormal];
    [self.btnDone setTitle:FSLocalizedForKey(@"kAppearanceCreateType") forState:UIControlStateNormal];
    [self.btnDone setBackgroundColor:self.fs_backgroundColor_B8];
    self.btnDone.titleLabel.textAlignment = NSTextAlignmentCenter;
    self.btnDone.titleLabel.font = self.fs_standardGeneralFont_GF2;
    [self.btnDone addTarget:self action:@selector(createTitle) forControlEvents:UIControlEventTouchUpInside];
    self.isCreateTitle =  false;
    self.itemsArray = [[NSMutableArray alloc] init];
    NSMutableArray *titleArray = [[NSMutableArray alloc] init];
    NSMutableArray *loactionReasonArray = [[NSMutableArray alloc] init];
    SignatureAppearanceModel *titleModel = [[SignatureAppearanceModel alloc ] initWithAppearance:false name:FSLocalizedForKey(@"kAppearanceType")];
    SignatureAppearanceModel *locationModel = [[SignatureAppearanceModel alloc ] initWithAppearanceOpen:false name:FSLocalizedForKey(@"kAppearanceLocation")];
    SignatureAppearanceModel *reasonModel = [[SignatureAppearanceModel alloc ] initWithAppearanceOpen:false name:FSLocalizedForKey(@"kAppearanceReason")];
    [titleArray addObject:titleModel];
    [self.itemsArray addObject:titleArray];
    [loactionReasonArray addObject:locationModel];
    [loactionReasonArray addObject:reasonModel];
    [self.itemsArray addObject:loactionReasonArray];
    [self loadData];
    self.dataSource = self.itemsArray;
}

- (void)viewDidAppear:(BOOL)animated{
    [super viewDidAppear:animated];
    if(self.initLoad){
        return;
    }
    self.initLoad = YES;
    [self renderSignature:self.currentSignature];
    NSIndexPath *indexPath = [NSIndexPath indexPathForRow:0 inSection:0];
    [self reloadRowsAtIndexPaths:indexPath];

}

- (void)loadData{
    self.currentSignature = self.signature.copy;
    //    CERT_INFO *info = [[CERT_INFO alloc] init];
    self.cerInfo = [[CERT_INFO alloc] init];
    NSString *certPath = [SIGNATURE_PATH stringByAppendingPathComponent:self.currentSignature.certMD5];
    [FSPDFCertUtil getCertInfo:certPath password:self.currentSignature.certPasswd certInfo:self.cerInfo];
    self.titleArray =  [TitleSignatureAppearanceData getTitleSignatureData];
    if(self.titleArray.count <= 0){
        AnnotationSignature *sig = [[AnnotationSignature alloc] init];
        sig.titleName = @"kAppearanceStandardStyle";
        sig.isUserNameOpen = YES;
        sig.isDistinguishedNameOpen = YES;
        sig.isReasonOpen = YES;
        sig.reason = @"kReasonAuthor";
        sig.isLocationOpen = YES;
        sig.location = @"";
        sig.isDateOpen = YES;
        sig.isVersionOpen = YES;
        sig.isLablesOpen = NO;
        [self.titleArray addObject:sig];
        [TitleSignatureAppearanceData insertTitleSignatureData:self.titleArray];
        [TitleSignatureAppearanceData insertRecentTitleSignatureData:@"kAppearanceStandardStyle"];
    }
    AnnotationSignature *sig = nil;
    if(self.signature.name){
        if(self.signature.titleName.length == 0){
            NSString *recentTitle = [TitleSignatureAppearanceData getRecentTitleSignature];
            sig = [TitleSignatureAppearanceData getTitleSignatureDataByKey:recentTitle];
            [self setReasonAndLocation:sig];
        }else{
            self.editedTitle = self.signature.titleName;
            sig = [TitleSignatureAppearanceData getTitleSignatureDataByKey:self.signature.titleName];
        }
    }else{
        NSString *recentTitle = [TitleSignatureAppearanceData getRecentTitleSignature];
        sig = [TitleSignatureAppearanceData getTitleSignatureDataByKey:recentTitle];
        [self setReasonAndLocation:sig];
    }
    [self setTitleStyle:sig];
    [self setSignDynamicInfo:sig];
}

- (void)loadOptionData{
    NSMutableArray *array = [[NSMutableArray alloc] init];
    SignatureAppearanceModel *model = [[SignatureAppearanceModel alloc ] initWithAppearance:false name:FSLocalizedForKey(@"kAppearanceOptionalName")];
    SignatureAppearanceModel *model2 = [[SignatureAppearanceModel alloc ] initWithAppearance:false name:FSLocalizedForKey(@"kAppearanceOptionalFoxitVersion")];
    SignatureAppearanceModel *model3 = [[SignatureAppearanceModel alloc ] initWithAppearance:false name:FSLocalizedForKey(@"kAppearanceOptionalDn")];
    SignatureAppearanceModel *model4 = [[SignatureAppearanceModel alloc ] initWithAppearance:false name:FSLocalizedForKey(@"kAppearanceOptionalDate")];
    SignatureAppearanceModel *model5 = [[SignatureAppearanceModel alloc ] initWithAppearance:false name:FSLocalizedForKey(@"kAppearanceOptionalLabels")];
    SignatureAppearanceModel *model6 = [[SignatureAppearanceModel alloc ] initWithAppearance:false name:FSLocalizedForKey(@"kAppearanceOptionalLogo")];
    [array addObject:model];
    [array addObject:model2];
    [array addObject:model3];
    [array addObject:model4];
    [array addObject:model5];
    [array addObject:model6];
    [self.itemsArray addObject:array];
    self.dataSource=self.itemsArray;
}

- (void)setTitleStyle:(AnnotationSignature *)sig{
    self.currentSignature.titleName = sig.titleName;
    self.currentSignature.isUserNameOpen = sig.isUserNameOpen;
    self.currentSignature.isDistinguishedNameOpen = sig.isDistinguishedNameOpen;
    if(self.currentSignature.isDistinguishedNameOpen){
        self.currentSignature.distinguishedName = [self getDn];
    }
    self.currentSignature.isDateOpen = sig.isDateOpen;
    self.currentSignature.isVersionOpen = sig.isVersionOpen;
    self.currentSignature.isLogoOpen = sig.isLogoOpen;
    self.currentSignature.isLablesOpen = sig.isLablesOpen;
    
}

- (void)setReasonAndLocation:(AnnotationSignature *)sig{
    self.currentSignature.isReasonOpen = sig.isReasonOpen;
    if( self.currentSignature.isReasonOpen){
        self.currentSignature.reason = sig.reason.length == 0? @"kReasonAuthor" : sig.reason;
    }else{
        self.currentSignature.reason = @"";
    }
    
    self.currentSignature.isLocationOpen = sig.isLocationOpen;
    if( self.currentSignature.isLocationOpen){
        if(sig.location.length == 0){
            self.currentSignature.location = @"";
        }else{
            self.currentSignature.location = sig.location;
        }
    }else{
        self.currentSignature.location = @"";
    }
}

- (void)setSignDynamicInfo:(AnnotationSignature *)sig{
    if (sig.isUserNameOpen){
        self.currentSignature.userName = self.cerInfo.certPublisher == nil ?@"":self.cerInfo.certPublisher;
    }

    if (sig.isVersionOpen){
        self.currentSignature.version = [self.extensionsManager.appInfoProvider getAppVersion];
        self.currentSignature.appName = [self.extensionsManager.appInfoProvider getAppName];
    }
 
}


- (void)renderSignature:(AnnotationSignature *)sig{
    UIImage *image = self.image;

    CGRect signaturRect = CGRectIntersection(self.signatureIv.frame, self.signatureIv.bounds);
    FSBitmap *bmp=[Utility  uiimageToFSBitmap:image];
    FSRectF *viewRect = [[FSRectF alloc] initWithLeft1:0 bottom1:0 right1:signaturRect.size.width top1:signaturRect.size.height];

    UIImage *img = [AnnotationSignature renderSignature:viewRect signature:sig bitmap:bmp isImage:self.signature.isPic isNewTime:true];
    //    UIImage *imgNew = img;
    //    if (self.clearWhiteBtn.on) imgNew = [img fs_replaceColor:[UIColor colorWithRed:1.0 green:1.0 blue:1.0 alpha:1.0] withTolerance:1.0];
//    self.signatureIv.contentMode = UIViewContentModeScaleAspectFit;
    self.signatureIv.image = img;

}



-(NSString *)getDn{
    NSString *publisher = self.cerInfo.certPublisher;
    NSString *organization = self.cerInfo.certOrganization;
    NSString *organizationUnit = self.cerInfo.certOrganizationUnit;
    NSString *email = self.cerInfo.certEmailInfo;
    if(publisher.length == 0){
        publisher = @"\"\"";
    }
    NSString *dn = @"CN=";
    dn = [dn stringByAppendingString:publisher];
    if(organization.length ==0){
        organization = @"\"\"";
    }
    dn = [dn stringByAppendingString:@",O="];
    dn = [dn stringByAppendingString:organization];
    
    if(organizationUnit.length ==0){
        organizationUnit = @"\"\"";
    }
    dn = [dn stringByAppendingString:@",OU="];
    dn = [dn stringByAppendingString:organizationUnit];
    
    if(email.length ==0){
        email = @"\"\"";
    }
    dn = [dn stringByAppendingString:@",E="];
    dn = [dn stringByAppendingString:email];
    return dn;
}

- (void) doneTItle{
    self.isCreateTitle = false;
    self.title = FSLocalizedForKey(@"kAppearancePreview");
    self.navBar.rightItem.enabled = YES;
    [self.btnDone setTitle:FSLocalizedForKey(@"kAppearanceCreateType") forState:UIControlStateNormal];
    [self.itemsArray removeObjectAtIndex:2];
    [self setTitleStyle:self.createNewSignature];
    [self setReasonAndLocation:self.createNewSignature];
    [self setSignDynamicInfo:self.createNewSignature];
    [self renderSignature:self.createNewSignature];
    [self reloadData];
    self.titleArray =  [TitleSignatureAppearanceData getTitleSignatureData];
    [self.titleArray addObject:self.createNewSignature];
    [TitleSignatureAppearanceData insertTitleSignatureData:self.titleArray];
    [TitleSignatureAppearanceData insertRecentTitleSignatureData:self.createNewSignature.titleName];
}

- (void)createTitle{
    if(self.isCreateTitle){
        [self doneTItle];
        return;
    }
    NSString *title = FSLocalizedForKey(@"kAppearanceTypeCreate");
    NSString *msg = FSLocalizedForKey(@"kAppearanceWarnType");
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:title message:msg preferredStyle:UIAlertControllerStyleAlert];
    @weakify(self)
    UIAlertAction *okAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        @strongify(self)
        BOOL isExists = [TitleSignatureAppearanceData isTitleExits:alertController.textFields[0].text];
        if(isExists){
            [self isExists];
            return;
        }
        
        self.createNewSignature = [[AnnotationSignature alloc] init];
        self.createNewSignature.titleName = alertController.textFields[0].text;
        self.isCreateTitle =  true;
        self.title = FSLocalizedForKey(@"kAppearanceType");
        self.navBar.rightItem.enabled = NO;
        [self.btnDone setTitle:FSLocalizedForKey(@"kDone") forState:UIControlStateNormal];
        [self renderSignature:self.createNewSignature];
        [self loadOptionData];
    }];
    
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel") style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {
        //        @strongify(self)
        
    }];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
        //        textField.accessibilityIdentifier = @"input_pageindex_id";
        textField.placeholder = FSLocalizedForKey(@"kAppearanceHintType");
        textField.keyboardType = UIKeyboardTypeDefault;
        textField.clearButtonMode = UITextFieldViewModeWhileEditing;
        textField.delegate = self;
        textField.accessibilityIdentifier = @"signature_alert_exist_tf_id";
        
        [textField setBlockForControlEvents:UIControlEventEditingChanged block:^(UITextField * _Nonnull sender) {
            //            @strongify(self)
            okAction.enabled = !(sender.text.length <= 0);
            
        }];
    }];
    okAction.enabled = NO;
    [alertController addAction:cancelAction];
    [alertController addAction:okAction];
    [self presentViewController:alertController animated:YES completion:nil];
}

- (void)isExists{
    UIAlertController *alertView = [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kAppearanceTypeExists") actions:@[FSLocalizedForKey(@"kOK") ] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
        
    }];
    [self presentViewController:alertView animated:YES completion:nil];
}

-(void)cancelSign{
    if(self.isCreateTitle){
        self.isCreateTitle = false;
        self.title = FSLocalizedForKey(@"kAppearancePreview");
        self.navBar.rightItem.enabled = YES;
        self.createNewSignature = nil;
        [self.btnDone setTitle:FSLocalizedForKey(@"kAppearanceCreateType") forState:UIControlStateNormal];
        [self.itemsArray removeObjectAtIndex:2];
        [self renderSignature:self.currentSignature];
        [self reloadData];
        return;
    }
    @weakify(self)
    [self dismissViewControllerAnimated:NO
                             completion:^{
        @strongify(self)
        if (self.delegate) {
            [self.delegate cancelSign:self.signature];
        }
    }];
}

-(void)saveSign{
    @weakify(self)
    [self dismissViewControllerAnimated:NO
                             completion:^{
        @strongify(self)
        if (self.delegate) {
            [self.delegate saveSign:self.currentSignature];
        }
    }];
}


-(void)startTitle{
    SignatureAppearanceTitleViewController *titleCtr = [[SignatureAppearanceTitleViewController alloc] initWithUIExtensionsManager:self->_extensionsManager title:self.currentSignature.titleName editedTitle:self.editedTitle];
    [titleCtr fs_setCustomTransitionAnimator:_extensionsManager.presentedVCAnimator];
    [self presentViewController:titleCtr animated:YES completion:nil];
    titleCtr.delegate = self;
}

-(void)startReason{
    SignatureAppearanceReasonViewController *reasonCtr = [[SignatureAppearanceReasonViewController alloc] initWithUIExtensionsManager:self->_extensionsManager reasonOpen:self.isCreateTitle ? self.createNewSignature.isReasonOpen:self.currentSignature.isReasonOpen reason:self.isCreateTitle ? self.createNewSignature.reason:self.currentSignature.reason];
    [reasonCtr fs_setCustomTransitionAnimator:_extensionsManager.presentedVCAnimator];
    [self presentViewController:reasonCtr animated:YES completion:nil];
    reasonCtr.delegate = self;
}

-(void)startLocation{
    SignatureAppearanceLocationViewController *locationCtr = [[SignatureAppearanceLocationViewController alloc] initWithUIExtensionsManager:self->_extensionsManager locationOpen:self.isCreateTitle ? self.createNewSignature.isLocationOpen :self.currentSignature.isLocationOpen location:self.isCreateTitle ? self.createNewSignature.location:self.currentSignature.location];
    [locationCtr fs_setCustomTransitionAnimator:_extensionsManager.presentedVCAnimator];
    [self presentViewController:locationCtr animated:YES completion:nil];
    locationCtr.delegate = self;
}


#pragma mark -- SignatureAppearanceReasonDelegate
- (void)reasonBack:(BOOL)isReasonOpen reason:(NSString *)reason{
    if(self.isCreateTitle){
        self.createNewSignature.isReasonOpen = isReasonOpen;
        self.createNewSignature.reason = reason;
        [self renderSignature:self.createNewSignature];
    }else{
        self.currentSignature.isReasonOpen = isReasonOpen;
        self.currentSignature.reason = reason;
        [self renderSignature:self.currentSignature];
    }
    NSArray *array = [[NSArray alloc] init];
    NSIndexPath *indexPath = [NSIndexPath indexPathForRow:0 inSection:0];
    NSIndexPath *indexPath2 = [NSIndexPath indexPathForRow:1 inSection:1];
    array = @[indexPath,indexPath2];
    [self reloadRowsAtIndexPaths:array];
}

#pragma mark -- SignatureAppearanceLocationDelegate
- (void)locationBack:(BOOL)isLocationOpen location:(NSString *)location{
    if(self.isCreateTitle){
        self.createNewSignature.isLocationOpen = isLocationOpen;
        self.createNewSignature.location = location;
        [self renderSignature:self.createNewSignature];
    }else{
        self.currentSignature.isLocationOpen = isLocationOpen;
        self.currentSignature.location = location;
        [self renderSignature:self.currentSignature];
    }
    NSArray *array = [[NSArray alloc] init];
    NSIndexPath *indexPath = [NSIndexPath indexPathForRow:0 inSection:0];
    NSIndexPath *indexPath2 = [NSIndexPath indexPathForRow:0 inSection:1];
    array = @[indexPath,indexPath2];
    [self reloadRowsAtIndexPaths:array];
}

#pragma mark -- SignatureAppearanceTitleDelegate
- (void)titleBack:(NSString *)titleName{
    self.createNewSignature = nil;
    AnnotationSignature *sig = [TitleSignatureAppearanceData getTitleSignatureDataByKey:titleName];
    self.currentSignature.titleName = titleName;
    [self setTitleStyle:sig];
    [self setReasonAndLocation:sig];
    [self renderSignature:self.currentSignature];
    [self reloadData];
}


#pragma mark -- UITextFieldDelegate
- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string{
    return YES;
}


#pragma mark -  table view datasource handler
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView{
    return self.itemsArray.count;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section{
    NSMutableArray *itemArray = self.itemsArray[section];
    return itemArray.count;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{
    [super tableView:tableView didSelectRowAtIndexPath:indexPath];
    if(indexPath.section == 0){
        if(!self.isCreateTitle){
            [self startTitle];
        }
    }else if (indexPath.section ==1){
        if(indexPath.row == 0){
            [self startLocation];
        }else{
            [self startReason];
        }
    }else{
        NSMutableArray *itemArray = self.itemsArray[indexPath.section];
        SignatureAppearanceModel *item = itemArray[indexPath.row];
        item.select = !item.select;
        
        if([item.name isEqualToString:FSLocalizedForKey(@"kAppearanceOptionalName")]){
            self.createNewSignature.isUserNameOpen = item.select;
            if (item.select) {
                self.createNewSignature.userName = self.cerInfo.certPublisher == nil ?@"":self.cerInfo.certPublisher;
            }
        }else if([item.name isEqualToString:FSLocalizedForKey(@"kAppearanceOptionalFoxitVersion")]){
            self.createNewSignature.isVersionOpen = item.select;
            if (item.select) {
                self.createNewSignature.version = [self.extensionsManager.appInfoProvider getAppVersion];
                self.createNewSignature.appName = [self.extensionsManager.appInfoProvider getAppName];
            }
        }else if([item.name isEqualToString:FSLocalizedForKey(@"kAppearanceOptionalDn")]){
            self.createNewSignature.isDistinguishedNameOpen = item.select;
            if (item.select) {
                self.createNewSignature.distinguishedName = [self getDn];
            }
        }else if([item.name isEqualToString:FSLocalizedForKey(@"kAppearanceOptionalDate")]){
            self.createNewSignature.isDateOpen = item.select;
        }else if([item.name isEqualToString:FSLocalizedForKey(@"kAppearanceOptionalLabels")]){
            self.createNewSignature.isLablesOpen = item.select;
        }else if([item.name isEqualToString:FSLocalizedForKey(@"kAppearanceOptionalLogo")]){
            self.createNewSignature.isLogoOpen = item.select;
        }
        [self renderSignature:self.createNewSignature];
        [self reloadData];
    }
}
- (NSString *)dequeueReusableIdentifierWithIndexPath:(NSIndexPath *)indexPath{
    NSString *identifier = nil;
    if(indexPath.section == 0){
        identifier = NSStringFromClass([SignatureAppearanceCellTableViewCell class]);
    }else if (indexPath.section == 1){
        identifier = NSStringFromClass([SignatureAppearanceCellTableViewCell class]);
    }else{
        identifier = NSStringFromClass([FSTitleEstimatedTableViewCell class]);
    }
    return identifier;
}


- (void)configureCell:(SignatureAppearanceCellTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(SignatureAppearanceModel *)dataItem{
    cell.estimatedTitleLabel.text = dataItem.name;
    cell.backgroundColor = self.fs_backgroundColor_B8;
    if(indexPath.section == 0){
        cell.rightLable.textColor = self.fs_textColor_T3;
        if(self.isCreateTitle){
            if([self.createNewSignature.titleName isEqualToString:@"kAppearanceStandardStyle"]){
                cell.rightLable.text = FSLocalizedForKey(@"kAppearanceStandardStyle");
            }else{
                cell.rightLable.text = self.createNewSignature.titleName;
            }
            cell.rightView.hidden = YES;
        }else{
            if([self.currentSignature.titleName isEqualToString:@"kAppearanceStandardStyle"]){
                cell.rightLable.text = FSLocalizedForKey(@"kAppearanceStandardStyle");
            }else{
                cell.rightLable.text = self.currentSignature.titleName;
            }
            cell.rightView.hidden = NO;
        }
        if (!cell.rightView) {
            UIImageView * selectImg = [[UIImageView alloc] init];
            selectImg.image = FSImageNamed(@"common_shou_gray");
            selectImg.accessibilityIdentifier = @"select_check_item_right_id";
            cell.rightView = selectImg;
            
        }
    }else if(indexPath.section == 1){
        cell.rightView.hidden = NO;
        cell.rightLable.textColor = self.fs_textColor_T3;
        if(indexPath.row == 0){
            if(self.isCreateTitle){
                if(self.createNewSignature.isLocationOpen){
                    cell.rightLable.text = self.createNewSignature.location;
                }else{
                    cell.rightLable.text = FSLocalizedForKey(@"kClose");
                }
            }else{
            
                if(self.currentSignature.isLocationOpen){
                    cell.rightLable.text = self.currentSignature.location;
                }else{
                    cell.rightLable.text = FSLocalizedForKey(@"kClose");
                }
            }
        }else{
            if(self.isCreateTitle){
                if(self.createNewSignature.isReasonOpen){
                    cell.rightLable.text = FSLocalizedForKey(self.createNewSignature.reason);
                }else{
                    cell.rightLable.text = FSLocalizedForKey(@"kClose");
                }
            }else{
                if(self.currentSignature.isReasonOpen){
                    cell.rightLable.text = FSLocalizedForKey(self.currentSignature.reason);
                }else{
                    cell.rightLable.text = FSLocalizedForKey(@"kClose");
                }
            }
        }
        if (!cell.rightView) {
            UIImageView * selectImg = [[UIImageView alloc] init];
            selectImg.image = FSImageNamed(@"common_shou_gray");
            selectImg.accessibilityIdentifier = @"select_check_item_right_id";
            cell.rightView = selectImg;
        }
    }else{
        if (!cell.leftView) {
            FSItem* selectItem = [FSItem createItemWithImage:FSImageNamed(@"common_unselect") imageSelected:FSImageNamed(@"common_select")];
            [BindThemeKeySel(selectItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem *  _Nonnull target, SEL  _Nonnull sel) {
                [target setImage:[FSImageNamed(@"common_select") fs_imageWithTintColor:target.fs_primaryColor] forState:UIControlStateSelected];
            }];
            selectItem.userInteractionEnabled = false;
            cell.leftView = selectItem;
        }
        ((FSItem *)cell.leftView).hidden = false;
        ((FSItem *)cell.leftView).selected = dataItem.select;
        if(dataItem.select){
            ((FSItem *)cell.leftView).accessibilityIdentifier = @"common_select_item_id";
        }else{
            ((FSItem *)cell.leftView).accessibilityIdentifier = @"common_no_select_item_id";
        }
    }
    
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section{
    if(section == 0){
        return 188;
    }else{
        return 24;
    }
}

- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section{
    if(section == 0){

        [self.headerView addSubview:self.signatureIv];
        [self.signatureIv mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(self.headerView.mas_left);
            make.right.mas_equalTo(self.headerView.mas_right);
            make.top.mas_equalTo(self.headerView.mas_top);
            make.bottom.mas_equalTo(self.headerView.fs_mas_bottom).offset(-10);
        }];
        UIView * cutView = [[UIView alloc] init];
        [self.headerView addSubview:cutView];
        cutView.backgroundColor = self.fs_backgroundColor_B9;
        [cutView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(cutView.superview.mas_left);
            make.right.mas_equalTo(cutView.superview.mas_right);
            make.height.mas_equalTo(10);
            make.bottom.mas_equalTo(self.headerView.mas_bottom);
        }];
        return self.headerView;
    }else if (section == 1){
        UIView * headerView2 = [[UIView alloc] init];
        headerView2.backgroundColor = self.fs_backgroundColor_B9;
        UILabel *lbText = [[UILabel alloc] init];
        [headerView2 addSubview:lbText];
        lbText.text = FSLocalizedForKey(@"kAppearanceSignText");
        lbText.font = self.fs_standardWeakFont_WF1;
        lbText.textColor = self.fs_textColor_T4;
        [lbText mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(headerView2.fs_mas_left).offset(16);
            make.centerY.mas_equalTo(headerView2);
            
        }];
        return headerView2;
    }else{
        UIView * headerView3 = [[UIView alloc] init];
        headerView3.backgroundColor = self.fs_backgroundColor_B9;
        UILabel *lbText = [[UILabel alloc] init];
        [headerView3 addSubview:lbText];
        lbText.text = FSLocalizedForKey(@"kAppearanceOptional");
        lbText.font = self.fs_standardWeakFont_WF1;
        lbText.textColor = self.fs_textColor_T4;
        [lbText mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(headerView3.fs_mas_left).offset(16);
            make.centerY.mas_equalTo(headerView3);
            
        }];
        return headerView3;
    }
    
}

- (UIView *)tableView:(UITableView *)tableView viewForFooterInSection:(NSInteger)section{
     if(section ==0){
        return nil;
    }
    if(self.itemsArray.count == 3){
        if(section == 1){
            return nil;
        }
    }
    UIView * footerView = [[UIView alloc] init];
    UIView * cutView = [[UIView alloc] init];
    [footerView addSubview:cutView];
    cutView.backgroundColor = self.fs_backgroundColor_B9;
    [cutView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(cutView.superview.mas_left);
        make.right.mas_equalTo(cutView.superview.mas_right);
        make.height.mas_equalTo(10);
        make.top.mas_equalTo(footerView.mas_top);
    }];
    [footerView addSubview:self.btnDone];
    [self.btnDone mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.btnDone.superview.mas_left);
        make.right.mas_equalTo(self.btnDone.superview.mas_right);
        make.top.mas_equalTo(cutView.mas_bottom);
        make.height.mas_equalTo(44);
    }];
    return footerView;
}

- (CGFloat)tableView:(UITableView *)tableView heightForFooterInSection:(NSInteger)section{
    if(self.itemsArray.count-1 == section){
        return 54;
    }else{
        return 0;
    }
}

@end
