/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "TitleSignatureAppearanceData.h"
#import "AnnotationSignature.h"

@implementation TitleSignatureAppearanceData

- (id)init {
    self = [super init];
    if (self) {
        // Custom initialization
    }
    return self;
}

+ (void)insertTitleSignatureData:(NSArray *)array{
    NSData *data = [NSKeyedArchiver archivedDataWithRootObject:array];
    [[NSUserDefaults standardUserDefaults] setObject:data forKey:SETTING_TITLE_SIGNATURE];
    [[NSUserDefaults standardUserDefaults] synchronize];
}

+ (NSMutableArray *)getTitleSignatureData{
    NSMutableArray *array = nil;
    NSData *data = [[NSUserDefaults standardUserDefaults] valueForKey:SETTING_TITLE_SIGNATURE];
    if (data && data.length > 0) {
        array = [NSMutableArray arrayWithArray:[NSKeyedUnarchiver unarchiveObjectWithData:data]];
    } else {
        array = [NSMutableArray array];
    }
    return array;
}

+ (int)getTitleSignatureDataCount{
    NSMutableArray *array = [self getTitleSignatureData];
    return (int)array.count;
}

+ (AnnotationSignature *)getTitleSignatureDataByKey:(NSString *)title{
    NSMutableArray *array = nil;
    NSData *data = [[NSUserDefaults standardUserDefaults] valueForKey:SETTING_TITLE_SIGNATURE];
    if (data && data.length > 0) {
        array = [NSMutableArray arrayWithArray:[NSKeyedUnarchiver unarchiveObjectWithData:data]];
    } else {
        array = [NSMutableArray array];
    }
    AnnotationSignature *sig;
    for (AnnotationSignature *obj in array) {
        if([obj.titleName isEqualToString:title]){
            sig = obj;
        }
    }
    return sig;
}

+ (NSMutableArray *)getTitleSignatureNames{
    NSMutableArray *array = nil;
    NSData *data = [[NSUserDefaults standardUserDefaults] valueForKey:SETTING_TITLE_SIGNATURE];
    if (data && data.length > 0) {
        array = [NSMutableArray arrayWithArray:[NSKeyedUnarchiver unarchiveObjectWithData:data]];
    } else {
        array = [NSMutableArray array];
    }
    NSMutableArray *titleNames = [[NSMutableArray alloc] init];
    for (AnnotationSignature *obj in array) {
        [titleNames addObject:obj.titleName];
    }
    NSMutableArray *resultArr = (NSMutableArray *)[[titleNames reverseObjectEnumerator] allObjects];

    return resultArr;
}

+ (BOOL)isTitleExits:(NSString *)title{
    NSMutableArray *array = nil;
    BOOL isExits = NO;
    NSData *data = [[NSUserDefaults standardUserDefaults] valueForKey:SETTING_TITLE_SIGNATURE];
    if (data && data.length > 0) {
        array = [NSMutableArray arrayWithArray:[NSKeyedUnarchiver unarchiveObjectWithData:data]];
    } else {
        array = [NSMutableArray array];
    }
    for (AnnotationSignature *obj in array) {
        if([obj.titleName isEqualToString:title]){
            isExits = YES;
            break;
        }
    }
    return isExits;
}


+ (void)insertRecentTitleSignatureData:(NSString *)title{
    [[NSUserDefaults standardUserDefaults] setObject:title forKey:SETTING_RECENT_TITLE_SIGNATURE];
    [[NSUserDefaults standardUserDefaults] synchronize];
}


+ (NSString *)getRecentTitleSignature{
    return [[NSUserDefaults standardUserDefaults] stringForKey:SETTING_RECENT_TITLE_SIGNATURE];
}


+(void)InternationalizeErrorTitle{
    NSMutableArray *titleArray =  [TitleSignatureAppearanceData getTitleSignatureData];
    NSArray * reasonArray = [NSArray arrayWithObjects:@"kReasonAuthor",@"kReasonReview",@"kReasonApprove",@"kReasonApproveLegallyBinding",@"kReasonAccuracyIntegrity",@"kReasonSignTerms",@"kReasonSpecified", nil];
    NSString *recentTitle = [TitleSignatureAppearanceData getRecentTitleSignature];
    for (AnnotationSignature * titleItem in titleArray) {
        NSArray *localizations = [[NSBundle mainBundle] localizations];
        for (NSString *languageCode in localizations) {
            if(![titleItem.titleName isEqualToString:@"kAppearanceStandardStyle"]){
                NSString *titleName = FSLocalizedForKeyWithLanguage(@"kAppearanceStandardStyle",languageCode);
                if(![recentTitle isEqualToString:@"kAppearanceStandardStyle"] && [recentTitle isEqualToString:FSLocalizedForKey(titleName)]){
                    recentTitle = @"kAppearanceStandardStyle";
                    [TitleSignatureAppearanceData insertRecentTitleSignatureData:recentTitle];
                }
                if([titleItem.titleName isEqualToString:FSLocalizedForKey(titleName)]){
                    titleItem.titleName = @"kAppearanceStandardStyle";
                    [TitleSignatureAppearanceData insertTitleSignatureData:titleArray];
                }
            }
            if(titleItem.reason != nil && titleItem.isReasonOpen && ![reasonArray containsObject:titleItem.reason]){
                if([titleItem.reason isEqualToString:@""]){
                    titleItem.reason = @"kReasonAuthor";
                    [TitleSignatureAppearanceData insertTitleSignatureData:titleArray];
                    continue;
                }
                for (NSString * item in reasonArray) {
                    NSString *reason = FSLocalizedForKeyWithLanguage(item,languageCode);
                    if([titleItem.reason isEqualToString:reason]){
                        titleItem.reason = item;
                        [TitleSignatureAppearanceData insertTitleSignatureData:titleArray];
                        break;
                    }
                }
            }
        }
    }
}
@end



