/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "UIExtensionsManager.h"

@protocol IToolHandler;
@protocol IRotationEventListener;
@class AnnotationSignature;
@class UIExtensionsManager;

typedef void (^onDocChanging)(NSString *newDocPath);
typedef NSString * (^onGetDocPath)(void);

@interface SignToolHandler : NSObject <IToolHandler, IRotationEventListener, IDocEventListener, IScrollViewEventListener> {
    FSRectF *_signatureRect;
    CGSize _signatureOriginalSize;

    float _maxWidth;
    float _minWidth;
    float _maxHeight;
    float _minHeight;
}
@property (nonatomic, assign) BOOL isPerformOnce;
@property (nonatomic, assign) BOOL isAdded;
@property (nonatomic, assign) BOOL isDeleting;
@property (nonatomic, assign) CGPoint signatureStartPoint;
@property (nonatomic, strong) AnnotationSignature *signature;
@property (nonatomic, copy) onDocChanging docChanging;
@property (nonatomic, copy) onGetDocPath getDocPath;
@property (nonatomic, assign) FSAnnotType type;
@property (nonatomic, copy) dispatch_block_t didSelectedSign;

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager;
- (void)changedSignImage;
- (void)signList;
- (void)onDocWillClose:(FSPDFDoc *)document;
- (void) delete;

- (FSSignature *)createSignature:(FSPDFPage *)page withParam:(DIGITALSIGNATURE_PARAM *)param;
- (BOOL)signSignature:(FSSignature *)sign withParam:(DIGITALSIGNATURE_PARAM *)param;
- (FSSignatureStates)verifyDigitalSignature:(NSString *)fileName signature:(FSSignature *)signature;
- (void)initSignature:(FSSignature *)sign withParam:(DIGITALSIGNATURE_PARAM *)param signature:(AnnotationSignature *)sig;
- (void)openCreateSign;

-(bool) isAddCertSignature;
-(void) gotoSignPage;
@end
