/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "SignatureListViewController.h"
#import "AnnotationSignature.h"

static const float STYLE_CELL_HEIGHT = 120;
static const float STYLE_CELL_HEIGHT_IPHONE = 120;
static const float STYLE_CELL_HEIGHT_SIGN = 178;

#define DIGITALSIGNATUREGROUP 1
#define HANDWRITINGSIGNATUREGROP 0
#define kColor [UIColor colorWithRed:0xE2/255.0f green:0xE2/255.0f blue:0xE2/255.0f alpha:1]

@interface SignatureListCell : UITableViewCell
@property (nonatomic, strong) UIImageView *imageThumbnail;
@property (nonatomic, strong) FSItem *moreItem;
@property (nonatomic, strong) UIImageView *selectImageView;
@property (nonatomic, copy) void (^moreShowClick)(SignatureListCell *cell);
@property (nonatomic, copy) void (^editItemClick)(SignatureListCell *cell);
@property (nonatomic, copy) void (^deleteItemClick)(SignatureListCell *cell);
@end

@implementation SignatureListCell

- (instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier{
    if (self = [super initWithStyle:style reuseIdentifier:reuseIdentifier]) {
        self.isAccessibilityElement = YES;
        self.clipsToBounds = YES;
        self.backgroundColor = self.fs_backgroundColor_B11;
        self.selectImageView = [[UIImageView alloc] init];
        self.selectImageView.accessibilityIdentifier = @"selectImageView_id";
        [BindThemeKeySel( self.selectImageView, setImage:) fs_updateThemeCompleted:^(UIImageView * _Nonnull target, SEL  _Nonnull sel) {
            target.image = FSImageNamedAndPrimaryColor(@"common_select");
        }];
        self.selectImageView.hidden = YES;
        [self.contentView addSubview:self.selectImageView];
       
        [self.selectImageView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(self.selectImageView.superview.mas_left).offset(10);
            make.centerY.mas_equalTo(self.selectImageView.superview.mas_centerY);
            make.size.equalTo(@(self.selectImageView.image.size));
        }];

        UIImageView *imageThumbnail = [[UIImageView alloc] init];
        imageThumbnail.contentMode = UIViewContentModeScaleAspectFit;
        self.imageThumbnail = imageThumbnail;
        [self.contentView addSubview:imageThumbnail];

        UIImage *moreImage = FSImageNamedAndColor(@"common_more_gray", [self.fs_iconColor_I3 fs_lightColor]);
        self.moreItem = [[FSItem alloc] init];
        self.moreItem.accessibilityIdentifier = @"common_more_gray";
        [self.moreItem setImage:moreImage forState:UIControlStateNormal];
        [self.moreItem addTarget:self action:@selector(moreClick) forControlEvents:UIControlEventTouchUpInside];
        [self.contentView addSubview:self.moreItem];
        [self.moreItem mas_makeConstraints:^(MASConstraintMaker *make) {
            make.right.mas_equalTo(self.moreItem.superview.mas_right).offset(-10);
            make.centerY.mas_equalTo(self.moreItem.superview.mas_centerY);
            make.size.equalTo(@(moreImage.size));
        }];
        
        [self.imageThumbnail mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(self.selectImageView.mas_right).offset(10);
            make.top.mas_equalTo(self.imageThumbnail.superview.mas_top).offset(10);
            make.bottom.mas_equalTo(self.imageThumbnail.superview.mas_bottom).offset(-10);
            make.right.mas_equalTo(self.imageThumbnail.superview.mas_centerX);
        }];
        
        
        UIView *divideView = [[UIView alloc] init];
        divideView.layer.zPosition = 0.01;
        divideView.backgroundColor = self.fs_dividerColor_P2;
        [self.contentView addSubview:divideView];
        
        [divideView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(divideView.superview);
            make.right.mas_equalTo(divideView.superview);
            make.height.equalTo(@(DIVIDE_TOOL_BAR_THICKNES));
            make.bottom.mas_equalTo(divideView.superview);
        }];
        
        if (UIAccessibilityIsVoiceOverRunning()) {
            self.accessibilityHint = FSLocalizedForKey(@"kTriggerOpVoiceover");
            self.accessibilityCustomActions = @[[[UIAccessibilityCustomAction alloc] initWithName:FSLocalizedForKey(@"kUnfoldMoreMenu") target:self selector:@selector(moreClick)]];
        }
    }
    return self;
}

- (void)moreClick {
    self.moreShowClick(self);
}

@end

@interface SignatureDigListCell : SignatureListCell

@end

@implementation SignatureDigListCell

- (instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier{
    if (self = [super initWithStyle:style reuseIdentifier:reuseIdentifier]) {
        
        [self.imageThumbnail mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(self.selectImageView.mas_right).offset(10);
            make.top.mas_equalTo(self.imageThumbnail.superview.mas_top).offset(10);
            make.bottom.mas_equalTo(self.imageThumbnail.superview.mas_bottom).offset(-10);
            make.right.mas_equalTo(self.moreItem.mas_left).offset(-10);
           
        }];
        self.imageThumbnail.contentMode = UIViewContentModeScaleAspectFit;
    }
    return self;
}
@end
    
@implementation SignatureListViewController

- (void)initialize{
    [super initialize];
    self.signatureArray = [NSMutableArray array];
}

- (void)viewDidLoad{
    [super viewDidLoad];
    
    self.view.backgroundColor = self.fs_backgroundColor_B2;
    self.tableView.backgroundColor = self.fs_backgroundColor_B2;
    self.title = FSLocalizedForKey(@"kSignListTitle");
    
    self.navBarLeftTitle = FSLocalizedForKey(@"kClose");

    self.navBarRightTitle = FSLocalizedForKey(@"kCreate");
    @weakify(self)
    self.navBar.rightItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [self addSignature];
    };
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];

    if (self.isFieldSigList) {
        self.signatureArray = [AnnotationSignature getCertSignatureList];
    } else {
        self.signatureArray = (NSMutableArray *) [self groupSignatureListArray:[AnnotationSignature getSignatureList]];
    }
    [self.tableView reloadData];
}

- (void)setSignatureArray:(NSMutableArray *)signatureArray
{
    if(self.isFieldSigList){
        _signatureArray = (NSMutableArray *)[[signatureArray reverseObjectEnumerator] allObjects];
    }else{
        NSMutableArray * tempArray = [[NSMutableArray alloc] init];
        for(int i = 0;i <signatureArray.count; i++){
            NSArray * array = signatureArray[i];
            array = [[array reverseObjectEnumerator] allObjects];
            [tempArray addObject:array];
        }
        _signatureArray = tempArray;
    }
}

- (UIExtensionsManager *)extensionManager{
    return ((FSViewControllerManager *)self.services).extensionsManager;
}

- (BOOL)emptyDataSetShouldDisplay:(UITableView *)tableView{
    
    if (self.isFieldSigList) {
        return !self.signatureArray.count;
    }
    if (self.signatureArray.count) {
        for (NSArray *signatureCount in self.signatureArray) {
            if (!self.extensionManager.config.loadForm) {
                return !signatureCount.count;
            }
            if (signatureCount.count) {
                return NO;
            }
        }
    }
    return YES;
}

- (NSString *)titleForEmptyDataSet{
    return FSLocalizedForKey(@"kNoSignatures");
}

- (UIImage *)imageForEmptyDataSet:(UITableView *)tableView {
    return FSImageNamedAndPrimaryColor(@"panel_signature_no_data");
}

- (void)navigationBarBackTouch{
    [super navigationBarBackTouch];
    if (self.delegate && [self.delegate respondsToSelector:@selector(cancelSignature)]) {
        [self.delegate cancelSignature];
    }
}

-(UIImage *)handlerRenderSign: (UIImage *)image sign: (AnnotationSignature *)sign cell:(SignatureListCell *)cell CGSize:(CGSize) size {
//    image = [Utility scaleToSize:image size:size];
    FSBitmap *bmp=[Utility  uiimageToFSBitmap:image];
    FSRectF *viewRect = [[FSRectF alloc] initWithLeft1:0 bottom1:0 right1:size.width top1:size.height];
    UIImage * img = [AnnotationSignature renderSignature:viewRect signature:sign bitmap:bmp isImage:sign.isPic isNewTime:false];
    return img;
}

#pragma mark -  table view delegate handler
- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    NSString *name;
    if (self.isFieldSigList) {
        name = [self.signatureArray objectAtIndex:indexPath.row];
    } else {
        name = [[self.signatureArray objectAtIndex:indexPath.section] objectAtIndex:indexPath.row];
    }
    AnnotationSignature *sig = [AnnotationSignature getSignature:name];
    if (sig.titleName.length >0 ){
        return STYLE_CELL_HEIGHT_SIGN;
    }else{
        return DEVICE_iPHONE ? STYLE_CELL_HEIGHT_IPHONE : STYLE_CELL_HEIGHT;
    }
   
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    NSString *name;
    if (self.isFieldSigList) {
        name = [self.signatureArray objectAtIndex:indexPath.row];
    } else {
        name = [[self.signatureArray objectAtIndex:indexPath.section] objectAtIndex:indexPath.row];
    }
    AnnotationSignature *sig = [AnnotationSignature getSignature:name];
    [AnnotationSignature setSignatureSelected:sig.name];
    [tableView reloadData];

    [self dismissViewControllerAnimated:YES
                             completion:^{
                                 if (self.delegate) {
                                     [self.delegate signatureListViewController:self selectSignature:sig];
                                 }
                             }];
}

#pragma mark -  table view datasource handler

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    if (self.isFieldSigList) {
        return self.signatureArray.count ? 1 : 0;
    }
    if (self.signatureArray.count) {
        for (NSArray *signatureCount in self.signatureArray) {
            if ((!self.extensionManager.config.loadForm && signatureCount.count) || ![self.extensionManager allowFormPermission]) {
                return 1;
            }
            if (signatureCount.count) {
                return self.signatureArray.count;
            }
        }
    }
    return 0;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    if (self.isFieldSigList) {
        return self.signatureArray.count;
    }
    return [self.signatureArray[section] count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *cellId = @"signatureListCellID";
    static NSString *cellDigId = @"signatureDigListCellID";
    SignatureListCell *cell;
    NSString *name;
    if (self.isFieldSigList) {
        name = [self.signatureArray objectAtIndex:indexPath.row];
    } else {
        name = [[self.signatureArray objectAtIndex:indexPath.section] objectAtIndex:indexPath.row];
    }
    UIImage *signImage = [AnnotationSignature getSignatureImage:name];
    AnnotationSignature *sig = [AnnotationSignature getSignature:name];
    

    if (cell == nil) {
        if(sig.titleName.length > 0){
            cell = [[SignatureDigListCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellDigId];
        }else{
            cell = [[SignatureListCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellId];
        }
        cell.selectedBackgroundView = [UIView new];
    }
    cell.selectedBackgroundView.backgroundColor = self.fs_dividerColor_P1;
    cell.contentView.backgroundColor = nil;
  
    typeof(self) __weak weakSelf = self;
    cell.moreShowClick = ^(SignatureListCell *editCell) {
        NSMutableArray *items = @[].mutableCopy;
        
        [items addObject:[[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kEdit") image:FSImageNamedAndNormalColor(@"fill_sign_signature_edit") action:^(FSPopoverItem * _Nonnull item, int index) {
            editCell.editItemClick(editCell);
        }]];
        
        [items addObject:[[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kDelete") image:FSImageNamedAndNormalColor(@"common_delete_gray") action:^(FSPopoverItem * _Nonnull item, int index) {
            editCell.deleteItemClick(editCell);
        }]];
        
        UIColor *oriColor = editCell.backgroundColor;
        if (DEVICE_iPHONE) editCell.backgroundColor = self.fs_backgroundColor_B2;
        [self.extensionManager.viewCtrlManager showMoreMenuWithItems:items.copy dismissCompletion:DEVICE_iPHONE ? ^{
            editCell.backgroundColor = oriColor;
        } : nil  sourceView:!DEVICE_iPHONE ? ^UIView * _Nonnull(UIPopoverPresentationController * _Nonnull popver) {
            popver.permittedArrowDirections = UIPopoverArrowDirectionRight;
            return editCell.moreItem;
        } : nil];
        
    };
    cell.editItemClick = ^(SignatureListCell *editCell) {
        [weakSelf dismissViewControllerAnimated:NO
                                     completion:^{
                                         NSString *name;
                                         if (weakSelf.isFieldSigList) {
                                             name = [weakSelf.signatureArray objectAtIndex:[tableView indexPathForCell:editCell].row];
                                         } else {
                                             name = [[weakSelf.signatureArray objectAtIndex:[tableView indexPathForCell:editCell].section] objectAtIndex:[tableView indexPathForCell:editCell].row];
                                         }
                                         AnnotationSignature *sig = [AnnotationSignature getSignature:name];
                                         [weakSelf.delegate signatureListViewController:weakSelf openSignature:sig];
                                     }];

    };
    cell.deleteItemClick = ^(SignatureListCell *editCell) {
        NSString *name;
        if (weakSelf.isFieldSigList) {
            name = [weakSelf.signatureArray objectAtIndex:[tableView indexPathForCell:editCell].row];
        } else {
            name = [[weakSelf.signatureArray objectAtIndex:[tableView indexPathForCell:editCell].section] objectAtIndex:[tableView indexPathForCell:editCell].row];
        }
        AnnotationSignature *sig = [AnnotationSignature getSignature:name];
        [AnnotationSignature removeSignatureResource:sig.name];
        [sig remove];
        if (weakSelf.isFieldSigList) {
            weakSelf.signatureArray = [AnnotationSignature getCertSignatureList];
        } else {
            weakSelf.signatureArray = (NSMutableArray *) [weakSelf groupSignatureListArray:[AnnotationSignature getSignatureList]];
        }
        [weakSelf.tableView reloadData];
        if (weakSelf.delegate) {
            [weakSelf.delegate signatureListViewController:weakSelf deleteSignature:sig];
        }
    };

    UIImageView *imageThumbnail = cell.imageThumbnail;
    CGSize imageThumbnailSize;
    if (sig.titleName.length > 0 && ![sig isSignInfoEmpty] && signImage != nil){
        dispatch_async(dispatch_get_main_queue(), ^{
            imageThumbnail.image = [self handlerRenderSign:signImage sign:sig cell:cell CGSize:imageThumbnail.frame.size];
        });
    }else{
        imageThumbnailSize = CGSizeMake((signImage.size.width / signImage.size.height) * (STYLE_CELL_HEIGHT - 20), (STYLE_CELL_HEIGHT - 20));
        if (signImage.size.width >= imageThumbnailSize.width || signImage.size.height >= imageThumbnailSize.height) {
            signImage = [Utility scaleToSize:signImage size:imageThumbnailSize];
        }
        imageThumbnail.image = signImage;
    }

    NSString *selectedName = nil;
    selectedName = [AnnotationSignature getSignatureSelected];

    if ([selectedName isEqualToString:name]) {
        cell.selectImageView.hidden = NO;
    } else {
        cell.selectImageView.hidden = YES;
    }

    if (indexPath.section == DIGITALSIGNATUREGROUP) {
        cell.imageThumbnail.accessibilityIdentifier = @"signature_with_cert_img_id";
    } else {
        cell.imageThumbnail.accessibilityIdentifier = @"signature_without_cert_img_id";
    }
    
    return cell;
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section {
    if (self.isFieldSigList || [self.extensionManager.pdfViewCtrl isRMSProtected] || [self emptyDataSetShouldDisplay:tableView]) {
        return 1;
    }
    return 24.f;
}

- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section {

    UIView *headerView = [[UIView alloc] init];
    headerView.backgroundColor = self.fs_backgroundColor_B2; ;
    if (self.isFieldSigList || [self.extensionManager.pdfViewCtrl isRMSProtected] || [self emptyDataSetShouldDisplay:tableView]) {
        return headerView;
    }

    UILabel *label = [[UILabel alloc] init];
    label.font = self.fs_standardWeakFont_WF1;
    label.textColor = self.fs_textColor_T3;
    [headerView addSubview:label];
    if (section == DIGITALSIGNATUREGROUP) {
        label.text = FSLocalizedForKey(@"kSignListSectionHadCert");
    } else {
        label.text = FSLocalizedForKey(@"kSignListSectionWithoutCert");
    }

    [label mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(label.superview.fs_mas_left).offset(ITEM_IMAGE_TOP_MARGIN_NORMAL);
        make.centerY.fs_equalToSuperview();
    }];
    return headerView;
}

#pragma mark - event handler

- (void)addSignature {
    if (DEVICE_iPHONE) {
        [self dismissViewControllerAnimated:NO
                                 completion:^{
                                     if (self.delegate) {
                                         [self.delegate signatureListViewController:self openSignature:nil];
                                     }
                                 }];
    } else {
        if (self.delegate) {
            [self.delegate signatureListViewController:self openSignature:nil];
        }
    }
}

- (NSArray *)groupSignatureListArray:(NSArray *)listArray {
    NSMutableArray *mutableArray = [[NSMutableArray alloc] init];
    NSMutableArray *certSignArray = [[NSMutableArray alloc] init];
    NSMutableArray *handleSignArray = [[NSMutableArray alloc] init];
    [mutableArray addObject:handleSignArray];
    if (![self.extensionManager.pdfViewCtrl isRMSProtected] && ([self.extensionManager.permissionProvider checkPermission:FSFunctionPDFDigitalSignAdd] == FSPermissionStateShow)) {
        [mutableArray addObject:certSignArray];
    }
    

    for (NSString *sigName in listArray) {
        AnnotationSignature *sig = [AnnotationSignature getSignature:sigName];
        if (sig.certMD5 && sig.certPasswd && sig.certFileName) {
            if (![self.extensionManager.pdfViewCtrl isRMSProtected]) {
                [certSignArray addObject:sigName];
            }
        } else {
            [handleSignArray addObject:sigName];
        }
    }
    return mutableArray;
}

@end
