/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "SignatureViewController.h"
#import "AnnotationSignature.h"
#import "FSMediaFileToPDFViewController.h"
#import "DigitalCertSelectCtr.h"
#import "SignatureView.h"
#import "UIExtensionsConfig+private.h"
#import "SignatureModule+Private.h"
#import "UIExtensionsManager+Private.h"
#import "FSBaseViews.h"
#import "SignatureAppearanceViewController.h"

@interface SignatureViewController ()<FSPropertyMainViewDelegate, FSMediaFileToPDFViewControllerDelegate,SignatureAppearanceViewDelegate>

@property (nonatomic, weak) FSMainToolbar *topbar;
@property (nonatomic, strong) FSToolbar *subTopbar;
@property (nonatomic, strong) FSToolbar *bottombar;
@property (nonatomic, strong) FSItem *fromAlbum;
@property (nonatomic, strong) FSItem *fromCamera;
@property (nonatomic, strong) FSItem *addCertItem;
@property (nonatomic, strong) FSItem *clearItem;
@property (nonatomic, strong) FSItem *propertyItem;
@property (nonatomic, strong) FSSwitch *clearWhiteBtn;
@property (nonatomic, strong) SignatureView *viewSignature;
@property (nonatomic, assign) int currentColor;
@property (nonatomic, assign) float currentLineWidth;
@property (nonatomic, copy) NSString *currentCertFilePath;
@property (nonatomic, copy) NSString *currentCertFileName;
@property (nonatomic, copy) NSString *currentCertPasswd;
@property (nonatomic, copy) NSString *currentCertMD5;
@property (nonatomic, strong) NSData *currentDib;
@property (nonatomic, assign) CGRect currentRectSigPart;
@property (nonatomic, assign) BOOL isContentCert;
@property (nonatomic, strong) UIImage *sourceImage;
@property (nonatomic, strong) FSPropertyMainView *propertyMainView;
@property (nonatomic, strong) SignatureAppearanceViewController *signatureCtr;
@property (nonatomic, assign) BOOL initLoad;
@end

@implementation SignatureViewController

typedef struct {
    float thresh;
} CustomThresh_;

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        self.extensionsManager = extensionsManager;
        SignatureModule *signatureModule = [extensionsManager getModuleByName:Module_Signature];
        @weakify(self);
        signatureModule.getSignatureViewImageBlock = ^id{
            @strongify(self);
            return [self.viewSignature getCurrentImage];
        };
        self.topbar = signatureModule.signatureViewTopBar;
    }
    return self;
}

- (void)dealloc{
    [self.propertyMainView unResisterEvent];
    [self.subTopbar removeFromSuperview];
    SignatureModule *signatureModule = [_extensionsManager getModuleByName:Module_Signature];
    signatureModule.getSignatureViewImageBlock = nil;
}

- (FSNavigationBar *)navBar{
    if ([self.topbar.toolbar isKindOfClass:[FSNavigationBar class]]) {
        return (FSNavigationBar *)self.topbar.toolbar;
    }
    return [super navBar];
}

- (FSToolbar *)bottombar{
    if (!_bottombar) {
        _bottombar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleBottom];
        
        _clearWhiteBtn = [[FSSwitch alloc] initWithFrame:CGRectZero];
        [_bottombar addItem:_clearWhiteBtn atPosition:FSItemPositionLeft];
        
        FSLabel *describel = [[FSLabel alloc] init];
        describel.font = self.fs_standardGeneralFont_GF2;
        describel.textColor = self.fs_textColor_T4;
        describel.text = FSLocalizedForKey(@"kRemoveWhiteBackG");
        [_bottombar addItem:describel atPosition:FSItemPositionLeft];
    }
    return _bottombar;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.view.backgroundColor = self.fs_backgroundColor_B11;
    [self.view addSubview:self.topbar.contentView];
    [self initSubView];
    
    if (self.viewSignature.fromDraw) {
        self.sourceImage = nil;
    }

}
- (void)viewDidAppear:(BOOL)animated{
    [super viewDidAppear:animated];
    if(self.initLoad){
        return;
    }
    self.initLoad = YES;
    if (self.currentSignature)
    {
        if (self.viewSignature.fromDraw) {
            [self loadSignature];
        }
    }else{
        self.viewSignature.hasChanged = NO;
    }
}

#if !_MAC_CATALYST_
- (UIInterfaceOrientationMask)supportedInterfaceOrientations {
    if (DEVICE_iPHONE || self.view.window.traitCollection.horizontalSizeClass == UIUserInterfaceSizeClassCompact) {
        return  UIInterfaceOrientationMaskLandscape;
    }
    return  UIInterfaceOrientationMaskAll;
}

- (UIInterfaceOrientation)preferredInterfaceOrientationForPresentation{
    if (DEVICE_iPHONE) {
        return UIInterfaceOrientationLandscapeRight;
    }
    return [UIApplication sharedApplication].statusBarOrientation;
}

- (BOOL)shouldAutorotate{
    if (DEVICE_iPHONE) {
         NSDictionary *dict = [[NSBundle mainBundle] infoDictionary];
           NSArray *supportedInterfaceOrientations = dict[@"UISupportedInterfaceOrientations"];
           for (NSString *orientation in supportedInterfaceOrientations) {
               if ([orientation isEqualToString:@"UIInterfaceOrientationLandscapeLeft"] || [orientation isEqualToString:@"UIInterfaceOrientationLandscapeRight"]) {
                   return YES;
               }
           }
         return NO;
    }
    return YES;
}
#endif

- (void)initSubView {

    self.navBarLeftTitle = FSLocalizedForKey(@"kCancel");
    self.navBarRightTitle = FSLocalizedForKey(@"kSave");
    self.title = FSLocalizedForKey(@"kAddSignature");
    self.navBar.hasDivide = NO;
    @weakify(self)
    self.navBar.leftItem.onTapClick = ^(FSItem *_item) {
        @strongify(self)
        [self cancelSign];
    };

    self.navBar.rightItem.onTapClick = ^(FSItem *_item) {
        @strongify(self)
        [self saveSign];
    };
    self.navBar.rightItem.enabled = NO;

    self.subTopbar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleDefault];
    self.subTopbar.accessibilityIdentifier = @"Sign_property_bar_id";
    NSArray<__kindof UIView *> *arrangeViews = [[self.topbar contentView] arrangedSubviews];
    for (UIView * view in arrangeViews) {
        if ([view.accessibilityIdentifier isEqualToString:@"Sign_property_bar_id"]) {
            [view removeFromSuperview];
            break;
        }
    }
    [[self.topbar contentView] addArrangedSubview:self.subTopbar];
    self.fromAlbum = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"common_from_album")];
    self.fromAlbum.accessibilityIdentifier = @"common_from_album";
    self.fromAlbum.accessibilityLabel = FSLocalizedForKey(@"kFromAlbum");
    self.fromAlbum.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        FSMediaFileToPDFViewController *mediaFileToPDFViewController = [[FSMediaFileToPDFViewController alloc] init];
        mediaFileToPDFViewController.delegate = self;
        [self presentViewController:mediaFileToPDFViewController animated:NO completion:^{
            [mediaFileToPDFViewController openAlbum];
        }];
    };

    self.fromCamera = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"common_from_camera")];
    self.fromCamera.accessibilityIdentifier = @"common_from_camera";
    self.fromCamera.accessibilityLabel = FSLocalizedForKey(@"kFromCamera");
    self.fromCamera.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        FSMediaFileToPDFViewController *mediaFileToPDFViewController = [[FSMediaFileToPDFViewController alloc] init];
        mediaFileToPDFViewController.delegate = self;
        [self presentViewController:mediaFileToPDFViewController animated:NO completion:^{
            [mediaFileToPDFViewController openCamera];
        }];
    };

    self.propertyItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"fill_sign_signature_add_cer")];
    [self.propertyItem setBackgroundImage:FSImageNamedAndNormalColor(@"common_unflod_backgroup") forState:FSItemControlStateAll];
    [self.propertyItem setBackgroundImage:[FSImageNamedAndNormalColor(@"common_unflod_backgroup") fs_applyingAlpha:IMAGE_DISABLED_ALPHA] forState:UIControlStateDisabled];
    self.propertyItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [self propertySign:_item];
    };
    [self.subTopbar addItem:self.propertyItem spacing:4.f atPosition:FSItemPositionRight];

    self.clearItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"fill_sign_sign_earser")];
    self.clearItem.accessibilityIdentifier = @"fill_sign_sign_earser";
    self.clearItem.accessibilityLabel = FSLocalizedForKey(@"kClear");
    self.clearItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [self clearSign];
    };
    [self.subTopbar addItem:self.clearItem atPosition:FSItemPositionRight];

    if (self.extensionsManager.config.loadForm && (self.isFieldSig || [self.extensionsManager allowFormPermission])) {
        self.addCertItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"fill_sign_signature_add_cer")];
        self.addCertItem.accessibilityIdentifier = @"fill_sign_signature_add_cer";
        self.addCertItem.accessibilityLabel = FSLocalizedForKey(@"kSelectCertVoiceOver");
        if (![_extensionsManager.pdfViewCtrl isRMSProtected]) {
            if (self.currentSignature) {
                self.currentCertFileName = self.currentSignature.certFileName;
                self.currentCertMD5 = self.currentSignature.certMD5;
                self.currentCertPasswd = self.currentSignature.certPasswd;
            }

            if (self.currentCertFileName) {
                self.addCertItem.selected = YES;
                NSString *msg = [FSLocalizedForKey(@"kCurrentSelectCert") stringByAppendingString:self.currentCertFileName];
                [self fs_currentSceneToastWithMsg:msg];
            } else {
                self.addCertItem.selected = NO;
            }

            [self.addCertItem addTarget:self action:@selector(selectCert) forControlEvents:UIControlEventTouchUpInside];
        }
        [self.subTopbar addItem:self.addCertItem atPosition:FSItemPositionRight];
    }

    [self setSignatureDefaultOption];

    // Do any additional setup after loading the view from its nib.

    self.viewSignature = [[SignatureView alloc] initWithSize:self.view.fs_size];
    [self.view addSubview:self.viewSignature];

    SignatureModule *signatureModule = [_extensionsManager getModuleByName:Module_Signature];
    self.viewSignature.signHasChangedCallback = ^(BOOL hasChanged) {
        @strongify(self)
        !signatureModule.signatureViewHasChanged ? : signatureModule.signatureViewHasChanged(hasChanged);
        if (hasChanged && [self.viewSignature getCurrentImage]) {
            self.navBar.rightItem.enabled = YES;
        } else {
            self.navBar.rightItem.enabled = NO;
        }
    };

    [self.view addSubview:self.bottombar];
    [self.bottombar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.equalTo(self.view.mas_bottom);
        make.leading.trailing.equalTo(self.view);
    }];

    self.currentColor = [Preference getIntValue:[self getName] type:@"Color" defaultValue:_extensionsManager.config.defaultSettings.signature.color.rgbValue];
    self.currentLineWidth = [Preference getFloatValue:[self getName] type:@"Linewidth" defaultValue:_extensionsManager.config.defaultSettings.signature.thickness];
    self.viewSignature.color = self.currentColor;
    self.viewSignature.diameter = self.currentLineWidth < 2.0 ? 2.0:self.currentLineWidth;

    [self.propertyItem setImage:FSImageNamedAndCustomColor(@"read_property_color_ring",[UIColor colorWithRGB:self.currentColor]) forState:UIControlStateNormal];
    self.propertyItem.accessibilityIdentifier = @"read_property";
    self.propertyItem.accessibilityLabel = FSLocalizedForKey(@"kProperty");
}

- (void)viewWillLayoutSubviews{
    [super viewWillLayoutSubviews];

    if (self.view.fs_width >= IPAD_PREFERENCE_BAR_WIDTH) {

        if (![self.subTopbar.centerItems containsItem:self.fromAlbum]) {
            [self.subTopbar removeItem:self.fromAlbum];
            [self.subTopbar addItem:self.fromAlbum spacing:24.f atPosition:FSItemPositionCenter];
        }

        if (![self.subTopbar.centerItems containsItem:self.fromCamera]) {
            [self.subTopbar removeItem:self.fromCamera];
            [self.subTopbar addItem:self.fromCamera spacing:24.f atPosition:FSItemPositionCenter];
        }

    }else{
        if (![self.subTopbar.leftItems containsItem:self.fromAlbum]) {
            [self.subTopbar removeItem:self.fromAlbum];
            [self.subTopbar addItem:self.fromAlbum atPosition:FSItemPositionLeft];
        }

        if (![self.subTopbar.leftItems containsItem:self.fromCamera]) {
            [self.subTopbar removeItem:self.fromCamera];
            [self.subTopbar addItem:self.fromCamera atPosition:FSItemPositionLeft];
        }
    }
}

- (void)makeConstraints{
    [super makeConstraints];

    [self.topbar.contentView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.right.top.fs_equalToSuperview();
    }];

    [self.subTopbar mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.right.fs_equalToSuperview();
        make.height.offset(TOOL_BAR_HEIGHT_NORMAL);
    }];

    [self.viewSignature mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.topbar.contentView.mas_bottom);
        make.bottom.equalTo(self.bottombar.mas_top);
        make.left.equalTo(self.view.mas_safeAreaLayoutGuideLeft);
        make.right.equalTo(self.view.mas_safeAreaLayoutGuideRight);
    }];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];

    if (self.viewSignature.hasChanged) {
        self.currentDib = [[self.viewSignature getCurrentDib] copy];
        self.currentRectSigPart = self.viewSignature.rectSigPart;
    }
}

- (void)viewWillTransitionToSize:(CGSize)size withTransitionCoordinator:(id<UIViewControllerTransitionCoordinator>)coordinator {
    [super viewWillTransitionToSize:size withTransitionCoordinator:coordinator];
#if _MAC_CATALYST_
    [self.viewSignature setNeedsDisplay];
#endif

    [coordinator animateAlongsideTransition:^(id <UIViewControllerTransitionCoordinatorContext> context)
      {
      } completion:^(id<UIViewControllerTransitionCoordinatorContext>  _Nonnull context) {
          if (!DEVICE_iPHONE) {
              if (self.viewSignature.picSignature) {
                  [self clearSign];
                  UIImage *resultImage = [self fitSmalImage:self.sourceImage];
                  [self clearSign];
                  [self.viewSignature loadPicSignature:resultImage];
              }
          }
      }];
}

- (void)selectCert {
    @weakify(self)
    [self.extensionsManager.permissionProvider checkPermission:FSFunctionPDFDigitalSignAdd checkPermissionState:^(FSPermissionState state) {
        @strongify(self)
        if (state == FSPermissionStateShow) {
            [self selectCertIntent];
        }
    }];
}

- (void)selectCertIntent
{
    DigitalCertSelectCtr *digitalCertSelectCtr = [[DigitalCertSelectCtr alloc] initWithServices:self.extensionsManager.viewCtrlManager];
    digitalCertSelectCtr.fs_presentByTopViewController = YES;
    digitalCertSelectCtr.title = FSLocalizedForKey(@"kSelectAddCert");
    [self.extensionsManager.viewCtrlManager presentViewController:digitalCertSelectCtr
                       animated:YES
                     completion:^{

                     }];
    typeof(self) __weak weakSelf = self;
    digitalCertSelectCtr.doneOperator = ^(NSString *path, NSString *passwd, NSString *md5) {
        NSString *msg = [FSLocalizedForKey(@"kCurrentSelectCert") stringByAppendingString:[path lastPathComponent]];
        SignatureViewController *strongSelf = weakSelf;
        if (strongSelf) {
            [strongSelf fs_currentSceneToastWithMsg:msg];
        }
        weakSelf.currentCertFileName = [path lastPathComponent];
        weakSelf.currentCertPasswd = passwd;
        weakSelf.currentCertMD5 = md5;
        [AnnotationSignature setCertFileToSiganatureSpace:md5 path:path];
        weakSelf.viewSignature.hasChanged = YES;
        weakSelf.isContentCert = YES;
    };
    digitalCertSelectCtr.cancelOperator = ^(NSString *path, NSString *passwd, NSString *md5) {
    };
}

- (void)updateSignature {
    if (self.currentCertFileName) {
       self.currentSignature = [AnnotationSignature getSignature:self.currentSignature.name];
    }

    self.currentSignature.rectSigPart = self.viewSignature.rectSigPart;
    self.currentSignature.color = self.viewSignature.color;
    self.currentSignature.diameter = self.viewSignature.diameter;
    self.currentSignature.certFileName = self.currentCertFileName;
    self.currentSignature.certMD5 = self.currentCertMD5;
    self.currentSignature.certPasswd = self.currentCertPasswd;
    self.currentSignature.isPic = !self.viewSignature.fromDraw;
    self.currentSignature.isClearBg = self.clearWhiteBtn.on;

    if (self.currentCertFileName) {
        UIImage *img = [self.viewSignature getCurrentImage];
        [self showAppearanceViewController:(AnnotationSignature *) self.currentSignature image:img];
        return;
    }

    [self setSignatureImage:self.currentSignature.name];
    [self.currentSignature update];
    [AnnotationSignature setSignatureSelected:self.currentSignature.name];
}

- (void)setSignatureImage:(NSString *)name {
    UIImage *img;
    NSData *data = [self.viewSignature getCurrentDib];

    if (self.clearWhiteBtn.on && !self.viewSignature.fromDraw && self.clearWhiteBtn.enabled){
        FSBitmap* bitmap = [Utility uiimageToFSBitmap:self.sourceImage];
        
        FSBitmap *bit = [bitmap fadeOutBackground:FSBitmapEnhanceAlgoGrayLevelTransformation threshold_algorithms:FSBitmapThresholdAlgoFixedThreshold threshold_level:1.0f];
        
        NSData *dataBuffer = [bit getBuffer];

        size_t width = [bit getWidth];
        size_t height = [bit getHeight];
        size_t totalBytes = [bit getWidth] * [bit getHeight] *4;
        uint8_t* rgbaData = (uint8_t *)malloc(totalBytes);
        [dataBuffer getBytes:rgbaData length:dataBuffer.length];
        CGDataProviderRef provider= CGDataProviderCreateWithData(NULL, rgbaData, width * height * 4, NULL);
          CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
          CGBitmapInfo bitmapInfo = kCGImageAlphaFirst | kCGBitmapByteOrder32Little;
          CGImageRef imageRef = CGImageCreate(width, height, 8, 32, width * 4, colorSpace, bitmapInfo, provider, NULL, true, kCGRenderingIntentDefault);
        UIImage *imgNew = [UIImage imageWithCGImage:imageRef];
        img = [self fitSmalImage:imgNew];
        data  = UIImagePNGRepresentation(img);
        [AnnotationSignature setSignatureImage:name img:img];
        
    }else{
        img = [self.viewSignature getCurrentImage];
        [AnnotationSignature setSignatureImage:name img:img];
    }

    [AnnotationSignature setSignatureDib:name data:data];
}

- (void)loadSignature {
    NSData *data = [AnnotationSignature getSignatureDib:self.currentSignature.name];
    AnnotationSignature* currentSignature = [AnnotationSignature getSignature:self.currentSignature.name];
    self.clearWhiteBtn.on = currentSignature.isClearBg;
    self.clearWhiteBtn.enabled = YES;
    CGRect rectSigPart = self.currentSignature.rectSigPart;
    if (self.viewSignature.hasChanged) {
        data = self.currentDib;
        rectSigPart = self.currentRectSigPart;
        [self.viewSignature loadSignature:data rect:rectSigPart];
        self.viewSignature.hasChanged = YES;
    } else {
        if (self.currentSignature.isPic) {
            if(currentSignature.isClearBg){
                self.clearWhiteBtn.enabled = NO;
            }
            UIImage *image = [UIImage imageWithData:data];
            self.sourceImage = image;
            [self.viewSignature loadPicSignature:image];
        }
        else
        {
            [self.viewSignature loadSignature:data rect:rectSigPart];
            self.viewSignature.hasChanged = NO;
        }
    }
}

- (void)addNewSignature {
    AnnotationSignature *sig = [[AnnotationSignature alloc] init];
    sig.rectSigPart = self.viewSignature.rectSigPart;
    sig.color = self.viewSignature.color;
    sig.diameter = self.viewSignature.diameter;
    sig.certFileName = self.currentCertFileName;
    sig.certMD5 = self.currentCertMD5;
    sig.certPasswd = self.currentCertPasswd;
    sig.isPic = !self.viewSignature.fromDraw;
    sig.isClearBg = self.clearWhiteBtn.on;
    if (_isContentCert) {
        UIImage *img;
        if (self.clearWhiteBtn.on && !self.viewSignature.fromDraw && self.clearWhiteBtn.enabled){
           
            FSBitmap* bitmap = [Utility uiimageToFSBitmap:self.sourceImage];
            FSBitmap *bit =  [bitmap fadeOutBackground:FSBitmapEnhanceAlgoGrayLevelTransformation threshold_algorithms:FSBitmapThresholdAlgoFixedThreshold threshold_level:1.0f];
            
            NSData *dataBuffer = [bit getBuffer];
            
            size_t width = [bit getWidth];
            size_t height = [bit getHeight];
            size_t totalBytes = [bit getWidth] * [bit getHeight] *4;
            uint8_t* rgbaData = (uint8_t *)malloc(totalBytes);
            [dataBuffer getBytes:rgbaData length:dataBuffer.length];
            CGDataProviderRef provider= CGDataProviderCreateWithData(NULL, rgbaData, width * height * 4, NULL);
            CGColorSpaceRef colorSpace = CGColorSpaceCreateDeviceRGB();
            CGBitmapInfo bitmapInfo = kCGImageAlphaFirst | kCGBitmapByteOrder32Little;
            CGImageRef imageRef = CGImageCreate(width, height, 8, 32, width * 4, colorSpace, bitmapInfo, provider, NULL, true, kCGRenderingIntentDefault);
            UIImage *imgNew = [UIImage imageWithCGImage:imageRef];
            img = [self fitSmalImage:imgNew];
        }else{
          img  = [self.viewSignature getCurrentImage];
        }
        [self showAppearanceViewController:(AnnotationSignature *) sig image: img];
        return;
    }
    NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
    BOOL firstCreate = [defaults boolForKey:@"firstCreateSignature"];
    if(!_isFieldSig && !firstCreate){
        [self.extensionsManager.pdfViewCtrl fs_currentSceneToastWithMsg:FSLocalizedForKey(@"kSignatureTapInsert")];
        [defaults setBool:YES forKey:@"firstCreateSignature"];
    }
    NSString *newName = [sig add];
    [self setSignatureImage:newName];
    [AnnotationSignature setSignatureSelected:newName];
}

- (void)setSignatureDefaultOption {
    AnnotationSignature *option = [AnnotationSignature getSignatureOption];
    self.viewSignature.color = option.color;
    self.viewSignature.diameter = option.diameter;
}

#pragma mark - event methods

- (NSString *)getName {
    return FSPreferenceModuleNameSignature;
}

- (void)propertySign:(FSItem *)item {
    FSPropertyMainView *propertyMainView = [[FSPropertyMainView alloc] initWithUIExtensionsManager:self.extensionsManager];
    [propertyMainView setPropertyColor:self.currentColor];
    [propertyMainView setLineWidth:self.currentLineWidth];
    [propertyMainView resetProperties:@[@(FSPropertyTypePropertyColor), @(FSPropertyTypeLineWidth)]];
    propertyMainView.arrowDirection = UIPopoverArrowDirectionUp;
    [propertyMainView setColorLayoutColors:@[UIColorHex(#000000),UIColorHex(#6F322E),UIColorHex(#215968),UIColorHex(#4F6229),UIColorHex(#000066),UIColorHex(#604A7B),UIColorHex(#777777)]];
    propertyMainView.delegate = self;
    
    DevicePosition *devicePos = [[DevicePosition alloc] init];
    devicePos.phoneRect = item.bounds;
    devicePos.inPhoneView = self.view;
    devicePos.padRect = item.bounds;
    devicePos.inPadView = item;
    [propertyMainView showPropertyMainView:devicePos];
    self.propertyMainView = propertyMainView;
}

#pragma mark topBaseBar method

- (void)cancelSign {
    dispatch_async(dispatch_get_main_queue(), ^{
        [self dismissViewControllerAnimated:NO
                                 completion:^{
                                     if (self.cancelHandler) {
                                         self.cancelHandler();
                                     }
                                 }];
    });
}

- (void)clearSign {
    [self.viewSignature clear];
    self.viewSignature.hasChanged = YES;
    self.clearWhiteBtn.on = NO;
    self.clearWhiteBtn.enabled = YES;
}

- (void)saveSign {
    if (self.viewSignature.hasChanged || self.clearWhiteBtn.on) {
        if (self.currentSignature == nil) //new
        {
            if (_isFieldSig) {
                if (_isContentCert) {
                    [self addNewSignature];
                } else {
                    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kFieldSignWithoutCert") preferredStyle:UIAlertControllerStyleAlert];
                    UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
                    [alertController addAction:action];
                    [[self fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
                    return;
                }
            } else {
                [self addNewSignature];
                if (!_isContentCert) {
                    [self dismissViewControllerAnimated:NO
                                             completion:^{
                        if (self.saveHandler) {
                            self.saveHandler();
                        }
                    }];
                }
            }
        } else //edit
        {
            [self updateSignature];
            if (!self.currentCertFileName){

                [self dismissViewControllerAnimated:NO
                                         completion:^{
                    if (self.saveHandler) {
                        self.saveHandler();
                    }
                }];
            }
        }
    }

}

- (void) showAppearanceViewController:(AnnotationSignature *)annotationSignature image:(UIImage *)img{
    self.signatureCtr = [[SignatureAppearanceViewController alloc] initWithUIExtensionsManager:self->_extensionsManager signature:annotationSignature image:img];
        [self.signatureCtr fs_setCustomTransitionAnimator:_extensionsManager.presentedVCAnimator];
        self.signatureCtr.delegate = self;
        [self presentViewController:self.signatureCtr animated:YES completion:nil];
}
- (void)cancelSign:(AnnotationSignature *)currentSignature{
    [self saveSignData:currentSignature];
}

 
- (void)saveSign:(AnnotationSignature *)currentSignature{
    [self saveSignData:currentSignature];
}

-(void)saveSignData:(AnnotationSignature *)currentSignature{
    if(currentSignature.titleName.length > 0){
        if (currentSignature.isDateOpen){
            NSDate *now = [NSDate date];
            FSDateTime *time = [Utility convert2FSDateTime:now];
            currentSignature.date = [Utility convertFSDateTime2NSDate:time];
        }
    }
    if (self.currentSignature == nil){
        NSString *newName = [currentSignature add];
        [self setSignatureImage:newName];
        [AnnotationSignature setSignatureSelected:newName];
        NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
        BOOL firstCreate = [defaults boolForKey:@"firstCreateSignature"];
        if(!_isFieldSig && !firstCreate){
            [self.extensionsManager.pdfViewCtrl fs_currentSceneToastWithMsg:FSLocalizedForKey(@"kSignatureTapInsert")];
            [defaults setBool:YES forKey:@"firstCreateSignature"];
        }
    }else{
        self.currentSignature = currentSignature;
        [self setSignatureImage:self.currentSignature.name];
        [self.currentSignature update];
        [AnnotationSignature setSignatureSelected:self.currentSignature.name];
    }
   
    @weakify(self)
    [self dismissViewControllerAnimated:NO
                             completion:^{
        @strongify(self)
        if (self.saveHandler) {
            self.saveHandler();
        }
    }];
}


- (UIImage*)fitSmalImage:(UIImage *)image
{
    if (nil == image) {
        return nil;
    }
    
    if (image.size.width < (MIN(self.view.fs_width, self.view.fs_height)-self.viewSignature.fs_top) && image.size.height < (MIN(self.view.fs_width, self.view.fs_height)-self.viewSignature.fs_top)) {
        return image;
    }
    CGSize size = [self fitsize:image.size];
    UIGraphicsBeginImageContext(size);
    CGRect rect = CGRectMake(0, 0, size.width, size.height);
    [image drawInRect:rect];
    UIImage *newImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return newImage;
}

- (CGSize)fitsize:(CGSize)thisSize
{
    if(thisSize.width == 0 && thisSize.height ==0)
        return CGSizeMake(0, 0);
    CGFloat wscale = thisSize.width/(MIN(self.view.fs_width, self.view.fs_height)-self.viewSignature.fs_top);
    CGFloat hscale = thisSize.height/(MIN(self.view.fs_width, self.view.fs_height)-self.viewSignature.fs_top);
    CGFloat scale = (wscale>hscale)?wscale:hscale;
    CGSize newSize = CGSizeMake((int)roundf(thisSize.width/scale), (int)roundf(thisSize.height/scale));
    return newSize;
}


- (UIImage *)getImageWithWarning {
    UIImage *img = [self.viewSignature getCurrentImage];
    if (!img) {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kSignatureEmpty") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
        [alertController addAction:action];
        [[self fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
    }
    return img;
}

#pragma mark -FSPropertyMainViewDelegate

- (void)propertyMainView:(FSPropertyMainView *)propertyMainView changedProperty:(FSPropertyType)property fromOldValue:(NSValue *)oldValue toValue:(NSValue *)newValue{
    if (property == FSPropertyTypePropertyColor) {
        int color = ((NSNumber *) newValue).intValue;
        self.viewSignature.color = color;
        self.currentColor = color;
        [Preference setIntValue:[self getName] type:@"Color" value:color];
        [self.propertyItem setImage:FSImageNamedAndCustomColor(@"read_property_color_ring",[UIColor colorWithRGB:self.currentColor]) forState:UIControlStateNormal];
    }else if (property == FSPropertyTypeLineWidth){
        float f = ((NSNumber *) newValue).floatValue; // 1~12
        self.viewSignature.diameter = f;
        self.currentLineWidth = f;
        [Preference setFloatValue:[self getName] type:@"Linewidth" value:f];
    }
}
#pragma mark -FSMediaFileToPDFViewControllerDelegate
- (void)mediaFileToPDFViewController:(FSMediaFileToPDFViewController *)mediaFileToPDFViewController didSelectedImage:(UIImage *)image{
    if (image) {
        self.sourceImage = image;
       
        UIImage *resultImage = [self fitSmalImage:image];
        [self clearSign];
        [self.viewSignature loadPicSignature:resultImage];
    }
}
- (void)mediaFileToPDFViewController:(FSMediaFileToPDFViewController *)mediaFileToPDFViewController didSelectedVideo:(AVPlayer *)video{}

@end
