/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSFormHiddenTextView.h"

@implementation FSFormHiddenTextView

- (BOOL)canPerformAction:(SEL)action withSender:(id)sender{
    if (self.menuControl.isMenuVisible) {
        return [self.menuControl canPerformAction:action withSender:sender];
    }
    if (self.menuControl) {
        return NO;
    }
    return [super canPerformAction:action withSender:sender];
}

- (NSMethodSignature *)methodSignatureForSelector:(SEL)sel {
    if (self.menuControl.isMenuVisible) {
        return [self.menuControl methodSignatureForSelector:sel];
    }
    if ([super methodSignatureForSelector:sel]) {
        return [super methodSignatureForSelector:sel];
    }
    return nil;
}

- (void)forwardInvocation:(NSInvocation *)anInvocation {
    if (self.menuControl.isMenuVisible) {
        return [self.menuControl forwardInvocation:anInvocation];
    }else{
        [super forwardInvocation:anInvocation];
    }
}

- (NSRange)convertTextRangeToRange:(UITextRange *)range{
    UITextPosition *beginning = self.beginningOfDocument;
    UITextPosition *start = range.start;
    UITextPosition *end = range.end;

    NSInteger location = [self offsetFromPosition:beginning toPosition:start];
    NSInteger length = [self offsetFromPosition:start toPosition:end];

    return NSMakeRange(location, length);
}

- (void)replaceRange:(UITextRange *)textRange withText:(NSString *)text{
    if (self.delegate && [self.delegate respondsToSelector:@selector(textView:shouldChangeTextInRange:replacementText:)]) {
        NSRange range = [self convertTextRangeToRange:textRange];
        if ([self.delegate textView:self shouldChangeTextInRange:range replacementText:text]) {
            [super replaceRange:textRange withText:text];
        }
        return;
    }
    [super replaceRange:textRange withText:text];
}

- (void)insertText:(NSString *)text{
    [super insertText:text];
}

- (nullable NSString *)textInRange:(UITextRange *)range{
    return [super textInRange:range];;
}

- (BOOL)shouldChangeTextInRange:(UITextRange *)range replacementText:(NSString *)text{
    return [super shouldChangeTextInRange:range replacementText:text];
}

- (void)setMarkedText:(nullable NSString *)markedText selectedRange:(NSRange)selectedRange{
    [super setMarkedText:markedText selectedRange:selectedRange];
}
- (void)unmarkText{
    [super unmarkText];
}

/* Methods for creating ranges and positions. */
- (nullable UITextRange *)textRangeFromPosition:(UITextPosition *)fromPosition toPosition:(UITextPosition *)toPosition{
    return [super textRangeFromPosition:fromPosition toPosition:toPosition];
}
- (nullable UITextPosition *)positionFromPosition:(UITextPosition *)position offset:(NSInteger)offset{
    return [super positionFromPosition:position offset:offset];
}
- (nullable UITextPosition *)positionFromPosition:(UITextPosition *)position inDirection:(UITextLayoutDirection)direction offset:(NSInteger)offset{
    return [super positionFromPosition:position inDirection:direction offset:offset];
}

/* Simple evaluation of positions */
- (NSComparisonResult)comparePosition:(UITextPosition *)position toPosition:(UITextPosition *)other{
    return [super comparePosition:position toPosition:other];
}
- (NSInteger)offsetFromPosition:(UITextPosition *)from toPosition:(UITextPosition *)toPosition{
    return [super offsetFromPosition:from toPosition:toPosition];
}


- (nullable UITextPosition *)positionWithinRange:(UITextRange *)range farthestInDirection:(UITextLayoutDirection)direction{
    return [super positionWithinRange:range farthestInDirection:direction];
}
- (nullable UITextRange *)characterRangeByExtendingPosition:(UITextPosition *)position inDirection:(UITextLayoutDirection)direction{
    return [super characterRangeByExtendingPosition:position inDirection:direction];
}


- (NSWritingDirection)baseWritingDirectionForPosition:(UITextPosition *)position inDirection:(UITextStorageDirection)direction{
    return [super baseWritingDirectionForPosition:position inDirection:direction];
}
- (void)setBaseWritingDirection:(NSWritingDirection)writingDirection forRange:(UITextRange *)range{
    [super setBaseWritingDirection:writingDirection forRange:range];
}

- (CGRect)firstRectForRange:(UITextRange *)range{
    return [super firstRectForRange:range];
}
- (CGRect)caretRectForPosition:(UITextPosition *)position{
    return [super caretRectForPosition:position];
}
- (NSArray<UITextSelectionRect *> *)selectionRectsForRange:(UITextRange *)range{
    return [super selectionRectsForRange:range];
}
@end
