/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSFormImageFieldHandler.h"
#import "FormAnnotHandler.h"
#import "FSFileAndImagePicker.h"

@interface FSFormImageFieldHandler ()<FSFileAndImagePickerDelegate>
@property (nonatomic, weak) FormAnnotHandler *formAnnotHandler;
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
@property (nonatomic, strong) FSFileAndImagePicker *mediaPicker;
@property (nonatomic, weak) FSWidget *currentWidget;
@end
@implementation FSFormImageFieldHandler

- (instancetype)initWithFormAnnotHandler:(FormAnnotHandler *)formAnnotHandler{
    self = [super init];
    if (self) {
        _formAnnotHandler = formAnnotHandler;
        _extensionsManager = formAnnotHandler.extensionsManager;
        self.mediaPicker = [[FSFileAndImagePicker alloc] init];
    }
    return self;
}

- (void)showMenu:(FSWidget *)widget{
    if (widget.nonempty) {
        int pageIndex = widget.pageIndex;
        FSPDFViewCtrl *pdfViewCtrl = self.extensionsManager.pdfViewCtrl;
        CGRect dvRect = [self.extensionsManager.pdfViewCtrl convertPageViewRectToDisplayViewRect:[pdfViewCtrl convertPdfRectToPageViewRect:widget.fsrect pageIndex:pageIndex] pageIndex:pageIndex];
        
        self.mediaPicker.extensionsManager = _extensionsManager;
        self.mediaPicker.fileType = FSMediaFileTypeImage;
        self.mediaPicker.delegate = self;
        @fs_weakify(self)
        self.mediaPicker.adjustMenusAction = ^NSMutableArray *(NSMutableArray<MenuItem *> * _Nonnull menus) {
            @fs_strongify(self)
            if ([widget hasMKEntry:FSAnnotMKEntryNormalIcon]) {
                MenuItem *clearItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kClear") object:self action:@selector(clear)];
                [menus addObject:clearItem];
            }
            return menus;
        };
        [self.mediaPicker showMenu:dvRect];
        _currentWidget = widget;
    }
}
- (void)clear{
    if (self.currentWidget.nonempty) {
        [self.currentWidget removeMKEntry:FSAnnotMKEntryNormalIcon];
        
        FSPDFViewCtrl *pdfViewCtrl = self.extensionsManager.pdfViewCtrl;
        
        [pdfViewCtrl lockRefresh];
        [self.currentWidget resetAppearanceStream];
        [pdfViewCtrl unlockRefresh];
        
        self.extensionsManager.isDocModified= YES;
        
        int pageIndex = self.currentWidget.pageIndex;
        CGRect refreshRect = [pdfViewCtrl convertPdfRectToPageViewRect:self.currentWidget.fsrect pageIndex:pageIndex];
        refreshRect = CGRectInset(refreshRect, -30, -30);
        [self.extensionsManager.pdfViewCtrl refresh:refreshRect pageIndex:pageIndex];
        
        
    }
}

#pragma mark <FSFileAndImagePickerDelegate>
- (void)fileAndImagePicker:(FSFileAndImagePicker *)fileAndImagePicker didPickImage:(UIImage *)image imagePath:(NSString *)imagePath{
    
    if (self.currentWidget.nonempty) {
        FSImage *fs_image;
        if (image) {
            fs_image = [Utility uiimageToFSImage:image];
        }else{
            fs_image = [[FSImage alloc] initWithPath:imagePath];
        }
        [self.currentWidget setMKNormalIconImage:fs_image frame_index:0];
        FSPDFViewCtrl *pdfViewCtrl = self.extensionsManager.pdfViewCtrl;
        
        [pdfViewCtrl lockRefresh];
        [self.currentWidget resetAppearanceStream];
        [pdfViewCtrl unlockRefresh];
        
        self.extensionsManager.isDocModified= YES;
        
        int pageIndex = self.currentWidget.pageIndex;
        CGRect refreshRect = [pdfViewCtrl convertPdfRectToPageViewRect:self.currentWidget.fsrect pageIndex:pageIndex];
        refreshRect = CGRectInset(refreshRect, -30, -30);
        [self.extensionsManager.pdfViewCtrl refresh:refreshRect pageIndex:pageIndex];
        
        [self.formAnnotHandler killFocus];
    }
}
- (void)fileAndImagePickerDidCancel:(FSFileAndImagePicker *)fileAndImagePicker {
    [self completed];
}

- (void)completed{
}
@end
