/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FormAnnotHandler.h"
#import "UIExtensionsConfig+private.h"
#import "PageNavigationModule.h"
#import "FSFormHiddenTextView.h"
#import "FSPermissionUtilty.h"
#import "FSFormImageFieldHandler.h"
#import "FSFillFormHistoryDataView.h"
#import "FSHistoryDataModule+Private.h"
#import "AudioVideoModule.h"

@interface FormAnnotHandler () <UITextViewDelegate, IGestureEventListener, IWindowFrameChangedListener>

@property (nonatomic, strong) FSFormHiddenTextView *hiddenTextField;
@property (nonatomic, assign) NSInteger annotCountTag;
@property (nonatomic, strong) NSMutableArray *menuItems;
@property (nonatomic, strong) FSPointF *currentMenuPointf;
@property (nonatomic, strong) FSPointF *lastPtF;
@property (nonatomic, strong) FSWidget *foucsWidget;
@property (nonatomic, strong) FSFormImageFieldHandler *imageFieldHandler;
@property (nonatomic, weak) FSViewPopoverController *datePickerPopVC;
@property (nonatomic, strong) UIDatePicker *datePicker;
@property (nonatomic, strong) FSFillFormHistoryDataView *historyDataView;
@property (nonatomic, assign) BOOL needBack;
@property (nonatomic, strong) FSPDFObject *soundObj;
@property (nonatomic, assign) BOOL isPanGestures;
@property (nonatomic, copy) NSString *lastMarkText;
@property (nonatomic, copy) NSString *lastContentText;
@end

@implementation FormAnnotHandler

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super initWithUIExtensionsManager:extensionsManager];
    if (self) {
        [self.extensionsManager registerRotateChangedListener:self];
        [self.extensionsManager registerWindowFrameChangedListener:self];
        _taskServer = self.extensionsManager.taskServer;
        _lock = dispatch_semaphore_create(1);
        _keyboardHeight = 0;
        _formFiller = nil;
        _form = nil;
        _focusedWidgetIndex = -1;
        _inputCharacterLocation = 0;
        _needRotate = NO;
        _annotCountTag = -1;
        _imageFieldHandler = [[FSFormImageFieldHandler alloc] initWithFormAnnotHandler:self];
        
        _historyDataView = [[FSFillFormHistoryDataView alloc] initWithFormAnnotHandler:self];
        
        if (!self.extensionsManager.config.defaultSettings.disableFormNavigationBar) {
            self.formNaviBar = [self buildFormNaviBar];
            [self.pdfViewCtrl addSubview:self.formNaviBar];
            [self.formNaviBar mas_makeConstraints:^(MASConstraintMaker *make) {
                if (DEVICE_iPHONE || [self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
                    make.left.right.bottom.mas_equalTo(self.pdfViewCtrl);
                }else{
                    make.centerX.mas_equalTo(self.pdfViewCtrl);
                    make.bottom.mas_equalTo(self.pdfViewCtrl.fs_mas_bottom).offset(-TOOL_BAR_HEIGHT_NORMAL);
                    make.width.offset([self.formNaviBar getWidthWithPosition:FSItemPositionLeft] + ITEM_MARGIN_NORMAL + [self.formNaviBar getWidthWithPosition:FSItemPositionRight]);
                }
            }];
            [self.formNaviBar setHidden:YES];
            [(PageNavigationModule *)[self.extensionsManager getModuleByName:@"PageNavigation"] setIsFormNavBarShowState:NO];
        }

        self.hiddenTextField = [[FSFormHiddenTextView alloc] init];
        self.hiddenTextField.autocorrectionType = UITextAutocorrectionTypeNo;
        self.hiddenTextField.hidden = YES;
        self.hiddenTextField.delegate = self;
        self.hiddenTextField.text = @"";
        
        [self.pdfViewCtrl addSubview:self.hiddenTextField];
        
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(keyboardWasShown:)
                                                     name:UIKeyboardDidShowNotification
                                                   object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(keyboardWasHidden:)
                                                     name:UIKeyboardWillHideNotification
                                                object:nil];
        
        [self.extensionsManager registerGestureEventListener:self];
    }
    return self;
}

- (void)dealloc{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
    [self.extensionsManager unregisterRotateChangedListener:self];
}

- (FSFiller *)formFiller{
    if (!self.form || [self.form isEmpty]) return nil;
    if (!_formFiller || [_formFiller isEmpty]) {
        _formFiller = [self.form getFormFiller];
        if (!_formFiller || [_formFiller isEmpty]) {
            FSCallbackSubstitute *fillerAssistCallbackSubstitute = [FSCallbackSubstitute fillerAssistCallbackSubstitute];
            fillerAssistCallbackSubstitute.currentTarget = self;
            _formFiller = [[FSFiller alloc] initWithForm:self.form assist:fillerAssistCallbackSubstitute];
        }
    }
    if (!_formFiller || [_formFiller isEmpty]) return nil;
    return _formFiller;
}

- (FSForm *)form{
    if ([self.pdfViewCtrl isDynamicXFA]) return nil;
    if (!_form || [_form isEmpty]) {
        FSPDFDoc *document = self.pdfViewCtrl.currentDoc;
        BOOL hasForm = NO;
        @try {
            hasForm = [document hasForm];
        } @catch (NSException *e) {
        }
        if (hasForm) {
            // creating form filler may block current thread if -[ExActionHandler alert:title:type:icon:] called
            _form = [[FSForm alloc] initWithDocument:document];
        }
    }
    if (!_form || [_form isEmpty]) return nil;
    return _form;
}

-(FSFieldType)getFormAnnotFieldType:(FSAnnot *)annot{
    if (annot.getType == FSAnnotWidget) {
        FSWidget *widget = [[FSWidget alloc] initWithAnnot:annot];
        FSField *field = [widget getField];
        FSFieldType fieldType = [field getType];
        return  fieldType;
    }
    return FSFieldTypeUnknown;
}

- (void)setFocus:(FSControl *)control isHidden:(BOOL)hidden {
    if (control && ![control isEmpty]) {
        @try {
            [self.formFiller setFocus:control];
        } @catch (NSException *exception) {
        }
    } else {
        @try {
            [self killFocus];
        } @catch (NSException *exception) {
        }
    }
    
    if (control && [self beginDatePickerPopup]) {
        [self.formNaviBar setHidden:YES];
    }else{
        if (self.datePickerPopVC) {
            [self.datePickerPopVC dismissViewControllerAnimated:YES completion:nil];
        }
        [self.formNaviBar setHidden:hidden];
    }
    [(PageNavigationModule *)[self.extensionsManager getModuleByName:@"PageNavigation"] setIsFormNavBarShowState:!hidden];

}

- (void)setFoucsWidget:(FSWidget *)foucsWidget{
    _foucsWidget = foucsWidget;
    [self.extensionsManager setCurrentAnnot:foucsWidget];
}

- (void)killFocus{
    if ([self.formFiller killFocus]) {
        [self focusLostFromControl:nil value:nil];
        if (self.extensionsManager.currentAnnot.nonempty && !self.foucsWidget.nonempty) {
            [self.extensionsManager setCurrentAnnot:nil];
        }
    }
}

- (BOOL)canNavigateTo:(FSControl *)control {
    if (![control isEmpty]) {
        FSField *field = [control getField];
        unsigned int fflags = [field getFlags];
        FSFieldType fieldType = [[control getField] getType];
        BOOL bRet = fieldType == FSFieldTypePushButton || fieldType == FSFieldTypeSignature || fflags & FSFieldFlagReadOnly;
        return !bRet;
    }
    else{
        return NO;
    }
}

- (BOOL)canShowKeybord:(FSControl *)control {
    if (_MAC_CATALYST_) return NO;
    if ([control getWidget].dateTextFieldFormat) return NO;
    FSField *field = [control getField];
    FSFieldType fieldType = [field getType];
    return FSFieldTypeTextField == fieldType || (FSFieldTypeComboBox == fieldType && ([field getFlags] & FSFieldFlagComboEdit));
}

- (UIDatePicker *)datePicker{
    if (!_datePicker) {
        UIDatePicker *datePicker = [[UIDatePicker alloc] init];
        datePicker.datePickerMode = UIDatePickerModeDate;
        datePicker.date = [NSDate date];
    #if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
        if (@available(iOS 13.4, *)) {
            datePicker.preferredDatePickerStyle = UIDatePickerStyleWheels;
        }
    #endif
        _datePicker = datePicker;
    }
    return _datePicker;
}

- (UIView *)inputAccessoryView{
    if (self.extensionsManager.config.defaultSettings.disableFormNavigationBar) return nil;
    UIView *contentView = [UIView new];
    self.textFormNaviBar = [self buildFormNaviBar];
    contentView.backgroundColor = self.textFormNaviBar.backgroundColor;
    [contentView addSubview:self.textFormNaviBar];
    
    FSHistoryDataModule *historyDataModule = [self.extensionsManager getModuleByName:Module_History_Data];
    FSHistoryDataConfig *config = [historyDataModule configForScene:FSInteractiveSceneFillingForms];
    if (config.enable && self.hiddenTextField.keyboardType != UIKeyboardTypeASCIICapable) {
        [contentView addSubview:self.historyDataView];
        
        [self.textFormNaviBar mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.top.fs_equalToSuperview().priorityHigh();
            if (DEVICE_iPAD && ![self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
                make.bottom.fs_equalToSuperview().priorityHigh();
            }else{
                make.bottom.mas_equalTo(self.historyDataView.mas_top).priorityHigh();
            }
            make.left.right.fs_equalToSuperview();
        }];
        
        if (DEVICE_iPAD && ![self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            contentView.fs_size = CGSizeMake(self.pdfViewCtrl.fs_width, TAB_BAR_HEIGHT_NORMAL);
            [self.historyDataView mas_remakeConstraints:^(MASConstraintMaker *make) {
                UIView *leftLastItem = [self.textFormNaviBar getItemAtPosition:FSItemPositionLeft index:1];
                UIView *rightLastItem = [self.textFormNaviBar getItemAtPosition:FSItemPositionRight index:1];
                make.left.equalTo(leftLastItem.mas_right).offset(ITEM_BOTH_SPACE_NORMAL);
                make.right.equalTo(rightLastItem.mas_left).offset(-ITEM_BOTH_SPACE_NORMAL);
                make.bottom.fs_equalToSuperview();
                make.height.offset(TAB_BAR_HEIGHT_NORMAL);
            }];
        }else{
            contentView.fs_size = CGSizeMake(self.pdfViewCtrl.fs_width, TAB_BAR_HEIGHT_NORMAL * 2);
            [self.historyDataView mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.left.right.bottom.fs_equalToSuperview();
                make.height.offset(TAB_BAR_HEIGHT_NORMAL);
            }];
        }
    }else{
        contentView.fs_size = CGSizeMake(self.pdfViewCtrl.fs_width, TAB_BAR_HEIGHT_NORMAL);
        [self.textFormNaviBar mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.top.bottom.fs_equalToSuperview().priorityHigh();
            if (DEVICE_iPHONE) {
                make.left.right.fs_equalToSuperview();
            }else{
                make.centerX.fs_equalToSuperview();
                make.width.offset(IPAD_PREFERENCE_BAR_WIDTH);
        
                CGFloat spacing = (IPAD_PREFERENCE_BAR_WIDTH -  [self.textFormNaviBar getWidthWithPosition:FSItemPositionLeft] - ITEM_MARGIN_NORMAL - [self.textFormNaviBar getWidthWithPosition:FSItemPositionRight])/5;
                
                for (FSToolbarItem *item in self.textFormNaviBar.leftItems.items) {
                    item.margin += spacing;
                }
                for (FSToolbarItem *item in self.textFormNaviBar.rightItems.items) {
                    item.margin += spacing;
                }

                [self.textFormNaviBar readdItemsForPosition:FSItemPositionLeft];
                [self.textFormNaviBar readdItemsForPosition:FSItemPositionRight];
            }
        }];
    }
    

    return contentView;
}

- (FSToolbar *)buildFormNaviBar {
    @weakify(self);
    FSToolbar *formNaviBar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleBottom];
    if (DEVICE_iPAD && ![self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
        [formNaviBar fs_setLayerCornerRadius:LAYER_BORDER_WIDTH_6];
    }
    
    FSItem *prevItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"common_previous") background:nil];
    prevItem.accessibilityLabel = FSLocalizedForKey(@"kPreviousBtnVoiceover");
    prevItem.onTapClick = ^(FSItem *_item) {
        @strongify(self);
        FSPDFDoc *doc = [self.pdfViewCtrl getDoc];
        int curPageIndex = -1;
        if (self.extensionsManager.currentAnnot)
            curPageIndex = [self.extensionsManager.currentAnnot pageIndex];
        else
            curPageIndex = [self.pdfViewCtrl getCurrentPage];
        BOOL canShowKeybord = NO;
        if (self.foucsWidget) {
            canShowKeybord = [self canShowKeybord:[self.foucsWidget getControl]];
        }
        int curIndex = self->_focusedWidgetIndex;
        int pageCount = [doc getPageCount];
        int findPageIndex = curPageIndex;
        while (true) {
            FSPDFPage *page = [doc getPage:findPageIndex];
            FSTabOrderMgr *tab = [[FSTabOrderMgr alloc] initWithPage:page];
            tab.orderType = FSTabOrderMgrRow;
            FSAnnot *curAnnot = curIndex == -1 ? nil : [page getAnnot:curIndex];
            FSAnnot *annot = curAnnot == nil ? [tab getLastAnnot] : [tab getPrevAnnot:curAnnot];
            FSAnnot *firstAnnot = [tab getFirstAnnot];
            if ([annot isEmpty]) {
                if ([curAnnot isEqualToAnnot:firstAnnot] || [firstAnnot isEmpty]){
                    if (findPageIndex == 0)
                        findPageIndex = pageCount - 1;
                    else
                        findPageIndex--;
                    curIndex = -1;
                }
                continue;
            }
            if ([annot getType] == FSAnnotWidget) {
                FSWidget *widget = [[FSWidget alloc] initWithAnnot:annot];
                if ([self canNavigateTo:[widget getControl]]) {
                    canShowKeybord = [self canShowKeybord:[widget getControl]];
                    FSRectF *rect = [annot getRect];

                    [self setFocus:[widget getControl] isHidden:NO];
                    if (canShowKeybord) {
                        if (findPageIndex != curPageIndex) [self.pdfViewCtrl gotoPage:findPageIndex animated:NO];
                        [self gotoFormField];
                    } else {
                        if (![self.pdfViewCtrl isContinuous]) {
                            CGRect pvRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:rect pageIndex:findPageIndex];
                            CGPoint pvPt = CGPointZero;
                            pvPt.x = pvRect.origin.x - (self.pdfViewCtrl.frame.size.width - pvRect.size.width) / 2;
                            pvPt.y = pvRect.origin.y - (self.pdfViewCtrl.frame.size.height - pvRect.size.height) / 2;
                            FSPointF *pdfPt = [self.pdfViewCtrl convertPageViewPtToPdfPt:pvPt pageIndex:findPageIndex];
                            [self.pdfViewCtrl gotoPage:findPageIndex withDocPoint:pdfPt animated:YES];
                        }else{
                            PDF_LAYOUT_MODE pageLayoutMode = [self.pdfViewCtrl getPageLayoutMode];
                            CGRect pvRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:rect pageIndex:findPageIndex];
                            
                            float orignY = pvRect.origin.y;
                            BOOL isTwoPagesMode = (pageLayoutMode == PDF_LAYOUT_MODE_TWO || pageLayoutMode == PDF_LAYOUT_MODE_TWO_LEFT || pageLayoutMode == PDF_LAYOUT_MODE_TWO_RIGHT || pageLayoutMode == PDF_LAYOUT_MODE_TWO_MIDDLE);
                            if (!isTwoPagesMode) {
                                orignY = pvRect.origin.y - CGRectGetHeight(self.pdfViewCtrl.frame)/2 ;
                            }
                            CGPoint newOrign = CGPointMake(pvRect.origin.x - CGRectGetWidth(self.pdfViewCtrl.frame)/2, orignY);
                            FSPointF *pdfPt = [self.pdfViewCtrl convertPageViewPtToPdfPt:newOrign pageIndex:findPageIndex];
                            [self.pdfViewCtrl gotoPage:findPageIndex withDocPoint:pdfPt animated:YES];
                        }
                        
                        
                    }
                    self->_focusedWidgetIndex = [annot getIndex];
                    break;
                }
            }
            curIndex = [annot getIndex];
        }
        
        if (canShowKeybord || (_MAC_CATALYST_ && self.editFormControlNeedTextInput)) {
            
            [self beginInput];
        } else{
            [self endTextInputWithExit];
        }
    };
    [formNaviBar addItem:prevItem spacing:ITEM_MARGIN_NORMAL atPosition:FSItemPositionLeft];

    FSItem *nextItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"common_next") background:nil];
    nextItem.accessibilityLabel = FSLocalizedForKey(@"kNextBtnVoiceover");
    nextItem.onTapClick = ^(FSItem *_item) {
        @strongify(self);
        FSPDFDoc *doc = [self.pdfViewCtrl getDoc];
        int curPageIndex = -1;
        if (self.extensionsManager.currentAnnot)
            curPageIndex = [self.extensionsManager.currentAnnot pageIndex];
        else
            curPageIndex = [self.pdfViewCtrl getCurrentPage];
        BOOL canShowKeybord = NO;
        if (self.foucsWidget) {
            canShowKeybord = [self canShowKeybord:[self.foucsWidget getControl]];
        }
        int pageCount = [doc getPageCount];
        int i = self->_focusedWidgetIndex;
        int curIndex = i;
        int findPageIndex = curPageIndex;
        while (true) {
            FSPDFPage *page = [doc getPage:findPageIndex];
            FSTabOrderMgr *tab = [[FSTabOrderMgr alloc] initWithPage:page];
            tab.orderType = FSTabOrderMgrRow;
            FSAnnot *curAnnot = curIndex == -1 ? nil : [page getAnnot:curIndex];
            FSAnnot *annot = curAnnot == nil ? [tab getFirstAnnot] : [tab getNextAnnot:curAnnot];
            FSAnnot *lastAnnot = [tab getLastAnnot];
            if ([annot isEmpty]) {
                if ([curAnnot isEqualToAnnot:lastAnnot] || [lastAnnot isEmpty]){
                    if (findPageIndex == pageCount - 1)
                        findPageIndex = 0;
                    else
                        findPageIndex++;
                    curIndex = -1;
                }
                continue;
            }
            if ([annot getType] == FSAnnotWidget) {
                FSWidget *widget = [[FSWidget alloc] initWithAnnot:annot];
                if ([self canNavigateTo:[widget getControl]]) {
                    canShowKeybord = [self canShowKeybord:[widget getControl]];
                    FSRectF *rect = [annot getRect];
                    
                    [self setFocus:[widget getControl] isHidden:NO];
                    if (canShowKeybord) {
                        if (findPageIndex != curPageIndex) [self.pdfViewCtrl gotoPage:findPageIndex animated:NO];
                        [self gotoFormField];
                    } else {
                        if (![self.pdfViewCtrl isContinuous]) {
                            CGRect pvRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:rect pageIndex:findPageIndex];
                            CGPoint pvPt = CGPointZero;
                            pvPt.x = pvRect.origin.x - (self.pdfViewCtrl.frame.size.width - pvRect.size.width) / 2;
                            pvPt.y = pvRect.origin.y - (self.pdfViewCtrl.frame.size.height - pvRect.size.height) / 2;
                            FSPointF *pdfPt = [self.pdfViewCtrl convertPageViewPtToPdfPt:pvPt pageIndex:findPageIndex];
                            [self.pdfViewCtrl gotoPage:findPageIndex withDocPoint:pdfPt animated:YES];
                        }else{
                            PDF_LAYOUT_MODE pageLayoutMode = [self.pdfViewCtrl getPageLayoutMode];
                            CGRect pvRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:rect pageIndex:findPageIndex];
                            
                            float orignY = pvRect.origin.y;
                            BOOL isTwoPagesMode = (pageLayoutMode == PDF_LAYOUT_MODE_TWO || pageLayoutMode == PDF_LAYOUT_MODE_TWO_LEFT || pageLayoutMode == PDF_LAYOUT_MODE_TWO_RIGHT || pageLayoutMode == PDF_LAYOUT_MODE_TWO_MIDDLE);
                            if (!isTwoPagesMode) {
                                orignY = pvRect.origin.y - CGRectGetHeight(self.pdfViewCtrl.frame)/2 ;
                            }
                            CGPoint newOrign = CGPointMake(pvRect.origin.x - CGRectGetWidth(self.pdfViewCtrl.frame)/2, orignY);
                            FSPointF *pdfPt = [self.pdfViewCtrl convertPageViewPtToPdfPt:newOrign pageIndex:findPageIndex];
                            [self.pdfViewCtrl gotoPage:findPageIndex withDocPoint:pdfPt animated:YES];
                        }
                        
                    }
                    self->_focusedWidgetIndex = [annot getIndex];
                    break;
                }
            }
            curIndex = [annot getIndex];
        }
        
        if (canShowKeybord || (_MAC_CATALYST_ && self.editFormControlNeedTextInput)) {
            [self beginInput];
        } else{
            [self endTextInputWithExit];
        }
    };
    [formNaviBar addItem:nextItem spacing:ITEM_MARGIN_NORMAL atPosition:FSItemPositionLeft];
    
    FSItem *doneItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kDone")];
    [BindThemeKeySel(doneItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem  *_Nonnull target, SEL  _Nonnull sel) {
        [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
    }];
    doneItem.onTapClick = ^(FSItem *_item) {
        @strongify(self);
        if (self.datePickerPopVC) {
            [self setDateToTextField:self.datePicker];
        }
        [self killFocus];
        if (self.datePickerPopVC) {
            [self.datePickerPopVC dismissViewControllerAnimated:YES completion:nil];
        }
    };
    [formNaviBar addItem:doneItem spacing:ITEM_MARGIN_NORMAL atPosition:FSItemPositionRight];

    FSItem *resetItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kReset")];
    [BindThemeKeySel(resetItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem  *_Nonnull target, SEL  _Nonnull sel) {
        [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
    }];
    resetItem.onTapClick = ^(FSItem *_item) {
        @strongify(self);
        
        [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingFormBarReset];
        
        FSPDFDoc *doc = [self.pdfViewCtrl getDoc];
        
        int curPageIndex = [self.pdfViewCtrl getCurrentPage];
        if (self.extensionsManager.currentPageIndex >= 0 ) {
            curPageIndex = self.extensionsManager.currentPageIndex;
        }
        
        FSPDFPage *page = [doc getPage:curPageIndex];
        FSAnnot *annot = [page getAnnot:self->_focusedWidgetIndex];
        if ([annot getType] == FSAnnotWidget) {
            FSWidget *widget = [[FSWidget alloc] initWithAnnot:annot];
            FSField *field = [widget getField];

            FSAnnotAttributes *tmpAttributes = self.attributesBeforeModify;
            @try {
                [self killFocus];
            } @catch (NSException *exception) {
            }
            
            [field reset];
            [self setFocus:[widget getControl] isHidden:NO];
            if (tmpAttributes) self.attributesBeforeModify = tmpAttributes;
            
            BOOL canShowKeybord = [self canShowKeybord:[widget getControl]];
            if (canShowKeybord || (_MAC_CATALYST_ && self.editFormControlNeedTextInput)) {
                [self beginInput];
            } else
                [self endTextInputWithExit];
            self->_focusedWidgetIndex = [widget getIndex];
            self.hiddenTextField.text = @"";
            CGRect newRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:annot.pageIndex];
            newRect = CGRectInset(newRect, -30, -30);
            [self.pdfViewCtrl refresh:newRect pageIndex:annot.pageIndex needRender:NO];
            self.extensionsManager.isDocModified = YES;
        }
    };
    [formNaviBar addItem:resetItem spacing:ITEM_MARGIN_NORMAL atPosition:FSItemPositionRight];
    return formNaviBar;
}
- (void)creatFormFiller{
    if (self.formFiller) {
        [self setEnableHighlightForm:self.extensionsManager.enableHighlightForm];
    }
}

- (UIColor *)borderColorForSelectedAnnot:(FSAnnot *)annot {
    return  self.extensionsManager.config.defaultSettings.commonlyUsed.color;
}

#pragma IAnnotHandler

- (FSAnnotType)getType {
    return FSAnnotWidget;
}

- (BOOL)isHitAnnot:(FSAnnot *)annot point:(FSPointF *)point {
    if ([annot isEmpty]) return NO;
    if ([self.extensionsManager annotsPermissionCanModify] && ![self.extensionsManager allAnnotPermission]) {
        return NO;
    }
    if (![FSPermissionUtilty canFillFormInDocument:self.extensionsManager]) {
        return NO;
    }
    FSAnnot *hitAnnot = nil;
    @try {
        hitAnnot = [[annot getPage] getAnnotAtPoint:point tolerance:5];
    } @catch (NSException *exception) {
        return NO;
    }
    if (hitAnnot && ![hitAnnot isEmpty] && [hitAnnot isEqualToAnnot:annot]) {
        FSWidget *widget = [[FSWidget alloc] initWithAnnot:hitAnnot];
        if (widget && ![widget isEmpty]) {
            FSField *field = [widget getField];
            if (field && ![field isEmpty] && ([field getFlags] & FSFieldFlagReadOnly))
                return NO;
        }
    }
    return [super isHitAnnot:annot point:point];
}

- (void)onAnnotDeselected:(FSAnnot *)annot {
    [super onAnnotDeselected:annot];
    [self endTextInputWithExit];
    [self.extensionsManager.thumbnailModule removeThumbnailCacheOfPageAtIndex:annot.pageIndex];
    if (self.foucsWidget.nonempty && [self.foucsWidget isEqualToAnnot:annot]) {
        [self killFocus];
    }
}

- (BOOL)addAnnot:(FSAnnot *)annot addUndo:(BOOL)addUndo {
    if (annot.getType != FSAnnotWidget) {
        return [super addAnnot:annot addUndo:addUndo];;
    }
    [self.extensionsManager.thumbnailModule removeThumbnailCacheOfPageAtIndex:annot.pageIndex];
    FSPDFPage *page = [annot getPage];
    if (addUndo) {
        FSAnnotAttributes *attributes = [FSAnnotAttributes attributesWithAnnot:annot];
        [self.extensionsManager addUndoItem:[UndoItem itemForUndoAddAnnotWithAttributes:attributes page:page annotHandler:self]];
    }
    
    if ([[self.form getFormFiller] isEmpty] && _formFiller) {
        _formFiller = nil;
    }
    if (!_formFiller || [_formFiller isEmpty]) {
        [self creatFormFiller];
    }
//    self.extensionsManager.isDocModified = YES;
    [self.extensionsManager onAnnotAdded:page annot:annot];
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        [self refreshPageForAnnot:annot reRenderPage:YES];
    });
    return YES;
}

- (BOOL)modifyAnnot:(FSAnnot *)annot addUndo:(BOOL)addUndo {
    if (annot.getType != FSAnnotWidget) {
        return [super modifyAnnot:annot addUndo:addUndo];;
    }
    if (![annot canModify]) {
        return NO;
    }
    FSPDFPage *page = [annot getPage];
    if (!page || [page isEmpty]) {
        return NO;
    }
    if (addUndo) {
        annot.modifiedDate = [NSDate date];
        [self.extensionsManager addUndoItem:[UndoItem itemForUndoModifyAnnotWithOldAttributes:self.attributesBeforeModify newAttributes:[FSAnnotAttributes attributesWithAnnot:annot] pdfViewCtrl:self.pdfViewCtrl page:page annotHandler:self]];
    }
    self.extensionsManager.isDocModified = YES;
    [self refreshPageForAnnot:annot reRenderPage:YES];
    return YES;
}

- (BOOL)removeAnnot:(FSAnnot *)annot addUndo:(BOOL)addUndo {
    if (annot.getType != FSAnnotWidget) {
        return [super removeAnnot:annot addUndo:addUndo];;
    }
    [self hideMenu];
    if (self.extensionsManager.propertyMainView.isShowing) {
        [self.extensionsManager.propertyMainView dismiss];
    }
    
    FSPDFPage *page = [annot getPage];
    if (!page || [page isEmpty]) {
        return NO;
    }
    
    int pageIndex = annot.pageIndex;
    CGRect rect = [self getAnnotRect:annot];
    rect = UIEdgeInsetsInsetRect(rect, self.annotRefreshRectEdgeInsets);
    rect = CGRectIntersection(rect, [self.pdfViewCtrl getPageView:pageIndex].bounds);
    
    [self.extensionsManager onAnnotWillDelete:page annot:annot];
    if (addUndo) {
        FSAnnotAttributes *attributes = self.attributesBeforeModify ?: [FSAnnotAttributes attributesWithAnnot:annot];
        [self.extensionsManager addUndoItem:[UndoItem itemForUndoDeleteAnnotWithAttributes:attributes page:page annotHandler:self]];
    }
    
    [self.pdfViewCtrl lockRefresh];
    FSWidget *widget = [[FSWidget alloc] initWithAnnot:annot];
    FSControl *control = [widget getControl];
    if (![control isEmpty]) {
        FSField *field = [control getField];
        if (![field isEmpty]) {
            if ([annot isEqualToAnnot:self.annot]) {
                [self setFocus:nil isHidden:YES];
            }
            
            if ([field getType] == FSFieldTypeSignature) {
                FSSignature *sig = [[FSSignature alloc] initWithField:field];
                [self.pdfViewCtrl.currentDoc removeSignature:sig];
            }else{
                FSForm *form = [[FSForm alloc] initWithDocument:self.pdfViewCtrl.currentDoc];
                FSControl *control = [widget getControl];
                if (![control isEmpty] && [field getControlCount] > 1) {
                    [form removeControl:control];
                }else{
                    [form removeField:field];
                }
            }
        }
    }

    [self.pdfViewCtrl unlockRefresh];
    [self.pdfViewCtrl refresh:rect pageIndex:pageIndex];
    
    [self.extensionsManager onAnnotDeleted:page annot:annot];
    
    self.attributesBeforeModify = nil;
    self.annotImage = nil;
    [self.extensionsManager setCurrentAnnot:nil];
    return YES;
}

- (void)canDragFormControlOnPageView:(int)pageIndex recognizer:(UIGestureRecognizer *)recognizer annot:(FSAnnot *_Nullable)annot{
    UIView *pageView = [self.pdfViewCtrl getPageView:pageIndex];
    CGPoint point = [recognizer locationInView:pageView];
    CGRect rect = [self.pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
    FSPointF *pdfPoint = [self.pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
    FSPDFPage *page = [self.pdfViewCtrl.currentDoc getPage:pageIndex];
    if (recognizer.state == UIGestureRecognizerStateBegan) {
        [self beginCanDragFormControlOnPageView:page annotRect:rect touchPoint:point pdfPoint:pdfPoint];
    }else if (recognizer.state == UIGestureRecognizerStateChanged){
        [self changeCanDragFormControlOnPageView:page annotRect:rect touchPoint:point pdfPoint:pdfPoint];
    }else if (recognizer.state == UIGestureRecognizerStateEnded || recognizer.state == UIGestureRecognizerStateCancelled ){
        [self endCanDragFormControlOnPageView:page annotRect:rect touchPoint:point pdfPoint:pdfPoint];
        self.currentMenuPointf = CopyPointF(_lastPtF);
    }
}

- (void)beginCanDragFormControlOnPageView:(FSPDFPage *)page annotRect:(CGRect)rect touchPoint:(CGPoint)point pdfPoint:(FSPointF *)pdfPoint{
    @try {
        _lastPtF = nil;
        if (CGRectContainsPoint(rect, point)) {
            _lastPtF = pdfPoint;
        }
        if (self.foucsWidget.isComboBox || (self.foucsWidget.isTextField && _lastPtF)) {
            [self.formFiller onLButtonDown:page point:pdfPoint flags:0];
        }
    } @catch (NSException *exception) {
    }
}

- (void)changeCanDragFormControlOnPageView:(FSPDFPage *)page annotRect:(CGRect)rect touchPoint:(CGPoint)point pdfPoint:(FSPointF *)pdfPoint{
    @try {
        if (CGRectContainsPoint(rect, point) && _lastPtF) {
            _lastPtF = pdfPoint;
        }
        if (self.foucsWidget.isComboBox || (self.foucsWidget.isTextField && _lastPtF)) {
            [self.formFiller onMouseMove:page point:pdfPoint flags:0];
        }
    } @catch (NSException *exception) {
    }
}

- (void)endCanDragFormControlOnPageView:(FSPDFPage *)page annotRect:(CGRect)rect touchPoint:(CGPoint)point pdfPoint:(FSPointF *)pdfPoint{
    @try {
        if (self.canDragFormControl) {
            if (!CGRectContainsPoint(rect, point) && _lastPtF) {
                pdfPoint = _lastPtF;
            }
        }
        if (self.foucsWidget.isComboBox || (self.foucsWidget.isTextField && _lastPtF)) {
            [self.formFiller onLButtonUp:page point:pdfPoint flags:0];
        }
        if (self.canDragFormControl) {
            if (self.editFormControlNeedTextInput && _lastPtF) {
                [self.formFiller onRButtonDown:page point:pdfPoint flags:0];
                [self.formFiller onRButtonUp:page point:pdfPoint flags:0];
            }
        }
    } @catch (NSException *exception) {
    }
}

- (BOOL)onPageViewLongPress:(int)pageIndex recognizer:(nonnull UILongPressGestureRecognizer *)recognizer annot:(FSAnnot *_Nullable)annot {
    
    if ([[self.pdfViewCtrl currentDoc] isXFA]) {
        return NO;
    }
    
    BOOL docHasSignature = [Utility isDocumentSigned:[self.pdfViewCtrl currentDoc]];
    BOOL allowEdit = docHasSignature ? NO : [FSPermissionUtilty canModifyContents:self.extensionsManager];
    
    if (![FSPermissionUtilty canFillFormInDocument:self.extensionsManager] || [annot isEmpty] || ![FSPermissionUtilty canAssemble:self.extensionsManager] || !allowEdit) {
        return NO;
    }
    if ([annot getType] != FSAnnotWidget) {
        return NO;
    }
    
    if (self.annot && ![self.annot isEmpty] && [self.annot isEqualToAnnot:annot]) {
        if (self.canDragFormControl) {
            [self canDragFormControlOnPageView:pageIndex recognizer:recognizer annot:annot];
        }else{
            if (recognizer.state == UIGestureRecognizerStateBegan) {
                if ([self.extensionsManager.currentAnnot isEqualToAnnot:annot]) {
                    [self.extensionsManager showMenu];
                }
            }
        }
    }else{
        FSWidget *widget = [[FSWidget alloc] initWithAnnot:annot];
        FSField *field = [widget getField];
        if (![field isEmpty]) {
            self.shouldShowMenuOnSelection = YES;
            return [super onPageViewLongPress:pageIndex recognizer:recognizer annot:annot];
        }
    }

    return NO;
}

- (BOOL)onPageViewPan:(int)pageIndex recognizer:(UIPanGestureRecognizer *)recognizer annot:(FSAnnot *)annot{
    if (self.shouldShowMenuOnSelection && !self.canDragFormControl) {
        return [super onPageViewPan:pageIndex recognizer:recognizer annot:annot];
    }
    if (self.canDragFormControl) {
        [self canDragFormControlOnPageView:pageIndex recognizer:recognizer annot:annot];
    }else{
        if (recognizer.state == UIGestureRecognizerStateBegan) {
            if (self.foucsWidget.isImageField) {
                self.extensionsManager.shouldShowMenu = YES;
            }
        }else if (recognizer.state == UIGestureRecognizerStateEnded || recognizer.state == UIGestureRecognizerStateCancelled ){
            if (self.foucsWidget.isImageField && self.extensionsManager.shouldShowMenu) {
                [_imageFieldHandler showMenu:self.foucsWidget];
                self.extensionsManager.shouldShowMenu = NO;
            }
        }
    }
    return YES;
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer annot:(FSAnnot *)annot {
    
    if (self.shouldShowMenuOnSelection) {
        if (self.foucsWidget) {
            [self killFocus];
        }
        if (self.extensionsManager.currentAnnot) {
            [self.extensionsManager setCurrentAnnot:nil];
        }
        self.shouldShowMenuOnSelection = NO;
        CGPoint point = [recognizer locationInView:[self.pdfViewCtrl getPageView:pageIndex]];
        FSPointF *pdfPoint = [self.pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
        if (pageIndex != annot.pageIndex || ![self isHitAnnot:annot point:pdfPoint]) {
            return YES;
        }
    }
    BOOL hasForm = NO;
    @try {
        hasForm = [self.pdfViewCtrl.currentDoc hasForm];
    } @catch (NSException *e) {
    }
    if (!hasForm) {
        return NO;
    }
    BOOL canFillForm = [FSPermissionUtilty canFillForm:self.extensionsManager];
    if (!canFillForm) {
        return NO;
    }
    
    UIView *pageView = [self.pdfViewCtrl getPageView:pageIndex];
    CGPoint point = [recognizer locationInView:pageView];
    FSPointF *pointF = [self.pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
    [self onMouseLButtonClick:[self.pdfViewCtrl.currentDoc getPage:pageIndex] point:pointF];
    if (!self.foucsWidget && self.annot.nonempty) {
        [self.extensionsManager setCurrentAnnot:nil];
        return YES;;
    }
    if (self.editFormControlNeedTextInput) {
        [self beginInput];
    }
    if (self.soundObj) {
        [self beginPlaySound];
    }
    return _hasFormChanged;
}

- (void)onMouseLButtonClick:(FSPDFPage *)page point:(FSPointF *)point{
    _hasFormChanged = NO;
    @try {
        [self.formFiller onLButtonDown:page point:point flags:0];
        [self.formFiller onLButtonUp:page point:point flags:0];
        if (self.extensionsManager.currentAnnot.nonempty && !self.foucsWidget.nonempty) {
            [self.extensionsManager setCurrentAnnot:nil];
        }
    } @catch (NSException *exception) {
    }
    BOOL hidden = !self.foucsWidget.nonempty;
    if (!hidden) {
        FSWidget *widget = [[FSWidget alloc] initWithAnnot:self.foucsWidget];
        FSField *field = [widget getField];
        FSFieldType fieldType = [field isEmpty] ? FSFieldTypeUnknown : [field getType];
        if (FSFieldTypePushButton == fieldType)
            hidden = YES;
    }
    if (self.foucsWidget && [self beginDatePickerPopup]) {
        [self.formNaviBar setHidden:YES];
    }else{
        if (self.datePickerPopVC) {
            [self.datePickerPopVC dismissViewControllerAnimated:YES completion:nil];
        }
        [self.formNaviBar setHidden:hidden];
    }

    [(PageNavigationModule *)[self.extensionsManager getModuleByName:@"PageNavigation"] setIsFormNavBarShowState:!hidden];
    [self.extensionsManager.containerView bringSubviewToFront:self.formNaviBar];
}

- (BOOL)onPageViewShouldBegin:(int)pageIndex recognizer:(UIGestureRecognizer *)gestureRecognizer annot:(FSAnnot *)annot {
    FormAnnotHandler *_self = [FSCallbackSubstitute fillerAssistCallbackSubstitute].currentTarget;
    if (_self != self && _self.datePickerPopVC) {
        [_self.datePickerPopVC dismissViewControllerAnimated:YES completion:^{
            _self.datePickerPopVC = nil;
        }];
    }
    [FSCallbackSubstitute fillerAssistCallbackSubstitute].currentTarget = self;
    BOOL canFillForm = [FSPermissionUtilty canFillForm:self.extensionsManager];
    if (!canFillForm) {
        return NO;
    }

    self.isPanGestures = [gestureRecognizer isKindOfClass:[UIPanGestureRecognizer class]];
    
    UIView *pageView = [self.pdfViewCtrl getPageView:pageIndex];
    CGPoint point = [gestureRecognizer locationInView:pageView];
    FSPointF *pdfPoint = [self.pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
    if (pageIndex == annot.pageIndex && [self isHitAnnot:annot point:pdfPoint]) {
        return YES;
    }
    return NO;
}

- (BOOL)onPageViewTouchesBegan:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event annot:(FSAnnot *)annot {
    self.isPanGestures = NO;
    [self onPageViewTouchesState:UIGestureRecognizerStateBegan pageIndex:pageIndex touches:touches withEvent:event annot:annot];
    return NO;
}

- (BOOL)onPageViewTouchesMoved:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event annot:(FSAnnot *)annot {
    [self onPageViewTouchesState:UIGestureRecognizerStateChanged pageIndex:pageIndex touches:touches withEvent:event annot:annot];
    return NO;
}

- (BOOL)onPageViewTouchesEnded:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event annot:(FSAnnot *)annot {
    [self onPageViewTouchesState:UIGestureRecognizerStateEnded pageIndex:pageIndex touches:touches withEvent:event annot:annot];
    return NO;
}

- (BOOL)onPageViewTouchesCancelled:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event annot:(FSAnnot *)annot {
    [self onPageViewTouchesState:UIGestureRecognizerStateCancelled pageIndex:pageIndex touches:touches withEvent:event annot:annot];
    return NO;
}

- (void)onPageViewTouchesState:(UIGestureRecognizerState)state pageIndex:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event annot:(FSAnnot *)annot{
    if (self.canDragFormControl && self.isPanGestures) {
        UIView *pageView = [self.pdfViewCtrl getPageView:pageIndex];
        CGPoint point = [[touches anyObject] locationInView:[self.pdfViewCtrl getPageView:pageIndex]];
        CGRect rect = [self.pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
        FSPointF *pdfPoint = [self.pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
        FSPDFPage *page = [self.pdfViewCtrl.currentDoc getPage:pageIndex];
        if (state == UIGestureRecognizerStateBegan) {
            [self beginCanDragFormControlOnPageView:page annotRect:rect touchPoint:point pdfPoint:pdfPoint];
        }else if (state == UIGestureRecognizerStateChanged){
            [self changeCanDragFormControlOnPageView:page annotRect:rect touchPoint:point pdfPoint:pdfPoint];
        }else if (state == UIGestureRecognizerStateEnded || state == UIGestureRecognizerStateCancelled ){
            [self endCanDragFormControlOnPageView:page annotRect:rect touchPoint:point pdfPoint:pdfPoint];
        }
    }
}

- (BOOL)shouldDrawAnnot:(FSAnnot *)annot inPDFViewCtrl:(FSPDFViewCtrl *)pdfViewCtrl {
    return YES;
}

- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context annot:(FSAnnot *)annot{
    if (self.shouldShowMenuOnSelection) {
        [super onDraw:pageIndex inContext:context annot:annot];
    }
}

- (void)resetProperties{
    FSWidget *widget = [[FSWidget alloc] initWithAnnot:self.extensionsManager.currentAnnot];
    FSFieldType fieldType = [self getFormAnnotFieldType:widget];
    FSField *field = [widget getField];
    FSControl *control = [widget getControl];
    FSDefaultAppearance *appearance = [control getDefaultAppearance];
    float fontSize = [appearance getText_size];
    FSFont *font = [appearance getFont];
    NSString *fontName = @"";
    if (font && ![font isEmpty]) {
        fontName = [font getName];
    }
    
    FSPropertyMainView *propertyMainView = self.extensionsManager.propertyMainView;
    [propertyMainView setPropertyColor:widget.color];
    [propertyMainView setFontName:fontName];
    propertyMainView.fontSize = fontSize;
    propertyMainView.lock = ![widget canModifyAppearanceWithExternsions:self.extensionsManager];
    propertyMainView.text = field.getName;

    if (fieldType == FSFieldTypeTextField) {
        [propertyMainView resetProperties:@[@(FSPropertyTypePropertyColor), @(FSPropertyTypeFontName), @(FSPropertyTypeFontSizeAuto)]];
    }else if (fieldType == FSFieldTypeCheckBox){
        [propertyMainView resetProperties:@[@(FSPropertyTypePropertyColor)]];
    }else if (fieldType == FSFieldTypeRadioButton){
        [propertyMainView resetProperties:@[@(FSPropertyTypePropertyColor), @(FSPropertyTypeTextForm)]];
    }else if (fieldType == FSFieldTypeComboBox || fieldType == FSFieldTypeListBox){
        propertyMainView.formWidget = widget;
        [propertyMainView resetProperties:@[@(FSPropertyTypePropertyColor), @(FSPropertyTypeFontName),@(FSPropertyTypeFontSizeAuto), @(FSPropertyTypeFormOptions)]];
    }
}

- (FSMenuOptions)menuOptionsForAnnot:(FSAnnot *)annot {
    FSMenuOptions options = FSMenuOptionNone;
    if (![annot.NM hasPrefix:@"FoxitPDFSDK"]) {
        return options;
    }
    if ([FSPermissionUtilty canAddAnnot:self.extensionsManager]) {
        if ([annot canDeleteWithExternsions:self.extensionsManager]) {
            options |= FSMenuOptionDelete;
        }
        if (annot && ![annot isEmpty]) {
            FSFieldType fieldType = [self getFormAnnotFieldType:annot];
            if (fieldType == FSFieldTypeTextField || fieldType == FSFieldTypeCheckBox || fieldType == FSFieldTypeRadioButton || fieldType == FSFieldTypeListBox || fieldType == FSFieldTypeComboBox){
                options |= FSMenuOptionStyle;
            }
        }
    }
    return options;
}

- (void)endOp:(BOOL)needDelay {
    [self.pdfViewCtrl refresh:CGRectZero pageIndex:[self.pdfViewCtrl getCurrentPage]];
}

- (void)beginInput{
    if (!self.hiddenTextField.isEditable) {
        return;
    }
    #if !_MAC_CATALYST_
        [self.formNaviBar setHidden:YES];
        [(PageNavigationModule *)[self.extensionsManager getModuleByName:@"PageNavigation"] setIsFormNavBarShowState:NO];
    #endif
        self.hiddenTextField.inputAccessoryView = [self inputAccessoryView];
        self.hiddenTextField.inputView = nil;
        [self.hiddenTextField reloadInputViews];
        self.hiddenTextField.text = @"";
        [self.hiddenTextField becomeFirstResponder];
        self.extensionsManager.menuControl.shouldBecomeFirstResponder = NO;

    #if _MAC_CATALYST_
        _keyboardHeight = self.textFormNaviBar.fs_height;
        [self gotoFormField];
    #endif
    
}

- (void)endTextInputWithExit {
    if (!self.hiddenTextField.isEditable) {
        return;
    }
    if (self.hiddenTextField) {
        _needBack = NO;
        [self.hiddenTextField resignFirstResponder];
        self.hiddenTextField.text = @"";
        self.extensionsManager.menuControl.shouldBecomeFirstResponder = YES;
        
        #if _MAC_CATALYST_
            [self.pdfViewCtrl setBottomOffset:0];
        #endif
    }
}

- (BOOL)beginDatePickerPopup{
    NSDateFormatter *dateFormat = self.foucsWidget.dateTextFieldFormat;
    if (dateFormat) {
        if (self.datePickerPopVC) {
            CGRect pvAnnotRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:self.foucsWidget.fsrect pageIndex:self.foucsWidget.pageIndex];
            if (!CGRectEqualToRect(pvAnnotRect, self.datePickerPopVC.popoverPresentationController.sourceRect)) {
                self.datePickerPopVC.popoverPresentationController.sourceRect = pvAnnotRect;
                [self.datePickerPopVC.popoverPresentationController.containerView setNeedsLayout];
                [self.datePickerPopVC.popoverPresentationController.containerView layoutIfNeeded];
            }
            return YES;
        }
        UIDatePicker *datePicker = [self datePicker];
        NSString *dataStr = [[self.foucsWidget getField] getValue];
        if (dataStr) {
            if (![dateFormat.dateFormat containsString:@"yyyy"]) {
                NSDateFormatter *makeUpYear = [[NSDateFormatter alloc] init];
                [makeUpYear setDateFormat:@"yyyy"];
                NSInteger currentYear = [[makeUpYear stringFromDate:[NSDate date]] integerValue];
                dataStr = [NSString stringWithFormat:@"%ld,%@",(long)currentYear, dataStr];
            }
            datePicker.date = [dateFormat dateFromString:dataStr];
        }
        @fs_weakify(self)
        [datePicker addBlockForControlEvents:UIControlEventValueChanged block:^(UIDatePicker * _Nonnull sender) {
            @fs_strongify(self)
            [self setDateToTextField:sender];
        }];

        UIView *customView = [UIView new];
        customView.backgroundColor = self.fs_backgroundColor_B2;
        UIView *formNaviBar = [self buildFormNaviBar];
        [customView addSubview:formNaviBar];
        [formNaviBar mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.right.top.fs_equalToSuperview();
        }];
        [customView addSubview:datePicker];
        [datePicker mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(formNaviBar.mas_bottom);
            make.left.right.bottom.fs_equalToSuperview();
            make.height.mas_offset(121);
        }];
        customView.frame = CGRectMake(0, 0, IPAD_PREFERENCE_BAR_WIDTH, 121+49);

        CGRect pvAnnotRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:self.foucsWidget.fsrect pageIndex:self.foucsWidget.pageIndex];
        self.datePickerPopVC = [self.extensionsManager.viewCtrlManager popPopoverCustomView:customView sourceView:(DEVICE_iPHONE || [self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) ? nil : ^UIView * _Nonnull(UIPopoverPresentationController * _Nonnull popover) {
            popover.permittedArrowDirections = UIPopoverArrowDirectionAny;
            return [self.pdfViewCtrl getPageView:self.foucsWidget.pageIndex];
        } sourceRect:pvAnnotRect];
        self.datePickerPopVC.autoresizesFollowContnetView = YES;
        self.datePickerPopVC.dismissCompletion = ^{
            @fs_strongify(self)
            [self killFocus];
        };
        self.datePickerPopVC.willRepositionPopoverToRectInView = ^(CGRect * _Nonnull rect, UIView *__autoreleasing *inView) {
            @fs_strongify(self)
            *rect = [self.pdfViewCtrl convertPdfRectToPageViewRect:self.foucsWidget.fsrect pageIndex:self.foucsWidget.pageIndex];
            *inView = [self.pdfViewCtrl getPageView:self.foucsWidget.pageIndex];
        };
        return YES;
    }
    return NO;
}

- (void)setDateToTextField:(UIDatePicker *)datePicker{
    NSDateFormatter *dateFormat = self.foucsWidget.dateTextFieldFormat;
    if (!dateFormat) {
        return;
    }
    NSString *text = [dateFormat stringFromDate:datePicker.date];
    [self setTextToCurrentTextFiled:text needInput:NO];
}

- (void)beginPlaySound{
    if (self.soundObj) {
        AudioVideoModule *audioVideoModule = [self.extensionsManager getModuleByName:Module_Audio_Video];
        NSString *medioFilePath = [Utility getDocumentSoundStreamTempFilePath:[self.soundObj getStream] PDFPath:self.extensionsManager.pdfViewCtrl.filePath];
        if (medioFilePath) {
            NSURL *fileURL = [NSURL fileURLWithPath:medioFilePath];
            [audioVideoModule audioPlayWithFile:fileURL];
        }
    }
}

- (void)setEnableHighlightForm:(BOOL)enableHighlightForm{
    [self.formFiller highlightFormFields:self.extensionsManager.enableHighlightForm];
    [self setHighlightFormColor:self.extensionsManager.highlightFormColor];
}

- (void)setHighlightFormColor:(UIColor *)highlightFormColor{
    if (self.extensionsManager.enableHighlightForm) {
        if (highlightFormColor) [self.formFiller setHighlightColor:highlightFormColor.fs_argbValue];
    }
}

- (BOOL)imageFieldBrowFileWithFileFilter:(NSString *)fileFilter{
    if (self.foucsWidget.nonempty) {
        if (self.foucsWidget.isImageField) {
            [_imageFieldHandler showMenu:self.foucsWidget];
            return YES;
        }
    }
    return NO;
}

- (void)setTextToCurrentTextFiled:(NSString *)text{
    [self setTextToCurrentTextFiled:text needInput:YES];
}

- (void)setTextToCurrentTextFiled:(NSString *)text needInput:(BOOL)needInput{
    if (self.foucsWidget.nonempty) {
        FSControl *foucsControl = [self.foucsWidget getControl];
        FSField *field = [[self foucsWidget] getField];
        @try {
            [self killFocus];
        } @catch (NSException *exception) {
        }
        [field setValue:@""];
        [self setFocus:foucsControl isHidden:NO];
        for(NSUInteger i=0; i< text.length; i++)
        {
            unichar cha = [text characterAtIndex:i];
            //Control character as space.
            if(cha == 0x2006) cha = FSFillerVkeySpace;
            if(cha == 0x0A) cha = FSFillerVkeyReturn;
            [self.formFiller onChar:cha flags:0];
        }
        if (needInput) {
            [self beginInput];
            //self.hiddenTextField.text = text;
            if (self.historyDataView.superview){
                [self.historyDataView textDidBeginEditing:text];
            }
        }
    }
}

#pragma mark - UITextViewDelegate

- (BOOL)textView:(UITextView *)textView shouldChangeTextInRange:(NSRange)range replacementText:(NSString *)text {
    if ([textView.text isEqualToString:@""] && [text isEqualToString:@""]) {
        [self.formFiller onChar:FSFillerVkeyBack flags:0];
    }
    return YES;
}

- (void)textViewDidChange:(UITextView *)textView {
    self.extensionsManager.isDocModified = YES;

    if (self.lastMarkText) {
        for (int i = 0; i < self.lastMarkText.length; i++) {
            [self.formFiller onChar:FSFillerVkeyBack flags:0];
        }
    }
    _changeText = textView.text;
    
    NSString *other = @"➋➌➍➎➏➐➑➒";
    if([other rangeOfString:_changeText].location != NSNotFound) {
        _changeText = [textView.text substringFromIndex:(textView.text.length - _changeText.length)];
    }

    for(NSUInteger i=0; i< _changeText.length; i++)
    {
        unichar cha = [_changeText characterAtIndex:i];
        //Control character as space.
        if(cha == 0x2006) cha = FSFillerVkeySpace;
        if(cha == 0x0A) cha = FSFillerVkeyReturn;
        [self.formFiller onChar:cha flags:0];
    }
    
    _changeText = @"";
    if (!self.editFormControlNeedTextInput) {
        [self endOp:YES];
    }
    
    if (!textView.markedTextRange || textView.markedTextRange.empty) {
        self.hiddenTextField.text = @"";
        self.lastMarkText = @"";
    }else{
        self.lastMarkText = [self.hiddenTextField textInRange:textView.markedTextRange];
    }
}

- (void)textViewDidBeginEditing:(UITextView *)textView{
    self.lastContentText = @"";
    if (self.historyDataView.superview){
        NSString *content = [[self.foucsWidget getField] getValue];
        [self.historyDataView textDidBeginEditing:content];
    }
}

- (void)textViewDidEndEditing:(UITextView *)textView{
    if (self.historyDataView.superview){
        [self.historyDataView textDidEndEditing:self.lastContentText];
    }
}

#pragma mark -
#pragma GCC diagnostic ignored "-Wdeprecated-declarations"


- (void)gotoFormField {
    
    dispatch_semaphore_wait(_lock, DISPATCH_TIME_FOREVER);
    FSAnnot *dmAnnot = self.extensionsManager.currentAnnot;
    if (dmAnnot && ![dmAnnot isEmpty]) {

        CGRect pvAnnotRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:dmAnnot.fsrect pageIndex:dmAnnot.pageIndex];
        CGRect dvAnnotRect = [self.pdfViewCtrl convertPageViewRectToDisplayViewRect:pvAnnotRect pageIndex:dmAnnot.pageIndex];
        
        CGFloat visibleHeight = CGRectGetHeight(self.pdfViewCtrl.bounds) - [self.extensionsManager topHeight];
        if (DEVICE_iPHONE) {
            CGFloat maxHeight = visibleHeight - _keyboardHeight - 88;
            if (CGRectGetHeight(dvAnnotRect) > maxHeight) {
                dvAnnotRect.size.height = maxHeight;
            }
        }
        
        PDF_LAYOUT_MODE pageLayoutMode = [self.pdfViewCtrl getPageLayoutMode];
        if (_needRotate) {
            CGPoint annotPVOrigin = [self.pdfViewCtrl convertDisplayViewPtToPageViewPt:pvAnnotRect.origin pageIndex:dmAnnot.pageIndex];
            FSPointF *annotPVOriginf = [self.pdfViewCtrl convertPageViewPtToPdfPt:annotPVOrigin pageIndex:dmAnnot.pageIndex];
            [self.pdfViewCtrl gotoPage:dmAnnot.pageIndex withDocPoint:annotPVOriginf animated:YES];
            _needRotate = NO;
            pvAnnotRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:dmAnnot.fsrect pageIndex:dmAnnot.pageIndex];
            dvAnnotRect = [self.pdfViewCtrl convertPageViewRectToDisplayViewRect:pvAnnotRect pageIndex:dmAnnot.pageIndex];
            if ([self.pdfViewCtrl isContinuous]) {
                dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.4 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                    [self gotoFormField];
                });
                dispatch_semaphore_signal(_lock);
                return;
            }
        }
        
        CGFloat dvAnnotMaxYExtra = CGRectGetHeight(self.pdfViewCtrl.bounds) - CGRectGetMaxY(dvAnnotRect);
        if (dvAnnotMaxYExtra < _keyboardHeight) {
            float bottomOffset = ({
                float dvOffsetY = _keyboardHeight - fabs(dvAnnotMaxYExtra) + 20;
                
                if (dvOffsetY + CGRectGetHeight(pvAnnotRect) > CGRectGetHeight(self.pdfViewCtrl.bounds)) {
                    dvOffsetY =  CGRectGetHeight(self.pdfViewCtrl.bounds) - _keyboardHeight - 88;
                }
                dvOffsetY;
            });
            if ([self.pdfViewCtrl isContinuous]) {
                int pageIndex = dmAnnot.pageIndex;
                if (pageIndex == [self.pdfViewCtrl getPageCount] - 1) {
                    [self.pdfViewCtrl setBottomOffset:bottomOffset];
                }else if(pageIndex == [self.pdfViewCtrl getPageCount] - 2 && (pageLayoutMode == PDF_LAYOUT_MODE_TWO || pageLayoutMode == PDF_LAYOUT_MODE_TWO_RIGHT)){
                    if (pageLayoutMode == PDF_LAYOUT_MODE_TWO && [self.pdfViewCtrl getPageCount] % 2 == 0) {
                        [self.pdfViewCtrl setBottomOffset:bottomOffset];
                    }
                    if (pageLayoutMode == PDF_LAYOUT_MODE_TWO_RIGHT && [self.pdfViewCtrl getPageCount] % 2 == 1) {
                        [self.pdfViewCtrl setBottomOffset:bottomOffset];
                    }
                }else{
                    CGRect pvRectToPdfDisplayRect = [[self.pdfViewCtrl getPageView:pageIndex] convertRect:pvAnnotRect toView:[self.pdfViewCtrl getDisplayView]];
                    
                    CGRect visibleRect = self.pdfViewCtrl.bounds;
                    float topHeight = [self.extensionsManager topHeight];
                    visibleRect.origin.y += topHeight;
                    visibleRect.size.height -= topHeight;
                    
                    float y = CGRectGetMinY(pvAnnotRect) - CGRectGetHeight(visibleRect) + CGRectGetHeight(pvAnnotRect) + _keyboardHeight + 20;
                    if (y < 0) {
                        [self.pdfViewCtrl setBottomOffset:bottomOffset];
                    }else{
                        CGPoint pt = CGPointMake(pvAnnotRect.origin.x - pvRectToPdfDisplayRect.origin.x, y);
                        FSPointF *fspt = [self.pdfViewCtrl convertPageViewPtToPdfPt:pt pageIndex:pageIndex];
                        [self.pdfViewCtrl gotoPage:dmAnnot.pageIndex withDocPoint:fspt animated:NO];
                    }
                }
            }else{
                [self.pdfViewCtrl setBottomOffset:bottomOffset];
            }
        }
//        else {
//            CGPoint dvPt = CGPointZero;
//            dvPt.x = dvAnnotRect.origin.x - (self.pdfViewCtrl.frame.size.width - dvAnnotRect.size.width) / 2;
//            dvPt.y = dvAnnotRect.origin.y - (self.pdfViewCtrl.frame.size.height - dvAnnotRect.size.height) / 2;
//            CGPoint pvPt = [self.pdfViewCtrl convertDisplayViewPtToPageViewPt:dvPt pageIndex:dmAnnot.pageIndex];
//            FSPointF *pdfPt = [self.pdfViewCtrl convertPageViewPtToPdfPt:pvPt pageIndex:dmAnnot.pageIndex];
//            if (![self.pdfViewCtrl isContinuous]) {
//                [self.pdfViewCtrl gotoPage:dmAnnot.pageIndex withDocPoint:pdfPt animated:YES];
//            }else {
//                FSRectF *rect = dmAnnot.getRect;
//                int curPageIndex = dmAnnot.pageIndex;
//                CGRect pvRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:rect pageIndex:curPageIndex];
//                
//                float orignY = pvRect.origin.y;
//                BOOL isTwoPagesMode = (pageLayoutMode == PDF_LAYOUT_MODE_TWO || pageLayoutMode == PDF_LAYOUT_MODE_TWO_LEFT || pageLayoutMode == PDF_LAYOUT_MODE_TWO_RIGHT || pageLayoutMode == PDF_LAYOUT_MODE_TWO_MIDDLE);
//                if (!isTwoPagesMode) {
//                    orignY = pvRect.origin.y - CGRectGetHeight(self.pdfViewCtrl.frame)/2 ;
//                }
//                CGPoint newOrign = CGPointMake(pvRect.origin.x - CGRectGetWidth(self.pdfViewCtrl.frame)/2, orignY);
//                FSPointF *pdfPt = [self.pdfViewCtrl convertPageViewPtToPdfPt:newOrign pageIndex:curPageIndex];
//                [self.pdfViewCtrl gotoPage:curPageIndex withDocPoint:pdfPt animated:YES];
//            }
//        }
    }
    dispatch_semaphore_signal(_lock);
}

- (void)keyboardWasShown:(NSNotification *)aNotification {
    if (![self.hiddenTextField isFirstResponder]) return;
    if (_keyboardShown)
        return;
    _keyboardShown = YES;
    NSDictionary *info = [aNotification userInfo];
    // Get the frame of the keyboard.
    NSValue *frame = nil;
    frame = [info objectForKey:UIKeyboardBoundsUserInfoKey];
    CGRect keyboardFrame = [frame CGRectValue];
    _keyboardHeight = keyboardFrame.size.height;
    [self gotoFormField];
}

- (void)keyboardWasHidden:(NSNotification *)aNotification {
    if (![self.hiddenTextField isFirstResponder]) return;
    _keyboardShown = NO;
    [self endOp:YES];
    FSAnnot *dmAnnot = self.extensionsManager.currentAnnot;
    if (dmAnnot && ![dmAnnot isEmpty]) {
        PDF_LAYOUT_MODE pageLayoutMode = [self.pdfViewCtrl getPageLayoutMode];
        if (pageLayoutMode == PDF_LAYOUT_MODE_SINGLE || dmAnnot.pageIndex == [self.pdfViewCtrl.currentDoc getPageCount] - 1 || pageLayoutMode == PDF_LAYOUT_MODE_TWO || pageLayoutMode == PDF_LAYOUT_MODE_TWO_LEFT || pageLayoutMode == PDF_LAYOUT_MODE_TWO_RIGHT || pageLayoutMode == PDF_LAYOUT_MODE_TWO_MIDDLE) {
            [self.pdfViewCtrl setBottomOffset:0];
        }
        else if ([self.pdfViewCtrl isContinuous]) {
            [self.pdfViewCtrl setBottomOffset:0];
        }
    }
}

- (void)setFormTimer:(int)uElapse lpTimerFunc:(FSTimer *)lpTimerFunc {
    [self killFormTimer];
    self.formTimer = [NSTimer scheduledTimerWithTimeInterval:(float) uElapse / (float) 1000
                                                      target:self
                                                    selector:@selector(handleFormTimer:)
                                                    userInfo:nil
                                                     repeats:YES];
    self.formTimerCallback = lpTimerFunc;
}

- (void)killFormTimer {
    self.formTimerCallback = nil;
    [self.formTimer invalidate];
    self.formTimer = nil;
}

- (void)handleFormTimer:(NSTimer *)timer {
    if (_formTimerCallback) {
        [_formTimerCallback onTimer:10];
    }
}

- (void)onCut{
    if (self.formFiller) {
        [self.formFiller onCut];
        [self.extensionsManager hideMenu];
    }
}

- (void)onCopy{
    if (self.formFiller) {
        [self.formFiller onCopy];
        [self.extensionsManager hideMenu];
    }
}

- (void)onPaste{
    if (self.formFiller) {
        [self.formFiller onPaste];
        [self.extensionsManager hideMenu];
    }
}

- (void)onDelete{
    if (self.formFiller) {
        [self.formFiller onDelete];
        [self.extensionsManager hideMenu];
    }
}

- (void)onSelectAll{
    if (self.formFiller) {
        if ([self.formFiller onSelectAll]) {
            [self clickRButtonWithPt:self.currentMenuPointf];
        }
    }
}

- (void)clickRButtonWithPt:(FSPointF *)pt{
    [self.formFiller onRButtonDown:self.annot.getPage point:pt flags:0];
    [self.formFiller onRButtonUp:self.annot.getPage point:pt flags:0];
}


- (void)showRButtonMenu{
    if (_menuItems.count) {
        
        self.hiddenTextField.menuControl = self.extensionsManager.menuControl;

        int pageIndex = self.annot.pageIndex;
        self.extensionsManager.menuControl.menuItems = _menuItems;
        CGRect dvRect = [self.pdfViewCtrl convertPageViewRectToDisplayViewRect:[self.pdfViewCtrl convertPdfRectToPageViewRect:self.annot.fsrect pageIndex:pageIndex] pageIndex:pageIndex];

        [self.extensionsManager.menuControl setRect:dvRect margin:20];
        [self.extensionsManager.menuControl showMenu];
    }
}

#pragma mark <FSFillerAssistCallback>

- (void)refresh:(FSPDFPage *)page rect:(FSRectF *)pdfRect {
    void (^block) (void) = ^{
        if (self.annotCountTag > 0) {
             int annotCount = [page getAnnotCount];
            if (self.annotCountTag != annotCount) {
               [[NSNotificationCenter defaultCenter] postNotificationName:NSNotificationNameAnnotLsitUpdate object:nil];
                self.annotCountTag = annotCount;
            }
        }
        int pageIndex = [page getIndex];
        CGRect rect = [self.pdfViewCtrl convertPdfRectToPageViewRect:pdfRect pageIndex:pageIndex];
        [self.pdfViewCtrl refresh:rect pageIndex:pageIndex needRender:YES clearCache:YES];
    };

    if ([NSThread isMainThread]) {
        block();
    } else {
        dispatch_async(dispatch_get_main_queue(), ^{
            block();
        });
    }
    _hasFormChanged = YES;
}

- (BOOL)setTimer:(int)elapse timer:(FSTimer *)timerFunc out_timer_id:(int *)out_timer_id {
    [self setFormTimer:elapse lpTimerFunc:timerFunc];
    *out_timer_id = 10;
    return YES;
}

- (BOOL)killTimer:(int)timer_id {
    [self killFormTimer];
    return YES;
}

- (void)focusGotOnControl:(FSControl *)control value:(NSString *)fieldValue {
    FSField *field = [control getField];
    FSFieldType type = [field getType];
    self.editFormControlNeedSetCursor = NO;
    self.editFormControlNeedTextInput = NO;
    self.editFormControlNeedDatePicker = NO;
    self.canDragFormControl = NO;
    _annotCountTag = [[[control getWidget] getPage] getAnnotCount];
    if (type != FSFieldTypeTextField)
        self.editFormControlNeedSetCursor = YES;
    if (type == FSFieldTypeTextField || (type == FSFieldTypeComboBox && ([field getFlags] & FSFieldFlagComboEdit))){
        if (![[control getWidget] dateTextFieldFormat]) {
            self.editFormControlNeedTextInput = YES;
        }else{
            self.editFormControlNeedDatePicker = YES;
        }
    }
    if (self.editFormControlNeedTextInput || type == FSFieldTypeComboBox)
        _canDragFormControl = YES;
    self.foucsWidget = [control getWidget];
    _focusedWidgetIndex = [_foucsWidget getIndex];
    if (_foucsWidget.nonempty && [_foucsWidget getField].flags & FSFieldFlagTextPassword) {
        self.hiddenTextField.keyboardType = UIKeyboardTypeASCIICapable;
    }else{
        self.hiddenTextField.keyboardType = UIKeyboardTypeDefault;
    }
    self.soundObj = [self.foucsWidget getButtonPressedSoundObjt];
}

- (void)focusLostFromControl:(FSControl *)control value:(NSString *)fieldValue {
    _annotCountTag = -1;
    _focusedWidgetIndex = -1;
    [self endTextInputWithExit];
    self.editFormControlNeedSetCursor = NO;
    self.editFormControlNeedTextInput = NO;
    self.editFormControlNeedDatePicker = NO;
    self.canDragFormControl = NO;
    self.shouldShowMenuOnSelection = NO;
    self.soundObj = nil;
    if (self.extensionsManager.currentAnnot && [self.extensionsManager.currentAnnot isEqualToAnnot:_foucsWidget]) {
        _foucsWidget = nil;
    }
    self.hiddenTextField.keyboardType = UIKeyboardTypeDefault;
    [self.formNaviBar setHidden:YES];
    [(PageNavigationModule *)[self.extensionsManager getModuleByName:@"PageNavigation"] setIsFormNavBarShowState:NO];
}

- (void *)createPopupMenu{
    _menuItems = @[].mutableCopy;
    return (__bridge void *)([UIMenuController sharedMenuController]);
}

- (BOOL)appendPopupMenuItem:(void *)h_popup_menu popupMenuItem:(FSFillerAssistCallbackPopupMenuItem)popupMenuItem default_string:(NSString *)default_string{
    if (popupMenuItem == FSFillerAssistCallbackPopupMenuItemCut) {
        MenuItem *item = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kCut") object:self action:@selector(onCut)];
        [_menuItems addObject:item];
    }else if (popupMenuItem == FSFillerAssistCallbackPopupMenuItemCopy) {
        MenuItem *item = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kCopy") object:self action:@selector(onCopy)];
        [_menuItems addObject:item];
    }else if (popupMenuItem == FSFillerAssistCallbackPopupMenuItemPaste) {
        MenuItem *item = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kPaste") object:self action:@selector(onPaste)];
        [_menuItems addObject:item];
    }else if (popupMenuItem == FSFillerAssistCallbackPopupMenuItemDelete) {
        MenuItem *item = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kDelete") object:self action:@selector(onDelete)];
        [_menuItems addObject:item];
    }else if (popupMenuItem == FSFillerAssistCallbackPopupMenuItemSelectAll) {
        MenuItem *item = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kSelectAll") object:self action:@selector(onSelectAll)];
        [_menuItems addObject:item];
    }
    return YES;
}

- (BOOL)enablePopupMenuItem:(void *)h_popup_menu popupMenuItem:(FSFillerAssistCallbackPopupMenuItem)popupMenuItem is_enabled:(BOOL)is_enabled{
    if (!is_enabled) {
        for (MenuItem *item in _menuItems.copy) {
            if (popupMenuItem == FSFillerAssistCallbackPopupMenuItemCut && [item.title isEqualToString:FSLocalizedForKey(@"kCut")]) {
                [_menuItems removeObject:item];
                break;
            }else if (popupMenuItem == FSFillerAssistCallbackPopupMenuItemCopy && [item.title isEqualToString:FSLocalizedForKey(@"kCopy")]) {
                [_menuItems removeObject:item];
                break;
            }else if (popupMenuItem == FSFillerAssistCallbackPopupMenuItemPaste && [item.title isEqualToString:FSLocalizedForKey(@"kPaste")]) {
                [_menuItems removeObject:item];
                break;
            }else if (popupMenuItem == FSFillerAssistCallbackPopupMenuItemDelete && [item.title isEqualToString:FSLocalizedForKey(@"kDelete")]) {
                [_menuItems removeObject:item];
                break;
            }else if (popupMenuItem == FSFillerAssistCallbackPopupMenuItemSelectAll && [item.title isEqualToString:FSLocalizedForKey(@"kSelectAll")]) {
                [_menuItems removeObject:item];
                break;
            }
        }
    }
    return _menuItems.count;
}


- (BOOL)showPopupMenu:(void *)h_popup_menu point:(FSPointF *)point{
    if (_menuItems.count) {
        _currentMenuPointf = _currentMenuPointf ? _currentMenuPointf : CopyPointF(point);
        [self showRButtonMenu];
        return YES;
    }
    return NO;
}

- (void)setClipboardText:(NSString *)text{
    [UIPasteboard generalPasteboard].string = text;
}

- (NSString *)getClipboardText{
    return [UIPasteboard generalPasteboard].string;
}

- (void)editingMonitorOnCaretChanged:(int)n_caret_pos{
}

- (void)editingMonitorOnContentChanged:(NSString *)content {
    if (self.historyDataView.superview) {
        [self.historyDataView textDidChange:content];
    }
    self.lastContentText = content;
}

#pragma mark IDocEventListener

- (void)onDocOpened:(FSPDFDoc *)document error:(int)error {
    if (error != FSErrSuccess) {
        return;
    }
    [self creatFormFiller];
}

- (void)onDocWillClose:(FSPDFDoc *)document {
    if (self.annot) {
        [self setFocus:nil isHidden:YES];
        [self endTextInputWithExit];
    }
    self->_focusedWidgetIndex = -1;
    if (document){
        _form = nil;
        _formFiller = nil;
    }
    _formTimerCallback = nil;
}

#pragma mark IRotationEventListener

- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration{
    if (_keyboardShown) {
        [self endTextInputWithExit];
        _needRotate = YES;
    }
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation {
    self.hiddenTextField.inputAccessoryView = [self inputAccessoryView];
    if (_needRotate) {
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.4 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
            [self beginInput];
        });
    }
}

#pragma mark -- IWindowFrameChangedListener
- (void)onWindowFrameChanged:(CGRect)newFrame horizontalSizeClassChanged:(BOOL)changed{
    if (DEVICE_iPAD && changed) {
        self.hiddenTextField.inputAccessoryView = [self inputAccessoryView];
        [self.formNaviBar mas_remakeConstraints:^(MASConstraintMaker *make) {
            if (DEVICE_iPHONE || [self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
                make.left.right.bottom.mas_equalTo(self.pdfViewCtrl);
            }else{
                make.centerX.mas_equalTo(self.pdfViewCtrl);
                make.bottom.mas_equalTo(self.pdfViewCtrl.fs_mas_bottom).offset(-TOOL_BAR_HEIGHT_NORMAL);
                make.width.offset([self.formNaviBar getWidthWithPosition:FSItemPositionLeft] + ITEM_MARGIN_NORMAL + [self.formNaviBar getWidthWithPosition:FSItemPositionRight]);
            }
        }];
        
        if (DEVICE_iPAD && ![self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            [self.formNaviBar fs_setLayerCornerRadius:LAYER_BORDER_WIDTH_6];
        }else{
            [self.formNaviBar fs_setLayerCornerRadius:0.0];
        }
        
        if (self.foucsWidget && self.datePickerPopVC) {
            [self.datePickerPopVC dismissViewControllerAnimated:YES completion:^{
                self.datePickerPopVC = nil;
                [self beginDatePickerPopup];
            }];
        }
    }
}
@end
