/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FreetextModule.h"
#import <FoxitRDK/FSPDFViewControl.h>

#import "FtAnnotHandler.h"
#import "FtToolHandler.h"
#import "Utility.h"
#import "FSMainToolbar+Private.h"
#import "FSReadToolbarItem.h"

@interface FreetextModule ()

@property (nonatomic, weak) FSMainToolbarItem *propertyItem;
@property (nonatomic, weak) FSReadToolbarItem *readToolbarItem;
@property (nonatomic, strong) FSReadToolbarItem * typeWirterReadToolbarItem;
@end

@implementation FreetextModule {
    UIExtensionsManager *__weak _extensionsManager;
}

- (NSString *)getName {
    return @"Freetext";
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        [self loadModule];
        FtAnnotHandler* annotHandler = [[FtAnnotHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerAnnotHandler:annotHandler];

        FtToolHandler* toolHandler = [[FtToolHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerToolHandler:toolHandler];
        [_extensionsManager registerRotateChangedListener:toolHandler];
        [_extensionsManager registerUndoEventListener:toolHandler];
        [_extensionsManager.pdfViewCtrl registerScrollViewEventListener:toolHandler];
        [_extensionsManager registerWindowFrameChangedListener:toolHandler];
    }
    return self;
}

- (void)loadModule {
    [_extensionsManager.toolbarManager addReadItem:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeTypewriter] atToolTag:FSMainTopbarToolTagHome];
    FSReadToolSettings* toolSetings = [_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeTypewriter];
    _typeWirterReadToolbarItem = [[FSReadToolbarItem alloc] initWithToolSettings:toolSetings];
    [_extensionsManager.toolbarManager addReadItem:_typeWirterReadToolbarItem atToolTag:FSMainTopbarToolTagComment];
    [_extensionsManager registerAnnotPropertyListener:self];
}

- (void)didClickedReadToolItem:(FSReadToolbarItem *)readToolbarItem {
    [_extensionsManager changeState:FSUIManagerStateAnnotTool];
    id<IToolHandler> toolHandler = [_extensionsManager getToolHandlerByName:Tool_Freetext];
    toolHandler.toolSettings =  readToolbarItem.toolSettings;
    [_extensionsManager setCurrentToolHandler:toolHandler];
    
    self.readToolbarItem = readToolbarItem;
    FSMainToolbarItem *propertyItem =  [_extensionsManager.toolbarManager getMainItemWithType:FSMainToolbarItemTypeProperty];
    propertyItem.item.onTapClick = ^(FSItem * _Nonnull _item) {
        [[UIApplication sharedApplication] sendAction:@selector(resignFirstResponder) to:nil from:nil forEvent:nil];
        
        CGRect rect = [_item convertRect:_item.bounds toView:self->_extensionsManager.pdfViewCtrl];
        if (DEVICE_iPHONE) {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:rect inView:self->_extensionsManager.pdfViewCtrl];
        } else {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:_item.bounds inView:_item];
        }
    };
    self.propertyItem = propertyItem;
    [_extensionsManager onUIInteractElementClicked:FSUIElementReadingCommentBarTypeWriter];
}

#pragma mark - IAnnotPropertyListener

- (void)onToolSettingsChanged:(FSReadToolSettings *)toolSettings {
    if (self.readToolbarItem && self.readToolbarItem.toolSettings == toolSettings) {
        self.readToolbarItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }else if (_typeWirterReadToolbarItem.toolSettings == toolSettings){
        self.typeWirterReadToolbarItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }
}

@end
