/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FreetextUtil.h"

@implementation FreetextUtil

//+ (UIButton*)createDoneButton:(UITextView*)textView
//{
//    UIDeviceOrientation o = [[UIDevice currentDevice] orientation];
//    if (DEVICE_iPHONE) {
//        if (((STYLE_CELLWIDTH_IPHONE * STYLE_CELLHEIHGT_IPHONE) >= (375 * 667)) && (o == UIDeviceOrientationLandscapeLeft || o == UIDeviceOrientationLandscapeRight)) {
//            UIView *doneView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 0, 1)];
//            doneView.backgroundColor = [UIColor clearColor];
//            textView.inputAccessoryView = doneView;
//        }
//    } else if (((STYLE_CELLWIDTH_IPHONE * STYLE_CELLHEIHGT_IPHONE) >= (375 * 667)) && (o == UIDeviceOrientationPortrait || o == UIDeviceOrientationPortraitUpsideDown)) {
//        UIView *doneView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, 0, 40)];
//        doneView.backgroundColor = [UIColor clearColor];
//        UIButton *doneBT = [UIButton buttonWithType:UIButtonTypeCustom];
//        [doneBT setBackgroundImage:FSImageNamedAndNormalColor(@"common_keyboard_done") forState:UIControlStateNormal];
//        [doneView addSubview:doneBT];
//        [doneBT mas_makeConstraints:^(MASConstraintMaker *make) {
//            make.right.equalTo(doneView.mas_right).offset(0);
//            make.top.equalTo(doneView.mas_top).offset(0);
//            make.size.mas_equalTo(CGSizeMake(40, 40));
//        }];
//        textView.inputAccessoryView = doneView;
//    }
//}

+ (void)setTextViewProperties:(UITextView*)textView UIExtensionsManager:(UIExtensionsManager *)extensionsManager toolSettings:(FSReadToolSettings *)toolSettings isCallout:(BOOL)isCallout
{
    textView.textContainerInset = UIEdgeInsetsMake(2.5, -4, 0, -4);
    textView.layer.borderColor = isCallout ? [UIColor clearColor].CGColor : extensionsManager.config.defaultSettings.annotations.textbox.color.CGColor;
    textView.layer.borderWidth = 1;
    textView.backgroundColor = [UIColor clearColor];
    textView.textColor = ({
        UInt32 color = toolSettings ? toolSettings.settingsColor : [extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeTypewriter].settingsColor;
        float opacity = toolSettings ? toolSettings.opacity : [extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeTypewriter].opacity;
        BOOL isMappingColorMode = (extensionsManager.pdfViewCtrl.colorMode == FSRendererColorModeMapping);
        if (isMappingColorMode && color == 0) {
            color = 16775930;
        }
        if (!isMappingColorMode && color == 16775930) {
            color = 0;
        }
        [UIColor colorWithRGB:color alpha:opacity];
    });
    textView.showsVerticalScrollIndicator = NO;
    textView.showsHorizontalScrollIndicator = NO;
    textView.scrollEnabled = NO;
    textView.clipsToBounds = NO;
}

#pragma mark - keyboard

+ (void)afterKeyboardShown:(NSNotification*)notification textView:(UITextView*)textView UIExtensionsManager:(UIExtensionsManager *)extensionsManager pageIndex:(int)pageIndex
{
    NSDictionary* info = [notification userInfo];
    FSPDFViewCtrl* pdfViewCtrl = extensionsManager.pdfViewCtrl;

    CGRect keyboardEndFrame = [[info objectForKey:UIKeyboardFrameEndUserInfoKey] CGRectValue];
    CGRect keyboardFrame = [textView convertRect:keyboardEndFrame toView:nil];
    CGRect textFrame = textView.frame;
    
    CGPoint oldPvPoint = [pdfViewCtrl convertDisplayViewPtToPageViewPt:CGPointMake(0, 0) pageIndex:pageIndex];
    FSPointF *oldPdfPoint = [pdfViewCtrl convertPageViewPtToPdfPt:oldPvPoint pageIndex:pageIndex];
    
    CGRect dvAnnotRect = [pdfViewCtrl convertPageViewRectToDisplayViewRect:textFrame pageIndex:pageIndex];
    float positionY;
    positionY = SCREEN_HEIGHT - dvAnnotRect.origin.y - dvAnnotRect.size.height;
    
    if (positionY < keyboardFrame.size.height) {
        float dvOffsetY = keyboardFrame.size.height - positionY + 40;
        CGRect offsetRect = CGRectMake(0, 0, 100, dvOffsetY);
        
        CGRect pvRect = [pdfViewCtrl convertDisplayViewRectToPageViewRect:offsetRect pageIndex:pageIndex];
        FSRectF *pdfRect = [pdfViewCtrl convertPageViewRectToPdfRect:pvRect pageIndex:pageIndex];
        float pdfOffsetY = pdfRect.top - pdfRect.bottom;
        
        PDF_LAYOUT_MODE layoutMode = [pdfViewCtrl getPageLayoutMode];
        if (layoutMode == PDF_LAYOUT_MODE_SINGLE ||
            layoutMode == PDF_LAYOUT_MODE_TWO ||
            layoutMode == PDF_LAYOUT_MODE_TWO_LEFT ||
            layoutMode == PDF_LAYOUT_MODE_TWO_RIGHT ||
            layoutMode == PDF_LAYOUT_MODE_TWO_MIDDLE) {
            float tmpPvOffset = pvRect.size.height;
            CGRect tmpPvRect = CGRectMake(0, 0, 10, tmpPvOffset);
            CGRect tmpDvRect = [pdfViewCtrl convertPageViewRectToDisplayViewRect:tmpPvRect pageIndex:pageIndex];
            [pdfViewCtrl setBottomOffset:tmpDvRect.size.height];
        } else if ([pdfViewCtrl isContinuous]) {
            if ([pdfViewCtrl getCurrentPage] == [pdfViewCtrl getPageCount] - 1) {
                float tmpPvOffset = pvRect.size.height;
                CGRect tmpPvRect = CGRectMake(0, 0, 10, tmpPvOffset);
                CGRect tmpDvRect = [pdfViewCtrl convertPageViewRectToDisplayViewRect:tmpPvRect pageIndex:pageIndex];
                [pdfViewCtrl setBottomOffset:tmpDvRect.size.height];
            } else {
                FSPointF *jumpPdfPoint = [[FSPointF alloc] init];
                [jumpPdfPoint set:oldPdfPoint.x y:oldPdfPoint.y - pdfOffsetY];
                [pdfViewCtrl gotoPage:pageIndex withDocPoint:jumpPdfPoint animated:YES];
            }
        }
    }
}

+ (void)adjustBottomOffset:(FSPDFViewCtrl *)pdfViewCtrl pageIndex:(int)pageIndex
{
    PDF_LAYOUT_MODE layoutMode = [pdfViewCtrl getPageLayoutMode];
    if (layoutMode == PDF_LAYOUT_MODE_SINGLE ||
        layoutMode == PDF_LAYOUT_MODE_TWO ||
        layoutMode == PDF_LAYOUT_MODE_TWO_LEFT ||
        layoutMode == PDF_LAYOUT_MODE_TWO_RIGHT ||
        layoutMode == PDF_LAYOUT_MODE_TWO_MIDDLE ||
        pageIndex == [pdfViewCtrl getPageCount] - 1) {
        [pdfViewCtrl setBottomOffset:0];
    }
}

+ (void)adjustTextViewFrame:(UITextView *)textView inPageView:(UIView *)pageView forMinPageInset:(CGFloat)inset {
    CGAffineTransform textViewTransform = textView.transform;
    textView.transform = CGAffineTransformIdentity;
    CGRect frame = textView.frame;
    CGSize constraintSize = CGSizeMake(frame.size.width, MAXFLOAT);
    CGSize size = [textView sizeThatFits:constraintSize];
    if (textView.text.length > 0) {
        frame = CGRectMake(frame.origin.x, frame.origin.y, frame.size.width, size.height);
    }
    if (frame.size.height >= (CGRectGetHeight(pageView.frame) - 20)) {
        frame.size.height = pageView.fs_height - 20;
    }
    if (textView.frame.size.width >= (CGRectGetWidth(pageView.frame) - 20)) {
        frame.size.width = pageView.fs_width - 20;
    }
    
    CGRect bounds = CGRectInset(pageView.bounds, inset, inset);
    if (!CGRectIntersectsRect(frame, bounds)) {
        textView.frame = frame;
        return;
    }
    if (CGRectGetMinX(frame) < CGRectGetMinX(bounds)) {
        frame = CGRectOffset(frame, CGRectGetMinX(bounds) - CGRectGetMinX(frame), 0);
    }
    if (CGRectGetMaxX(frame) > CGRectGetMaxX(bounds)) {
        frame = CGRectOffset(frame, -(CGRectGetMaxX(frame) - CGRectGetMaxX(bounds)), 0);
    }
    if (CGRectGetMinY(frame) < CGRectGetMinY(bounds)) {
        frame = CGRectOffset(frame, 0, CGRectGetMinY(bounds) - CGRectGetMinY(frame));
    }
    if (CGRectGetMaxY(frame) > CGRectGetMaxY(bounds)) {
        frame = CGRectOffset(frame, 0, -(CGRectGetMaxY(frame) - CGRectGetMaxY(bounds)));
    }
    textView.frame = frame;
    textView.transform = textViewTransform;
}

//for callout

+ (CGRect)calculateBorderRectWithInnerRect:(CGRect)innerRect startPoint:(CGPoint)startPoint kneePoint:(CGPoint)kneePoint
{
    CGRect lineRect;
    lineRect.origin.x = MIN(startPoint.x, kneePoint.x);
    lineRect.origin.y = MIN(startPoint.y, kneePoint.y);
    lineRect.size.width = ABS(startPoint.x - kneePoint.x);
    lineRect.size.height = ABS(startPoint.y - kneePoint.y);
    return CGRectUnion(lineRect, innerRect);
}

+ (CGRect)resetCalloutAppearanceWhenTextviewChanged:(FSPDFViewCtrl*)pdfViewCtrl annot:(FSFreeText*)annot textFrame:(CGRect)textFrame
{
    if (!annot || [annot isEmpty]) return CGRectZero;
    CGRect newInnerRect = textFrame;
    CGRect newBorderRect;
    
    CGRect oldBorderRect = [pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:annot.pageIndex];
    CGRect oldInnerRect = [pdfViewCtrl convertPdfRectToPageViewRect:[annot getInnerRect] pageIndex:annot.pageIndex];
    FSPointFArray *calloutlinePoints = [annot getCalloutLinePoints];
    CGPoint startPoint = [pdfViewCtrl convertPdfPtToPageViewPt:[calloutlinePoints getAt:0] pageIndex:annot.pageIndex];
    CGPoint kneePoint = [pdfViewCtrl convertPdfPtToPageViewPt:[calloutlinePoints getAt:1] pageIndex:annot.pageIndex];
    CGPoint endPoint = [pdfViewCtrl convertPdfPtToPageViewPt:[calloutlinePoints getAt:2] pageIndex:annot.pageIndex];
    

    //calculate new points
    if (endPoint.x <= oldInnerRect.origin.x + 2.0) {
        //left
        endPoint.x = newInnerRect.origin.x;
        endPoint.y = newInnerRect.origin.y + newInnerRect.size.height / 2.0;
        kneePoint.x = endPoint.x - 20.0;
        kneePoint.y = endPoint.y;
    } else if (endPoint.x >= (oldInnerRect.origin.x + oldInnerRect.size.width) - 2.0) {
        //right
        endPoint.x = newInnerRect.origin.x + newInnerRect.size.width;
        endPoint.y = newInnerRect.origin.y + newInnerRect.size.height / 2.0;
        kneePoint.x = endPoint.x + 20.0;
        kneePoint.y = endPoint.y;
    } else if (endPoint.y <= oldInnerRect.origin.y + 2.0) {
        //top
        endPoint.y = newInnerRect.origin.y;
        endPoint.x = newInnerRect.origin.x + newInnerRect.size.width / 2.0;
        kneePoint.y = endPoint.y - 20.0;
        kneePoint.x = endPoint.x;
    } else {
        //bottom
        endPoint.y = newInnerRect.origin.y + newInnerRect.size.height;
        endPoint.x = newInnerRect.origin.x + newInnerRect.size.width / 2.0;
        kneePoint.y = endPoint.y + 20.0;
        kneePoint.x = endPoint.x;
    }
    
    
    newBorderRect = [FreetextUtil calculateBorderRectWithInnerRect:newInnerRect startPoint:startPoint kneePoint:kneePoint];
    annot.fsrect = [pdfViewCtrl convertPageViewRectToPdfRect:newBorderRect pageIndex:annot.pageIndex];
    [annot setInnerRect:[pdfViewCtrl convertPageViewRectToPdfRect:newInnerRect pageIndex:annot.pageIndex]];
    FSPointFArray *points = [[FSPointFArray alloc] init];
    [points add:[pdfViewCtrl convertPageViewPtToPdfPt:startPoint pageIndex:annot.pageIndex]];
    [points add:[pdfViewCtrl convertPageViewPtToPdfPt:kneePoint pageIndex:annot.pageIndex]];
    [points add:[pdfViewCtrl convertPageViewPtToPdfPt:endPoint pageIndex:annot.pageIndex]];
    [annot setCalloutLinePoints:points];
    
    CGRect refreshRect = CGRectInset(CGRectUnion(oldBorderRect, newBorderRect), -30, -30);
    return refreshRect;
}

#define DEFAULT_KENNTOEND_WIDTH 30
+(void)resetKneeAndEndingPt:(FSPDFViewCtrl *)pdfViewCtrl pageIndex:(int)pageIndex textRect:(CGRect)textRect pStartingPt:(CGPoint *)pStartingPt  pKneePt:(CGPoint *)pKneePt pEndingPt:(CGPoint *)pEndingPt{
    [self resetKneeAndEndingPt:pdfViewCtrl pageIndex:pageIndex textRect:textRect kennToEndW:DEFAULT_KENNTOEND_WIDTH pStartingPt:pStartingPt pKneePt:pKneePt pEndingPt:pEndingPt];
}
+(void)resetKneeAndEndingPt:(FSPDFViewCtrl *)pdfViewCtrl pageIndex:(int)pageIndex textRect:(CGRect)textRect kennToEndW:(CGFloat)kennToEndW pStartingPt:(CGPoint *)pStartingPt  pKneePt:(CGPoint *)pKneePt pEndingPt:(CGPoint *)pEndingPt {
    CGPoint startingPt = *pStartingPt;
    CGPoint kneePt = *pKneePt;
    CGPoint endingPt = *pEndingPt;
    if (kneePt.x < endingPt.x) {
        if (startingPt.x > CGRectGetMinX(textRect) && startingPt.x < CGRectGetMaxX(textRect) && startingPt.y < CGRectGetMinY(textRect)) {
            endingPt.x = (CGRectGetMinX(textRect) + CGRectGetMaxX(textRect)) / 2;
            endingPt.y = CGRectGetMinY(textRect);
            kneePt.x = endingPt.x;
            kneePt.y = endingPt.y - [Utility convertWidth:kennToEndW fromPDFToPageView:pdfViewCtrl pageIndex:pageIndex];
        } else if (startingPt.x > CGRectGetMinX(textRect) && startingPt.x < CGRectGetMaxX(textRect) && startingPt.y > CGRectGetMaxY(textRect)) {
            endingPt.x = (CGRectGetMinX(textRect) + CGRectGetMaxX(textRect)) / 2;
            endingPt.y = CGRectGetMaxY(textRect);
            kneePt.x = endingPt.x;
            kneePt.y = endingPt.y + [Utility convertWidth:kennToEndW fromPDFToPageView:pdfViewCtrl pageIndex:pageIndex];
        } else if (startingPt.x > CGRectGetMaxX(textRect)) {
            endingPt.x = CGRectGetMaxX(textRect);
            endingPt.y = (CGRectGetMinY(textRect) + CGRectGetMaxY(textRect)) / 2;
            kneePt.x = endingPt.x + [Utility convertWidth:kennToEndW fromPDFToPageView:pdfViewCtrl pageIndex:pageIndex];
            kneePt.y = endingPt.y;
        } else {
            endingPt.x = CGRectGetMinX(textRect);
            endingPt.y = (CGRectGetMinY(textRect) + CGRectGetMaxY(textRect)) / 2;
            kneePt.x = endingPt.x - [Utility convertWidth:kennToEndW fromPDFToPageView:pdfViewCtrl pageIndex:pageIndex];
            kneePt.y = endingPt.y;
        }
    } else if (kneePt.x > endingPt.x) {
        if (startingPt.x < CGRectGetMaxX(textRect) && startingPt.x > CGRectGetMinX(textRect) && startingPt.y < CGRectGetMinY(textRect)) {
            endingPt.x = (CGRectGetMinX(textRect) + CGRectGetMaxX(textRect)) / 2;
            endingPt.y = CGRectGetMinY(textRect);
            kneePt.x = endingPt.x;
            kneePt.y = endingPt.y - [Utility convertWidth:kennToEndW fromPDFToPageView:pdfViewCtrl pageIndex:pageIndex];
        } else if (startingPt.x < CGRectGetMaxX(textRect) && startingPt.x > CGRectGetMinX(textRect) && startingPt.y > CGRectGetMaxY(textRect)) {
            endingPt.x = (CGRectGetMinX(textRect) + CGRectGetMaxX(textRect)) / 2;
            endingPt.y = CGRectGetMaxY(textRect);
            kneePt.x = endingPt.x;
            kneePt.y = endingPt.y + [Utility convertWidth:kennToEndW fromPDFToPageView:pdfViewCtrl pageIndex:pageIndex];
        } else if (startingPt.x < CGRectGetMinX(textRect)) {
            endingPt.x = CGRectGetMinX(textRect);
            endingPt.y = (CGRectGetMinY(textRect) + CGRectGetMaxY(textRect)) / 2;
            kneePt.x = endingPt.x - [Utility convertWidth:kennToEndW fromPDFToPageView:pdfViewCtrl pageIndex:pageIndex];
            kneePt.y = endingPt.y;
        } else {
            endingPt.x = CGRectGetMaxX(textRect);
            endingPt.y = (CGRectGetMinY(textRect) + CGRectGetMaxY(textRect)) / 2;
            kneePt.x = endingPt.x + [Utility convertWidth:kennToEndW fromPDFToPageView:pdfViewCtrl pageIndex:pageIndex];
            kneePt.y = endingPt.y;
        }
    } else if (kneePt.y < endingPt.y) {
        if (startingPt.x < CGRectGetMinX(textRect)) {
            endingPt.x = CGRectGetMinX(textRect);
            endingPt.y = (CGRectGetMinY(textRect) + CGRectGetMaxY(textRect)) / 2;
            kneePt.x = endingPt.x - [Utility convertWidth:kennToEndW fromPDFToPageView:pdfViewCtrl pageIndex:pageIndex];
            kneePt.y = endingPt.y;
        } else if (startingPt.x > CGRectGetMaxX(textRect)) {
            endingPt.x = CGRectGetMaxX(textRect);
            endingPt.y = (CGRectGetMinY(textRect) + CGRectGetMaxY(textRect)) / 2;
            kneePt.x = endingPt.x + [Utility convertWidth:kennToEndW fromPDFToPageView:pdfViewCtrl pageIndex:pageIndex];
            kneePt.y = endingPt.y;
        } else if (startingPt.y > CGRectGetMaxY(textRect) && startingPt.x > CGRectGetMinX(textRect) && startingPt.x < CGRectGetMaxX(textRect)) {
            endingPt.x = (CGRectGetMinX(textRect) + CGRectGetMaxX(textRect)) / 2;
            endingPt.y = CGRectGetMaxY(textRect);
            kneePt.x = endingPt.x;
            kneePt.y = endingPt.y + [Utility convertWidth:kennToEndW fromPDFToPageView:pdfViewCtrl pageIndex:pageIndex];
        } else {
            endingPt.x = (CGRectGetMinX(textRect) + CGRectGetMaxX(textRect)) / 2;
            endingPt.y = CGRectGetMinY(textRect);
            kneePt.x = endingPt.x;
            kneePt.y = endingPt.y - [Utility convertWidth:kennToEndW fromPDFToPageView:pdfViewCtrl pageIndex:pageIndex];
        }
    } else if (kneePt.y > endingPt.y) {
        if (startingPt.x < CGRectGetMinX(textRect)) {
            endingPt.x = CGRectGetMinX(textRect);
            endingPt.y = (CGRectGetMinY(textRect) + CGRectGetMaxY(textRect)) / 2;
            kneePt.x = endingPt.x - [Utility convertWidth:kennToEndW fromPDFToPageView:pdfViewCtrl pageIndex:pageIndex];
            kneePt.y = endingPt.y;
        } else if (startingPt.x > CGRectGetMaxX(textRect)) {
            endingPt.x = CGRectGetMaxX(textRect);
            endingPt.y = (CGRectGetMinY(textRect) + CGRectGetMaxY(textRect)) / 2;
            kneePt.x = endingPt.x + [Utility convertWidth:kennToEndW fromPDFToPageView:pdfViewCtrl pageIndex:pageIndex];
            kneePt.y = endingPt.y;
        } else if (startingPt.y < CGRectGetMinY(textRect) && startingPt.x > CGRectGetMinX(textRect) && startingPt.x < CGRectGetMaxX(textRect)) {
            endingPt.x = (CGRectGetMinX(textRect) + CGRectGetMaxX(textRect)) / 2;
            endingPt.y = CGRectGetMinY(textRect);
            kneePt.x = endingPt.x;
            kneePt.y = endingPt.y - [Utility convertWidth:kennToEndW fromPDFToPageView:pdfViewCtrl pageIndex:pageIndex];
        } else {
            endingPt.x = (CGRectGetMinX(textRect) + CGRectGetMaxX(textRect)) / 2;
            endingPt.y = CGRectGetMaxY(textRect);
            kneePt.x = endingPt.x;
            kneePt.y = endingPt.y + [Utility convertWidth:kennToEndW fromPDFToPageView:pdfViewCtrl pageIndex:pageIndex];
        }
    }
    pStartingPt->x = startingPt.x;
    pStartingPt->y = startingPt.y;
    pKneePt->x = kneePt.x;
    pKneePt->y = kneePt.y;
    pEndingPt->x = endingPt.x;
    pEndingPt->y = endingPt.y;
}


@end
