/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSFullScreenBottombar.h"
#import "PanelController+private.h"
#import "PageNavigationModule.h"

@interface FSFullScreenBottombar ()
@property (nonatomic, strong) FSItem *pageLabelItem;
@property (nonatomic, weak) UIExtensionsManager *extensionsMgr;
@end

@implementation FSFullScreenBottombar

- (instancetype)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager
{
    self = [super init];
    if (self) {
        _extensionsMgr = extensionsManager;
        [self arrangeView];
    }
    
    return self;
}

- (void)setHidden:(BOOL)hidden{
    [super setHidden:hidden];
    
    if (hidden) {
        if (self.toolItem.selected) {
            self.toolItem.selected = NO;
            [self.extensionsMgr.fullScreenToolbarMgr showFullScreenToolbar:NO];
        }
    }else{
        [self updateSubViewConstraints];
        BOOL bToolBtnEnable = ![self.extensionsMgr.pdfViewCtrl isDynamicXFA] && self.extensionsMgr.enableModifyDocument;
        if (!bToolBtnEnable) {
            self.toolItem.enabled = bToolBtnEnable;
            return;
        }
        
        if (self.extensionsMgr.currentState == FSUIManagerStateReflow || self.extensionsMgr.currentState == FSUIManagerStateFillSign || self.extensionsMgr.currentState == FSUIManagerStateEditor) {
            bToolBtnEnable = NO;
        }else{
            bToolBtnEnable = YES;
        }
        

        FSMainTopbarToolTagItem *homeTagItem = nil;
        NSArray <FSMainTopbarToolTagItem*> *tagItems = self.extensionsMgr.topToolbar.tagItems;
        for (FSMainTopbarToolTagItem* tagItem in tagItems) {
            if (tagItem.toolTag == FSMainTopbarToolTagHome) {
                homeTagItem = tagItem;
                break;
            }
        }
        
        if (homeTagItem && homeTagItem.readToolbarItems.count == 1) {
            FSReadToolbarItem *readItem = homeTagItem.readToolbarItems[0];
            FSMainToolbarItem *mainItem = readItem.customItem;
            FSMainToolbarItemType type = mainItem.itemType;
            if (type == FSMainToolbarItemTypeAddTool) {
                bToolBtnEnable = NO;
            }
        }else if (!homeTagItem){
            bToolBtnEnable = NO;
        }
        
        self.toolItem.enabled = bToolBtnEnable;
    }
}

- (void)updateSubViewConstraints{
    if (self.bookMartItem) {
        self.bookMartItem.hidden = ([self.extensionsMgr.permissionProvider checkPermission:FSFunctionPDFBookMark] == FSPermissionStateHide);
        
        if (!self.bookMartItem.hidden) {
            self.bookMartItem.hidden = [self.extensionsMgr getToolbarItemHiddenStatus][@"ReadingBookmark"];
            [self.bookMartItem mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.right.equalTo(self.mas_safeAreaLayoutGuide).offset(-14);
                make.size.mas_equalTo(CGSizeMake(20, 20));
                make.bottom.equalTo(self);
            }];
        }
    }
    
    self.toolItem.hidden = ([self.extensionsMgr.permissionProvider checkPermission:FSFunctionSmallBottomBarToolItem] == FSPermissionStateHide);
    if (!self.toolItem.hidden) {
        [self.toolItem mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.size.mas_equalTo(CGSizeMake(20, 20));
            make.bottom.equalTo(self);
            if (self.bookMartItem && !self.bookMartItem.hidden) {
                make.right.equalTo(self.bookMartItem.mas_left).offset(-14);
            }else{
                make.right.equalTo(self.mas_safeAreaLayoutGuide).offset(-14);
            }
        }];
    }
    
    if (self.pageLabelItem) {
        [self.pageLabelItem mas_remakeConstraints:^(MASConstraintMaker *make) {
            if (self.toolItem && !self.toolItem.hidden) {
                make.right.equalTo(self.toolItem.mas_left).offset(-16);
            }else{
                make.right.equalTo(self.mas_safeAreaLayoutGuide).offset(-14);
            }
            make.bottom.equalTo(self);
        }];
    }
}

- (void)arrangeView{
    
    if (self.extensionsMgr.config.loadReadingBookmark) {
        self.bookMartItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"full_screen_book_mark")];
        self.bookMartItem.accessibilityLabel = FSLocalizedForKey(@"kAddBookMarkItemVoiceOver");
        self.bookMartItem.imageView.contentMode = UIViewContentModeCenter;

        [BindThemeKeySel(self.bookMartItem, setImage:forState:) fs_updateThemeCompleted:^(id  _Nonnull target, SEL  _Nonnull sel) {
            [target setImage:FSImageNamedAndPrimaryColor(@"full_screen_book_mark_select") forState:UIControlStateSelected];
        }];
        [self.bookMartItem addTarget:self action:@selector(bookmartBtnPress:) forControlEvents:UIControlEventTouchUpInside];
        [self addSubview:self.bookMartItem];
        [self.bookMartItem mas_makeConstraints:^(MASConstraintMaker *make) {
            make.right.equalTo(self.mas_safeAreaLayoutGuide).offset(-14);
            make.size.mas_equalTo(CGSizeMake(20, 20));
            make.bottom.equalTo(self);
        }];
    }
    
    self.toolItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"full_screen_tool")];
    self.toolItem.accessibilityLabel = FSLocalizedForKey(@"kHomeToolBarVoiceOver");

    [BindThemeKeySel(self.toolItem, setImage:forState:) fs_updateThemeCompleted:^(id  _Nonnull target, SEL  _Nonnull sel) {
        [target setImage:FSImageNamedAndPrimaryColor(@"full_screen_tool_select") forState:UIControlStateSelected];
    }];
    self.toolItem.imageView.contentMode = UIViewContentModeCenter;
    [self.toolItem addTarget:self action:@selector(toolBtnPress:) forControlEvents:UIControlEventTouchUpInside];
    [self addSubview:self.toolItem];
    [self.toolItem mas_makeConstraints:^(MASConstraintMaker *make) {
        make.size.mas_equalTo(CGSizeMake(20, 20));
        make.bottom.equalTo(self);
        if (self.bookMartItem) {
            make.right.equalTo(self.bookMartItem.mas_left).offset(-14);
        }else{
            make.right.equalTo(self.mas_safeAreaLayoutGuide).offset(-14);
        }
    }];

    if (self.extensionsMgr.config.loadPageNavigation){
        self.pageLabelItem = [FSItem new];
        [self.pageLabelItem addTarget:self action:@selector(pageBtnPress) forControlEvents:UIControlEventTouchUpInside];
        [self addSubview:self.pageLabelItem];
        [self.pageLabelItem mas_makeConstraints:^(MASConstraintMaker *make) {
            make.right.equalTo(self.toolItem.mas_left).offset(-16);
            make.bottom.equalTo(self);
        }];
        [self.pageLabelItem setTitleEdgeInsets:UIEdgeInsetsMake(5, 20, 5, 0)];
        [self.pageLabelItem setTitleColor:self.fs_textColor_T3 forState:UIControlStateNormal];
        self.pageLabelItem.font = self.fs_standardWeakFont_WF1;
    }
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(bookmarkUpdateNotificationButtonState) name:FSBookmarkUpdateNotification object:nil];
    [self bookmarkUpdateNotificationButtonState];
}

- (void)pageBtnPress{
    [(PageNavigationModule*)[self.extensionsMgr getModuleByName:@"PageNavigation"] showGotoPageAlertView];
}

- (void)toolBtnPress:(UIButton*)button{
    [self.extensionsMgr onUIInteractElementClicked:FSUIElementReadingFullScreenToolbar];
    button.selected = !button.selected;
    [self.extensionsMgr.fullScreenToolbarMgr showFullScreenToolbar:button.selected];
}

- (void)bookmartBtnPress:(UIButton*)button{
    [self.extensionsMgr onUIInteractElementClicked:FSUIElementReadingFullScreenBookmark];
    button.selected = !button.selected;
    int currentPage = self.extensionsMgr.pdfViewCtrl.getCurrentPage;
    if (button.selected) {
        if (currentPage >= 0 && currentPage < self.extensionsMgr.pdfViewCtrl.getPageCount) {
            @try {
                [self.extensionsMgr.pdfViewCtrl.currentDoc insertReadingBookmark:-1
                                                                           title:[NSString stringWithFormat:FSLocalizedForKey(@"kPage"), currentPage + 1]
                                                                 dest_page_index:currentPage];
            } @catch (NSException *exception) {
                button.selected = NO;
                return;
            }
            
            [self.extensionsMgr supportsMultipleScenesOp:^(UIExtensionsManager * _Nonnull ext) {
                [ext.panelController reloadReadingBookmarkPanel];
            }];

            [[NSNotificationCenter defaultCenter] postNotificationName:FSBookmarkUpdateNotification object:nil];
        }
    }else{
        
        FSReadingBookmark *bookmark = [Utility getReadingBookMarkAtPage:self.extensionsMgr.pdfViewCtrl.getDoc page:currentPage];
        if (bookmark) {
            [self.extensionsMgr.pdfViewCtrl.currentDoc removeReadingBookmark:bookmark];
            [self.extensionsMgr supportsMultipleScenesOp:^(UIExtensionsManager * _Nonnull ext) {
                [ext.panelController reloadReadingBookmarkPanel];
            }];
            [[NSNotificationCenter defaultCenter] postNotificationName:FSBookmarkUpdateNotification object:nil];
        }
    }
}

-(void)layoutSubviews{
    [super layoutSubviews];
    [self mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self.superview);
        make.bottom.equalTo(self.superview).offset(-self.superview.safeAreaInsets.bottom);
        make.height.mas_equalTo(20);
    }];
}

- (void)onPageChange:(int)pageIndex
{
    [self.pageLabelItem setTitle:[Utility getDisplayPageLabel:pageIndex needTotal:YES useLogicalPageNumbers:self.extensionsMgr.useLogicalPageNumbers pdfViewCtrl:self.extensionsMgr.pdfViewCtrl] forState:UIControlStateNormal];
    self.pageLabelItem.accessibilityLabel = self.pageLabelItem.currentTitle;
    FSReadingBookmark *bookmark = [Utility getReadingBookMarkAtPage:self.extensionsMgr.pdfViewCtrl.getDoc page:pageIndex];
    if (bookmark) {
        self.bookMartItem.selected = YES;
    }else{
        self.bookMartItem.selected = NO;
    }
}

- (void)bookmarkUpdateNotificationButtonState {
    FSReadingBookmark *bookmark = [Utility getReadingBookMarkAtPage:self.extensionsMgr.pdfViewCtrl.getDoc page:self.extensionsMgr.pdfViewCtrl.getCurrentPage];
    if (bookmark) {
        self.bookMartItem.selected = YES;
    }else{
        self.bookMartItem.selected = NO;
    }
}

@end
