/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSFullScreenToolbarMgr.h"
#import "UIExtensionsManager+Private.h"
#import "FSMainToolbar+Private.h"
#import "FSPermissionUtilty.h"

#define FullScreenSpace  0

@interface FSFullScreenToolbarMgr () <IRotationEventListener,IPageNumberListener,IPageEventListener,IDocEventListener,IToolEventListener,IWindowFrameChangedListener,IViewAppleEventListener>

@property(nonatomic, assign) CGPoint oldPoint;
@property(nonatomic, assign) BOOL userChanged;

@end

@implementation FSFullScreenToolbarMgr

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager{
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
    }
    return self;
}

- (UIView *)smallTopToolbar{
    if (!_smallTopToolbar) {
        _smallTopToolbar = [[FSFullScreenTopbar alloc] init];
        _smallTopToolbar.extensionsMgr = self.extensionsManager;
        _smallTopToolbar.hidden = YES;
    }
    return _smallTopToolbar;
}

- (UIView *)smallBottomToolbar{
    if (!_smallBottomToolbar) {
        _smallBottomToolbar = [[FSFullScreenBottombar alloc] initWithExtensionsManager:self.extensionsManager];
        _smallBottomToolbar.backgroundColor = [UIColor clearColor];
        _smallBottomToolbar.hidden = YES;
    }
    return _smallBottomToolbar;
}

- (void)buildFullScreenToolbar{
    
    [_extensionsManager.pdfViewCtrl registerPageEventListener:self];
    [_extensionsManager registerPageNumberListener:self];
    [_extensionsManager registerToolEventListener:self];
    
    [self installTopbar];
    [self installBottombar];
    [self installFullScreenToolbar];
}

- (void)installTopbar{
    if (self.extensionsManager.topToolbar) {
        [self.extensionsManager.containerView insertSubview:self.smallTopToolbar belowSubview:[self.extensionsManager.topToolbar getToolbarContentView]];
    }else{
        [self.extensionsManager.containerView addSubview:self.smallTopToolbar];
    }
    [self.smallTopToolbar mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.extensionsManager.containerView.superview);
        make.left.right.equalTo(self.extensionsManager.containerView.superview.mas_safeAreaLayoutGuide);
        CGFloat safeAreaTop = self.extensionsManager.pdfViewCtrl.fs_currentSceneKeyWindow.safeAreaInsets.top;
        if (safeAreaTop >20 ) {
            safeAreaTop += 25;
        }else if (safeAreaTop < 20){
            safeAreaTop = 30;
        }
        make.height.mas_equalTo(safeAreaTop);
    }];
}

- (void)installBottombar{
    if (self.extensionsManager.bottomToolbar) {
        [self.extensionsManager.containerView insertSubview:self.smallBottomToolbar belowSubview:[self.extensionsManager.bottomToolbar getToolbarContentView]];
    }else{
        [self.extensionsManager.containerView addSubview:self.smallBottomToolbar];
    }
    [self.smallBottomToolbar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self.smallBottomToolbar.superview);
        make.bottom.mas_equalTo(self.smallBottomToolbar.superview.fs_mas_bottom);
        make.height.mas_equalTo(20);
    }];
}

- (void)removeTopbar{
    [self.smallTopToolbar removeFromSuperview];
}

- (void)removeBottombar{
    [self.smallBottomToolbar removeFromSuperview];
}

- (void)setHiddenBottomToolbar:(BOOL)hidden{
    if (!self.extensionsManager.pdfViewCtrl.currentDoc || !self.smallBottomToolbar.superview || (self.smallBottomToolbar.hidden == hidden && hidden)) {
        return;
    }
    
    if (self.extensionsManager.currentState == FSUIManagerStateNormal ||
        self.extensionsManager.currentState == FSUIManagerStateFillSign ||
        self.extensionsManager.currentState == FSUIManagerStateEditor ||
        self.extensionsManager.currentState == FSUIManagerStateCreateForm ||
        (DEVICE_iPAD && ![self.extensionsManager.pdfViewCtrl fs_currentSceneiPadHorizontalCompact] && (self.extensionsManager.currentState == FSUIManagerStateReflow ||
                         self.extensionsManager.currentState == FSUIManagerStateSpeech ||
                         self.extensionsManager.currentState == FSUIManagerStatePanZoom ||
                         self.extensionsManager.currentState == FSUIManagerStateAutoFlip))) {
        self.smallBottomToolbar.hidden = hidden;
    }else if (self.extensionsManager.currentState == FSUIManagerStateRedaction ||
              (DEVICE_iPAD && [self.extensionsManager.pdfViewCtrl fs_currentSceneiPadHorizontalCompact] && (self.extensionsManager.currentState == FSUIManagerStateReflow ||
                               self.extensionsManager.currentState == FSUIManagerStateSpeech ||
                               self.extensionsManager.currentState == FSUIManagerStatePanZoom ||
                               self.extensionsManager.currentState == FSUIManagerStateAutoFlip))){
        self.smallBottomToolbar.hidden = YES;
    }
}

- (void)setHiddenTopToolbar:(BOOL)hidden{
    if (!self.extensionsManager.pdfViewCtrl.currentDoc || (self.smallTopToolbar.hidden == hidden && hidden)) {
        return;
    }
    
    BOOL isLandscape = UIInterfaceOrientationIsLandscape([UIApplication sharedApplication].statusBarOrientation);
    
    if (self.extensionsManager.currentState == FSUIManagerStateNormal ||
        self.extensionsManager.currentState == FSUIManagerStateFillSign ||
        self.extensionsManager.currentState == FSUIManagerStateEditor ||
        self.extensionsManager.currentState == FSUIManagerStateCreateForm ||
        (DEVICE_iPAD && ![self.extensionsManager.pdfViewCtrl fs_currentSceneiPadHorizontalCompact] && (self.extensionsManager.currentState == FSUIManagerStateReflow ||
                         self.extensionsManager.currentState == FSUIManagerStateSpeech ||
                         self.extensionsManager.currentState == FSUIManagerStatePanZoom ||
                         self.extensionsManager.currentState == FSUIManagerStateAutoFlip))) {

        if (DEVICE_iPHONE && isLandscape) {
            [self.extensionsManager setStatusBarHidden:YES];
            [self.extensionsManager.pdfViewCtrl.fs_viewController setNeedsStatusBarAppearanceUpdate];
        }else{
            [self.extensionsManager setStatusBarHidden:!hidden];
            [self.extensionsManager.pdfViewCtrl.fs_viewController setNeedsStatusBarAppearanceUpdate];
        }
        self.smallTopToolbar.hidden = hidden;
        self.smallBottomToolbar.hidden = hidden;
        [self resetPdfViewCtrlSafeArea];
    }else if (self.extensionsManager.currentState == FSUIManagerStateRedaction ||
              (DEVICE_iPAD && [self.extensionsManager.pdfViewCtrl fs_currentSceneiPadHorizontalCompact] && (self.extensionsManager.currentState == FSUIManagerStateReflow ||
                               self.extensionsManager.currentState == FSUIManagerStateSpeech ||
                               self.extensionsManager.currentState == FSUIManagerStatePanZoom ||
                               self.extensionsManager.currentState == FSUIManagerStateAutoFlip))){
        self.smallTopToolbar.hidden = YES;
        self.smallBottomToolbar.hidden = YES;
    }
    else{
        if (!self.smallTopToolbar.hidden) {
            [self.extensionsManager setStatusBarHidden:NO];
            [self.extensionsManager.pdfViewCtrl.fs_viewController setNeedsStatusBarAppearanceUpdate];
            self.smallTopToolbar.hidden = YES;
            [self setHiddenBottomToolbar:YES];
            [self resetPdfViewCtrlSafeArea];
        }
    }
}

- (void)dealloc
{
    [_extensionsManager.pdfViewCtrl unregisterDocEventListener:self];
    [_extensionsManager.pdfViewCtrl unregisterPageEventListener:self];
    [_extensionsManager.pdfViewCtrl unregisterViewAppleEventListener:self];
    [_extensionsManager unregisterRotateChangedListener:self];
    [_extensionsManager unregisterWindowFrameChangedListener:self];
    [_extensionsManager unregisterPageNumerListener:self];
}

- (void)installFullScreenToolbar{
    self.floatyToolbar = [[FSScrollToolbar alloc] initWithStyle:FSToolbarStyleDefault];
    self.floatyToolbar.verticalScrollBarEdgeInsets = UIEdgeInsetsMake(3, 0, 0, 0);
    self.floatyToolbar.axis = UILayoutConstraintAxisVertical;
    self.floatyToolbar.hidden = YES;
    self.floatyToolbar.layer.cornerRadius = 4;
    self.floatyToolbar.clipsToBounds = YES;
    
    [self.extensionsManager.containerView addSubview:self.floatyToolbar];
    [self.floatyToolbar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.size.mas_equalTo(CGSizeMake(self.extensionsManager.topToolbar.verticalContentWidth, [self getToolbarHeight]));
        make.centerY.equalTo(self.floatyToolbar.superview.mas_safeAreaLayoutGuide);
        make.right.equalTo(self.floatyToolbar.superview.mas_safeAreaLayoutGuide).offset(-16);
    }];
}

- (void)registerEventListens{
    [_extensionsManager.pdfViewCtrl registerDocEventListener:self];
    [_extensionsManager.pdfViewCtrl registerPageEventListener:self];
    [_extensionsManager.pdfViewCtrl registerViewAppleEventListener:self];
    [_extensionsManager registerRotateChangedListener:self];
    [_extensionsManager registerWindowFrameChangedListener:self];
    [_extensionsManager registerPageNumberListener:self];
}

- (void)resetToolbarPosition:(BOOL)changePos{
    if (self.floatyToolbar.hidden) {
        return;
    }

    CGFloat containerViewWidth = self.extensionsManager.containerView.fs_width;
    
    CGSize size = CGSizeMake(self.extensionsManager.topToolbar.verticalContentWidth, [self getToolbarHeight]);
    CGFloat newX = self.floatyToolbar.fs_left;
    if (newX < self.floatyToolbar.superview.safeAreaInsets.left) {
        newX = self.floatyToolbar.superview.safeAreaInsets.left;
    }
    if (newX + size.width > containerViewWidth - self.floatyToolbar.superview.safeAreaInsets.right) {
        newX = containerViewWidth - self.floatyToolbar.superview.safeAreaInsets.right - size.width;
    }
    CGFloat newY = self.floatyToolbar.fs_top;
    if (newY < self.extensionsManager.smallTopToolbar.fs_bottom) {
        newY = self.extensionsManager.smallTopToolbar.fs_bottom;
    }

    if (newY + size.height > self.extensionsManager.smallBottomToolbar.fs_top) {
        newY = self.extensionsManager.smallBottomToolbar.fs_top - size.height;
    }
    
    if (changePos) {
        [self.floatyToolbar mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.size.mas_equalTo(size);
            make.left.mas_equalTo(newX);
            make.top.mas_equalTo(newY);
        }];
    }else{

        [self.floatyToolbar mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.size.mas_equalTo(size);
            make.centerY.equalTo(self.floatyToolbar.superview.mas_safeAreaLayoutGuide);
            make.right.equalTo(self.floatyToolbar.superview.mas_safeAreaLayoutGuide).offset(-16);
        }];
    }
}

- (CGFloat)getToolbarHeight{
    CGFloat totalheight = 488;
    
    if (DEVICE_iPHONE) {
        if (UIInterfaceOrientationIsLandscape([UIApplication sharedApplication].statusBarOrientation)) {
            totalheight = 300;
        }else{
            totalheight = 407;
        }
    }
    
    int totalCount = (int)(self.floatyToolbar.centerItems.items.count + self.floatyToolbar.rightItems.items.count);
    CGFloat realHeight = 0;
    if (totalCount > 0) {
        realHeight = (totalCount-1) * 40.0 + 20;
    }
    else{
        return totalheight;
    }
    return MIN(realHeight, totalheight)+self.floatyToolbar.verticalScrollBarEdgeInsets.top;
}

- (void)addPanItemToTagItem{
    
    FSMainToolbarItem *panItem = [[FSMainToolbarItem alloc] initWithExtensionsManager:self.extensionsManager itemType:FSMainToolbarItemTypePan];
    UIPanGestureRecognizer *pan = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(panSubMenuTopbarWithRecognizer:)];
    [panItem.item addGestureRecognizer:pan];

    if (panItem) {
        panItem.item.transform = CGAffineTransformMakeRotation(M_PI_2);
        [self.floatyToolbar addItem:panItem.customView spacing:FullScreenSpace atPosition:FSScrollToolbarItemPositionRight];
    }
}

- (void)panSubMenuTopbarWithRecognizer:(UIPanGestureRecognizer *)recognizer{
    CGPoint point = [recognizer locationInView:self.extensionsManager.containerView];

    if (recognizer.state == UIGestureRecognizerStateBegan) {
        [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingFullScreenToolbarDrag];
    }else{
        
        CGSize size = self.floatyToolbar.fs_size;
        
        CGFloat moveX = point.x - self.oldPoint.x;
        CGFloat moveY = point.y - self.oldPoint.y;
        CGFloat newX = self.floatyToolbar.fs_left + moveX;
        if (newX < self.floatyToolbar.superview.safeAreaInsets.left) {
            newX = self.floatyToolbar.superview.safeAreaInsets.left;
        }
        if (newX + self.floatyToolbar.fs_width > self.extensionsManager.containerView.fs_width - self.floatyToolbar.superview.safeAreaInsets.right) {
            newX = self.extensionsManager.containerView.fs_width - self.floatyToolbar.superview.safeAreaInsets.right - self.floatyToolbar.fs_width;
        }
        CGFloat newY = self.floatyToolbar.fs_top + moveY;
        if (newY < self.extensionsManager.smallTopToolbar.fs_bottom) {
            newY = self.extensionsManager.smallTopToolbar.fs_bottom;
        }
        if (newY + self.floatyToolbar.fs_height > self.extensionsManager.smallBottomToolbar.fs_top) {
            newY = self.extensionsManager.smallBottomToolbar.fs_top - self.floatyToolbar.fs_height;
        }
        [self.floatyToolbar mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.size.mas_equalTo(size);
            make.left.mas_equalTo(newX);
            make.top.mas_equalTo(newY);
        }];
        
    }
    
    self.oldPoint = point;
    self.userChanged = YES;
}

- (void)showFullScreenToolbar:(BOOL)bShow{
    if (!self.floatyToolbar) {
        return;
    }
    
    if (bShow) {
        
        [self.floatyToolbar removeItemAtPosition:FSScrollToolbarItemPositionRight];
        [self.floatyToolbar removeItemAtPosition:FSScrollToolbarItemPositionScroll];
        
        FSMainTopbarToolTagItem *homeTagItem  = nil;
        NSArray <FSMainTopbarToolTagItem*> *tagItems = self.extensionsManager.topToolbar.tagItems;
        for (FSMainTopbarToolTagItem* tagItem in tagItems) {
            if (tagItem.toolTag == FSMainTopbarToolTagHome) {
                homeTagItem = tagItem;
                break;
            }
        }
        
        [self addPanItemToTagItem];
        
        for (FSMainToolbarItem *mainItem in homeTagItem.mainToolbarItems) {
            
            if (mainItem.itemType == FSMainToolbarItemTypeAddTool) {
                continue;
            }else if (mainItem.itemType == FSMainToolbarItemTypeDivide) {
                FSMainToolbarItem *divideItem = [[FSMainToolbarItem alloc] initWithExtensionsManager:self.extensionsManager itemType:FSMainToolbarItemTypeDivide];
                divideItem.item.transform = CGAffineTransformMakeRotation(M_PI_2);
                [self.floatyToolbar addItem:divideItem.customView spacing:FullScreenSpace atPosition:FSScrollToolbarItemPositionRight];
  
            }else if (mainItem.itemType == FSMainToolbarItemTypePan) {
                continue;
            }else{
                
                [self.floatyToolbar addItem:mainItem.customView spacing:FullScreenSpace atPosition:FSScrollToolbarItemPositionRight];
            }
        }
        for (FSReadToolbarItem *readItem in homeTagItem.readToolbarItems) {
            if (readItem.toolbarItem.itemType == FSMainToolbarItemTypeAddTool) {
                continue;
            }
            if (readItem.readToolbarItemType == FSReadToolbarItemTypeExist) {
                if (!readItem.extensionsManager) {
                    readItem.extensionsManager = self.extensionsManager;
                }
                if (readItem.toolbarItem) {
                    [self.floatyToolbar addItem:readItem.toolbarItem.customView spacing:FullScreenSpace atPosition:FSScrollToolbarItemPositionScroll];
                }
            }else{
                [self.floatyToolbar addItem:readItem.toolbarItem.customView spacing:FullScreenSpace atPosition:FSScrollToolbarItemPositionScroll];
            }
        }
        self.floatyToolbar.hidden = NO;
        
        [self resetToolbarPosition:self.userChanged];
        
    }else{
        self.floatyToolbar.hidden = YES;
        [self.extensionsManager.topToolbar resetHomeTagItem];
        [self.extensionsManager setCurrentToolHandler:nil];
        if (self.extensionsManager.currentState == FSUIManagerStateAnnotTool) {
            [self.extensionsManager changeState:FSUIManagerStateNormal];
        }
    }
}

- (void)resetPdfViewCtrlSafeArea{
    UIEdgeInsets insets = self.extensionsManager.pdfViewCtrl.fs_viewController.additionalSafeAreaInsets;
    if (UIEdgeInsetsEqualToEdgeInsets(insets, UIEdgeInsetsZero)) {
        insets = self.extensionsManager.pdfViewCtrl.fs_currentSceneKeyWindow.safeAreaInsets;
    }
    [self.extensionsManager.pdfViewCtrl.fs_viewController setNeedsStatusBarAppearanceUpdate];
    self.extensionsManager.pdfViewCtrl.fs_viewController.additionalSafeAreaInsets = insets;
    if (UIEdgeInsetsEqualToEdgeInsets(insets,self.extensionsManager.pdfViewCtrl.fs_currentSceneKeyWindow.safeAreaInsets)) {
        insets = UIEdgeInsetsZero;
    }
    self.extensionsManager.pdfViewCtrl.fs_viewController.additionalSafeAreaInsets = insets;
    if (self.smallBottomToolbar.superview) {
        [self.smallBottomToolbar mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.right.equalTo(self.smallBottomToolbar.superview);
            make.bottom.mas_equalTo(self.smallBottomToolbar.superview.fs_mas_bottom);
            make.height.mas_equalTo(20);
        }];
    }
}

#pragma mark --IRotationEventListener
- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation{
    BOOL isLandscape = UIInterfaceOrientationIsLandscape([UIApplication sharedApplication].statusBarOrientation);
    
    if (!self.extensionsManager.pdfViewCtrl.currentDoc) {
        return;
    }
    
    if (DEVICE_iPHONE && isLandscape) {
        [self.extensionsManager setStatusBarHidden:YES];
        [self.extensionsManager.pdfViewCtrl.fs_viewController setNeedsStatusBarAppearanceUpdate];
    }else{
        if (self.extensionsManager.currentIsFullScreen  && _extensionsManager.topToolbar.hidden) {
            [self.extensionsManager setStatusBarHidden:YES];
            [self.extensionsManager.pdfViewCtrl.fs_viewController setNeedsStatusBarAppearanceUpdate];
        }else{
            [self.extensionsManager setStatusBarHidden:NO];
            [self.extensionsManager.pdfViewCtrl.fs_viewController setNeedsStatusBarAppearanceUpdate];
            [self resetPdfViewCtrlSafeArea];
            if ([[UIDevice currentDevice].systemVersion floatValue] < 12.0) {
                [self.extensionsManager.pdfViewCtrl.fs_viewController.view setNeedsLayout];
            }
        }
    }
    [self resetToolbarPosition:self.userChanged];

}

#pragma mark --IWindowFrameChangedListener
- (void)onWindowFrameChanged:(CGRect)newFrame horizontalSizeClassChanged:(BOOL)changed{
    if (DEVICE_iPAD && changed) {
        if (!self.floatyToolbar.hidden) {
            [self showFullScreenToolbar:YES];
        }
    }
}

- (void)onPageChanged:(int)oldIndex currentIndex:(int)currentIndex{
    if (self.smallBottomToolbar) {
        [self.smallBottomToolbar onPageChange:currentIndex];
    }
}

- (void)updatePageNumber{
    if (self.smallBottomToolbar) {
        [self.smallBottomToolbar onPageChange:self.extensionsManager.pdfViewCtrl.getCurrentPage];
    }
}

- (void)onPagesRemoved:(NSArray<NSNumber *> *)indexes{
    if (self.smallBottomToolbar) {
        [self.smallBottomToolbar onPageChange:self.extensionsManager.pdfViewCtrl.getCurrentPage];
    }
}

#pragma mark IDocEventListener
- (void)onDocOpened:(FSPDFDoc *)document error:(int)error {
    if (error != FSErrSuccess) {
        return;
    }

    if ([FSPermissionUtilty canAssemble:self->_extensionsManager]) {
        self.smallBottomToolbar.bookMartItem.enabled = YES;
        self.smallBottomToolbar.bookMartItem.alpha = 1;
    } else {
        self.smallBottomToolbar.bookMartItem.enabled = NO;
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
        if (@available(iOS 13.0, *)) {
            if (self.extensionsManager.pdfViewCtrl.traitCollection.userInterfaceStyle == UIUserInterfaceStyleDark){
                self.smallBottomToolbar.bookMartItem.alpha = 0.3;
            }else{
                self.smallBottomToolbar.bookMartItem.alpha = 0.5;
            }
        }else
#endif
        {
            self.smallBottomToolbar.bookMartItem.alpha = 0.5;
        }
    }
}

- (void)onDocClosed:(FSPDFDoc *)document error:(int)error {
    self.smallBottomToolbar.bookMartItem.enabled = YES;
    self.smallBottomToolbar.bookMartItem.alpha = 1;
}

#pragma mark --IToolEventListener
- (void)onToolChanged:(NSString *)lastToolName CurrentToolName:(NSString *)toolName{
//    if (!toolName.length &&  lastToolName.length > 0) {
//        if (!self.floatyToolbar.hidden) {
//            if (self.extensionsManager.currentState != self.currentState) {
//                [self.extensionsManager changeState:self.currentState];
//            }
//        }
//    }
}
#pragma mark -- IViewAppleEventListener
- (void)layoutSubviewsFromPDFViewCtrl:(FSPDFViewCtrl *)pdfViewCtrl{
    
    if (DEVICE_iPAD) {
        [self resetToolbarPosition:self.userChanged];
    }
}

@end
