/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSFullScreenTopbar.h"
#import "MenuView+Private.h"


@interface FSFullScreenTopbar ()
@property (nonatomic, strong) UILabel *timeLabel;
@property (nonatomic, strong) NSTimer *timer;
@property (nonatomic, strong) UIView *bgView;
@property (nonatomic, strong) UIButton *closeBtn;
@property (nonatomic, strong) UIView *lineView;
@end

@implementation FSFullScreenTopbar

- (instancetype)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        [self arrangeView];
    }

    return self;
}

- (void)arrangeView{
    
    UIView *bgView = [[UIView alloc] init];
    bgView.backgroundColor = self.fs_backgroundColor_B1;
    [self addSubview:bgView];
    self.bgView = bgView;
    [bgView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.equalTo(self).offset(-10);
        make.left.width.top.equalTo(self);
    }];
    
    UIButton *closeBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    closeBtn.accessibilityLabel = FSLocalizedForKey(@"kExitFullScreenVoiceOver");
    [closeBtn setImage:FSImageNamedAndNormalColor(@"full_screen_close") forState:UIControlStateNormal];
    [closeBtn setImage:[FSImageNamedAndNormalColor(@"full_screen_close") fs_applyingAlpha:IMAGE_HIGHLIGHTED_ALPHA] forState:UIControlStateHighlighted];
    [closeBtn addTarget:self action:@selector(closeBtnPress) forControlEvents:UIControlEventTouchUpInside];
    [closeBtn setImageEdgeInsets:UIEdgeInsetsMake(-10, 0, 0, 0)];
    [self addSubview:closeBtn];
    self.closeBtn = closeBtn;
    [closeBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.mas_safeAreaLayoutGuide).offset(9);
        make.size.mas_equalTo(CGSizeMake(30, 30));
        make.bottom.equalTo(self);
    }];

    self.timeLabel = [[UILabel alloc] init];
    self.timeLabel.textColor = self.fs_textColor_T2;
    self.timeLabel.font = self.fs_standardWeakFont_WF1;
    self.timeLabel.textAlignment = NSTextAlignmentCenter;
    [self addSubview:self.timeLabel];
    [self.timeLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(self);
        make.size.mas_equalTo(CGSizeMake(130, 20));
        make.bottom.equalTo(bgView);
    }];

    self.moreBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    self.moreBtn.accessibilityLabel = FSLocalizedForKey(@"kMore");
    [self.moreBtn setImage:FSImageNamedAndNormalColor(@"full_screen_more") forState:UIControlStateNormal];
    [self.moreBtn setImage:[FSImageNamedAndNormalColor(@"full_screen_more") fs_applyingAlpha:IMAGE_HIGHLIGHTED_ALPHA] forState:UIControlStateHighlighted];
    [self.moreBtn addTarget:self action:@selector(moreBtnPress) forControlEvents:UIControlEventTouchUpInside];
    [self.moreBtn setImageEdgeInsets:UIEdgeInsetsMake(-10, 0, 0, 0)];
    [self addSubview:self.moreBtn];
    [self.moreBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.mas_safeAreaLayoutGuide).offset(-9);
        make.size.mas_equalTo(CGSizeMake(30, 30));
        make.bottom.equalTo(self);
    }];
    
    UIView *lineView = [[UIView alloc] init];
    lineView.backgroundColor = self.fs_dividerColor_P1;
    [self addSubview:lineView];
    self.lineView = lineView;
    [lineView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.height.mas_equalTo(DIVIDE_TOOL_BAR_THICKNES);
        make.left.width.equalTo(self);
        make.bottom.equalTo(bgView);
    }];
    
    self.backgroundColor= [UIColor clearColor];
}

- (float)getDisplayHeight{
    return self.bgView.fs_height;
}

- (void)layoutSubviews{
    [super layoutSubviews];
    CGFloat safeAreaTop = self.superview.safeAreaInsets.top;
    if (safeAreaTop >20 ) {
        safeAreaTop += 25;
    }else if (safeAreaTop <= 20){
        safeAreaTop = 30;
    }
    [self mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.right.top.equalTo(self.superview);
        make.height.mas_equalTo(safeAreaTop);
    }];
}

- (void)setHidden:(BOOL)hidden{

    [super setHidden:hidden];

    if (hidden) {
        if (self.timer) {
            [self.timer invalidate];
            self.timer = nil;
        }
    }else{
        [self setTimeContent];
        self.timer = [NSTimer scheduledTimerWithTimeInterval:1 repeats:YES block:^(NSTimer * _Nonnull timer) {
            [self setTimeContent];
        }];
        if (self.extensionsMgr.pdfViewCtrl.isNightMode) {
            self.bgView.backgroundColor = [UIColor clearColor];
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
            if (@available(iOS 13.0, *)) {
                self.overrideUserInterfaceStyle = UIUserInterfaceStyleDark;
            } else
#endif
            {
                [self.closeBtn setImage:FSImageNamedAndColor(@"full_screen_close", self.fs_iconColor_I3) forState:UIControlStateNormal];
                self.timeLabel.textColor = UIColorHex(#808080);
                [self.moreBtn setImage:FSImageNamedAndColor(@"full_screen_more", self.fs_iconColor_I3) forState:UIControlStateNormal];
                self.lineView.backgroundColor = UIColorHex(#36404A);
            }
        }else{
            self.bgView.backgroundColor = self.fs_backgroundColor_B1;
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
            if (@available(iOS 13.0, *)) {
                self.overrideUserInterfaceStyle = UIUserInterfaceStyleUnspecified;
            } else
#endif
            {
                [self.closeBtn setImage:FSImageNamedAndNormalColor(@"full_screen_close") forState:UIControlStateNormal];
                self.timeLabel.textColor = self.fs_textColor_T2;
                [self.moreBtn setImage:FSImageNamedAndNormalColor(@"full_screen_more") forState:UIControlStateNormal];
                self.lineView.backgroundColor = self.fs_dividerColor_P1;
            }
        }
    }
}

- (void)setTimeContent{
    NSDateFormatter *format = [[NSDateFormatter alloc] init];
    format.dateFormat = @"h:mm a";
    NSString *dateStr = [format stringFromDate:[NSDate date]];
    self.timeLabel.text = dateStr;
    self.timeLabel.accessibilityLabel = dateStr;
}

- (void)closeBtnPress{
    if (self.extensionsMgr.currentState == FSUIManagerStateAnnotTool) {
        [self.extensionsMgr changeState:self.extensionsMgr.lastManagerStateWhenAnnotTool];
    }
    [self.extensionsMgr setFullScreen:NO];
}

- (void)moreBtnPress{
    if (_extensionsMgr.moreItemClickedCallback) {
        _extensionsMgr.moreItemClickedCallback(self.moreBtn);
    }
    else
    {
        self.extensionsMgr.currentAnnot = nil;
        self.extensionsMgr.currentWidget = nil;
        self.extensionsMgr.more.hidden = NO;
    }
}

- (void)dealloc
{
    MARK;
}

@end
