/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSHistoryDataView.h"
#import "FSCollectionView.h"
#import "FSHistoryDataCollectionCell.h"

static NSString *const kFSHistoryDataCollectionCellID = @"FSHistoryDataCollectionCell";
@interface FSHistoryDataView ()

@end
@implementation FSHistoryDataView

- (instancetype)init{
    self = [super init];
    if (self) {
        [self initialize];
        
    }
    return self;
}

- (void)initialize{
    [self addCollectionView];
}

- (void)addCollectionView{
    
    UICollectionViewFlowLayout *layout = [[UICollectionViewFlowLayout alloc] init];
    layout.minimumLineSpacing = 1;
    layout.scrollDirection = UICollectionViewScrollDirectionHorizontal;
    _collectionView = [[FSCollectionView alloc] initWithFrame:CGRectZero collectionViewLayout:layout];
    [self addSubview:_collectionView];
    _collectionView.showsHorizontalScrollIndicator = NO;
    _collectionView.delegate = self;
    _collectionView.dataSource = self;
           
    [_collectionView registerClass:[FSHistoryDataCollectionCell class] forCellWithReuseIdentifier:kFSHistoryDataCollectionCellID];
    [self addSubview:_collectionView];
    
    [_collectionView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.right.top.bottom.fs_equalToSuperview();
    }];
}

- (FSHistoryDataItem *)dataForIndexPath:(NSIndexPath *)indexPath{
    return self.dataSource[indexPath.item];
}

- (__kindof FSHistoryDataCollectionCell *)collectionView:(nonnull UICollectionView *)collectionView dequeueReusableCellWithIdentifier:(NSString *)identifier forIndexPath:(NSIndexPath *)indexPath withDataItem:(FSHistoryDataItem *)dataItem{
    FSHistoryDataCollectionCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:identifier forIndexPath:indexPath];
    return cell;
}

- (void)configureCell:(FSHistoryDataCollectionCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(FSHistoryDataItem *)dataItem{}

- (nonnull __kindof UICollectionViewCell *)collectionView:(nonnull UICollectionView *)collectionView cellForItemAtIndexPath:(nonnull NSIndexPath *)indexPath {
    FSHistoryDataItem *dataItem = [self dataForIndexPath:indexPath];
    FSHistoryDataCollectionCell *cell = [self collectionView:collectionView dequeueReusableCellWithIdentifier:kFSHistoryDataCollectionCellID forIndexPath:indexPath withDataItem:dataItem];
    [self configureCell:cell atIndexPath:indexPath withDataItem:dataItem];
    return cell;
}

- (NSInteger)collectionView:(nonnull UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return self.dataSource.count;
}

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView{
    return 1;
}

@end
