/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "ImageAnnotHandler.h"
#import "FSPermissionUtilty.h"


@interface ImageAnnotHandler()
@property (nonatomic, assign) CGPoint oldTouchPoin;
@end

@implementation ImageAnnotHandler

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    if (self = [super initWithUIExtensionsManager:extensionsManager]) {
        _minImageWidthInPage = 0.1;
        _maxImageWidthInPage = 0.5;
        _minImageHeightInPage = 0.1;
        _maxImageHeightInPage = 0.5;
    }
    return self;
}

- (FSAnnotType)getType {
    return FSAnnotScreen;
}

- (BOOL)keepAspectRatioForResizingAnnot:(FSAnnot *)annot {
    if (![self.extensionsManager annotsPermissionCanModify:annot]) {
        return NO;
    }
    return YES;
}

- (CGSize)minSizeForResizingAnnot:(FSAnnot *)annot {
    UIView *pageView = [self.pdfViewCtrl getPageView:annot.pageIndex];
    CGFloat pageWidth = CGRectGetWidth(pageView.bounds);
    CGFloat pageHeight = CGRectGetHeight(pageView.bounds);
    CGFloat minW = self.minImageWidthInPage * pageWidth;
    CGFloat minH = self.minImageHeightInPage * pageHeight;
    return CGSizeMake(minW, minH);
}

- (CGSize)maxSizeForResizingAnnot:(FSAnnot *)annot {
    UIView *pageView = [self.pdfViewCtrl getPageView:annot.pageIndex];
    CGFloat pageWidth = CGRectGetWidth(pageView.bounds);
    CGFloat pageHeight = CGRectGetHeight(pageView.bounds);
    CGFloat maxW = self.maxImageWidthInPage * pageWidth;
    CGFloat maxH = self.maxImageHeightInPage * pageHeight;
    return CGSizeMake(maxW, maxH);
}

- (void)resetProperties{

    FSAnnot *annot = self.extensionsManager.currentAnnot;
    FSPropertyMainView *propertyMainView = self.extensionsManager.propertyMainView;
    [propertyMainView setOpacity:annot.opacity];
    [propertyMainView setLock:![annot canModifyAppearanceWithExternsions:self.extensionsManager]];
    [propertyMainView setPropertyColor:self.fs_primaryColor.rgbValue];
    FSRotation rotation = [[[FSScreen alloc] initWithAnnot:self.annot] getRotation];
    if (rotation != 0) {
        rotation = 4 - rotation;
    }
    [propertyMainView setRotation:[Utility valueForRotation:rotation]];
    [propertyMainView resetProperties:@[ @(FSPropertyTypeRotation), @(FSPropertyTypeOpacity)]];
}

- (UIColor *)borderColorForSelectedAnnot:(FSAnnot *)annot {
    return [UIColor colorWithRGB:0x179cd8];
}

- (FSMenuOptions)menuOptionsForAnnot:(FSAnnot *)annot {
    FSMenuOptions options = FSMenuOptionNone;
    if (![FSPermissionUtilty canAddAnnot:self.extensionsManager]) {
        options = FSMenuOptionNone;
    }
    if (annot.canModify && [FSPermissionUtilty canAddAnnot:self.extensionsManager]) {
        options = FSMenuOptionStyle;
        if ([self.extensionsManager annotPermissionCanFlatten:annot]) {
            options |= FSMenuOptionFlatten;
        }
        if ([annot canDeleteWithExternsions:self.extensionsManager]) {
            options |= FSMenuOptionDelete;
        }
    }
    return options;
}

@end
