/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "ImageModule.h"
#import "FSFileAndImagePicker.h"
#import "ImageAnnotHandler.h"
#import "ImageToolHandler.h"
#import "Utility.h"
#import <FoxitRDK/FSPDFViewControl.h>

@interface ImageModule ()

@property (nonatomic, weak) FSMainToolbarItem *propertyItem;
@property (nonatomic, weak) FSReadToolbarItem *readToolbarItem;
@end

@implementation ImageModule {
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;
}

- (NSString *)getName {
    return @"Image";
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        [self loadModule];
        ImageToolHandler *toolHandler = [[ImageToolHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerToolHandler:toolHandler];
        ImageAnnotHandler *annotHandler = [[ImageAnnotHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerAnnotHandler:annotHandler];

        annotHandler.minImageWidthInPage = annotHandler.minImageHeightInPage = 0.1;
        annotHandler.maxImageWidthInPage = annotHandler.maxImageHeightInPage = 0.7;
    }
    return self;
}

- (void)loadModule {
    FSReadToolbarItem *imageItem = [[FSReadToolbarItem alloc] initWithToolSettings:[_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeImageAnnotation]];
    [_extensionsManager.toolbarManager addReadItem:imageItem atToolTag:FSMainTopbarToolTagComment];
}

- (void)didClickedReadToolItem:(FSReadToolbarItem *)readToolbarItem {
    [_extensionsManager changeState:FSUIManagerStateAnnotTool];
    ImageToolHandler *toolHanlder = (ImageToolHandler *) [_extensionsManager getToolHandlerByName:Tool_Image];
    toolHanlder.toolSettings = readToolbarItem.toolSettings;
    [_extensionsManager setCurrentToolHandler:toolHanlder];

    self.readToolbarItem = readToolbarItem;
    FSMainToolbarItem *propertyItem =  [_extensionsManager.toolbarManager getMainItemWithType:FSMainToolbarItemTypeProperty];
    propertyItem.item.onTapClick = ^(FSItem * _Nonnull _item) {
        CGRect rect = [_item convertRect:_item.bounds toView:self->_extensionsManager.pdfViewCtrl];
        if (DEVICE_iPHONE) {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:rect inView:self->_extensionsManager.pdfViewCtrl];
        } else {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:_item.bounds inView:_item];
        }
    };
    self.propertyItem = propertyItem;
    
    [_extensionsManager onUIInteractElementClicked:FSUIElementReadingCommentBarImage];
}


@end
