/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSInsertPageColorViewController.h"
#import "ColorBox.h"

@interface FSInsertPageColorViewController ()

@property (nonatomic, strong) UIView *colorBoxContainer;
@property (nonatomic, strong) UIStackView *colorItems;
@property (nonatomic, strong) ColorBox *colorBox;
@property (nonatomic, strong) FSItem *customColorItem;
@property (nonatomic, strong) UIColor *customColor;
@end

@implementation FSInsertPageColorViewController

- (ColorBox *)colorBox{
    if(!_colorBox){
        _colorBox = [ColorBox new];
    }
    return _colorBox;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    self.title = FSLocalizedForKey(@"kPageColor");
    self.navBarRightTitle = FSLocalizedForKey(@"kOK");
    @weakify(self)
    self.navBar.rightItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        !self.selectedColor ? : self.selectedColor(self.currentColor);
        [self.services popViewControllerAnimated:YES];
    };
    
    self.colorBoxContainer = [UIView new];
    [self.view addSubview:self.colorBoxContainer];
    
    self.colorBox = [ColorBox new];
    self.colorBox.CallBackInt = ^(long property, UInt32 value) {
        @strongify(self)

        UIColor *customColor = [UIColor colorWithRGB:value];
        self.currentColor = customColor;
        self.customColor = customColor;
        self.customColorItem.onTapClick(self.customColorItem);
        UIImage *image = [FSImageNamed(@"thumbnail_color_ring") fs_addImage:[FSImageNamed(@"thumbnail_ring") fs_imageWithTintColor:self.customColor]];
        [self.customColorItem setImage:image forState:UIControlStateSelected | UIControlStateHighlighted | UIControlStateDisabled];
        [self.customColorItem setImage:image forState:UIControlStateNormal];
        
    };
    [self.colorBoxContainer addSubview:self.colorBox];
    
    self.colorItems = [UIStackView new];
    self.colorItems.distribution = UIStackViewDistributionEqualSpacing;
    [self.colorBoxContainer addSubview:self.colorItems];
    
    [self addColorItems];
}

- (void)makeConstraints{
    [super makeConstraints];
    
    [self.colorBoxContainer mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.fs_equalToSuperview().priorityHigh();
        make.top.equalTo(self.navBar.mas_bottom).priorityHigh();
        make.centerX.fs_equalToSuperview();
        
        make.height.offset(263.f);
    }];
    
    [self.colorBox mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.colorBox.superview.fs_mas_left).offset(ITEM_MARGIN_NORMAL);
        make.right.equalTo(self.colorBox.superview.fs_mas_right).offset(-ITEM_MARGIN_NORMAL);
        make.top.equalTo(self.colorBox.superview.mas_top).offset(ITEM_MARGIN_NORMAL);
        make.width.mas_lessThanOrEqualTo(MIN_BAR_WIDTH_IN_VIEW(self.view));
        make.height.offset(160.f);
    }];
    
    [self.colorItems mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self.colorBox);
        make.top.equalTo(self.colorBox.mas_bottom).offset(ITEM_MEDIUM_SPACE_NORMAL);
        make.height.offset(32.f);
    }];
}

- (void)addColorItems{
    NSArray *tmp = @[ UIColor.whiteColor, UIColorHex(#FFFFBF), UIColorHex(#DDDDFF), UIColorHex(#669933), UIColorHex(D8D8D8), UIColorHex(#B0B0B0), UIColor.blackColor, UIColor.clearColor];
    
    __block FSItem *currentItem = nil;
    for (int i = 0; i < tmp.count; i++ ) {
        UIColor *color = tmp[i];
        UIImage *image = nil;
        if (i == tmp.count - 1) {
            image = FSImageNamed(@"thumbnail_ring_none");
            if (!currentItem) {
                self.customColor = self.currentColor;
                image = [FSImageNamed(@"thumbnail_color_ring") fs_addImage:[FSImageNamed(@"thumbnail_ring") fs_imageWithTintColor:self.currentColor]];
            }
        }else{
            image = [UIImage fs_imageWithColor:color size:CGSizeMake(24.f, 24.f)];
        }
        FSItem *colorItem = [FSItem new];
        colorItem.accessibilityLabel = [NSString stringWithFormat:@"%@ %@", FSLocalizedForKey(@"kColorValueAccessibilityLabel"), [color hexString]];
        [colorItem setImage:image forState:FSItemControlStateAll];

        colorItem.imageView.backgroundColor = color;
        [colorItem fs_setLayerCornerRadius:LAYER_BORDER_RADIUS_3];
        
        @weakify(self)
        colorItem.onTapClick = ^(FSItem * _Nonnull _item) {
            if (currentItem != _item) {
                @strongify(self)
                [currentItem fs_setLayerBorderWidth:0 borderColor:nil];
                currentItem.selected = NO;
                currentItem = _item;
                currentItem.selected = YES;
                [currentItem fs_setLayerBorderWidth:LAYER_BORDER_WIDTH_1 borderColor:self.fs_primaryColor];
                
                if (self.customColorItem == currentItem) {
                    if (!self.customColor) {
                        self.colorBox.CallBackInt(0, self.currentColor.rgbValue);
                        self.customColor = nil;
                    }else{
                        self.currentColor = self.customColor;
                    }
                }else{
                    self.currentColor = currentItem.imageView.backgroundColor;
                }
            }
        };
        
        if (i == tmp.count - 1) {
            self.customColorItem = colorItem;
            self.customColorItem.accessibilityLabel = FSLocalizedForKey(@"kCustomColorAccessibilityLabel");
            if (!currentItem) {
                colorItem.onTapClick(colorItem);
            }
            [colorItem setImageEdgeInsets:UIEdgeInsetsMake(1, 1, 1, 1)];
        }else{
            if (color.rgbaValue == self.currentColor.rgbaValue) {
                colorItem.onTapClick(colorItem);
            }
            if (i == 0) {
                [colorItem.imageView fs_setLayerBorderWidth:LAYER_BORDER_WIDTH_05 borderColor:self.fs_backgroundColor_B3];
            }
            [colorItem setImageEdgeInsets:UIEdgeInsetsMake(4.f, 4.f, 4.f, 4.f)];
            [colorItem.imageView fs_setLayerCornerRadius:12.f];
            
        }
        
        [self.colorItems addArrangedSubview:colorItem];
    }
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
