/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSInsertPageConfigVCtrl.h"
#import "FSInsertPageSizeOrOriVCtrl.h"
#import "FSDataItem.h"

@interface FSInsertPageConfigVCtrl ()<UITextFieldDelegate, UIScrollViewDelegate,IRotationEventListener>
{
    __weak UIExtensionsManager *_extensionsManager;
}

@property (nonatomic, strong, readwrite) FSViewControllerManager *services;
@property (nonatomic, weak) UITextField *pageCountTextFiled;
@property (nonatomic, weak) UILabel *pageSize;
@property (nonatomic, weak) UIView *pageColor;
@property (nonatomic, weak) UILabel *pageOrientation;

@property (nonatomic, strong) UILabel *pageStyleLab;


@property (nonatomic, assign) NSUInteger selectedSizeTypeRow;
@property (nonatomic, assign) NSUInteger selectedOrientationTypeRow;
@property (nonatomic, strong) UIColor *selectedColor;


@property (nonatomic, strong) UIButton *curselectPageTexturebtn;

@property (nonatomic, strong) UIScrollView *pageScrollView;

@property (nonatomic, strong) FSItem  *okItem;
@end

@interface FSInsertPageDataItem : FSDataItem

@property (nonatomic, copy) NSString *title;

@property (nonatomic, strong) id value;
@end

@implementation FSInsertPageDataItem @end

@implementation FSInsertPageConfigVCtrl

@dynamic services;

- (instancetype)initWithServices:(id<FSViewControllerServices>)services params:(nullable id)params{
    self = [super initWithServices:services params:params];
    if (self) {
        _extensionsManager = self.services.extensionsManager;
    }
    return self;
}

- (void)initialize{
    [super initialize];
    self.defaultEstimatedCellStyle = FSEstimatedTableViewCellStyleTitle;
    self.shouldLoadDataOnViewDidLoad = YES;
    self.selectedColor = UIColor.whiteColor;
}

- (UIScrollView *)pageScrollView{
    if (!_pageScrollView) {
        _pageScrollView = [UIScrollView new];
        _pageScrollView.delegate = self;
        _pageScrollView.showsHorizontalScrollIndicator = NO;
    }
    return _pageScrollView;
}

- (UILabel *)pageStyleLab{
    if (!_pageStyleLab) {
        _pageStyleLab = [UILabel new];
        _pageStyleLab.text = FSLocalizedForKey(@"kPageStyle");
        _pageStyleLab.textColor = self.fs_textColor_T3;
        _pageStyleLab.font = self.fs_standardWeakFont_WF1;
        [self.view addSubview:_pageStyleLab];
    }
    return _pageStyleLab;
}

- (void)makeConstraints{
    [super makeConstraints];
    
    [self.pageStyleLab mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.navBar.mas_bottom).offset(ITEM_IMAGE_TOP_MARGIN_NORMAL);
        make.left.equalTo(self.pageStyleLab.superview.fs_mas_left).offset(ITEM_MARGIN_NORMAL);
        make.height.offset(22.f);
    }];
    
    [self.pageScrollView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.pageStyleLab.mas_bottom).offset(ITEM_IMAGE_TOP_MARGIN_NORMAL);
        make.left.equalTo(self.view.fs_mas_left);
        make.right.equalTo(self.view.fs_mas_right);
        make.height.offset(116.f);
    }];
    
    [self.tableView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.right.fs_equalToSuperview();
        make.bottom.equalTo(self.tableView.superview);
        make.top.equalTo(self.navBar.mas_bottom).offset(190.f);
    }];
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.title = FSLocalizedForKey(@"kAddPage");
    self.navBarLeftTitle = FSLocalizedForKey(@"kCancel");
    self.navBarRightTitle = FSLocalizedForKey(@"kOK");
    
    self.tableView.alwaysBounceVertical = NO;
    
    FSItem *okItem = self.navBar.rightItem;
    @weakify(self)
    okItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [self clickedOK:_item];
    };
    self.okItem = okItem;
    
    [self.view addSubview:self.pageScrollView];
    
    self.selectedSizeTypeRow = 0;
    self.selectedOrientationTypeRow = 0;
    
    self.view.backgroundColor = self.fs_backgroundColor_B2;
}

- (void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    [self createPageTextureSelect];
}

- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
}

- (void)loadData{
    NSArray *titles = @[ FSLocalizedForKey(@"kPageCount"), FSLocalizedForKey(@"kPageSize"), FSLocalizedForKey(@"kPageColor"), FSLocalizedForKey(@"kOrientation")];
    NSArray *defaultValue = @[ @"1", FSLocalizedForKey(@"kLetter"), self.selectedColor, FSLocalizedForKey(@"kPortrait")];
    NSMutableArray *tmp = @[].mutableCopy;
    for (int i = 0 ; i < titles.count; i++) {
        FSInsertPageDataItem *item = [FSInsertPageDataItem new];
        item.title = titles[i];
        item.value = defaultValue[i];
        [tmp addObject:item];
    }
    self.dataSource = @[tmp.copy];
}

- (void)configureCell:(FSTitleEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(FSInsertPageDataItem *)dataItem{
    if (!cell.installed) {
        cell.titleDefaultTextColor = self.fs_textColor_T3;
        if (indexPath.row == 0) {
            UITextField *pageCountTextFiled = [UITextField new];
            pageCountTextFiled.delegate = self;
            pageCountTextFiled.font = self.fs_standardGeneralFont_GF3;
            pageCountTextFiled.textColor = self.fs_textColor_T4;
            pageCountTextFiled.textAlignment = NSTextAlignmentRight;
            pageCountTextFiled.borderStyle = UITextBorderStyleNone;
            pageCountTextFiled.keyboardType = UIKeyboardTypePhonePad;
            [cell.contentView addSubview:pageCountTextFiled];
            self.pageCountTextFiled = pageCountTextFiled;
            [pageCountTextFiled mas_makeConstraints:^(MASConstraintMaker *make) {
                make.right.fs_equalToSuperview().offset(-ITEM_MARGIN_NORMAL);
                make.centerY.fs_equalToSuperview();
                make.width.fs_equalToSuperview().multipliedBy(0.5);
            }];
        } else{
            UIImageView *accessoryImageView = [UIImageView new];
            cell.rightView = accessoryImageView;
            
            UILabel *label = [UILabel new];
            label.font = self.fs_standardGeneralFont_GF3;
            label.textColor = self.fs_textColor_T4;
            label.textAlignment = NSTextAlignmentRight;
            
            [cell.contentView addSubview:label];
            
            [label mas_makeConstraints:^(MASConstraintMaker *make) {
                make.right.equalTo(accessoryImageView.mas_left);
                make.centerY.fs_equalToSuperview();
            }];
            if (indexPath.row == 1) {
                self.pageSize = label;
            }else if (indexPath.row == 2){
                [label removeFromSuperview];
                
                UIView *pageColor = [UIView new];
                [pageColor fs_setLayerCornerRadius:LAYER_BORDER_WIDTH_05];
                [pageColor fs_setLayerBorderWidth:LAYER_BORDER_WIDTH_1 borderColor:self.fs_dividerColor_P1];
                
                [cell.contentView addSubview:pageColor];
                self.pageColor = pageColor;
                [pageColor mas_makeConstraints:^(MASConstraintMaker *make) {
                    make.right.equalTo(accessoryImageView.mas_left);
                    make.centerY.fs_equalToSuperview();
                    make.size.equalTo(@(CGSizeMake(22.f, 22.f)));
                }];
            }else if (indexPath.row == 3){
                self.pageOrientation = label;
            }
        }
    }
    if (indexPath.row == 0) {
        self.pageCountTextFiled.text = dataItem.value;
    }else{
       ((UIImageView *)cell.rightView).image = FSImageNamedAndNormalColor(@"thumbnail_next");
        if (indexPath.row == 1) {
            self.pageSize.text = dataItem.value;
        }else if (indexPath.row == 2){
            self.pageColor.backgroundColor = dataItem.value;
        }else if (indexPath.row == 3){
            self.pageOrientation.text = dataItem.value;
        }
    }
    cell.estimatedTitleLabel.text = dataItem.title;
    
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{
    if (indexPath.row == 0) {
        [self.pageCountTextFiled becomeFirstResponder];
    }else if(indexPath.row == 1){
        FSInsertPageSizeOrOriVCtrl *insertPageSizeOrOriVCtrl = [self.services createViewCtrlType:FSViewCtrlTypeInsertPageSizeOrOri params:@(0)];
        [insertPageSizeOrOriVCtrl setSelectedType:^(NSUInteger selectedRow, NSString * _Nullable selectedStr) {
            self.selectedSizeTypeRow = selectedRow;
            self.pageSize.text = selectedStr;
        }];
        insertPageSizeOrOriVCtrl.selectedRow = self.selectedSizeTypeRow;
        [self.services pushViewController:insertPageSizeOrOriVCtrl animated:YES];

    }else if (indexPath.row == 2){
        FSInsertPageColorViewController *colorSelectVC = [self.services createViewCtrlType:FSViewCtrlTypeInsertPageColor];
        colorSelectVC.currentColor = self.selectedColor;
        [colorSelectVC setSelectedColor:^(UIColor *color) {
            self.selectedColor = color;
            self.pageColor.backgroundColor = color;
        }];
        [self.services pushViewController:colorSelectVC animated:YES];
    }else if (indexPath.row == 3){
        FSInsertPageSizeOrOriVCtrl *insertPageSizeOrOriVCtrl = [self.services createViewCtrlType:FSViewCtrlTypeInsertPageSizeOrOri params:@(1)];
        [insertPageSizeOrOriVCtrl setSelectedType:^(NSUInteger selectedRow, NSString * _Nullable selectedStr) {
            self.selectedOrientationTypeRow = selectedRow;
            self.pageOrientation.text = selectedStr;
        }];
        insertPageSizeOrOriVCtrl.selectedRow = self.selectedOrientationTypeRow;
        [self.services pushViewController:insertPageSizeOrOriVCtrl animated:YES];
    }
    [super tableView:tableView didSelectRowAtIndexPath:indexPath];
}

- (void)clickedOK:(id)sender {

    FSPDFPageSize pageSize = FSPDFPageSizeLetter;
    switch (self.selectedSizeTypeRow) {
        case 0:
            pageSize = FSPDFPageSizeLetter;
            break;
        case 1:
            pageSize = FSPDFPageSizeA3;
            break;
        case 2:
            pageSize = FSPDFPageSizeA4;
            break;
        case 3:
            pageSize = FSPDFPageSizeLegal;
            break;
            
        default:
            break;
    }
    if (self.selectedSizeTypeRow != 4) {
        if([_extensionsManager.pdfViewCtrl insertPages:self.delegate.insertIndex pageSize:pageSize style:(int)(_curselectPageTexturebtn.tag - 200) color:_selectedColor.fs_argbValue rotation:(FSRotation)self.selectedOrientationTypeRow count:_pageCountTextFiled.text.intValue]) {
            if (self.delegate && [self.delegate respondsToSelector:@selector(insertPageSucceed:count:)]) {
                [self.delegate insertPageSucceed:_extensionsManager.pdfViewCtrl.currentDoc count:_pageCountTextFiled.text.intValue];
            }
            [self dismissViewControllerAnimated:YES completion:nil];
        }
    }else{
        CGFloat width = 72 * 11;
        CGFloat height = 72 * 17;
        if ([_extensionsManager.pdfViewCtrl insertPages:self.delegate.insertIndex width:width height:height style:(int)(_curselectPageTexturebtn.tag - 200) color:_selectedColor.fs_argbValue rotation:(FSRotation)self.selectedOrientationTypeRow count:_pageCountTextFiled.text.intValue]) {
            if (self.delegate && [self.delegate respondsToSelector:@selector(insertPageSucceed:count:)]) {
                [self.delegate insertPageSucceed:_extensionsManager.pdfViewCtrl.currentDoc count:_pageCountTextFiled.text.intValue];
            }
            [self dismissViewControllerAnimated:YES completion:nil];
        }
    }

}

- (void)createPageTextureSelect{
    [self.pageScrollView layoutIfNeeded];
    CGFloat scrW = self.view.fs_width - 30 * 2;
    CGFloat spacing = scrW/12;
    CGFloat pageTextureBtnW = (scrW - spacing * 2)/ 3;
    if (self.view.fs_width > 500) {
        pageTextureBtnW = (scrW - spacing * 2)/ 4;
    }
    CGFloat pageTextureBtnH = CGRectGetHeight(self.pageScrollView.frame);
    [self.pageScrollView setContentSize:CGSizeMake(scrW * 2  , CGRectGetHeight(self.pageScrollView.frame))];
    if ((DEVICE_iPAD && ![self fs_currentSceneiPadHorizontalCompact]) || (UIInterfaceOrientationIsLandscape([UIApplication sharedApplication].statusBarOrientation) && DEVICE_iPHONE)) {
        spacing = scrW/20;
        pageTextureBtnW = (scrW - spacing * 4)/ 5;
        [self.pageScrollView setContentSize:CGSizeMake(scrW  , CGRectGetHeight(self.pageScrollView.frame))];
    }
    
    NSArray *imgName = @[ @"", @"thumb_lined", @"thumb_grid", @"thumb_graph", @"thumb_music" ];
    NSArray *titleStr = @[ FSLocalizedForKey(@"kBlank"), FSLocalizedForKey(@"kLined"), FSLocalizedForKey(@"kGrid"), FSLocalizedForKey(@"kGraph"), FSLocalizedForKey(@"kMusic") ];
    void (^createPageTexture)(void) = ^{
        
        for (int i = 0; i < imgName.count; i++) {
            UIImage *image = nil;
            if (i == 0) {
                image = FSImageNamed(imgName[1]);
                image = [UIImage fs_imageWithColor:[UIColor whiteColor] size:image.size];
            }else{
                image = FSImageNamed(imgName[i]);
            }
            
            image = [self imageWithBorderWidth:2.f borderColor:[UIColor colorWithRed:239.f/255.0 green:239.f/255.0 blue:244.f/255.0 alpha:1] image:image];
            
            UIButton *pageTextureBtn = [[UIButton alloc] initWithFrame:CGRectMake(pageTextureBtnW * i + spacing * (i + 1),0 , pageTextureBtnW, pageTextureBtnH - 20)];
            pageTextureBtn.accessibilityLabel = titleStr[i];
            pageTextureBtn.tag = 200 + i;
            [pageTextureBtn addTarget:self action:@selector(didSelectedPageTexture:) forControlEvents:UIControlEventTouchUpInside];
            if (i == 0) {
                pageTextureBtn.selected = YES;
                self->_curselectPageTexturebtn = pageTextureBtn;
            }
            [pageTextureBtn setImage:image forState:UIControlStateNormal];
            @weakify(self)
            [BindThemeKeySel(pageTextureBtn, setImage:forState:) fs_updateThemeCompleted:^(UIButton * _Nonnull target, SEL  _Nonnull sel) {
                @strongify(self)
                [target setImage:[self imageWithBorderWidth:2.f borderColor:target.fs_primaryColor image:image] forState:UIControlStateSelected];
                [target setImage:[image fs_applyingAlpha:IMAGE_HIGHLIGHTED_ALPHA] forState:UIControlStateHighlighted];
            }];
            
            [self.pageScrollView addSubview:pageTextureBtn];
            
            CGRect labRect = pageTextureBtn.frame;
            labRect.origin.y = CGRectGetMaxY(pageTextureBtn.frame);
            labRect.size.height = 20.f;
            UILabel *titleLab = [[UILabel alloc] initWithFrame:labRect];
            titleLab.isAccessibilityElement = NO;
            titleLab.textColor = [UIColor colorWithRed:111/255.f green:113/255.f blue:121/255.f alpha:1];
            titleLab.font = [UIFont systemFontOfSize:15.f];
            titleLab.textAlignment = NSTextAlignmentCenter;
            titleLab.text = titleStr[i];
            titleLab.tag = 300 + i;
            
            [self.pageScrollView addSubview:titleLab];
            
            [self.pageScrollView setContentSize:CGSizeMake(CGRectGetMaxX(pageTextureBtn.frame) + spacing , CGRectGetHeight(self.pageScrollView.frame))];
        }
    };
#if _MAC_CATALYST_
    if (self.pageScrollView.subviews.count > 1) {
#else
    if (self.pageScrollView.subviews.count) {
#endif
        for (int i = 0 ; i < self.pageScrollView.subviews.count; i++ ) {
            CGRect frame = CGRectMake(pageTextureBtnW * i + spacing * (i + 1), 0, pageTextureBtnW, pageTextureBtnH - 20);
            UIView *pageTextureBtn = [self.pageScrollView viewWithTag:200+i];
            if (!pageTextureBtn && ((imgName.count - 1) < i)) {
                break;
            }
            pageTextureBtn.frame = frame;
            UIView *titleLab = [self.pageScrollView viewWithTag:300+i];
            frame.origin.y = CGRectGetMaxY(frame);
            frame.size.height = 20.f;
            titleLab.frame = frame;
            
            [self.pageScrollView setContentSize:CGSizeMake(CGRectGetMaxX(pageTextureBtn.frame) + spacing , CGRectGetHeight(self.pageScrollView.frame))];
        }
    }else{
        createPageTexture();
    }
}

- (void)didSelectedPageTexture:(UIButton *)btn{
    if (btn != _curselectPageTexturebtn) {
        btn.selected = YES;
        _curselectPageTexturebtn.selected = NO;
        _curselectPageTexturebtn = btn;
    }
}

- (void)viewWillTransitionToSize:(CGSize)size withTransitionCoordinator:(id<UIViewControllerTransitionCoordinator>)coordinator{
    [super viewWillTransitionToSize:size withTransitionCoordinator:coordinator];
    [coordinator animateAlongsideTransition:nil
                                 completion:^(id<UIViewControllerTransitionCoordinatorContext>  _Nonnull context) {[self createPageTextureSelect];}];
}

#pragma -mark IRotationEventListener
- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation{
    [self createPageTextureSelect];
}

#pragma -mark UIScrollViewDelegate

#pragma -mark UITextFieldDelegate
- (void)textFieldDidEndEditing:(UITextField *)textField{
    if (!textField.text.length) {
        textField.text = @"1";
        NSString *title = FSLocalizedForKey(@"kWarning");
        NSString *message = [NSString stringWithFormat:@"%@ 1-100", FSLocalizedForKey(@"kEnterPageNumber")];
        FSAlertViewCtrlShow (title, message);
    }
}

- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string{
    if (string.length == 0 && range.length == 1) {
        return YES;
    }
    NSString *currentString = [textField.text stringByAppendingString:string];
    if (currentString.intValue>100) {
        textField.text = @"100";
    }
    NSString *regex = @"(^[0-9]{1,3})";
    NSPredicate *pred = [NSPredicate predicateWithFormat:@"SELF MATCHES %@", regex];
    BOOL flag = [pred evaluateWithObject:currentString];
    if (!flag || currentString.intValue>100 || !currentString.intValue) {
        NSString *title = FSLocalizedForKey(@"kWarning");
        NSString *message = [NSString stringWithFormat:@"%@ 1-100", FSLocalizedForKey(@"kEnterPageNumber")];
        FSAlertViewCtrlShow (title, message);
        return NO;
    }
    return flag;
}

- (UIImage *)imageWithBorderWidth:(CGFloat)borderW borderColor:(UIColor *)color image:(UIImage *)image{
    
    CGSize size = CGSizeMake(image.size.width + 2 * borderW, image.size.height + 2 * borderW);
    if (size.width <= 0 || size.height <= 0) return nil;
    UIGraphicsBeginImageContextWithOptions(size, NO, 0);
    UIBezierPath *path = [UIBezierPath bezierPathWithRect:CGRectMake(0, 0, size.width, size.height)];
    [color set];
    [path fill];

    path = [UIBezierPath bezierPathWithRect:CGRectMake(borderW, borderW, image.size.width, image.size.height)];
    [path addClip];
    [image drawInRect:CGRectMake(borderW, borderW, image.size.width, image.size.height)];
    UIImage *newImage = UIGraphicsGetImageFromCurrentImageContext();

    UIGraphicsEndImageContext();
    return newImage;
}

@end
