/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSInsertPageSizeOrOriVCtrl.h"
#import "FSDataItem.h"

@interface FSInsertPageConfigTypeItem : FSDataItem
@property (nonatomic, copy) NSString *title;
@end

@implementation FSInsertPageConfigTypeItem @end

@interface FSInsertPageSizeOrOriVCtrl ()<UITableViewDelegate, UITableViewDataSource>
@property (nonatomic, strong) FSItem *okItem;

@property (nonatomic, strong) FSInsertPageConfigTypeItem *currentCellItem;
@end

@implementation FSInsertPageSizeOrOriVCtrl

- (void)initialize{
    [super initialize];
    self.defaultEstimatedCellStyle = FSEstimatedTableViewCellStyleTitle;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    self.title = FSLocalizedForKey(@"kPageSize");
    if ([self.params intValue] == 1) {
        self.title = FSLocalizedForKey(@"kOrientation");
    }
    self.navBarRightTitle = FSLocalizedForKey(@"kOK");
    @weakify(self)
    self.navBar.rightItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [self clickedOK:_item];
    };
    
    [self creatDataSource];
    
    self.tableView.separatorStyle = UITableViewCellSeparatorStyleNone;
    
    [self tableView:self.tableView selectRowAtIndexPath:[NSIndexPath indexPathForRow:self.selectedRow inSection:0]];
}

- (void)creatDataSource{
    NSArray *tempArr = @[ FSLocalizedForKey(@"kLetter"), @"A3", @"A4", FSLocalizedForKey(@"kLegal"), FSLocalizedForKey(@"kLedger") ];
    if ([self.params intValue] == 1) {
        tempArr = @[ FSLocalizedForKey(@"kPortrait"), FSLocalizedForKey(@"kLandscape")];
    }
    NSMutableArray *dataSource = @[].mutableCopy;
    for (int i = 0 ; i < tempArr.count; i++) {
        FSInsertPageConfigTypeItem *item = [FSInsertPageConfigTypeItem new];
        item.title = tempArr[i];
        item.selected = NO;
        if (i == self.selectedRow) {
            item.selected = YES;
            self.currentCellItem = item;
        }
        [dataSource addObject:item];
    }
    self.dataSource = @[dataSource.copy];
}

- (void)tableView:(UITableView *)tableView selectRowAtIndexPath:(NSIndexPath *)indexPath{
    
    [tableView selectRowAtIndexPath:indexPath animated:NO scrollPosition:0];
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

- (IBAction)clickedOK:(id)sender {
    !self.selectedType?:self.selectedType(self.selectedRow, self.currentCellItem.title);
    [self.services popViewControllerAnimated:YES];
}

- (void)configureCell:(FSTitleEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(FSInsertPageConfigTypeItem *)dataItem{
    cell.selectedBackgroundColor = self.fs_dividerColor_P1;
    cell.contentView.backgroundColor = nil;
    if (!cell.rightView) {
        UIImageView *imageView = [UIImageView new];
        [BindThemeKeySel(imageView, setImage:) fs_updateThemeCompleted:^(UIImageView * _Nonnull target, SEL  _Nonnull sel) {
            target.image = FSImageNamedAndPrimaryColor(@"common_select_check");
        }];
        imageView.hidden = YES;
        cell.rightView = imageView;
    }
    cell.isAccessibilityElement = YES;
    cell.accessibilityLabel = dataItem.title;
    if (dataItem.selected) {
        cell.accessibilityTraits = UIAccessibilityTraitSelected;
    }else{
        cell.accessibilityTraits = UIAccessibilityTraitNone;
    }
    cell.rightView.hidden = !dataItem.selected;
    cell.estimatedTitleLabel.text = dataItem.title;
}

#pragma -mark UITableViewDelegate

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{
    
    NSIndexPath *lastIndexPath = [NSIndexPath indexPathForRow:[self.dataSource indexOfObject:self.currentCellItem] inSection:0];
    self.currentCellItem.selected = NO;
    FSInsertPageConfigTypeItem *item = [self dataForAtIndexPath:indexPath];
    item.selected = YES;
    self.currentCellItem = item;
    
    [tableView reloadRowsAtIndexPaths:@[lastIndexPath, indexPath] withRowAnimation:0];
    self.selectedRow = indexPath.row;
    
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}


@end

