/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "LineModule.h"
#import <FoxitRDK/FSPDFViewControl.h>

#import "LineAnnotHandler.h"
#import "LineToolHandler.h"
#import "Utility.h"

@interface LineModule ()

@property (nonatomic, weak) FSMainToolbarItem *propertyItem;
@property (nonatomic, weak) FSReadToolbarItem *readToolbarItem;
@property (nonatomic, strong) FSReadToolbarItem *lineItem;
@property (nonatomic, strong) FSReadToolbarItem *arrowItem;
@end

@implementation LineModule {
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;
    FSAnnotType _annotType;
    BOOL _isArrLine;
}

- (NSString *)getName {
    return @"Line";
}

#pragma mark init
- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;

        [self loadModule];
        LineAnnotHandler* annotHandler = [[LineAnnotHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerAnnotHandler:annotHandler];

        LineToolHandler* toolHandler = [[LineToolHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerToolHandler:toolHandler];
    }
    return self;
}

- (FSAnnotType)annotType{
    return FSAnnotLine;
}

- (void)loadModule {
    if ([_extensionsManager.config.tools containsObject:Tool_Line]) {
        _lineItem = [[FSReadToolbarItem alloc] initWithToolSettings:[_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeLine]];
        [_extensionsManager.toolbarManager addReadItem:_lineItem atToolTag:FSMainTopbarToolTagDrawing];
    }
    
    if ([_extensionsManager.config.tools containsObject:Tool_Arrow]) {
        _arrowItem = [[FSReadToolbarItem alloc] initWithToolSettings:[_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeArrow]];
        [_extensionsManager.toolbarManager addReadItem:_arrowItem atToolTag:FSMainTopbarToolTagDrawing];
    }
    [_extensionsManager registerAnnotPropertyListener:self];
}

- (void)didClickedReadToolItem:(FSReadToolbarItem *)readToolbarItem {
    [_extensionsManager changeState:FSUIManagerStateAnnotTool];
    LineToolHandler *toolHandler = [_extensionsManager getToolHandlerByName:Tool_Line];
    if (readToolbarItem.settingsType == FSReadToolSettingsTypeLine) {
        self->_isArrLine = NO;
    }
    if (readToolbarItem.settingsType == FSReadToolSettingsTypeArrow) {
        self->_isArrLine = YES;
    }
    toolHandler.type = self.annotType;
    toolHandler.isArrowLine = _isArrLine;
    toolHandler.isDistanceTool = NO;
    toolHandler.toolSettings = readToolbarItem.toolSettings;
    [_extensionsManager setCurrentToolHandler:toolHandler];
    
    self.readToolbarItem = readToolbarItem;
    FSMainToolbarItem *propertyItem =  [_extensionsManager.toolbarManager getMainItemWithType:FSMainToolbarItemTypeProperty];
    propertyItem.item.onTapClick = ^(FSItem * _Nonnull _item) {
        CGRect rect = [_item convertRect:_item.bounds toView:self->_extensionsManager.pdfViewCtrl];
        if (DEVICE_iPHONE) {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:rect inView:self->_extensionsManager.pdfViewCtrl];
        } else {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:_item.bounds inView:_item];
        }
    };
    
    if (readToolbarItem.toolSettings.settingsType == FSReadToolSettingsTypeLine) {
        [_extensionsManager onUIInteractElementClicked:FSUIElementReadingDrawingBarLine];
    }
    else if (readToolbarItem.toolSettings.settingsType == FSReadToolSettingsTypeArrow) {
        [_extensionsManager onUIInteractElementClicked:FSUIElementReadingDrawingBarArrow];
    }
}

- (void)onToolSettingsChanged:(FSReadToolSettings *)toolSettings {
    if (self.readToolbarItem && self.readToolbarItem.toolSettings == toolSettings) {
        self.readToolbarItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }else if (_lineItem.toolSettings == toolSettings){
        _lineItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }else if (_arrowItem.toolSettings == toolSettings){
        _arrowItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }
}

@end
