/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSLinkEditViewController.h"
#import "FSLinkThumbnailViewController.h"

@interface FSLinkEditViewController () <UITextFieldDelegate>

@property(nonatomic, assign) int selectRow;
@property(nonatomic, strong) UIButton *pageCheckBtn;
@property(nonatomic, strong) UIButton *webCheckBtn;
@property(nonatomic, strong) UITextField *pageTF;
@property(nonatomic, strong) UITextField *webTF;
@property(nonatomic, strong) UILabel *pageNoticeLable;
@end

@implementation FSLinkEditViewController

- (void)viewDidLoad{
    [super viewDidLoad];
    self.title = FSLocalizedForKey(@"kSettings");
    self.navBarLeftTitle = FSLocalizedForKey(@"kCancel");
    self.navBarRightTitle = FSLocalizedForKey(@"kDone");
    [self.navBar.rightItem addTarget:self action:@selector(donePress) forControlEvents:UIControlEventTouchUpInside];
    self.selectRow = 0;
    self.tableView.backgroundColor = self.fs_backgroundColor_B2;
    self.navBar.rightItem.enabled = NO;

    if (self.linkAnnot && ![self.linkAnnot isEmpty]) {
        FSLink *link = [[FSLink alloc] initWithAnnot:self.linkAnnot];
        FSAction *action = [link getAction];
        if (action && ![action isEmpty]) {
            FSActionType type = [action getType];
            if (type == FSActionTypeGoto) {
                FSGotoAction *gotoAction = [[FSGotoAction alloc] initWithAction:action];
                FSDestination *dest = [gotoAction getDestination];
                if (dest && ![dest isEmpty]) {
                    int jumpIndex = [dest getPageIndex:_extensionsMgr.pdfViewCtrl.getDoc];
                    if (jumpIndex >= 0 && jumpIndex < _extensionsMgr.pdfViewCtrl.getPageCount) {
                        self.pageTF.text = [NSString stringWithFormat:@"%d",jumpIndex+1];
                    }
                }
            }else if (type == FSActionTypeURI){
                FSURIAction *uriAction = [[FSURIAction alloc] initWithAction:action];
                NSString *uriStr = [uriAction getURI];
                if (uriStr && uriStr.length > 0) {
                    self.webTF.text = uriStr;
                    [self.webTF becomeFirstResponder];
                }
            }
        }

    }
}

- (UIButton *)pageCheckBtn{
    if (!_pageCheckBtn) {
        _pageCheckBtn = [UIButton buttonWithType:UIButtonTypeCustom];
        [_pageCheckBtn setImage:FSImageNamedAndPrimaryColor(@"common_blue_select") forState:UIControlStateNormal];
        [BindThemeKeySel(_pageCheckBtn, setImage:forState:) fs_updateThemeCompleted:^(FSItem *  _Nonnull target, SEL  _Nonnull sel) {
      
            if (target.currentImage && target.state == UIControlStateNormal) {
                [target setImage:FSImageNamedAndPrimaryColor(@"common_blue_select") forState:UIControlStateNormal];
            }
        }];
    }
    return _pageCheckBtn;
}

- (UIButton *)webCheckBtn{
    if (!_webCheckBtn) {
        _webCheckBtn = [UIButton buttonWithType:UIButtonTypeCustom];
        [_webCheckBtn setImage:FSImageNamedAndPrimaryColor(@"common_blue_select") forState:UIControlStateNormal];
        [BindThemeKeySel(_webCheckBtn, setImage:forState:) fs_updateThemeCompleted:^(FSItem *  _Nonnull target, SEL  _Nonnull sel) {
      
            if (target.currentImage && target.state == UIControlStateNormal) {
                [target setImage:FSImageNamedAndPrimaryColor(@"common_blue_select") forState:UIControlStateNormal];
            }
        }];
    }
    return _webCheckBtn;
}

- (UITextField *)pageTF{
    
    if (!_pageTF) {
        _pageTF = [[UITextField alloc] init];
        _pageTF.accessibilityIdentifier = @"link_to_page_id";
        _pageTF.clearButtonMode = UITextFieldViewModeWhileEditing;
        _pageTF.font = self.fs_standardGeneralFont_GF2;
        _pageTF.textColor = self.fs_textColor_T4;
        _pageTF.text = [NSString stringWithFormat:@"%d",self.extensionsMgr.pdfViewCtrl.getCurrentPage+1];
        _pageTF.keyboardType = UIKeyboardTypeNumberPad;
        _pageTF.delegate = self;
        UIButton *clearButton = [_pageTF valueForKey:@"_clearButton"];
        if (clearButton) {
            [clearButton setImage:FSImageNamed(@"edit_link_clear") forState:UIControlStateNormal];
        }
    }
    return _pageTF;
}

- (UITextField *)webTF{
    if (!_webTF) {
        _webTF = [[UITextField alloc] init];
        _webTF.accessibilityIdentifier = @"link_to_web_id";
        _webTF.clearButtonMode = UITextFieldViewModeWhileEditing;
        _webTF.font = self.fs_standardGeneralFont_GF2;
        _webTF.textColor = self.fs_textColor_T4;
        _webTF.delegate = self;
        NSMutableDictionary *attrs = [NSMutableDictionary dictionary];
        attrs[NSFontAttributeName] = self.fs_standardGeneralFont_GF2;
        attrs[NSForegroundColorAttributeName] = self.fs_textColor_T2;
        NSAttributedString *attStr = [[NSAttributedString alloc] initWithString:FSLocalizedForKey(@"kEnterWebLink") attributes:attrs];
        _webTF.attributedPlaceholder = attStr;
        UIButton *clearButton = [_webTF valueForKey:@"_clearButton"];
        if (clearButton) {
            [clearButton setImage:FSImageNamed(@"edit_link_clear") forState:UIControlStateNormal];
        }
    }
    return _webTF;
}

-(NSInteger)numberOfSectionsInTableView:(UITableView *)tableView{
    return 1;
}

-(NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section{
    return 2;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath{
    return 50.f;
}


-(CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section{
    return 32;
}

-(UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section{
    UIView *headerView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, self.tableView.bounds.size.width, 32)];
    
    headerView.backgroundColor = self.fs_backgroundColor_B2;
    UILabel *headerLabel = [[UILabel alloc] init];
    [headerView addSubview:headerLabel];
    headerLabel.textColor = self.fs_textColor_T3;
    headerLabel.font = [UIFont systemFontOfSize:12];
    headerLabel.text = FSLocalizedForKey(@"kLinkHeader");
    [headerLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(headerView).offset(16);
        make.top.equalTo(headerView).offset(8);
        make.right.bottom.equalTo(headerView);
    }];
    
    return headerView;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath{
    static NSString *cellidentifer = @"CellIdentifer";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellidentifer];
    if (cell == nil) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellidentifer];
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
        cell.backgroundColor = self.fs_backgroundColor_B1;
        cell.contentView.backgroundColor = cell.backgroundColor;
        CGSize gotoPageSize = [Utility getTextSize:FSLocalizedForKey(@"kGoToPage") fontSize:15 maxSize:CGSizeMake(200, 22)];
        CGSize openWebLinkSize = [Utility getTextSize:FSLocalizedForKey(@"kOpenWebLink") fontSize:15 maxSize:CGSizeMake(200, 22)];
        CGFloat offsetWidth = openWebLinkSize.width - gotoPageSize.width;
        
        if (indexPath.row == 0) {
            [cell.contentView addSubview:self.pageCheckBtn];
            [cell.contentView addSubview:self.pageTF];
            [self.pageTF becomeFirstResponder];
            
            UILabel *noticeLable = [[UILabel alloc] init];
            noticeLable.font = self.fs_standardGeneralFont_GF2;
            noticeLable.textColor = self.fs_textColor_T4;
            noticeLable.text = FSLocalizedForKey(@"kGoToPage");
            [cell.contentView addSubview:noticeLable];
            self.pageNoticeLable = noticeLable;
            
            UIButton *thumbNailBtn = [UIButton buttonWithType:UIButtonTypeCustom];
            [thumbNailBtn setImage:FSImageNamedAndPrimaryColor(@"eidt_link_thumb_nail") forState:UIControlStateNormal];
            [BindThemeKeySel(_webCheckBtn, setImage:forState:) fs_updateThemeCompleted:^(FSItem *  _Nonnull target, SEL  _Nonnull sel) {
          
                if (target.currentImage && target.state == UIControlStateNormal) {
                    [target setImage:FSImageNamedAndPrimaryColor(@"eidt_link_thumb_nail") forState:UIControlStateNormal];
                }
            }];
            [thumbNailBtn addTarget:self action:@selector(thumbNailBtnPress) forControlEvents:UIControlEventTouchUpInside];
            [cell.contentView addSubview:thumbNailBtn];
            
            [self.pageCheckBtn mas_makeConstraints:^(MASConstraintMaker *make) {
                make.left.mas_equalTo(8);
                make.size.mas_equalTo(CGSizeMake(40, 40));
                make.centerY.equalTo(self.pageCheckBtn.superview);
            }];
            [noticeLable mas_makeConstraints:^(MASConstraintMaker *make) {
                make.left.equalTo(self.pageCheckBtn.mas_right).offset(8);
                make.height.mas_equalTo(22);
                if (self.view.fs_width > 320) {
                    make.width.mas_equalTo(gotoPageSize.width);
                }else{
                    make.width.mas_equalTo(gotoPageSize.width).priorityHigh();
                    make.width.mas_lessThanOrEqualTo(130);
                }
                make.centerY.equalTo(noticeLable.superview);
            }];
            
            [thumbNailBtn mas_makeConstraints:^(MASConstraintMaker *make) {
                make.right.mas_equalTo(-4);
                make.centerY.equalTo(thumbNailBtn.superview);
                make.size.mas_equalTo(CGSizeMake(40, 40));
            }];
            
            [self.pageTF mas_makeConstraints:^(MASConstraintMaker *make) {
                make.left.equalTo(noticeLable.mas_right).offset(16+ (offsetWidth > 0 ? offsetWidth : 0));
                make.right.equalTo(thumbNailBtn.mas_left).offset(-10);
                make.top.bottom.fs_equalToSuperview();
                make.centerY.equalTo(self.pageTF.superview);
            }];
        }else if (indexPath.row == 1){
            [cell.contentView addSubview:self.webCheckBtn];
            [cell.contentView addSubview:self.webTF];
            
            UILabel *noticeLable = [[UILabel alloc] init];
            noticeLable.font = self.fs_standardGeneralFont_GF2;
            noticeLable.textColor = self.fs_textColor_T4;
            noticeLable.text = FSLocalizedForKey(@"kOpenWebLink");
            [cell.contentView addSubview:noticeLable];
            

            [self.webCheckBtn mas_makeConstraints:^(MASConstraintMaker *make) {
                make.left.mas_equalTo(8);
                make.size.mas_equalTo(CGSizeMake(40, 40));
                make.centerY.equalTo(self.webCheckBtn.superview);
            }];
            
            [noticeLable mas_makeConstraints:^(MASConstraintMaker *make) {
                make.left.equalTo(self.webCheckBtn.mas_right).offset(8);
                make.height.mas_equalTo(22);
                make.width.mas_equalTo(openWebLinkSize.width);
                make.centerY.equalTo(noticeLable.superview);
            }];

            [self.webTF mas_makeConstraints:^(MASConstraintMaker *make) {
                make.left.equalTo(noticeLable.mas_right).offset(16+(offsetWidth > 0 ? 0 : -1*offsetWidth));
                make.right.equalTo(self.webTF.superview).offset(-10);
                make.top.bottom.fs_equalToSuperview();
                make.centerY.equalTo(self.webTF.superview);
            }];
        }
    }else{
        if (indexPath.row == 0) {
            CGSize gotoPageSize = [Utility getTextSize:FSLocalizedForKey(@"kGoToPage") fontSize:15 maxSize:CGSizeMake(200, 22)];
            [self.pageNoticeLable mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.left.equalTo(self.pageCheckBtn.mas_right).offset(8);
                make.height.mas_equalTo(22);
                if (self.view.fs_width > 320) {
                    make.width.mas_equalTo(gotoPageSize.width);
                }else{
                    make.width.mas_equalTo(gotoPageSize.width).priorityHigh();
                    make.width.mas_lessThanOrEqualTo(130);
                }
                make.centerY.equalTo(self.pageNoticeLable.superview);
            }];
        }
    }

    return cell;
}

- (void)viewWillTransitionToSize:(CGSize)size withTransitionCoordinator:(id<UIViewControllerTransitionCoordinator>)coordinator{
    [super viewWillTransitionToSize:size withTransitionCoordinator:coordinator];
    [coordinator animateAlongsideTransition:^(id<UIViewControllerTransitionCoordinatorContext>  _Nonnull context) {

    } completion:^(id<UIViewControllerTransitionCoordinatorContext>  _Nonnull context) {
        [self reloadData];
    }];
}

- (void)thumbNailBtnPress{
    FSLinkThumbnailViewController *linkThumbnailVC = [[FSLinkThumbnailViewController alloc] initWithServices:self.extensionsMgr.viewCtrlManager];
    linkThumbnailVC.startPageIndex = [self.pageTF.text intValue] -1;
    FSNavigationController *linkNavCtrl = [[FSNavigationController alloc] initWithRootViewController:linkThumbnailVC];
    if (DEVICE_iPHONE) {
        linkNavCtrl.modalPresentationStyle = UIModalPresentationFullScreen;
    }else{
        linkNavCtrl.modalPresentationStyle = UIModalPresentationFormSheet;
    }
    linkThumbnailVC.doneBlock = ^(int pageIndex) {
        
        dispatch_async(dispatch_get_main_queue(), ^{
            if (self.pageTF) {
                self.pageTF.text = [NSString stringWithFormat:@"%d",pageIndex+1];
                if ([self.pageTF canBecomeFirstResponder]) {
                    [self.pageTF becomeFirstResponder];
                    self.navBar.rightItem.enabled = YES;
                }
            }
        });

    };
    [self presentViewController:linkNavCtrl animated:YES completion:nil];
}

- (void)textFieldDidBeginEditing:(UITextField *)textField{
    if (textField == self.pageTF) {
        self.selectRow = 0;
        self.navBar.rightItem.enabled = self.pageTF.text.length > 0;
        
    }else{
        self.selectRow = 1;
        self.navBar.rightItem.enabled = self.webTF.text.length > 0;
        
    }

    if (self.selectRow == 0) {
        self.pageCheckBtn.hidden = NO;
        self.webCheckBtn.hidden = YES;
    }else if (self.selectRow == 1){
        self.pageCheckBtn.hidden = YES;
        self.webCheckBtn.hidden = NO;
    }
}

- (BOOL)validateNumber:(NSString*)number {
    BOOL res = YES;
    NSCharacterSet* tmpSet = [NSCharacterSet characterSetWithCharactersInString:@"0123456789"];
    int i = 0;
    while (i < number.length) {
        NSString * string = [number substringWithRange:NSMakeRange(i, 1)];
        NSRange range = [string rangeOfCharacterFromSet:tmpSet];
        if (range.length == 0) {
            res = NO;
            break;
        }
        i++;
    }
    return res;
}

- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string{
    
    NSString *textStr = [textField.text stringByReplacingCharactersInRange:range withString:string];
    
    
    if (self.selectRow == 0) {
        if (![self validateNumber:string]) {
            return NO;
        }
        self.navBar.rightItem.enabled = textStr.length > 0;
    }else if (self.selectRow == 1){
        self.navBar.rightItem.enabled = [textStr stringByTrim].length > 0;
    }
    
    return YES;
}

- (BOOL)textFieldShouldClear:(UITextField *)textField{

    self.navBar.rightItem.enabled = NO;

    return YES;
}


-(BOOL)emptyDataSetShouldDisplay:(UITableView *)tableView{
    return NO;
}


- (void)donePress{

    NSString *indexStr = self.pageTF.text;
    NSString *webAddrStr = [self.webTF.text stringByTrim];
    int pageIndex = 0;
    BOOL bPageValid = YES;
    BOOL bWebValid = YES;
    
    int allCount = self.extensionsMgr.pdfViewCtrl.getPageCount;
    if (!indexStr || indexStr.length <= 0 ) {
        bPageValid = NO;
    }else{
        pageIndex = [indexStr intValue];
        if (pageIndex<0 || pageIndex>allCount) {
            bPageValid = NO;
        }
    }

    if (!webAddrStr || webAddrStr.length <= 0) {
        webAddrStr = @"";
        bWebValid = NO;
    }
    
    if (self.selectRow == 0) {
        if (!bPageValid) {
            [UIAlertController fs_alert_OK_WithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kInvalidPageCount") actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                self.pageTF.text = [NSString stringWithFormat:@"%d",self.extensionsMgr.pdfViewCtrl.getCurrentPage+1];
                [self.pageTF becomeFirstResponder];
            } onCtrl:[_extensionsMgr.pdfViewCtrl fs_currentSceneTopMostViewController]];
            
            return;
        }
    }else{
        if (!bWebValid) {
            [UIAlertController fs_alert_OK_WithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kInvalidWebAddr") actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                self.webTF.text = [self.webTF.text stringByTrim];
                self.navBar.rightItem.enabled = NO;
                [self.webTF becomeFirstResponder];
            } onCtrl:[_extensionsMgr.pdfViewCtrl fs_currentSceneTopMostViewController]];
            return;
        }
    }
    NSMutableDictionary *dict = [NSMutableDictionary dictionary];
    [dict setObject:[NSNumber numberWithInt:pageIndex] forKey:kLinkPageIndex];
    [dict setObject:[NSNumber numberWithInt:self.selectRow] forKey:kLinkSelectRow];
    [dict setObject:webAddrStr forKey:kLinkWebAddr];
    
    if (self.doneActionBlock) {
        self.doneActionBlock(dict);
    }
    
    [self dismissViewControllerAnimated:YES completion:^{

    }];
}

- (void)navigationBarBackTouch{
    
    if (self.cancelActionBlock) {
        self.cancelActionBlock();
    }
    
    [self dismissViewControllerAnimated:YES completion:^{

    }];
}

@end
