/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "LinkAnnotHandler.h"
#import "LinkToolHandler.h"
#import "FSLinkEditViewController.h"
#import "FSPermissionUtilty.h"

static NSString *LINK_DES_TYPE = @"DesType";
static NSString *LINK_DES_INDEX = @"DesIndex";
static NSString *LINK_DES_RECT = @"DesRect";
static NSString *LINK_DES_URL = @"DesURL";
static NSString *LINK_DES_AREA = @"DesArea";
static NSString *LINK_DES_FILE = @"DesFile";

@interface LinkAnnotHandler ()

@property (nonatomic, strong) NSMutableDictionary *dictAnnotLink;
//@property (nonatomic, strong) NSDictionary *jumpDict;
@property(nonatomic, strong) NSMutableArray<UILabel*> *labelArray;
@property (nonatomic, strong) FSAnnot *annot;

- (void)reloadAnnotLink:(FSPDFPage *)dmpage;
- (void)loadAnnotLink:(FSPDFPage *)dmpage;

@end

@implementation LinkAnnotHandler
@synthesize annot = _annot;
- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    if (self = [super initWithUIExtensionsManager:extensionsManager]) {
        _dictAnnotLink = [[NSMutableDictionary alloc] init];
        _labelArray = [NSMutableArray array];
    }
    return self;
}

- (FSAnnotType)getType {
    return FSAnnotLink;
}

- (BOOL)shouldDrawAnnot:(FSAnnot *)annot inPDFViewCtrl:(FSPDFViewCtrl *)pdfViewCtrl {
    return YES;
}

- (BOOL)onPageViewShouldBegin:(int)pageIndex recognizer:(UIGestureRecognizer *)gestureRecognizer annot:(FSAnnot *)annot {
    if (!self.extensionsManager.enableLinks) {
        return NO;
    }
    if (![gestureRecognizer isKindOfClass:[UITapGestureRecognizer class]]) {
        return NO;
    }
    CGPoint point = [gestureRecognizer locationInView:[self.pdfViewCtrl getPageView:pageIndex]];
    __block BOOL ret = NO;
    id linkArray = nil;
    @synchronized(_dictAnnotLink) {
        [self reloadAnnotLink:[self.pdfViewCtrl.currentDoc getPage:pageIndex]];
        linkArray = [_dictAnnotLink objectForKey:@(pageIndex)];
    }
    if (linkArray && linkArray != [NSNull null]) {
        [linkArray enumerateObjectsWithOptions:NSEnumerationReverse
                                    usingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
                                        NSDictionary *linkDict = obj;
                                        NSArray *desAreaArray = [linkDict objectForKey:LINK_DES_AREA];
                                        [desAreaArray enumerateObjectsUsingBlock:^(id obj2, NSUInteger idx2, BOOL *stop2) {
                                            NSArray *pointsArray = obj2;
                                            FSPointF *dibPoint = [self.pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
                                            CGPoint point = CGPointMake(dibPoint.x, dibPoint.y);
                                            if ([Utility isPointInPolygon:point polygonPoints:pointsArray]) {
                                                [self.pdfViewCtrl refresh:CGRectZero pageIndex:pageIndex];
                                                ret = YES;
                                                *stop2 = YES;
                                                *stop = YES;
                                            }
                                        }];
                                    }];
    }
    return ret;
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer annot:(FSAnnot *)annot {
    if (!self.extensionsManager.enableLinks) {
        return NO;
    }
    
    FSPDFPage *curPage = [self.extensionsManager.pdfViewCtrl.currentDoc getPage:pageIndex];
    if (!curPage || [curPage isEmpty]) {
        return NO;
    }
    
    CGPoint point = [recognizer locationInView:[self.pdfViewCtrl getPageView:annot.pageIndex]];
    FSPointF *pdfPoint = [self.pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:annot.pageIndex];
    if (self.extensionsManager.currentAnnot && [Utility isAnnot:self.extensionsManager.currentAnnot uniqueToAnnot:annot]) {
        if (!(pageIndex == annot.pageIndex && [self isHitAnnot:annot point:pdfPoint])) {
            [self.extensionsManager setCurrentAnnot:nil];
        } else if (self.extensionsManager.shouldShowMenu && !self.extensionsManager.shouldShowPropertyBar) {
            [self showMenu];
        }
        
        return YES;
    }
    
    if (self.extensionsManager.currentToolHandler && [self.extensionsManager.currentToolHandler isKindOfClass:[LinkToolHandler class]]) {
        point = [recognizer locationInView:[self.pdfViewCtrl getPageView:pageIndex]];
        FSPointF *pdfPoint = [self.pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
        if (self.extensionsManager.currentAnnot) {
            if ((pageIndex != annot.pageIndex || ![self isHitAnnot:annot point:pdfPoint]) && recognizer.state ==  UIGestureRecognizerStateEnded) {
                [self.extensionsManager setCurrentAnnot:nil];
            }
        } else {
            [self.extensionsManager setCurrentAnnot:annot];
            #if _MAC_CATALYST_
                recognizer.enabled = NO;
                recognizer.enabled = YES;
            #endif
        }
        return YES;
    }else{
        if (self.extensionsManager.config.disableLink) {
            return YES;
        }
        
        point = [recognizer locationInView:[self.pdfViewCtrl getPageView:pageIndex]];
        __block BOOL ret = NO;
        id linkArray = nil;
        @synchronized(_dictAnnotLink) {
            [self reloadAnnotLink:[self.pdfViewCtrl.currentDoc getPage:pageIndex]];
            linkArray = [_dictAnnotLink objectForKey:@(pageIndex)];
        }
        if (linkArray && linkArray != [NSNull null]) {
            [linkArray enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
                NSDictionary *linkDict = obj;
                NSArray *desAreaArray = [linkDict objectForKey:LINK_DES_AREA];
                [desAreaArray enumerateObjectsUsingBlock:^(id obj2, NSUInteger idx2, BOOL *stop2) {
                    NSArray *pointsArray = obj2;
                    FSPointF *dibPoint = [self.pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
                    CGPoint point = CGPointMake(dibPoint.x, dibPoint.y);
                    if ([Utility isPointInPolygon:point polygonPoints:pointsArray]) {
                        [self.pdfViewCtrl refresh:CGRectZero pageIndex:pageIndex];

                        BOOL isCustomLink = [self.extensionsManager onLinkOpen:linkDict LocationInfo:point];
                        if (!isCustomLink){
                            int type = [[linkDict objectForKey:LINK_DES_TYPE] intValue];
                            if (type == FSActionTypeGoto) {
                                [self jumpToPageWithDict:linkDict];
                            }
                            if (type == FSActionTypeURI) {
                                NSString *urlStr = [linkDict objectForKey:LINK_DES_URL];
                                NSURL *URL = [NSURL URLWithString:[urlStr stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLQueryAllowedCharacterSet]]];
                                NSString *scheme = URL.scheme;
                                if (!scheme || scheme.length <= 0) {
                                    NSString *urlStr = [URL absoluteString];
                                    if (urlStr) {
                                        urlStr = [@"http://" stringByAppendingString:urlStr];
                                    }
                                    URL = [NSURL URLWithString:urlStr];
                                }
                                
                                if (!URL) {
                                    [UIAlertController fs_alert_OK_WithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kInvalidLinkUrl") onCtrl:[self.pdfViewCtrl fs_currentSceneTopMostViewController]];
                                    ret = YES;
                                    return;
                                }
                                
                                BOOL isExit = [[UIApplication sharedApplication] canOpenURL:URL];
                                if (isExit) {
                                    [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kSecurityWarning") message:[NSString stringWithFormat:@"%@%@",FSLocalizedForKey(@"kLinkingToWeb"),urlStr] actions:@[FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                                        if (index == 1) {
                                            [[UIApplication sharedApplication] openURL:URL options:@{} completionHandler:nil];
                                        }
                                    } onCtrl:[self.pdfViewCtrl fs_currentSceneTopMostViewController]];
                                    
                                } else if (scheme && scheme.length > 0) {
                                    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:nil message:FSLocalizedForKey(@"kNoAppropriateApplication") preferredStyle:UIAlertControllerStyleAlert];
                                    UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleCancel handler:nil];
                                    [alertController addAction:action];
                                    [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
                                } else {
                                    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:nil message:FSLocalizedForKey(@"kInvalidUrl") preferredStyle:UIAlertControllerStyleAlert];
                                    UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleCancel handler:nil];
                                    [alertController addAction:action];
                                    [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
                                }
                            }
                            if (type == FSActionTypeLaunch) {
                                FSFileSpec *file = [linkDict objectForKey:LINK_DES_FILE];
                                if (![file isEmpty]) {
                                    NSString *fileName = [file getFileName];
                                    fileName = [fileName stringByReplacingOccurrencesOfString:@":" withString:@"/"];
                                    NSString *path = [NSString stringWithFormat:@"%@%@%@", [self.extensionsManager.pdfViewCtrl.filePath stringByDeletingLastPathComponent], @"/", fileName];
                                    NSFileManager *fileManager = [NSFileManager defaultManager];
                                    if (!path || path == nil || ![fileManager fileExistsAtPath:path]) {
                                        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:nil message:FSLocalizedForKey(@"kUnfoundOrCannotOpen") preferredStyle:UIAlertControllerStyleAlert];
                                        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleCancel handler:nil];
                                        [alertController addAction:action];
                                        [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
                                        ret = YES;
                                        return;
                                    }
                                    
                                    NSURL *URL = [[NSURL alloc] initFileURLWithPath:path];
                                    NSString *ext = [path pathExtension];
                                    if ([ext isEqualToString:@"pdf"]) {
                                        [self setDocWithPath:path dict:linkDict];
                                    } else {
                                        BOOL success = [[UIApplication sharedApplication] openURL:URL];
                                        if (!success) {
                                            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:nil message:FSLocalizedForKey(@"kNoAppropriateApplication") preferredStyle:UIAlertControllerStyleAlert];
                                            UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleCancel handler:nil];
                                            [alertController addAction:action];
                                            [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
                                        }
                                    }
                                }
                            }
                            if (type == FSActionTypeGoToR) {
                                FSFileSpec *file = [linkDict objectForKey:LINK_DES_FILE];
                                if (![file isEmpty]) {
                                    NSString *fileName = [file getFileName];
                                    fileName = [fileName stringByReplacingOccurrencesOfString:@":" withString:@"/"];
                                    NSString *path = [NSString stringWithFormat:@"%@%@%@", [self.extensionsManager.pdfViewCtrl.filePath stringByDeletingLastPathComponent], @"/", fileName];
                                    NSFileManager *fileManager = [NSFileManager defaultManager];
                                    if (!path || path == nil || ![fileManager fileExistsAtPath:path]) {
                                        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:nil message:FSLocalizedForKey(@"kUnfoundOrCannotOpen") preferredStyle:UIAlertControllerStyleAlert];
                                        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleCancel handler:nil];
                                        [alertController addAction:action];
                                        [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
                                        ret = YES;
                                        return;
                                    }

                                    [self setDocWithPath:path dict:linkDict];
                                }
                            }
                        }
                        ret = YES;
                        *stop2 = YES;
                        *stop = YES;
                    }
                }];
            }];
        }
        return ret;
    }
    
    return NO;
}

- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context annot:(FSAnnot *)annot {
    [self onRealPageViewDraw:[self.pdfViewCtrl.currentDoc getPage:pageIndex] inContext:context pageIndex:pageIndex];
    [super onDraw:pageIndex inContext:context annot:annot];
}

- (BOOL)onPageViewLongPress:(int)pageIndex recognizer:(nonnull UILongPressGestureRecognizer *)recognizer annot:(FSAnnot *_Nullable)annot {
   
    return NO;
}

//draw link
- (void)onRealPageViewDraw:(FSPDFPage *)page inContext:(CGContextRef)context pageIndex:(int)pageIndex {

    if (!self.extensionsManager.enableHighlightLinks)
        return;
    UIView *pageView = [[self pdfViewCtrl] getPageView:pageIndex];
    if (!pageView) return;
    
    for (UILabel *label in self.labelArray) {
        [label removeFromSuperview];
    }
    [self.labelArray removeAllObjects];
    BOOL needShowText = NO;
    if (self.extensionsManager.currentToolHandler &&
        [self.extensionsManager.currentToolHandler isKindOfClass:[LinkToolHandler class]] &&
        self.extensionsManager.pdfViewCtrl.getCurrentPage == [page getIndex]) {
        needShowText = YES;
    }
    
    [self reloadAnnotLink:page];

    NSArray *array = nil;
    @try {
        array = [_dictAnnotLink objectForKey:[NSNumber numberWithInt:[page getIndex]]];
    } @catch (NSException *exception) {
    }
   
    if (array && ((id) array != [NSNull null])) {

        CGFloat red, green, blue, alpha;
        [self.extensionsManager.linksHighlightColor getRed:&red green:&green blue:&blue alpha:&alpha];
        CGContextSetRGBFillColor(context, red, green, blue, alpha);
        [array enumerateObjectsWithOptions:NSEnumerationReverse
                                usingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
                                    NSDictionary *linkDict = obj;
                                    NSArray *desAreaArray = [linkDict objectForKey:LINK_DES_AREA];
                                    int jumpType = [[linkDict objectForKey:LINK_DES_TYPE] intValue];
                                    int jumpIndex = [[linkDict objectForKey:LINK_DES_INDEX] intValue];
                                    NSString* jumpStr = [linkDict objectForKey:LINK_DES_URL];
                                    [desAreaArray enumerateObjectsUsingBlock:^(id obj2, NSUInteger idx2, BOOL *stop2) {
                                        NSArray *pointArray = obj2;
                                        __block CGRect linkRect = CGRectZero;
                                        CGContextBeginPath(context);
                                        __block CGFloat minX = 0;
                                        __block CGFloat minY = 0;
                                        __block CGFloat maxX = 0;
                                        __block CGFloat maxY = 0;
                                        [pointArray enumerateObjectsUsingBlock:^(id obj3, NSUInteger idx3, BOOL *stop3) {
                                            CGPoint p = [obj3 CGPointValue];
                                            FSPointF *p2 = [[FSPointF alloc] init];
                                            p2.x = p.x;
                                            p2.y = p.y;
                                            CGPoint pp = [self.pdfViewCtrl convertPdfPtToPageViewPt:p2 pageIndex:[page getIndex]];
                                            p.x = pp.x;
                                            p.y = pp.y;

                                            p.x = p.x > pageView.fs_width ? pageView.fs_width : p.x;
                                            p.x = p.x < 0 ? 0 : p.x;
                                            
                                            p.y = p.y > pageView.fs_height ? pageView.fs_height : p.y;
                                            p.y = p.y < 0 ? 0 : p.y;
                                            
                                            if (idx3 == 0) {
                                                CGContextMoveToPoint(context, p.x, p.y);
                                                minX = p.x;
                                                minY = p.y;
                                            } else {
                                                CGContextAddLineToPoint(context, p.x, p.y);
                                            }
                                            if (minX > p.x) {
                                                minX = p.x;
                                            }
                                            if (maxX < p.x) {
                                                maxX = p.x;
                                            }
                                            if (minY > p.y) {
                                                minY = p.y;
                                            }
                                            if (maxY < p.y) {
                                                maxY = p.y;
                                            }
                                        }];
                                        CGContextClosePath(context);
                                        CGContextSetRGBFillColor(context, red, green, blue, alpha);
                                        CGContextFillPath(context);
                                        if (needShowText) {
                                            linkRect = CGRectMake(minX, minY, maxX-minX, maxY-minY);
                                            NSString *str = @"";
                                            if (jumpType == FSActionTypeGoto) {
                                                str = [NSString stringWithFormat:@"%@ %d",FSLocalizedForKey(@"kToPage"),jumpIndex+1];
                                            }else if (jumpType == FSActionTypeURI) {
                                                str = [NSString stringWithFormat:@"%@%@",FSLocalizedForKey(@"kLinkTo"),jumpStr?jumpStr:@""];
                                            }
                                            [self toDrawTextWithRect:linkRect str:str context:context];
                                        }
                                         
                                    }];
                                }];
    }
}

- (void)toDrawTextWithRect:(CGRect)rect str:(NSString*)str context:(CGContextRef)context{
    if( str == nil || context == nil)
        return;
    
    NSMutableParagraphStyle *textStyle = [[NSMutableParagraphStyle defaultParagraphStyle] mutableCopy];
//    textStyle.lineBreakMode = NSLineBreakByWordWrapping;
    textStyle.alignment = NSTextAlignmentCenter;
    
    float fontSize = 14.0f;
    CGRect textrect = CGRectZero;
    NSDictionary *attributes = nil;
    
    while (fontSize > 6) {

        UIFont  *font = [UIFont systemFontOfSize:fontSize];
        attributes = @{NSFontAttributeName:font,NSForegroundColorAttributeName:self.fs_textColor_T2,NSParagraphStyleAttributeName:textStyle,NSBackgroundColorAttributeName:[UIColor clearColor]};
        textrect = [str boundingRectWithSize:CGSizeMake(rect.size.width, MAXFLOAT) options:NSStringDrawingUsesLineFragmentOrigin attributes:attributes context:nil];
        
        if (textrect.size.height > rect.size.height) {
            fontSize--;
        }else{
            break;
        }
        
    }
    if (fontSize > 6) {
        CGFloat marginTop = (rect.size.height - textrect.size.height)/2;
        if (marginTop < 0) marginTop = 0;
        if (textrect.size.height > rect.size.height) textrect.size.height = rect.size.height;

        CGRect r = CGRectMake(rect.origin.x, rect.origin.y + marginTop,rect.size.width, textrect.size.height);
        [str drawInRect:r withAttributes:attributes];
    }
 
}

#pragma mark private

//- (void)setJumpDocWithPath:(NSString *)path dict:(NSDictionary *)dict {
//    FSPDFDoc *doc = [[FSPDFDoc alloc] initWithPath:path];
//    __block FSErrorCode status = [doc load:nil];
//    if (status == FSErrPassword) {
//        AlertView *alert = [[AlertView alloc] initWithTitle:nil
//                                                    message:@"kDocNeedPassword"
//                                         buttonClickHandler:^(AlertView *alertView, NSInteger buttonIndex) {
//                                             if (buttonIndex == 0) {
//                                                 return;
//                                             } else if (buttonIndex == 1) {
//                                                 NSString *password = [alertView textFieldAtIndex:1].text;
//                                                 status = [doc load:password];
//                                                 if (status == FSErrSuccess) {
//                                                     [self.extensionsManager.pdfViewCtrl openDoc:path password:password completion:nil];
//                                                     _jumpDict = [[NSDictionary alloc] initWithDictionary:dict];
//                                                 }
//                                             }
//                                         }
//                                          cancelButtonTitle:@"kCancel"
//                                          otherButtonTitles:@"kOK", nil];
//        [alert show];
//    } else if (status == FSErrSuccess) {
//        [self.extensionsManager.pdfViewCtrl openDoc:path password:nil completion:nil];
//        _jumpDict = [[NSDictionary alloc] initWithDictionary:dict];
//    }
//}

- (void)setDocWithPath:(NSString *)path dict:(NSDictionary *)dict {
    if (path == nil) {
        return;
    }
    
    __weak typeof(self) weakSelf = self;
    void (^jumpblock)(BOOL) = ^(BOOL isNeedSave){
        if (self.extensionsManager.isMultiFileMode) {
            BOOL isOpen = [weakSelf.extensionsManager.delegate uiextensionsManager:weakSelf.extensionsManager openNewDocAtPath:path shouldCloseCurrentDoc:NO completionCallBack:^{}];
            if (isOpen) {
                [weakSelf jumpToPageWithDict:dict];
            }
            return ;
        }
        if (isNeedSave) {
            [weakSelf.extensionsManager saveAndCloseCurrentDoc:^(BOOL success) {
                if (success) {
                    __strong typeof(weakSelf) strongSelf = weakSelf;
                    strongSelf.extensionsManager.isDocModified = NO;
                    [strongSelf.pdfViewCtrl openDoc:path
                                           password:nil
                                         completion:^(FSErrorCode error) {
                                             [weakSelf jumpToPageWithDict:dict];
                                         }];
                }
            }];
        }else{
            weakSelf.extensionsManager.isDocModified = NO;
            __strong typeof(weakSelf) strongSelf = weakSelf;
            [strongSelf.pdfViewCtrl openDoc:path
                                   password:nil
                                 completion:^(FSErrorCode error) {
                                     
                                     [weakSelf jumpToPageWithDict:dict];
                                 }];
        }
    };
    
    if (self.extensionsManager.isDocModified && !self.extensionsManager.isMultiFileMode) {
        
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:nil message:FSLocalizedForKey(@"kShouldSaveFileTips") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *cancelaction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel") style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {
            jumpblock(NO);
        }];
        
        UIAlertAction *okaction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
            jumpblock(YES);
        }];
        
        [alertController addAction:cancelaction];
        [alertController addAction:okaction];
        [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
    }else{
        jumpblock(NO);
    }
}

- (void)jumpToPageWithDict:(NSDictionary *)dict {
    int jumpIndex = [[dict objectForKey:LINK_DES_INDEX] intValue];
    if (jumpIndex >= 0 && jumpIndex < [self.extensionsManager.pdfViewCtrl.currentDoc getPageCount]) {
        //prevent sometimes it's faster than return YES
        double delayInSeconds = 0.1;
        dispatch_time_t popTime = dispatch_time(DISPATCH_TIME_NOW, delayInSeconds * NSEC_PER_SEC);
        dispatch_after(popTime, dispatch_get_main_queue(), ^(void) {
            FSPointF *point = [[FSPointF alloc] init];
            point.x = 0;
            if (dict[LINK_DES_RECT]) {
                FSRectF *desDibRect = [Utility CGRect2FSRectF:[dict[LINK_DES_RECT] CGRectValue]];
                point.y = desDibRect.top;
            } else {
                FSPDFPage *page = nil;
                @try {
                    page = [self.extensionsManager.pdfViewCtrl.currentDoc getPage:jumpIndex];
                } @catch (NSException *exception) {
                }
                point.y = page ? [page getHeight] : 0;
            }
            [self.pdfViewCtrl gotoPage:jumpIndex withDocPoint:point animated:YES];
        });
    }
}

#pragma mark link
- (void)loadAnnotLink:(FSPDFPage *)dmpage {
    NSMutableArray *array = nil;
    @synchronized(_dictAnnotLink) {
        @try {
            array = [_dictAnnotLink objectForKey:[NSNumber numberWithInt:[dmpage getIndex]]];
        } @catch (NSException *exception) {
        }
    }
    if (!array) {
        if (!dmpage || [dmpage isEmpty]) {
            return;
        }
        int linkCount = 0;
        @try {
            linkCount = [dmpage getAnnotCount];
            if (linkCount > 0) {
                array = [NSMutableArray array];
            }
        } @catch (NSException *exception) {
            return;
        }
        
        for (int i = 0; i < linkCount; i++) {
            FSAnnot *annot = [dmpage getAnnot:i];
            if (!annot || [annot isEmpty]) {
                continue;
            }

            if (FSAnnotLink != [annot getType])
                continue;
            FSLink *link = [[FSLink alloc] initWithAnnot:annot];
            FSAction *action = nil;
            @try {
                action = [link getAction];
            } @catch (NSException *exception) {
                NSLog(@"%@", exception.description);
            }
            while (action && ![action isEmpty]) {
                NSMutableDictionary *dict = [NSMutableDictionary dictionary];

                BOOL support = NO;
                if ([action getType] == FSActionTypeGoto) {
                    FSGotoAction *gotoAction = [[FSGotoAction alloc] initWithAction:action];
                    FSDestination *dest = [gotoAction getDestination];
                    if (dest && ![dest isEmpty]) {
                        FSRectF *rect = [[FSRectF alloc] init];
                        rect.left = [dest getLeft];
                        rect.bottom = [dest getBottom];
                        rect.top = [dest getTop];
                        rect.right = [dest getRight];
                        int desIndex = [dest getPageIndex:[dmpage getDocument]];

                        CGRect desRect = [Utility FSRectF2CGRect:rect];
                        [dict setValue:@(FSActionTypeGoto) forKey:LINK_DES_TYPE];
                        [dict setValue:@(desIndex) forKey:LINK_DES_INDEX];
                        [dict setValue:[NSValue valueWithCGRect:desRect] forKey:LINK_DES_RECT];
                        
                    }else{
                        [dict setValue:@(FSActionTypeGoto) forKey:LINK_DES_TYPE];
                        [dict setValue:@(0) forKey:LINK_DES_INDEX];
                        [dict setValue:[NSValue valueWithCGRect:CGRectZero] forKey:LINK_DES_RECT];
                    }
                    support = YES;
                }
                if ([action getType] == FSActionTypeGoToR) {
                    FSRemoteGotoAction *gotoRAction = [[FSRemoteGotoAction alloc] initWithAction:action];
                    //fileSpec
                    FSFileSpec *file = [gotoRAction getFileSpec];
                    if (![file isEmpty]) {
                        [dict setValue:file forKey:LINK_DES_FILE];
                        support = YES;
                        [dict setValue:@(FSActionTypeGoToR) forKey:LINK_DES_TYPE];
                        FSDestination *dest = [gotoRAction getDestination];
                        if (dest && ![dest isEmpty]) {
                            FSRectF *rect = [[FSRectF alloc] init];
                            rect.left = [dest getLeft];
                            rect.bottom = [dest getBottom];
                            rect.top = [dest getTop];
                            rect.right = [dest getRight];
                            int desIndex = [dest getPageIndex:[dmpage getDocument]];

                            CGRect desRect = [Utility FSRectF2CGRect:rect];
                            [dict setValue:@(desIndex) forKey:LINK_DES_INDEX];
                            [dict setValue:[NSValue valueWithCGRect:desRect] forKey:LINK_DES_RECT];
                        } else {
                            [dict setValue:@0 forKey:LINK_DES_INDEX];
                        }
                    }
                }
                if ([action getType] == FSActionTypeLaunch) {
                    FSLaunchAction *launchAction = [[FSLaunchAction alloc] initWithAction:action];
                    //fileSpec
                    FSFileSpec* file = [launchAction getFileSpec];
                    [dict setValue:file forKey:LINK_DES_FILE];
                    [dict setValue:@(FSActionTypeLaunch) forKey:LINK_DES_TYPE];
                    support = YES;
                }

                if ([action getType] == FSActionTypeURI) {
                    FSURIAction *uriAction = [[FSURIAction alloc] initWithAction:action];
                    NSString *uri = [uriAction getURI];
                    [dict setValue:uri forKey:LINK_DES_URL];
                    [dict setValue:@(FSActionTypeURI) forKey:LINK_DES_TYPE];
                    support = YES;
                }
                if (support) {
                    NSArray *desArea = [self getAnnotationQuad:annot];
                    if (!desArea) {
                        FSRectF *rect = [annot getRect];
    
                        CGPoint point1;
                        point1.x = rect.left;
                        point1.y = rect.bottom;
                        NSValue *value1 = [NSValue valueWithCGPoint:point1];
                        CGPoint point2;
                        point2.x = rect.right;
                        point2.y = rect.bottom;
                        NSValue *value2 = [NSValue valueWithCGPoint:point2];
                        CGPoint point3;
                        point3.x = rect.right;
                        point3.y = rect.top;
                        NSValue *value3 = [NSValue valueWithCGPoint:point3];
                        CGPoint point4;
                        point4.x = rect.left;
                        point4.y = rect.top;
                        NSValue *value4 = [NSValue valueWithCGPoint:point4];
                        NSArray *arrayQuad = [NSArray arrayWithObjects:value1, value2, value3, value4, nil];
                        desArea = [NSArray arrayWithObject:arrayQuad];
                    }

                    if (desArea) {
                        [dict setValue:desArea forKey:LINK_DES_AREA];
                    }
                }
                
                if (dict.count > 0) {
                    [array addObject:dict];
                }
                if ([action getSubActionCount] > 0)
                    action = [action getSubAction:0];
                else
                    action = nil;
            }
        }
        @synchronized(_dictAnnotLink) {
            [_dictAnnotLink setObject:array ? array : [NSNull null] forKey:[NSNumber numberWithInt:[dmpage getIndex]]];
        }
    }
}

- (void)reloadAnnotLink:(FSPDFPage *)dmpage {
    if (!dmpage || [dmpage isEmpty]) {
        return;
    }
    @synchronized(self) {
        @synchronized(_dictAnnotLink) {
            [_dictAnnotLink removeObjectForKey:[NSNumber numberWithInt:[dmpage getIndex]]];
        }
        [self loadAnnotLink:dmpage];
    }
}

- (NSArray *)getAnnotationQuad:(FSAnnot *)annot {
    if ([annot getType] != FSAnnotLink)
        return nil;
    FSLink *link = [[FSLink alloc] initWithAnnot:annot];
    FSQuadPointsArray *quads = [link getQuadPoints];
    unsigned long quadCount = [quads getSize];
    if (quadCount == 0) {
        return nil;
    }

    NSMutableArray *array = [NSMutableArray array];
    for (int i = 0; i < quadCount; i++) {
        FSQuadPoints *quadPoints = [quads getAt:i];
        if (!quadPoints) {
            break;
        }

        CGPoint point1;
        point1.x = [[quadPoints getFirst] getX];
        point1.y = [[quadPoints getFirst] getY];
        NSValue *value1 = [NSValue valueWithCGPoint:point1];
        CGPoint point2;
        point2.x = [[quadPoints getSecond] getX];
        point2.y = [[quadPoints getSecond] getY];
        NSValue *value2 = [NSValue valueWithCGPoint:point2];
        CGPoint point3;
        point3.x = [[quadPoints getThird] getX];
        point3.y = [[quadPoints getThird] getY];
        NSValue *value3 = [NSValue valueWithCGPoint:point3];
        CGPoint point4;
        point4.x = [[quadPoints getFourth] getX];
        point4.y = [[quadPoints getFourth] getY];
        NSValue *value4 = [NSValue valueWithCGPoint:point4];
        NSArray *arrayQuad = [NSArray arrayWithObjects:value1, value2, value3, value4, nil];
        [array addObject:arrayQuad];
    }

    return array;
}

- (void)onAnnotDeselected:(FSAnnot *)annot {
    [self hideMenu];
    if (self.extensionsManager.propertyMainView.isShowing) {
        [self.extensionsManager.propertyMainView dismiss];
    }
    if ([self shouldDrawAnnot:annot inPDFViewCtrl:self.pdfViewCtrl]) {
        [self refreshPageForAnnot:self.annot reRenderPage:NO];
    } else {
        [self refreshPageForAnnot:self.annot reRenderPage:YES];
    }

    FSLinkAttributes *linkAttributes = (FSLinkAttributes*)self.attributesBeforeModify;
    if (linkAttributes && ![linkAttributes isEqualToAttributes:(FSLinkAttributes*)[FSAnnotAttributes attributesWithAnnot:annot] doc:self.extensionsManager.pdfViewCtrl.getDoc]) {
        [self modifyAnnot:annot addUndo:YES];
    }

    self.attributesBeforeModify = nil;
    self.annotImage = nil;
    self.annot = nil;
}

- (FSMenuOptions)menuOptionsForAnnot:(FSAnnot *)annot {
    FSMenuOptions options = FSMenuOptionNone;
    if ([annot canModifyWithExternsions:self.extensionsManager] && [FSPermissionUtilty canAddEditAnnot:self.extensionsManager]) {
        
        options = FSMenuOptionEdit;
        if ([annot canDeleteWithExternsions:self.extensionsManager] && [annot getType] == FSAnnotLink) {
            options |= FSMenuOptionDelete;
        }
    }

    if (![FSPermissionUtilty canAddEditAnnot:self.extensionsManager]) {
        options = FSMenuOptionNone;
    }
    return options;
}

- (void)edit {
    
    if ([self.annot getType] != FSAnnotLink) {
        return;
    }
    
    FSLinkEditViewController *linkEditVC = [[FSLinkEditViewController alloc] init];
    linkEditVC.extensionsMgr = self.extensionsManager;
    linkEditVC.linkAnnot = self.annot;
    linkEditVC.doneActionBlock = ^(NSDictionary * _Nonnull dict) {
        
        if (self.annot && dict) {
            
            FSLink *curAnnot = [[FSLink alloc] initWithAnnot:self.annot];
            
            int pageIndex = [[dict objectForKey:kLinkPageIndex] intValue] - 1;
            NSString *webStr = [dict objectForKey:kLinkWebAddr];
            int jumpType = [[dict objectForKey:kLinkSelectRow] intValue];
            if (pageIndex < 0 || pageIndex >= self.extensionsManager.pdfViewCtrl.getPageCount) {
                pageIndex = self.extensionsManager.pdfViewCtrl.getCurrentPage;
            }
            if (!webStr) {
                webStr = @"";
            }

            // Add action for link annotation
            FSPDFDoc* doc = [[self.annot getPage] getDocument];
            if (jumpType == 0) {
                FSGotoAction* action = [[FSGotoAction alloc] initWithAction:[FSAction create:doc action_type:FSActionTypeGoto]];
                FSDestination* dest = [FSDestination createXYZ:self.extensionsManager.pdfViewCtrl.getDoc page_index:pageIndex left:0 top:0 zoom_factor:1.0f];
                [action setDestination:dest];
                [curAnnot setAction:action];
            }else{
                FSURIAction* action = [[FSURIAction alloc] initWithAction:[FSAction create:doc action_type:FSActionTypeURI]];
                [action setTrackPositionFlag:YES];
                [action setURI:webStr];
                [curAnnot setAction:action];
            }
            
            [self.pdfViewCtrl lockRefresh];
            [self.annot resetAppearanceStream];
            [self.pdfViewCtrl unlockRefresh];
            
        }
        
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
            if (self.extensionsManager.currentAnnot) {
                [self.extensionsManager setCurrentAnnot:nil];
            }
            [self.pdfViewCtrl refresh:self.extensionsManager.pdfViewCtrl.getCurrentPage needRender:YES];
        });
        
    };
    linkEditVC.cancelActionBlock = ^{
        if (self.extensionsManager.currentAnnot) {
            [self.extensionsManager setCurrentAnnot:nil];
        }
        if (self.annot) {
        }
    };
    FSNavigationController *navCtrl = [[FSNavigationController alloc] initWithRootViewController:linkEditVC];
    if (DEVICE_iPHONE) {
        navCtrl.modalPresentationStyle = UIModalPresentationFullScreen;
    }else{
        navCtrl.modalPresentationStyle = UIModalPresentationFormSheet;
    }
    [[self.pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:navCtrl animated:YES completion:nil];
}

#pragma mark IDocEventListener

- (void)onDocOpened:(FSPDFDoc *)document error:(int)error {
    if (error != FSErrSuccess) {
        return;
    }
    //    if(_jumpDict){
    //        [self jumpToPageWithDict:_jumpDict];
    //        _jumpDict = nil;
    //    }
}
- (void)onDocClosed:(FSPDFDoc *)document error:(int)error {
    [self.dictAnnotLink removeAllObjects];
}

#pragma mark IPageEventListener
- (void)onPagesWillRemove:(NSArray<NSNumber *> *)indexes {
}

- (void)onPagesWillMove:(NSArray<NSNumber *> *)indexes dstIndex:(int)dstIndex {
}

- (void)onPagesWillRotate:(NSArray<NSNumber *> *)indexes rotation:(int)rotation {
}

- (void)onPagesRemoved:(NSArray<NSNumber *> *)indexes {
    [self.dictAnnotLink removeAllObjects];
}

- (void)onPagesMoved:(NSArray<NSNumber *> *)indexes dstIndex:(int)dstIndex {
    [self.dictAnnotLink removeAllObjects];
}

- (void)onPagesRotated:(NSArray<NSNumber *> *)indexes rotation:(int)rotation {
}

- (void)onPagesInsertedAtRange:(NSRange)range {
    [self.dictAnnotLink removeAllObjects];
}

@end
