/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "LinkModule.h"
#import "LinkAnnotHandler.h"
#import "LinkToolHandler.h"

@interface LinkModule () <IStateChangeListener, IGestureEventListener, IDocEventListener, IPageEventListener,IToolEventListener> {
    UIExtensionsManager *__weak _extensionsManager;

}

@property(nonatomic, strong) LinkAnnotHandler* annotHandler;
@property(nonatomic, strong) LinkToolHandler* toolHandle;

@end

@implementation LinkModule

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        self.toolHandle = [[LinkToolHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerToolHandler:self.toolHandle];
        self.annotHandler = [[LinkAnnotHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerAnnotHandler:self.annotHandler];
        
        [_extensionsManager.pdfViewCtrl registerDocEventListener:self.annotHandler];
        [_extensionsManager.pdfViewCtrl registerPageEventListener:self.annotHandler];
  
        [_extensionsManager.pdfViewCtrl registerGestureEventListener:self];
        [_extensionsManager.pdfViewCtrl registerDocEventListener:self];
        [_extensionsManager.pdfViewCtrl registerPageEventListener:self];
        [_extensionsManager registerStateChangeListener:self];
        [_extensionsManager registerToolEventListener:self];
        
        [self loadModule];
    }
    return self;
}

- (void)dealloc
{

    if (self.toolHandle) {
        [_extensionsManager registerToolHandler:self.toolHandle];
    }
    if (self.annotHandler) {
        [_extensionsManager registerAnnotHandler:self.annotHandler];
    }
    
    [_extensionsManager.pdfViewCtrl unregisterDocEventListener:self.annotHandler];
    [_extensionsManager.pdfViewCtrl unregisterPageEventListener:self.annotHandler];
    [_extensionsManager.pdfViewCtrl unregisterGestureEventListener:self];
    [_extensionsManager.pdfViewCtrl unregisterDocEventListener:self];
    [_extensionsManager.pdfViewCtrl unregisterPageEventListener:self];
    [_extensionsManager unregisterStateChangeListener:self];
    [_extensionsManager unregisterToolEventListener:self];
}

- (NSString *)getName {
    return @"Link";
}

- (void)loadModule {

    [_extensionsManager.toolbarManager addReadItem:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeLink withItemContentType:FSItemContentTypeImageAndText] atToolTag:FSMainTopbarToolTagEdit];
}

- (void)didClickedReadToolItem:(FSReadToolbarItem *)readToolbarItem {
    @weakify(self)
    [_extensionsManager.permissionProvider checkPermission:FSFunctionEdit checkPermissionState:^(FSPermissionState state) {
        @strongify(self)
        if (state == FSPermissionStateShow) {

            LinkToolHandler *toolHanlder = (LinkToolHandler *) [self->_extensionsManager getToolHandlerByName:Tool_Link];
            [self->_extensionsManager setCurrentToolHandler:toolHanlder];
            toolHanlder.toolSettings = readToolbarItem.toolSettings;

            [self fs_toastWithMsg:FSLocalizedForKey(@"kLinkTapNotice") inView:[self->_extensionsManager.pdfViewCtrl fs_currentSceneTopMostViewController].view];
        }else{
            readToolbarItem.toolbarItem.item.selected = NO;
        }
    }];
}

- (void)onStateChanged:(FSUIManagerState)state {
    
}

- (BOOL)onPan:(UIPanGestureRecognizer *)recognizer {

    CGPoint point = [recognizer locationInView:[_extensionsManager.pdfViewCtrl getDisplayView]];
    int pageIndex = [_extensionsManager.pdfViewCtrl getPageIndex:point];
    if (self.toolHandle != nil) {
        if ([self.toolHandle onPageViewPan:pageIndex recognizer:recognizer]) {
            return YES;
        }
    }

    return NO;
}

- (void)onToolChanged:(nonnull NSString *)lastToolName CurrentToolName:(nonnull NSString *)toolName {
    if ([lastToolName isEqual:Tool_Link] || [toolName isEqual:Tool_Link]) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self->_extensionsManager.pdfViewCtrl refresh:self->_extensionsManager.pdfViewCtrl.getCurrentPage needRender:YES];
        });
    }
}

@end
