/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSCircleProgressView.h"

@interface FSCircleProgressView ()
{
    CAShapeLayer *backGroundLayer;
    CAShapeLayer *frontFillLayer;
    UIBezierPath *backGroundBezierPath;
    UIBezierPath *frontFillBezierPath;
}
@property(nonatomic,strong)UILabel *progressLabel;
@end

@implementation FSCircleProgressView

@synthesize progressColor = _progressColor;
@synthesize progressTrackColor = _progressTrackColor;
@synthesize progressValue = _progressValue;

- (instancetype)initWithCoder:(NSCoder *)aDecoder
{
    if (self = [super initWithCoder:aDecoder]) {
        [self arrangeView];
    }
    return self;
}
- (instancetype)initWithFrame:(CGRect)frame
{
    if (self = [super initWithFrame:frame])
    {
        [self arrangeView];
        
    }
    return self;

}

- (void)arrangeView
{
    self.progressWidth = 5;
    self.progressTrackWidth = 5;
    
    //background layer
    backGroundLayer = [CAShapeLayer layer];
    backGroundLayer.fillColor = nil;
    backGroundLayer.frame = self.bounds;
    
    //fill layer
    frontFillLayer = [CAShapeLayer layer];
    frontFillLayer.fillColor = nil;
    frontFillLayer.frame = self.bounds;
    
    [self.layer addSublayer:backGroundLayer];
    [self.layer addSublayer:frontFillLayer];
    
    [self addSubview:self.progressLabel];
    self.progressLabel.text = [NSString stringWithFormat:@"%d%%",(int)self.progressValue];
    [self.progressLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.bottom.top.equalTo(self);
    }];
}

- (void)layoutSubviews{
    [super layoutSubviews];
    
    backGroundLayer.frame = self.bounds;
    frontFillLayer.frame = self.bounds;
    
    backGroundBezierPath = [UIBezierPath bezierPathWithArcCenter:CGPointMake(self.fs_width/2.0, self.fs_height/2.0) radius:(self.fs_width-self.progressTrackWidth*1.5)/2.f startAngle:0 endAngle:M_PI*2
                                                       clockwise:YES];
    backGroundLayer.path = backGroundBezierPath.CGPath;
}

-(UILabel *)progressLabel{
    if (!_progressLabel) {
        _progressLabel = [[UILabel alloc] init];
        _progressLabel.textAlignment = NSTextAlignmentCenter;
        _progressLabel.font = self.fs_standardWeakFont_WF1;
    }
    return _progressLabel;
}

- (void)setProgressTextFont:(UIFont *)progressTextFont{
    _progressTextFont = progressTextFont;
    
    if (self.progressLabel) {
        self.progressLabel.font = progressTextFont;
    }
}

- (void)setProgressTextColor:(UIColor *)progressTextColor
{
    _progressTextColor = progressTextColor;
    if (self.progressLabel) {
        self.progressLabel.textColor = progressTextColor;
    }
}

- (void)setProgressColor:(UIColor *)progressColor
{
    _progressColor = progressColor;
    frontFillLayer.strokeColor = progressColor.CGColor;
}

- (UIColor *)progressColor
{
    return _progressColor;
}

- (void)setProgressTrackColor:(UIColor *)progressTrackColor
{
    _progressTrackColor = progressTrackColor;
    backGroundLayer.strokeColor = progressTrackColor.CGColor;
    backGroundBezierPath = [UIBezierPath bezierPathWithArcCenter:CGPointMake(self.fs_width/2.0, self.fs_height/2.0) radius:(self.fs_width-4-self.progressTrackWidth)/2.f startAngle:0 endAngle:M_PI*2
                                                       clockwise:YES];
    backGroundLayer.path = backGroundBezierPath.CGPath;
}

- (UIColor *)progressTrackColor
{
    return _progressTrackColor;
}

- (void)setProgressValue:(CGFloat)progressValue
{
    _progressValue = progressValue;
    frontFillBezierPath = [UIBezierPath bezierPathWithArcCenter:CGPointMake(self.fs_width/2.0, self.fs_height/2.0) radius:(CGRectGetWidth(self.bounds)-self.progressWidth)/2.f startAngle:-M_PI_2 endAngle:(2*M_PI)*(progressValue/100.0)-M_PI_2 clockwise:YES];
    frontFillLayer.path = frontFillBezierPath.CGPath;
    self.progressLabel.text = [NSString stringWithFormat:@"%d%%",(int)progressValue];
}

- (CGFloat)progressValue
{
    return _progressValue;
}

- (void)setProgressWidth:(CGFloat)progressWidth
{
    _progressWidth = progressWidth;
    frontFillLayer.lineWidth = progressWidth;
}

- (void)setProgressTrackWidth:(CGFloat)progressTrackWidth
{
    _progressTrackWidth = progressTrackWidth;
    backGroundLayer.lineWidth = progressTrackWidth;
}


@end
