/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSFileInfoHeadView.h"

@interface FSFileInfoHeadView ()

@property(nonatomic , strong)UIView *imageBGView;

@end

@implementation FSFileInfoHeadView

/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
    // Drawing code
}
*/

- (instancetype)initWithFrame:(CGRect)frame{
    if (self = [super initWithFrame:frame])
    {
        self.accessibilityIdentifier = @"fileInfoHeader_id";
    }
    [self arrangeView];
    return self;
}

- (UIView *)imageBGView{
    if (!_imageBGView) {
        _imageBGView = [[UIImageView alloc] init];
    }
    return _imageBGView;
}

- (UIImageView *)imageView{
    if (!_imageView) {
        _imageView = [[UIImageView alloc] init];
        _imageView.contentMode = UIViewContentModeScaleAspectFit;
        _imageView.layer.borderWidth = 1;
        _imageView.layer.borderColor = self.fs_dividerColor_P2.CGColor;
        _imageView.image = FSImageNamed(@"file_list_PDF");
    }
    return _imageView;
}

- (UILabel *)titlLabel{
    if (!_titlLabel) {
        _titlLabel = [[UILabel alloc] init];
        _titlLabel.numberOfLines = 0;
        _titlLabel.textAlignment = NSTextAlignmentCenter;
        _titlLabel.textColor = self.fs_textColor_T4;
        _titlLabel.font = self.fs_standardGeneralFont_GF1;
    }
    return _titlLabel;
}

- (void)setViewConstraint{
    
    [self.imageBGView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(132);
        make.height.mas_equalTo(132);
        make.top.mas_equalTo(16);
        make.centerX.equalTo(self);
    }];
    
    [self.imageView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.mas_equalTo(100);
        make.height.mas_equalTo(132);
        make.center.equalTo(self.imageView.superview);
    }];
    
    [self.titlLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.imageView.mas_bottom).offset(12);
        make.bottom.equalTo(self).offset(-16);
        make.left.equalTo(self).offset(16);
        make.right.equalTo(self).offset(-16);
    }];
}

- (void)arrangeView{
    [self addSubview:self.imageBGView];
    [self.imageBGView addSubview:self.imageView];
//    [self addSubview:self.imageView];
    [self addSubview:self.titlLabel];
    [self setViewConstraint];
}

- (void)remakeImageViewContraints:(BOOL)isVertical{
    [self.imageView mas_remakeConstraints:^(MASConstraintMaker *make) {
        if (isVertical) {
            make.size.mas_equalTo(CGSizeMake(100, 132));
        }else{
            make.size.mas_equalTo(CGSizeMake(132, 100));
        }
        
        make.center.equalTo(self.imageView.superview);
    }];
}

#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
- (void)traitCollectionDidChange:(UITraitCollection *)previousTraitCollection{
    [super traitCollectionDidChange:previousTraitCollection];
    if (@available(iOS 13.0, *)) {
        if ([self.traitCollection hasDifferentColorAppearanceComparedToTraitCollection:previousTraitCollection]) {
            _imageView.layer.borderColor = self.fs_dividerColor_P2.CGColor;
        }
    }
}
#endif

@end
