/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSFileInfoTableViewCell.h"

@interface FSFileInfoTableViewCell () <UITextFieldDelegate>

@end

@implementation FSFileInfoTableViewCell 

-(instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier{
    if (self = [super initWithStyle:style reuseIdentifier:reuseIdentifier]){
        self.backgroundColor = self.fs_backgroundColor_B1;
    }
    [self arrageView];
    return self;
}

- (void)awakeFromNib {
    [super awakeFromNib];
    // Initialization code
}

- (UILabel *)leftLabel{
    if (!_leftLabel) {
        _leftLabel = [[UILabel alloc] init];
        _leftLabel.textColor = self.fs_textColor_T3;
        _leftLabel.font = self.fs_standardGeneralFont_GF2;
        _leftLabel.numberOfLines = 0;
    }
    return _leftLabel;
}

- (UILabel *)rightLabel{
    if (!_rightLabel) {
        _rightLabel = [[UILabel alloc] init];
        _rightLabel.textColor = self.fs_textColor_T4;
        _rightLabel.font = self.fs_standardGeneralFont_GF2;
        _rightLabel.hidden = NO;
    }
    return _rightLabel;
}

- (UITextField *)rightTextField{
    if (!_rightTextField) {
        _rightTextField = [[UITextField alloc] init];
        _rightTextField.textColor = self.fs_textColor_T4;
        _rightTextField.font = self.fs_standardGeneralFont_GF2;
        _rightTextField.hidden = YES;
        _rightTextField.delegate = self;
    }
    return _rightTextField;
}

- (void)arrageView{
    [self.contentView addSubview:self.leftLabel];
    [self.contentView addSubview:self.rightLabel];
    [self.contentView addSubview:self.rightTextField];
    [self setViewConstraints];
}

- (void)setViewConstraints{
    [self.leftLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView).offset(16);
        make.height.mas_equalTo(36);
        make.width.mas_equalTo(112);
        make.centerY.equalTo(self.contentView);
    }];
    
    [self.rightLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.leftLabel.mas_right).offset(24);
        make.height.mas_equalTo(18);
        make.right.equalTo(self.contentView).offset(-10);
        make.centerY.equalTo(self.contentView);
    }];
    
    [self.rightTextField mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.top.right.bottom.equalTo(self.rightLabel);
    }];
}

- (void)setSelected:(BOOL)selected animated:(BOOL)animated {
    [super setSelected:selected animated:animated];

    // Configure the view for the selected state
}

- (BOOL)textFieldShouldBeginEditing:(UITextField *)textField
{
    if ([self.extensionsMgr.permissionProvider checkPermission:FSFunctionDocInfoEdit] == FSPermissionStateShow) {
        return YES;
    }else{
        [self.extensionsMgr.permissionProvider checkPermission:FSFunctionDocInfoEdit checkPermissionState:^(FSPermissionState state) {
            if (state == FSPermissionStateShow) {
                [textField becomeFirstResponder];
            }
        }];
        return NO;
    }
}

- (void)textFieldDidEndEditing:(UITextField *)textField reason:(UITextFieldDidEndEditingReason)reason{
    if (self.metaKeyStr && self.metadata) {
        NSString *contentText = textField.text;
        if (contentText) {
            NSArray<NSString*> *strArray = @[contentText];
            [self.metadata setValues:self.metaKeyStr value_array:strArray];
            self.extensionsMgr.isDocModified = YES;
        }

    }
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField
{
    [textField resignFirstResponder];
    return NO;
}

@end
