/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSReduceFileSizeView.h"
#import "FSCircleProgressView.h"

@interface FSReduceFileSizeView ()

@property(nonatomic, strong) UIView *contentView;
@property(nonatomic, strong) UILabel *msgLable;
@property(nonatomic, strong) NSString *msgStr;
@property(nonatomic, assign) BOOL isSuccess;
@property(nonatomic, strong) UIImageView *beforeImageView;
@property(nonatomic, strong) UIImageView *arrowImageView;
@property(nonatomic, strong) UIImageView *afterImageView;


@property(nonatomic, strong) FSCircleProgressView *progressView;
@end

@implementation FSReduceFileSizeView

/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
    // Drawing code
}
*/

-(instancetype)initWithState:(BOOL)sucess msg:(NSString*)msg{
    if (self = [super init]) {
        
        self.msgStr = msg;
        self.isSuccess = sucess;
        [self arrangeView:NO];
        
    }
    
    return self;
}

-(instancetype)initWithProgress:(CGFloat)progress{
    if (self = [super init]) {
        
        self.progressValue = progress;
        [self arrangeView:YES];
        
    }
    
    return self;
}


- (void)arrangeView:(BOOL)bProgressView{
    
    UIButton *backBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    [self addSubview:backBtn];
    backBtn.backgroundColor = [UIColor clearColor];
    [backBtn addTarget:self action:@selector(backBtnPress) forControlEvents:UIControlEventTouchUpInside];
    [backBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.top.bottom.equalTo(self);
    }];
    
    self.contentView = [[UIView alloc] init];
    [self addSubview:self.contentView];
    self.contentView.backgroundColor = self.fs_backgroundColor_B1;
    [self.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.mas_bottom);
        make.height.mas_equalTo(240);
        if (DEVICE_iPAD && ![self fs_currentSceneiPadHorizontalCompact]) {
            make.width.mas_equalTo(375);
            make.centerX.equalTo(self);
        }else{
            make.width.mas_equalTo(MIN(self.fs_currentSceneKeyWindow.bounds.size.width, self.fs_currentSceneKeyWindow.bounds.size.height));
            make.centerX.equalTo(self);
        }
    }];
    
    if (bProgressView) {
        [self arrangeProgressContentView];
        [self showProgressAnimated];
    }else{
        [self arrangeResultContentView];
        [self showResultAnimated];
    }
}

-(void)setProgressValue:(CGFloat)progressValue{
    _progressValue = progressValue;
    
    if (self.progressView) {
        self.progressView.progressValue = progressValue;
    }
}

- (void)arrangeResultContentView{
    
    UILabel *titleLable = [[UILabel alloc] init];
    [self.contentView addSubview:titleLable];
    titleLable.text = FSLocalizedForKey(@"kReduceFileSize");
    titleLable.font = self.fs_standardGeneralFont_GF1;
    titleLable.textColor = self.fs_textColor_T4;
    titleLable.textAlignment = NSTextAlignmentCenter;
    [titleLable mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView).offset(40);
        make.right.equalTo(self.contentView).offset(-40);
        make.top.mas_equalTo(13);
        make.height.mas_equalTo(22);
    }];
    
    UIButton *closeBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    closeBtn.accessibilityIdentifier = @"more_reduce_size_close";
    [self addSubview:closeBtn];
    [closeBtn setImage:FSImageNamedAndNormalColor(@"more_reduce_size_close") forState:UIControlStateNormal];
    closeBtn.backgroundColor = [UIColor clearColor];
    [closeBtn addTarget:self action:@selector(backBtnPress) forControlEvents:UIControlEventTouchUpInside];
    [closeBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.contentView).offset(-8);
        make.centerY.equalTo(titleLable);
        make.size.mas_equalTo(CGSizeMake(24, 24));
    }];
    
    self.msgLable = [[UILabel alloc] init];
    self.msgLable.accessibilityIdentifier = @"reduce_msgLabel_id";
    [self addSubview:self.msgLable];
    self.msgLable.text = self.msgStr;
    self.msgLable.font = [UIFont systemFontOfSize:15];
    self.msgLable.textColor = self.fs_textColor_T4;
    self.msgLable.textAlignment = NSTextAlignmentCenter;
    [self.msgLable mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView).offset(40);
        make.right.equalTo(self.contentView).offset(-40);
        make.bottom.equalTo(self.contentView).offset(-64);
        make.height.mas_equalTo(20);
    }];
    
    self.beforeImageView = [[UIImageView alloc] initWithImage:FSImageNamedAndNormalColor(@"more_reduce_size_before")];
    [self.contentView addSubview:self.beforeImageView];
    
    self.arrowImageView = [[UIImageView alloc] initWithImage:FSImageNamedAndPrimaryColor(@"more_reduce_size_arrow")];
    [BindThemeKeySel(self.arrowImageView, setImage:) fs_updateThemeCompleted:^(UIImageView * _Nonnull target, SEL  _Nonnull sel) {
        [target setImage:FSImageNamedAndPrimaryColor(@"more_reduce_size_arrow")];
    }];
    [self.contentView addSubview:self.arrowImageView];
    
    self.afterImageView = [[UIImageView alloc] initWithImage:FSImageNamedAndPrimaryColor(@"more_reduce_size_after")];
    [BindThemeKeySel(self.afterImageView, setImage:) fs_updateThemeCompleted:^(UIImageView * _Nonnull target, SEL  _Nonnull sel) {
        [target setImage:FSImageNamedAndPrimaryColor(@"more_reduce_size_after")];
    }];
    [self.contentView addSubview:self.afterImageView];
    if (DEVICE_iPAD && ![self fs_currentSceneiPadHorizontalCompact]) {
        self.contentView.layer.cornerRadius = 8;
        self.clipsToBounds = YES;
    }
    
    [self.arrowImageView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(self.contentView);
        make.size.mas_equalTo(CGSizeMake(18, 13));
        make.top.mas_equalTo(101);
        
    }];
    
    [self.beforeImageView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.arrowImageView.mas_left).offset(-16);
        make.centerY.equalTo(self.arrowImageView);
        if (self.isSuccess) {
            make.size.mas_equalTo(CGSizeMake(56, 56));
        }else{
            make.size.mas_equalTo(CGSizeMake(40, 40));
        }
    }];
    
    [self.afterImageView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.arrowImageView.mas_right).offset(16);
        make.centerY.equalTo(self.arrowImageView);
        if (self.isSuccess) {
            make.size.mas_equalTo(CGSizeMake(40, 40));
        }else{
            make.size.mas_equalTo(CGSizeMake(56, 56));
        }
    }];
}

- (void)arrangeProgressContentView{
    
    UIButton *closeBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    [self addSubview:closeBtn];
    [closeBtn setImage:FSImageNamedAndNormalColor(@"more_reduce_size_close") forState:UIControlStateNormal];
    closeBtn.backgroundColor = [UIColor clearColor];
    [closeBtn addTarget:self action:@selector(backBtnPress) forControlEvents:UIControlEventTouchUpInside];
    [closeBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(self.contentView).offset(-8);
        make.top.equalTo(self.contentView).offset(8);
        make.size.mas_equalTo(CGSizeMake(24, 24));
    }];
    
    self.progressView = [[FSCircleProgressView alloc] init];
    self.progressView.progressColor = self.fs_primaryColor;
    self.progressView.progressTrackColor = self.fs_textColor_T2;
    self.progressView.progressWidth = 4;
    self.progressView.progressColor = self.fs_primaryColor;
    self.progressView.progressTrackWidth = 2;
    self.progressView.progressTextFont = [UIFont systemFontOfSize:12];
    [self.contentView addSubview:self.progressView];
    [self.progressView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.size.mas_equalTo(CGSizeMake(48, 48));
        make.top.mas_equalTo(68);
        make.centerX.equalTo(self.contentView);
    }];
    BindThemeKeyPath(self.progressView, progressColor);
    
    UILabel *titleLable = [[UILabel alloc] init];
    [self.contentView addSubview:titleLable];
    titleLable.text = FSLocalizedForKey(@"kOptimizing");
    titleLable.font = [UIFont systemFontOfSize:15];
    titleLable.textColor = self.fs_textColor_T4;
    titleLable.textAlignment = NSTextAlignmentCenter;
    [titleLable mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView).offset(40);
        make.right.equalTo(self.contentView).offset(-40);
        make.top.equalTo(self.progressView.mas_bottom).offset(36);
        make.height.mas_equalTo(20);
    }];
    
    if (DEVICE_iPAD && ![self fs_currentSceneiPadHorizontalCompact]) {
        self.contentView.layer.cornerRadius = 8;
        self.clipsToBounds = YES;
    }
    
}

- (void)backBtnPress{
    [self removeFromSuperview];
}

- (void)showResultAnimated{
 
    self.backgroundColor = [UIColor colorWithWhite:0 alpha:0.1];
    [self layoutIfNeeded];
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.01 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        [UIView animateWithDuration:0.3 animations:^{
            
            [self.contentView mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.height.mas_equalTo(240);
                if (DEVICE_iPAD && ![self fs_currentSceneiPadHorizontalCompact]) {
                    make.width.mas_equalTo(375);
                    make.centerX.centerY.equalTo(self);
                }else{
                    make.width.mas_equalTo(MIN(self.fs_currentSceneKeyWindow.bounds.size.width, self.fs_currentSceneKeyWindow.bounds.size.height));
                    make.centerX.equalTo(self);
                    make.bottom.equalTo(self.mas_bottom);
                }
            }];
            self.backgroundColor = [UIColor colorWithWhite:0 alpha:0.4];
            [self layoutIfNeeded];
        } completion:^(BOOL finished) {
            
        }];
    });

    
}

- (void)showProgressAnimated{
 

    [self.contentView mas_remakeConstraints:^(MASConstraintMaker *make) {
    make.height.mas_equalTo(240);
    if (DEVICE_iPAD && ![self fs_currentSceneiPadHorizontalCompact]) {
        make.width.mas_equalTo(375);
        make.centerX.centerY.equalTo(self);
        make.top.equalTo(self.mas_bottom);
    }else{
        make.width.mas_equalTo(MIN(self.fs_currentSceneKeyWindow.bounds.size.width, self.fs_currentSceneKeyWindow.bounds.size.height));
        make.centerX.equalTo(self);
        make.bottom.equalTo(self.mas_bottom);
    }
    }];
    self.backgroundColor = [UIColor colorWithWhite:0 alpha:0.4];
    [self layoutIfNeeded];


    
}

- (void)layoutSubviews{
    [super layoutSubviews];
    if (self.contentView && self.superview) {
        [self.contentView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.height.mas_equalTo(240);
            if (DEVICE_iPAD && ![self fs_currentSceneiPadHorizontalCompact]) {
                make.width.mas_equalTo(375);
                make.centerX.centerY.equalTo(self);
            }else{
                make.width.mas_equalTo(MIN(self.bounds.size.width, self.bounds.size.height));
                make.centerX.equalTo(self);
                make.bottom.equalTo(self.mas_bottom);
            }
        }];
    }
    
    if (DEVICE_iPAD && ![self fs_currentSceneiPadHorizontalCompact]) {
        self.contentView.layer.cornerRadius = 8;
        self.clipsToBounds = YES;
    }else{
        self.contentView.layer.cornerRadius = 0;
        self.clipsToBounds = YES;
    }
}

@end
