/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSTransParentContainerViewController.h"

#define HeightSpacing 10

@interface FSTransParentContainerViewController ()
@property (nonatomic, strong) UIViewController<FSTransParentControllerProtocol> *targetVC;
@property (nonatomic, strong) UIView *targetView;
@property (nonatomic, strong) UIView *containerView;
@property (nonatomic, strong) UIControl *backCtrl;
@property (nonatomic, assign) NSInteger containerHeight;
@property(nonatomic, assign) NSInteger expectedHeight;
@end

@implementation FSTransParentContainerViewController

-(instancetype)initWithViewController:(UIViewController<FSTransParentControllerProtocol> *)viewController expectedHeight:(NSInteger)height{
    if (self = [super init]){
        self.modalPresentationStyle = UIModalPresentationOverCurrentContext;
        self.targetVC = viewController;
        self.targetVC.containerViewCtrl = self;
        self.expectedHeight = height;
        self.targetView = self.targetVC.view;
    }
    
    return self;
}

-(instancetype)initWithView:(UIView *)view expectedHeight:(NSInteger)height{
    if (self = [super init]){
        self.modalPresentationStyle = UIModalPresentationOverCurrentContext;
        self.targetView = view;
        self.expectedHeight = height;
    }
    
    return self;
}

- (UIControl *)backCtrl{
    if (!_backCtrl) {
        _backCtrl = [[UIControl alloc] init];
        _backCtrl.backgroundColor = [UIColor colorWithWhite:0 alpha:0.3];
        [_backCtrl addTarget:self action:@selector(dismissSelf) forControlEvents:UIControlEventTouchUpInside];
    }
    
    return _backCtrl;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    [self.view addSubview:self.backCtrl];
    [self.backCtrl mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.top.right.bottom.equalTo(self.view);
    }];
    self.containerView = [[UIView alloc] init];
    
    if (_targetView){
        [self.containerView addSubview:_targetView];
        self.containerView.backgroundColor = self.targetView.backgroundColor;
        [_targetView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(self.containerView).offset(HeightSpacing);
            make.left.right.bottom.equalTo(self.containerView);
        }];
    }
    
    UIPanGestureRecognizer *panGesture = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(handlePan:)];
    [self.containerView addGestureRecognizer:panGesture];
    [self makeBeginAnimateConstraints];
}

- (void)makeBeginAnimateConstraints{
    if (_containerHeight <= 0) {
        return;
    }
    NSInteger height = 0;
    NSInteger width = MIN([UIScreen mainScreen].bounds.size.height, [UIScreen mainScreen].bounds.size.width);
    if (_containerHeight > [UIScreen mainScreen].bounds.size.height-44) {
        height = [UIScreen mainScreen].bounds.size.height-44;
    }else{
        height = _containerHeight;
    }
    [_containerView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.view.mas_bottom);
        make.centerX.equalTo(self.view);
        make.height.mas_equalTo(height);
        make.width.mas_equalTo(width);
    }];
    
    [UIView animateWithDuration:0.4 animations:^{
        [self.containerView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(self.view.mas_bottom);
            make.centerX.equalTo(self.view);
            make.height.mas_equalTo(height);
            make.width.mas_equalTo(width);
        }];
    } completion:^(BOOL finished) {
        
    }];
    
}

- (void)handlePan:(UIPanGestureRecognizer*)panGesture{
    
}

- (void)setExpectedHeight:(NSInteger)expectedHeight{
    _expectedHeight = expectedHeight;
    self.containerHeight = expectedHeight + HeightSpacing;
}

- (void)dismissSelf{
    
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
