/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FileInformationViewController.h"
#import "UINavigationItem+IOS7PaddingAdditions.h"
#import "MenuView+Private.h"
#import "PermissionViewController.h"
#import "Utility.h"
#import "FSFileInfoHeadView.h"
#import "FSFileInfoTableViewCell.h"

#define LABLEHIGHT 20
#define LABLEWIDTH 100
#define LABLEVALUEWIDTH 300
#define LEFTMARGEN 20
#define TOPMARGEN 20

static NSString *Cellidentifer = @"CellIdentifer";
static NSString *PermissionCellidentifer = @"PermisssionCellIdentifer";

@interface FileInformationViewController ()<IFSUIExtensionsModifiedEventListener>

@property (nonatomic, weak) FSPDFViewCtrl *pdfViewCtrl;
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
@property (nonatomic, strong) FSFileInfoHeadView *fileInfoHeadView;
@property (nonatomic, strong) FSMetadata *metadata;
@property (nonatomic, assign) BOOL didViewDidAppear;
@end

@implementation FileInformationViewController

-(instancetype)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil{
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
        self.shouldShowNavigationBar = YES;
    }
    return self;
}

- (void)setUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    _extensionsManager = extensionsManager;
    _pdfViewCtrl = extensionsManager.pdfViewCtrl;
}

- (void)viewDidLoad {
    [super viewDidLoad];

    if (self.showStyle == FSNavigationBarShowStylePresent) {
        self.navBarLeftTitle = FSLocalizedForKey(@"kClose");
    }
    
    CGRect tableRect = CGRectMake(0, 0, self.view.frame.size.width, self.view.frame.size.height);
    self.tableView = [[UITableView alloc] initWithFrame:tableRect style:UITableViewStylePlain];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    [self.view addSubview:self.tableView];
    self.view.backgroundColor = self.fs_backgroundColor_B1;
    self.tableView.backgroundColor = self.fs_backgroundColor_B1;
    self.fileInfoHeadView = [[FSFileInfoHeadView alloc] initWithFrame:CGRectMake(0, 0, self.tableView.frame.size.width, 198)];
    self.tableView.tableHeaderView = self.fileInfoHeadView;
    
    UITapGestureRecognizer *tableViewGesture = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(commentTableViewTouchInSide)];
    tableViewGesture.numberOfTapsRequired = 1;
    tableViewGesture.cancelsTouchesInView = NO;
    [self.tableView addGestureRecognizer:tableViewGesture];

    self.tableView.separatorStyle = UITableViewCellSeparatorStyleNone;
    self.tableView.translatesAutoresizingMaskIntoConstraints = false;
    [self.tableView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.navBar.mas_bottom);
        if (@available(iOS 11.0, *)) {
            make.left.right.bottom.equalTo(self.view.mas_safeAreaLayoutGuide);
        }else{
            make.left.right.bottom.equalTo(self.view);
        } 
    }];

    [self initNavigationBar];

    [self getDocInfo];
    
    FSPDFDoc *document = _pdfViewCtrl.currentDoc;
    self.metadata = [[FSMetadata alloc] initWithDocument:document];
    
}

- (void)commentTableViewTouchInSide{
    [self.view endEditing:YES];
}

- (void)viewWillDisappear:(BOOL)animated {
    
    [super viewWillDisappear:animated];
    [[NSNotificationCenter defaultCenter] removeObserver:self];

    [_extensionsManager unregisterExtensionsModifiedEventListener:self];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    //[[UIApplication sharedApplication] setStatusBarStyle:UIStatusBarStyleDefault animated:NO];
    [self.tableView reloadData];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(orientationDidChange:) name:UIDeviceOrientationDidChangeNotification object:nil];
    
    [_extensionsManager registerExtensionsModifiedEventListener:self];
    _didViewDidAppear = NO;
}

- (void)viewDidAppear:(BOOL)animated{
    [super viewDidAppear:animated];
    _didViewDidAppear = YES;
}

- (void)viewDidLayoutSubviews{
    [super viewDidLayoutSubviews];
#if _MAC_CATALYST_
    if (!_didViewDidAppear) {
        [self commentTableViewTouchInSide];
    }
#endif
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)orientationDidChange:(NSNotification *)aNotification {
    [self.tableView reloadData];
}

#pragma mark - NavigationController methods
- (void)initNavigationBar {
    [self.navBar setTitle:FSLocalizedForKey(@"kDocinfo")];
    StateBarThemeKeyPath(self.navBar.stateBar, backgroundColor);
    [BindThemeKeySel(self.navBar.stateBar, backgroundView) fs_updateThemeCompleted:^(UIView*  _Nonnull target, SEL  _Nonnull sel) {
        StateBarThemeKeyPath(target, backgroundColor);
    }];
}

#pragma mark - Table view data source

- (CGFloat)tableView:(UITableView *)tableView heightForFooterInSection:(NSInteger)section {
    return 0.001;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    if (indexPath.section == 0) {
        return 44;
    }else if (indexPath.section == 1) {
        return 50;
    }
    return 44;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    if (section == 0) {
        return 14;
    }
    return 1;
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section{
    if (section == 0) {
        return 32;
    }
    return 24;
}

- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section{
    int height = 32;
    if (section != 0) {
        height = 24;
    }
    UIView *headerView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, CGRectGetWidth(self.tableView.bounds), height)];
    headerView.backgroundColor = self.fs_backgroundColor_B1;
    
    UILabel *headerLabel = [[UILabel alloc] init];
    if (section == 0) {
        headerLabel.text = FSLocalizedForKey(@"kHeaderDescription");
    }else{
        headerLabel.text = [FSLocalizedForKey(@"kSecurity") uppercaseString];
    }
    headerLabel.textColor = self.fs_textColor_T3;
    headerLabel.font = [UIFont systemFontOfSize:12];
    [headerView addSubview:headerLabel];
    [headerLabel mas_makeConstraints:^(MASConstraintMaker *make) {

        make.top.bottom.equalTo(headerView);
        make.left.equalTo(headerView).offset(16);
        make.right.equalTo(headerView).offset(-16);
    }];
    return headerView;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 2;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {

    UITableViewCell *tableViewCell = nil;
    if (indexPath.section == 0) {
        NSString *cellIdentifierStr = [NSString stringWithFormat:@"%@_%ld",Cellidentifer,(long)indexPath.row];
        FSFileInfoTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifierStr];
        if (!cell) {
            cell = [[FSFileInfoTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellIdentifierStr];
            cell.selectionStyle = UITableViewCellSelectionStyleNone;
            cell.extensionsMgr = self.extensionsManager;
        }
        tableViewCell = cell;
        cell.leftLabel.numberOfLines = 0;
        cell.rightTextField.enabled = !self.extensionsManager.isViewSignedDocument && self.extensionsManager.enableModifyDocument;
        cell.rightTextField.returnKeyType = UIReturnKeyDone;
        switch (indexPath.row) {
            case 0:
            {
                NSString *extensionString = [[self.extensionsManager.pdfViewCtrl.filePath pathExtension] uppercaseString];
                NSString *versionString = [NSString stringWithFormat:@"%0.1f",[[self.extensionsManager.pdfViewCtrl getDoc] getFileVersion]/10.0];
                cell.leftLabel.text = [NSString stringWithFormat:@"%@", FSLocalizedForKey(@"kPDFVersion")];
                cell.rightLabel.text = [NSString stringWithFormat:@"%@-%@",extensionString, versionString];
            }
                break;
            case 1:
            {
                cell.leftLabel.text = [NSString stringWithFormat:@"%@", FSLocalizedForKey(@"kFastWebView")];
                BOOL isFastWebViewPDF = [self.extensionsManager.pdfViewCtrl getDoc].isLinearized;
                cell.rightLabel.text = isFastWebViewPDF ? FSLocalizedForKey(@"kYes") : FSLocalizedForKey(@"kNo");
            }
                break;
            case 2:
            {
                cell.leftLabel.text = [NSString stringWithFormat:@"%@", FSLocalizedForKey(@"kTaggedPDF")];
                BOOL isTaggedPDF = [self.extensionsManager.pdfViewCtrl getDoc].isTaggedPDF;
                cell.rightLabel.text = isTaggedPDF ? FSLocalizedForKey(@"kYes") : FSLocalizedForKey(@"kNo");
            }
                break;
            case 3:
            {
                cell.leftLabel.text = [NSString stringWithFormat:@"%@", FSLocalizedForKey(@"kPageSize")];
                int currentPageindex = [self->_pdfViewCtrl getCurrentPage];
                float width = 0;
                float height = 0;
                cell.rightLabel.text = @"";
                if ([self.pdfViewCtrl isDynamicXFA]) {
                    FSXFAPage *xfaPage = [[self.extensionsManager.pdfViewCtrl getXFADoc] getPage:currentPageindex];
                    if ([xfaPage isEmpty]) {
                        cell.rightLabel.text = @"";
                    }else{
                        width = [xfaPage getWidth];
                        height = [xfaPage getHeight];
                    }
                }else{
                    FSPDFPage *pdfPage = [self.extensionsManager.pdfViewCtrl.currentDoc getPage:currentPageindex];
                    if ([pdfPage isEmpty]) {
                        cell.rightLabel.text = @"";
                    }else{
                        width = [pdfPage getWidth];
                        height = [pdfPage getHeight];
                    }
                }
                if (width != 0 && height != 0){
                    FSMeasurementUnits measureUnits = self.extensionsManager.measurementUnits;
                    switch (measureUnits) {
                        case FSMeasurementCM:
                            cell.rightLabel.text = [NSString stringWithFormat:@"%0.2f*%0.2f %@",width/72*2.54f,height/72*2.54f,@"cm"];
                            break;
                        case FSMeasurementMM:
                            cell.rightLabel.text = [NSString stringWithFormat:@"%0.2f*%0.2f %@",width/72*25.4f,height/72*25.4f,@"mm"];
                            break;
                        default:
                            cell.rightLabel.text = [NSString stringWithFormat:@"%0.2f*%0.2f %@",width/72,height/72,FSLocalizedForKey(@"kInch")];
                            break;
                    }
                }
            }
                break;
            case 4:
            {
                cell.leftLabel.text = [NSString stringWithFormat:@"%@", FSLocalizedForKey(@"kPages")];
                cell.rightLabel.text = [NSString stringWithFormat:@"%d",[[self.extensionsManager.pdfViewCtrl getDoc] getPageCount]];
            }
                break;
            case 5:
            {
                cell.leftLabel.text = [NSString stringWithFormat:@"%@", FSLocalizedForKey(@"kTitle")];
                [self setValuetoUITextField:cell key:@"Title" metaData:self.metadata];
                cell.rightTextField.placeholder = FSLocalizedForKey(@"kTitleNotice");
            }
                break;
            case 6:
            {
                cell.leftLabel.text = [NSString stringWithFormat:@"%@", FSLocalizedForKey(@"kFileSubject")];
                [self setValuetoUITextField:cell key:@"Subject" metaData:self.metadata];
                cell.rightTextField.placeholder = FSLocalizedForKey(@"kSubjectNotice");
            }
                break;
            case 7:
            {
                cell.leftLabel.text = [NSString stringWithFormat:@"%@", FSLocalizedForKey(@"kCreator")];
                [self setValuetoUITextField:cell key:@"Creator" metaData:self.metadata];
                cell.rightTextField.placeholder = FSLocalizedForKey(@"kCreatorNotice");

            }
                break;
            case 8:
            {
                cell.leftLabel.text = [NSString stringWithFormat:@"%@", FSLocalizedForKey(@"kFileAuthor")];
                [self setValuetoUITextField:cell key:@"Author" metaData:self.metadata];
                cell.rightTextField.placeholder = FSLocalizedForKey(@"kAuthorNotice");
            }
                break;
            case 9:
            {
                cell.leftLabel.text = [NSString stringWithFormat:@"%@", FSLocalizedForKey(@"kProducer")];
                [self setValuetoUITextField:cell key:@"Producer" metaData:self.metadata];
                cell.rightTextField.placeholder = FSLocalizedForKey(@"kProducerNotice");
            }
                break;
            case 10:
            {
                cell.leftLabel.text = [NSString stringWithFormat:@"%@", FSLocalizedForKey(@"kKeywords")];
                [self setValuetoUITextField:cell key:@"Keywords" metaData:self.metadata];
                cell.rightTextField.placeholder = FSLocalizedForKey(@"kKeywordsNotice");
            }
                break;
            case 11:
            {
                cell.leftLabel.text = [NSString stringWithFormat:@"%@", FSLocalizedForKey(@"kDocumentSize")];
                unsigned long long fileSize = 0;
                NSFileManager *manager = [NSFileManager defaultManager];
                if ([manager fileExistsAtPath:self.extensionsManager.pdfViewCtrl.filePath]) {
                    fileSize = [[manager attributesOfItemAtPath:self.extensionsManager.pdfViewCtrl.filePath error:nil] fileSize];
                }
                cell.rightLabel.text = [Utility displayFileSize:fileSize];
            }
                break;
            case 12:
            {
                cell.leftLabel.text = [NSString stringWithFormat:@"%@", FSLocalizedForKey(@"kModifyDate")];
                cell.rightLabel.text = [Utility displayDateInMDYHMS:[Utility convertFSDateTime2NSDate:[self.metadata getModifiedDateTime]]];
            }
                break;
            case 13:
            {
                cell.leftLabel.text = [NSString stringWithFormat:@"%@", FSLocalizedForKey(@"kCreateDate")];
                cell.rightLabel.text = [Utility displayDateInMDYHMS: [Utility convertFSDateTime2NSDate:[self.metadata getCreationDateTime]]];
            }
                break;

            default:
                break;
        }
        
    } else if (indexPath.section == 1) {
        
        UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:PermissionCellidentifer];
        if (cell == nil) {
            cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:PermissionCellidentifer];
            cell.backgroundColor = self.fs_backgroundColor_B1;
            cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
            UILabel *label = [[UILabel alloc] init];
            label.font = [UIFont systemFontOfSize:15.0f];
            label.backgroundColor = [UIColor clearColor];
            label.frame = CGRectMake(16, 5, 200, 30);
            label.textColor = self.fs_textColor_T4;
            [cell.contentView addSubview:label];
            label.tag = 9999;
        }
        cell.selectionStyle = UITableViewCellSelectionStyleDefault;
        cell.selectedBackgroundView = [UIView new];
        cell.selectedBackgroundView.backgroundColor = self.fs_dividerColor_P1;
        tableViewCell = cell;

        UILabel *label = [cell.contentView viewWithTag:9999];
        if (label) {
            FSPDFDocEncryptType type = [_pdfViewCtrl.currentDoc getEncryptionType];
            //BOOL isStdEncrypted = (FSPDFDocEncryptPassword == type);

            id<FSMenuView> customView = [self.extensionsManager.menuViewManager getMenuViewForMenuViewType:FSMenuViewTypeViewSecurityPermission];
            FSActionControlAttribute *attribute;
            if ([customView respondsToSelector:@selector(getActionControlAttribute)]) {
                attribute = [customView getActionControlAttribute];
            }
            if (attribute) {
                label.fs_width = tableView.fs_width - 100;
                label.text = attribute.normalTitle;
            }else{
                if (FSPDFDocEncryptPassword == type) {
                    label.text = [NSString stringWithFormat:@"%@", FSLocalizedForKey(@"kPasswordEncryption")];
                } else if (FSPDFDocEncryptRMS == type || [[_pdfViewCtrl.filePath pathExtension].lowercaseString isEqualToString:@"ppdf"] || [_pdfViewCtrl isRMSProtected]) {
                    label.text = [NSString stringWithFormat:@"%@", FSLocalizedForKey(@"kRMSEncryption")];
                }else {
                    label.text = [NSString stringWithFormat:@"%@", FSLocalizedForKey(@"kNoEncryption")];
                }
            }
        }
    }

    tableViewCell.backgroundColor = self.fs_backgroundColor_B1;
    return tableViewCell;
}

- (void)setValuetoLabel:(UILabel*)label key:(NSString*)key metaData:(FSMetadata*)metaData{
    NSArray<NSString *> *metaArray = [metaData getValues:key];
    if (metaArray.count > 0) {
        label.text = metaArray[0];
    }else{
        label.text = @"";
    }
}

- (void)setValuetoUITextField:(FSFileInfoTableViewCell*)cell key:(NSString*)key metaData:(FSMetadata*)metaData{
    NSArray<NSString *> *metaArray = [metaData getValues:key];
    if (metaArray.count > 0) {
        cell.rightTextField.text = metaArray[0];
    }else{
        cell.rightTextField.text = @"";
    }
    cell.metaKeyStr = key;
    cell.metadata = metaData;
    cell.rightLabel.hidden = YES;
    cell.rightTextField.hidden = NO;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    if (indexPath.section == 0) {
    } else if (indexPath.section == 1) {
        [self viewSecurityPermission];
    } else if (indexPath.section == 2) {
    }
}

- (void)getDocInfo{
    if (self.extensionsManager.pdfViewCtrl && ![self.extensionsManager.pdfViewCtrl.currentDoc isEmpty]) {
                
        FSPDFDoc *document = self.extensionsManager.pdfViewCtrl.currentDoc;
        if (document == nil) {
            return;
        }

        unsigned long long fileSize = 0;
        NSFileManager *manager = [NSFileManager defaultManager];
        if ([manager fileExistsAtPath:self.extensionsManager.pdfViewCtrl.filePath]) {
            fileSize = [[manager attributesOfItemAtPath:self.extensionsManager.pdfViewCtrl.filePath error:nil] fileSize];
        }
        
        self.fileInfoHeadView.titlLabel.text = [[self.extensionsManager.pdfViewCtrl.filePath lastPathComponent] stringByDeletingPathExtension];
             
        FSPDFPage*  pdfPage = [self.extensionsManager.pdfViewCtrl.currentDoc getPage:0];
        if ([pdfPage isEmpty])
            return;
        float pageWidth = [pdfPage getWidth];
        float pageHeight = [pdfPage getHeight];
        float thumbnailWidth = 0;
        float thumbnailHeight = 0;
        
        FSRotation rotation = (FSRotation)[self.extensionsManager.pdfViewCtrl getViewRotation];
        if (rotation == FSRotation90 || rotation == FSRotation270) {
            float tmp = pageWidth;
            pageWidth =  pageHeight;
            pageHeight = tmp;
        }
        
        if (pageWidth > pageHeight) {
            thumbnailWidth = 132;
            thumbnailHeight = thumbnailWidth * pageHeight / pageWidth;
        }else{
            thumbnailHeight = 132;
            thumbnailWidth = thumbnailHeight * pageWidth / pageHeight;
        }
        
        
        @weakify(self)
        dispatch_async(dispatch_get_global_queue(0, 0), ^{
            @strongify(self)
            [Utility drawPage:pdfPage targetSize:CGSizeMake(thumbnailWidth, thumbnailHeight) shouldDrawAnnotation:YES progressive:^(UIImage * _Nonnull image, BOOL finish) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    [self.fileInfoHeadView remakeImageViewContraints:thumbnailWidth < thumbnailHeight];
                    self.fileInfoHeadView.imageView.image = image;
                });
            } needPause:^BOOL{
                return  NO;
            } rotation:rotation enableForPrint:NO pdfViewCtrl:self.extensionsManager.pdfViewCtrl];
        });
    }
}

- (void)viewSecurityPermission {
    id<FSMenuView> customView = [self.extensionsManager.menuViewManager getMenuViewForMenuViewType:FSMenuViewTypeViewSecurityPermission];
    if (customView) {
        if ([customView respondsToSelector:@selector(shouldPresentByRDK)]) {
            if (![customView shouldPresentByRDK]) {
                if ([customView respondsToSelector:@selector(presentActionInMenuViewByViewController:)]) {
                    [customView presentActionInMenuViewByViewController:^UIViewController * _Nonnull{
                        return self.navigationController;
                    }];
                    return;
                }
                if ([customView respondsToSelector:@selector(presentActionInMenuView)]) {
                    [customView presentActionInMenuView];
                }
                return;
            }
        }
        if ([customView respondsToSelector:@selector(getCustomView)]) {
            customView.containerController = [self.extensionsManager.viewCtrlManager pushViewCtrlType:FSViewCtrlTypeCustomViewContainer params:[customView getCustomView]];
        }
        return;
    }
    FSPDFDoc *document = _pdfViewCtrl.currentDoc;
    PermissionViewController *permissionCtrl = [[PermissionViewController alloc] init];
    BOOL docHasSignature = [Utility isDocumentSigned:document];
    permissionCtrl.allowOwner = [_pdfViewCtrl isOwner];
    permissionCtrl.allowPrint = [Utility canPrintDocument:_pdfViewCtrl.getDoc];
    permissionCtrl.allowFillForm = [Utility canFillForm:_pdfViewCtrl];
    permissionCtrl.allowAssemble = docHasSignature ? NO : [Utility canAssemble:_pdfViewCtrl];
    permissionCtrl.allowAnnotate = [Utility canAddAnnot:_pdfViewCtrl];
    permissionCtrl.allowEdit = docHasSignature ? NO : [Utility canModifyContents:_pdfViewCtrl];
    permissionCtrl.allowExtractAccess = [Utility canCopyForAssess:_pdfViewCtrl];
    permissionCtrl.allowExtract = [Utility canExtractContents:_pdfViewCtrl];
    permissionCtrl.extensionsManager = self.extensionsManager;

    [self.navigationController pushViewController:permissionCtrl animated:YES];
}

- (void)navigationBarBackTouch{

    [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] dismissViewControllerAnimated:YES completion:^{
    }];
}
#pragma mark - IFSUIExtensionsModifiedEventListener
- (void)extensions:(UIExtensionsManager *)extensions modifiedEventDealSel:(SEL)sel args:(NSArray *)args{
    if (extensions != _extensionsManager ) {
        [self getDocInfo];
        [self.tableView reloadData];
    }
}

@end
