/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSMoreGroupDetailViewController.h"
#import "MoreView.h"
#import "MenuView+Private.h"

@interface FSMoreGroupDetailViewController ()

@end

@implementation FSMoreGroupDetailViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.navBar.backgroundColor = self.fs_backgroundColor_B1;
 
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

-(CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section{
    return 0;
}

-(__kindof UITableViewCell *)tableView:(UITableView *)tableView dequeueReusableCellWithIdentifier:(NSString *)identifier forIndexPath:(NSIndexPath *)indexPath withDataItem:(id)dataItem{
    
    identifier = NSStringFromClass([FSTitleEstimatedTableViewCell class]);
    
    return  [super tableView:tableView dequeueReusableCellWithIdentifier:identifier forIndexPath:indexPath withDataItem:dataItem];
}

- (void)configureCell:(FSTitleEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(FSPopoverItem *)item{
    cell.isAccessibilityElement = YES;
    cell.selectedBackgroundColor = self.fs_dividerColor_P1;
    cell.contentView.backgroundColor = nil;
    if (!cell.installed) {
        cell.leftMargin = POPOVER_LABEL_LEFT_ICON_MARGINT;
        cell.mediumSpace = cell.leftMargin;
    }

    if (indexPath.section < self.dataSource.count) {
        NSArray<MoreMenuItem*> *itemArray = self.dataSource[indexPath.section];
        if (itemArray && indexPath.row < itemArray.count) {
            
            MoreMenuItem *menuItem = itemArray[indexPath.row];
            if (menuItem) {
                if (!cell.leftView) {
                    UIImageView *imageView = [UIImageView new];
                    imageView.fs_size = CGSizeMake(40.f, 40.f);
                    imageView.contentMode = UIViewContentModeScaleAspectFit;
                    cell.leftView = imageView;
                }
                
                ((UIImageView *)cell.leftView).image = menuItem.image;
                cell.estimatedTitleLabel.text = menuItem.title;
                cell.estimatedTitleLabel.textColor = cell.titleDefaultTextColor;
                cell.userInteractionEnabled = YES;
                cell.accessibilityLabel = menuItem.title;
                cell.accessibilityTraits = UIAccessibilityTraitNone;
                if(!menuItem.enable) {
                    cell.accessibilityTraits = UIAccessibilityTraitNotEnabled;
                    cell.userInteractionEnabled = NO;
                    cell.estimatedTitleLabel.textColor = cell.fs_iconColor_I2;
                    ((UIImageView *)cell.leftView).image = [Utility unableImage:menuItem.image];
                }
                if (menuItem.image.accessibilityIdentifier) {
                    ((UIImageView *)cell.leftView).accessibilityIdentifier = menuItem.image.accessibilityIdentifier;
                }
            }
        }
    }

}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    

    if (indexPath.section < self.dataSource.count) {
        NSArray<MoreMenuItem*> *itemArray = self.dataSource[indexPath.section];
        if (itemArray && indexPath.row < itemArray.count) {
            
            MoreMenuItem *menuItem = itemArray[indexPath.row];
            if (menuItem) {
                if ([menuItem.callBack respondsToSelector:@selector(onClick:)]) {
                    [self.extensionsMgr.more setHidden:YES animate:YES completion:^{
                        [menuItem.callBack onClick:menuItem];
                    }];
                }
            }
        }
    }
    
}

@end
