/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSMoreHeaderView.h"

@interface FSMoreHeaderView ()

@property (nonatomic, strong) UIView *roundedRectangleView;
@property (nonatomic, strong) UIView *imageBGView;
@property (nonatomic, strong) UIImageView *localIconImageView;
@end

@implementation FSMoreHeaderView

/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
    // Drawing code
}
*/

- (instancetype)init{
    if (self = [super init]) {
        [self arrangeRoundView];
    }
    return  self;
}

- (UIView *)imageBGView{

    if (!_imageBGView) {
        _imageBGView = [[UIView alloc] init];
    }
    return _imageBGView;
    
}

- (UIView *)roundedRectangleView{
    if (!_roundedRectangleView) {
        _roundedRectangleView = [[UIView alloc] init];
        _roundedRectangleView.backgroundColor = self.fs_backgroundColor_B3;
        _roundedRectangleView.layer.cornerRadius = 2;
        _roundedRectangleView.layer.masksToBounds = YES;
    }
    return _roundedRectangleView;
}

- (UIImageView *)pdfImageView{
    if (!_pdfImageView) {
        _pdfImageView = [[UIImageView alloc] init];
        _pdfImageView.contentMode = UIViewContentModeScaleAspectFit;
        _pdfImageView.layer.borderWidth = 1;
        _pdfImageView.layer.borderColor = self.fs_dividerColor_P2.CGColor;
        _pdfImageView.image = FSImageNamed(@"file_list_PDF");
    }
    return _pdfImageView;
}

- (UIImageView *)localIconImageView{
    if (!_localIconImageView) {
        _localIconImageView = [[UIImageView alloc] init];
        _localIconImageView.contentMode = UIViewContentModeScaleAspectFit;
    }
    return _localIconImageView;
}

- (UILabel *)pdfNameLabel{
    if (!_pdfNameLabel) {
        _pdfNameLabel = [[UILabel alloc] init];
        _pdfNameLabel.numberOfLines = 2;
        _pdfNameLabel.textAlignment = NSTextAlignmentLeft;
        _pdfNameLabel.textColor = self.fs_textColor_T4;
        _pdfNameLabel.font = self.fs_standardGeneralFont_GF2;
    }
    return _pdfNameLabel;
}

- (UILabel *)pdfInfoLabel{
    if (!_pdfInfoLabel) {
        _pdfInfoLabel = [[UILabel alloc] init];
        _pdfInfoLabel.numberOfLines = 1;
        _pdfInfoLabel.textAlignment = NSTextAlignmentLeft;
        _pdfInfoLabel.textColor = self.fs_textColor_T3;
        _pdfInfoLabel.font = self.fs_standardWeakFont_WF2;
    }
    return _pdfInfoLabel;
}

- (void)setLocalIconImage:(UIImage *)localIconImage
{
    _localIconImage = localIconImage;
    self.localIconImageView.image = _localIconImage;
}

- (void)arrangeRoundView{
    [self addSubview:self.roundedRectangleView];
    [self.roundedRectangleView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(self);
        make.top.equalTo(self).offset(3);
        make.size.mas_equalTo(CGSizeMake(31, 4));
    }];
}

- (void)removeAllSubViews{
    for (UIView *view in self.subviews) {
        if (view == self.roundedRectangleView) {
            continue;
        }else{
            [view removeFromSuperview];
        }
    }
}

- (void)arrangeView{
    
    [self addSubview:self.imageBGView];
    [self.imageBGView addSubview:self.pdfImageView];
    [self addSubview:self.pdfNameLabel];
    [self addSubview:self.pdfInfoLabel];
    [self addSubview:self.localIconImageView];
    [self setViewConstraints];
    
    [self setBackgroundImage:[UIImage fs_imageWithColor:self.fs_dividerColor_P1 size:CGSizeMake(1, 1)] forState:UIControlStateHighlighted];
    
    [self addTarget:self action:@selector(tapHandle:) forControlEvents:UIControlEventTouchUpInside];
    
    [self setViewConstraints];
}

- (void)tapHandle:(UIButton *)btn{
    if (self.tapHandleBlock) {
        self.tapHandleBlock();
    }
}

- (void)setViewConstraints{

    [self.imageBGView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self).offset(ITEM_MARGIN_NORMAL-4);
        make.centerY.equalTo(self).offset(3.5);
        make.size.mas_equalTo(CGSizeMake(40, 40));
        
    }];
    
    [self.pdfImageView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.center.equalTo(self.imageBGView);
        make.size.mas_equalTo(CGSizeMake(32, 40));
    }];
    
    [self.pdfNameLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.imageBGView.mas_right).offset(ITEM_MARGIN_NORMAL-4);
        make.right.mas_equalTo(self.pdfNameLabel.superview).offset(-ITEM_MARGIN_NORMAL);
        make.top.fs_equalToSuperview().offset(21);
    }];

    [self.pdfInfoLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.pdfNameLabel.mas_bottom).offset(3);
        make.left.mas_equalTo(self.pdfNameLabel).priorityHigh();
        make.bottom.fs_equalToSuperview().offset(-14);
    }];
    
    [self.localIconImageView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(self.pdfNameLabel.mas_bottom);
        make.left.mas_equalTo(self.pdfInfoLabel.mas_right).offset(4);
        make.width.height.mas_equalTo(16);
    }];
}

- (void)remakeImageViewContraints:(BOOL)isVertical{
    [self.pdfImageView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.center.equalTo(self.imageBGView);
        if (isVertical) {
            make.size.mas_equalTo(CGSizeMake(32, 40));
        }else{
            make.size.mas_equalTo(CGSizeMake(40, 32));
        }
        
    }];
}

- (BOOL)isAccessibilityElement
{
    return YES;
}
@end
