/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSMoreTableViewController.h"
#import "FSMoreGroupDetailViewController.h"
#import "FSMoreHeaderView.h"
#import "MenuView+Private.h"

#define kSectionFirstIdentifier @"FSTitleEstimatedTableViewCellNoImage"

@interface FSMoreTableViewController () <UIGestureRecognizerDelegate>
@property (nonatomic, assign, readwrite) FSTableViewPopoverShowStyle popoverShowStyle;
@property (nonatomic, strong) UIImage* coverImage;
@property (nonatomic, strong) NSString *pdfName;
@property (nonatomic, strong) NSString *pdfInfo;
@property (nonatomic, strong) FSMoreHeaderView *moreHeaderView;
@property (nonatomic, strong) UIPanGestureRecognizer *panRecognizer;
@property (nonatomic, strong) UIViewController *vc;
@end

@implementation FSMoreTableViewController
@synthesize popoverShowStyle = _popoverShowStyle;

- (instancetype)initWithServices:(id<FSViewControllerServices>)services showStyle:(FSTableViewPopoverShowStyle)showStyle {
    self = [super initWithServices:services];
    if (self) {
        self.popoverShowStyle = showStyle;
        self.popoverSize = CGSizeZero;
        [self configureModalPresentationStyle];
    }
    return self;
}

- (FSPresentedViewControllerAnimator *)animator{
    return self.transitioningDelegate;
}

- (void)resetPopverSize{
    CGSize popoverSize = self.popoverSize;
    popoverSize.height = [self getTableViewHeight];
    self.popoverSize = popoverSize;
    
    if (_popoverShowStyle == FSTableViewPopoverShowStyleDefault) {
        FSPresentedViewControllerAnimator *animator = self.animator;
        animator.size = popoverSize;
        animator.maxVerticalHeigh = popoverSize.height;
    }else{
        self.preferredContentSize = popoverSize;
    }
}

- (CGFloat)getTableViewHeight{
    
    CGFloat height = [self getMoreHeaderViewHeight];
    
    if (_groupDic.count > 0) {
        for (int i = 0; i < _groupDic.count ; i ++) {
            height += PREFERENCE_CELL_ROW_HEIGHT_NORMAL;
        }
        height += DIVIDE_VIEW_THICKNES;
    }

    if (_menuItemArray.count > 0) {
        for (int i = 0; i < _menuItemArray.count ; i ++) {
            height += PREFERENCE_CELL_ROW_HEIGHT_NORMAL;
        }
        height += DIVIDE_VIEW_THICKNES;
    }
    
    if ([self.extensionsMgr fs_currentSceneIsCompact]) {
        int maxHeight = [UIScreen mainScreen].bounds.size.height-44;
        if (@available(iOS 11.0, *)) {
            height += self.extensionsMgr.pdfViewCtrl.fs_currentSceneKeyWindow.safeAreaInsets.bottom;
            maxHeight = maxHeight - self.extensionsMgr.pdfViewCtrl.fs_currentSceneKeyWindow.safeAreaInsets.top;
        }
  
        height = MIN(height, maxHeight);
    }
    return height;
}

- (float)getMoreHeaderViewHeight{
    if (self.thumbnailHidden) {
        return 10.0;
    }
    
    float pdfNameLabelHeight = 0;
    float moreHeaderHeight = 0;
    NSString *pdfName = [[self.extensionsMgr.pdfViewCtrl.filePath lastPathComponent] stringByDeletingPathExtension];
    int width = self.extensionsMgr.pdfViewCtrl.fs_width-84;
    if (DEVICE_iPAD && ![self.extensionsMgr fs_currentSceneiPadHorizontalCompact]) {
        width = 390-84;
    }
    CGSize size = [Utility getTextSize:pdfName fontSize:15.0 maxSize:CGSizeMake(width, 1000)];
    pdfNameLabelHeight = size.height;
    if (pdfNameLabelHeight > 36) {
        pdfNameLabelHeight = 36;
    }
    
    moreHeaderHeight = pdfNameLabelHeight + 56;
    return moreHeaderHeight;
}

- (void)configureModalPresentationStyle{
    UIExtensionsManager *ex = ((FSViewControllerManager *)self.services).extensionsManager;
    if (self.popoverShowStyle == FSTableViewPopoverShowStyleDefault) {
        
        if (ex.pdfViewCtrl.fs_width > ex.pdfViewCtrl.fs_height) {
            self.popoverSize = CGSizeMake(ex.pdfViewCtrl.fs_height, 0);
        }else{
            self.popoverSize = CGSizeMake(ex.pdfViewCtrl.fs_width, 0);
        }
    }else{
        self.popoverSize = CGSizeMake(ex.pdfViewCtrl.fs_width, 0);
    }
    [super configureModalPresentationStyle];

    self.animator.dismissViewControllerCompletion = ^{
        [self.services.navigationControllerStack popNavigationController];
    };
}

- (void)loadView{

    self.view = [[UIView alloc] init];
}

- (void)viewDidLoad{
    [super viewDidLoad];
    
    [self setMoreHeadView];
    
    self.view.backgroundColor = self.tableView.backgroundColor;
    [self.tableView registerClass:[FSTitleEstimatedTableViewCell class] forCellReuseIdentifier:kSectionFirstIdentifier];
    self.tableView.bounces = NO;
    [self.tableView removeFromSuperview];
    [self.view addSubview:self.tableView];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    
    [self.tableView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.moreHeaderView.mas_bottom);
        make.left.right.bottom.equalTo(self.view);
    }];
    
  
    if (self.navigationController.view) {
        self.panRecognizer = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(handlePanFrom:)];
        self.panRecognizer.delegate = self;
        [self.navigationController.view addGestureRecognizer:self.panRecognizer];
    }
    
    
    
}

- (void)viewDidLayoutSubviews{
    [super viewDidLayoutSubviews];

}

- (void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    
    self.panRecognizer.enabled = YES;
    [self getDocInfo];
}

- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    
    self.panRecognizer.enabled = NO;
}

- (void)setMoreHeadView{
    
    if (!self.moreHeaderView.superview) {
        if (!self.moreHeaderView) {
            self.moreHeaderView = [[FSMoreHeaderView alloc] init];
        }
        [self.view addSubview:self.moreHeaderView];
    }
    
    [self.moreHeaderView removeAllSubViews];

    [self.moreHeaderView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.top.right.left.equalTo(self.view);
        if (self.thumbnailHidden) {
            make.height.mas_equalTo(10);
        }
    }];
    
    if (!self.thumbnailHidden) {
        [self.moreHeaderView arrangeView];
    }
    
    
    @weakify(self)
    self.moreHeaderView.tapHandleBlock = ^{
        @strongify(self)
        if (!self.thumbnailHidden) {
            [self.extensionsMgr.more setHidden:YES animate:YES completion:^{
                [self.extensionsMgr.more openFileInfo];
            }];
        }
        
        [self.extensionsMgr onUIInteractElementClicked:FSUIElementReadingMoreFileProperty];
    };
}

- (void)setFileLocationIcon:(UIImage *)icon
{
    if (!self.thumbnailHidden && self.moreHeaderView) {
        self.moreHeaderView.localIconImage = icon;
    }
}

- (void)setThumbnailHidden:(BOOL)thumbnailHidden{
    _thumbnailHidden = thumbnailHidden;
    [self setMoreHeadView];
}

- (void)getDocInfo{
    
    if (self.extensionsMgr.pdfViewCtrl && ![self.extensionsMgr.pdfViewCtrl.currentDoc isEmpty] ){
                

        FSPDFDoc *document = self.extensionsMgr.pdfViewCtrl.currentDoc;
        if (document == nil) {
            return;
        }
        FSMetadata *metadata = [[FSMetadata alloc] initWithDocument:document];

        unsigned long long fileSize = 0;
        NSFileManager *manager = [NSFileManager defaultManager];
        if ([manager fileExistsAtPath:self.extensionsMgr.pdfViewCtrl.filePath]) {
            fileSize = [[manager attributesOfItemAtPath:self.extensionsMgr.pdfViewCtrl.filePath error:nil] fileSize];
        }
        NSString *sizeStr = [Utility displayFileSize:fileSize];
        NSString *createTimeStr = [Utility displayDateInMDY:[Utility convertFSDateTime2NSDate:[metadata getCreationDateTime]]];
        if (createTimeStr == nil) {
            createTimeStr = @"";
        }else{
            createTimeStr = [NSString stringWithFormat:@" · %@",createTimeStr];
        }
        if (sizeStr == nil) {
            sizeStr = @"";
        }else{
            sizeStr = [NSString stringWithFormat:@" · %@",sizeStr];
        }
        self.pdfName = [[self.extensionsMgr.pdfViewCtrl.filePath lastPathComponent] stringByDeletingPathExtension];
        self.pdfInfo = [NSString stringWithFormat:@"%@%@%@",
                        [[self.extensionsMgr.pdfViewCtrl.filePath pathExtension] uppercaseString],
                        createTimeStr,sizeStr];
        self.moreHeaderView.pdfNameLabel.text = self.pdfName;
        self.moreHeaderView.pdfInfoLabel.text = self.pdfInfo;
        self.moreHeaderView.accessibilityLabel = [NSString stringWithFormat:@"%@ %@",self.pdfName,self.pdfInfo];
        FSPDFPage*  pdfPage = [self.extensionsMgr.pdfViewCtrl.currentDoc getPage:0];
        if ([pdfPage isEmpty]){
            dispatch_async(dispatch_get_main_queue(), ^{
                [self.moreHeaderView remakeImageViewContraints:YES];
                self.moreHeaderView.pdfImageView.image = FSImageNamed(@"file_list_PDF");
            });
            return;
        }
        float pageWidth = [pdfPage getWidth];
        float pageHeight = [pdfPage getHeight];
        float thumbnailWidth = 0;
        float thumbnailHeight = 0;
        
        FSRotation rotation = (FSRotation)[self.extensionsMgr.pdfViewCtrl getViewRotation];
        if (rotation == FSRotation90 || rotation == FSRotation270) {
            float tmp = pageWidth;
            pageWidth =  pageHeight;
            pageHeight = tmp;
        }
        
        if (pageWidth > pageHeight) {
            thumbnailWidth = 40;
            thumbnailHeight = thumbnailWidth * pageHeight / pageWidth;
        }else{
            thumbnailHeight = 40;
            thumbnailWidth = thumbnailHeight * pageWidth / pageHeight;
        }

        @weakify(self)
        dispatch_async(dispatch_get_global_queue(0, 0), ^{
            @strongify(self)

            [Utility drawPage:pdfPage targetSize:CGSizeMake(thumbnailWidth, thumbnailHeight) shouldDrawAnnotation:YES progressive:^(UIImage * _Nonnull image, BOOL finish) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    self.coverImage = image;
                    
                    if (!self.thumbnailHidden) {
                        [self.moreHeaderView remakeImageViewContraints:thumbnailWidth < thumbnailHeight];
                        self.moreHeaderView.pdfImageView.image = image;
                    }
                });
            } needPause:^BOOL{
                return  NO;
            } rotation:rotation enableForPrint:NO pdfViewCtrl:self.extensionsMgr.pdfViewCtrl];
        });
    }
}


-(__kindof UITableViewCell *)tableView:(UITableView *)tableView dequeueReusableCellWithIdentifier:(NSString *)identifier forIndexPath:(NSIndexPath *)indexPath withDataItem:(id)dataItem{
    
    if (indexPath.section == 0) {
        identifier = kSectionFirstIdentifier;
    }
    
    return  [super tableView:tableView dequeueReusableCellWithIdentifier:identifier forIndexPath:indexPath withDataItem:dataItem];
}

#pragma mark UITableViewDataSource UITableViewDelegate

- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section{
    UIView *view = [UIView new];
    view.backgroundColor = self.fs_dividerColor_P1;
    if (section == 0) {
        if (self.thumbnailHidden) {
            view.backgroundColor = UIColor.clearColor;
        }else{
            view.backgroundColor = self.fs_dividerColor_P1;
        }
    }
    return view;
}

-(CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(nonnull NSIndexPath *)indexPath{

    return PREFERENCE_CELL_ROW_HEIGHT_NORMAL;
}

- (UIView *)tableView:(UITableView *)tableView viewForFooterInSection:(NSInteger)section{
    UIView *view = [UIView new];
    view.backgroundColor = [UIColor clearColor];
    return view;
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section{

    if (section == 0 || section == 1) {
        return DIVIDE_VIEW_THICKNES;
    }
    return 0.001;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {

    return 2;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {

    if (section == 0) {
        return [[self.groupDic allKeys] count];
    }else if (section == 1){
        return _menuItemArray.count;
    }else{
        return 0;
    }
}

- (void)configureCell:(FSEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(FSPopoverItem *)item{
    cell.selectedBackgroundColor = self.fs_dividerColor_P1;
    cell.contentView.backgroundColor = nil;
    if (!cell.installed) {
        cell.leftMargin = POPOVER_LABEL_LEFT_ICON_MARGINT;
        cell.mediumSpace = cell.leftMargin;
    }
    if (indexPath.section == 0) {
        FSTitleEstimatedTableViewCell *titleCell = (FSTitleEstimatedTableViewCell*)cell;
        if (indexPath.row < _groupTags.count) {
            MoreMenuGroup *groupItem = [_groupDic objectForKey:_groupTags[indexPath.row]];
            
            if (groupItem) {
                titleCell.estimatedTitleLabel.text = groupItem.title;
                if (!titleCell.rightView) {
                    FSItem *moreItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"common_accessory_gray")];
                    moreItem.accessibilityLabel = FSLocalizedForKey(@"kMore");
                    moreItem.userInteractionEnabled = NO;
                    titleCell.rightView = moreItem;
                }
            }
        }
        
    }else if (indexPath.section == 1){
        FSTitleEstimatedTableViewCell *titleCell = (FSTitleEstimatedTableViewCell*)cell;
        if (indexPath.row < _menuItemArray.count) {
            MoreMenuItem *menuItem = _menuItemArray[indexPath.row];
            
            if (menuItem) {
                if (!titleCell.leftView) {
                    UIImageView *imageView = [UIImageView new];
                    imageView.fs_size = CGSizeMake(40.f, 40.f);
                    imageView.contentMode = UIViewContentModeScaleAspectFit;
                    titleCell.leftView = imageView;
                }
                
                ((UIImageView *)titleCell.leftView).image = menuItem.image;
                titleCell.estimatedTitleLabel.text = menuItem.title;
                titleCell.estimatedTitleLabel.textColor = titleCell.titleDefaultTextColor;
                titleCell.userInteractionEnabled = YES;
                
                if(!menuItem.enable) {
                    titleCell.userInteractionEnabled = NO;
                    titleCell.estimatedTitleLabel.textColor = titleCell.fs_iconColor_I2;
                    ((UIImageView *)titleCell.leftView).image = [Utility unableImage:menuItem.image];
                }
            }
        }
    }
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    
    if (indexPath.section == 0) {
        
        MoreMenuGroup *group = [self.groupDic objectForKey:[self.groupTags objectAtIndex:indexPath.row]];
        if ([group.callBack respondsToSelector:@selector(onClick:)]) {
            [self.extensionsMgr.more setHidden:YES animate:YES completion:^{
                [group.callBack onClick:group];
            }];
            
        }else{

//            if (group.tag == TAG_GROUP_PROTECT || group.tag == TAG_GROUP_COMMENT_FIELD) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    NSArray<MoreMenuItem*> *menuItems = [group getItems];
                    if (menuItems) {
                        FSMoreGroupDetailViewController *groupDetailVC = [[FSMoreGroupDetailViewController alloc] init];
                        groupDetailVC.extensionsMgr = self.extensionsMgr;
                        groupDetailVC.title = group.title;
                        groupDetailVC.dataSource = @[menuItems];
                        [self.navigationController pushViewController:groupDetailVC animated:YES];
                    }
                    
                });
//            }
        }
    }else if (indexPath.section == 1) {
        if (indexPath.row < _menuItemArray.count) {
            
            MoreMenuItem *menuItem = _menuItemArray[indexPath.row];
            if ([menuItem.callBack respondsToSelector:@selector(onClick:)]) {
                [self.extensionsMgr.more setHidden:YES animate:YES completion:^{
                    [menuItem.callBack onClick:menuItem];
                }];
            }
        }

    }
    

}

#pragma mark handle Pan Gesture

- (void)handlePanFrom:(UIPanGestureRecognizer*)recognizer{

    if (DEVICE_iPAD && ![self.extensionsMgr fs_currentSceneiPadHorizontalCompact]) {
        return;
    }
    
    static CGPoint touchPoint = {0,0};
    if (recognizer.state == UIGestureRecognizerStateBegan) {
        touchPoint = [recognizer locationInView:self.view];
    }
    BOOL needOperate = YES;
    CGRect tableViewRect = [self.view convertRect:self.tableView.bounds fromView:self.tableView];
    if (CGRectContainsPoint(tableViewRect, touchPoint)) {
        //clicked tableview and tableview offset >0 .
        if (self.tableView.contentOffset.y > 0) {
            needOperate = NO;
        }
    }
    if (self.handlePanBlock) {
        self.handlePanBlock(recognizer,needOperate);
    }
    
}

#pragma mark UIGestureRecognizerDelegate

-(BOOL)gestureRecognizer:(UIGestureRecognizer*) gestureRecognizer shouldRecognizeSimultaneouslyWithGestureRecognizer:(UIGestureRecognizer*)otherGestureRecognizer{

    if ([otherGestureRecognizer.view isKindOfClass:[UITableView class]]) {
            return YES;

        }

    return NO;
}


#pragma mark Empty Data

- (BOOL)emptyDataSetShouldDisplay:(UITableView *)tableView {
    return (_groupDic.count <= 0 && _menuItemArray.count <= 0);
}

- (NSString *)titleForEmptyDataSet{
    return @"";
}

- (void)dealloc{
    
}

@end
