/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "MoreView.h"
#import "MenuView+Private.h"
#import "PrivateDefine.h"
#import "Utility.h"
#import "FSNavigationController.h"
#import "FSMoreTableViewController.h"
#import "UIExtensionsManager+Private.h"
#import "FSMacro.h"
#import "FSMoreAnimateView.h"
#import "FileInformationViewController.h"
#import "FSMoreGroupDetailViewController.h"
#import "FSMainToolbar+Private.h"

#define MinScreenWidth MIN([UIScreen mainScreen].bounds.size.width, [UIScreen mainScreen].bounds.size.height)

//#define iOS11Later ([[UIDevice currentDevice] systemVersion].floatValue >= 11.0)

int const TAG_GROUP_PROTECT = 10;
int const TAG_GROUP_COMMENT_FIELD = 20;

int const TAG_ITEM_REDACTION = 10;
int const TAG_ITEM_PASSWORD = 20;
int const TAG_ITEM_CERTIFICATE = 30;

int const TAG_ITEM_IMPORTCOMMENT = 40;
int const TAG_ITEM_EXPORTCOMMENT = 50;
int const TAG_ITEM_SUMARIZECOMMENT = 60;
int const TAG_ITEM_EXPORTHIGHLIGHTEDTEXT = 65;
int const TAG_ITEM_RESETFORM = 70;
int const TAG_ITEM_IMPORTFORM = 80;
int const TAG_ITEM_EXPORTFORM = 90;

int const TAG_ITEM_SAVE_AS = 110;
int const TAG_ITEM_REDUCEFILESIZE = 120;
int const TAG_ITEM_WIRELESSPRINT = 130;
int const TAG_ITEM_FLATTEN = 140;
int const TAG_ITEM_SCREENCAPTURE = 150;
int const TAG_ITEM_INFO = 160;

@interface MoreMenuView () <UITableViewDelegate, UITableViewDataSource, IWindowFrameChangedListener>

//@property (nonatomic, strong) FSMoreTableViewController *tableViewCtrl;
@property (nonatomic, strong) UINavigationController *navi;
@property (nonatomic, strong) NSMutableDictionary<NSNumber*,MoreMenuGroup*> *groupDic;
@property (nonatomic, strong) NSMutableArray<NSNumber*> *groupTags;
@property (nonatomic, strong) NSMutableArray<MoreMenuItem*> *menuItemArray;
@property (nonatomic, strong) NSMutableArray<MoreMenuItem*> *removedmenuItemArray;
@property (nonatomic, strong) NSMutableDictionary<NSNumber*,MoreMenuGroup*> *moreViewRemoveGropDic;
@property (nonatomic, strong) NSMutableDictionary<NSNumber*,NSMutableArray*> *moreViewCopyDic;
@property (nonatomic, weak) UIExtensionsManager *extensionsMgr;
@property (nonatomic, strong) FSMoreAnimateView *animateView;
@property (nonatomic, assign) CGFloat maxPopoverHeight;
@property (nonatomic, assign) CGFloat oldPanChangeHeight;
@property (nonatomic, assign) CGFloat endPanHeight;
@property (nonatomic, assign) BOOL bUpDirection;
@property (nonatomic, assign) BOOL bSetOldValue;
@property (nonatomic, weak) FSViewPopoverController *morePopoverViewCtrl;
@property (nonatomic, weak) FSViewPopoverController *filePopoverViewCtrl;
@property (nonatomic, assign) BOOL thumbnailItemHidden;
@end

@implementation MoreMenuView

- (id)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    if (self = [super init]) {
        
        _extensionsMgr = extensionsManager;
        _groupDic = [[NSMutableDictionary alloc] init];
        _groupTags = [[NSMutableArray alloc] init];
        _menuItemArray = [[NSMutableArray alloc] init];
        _removedmenuItemArray = [[NSMutableArray alloc] init];
        _maxPopoverHeight = 0;
        _endPanHeight = 0;
        _hidden = YES;
        
        _tableViewCtrl = [self.extensionsMgr.viewCtrlManager createViewCtrlType:FSViewCtrlTypeMoreTablePopover];
        _tableViewCtrl.groupDic = _groupDic;
        _tableViewCtrl.groupTags = _groupTags;
        _tableViewCtrl.menuItemArray = _menuItemArray;
        _navi = [[FSNavigationController alloc] initWithRootViewController:_tableViewCtrl];
        _tableViewCtrl.tableView.backgroundColor = self.fs_backgroundColor_B1;
        _tableViewCtrl.tableView.separatorColor = DividingLineColor;
        _tableViewCtrl.extensionsMgr = self.extensionsMgr;
        _tableViewCtrl.thumbnailHidden = self.thumbnailItemHidden;
   
        @weakify(self)
        _tableViewCtrl.handlePanBlock = ^(UIPanGestureRecognizer * _Nonnull panGesture, BOOL needOperate) {
            @strongify(self)
            if (DEVICE_iPHONE || [_extensionsMgr fs_currentSceneiPadHorizontalCompact]) {
                [self handlePanGesture:panGesture needOperate:needOperate];
            }
        };
   
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(statusBarOrientationChange:)
                                                   name:UIDeviceOrientationDidChangeNotification object:nil];

        [self.extensionsMgr registerWindowFrameChangedListener:self];
    }
    return self;
}

- (void)reloadData {
    [_groupTags removeAllObjects];
    [_groupTags addObjectsFromArray:[self.groupDic allKeys]];
    [self.groupTags sortUsingComparator:^NSComparisonResult(id obj1, id obj2) {

        NSComparisonResult result = [obj1 compare:obj2];
        return result;
    }];

    if (_tableViewCtrl) {
        _tableViewCtrl.groupDic = _groupDic;
        _tableViewCtrl.groupTags = _groupTags;
        _tableViewCtrl.menuItemArray = _menuItemArray;
        [_tableViewCtrl.tableView reloadData];
    }

    UIViewController *firstVC = [self.navi.viewControllers lastObject];
    if (firstVC && [firstVC isKindOfClass:[FSMoreGroupDetailViewController class]]) {
        [((FSMoreGroupDetailViewController*)firstVC).tableView reloadData];
    }
    
}

- (void)setMenuTitle:(NSString *)title {
    _tableViewCtrl.title = title;
}

- (void)setFileLocationIcon:(UIImage *)icon
{
    [_tableViewCtrl setFileLocationIcon:icon];
}

- (NSMutableSet *)listeners{
    if(!_listeners){
        _listeners = [NSMutableSet set];
    }
    return _listeners;
}


- (FSMoreAnimateView *)animateView{
    if (!_animateView) {
        _animateView = [[FSMoreAnimateView alloc] init];
        _animateView.backgroundColor = [UIColor colorWithWhite:0.0f alpha:0.3];
        @weakify(self)
        _animateView.tapBackgroundBlock = ^{
            @strongify(self)
            [self setHidden:YES animate:YES];
        };
    }
    return _animateView;
}

- (UIView *)getContentView {
    return self.navi.view;
}

- (void)getDeepCopyDataSource {
    if (!_moreViewCopyDic) {
        NSMutableDictionary *groupDicArr = [[NSMutableDictionary alloc] init];
        for (id key in _groupDic) {
            MoreMenuGroup *group = [_groupDic objectForKey:key];

            NSMutableArray *mvMenuItemArr = [[NSMutableArray alloc] init];

            for (MoreMenuItem *item in [group getItems]) {
                [mvMenuItemArr addObject:item];
            }

            [groupDicArr setObject:mvMenuItemArr forKey:key];
        }

        _moreViewCopyDic = [groupDicArr mutableCopy];
    }
}

- (void)closeFileInfo:(void (^ __nullable)(void))completion{
    
    if (self.filePopoverViewCtrl) {
        [self.filePopoverViewCtrl dismissViewControllerAnimated:YES completion:completion];
    }
}

- (void)openFileInfo {
    
    FileInformationViewController *fileInfoCtr = [[FileInformationViewController alloc] initWithNibName:nil bundle:[NSBundle bundleForClass:[self class]]];
    [fileInfoCtr setUIExtensionsManager:self.extensionsMgr];
    FSNavigationController *fileInfoNavCtr = [[FSNavigationController alloc] initWithRootViewController:fileInfoCtr];
    if (DEVICE_iPAD) {
        fileInfoNavCtr.modalPresentationStyle = UIModalPresentationFormSheet;
    }else{
        fileInfoNavCtr.modalPresentationStyle = UIModalPresentationFullScreen;
    }
    [[self.extensionsMgr.pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:fileInfoNavCtr animated:YES completion:nil];
    
}

#pragma mark - orientationChange

-(void)statusBarOrientationChange:(NSNotification*)notify{

    [self reLayoutIphoneMoreView];
}

- (void)reLayoutIphoneMoreView{
    dispatch_async(dispatch_get_main_queue(), ^{
        if (self.animateView.superview && self.navi.view.superview && !self.hidden && [self.extensionsMgr fs_currentSceneIsCompact]) {
            self.maxPopoverHeight = [self.tableViewCtrl getTableViewHeight];
            
            [self.navi.view mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.height.mas_equalTo(self.maxPopoverHeight);
                make.centerX.equalTo(self.animateView);
                make.bottom.mas_equalTo(self.animateView.mas_bottom);
                make.width.mas_equalTo(MIN(self.animateView.frame.size.width, MinScreenWidth));
            }];
            [self.animateView layoutIfNeeded];
        }
    });
}


#pragma mark - more view group item element hide/show
- (void)setMoreViewItemHiddenWithGroup:(NSUInteger)groupTag andItemTag:(NSUInteger)itemTag hidden:(BOOL)isHidden {
    [self getDeepCopyDataSource];

    if (isHidden) {
        [self removeMenuItem:groupTag WithItemTag:itemTag];
    } else {
        if ([[_moreViewCopyDic allKeys] containsObject:@(groupTag)]) {
            NSArray *groupArr = [_moreViewCopyDic objectForKey:@(groupTag)];

            MoreMenuItem *waitAddItem = [[MoreMenuItem alloc] init];
            for (MoreMenuItem *item in groupArr) {
                if (item.tag == itemTag) {
                    waitAddItem = item;
                    break;
                }
            }

            MoreMenuGroup *group = [_groupDic objectForKey:@(groupTag)];
            if (!group) {
                group = [[MoreMenuGroup alloc] init];
                group.tag = groupTag;
                switch (groupTag) {
                case TAG_GROUP_PROTECT:
                    group.title = FSLocalizedForKey(@"kProtect");
                    break;
                    case TAG_GROUP_COMMENT_FIELD:
                    group.title = FSLocalizedForKey(@"kComments&Fields");
                    break;
                default:
                    break;
                }
                [self addGroup:group];
            }
            if (![[group getItems] containsObject:waitAddItem]) {
                [self addMenuItem:groupTag withItem:waitAddItem];
            }
        }
    }
}

#pragma mark - more view group element hide/show
- (void)setMoreViewItemHiddenWithGroup:(NSUInteger)groupTag hidden:(BOOL)isHidden {
    if (!_moreViewRemoveGropDic) {
        _moreViewRemoveGropDic = [_groupDic mutableCopy];
    }

    if (isHidden) {
        [self removeGroup:groupTag];
    } else {
        if ([[_moreViewRemoveGropDic allKeys] containsObject:@(groupTag)]) {
            if (![[_groupDic allKeys] containsObject:@(groupTag)]) {
                MoreMenuGroup *group = [_moreViewRemoveGropDic objectForKey:@(groupTag)];
                [self addGroup:group];
            }
        }
    }
}

- (void)addGroup:(MoreMenuGroup *)group {
    [self.groupDic setObject:group forKey:@(group.tag)];
    [_groupTags removeAllObjects];
    [_groupTags addObjectsFromArray:[self.groupDic allKeys]];
    [self.groupTags sortUsingComparator:^NSComparisonResult(id obj1, id obj2) {

        NSComparisonResult result = [obj1 compare:obj2];
        return result;
    }];
    [self reloadData];
}

- (void)addMenuItem:(NSUInteger)groupTag withItem:(MoreMenuItem *)item {
    MoreMenuGroup *group = [self.groupDic objectForKey:@(groupTag)];
    [[group getItems] addObject:item];
    [[group getItems] sortUsingComparator:^NSComparisonResult(id obj1, id obj2) {

        MoreMenuItem *item1 = (MoreMenuItem *) obj1;
        MoreMenuItem *item2 = (MoreMenuItem *) obj2;
        NSComparisonResult result = [@(item1.tag) compare:@(item2.tag)];
        return result;
    }];
}

- (void)removeMenuItem:(NSUInteger)groupTag WithItemTag:(NSUInteger)itemTag {
    if ([self.groupDic objectForKey:@(groupTag)]) {
        MoreMenuGroup *group = [self.groupDic objectForKey:@(groupTag)];
        NSMutableArray *items = [group getItems];
        for (int i = 0; i < [items count]; i++) {
            MoreMenuItem *item = [items objectAtIndex:i];
            if (item.tag == itemTag) {
                [items removeObject:item];
                break;
            }
        }
        [[group getItems] sortUsingComparator:^NSComparisonResult(id obj1, id obj2) {

            MoreMenuItem *item1 = (MoreMenuItem *) obj1;
            MoreMenuItem *item2 = (MoreMenuItem *) obj2;
            NSComparisonResult result = [@(item1.tag) compare:@(item2.tag)];
            return result;
        }];

        if ([group getItems].count == 0) {
            [self removeGroup:groupTag];
            return;
        }

        [self reloadData];
    }
}

- (void)removeGroup:(NSUInteger)tag {
    if ([self.groupDic objectForKey:@(tag)]) {
        [self.groupDic removeObjectForKey:@(tag)];
        [_groupTags removeAllObjects];
        [_groupTags addObjectsFromArray:[self.groupDic allKeys]];

        [self reloadData];
    }
}

- (MoreMenuGroup *)getGroup:(NSUInteger)tag {
    return [self.groupDic objectForKey:@(tag)];
}

- (void)addIndividualMenuItem:(MoreMenuItem *)item {
    
    if (!item) {
        return;
    }
    [_menuItemArray addObject:item];
    [_menuItemArray sortUsingComparator:^NSComparisonResult(id obj1, id obj2) {

        MoreMenuItem *item1 = (MoreMenuItem *) obj1;
        MoreMenuItem *item2 = (MoreMenuItem *) obj2;
        NSComparisonResult result = [@(item1.tag) compare:@(item2.tag)];
        return result;
    }];
}

- (void)removeIndividualMenuItemWithTag:(NSUInteger)itemTag {

    for (int i = 0; i < [_menuItemArray count]; i++) {
        MoreMenuItem *item = [_menuItemArray objectAtIndex:i];
        if (item.tag == itemTag) {
            [_menuItemArray removeObject:item];
            break;
        }
    }
    [self reloadData];
}


- (MoreMenuItem*)getIndividualMenuItemWithTag:(NSUInteger)itemTag{
    for (int i = 0; i < [_menuItemArray count]; i++) {
        MoreMenuItem *item = [_menuItemArray objectAtIndex:i];
        if (item.tag == itemTag) {
            return item;
        }
    }
    return nil;
}

- (void)setIndividualMenuItemHiddenWithItemTag:(NSUInteger)itemTag hidden:(BOOL)isHidden{

    if (isHidden) {
        MoreMenuItem *hiddenMenuItem = nil;
        for (MoreMenuItem *menuItem in self.menuItemArray) {
            if (menuItem.tag == itemTag) {
                hiddenMenuItem = menuItem;
                break;
            }
        }
        if (hiddenMenuItem) {
            [self.menuItemArray removeObject:hiddenMenuItem];
            [self.removedmenuItemArray addObject:hiddenMenuItem];
        }

    } else {

        MoreMenuItem *hiddenMenuItem = nil;
        for (MoreMenuItem *menuItem in self.removedmenuItemArray) {
            if (menuItem.tag == itemTag) {
                hiddenMenuItem = menuItem;
                break;
            }
        }
        if (hiddenMenuItem) {
            [self.removedmenuItemArray removeObject:hiddenMenuItem];
            [self addIndividualMenuItem:hiddenMenuItem];
        }
    }
}

- (void)setTitleBarHidden:(BOOL)isHidden{
    self.thumbnailItemHidden = isHidden;
    self.tableViewCtrl.thumbnailHidden = self.thumbnailItemHidden;
    [self setIndividualMenuItemHiddenWithItemTag:TAG_ITEM_INFO hidden:self.thumbnailItemHidden];
    [self reloadData];
}

- (void)registerEventListener:(id<IMoreMenuViewListener>)listener{
    [self.listeners addObject:listener];
}
- (void)unregisterEventListener:(id<IMoreMenuViewListener>)listener{
    [self.listeners removeObject:listener];
}

#pragma mark MoreView panGesture Animate

- (void)handlePanGesture:(UIPanGestureRecognizer *)recognizer needOperate:(BOOL)needOperate{
    
    if (!needOperate) {
        self.bSetOldValue = NO;
        return;
    }
    
    CGPoint translatedPoint = [recognizer translationInView:self.tableViewCtrl.navigationController.view];
    if ([recognizer state] == UIGestureRecognizerStateBegan) {

    }else if ([recognizer state] == UIGestureRecognizerStateChanged) {

        if (translatedPoint.y != self.oldPanChangeHeight && self.bSetOldValue) {
            CGRect naviRect = self.navi.view.frame;
            naviRect.origin.y += translatedPoint.y - self.oldPanChangeHeight;
            if (naviRect.origin.y <= self.animateView.frame.size.height-self.maxPopoverHeight) {
                naviRect.origin.y = self.animateView.frame.size.height-self.maxPopoverHeight;
            }

            [self.navi.view mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.top.mas_equalTo(naviRect.origin.y);
                make.height.mas_equalTo(self.maxPopoverHeight);
                make.centerX.equalTo(self.animateView);
                make.width.mas_equalTo(MIN(self.animateView.frame.size.width, MinScreenWidth));
            }];
            
            if (translatedPoint.y < self.oldPanChangeHeight) {
                self.bUpDirection = YES;
            }else{
                self.bUpDirection = NO;
            }
        }
        self.oldPanChangeHeight = translatedPoint.y;
        self.bSetOldValue = YES;
    }else if (([recognizer state] == UIGestureRecognizerStateEnded) || ([recognizer state] == UIGestureRecognizerStateCancelled)) {

        self.endPanHeight = translatedPoint.y;
        self.bSetOldValue = NO;
        CGRect naviRect = self.navi.view.frame;
        int distance = naviRect.origin.y - (self.animateView.frame.size.height-self.maxPopoverHeight);

        if (distance > 0 ) {
            if (distance < 100  && self.bUpDirection ) {
                self.hidden = NO;
            }else{
                self.hidden = YES;
            }
        }

    }
}

#pragma mark MoreView show Animate

- (void)setHidden:(BOOL)hidden{
    [self setHidden:hidden animate:YES];
}

- (void)setHidden:(BOOL)hidden animate:(BOOL)animated{
    [self setHidden:hidden animate:animated completion:nil];
}

- (void)setHidden:(BOOL)hidden animate:(BOOL)animated completion:(void (^ __nullable)(void))completion{
    
    if (_hidden == hidden && hidden) {
        return;
    }
    _hidden = hidden;
    
    if (_hidden) {
        [self.extensionsMgr delayAutoFullScreen];
    }else{
        [self.extensionsMgr cancelAutoFullScreen];
    }
    
    void (^block)(void) = ^{
        !completion ? : completion();
        if (hidden) {
            [self.navi popToRootViewControllerAnimated:NO];
            [self.extensionsMgr delayAutoFullScreen];
        }
    };
    if ([self.extensionsMgr fs_currentSceneIsCompact]) {
        [self setCompactScreenHidden:hidden animate:animated completion:block];
    }else{
        [self setRegularScreenHidden:hidden animate:animated completion:block];
    }
    
    for (id<IMoreMenuViewListener> listener in self.listeners) {
        if ([listener respondsToSelector:@selector(onMoreMenuChanged:)]) {
            [listener onMoreMenuChanged:hidden];
        }
    }
}
    
- (void)setCompactScreenHidden:(BOOL)hidden animate:(BOOL)animated completion:(void (^ __nullable)(void))completion{
    self.maxPopoverHeight = [self.tableViewCtrl getTableViewHeight];
    
    if (hidden) {
        void (^hideMenu)(void) = ^{
            
            [self.navi.view mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.top.equalTo(self.animateView.mas_bottom);
                make.height.mas_equalTo(self.maxPopoverHeight);
                make.centerX.equalTo(self.animateView);
                make.width.mas_equalTo(MIN(self.animateView.frame.size.width, MinScreenWidth));
            }];
            self.animateView.backgroundColor = [UIColor colorWithWhite:0.0f alpha:0.1];
            [self.animateView layoutIfNeeded];
        };
        
        if (animated) {
            [UIView setAnimationsEnabled:YES];
            [UIView animateWithDuration:0.3
                             animations:hideMenu
                             completion:^(BOOL finished) {
                [self.navi.view removeFromSuperview];
                [self.animateView removeFromSuperview];
                if (completion) {
                    completion();
                }
            }];
        } else {
            hideMenu();
            [self.animateView removeFromSuperview];
            if (completion) {
                completion();
            }
        }
    }else{
        
        if (!self.animateView.superview) {
            [self.extensionsMgr.pdfViewCtrl addSubview:self.animateView];
            [self.animateView addSubview:self.navi.view];
            [self.animateView mas_makeConstraints:^(MASConstraintMaker *make) {
                make.left.top.bottom.equalTo(self.extensionsMgr.pdfViewCtrl);
                make.width.equalTo(self.extensionsMgr.pdfViewCtrl);
                
            }];
    
            [self.navi.view mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.top.equalTo(self.animateView.mas_bottom);
                make.height.mas_equalTo(self.maxPopoverHeight);
                make.centerX.equalTo(self.animateView);
                make.width.mas_equalTo(MIN(self.extensionsMgr.pdfViewCtrl.frame.size.width, MinScreenWidth));
            }];
            [self.animateView layoutIfNeeded];
        }
        
        void (^hideMenu)(void) = ^{
            
            [self.navi.view mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.height.mas_equalTo(self.maxPopoverHeight);
                make.centerX.bottom.equalTo(self.animateView);
                make.width.mas_equalTo(MIN(self.animateView.frame.size.width, MinScreenWidth));
            }];
            self.animateView.backgroundColor = [UIColor colorWithWhite:0.0f alpha:0.3];
            [self.animateView layoutIfNeeded];
        };
        
        if (animated) {
            [UIView setAnimationsEnabled:YES];
            [UIView animateWithDuration:0.3
                             animations:hideMenu
                             completion:^(BOOL finished) {
                if (completion) {
                    completion();
                }
            }];
        } else {
            hideMenu();
            if (completion) {
                completion();
            }
        }
    }
}

- (void)setRegularScreenHidden:(BOOL)hidden animate:(BOOL)animated completion:(void (^ __nullable)(void))completion{
    
    self.maxPopoverHeight = [self.tableViewCtrl getTableViewHeight];
    self.navi.view.frame = CGRectMake(0, 0, 390, self.maxPopoverHeight);
    
    if (hidden) {
        if (self.morePopoverViewCtrl) {
            self->_hidden = YES;
            [self.morePopoverViewCtrl dismissViewControllerAnimated:animated completion:completion];
            [self.extensionsMgr delayAutoFullScreen];
            self.morePopoverViewCtrl = nil;
        }
    }else{
        self.morePopoverViewCtrl = [self.extensionsMgr.viewCtrlManager popPopoverCustomView:[self getContentView] sourceView:^UIView * _Nonnull(UIPopoverPresentationController * _Nonnull popover) {
            popover.permittedArrowDirections = UIPopoverArrowDirectionUp;
            UIView *btnView = nil;
            if (self.extensionsMgr.smallTopToolbar && !self.extensionsMgr.smallTopToolbar.hidden) {
                btnView = self.extensionsMgr.smallTopToolbar.moreBtn;
            }else{
                FSMainToolbarItem *moreItem = [self.extensionsMgr.toolbarManager getMainItemWithType:FSMainToolbarItemTypeMore];
                btnView = moreItem.item;
            }
            return btnView;
        }];
        @fs_weakify(self)
        self.morePopoverViewCtrl.dismissCompletion = ^{
            @fs_strongify(self)
            [self setHidden:YES animate:NO];
        };
        self.morePopoverViewCtrl.autoresizesFollowContnetView = YES;
        !completion ? : completion();
    }
}

#pragma mark -IWindowFrameChangedListener
- (void)onWindowFrameChanged:(CGRect)newFrame horizontalSizeClassChanged:(BOOL)changed {
    if (DEVICE_iPAD && changed) {
        if ([self.extensionsMgr fs_currentSceneiPadHorizontalCompact]) {
            if (self.morePopoverViewCtrl && !self.hidden) {
                @fs_weakify(self)
                [self setRegularScreenHidden:YES animate:YES completion:^{
                    @fs_strongify(self)
                    self.morePopoverViewCtrl = nil;
                    self.hidden = NO;
                }];
            }else if (!self.morePopoverViewCtrl && !self.hidden){
                [self statusBarOrientationChange:nil];
            }
        }else{
            if (!self.morePopoverViewCtrl && !self.hidden) {
                @fs_weakify(self)
                [self setCompactScreenHidden:YES animate:YES completion:^{
                    @fs_strongify(self)
                    self.hidden = NO;
                }];
            }

        }
    }else if ([self.extensionsMgr fs_currentSceneiPadHorizontalCompact]){
        [self reLayoutIphoneMoreView];
    }
        
}

@end
