/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "MoreModule.h"
#import "FileInformationViewController.h"
#import "MoreView.h"
#import "ScreenCaptureViewController.h"
#import "ScreenCaptureNaviController.h"
#import "SaveAsViewController.h"
#import "MenuView+Private.h"
#import "PasswordModule.h"
#import "FSReduceFileSizeView.h"
#import "FormModule.h"
#import "FSPermissionUtilty.h"

@interface MoreModule ()

@property (nonatomic, weak) FSMainToolbarItem *moreItem;
@property (nonatomic, weak) MoreMenuView *moreMenu;

@property (nonatomic, strong) MoreMenuGroup *othersGroup;
@property (nonatomic, strong) MoreMenuItem *saveItem;
@property (nonatomic, strong) MoreMenuItem *fileInfoItem;
@property (nonatomic, strong) MoreMenuItem *reduceFileSizeItem;
@property (nonatomic, strong) MoreMenuItem *wirelessPrintItem;
@property (nonatomic, strong) MoreMenuItem *flattenItem;
@property (nonatomic, strong) MoreMenuItem *cropScreenItem;
@property (nonatomic, strong) MoreMenuItem *infoItem;

@property (nonatomic, weak) FSPDFViewCtrl *pdfViewCtrl;
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
@property (nonatomic, strong) FSReduceFileSizeView *noticeView;
@end

@implementation MoreModule

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        self.extensionsManager = extensionsManager;
        self.pdfViewCtrl = extensionsManager.pdfViewCtrl;
        [self loadModule];
    }
    return self;
}

- (void)loadModule {
    [_extensionsManager registerRotateChangedListener:self];
    [_extensionsManager.pdfViewCtrl registerDocEventListener:self];
    _extensionsManager.more = [[MoreMenuView alloc] initWithUIExtensionsManager:_extensionsManager];
    self.moreMenu = _extensionsManager.more;

    self.moreItem = [_extensionsManager.toolbarManager addMainItemWithType:FSMainToolbarItemTypeMore];
    [self.moreItem.item addTarget:self action:@selector(onClickMoreButton:) forControlEvents:UIControlEventTouchUpInside];

    [self addIndividualItems];    
}

- (void)dealloc
{
    [_extensionsManager unregisterRotateChangedListener:self];
    [_extensionsManager.pdfViewCtrl unregisterDocEventListener:self];
}

- (void)addIndividualItems{
    //add individual menuItems
    self.saveItem = [[MoreMenuItem alloc] init];
    self.saveItem.tag =  TAG_ITEM_SAVE_AS;
    self.saveItem.callBack = self;
    self.saveItem.title = FSLocalizedForKey(@"kSaveAs");
    self.saveItem.image = FSImageNamedAndNormalColor(@"more_save_save_as");
    [self.moreMenu addIndividualMenuItem:self.saveItem];

    self.reduceFileSizeItem = [[MoreMenuItem alloc] init];
    self.reduceFileSizeItem.tag = TAG_ITEM_REDUCEFILESIZE;
    self.reduceFileSizeItem.callBack = self;
    self.reduceFileSizeItem.title = FSLocalizedForKey(@"kReduceFileSize");
    self.reduceFileSizeItem.image = FSImageNamedAndNormalColor(@"more_reduce_size");;
    [self.moreMenu addIndividualMenuItem:self.reduceFileSizeItem];
    
    self.wirelessPrintItem = [[MoreMenuItem alloc] init];
    self.wirelessPrintItem.tag = TAG_ITEM_WIRELESSPRINT;
    self.wirelessPrintItem.callBack = self;
    self.wirelessPrintItem.title = FSLocalizedForKey(@"kAirPrint");
    self.wirelessPrintItem.image = FSImageNamedAndNormalColor(@"more_print_icon");
    [self.moreMenu addIndividualMenuItem:self.wirelessPrintItem];
    
    self.flattenItem = [[MoreMenuItem alloc] init];
    self.flattenItem.tag = TAG_ITEM_FLATTEN;
    self.flattenItem.callBack = self;
    self.flattenItem.title = FSLocalizedForKey(@"kFlatten");
    self.flattenItem.image = FSImageNamedAndNormalColor(@"more_flatten_Icon");
    [self.moreMenu addIndividualMenuItem:self.flattenItem];
    
    self.cropScreenItem = [[MoreMenuItem alloc] init];
    self.cropScreenItem.tag = TAG_ITEM_SCREENCAPTURE;
    self.cropScreenItem.callBack = self;
    self.cropScreenItem.title = FSLocalizedForKey(@"kScreenCapture");
    self.cropScreenItem.image = FSImageNamedAndNormalColor(@"more_screen_screen_icon");
    [self.moreMenu addIndividualMenuItem:self.cropScreenItem];
    
    self.infoItem = [[MoreMenuItem alloc] init];
    self.infoItem.tag = TAG_ITEM_INFO;
    self.infoItem.callBack = self;
    self.infoItem.title = FSLocalizedForKey(@"kShowInfo");
    self.infoItem.image = FSImageNamedAndNormalColor(@"more_file_info_icon");
    [self.moreMenu addIndividualMenuItem:self.infoItem];
}

- (void)onClickMoreButton:(UIButton *)button {
    if (_extensionsManager.moreItemClickedCallback) {
        _extensionsManager.moreItemClickedCallback(button);
    }
    else
    {
        if (![self.extensionsManager allowFormPermission]) {
            FormModule *formModule = (FormModule *)[self.extensionsManager getModuleByName:@"Form"];
            [formModule resetMoreViewItem];
        }
        _extensionsManager.currentAnnot = nil;
        _extensionsManager.currentWidget = nil;
        self.moreMenu.hidden = NO;
    }
}

- (void)onClick:(id<MoreItemProtocol>)item {

    [self operateMoreClick:item];

}

- (void)operateMoreClick:(id<MoreItemProtocol>)item{
    if (item.tag == TAG_ITEM_SAVE_AS){
        [_extensionsManager onUIInteractElementClicked:FSUIElementReadingMoreSaveAs];
        [self openSaveAs];

    }else if (item.tag == TAG_ITEM_REDUCEFILESIZE) {
        [_extensionsManager onUIInteractElementClicked:FSUIElementReadingMoreReduceFileSize];
        @weakify(self)
        [_extensionsManager.permissionProvider checkPermission:FSFunctionReduce checkPermissionState:^(FSPermissionState state) {
            @strongify(self)
            if (state == FSPermissionStateShow) {
                [self reduceFileSize];
            }
        }];
    }else if (item.tag == TAG_ITEM_WIRELESSPRINT) {
        [_extensionsManager onUIInteractElementClicked:FSUIElementReadingMorePrint];
        [self wirelessPrint];
        
    }else if (item.tag == TAG_ITEM_FLATTEN) {
        [_extensionsManager onUIInteractElementClicked:FSUIElementReadingMoreFlatten];
        @weakify(self)
        [_extensionsManager.permissionProvider checkPermission:FSFunctionFlatten checkPermissionState:^(FSPermissionState state) {
            @strongify(self)
            if (state == FSPermissionStateShow) {
                [self flatten];
            }
        }];

    }else if(item.tag == TAG_ITEM_SCREENCAPTURE) {
        [_extensionsManager onUIInteractElementClicked:FSUIElementReadingMoreScreenCapture];
        
        if (!([_extensionsManager.permissionProvider checkPermission:FSFunctionCanSaveToLocal] == FSPermissionStateShow)) {
            return;
        }
        [self cropScreen];
    }else if(item.tag == TAG_ITEM_INFO) {
        if (!_moreMenu.tableViewCtrl.thumbnailHidden) {
            [self->_extensionsManager.more openFileInfo];
        }
        [_extensionsManager onUIInteractElementClicked:FSUIElementReadingMoreFileProperty];
        
    }
}

- (void)openSaveAs{
    SaveAsViewController *VC = [self.extensionsManager.viewCtrlManager createViewCtrlType:FSViewCtrlTypeSaveAs];
    FSNavigationController *saveNavCtrl = [[FSNavigationController alloc] initWithRootViewController:VC];
    [VC setUIExtensionsManager:self.extensionsManager];
    if (DEVICE_iPAD) {
        saveNavCtrl.modalPresentationStyle = UIModalPresentationFormSheet;
    }else{
        saveNavCtrl.modalPresentationStyle = UIModalPresentationFullScreen;
    }
    [self.extensionsManager.viewCtrlManager presentViewController:saveNavCtrl animated:YES completion:nil];
}

- (void)openFileInfo {
    FileInformationViewController *fileInfoCtr = [[FileInformationViewController alloc] initWithNibName:nil bundle:[NSBundle bundleForClass:[self class]]];
    [fileInfoCtr setUIExtensionsManager:_extensionsManager];
    FSNavigationController *fileInfoNavCtr = [[FSNavigationController alloc] initWithRootViewController:fileInfoCtr];
    if (DEVICE_iPAD) {
        fileInfoNavCtr.modalPresentationStyle = UIModalPresentationFormSheet;
        fileInfoNavCtr.modalTransitionStyle = UIModalTransitionStyleCoverVertical;
    }
    [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:fileInfoNavCtr animated:YES completion:nil];
}

- (void)reduceFileSize {
    
    BOOL docHasSignature = [Utility isDocumentSigned:[_pdfViewCtrl currentDoc]];
    BOOL allowEdit = docHasSignature ? NO : [FSPermissionUtilty canModifyContents:_extensionsManager];
    BOOL canFlatten = !(!allowEdit || ![FSPermissionUtilty canAddAnnot:_extensionsManager]);
    BOOL canOptimizeDoc = canFlatten && ![[_pdfViewCtrl currentDoc] isXFA] && ![Utility simpleCheckPDFA:[_pdfViewCtrl currentDoc]];
    
    if (([_pdfViewCtrl isRMSProtected] && ![_pdfViewCtrl isOwner]) || !canOptimizeDoc) {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kRMSNoAccess") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
        [alertController addAction:action];
        [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
        return;
    }
    
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kReduceFileSizeDescription") preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel") style:UIAlertActionStyleCancel handler:nil];
    UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                     style:UIAlertActionStyleDefault
                                                   handler:^(UIAlertAction *action) {
        [self.extensionsManager supportsMultipleScenesOp:^(UIExtensionsManager * _Nonnull ext) {
            if (ext.currentAnnot) {
                [ext setCurrentAnnot:nil];
            }
        }];
        [self operatePdfDocWithFlatten:NO];
    }];
    [alertController addAction:cancelAction];
    [alertController addAction:action];
    [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
}

- (void)flatten {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kFlatten") message:FSLocalizedForKey(@"kFlattenTip") preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel") style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {
        
    }];
    UIAlertAction *flattenAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kFlatten") style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        BOOL docHasSignature = [Utility isDocumentSigned:[self->_pdfViewCtrl currentDoc]];
        BOOL allowEdit = docHasSignature ? NO : [FSPermissionUtilty canModifyContents:self->_extensionsManager];
        BOOL canFlatten = !(!allowEdit || ![FSPermissionUtilty canAddAnnot:self->_extensionsManager]);
        
        if (([self->_pdfViewCtrl isRMSProtected] && ![self->_pdfViewCtrl isOwner]) || !canFlatten)  {
            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kRMSNoAccess") preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
            [alertController addAction:action];
            [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
            return;
        }
        [self.extensionsManager supportsMultipleScenesOp:^(UIExtensionsManager * _Nonnull ext) {
            if (ext.currentAnnot) {
                [ext setCurrentAnnot:nil];
            }
        }];
        [self operatePdfDocWithFlatten:YES];
    }];
    [alertController addAction:cancelAction];
    [alertController addAction:flattenAction];
    [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
}

- (void)wirelessPrint {
    if (![Utility canPrintDocument:_pdfViewCtrl.getDoc]) {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kRMSNoAccess") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
        [alertController addAction:action];
        [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
        return;
    }
    
    UIPrintInteractionCompletionHandler completion = ^(UIPrintInteractionController *_Nonnull printInteractionController, BOOL completed, NSError *_Nullable error) {
        if (error) {
            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(error.localizedDescription) preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
            [alertController addAction:action];
            [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
        }
    };
    NSString *fileName = self.pdfViewCtrl.filePath.lastPathComponent;
    if ([self.pdfViewCtrl isDynamicXFA]) {
        if (DEVICE_iPHONE) {
            [Utility printXFADoc:[self.pdfViewCtrl getXFADoc] animated:YES jobName:fileName delegate:nil completionHandler:completion];
        } else {
            CGRect fromRect = [self.pdfViewCtrl convertRect:self.moreItem.item.bounds fromView:self.moreItem.item];
            [Utility printXFADoc:[self.pdfViewCtrl getXFADoc] fromRect:fromRect inView:self.pdfViewCtrl animated:YES jobName:fileName delegate:nil completionHandler:completion];
        }
        return ;
    }
    
    if (DEVICE_iPHONE) {
        [Utility printDoc:[self.pdfViewCtrl currentDoc] animated:YES inWindow:[self.pdfViewCtrl fs_currentSceneKeyWindow] jobName:fileName delegate:nil completionHandler:completion];
    } else {
        CGRect fromRect = [self.pdfViewCtrl convertRect:self.moreItem.item.bounds fromView:self.moreItem.item];
        [Utility printDoc:[self.pdfViewCtrl currentDoc] fromRect:fromRect inView:self.pdfViewCtrl animated:YES inWindow:[self.pdfViewCtrl fs_currentSceneKeyWindow] jobName:fileName delegate:nil completionHandler:completion];
    }
}

- (void)cropScreen {
    if (![Utility canExtractContents:_pdfViewCtrl]) {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kRMSNoAccess") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
        [alertController addAction:action];
        [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
        return;
    }
    
    if (![Utility canCopyForAssess:self.pdfViewCtrl]) {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kRMSNoAccess") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:nil];
        [alertController addAction:action];
        [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
        return;
    }
    
    __weak __typeof__(self) weakSelf = self;
    void(^start)(void) = ^(void)
    {
        [weakSelf.extensionsManager setFullScreen:YES];

        ScreenCaptureViewController *screenCaptureViewController = [[ScreenCaptureViewController alloc] initWithNibName:@"ScreenCaptureViewController" bundle:[NSBundle bundleForClass:[self class]]];

        screenCaptureViewController.pdfViewCtrl = self->_extensionsManager.pdfViewCtrl;
        screenCaptureViewController.screenCaptureCompelementHandler = ^(CGRect area) {
        };
        screenCaptureViewController.screenCaptureClosedHandler = ^() {
            [weakSelf.extensionsManager setFullScreen:NO];
        };
        
        ScreenCaptureNaviController *shotNavCtr = [[ScreenCaptureNaviController alloc] initWithRootViewController:screenCaptureViewController];
        if (DEVICE_iPAD) {
            shotNavCtr.modalPresentationStyle = UIModalPresentationFullScreen;
            shotNavCtr.modalTransitionStyle = UIModalTransitionStyleCoverVertical;
        }else{
            shotNavCtr.modalPresentationStyle = UIModalPresentationFullScreen;
        }
        [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:shotNavCtr animated:YES completion:nil];
    
        
    };
    
    start();
}

- (NSString *)pathForTemporaryFileWithSuffix:(NSString *)suffix
{
    CFUUIDRef uuid = CFUUIDCreate(NULL);
    assert(uuid != NULL);

    CFStringRef uuidStr = CFUUIDCreateString(NULL, uuid);
    assert(uuidStr != NULL);

    NSString *result = [NSTemporaryDirectory() stringByAppendingPathComponent:[NSString stringWithFormat:@"%@-%@", uuidStr, suffix]];
    assert(result != nil);

    CFRelease(uuidStr);
    CFRelease(uuid);

    return result;
}

- (void)operatePdfDocWithFlatten:(BOOL)bFlatten{
    
    NSString *pdfFilePath = self.extensionsManager.pdfViewCtrl.filePath;
    NSFileManager *fileManager = [[NSFileManager alloc] init];
    unsigned long long beforeCompressSize = [self pdfFileSize];
    
    if ([fileManager fileExistsAtPath:pdfFilePath]) {
        
        if (self.extensionsManager.isViewSignedDocument) {
            FSAlertViewCtrlShowOnCtrl(FSLocalizedForKey(@"kWarning"), FSLocalizedForKey(@"kCouldNotSavedTip"), self.extensionsManager.pdfViewCtrl);
            return;
        }
        
        
        if (pdfFilePath.length) {
            
            if (bFlatten) {
                if (self->_extensionsManager.currentToolHandler.type == FSAnnotInk || self->_extensionsManager.currentToolHandler.type == FSAnnotPSInk) {
                    [self->_extensionsManager.currentToolHandler onDeactivate];
                }
                dispatch_semaphore_t wait = dispatch_semaphore_create(0);
                [[self->_pdfViewCtrl fs_currentSceneTopMostViewController].view fs_showHUDLoading:FSLocalizedForKey(@"kSaving") process:^{
                    FSXFADoc *xfaDoc = [self.pdfViewCtrl getXFADoc];
                    if (![xfaDoc isEmpty] && xfaDoc && [xfaDoc getType] == FSXFADocDynamic) {
                        [xfaDoc flattenTo:pdfFilePath];
                    }else{
                        FSPDFDoc *doc = [self.pdfViewCtrl currentDoc];
                        for (int i = 0 ; i < [doc getPageCount]; i ++){
                            [self.pdfViewCtrl flattenPageAtIndex:i display:YES options:FSPDFPageFlattenAll];
                            dispatch_async(dispatch_get_main_queue(), ^{
                                [self.pdfViewCtrl refresh:CGRectZero pageIndex:i needRender:YES clearCache:YES];
                            });
                        }
                        
                    }
                    dispatch_semaphore_signal(wait);
                }];
                dispatch_semaphore_wait(wait, DISPATCH_TIME_FOREVER);
                dispatch_async(dispatch_get_main_queue(), ^{
                    
                    [[NSNotificationCenter defaultCenter] postNotificationName:NSNotificationNameAnnotLsitUpdate object:nil];
                    [[NSNotificationCenter defaultCenter] postNotificationName:NSNotificationNameAttachmentLsitUpdate object:nil];
                    [[NSNotificationCenter defaultCenter] postNotificationName:NSNotificationNameFlattenDoc object:nil];
                    
                    self.extensionsManager.isDocModified = YES;
                    [self.extensionsManager clearUndoRedo];

                    self.extensionsManager.isDocModified = YES;
                    [self fs_toastWithMsg:FSLocalizedForKey(@"kFlattenSuccessful") inView:[self->_pdfViewCtrl fs_currentSceneTopMostViewController].view];
                });
                
            }else{
                //[fileManager removeItemAtPath:pdfFilePath error:nil];
                NSString *tmpFilePath = [self pathForTemporaryFileWithSuffix:[pdfFilePath lastPathComponent]];
                [[self->_pdfViewCtrl fs_currentSceneTopMostViewController].view fs_showHUDLoading:FSLocalizedForKey(@"kOptimizing") process:^{
                    
                    FSImageSettings *imageSettings = [[FSImageSettings alloc] init];
                    [imageSettings setQuality:FSImageSettingsImageCompressQualityMedium];
                    [imageSettings setCompressionMode:FSImageSettingsImageCompressjpeg];
                    
                    FSMonoImageSettings *monoSettings = [[FSMonoImageSettings alloc] init];
                    [monoSettings setQuality:FSMonoImageSettingsImageCompressQualityLossy];
                    [monoSettings setCompressionMode:FSMonoImageSettingsImageCompressjbig2];
                    
                    FSOptimizerSettings *optitmizeSettings = [[FSOptimizerSettings alloc] init];
                    [optitmizeSettings setColorGrayImageSettings:imageSettings];
                    [optitmizeSettings setMonoImageSettings:monoSettings];
                    [optitmizeSettings setOptimizerOptions:FSOptimizerSettingsOptimizerCompressImages];
                    
                    FSProgressive *progressive;
                    
                    @try {
                        progressive = [FSOptimizer optimize:[self.pdfViewCtrl currentDoc] settings:optitmizeSettings pause:nil];
                        
                        if (progressive) {
                            FSProgressiveState proState = FSProgressiveToBeContinued;
                            while (proState == FSProgressiveToBeContinued) {
                                proState = [progressive resume];
                            }
                            if (proState != FSProgressiveFinished){
                                return;
                            }
                        }
                    } @catch (NSException *exception) {
                        dispatch_async(dispatch_get_main_queue(), ^{
                            [UIAlertController fs_alert_OK_WithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kOptimizeFailed") onCtrl:self.pdfViewCtrl.fs_viewController];
                        });
                        return;
                    }
                    
                    dispatch_async(dispatch_get_main_queue(), ^{
                        if (self.pdfViewCtrl) {
                            if ([[self.pdfViewCtrl currentDoc] getSignatureCount] > 0) {
                                [self.pdfViewCtrl saveDoc:tmpFilePath flag:FSPDFDocSaveFlagIncremental];
                            }else{
                                [self.pdfViewCtrl saveDoc:tmpFilePath flag:FSPDFDocSaveFlagRemoveRedundantObjects];
                            }
                        }
                        
                        if ([fileManager fileExistsAtPath:tmpFilePath]) {
                            NSDictionary<NSFileAttributeKey, id> *attr = [fileManager attributesOfItemAtPath:tmpFilePath error:nil];
                            if (attr.fileSize <= 0) return;
                            
                            BOOL sucess = [fileManager removeItemAtPath:pdfFilePath error:nil];
                            if (sucess) [fileManager moveItemAtPath:tmpFilePath toPath:pdfFilePath error:nil];
                        }
                        
                        [self reopenPdfDoc:pdfFilePath beforeSize:beforeCompressSize];
                    });
                }];
            }
        }
    }
}

- (void)reopenPdfDoc:(NSString*)pdfFilePath beforeSize:(unsigned long long)beforeSize{
    
    __block unsigned long long afterCompressSize = 0;
    
    dispatch_async(dispatch_get_main_queue(), ^{
        afterCompressSize = [self pdfFileSize];
        if (self.extensionsManager) {
            void(^popCompressTipBlcok)(void) = ^(){
                if (beforeSize > 0) {
                    BOOL isSuccess = YES;
                    if (afterCompressSize > beforeSize) {
                        isSuccess = NO;
                    }
                    NSString *msgStr = [NSString stringWithFormat:FSLocalizedForKey(@"kOptimizeSizeTip"),[Utility displayFileSize:beforeSize],[Utility displayFileSize:afterCompressSize]];
                    self.noticeView = [[FSReduceFileSizeView alloc] initWithState:isSuccess msg:msgStr];
                    [self.extensionsManager.pdfViewCtrl addSubview:self.noticeView];
                    [self.noticeView mas_makeConstraints:^(MASConstraintMaker *make) {
                        make.left.right.top.bottom.equalTo(self.extensionsManager.pdfViewCtrl);
                    }];
                }
            };
            if ([self.extensionsManager.delegate respondsToSelector:@selector(uiextensionsManager:openNewDocAtPath:shouldCloseCurrentDoc:completionCallBack:)]) {
                [self.extensionsManager.delegate uiextensionsManager:self.extensionsManager openNewDocAtPath:pdfFilePath shouldCloseCurrentDoc:YES completionCallBack:^{}];
                dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                    popCompressTipBlcok();
                });
            } else {
                [self.extensionsManager.pdfViewCtrl openDoc:pdfFilePath
                                                   password:self.extensionsManager.passwordModule.inputPassword
                                                 completion:^(FSErrorCode error) {
                    if (error != FSErrSuccess) {
                        [self.extensionsManager.pdfViewCtrl closeDoc:nil];
                        if (self.extensionsManager.goBack)
                            self.extensionsManager.goBack();
                    }else{
                        popCompressTipBlcok();
                    }
                }];
            }
            self.extensionsManager.isDocModified = NO;
            self.extensionsManager.more.hidden = YES;
        }
    });

}

- (unsigned long long)pdfFileSize{
    unsigned long long fileSize = 0;
    NSFileManager *manager = [NSFileManager defaultManager];
    if ([manager fileExistsAtPath:self.pdfViewCtrl.filePath]) {
        fileSize = [[manager attributesOfItemAtPath:self.pdfViewCtrl.filePath error:nil] fileSize];
    }
    return fileSize;
}

#pragma mark --IDocEventListener
- (void)onDocClosed:(FSPDFDoc *)document error:(int)error{
    if (self.noticeView && self.noticeView.superview) {
        [self.noticeView removeFromSuperview];
        self.noticeView = nil;
    }
}

- (void)onDocOpened:(FSPDFDoc *)document error:(int)error{
    
    BOOL docHasSignature = [Utility isDocumentSigned:[_pdfViewCtrl currentDoc]];
    BOOL allowEdit = docHasSignature ? NO : [FSPermissionUtilty canModifyContents:_extensionsManager];
    BOOL canFlatten = !(!allowEdit || ![FSPermissionUtilty canAddAnnot:_extensionsManager]) && [_extensionsManager annotsPermissionCanModify] && [_extensionsManager annotsPermissionCanDelete];
    BOOL canOptimizeDoc = canFlatten && ![[_pdfViewCtrl currentDoc] isXFA] && ![Utility simpleCheckPDFA:[_pdfViewCtrl currentDoc]];
    
    if (self.reduceFileSizeItem) {
        if (![FSLibrary hasModuleLicenseRight:FSModuleNameOptimizer] || ([_pdfViewCtrl isRMSProtected] && ![_pdfViewCtrl isOwner]) || !canOptimizeDoc) {
            self.reduceFileSizeItem.enable = NO;
        }else{
            self.reduceFileSizeItem.enable = YES;
        }
    }
    
    if (self.flattenItem) {
        if (([_pdfViewCtrl isRMSProtected] && ![_pdfViewCtrl isOwner]) || !canFlatten)  {
            self.flattenItem.enable = NO;
        }else{
            self.flattenItem.enable = YES;
        }
    }
    
    if (self.wirelessPrintItem) {
        if (![Utility canPrintDocument:_pdfViewCtrl.getDoc]) {
            self.wirelessPrintItem.enable = NO;
        }else{
            self.wirelessPrintItem.enable = YES;
        }
    }
    
    if (self.cropScreenItem) {
        if (![Utility canExtractContents:_pdfViewCtrl] || ![Utility canCopyForAssess:self.pdfViewCtrl]) {
            self.cropScreenItem.enable = NO;
        }else{
            self.cropScreenItem.enable = YES;
        }
    }

    if (self.saveItem) {
        self.saveItem.enable = self.extensionsManager.enableModifyDocument;
    }

}

@end
