/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "PermissionViewController.h"
#import "MenuView+Private.h"


@implementation PessmissionCell

- (id)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier {
    if (self = [super initWithStyle:style reuseIdentifier:reuseIdentifier]) {
        self.permissionLabel = [[UILabel alloc] init];
        self.permissionLabel.font = [UIFont systemFontOfSize:15.0f];
        self.permissionLabel.backgroundColor = [UIColor clearColor];
        self.permissionLabel.textColor = self.fs_textColor_T4;;

        self.allowLabel = [[UILabel alloc] init];
        self.allowLabel.font = [UIFont systemFontOfSize:15.0f];
        self.allowLabel.textAlignment = NSTextAlignmentRight;
        self.allowLabel.backgroundColor = [UIColor clearColor];
        self.allowLabel.textColor = self.fs_textColor_T4;

        [self.contentView addSubview:self.permissionLabel];
        [self.contentView addSubview:self.allowLabel];
        
        [self.allowLabel mas_makeConstraints:^(MASConstraintMaker *make) {
            make.right.equalTo(self.contentView).offset(-16);
            make.centerY.height.equalTo(self.contentView);
        }];
        
        [self.permissionLabel mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(self.contentView).offset(16);
            make.right.equalTo(self.allowLabel.mas_left).offset(-10);
            make.centerY.height.equalTo(self.contentView);
        }];
        
        [self.permissionLabel setContentCompressionResistancePriority:UILayoutPriorityFittingSizeLevel forAxis:UILayoutConstraintAxisHorizontal];
    }
    return self;
}

@end

@interface PermissionViewController ()

@end

@implementation PermissionViewController

- (id)init {
    self = [super init];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
}

- (void)viewDidLoad {
    [super viewDidLoad];

    [self.tableView setBackgroundView:nil];
    self.view.backgroundColor = self.fs_backgroundColor_B1;;
    self.tableView.backgroundColor = self.fs_backgroundColor_B1;
    self.tableView.separatorStyle = FSTableViewSectionStyleNone;
    self.tableView.allowsSelection = NO;
    
    [self initNavigationBar];
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    //[[UIApplication sharedApplication] setStatusBarStyle:UIStatusBarStyleDefault animated:NO];
    if ([self.navigationController.navigationBar respondsToSelector:@selector(titleTextAttributes)]) {
        self.navigationController.navigationBar.titleTextAttributes = [NSDictionary dictionaryWithObjectsAndKeys:[UIColor whiteColor], NSForegroundColorAttributeName, nil];
    }
    [self.tableView reloadData];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - NavigationController methods
- (void)initNavigationBar {
    [self.navBar setTitle:FSLocalizedForKey(@"kPermission")];
}

- (void)navigationBarBackTouch{
    [self.navigationController popViewControllerAnimated:YES];
}

#pragma mark - Table view data source

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section{
    return 0.0;
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    // Return the number of sections.
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return 7;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath{
    return 44;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *CellIdentifier = @"CellPermission";
    PessmissionCell *cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
    if (cell == nil) {
        cell = [[PessmissionCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:CellIdentifier];
    }
    cell.backgroundColor = self.fs_backgroundColor_B1;
    switch (indexPath.row) {
    case 0:
        cell.permissionLabel.text = FSLocalizedForKey(@"kPermissionPrint");
        if (self.allowPrint) {
            cell.allowLabel.text = FSLocalizedForKey(@"kEnable");
        } else {
            cell.allowLabel.text = FSLocalizedForKey(@"kDisable");
        }
        break;
    case 1:
        cell.permissionLabel.text = FSLocalizedForKey(@"kRMSFILLFORM");
        if (self.allowFillForm) {
            cell.allowLabel.text = FSLocalizedForKey(@"kEnable");
        } else {
            cell.allowLabel.text = FSLocalizedForKey(@"kDisable");
        }
        break;
    case 2:
        cell.permissionLabel.text = FSLocalizedForKey(@"kRMSASSEMBLE");
        if (self.allowAssemble) {
            cell.allowLabel.text = FSLocalizedForKey(@"kEnable");
        } else {
            cell.allowLabel.text = FSLocalizedForKey(@"kDisable");
        }
        break;
    case 3:
        cell.permissionLabel.text = FSLocalizedForKey(@"kRMSANNOTATE");
        if (self.allowAnnotate) {
            cell.allowLabel.text = FSLocalizedForKey(@"kEnable");
        } else {
            cell.allowLabel.text = FSLocalizedForKey(@"kDisable");
        }
        break;
    case 4:
        cell.permissionLabel.text = FSLocalizedForKey(@"kRMSEDIT");
        if (self.allowEdit) {
            cell.allowLabel.text = FSLocalizedForKey(@"kEnable");
        } else {
            cell.allowLabel.text = FSLocalizedForKey(@"kDisable");
        }
        break;
    case 5:
        cell.permissionLabel.text = FSLocalizedForKey(@"kRMSEXTRACTACCESS");
        if (self.allowExtractAccess) {
            cell.allowLabel.text = FSLocalizedForKey(@"kEnable");
        } else {
            cell.allowLabel.text = FSLocalizedForKey(@"kDisable");
        }
        break;
    case 6:
        cell.permissionLabel.text = FSLocalizedForKey(@"kRMSEXTRACT");
        if (self.allowExtract) {
            cell.allowLabel.text = FSLocalizedForKey(@"kEnable");
        } else {
            cell.allowLabel.text = FSLocalizedForKey(@"kDisable");
        }
        break;
    default:
        break;
    }
    
    return cell;
}

- (BOOL)emptyDataSetShouldDisplay:(UITableView *)tableView {
    return NO;
}

@end
