/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/

#import "QualityViewController.h"
#import "FSSelectStyleObj.h"

@interface QualityViewController ()
@property (nonatomic, strong) FSSelectStyleObj *currentColorObj;
@property (nonatomic, strong) FSSelectStyleObj *currentMonoObj;
@end

@implementation QualityViewController

- (void)initialize{
    [super initialize];
    self.defaultEstimatedCellStyle = FSEstimatedTableViewCellStyleTitle;
    self.sectionStyle = FSTableViewSectionStyleLikeGrouped;
    self.shouldLoadDataOnViewDidLoad = YES;
    self.cellSelectedColor = self.fs_dividerColor_P1;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    self.title = FSLocalizedForKey(@"kQuality");
    self.tableView.backgroundColor = self.fs_backgroundColor_B9;
    
    [self.navBar.leftItem addTarget:self action:@selector(clickedCancel:) forControlEvents:UIControlEventTouchUpInside];
}

- (void)tableView:(UITableView *)tableView selectRowAtIndexPath:(NSIndexPath *)indexPath{
    
    [tableView selectRowAtIndexPath:indexPath animated:NO scrollPosition:0];
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

- (void)loadData{
    NSArray *tempArr = @[ FSLocalizedForKey(@"kMinimum"), FSLocalizedForKey(@"kLow"), FSLocalizedForKey(@"kMedium"), FSLocalizedForKey(@"kHigh"), FSLocalizedForKey(@"kMaximum")];
    NSMutableArray *styleData = @[].mutableCopy;
    NSMutableArray *tempStyle = @[].mutableCopy;
    for (int i = 0 ; i < tempArr.count; i++) {
        FSSelectStyleObj *obj = [[FSSelectStyleObj alloc] init];
        obj.styleText = tempArr[i];
        obj.selected = NO;
        obj.styleID = i;
        if (i == (self.colorCompressQuality - 1)) {
            obj.selected = YES;
            self.currentColorObj = obj;
        }
        [tempStyle addObject:obj];
    }
    [styleData addObject:tempStyle.copy];
    tempArr = @[ FSLocalizedForKey(@"kLossy"), FSLocalizedForKey(@"kLossless")];
    tempStyle = @[].mutableCopy;
    for (int i = 0 ; i < tempArr.count; i++) {
        FSSelectStyleObj *obj = [[FSSelectStyleObj alloc] init];
        obj.styleText = tempArr[i];
        obj.selected = NO;
        obj.styleID = i;
        if (i == (self.monoCompressQuality - 1)) {
            obj.selected = YES;
            self.currentMonoObj = obj;
        }
        [tempStyle addObject:obj];
    }
    [styleData addObject:tempStyle.copy];
    self.dataSource = styleData.copy;
}

- (void)clickedCancel:(id)sender {
    [self.navigationController popViewControllerAnimated:YES];
}

#pragma mak UITableViewDataSource UITableViewDelegate

- (NSString *)titles1ForAtSection:(NSUInteger)section{
    if (section == 0) {
        return FSLocalizedForKey(@"kGrayscaleImages");
    }
    return FSLocalizedForKey(@"kMonochromeImages");
}

- (void)configureCell:(FSTitleEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(FSSelectStyleObj *)obj{
    if (!cell.rightView) {
        UIImageView *imageView = [UIImageView new];
        [BindThemeKeySel(imageView, setImage:) fs_updateThemeCompleted:^(UIImageView * _Nonnull target, SEL  _Nonnull sel) {
            target.image = FSImageNamedAndPrimaryColor(@"common_select_check");
        }];
        imageView.hidden = YES;
        cell.rightView = imageView;
    }
    cell.contentView.backgroundColor = [UIColor clearColor];
    cell.hightlightBackgroundColor = self.fs_dividerColor_P1;
    cell.estimatedTitleLabel.text = obj.styleText;
    cell.rightView.hidden = !obj.selected;
    if (obj.selected) {
        cell.accessibilityTraits = UIAccessibilityTraitSelected;
    }else{
        cell.accessibilityTraits = UIAccessibilityTraitNone;
    }
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    NSArray *tmp = self.dataSource[indexPath.section];
    NSIndexPath *lastIndexPath = [NSIndexPath indexPathForRow:[tmp indexOfObject:self.currentColorObj] inSection:indexPath.section];
    FSSelectStyleObj *obj = tmp[indexPath.row];
    if (indexPath.section == 0) {
        self.currentColorObj.selected = NO;
        self.currentColorObj = obj;
        !self.didSelectColorBlock ? : self.didSelectColorBlock(indexPath.row, obj.styleText);
    }else{
        self.currentMonoObj.selected = NO;
        self.currentMonoObj = obj;
        !self.didSelectMonoBlock ? : self.didSelectMonoBlock(indexPath.row, obj.styleText);
    }
    obj.selected = YES;
    [tableView reloadRowsAtIndexPaths:@[lastIndexPath, indexPath] withRowAnimation:0];
}

- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section
{
   UIView *headerView = [super tableView:tableView viewForHeaderInSection:section];
   headerView.backgroundColor = self.fs_backgroundColor_B9;
   return headerView;
}
/*
 #pragma mark - Navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
