/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "MultipleSelectionModule.h"
#import "MultipleSelectionToolHandler.h"
#import "Utility.h"
#import <FoxitRDK/FSPDFViewControl.h>
#import "UIExtensionsConfig.h"
#import "FSPermissionUtilty.h"

@interface MultipleSelectionModule ()<IDocEventListener, IToolEventListener, IMoreMenuViewListener>
@property (nonatomic, weak) MultipleSelectionToolHandler *toolHandler;
@end

@implementation MultipleSelectionModule {
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;
    NSMutableArray *_multipleSelectionItems;
}

- (NSString *)getName {
    return @"MultipleSelection";
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        [self loadModule];
        
        MultipleSelectionToolHandler *toolHandler = [[MultipleSelectionToolHandler alloc] initWithUIExtensionsManager:extensionsManager];
        _toolHandler = toolHandler;
        [_extensionsManager registerToolHandler:toolHandler];
        [_extensionsManager registerToolEventListener:self];
        [_pdfViewCtrl registerDocEventListener:self];
        [_extensionsManager.more registerEventListener:self];
    }
    return self;
}
- (void)loadModule {
    [_extensionsManager registerAnnotPropertyListener:self];
    _multipleSelectionItems = @[].mutableCopy;
}

- (BOOL)addMultipleSelectionItem:(FSMainToolbarItem *)multipleSelectionItem{
    if (multipleSelectionItem.itemType == FSMainToolbarItemTypeMultiSelect) {
        @weakify(self)
        multipleSelectionItem.item.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            id<IToolHandler> toolHandler = [self->_extensionsManager getToolHandlerByName:Tool_Multiple_Selection];
            if (self->_extensionsManager.currentToolHandler == toolHandler) {
                [self->_extensionsManager changeState:FSUIManagerStateNormal];
                [self->_extensionsManager setCurrentToolHandler:nil];
            }else{
                if (self->_extensionsManager.currentToolHandler) {
                    [self->_extensionsManager setCurrentToolHandler:nil];
                }
                [self->_extensionsManager changeState:FSUIManagerStateAnnotTool];
                [self->_extensionsManager setCurrentToolHandler:toolHandler];
            }
            [self->_extensionsManager onUIInteractElementClicked:FSUIElementReadingToolbarMultipleSelection];
        };
        [_multipleSelectionItems addObject:multipleSelectionItem];
        return YES;
    }
    return NO;
}

- (void)multipleSelectionItemsSelect:(BOOL)select{
    for (FSMainToolbarItem *multipleSelectionItem in _multipleSelectionItems) {
        multipleSelectionItem.item.selected = select;
    }
}

#pragma mark- IDocEventListener
- (void)onDocOpened:(FSPDFDoc * _Nullable)document error:(int)error{
    if (error == FSErrSuccess) {

        BOOL canAddAnnot = [FSPermissionUtilty canAddAnnot:_extensionsManager];
        for (FSMainToolbarItem *multipleSelectionItem in _multipleSelectionItems) {
            multipleSelectionItem.item.enabled = canAddAnnot;
        }
    }
}

#pragma mark- IToolEventListener
- (void)onToolChanged:(NSString *)lastToolName CurrentToolName:(NSString *)toolName{
    [self multipleSelectionItemsSelect:[toolName isEqualToString:Tool_Multiple_Selection]];
}

#pragma mark -- IMoreMenuViewListener
- (void)onMoreMenuChanged:(BOOL)isHidden{
    if (!isHidden) {
        [_toolHandler onDeselectAnnots];
    }
}

@end
