/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "NoteAnnotHandler.h"
#import "NoteModule.h"

@implementation NoteAnnotHandler

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    if (self = [super initWithUIExtensionsManager:extensionsManager]) {
        self.pageEdgeInsets = UIEdgeInsetsMake(2, 2, 2, 2);
        self.annotSelectionEdgeInsets = UIEdgeInsetsMake(-20, -20, -20, -20);
        self.annotBorderEdgeInsets = UIEdgeInsetsMake(-2, -2, -2, -2);
    }
    return self;
}

- (FSAnnotType)getType {
    return FSAnnotNote;
}

- (BOOL)canResizeAnnot:(FSAnnot *)annot {
    return NO;
}

- (void)resetProperties{
    FSAnnot *annot = self.annot;
    FSPropertyMainView *propertyMainView = self.extensionsManager.propertyMainView;
    [propertyMainView setPropertyColor:annot.color];
    [propertyMainView setOpacity:annot.opacity];
    [propertyMainView setIcon:annot.icon];
    [propertyMainView setLock:![annot canModifyAppearanceWithExternsions:self.extensionsManager]];
    [propertyMainView resetProperties:@[@(FSPropertyTypePropertyColor), @(FSPropertyTypeOpacity), @(FSPropertyTypeShapeNote)]];
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer annot:(FSAnnot *)annot {
    CGPoint point = [recognizer locationInView:[self.pdfViewCtrl getPageView:annot.pageIndex]];
    FSPointF *pdfPoint = [self.pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:annot.pageIndex];
    if ([Utility isAnnot:self.extensionsManager.currentAnnot uniqueToAnnot:annot]) {
        if (!(pageIndex == annot.pageIndex && [self isHitAnnot:annot point:pdfPoint])) {
            [self.extensionsManager setCurrentAnnot:nil];
        } else {
            if (self.extensionsManager.shouldShowMenu && !self.extensionsManager.shouldShowPropertyBar) {
                [self showMenu];
            }
        }
    } else {
        self.shouldShowMenuOnSelection = NO;
        [self.extensionsManager setCurrentAnnot:annot];
        self.shouldShowMenuOnSelection = YES;
        @weakify(self)
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.4 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
            @strongify(self)
            [self comment];
        });
    }
    return YES;
}

@end
