/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "NoteModule.h"
#import <FoxitRDK/FSPDFViewControl.h>

#import "NoteAnnotHandler.h"
#import "NoteToolHandler.h"
#import "Utility.h"

@interface NoteModule () {
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;
}

@property (nonatomic, weak) FSMainToolbarItem *propertyItem;
@property (nonatomic, weak) FSReadToolbarItem *readToolbarItem;
@property (nonatomic, strong) FSReadToolbarItem *noteReadToolbarItem;
@end

@implementation NoteModule

- (NSString *)getName {
    return @"Note";
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        [self loadModule];
        
        NoteToolHandler* toolHanlder = [[NoteToolHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_pdfViewCtrl registerDocEventListener:toolHanlder];
        [_extensionsManager registerToolHandler:toolHanlder];
        
        NoteAnnotHandler* annotHandler = [[NoteAnnotHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerAnnotHandler:annotHandler];

    }
    return self;
}

- (void)loadModule {
    [_extensionsManager.toolbarManager addReadItem:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeNote] atToolTag:FSMainTopbarToolTagHome];
    FSReadToolSettings * toolSettings = [_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeNote];
    _noteReadToolbarItem = [[FSReadToolbarItem alloc] initWithToolSettings:toolSettings];
    [_extensionsManager.toolbarManager addReadItem:_noteReadToolbarItem atToolTag:FSMainTopbarToolTagComment];
    [_extensionsManager registerAnnotPropertyListener:self];
}

- (void)didClickedReadToolItem:(FSReadToolbarItem *)readToolbarItem {

    id<IToolHandler> toolHandler = [_extensionsManager getToolHandlerByName:Tool_Note];
    toolHandler.toolSettings = readToolbarItem.toolSettings;
    [_extensionsManager setCurrentToolHandler:toolHandler];
    [_extensionsManager changeState:FSUIManagerStateAnnotTool];
    
    self.readToolbarItem = readToolbarItem;
    FSMainToolbarItem *propertyItem =  [_extensionsManager.toolbarManager getMainItemWithType:FSMainToolbarItemTypeProperty];
    propertyItem.item.onTapClick = ^(FSItem * _Nonnull _item) {
        CGRect rect = [_item convertRect:_item.bounds toView:self->_extensionsManager.pdfViewCtrl];
        if (DEVICE_iPHONE) {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:rect inView:self->_extensionsManager.pdfViewCtrl];
        } else {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:_item.bounds inView:_item];
        }
    };
    self.propertyItem = propertyItem;
    [_extensionsManager onUIInteractElementClicked:FSUIElementReadingCommentBarNote];
}

#pragma mark - IAnnotPropertyListener

- (void)onToolSettingsChanged:(FSReadToolSettings *)toolSettings {
    if (self.readToolbarItem && self.readToolbarItem.toolSettings == toolSettings) {
        self.readToolbarItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }else if (_noteReadToolbarItem.toolSettings == toolSettings){
        self.noteReadToolbarItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }
}

@synthesize annotType;

@end
