/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSAddOutlineViewController.h"

@interface FSAddOutlineViewController ()
@property (nonatomic,strong) UIView *containerView;
@property (nonatomic,strong) UILabel *titleLable;
@property (nonatomic,strong) UILabel *contentLable;
@property (nonatomic,strong) UILabel *pageNumberLable;
@property (nonatomic,strong) UITextField *contentFT;
@property (nonatomic,strong) UITextField *pageNumberFT;
@end

@implementation FSAddOutlineViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.

    UIButton *bgBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    bgBtn.backgroundColor = [UIColor colorWithWhite:0 alpha:0.3];
    [bgBtn addTarget:self action:@selector(pressBgBtn) forControlEvents:UIControlEventTouchUpInside];
    [self.view addSubview:bgBtn];
    [bgBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.top.bottom.equalTo(self.view);
    }];
    
    self.containerView = [UIView new];
    self.containerView.backgroundColor = UIColorHex_DarkMode(#F2F2F2, #1E1E1E);
    self.containerView.layer.cornerRadius = 12;
    self.containerView.clipsToBounds = YES;
    [self.view addSubview:self.containerView];
    [self.containerView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.equalTo(self.view).multipliedBy(0.8);
        make.centerX.equalTo(self.view);
        make.size.mas_equalTo(CGSizeMake(270, 248));
    }];
    
    [self arrangeView];
    
    UITapGestureRecognizer *tapBgGes = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(viewTapped:)];
    tapBgGes.cancelsTouchesInView = NO;
    [self.containerView addGestureRecognizer:tapBgGes];

    if (self.editType == EditOutlineTypeAdd) {
        self.pageNumberFT.text = [NSString stringWithFormat:@"%d",[_extensionsManager.pdfViewCtrl getCurrentPage] + 1] ;
    }else{
        if (self.outlineStr){
            self.contentFT.text = self.outlineStr;
        }
        
        if (self.pageNumber > 0 && self.pageNumber <= self.extensionsManager.pdfViewCtrl.getPageCount) {
            self.pageNumberFT.text = [NSString stringWithFormat:@"%d",self.pageNumber];
        }
        
    }
    
    if (self.editType == EditOutlineTypeEditFromPage) {
        [self.pageNumberFT becomeFirstResponder];
    }else{
        [self.contentFT becomeFirstResponder];
    }
    
}

- (void)pressBgBtn{
    [self resignAllFirstResponder];
}

- (void)viewTapped:(UITapGestureRecognizer*)tapGr{
    [self resignAllFirstResponder];
}

- (void)resignAllFirstResponder{
    if (self.contentFT) {
        [self.contentFT resignFirstResponder];
    }
    
    if (self.pageNumberFT) {
        [self.pageNumberFT resignFirstResponder];
    }
}

- (void)arrangeView{
    self.titleLable = [UILabel new];
    self.titleLable.textColor = UIColorHex_DarkMode(#000000, #FFFFFF);
    self.titleLable.font = [UIFont boldSystemFontOfSize:17.0f];
    if (self.editType == EditOutlineTypeAdd) {
        self.titleLable.text = FSLocalizedForKey(@"kAddOutline");
    }else{
        self.titleLable.text = FSLocalizedForKey(@"kEditOutline");
    }
    self.titleLable.textAlignment = NSTextAlignmentCenter;
    [self.containerView addSubview:self.titleLable];
    [self.titleLable mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.width.equalTo(self.containerView);
        make.top.mas_equalTo(24);
        make.height.mas_equalTo(24);
    }];
    
    self.contentLable = [UILabel new];
    self.contentLable.textColor = UIColorHex_DarkMode(#000000, #FFFFFF);
    self.contentLable.font = [UIFont systemFontOfSize:13.0f];
    self.contentLable.text = FSLocalizedForKey(@"kEditOutlineInputName");
    self.contentLable.textAlignment = NSTextAlignmentCenter;
    [self.containerView addSubview:self.contentLable];
    [self.contentLable mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(16);
        make.right.mas_equalTo(-16);
        make.top.mas_equalTo(self.titleLable.mas_bottom).offset(8);
        make.height.mas_equalTo(16);
    }];
    
    UIView *contentBgView = [UIView new];
    contentBgView.backgroundColor = self.fs_backgroundColor_B1;
    contentBgView.layer.cornerRadius = 8;
    contentBgView.clipsToBounds = YES;
    contentBgView.layer.borderColor = self.fs_dividerColor_P3.CGColor;
    contentBgView.layer.borderWidth = 0.5f;
    [self.containerView addSubview:contentBgView];
    [contentBgView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(22);
        make.right.mas_equalTo(-16);
        make.top.mas_equalTo(self.contentLable.mas_bottom).offset(8);
        make.height.mas_equalTo(32);
    }];
    
    self.contentFT = [UITextField new];
    self.contentFT.textColor = self.fs_textColor_T4;
    self.contentFT.font = [UIFont systemFontOfSize:13.0f];
    self.contentFT.placeholder = FSLocalizedForKey(@"kEditOutlineName");
    self.contentFT.clearButtonMode = UITextFieldViewModeNever;
    [contentBgView addSubview:self.contentFT];
    [self.contentFT mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(10);
        make.right.mas_equalTo(-10);
        make.top.bottom.mas_equalTo(contentBgView);
    }];
    
    self.pageNumberLable = [UILabel new];
    self.pageNumberLable.textColor = UIColorHex_DarkMode(#000000, #FFFFFF);
    self.pageNumberLable.font = [UIFont systemFontOfSize:13.0f];
    self.pageNumberLable.text = FSLocalizedForKey(@"kEditOutlineInputPage");;
    self.pageNumberLable.textAlignment = NSTextAlignmentCenter;
    [self.containerView addSubview:self.pageNumberLable];
    [self.pageNumberLable mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(16);
        make.right.mas_equalTo(-16);
        make.top.mas_equalTo(contentBgView.mas_bottom).offset(8);
        make.height.mas_equalTo(16);
    }];
    
    UIView *pageNumberBgView = [UIView new];
    pageNumberBgView.backgroundColor = self.fs_backgroundColor_B1;
    pageNumberBgView.layer.cornerRadius = 8;
    pageNumberBgView.clipsToBounds = YES;
    pageNumberBgView.layer.borderColor = self.fs_dividerColor_P3.CGColor;
    pageNumberBgView.layer.borderWidth = 0.5f;
    [self.containerView addSubview:pageNumberBgView];
    [pageNumberBgView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(22);
        make.right.mas_equalTo(-16);
        make.top.mas_equalTo(self.pageNumberLable.mas_bottom).offset(8);
        make.height.mas_equalTo(32);
    }];

    self.pageNumberFT = [UITextField new];
    self.pageNumberFT.textColor = self.fs_textColor_T4;
    self.pageNumberFT.font = [UIFont systemFontOfSize:13.0f];
    self.pageNumberFT.clearButtonMode = UITextFieldViewModeNever;
    self.pageNumberFT.keyboardType = UIKeyboardTypeNumberPad;
    [pageNumberBgView addSubview:self.pageNumberFT];
    [self.pageNumberFT mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(10);
        make.right.mas_equalTo(-10);
        make.top.bottom.mas_equalTo(pageNumberBgView);
    }];

    UIButton *cancelBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    [cancelBtn setTitleColor:UIColorHex_DarkMode(#007AFF, #0A84FF) forState:UIControlStateNormal];
    cancelBtn.titleLabel.font = [UIFont boldSystemFontOfSize:17.0f];
    [cancelBtn setTitle:FSLocalizedForKey(@"kCancel") forState:UIControlStateNormal];
    [cancelBtn addTarget:self action:@selector(pressCancelBtn) forControlEvents:UIControlEventTouchUpInside];
    [self.containerView addSubview:cancelBtn];
    
    UIButton *okBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    [okBtn setTitleColor:UIColorHex_DarkMode(#007AFF, #0A84FF) forState:UIControlStateNormal];
    [okBtn setTitle:FSLocalizedForKey(@"kOK") forState:UIControlStateNormal];
    okBtn.titleLabel.font = [UIFont boldSystemFontOfSize:17.0f];
    [okBtn addTarget:self action:@selector(pressOkBtn) forControlEvents:UIControlEventTouchUpInside];
    [self.containerView addSubview:okBtn];
    
    [cancelBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.bottom.equalTo(self.containerView);
        make.height.mas_equalTo(48);
        make.right.mas_equalTo(okBtn.mas_left);
        make.width.mas_equalTo(okBtn.mas_width);
    }];
    
    [okBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(cancelBtn.mas_right);
        make.height.mas_equalTo(48);
        make.right.bottom.mas_equalTo(self.containerView);
    }];
    
    UIView *horizontalLine = [UIView new];
    horizontalLine.backgroundColor = self.fs_dividerColor_P1;
    [self.containerView addSubview:horizontalLine];
    [horizontalLine mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.width.equalTo(self.containerView);
        make.height.mas_equalTo(1);
        make.top.mas_equalTo(cancelBtn);
    }];
    
    UIView *verticalLine = [UIView new];
    verticalLine.backgroundColor = self.fs_dividerColor_P1;
    [self.containerView addSubview:verticalLine];
    [verticalLine mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.mas_equalTo(cancelBtn);
        make.bottom.equalTo(self.containerView);
        make.width.mas_equalTo(1);
        make.right.mas_equalTo(cancelBtn);
    }];
    
}

- (void)setEditType:(EditOutlineType)editType{
    _editType = editType;
    if (_editType == EditOutlineTypeAdd) {
        self.titleLable.text = FSLocalizedForKey(@"kDone");
    }else{
        
    }
}

- (void)pressCancelBtn{
    [self dismissViewControllerAnimated:YES completion:^{
        
    }];
}

- (void)pressOkBtn{
    
    NSString *indexStr = self.pageNumberFT.text;
    NSString *outlineStr = [self.contentFT.text stringByTrim];
    int pageIndex = 0;
    BOOL bPageValid = YES;
    BOOL bOutlineValid = YES;
    
    int totalCount = self.extensionsManager.pdfViewCtrl.getPageCount;
    if (!indexStr || indexStr.length <= 0 ) {
        bPageValid = NO;
    }else{
        pageIndex = [indexStr intValue];
        if (pageIndex < 1 || pageIndex > totalCount) {
            bPageValid = NO;
        }
    }

    if (!outlineStr || outlineStr.length <= 0) {
        bOutlineValid = NO;
    }
    
    if (!bPageValid) {
        [UIAlertController fs_alert_OK_WithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kOutlinePageInvalid") actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
            self.pageNumberFT.text = [NSString stringWithFormat:@"%d",self.extensionsManager.pdfViewCtrl.getCurrentPage+1];
            [self.pageNumberFT becomeFirstResponder];
        } onCtrl:[self fs_currentSceneTopMostViewController]];
        
        return;
    }

    if (!bOutlineValid) {
        [UIAlertController fs_alert_OK_WithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kOutlineNameInvalid") actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
            [self.contentFT becomeFirstResponder];
        } onCtrl:[self fs_currentSceneTopMostViewController]];
        return;
    }

    self.outlineStr = self.contentFT.text;
    self.pageNumber = pageIndex;

    [self dismissViewControllerAnimated:YES completion:^{
        if (self.editOutlineBlock) {
            self.editOutlineBlock(self.outlineStr, self.pageNumber);
        }
    }];
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
