/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSMoveOutlineViewController.h"
#import "FSOutlineTableViewCell.h"

@interface FSMoveOutlineViewController ()

@end

@implementation FSMoveOutlineViewController

- (void)initialize{
    [super initialize];
    self.shouldShowNavigationBar = YES;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    if (self.navigationController.viewControllers.count == 1) {
        self.navBarLeftTitle = FSLocalizedForKey(@"kClose");
    }
    self.navBarRightTitle = FSLocalizedForKey(@"kOK");
    [self.navBar.rightItem addTarget:self action:@selector(donePress) forControlEvents:UIControlEventTouchUpInside];
    [self.navBar setTitle:FSLocalizedForKey(@"kMoveOutline")];
}

- (void)donePress{
    FSBookmark *curBookbark = nil;
    
    if (self.moveBookmark) {
        curBookbark = self.moveBookmark;
    }else{
        if (self.currentBookmarks.count > 0) {
            curBookbark = [self.currentBookmarks objectAtIndex:self.currentBookmarks.count - 1];
        
            if (self.moveTreeLevel == self.treeLevel &&
                self.moveIndexPaths.count == 1 &&
                self.moveIndexPaths[0].row == self.currentBookmarks.count - 1) {
                [self dismissViewControllerAnimated:YES completion:^{
                    if (self.moveOutlineBlock) {
                        self.moveOutlineBlock();
                    }
                }];
                return;
            }
        }
    }
   
    FSBookmark *lastBookmark = nil;
    for (int i=0; i<self.moveBookmarks.count; i++) {
        FSBookmark *bookmark = [self.moveBookmarks objectAtIndex:i];
        if (i == 0) {
            if (curBookbark) {
                [bookmark moveTo:curBookbark position:FSBookmarkPosNextSibling];
            }else{
                [bookmark moveTo:self.currentParentBookmark position:FSBookmarkPosFirstChild];
            }
        }else{
            [bookmark moveTo:lastBookmark position:FSBookmarkPosNextSibling];
        }
        lastBookmark = bookmark;
    }
    self->_extensionsManager.isDocModified = YES;
    [self dismissViewControllerAnimated:YES completion:^{
        if (self.moveOutlineBlock) {
            self.moveOutlineBlock();
        }
    }];
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

- (void)configureCell:(FSOutlineTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(FSBookmark *)bookmark{
    [super configureCell:cell atIndexPath:indexPath withDataItem:bookmark];
    @weakify(self)
    cell.isMoving = YES;
    BOOL isMoveItem = NO;
    if (self.treeLevel == self.moveTreeLevel) {
        for (NSIndexPath *moveIndexPath in self.moveIndexPaths) {
            if (moveIndexPath.row == indexPath.row) {
                isMoveItem = YES;
                break;
            }
        }
    }
    
    if (isMoveItem) {
        cell.leftView.hidden = YES;
    }else{
        cell.leftView.hidden = NO;
    }
    
    if (!cell.leftView.hidden) {
        FSItem *outlineItem = (FSItem *)cell.leftView;
        outlineItem.onTapClick = ^(FSItem * _Nonnull item) {
            @strongify(self)
            FSMoveOutlineViewController *subOutlineViewCtrl = [[FSMoveOutlineViewController alloc] initWithServices:self.services];
            subOutlineViewCtrl->_extensionsManager = self->_extensionsManager;
            subOutlineViewCtrl->_pdfViewCtrl = self->_pdfViewCtrl;
            subOutlineViewCtrl.hasParentBookmark = YES;
            subOutlineViewCtrl.treeLevel = self.treeLevel + 1;
            subOutlineViewCtrl.moveTreeLevel = self.moveTreeLevel;
            subOutlineViewCtrl.editing = self.editing;
            subOutlineViewCtrl.panel = self.panel;
            subOutlineViewCtrl.allBookmarks = self.allBookmarks;
            subOutlineViewCtrl.moveBookmarks = self.moveBookmarks;
            subOutlineViewCtrl.moveIndexPaths = self.moveIndexPaths;
            subOutlineViewCtrl.commonBlock = ^{
        
            };
            [subOutlineViewCtrl loadData:bookmark];
            subOutlineViewCtrl.bookmarkGotoPageHandler = self.bookmarkGotoPageHandler;
            subOutlineViewCtrl.moveOutlineBlock = self.moveOutlineBlock;
            [self.services pushViewController:subOutlineViewCtrl animated:YES];
        };
    }
    
    if (isMoveItem) {
        cell.selectedBackgroundView.backgroundColor = self.fs_backgroundColor_B1;
    }else{
        cell.selectedBackgroundView.backgroundColor = self.fs_dividerColor_P1;
    }
    
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    
    self.moveBookmark = [self.currentBookmarks objectAtIndex:indexPath.row];
}

- (NSIndexPath *)tableView:(UITableView *)tableView willSelectRowAtIndexPath:(NSIndexPath *)indexPath{
    BOOL isMoveItem = NO;
    if (self.treeLevel == self.moveTreeLevel) {
        for (NSIndexPath *moveIndexPath in self.moveIndexPaths) {
            if (moveIndexPath.row == indexPath.row) {
                isMoveItem = YES;
                break;
            }
        }
    }
    if (!isMoveItem) {
        return indexPath;
    }else{
        return nil;
    }
}

- (void)setSearchBarHidden:(BOOL)hidden{
    //This method cannot be deleted. It is used to hide the search function
}
@end
