/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "OutlineViewController.h"
#import "PanelController+private.h"
#import "FSBookmark+Extensions.h"
#import "FSPermissionUtilty.h"
#import "FSAddOutlineViewController.h"
#import "FSOutlineTableViewCell.h"
#import "FSMoveOutlineViewController.h"


@implementation OutlineButton

@end

@interface OutlineViewController () <IPanelChangedListener> {
    __weak FSPanelController *_panelController;
}
@property (nonatomic, strong, readwrite) NSMutableArray *currentBookmarks;
@property (nonatomic, strong, readwrite) FSBookmark *currentParentBookmark;
@property (nonatomic, strong) FSItem *backRoot;
@property (nonatomic, strong) FSItem *editMoveItem;
@property (nonatomic, strong) NSIndexPath *editiIndexPath;
@property (nonatomic, weak) OutlineViewController *parentOutlineVC;
@property (nonatomic, weak) OutlineViewController *childOutlineVC;
@end

@implementation OutlineViewController

@synthesize bookmarkGotoPageHandler = _bookmarkGotoPageHandler;
@synthesize hasParentBookmark;

- (id)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager panelController:(FSPanelController *)panelController{
    self = [super initWithServices:extensionsManager.viewCtrlManager];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _panelController = panelController;
        _bookmarkGotoPageHandler = nil;
    }
    return self;
}

- (id)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager{
    self = [super initWithServices:nil];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _bookmarkGotoPageHandler = nil;
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    
    if (self.canEditBookmark) {
        FSItem *addItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kAdd")];
        addItem.font = self.fs_standardGeneralFont_GF2;
        addItem.enabled = self.hasParentBookmark ? self.parentOutlineVC.addItem.enabled : YES;
        [addItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
        [addItem addTarget:self action:@selector(clickedAddItem:) forControlEvents:UIControlEventTouchUpInside];
        [self.bottomBar addItem:addItem atPosition:FSItemPositionRight];
        _addItem = addItem;
        
        [self.editLeftItem setTitle:FSLocalizedForKey(@"kSelectAll") forState:UIControlStateNormal];
        [self.editLeftItem setTitle:FSLocalizedForKey(@"kDeselectAll") forState:UIControlStateSelected];
        [self.editRightItem setTitle:@"" forState:UIControlStateNormal];
        [self.editRightItem setImage:FSImageNamedAndNormalColor(@"common_delete_gray") forState:UIControlStateNormal];
        
        self.editMoveItem = [FSItem createItemWithImage:FSImageNamed(@"panel_outline_move")];
        self.editMoveItem.font = self.fs_standardGeneralFont_GF2;
        [self.editMoveItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
        [self.editMoveItem addTarget:self action:@selector(clickedMoveItem:) forControlEvents:UIControlEventTouchUpInside];
        [self.bottomBar addItem:self.editMoveItem atPosition:FSItemPositionRight];
        [self.editBar addItem:self.editMoveItem atPosition:FSItemPositionRight];
        self.editMoveItem.enabled = NO;
        
        FSItem *editAddItem = [FSItem createItemWithImage:FSImageNamed(@"panel_outline_add")];
        editAddItem.font = self.fs_standardGeneralFont_GF2;
        [editAddItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
        [editAddItem addTarget:self action:@selector(clickedAddItem:) forControlEvents:UIControlEventTouchUpInside];
        [self.editBar addItem:editAddItem atPosition:FSItemPositionRight];
    }
    self.bottomBar.hidden = !self.canEditBookmark;

    self.placeholder = FSLocalizedForKey(@"kSearch");
    self.cancelTitle = FSLocalizedForKey(@"kCancel");
    [self.editLeftItem setTitle:FSLocalizedForKey(@"kSelectAll") forState:UIControlStateNormal];
    [self.editLeftItem setTitle:FSLocalizedForKey(@"kDeselectAll") forState:UIControlStateSelected];
    [self.editRightItem setTitle:@"" forState:UIControlStateNormal];
    [self.editRightItem setImage:FSImageNamedAndNormalColor(@"common_delete_gray") forState:UIControlStateNormal];
    self.editRightItem.accessibilityLabel = FSLocalizedForKey(@"kDelete");
    
    [self.tableView fs_registerClass:[FSOutlineTableViewCell class]];
    
    if (self.treeLevel == 0 && _panelController) {
        [_panelController registerPanelChangedListener:self];
    }
}

- (void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    if (self.editing && !self.panel.rightItem.selected) {
        self.panel.rightItem.selected = YES;
    }
}

#pragma mark - rewrite

- (void)initialize{
    [super initialize];
    self.shouldShowNavigationBar = NO;
    self.shouldShowBottomBar = YES;
    self.defaultEstimatedCellStyle = FSEstimatedTableViewCellStyleTitle;
    self.editingImageNormal = FSImageNamedAndNormalColor(@"common_unselect");
    self.editingImageSelected = FSImageNamed(@"common_select");
}

- (void)setCanEditBookmark:(BOOL)canEditBookmark{
    _canEditBookmark = canEditBookmark;
}

- (void)loadData{
    if (!self.hasParentBookmark) {
        [self loadData:nil];
    }
}

- (BOOL)searchBarShouldBeginEditing:(UISearchBar *)searchBar{
    if (self.editing) {
        self.editing = NO;
    }
    [_panelController.panel.pagerContentView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(_panelController.panel.contentView.fs_mas_left);
        make.right.mas_equalTo(_panelController.panel.contentView.fs_mas_right);
        make.bottom.top.fs_equalToSuperview();
    }];
    [_panelController.panel.contentView layoutIfNeeded];
    _panelController.panel.pagerContentView.scrollEnabled = NO;
    return YES;
}

- (void)enteredSearchState{
    if (!self.allBookmarks.count) {
        self.allBookmarks = @[].mutableCopy;
        FSBookmark *rootBookmark = [[_pdfViewCtrl getDoc] getRootBookmark];
        [self getAllBookmarksWithBookmark:rootBookmark];
    }
    self.dataSource = @[self.allBookmarks];

    if (self.canEditBookmark) {
        [self.tableView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(self.tableView.superview.mas_safeAreaLayoutGuideLeft);
            make.right.mas_equalTo(self.tableView.superview.mas_safeAreaLayoutGuideRight);
            make.bottom.equalTo(self.tableView.superview);
            make.top.equalTo(self.shouldShowNavigationBar ? self.navBar.mas_bottom : self.tableView.superview);
        }];
    }
}
- (void)exitedSearchState{
    self.dataSource = self.currentBookmarks ? @[self.currentBookmarks] : nil;

    if (self.canEditBookmark) {
        [self.tableView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(self.tableView.superview.mas_safeAreaLayoutGuideLeft);
            make.right.mas_equalTo(self.tableView.superview.mas_safeAreaLayoutGuideRight);
            make.bottom.equalTo(self.shouldShowBottomBar ? self.bottomBar.mas_top : self.tableView.superview);
            make.top.equalTo(self.shouldShowNavigationBar ? self.navBar.mas_bottom : self.tableView.superview);
        }];
    }
}


- (void)setSubviewTableViewLayoutMargins:(UIEdgeInsets)subviewTableViewLayoutMargins{
    [super setSubviewTableViewLayoutMargins:subviewTableViewLayoutMargins];
    self.childOutlineVC.needMakeUpSectionHeaderInset = self.needMakeUpSectionHeaderInset;
    self.childOutlineVC.subviewTableViewLayoutMargins = self.subviewTableViewLayoutMargins;
    self.childOutlineVC.editBar.subviewContentViewLayoutMargins = self.editBar.subviewContentViewLayoutMargins;
    self.childOutlineVC.bottomBar.subviewContentViewLayoutMargins = self.bottomBar.subviewContentViewLayoutMargins;
}

- (NSArray *)getAllBookmarksWithBookmark:(FSBookmark *)bookmark{
    @try {
        if (bookmark && ![bookmark isEmpty] && !bookmark.isRoot) {
            [self.allBookmarks addObject:bookmark];
        }
        NSArray *bookmarks = [bookmark getChildBookmarks];
        for (FSBookmark *_bookmark in bookmarks) {
            NSArray *_bookmarks = [_bookmark getChildBookmarks];
            if (_bookmarks.count) {
                [self getAllBookmarksWithBookmark:_bookmark];
            }else{
                [self.allBookmarks addObject:_bookmark];
            }
        }
        return self.allBookmarks.copy;
    } @catch (NSException *_) {
        return self.allBookmarks.copy;
    }
}

- (void)clickedAddItem:(FSItem *)item{
    [self showEditOutlineView:nil pageNumber:0 editType:EditOutlineTypeAdd];
}

- (void)editOutLineWithType:(EditOutlineType)editType outlineName:(NSString *)outlineStr pageNumber:(int)pageNumber
{
    NSInteger index = -1;
    FSBookmark *curBookmark = nil;
    if (editType == EditOutlineTypeAdd) {
        if (self.selectedDataSource.count) {
            curBookmark = [self.selectedDataSource objectAtIndex:0];
            index = [self.selectedIndexPaths objectAtIndex:0].row;
        }else{
            if (self.currentBookmarks.count) {
                curBookmark = [self.currentBookmarks objectAtIndex:self.currentBookmarks.count-1];
                index = self.currentBookmarks.count-1;
            }
            
        }
        if (curBookmark) {
            FSBookmark *newBookmark = [curBookmark insert:outlineStr position:FSBookmarkPosNextSibling];
            FSDestination* dest = [FSDestination createXYZ:self->_extensionsManager.pdfViewCtrl.getDoc page_index:pageNumber-1 left:0 top:0 zoom_factor:1.0f];
            [newBookmark setDestination:dest];
            
            [self.allBookmarks removeAllObjects];
            [self.currentBookmarks insertObject:newBookmark atIndex:index+1];
            [self insertRowsAtIndexPath:[NSIndexPath indexPathForRow:index+1 inSection:0]];
        }else{
            
            if (!self.currentParentBookmark) {
                self.currentParentBookmark = [[self->_pdfViewCtrl getDoc] createRootBookmark];
                if (!self.currentParentBookmark || [self.currentParentBookmark isEmpty]) {
                    return;
                }
            }
            
            FSBookmark *newBookmark = [self.currentParentBookmark insert:outlineStr position:FSBookmarkPosFirstChild];
            FSDestination* dest = [FSDestination createXYZ:self->_extensionsManager.pdfViewCtrl.getDoc page_index:pageNumber-1 left:0 top:0 zoom_factor:1.0f];
            [newBookmark setDestination:dest];
            [self.allBookmarks removeAllObjects];
            self.currentBookmarks = [[self.currentParentBookmark getChildBookmarks] mutableCopy];
            if (self.currentBookmarks) self.dataSource = @[self.currentBookmarks];
            else self.dataSource = nil;
            [self.tableView reloadData];
        }
    }else{
        if (self.editiIndexPath) {
            curBookmark = [self.currentBookmarks objectAtIndex:self.editiIndexPath.row];
            curBookmark.title = outlineStr;
            FSDestination* dest = [FSDestination createXYZ:self->_extensionsManager.pdfViewCtrl.getDoc page_index:pageNumber-1 left:0 top:0 zoom_factor:1.0f];
            [curBookmark setDestination:dest];
            [self.tableView reloadRowsAtIndexPaths:@[self.editiIndexPath] withRowAnimation:UITableViewRowAnimationFade];
        }
    }
    self->_extensionsManager.isDocModified = YES;
}

- (void)showEditOutlineView:(NSString *)outlineStr pageNumber:(int)pageNumber editType:(EditOutlineType)editType{
    FSAddOutlineViewController *addOutlineVC = [[FSAddOutlineViewController alloc] init];
    addOutlineVC.modalPresentationStyle = UIModalPresentationOverFullScreen;
    addOutlineVC.modalTransitionStyle = UIModalTransitionStyleCrossDissolve;
    addOutlineVC.extensionsManager = _extensionsManager;
    addOutlineVC.editType = editType;
    if (editType == EditOutlineTypeEditFromName || editType == EditOutlineTypeEditFromPage) {
        if (outlineStr) {
            addOutlineVC.outlineStr = outlineStr;
        }
        
        if (pageNumber>0 && pageNumber <= _extensionsManager.pdfViewCtrl.getPageCount) {
            addOutlineVC.pageNumber = pageNumber;
        }else{
            addOutlineVC.pageNumber = _extensionsManager.pdfViewCtrl.getCurrentPage + 1;
        }
    }
    @weakify(self)
    addOutlineVC.editOutlineBlock = ^(NSString * _Nonnull outlineStr, int pageNumber) {
        @strongify(self)
        [self editOutLineWithType:editType outlineName:outlineStr pageNumber:pageNumber];
    };
    [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:addOutlineVC animated:YES completion:nil];
}

- (void)clickedMoveItem:(FSItem *)item{
    @fs_weakify(self)
    FSMoveOutlineViewController *outlineVC = [[FSMoveOutlineViewController alloc] initWithExtensionsManager:self->_extensionsManager];
    outlineVC.panel = self.panel;
    outlineVC.hasParentBookmark = NO;
    outlineVC.treeLevel = 0;
    [outlineVC loadData:nil];
    outlineVC.shouldShowBottomBar = NO;
    outlineVC.moveBookmarks = self.selectedDataSource;
    outlineVC.moveIndexPaths = self.selectedIndexPaths;
    outlineVC.moveTreeLevel = self.treeLevel;
    outlineVC.moveOutlineBlock = ^{
        @fs_strongify(self)
        [self.allBookmarks removeAllObjects];
        [self deselectAll];
        for (OutlineViewController *VC in self.navigationController.viewControllers) {
            if ([VC isKindOfClass:[OutlineViewController class]]) {
                if (VC.currentParentBookmark) {
                    [VC loadData:VC.currentParentBookmark];
                }
            }
        }
        if ([self emptyDataSetShouldDisplay:self.tableView] && self.hasParentBookmark) {
            [self.services popViewControllerAnimated:YES];
        }
    };
    outlineVC.bookmarkGotoPageHandler = self.bookmarkGotoPageHandler;
    FSNavigationController *outlineNavCtrl = [[FSNavigationController alloc] initWithRootViewController:outlineVC];
    if (DEVICE_iPHONE) {
        outlineNavCtrl.modalPresentationStyle = UIModalPresentationFullScreen;
    }else{
        outlineNavCtrl.modalPresentationStyle = UIModalPresentationFormSheet;
    }
    [[self->_pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:outlineNavCtrl animated:YES completion:nil];
}

- (void)clickedSearchCancelItem{
    [super clickedSearchCancelItem];
    [_panelController.panel.pagerContentView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(_panelController.panel.contentView.mas_left);
        make.right.mas_equalTo(_panelController.panel.contentView.mas_right);
        make.bottom.fs_equalToSuperview();
        make.top.mas_equalTo(_panelController.panel.topBar.mas_bottom);
    }];
    [_panelController.panel.contentView layoutIfNeeded];
    _panelController.panel.pagerContentView.scrollEnabled = YES;
    self.panel.rightItem.selected = self.editing;
}

- (void)setSelectedIndexPaths:(NSArray<NSIndexPath *> *)selectedIndexPaths{
    [super setSelectedIndexPaths:selectedIndexPaths];
    NSArray *bookmark = [[self dataSource] firstObject];
    if (bookmark.count) {
        self.editLeftItem.selected = self.selectedIndexPaths.count ==  bookmark.count;
    }
}

- (void)clickedEditLeftItem{
    if ([self currentBookmarks].count > 0) {
        self.editLeftItem.selected = !self.editLeftItem.selected;
        if (self.editLeftItem.selected) {
            [self selectAll];
        }else{
            [self deselectAll];
        }
    }
}

- (void)clickedEditRightItem{
    if ([self selectedIndexPaths].count > 0 && [self selectedDataSource].count > 0) {
        void(^block)(void) = ^{
            for (FSBookmark *bookmark in self.selectedDataSource) {
                if ([self->_pdfViewCtrl.currentDoc removeBookmark:bookmark]) {
                    [self.currentParentBookmark removeChildBookmark:bookmark];
                }
            }
            [self.allBookmarks removeAllObjects];
#if _MAC_CATALYST_
            NSMutableArray *selectedDataSource = @[].mutableCopy;
            for (NSIndexPath *indexPath in self.selectedIndexPaths) {
                FSBookmark *bookmark = [self dataForAtIndexPath:indexPath];
                [selectedDataSource addObject:bookmark];
            }
            [self.currentBookmarks removeObjectsInArray:selectedDataSource];
#else
            [self.currentBookmarks removeObjectsInArray:self.selectedDataSource];
#endif
            [self.tableView deleteRowsAtIndexPaths:self.selectedIndexPaths withRowAnimation:UITableViewRowAnimationFade];
            if (self.hasParentBookmark && !self.currentBookmarks.count) {
                !self.commonBlock ? : self.commonBlock();
                if (!self.canEditBookmark) {
                    [self.services popViewControllerAnimated:YES];
                }
            }else if(!self.hasParentBookmark && !self.currentBookmarks.count){
                self.dataSource = nil;
            }
            self->_extensionsManager.isDocModified = YES;
            [super clickedEditRightItem];
            if ([self emptyDataSetShouldDisplay:self.tableView] && self.hasParentBookmark) {
                [self.services popViewControllerAnimated:YES];
            }
        };
        if (self.selectedDataSource.count != self.currentBookmarks.count) {
            [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kConfirm") message:FSLocalizedForKey(@"kDeleteOutline") actions:@[ FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                if (index == 1) {
                    block();
                }
            } onCtrl:[self->_pdfViewCtrl fs_currentSceneTopMostViewController]];
        }else{
            [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kConfirm") message:FSLocalizedForKey(@"kClearOutline") actions:@[ FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                if (index == 1) {
                    block();
                }
            } onCtrl:[self->_pdfViewCtrl fs_currentSceneTopMostViewController]];
        }
    }
}

- (void)setSelectedDataSource:(NSArray *)selectedDataSource{
    [super setSelectedDataSource:selectedDataSource];
    self.editMoveItem.enabled = selectedDataSource.count;
}

- (void)setEditing:(BOOL)editing{
    [super setEditing:editing];
    _panel.rightItem.selected = editing;
}

- (void)loadData:(FSBookmark *)parentBookmark {
    if (!_pdfViewCtrl.getDoc) {
        return;
    }
    FSBookmark *rootBookmark = [[_pdfViewCtrl getDoc] getRootBookmark];
    if (!rootBookmark || [rootBookmark isEmpty]) {
        return;
    }
    if (!parentBookmark) {
        parentBookmark = rootBookmark;
    }
    self.currentParentBookmark = parentBookmark;
    self.currentBookmarks = [parentBookmark getChildBookmarks].mutableCopy;
    if (self.currentBookmarks) self.dataSource = @[self.currentBookmarks];
    else self.dataSource = nil;
}

- (CGFloat)verticalOffsetForEmptyDataSet:(UITableView *)tableView{
    if (self.hasParentBookmark && !self.enteredSearch) {
        return PREFERENCE_CELL_ROW_HEIGHT_NORMAL/2;
    }
    return 0;
}

- (BOOL)emptyDataSetShouldDisplay:(UITableView *)tableView{
    BOOL emptyDataSetShouldDisplay = [super emptyDataSetShouldDisplay:tableView];
    if (!self.hasParentBookmark) {
        self.panel.rightItem.enabled = !emptyDataSetShouldDisplay && [FSPermissionUtilty canAssemble:_extensionsManager];
        if (!self.panel.rightItem.enabled) self.panel.rightItem.selected = NO;
    }
    if (emptyDataSetShouldDisplay) {
        if (self.editing) {
            self.editing = NO;
        }
        if (!self.searchBarContentView.hidden && !self.keywords.length) {
            [self setSearchBarHidden:YES];
        }
    }
    return emptyDataSetShouldDisplay;
}

- (NSString *)titleForEmptyDataSet{
    if (self.keywords.length) return FSLocalizedForKey(@"kNoResultsTip");
    return FSLocalizedForKey(@"kNoOutline");
}

- (UIImage *)imageForEmptyDataSet:(UITableView *)tableView {
    if (self.keywords.length) return FSImageNamedAndPrimaryColor(@"search_result_no_data");
    return FSImageNamedAndPrimaryColor(@"panel_outline_no_data");
}


- (BOOL)evaluateWithObject:(FSBookmark *)bookmark{
    @try {
        return [super evaluateWithObject:bookmark.title];
    } @catch (NSException *exception) {
        return NO;
    }
    return NO;
}

- (NSArray<UILabel *> *)keywordsHighlightLabels:(FSTitleEstimatedTableViewCell *)cell{
    return @[cell.estimatedTitleLabel];
}

- (NSString *)dequeueReusableIdentifierWithIndexPath:(NSIndexPath *)indexPath{
    NSString *identifier = NSStringFromClass([FSOutlineTableViewCell class]);
    return identifier;
}

- (void)configureCell:(FSOutlineTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(FSBookmark *)bookmark{
    cell.editingHiddenRightView = NO;
    if (!cell.installed) {
        cell.leftMargin = TITLE_LEFT_ICON_MARGINT;
        cell.rightMargin = ITEM_RIGHT_MARGIN_NORMAL;
        cell.mediumSpace = TITLE_LEFT_ICON_MARGINT;
    }
    cell.hightlightBackgroundColor = self.fs_dividerColor_P1;
    cell.canEditBookmark = self.canEditBookmark;
    if (!cell.leftView) {
        FSItem *outlineItem =  [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"common_accessory_gray")];
        outlineItem.accessibilityLabel = FSLocalizedForKey(@"kUnfoldVoiceOver");
        cell.leftView = outlineItem;
    }
    @weakify(self)
    if (self.enteredSearch) {
        [((FSItem *)cell.leftView) setImage:[UIImage fs_imageWithColor:[UIColor clearColor] size:CGSizeMake(1, 1)] forState:UIControlStateNormal];
    }else{
        [((FSItem *)cell.leftView) setImage:FSImageNamedAndNormalColor(@"common_accessory_gray") forState:UIControlStateNormal];
    }

    if (!cell.rightView) {
        FSItem *pageNumItem = [FSItem createItemWithTitle:nil];
        [pageNumItem setTitleColor:self.fs_textColor_T3 forState:UIControlStateNormal];
        pageNumItem.font = self.fs_standardGeneralFont_GF2;
        pageNumItem.titleLabel.textAlignment = NSTextAlignmentRight;
        pageNumItem.titleLabel.numberOfLines = 0;
        cell.rightView = pageNumItem;
    }
    
    FSItem *pageNumItem = (FSItem *)cell.rightView;
    pageNumItem.enabled = self.editing && self.canEditBookmark;
    @try {
        int num = [bookmark.destination getPageIndex:_pdfViewCtrl.currentDoc];
        NSString *title;
        title = num < 0 ? @"" : [NSString stringWithFormat:@"%d",[bookmark.destination getPageIndex:_pdfViewCtrl.currentDoc] + 1];
        [pageNumItem setTitle:title forState:UIControlStateNormal];
    } @catch (NSException *exception) {
        [pageNumItem setTitle:@"" forState:UIControlStateNormal];
    }
    pageNumItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        self.editiIndexPath = indexPath;
        int pageNumber = [bookmark.destination getPageIndex:self->_pdfViewCtrl.currentDoc];
        [self showEditOutlineView:bookmark.title pageNumber:pageNumber+1 editType:EditOutlineTypeEditFromPage];
    };
    
    BOOL hasChild = [[bookmark getFirstChild] isEmpty] ? NO : YES;
    cell.leftView.hidden = !hasChild;
    if (!cell.leftView.hidden) cell.leftView.hidden = self.enteredSearch;
    
    if (!cell.leftView.hidden) {
        FSItem *outlineItem = (FSItem *)cell.leftView;
        outlineItem.onTapClick = ^(FSItem * _Nonnull item) {
            @strongify(self)
            [self deselectAll];
            OutlineViewController *subOutlineViewCtrl = [[OutlineViewController alloc] initWithExtensionsManager:self->_extensionsManager panelController:self->_panelController];
            subOutlineViewCtrl.canEditBookmark = self.canEditBookmark;
            subOutlineViewCtrl.hasParentBookmark = YES;
            subOutlineViewCtrl.parentOutlineVC = self;
            self.childOutlineVC = subOutlineViewCtrl;
            subOutlineViewCtrl.treeLevel = self.treeLevel + 1;
            subOutlineViewCtrl.editing = self.editing;
            subOutlineViewCtrl.panel = self.panel;
            subOutlineViewCtrl.allBookmarks = self.allBookmarks;
            subOutlineViewCtrl.needMakeUpSectionHeaderInset = self.needMakeUpSectionHeaderInset;
            subOutlineViewCtrl.subviewTableViewLayoutMargins = self.subviewTableViewLayoutMargins;
            subOutlineViewCtrl.editBar.subviewContentViewLayoutMargins = self.editBar.subviewContentViewLayoutMargins;
            subOutlineViewCtrl.bottomBar.subviewContentViewLayoutMargins = self.bottomBar.subviewContentViewLayoutMargins;
            subOutlineViewCtrl.commonBlock = ^{
                @strongify(self)
                [self.tableView reloadRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationNone];
            };
            [subOutlineViewCtrl loadData:bookmark];
            subOutlineViewCtrl.bookmarkGotoPageHandler = self.bookmarkGotoPageHandler;
            [self.services pushViewController:subOutlineViewCtrl animated:YES];
        };
    }
    
    cell.estimatedTitleLabel.text = [bookmark getTitle];
    cell.estimatedTitleLabel.accessibilityIdentifier = @"outline_id";
    
    [cell.titleLableItem setTitle:[bookmark getTitle] forState:UIControlStateNormal];
    cell.titleLableItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        self.editiIndexPath = indexPath;
        @try {
            int pageNumber = [bookmark.destination getPageIndex:self->_pdfViewCtrl.currentDoc];
            [self showEditOutlineView:bookmark.title pageNumber:pageNumber+1 editType:EditOutlineTypeEditFromName];
        } @catch (NSException *exception) {

        }
    };
    cell.titleLableItem.enabled = self.editing && self.canEditBookmark;
    [super configureCell:cell atIndexPath:indexPath withDataItem:bookmark];
}

#pragma mark - Table view data source

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section{
    if (self.hasParentBookmark && !self.enteredSearch) {
        return PREFERENCE_CELL_ROW_HEIGHT_NORMAL;
    }
    return 0.01;
}

- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section{
    UIView *view = [UIView new];
    if (self.hasParentBookmark && !self.enteredSearch) {
        view.backgroundColor = self.fs_backgroundColor_B1;

        _backRoot = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"panel_outline_list_back_gray")];
        _backRoot.accessibilityLabel = FSLocalizedForKey(@"kBackToTopVoiceover");

        @weakify(self)
        _backRoot.onTapClick = ^(FSItem * _Nonnull item) {
            @strongify(self)
            [self.services popToRootViewControllerAnimated:YES];
        };
        [view addSubview:_backRoot];
        
        
        FSItem *preItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"panel_back_dot_gray")];
        preItem.accessibilityLabel = FSLocalizedForKey(@"kBack");
        preItem.onTapClick = ^(FSItem * _Nonnull item) {
            @strongify(self)
            [self.services popViewControllerAnimated:YES];
        };
        [view addSubview:preItem];

        [_backRoot mas_makeConstraints:^(MASConstraintMaker *make) {
            make.centerY.fs_equalToSuperview();
            make.left.mas_equalTo(view.mas_safeAreaLayoutGuideLeft).offset(TITLE_LEFT_ICON_MARGINT);
        }];
        [preItem mas_makeConstraints:^(MASConstraintMaker *make) {
            make.centerY.fs_equalToSuperview();
            make.left.equalTo(_backRoot.mas_right);
        }];
        
        return view;
    }
    return view;
}

- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
    if(self.currentBookmarks.count >1){
        return self.canEditBookmark;
    }else{
        return NO;
    }
    
}

- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)sourceIndexPath toIndexPath:(NSIndexPath *)destinationIndexPath
{
    if (sourceIndexPath.row == destinationIndexPath.row) {
        return;
    }
    FSBookmark *sourceBookmark = [self.currentBookmarks objectAtIndex:sourceIndexPath.row];
    FSBookmark *destBookmark = [self.currentBookmarks objectAtIndex:destinationIndexPath.row];
    [self.allBookmarks removeAllObjects];
    [self.currentBookmarks removeObject:sourceBookmark];
    [self.currentBookmarks insertObject:sourceBookmark atIndex:destinationIndexPath.row];
    if (sourceIndexPath.row > destinationIndexPath.row) {
        [sourceBookmark moveTo:destBookmark position:FSBookmarkPosPrevSibling];
    }else if (sourceIndexPath.row < destinationIndexPath.row){
        [sourceBookmark moveTo:destBookmark position:FSBookmarkPosNextSibling];
    }
    _extensionsManager.isDocModified = YES;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    if (self.editing) {
        [super tableView:tableView didSelectRowAtIndexPath:indexPath];
        return;
    }
    [super tableView:tableView didSelectRowAtIndexPath:indexPath];
    FSBookmark *bookmarkItem = [self dataForAtIndexPath:indexPath];
    FSDestination *dest = [bookmarkItem getDestination];
    if (!dest || [dest isEmpty])
        return;
    int pageIndex = [dest getPageIndex:_pdfViewCtrl.currentDoc];
    if (pageIndex < 0) return;
    _panelController.isHidden = YES;
    if (dest.getZoomMode == FSDestinationZoomFitPage) {
       [_pdfViewCtrl gotoPage:pageIndex animated:NO];
    }else{
        FSPointF *point = [[FSPointF alloc] initWithX:[dest getLeft] Y:[dest getTop]];
        [_pdfViewCtrl gotoPage:pageIndex withDocPoint:point animated:NO];
    }
    [self clickedSearchCancelItem];
}

- (void)viewSafeAreaInsetsDidChange{
    [super viewSafeAreaInsetsDidChange];
    [self.view setNeedsLayout];
}

#pragma mark - methods
- (void)clearAllBookmark {
    for (FSBookmark *bookmark in self.currentBookmarks) {
        [_pdfViewCtrl.currentDoc removeBookmark:bookmark];
    }
    [self.allBookmarks removeAllObjects];
    self.currentBookmarks = nil;
    self.dataSource = nil;
}

- (void)clearData {
    if (self.tableView.window) {
        [self clickedSearchCancelItem];
    }
    if (self.navigationController.viewControllers.count > 1) [self.navigationController popToRootViewControllerAnimated:NO];
    if (self.editing) {
        self.editing = NO;
        self.panel.rightItem.selected = self.editing;
    }
    self.allBookmarks = nil;
    self.currentBookmarks = nil;
    self.dataSource = nil;
    self.currentParentBookmark = nil;
}


#pragma mark - IPanelChangedListener
- (void)onPanelChanged:(BOOL)isHidden {
    if (isHidden) {
        for (OutlineViewController *outlineCtrl in self.navigationController.viewControllers) {
            outlineCtrl.editing = NO;
        }
    }
}

- (void)dealloc{

}

@end
