/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSPageColorCell.h"

@interface FSPageColorCell()
@property(nonatomic, strong)FSItem* colorItem;
@property(nonatomic, strong)FSItem* selectedItem;
@end

@implementation FSPageColorCell
- (instancetype)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        self.isAccessibilityElement = YES;
        [self initSubView];
    }
    return self;
}

- (void)initSubView
{
    self.contentView.backgroundColor = self.fs_backgroundColor_B1;
    self.contentView.frame = self.bounds;
    FSItem* selectedItem = [[FSItem alloc] init];
    _selectedItem = selectedItem;
    [self.contentView addSubview:selectedItem];
    [selectedItem mas_makeConstraints:^(MASConstraintMaker *make) {
        make.center.equalTo(selectedItem.superview);
        make.size.equalTo(selectedItem.superview);
    }];
    [selectedItem setBackgroundColor:self.fs_backgroundColor_B1];
    [selectedItem fs_setLayerBorderWidth:LAYER_BORDER_WIDTH_1 borderColor:self.fs_primaryColor];
    [selectedItem fs_setLayerCornerRadius:LAYER_BORDER_RADIUS_3];
    [selectedItem setHidden:YES];
    selectedItem.userInteractionEnabled = NO;
    
    FSItem* colorItem = [[FSItem alloc] init];
    _colorItem = colorItem;
    [self.contentView addSubview:colorItem];
    [colorItem mas_makeConstraints:^(MASConstraintMaker *make) {
        make.center.equalTo(colorItem.superview);
        make.size.mas_equalTo(CGSizeMake(26, 26));
    }];
    [colorItem setImage:FSImageNamed(@"setting_pagecolor_ring_white") forState:UIControlStateNormal];
    colorItem.userInteractionEnabled = NO;
}

- (void)setCellInfo:(UIColor*)color
{
    if (color == UIColor.whiteColor)
    {
        [_colorItem mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.center.equalTo(self.colorItem.superview);
            make.size.mas_equalTo(CGSizeMake(26, 26));
        }];
        [_colorItem setImage:FSImageNamed(@"setting_pagecolor_ring_white") forState:UIControlStateNormal];
    }
    else
    {
        [_colorItem mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.center.equalTo(self.colorItem.superview);
            make.size.mas_equalTo(CGSizeMake(24, 24));
        }];
        [_colorItem setImage:[FSImageNamed(@"setting_pagecolor_ring_white") fs_imageWithTintColor:color] forState:UIControlStateNormal];
    }
    
    _colorItem.accessibilityIdentifier = [NSString stringWithFormat:@"color_item_%@",[color hexString]];
    self.accessibilityLabel = [NSString stringWithFormat:@"%@ %@", FSLocalizedForKey(@"kColorValueAccessibilityLabel"), [color hexString]];
}

- (void)setSelected:(BOOL)selected
{
    _selectedItem.hidden = !selected;
}
@end
