/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "PageNavigationModule.h"
//#import "AppDelegate.h"
#import <QuartzCore/QuartzCore.h>
#import "FSPageNavigationSlider.h"
#import "NSString+FSAdd.h"

#define kPageSliderHeight 31

@interface PageNavigationModule () <UITextFieldDelegate>{
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;
    FSUIManagerState _oldReadFrameState;
}

@property (strong, nonatomic) UIView *pageNumView;
@property (strong, nonatomic) UILabel *totalNumLabel;
@property (strong, nonatomic) UIImageView *prevImage;
@property (strong, nonatomic) UIImageView *nextImage;
@property (strong, nonatomic) FSPageNavigationSlider *pageSlider;
@property (assign, nonatomic) int indexForSliderDown;
@property (assign, nonatomic) BOOL isFristChangeForSlider;
@property (assign, nonatomic) CGSize reflowScrollViewContentSize;
@property (assign, nonatomic) BOOL isKeyboardShow;
@end

@implementation PageNavigationModule

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    if (self = [super init]) {
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _extensionsManager = extensionsManager;
        [self initSubViews];
        [self loadModule];
        _oldReadFrameState = FSUIManagerStateNormal;
        [_pdfViewCtrl registerLayoutEventListener:self];
        _isFristChangeForSlider = YES;
        _isFormNavBarShowState = NO;
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardDidShow) name:UIKeyboardDidShowNotification object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardDidHide) name:UIKeyboardWillHideNotification object:nil];

    }
    return self;
}

- (void)onLayoutFinished
{
    [self setPageSliderLayout:_pdfViewCtrl.fs_width];
}

- (NSString *)getName {
    return @"PageNavigation";
}

- (void)initSubViews {
    self.totalNumLabel = [[UILabel alloc] init];
    self.totalNumLabel.accessibilityIdentifier = @"page_num";
    self.totalNumLabel.userInteractionEnabled = YES;
    [self.totalNumLabel addGestureRecognizer:[[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(showGotoPageToolbar:)]];

    self.prevImage = [[UIImageView alloc] init];
    self.prevImage.accessibilityIdentifier = @"pre_page";
    self.prevImage.image = FSImageNamed(@"page_number_goto_prev");
    self.prevImage.userInteractionEnabled = YES;
    [self.prevImage addGestureRecognizer:[[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handleGotoPrevView:)]];

    self.nextImage = [[UIImageView alloc] init];
    self.nextImage.accessibilityIdentifier = @"next_page";
    self.nextImage.image = FSImageNamed(@"page_number_goto_next");
    self.nextImage.userInteractionEnabled = YES;
    [self.nextImage addGestureRecognizer:[[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(handleGotoNextView:)]];

    self.pageNumView = [[UIView alloc] init];
    [self.pageNumView addSubview:self.totalNumLabel];
    [self.pageNumView addSubview:self.prevImage];
    [self.pageNumView addSubview:self.nextImage];

    [self.prevImage mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.pageNumView).offset(4);
        make.centerY.mas_equalTo(self.pageNumView);
        make.size.mas_equalTo(self.prevImage.image.size);
    }];
    
    [self.nextImage mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(self.pageNumView).offset(-4);
        make.centerY.mas_equalTo(self.pageNumView);
        make.size.mas_equalTo(self.nextImage.image.size);
    }];
    
    [self addPageNumberView];
    [self initPageSlider];
}

- (void)initPageSlider
{
    _pageSlider = [[FSPageNavigationSlider alloc] init];
    _pageSlider.accessibilityIdentifier = @"page_slide_button";
    _pageSlider.minimumTrackTintColor = UIColor.clearColor;
    _pageSlider.maximumTrackTintColor = UIColor.clearColor;
    _pageSlider.minimumValue = 0;
    [_pageSlider addTarget:self action:@selector(onPageNumberSliderValueChanged) forControlEvents:UIControlEventValueChanged];
    [_pageSlider addTarget:self action:@selector(onPageNumberSliderTouchUpInside) forControlEvents:UIControlEventTouchUpInside];
    [BindThemeKeySel(_pageSlider, setThumbImage:forState:) fs_updateThemeCompleted:^(UISlider*  _Nonnull target, SEL  _Nonnull sel) {
        [target setThumbImage:[FSImageNamed(@"page_number_slider_thumb_bg") fs_addImage:[FSImageNamed(@"page_number_slider_thumb_icon") fs_imageWithTintColor:target.fs_primaryColor]] forState:UIControlStateNormal];
    }];
    [_extensionsManager.containerView addSubview:_pageSlider];
    [_pageSlider mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.mas_equalTo(self.pageSlider.superview.fs_mas_bottom).offset([self checkIsIphoneDevice] ? -(49+8) : -8);
        make.height.mas_equalTo(kPageSliderHeight);
        make.left.mas_equalTo(self.pageSlider.superview.fs_mas_left).offset(8);
        make.right.mas_equalTo(self.pageSlider.superview.fs_mas_right).offset(-8);
    }];
}

- (void)setPageSliderLayoutWithHorization
{
    if (_pageSlider.superview) {
        CGFloat space = 8;
        CGFloat sliderHeight = kPageSliderHeight;
       
        UIView *currentBottom = [_extensionsManager.containerView hitTest:CGPointMake(_extensionsManager.containerView.fs_right/2 - 10, _extensionsManager.containerView.fs_bottom - 10) withEvent:nil];
        FSToolbar *currentBottomBar = [currentBottom fs_getSuperviewOfClassName:NSStringFromClass([FSToolbar class])];
        if (!currentBottomBar && _extensionsManager.bottomToolbar && !_extensionsManager.bottomToolbar.hidden) {
            currentBottomBar = [_extensionsManager.bottomToolbar toolbar];
        }
        CGFloat bottomHeight = (currentBottomBar || (![self checkIsBottombarHidden] && [self checkIsIphoneDevice])) ? -(TAB_BAR_HEIGHT_NORMAL + 8) : _extensionsManager.smallBottomToolbar && !(_extensionsManager.smallBottomToolbar.hidden) ? -28 : -8;

        [_pageSlider mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.bottom.mas_equalTo(self.pageSlider.superview.fs_mas_bottom).offset(bottomHeight);
            make.height.mas_equalTo(sliderHeight);
            make.left.mas_equalTo(self.pageSlider.superview.fs_mas_left).offset(space);
            make.right.mas_equalTo(self.pageSlider.superview.fs_mas_right).offset(-space);
        }];
        if (_pageNumView.superview) {
            [_pageNumView mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.height.mas_equalTo(26);
                make.left.mas_equalTo(self.pageNumView.superview.fs_mas_left).offset([self checkIsIphoneDevice] ? 16 : 44);
                make.bottom.mas_equalTo(self.pageSlider.mas_top).offset(-8);
            }];
        }
    }

    _pageSlider.transform = CGAffineTransformIdentity;
    if (_pdfViewCtrl.pageBindingEdge == PDF_PAGE_BINDING_RIGHT_EDGE) {
        _pageSlider.transform = CGAffineTransformScale(_pageSlider.transform, -1.0, 1.0);
    }
}

- (CGFloat)getTopbarHeight
{
    if (!_extensionsManager.topToolbar.hidden)
    {
        return _extensionsManager.topToolbar.contentView.frame.size.height;
    }
    FSUIManagerState state = [_extensionsManager getState];
    if (state == FSUIManagerStateReflow || state == FSUIManagerStateSpeech || state == FSUIManagerStateRedaction || state == FSUIManagerStatePanZoom || state == FSUIManagerStateAutoFlip || state == FSUIManagerStateSearch) {
        if (!_extensionsManager.currentIsFullScreen) {
            for (UIView *subViews in _extensionsManager.containerView.subviews) {
                if ([subViews isKindOfClass:[FSToolbar class]] && ((FSToolbar*)subViews).style == FSToolbarStyleTop && !subViews.hidden && ((FSToolbar*)subViews).tag == TOPBAR_PAGESLIDER_TAG)
                {
                    return subViews.frame.size.height;
                }
            }
        }
    }
    
    
    return 0;
}

- (void)setPageSliderLayout:(CGFloat)pdfViewWidth
{
    BOOL isVertical = [self checkPageSliderIsVertical];
    CGFloat space = 8;
    CGFloat sliderHeight = kPageSliderHeight;
    if (isVertical) {
        
        CGFloat bottomAreaHeight = _pdfViewCtrl.fs_safeAreaInsets.bottom;
        CGFloat topHeight = [self getTopbarHeight];
        CGFloat bottomHeight = ([self checkIsIphoneDevice] ? TAB_BAR_HEIGHT_NORMAL : 0) + bottomAreaHeight;
        if (_extensionsManager.currentIsFullScreen && (_extensionsManager.topToolbar.hidden || !_extensionsManager.topToolbar))
        {
            CGFloat fullScreenTopbarHeight = 15;
            if (_pdfViewCtrl.fs_safeAreaInsets.top == 0 ) {
                fullScreenTopbarHeight = 20;
            }else if (_extensionsManager.fullScreenToolbarMgr.smallTopToolbar.hidden){
                fullScreenTopbarHeight = 0;
            }
            
            topHeight = fullScreenTopbarHeight + _pdfViewCtrl.fs_safeAreaInsets.top;
            bottomHeight = bottomAreaHeight + (_extensionsManager.fullScreenToolbarMgr.smallBottomToolbar.hidden ? 0 : 20);
        }
        if (topHeight < 28) {
            topHeight = 28;
        }
        CGFloat sliderWidth = _pdfViewCtrl.fs_height-topHeight-bottomHeight-space*2;
        CGFloat centerYoffset = (topHeight-bottomHeight)/2;
        [_pageSlider mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.centerX.mas_equalTo(self.pageSlider.superview);
            make.centerY.mas_equalTo(self.pageSlider.superview).offset(centerYoffset);
            make.height.mas_equalTo(sliderHeight);
            make.width.mas_equalTo(sliderWidth);
        }];
        
        CGAffineTransform trans = CGAffineTransformMakeRotation(M_PI_2);
        
        CGFloat posFag =  1;
        if (_extensionsManager.toolbarManager.mainTopbar.subitemContentPosition == FSTopbarSubitemContentViewPositionRight && ![self checkIsIphoneDevice]) {
            posFag = 1;
            _pageSliderPos = FSPageSliderPosLeft;
        }else {
            posFag = -1;
            _pageSliderPos = FSPageSliderPosRight;
        }

        CGAffineTransform translate = CGAffineTransformTranslate(trans, 0, posFag*(pdfViewWidth-sliderHeight)/2 - posFag*space - posFag*_pdfViewCtrl.fs_safeAreaInsets.right);
        _pageSlider.transform = translate;
    }
    else
    {
        _pageSliderPos = FSPageSliderPosBottom;
        [self setPageSliderLayoutWithHorization];
    }
}

- (void)onPageNumberSliderTouchUpInside
{
    [_extensionsManager delayAutoFullScreen];
    _isFristChangeForSlider = YES;
    int index = [_pdfViewCtrl getCurrentPage];
    if (index != _indexForSliderDown) {
//        [_pdfViewCtrl addPageJumpedNode:_indexForSliderDown index:index animated:YES];
    }
}

- (void)onPageNumberSliderValueChanged
{
    if (_isFristChangeForSlider) {
        _isFristChangeForSlider = NO;
        _indexForSliderDown = [_pdfViewCtrl getCurrentPage];
        [_extensionsManager cancelAutoFullScreen];
    }
    if ([self checkPageSliderIsVertical]) {
        UIScrollView* scrollView = [_pdfViewCtrl getPageContainer];
        scrollView.contentOffset = CGPointMake(scrollView.contentOffset.x, _pageSlider.value);
    }else{
        [self gotoPage:_pageSlider.value animated:NO isJump:NO];
    }
}

- (BOOL)checkIsBottombarHidden
{
    return _extensionsManager.currentIsFullScreen && _extensionsManager.bottomToolbar.hidden;
}

- (BOOL)checkIsFullScreenForReading
{
    return _extensionsManager.currentIsFullScreen;
}

- (void)showGotoPageAlertView
{
    [_extensionsManager onUIInteractElementClicked:FSUIElementReadingGotoPage];
    
    [_extensionsManager cancelAutoFullScreen];
    NSString *title = FSLocalizedForKey(@"kGotoPage");
    NSString *msg = FSLocalizedForKey(@"kEnterPageNumber");
    NSString *pageNumStr = [NSString stringWithFormat:@" (%d/%d)",[_pdfViewCtrl getCurrentPage]+1, [_pdfViewCtrl getPageCount]];
    if (_extensionsManager.useLogicalPageNumbers){
        NSString *title = [Utility getPageLabelTitle:[_pdfViewCtrl getCurrentPage]  doc:[_pdfViewCtrl currentDoc]];
        if (title.length){
            pageNumStr = [NSString stringWithFormat:@" (%@(%d/%d))",title, [_pdfViewCtrl getCurrentPage]+1, [_pdfViewCtrl getPageCount]];
        }
    }
    msg = [msg stringByAppendingString:pageNumStr];
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:title message:msg preferredStyle:UIAlertControllerStyleAlert];
    @weakify(self)
    UIAlertAction *okAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        @strongify(self)
        [self->_extensionsManager delayAutoFullScreen];
        NSString *stringPage = [alertController.textFields.firstObject.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
        int pageIndex = 0;
        
        if (self->_extensionsManager.useLogicalPageNumbers){
            FSPDFDoc *doc = self->_pdfViewCtrl.getDoc;
            FSPageLabels *pageLabels = [[FSPageLabels alloc] initWithDocument:doc];
            if ([pageLabels hasPageLabel:0]){
                if (![pageLabels isEmpty]){
                    for (int i = 0; i < [doc getPageCount]; i++) {
                        NSString *title = [pageLabels getPageLabelTitle:i];
                        if ([title isEqualToString:stringPage]){
                            pageIndex = i + 1;
                            break;
                        }
                    }
                }
            }
            if (pageIndex == 0){
                if ([stringPage fs_isNumber]){
                    if (stringPage != nil && stringPage.length != 0) {
                        pageIndex = stringPage.intValue;
                    }
                }
                if (pageIndex == 0) {
                    FSAlertViewCtrlShowOnCtrl(FSLocalizedForKey(@"kWarning"), FSLocalizedFormatForKey(@"kInvalidLogicalPageNumber", stringPage), self->_pdfViewCtrl.fs_viewController);
                    return;
                }
            }
        }else{
            if ([stringPage fs_isNumber]){
                if (stringPage != nil && stringPage.length != 0) {
                    pageIndex = stringPage.intValue;
                }
            }
        }
        
        [self gotoPageWithPageIndex:pageIndex];
    }];
   
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel") style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {
        @strongify(self)
        [self quitGotoMode];
    }];
    [alertController addTextFieldWithConfigurationHandler:^(UITextField * _Nonnull textField) {
        textField.accessibilityIdentifier = @"input_pageindex_id";
        textField.clearButtonMode = UITextFieldViewModeWhileEditing;
        textField.delegate = self;
        if (!self->_extensionsManager.useLogicalPageNumbers){
            textField.keyboardType = UIKeyboardTypeNumberPad;
        }
        [textField setBlockForControlEvents:UIControlEventEditingChanged block:^(UITextField * _Nonnull sender) {
            @strongify(self)
            if (!self->_extensionsManager.useLogicalPageNumbers){
                okAction.enabled = !(sender.text.intValue > [self->_pdfViewCtrl getPageCount] || sender.text.intValue <= 0);
            }else{
                okAction.enabled = sender.text.length;
            }
        }];
    }];
    okAction.enabled = NO;
    [alertController addAction:cancelAction];
    [alertController addAction:okAction];
    [_extensionsManager.viewCtrlManager presentViewController:alertController animated:YES completion:nil];
}


#pragma mark -- UITextFieldDelegate
- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string
{
    if (!self->_extensionsManager.useLogicalPageNumbers){
        NSCharacterSet *charSet = [[NSCharacterSet characterSetWithCharactersInString:@"0123456789"] invertedSet];
        NSString *filteredStr = [[string componentsSeparatedByCharactersInSet:charSet] componentsJoinedByString:@""];
        if ([string isEqualToString:filteredStr]) {
            return YES;
        }
        return NO;
    }
    return YES;
}

- (void)addPageNumberView {
    self.prevImage.hidden = YES;
    self.nextImage.hidden = YES;

    [_extensionsManager.containerView addSubview:_pageNumView];
    self.pageNumView.layer.cornerRadius = 13;
    self.pageNumView.backgroundColor = [UIColor colorWithWhite:0.0 alpha:0.425];
}

- (void)removePageNumberView {
    UIView *pageNumView = self.pageNumView;
    dispatch_async(dispatch_get_main_queue(), ^{
        [pageNumView removeFromSuperview];
    });
}

- (void)gotoPageWithPageIndex:(int)pageIndex {
    if (pageIndex <= 0 || pageIndex > [_pdfViewCtrl getPageCount]) {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:[NSString stringWithFormat:@"%@ %d - %d", FSLocalizedForKey(@"kWrongPageNumber"), 1, [_pdfViewCtrl getPageCount]] preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                         style:UIAlertActionStyleDefault
                                                       handler:^(UIAlertAction *action) {
                                                           [self showGotoPageToolbar:nil];
                                                       }];
        [alertController addAction:action];
        [_pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
    } else {
        [self gotoPage:pageIndex - 1 animated:NO isJump:YES];
    }
}

- (BOOL)gotoPage:(int)index animated:(BOOL)animated isJump:(BOOL)isJump{
    int pageCount = [_pdfViewCtrl getPageCount];
    if ([_pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_LEFT || [_pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_RIGHT || [_pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_MIDDLE) {
        pageCount = pageCount +1;
    }
    
    if (index >= 0 && index < pageCount) {
        [_pdfViewCtrl gotoPage:index animated:animated needJumpedNode:isJump];
        return YES;
    }
    FSCLog(@"Attempt to go to page index out of range: %d", index);
    return NO;
}

- (void)quitGotoMode {
    [_extensionsManager delayAutoFullScreen];
    [self showPageNumberWithState:NO pdfViewWidth:_pdfViewCtrl.fs_width];
}

- (void)showPageNum:(BOOL)isShow
{
    if (!isShow) {
        self.pageNumView.hidden = YES;
        self.pageSlider.hidden = YES;
    }else{
        [self showPageNumberWithState:NO pdfViewWidth:_pdfViewCtrl.fs_width];
    }
}

- (void)showGotoPageToolbar:(UITapGestureRecognizer *)recognizer {
    if (_extensionsManager.getState == FSUIManagerStateSearch) {
        [_extensionsManager changeState:FSUIManagerStateNormal];
    }
    [self showGotoPageAlertView];
    [self showPageNumberWithState:NO pdfViewWidth:_pdfViewCtrl.fs_width];
}

- (void)handleGotoPrevView:(UITapGestureRecognizer *)recognizer {
    [_extensionsManager setCurrentAnnot:nil];

    if ([_pdfViewCtrl hasPrevView]) {
        [_pdfViewCtrl gotoPrevView:NO];
    }
}

- (void)handleGotoNextView:(UITapGestureRecognizer *)recognizer {
    [_extensionsManager setCurrentAnnot:nil];

    if ([_pdfViewCtrl hasNextView]) {
        [_pdfViewCtrl gotoNextView:NO];
    }
}

- (void)updatePageNumView {

    _totalNumLabel.text = [Utility getDisplayPageLabel:[_pdfViewCtrl getCurrentPage] needTotal:YES useLogicalPageNumbers:_extensionsManager.useLogicalPageNumbers pdfViewCtrl:_pdfViewCtrl];
    _totalNumLabel.font = self.fs_standardGeneralFont_GF3;
    
    [self updateTotalNumLabelConstraints];

    if (([self checkIsFullScreenForReading] && _extensionsManager.isFullScreen) ||
        !_totalNumLabel.text.length ||
        [_pdfViewCtrl getPageCount] == 1) {
        self.pageNumView.hidden = YES;
    }else{
        self.pageNumView.hidden = NO;
    }
}

- (void)updateTotalNumLabelConstraints{
    [self.totalNumLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.centerY.mas_equalTo(self.pageNumView);
        if (self.prevImage.hidden) {
            make.left.mas_equalTo(self.pageNumView).offset(10.f);
        }else{
            make.left.mas_equalTo(self.prevImage.mas_right);
        }

        if (self.nextImage.hidden) {
            make.right.mas_equalTo(self.pageNumView).offset(-10.f);
        }else{
            make.right.mas_equalTo(self.nextImage.mas_left);
        }
    }];
}

- (void)setPageSliderShowState
{
    UIScrollView* scrollView = (UIScrollView*)[_pdfViewCtrl getPageContainer];
    if ([self checkPageSliderIsVertical]) {
        CGFloat height = scrollView.frame.size.height;
        self.pageSlider.hidden = (height >= _pdfViewCtrl.getVScrollRange) || _pdfViewCtrl.getPageCount <= 1 ? YES : NO;
    }else{
        CGFloat width = scrollView.frame.size.width;
        self.pageSlider.hidden = (width >= _pdfViewCtrl.getHScrollRange) || _pdfViewCtrl.getPageCount <= 1 ? YES : NO;
    }
}

- (void)showPageNumberWithState:(BOOL)isOnPageChange pdfViewWidth:(CGFloat)pdfViewWidth
{
    if (_extensionsManager.currentIsFullScreen  && _extensionsManager.topToolbar.hidden /*&& !isOnPageChange*/) {
        self.pageNumView.hidden = YES;
        self.pageSlider.hidden = YES;
        [self updatePageNumView];
    } else {
        [self changePageNumberStateWithShow];
        [self setPageSliderShowState];
        [self setPageSliderLayout:pdfViewWidth];
        [self setPageCountLabel];
        
        [self setPageSliderShowState];
    }
}


- (void)setPageCountLabel{
    if (!self.pageNumView.isHidden)
    {
        if ([_pdfViewCtrl hasPrevView]) {
            [self updateTotalNumLabelConstraints];
            self.prevImage.hidden = NO;
        }else{
            self.prevImage.hidden = YES;
        }
        if ([_pdfViewCtrl hasNextView]) {
            [self updateTotalNumLabelConstraints];
            self.nextImage.hidden = NO;
        }else{
            self.nextImage.hidden = YES;
        }
        
        self.totalNumLabel.textColor = self.fs_textColor_T1;
    }
    [self updatePageNumView];
}

- (void)loadModule {
    [_pdfViewCtrl registerScrollViewEventListener:self];
    [_pdfViewCtrl registerGestureEventListener:self];
    [_pdfViewCtrl registerPageEventListener:self];
    [_pdfViewCtrl registerDocEventListener:self];

    [_extensionsManager registerFullScreenListener:self];
    [_extensionsManager registerStateChangeListener:self];
    [_extensionsManager registerRotateChangedListener:self];
    
    [_extensionsManager registerPageNumberListener:self];
    [_extensionsManager.toolbarManager registerTopbarEventListener:self];
    [_extensionsManager registerWindowFrameChangedListener:self];
}

#pragma pageEventListener methods

- (void)onPageChanged:(int)oldIndex currentIndex:(int)currentIndex {
    [self showPageNumberWithState:YES pdfViewWidth:_pdfViewCtrl.fs_width];
    
    [self setSliderCurrentValue];
    
    if (UIAccessibilityIsVoiceOverRunning()) {
        UIView *element = [_pdfViewCtrl getPageView:currentIndex];
        UIAccessibilityPostNotification(UIAccessibilityScreenChangedNotification, element);
    }
}

- (void)changePageNumberStateWithShow
{
    [self updatePageNumView];
    if (!([self checkIsFullScreenForReading] && _extensionsManager.isFullScreen)) {
        CGFloat space = 8;
        CGFloat bottomBarHeight = 0;
        if (![self checkIsBottombarHidden] && [self checkIsIphoneDevice]) {
            bottomBarHeight = TAB_BAR_HEIGHT_NORMAL;
        }else{
            bottomBarHeight = !_extensionsManager.smallBottomToolbar.hidden ? 20 : 0;
        }
        if ([_extensionsManager getState] == FSUIManagerStateComparision)
        {
            bottomBarHeight = 0;
        }
        CGFloat sliderHeight = kPageSliderHeight;
        CGFloat bottomHeight = 0;
        if ([self checkPageSliderIsVertical]) {
            bottomHeight = bottomBarHeight  + space*2;
        }else{
            bottomHeight = bottomBarHeight  + space*2+sliderHeight;
        }
        
        if ([_pageNumView superview]) {
            [self.pageNumView mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.height.mas_equalTo(26);
                make.left.mas_equalTo(self.pageNumView.superview.fs_mas_left).offset([self checkIsIphoneDevice] ? 16 : 44);
                make.bottom.mas_equalTo(self.pageNumView.superview.fs_mas_bottom).offset(-1*bottomHeight);
            }];
        }
    }
}

- (void)onPageJumped {
    [self setPageCountLabel];
}

- (void)onPagesRemoved:(NSArray<NSNumber *> *)indexes {
    // If the document has only one page, there should not be a previous page and next page
    if ([_pdfViewCtrl getPageCount] == 1) {
        [_pdfViewCtrl clearPrevNextStack];
    }
    [self setPageCountLabel];
    [self setPageSliderShowState];
    [self setPageSliderMaxMinValue];
}

- (void)onPagesInsertedAtRange:(NSRange)range {
    // If insert a page, the data in the stack becomes invalid and needs to be empty
    [_pdfViewCtrl clearPrevNextStack];
    [self setPageCountLabel];
    [self setPageSliderShowState];
    [self setPageSliderMaxMinValue];
}

- (void)onPagesRotated:(NSArray<NSNumber *> *)indexes rotation:(int)rotation{
    [self setPageSliderMaxMinValue];
}

- (void)onPagesMoved:(NSArray<NSNumber *> *)indexes dstIndex:(int)dstIndex {
    // If insert a page, the data in the stack becomes invalid and needs to be empty
    [_pdfViewCtrl clearPrevNextStack];
    [self setPageCountLabel];
    [self setPageSliderShowState];
}

#pragma docEventlistener methods

- (void)onDocWillOpen {
}

- (void)onDocOpened:(FSPDFDoc *)document error:(int)error {
    if(error != FSErrSuccess) return;
    [self setPageCountLabel];
    [self setPageSliderMaxMinValue];
    
    _pageSlider.value = 0;
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        [self layoutPdfViewContentInsets:YES];
    });
    
}

- (void)onDocWillClose:(FSPDFDoc *)document {

}

- (void)onDocClosed:(FSPDFDoc *)document error:(int)error {
}

- (void)onDocWillSave:(FSPDFDoc *)document {
}

- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration {
    if (!_pdfViewCtrl.currentDoc) {
        return ;
    }

    self.pageNumView.hidden = YES;
    self.pageSlider.hidden = YES;
}

- (void)willAnimateRotationToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration{
    [self layoutPdfViewContentInsets:NO];
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation {
    if (!_pdfViewCtrl.currentDoc) {
        return ;
    }
    [self setPageSliderMaxMinValue];
    [self setSliderCurrentValue];
    [self showPageNumberWithState:NO pdfViewWidth:_pdfViewCtrl.fs_width];
    
}

- (BOOL)checkPageSliderIsVertical
{
    return _pdfViewCtrl.isContinuous || [_extensionsManager getState] == FSUIManagerStateReflow;
}

- (void)setPageSliderMaxMinValue
{
    self.pageSlider.minimumValue = 0;
    if ([self checkPageSliderIsVertical]) {
        UIScrollView* scrollView = (UIScrollView*)[_pdfViewCtrl getPageContainer];
        CGFloat height = scrollView.frame.size.height;
        self.pageSlider.minimumValue = -scrollView.contentInset.top;
        self.pageSlider.maximumValue = _pdfViewCtrl.getVScrollRange - height + scrollView.contentInset.bottom;
    }else{
        self.pageSlider.maximumValue = _pdfViewCtrl.getPageCount - 1;
    }
}

- (void)setSliderCurrentValue
{
    if ([self checkPageSliderIsVertical]) {
        self.pageSlider.value = _pdfViewCtrl.getPageContainer.contentOffset.y;
    }else{
        
        PDF_LAYOUT_MODE pageMode = [_pdfViewCtrl getPageLayoutMode];
        if (pageMode == PDF_LAYOUT_MODE_TWO_LEFT || pageMode == PDF_LAYOUT_MODE_TWO_RIGHT || pageMode == PDF_LAYOUT_MODE_TWO_MIDDLE || pageMode == PDF_LAYOUT_MODE_TWO) {
            if (_pdfViewCtrl.getCurrentPage == _pdfViewCtrl.getPageCount - 2) {
                self.pageSlider.value = _pdfViewCtrl.getCurrentPage + 1;
            }else{
                self.pageSlider.value = _pdfViewCtrl.getCurrentPage;
            }
        }else{
            self.pageSlider.value = _pdfViewCtrl.getCurrentPage;
        }
    }
}

- (void)layoutPdfViewContentInsets:(BOOL)moveOffset{
    CGFloat bottomAreaHeight = _pdfViewCtrl.fs_currentSceneKeyWindow.fs_safeAreaInsets.bottom;
    CGFloat topEdge = [self getTopbarHeight];
    CGFloat bottomEdge = ([self checkIsIphoneDevice] ? TAB_BAR_HEIGHT_NORMAL : 0) + bottomAreaHeight;

    if (self->_extensionsManager.currentIsFullScreen && self->_extensionsManager.topToolbar.hidden && self->_extensionsManager.smallTopToolbar)
    {
        CGFloat fullScreenTopbarHeight = 15;
        if (!self->_extensionsManager.smallTopToolbar.hidden && self->_pdfViewCtrl.fs_currentSceneKeyWindow.fs_safeAreaInsets.top == 0) {
            fullScreenTopbarHeight = 20;
        }else if (self->_extensionsManager.smallTopToolbar.hidden){
            fullScreenTopbarHeight = 0;
        }
        CGFloat safeAreaTop = self->_pdfViewCtrl.fs_currentSceneKeyWindow.fs_safeAreaInsets.top;
        if (self->_extensionsManager.smallTopToolbar.hidden) {
            safeAreaTop = 0;
        }
        topEdge = fullScreenTopbarHeight + safeAreaTop;
        bottomEdge = bottomAreaHeight;
    }

    [self->_pdfViewCtrl setPageViewEdgeInsets:UIEdgeInsetsMake(topEdge, 0, bottomEdge, 0) moveOffset:moveOffset];
    
    [self setPageSliderMaxMinValue];
    [self setSliderCurrentValue];
}

- (void)keyboardDidShow{
    self.isKeyboardShow = YES;
}

- (void)keyboardDidHide{
    self.isKeyboardShow = NO;
}

#pragma - IFullScreenListener

- (void)onFullScreen:(BOOL)isFullScreen {
    if (!isFullScreen) {
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.3 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
            [self layoutPdfViewContentInsets:NO];
            [self changePageNumberStateWithShow];
            [self setPageSliderShowState];
            [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(delayOnFullScreen) object:nil];
            [self performSelector:@selector(delayOnFullScreen) withObject:nil afterDelay:5];
            [self setPageSliderLayout:self->_pdfViewCtrl.fs_width];
            
        });

    }else{
        [self layoutPdfViewContentInsets:NO];
        [self setPageSliderMaxMinValue];
        [self setSliderCurrentValue];
        [self showPageNumberWithState:NO  pdfViewWidth:_pdfViewCtrl.fs_width];
        
    }
    
}

- (void)delayOnFullScreen
{
//    [self setPageSliderMaxMinValue];
//    [self setSliderCurrentValue];
//    [self showPageNumberWithState:NO  pdfViewWidth:_pdfViewCtrl.fs_width];
    self.pageNumView.hidden = YES;
    self.pageSlider.hidden = YES;
}

#pragma mark IPageNumberListener
-(void)updatePageNumber {
    [self setPageSliderMaxMinValue];
    [self showPageNumberWithState:NO pdfViewWidth:_pdfViewCtrl.fs_width];
    [self setSliderCurrentValue];
    
    PDF_LAYOUT_MODE pageMode = [_pdfViewCtrl getPageLayoutMode];
    if (pageMode == PDF_LAYOUT_MODE_TWO_LEFT || pageMode == PDF_LAYOUT_MODE_TWO_RIGHT || pageMode == PDF_LAYOUT_MODE_TWO_MIDDLE) {
        if(_pdfViewCtrl.getCurrentPage == 1 && ![_pdfViewCtrl isContinuous]) {
            if(_pdfViewCtrl.IsChangeLayoutMode ){
                _pageSlider.value = _pdfViewCtrl.getCurrentPage - 1;
                _pdfViewCtrl.IsChangeLayoutMode = NO;
            }
        }
    }
}

#pragma mark IGestureEventListener

- (BOOL)onTap:(UITapGestureRecognizer *)recognizer {

    return NO;
}

- (BOOL)onLongPress:(UILongPressGestureRecognizer *)recognizer {
    return NO;
}

#pragma - (void) onStateChanged : (int) state;

- (void)onStateChanged:(FSUIManagerState)state{
    [self showPageNumberWithState:NO pdfViewWidth:_pdfViewCtrl.fs_width];
    if (state == FSUIManagerStateReflow) {
        [self layoutPdfViewContentInsets:YES];

    }else{
        [self layoutPdfViewContentInsets:NO];
    }
}

- (void)setIsFormNavBarShowState:(BOOL)isFormNavBarShowState
{
    _isFormNavBarShowState = isFormNavBarShowState;
    if (isFormNavBarShowState) {
        [self showPageNum:NO];
    }
    else
    {
        if (![self.pageSlider isHidden]) {
            [self setPageSliderLayout:_pdfViewCtrl.fs_width];
        }
    }
}

#pragma mark-- IScrollViewEventListener

- (void)onScrollViewWillBeginDragging:(UIScrollView *)scrollView
{
    if (!(self->_extensionsManager.currentIsFullScreen)) {
        [self changePageNumberStateWithShow];
        [self setPageSliderShowState];
        [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(delayOnFullScreen) object:nil];
        [self performSelector:@selector(delayOnFullScreen) withObject:nil afterDelay:5];
    }
}

- (void)onScrollViewDidScroll:(UIScrollView *)scrollView
{
    if (scrollView != _pdfViewCtrl.getPageContainer) {
        return;
    }
    if (_extensionsManager.currentState == FSUIManagerStateReflow) {
        if (!CGSizeEqualToSize(scrollView.contentSize, self.reflowScrollViewContentSize)) {
            self.reflowScrollViewContentSize = scrollView.contentSize;
            [self setPageSliderMaxMinValue];
            [self setSliderCurrentValue];
        }
    }else{
        [self setPageSliderMaxMinValue];
        [self setSliderCurrentValue];
    }
}

- (void)onScrollViewWillBeginZooming:(UIScrollView *)scrollView{
    if (scrollView != _pdfViewCtrl.getPageContainer) {
        return;
    }

    [self setPageSliderMaxMinValue];
    [self setSliderCurrentValue];
}

- (void)onScrollViewDidZoom:(UIScrollView *)scrollView{
    if (scrollView != _pdfViewCtrl.getPageContainer) {
        return;
    }

    [self setPageSliderMaxMinValue];
    [self setSliderCurrentValue];
}

- (void)onScrollViewDidEndZooming:(UIScrollView *)scrollView{
    
    if (scrollView != _pdfViewCtrl.getPageContainer) {
        return;
    }

    [self setPageSliderMaxMinValue];
    [self setSliderCurrentValue];
}

#pragma mark-- IFSMainTopbarEventListener
- (void)onSubitemViewChanged:(FSTopbarSubitemContentViewPosition)position
{
    [self showPageNumberWithState:NO pdfViewWidth:_pdfViewCtrl.fs_width];
}

#pragma mark-- IWindowFrameChangedListener
- (void)onWindowFrameChanged:(CGRect)newFrame horizontalSizeClassChanged:(BOOL)changed
{
    if (DEVICE_iPAD ) {
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
            [self showPageNumberWithState:NO pdfViewWidth:newFrame.size.width];
        });
    }
    [self layoutPdfViewContentInsets:NO];
}

- (BOOL)checkIsIphoneDevice
{
    return DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact];
}
@end
