/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "PanAndZoomViewController.h"
#import "Utility.h"
#import "PageNavigationModule.h"
#import "FSViewSettingManager.h"
#import "FSSettingBar+private.h"
@interface borderView : UIView
@property(nonatomic, strong) UIColor* borderColor;
- (CGRect)receiveImageViewRect:(CGRect)rect;
@property(nonatomic, weak) UIExtensionsManager * extensionsManager;
@end

@implementation borderView
{
    CGRect _imageViewFrame;
    FSPDFViewCtrl*__weak _pdfViewCtrl;
}

- (instancetype)initWithFrame:(CGRect)frame pdfViewCtrl:(FSPDFViewCtrl*)pdfViewCtrl{
    self = [super initWithFrame:frame];
    _pdfViewCtrl = pdfViewCtrl;
    
    self.contentMode = UIViewContentModeRedraw;
    UIPanGestureRecognizer* panGesture = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(handlePan:)];
    [self addGestureRecognizer:panGesture];
    
    self.borderColor = self.fs_primaryColor;
    [BindThemeKeySel(self, setNeedsDisplay) fs_updateThemeCompleted:^(borderView*  _Nonnull target, SEL  _Nonnull sel) {
        target.borderColor = target.fs_primaryColor;
        [target setNeedsDisplay];
    }];
    
    return self;
}

- (void)drawRect:(CGRect)rect {
    
    //draw rect line
    CGMutablePathRef path = CGPathCreateMutable();
    CGRect originRec = self.bounds;
    originRec = CGRectInset(originRec, 0, 2);
    CGPathAddRect(path,NULL, originRec);
    CGContextRef currentContext = UIGraphicsGetCurrentContext();
    CGContextAddPath(currentContext, path);
    
    [self.borderColor setStroke];
 
    CGContextSetLineWidth(currentContext, 2.0f);
    
    CGContextDrawPath(currentContext, kCGPathStroke);
    
    CGPathRelease(path);
}

- (CGRect)receiveImageViewRect:(CGRect)rect {
    _imageViewFrame = rect;
    return rect;
}

- (void)handlePan:(UIPanGestureRecognizer*)recognizer
{
    if (recognizer.state == UIGestureRecognizerStateBegan) {
        [_extensionsManager cancelAutoFullScreen];
    }else if (recognizer.state == UIGestureRecognizerStateChanged){
        CGPoint translation = [recognizer translationInView:self];
        recognizer.view.center = CGPointMake(recognizer.view.center.x + translation.x,
                                             recognizer.view.center.y + translation.y);
        //set contraint border
        if ( !CGRectContainsRect(_imageViewFrame, recognizer.view.frame) ) {
            CGPoint oldTranslation = translation;
            if (CGRectGetMinX(_imageViewFrame) - CGRectGetMinX(recognizer.view.frame) > 0) {
                translation.x += CGRectGetMinX(_imageViewFrame) - CGRectGetMinX(recognizer.view.frame);
            }
            if (CGRectGetMaxX(_imageViewFrame) - CGRectGetMaxX(recognizer.view.frame) < 0) {
                translation.x += CGRectGetMaxX(_imageViewFrame) - CGRectGetMaxX(recognizer.view.frame);
            }
            if (CGRectGetMinY(_imageViewFrame) - CGRectGetMinY(recognizer.view.frame) > 0) {
                translation.y += CGRectGetMinY(_imageViewFrame) - CGRectGetMinY(recognizer.view.frame);
            }
            if (CGRectGetMaxY(_imageViewFrame) - CGRectGetMaxY(recognizer.view.frame) < 0) {
                translation.y += CGRectGetMaxY(_imageViewFrame) - CGRectGetMaxY(recognizer.view.frame);
            }
            recognizer.view.center = CGPointMake(recognizer.view.center.x - oldTranslation.x,
                                                 recognizer.view.center.y - oldTranslation.y);
            
            recognizer.view.center = CGPointMake(recognizer.view.center.x + translation.x,
                                                 recognizer.view.center.y + translation.y);
        }
        CGFloat scale = _pdfViewCtrl.bounds.size.width / recognizer.view.frame.size.width;
        [_pdfViewCtrl scrollDisplayView:translation.x * scale distanceY:translation.y * scale];
        
        [recognizer setTranslation:CGPointZero inView:self];
    }else{
        [_extensionsManager delayAutoFullScreen];
    }
    
}
@end

@interface PanZoomView ()<IThumbnailEventListener>
{
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;
    
    CGRect _tempSelfFrame;
    
    borderView * _borderView;
    CGRect _imageViewFrame;
    CGFloat _aspectRatio;
    int _pageIndex;
    
}
@property (nonatomic, assign) BOOL showPanZoom;
@property (nonatomic, assign) BOOL needPauseNow;
@property (nonatomic, strong) FSToolbar* topToolBar;
@property (nonatomic, strong) FSToolbar* bottomToolBar;
@property (nonatomic, strong, nullable) UIView *panZoomBottomContentView;
@property (nonatomic, strong) UISlider* zoomSlider;
@property (nonatomic, strong) UIBarButtonItem* prevPageButtonItem;
@property (nonatomic, strong) UIBarButtonItem* nextPageButtonItem;
@property (nonatomic, assign) BOOL isSelfState;
@property (nonatomic, strong) FSItem* closeItem;
@property (nonatomic, strong) FSItem* titleItem;
@property (nonatomic, strong) UIPanGestureRecognizer *pan;
@property (nonatomic, strong) FSItem* exitItem;
@property (nonatomic, strong) UIButton *nextPageButton;
@property (nonatomic, strong) UIButton *zoomOutButton;
@end
@implementation PanZoomView

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager*)uiextensionManager {
    _extensionsManager = uiextensionManager;
    _pdfViewCtrl = uiextensionManager.pdfViewCtrl;
    _pageIndex = [_pdfViewCtrl getCurrentPage];
    _showPanZoom = NO;
    _isSelfState = NO;
    
    [self correctPageIndexInCoverMode];
    
    [self correctPageIndexInFacingMode];

    self = [super initWithFrame:CGRectZero];
    //gesture
    [_extensionsManager registerAnnotEventListener:self];
    [_extensionsManager registerRotateChangedListener:self];
    [_pdfViewCtrl registerPageEventListener:self];
    [_pdfViewCtrl registerScrollViewEventListener:self];
    [_pdfViewCtrl registerLayoutEventListener:self];
    [_pdfViewCtrl registerDocEventListener:self];
    [_pdfViewCtrl registerGestureEventListener:self];
    [_extensionsManager registerStateChangeListener:self];
    [_extensionsManager registerFullScreenListener:self];
    [_extensionsManager.settingBar.viewSettingManager registerEventListener:self];
    [_extensionsManager.panelController registerPanelChangedListener:self];
    [_extensionsManager.thumbnailModule registerThumbnailEventListener:self];  
    [_extensionsManager registerWindowFrameChangedListener:self];
    
    UIPanGestureRecognizer* panGesture = [[UIPanGestureRecognizer alloc] initWithTarget:_extensionsManager.settingBar.viewSettingManager action:@selector(moveBottombarWithRecognizer:)];
    [self addGestureRecognizer:panGesture];

    self.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin | UIViewAutoresizingFlexibleTopMargin | UIViewAutoresizingFlexibleBottomMargin | UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
    self.backgroundColor = self.fs_backgroundColor_B4;
    self.opaque = YES;
    self.userInteractionEnabled = YES;
    
    self.layer.borderColor = self.fs_textColor_T2.CGColor;
    self.layer.borderWidth = 2;
    
    self.imageView = [[UIImageView alloc] initWithFrame:CGRectZero];
    self.imageView.contentMode = UIViewContentModeScaleAspectFit;
    [self addSubview:self.imageView];
    [self bringSubviewToFront:self.imageView];
    
    _borderView = [[borderView alloc] initWithFrame:CGRectZero pdfViewCtrl:_pdfViewCtrl];
    _borderView.opaque = NO;
    _borderView.userInteractionEnabled = YES;
    _borderView.extensionsManager = _extensionsManager;
    [self addSubview:_borderView];
    [self bringSubviewToFront:_borderView];

    [self bulidToolbar];
    [self setHidden:YES];
    return self;
}

#pragma mark -- IFullScreenListener
- (void)onFullScreen:(BOOL)isFullScreen {
    if ([_extensionsManager getState] == FSUIManagerStatePanZoom && !(isFullScreen && _extensionsManager.currentToolHandler != nil)) {
        [self showPanZoom:!isFullScreen animation:YES];
    }
}

- (void)bulidToolbar
{
    [self.topToolBar setHidden:YES];
    [self.bottomToolBar setHidden:YES];
}

- (FSToolbar *)topToolBar
{
    if (_topToolBar == nil) {
        FSToolbar* topToolBar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleTop];
        topToolBar.tag = TOPBAR_PAGESLIDER_TAG;
        _topToolBar = topToolBar;
        BindThemeKeyPath(topToolBar.stateBar, backgroundColor);
        [_extensionsManager.containerView addSubview:topToolBar];
        FSItem* closeItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kClose")];
        _closeItem = closeItem;
        [closeItem setFont:self.fs_standardGeneralFont_GF1];
        [BindThemeKeySel(closeItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem*  _Nonnull target, SEL  _Nonnull sel) {
            [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
        }];
        @weakify(self)
        closeItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            [self showPanZoom:NO animation:NO];
            [self->_extensionsManager changeState:FSUIManagerStateNormal];
            [self->_extensionsManager setFullScreen:NO ];
            [self->_extensionsManager.settingBar.viewSettingManager setItemSelectedWithType:PANZOOM selected:NO];
        };
        [topToolBar addItem:closeItem atPosition:FSItemPositionLeft];
        
        FSItem* titleItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kPanandZoom")];
        titleItem.accessibilityTraits = UIAccessibilityTraitHeader;
        titleItem.customAccessibilityTraits = UIAccessibilityTraitHeader;
        _titleItem = titleItem;
        [titleItem setFont:self.fs_standardGeneralFont_GF1];
        [titleItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
        [topToolBar addSubview:titleItem];
        titleItem.titleLabel.textAlignment = NSTextAlignmentCenter;
        
        CGSize closeItemSize = [Utility getTextSize:FSLocalizedForKey(@"kClose") fontSize:closeItem.titleLabel.font.pointSize maxSize:CGSizeMake(150, 30)];
        CGFloat titleItemWidth = (_pdfViewCtrl.fs_width/2 - 16*2 - closeItemSize.width)*2;
        
        [titleItem mas_makeConstraints:^(MASConstraintMaker *make) {
            make.center.mas_equalTo(titleItem.superview);
            make.size.mas_equalTo(CGSizeMake(titleItemWidth, 20));
        }];

        [topToolBar mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.right.top.mas_equalTo(topToolBar.superview);
        }];
        titleItem.userInteractionEnabled = NO;
    }
    return _topToolBar;
}

- (void)resetTitleItem
{
    if (_titleItem) {
        CGSize closeItemSize = [Utility getTextSize:FSLocalizedForKey(@"kClose") fontSize:_closeItem.titleLabel.font.pointSize maxSize:CGSizeMake(150, 30)];
        CGFloat titleItemWidth = (_pdfViewCtrl.fs_width/2 - 16*2 - closeItemSize.width)*2;
        
        [_titleItem mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.center.mas_equalTo(_titleItem.superview);
            make.size.mas_equalTo(CGSizeMake(titleItemWidth, 20));
        }];
    }
}

- (FSToolbar *)bottomToolBar
{
    if (_bottomToolBar == nil) {
        FSToolbar* bottomToolBar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleBottom];
        [_extensionsManager.containerView addSubview:bottomToolBar];
        bottomToolBar.hasDivide = NO;
        _bottomToolBar = bottomToolBar;
        //zoom tool
        UIView *contentView = [[UIView alloc] init];
        contentView.backgroundColor = [UIColor fs_clearColor];
        [bottomToolBar addSubview:contentView];
        
        [contentView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.right.top.bottom.mas_equalTo(bottomToolBar);
        }];
        
        self.panZoomBottomContentView = contentView;
        UIButton *zoomOutButton = [Utility createButtonWithImage:FSImageNamedAndNormalColor(@"pan_zoom_out")];
        zoomOutButton.accessibilityLabel = FSLocalizedForKey(@"kPanZoomOutVoiceOver");
        _zoomOutButton = zoomOutButton;
        zoomOutButton.enabled = NO;
        [contentView addSubview:zoomOutButton];
        
        UIButton *zoomInButton = [Utility createButtonWithImage:FSImageNamedAndNormalColor(@"pan_zoom_in")];
        zoomInButton.accessibilityLabel = FSLocalizedForKey(@"kPanZoomInVoiceOver");
        zoomInButton.enabled = NO;
        [contentView addSubview:zoomInButton];
        
        //page change tool
        UIButton *prevPageButton = [Utility createButtonWithImage:FSImageNamedAndNormalColor(@"pan_zoom_prev_page")];
        prevPageButton.accessibilityLabel = FSLocalizedForKey(@"kPanZoomPreviousPageVoiceOver");
        [prevPageButton setImage:[FSImageNamedAndNormalColor(@"pan_zoom_prev_page") fs_imageWithTintColor:self.fs_textColor_T2] forState:UIControlStateDisabled];
        [contentView addSubview:prevPageButton];
        [prevPageButton addTarget:self action:@selector(onClickPrevPageButton) forControlEvents:UIControlEventTouchUpInside];
        self.prevPageButtonItem = [[UIBarButtonItem alloc] initWithCustomView:prevPageButton];
        
        UIButton *nextPageButton = [Utility createButtonWithImage:FSImageNamedAndNormalColor(@"pan_zoom_next_page")];
        nextPageButton.accessibilityLabel = FSLocalizedForKey(@"kPanZoomNextPageVoiceOver");
        _nextPageButton = nextPageButton;
        [nextPageButton setImage:[FSImageNamedAndNormalColor(@"pan_zoom_next_page") fs_imageWithTintColor:self.fs_textColor_T2] forState:UIControlStateDisabled];
        [contentView addSubview:nextPageButton];
        [nextPageButton addTarget:self action:@selector(onClickNextPageButton) forControlEvents:UIControlEventTouchUpInside];
        self.nextPageButtonItem = [[UIBarButtonItem alloc] initWithCustomView:nextPageButton];
    
        self.zoomSlider = [[UISlider alloc] initWithFrame:CGRectZero];
        self.zoomSlider.maximumValue = 10.0;
        self.zoomSlider.minimumValue = 1.0;
        self.zoomSlider.continuous = YES;
        [self.zoomSlider addTarget:self action:@selector(onZoomSliderValueChanged:) forControlEvents:UIControlEventValueChanged];
        [self.zoomSlider addTarget:self action:@selector(onZoomSliderTouch) forControlEvents:UIControlEventTouchDown];
        [self.zoomSlider addTarget:self action:@selector(onZoomSliderLeave) forControlEvents:UIControlEventTouchUpInside | UIControlEventTouchUpOutside | UIControlEventTouchCancel];
        [self.zoomSlider setThumbImage:FSImageNamed(@"pan_zoom_slider_thumb") forState:UIControlStateNormal];
        [BindThemeKeySel(self.zoomSlider, setNeedsDisplay) fs_updateThemeCompleted:^(UISlider*  _Nonnull target, SEL  _Nonnull sel) {
            target.minimumTrackTintColor = target.fs_primaryColor;
        }];
   
        [contentView addSubview:self.zoomSlider];
        
        [zoomOutButton mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(bottomToolBar.fs_mas_left).offset(ITEM_MEDIUM_SPACE_NORMAL);
            make.centerY.mas_equalTo(bottomToolBar);
        }];
        [self reLayoutBottomToolbar];
        
        [prevPageButton mas_makeConstraints:^(MASConstraintMaker *make) {
            make.right.equalTo(nextPageButton.mas_left).offset(-ITEM_MEDIUM_SPACE_NORMAL);
            make.size.mas_equalTo(zoomOutButton);
            make.centerY.mas_equalTo(zoomOutButton);
        }];
        
        [zoomInButton mas_makeConstraints:^(MASConstraintMaker *make) {
              make.right.equalTo(prevPageButton.mas_left).offset(-ITEM_MEDIUM_SPACE_NORMAL);
              make.size.mas_equalTo(zoomOutButton);
              make.centerY.mas_equalTo(zoomOutButton);
          }];
        
        [self.zoomSlider mas_makeConstraints:^(MASConstraintMaker *make) {
            make.right.equalTo(zoomInButton.mas_left).offset(-ITEM_MEDIUM_SPACE_NORMAL);
            make.centerY.mas_equalTo(zoomOutButton);
            make.left.equalTo(zoomOutButton.mas_right).offset(ITEM_MEDIUM_SPACE_NORMAL);
        }];
 
        [self resetPageButtonState];
        
    }
    return _bottomToolBar;
}

- (void)onZoomSliderValueChanged:(UISlider*)slider {
    FSMessageDebouncePerformDefault(^{
        CGFloat scaleFactor = slider.value;
        [self->_pdfViewCtrl lockRefresh];
        [self->_pdfViewCtrl setZoom:scaleFactor withSlider: YES];
        [self->_pdfViewCtrl unlockRefresh];
        [self->_extensionsManager.settingBar.viewSettingManager setItemSelectedWithType:FITPAGE selected:NO];
        [self->_extensionsManager.settingBar.viewSettingManager setItemSelectedWithType:FITWIDTH selected:NO];
    });
}

- (void)onZoomSliderTouch{
    [_extensionsManager cancelAutoFullScreen];
}

- (void)onZoomSliderLeave{
    [_extensionsManager delayAutoFullScreen];
}

- (BOOL)checkPageSliderIsVertical
{
    return _pdfViewCtrl.isContinuous || [_extensionsManager getState] == FSUIManagerStateReflow;
}

- (void)resetPageButtonState{
    if ([self checkPageSliderIsVertical]) {
        self.nextPageButtonItem.enabled = ([_pdfViewCtrl getPageCount] - 1 != _pdfViewCtrl.getCurrentPage) &&
        _pdfViewCtrl.getPageContainer.contentOffset.y+_pdfViewCtrl.getPageContainer.frame.size.height < _pdfViewCtrl.getPageContainer.contentSize.height;
        self.prevPageButtonItem.enabled = (_pdfViewCtrl.getCurrentPage != 0) &&
        _pdfViewCtrl.getPageContainer.contentOffset.y > -_pdfViewCtrl.getPageContainer.contentInset.top;
    }else{
        self.nextPageButtonItem.enabled = [_pdfViewCtrl getPageCount] - 1 != _pdfViewCtrl.getCurrentPage;
        self.prevPageButtonItem.enabled = _pdfViewCtrl.getCurrentPage != 0;
    }
}

- (void)onClickPrevPageButton {
    if ([_extensionsManager canAutoHideTopBottomToolBar]) {
        [_extensionsManager delayHideTopBottomToolBar];
    }
    [_pdfViewCtrl gotoPrevPage:NO];
    
    [self resetPageButtonState];
    
    CGFloat scaleFactor = self.zoomSlider.value;
    [_pdfViewCtrl setZoom:scaleFactor withSlider: NO];
}

- (void)onClickNextPageButton {
    if ([_extensionsManager canAutoHideTopBottomToolBar]) {
        [_extensionsManager delayHideTopBottomToolBar];
    }
    [_pdfViewCtrl gotoNextPage:NO];
    
    [self resetPageButtonState];
    
    CGFloat scaleFactor = self.zoomSlider.value;
    [_pdfViewCtrl setZoom:scaleFactor withSlider: NO];
}

#pragma mark --IPanelChangedListener
- (void)onPanelChanged:(BOOL)isHidden
{
    if ([_extensionsManager getState] == FSUIManagerStatePanZoom) {
        if (!isHidden) {
            [_extensionsManager changeState:FSUIManagerStateNormal];
        }
    }
}

#pragma mark --IGestureEventListener

- (BOOL)onTap:(UITapGestureRecognizer *)recognizer {
    if ([_extensionsManager getState] == FSUIManagerStatePanZoom) {
        [_extensionsManager setFullScreen:!self.bottomToolBar.hidden];
        return YES;
    }
    return NO;
}

- (void)showPanZoom:(BOOL)isShow animation:(BOOL)animation
{

    _showPanZoom = isShow;
    [self showToolBar:isShow animation:animation];
    [self setHidden:!isShow];
    if (isShow) {
        _pageIndex = [_pdfViewCtrl getCurrentPage];
        self.zoomSlider.value = [_pdfViewCtrl getZoom];
    
        [self resetPageButtonState];
        
        [self resetTitleItem];
        [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(delayShowPanZoomThumbnailView) object:nil];
        [self performSelector:@selector(delayShowPanZoomThumbnailView) withObject:nil afterDelay:0.1];
    }
    [_extensionsManager setStatusBarHidden:_extensionsManager.currentIsFullScreen && _topToolBar.hidden];
    [_extensionsManager.pdfViewCtrl.fs_viewController setNeedsStatusBarAppearanceUpdate];
    [_extensionsManager.settingBar.viewSettingManager resetPdfViewCtrlSafeArea];
}

- (void)delayShowPanZoomThumbnailView
{
    FSPDFPage* currentPage = [_pdfViewCtrl.currentDoc getPage:_pageIndex];
    if (!currentPage || [currentPage isEmpty]) {
        self.imageView.image = nil;
        return;
    }
    [self drawSubView:currentPage needDrawThumbnail:YES];
}

- (void)showToolBar:(BOOL)isShow animation:(BOOL)animation{
    if (self.bottomToolBar.hidden != isShow) {
        return;
    }

    if (!isShow) {
        if (![self checkIsIphoneDevice]) {
            [_extensionsManager setHiddenTopToolbar:YES];
        }
        self.topToolBar.hidden = YES;
        self.bottomToolBar.hidden  = YES;
    } else {
        if (![self checkIsIphoneDevice]) {
            self.topToolBar.hidden = YES;
            [_extensionsManager setHiddenTopToolbar:NO];
        }
        else
        {
            [_extensionsManager setHiddenTopToolbar:YES];
            [_extensionsManager setHiddenBottomToolbar:YES];
            self.topToolBar.hidden = NO;
        }
        self.bottomToolBar.hidden  = NO;
        [self.bottomToolBar.superview bringSubviewToFront:self.bottomToolBar];
    }
    if (animation)
    {
        [Utility addAnimation:self.topToolBar.layer type:self.topToolBar.hidden?kCATransitionReveal:kCATransitionMoveIn subType:self.topToolBar.hidden?kCATransitionFromTop: kCATransitionFromBottom timeFunction:kCAMediaTimingFunctionEaseInEaseOut duration:0.3];
        
        [Utility addAnimation:self.bottomToolBar.layer type:self.bottomToolBar.hidden?kCATransitionReveal:kCATransitionMoveIn subType:self.bottomToolBar.hidden?kCATransitionFromBottom: kCATransitionFromTop timeFunction:kCAMediaTimingFunctionEaseInEaseOut duration:0.3];
    }
    [(PageNavigationModule *)[_extensionsManager getModuleByName:@"PageNavigation"] showPageNum: isShow];
    
    [(PageNavigationModule *)[_extensionsManager getModuleByName:@"PageNavigation"] layoutPdfViewContentInsets:NO];
}

- (void)dealloc{
    self.needPauseNow = YES;
}

- (void)removeFromSuperview{
    self.needPauseNow = YES;
    [super removeFromSuperview];
}

- (void)correctPageIndexInCoverMode{
    PDF_LAYOUT_MODE layoutMode = [_pdfViewCtrl getPageLayoutMode];
    if (!_pdfViewCtrl.isContinuous) {
        if ( layoutMode == PDF_LAYOUT_MODE_TWO_RIGHT )
        {
            if (_pageIndex == 0) {
                _pageIndex = 0;
            }else{
                _pageIndex = _pageIndex -1;
            }
        }
    }
}

- (BOOL)correctPageIndexInFacingMode
{
    PDF_LAYOUT_MODE layoutMode = [_pdfViewCtrl getPageLayoutMode];
    if ( layoutMode == PDF_LAYOUT_MODE_TWO ||
        layoutMode == PDF_LAYOUT_MODE_TWO_RIGHT )
    {
        BOOL isLeftPage = NO;
        
        if (layoutMode == PDF_LAYOUT_MODE_TWO) {
            isLeftPage = _pageIndex % 2 == 0;
        } else {
            if (_pageIndex == 0) {
                _pageIndex = 0 ;
                return YES;
            }
            isLeftPage = _pageIndex % 2 == 1;
        }
        
        if ( isLeftPage ) {
            CGRect rect = [_pdfViewCtrl convertDisplayViewRectToPageViewRect:_pdfViewCtrl.bounds pageIndex:(_pageIndex + 1)];
            if (rect.origin.x > 0) {
                _pageIndex += 1;
                return YES;
            }
        } else {
            CGRect rect = [_pdfViewCtrl convertDisplayViewRectToPageViewRect:_pdfViewCtrl.bounds pageIndex:_pageIndex];
            if (rect.origin.x < 0) {
                _pageIndex -= 1;
                return YES;
            }
        }
    }
    return NO;
}

- (void)drawSubView:(FSPDFPage*)currentPage needDrawThumbnail:(BOOL)needDrawThumbnail {
    if (!_showPanZoom) {
        return;
    }
    float pageWidth = [currentPage getWidth];
    float pageHeight = [currentPage getHeight];
    
    FSRotation rotation = (FSRotation)[_pdfViewCtrl getViewRotation];
    if (rotation == FSRotation90 || rotation == FSRotation270) {
        float tmp = pageWidth;
        pageWidth =  pageHeight;
        pageHeight = tmp;
    }
    
    _aspectRatio = pageWidth / pageHeight;
    //during rotation, self.frame will be set to CGRectZero.
    CGFloat thumbnailWidth = 0;
    if ( CGRectEqualToRect(self.frame, CGRectZero) ) {
        thumbnailWidth = MIN(_tempSelfFrame.size.width, _tempSelfFrame.size.height * _aspectRatio);
    } else {
        thumbnailWidth = MIN(self.frame.size.width, self.frame.size.height * _aspectRatio);
    }
    CGFloat thumbnailHeight = thumbnailWidth / _aspectRatio;
    CGSize thumbnailSize = CGSizeMake(thumbnailWidth, thumbnailHeight);
    
    if (needDrawThumbnail) {
        self.needPauseNow = NO;
        self.imageView.image = nil;
        dispatch_async(dispatch_get_global_queue(0, 0), ^{
            [Utility drawPage:currentPage targetSize:thumbnailSize shouldDrawAnnotation:YES progressive:^(UIImage * _Nonnull image, BOOL finish) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    [self.imageView setImage:image];
                });
            } needPause:^BOOL{
                return  self.needPauseNow;
            } rotation:rotation enableForPrint:NO pdfViewCtrl:self->_pdfViewCtrl];
        });
    }
    
    //calculate image view frame
    CGFloat originX, originY, width, height;
    if (_aspectRatio > 1) {
        width = self.bounds.size.width;
        height = self.bounds.size.height / _aspectRatio;
        originX = self.bounds.origin.x;
        originY = (self.bounds.size.height - height) / 2.0;
        
    } else {
        height = self.bounds.size.height;
        width = height * _aspectRatio;
        originY = self.bounds.origin.y;
        originX = (self.bounds.size.width - width) / 2.0;
    }
    self.imageView.frame = CGRectMake(originX, originY, width, height);
    _imageViewFrame = self.imageView.frame;
    
    [_borderView receiveImageViewRect:_imageViewFrame];
    _borderView.frame = [self calculatorCurrentBorderViewRect];
}

#pragma IStateChangeListener
-(void)onStateChanged:(FSUIManagerState)state lastState:(FSUIManagerState)lastState
{
    if (state != FSUIManagerStatePanZoom && state != FSUIManagerStateAnnotTool) {
        if (_isSelfState) {
            _isSelfState = NO;
            _showPanZoom = NO;
            [self setHidden:YES];
            [self.bottomToolBar setHidden:YES];
            [self.topToolBar setHidden:YES];
            [_extensionsManager.settingBar.viewSettingManager setItemSelectedWithType:PANZOOM selected:NO];
            [self removeFromSuperview];
        }
    }
    else if (state == FSUIManagerStatePanZoom) {
    
        [_extensionsManager.settingBar.viewSettingManager setItemSelectedWithType:PANZOOM selected:YES];
        _isSelfState = YES;
        
        if (!self.superview) {
            [self showPanZoom:YES animation:YES];
            [self reLayoutBottomToolbar];
            [_extensionsManager setHiddenPanel:YES];
            [_extensionsManager.containerView addSubview:self];
            if ([self checkIsIphoneDevice]) {
                [_bottomToolBar mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.left.right.bottom.mas_equalTo(self.bottomToolBar.superview);
                }];
            }
            else
            {
                [_bottomToolBar mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.bottom.equalTo(self.bottomToolBar.superview.fs_mas_bottom).offset(-40);
                    make.centerX.equalTo(self.bottomToolBar.superview);
                    make.width.mas_equalTo(435);
                }];
            }
            [self mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.centerX.equalTo(self.superview);
                make.bottom.equalTo(self.superview.fs_mas_bottom).offset([self checkIsIphoneDevice]?-136: -97);
                make.size.mas_equalTo(CGSizeMake(180, 182));
            }];
        }
    }
}

#pragma pageEventListener methods

- (void)onPageChanged:(int)oldIndex currentIndex:(int)currentIndex
{
    if (!self.superview) {
        return;
    }
    _pageIndex = currentIndex;
    [self correctPageIndexInCoverMode];
    [self correctPageIndexInFacingMode];
    
    PDF_LAYOUT_MODE layoutMode = [_pdfViewCtrl getPageLayoutMode];
    if (_pdfViewCtrl.isContinuous) {
        if ( layoutMode == PDF_LAYOUT_MODE_TWO_LEFT || layoutMode == PDF_LAYOUT_MODE_TWO_RIGHT || layoutMode == PDF_LAYOUT_MODE_TWO_MIDDLE)
        {
            _pageIndex = currentIndex == 1 ? 2 : _pageIndex;
        }
    }
    
    [self resetPageButtonState];
    
    if (_pdfViewCtrl.currentDoc != nil) {
        [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(delayShowPanZoomThumbnailView) object:nil];
        [self performSelector:@selector(delayShowPanZoomThumbnailView) withObject:nil afterDelay:0.1];
    }
}

- (void)onLayoutFinished
{
    if (_isSelfState)
    {
        _borderView.frame = [self calculatorCurrentBorderViewRect];
        
        [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(delayShowPanZoomThumbnailView) object:nil];
        [self performSelector:@selector(delayShowPanZoomThumbnailView) withObject:nil afterDelay:0.1];
    }
}

#pragma rotate

-(void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration
{
    
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation
{
    if (_isSelfState) {
        _borderView.frame = [self calculatorCurrentBorderViewRect];
        FSPDFPage* currentPage = [_pdfViewCtrl.currentDoc getPage:_pageIndex];
        if ([currentPage isEmpty]) return;
        [self drawSubView:currentPage needDrawThumbnail:NO];
        [self resetTitleItem];
    }
}

#pragma annotEventListener methods

-(void)onAnnotDeleted:(FSPDFPage *)page annot:(FSAnnot *)annot
{
    [self drawSubView:page needDrawThumbnail:YES];
}

-(void)onAnnotFlattened:(FSPDFPage *)page annot:(FSAnnot *)annot {
    [self drawSubView:page needDrawThumbnail:YES];
}

-(void)onAnnotAdded:(FSPDFPage *)page annot:(FSAnnot *)annot
{
    [self drawSubView:page needDrawThumbnail:YES];
}

-(void)onAnnotModified:(FSPDFPage *)page annot:(FSAnnot *)annot
{
    [self drawSubView:page needDrawThumbnail:YES];
}

#pragma scrollViewEventListener methods

-(void)onScrollViewDidEndDragging:(UIScrollView *)scrollView willDecelerate:(BOOL)decelerate
{
    if (decelerate) {
        return;
    }
    if ([self correctPageIndexInFacingMode]) {
        FSPDFPage* currentPage = [_pdfViewCtrl.currentDoc getPage:_pageIndex];
        if ([currentPage isEmpty]) return;
        [self drawSubView:currentPage needDrawThumbnail:NO];
    } else {
        _borderView.frame = [self calculatorCurrentBorderViewRect];
    }
    
    self.zoomSlider.value = [_pdfViewCtrl getZoom];
}

-(void)onScrollViewDidEndDecelerating:(UIScrollView *)scrollView
{
    if ([self correctPageIndexInFacingMode]) {
        FSPDFPage* currentPage = [_pdfViewCtrl.currentDoc getPage:_pageIndex];
        if ([currentPage isEmpty]) return;
        [self drawSubView:currentPage needDrawThumbnail:NO];
    } else {
        _borderView.frame = [self calculatorCurrentBorderViewRect];
    }
    
    self.zoomSlider.value = [_pdfViewCtrl getZoom];
}

- (void)onScrollViewDidEndZooming:(UIScrollView *)scrollView
{
    if ([self correctPageIndexInFacingMode]) {
        FSPDFPage* currentPage = [_pdfViewCtrl.currentDoc getPage:_pageIndex];
        if ([currentPage isEmpty]) return;
        [self drawSubView:currentPage needDrawThumbnail:NO];
    } else {
        _borderView.frame = [self calculatorCurrentBorderViewRect];
    }
    
    self.zoomSlider.value = [_pdfViewCtrl getZoom];
}

- (CGRect)calculatorCurrentBorderViewRect
{
    if (_pageIndex >= _pdfViewCtrl.currentDoc.getPageCount) {
        return CGRectMake(0, 0, 0, 0);
    }
    FSPDFPage* currentPage = [_pdfViewCtrl.currentDoc getPage:_pageIndex];
    if ([currentPage isEmpty]) {
        return CGRectMake(0, 0, 0, 0);
    }
    float fullPageWidth = [currentPage getWidth];
    float fullPageHeight = [currentPage getHeight];

    FSRotation viewRotation = (FSRotation)[_pdfViewCtrl getViewRotation];
    if (viewRotation == FSRotation90 || viewRotation == FSRotation270) {
        float tmp = fullPageWidth;
        fullPageWidth =  fullPageHeight;
        fullPageHeight = tmp;
    }
    
    CGRect rect = [_pdfViewCtrl convertDisplayViewRectToPageViewRect:_pdfViewCtrl.bounds pageIndex:_pageIndex];
    if (rect.origin.x < 0) {
        rect.size.width = rect.size.width + rect.origin.x;
        rect.origin.x = 0;
    }
    if (rect.origin.y < 0) {
        rect.size.height = rect.size.height + rect.origin.y;
        rect.origin.y = 0;
    }
    FSRectF* pdfRect = [_pdfViewCtrl convertPageViewRectToPdfRect:rect pageIndex:_pageIndex];
    //rate
    CGFloat xRate = pdfRect.left / fullPageWidth;
    CGFloat widthRate = (pdfRect.right - pdfRect.left) / fullPageWidth;
    CGFloat yRate = 1.0 - pdfRect.top / fullPageHeight;
    CGFloat heightRate = (pdfRect.top - pdfRect.bottom) / fullPageHeight;
    
    //adjust rect rate for page rotation
    FSPDFPage* page = [_pdfViewCtrl.currentDoc getPage:_pageIndex];
    if ([page isEmpty]) return CGRectMake(0, 0, 0, 0);
    FSRotation rotation = [page getRotation];
    rotation = (FSRotation) ((rotation + viewRotation) % 4);
    if (rotation == FSRotation90) {
        xRate = pdfRect.bottom / fullPageWidth;
        widthRate = (pdfRect.top - pdfRect.bottom) / fullPageWidth;
        yRate = pdfRect.left / fullPageHeight;
        heightRate = (pdfRect.right - pdfRect.left) / fullPageHeight;
    } else if (rotation == FSRotation270) {
        xRate = 1.0 - pdfRect.top / fullPageWidth;
        widthRate = (pdfRect.top - pdfRect.bottom) / fullPageWidth;
        yRate = 1.0 - pdfRect.right / fullPageHeight;
        heightRate = (pdfRect.right - pdfRect.left) / fullPageHeight;
    } else if (rotation == FSRotation180) {
        xRate = 1.0 - pdfRect.right / fullPageWidth;
        widthRate = (pdfRect.right - pdfRect.left) / fullPageWidth;
        yRate = pdfRect.bottom / fullPageHeight;
        heightRate = (pdfRect.top - pdfRect.bottom) / fullPageHeight;
    }
    
    //new frame in imageView
    _imageViewFrame = self.imageView.frame;

    CGFloat newWidth = widthRate * _imageViewFrame.size.width;
    CGFloat newX = _imageViewFrame.origin.x + _imageViewFrame.size.width * xRate;
    CGFloat newHeight = heightRate * _imageViewFrame.size.height;
    CGFloat newY = _imageViewFrame.origin.y + _imageViewFrame.size.height * yRate;
    
    if (newY + newHeight > _imageViewFrame.origin.y + _imageViewFrame.size.height) {
        newHeight = _imageViewFrame.origin.y + _imageViewFrame.size.height - newY;
    }
    if (newX + newWidth > _imageViewFrame.origin.x + _imageViewFrame.size.width) {
        newWidth = _imageViewFrame.origin.x + _imageViewFrame.size.width - newX;
    }
    if (isnan(newX))  newX = 0;
    if (isnan(newY))  newY = 0;
    if (isnan(newWidth))  newWidth = 0;
    if (isnan(newHeight))  newHeight = 0;
    
    CGRect borderViewRect = CGRectMake(newX, newY, newWidth, newHeight);
    if ( !CGRectIntersectsRect(borderViewRect, _imageViewFrame) )
    {
        return CGRectZero;
    }
    
    return borderViewRect;
}

#pragma mark -- IDocEventListener
- (void)onDocWillClose:(FSPDFDoc *)document{
    if (_isSelfState) {
         [_extensionsManager changeState:FSUIManagerStateNormal];
    }
}

#pragma mark -- IThumbnailEventListener
- (void)onThumbnailChanged:(BOOL)isHidden
{
    if ([_extensionsManager getState] == FSUIManagerStatePanZoom) {
        _bottomToolBar.hidden = !isHidden;
        self.hidden = !isHidden;
        if (isHidden) {
            [_extensionsManager setFullScreen:NO];
        }
    }
}

-(void)reLayoutBottomToolbar
{
    if ([self checkIsIphoneDevice]) {
        [_bottomToolBar mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.right.bottom.mas_equalTo(_bottomToolBar.superview);
        }];
        
        [_nextPageButton mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.right.equalTo(_bottomToolBar.fs_mas_right).offset(-ITEM_MEDIUM_SPACE_NORMAL);
            make.size.mas_equalTo(_zoomOutButton);
            make.centerY.mas_equalTo(_zoomOutButton);
        }];
        
        if (_exitItem && _exitItem.superview) {
            [_exitItem removeFromSuperview];
        }
        if (_pan && [_bottomToolBar.gestureRecognizers containsObject:_pan]) {
            [_bottomToolBar removeGestureRecognizer:_pan];
        }
        [self.bottomToolBar fs_setShadowColor:nil];
    }else {
        [self.bottomToolBar fs_setShadowColor:self.fs_dividerColor_P3];
        _bottomToolBar.layer.cornerRadius = 6;
        [_bottomToolBar mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.bottom.equalTo(_bottomToolBar.superview.fs_mas_bottom).offset(-40);
            make.centerX.equalTo(_bottomToolBar.superview);
            make.width.mas_equalTo(435);
        }];
        
        if (!_pan) {
            _pan = [[UIPanGestureRecognizer alloc] initWithTarget:_extensionsManager.settingBar.viewSettingManager action:@selector(moveBottombarWithRecognizer:)];
        }
        if (![_bottomToolBar.gestureRecognizers containsObject:_pan]) {
            [_bottomToolBar addGestureRecognizer:_pan];
        }
        
        if (!_exitItem) {
            @fs_weakify(self)
            _exitItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"setting_exit")];
            _exitItem.accessibilityLabel = FSLocalizedForKey(@"kExitVoiceOver");
            _exitItem.onTapClick = ^(FSItem * _Nonnull _item) {
                @fs_strongify(self)
                [self->_extensionsManager changeState:FSUIManagerStateNormal];
            };
        }
        if (!_exitItem.superview) {
            [self.panZoomBottomContentView addSubview:_exitItem];
            [_exitItem mas_makeConstraints:^(MASConstraintMaker *make) {
                make.right.mas_equalTo(-16);
                make.centerY.mas_equalTo(_exitItem.superview);
                make.width.height.mas_equalTo(40);
            }];
        }
        
        [_nextPageButton mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.right.equalTo(_exitItem.mas_left).offset(-ITEM_MEDIUM_SPACE_NORMAL);
            make.size.mas_equalTo(_zoomOutButton);
            make.centerY.mas_equalTo(_zoomOutButton);
        }];
        
        
    }
}

- (void)layoutSubviews{
    [super layoutSubviews];
    [self resetTitleItem];
}

#pragma mark-- IWindowFrameChangedListener
- (void)onWindowFrameChanged:(CGRect)newFrame horizontalSizeClassChanged:(BOOL)changed
{
    if (DEVICE_iPAD && changed) {
        if ([_extensionsManager getState] == FSUIManagerStatePanZoom) {
            [self reLayoutBottomToolbar];
            self.bottomToolBar.hidden = YES;
            
            [_extensionsManager setCurrentIsFullScreen:NO];
            [self showPanZoom:YES animation:NO];
        }else if ([_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]){
            [self showPanZoom:NO animation:NO];
        }
    }
}

- (BOOL)checkIsIphoneDevice
{
    return DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact];
}
@end
