/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "AnnotationPanel.h"
#import "FSAnnotationListViewController.h"
#import "AnnotationStruct.h"
#import "PanelController+private.h"
#import "PanelHost.h"
#import "FSPermissionUtilty.h"

#define ENLARGE_EDGE 3

@interface AnnotationPanel ()

@property (nonatomic, strong) TabItem *tabItem;

@end

@implementation AnnotationPanel

- (void)initialize{
    
    self.annotationCtrl = [[FSAnnotationListViewController alloc] initWithExtensionsManager:self.extensionsManager  module:self];
    self.reflowAnnotationCtrl = [[FSAnnotationListViewController alloc] initWithExtensionsManager:self.extensionsManager  module:self];
    self.reflowAnnotationCtrl.isReflow = YES;
//    self.contentViewCtr = self.annotationCtrl;
    [self.panelController registerPanelChangedListener:self.annotationCtrl];
    [self.panelController registerPanelChangedListener:self.reflowAnnotationCtrl];
    
    [self.contentView addSubview:self.annotationCtrl.view];
    [self.annotationCtrl.view mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(0);
        make.right.equalTo(self.contentView.mas_right).offset(0);
        make.top.equalTo(self.contentView.mas_top).offset(0);
        make.bottom.equalTo(self.contentView.mas_bottom).offset(0);
    }];

    self.title =  FSLocalizedForKey(@"kComments");
}

- (void)load {
    [self.pdfViewCtrl registerDocEventListener:self];
    [self.pdfViewCtrl registerPageEventListener:self.annotationCtrl];
    [self.pdfViewCtrl registerPageEventListener:self.reflowAnnotationCtrl];
    [self.panelController.panel insertSpec:self atIndex:2];
}

- (void)unload {
    [self.pdfViewCtrl unregisterDocEventListener:self];
    [self.pdfViewCtrl unregisterPageEventListener:self.annotationCtrl];
    [self.pdfViewCtrl unregisterPageEventListener:self.reflowAnnotationCtrl];
    [self.panelController.panel removeSpec:self];
    self.panelController.panel.currentSpec = nil;
}

- (void)clickedRightItem{
    if([self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_REFLOW){
        self.reflowAnnotationCtrl.editing = !self.reflowAnnotationCtrl.editing;
    }else{
        self.annotationCtrl.editing = !self.annotationCtrl.editing;
    }
}

- (void)onDocWillOpen {
}

- (void)onDocOpened:(FSPDFDoc *)document error:(int)error {
    if (error != FSErrSuccess) {
        return;
    }

    dispatch_async(dispatch_get_main_queue(), ^{
        [self->_annotationCtrl loadData];
        //update clear button state
        self.rightItem.enabled = [FSPermissionUtilty canAddAnnot:self.extensionsManager];
    });
}

- (void)onDocClosed:(FSPDFDoc *)document error:(int)error {
    [self->_annotationCtrl clearData];
    [self->_reflowAnnotationCtrl clearData];
    dispatch_main_async_safe(^{
        if (self->_annotationCtrl.editing) {
            self->_annotationCtrl.editing = NO;
        }
        if (self->_reflowAnnotationCtrl.editing) {
            self->_reflowAnnotationCtrl.editing = NO;
        }
    });
}

- (void)onDocWillSave:(FSPDFDoc *)document {
}

- (int)getType {
    return FSPanelTypeAnnotation;
}

- (TabItem *)getTabItem{
    if (!_tabItem) {
        _tabItem = [TabItem new];
        _tabItem.image = FSImageNamedAndNormalColor(@"panel_comment_gray");
        _tabItem.selectImage = [_tabItem.image fs_imageWithTintColor:self.fs_primaryColor];
    }
    return _tabItem;
}

- (void)onActivated {
    [super onActivated];
    
    [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingPanelComment];
    
    BOOL isHidden = [self.panelController isPanelHidden:FSPanelTypeAnnotation];
    if(!isHidden){
        if([self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_REFLOW){
            if(![self.reflowAnnotationCtrl.view isDescendantOfView:self.contentView]){
                [self.contentView addSubview:self.reflowAnnotationCtrl.view];
                [self.reflowAnnotationCtrl.view mas_makeConstraints:^(MASConstraintMaker *make) {
                    make.left.equalTo(self.contentView.mas_left).offset(0);
                    make.right.equalTo(self.contentView.mas_right).offset(0);
                    make.top.equalTo(self.contentView.mas_top).offset(0);
                    make.bottom.equalTo(self.contentView.mas_bottom).offset(0);
                }];
            }
            [self.annotationCtrl.view setHidden:YES];
            [self.reflowAnnotationCtrl.view setHidden:NO];
            [self.reflowAnnotationCtrl loadData];

        }else{
            [self.reflowAnnotationCtrl.view setHidden:YES];
            [self.annotationCtrl.view setHidden:NO];
           
        }
        
    }
}

- (void)onDeactivated {
    [super onDeactivated];
    self.annotationCtrl.editing = NO;
    self.reflowAnnotationCtrl.editing = NO;
}

- (void)quitSearch{
    if([self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_REFLOW){
        [self.annotationCtrl clickedSearchCancelItem];
    }else{
        [self.annotationCtrl clickedSearchCancelItem];
    }
    
}

@end
