/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "AttachmentPanel.h"
#import "PanelController+private.h"
#import "PanelHost.h"
#import "FSPermissionUtilty.h"

@interface AttachmentPanel ()
@property (nonatomic, strong) TabItem *tabItem;
@end

@implementation AttachmentPanel

- (void)initialize{
    
    self.attachmentCtr = [[AttachmentViewController alloc] initWithExtensionsManager:self.extensionsManager module:self];
    self.contentViewCtr = self.attachmentCtr;
    [self.contentView addSubview:self.attachmentCtr .view];
    [self.attachmentCtr.view mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(0);
        make.right.equalTo(self.contentView.mas_right).offset(0);
        make.top.equalTo(self.contentView.mas_top).offset(0);
        make.bottom.equalTo(self.contentView.mas_bottom).offset(0);
    }];
    
    self.title =  FSLocalizedForKey(@"kAttachments");
}

- (void)load {
    [self.pdfViewCtrl registerDocEventListener:self];
    [self.panelController.panel insertSpec:self atIndex:3];
}

- (void)unload {
    [self.pdfViewCtrl unregisterDocEventListener:self];
    [self.panelController.panel removeSpec:self];
}

- (int)getType {
    return FSPanelTypeAttachment;
}


- (TabItem *)getTabItem{
    if (!_tabItem) {
        _tabItem = [TabItem new];
        _tabItem.image = FSImageNamedAndNormalColor(@"panel_attachment_gray");
        _tabItem.selectImage = [_tabItem.image fs_imageWithTintColor:self.fs_primaryColor];
    }
    return _tabItem;
}

- (void)onActivated {
    [super onActivated];
    
    [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingPanelAttachment];
}

- (void)onDeactivated {
    [super onDeactivated];
    [self.attachmentCtr hideCellEditView];
    self.attachmentCtr.editing = NO;
}

- (void)clickedRightItem {
    self.attachmentCtr.editing = !self.attachmentCtr.editing;
}

- (void)cancelAttachment {
    self.panelController.isHidden = YES;
}

- (void)onDocOpened:(FSPDFDoc *)document error:(int)error {
    if (error != FSErrSuccess) {
        return;
    }
    BOOL canModify = [FSPermissionUtilty canModifyContents:self.extensionsManager];
    dispatch_async(dispatch_get_main_queue(), ^{
        self.rightItem.enabled = canModify;
        self.rightItem.hidden = ([self.extensionsManager.permissionProvider checkPermission:FSFunctionAttachmentsPanelEditItem] == FSPermissionStateHide);
        self.attachmentCtr.addItem.enabled = canModify;
        self.attachmentCtr.addItem.hidden = ([self.extensionsManager.permissionProvider checkPermission:FSFunctionAttachmentsPanelAddItem] == FSPermissionStateHide);
    });
}

@end
