/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSBasePanel.h"

@interface FSBasePanel ()<IPanelSpec>
@property (nonatomic, weak) FSItem *titleItem;
@property (nonatomic, strong) FSToolbar *toolbar;
@end

static NSString *const kNavigationKeyPanel = @"kNavigationKeyPanel";
@implementation FSBasePanel
@synthesize navController = _navController;
- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager panelController:(FSPanelController *)panelController{
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _panelController = panelController;
        
        self.toolbar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleDefault];
        self.toolbar.backgroundColor = [UIColor fs_clearColor];
        _contentView = [UIView new];
        
        @weakify(self)
        
        FSItem *cancelItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kClose")];
        cancelItem.font = self.fs_standardGeneralFont_GF2;
        cancelItem.accessibilityLabel = FSLocalizedForKey(@"kClose");
        cancelItem.onTapClick = ^(FSItem * _Nonnull item) {
            @strongify(self)
            [self clickedLeftItem];
        };
        [BindThemeKeySel(cancelItem, setTitleColor:forState:)
         fs_updateThemeCompleted:^(FSItem * _Nonnull target, SEL  _Nonnull sel) {
            [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
        }];
        [self.toolbar addItem:cancelItem atPosition:FSItemPositionLeft];
        _leftItem = cancelItem;
        if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            _leftItem.hidden = NO;
        }else{
            _leftItem.hidden = YES;
        }

        
        FSItem *titleItem = [FSItem createHeadlineItemWithTitle:nil];
        titleItem.accessibilityTraits = UIAccessibilityTraitHeader;
        titleItem.customAccessibilityTraits = UIAccessibilityTraitHeader;
        [titleItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
        [self.toolbar addItem:titleItem atPosition:FSItemPositionCenter];
        self.titleItem = titleItem;
        
        FSItem *editItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kEdit")];
        editItem.font = self.fs_standardGeneralFont_GF2;
        [editItem setTitle:FSLocalizedForKey(@"kDone") forState:UIControlStateSelected];
        editItem.onTapClick = ^(FSItem * _Nonnull item) {
            @strongify(self)
            item.selected = !item.selected;
            [self clickedRightItem];
        };
        [BindThemeKeySel(editItem, setTitleColor:forState:)
         fs_updateThemeCompleted:^(FSItem * _Nonnull target, SEL  _Nonnull sel) {
            [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
            [target setTitleColor:target.fs_primaryColor forState:UIControlStateSelected];
        }];
        [self.toolbar addItem:editItem atPosition:FSItemPositionRight];
        _rightItem = editItem;
        
        [self initialize];
    }
    return self;
}

- (void)initialize {}

- (void)setTitle:(NSString *)title{
    _title = title;
    [_titleItem setTitle:title forState:UIControlStateNormal];
    _title.accessibilityLabel = title;
}

- (void)clickedLeftItem{
    _panelController.isHidden = YES;
}
- (void)clickedRightItem{}

- (void)load {}

- (void)unload {}

- (int)getType {
    return 0;
}

- (UIView *)getTopToolbar {
    return self.toolbar;
}

- (UIView *)getContentView {
    return self.contentView;
}

- (TabItem *)getTabItem{
    return nil;
}

- (void)onActivated {
    if (self.navController) {
        [self.extensionsManager.viewCtrlManager.navigationControllerStack pushNavigationController:self.navController];
    }
}

- (void)onDeactivated {
    if (self.navController) {
        FSNavigationController *topNavigationController = (FSNavigationController *)[self.extensionsManager.viewCtrlManager.navigationControllerStack topNavigationController];
        if ([topNavigationController isKindOfClass:[FSNavigationController class]]) {
            if ([topNavigationController.navigationKey isEqualToString:kNavigationKeyPanel]) {
                [self.extensionsManager.viewCtrlManager.navigationControllerStack popNavigationController];
            }
        }
    }
}

- (void)resetToolBar{
    if (DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
        _leftItem.hidden = NO;
    }else{
        _leftItem.hidden = YES;
    }
}

- (void)setNavController:(FSNavigationController *)navController{
    navController.navigationKey = kNavigationKeyPanel;
    _navController = navController;
}

- (void)quitSearch{};

@end
