/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSDigitalSignatureViewController.h"
#import "DigitalSignaturePanel.h"
#import "FSPermissionUtilty.h"

@interface FSDigitalSignatureViewController ()
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
@property (nonatomic, weak) FSPDFViewCtrl *pdfViewCtrl;
@property (nonatomic, weak) FSPanelController *panelController;

@property (nonatomic, weak) DigitalSignaturePanel *signaturePanel;

@end

@implementation FSDigitalSignatureViewController

- (void)initialize{
    [super initialize];
    self.shouldShowNavigationBar = NO;
    self.defaultEstimatedCellStyle = FSEstimatedTableViewCellStyleSubject;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.cellSelectedColor = self.fs_dividerColor_P1;

}

- (id)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager module:(DigitalSignaturePanel *)signaturePanel{
    self = [super initWithServices:extensionsManager.viewCtrlManager];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _panelController = signaturePanel.panelController;
        _signaturePanel = signaturePanel;

    }
    return self;
}

- (id)dataForAtIndexPath:(NSIndexPath *)indexPath{
    NSArray *keys = [self.digitalsigDict allKeys];
    int pageIndex = [keys containsObject:@(-1)] ? -1 : (int)indexPath.section;
    
    NSArray *rowsData = [self.digitalsigDict objectForKey:@(pageIndex)];
    return rowsData[indexPath.row];
}

- (NSString *)titleForEmptyDataSet{
    return FSLocalizedForKey(@"kNoDigitalSignatures");
}

- (UIImage *)imageForEmptyDataSet:(UITableView *)tableView {
    return FSImageNamedAndPrimaryColor(@"panel_signature_no_data");
}

- (BOOL)emptyDataSetShouldDisplay:(UITableView *)tableView{
    BOOL emptyDataSetShouldDisplay = [super emptyDataSetShouldDisplay:tableView];
    _signaturePanel.rightItem.enabled = !emptyDataSetShouldDisplay;
    return emptyDataSetShouldDisplay;
}

#pragma mark - UITableViewDelegate digitalsigDict & delegate

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView{
    return [self.digitalsigDict count];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    NSArray *keys = [self.digitalsigDict allKeys];
    int pageIndex = [keys containsObject:@(-1)] ? -1 : (int)section;
    return [[self.digitalsigDict objectForKey:@(pageIndex)] count];
}

- (void)configureCell:(FSSubjectEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(NSMutableDictionary *)cellData{
    if (!cell.leftView) {
        UIImageView *imageView = [UIImageView new];
        cell.leftView = imageView;
    }
    
    if (!cell.rightView) {
        FSItem *rightView = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"common_more_gray")];
        rightView.accessibilityLabel = FSLocalizedForKey(@"kMore");
        cell.rightView = rightView;
    }
    
    [cell.rightView setHidden:NO];
    FSSignature *signature = [self.pdfViewCtrl.currentDoc getSignature:(int)indexPath.row];
    FSControl *control = [signature getControl:0];
    if (![control isEmpty]) {
        FSWidget *widget = [control getWidget];
        if ([widget isEmpty]) {
            [cell.rightView setHidden:YES];
        }
    }
    
    if (![[cellData objectForKey:@"isSigned"] boolValue]){
        ((FSItem *)cell.rightView).enabled = [FSPermissionUtilty canFillFormInDocument:_extensionsManager];
        [cell.rightView setHidden:![FSPermissionUtilty canFillFormInDocument:_extensionsManager]];
    }else{
        ((FSItem *)cell.rightView).enabled = YES;
    }
    
    ((FSItem *)cell.rightView).onTapClick = ^(FSItem * _Nonnull item) {
        NSArray *tmp = nil;
        BOOL isSigned = NO;
        if ([[cellData objectForKey:@"isSigned"] boolValue]) {
            isSigned = YES;
            @weakify(self)
            FSPopoverItem *item1 = [[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kDigitalSignatureBtnView") image:FSImageNamedAndNormalColor(@"panel_check_gray") action:^(FSPopoverItem * _Nonnull popoverItem, int index) {
                @strongify(self)
                [self.pdfViewCtrl lockRefresh];
                [self.signaturePanel viewSignature:[[cellData objectForKey:@"pageIndex"] intValue] signatureIndex:(int)indexPath.row uuid:[cellData objectForKey:@"uuid"]];
                [self.pdfViewCtrl unlockRefresh];
            }];
            
            FSPopoverItem *item2 = [[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kDigitalSignatureBtnVerify") image:FSImageNamedAndNormalColor(@"sign_verification_gray") action:^(FSPopoverItem * _Nonnull popoverItem, int index) {
                @strongify(self)
                [self.signaturePanel verifySignature:[[cellData objectForKey:@"pageIndex"] intValue] signatureIndex:(int)indexPath.row  uuid:[cellData objectForKey:@"uuid"]];

            }];
            tmp = @[item1,item2];
        }else{
            @weakify(self)
            FSPopoverItem *item1 = [[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kSignAction") image: FSImageNamedAndNormalColor(@"panel_signature_list_gray") action:^(FSPopoverItem * _Nonnull popoverItem, int index) {
                @strongify(self)
                [self.signaturePanel signSignature:[[cellData objectForKey:@"pageIndex"] intValue] signatureIndex:(int)indexPath.row  uuid:[cellData objectForKey:@"uuid"]];
            }];
            tmp = @[item1];
        }
        UIColor *oriColor = item.superview.backgroundColor;
        if (DEVICE_iPHONE) item.superview.backgroundColor = self.fs_backgroundColor_B2;
        [self.extensionsManager.viewCtrlManager showMoreMenuWithItems:tmp dismissCompletion:DEVICE_iPHONE ? ^{
            item.superview.backgroundColor = oriColor;
        } : nil sourceView:!DEVICE_iPHONE ? ^UIView * _Nonnull(UIPopoverPresentationController * _Nonnull popover) {
            popover.permittedArrowDirections = UIPopoverArrowDirectionRight;
            return item;
        } : nil];
    };
    
    NSString *fieldTitle = @"";
    NSString *fieldDetail = @"";
    
    if ([[cellData objectForKey:@"isSigned"] boolValue]) {
        NSString *version = @"";
        NSString *signer = [NSString stringWithFormat:FSLocalizedForKey(@"kDigitalSignatureSignedTitleSigner"), [cellData objectForKey:@"signer"]];
        fieldTitle = [NSString stringWithFormat:@"%@%@",version,signer];
        
        FSDateTime *dateTime = [cellData objectForKey:@"signdate"];
        NSDate *signdate = [Utility convertFSDateTime2NSDate:dateTime];
        NSString *signdateString = [Utility displayDateInMDYHM:signdate];
        
        fieldDetail = signdateString;
        
        ((UIImageView *)cell.leftView).image = FSImageNamedAndNormalColor(@"panel_signed_gray");
    }else{
        fieldTitle = FSLocalizedForKey(@"kDigitalSignatureUnsigned");
        fieldDetail = [cellData objectForKey:@"signName"];
        
        ((UIImageView *)cell.leftView).image = FSImageNamedAndNormalColor(@"panel_unsign_gray");
    }
    cell.estimatedTitleLabel.text = fieldTitle;
    cell.estimatedSubjectLabel.text = fieldDetail;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:NO];
    
    NSArray *keys = [self.digitalsigDict allKeys];
    int pageIndex = [keys containsObject:@(-1)] ? -1 : (int)indexPath.section;
    
    NSArray *rowsData = [self.digitalsigDict objectForKey:@(pageIndex)];
    NSMutableDictionary *cellData = rowsData[indexPath.row];
    
    [self.signaturePanel gotoPage:[[cellData objectForKey:@"pageIndex"] intValue] signatureIndex:(int)indexPath.row uuid:[cellData objectForKey:@"uuid"] isShowSignList:YES];
}

@end
