/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "OutlinePanel.h"
#import "OutlineViewController.h"
#import "PanelController+private.h"
#import "PanelHost.h"
#import "FSPermissionUtilty.h"

@interface OutlinePanel ()

@property (nonatomic, strong) TabItem *tabItem;
@property (nonatomic, strong) OutlineViewController *outlineCtrl;
@end

@implementation OutlinePanel

- (void)initialize{
    
    _outlineCtrl = [[OutlineViewController alloc] initWithExtensionsManager:self.extensionsManager panelController:self.panelController];
    _outlineCtrl.canEditBookmark = YES;
    _outlineCtrl.panel = self;
    _outlineCtrl.hasParentBookmark = NO;
    _outlineCtrl.treeLevel = 0;
    _outlineCtrl.view.backgroundColor = [UIColor clearColor];
    self.contentViewCtr = _outlineCtrl;
    _outlineCtrl.bookmarkGotoPageHandler = ^(int page) {
    };

    self.navController = [[FSNavigationController alloc] initWithRootViewController:_outlineCtrl];
    [self.contentView addSubview:self.navController.view];
    [self.navController.view mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left).offset(0);
        make.right.equalTo(self.contentView.mas_right).offset(0);
        make.top.equalTo(self.contentView.mas_top).offset(0);
        make.bottom.equalTo(self.contentView.mas_bottom).offset(0);
    }];
    
    self.title =  FSLocalizedForKey(@"kOutline");
    
}

-(void)onActivated {
    [super onActivated];
    [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingPanelOutline];
}

- (void)onDeactivated{
    [super onDeactivated];
    for (OutlineViewController *outlineCtrl in self.navController.viewControllers) {
        outlineCtrl.editing = NO;
    }
}

- (void)clickedRightItem{
    for (OutlineViewController *outlineCtrl in self.navController.viewControllers) {
        outlineCtrl.editing = !outlineCtrl.editing;
    }
}

- (void)load {
    [self.pdfViewCtrl registerDocEventListener:self];
    [self.pdfViewCtrl registerPageEventListener:self];
    [self.panelController.panel insertSpec:self atIndex:1];
}

- (void)unload {
    [self.pdfViewCtrl unregisterDocEventListener:self];
    [self.pdfViewCtrl unregisterPageEventListener:self];
    [self.panelController.panel removeSpec:self];
}

- (void)onDocWillOpen {
}

- (void)onDocOpened:(FSPDFDoc *)document error:(int)error {
    if (error == FSErrSuccess) {
        [_outlineCtrl loadData:nil];
        _outlineCtrl.addItem.enabled = [FSPermissionUtilty canAssemble:self.extensionsManager];
    }
}

- (void)onDocWillClose:(FSPDFDoc *)document {
    [_outlineCtrl clearData];
}

- (void)onDocClosed:(FSPDFDoc *)document error:(int)error {
    
}

- (void)onDocWillSave:(FSPDFDoc *)document {
}

- (int)getType {
    return FSPanelTypeOutline;
}

- (TabItem *)getTabItem{
    if (!_tabItem) {
        _tabItem = [TabItem new];
        _tabItem.image = FSImageNamedAndNormalColor(@"panel_outline_list_gray");
        _tabItem.selectImage = [_tabItem.image fs_imageWithTintColor:self.fs_primaryColor];
    }
    return _tabItem;
}

#pragma mark IPageEventListener
- (void)onPagesWillRemove:(NSArray<NSNumber *> *)indexes {
}

- (void)onPagesWillMove:(NSArray<NSNumber *> *)indexes dstIndex:(int)dstIndex {
}

- (void)onPagesWillRotate:(NSArray<NSNumber *> *)indexes rotation:(int)rotation {
}

- (void)onPagesRemoved:(NSArray<NSNumber *> *)indexes {
    [_outlineCtrl clearData];
    [_outlineCtrl loadData:nil];
}

- (void)onPagesMoved:(NSArray<NSNumber *> *)indexes dstIndex:(int)dstIndex {
    [_outlineCtrl clearData];
    [_outlineCtrl loadData:nil];
}

- (void)onPagesRotated:(NSArray<NSNumber *> *)indexes rotation:(int)rotation {
}

- (void)quitSearch{
    [self.outlineCtrl clickedSearchCancelItem];
}

@end
