/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "ReadingBookmarkPanel.h"
#import "PanelController+private.h"
#import "PanelHost.h"
#import "ReadingBookmarkModule.h"
#import "ReadingBookmarkViewController.h"
#import "FSPermissionUtilty.h"

@interface ReadingBookmarkPanel ()<IDocEventListener, IPageEventListener>

@property (nonatomic, assign) BOOL canEdit;
@property (nonatomic, strong) TabItem *tabItem;
@property (nonatomic, strong) ReadingBookmarkViewController *bookmarkCtrl;
@end

@implementation ReadingBookmarkPanel

- (void)initialize{
    
    self.bookmarkCtrl = [[ReadingBookmarkViewController alloc] initWithExtensionsManager:self.extensionsManager panelController:self.panelController];
    self.bookmarkCtrl.panel = self;
    //        _bookmarkCtrl.view.autoresizingMask = UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleWidth;
    self.contentViewCtr = self.bookmarkCtrl;
    _bookmarkCtrl.bookmarkGotoPageHandler = ^(int page) {

    };
    _bookmarkCtrl.bookmarkSelectionHandler = ^() {

    };

    @weakify(self)
    _bookmarkCtrl.bookmarkDeleteHandler = ^() {
        @strongify(self)
        [[NSNotificationCenter defaultCenter] postNotificationName:FSBookmarkUpdateNotification object:nil];
        self.extensionsManager.isDocModified = YES;
        [self.extensionsManager supportsMultipleScenesOp:^(UIExtensionsManager * _Nonnull ext) {
            if (ext != self.extensionsManager) [(ReadingBookmarkPanel *)[ext.panelController getPanel:FSPanelTypeReadingBookmark] reloadData];
        }];
    };
    
    self.bookmarkCtrl.view.frame = self.contentView.bounds;
    [self.contentView addSubview:self.bookmarkCtrl.view];
    [self.bookmarkCtrl.view mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.contentView.mas_left);
        make.right.equalTo(self.contentView.mas_right);
        make.top.equalTo(self.contentView.mas_top);
        make.bottom.equalTo(self.contentView.mas_bottom);
    }];
    
    self.title =  FSLocalizedForKey(@"kBookmarks");
}

- (void)clickedRightItem{
    self.bookmarkCtrl.editing = !self.bookmarkCtrl.editing;
}

- (void)load {
    [self.pdfViewCtrl registerDocEventListener:self];
    [self.pdfViewCtrl registerPageEventListener:self];
    [self.panelController.panel insertSpec:self atIndex:0];
}

- (void)unload {
    [self.pdfViewCtrl unregisterDocEventListener:self];
    [self.pdfViewCtrl unregisterPageEventListener:self];
    [self.panelController.panel removeSpec:self];
}

- (void)editBookMark {
    _bookmarkCtrl.isContentEditing = !_bookmarkCtrl.isContentEditing;
}

- (void)reloadData {
    [_bookmarkCtrl loadData];
}

- (void)setEditEnable:(BOOL)enable{
    self.rightItem.enabled = _canEdit & enable;
    if (!self.rightItem.enabled && self.rightItem.selected) {
        self.rightItem.selected = NO;
    }
}

#pragma mark--- IDocEventListener

- (void)onDocOpened:(FSPDFDoc *)document error:(int)error {
    if (error != FSErrSuccess) {
        return;
    }
    //    [_bookmarkCtrl clearData:NO];
    [Utility bookmarkMigration:document];
    _canEdit = [FSPermissionUtilty canAssemble:self.extensionsManager];
    [_bookmarkCtrl loadData];
}

- (void)onDocWillClose:(FSPDFDoc *)document {
    [_bookmarkCtrl clearData:NO];
}

- (void)onDocClosed:(FSPDFDoc *)document error:(int)error {
}

- (void)onDocWillSave:(FSPDFDoc *)document {
}

#pragma mark IPageEventListener
- (void)onPagesWillRemove:(NSArray<NSNumber *> *)indexes {
}

- (void)onPagesWillMove:(NSArray<NSNumber *> *)indexes dstIndex:(int)dstIndex {
}

- (void)onPagesWillRotate:(NSArray<NSNumber *> *)indexes rotation:(int)rotation {
}

- (void)onPagesRemoved:(NSArray<NSNumber *> *)indexes {
    NSMutableArray *readingbookMarksArray =[NSMutableArray array];
    
    FSPDFDoc *doc = self.pdfViewCtrl.currentDoc;
    int count = [doc getReadingBookmarkCount];
    for (int i = count - 1; i > -1; i--) {
        FSReadingBookmark *rb = [doc getReadingBookmark:i];
        int index = [rb getPageIndex];
        [readingbookMarksArray addObject:@(index)];
        if (-1 == index) {
            [doc removeReadingBookmark:rb];
        }
    }
    
    NSMutableArray *oldPageArray =[NSMutableArray array];
    int oldpageCount = [self.pdfViewCtrl getPageCount] + (int)[indexes count];
    for (int i = 0; i < oldpageCount; i++) {
        [oldPageArray addObject:@(i)];
    }
    
    for (int i = 0; i < indexes.count; i++) {
        [oldPageArray removeObject: [indexes objectAtIndex:i]];
    }

    if (![readingbookMarksArray containsObject:@(-1)]) {
        int count = (int)[readingbookMarksArray count];
        for (int i = count - 1; i > -1; i--) {
            FSReadingBookmark *rb = [doc getReadingBookmark:i];
            int index = [rb getPageIndex];
            [readingbookMarksArray addObject:@(index)];
            if ([indexes containsObject:@(index)]) {
                [doc removeReadingBookmark:rb];
            }
            if ([oldPageArray containsObject:@(index)] && rb.pageIndex != [oldPageArray indexOfObject:@(index)]) {
                int newPageIndex = (int)[oldPageArray indexOfObject:@(index)];
                rb.pageIndex = newPageIndex;
            }
        }
    }

    [_bookmarkCtrl clearData:NO];
    [_bookmarkCtrl loadData];

    dispatch_async(dispatch_get_main_queue(), ^{
        self.rightItem.enabled = YES;
    });
}

- (void)onPagesMoved:(NSArray<NSNumber *> *)indexes dstIndex:(int)dstIndex {
    [_bookmarkCtrl clearData:NO];
    [_bookmarkCtrl loadData];

    dispatch_async(dispatch_get_main_queue(), ^{
        self.rightItem.enabled = YES;
    });
}

- (void)onPagesRotated:(NSArray<NSNumber *> *)indexes rotation:(int)rotation {
}

- (int)getType {
    return FSPanelTypeReadingBookmark;
}

- (TabItem *)getTabItem{
    if (!_tabItem) {
        _tabItem = [TabItem new];
        _tabItem.image = FSImageNamedAndNormalColor(@"panel_bookmark_gray");
        _tabItem.selectImage = [_tabItem.image fs_imageWithTintColor:self.fs_primaryColor];
    }
    return _tabItem;
}

-(void)onActivated {
    [super onActivated];
    [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingPanelBookmark];
}

- (void)onDeactivated {
    [super onDeactivated];
    [self.bookmarkCtrl hideCellEditView];
    self.bookmarkCtrl.editing = NO;
}

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    return YES;
}

- (void)applicationWillResignActive:(UIApplication *)application {
}

- (void)applicationDidEnterBackground:(UIApplication *)application {
}

- (void)applicationWillEnterForeground:(UIApplication *)application {
}

- (void)applicationDidBecomeActive:(UIApplication *)application {
}

- (void)applicationWillTerminate:(UIApplication *)application {
}

- (BOOL)application:(UIApplication *)application openURL:(NSURL *)url sourceApplication:(NSString *)sourceApplication annotation:(id)annotation {
    return NO;
}

- (void)quitSearch{
    [self.bookmarkCtrl clickedSearchCancelItem];
}

@end
