/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "EncryptOptionViewController.h"
#import "FSTableView.h"

@implementation EncryptOptionCell
@end

@interface EncryptOptionViewController () <UIPopoverPresentationControllerDelegate,UITableViewDelegate,UITableViewDataSource>
//@property (nonatomic, strong) UIButton *buttonDone;
@property (nonatomic, strong) FSTableView *tableView;

- (void)buttonCancelClicked:(id)sender;
- (void)buttonDoneClicked:(id)sender;
- (void)initNavigationBar;
- (void)refreshInterface;

@end

@implementation EncryptOptionViewController

@synthesize cellOpenDoc;
@synthesize switchOpenDoc;
@synthesize cellPrintDoc;
@synthesize switchPrintDoc;
@synthesize cellCopyAccessibility;
@synthesize switchCopyAccessibility;
@synthesize cellOpenDocPassword;
@synthesize textboxOpenDocPassword;
@synthesize cellOtherPassword;
@synthesize textboxOtherPassword;
@synthesize cellEncryptRMS;
@synthesize buttonEncryptRMS;
@synthesize optionHandler = _optionHandler;
@synthesize rmsHandler = _rmsHandler;

#pragma mark - life cycle

- (id)initWithStyle:(UITableViewStyle)style {
    self = [super init];
    if (self) {
        self.shouldShowNavigationBar = YES;
    }
    return self;
}

-(instancetype)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil{
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]){
        self.shouldShowNavigationBar = YES;
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(StateOfDoneButton) name:UITextFieldTextDidChangeNotification object:nil];
    self.tableView = [[FSTableView alloc] init];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.view.backgroundColor = self.fs_backgroundColor_B9;
    self.tableView.backgroundColor = self.fs_backgroundColor_B9;
    self.tableView.separatorStyle = UITableViewCellSeparatorStyleNone;
    if (DEVICE_iPHONE) {
        [self.tableView setBackgroundView:nil];
    }
    [self.view addSubview:self.tableView];
    [self.tableView mas_makeConstraints:^(MASConstraintMaker *make) {
        if (@available(iOS 11.0, *)) {
            make.left.right.bottom.equalTo(self.view.mas_safeAreaLayoutGuide);
        }else{
            make.left.right.bottom.equalTo(self.view);
        }
        make.top.equalTo(self.navBar.mas_bottom);
    }];

    [self refreshInterface];
    
    [self initNavigationBar];
}

/*
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return YES;
}
*/

#pragma mark - Table view data source
//TODO
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    //disable RMS encrypt
    int rowCount = 2; //info cells
    if (switchOpenDoc.on) {
        rowCount++; //show open doc password
    }
    if (_switchAddLimitation.on) {
        rowCount += 8;
    }
    return rowCount;
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section{
    return 0;
}

- (CGFloat)tableView:(UITableView *)tableView heightForFooterInSection:(NSInteger)section{
    if (section == 0 || (!switchOpenDoc.on && section == 1) || (switchOpenDoc.on && section == 2)) {
        UILabel *footerLabel = [[UILabel alloc] init];
        footerLabel.numberOfLines = 0;
        footerLabel.font = self.fs_standardWeakFont_WF2;
        if (section == 0) {
            footerLabel.text = FSLocalizedForKey(@"kEncryptionOpenDocSectionFooter");
        }else{
            footerLabel.text = FSLocalizedForKey(@"kEncryptionAddLimitationFooter");
        }
        CGSize size = [footerLabel sizeThatFits:CGSizeMake(self.tableView.fs_width - 32, CGFLOAT_MAX)];
        return size.height + 16;
    }else if ((switchOpenDoc.on && section == 1)) {
        return 8;
    }
    return 0;
}

- (UIView *)tableView:(UITableView *)tableView viewForFooterInSection:(NSInteger)section{
    UIView *footerView = nil;
    if (section == 0 || (!switchOpenDoc.on && section == 1) || (switchOpenDoc.on && section == 2)) {
        footerView = [[UIView alloc] initWithFrame:CGRectMake(0, 0, self.tableView.bounds.size.width, 32)];
        footerView.backgroundColor = self.fs_backgroundColor_B9;
        UILabel *footerLabel = [[UILabel alloc] init];
        footerLabel.numberOfLines = 0;
        [footerView addSubview:footerLabel];
        footerLabel.textColor = self.fs_textColor_T3;
        footerLabel.font = self.fs_standardWeakFont_WF2;
        if (section == 0) {
            footerLabel.text = FSLocalizedForKey(@"kEncryptionOpenDocSectionFooter");
        }else{
            footerLabel.text = FSLocalizedForKey(@"kEncryptionAddLimitationFooter");
        }
        [footerLabel mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(footerView).offset(16);
            make.top.equalTo(footerView).offset(8);
            make.right.equalTo(footerView).offset(-16);
            make.bottom.equalTo(footerView).offset(-8);
        }];
    }
    return footerView;

}

//TODO:
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return 1;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 60.f;
}

//TODO:
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    NSInteger section = indexPath.section;
    if (section == 0) {
        cellOpenDoc.contentView.backgroundColor = self.fs_backgroundColor_B8;
        return cellOpenDoc;
    }
    if (switchOpenDoc.on) {
        section--;
    }
    if (section == 0) {
        cellOpenDocPassword.contentView.backgroundColor = self.fs_backgroundColor_B8;
        return cellOpenDocPassword;
    }
    if (section == 1) {
        _cellAddLimitation.contentView.backgroundColor = self.fs_backgroundColor_B8;
        return _cellAddLimitation;
    }
    if (section == 2) {
        cellPrintDoc.contentView.backgroundColor = self.fs_backgroundColor_B8;
        return cellPrintDoc;
    }
    if (section == 3) {
        _cellFillForm.contentView.backgroundColor = self.fs_backgroundColor_B8;
        return _cellFillForm;
    }
    if (section == 4) {
        _cellAnnotDoc.contentView.backgroundColor = self.fs_backgroundColor_B8;
        return _cellAnnotDoc;
    }
    if (section == 5) {
        _cellAssembleDoc.contentView.backgroundColor = self.fs_backgroundColor_B8;
        return _cellAssembleDoc;
    }
    if (section == 6) {
        _cellEditDocument.contentView.backgroundColor = self.fs_backgroundColor_B8;
        return _cellEditDocument;
    }
    if (section == 7) {
        cellCopyAccessibility.contentView.backgroundColor = self.fs_backgroundColor_B8;
        return cellCopyAccessibility;
    }
    if (section == 8) {
        _cellExtractContent.contentView.backgroundColor = self.fs_backgroundColor_B8;
        return _cellExtractContent;
    }
    if (_switchAddLimitation.on && section == 9) {
        cellOtherPassword.contentView.backgroundColor = self.fs_backgroundColor_B8;
        return cellOtherPassword;
    }

    return nil;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

#pragma mark - UITextField delegate handler

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    [textField resignFirstResponder];
    return YES;
}

- (void)StateOfDoneButton {
    if (self.textboxOtherPassword.text.length == 0 && self.textboxOpenDocPassword.text.length == 0) {
        self.navBar.rightItem.enabled = NO;
    } else {
        self.navBar.rightItem.enabled = YES;
    }
}

#pragma mark - UINavigationController delegate handler

- (void)navigationController:(UINavigationController *)navigationController willShowViewController:(UIViewController *)viewController animated:(BOOL)animated {

}

#pragma mark - event handler

- (IBAction)switchOpenDocValueChanged:(id)sender {
    if (!((UISwitch *) sender).on) {
        self.textboxOpenDocPassword.text = nil;
    }
    [self StateOfDoneButton];
    [self.tableView reloadData];
}

- (IBAction)switchPrintDocValueChanged:(id)sender {
    [self.tableView reloadData];
}

- (IBAction)switchCopyAccessibilityValueChanged:(id)sender {
    if (!((UISwitch *) sender).on) {
        self.switchExtractContent.on = NO;
    }
    [self.tableView reloadData];
}

- (IBAction)switchAnnotDocValueChanged:(id)sender {
    if (((UISwitch *) sender).on) {
        self.switchFillForm.on = YES;
    }
    [self.tableView reloadData];
}

- (IBAction)switchAssembleDocValueChanged:(id)sender {
    if (!((UISwitch *) sender).on) {
        self.switchEditDocument.on = NO;
    }
    [self.tableView reloadData];
}

- (IBAction)encryptUsingRMS:(id)sender {
    if (self.rmsHandler != nil) {
        self.rmsHandler(self);
    }

    [self close];
}

- (IBAction)switchFillFormValueChanged:(id)sender {
    if (!((UISwitch *) sender).on) {
        self.switchAnnotDoc.on = NO;
        self.switchEditDocument.on = NO;
    }
    [self.tableView reloadData];
}

- (IBAction)switchEditDocumentValueChanged:(id)sender {
    if (((UISwitch *) sender).on) {
        self.switchAssembleDoc.on = YES;
        self.switchFillForm.on = YES;
    }
    [self.tableView reloadData];
}

- (IBAction)switchExtractContentValueChanged:(id)sender {
    if (((UISwitch *) sender).on) {
        self.switchCopyAccessibility.on = YES;
    }
    [self.tableView reloadData];
}

- (IBAction)switchAddLimitationValueChanged:(id)sender {
    if (!((UISwitch *) sender).on) {
        self.textboxOtherPassword.text = nil;
    }
    [self StateOfDoneButton];
    [self.tableView reloadData];
}

- (void)navigationBarBackTouch{
    if (self.optionHandler != nil) {
        self.optionHandler(self, YES, nil, nil, NO, NO, NO, NO, NO, NO, NO, NO);
    }
    [self close];
}

//- (void)buttonCancelClicked:(id)sender {
//    if (self.optionHandler != nil) {
//        self.optionHandler(self, YES, nil, nil, NO, NO, NO, NO, NO, NO, NO, NO);
//    }
//    [self close];
//}

- (void)close {
    [self dismissViewControllerAnimated:YES completion:nil];
}

- (void)buttonDoneClicked:(id)sender {
    
    if ([self->textboxOpenDocPassword.text isEqualToString:self->textboxOtherPassword.text]) {
        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kPasswordNotSame") preferredStyle:UIAlertControllerStyleAlert];
        UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                         style:UIAlertActionStyleCancel
                                                       handler:^(UIAlertAction *_Nonnull action) {
                                                           [self popoverDidCancel];
                                                       }];
        [alertController addAction:action];
        [self presentAlertController:alertController];
        return;
    }
    
    [self dismissViewControllerAnimated:YES completion:^{
        if (self->switchOpenDoc.on && self->textboxOpenDocPassword.text.length == 0) {
            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kEncryptMissOpenDocPass") preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                             style:UIAlertActionStyleCancel
                                                           handler:^(UIAlertAction *_Nonnull action) {
                                                               [self popoverDidCancel];
                                                           }];
            [alertController addAction:action];
            [self presentAlertController:alertController];
            return;
        }
        if ((self->switchPrintDoc.on || self->switchCopyAccessibility.on || self->_switchAnnotDoc.on || self->_switchAssembleDoc.on || self->_switchExtractContent.on || self->_switchFillForm.on || self->_switchEditDocument.on) && self->textboxOtherPassword.text.length == 0 && self->textboxOpenDocPassword.text.length == 0) {
            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kEncryptMissOtherPass") preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                             style:UIAlertActionStyleCancel
                                                           handler:^(UIAlertAction *_Nonnull action) {
                                                               [self popoverDidCancel];
                                                           }];
            [alertController addAction:action];
            [self presentAlertController:alertController];
            return;
        }
        if (self.optionHandler != nil) {
            NSString *openDocPass = (self->textboxOpenDocPassword.text.length == 0) ? nil : self->textboxOpenDocPassword.text;
            NSString *otherPass = (self->textboxOtherPassword.text.length == 0) ? nil : self->textboxOtherPassword.text;
            if (openDocPass == nil && otherPass == nil) //if none password is enter, tell user to give up
            {
                UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kConfirm") message:FSLocalizedForKey(@"kEncryptionDoneNoPass") preferredStyle:UIAlertControllerStyleAlert];
                UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kNo")
                                                                       style:UIAlertActionStyleCancel
                                                                     handler:^(UIAlertAction *action) {
                                                                         self.optionHandler(self, YES, nil, nil, NO, NO, NO, NO, NO, NO, NO, NO);
                                                                         [self close];
                                                                     }];
                UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kYes") style:UIAlertActionStyleDefault handler:nil];
                [alertController addAction:cancelAction];
                [alertController addAction:action];
                [self presentAlertController:alertController];
                return;
            }
            self.optionHandler(self, NO, openDocPass, otherPass, self->switchPrintDoc.on, NO, self->_switchFillForm.on, self->_switchAnnotDoc.on, self->_switchAssembleDoc.on, self->_switchEditDocument.on, self->switchCopyAccessibility.on, self->_switchExtractContent.on);
        }
    }];
}

#pragma mark pop over

- (void)presentAlertController:(UIAlertController *)alertController {
    [[self fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
    self.currentVC = alertController;
}

- (void)popoverDidCancel {
    self.currentVC = nil;
}

#pragma mark - Private methods

- (void)initNavigationBar {

    [self setTitle:FSLocalizedForKey(@"kEncryptionTitle")];
    self.navBarLeftTitle = FSLocalizedForKey(@"kClose");
    self.navBarRightTitle = FSLocalizedForKey(@"kDone");
    [self.navBar.rightItem setTitleColor:self.fs_textColor_T2 forState:UIControlStateDisabled];
    [self.navBar.rightItem addTarget:self action:@selector(buttonDoneClicked:) forControlEvents:UIControlEventTouchUpInside];
    self.navBar.rightItem.enabled = NO;
    StateBarThemeKeyPath(self.navBar.stateBar, backgroundColor);
    [BindThemeKeySel(self.navBar.stateBar, backgroundView) fs_updateThemeCompleted:^(UIView*  _Nonnull target, SEL  _Nonnull sel) {
        StateBarThemeKeyPath(target, backgroundColor);
    }];
}

//TODO
- (void)refreshInterface {
    id labelTitle = [cellOpenDoc.contentView viewWithTag:100];
    if ([labelTitle isKindOfClass:[UILabel class]]) {
        ((UILabel *) labelTitle).text = FSLocalizedForKey(@"kCellEncryptOpenDocumentTitle");
        ((UILabel *) labelTitle).textColor = self.fs_textColor_T4;
    }
    labelTitle = [cellPrintDoc.contentView viewWithTag:100];
    if ([labelTitle isKindOfClass:[UILabel class]]) {
        ((UILabel *) labelTitle).text = FSLocalizedForKey(@"kCellEncryptPrintDocumentTitle");
        ((UILabel *) labelTitle).textColor = self.fs_textColor_T4;
    }
    labelTitle = [cellCopyAccessibility.contentView viewWithTag:100];
    if ([labelTitle isKindOfClass:[UILabel class]]) {
        ((UILabel *) labelTitle).text = FSLocalizedForKey(@"kCellEncryptCopyContentTitle");
        ((UILabel *) labelTitle).textColor = self.fs_textColor_T4;
    }
    labelTitle = [_cellAnnotDoc.contentView viewWithTag:100];
    if ([labelTitle isKindOfClass:[UILabel class]]) {
        ((UILabel *) labelTitle).text = FSLocalizedForKey(@"kRMSANNOTATE");
        ((UILabel *) labelTitle).textColor = self.fs_textColor_T4;
    }
    labelTitle = [_cellAssembleDoc.contentView viewWithTag:100];
    if ([labelTitle isKindOfClass:[UILabel class]]) {
        ((UILabel *) labelTitle).text = FSLocalizedForKey(@"kRMSASSEMBLE");
        ((UILabel *) labelTitle).textColor = self.fs_textColor_T4;
    }
    labelTitle = [cellOpenDocPassword.contentView viewWithTag:100];
    if ([labelTitle isKindOfClass:[UILabel class]]) {
        ((UILabel *) labelTitle).text = FSLocalizedForKey(@"kCellEncryptPasswordTitle");
        ((UILabel *) labelTitle).textColor = self.fs_textColor_T4;
    }
    labelTitle = [cellOpenDocPassword.contentView viewWithTag:101];
    if ([labelTitle isKindOfClass:[UILabel class]]) {
        ((UILabel *) labelTitle).textColor = self.fs_textColor_T4;
    }
    labelTitle = [cellOtherPassword.contentView viewWithTag:100];
    if ([labelTitle isKindOfClass:[UILabel class]]) {
        ((UILabel *) labelTitle).text = FSLocalizedForKey(@"kCellEncryptPasswordTitle");
        ((UILabel *) labelTitle).textColor = self.fs_textColor_T4;
    }
    labelTitle = [cellOtherPassword.contentView viewWithTag:101];
    if ([labelTitle isKindOfClass:[UILabel class]]) {
        ((UILabel *) labelTitle).textColor = self.fs_textColor_T4;
    }
    labelTitle = [_cellAddLimitation.contentView viewWithTag:100];
    if ([labelTitle isKindOfClass:[UILabel class]]) {
        ((UILabel *) labelTitle).text = FSLocalizedForKey(@"kPwdDocumentLimitation");
        ((UILabel *) labelTitle).textColor = self.fs_textColor_T4;
    }
    labelTitle = [_cellFillForm.contentView viewWithTag:100];
    if ([labelTitle isKindOfClass:[UILabel class]]) {
        ((UILabel *) labelTitle).text = FSLocalizedForKey(@"kPwdFillForm");
        ((UILabel *) labelTitle).textColor = self.fs_textColor_T4;
    }
    labelTitle = [_cellEditDocument.contentView viewWithTag:100];
    if ([labelTitle isKindOfClass:[UILabel class]]) {
        ((UILabel *) labelTitle).text = FSLocalizedForKey(@"kPwdEditDocument");
        ((UILabel *) labelTitle).textColor = self.fs_textColor_T4;
    }
    labelTitle = [_cellExtractContent.contentView viewWithTag:100];
    if ([labelTitle isKindOfClass:[UILabel class]]) {
        ((UILabel *) labelTitle).text = FSLocalizedForKey(@"kPwdExtractContent");
        ((UILabel *) labelTitle).textColor = self.fs_textColor_T4;
    }
    textboxOpenDocPassword.placeholder = FSLocalizedForKey(@"kRequiredPlaceHolder");
    textboxOtherPassword.placeholder = FSLocalizedForKey(@"kRequiredPlaceHolder");
    [buttonEncryptRMS setTitle:FSLocalizedForKey(@"kRMSEncrypt") forState:UIControlStateNormal];
}

@end

