/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "EraseModule.h"
#import <FoxitRDK/FSPDFViewControl.h>

#import "EraseToolHandler.h"
#import "Utility.h"

@interface EraseModule ()

@property (nonatomic, weak) FSMainToolbarItem *propertyItem;
@property (nonatomic, weak) FSReadToolbarItem *readToolbarItem;
@end

@implementation EraseModule {
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;
}

- (NSString *)getName {
    return @"Eraser";
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        [self loadModule];
        EraseToolHandler* toolHandler = [[EraseToolHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerToolHandler:toolHandler];
    }
    return self;
}

- (void)loadModule {
    [_extensionsManager.toolbarManager addReadItem:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeEraser] atToolTag:FSMainTopbarToolTagHome];
    FSReadToolbarItem *eraseItem = [[FSReadToolbarItem alloc] initWithToolSettings:[_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeEraser]];
    [_extensionsManager.toolbarManager addReadItem:eraseItem atToolTag:FSMainTopbarToolTagDrawing];
}

- (void)didClickedReadToolItem:(FSReadToolbarItem *)readToolbarItem {
    [_extensionsManager changeState:FSUIManagerStateAnnotTool];
    id<IToolHandler> toolHandler = [_extensionsManager getToolHandlerByName:Tool_Eraser];
    toolHandler.toolSettings = readToolbarItem.toolSettings;
    [_extensionsManager setCurrentToolHandler:toolHandler];
    
    self.readToolbarItem = readToolbarItem;
    FSMainToolbarItem *propertyItem =  [_extensionsManager.toolbarManager getMainItemWithType:FSMainToolbarItemTypeProperty];
    propertyItem.item.onTapClick = ^(FSItem * _Nonnull _item) {
        CGRect rect = [_item convertRect:_item.bounds toView:self->_extensionsManager.pdfViewCtrl];
        if (DEVICE_iPHONE) {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:rect inView:self->_extensionsManager.pdfViewCtrl];
        } else {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:_item.bounds inView:_item];
        }
    };
    self.propertyItem = propertyItem;
    
    [_extensionsManager onUIInteractElementClicked:FSUIElementReadingDrawingBarEraser];
}

@end
