/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "EraseUtil.h"

@implementation EraseUtil

+ (double)distanceOfTwoPoints:(FSPointF *)point1 point2:(FSPointF *)point2 {
    double dx = point1.x - point2.x;
    double dy = point1.y - point2.y;
    return sqrt(dx * dx + dy * dy);
}

+ (BOOL)isInsideCircle:(FSPointF *)point circleCenterPoint:(FSPointF *)circleCenterPoint radius:(CGFloat)radius{
    double dist = [self distanceOfTwoPoints:point point2:circleCenterPoint];
    if (dist < radius) {
        return YES;
    }
    return NO;
}

+ (void)addCenterPoint:(FSPointF *)p1 p2:(FSPointF *)p2 radius:(CGFloat)radius array:(NSMutableArray<FSPointF *> *)array{
    if ([self distanceOfTwoPoints:p1 point2:p2] > radius) {
        FSPointF *centerPoint = [[FSPointF alloc] init];
        centerPoint.x = (p1.x + p2.x) / 2;
        centerPoint.y = (p1.y + p2.y) / 2;

        [array addObject:centerPoint];

        [self addCenterPoint:p1 p2:centerPoint radius:radius array:array];
        [self addCenterPoint:p2 p2:centerPoint radius:radius array:array];
    }
}

+ (void)getU:(FSPointF *)point1 point2:(FSPointF *)point2 circleCenterPoint:(FSPointF *)point u1:(double *)u1 u2:(double *)u2 radius:(CGFloat)radius{
    double a = (point2.x - point1.x) * (point2.x - point1.x) + (point2.y - point1.y) * (point2.y - point1.y);
    double b = 2 * ((point2.x - point1.x) * (point1.x - point.x) + (point2.y - point1.y) * (point1.y - point.y));
    double c = point.x * point.x + point.y * point.y + point1.x * point1.x + point1.y * point1.y - 2 * (point.x * point1.x + point.y * point1.y) - radius * radius;

    double d = sqrt(b * b - 4 * a * c);
    *u1 = (-b + d) / (2 * a);
    *u2 = (-b - d) / (2 * a);
}

+ (double)cg_distanceOfTwoPoints:(CGPoint)point1 point2:(CGPoint)point2 {
    double dx = point1.x - point2.x;
    double dy = point1.y - point2.y;
    return sqrt(dx * dx + dy * dy);
}

+ (BOOL)cg_isInsideCircle:(CGPoint)point circleCenterPoint:(CGPoint)circleCenterPoint radius:(CGFloat)radius{
    double dist = [self cg_distanceOfTwoPoints:point point2:circleCenterPoint];
    if (dist < radius) {
        return YES;
    }
    return NO;
}

+ (void)cg_addCenterPoint:(CGPoint)p1 p2:(CGPoint)p2 radius:(CGFloat)radius array:(NSMutableArray<NSValue *> *)array{
    if ([self cg_distanceOfTwoPoints:p1 point2:p2] > radius) {
        CGPoint centerPoint = {(p1.x + p2.x) / 2, (p1.y + p2.y) / 2};

        [array addObject:[NSValue valueWithCGPoint:centerPoint]];

        [self cg_addCenterPoint:p1 p2:centerPoint radius:radius array:array];
        [self cg_addCenterPoint:p2 p2:centerPoint radius:radius array:array];
    }
}

+ (void)cg_getU:(CGPoint)point1 point2:(CGPoint)point2 circleCenterPoint:(CGPoint)point u1:(double *)u1 u2:(double *)u2 radius:(CGFloat)radius{
    double a = (point2.x - point1.x) * (point2.x - point1.x) + (point2.y - point1.y) * (point2.y - point1.y);
    double b = 2 * ((point2.x - point1.x) * (point1.x - point.x) + (point2.y - point1.y) * (point1.y - point.y));
    double c = point.x * point.x + point.y * point.y + point1.x * point1.x + point1.y * point1.y - 2 * (point.x * point1.x + point.y * point1.y) - radius * radius;

    double d = sqrt(b * b - 4 * a * c);
    *u1 = (-b + d) / (2 * a);
    *u2 = (-b - d) / (2 * a);
}

+ (NSMutableArray<NSMutableArray<FSPointF *> *> *)getPointLinesFromInkAnnot:(FSInk *)inkAnnot {
    NSMutableArray<NSMutableArray<FSPointF *> *> *arrayLines = [NSMutableArray<NSMutableArray<FSPointF *> *> array];
    FSPath *path = [inkAnnot getInkList];
    FSPointF *lastPt = nil;
    for (int i = 0; i < [path getPointCount]; i++) {
        FSPointF *pt = [path getPoint:i];
        FSPathPointType type = [path getPointType:i];
        if (type == FSPathTypeMoveTo) {
            [arrayLines addObject:[NSMutableArray<FSPointF *> arrayWithObject:pt]];
        } else {
            if (fabs(lastPt.x - pt.x) > 1e-3 || fabs(lastPt.y - pt.y) > 1e-3) {
                [[arrayLines lastObject] addObject:pt];
            }
        }
        lastPt = pt;
    }
    return arrayLines;
}

+ (NSMutableArray<FSPointF *> *)getAllPointFromInkAnnot:(FSInk *)inkAnnot {
    NSMutableArray<FSPointF *> *arrayPoints = [NSMutableArray<FSPointF *> array];
    FSPath *path = [inkAnnot getInkList];
    for (int i = 0; i < [path getPointCount]; i++) {
        FSPointF *pt = [path getPoint:i];
        [arrayPoints addObject:pt];
    }
    return arrayPoints;
}


@end
