/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "PencilAnnotHandler.h"
#import "FSPermissionUtilty.h"

@implementation PencilAnnotHandler

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    if (self = [super initWithUIExtensionsManager:extensionsManager]) {
        self.annotSelectionEdgeInsets = UIEdgeInsetsMake(-20, -20, -20, -20);
        self.annotRefreshRectEdgeInsets = UIEdgeInsetsMake(-30, -30, -30, -30);
    }
    return self;
}

- (FSAnnotType)getType {
    return FSAnnotInk;
}

- (void)onAnnotSelected:(FSAnnot *)annot {
    if ([[self.extensionsManager.currentToolHandler getName] isEqualToString:Tool_Eraser]) {
        //noop, todo, add to undo/redo
        return;
    }
    [super onAnnotSelected:annot];
}



- (FSMenuOptions)menuOptionsForAnnot:(FSAnnot *)annot {
    FSMenuOptions options = FSMenuOptionOpen;
    if ([self.extensionsManager annotPermissionCanFlatten:annot]) {
        options |= FSMenuOptionFlatten;
    }
    if (![FSPermissionUtilty canAddAnnot:self.extensionsManager]) {
        options = FSMenuOptionOpen;
    }
    if (annot.canModify && [FSPermissionUtilty canAddAnnot:self.extensionsManager]) {
        options |= FSMenuOptionStyle;
        if ([annot canDeleteWithExternsions:self.extensionsManager]) {
            options |= FSMenuOptionDelete;
        }
    }
    if (annot.canReply && [FSPermissionUtilty canAddAnnot:self.extensionsManager]) {
        if ([self.extensionsManager annotsPermissionCanReply:annot]) {
            options |= FSMenuOptionReply;
        }
    }
    if ([FSPermissionUtilty canAddAnnot:self.extensionsManager] && self.extensionsManager.config.defaultSettings.enableHandwritingRecognition) {
        options |= FSMenuOptionRecognizeText;
    }
    
    return options;
}

- (void)resetProperties{

    FSAnnot *annot = self.extensionsManager.currentAnnot;
    FSPropertyMainView *propertyMainView = self.extensionsManager.propertyMainView;
    [propertyMainView setLineWidthLayoutValue:FSLineWidthLayoutValueHighlighter];
    [propertyMainView setPropertyColor:annot.color];
    [propertyMainView setOpacity:annot.opacity];
    [propertyMainView setLineWidth:annot.lineWidth];
    [propertyMainView setLock:![annot canModifyAppearanceWithExternsions:self.extensionsManager]];

    [propertyMainView resetProperties:@[ @(FSPropertyTypePropertyColor),@(FSPropertyTypeLineWidth) , @(FSPropertyTypeOpacity)]];
}

- (BOOL)shouldDrawAnnot:(FSAnnot *)annot inPDFViewCtrl:(FSPDFViewCtrl *)pdfViewCtrl{
    if (self.extensionsManager.currentToolHandler && [self.extensionsManager.currentToolHandler respondsToSelector:@selector(shouldDrawAnnot:inPDFViewCtrl:)]){
        return [self.extensionsManager.currentToolHandler shouldDrawAnnot:annot inPDFViewCtrl:pdfViewCtrl];
    }
    if ([annot isEqualToAnnot:self.drawingInkAnnot]) {
        return NO;
    }
    return [super shouldDrawAnnot:annot inPDFViewCtrl:pdfViewCtrl];
}

@end
