/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "PencilModule.h"
#import "PencilAnnotHandler.h"
#import "PencilToolHandler.h"
#import "Utility.h"
#import <FoxitRDK/FSPDFViewControl.h>
#import <CoreBluetooth/CoreBluetooth.h>

#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_12_1
@interface PencilModule ()<CBCentralManagerDelegate, UIPencilInteractionDelegate>
#else
@interface PencilModule ()<CBCentralManagerDelegate>
#endif

@property (nonatomic, weak) FSMainToolbarItem *propertyItem;
@property (nonatomic, weak) FSReadToolbarItem *readToolbarItem;
@property (nonatomic, strong) FSReadToolbarItem *pencilItem;
@property (nonatomic, strong) CBCentralManager *m_centralManager;

@end

@implementation PencilModule {
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;
}

- (NSString *)getName {
    return @"Pencil";
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        [self loadModule];
        PencilAnnotHandler* annotHandler = [[PencilAnnotHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerAnnotHandler:annotHandler];

        PencilToolHandler* toolHandler = [[PencilToolHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerToolHandler:toolHandler];
        [_extensionsManager registerAnnotPropertyListener:toolHandler];
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_12_1
        if (@available(iOS 12.1, *)) {
            UIPencilInteraction *pencilInteraction = [[UIPencilInteraction alloc] init];
            pencilInteraction.delegate = self;
            [_pdfViewCtrl addInteraction:pencilInteraction];
        } else {
            // Fallback on earlier versions
        }
#endif
    }
    return self;
}

#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_12_1
- (void)pencilInteractionDidTap:(UIPencilInteraction *)interaction API_AVAILABLE(ios(12.1)){
    if (_extensionsManager.currentToolHandler) {
        NSString* currentToolName = [_extensionsManager.currentToolHandler getName];
        if ([currentToolName isEqualToString:Tool_Pencil]) {
            [self switchPencilEraser:FSReadToolSettingsTypeEraser];
        }
        
        if ([currentToolName isEqualToString:Tool_Eraser]) {
            [self switchPencilEraser:FSReadToolSettingsTypePencil];
        }
    }
}
#endif

- (void)switchPencilEraser:(FSReadToolSettingsType)toolSettingType{
    
    if (_extensionsManager.getState != FSUIManagerStateNormal) {
        [_extensionsManager changeState:FSUIManagerStateNormal];
    }
    
    BOOL isExistPencil = NO;
    if (!_extensionsManager.fullScreenToolbarMgr.floatyToolbar.hidden) {
        for (FSMainTopbarToolTagItem *tagItem in _extensionsManager.toolbarManager.mainTopbar.tagItems) {
            if (tagItem.toolTag == FSMainTopbarToolTagHome) {
                for (FSReadToolbarItem *readItem in tagItem.readToolbarItems) {
                    if (readItem.settingsType == toolSettingType) {
                        [readItem selectFormSuperBar];
                        isExistPencil = YES;
                        break;
                    }
                }
                break;
            }
        }
    }
    
    if (!isExistPencil) {
        if (_extensionsManager.currentIsFullScreen) {
            [_extensionsManager setFullScreen:NO];
        }
        
        isExistPencil = NO;
        FSMainTopbarToolTagItem *currentItem = _extensionsManager.toolbarManager.mainTopbar.currentTagItem;
        for (FSReadToolbarItem *readItem in currentItem.readToolbarItems) {
            if (readItem.settingsType == toolSettingType) {
                [readItem selectFormSuperBar];
                isExistPencil = YES;
                break;
            }
        }
        
        if (!isExistPencil) {
            [_extensionsManager.toolbarManager.mainTopbar switchToTag:FSMainTopbarToolTagDrawing];
            FSMainTopbarToolTagItem *currentItem = _extensionsManager.toolbarManager.mainTopbar.currentTagItem;
            for (FSReadToolbarItem *readItem in currentItem.readToolbarItems) {
                if (readItem.settingsType == toolSettingType) {
                    [readItem selectFormSuperBar];
                    break;
                }
            }
        }
    }
}

- (void)loadModule {
    self.isApplePencilConnect = NO;
    
    [_extensionsManager.toolbarManager addReadItem:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypePencil] atToolTag:FSMainTopbarToolTagHome];
    _pencilItem = [[FSReadToolbarItem alloc] initWithToolSettings:[_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypePencil]];
    [_extensionsManager.toolbarManager addReadItem:_pencilItem atToolTag:FSMainTopbarToolTagDrawing];
    
    FSReadToolbarItem *highlighItem = [[FSReadToolbarItem alloc] initWithToolSettings:[_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeHighlighter]];
    [_extensionsManager.toolbarManager addReadItem:highlighItem atToolTag:FSMainTopbarToolTagDrawing];
    [_extensionsManager registerAnnotPropertyListener:self];
    
    [_extensionsManager.pdfViewCtrl registerDocEventListener:self];
}

- (void)didClickedReadToolItem:(FSReadToolbarItem *)readToolbarItem {
    [_extensionsManager changeState:FSUIManagerStateAnnotTool];
    id<IToolHandler> toolHandler = [_extensionsManager getToolHandlerByName:Tool_Pencil];
    toolHandler.toolSettings = readToolbarItem.toolSettings;
    [_extensionsManager setCurrentToolHandler:toolHandler];

    self.readToolbarItem = readToolbarItem;
    FSMainToolbarItem *propertyItem =  [_extensionsManager.toolbarManager getMainItemWithType:FSMainToolbarItemTypeProperty];
    propertyItem.item.onTapClick = ^(FSItem * _Nonnull _item) {
        CGRect rect = [_item convertRect:_item.bounds toView:self->_extensionsManager.pdfViewCtrl];
        if (DEVICE_iPHONE) {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:rect inView:self->_extensionsManager.pdfViewCtrl];
        } else {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:_item.bounds inView:_item];
        }
    };
    self.propertyItem = propertyItem;
    
    if (readToolbarItem.toolSettings.settingsType == FSReadToolSettingsTypePencil) {
        [_extensionsManager onUIInteractElementClicked:FSUIElementReadingDrawingBarPencil];
    }
    else if (readToolbarItem.toolSettings.settingsType == FSReadToolSettingsTypeHighlighter) {
        [_extensionsManager onUIInteractElementClicked:FSUIElementReadingDrawingBarHighlight];
    }
    
    [self resetApplePencilConnect];
}

#pragma mark - IAnnotPropertyListener

- (void)onToolSettingsChanged:(FSReadToolSettings *)toolSettings {
    if (self.readToolbarItem && self.readToolbarItem.toolSettings == toolSettings) {
        self.readToolbarItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }else if (_pencilItem.toolSettings == toolSettings){
        _pencilItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }
}

#pragma mark - IDocEventListener
- (void)onDocWillOpen{
    [self initModuleData];
}

- (void)initModuleData
{
    if (_extensionsManager.disableFingerWhenLinkApplePencil){
        if (!_m_centralManager) {
            _m_centralManager = [[CBCentralManager alloc] initWithDelegate:self queue:nil options:nil];
        }
    }else{
        _m_centralManager = nil;
        _isApplePencilConnect = NO;
    }
}

#pragma mark - CBCentralManagerDelegate
- (void)centralManagerDidUpdateState:(CBCentralManager *)central
{
    _isApplePencilConnect = NO;
    if ([central state] == CBManagerStatePoweredOn)
    {
        NSArray* myArray = [NSArray arrayWithObject:[CBUUID UUIDWithString:@"180A"]];
        
        NSArray* peripherals =
        [_m_centralManager retrieveConnectedPeripheralsWithServices:myArray];
        for (CBPeripheral* peripheral in peripherals)
        {
            if ([[peripheral name] isEqualToString:@"Apple Pencil"])
            {
                _isApplePencilConnect = YES;
                break;
            }
        }
    }
}
- (void)centralManager:(CBCentralManager *)central didConnectPeripheral:(CBPeripheral *)peripheral
{
    if ([[peripheral name] isEqualToString:@"Apple Pencil"])
    {
        _isApplePencilConnect = YES;
    }
}

- (void)centralManager:(CBCentralManager *)central didDisconnectPeripheral:(CBPeripheral *)peripheral error:(nullable NSError *)error
{
    if ([[peripheral name] isEqualToString:@"Apple Pencil"])
    {
        _isApplePencilConnect = NO;
    }
}

- (void)resetApplePencilConnect
{
    _isApplePencilConnect = NO;
    if ([_m_centralManager state] == CBManagerStatePoweredOn)
    {
        // Device information UUID
        NSArray* myArray = [NSArray arrayWithObject:[CBUUID UUIDWithString:@"180A"]];
        
        NSArray* peripherals =
        [_m_centralManager retrieveConnectedPeripheralsWithServices:myArray];
        for (CBPeripheral* peripheral in peripherals)
        {
            if ([[peripheral name] isEqualToString:@"Apple Pencil"])
            {
                _isApplePencilConnect = YES;
                break;
            }
        }
    }
}

@end
