/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "PencilToolHandler.h"
#import "PencilAnnotHandler.h"
#import "PaintUtility.h"
#import "PencilModule.h"

@interface PencilCAShapeView : UIView

@property (nonatomic, assign) int pageIndex;

@end

@implementation PencilCAShapeView

@end

@interface PencilToolHandler () <IAnnotPropertyListener,IAnnotEventListener, FSPSICallback, IGestureEventListener,IScrollViewEventListener, ILayoutEventListener>

@property (nonatomic, strong) FSInk *annot;
@property (nonatomic, strong) FSPSI *psiHandler;
@property (nonatomic, strong) FSBitmap *psiBitMap;
@property (nonatomic, assign) CGRect psiRect;
@property (nonatomic, assign) BOOL isMoving;
@property (nonatomic, assign) BOOL isZooming;
@property (nonatomic, assign) CGPoint lastPoint;
@property (nonatomic, assign) CGRect lastRect;
@property (nonatomic, assign) int currentPageIndex;

@property (nonatomic, strong) UIBezierPath *curPath;
@property (nonatomic, strong) CAShapeLayer *curlayer;
@property (nonatomic, strong) NSMutableDictionary *bezierViews;
//@property (nonatomic, strong) PencilCAShapeView *bezierView;

@property (nonatomic, strong) NSMutableArray *linesArray;
@property (nonatomic, strong) NSMutableArray *redoLinesArray;
@property (nonatomic, assign) BOOL isEndDrawingLine;

@property (nonatomic, strong) FSAnnotAttributes *attributesBeforeModify;
@property (nonatomic, assign) BOOL added;

@property (nonatomic, strong) UITouch *currentTouch;
@property (nonatomic, strong) NSString *nmStr;
@property (nonatomic, strong) UITapGestureRecognizer *curDoubleTapGesture;
@property (nonatomic, assign) CGFloat beginZoomScale;
@property (nonatomic, assign) CGFloat changeZoomScale;
@end

@implementation PencilToolHandler {
    __weak UIExtensionsManager *_extensionsManager;
    __weak FSPDFViewCtrl *_pdfViewCtrl;
    FSTaskServer *_taskServer;
    void *dibBuf;
    CGPoint _vertice[4];
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _taskServer = _extensionsManager.taskServer;
        _type = FSAnnotInk;
        
        _linesArray = [NSMutableArray array];
        _redoLinesArray = [NSMutableArray array];
        _bezierViews = @{}.mutableCopy;
        [_extensionsManager registerAnnotEventListener:self];
        [_pdfViewCtrl registerGestureEventListener:self];
        [_extensionsManager registerToolEventListener:self];
        [_pdfViewCtrl registerScrollViewEventListener:self];
        [_pdfViewCtrl registerLayoutEventListener:self];
        
        self.undoItems = [NSMutableArray<UndoItem*> array];
        self.redoItems = [NSMutableArray<UndoItem*> array];
    }
    return self;
}

- (NSString *)getName {
    return Tool_Pencil;
}

- (BOOL)isEnabled {
    return YES;
}

- (void)onActivate {
    _isEndDrawingLine = NO;
}

- (void)onDeactivate {
    _isEndDrawingLine = YES;
    self.annot = nil;
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        if (!self.annot) {
            for (UIView *bezierView in self.bezierViews.allValues) {
                if (bezierView.superview){
                    [bezierView removeFromSuperview];
                }
            }
            [self.bezierViews removeAllObjects];
        }
    });
}


- (void)onAnnotWillDelete:(FSPDFPage *)page annot:(FSAnnot *)annot{
    if ([self.annot isEqualToAnnot:annot]) {
        self.annot = nil;
    }
}

- (void)onAnnotWillFlatten:(FSPDFPage *)page annot:(FSAnnot *)annot{
    if ([self.annot isEqualToAnnot:annot]) {
        self.annot = nil;
    }
}

- (void)onAnnotAdded:(FSPDFPage *)page annot:(FSAnnot *)annot{
}

- (void)setAnnot:(FSInk *)annot{
    
    if (_annot && (!annot || [annot isEmpty])) {
        id<IAnnotHandler> annotHandler = [_extensionsManager getAnnotHandlerByAnnot:_annot];
        if ([annotHandler isKindOfClass: NSClassFromString(@"PencilAnnotHandler")]){
            ((PencilAnnotHandler *)annotHandler).drawingInkAnnot = nil;
        }
    }
    
    self.attributesBeforeModify = nil;
    self.added = NO;
    if (self.toolSettings.pencilStyle == FSPenStyleBrush) {
        if (!annot || [annot isEmpty]) {
            if (self.psiHandler) {
                self.psiHandler = nil;
                self.psiBitMap = nil;
                self.psiRect = CGRectZero;
                free(dibBuf);
                dibBuf = NULL;
            }
        }
    }
    
    if ((!annot || [annot isEmpty]) &&
        (_annot && ![_annot isEmpty])) {

        CGRect lineRect = CGRectZero;
        if (_linesArray && _linesArray.count > 0) {

            lineRect = [self getRectFromPoints:_linesArray];
            if (CGRectIsEmpty(lineRect)) {
                lineRect = CGRectMake(self.lastPoint.x - 5 > 0 ? self.lastPoint.x - 5 : 0, self.lastPoint.y - 5 > 0 ? self.lastPoint.y - 5 : 0, 10, 10);
            }
            float scale = [_pdfViewCtrl getPageViewWidth:[_annot pageIndex]] / 1000;
            float margin = self.annot.lineWidth * 3 * scale;
            lineRect = CGRectInset(lineRect, -margin, -margin);
            lineRect = [Utility getStandardRect:lineRect];

            id<IAnnotHandler> annotHandler = [_extensionsManager getAnnotHandlerByType:FSAnnotInk];
            [annotHandler addAnnot:self.annot addUndo:NO];
        }else{
            FSPDFPage *page = [_annot getPage];
            if (!page || [page isEmpty]) {
                return;
            }
            [page removeAnnot:_annot];
        }
        
        [_linesArray removeAllObjects];
        [_redoLinesArray removeAllObjects];
        [_extensionsManager addUndoItems:_undoItems redoItems:_redoItems];
        [_undoItems removeAllObjects];
        [_redoItems removeAllObjects];
        
        for (UIView *bezierView in self.bezierViews.allValues) {
            if (bezierView.superview){
                [bezierView removeFromSuperview];
            }
        }
        [self.bezierViews removeAllObjects];
        

        if (!CGRectEqualToRect(lineRect, CGRectZero)) {
            
            if (self.isZooming) {
                CGRect zoomLineRect = CGRectZero;
                zoomLineRect.origin.x = lineRect.origin.x * self.changeZoomScale;
                zoomLineRect.origin.y = lineRect.origin.y * self.changeZoomScale;
                zoomLineRect.size.width = lineRect.size.width * self.changeZoomScale;
                zoomLineRect.size.height = lineRect.size.height * self.changeZoomScale;
                lineRect = zoomLineRect;
            }
            
            [_pdfViewCtrl lockRefresh];
            FSPath *path = [self.annot getInkList];
            if ([path getPointType:([path getPointCount] -1)] == FSPathTypeMoveTo) {
                [path removePoint:([path getPointCount] -1)];
                [self.annot setInkList:path];
            }
            [self.annot resetAppearanceStream];
            [_pdfViewCtrl unlockRefresh];
            [_pdfViewCtrl refresh:lineRect pageIndex:[_annot pageIndex]];
        }
        
    }
    
    _annot = annot;
}

- (void)addAnnot:(FSInk *)annot{
    
    FSPDFPage *page = [annot getPage];
    if (!page || [page isEmpty]) {
        return;
    }
    
    id<IAnnotHandler> annotHandler = [_extensionsManager getAnnotHandlerByType:FSAnnotInk];
    FSAnnotAttributes * attributesBeforeModify = [FSAnnotAttributes attributesWithAnnot:annot];
    UndoItem *undoItem = [UndoItem itemForUndoAddAnnotWithAttributes:attributesBeforeModify page:page annotHandler:annotHandler];
    [self addUndoItem:undoItem];
    self.attributesBeforeModify = attributesBeforeModify;
    _extensionsManager.isDocModified = YES;
}

- (void)modifyAnnot:(FSInk *)annot{
    if (![annot canModify]) {
        return;
    }
    FSPDFPage *page = [annot getPage];
    if (!page || [page isEmpty]) {
        return;
    }
    id<IAnnotHandler> annotHandler = [_extensionsManager getAnnotHandlerByType:FSAnnotInk];
    annot.modifiedDate = [NSDate date];
    FSAnnotAttributes *attributesBeforeModify = [FSAnnotAttributes attributesWithAnnot:annot];
    UndoItem *undoItem = [UndoItem itemForUndoModifyAnnotWithOldAttributes:self.attributesBeforeModify newAttributes:attributesBeforeModify pdfViewCtrl:_pdfViewCtrl page:page annotHandler:annotHandler];
    [self addUndoItem:undoItem];
    self.attributesBeforeModify = attributesBeforeModify;
    _extensionsManager.isDocModified = YES;
}

// PageView Gesture+Touch
- (BOOL)onPageViewLongPress:(int)pageIndex recognizer:(UILongPressGestureRecognizer *)recognizer {
    return YES;
}

- (BOOL)onPageViewTap:(int)pageIndex recognizer:(UITapGestureRecognizer *)recognizer {
    PencilModule *pencilModule = (PencilModule *)[_extensionsManager getModuleByName:Tool_Pencil];
    if (pencilModule.isApplePencilConnect) {
        if (self.currentTouch.type == UITouchTypeStylus) {
            return YES;
        }else{
            return NO;
        }
    }
    
    return YES;
}

- (BOOL)onPageViewPan:(int)pageIndex recognizer:(UIPanGestureRecognizer *)recognizer {
    return YES;
}

- (BOOL)onPageViewShouldBegin:(int)pageIndex recognizer:(UIGestureRecognizer *)gestureRecognizer {

    if (pageIndex >= 0) {
        UITapGestureRecognizer *doubleTapGesture = [_pdfViewCtrl getPageViewDoubleTapGesture:pageIndex];
        if (doubleTapGesture && self.curDoubleTapGesture != doubleTapGesture) {
            self.curDoubleTapGesture.enabled = YES;
            doubleTapGesture.enabled = NO;
            self.curDoubleTapGesture = doubleTapGesture;
        }
    }
    
    PencilModule *pencilModule = (PencilModule *)[_extensionsManager getModuleByName:Tool_Pencil];
    if (pencilModule.isApplePencilConnect) {
        if (self.currentTouch.type == UITouchTypeDirect) {
            return NO;
        }
        if (self.currentTouch.type == UITouchTypeIndirect) {
            return NO;
        }
    }
    
    if ([gestureRecognizer isKindOfClass:[UITapGestureRecognizer class]]) {
        if (((UITapGestureRecognizer*)gestureRecognizer).numberOfTapsRequired == 2) {
            return NO;
        }
    }
    
    return YES;
}

- (FSInk *)createInkAnnotationInPage:(int)pageIndex atPos:(FSPointF *)pos {
    FSPDFPage *page = [_pdfViewCtrl.currentDoc getPage:pageIndex];
    if (!page || [page isEmpty])
        return nil;

    CGRect rect = CGRectMake(pos.x - 20, pos.y - 20, 40, 40);
    FSInk *annot = [[FSInk alloc] initWithAnnot:[page addAnnot:FSAnnotInk rect:[Utility CGRect2FSRectF:rect]]];
    annot.NM = [Utility getUUID];
    annot.author = _extensionsManager.annotAuthor;
    annot.color = self.toolSettings.settingsColor;
    annot.opacity = self.toolSettings.opacity;
    annot.lineWidth = self.toolSettings.thickness;
    annot.createDate = [NSDate date];
    annot.modifiedDate = [NSDate date];
    annot.flags = FSAnnotFlagPrint;
    annot.subject = @"Pencil";
    
    FSPath *path = [[FSPath alloc] init];
    [path moveTo:pos];
    [annot setInkList:path];
    
    if (self.toolSettings.pencilStyle == FSPenStylePen) {
        PencilCAShapeView *bezierView = self.bezierViews[@(pageIndex)];
        if (!bezierView){
            bezierView = [[PencilCAShapeView alloc] init];
            bezierView.backgroundColor = [UIColor clearColor];
            bezierView.userInteractionEnabled = NO;
            self.bezierViews[@(pageIndex)] =  bezierView;
        }else{
            [bezierView.layer.sublayers.copy enumerateObjectsUsingBlock:^(CALayer *_Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
                [obj removeFromSuperlayer];
            }];
            [bezierView removeFromSuperview];
        }

        UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
        CGRect b = pageView.bounds;
        bezierView.frame = b;
        [pageView addSubview:bezierView];
        bezierView.pageIndex = pageIndex;
    }
    
    id<IAnnotHandler> annotHandler = [_extensionsManager getAnnotHandlerByAnnot:annot];
    if ([annotHandler isKindOfClass: NSClassFromString(@"PencilAnnotHandler")]){
        ((PencilAnnotHandler *)annotHandler).drawingInkAnnot = annot;
    }
    
    return annot;
}

- (void)createPSIInPage:(int)pageIndex{
    if (self.psiHandler) {
        return;
    }
    FSPDFPage *page = [[_pdfViewCtrl currentDoc] getPage:pageIndex];
    if (!page || [page isEmpty]) {
        return;
    }
    UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
    int width = pageView.fs_width;
    int height = pageView.fs_height;
    NSUInteger size = width * height * 4;
    dibBuf = malloc(size);
    self.psiBitMap = [[FSBitmap alloc] initWithWidth:width height:height format:FSBitmapDIBArgb buffer:[NSData dataWithBytesNoCopy:(unsigned char *) dibBuf length:size freeWhenDone:NO] pitch:0];
    memset(dibBuf, 0x00, size);

    self.psiHandler = [[FSPSI alloc] initWithBitmap:self.psiBitMap simulate:YES];
    [self.psiHandler setOpacity:self.toolSettings.opacity];
    [self.psiHandler setColor:self.toolSettings.settingsColor];
    CGFloat diameter = [Utility convertWidth:self.toolSettings.thickness fromPDFToPageView:_extensionsManager.pdfViewCtrl pageIndex:pageIndex]*1.2;
    [self.psiHandler setDiameter:diameter < 1 ? 1 : diameter];
    [self.psiHandler setCallback:self];
    self.psiRect = CGRectZero;
}

- (BOOL)onPageViewTouchesBegan:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    PencilModule *pencilModule = (PencilModule *)[_extensionsManager getModuleByName:Tool_Pencil];
    UITouch *touch = [touches anyObject];
    
    if (!pencilModule.isApplePencilConnect) {
        if (touch.type == UITouchTypeStylus) {
            pencilModule.isApplePencilConnect = YES;
        }
    }
    
    if (pencilModule.isApplePencilConnect) {
        self.currentTouch = touch;
        
        if (touch.type == UITouchTypeDirect) {
            return NO;
        }
        if (touch.type == UITouchTypeIndirect) {
            return NO;
        }
    }
    
    if (self.currentPageIndex != -1 && self.currentPageIndex != pageIndex) {
        [self onDeactivate];
    }
    self.currentPageIndex = pageIndex;
    _isEndDrawingLine = NO;

    UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
    CGPoint point = [[touches anyObject] preciseLocationInView:pageView];
    if (point.x < 0 || point.x > pageView.bounds.size.width ||
        point.y < 0 || point.y > pageView.bounds.size.height) {
        return NO;
    }

    _isMoving = YES;
    
    NSMutableArray *pointArray = [NSMutableArray array];
    NSValue *pointValue = [NSValue valueWithCGPoint:point];
    [pointArray addObject:pointValue];
    NSDictionary *dic = [NSMutableDictionary dictionaryWithObjectsAndKeys: pointArray, @"pointArray", nil];
    [_linesArray addObject:dic];
    
    FSPointF *dibPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];

    if (self.annot == nil || [self.annot isEmpty]) //this is the first line
    {
        self.annot = [self createInkAnnotationInPage:pageIndex atPos:dibPoint];
    } else {
        if (pageIndex != self.annot.pageIndex) {
            return NO;
        }
        
        FSPath *path = [self.annot getInkList];
        [path moveTo:dibPoint];
        [self.annot setInkList:path];
    }
    _lastPoint = point;
    if (self.toolSettings.pencilStyle == FSPenStylePen) {
      
        FSRectF *lineFSRect = [[FSRectF alloc] initWithLeft1:0 bottom1:0 right1:self.annot.lineWidth top1:self.annot.lineWidth];
        CGRect lineRect = [_pdfViewCtrl convertPdfRectToPageViewRect:lineFSRect pageIndex:pageIndex];
        float lineWidth = lineRect.size.width;
        
        self.curPath = [UIBezierPath bezierPath];
        self.curPath.lineCapStyle = kCGLineCapRound;
        self.curPath.lineJoinStyle = kCGLineJoinRound;
        [self.curPath moveToPoint:point];
        
        CAShapeLayer *curlayer = [[CAShapeLayer alloc] init];
        curlayer = [[CAShapeLayer alloc] init];
        curlayer.zPosition = 1;
        curlayer.lineCap = @"round";
        curlayer.lineJoin = @"round";
        curlayer.backgroundColor = [UIColor clearColor].CGColor;
        curlayer.fillColor = [UIColor clearColor].CGColor;
        
        curlayer.lineWidth = lineWidth;
        curlayer.strokeColor = [UIColor colorWithRGB:self.annot.color].CGColor;
        curlayer.path = self.curPath.CGPath;
        self.curlayer = curlayer;
        PencilCAShapeView *bezierView = self.bezierViews[@(pageIndex)];
        bezierView.userInteractionEnabled = NO;
        bezierView.opaque = NO;
        bezierView.layer.opacity = self.annot.opacity;
        [bezierView.layer addSublayer:curlayer];
        
        memset(_vertice, 0, sizeof(_vertice));
        _vertice[0] = point;
        
    }else{
        [self createPSIInPage:pageIndex];
        if (CGRectEqualToRect(self.psiRect, CGRectZero)) {
            self.psiRect = CGRectMake(point.x, point.y, 0, 0);
        }else{
            self.psiRect = CGRectUnion(self.psiRect, CGRectMake(point.x, point.y, 0, 0));
        }
        FSPointF *pointf = [[FSPointF alloc] initWithX:point.x Y:point.y];
        [self.psiHandler addPoint:pointf type:FSPathTypeMoveTo pressure:1];
        self.annot.simulatePSI = YES;
    }
    
    return YES;
}

- (BOOL)onPageViewTouchesMoved:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    PencilModule *pencilModule = (PencilModule *)[_extensionsManager getModuleByName:Tool_Pencil];
    if (pencilModule.isApplePencilConnect) {
        UITouch *touch = [touches anyObject];
        if (touch.type == UITouchTypeDirect) {
            return NO;
        }
        if (touch.type == UITouchTypeIndirect) {
            return NO;
        }
    }
    
    if (_isMoving && !_isZooming && self.currentPageIndex == pageIndex) {
        UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
        CGPoint point = [[touches anyObject] preciseLocationInView:pageView];
        if (point.x < 0 || point.x > pageView.bounds.size.width ||
            point.y < 0 || point.y > pageView.bounds.size.height) {
            return NO;
        }
        if (CGPointEqualToPoint(_lastPoint, point)) {
            return YES;
        }
        __weak NSMutableArray *pointArray = [_linesArray lastObject][@"pointArray"];
        NSValue *pointValue = [NSValue valueWithCGPoint:point];
        [pointArray addObject:pointValue];
        
        if (self.annot == nil || [self.annot isEmpty]) //this is the first line
        {
            FSPointF *dibPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
            self.annot = [self createInkAnnotationInPage:pageIndex atPos:dibPoint];
            if (self.toolSettings.pencilStyle == FSPenStylePen) {
                CGRect cgRect = [_pdfViewCtrl convertPdfRectToPageViewRect:self.annot.fsrect pageIndex:pageIndex];
                [_pdfViewCtrl refresh:cgRect pageIndex:pageIndex];
            }else{
                [self createPSIInPage:pageIndex];
            }
            _lastPoint = point;
            return YES;
        } else {
            if (pageIndex != self.annot.pageIndex) {
                //annotation cannot cross page
                return NO;
            } else {
                FSPointF *dibPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
                
               
                FSPath *path = [self.annot getInkList];
                if ([path getPointCount] == 0) {
                    FSPath *path = [[FSPath alloc] init];
                    [path moveTo:dibPoint];
                    [self.annot setInkList:path];
                    return YES;
                }
                [path lineTo:dibPoint];
                [self.annot setInkList:path];

                if (self.toolSettings.pencilStyle == FSPenStylePen) {
                    if (pointArray.count == 2){
                        _vertice[1] = _vertice[0];
                        _vertice[2] = [pointArray[pointArray.count - 2] CGPointValue];
                        _vertice[3] = point;
                    }else if (pointArray.count >= 3){
                        _vertice[1] = [pointArray[pointArray.count - 3] CGPointValue];
                        _vertice[2] = [pointArray[pointArray.count - 2] CGPointValue];
                        _vertice[3] = point;
                        
                        NSArray *cp = [PaintUtility calcBesselControlPoints:_vertice rate:0.8f];
                        if (cp){
                            [self.curPath addCurveToPoint:_vertice[2] controlPoint1:[cp[0] CGPointValue] controlPoint2:[cp[1] CGPointValue]];
                            
                            PencilCAShapeView *bezierView = self.bezierViews[@(pageIndex)];
                            self.curlayer.frame = bezierView.bounds;
                            self.curlayer.position = bezierView.center;
                            self.curlayer.path = self.curPath.CGPath;
                            
                            _vertice[0] = _vertice[1];
                        }
                    }

                }else{
                    if (CGRectEqualToRect(self.psiRect, CGRectZero)) {
                        self.psiRect = CGRectMake(point.x, point.y, 0, 0);
                    }else{
                        self.psiRect = CGRectUnion(self.psiRect, CGRectMake(point.x, point.y, 0, 0));
                    }
                    FSPointF *pointf = [[FSPointF alloc] initWithX:point.x Y:point.y];
                    [self.psiHandler addPoint:pointf type:FSPathTypeLineTo pressure:1];
                    float scale = [_pdfViewCtrl getPageViewWidth:pageIndex] / 1000;
                    float margin = self.annot.lineWidth * 3 * scale;
                    CGRect refreshRect = CGRectInset(self.psiRect, -margin, -margin);
                    refreshRect = [Utility getStandardRect:refreshRect];
                    [_pdfViewCtrl refresh:refreshRect pageIndex:pageIndex needRender:NO];
                }
                
                _lastPoint = point;
                return YES;
            }
        }
    }
    return NO;
}

- (BOOL)onPageViewTouchesEnded:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    PencilModule *pencilModule = (PencilModule *)[_extensionsManager getModuleByName:Tool_Pencil];
    if (pencilModule.isApplePencilConnect) {
        UITouch *touch = [touches anyObject];
        if (touch.type == UITouchTypeDirect) {
            return NO;
        }
        
        if (touch.type == UITouchTypeIndirect) {
            return NO;
        }
    }

    _isMoving = NO;
    
    if (pageIndex != self.currentPageIndex) {
        pageIndex = self.currentPageIndex;
    }
    UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
    CGPoint point = [[touches anyObject] preciseLocationInView:pageView];
    if (point.x < 0) {
        point.x = 0;
    }else if (point.x > pageView.bounds.size.width) {
        point.x = pageView.bounds.size.width;
    }
    
    if (point.y < 0) {
        point.y = 0;
    }else if (point.y > pageView.bounds.size.height) {
        point.y = pageView.bounds.size.height;
    }


    self.lastPoint = point;
    
    FSPointF *dibPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
    FSPath *path = [self.annot getInkList];
    if ([path getPointCount] == 0) {
        FSPath *path = [[FSPath alloc] init];
        [path moveTo:dibPoint];
        [self.annot setInkList:path];
        return YES;
    }
    [path lineTo:dibPoint];
    [self.annot setInkList:path];

    NSMutableArray *pointArray = [_linesArray lastObject][@"pointArray"];
    [_linesArray lastObject][@"isEnd"] = @(YES);
    NSValue *pointValue = [NSValue valueWithCGPoint:point];
    
    if (self.toolSettings.pencilStyle != FSPenStylePen) {
        if (CGRectEqualToRect(self.psiRect, CGRectZero)) {
            self.psiRect = CGRectMake(point.x, point.y, 0, 0);
        }else{
            self.psiRect = CGRectUnion(self.psiRect, CGRectMake(point.x, point.y, 0, 0));
        }
        FSPointF *pointf = [[FSPointF alloc] initWithX:point.x Y:point.y];
        if (pointArray.count == 1) {
            //Fixed the problem that brush cannot draw dots after pageview is enlarged
            NSValue *pointValue1 = [NSValue valueWithCGPoint:CGPointMake(point.x+1, point.y)];
            [pointArray addObject:pointValue1];
            pointf = [[FSPointF alloc] initWithX:point.x+1 Y:point.y];
            [self.psiHandler addPoint:pointf type:FSPathTypeLineTo pressure:1];
            self.psiRect = CGRectUnion(self.psiRect, CGRectMake(point.x+1, point.y, 0, 0));
            
            FSPointF *pdfPt = [_pdfViewCtrl convertPageViewPtToPdfPt:CGPointMake(point.x+1, point.y) pageIndex:pageIndex];
            FSPath *path = [self.annot getInkList];
            [path lineTo:pdfPt];
            [self.annot setInkList:path];
            
            float offset = [_pdfViewCtrl getZoom] > 1.5 ? [_pdfViewCtrl getZoom]: 1.5;
            float scale = [_pdfViewCtrl getPageViewWidth:pageIndex] / 1000;
            float margin = self.annot.lineWidth * scale*0.8;
            if (offset < margin) {
                offset = margin;
            }
            NSValue *pointValue = [NSValue valueWithCGPoint:CGPointMake(point.x+offset, point.y)];
            [pointArray addObject:pointValue];
            pointf = [[FSPointF alloc] initWithX:point.x+offset Y:point.y];
            [self.psiHandler addPoint:pointf type:FSPathTypeLineToCloseFigure pressure:1];
            self.psiRect = CGRectUnion(self.psiRect, CGRectMake(point.x+offset, point.y, 0, 0));

            pdfPt = [_pdfViewCtrl convertPageViewPtToPdfPt:CGPointMake(point.x+offset, point.y) pageIndex:pageIndex];
            path = [self.annot getInkList];
            [path lineTo:pdfPt];
            [self.annot setInkList:path];
        }else{
            [pointArray addObject:pointValue];
            [self.psiHandler addPoint:pointf type:FSPathTypeLineToCloseFigure pressure:1];
        }
        
        CGRect lineRect = [self getRectFromPoints:_linesArray];
        if (CGRectIsEmpty(lineRect)) {
            lineRect = CGRectMake(point.x - 5 > 0 ? point.x - 5 : 0, point.y - 5 > 0 ? point.y - 5 : 0, 10, 10);
        }
        float scale = [_pdfViewCtrl getPageViewWidth:pageIndex] / 1000;
        float margin = self.annot.lineWidth * 3 * scale;
        lineRect = CGRectInset(lineRect, -margin, -margin);
        lineRect = [Utility getStandardRect:lineRect];
        [_pdfViewCtrl refresh:lineRect pageIndex:pageIndex needRender:NO];

    }else{
        if (pointArray.count == 1) {
            NSValue *pointValue = [NSValue valueWithCGPoint:CGPointMake(point.x+0.001f, point.y+0.001f)];
            [pointArray addObject:pointValue];
            
            [self.curPath addLineToPoint:CGPointMake(point.x + 0.001f, point.y + 0.001f)];
            
        }else{
            [pointArray addObject:pointValue];
            
            if (pointArray.count == 2){
                _vertice[1] = _vertice[0];
            }else if (pointArray.count >=3 ){
                _vertice[0] = [pointArray[pointArray.count - 3] CGPointValue];
                _vertice[1] = [pointArray[pointArray.count - 2] CGPointValue];

            }
            _vertice[2] = point;
            _vertice[3] = point;

            NSArray *cp = [PaintUtility calcBesselControlPoints:_vertice rate:0.8f];
            [self.curPath addCurveToPoint:_vertice[2] controlPoint1:[cp[0] CGPointValue] controlPoint2:[cp[1] CGPointValue]];
        }
        
        PencilCAShapeView *bezierView = self.bezierViews[@(pageIndex)];
        self.curlayer.frame = bezierView.bounds;
        self.curlayer.position = bezierView.center;
        self.curlayer.path = self.curPath.CGPath;
    }
    
    if (_linesArray.count == 1) {
        [self addAnnot:self.annot];
    }else{
        [self modifyAnnot:self.annot];
    }
    return YES;
}

- (BOOL)onPageViewTouchesCancelled:(int)pageIndex touches:(NSSet *)touches withEvent:(UIEvent *)event {
    PencilModule *pencilModule = (PencilModule *)[_extensionsManager getModuleByName:Tool_Pencil];
    if (pencilModule.isApplePencilConnect) {
        UITouch *touch = [touches anyObject];
        if (touch.type == UITouchTypeDirect) {
            return NO;
        }
        
        if (touch.type == UITouchTypeIndirect) {
            return NO;
        }
    }
    
    return NO;
}

- (CGRect)getRectFromPoints:(NSArray *)array {
    NSMutableArray *pointsXArr = [NSMutableArray new];
    NSMutableArray *pointsYArr = [NSMutableArray new];
    
    for (NSUInteger i = 0; i < _linesArray.count; i++) {
        __weak NSMutableArray *pointArray = _linesArray[i][@"pointArray"];
        
        for (NSUInteger j = 0; j < pointArray.count; j++) {
            __weak NSValue *pointValue = pointArray[j];
            CGPoint point = [pointValue CGPointValue];
            [pointsXArr addObject:@(point.x)];
            [pointsYArr addObject:@(point.y)];
        }
    }

    CGFloat xmaxValue = [[pointsXArr valueForKeyPath:@"@max.floatValue"] floatValue];
    CGFloat xminValue = [[pointsXArr valueForKeyPath:@"@min.floatValue"] floatValue];
    
    CGFloat ymaxValue = [[pointsYArr valueForKeyPath:@"@max.floatValue"] floatValue];
    CGFloat yminValue = [[pointsYArr valueForKeyPath:@"@min.floatValue"] floatValue];
    
    CGRect result = CGRectMake(xminValue, yminValue, fabs(xmaxValue-xminValue), fabs(ymaxValue-yminValue));
    result = CGRectInset(result, -20, -20);
    return result;
}

- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context {
    if (_isEndDrawingLine || _extensionsManager.currentToolHandler != self || pageIndex != self.currentPageIndex) {
        return ;
    }
    if (self.toolSettings.pencilStyle == FSPenStylePen) {
    }else{

        @try {
            float scale = [_pdfViewCtrl getPageViewWidth:pageIndex] / 1000;
            float margin = self.annot.lineWidth * 3 * scale;
            CGRect psiRect = CGRectInset(self.psiRect, -margin, -margin);
            UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
            psiRect = CGRectIntersection(psiRect, pageView.bounds);
            psiRect = [Utility getStandardRect:psiRect];
            
            if ([self.psiBitMap getWidth] != (int)pageView.fs_width || [self.psiBitMap getHeight] != (int)pageView.fs_height || CGRectEqualToRect(psiRect, CGRectZero)) {
                return;
            }

            int stride = psiRect.size.width * 4;
            int size = stride * psiRect.size.height;
            void *pBuf = malloc(size);
            memset(pBuf, 0x00, size);

            for (int i = 0; i < psiRect.size.height; i++) {
                if (dibBuf == NULL) {
                    return;
                }
                memcpy((void *) ((long) pBuf + i * stride), (void *) ((long) dibBuf + ((int) pageView.fs_width * 4) * (i + (int) psiRect.origin.y) + (int) psiRect.origin.x * 4), stride);
            }

            UIImage *img = [Utility dib2img:pBuf size:size dibWidth:psiRect.size.width dibHeight:psiRect.size.height withAlpha:YES];
            CGContextSaveGState(context);
            CGContextTranslateCTM(context, psiRect.origin.x, psiRect.origin.y);
            CGContextTranslateCTM(context, 0, psiRect.size.height);
            CGContextScaleCTM(context, 1.0, -1.0);
            CGContextTranslateCTM(context, -psiRect.origin.x, -psiRect.origin.y);
            CGContextDrawImage(context, psiRect, img.CGImage);

            CGContextRestoreGState(context);
        } @catch (NSException *exception) {
            
        } @finally {
            
        }
        
    }
}


#pragma FSPSICallback
- (void)refresh:(FSPSI *)psi rect:(FSRectF *)rectf {
}

#pragma mark - IGestureEventListener
- (BOOL)onDoubleTap:(UITapGestureRecognizer *)gestureRecognizer{
    PencilModule *pencilModule = (PencilModule *)[_extensionsManager getModuleByName:Tool_Pencil];
    if (pencilModule.isApplePencilConnect) {
        if (self.currentTouch.type == UITouchTypeStylus) {
            return YES;
        }
    }
    
    return _extensionsManager.currentToolHandler == self;
}

-(BOOL)onTap:(UITapGestureRecognizer *)gestureRecognizer
{
    PencilModule *pencilModule = (PencilModule *)[_extensionsManager getModuleByName:Tool_Pencil];
    if (pencilModule.isApplePencilConnect) {
        if (self.currentTouch.type == UITouchTypeStylus) {
            return YES;
        }
    }
    return _extensionsManager.currentToolHandler == self;
}

- (BOOL)onPageGestureRecognizer:(int)pageIndex gestureRecognizer:(UIGestureRecognizer *)gestureRecognizer shouldReceiveTouch:(UITouch *)touch
{
    self.currentTouch = touch;

    PencilModule *pencilModule = (PencilModule *)[_extensionsManager getModuleByName:Tool_Pencil];
    if (pencilModule.isApplePencilConnect) {
        if (self.currentTouch.type == UITouchTypeDirect) {
            return NO;
        }
        if (self.currentTouch.type == UITouchTypeIndirect) {
            return NO;
        }
    }
    return NO;
}

- (void)addUndoItem:(UndoItem *)undoItem {
    if (!undoItem)
        return;
    [self.undoItems addObject:undoItem];
    [self.redoItems removeAllObjects];
    [_redoLinesArray removeAllObjects];
    for (id<IFSUndoEventListener> listener in _extensionsManager.undoListeners) {
        if ([listener respondsToSelector:@selector(onUndoChanged)]) {
            [listener onUndoChanged];
        }
    }
}

- (void)undo {

    for (id<IFSUndoEventListener> listener in _extensionsManager.undoListeners) {
        if ([listener respondsToSelector:@selector(onWillUndo)]) {
            [listener onWillUndo];
        }
    }
    
    UndoItem *item = [self.undoItems objectAtIndex:self.undoItems.count - 1];
    int pageIndex = item.pageIndex;
    CGRect lineRect = [self getRectFromPoints:_linesArray];
    if (CGRectIsEmpty(lineRect)) {
        UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
        lineRect = pageView.bounds;
    }
    float scale = [_pdfViewCtrl getPageViewWidth:pageIndex] / 1000;
    float margin = self.annot.lineWidth * 3 * scale;
    lineRect = CGRectInset(lineRect, -margin, -margin);
    lineRect = [Utility getStandardRect:lineRect];
    
    if (_linesArray.count > 0) {
        
        NSMutableDictionary* removeDict = [_linesArray lastObject];
        [_linesArray removeLastObject];
        [_redoLinesArray addObject:removeDict];
        [self.redoItems addObject:item];
        [self.undoItems removeObject:item];
        
        if (self.toolSettings.pencilStyle == FSPenStylePen) {
            PencilCAShapeView *bezierView = self.bezierViews[@(pageIndex)];
            CALayer *la = [bezierView.layer.sublayers lastObject];
            [la removeFromSuperlayer];
            removeDict[@"bezierLayer"] = la;
        }else{

            if (self.psiHandler) {
                [self.psiHandler setCallback:nil];
                self.psiHandler = nil;
                self.psiBitMap = nil;
                self.psiRect = CGRectZero;
                free(dibBuf);
                dibBuf = NULL;
            }
            [self createPSIInPage:pageIndex];

            for (NSDictionary *dict in _linesArray) {

                __weak NSMutableArray *pointArray = dict[@"pointArray"];
                if (pointArray.count > 0) {
                    for (int i=0; i < pointArray.count; i++) {
                        NSValue *pointValue = pointArray[i];
                        CGPoint point = [pointValue CGPointValue];
                        FSPointF *pointf = [[FSPointF alloc] initWithX:point.x Y:point.y];

                        if (CGRectEqualToRect(self.psiRect, CGRectZero)) {
                            self.psiRect = CGRectMake(point.x, point.y, 0, 0);
                        }else{
                            self.psiRect = CGRectUnion(self.psiRect, CGRectMake(point.x, point.y, 0, 0));
                        }
                        
                        if (i == 0) {
                            [self.psiHandler addPoint:pointf type:FSPathTypeMoveTo pressure:1];
                        }else if (i == pointArray.count-1){
                            [self.psiHandler addPoint:pointf type:FSPathTypeLineToCloseFigure pressure:1];
                        }else{
                            [self.psiHandler addPoint:pointf type:FSPathTypeLineTo pressure:1];
                        }
                    }
                }
            }
        }
    }
    
    if (self.annot) {
        FSInkAttributes *inkAttributes = (FSInkAttributes *)[item getUndoOldAttributes];
        if (inkAttributes) {
            [self.annot setInkList:inkAttributes.inkList];
        }else if (_linesArray.count <= 0 && [self.annot.inkList getPointCount] > 0) {
            FSPDFPage *page = [_annot getPage];
            if (!page || [page isEmpty]) {
                return;
            }
            [page removeAnnot:_annot];
            self.annot = nil;
        }
    }
    if (self.toolSettings.pencilStyle == FSPenStylePen) {

    }else{
        [_pdfViewCtrl refresh:lineRect pageIndex:pageIndex needRender:NO];
    }
    
    for (id<IFSUndoEventListener> listener in _extensionsManager.undoListeners) {
        if ([listener respondsToSelector:@selector(onUndoChanged)]) {
            [listener onUndoChanged];
        }
    }
    
    for (id<IFSUndoEventListener> listener in _extensionsManager.undoListeners) {
        if ([listener respondsToSelector:@selector(onDidUndo)]) {
            [listener onDidUndo];
        }
    }
}

- (void)redo {
    
    for (id<IFSUndoEventListener> listener in _extensionsManager.undoListeners) {
        if ([listener respondsToSelector:@selector(onWillRedo)]) {
            [listener onWillRedo];
        }
    }

    id undoObj = [self.redoItems objectAtIndex:self.redoItems.count - 1];
    UndoItem *item = (UndoItem *)undoObj;
    int pageIndex = item.pageIndex;
    if (_redoLinesArray.count > 0) {
        NSMutableDictionary* removeDict = [_redoLinesArray lastObject];
        [_redoLinesArray removeLastObject];
        [_linesArray addObject:removeDict];
        [self.undoItems addObject:item];
        [self.redoItems removeObject:item];
        
        if (self.toolSettings.pencilStyle == FSPenStylePen) {
            PencilCAShapeView *bezierView = self.bezierViews[@(pageIndex)];
            CAShapeLayer *la = removeDict[@"bezierLayer"];
            [bezierView.layer addSublayer:la];
        }else{
            
            if (!self.psiHandler) {
                [self createPSIInPage:pageIndex];
            }
            __weak NSMutableArray *pointArray = [_linesArray lastObject][@"pointArray"];
            if (pointArray.count > 0) {
                for (int i=0; i < pointArray.count; i++) {
                    NSValue *pointValue = pointArray[i];
                    CGPoint point = [pointValue CGPointValue];
                    FSPointF *pointf = [[FSPointF alloc] initWithX:point.x Y:point.y];
                    
                    if (CGRectEqualToRect(self.psiRect, CGRectZero)) {
                        self.psiRect = CGRectMake(point.x, point.y, 0, 0);
                    }else{
                        self.psiRect = CGRectUnion(self.psiRect, CGRectMake(point.x, point.y, 0, 0));
                    }
                    
                    if (i == 0) {
                        [self.psiHandler addPoint:pointf type:FSPathTypeMoveTo pressure:1];
                    }else if (i == pointArray.count-1){
                        [self.psiHandler addPoint:pointf type:FSPathTypeLineToCloseFigure pressure:1];
                    }else{
                        [self.psiHandler addPoint:pointf type:FSPathTypeLineTo pressure:1];
                    }
                }
            }
            
        }
    }
    
    if (self.annot) {
        FSInkAttributes *inkAttributes = (FSInkAttributes *)[item getUndoAttributes];
        [self.annot setInkList:inkAttributes.inkList];
    }
    
    if (self.toolSettings.pencilStyle == FSPenStylePen) {
    }else{
        CGRect lineRect = [self getRectFromPoints:_linesArray];
        if (CGRectIsEmpty(lineRect)) {
            UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
            lineRect = pageView.bounds;
        }
        float scale = [_pdfViewCtrl getPageViewWidth:pageIndex] / 1000;
        float margin = self.annot.lineWidth * 3 * scale;
        lineRect = CGRectInset(lineRect, -margin, -margin);
        lineRect = [Utility getStandardRect:lineRect];
        [_pdfViewCtrl refresh:lineRect pageIndex:pageIndex needRender:NO];
    }

        
    for (id<IFSUndoEventListener> listener in _extensionsManager.undoListeners) {
        if ([listener respondsToSelector:@selector(onUndoChanged)]) {
            [listener onUndoChanged];
        }
    }
    for (id<IFSUndoEventListener> listener in _extensionsManager.undoListeners) {
        if ([listener respondsToSelector:@selector(onDidRedo)]) {
            [listener onDidRedo];
        }
    }
}

- (BOOL)canRedo {
    return self.redoItems.count > 0;
}


- (BOOL)canUndo {
    return self.undoItems.count > 0;
}


- (void)clearUndoRedo {
    if (self.undoItems) {
        [self.undoItems removeAllObjects];
    }
    if (self.redoItems) {
        [self.redoItems removeAllObjects];
    }
}


#pragma IAnnotPropertyListener
- (void)onToolSettingsChanged:(FSReadToolSettings *)toolSettings {
    if (toolSettings.settingsType == self.toolSettings.settingsType) {
        if (self.annot) {
            self.annot = nil;
        }
    }
}

#pragma mark IToolEventListener

- (void)onToolChanged:(NSString *)lastToolName CurrentToolName:(NSString *)toolName {
    if ([toolName isEqualToString:Tool_Pencil]) {

    }else{
        if (self.curDoubleTapGesture) {
            self.curDoubleTapGesture.enabled = YES;
            self.curDoubleTapGesture = nil;
        }
        if (self.annot) {
            _isEndDrawingLine = YES;
            self.annot = nil;
        }
        
    }
}

#pragma mark-- IScrollViewEventListener

- (void)onScrollViewWillBeginZooming:(UIScrollView *)scrollView{
    self.isEndDrawingLine = YES;
    self.beginZoomScale = [_pdfViewCtrl getZoom];
    self.isZooming = YES;
}

- (void)onScrollViewDidEndZooming:(UIScrollView *)scrollView{
    
    self.changeZoomScale = [_pdfViewCtrl getZoom] / self.beginZoomScale;
    
    if (self.annot) {
        self.annot = nil;
    }
    self.isZooming = NO;
    self.changeZoomScale = 1;
}

#pragma mark --ILayoutEventListener
- (void)onLayoutModeChanged:(PDF_LAYOUT_MODE)oldLayoutMode newLayoutMode:(PDF_LAYOUT_MODE)newLayoutMode{
    if (self.annot) {
        self.annot = nil;
    }
}

@synthesize toolSettings;

@end
