/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */
#if !TARGET_OS_MACCATALYST
#import "FSLanguageContainerModel.h"
#import <MLKitDigitalInkRecognition/MLKitDigitalInkRecognition.h>

@implementation FSLanguageContainerModel


- (instancetype)initWithAppearance:(NSString *) languageTag{
    self = [super init];
    if (self) {
        self.languageTag = languageTag;
    }
    return self;
}

- (instancetype)initWithAppearance:(NSString *) name languageTag:(NSString *) languageTag{
    self = [super init];
    if (self) {
        self.name = name;
        self.languageTag = languageTag;
    }
    return self;
}

+ (NSString *) getCurrentRegionalLanguage{
    NSString *currentLanguage = @"";
    NSString *fullLanguage = [[NSLocale preferredLanguages] objectAtIndex:0];
    NSLocale *currentLocale = [NSLocale currentLocale];
    NSString *language = [currentLocale objectForKey:NSLocaleLanguageCode];
    if([language isEqualToString:@"en"]){
        currentLanguage = MLKDigitalInkRecognitionModelIdentifierEnUs.languageTag;
    }else if([language isEqualToString:@"de"]){
        currentLanguage = MLKDigitalInkRecognitionModelIdentifierDeDe.languageTag;
    }else if([language isEqualToString:@"pt"]){
        currentLanguage = MLKDigitalInkRecognitionModelIdentifierPtBr.languageTag;
    }else if([language isEqualToString:@"it"]){
        currentLanguage = MLKDigitalInkRecognitionModelIdentifierItIt.languageTag;
    }else if([language isEqualToString:@"fr"]){
        currentLanguage = MLKDigitalInkRecognitionModelIdentifierFrFr.languageTag;
    }else if([language isEqualToString:@"ko"]){
        currentLanguage = MLKDigitalInkRecognitionModelIdentifierKo.languageTag;
    }else if([language isEqualToString:@"zh"]){
        if([fullLanguage isEqualToString:@"zh-Hans-CN"] ){
            currentLanguage = MLKDigitalInkRecognitionModelIdentifierZhHaniCn.languageTag;
        }else{
            currentLanguage = MLKDigitalInkRecognitionModelIdentifierZhHaniTw.languageTag;
        }
    }else if([language isEqualToString:@"es"]){
        currentLanguage = MLKDigitalInkRecognitionModelIdentifierEs.languageTag;
    }else if([language isEqualToString:@"ru"]){
        currentLanguage = MLKDigitalInkRecognitionModelIdentifierRu.languageTag;
    }else if([language isEqualToString:@"nl"]){
        currentLanguage = MLKDigitalInkRecognitionModelIdentifierNlNl.languageTag;
    }else if([language isEqualToString:@"ja"]){
        currentLanguage = MLKDigitalInkRecognitionModelIdentifierJa.languageTag;
    }else{
        currentLanguage = MLKDigitalInkRecognitionModelIdentifierEnUs.languageTag;
    }
    return currentLanguage;
}

+ (NSString *) getLanguagesKey:(NSString *) tag{
    NSString *language = @"";
    if([tag isEqualToString:MLKDigitalInkRecognitionModelIdentifierEnUs.languageTag]){
        language = FSLocalizedForKey(@"kOCRInkEnglish");
    }else if([tag isEqualToString:MLKDigitalInkRecognitionModelIdentifierDeDe.languageTag]){
        language = FSLocalizedForKey(@"kOCRInkGerman");
    }else if([tag isEqualToString:MLKDigitalInkRecognitionModelIdentifierPtBr.languageTag]){
        language = FSLocalizedForKey(@"kOCRInkPortuguese");
    }else if([tag isEqualToString:MLKDigitalInkRecognitionModelIdentifierItIt.languageTag]){
        language = FSLocalizedForKey(@"kOCRInkItalian");
    }else if([tag isEqualToString:MLKDigitalInkRecognitionModelIdentifierFrFr.languageTag]){
        language = FSLocalizedForKey(@"kOCRInkFrench");
    }else if([tag isEqualToString:MLKDigitalInkRecognitionModelIdentifierKo.languageTag]){
        language = FSLocalizedForKey(@"kOCRInkKorean");
    }else if([tag isEqualToString:MLKDigitalInkRecognitionModelIdentifierZhHaniTw.languageTag]){
        language = FSLocalizedForKey(@"kOCRInkTranditionalChinese");
    }else if([tag isEqualToString:MLKDigitalInkRecognitionModelIdentifierZhHaniCn.languageTag]){
        language = FSLocalizedForKey(@"kOCRInkSimplifiedChinese");
    }else if([tag isEqualToString:MLKDigitalInkRecognitionModelIdentifierEs.languageTag]){
        language = FSLocalizedForKey(@"kOCRInkSpanish");
    }else if([tag isEqualToString:MLKDigitalInkRecognitionModelIdentifierRu.languageTag]){
        language = FSLocalizedForKey(@"kOCRInkRussian");
    }else if([tag isEqualToString:MLKDigitalInkRecognitionModelIdentifierNlNl.languageTag]){
        language = FSLocalizedForKey(@"kOCRInkDutch");
    }else if([tag isEqualToString:MLKDigitalInkRecognitionModelIdentifierJa.languageTag]){
        language = FSLocalizedForKey(@"kOCRInkJapanese");
    }else{
        language = FSLocalizedForKey(@"kOCRInkEnglish");
    }
    return language;
}
@end

#endif
