/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */
#if !TARGET_OS_MACCATALYST
#import "FSPencilLanguagesViewController.h"
#import "FSCloudSyncStatusView.h"

#import <MLKitDigitalInkRecognition/MLKitDigitalInkRecognition.h>

@interface FSPencilLanguagesViewController () <StrokeManagerDelegate>
@property (nonatomic, strong) StrokeManager *strokeManager;
@property (nonatomic, assign) NSInteger languageCount;
@property (nonatomic, assign) NSInteger downPosition;
@property (nonatomic, assign) NSInteger i;
@property (nonatomic, strong) NSMutableArray <FSLanguageContainerModel *> *languagesList;

@end

@implementation FSPencilLanguagesViewController


- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    if (self = [super init]) {
        self.extensionsManager = extensionsManager;
    }
    return self;
}
- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager index:(NSInteger)index {
    if (self = [super init]) {
        self.extensionsManager = extensionsManager;
        self.strokeManager = [[StrokeManager alloc] initWithDelegate:self];
        self.index = index;
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    self.tableView.backgroundColor = self.fs_backgroundColor_B9;
    self.title = FSLocalizedForKey(@"kOCRInkCurrentLanguages");
    self.navBar.hasDivide = YES;
    @weakify(self)
    self.navBar.leftItem.onTapClick = ^(FSItem *_item) {
        @strongify(self)
        [self cancelBack];
    };

    [self loadData];

}


- (void)loadData{
    self.i = 0;
    self.languageCount = 12;
    self.languagesList = [[NSMutableArray alloc] init];
    for (MLKDigitalInkRecognitionModelIdentifier *modelIdentifier in MLKDigitalInkRecognitionModelIdentifier.allModelIdentifiers) {
        if([MLKDigitalInkRecognitionModelIdentifierEnUs.languageTag isEqualToString:modelIdentifier.languageTag]
           || [MLKDigitalInkRecognitionModelIdentifierDeDe.languageTag isEqualToString:modelIdentifier.languageTag]
           || [MLKDigitalInkRecognitionModelIdentifierPtBr.languageTag isEqualToString:modelIdentifier.languageTag]
           || [MLKDigitalInkRecognitionModelIdentifierItIt.languageTag isEqualToString:modelIdentifier.languageTag]
           || [MLKDigitalInkRecognitionModelIdentifierFrFr.languageTag isEqualToString:modelIdentifier.languageTag]
           || [MLKDigitalInkRecognitionModelIdentifierKo.languageTag isEqualToString:modelIdentifier.languageTag]
           || [MLKDigitalInkRecognitionModelIdentifierZhHaniCn.languageTag isEqualToString:modelIdentifier.languageTag]
           || [MLKDigitalInkRecognitionModelIdentifierZhHaniTw.languageTag isEqualToString:modelIdentifier.languageTag]
           || [MLKDigitalInkRecognitionModelIdentifierEs.languageTag isEqualToString:modelIdentifier.languageTag]
           || [MLKDigitalInkRecognitionModelIdentifierRu.languageTag isEqualToString:modelIdentifier.languageTag]
           || [MLKDigitalInkRecognitionModelIdentifierNlNl.languageTag isEqualToString:modelIdentifier.languageTag]
           || [MLKDigitalInkRecognitionModelIdentifierJa.languageTag isEqualToString:modelIdentifier.languageTag]){
            FSLanguageContainerModel * model = [[FSLanguageContainerModel alloc] initWithAppearance:[FSLanguageContainerModel getLanguagesKey:modelIdentifier.languageTag] languageTag:modelIdentifier.languageTag];
            model.select = NO;
            [self.languagesList addObject:model];
            if ([self.strokeManager isLanguageDownloaded:model.languageTag]) {
                model.downloadStatus = DOWNLOADED;
            } else {
                model.downloadStatus = DOWNLOAD;
            }
            [self addData];
            
        }
        
    }
}
- (void) addData{
    self.i++;
    if (self.i == self.languageCount) {
        NSMutableArray<FSLanguageContainerModel*> *array = [[NSMutableArray alloc] init];
        FSLanguageContainerModel * model = [[FSLanguageContainerModel alloc] initWithAppearance:MLKDigitalInkRecognitionModelIdentifierZhHaniCn.languageTag];
        FSLanguageContainerModel * model1 = [[FSLanguageContainerModel alloc] initWithAppearance:MLKDigitalInkRecognitionModelIdentifierZhHaniTw.languageTag];
        FSLanguageContainerModel * model2 = [[FSLanguageContainerModel alloc] initWithAppearance:MLKDigitalInkRecognitionModelIdentifierNlNl.languageTag];
        FSLanguageContainerModel * model3 = [[FSLanguageContainerModel alloc] initWithAppearance:MLKDigitalInkRecognitionModelIdentifierEnUs.languageTag];
        FSLanguageContainerModel * model4 = [[FSLanguageContainerModel alloc] initWithAppearance:MLKDigitalInkRecognitionModelIdentifierFrFr.languageTag];
        FSLanguageContainerModel * model5 = [[FSLanguageContainerModel alloc] initWithAppearance:MLKDigitalInkRecognitionModelIdentifierDeDe.languageTag];
        FSLanguageContainerModel * model6 = [[FSLanguageContainerModel alloc] initWithAppearance:MLKDigitalInkRecognitionModelIdentifierItIt.languageTag];
        FSLanguageContainerModel * model7 = [[FSLanguageContainerModel alloc] initWithAppearance:MLKDigitalInkRecognitionModelIdentifierJa.languageTag];
        FSLanguageContainerModel * model8 = [[FSLanguageContainerModel alloc] initWithAppearance:MLKDigitalInkRecognitionModelIdentifierKo.languageTag];
        FSLanguageContainerModel * model9 = [[FSLanguageContainerModel alloc] initWithAppearance:MLKDigitalInkRecognitionModelIdentifierPtBr.languageTag];
        FSLanguageContainerModel * model10 = [[FSLanguageContainerModel alloc] initWithAppearance:MLKDigitalInkRecognitionModelIdentifierRu.languageTag];
        FSLanguageContainerModel * model11 = [[FSLanguageContainerModel alloc] initWithAppearance:MLKDigitalInkRecognitionModelIdentifierEs.languageTag];
        [array addObject:model];
        [array addObject:model1];
        [array addObject:model2];
        [array addObject:model3];
        [array addObject:model4];
        [array addObject:model5];
        [array addObject:model6];
        [array addObject:model7];
        [array addObject:model8];
        [array addObject:model9];
        [array addObject:model10];
        [array addObject:model11];
        NSMutableArray<FSLanguageContainerModel*> *tempArray = [[NSMutableArray alloc] init];
        for (int i = 0 ;i < array.count;i++){
            for (int j = 0 ;j < array.count;j++){
                FSLanguageContainerModel * obj1 = array[i];
                FSLanguageContainerModel * obj2 = self.languagesList[j];
                if ([obj1.languageTag isEqualToString:obj2.languageTag]){
                    [tempArray addObject:obj2];
                }
            }
        }
        [self.languagesList removeAllObjects];
        [self.languagesList addObjectsFromArray:tempArray];
        
        NSString *language = [FSLanguageContainerModel getCurrentRegionalLanguage];
        int index = 0;
        FSLanguageContainerModel *temp = nil;
        for (int i = 0 ;i < self.languagesList.count;i++){
            if ([self.languagesList[i].languageTag isEqualToString:language] && i != 0){
                index = i;
                if(index != 0){
                    temp = self.languagesList[index];
                    [self.languagesList removeObjectAtIndex:index];
                }
            }
        }
        if (index != 0 && temp != nil){
            [self.languagesList insertObject:temp atIndex:0];
        }
        NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
        NSString *inkLanguages = [defaults stringForKey:@"InkLanguages"];
        if (inkLanguages.length != 0){
            for (int i = 0 ;i < self.languagesList.count;i++){
                if ([self.languagesList[i].languageTag isEqualToString:inkLanguages]){
                    self.index = i;
                }
            }
        }
        self.languagesList[self.index].select = YES;
        self.dataSource = self.languagesList;
    }
}

-(void)cancelBack{
    @weakify(self)
    [self dismissViewControllerAnimated:NO completion:^{
        @strongify(self)
        if (self.delegate) {
            [self.delegate languagesCallBack:self.index model:self.languagesList[self.index]];
        }
    }];
}

- (void)downloadModel:(FSItem *)fsItem indexPath:(NSIndexPath *) indexPath model:(FSLanguageContainerModel *)item{
    item.downloadStatus = DOWNLOADING;
    [self.strokeManager downloadModel:item.languageTag];
    [self reloadRowsAtIndexPaths:indexPath];
    
}

- (void)deleteModel:(FSItem *)fsItem indexPath:(NSIndexPath *) indexPath model:(FSLanguageContainerModel *)item{
    item.downloadStatus = DOWNLOAD;
    [self.strokeManager deleteModel:item.languageTag];
    [self reloadRowsAtIndexPaths:indexPath];
    
}

#pragma mark -  table view delegate handler

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView{
    return 1;
}

- (FSLanguageContainerModel *)dataForAtIndexPath:(NSIndexPath *)indexPath{
    return self.languagesList[indexPath.row];
}

- (void)configureCell:(FSTitleEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(FSLanguageContainerModel *)dataItem{
    cell.backgroundColor = self.fs_backgroundColor_B8;
    
    cell.estimatedTitleLabel.text = dataItem.name;
    if (!cell.rightView) {
        cell.rightView = [[UIView alloc] init];
        FSItem* lbDownLoad = [FSItem createItemWithTitle:FSLocalizedForKey(@"kOCRInkDownload")];
        lbDownLoad.accessibilityTraits = UIAccessibilityTraitHeader;
        lbDownLoad.customAccessibilityTraits = UIAccessibilityTraitHeader;
        [lbDownLoad setFont:self.fs_standardGeneralFont_GF2];
        [lbDownLoad setTitleColor:self.fs_primaryColor forState:UIControlStateNormal];
        lbDownLoad.accessibilityIdentifier = @"lbDownLoad_item_id";
        lbDownLoad.tag = 1;
        [cell.rightView addSubview:lbDownLoad];
        [lbDownLoad mas_remakeConstraints:^(MASConstraintMaker *make) {
//            make.size.mas_equalTo(CGSizeMake(120, 40));
            make.right.mas_equalTo(lbDownLoad.superview.mas_right).offset(-12);
//            make.left.mas_equalTo(lbDownLoad.superview.mas_left);
            make.top.mas_equalTo(lbDownLoad.superview.mas_top);
            make.bottom.mas_equalTo(lbDownLoad.superview.mas_bottom);
         
        }];
        
        FSItem* selectItem = [FSItem createItemWithImage:FSImageNamed(@"common_select_check")];
        [BindThemeKeySel(selectItem, setImage:forState:) fs_updateThemeCompleted:^(FSItem*  _Nonnull target, SEL  _Nonnull sel) {
            [target setImage:[FSImageNamed(@"common_select_check") fs_imageWithTintColor:target.fs_primaryColor] forState:UIControlStateNormal];
        }];
        selectItem.accessibilityIdentifier = @"select_item_id";
        selectItem.tag = 2;
        [cell.rightView addSubview:selectItem];
        [selectItem mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.right.mas_equalTo(selectItem.superview.mas_right);
//            make.left.mas_equalTo(selectItem.superview.mas_left);
            make.top.mas_equalTo(selectItem.superview.mas_top);
            make.bottom.mas_equalTo(selectItem.superview.mas_bottom);
//            make.size.mas_equalTo(CGSizeMake(40, 40));
        }];
       
        FSItem *deleteItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"common_delete_gray")];
        
        deleteItem.accessibilityIdentifier = @"delete_item_id";
        deleteItem.tag = 3;
        [cell.rightView addSubview:deleteItem];
        [deleteItem mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.right.mas_equalTo(deleteItem.superview.mas_right);
//            make.left.mas_equalTo(deleteItem.superview.mas_left);
            make.top.mas_equalTo(deleteItem.superview.mas_top);
            make.bottom.mas_equalTo(deleteItem.superview.mas_bottom);
//            make.size.mas_equalTo(CGSizeMake(40, 40));

        }];
        FSCloudSyncStatusView *stateView = [[FSCloudSyncStatusView alloc] initWithFrame:CGRectMake(0, 0, 40, 40)];
        stateView.backgroundColor = [UIColor clearColor];
        stateView.tag = 4;
        [cell.rightView addSubview:stateView];
        [stateView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.right.mas_equalTo(stateView.superview.mas_right).offset(-12);
//            make.left.mas_equalTo(downLoadItem.superview.mas_left);
            make.top.mas_equalTo(stateView.superview.mas_top);
            make.bottom.mas_equalTo(stateView.superview.mas_bottom);
//            make.size.mas_equalTo(CGSizeMake(40, 40));

        }];
       
        [ cell.estimatedTitleLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.right.mas_equalTo(cell.rightView.superview.mas_left);
            make.left.mas_equalTo(cell.estimatedTitleLabel.superview.mas_left);
            make.top.mas_equalTo( cell.estimatedTitleLabel.superview.mas_top);
            make.bottom.mas_equalTo( cell.estimatedTitleLabel.superview.mas_bottom);
//            make.size.mas_equalTo(CGSizeMake(40, 40));

        }];
    }
    
    FSItem *item = cell.rightView.subviews[0];
    @weakify(self)
    item.onTapClick = ^(FSItem *_item) {
        @strongify(self)
        [self downloadModel:_item indexPath:indexPath model:dataItem];
    };
    FSItem *item2 = cell.rightView.subviews[2];
    item2.onTapClick = ^(FSItem *_item) {
        @strongify(self)
        [self deleteModel:_item indexPath:indexPath model:dataItem];
    };
    if(dataItem.downloadStatus == DOWNLOAD){
        cell.rightView.subviews[0].hidden = NO;
        cell.rightView.subviews[1].hidden = YES;
        cell.rightView.subviews[2].hidden = YES;
        cell.rightView.subviews[3].hidden = YES;

    }else if(dataItem.downloadStatus == DOWNLOADED){
        cell.rightView.subviews[0].hidden = YES;
        cell.rightView.subviews[3].hidden = YES;
        if(dataItem.select){
            cell.rightView.subviews[1].hidden = NO;
            cell.rightView.subviews[2].hidden = YES;
        }else{
            cell.rightView.subviews[1].hidden = YES;
            cell.rightView.subviews[2].hidden = NO;
        }
    }else{
        cell.rightView.subviews[0].hidden = YES;
        cell.rightView.subviews[1].hidden = YES;
        cell.rightView.subviews[2].hidden = YES;
        cell.rightView.subviews[3].hidden = NO;
        FSCloudSyncStatusView *stateView  = cell.rightView.subviews[3];
        stateView.statusType = FSCloudSyncFileStatusTypeSyncing;
       
    }
    [cell.contentView layoutIfNeeded];
  
}


- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{
    [super tableView:tableView didSelectRowAtIndexPath:indexPath];
    if(self.languagesList[indexPath.row].downloadStatus != DOWNLOADED){
        return;
    }
  
    [self.strokeManager selectLanguage:self.languagesList[indexPath.row].languageTag];
    NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
    [defaults setObject:self.languagesList[indexPath.row].languageTag forKey:@"InkLanguages"];
    
    
    self.index = indexPath.row;
    for (int i = 0 ;i < self.languagesList.count;i++){
        if (i == self.index){
            self.languagesList[i].select = YES;
        }else{
            if (self.languagesList[i].select == YES){
                self.languagesList[i].select = NO;
            }
        }
    }
    [self reloadData];
}

#pragma mark - StrokeManagerDelegate
- (void)displayMessage:(NSString *)message {

}

- (void)downModelSuccess:(NSString *)tag{
    for(int i = 0;i < self.languagesList.count; i++){
        FSLanguageContainerModel *model = self.languagesList[i];
        if([model.languageTag isEqualToString:tag] && model.downloadStatus == DOWNLOADING){
            model.downloadStatus = DOWNLOADED;
            NSIndexPath *indexPath = [NSIndexPath indexPathForRow:i inSection:0];
            [self reloadRowsAtIndexPaths:indexPath];
            
        }
    }
}

- (void)inkSuccess:(NSString *)text{
    
}


@end

#endif
