/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */
#if !TARGET_OS_MACCATALYST
#import "FSPencilLanguagesViewController.h"
#import "FSPencilRecognizeTextViewController.h"
#import "StrokeManager.h"
#import "FSLanguageContainerModel.h"
#import <MLKitDigitalInkRecognition/MLKitDigitalInkRecognition.h>

@interface FSPencilRecognizeTextViewController ()<FSPencilLanguagesDelegate, StrokeManagerDelegate>
@property (nonatomic, assign) NSInteger index;
@property (nonatomic, strong) NSString *inkLanguages;
@property (nonatomic, strong) FSLanguageContainerModel *item;
@property (nonatomic, strong) StrokeManager *strokeManager;
@property (nonatomic, strong) UILabel *lbContent;
@property (nonatomic, strong) UITextView *textInk;
@property (nonatomic, assign) BOOL first;
@property (nonatomic, assign) int keyboardHeight;
@property (nonatomic, strong) NSMutableArray <FSAnnot *> *annots;
@end

@implementation FSPencilRecognizeTextViewController

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    if (self = [super init]) {
        self.extensionsManager = extensionsManager;
    }
    return self;
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager annots:(NSMutableArray <FSAnnot *>*) annots{
    if (self = [super init]) {
        self.extensionsManager = extensionsManager;
        self.annots = annots.mutableCopy;
        self.strokeManager = [[StrokeManager alloc] initWithDelegate:self];

    }
    return self;
}

- (UIInterfaceOrientationMask)supportedInterfaceOrientations {
    return UIInterfaceOrientationMaskPortrait;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    [self initView];
    [self loadData];
 
}
- (void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];


    [[IQKeyboardManager sharedManager] setEnable:NO];

    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardWillShow:) name:UIKeyboardWillShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardWillHide:) name:UIKeyboardWillHideNotification object:nil];

}
- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    [[NSNotificationCenter defaultCenter] removeObserver:self];
   
}

- (void)initView{
    self.navBarLeftTitle = FSLocalizedForKey(@"kCancel");
    self.navBarRightTitle = FSLocalizedForKey(@"kOK");
    self.title = FSLocalizedForKey(@"kOCRInkRecognizeText");
    self.navBar.hasDivide = NO;
    self.keyboardHeight = 0;
    @weakify(self)
    self.navBar.leftItem.onTapClick = ^(FSItem *_item) {
        @strongify(self)
        [self cancel];
    };
    
    self.navBar.rightItem.onTapClick = ^(FSItem *_item) {
        @strongify(self)
        [self dismissViewControllerAnimated:NO completion:^{
            [self.delegate languagesCallBack:self.annots text:self.textInk.text];
        }];
       
    };
    
    UIView *lbView = [[UIView alloc] init];
    [self.view addSubview:lbView];
    [lbView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.view.mas_left);
        make.right.mas_equalTo(self.view.mas_right);
        make.top.mas_equalTo(self.navBar.mas_bottom);
        make.height.mas_equalTo(32);
    }];
    lbView.backgroundColor = self.fs_backgroundColor_B9;
    UILabel *lbLanguage = [[UILabel alloc] init];
    [lbView addSubview:lbLanguage];
    [lbLanguage mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.view.mas_left).offset(16);
        make.top.mas_equalTo(lbView.mas_top).offset(4);
        make.bottom.mas_equalTo(lbView.mas_bottom);
    }];
    lbLanguage.text = FSLocalizedForKey(@"kOCRInkLanguages");
    lbLanguage.font = self.fs_standardWeakFont_WF1;
    lbLanguage.textAlignment = NSTextAlignmentCenter;
    lbLanguage.textColor = self.fs_textColor_T3;
    
    
    UIView *languaesView = [[UIView alloc] init];
    [self.view addSubview:languaesView];
    [languaesView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.view.mas_left);
        make.right.mas_equalTo(self.view.mas_right);
        make.top.mas_equalTo(lbView.mas_bottom);
        make.height.mas_equalTo(44);
    }];
    languaesView.backgroundColor = self.fs_backgroundColor_B8;
    UITapGestureRecognizer *gesture = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(tapGesture:)];
    [languaesView addGestureRecognizer:gesture];
    
    UILabel *lbLanguageText = [[UILabel alloc] init];
    [languaesView addSubview:lbLanguageText];
    [lbLanguageText mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.view.mas_left).offset(16);
        make.centerY.equalTo(lbLanguageText.superview);
    }];
    lbLanguageText.text = FSLocalizedForKey(@"kOCRInkCurrentLanguages");
    lbLanguageText.font = self.fs_standardGeneralFont_GF2;
    lbLanguageText.textAlignment = NSTextAlignmentCenter;
    lbLanguageText.textColor = self.fs_textColor_T4;
  
    UIImageView *ivRight = [[UIImageView alloc] initWithImage:FSImageNamedAndNormalColor(@"common_accessory_gray")];
    [languaesView addSubview:ivRight];
    [ivRight mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(ivRight.superview.fs_mas_right).offset(-4);
        make.centerY.equalTo(ivRight.superview);
    }];


    self.lbContent = [[UILabel alloc] init];
    [languaesView addSubview:self.lbContent];
    [self.lbContent mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(ivRight.mas_left);
        make.centerY.equalTo(self.lbContent.superview);
    }];
    self.lbContent.text = FSLocalizedForKey(@"kOCRInkCurrentLanguages");
    self.lbContent.font = self.fs_standardGeneralFont_GF2;
    self.lbContent.textAlignment = NSTextAlignmentCenter;
    self.lbContent.textColor = self.fs_textColor_T13;
    self.lbContent.accessibilityIdentifier = @"current_languages_id";

    UIView *lbTextView = [[UIView alloc] init];
    [self.view addSubview:lbTextView];
    [lbTextView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.view.mas_left);
        make.right.mas_equalTo(self.view.mas_right);
        make.top.mas_equalTo(languaesView.mas_bottom);
        make.height.mas_equalTo(32);
    }];
    lbTextView.backgroundColor = self.fs_backgroundColor_B9;
    UILabel *lbText = [[UILabel alloc] init];
    [lbTextView addSubview:lbText];
    [lbText mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.view.mas_left).offset(16);
        make.top.mas_equalTo(lbTextView.mas_top).offset(4);
        make.bottom.mas_equalTo(lbTextView.mas_bottom);
    }];
    lbText.text = FSLocalizedForKey(@"kOCRInkText");
    lbText.font = self.fs_standardWeakFont_WF1;
    lbText.textAlignment = NSTextAlignmentCenter;
    lbText.textColor = self.fs_textColor_T3;
    
    UIView *inkView = [[UIView alloc] init];
    [self.view addSubview:inkView];
    [inkView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.view.mas_left);
        make.right.mas_equalTo(self.view.mas_right);
        make.top.mas_equalTo(lbTextView.mas_bottom);
        make.bottom.mas_equalTo(self.view.mas_bottom);

    }];
    inkView.backgroundColor = self.fs_backgroundColor_B8;

    self.textInk = [[UITextView alloc] init];
    self.textInk.font = self.fs_standardGeneralFont_GF2;
    self.textInk.textColor = self.fs_textColor_T4;
//    self.textInk.borderStyle = UITextBorderStyleNone;
    self.textInk.textAlignment = NSTextAlignmentCenter;
    self.textInk.accessibilityIdentifier = @"ink_recognize_tv_id";
    self.textInk.backgroundColor = self.fs_backgroundColor_B8;
    [inkView addSubview:self.textInk];
    [self.textInk mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.textInk.superview.mas_left).offset(16);
        make.right.mas_equalTo(self.textInk.superview.mas_right).offset(-16);
        make.top.mas_equalTo(self.textInk.superview.mas_top).offset(16);
        make.bottom.mas_equalTo(self.textInk.superview.mas_bottom).offset(-16);
    }];
}
- (void)tapGesture:(UITapGestureRecognizer *)gesture {
    [self showLanguagesViewController];
}
- (void)loadData{
    NSString *languages = [FSLanguageContainerModel getCurrentRegionalLanguage];
    NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
    NSString *inkLanguages = [defaults stringForKey:@"InkLanguages"];
    if(inkLanguages.length != 0){
        languages = inkLanguages;
    }
    self.lbContent.text = [FSLanguageContainerModel getLanguagesKey:languages];
    for (MLKDigitalInkRecognitionModelIdentifier *modelIdentifier in MLKDigitalInkRecognitionModelIdentifier.allModelIdentifiers) {
        if([languages isEqualToString:modelIdentifier.languageTag]){
            [self.strokeManager selectLanguage:languages];
            BOOL isLanguageDown = [self.strokeManager isLanguageDownloaded:languages];
            [self.view fs_showHUDLoading:@""];
            if(isLanguageDown){
                [self recognitionInk];
            }else{
                self.first = YES;
              [self.strokeManager downloadModel];
            }
        }
    }
}

- (void)recognitionInk{
    [self.strokeManager clear];
    NSMutableArray<NSMutableArray<NSMutableArray<FSPointF *> *> * > * inkListss = [[NSMutableArray alloc] init];
    int index = 0;
    for (FSAnnot *annot in self.annots.copy) {
        if (annot.type != FSAnnotInk) {
            [self.annots removeObject:annot];
            continue;
        }
        FSInk *ink = [[FSInk alloc] initWithAnnot:annot];
        NSMutableArray<NSMutableArray<FSPointF *> *> *inkLists = [self getAllPointFromInkAnnot:ink];
        if(inkLists != nil){
            [inkListss addObject:inkLists];
        }
        index = annot.pageIndex;
    }
    if(inkListss.count > 0){
        [self.strokeManager addAtPoint:[self.extensionsManager pdfViewCtrl] annots:inkListss index:index];
        [self.strokeManager recognizeInk];
    }
}


- (void)cancel{
    [self dismissViewControllerAnimated:NO completion:^{}];
}

- (void)showLanguagesViewController{
    FSPencilLanguagesViewController *viewController = [[FSPencilLanguagesViewController alloc] initWithUIExtensionsManager:self->_extensionsManager index:self.index];
    [viewController fs_setCustomTransitionAnimator:_extensionsManager.presentedVCAnimator];
    [self presentViewController:viewController animated:YES completion:nil];
    viewController.delegate = self;
    viewController.index = self.index;
    NSUserDefaults* defaults = [NSUserDefaults standardUserDefaults];
    self.inkLanguages = [defaults stringForKey:@"InkLanguages"];
}

- (NSMutableArray<NSMutableArray<FSPointF *> *> *)getAllPointFromInkAnnot:(FSInk *)inkAnnot {
    NSMutableArray<NSMutableArray<FSPointF *> *> *arrayss =  [NSMutableArray<NSMutableArray<FSPointF *> *>  array] ;
    NSMutableArray<FSPointF *> *arrayPoints =  [NSMutableArray<FSPointF *>  array];
    FSPath *path = [inkAnnot getInkList];
    int ptCount = [path getPointCount];
    for (int i = 0; i < ptCount; i++) {
        if([path getPointType:i] == FSPathTypeMoveTo){
            arrayPoints = [NSMutableArray<FSPointF *> array];
        }
        if(arrayPoints != nil){
            FSPointF *pt = [path getPoint:i];
            if(arrayss.count == 0 && i == 1){
                FSPointF *pt2 = [path getPoint:i-1];
                if(pt.x == pt2.x+1 && pt.y == pt2.y){
                    continue;
                }
            }
            [arrayPoints addObject:pt];
        }
        if(i == ptCount - 1 || ((i + 1) < ptCount && [path getPointType:i+1] == FSPathTypeMoveTo)){
            [arrayss addObject:arrayPoints];
        }
    }
    return arrayss;
}

#pragma mark - FSPencilLanguagesDelegate
- (void)languagesCallBack:(NSInteger)index model:(FSLanguageContainerModel *)item{
    self.index = index;
    [self.strokeManager selectLanguage:item.languageTag];
    NSString *language = @"";
    if(self.item != nil){
        language = [self.item languageTag];
    }else{
        if(self.inkLanguages.length != 0){
            language = self.inkLanguages;
        }else{
            language = [FSLanguageContainerModel getCurrentRegionalLanguage];
        }
    }
    if(![language isEqualToString:item.languageTag]){
        [self.view fs_showHUDLoading:@""];
        [self recognitionInk];
    }
    self.item = item;
    [self.lbContent setText:[FSLanguageContainerModel getLanguagesKey:[item languageTag]]];
    self.inkLanguages = @"";
}


#pragma mark - StrokeManagerDelegate

- (void)displayMessage:(NSString *)message {
    if ([message isEqualToString:@"Recognition error"]){
        [self fs_toastWithMsg:FSLocalizedForKey(@"kOCRInkRecognitionFailed") inView:self.view];
        
    }
}

- (void)downModelSuccess:(NSString *)tag{
    if(self.first){
        [self recognitionInk];
        self.first = NO;
    }
}

- (void)inkSuccess:(NSString *)text{
    self.textInk.text = text;
    [self.view fs_hideHUDLoading];
}


- (void)adjustTextViewFrame{
    if(self.keyboardHeight == 0){
        [self.textInk mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(self.textInk.superview.mas_left).offset(16);
            make.right.mas_equalTo(self.textInk.superview.mas_right).offset(-16);
            make.top.mas_equalTo(self.textInk.superview.mas_top).offset(16);
            make.bottom.mas_equalTo(self.textInk.superview.mas_bottom).offset(-16);
        }];
    }else{
        float inkHeight= self.textInk.frame.size.height - self.keyboardHeight - 16;
        [self.textInk mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(self.textInk.superview.mas_left).offset(16);
            make.right.mas_equalTo(self.textInk.superview.mas_right).offset(-16);
            make.top.mas_equalTo(self.textInk.superview.mas_top).offset(16);
            make.height.mas_equalTo(inkHeight);
        }];
    }
}

#pragma mark - UIKeyboad Notification methods
-(void)keyboardWillShow:(NSNotification*)notification{
    [[IQKeyboardManager sharedManager] setEnable:YES];
    NSDictionary *userInfo = [notification userInfo];
    NSValue *keyboardFrameValue = [userInfo objectForKey:UIKeyboardFrameEndUserInfoKey];
    CGRect keyboardFrame = [keyboardFrameValue CGRectValue];
    self.keyboardHeight = keyboardFrame.size.height;
    [UIView animateWithDuration:ANIMATE_DURATION_NORMAL animations:^{
        [self adjustTextViewFrame];
    } completion:^(BOOL finished) {
            
    }];

}
- (void)keyboardWillHide:(NSNotification*)notification{
    [[IQKeyboardManager sharedManager] setEnable:NO];
    self.keyboardHeight = 0;
    [UIView animateWithDuration:ANIMATE_DURATION_NORMAL animations:^{
        [self adjustTextViewFrame];
    } completion:^(BOOL finished) {

    }];
}

@end

#endif
