/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */
#if !TARGET_OS_MACCATALYST
#import <Foundation/Foundation.h>


@class MLKInk;
/**
 * Stores a piece of ink that has been sent to the recognizer, along with the recognition result
 * when it is returned. An array of these is used to represent the ink and results on the screen.
 */
@interface RecognizedInk : NSObject

/** Ink, displayed to the user. */
@property(nonatomic) MLKInk * _Nullable ink;
/** Top recognition result candidate for the ink, displayed to the user. */
@property(copy, nullable, nonatomic) NSString *text;

/**
 * Creates a `RecognizedInk` with the given `Ink` once it is sent for recognition. Text is initially
 * empty.
 * @param ink The ink to be stored in the `RecognizedInk` object.
 * @return the initialized `RecognizedInk` object.
 */
- (nullable instancetype)initWithInk:(MLKInk *_Nullable)ink NS_DESIGNATED_INITIALIZER;

- (nullable instancetype)init NS_UNAVAILABLE;

@end
#endif
