/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "PolygonModule.h"
#import "PolygonAnnotHandler.h"
#import "PolygonToolHandler.h"
#import "Utility.h"
#import <FoxitRDK/FSPDFViewControl.h>

@interface PolygonModule ()

@property (nonatomic, weak) FSMainToolbarItem *propertyItem;
@property (nonatomic, weak) FSReadToolbarItem *readToolbarItem;
@property (nonatomic, strong) FSReadToolbarItem *polygonItem;
@property (nonatomic, strong) FSReadToolbarItem *cloudItem;
@property (nonatomic) BOOL isPolygon;

@end

@implementation PolygonModule {
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;
    FSAnnotType _annotType;
}

- (NSString *)getName {
    return @"Polygon";
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _isPolygon = YES;
        [self loadModule];
        PolygonAnnotHandler *annotHandler = [[PolygonAnnotHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerAnnotHandler:annotHandler];

        PolygonToolHandler *toolHandler = [[PolygonToolHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerToolHandler:toolHandler];
        [_extensionsManager registerAnnotPropertyListener:toolHandler];

        annotHandler.minVertexDistance = 5;
        toolHandler.minVertexDistance = 5;
    }
    return self;
}

- (FSAnnotType)annotType{
    return _annotType;
}

- (void)setAnnotType:(FSAnnotType)annotType{
    _annotType = annotType;
    if (annotType == FSAnnotPolygon) {
        self.isPolygon = YES;
    }
    if (annotType == FSAnnotCloud) {
        self.isPolygon = NO;
    }
}

- (void)loadModule {
    _polygonItem = [[FSReadToolbarItem alloc] initWithToolSettings:[_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypePolygon]];
    [_extensionsManager.toolbarManager addReadItem:_polygonItem atToolTag:FSMainTopbarToolTagDrawing];
    
    _cloudItem = [[FSReadToolbarItem alloc] initWithToolSettings:[_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeCloud]];
    [_extensionsManager.toolbarManager addReadItem:_cloudItem atToolTag:FSMainTopbarToolTagDrawing];
    [_extensionsManager registerAnnotPropertyListener:self];
}

- (void)didClickedReadToolItem:(FSReadToolbarItem *)readToolbarItem {
    [_extensionsManager changeState:FSUIManagerStateAnnotTool];
    PolygonToolHandler *toolHandler = (PolygonToolHandler *) [_extensionsManager getToolHandlerByName:Tool_Polygon];
    toolHandler.toolSettings = readToolbarItem.toolSettings;
    toolHandler.isPolygon = self.isPolygon;
    [_extensionsManager setCurrentToolHandler:toolHandler];


    self.readToolbarItem = readToolbarItem;
    FSMainToolbarItem *propertyItem =  [_extensionsManager.toolbarManager getMainItemWithType:FSMainToolbarItemTypeProperty];
    propertyItem.item.onTapClick = ^(FSItem * _Nonnull _item) {
        CGRect rect = [_item convertRect:_item.bounds toView:self->_extensionsManager.pdfViewCtrl];
        if (DEVICE_iPHONE) {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:rect inView:self->_extensionsManager.pdfViewCtrl];
        } else {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:_item.bounds inView:_item];
        }
    };
    self.propertyItem = propertyItem;
    
    if (readToolbarItem.toolSettings.settingsType == FSReadToolSettingsTypePolygon) {
        [_extensionsManager onUIInteractElementClicked:FSUIElementReadingDrawingBarPolygon];
    }
    else if (readToolbarItem.toolSettings.settingsType == FSReadToolSettingsTypeCloud) {
        [_extensionsManager onUIInteractElementClicked:FSUIElementReadingDrawingBarCloud];
    }

}


#pragma mark - IAnnotPropertyListener

- (void)onToolSettingsChanged:(FSReadToolSettings *)toolSettings {
    if (self.readToolbarItem && self.readToolbarItem.toolSettings == toolSettings) {
        self.readToolbarItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }else if (_polygonItem.toolSettings == toolSettings){
        _polygonItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }else if (_cloudItem.toolSettings == toolSettings){
        _cloudItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }
}

@end
