/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSColorLayout.h"
#import "ColorBox.h"

FSSYNTH_INTERFACE_IMP_CLASS(_FSPropertyColorStackView, UIStackView)
@interface FSColorLayout ()
@property (nonatomic, strong) _FSPropertyColorStackView *colorStackView;
@property (nonatomic, strong) UIColor *customColor;
@property (nonatomic, strong) FSItem *customColorItem;

@property (nonatomic, strong) ColorBox *colorBox;
@property (nonatomic, strong) FSItem *currentItem;

@property (nonatomic, strong) NSMutableArray *allColorItems;

@property (nonatomic, strong) UIColor *currentColor;
@end

@implementation FSColorLayout

- (instancetype)initWithPropertyType:(FSPropertyType)propertyType colors:(NSArray <UIColor *> *)colors currentColor:(UIColor *)currentColor{
    self = [super initWithPropertyType:propertyType];
    if (self) {
        _currentColor = currentColor;
        _colors = colors;
        [self addColorsItem];
        [self addCustomColorItem];
        self.unfoldItem.hidden = YES;
    }
    return self;
}
- (_FSPropertyColorStackView *)colorStackView{
    if(!_colorStackView){
        _colorStackView = [_FSPropertyColorStackView new];
        _colorStackView.distribution = UIStackViewDistributionEqualSpacing;
        _colorStackView.alignment = UIStackViewAlignmentCenter;
        _allColorItems = @[].mutableCopy;
    }
    return _colorStackView;
}

- (void)setColors:(NSArray<UIColor *> *)colors{
    _colors = colors.copy;
    [self.colorStackView.subviews makeObjectsPerformSelector:@selector(removeFromSuperview)];
    [self addColorsItem];
    [self addCustomColorItem];
}

- (void)setCurrentColor:(UIColor *)currentColor{
    _currentColor = currentColor;
    BOOL isExist = NO;
    for (FSItem *colorItem in _allColorItems) {
        if (colorItem.imageView.backgroundColor.rgbValue == currentColor.rgbValue) {
            if (colorItem == self.currentItem) continue;
            [self didClickedColorItem:colorItem];
            isExist = YES;
            break;
        }
    }
    if (!isExist) {
        self.customColor = currentColor;
        UIImage *image = [FSImageNamed(@"thumbnail_color_ring") fs_addImage:[FSImageNamed(@"thumbnail_ring") fs_imageWithTintColor:self.customColor]];
        [self.customColorItem setImage:image forState:UIControlStateSelected | UIControlStateHighlighted | UIControlStateDisabled];
        [self.customColorItem setImage:image forState:UIControlStateNormal];
    }
}

- (void)addContentView{
    [self addSubview:self.colorStackView];
    [self.colorStackView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.fs_mas_left).offset(ITEM_MARGIN_NORMAL);
        make.right.equalTo(self.fs_mas_right).offset(-ITEM_MARGIN_NORMAL);
        make.bottom.top.fs_equalToSuperview();
        make.height.offset(58.f);
    }];
}

- (void)addColorsItem{
    for (int i = 0; i < _colors.count; i ++) {
        UIColor *color = _colors[i];
        UIImage *image = nil;
        image = [UIImage fs_imageWithColor:color size:CGSizeMake(24.f, 24.f)];
        FSItem *colorItem = [FSItem new];
        colorItem.accessibilityIdentifier = [NSString stringWithFormat:@"color_item_%@",[color hexString]];
        colorItem.accessibilityLabel = [NSString stringWithFormat:@"%@ %@", FSLocalizedForKey(@"kColorValueAccessibilityLabel"), [color hexString]];
        if (!(self.propertyType == FSPropertyTypeFillColor && color.rgbaValue == 0xFFFFFF00)){
            [colorItem setImage:image forState:FSItemControlStateAll];
        }
        
        colorItem.imageView.backgroundColor = color;
        [colorItem fs_setLayerCornerRadius:LAYER_BORDER_RADIUS_3];
        
        @weakify(self)
        colorItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            [self didClickedColorItem:_item];
        };
        
        if (color.rgbaValue == self.currentColor.rgbaValue) {
            colorItem.onTapClick(colorItem);
        }
        if (self.propertyType == FSPropertyTypeFillColor && color.rgbaValue == 0xFFFFFF00){
            [colorItem setImage:FSImageNamed(@"read_property_clearColor") forState:FSItemControlStateAll];
            [colorItem setImageEdgeInsets:UIEdgeInsetsMake(4.f, 4.f, 4.f, 4.f)];
            [self.colorStackView addArrangedSubview:colorItem];
            [_allColorItems addObject:colorItem];
            continue;
        }
        if (color.rgbValue == [UIColor whiteColor].rgbValue) {
            [colorItem.imageView fs_setLayerBorderWidth:LAYER_BORDER_WIDTH_05 borderColor:self.fs_backgroundColor_B3];
        }
        [colorItem setImageEdgeInsets:UIEdgeInsetsMake(4.f, 4.f, 4.f, 4.f)];
        [colorItem.imageView fs_setLayerCornerRadius:12.f];
        
        [self.colorStackView addArrangedSubview:colorItem];
        [_allColorItems addObject:colorItem];
    }
}

- (void)addCustomColorItem{
    UIImage *image = FSImageNamed(@"thumbnail_ring_none");
    if (!self.currentItem) {
        self.customColor = self.currentColor;
        if (self.customColor) {
            image = [FSImageNamed(@"thumbnail_color_ring") fs_addImage:[FSImageNamed(@"thumbnail_ring") fs_imageWithTintColor:self.currentColor]];
        }
    }
    
    FSItem *colorItem = [FSItem new];
    colorItem.accessibilityIdentifier = @"color_item_custom";
    colorItem.accessibilityLabel = FSLocalizedForKey(@"kCustomColorAccessibilityLabel");
    [colorItem setImage:image forState:FSItemControlStateAll];
    colorItem.imageView.backgroundColor = [UIColor clearColor];
    [colorItem fs_setLayerCornerRadius:LAYER_BORDER_RADIUS_3];
    
    @weakify(self)
    colorItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [UIView animateWithDuration:ANIMATE_DURATION_NORMAL animations:^{
            [self didClickedColorItem:_item];
            [self addAttachedContentView];
        }];
    };
    
    self.customColorItem = colorItem;
    if (!self.currentItem) {
        [self didClickedColorItem:colorItem];
    }
    [colorItem setImageEdgeInsets:UIEdgeInsetsMake(1, 1, 1, 1)];
    [self.colorStackView addArrangedSubview:colorItem];
    [_allColorItems addObject:colorItem];
}

- (void)didClickedColorItem:(FSItem *)colorItem{
    if (self.currentItem != colorItem) {
        UIColor *oldColor = self.currentColor;

        [self.currentItem fs_setLayerBorderWidth:0 borderColor:nil];
        self.currentItem.selected = NO;
        self.currentItem = colorItem;
        self.currentItem.selected = YES;
        
        [BindThemeKeySel(self.currentItem, fs_setLayerBorderWidth:borderColor:) fs_updateThemeCompleted:^(FSItem * _Nonnull target, SEL  _Nonnull sel) {
            if (target.selected) {
                [target fs_setLayerBorderWidth:1.f borderColor:target.fs_primaryColor];
            }else{
                [target fs_setLayerBorderWidth:0 borderColor:nil];
            }
        }];

        if (self.customColorItem == self.currentItem) {
            if (!self.customColor) {
                self.colorBox.CallBackInt(0, self.currentColor.rgbValue);
                self.customColor = nil;
            }else{
                _currentColor = self.customColor;
            }
        }else{
            _currentColor = self.currentItem.imageView.backgroundColor;
        }
        
        if (self.delegate && [self.delegate respondsToSelector:@selector(propertyLayout:changedProperty:fromOldValue:toValue:)]) {
            NSNumber *toValue = (self.propertyType == FSPropertyTypeFillColor && self.currentColor.rgbaValue == 0xFFFFFF00) ? @(self.currentColor.rgbaValue) : @(self.currentColor.rgbValue);
            [self.delegate propertyLayout:self changedProperty:self.propertyType fromOldValue:@(oldColor.rgbValue) toValue:toValue];
        }
    }
}

- (void)initBackItem{
    [self.backItem setTitle:FSLocalizedForKey(@"kCustomizeColors") forState:UIControlStateNormal];
}

- (void)addAttachedContentView{
    [self.attachedContentView addSubview:self.colorStackView];
    [_colorStackView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.equalTo(self.attachedSubview);
        make.top.equalTo(self.attachedSubview.mas_bottom).offset(ITEM_MEDIUM_SPACE_NORMAL);
        make.bottom.fs_equalToSuperview();
    }];
    ! self.addSubview ? : self.addSubview(self.attachedContentView);
}

- (void)addAttachedSubView{
    [self.attachedContentView addSubview:self.colorBox];
    [self.attachedSubview mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.fs_equalToSuperview().offset(ITEM_MARGIN_NORMAL);
        make.right.fs_equalToSuperview().offset(-ITEM_MARGIN_NORMAL);
        make.height.offset(160.f);
        make.top.equalTo(self.backItem.mas_bottom).offset(4.f);
        make.bottom.fs_equalToSuperview().priorityLow();
    }];
}

- (UIView *)attachedSubview{
    return self.colorBox;
}

- (UIView *)contentView{
    return self.colorStackView;
}

- (ColorBox *)colorBox{
    if(!_colorBox){
        _colorBox = [ColorBox new];
        @weakify(self)
        _colorBox.CallBackInt = ^(long property, UInt32 value) {
            @strongify(self)
            UIColor *oldColor = self.currentColor;
            UIColor *customColor = [UIColor colorWithRGB:value];
            self->_currentColor = customColor;
            self.customColor = customColor;
            [self didClickedColorItem:self.customColorItem];
            UIImage *image = [FSImageNamed(@"thumbnail_color_ring") fs_addImage:[FSImageNamed(@"thumbnail_ring") fs_imageWithTintColor:self.customColor]];
            [self.customColorItem setImage:image forState:UIControlStateSelected | UIControlStateHighlighted | UIControlStateDisabled];
            [self.customColorItem setImage:image forState:UIControlStateNormal];
            
            if (self.delegate && [self.delegate respondsToSelector:@selector(propertyLayout:changedProperty:fromOldValue:toValue:)]) {
                [self.delegate propertyLayout:self changedProperty:self.propertyType fromOldValue:@(oldColor.rgbValue) toValue:@(self.currentColor.rgbValue)];
            }
        };
    }
    return _colorBox;
}

@end
