/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSDistanceUnitLayout.h"

#define Knum @"^[0-9]*$"
@interface FSDistanceUnitLayout ()<UITextFieldDelegate>
{
    NSArray *_dataSource;
}

@property (nonatomic, strong) NSString *currentDistanceUnitName;
@property (nonatomic, strong) NSString *currentDistanceCustomUnitName;

@property (nonatomic, strong) UIButton *distanceUnitBtn;
@property (nonatomic, strong) UITextField *unitTextField;
@property (nonatomic, strong) UITextField *customTextField;
@property (nonatomic, strong) UIButton *customUnitBtn;

@property (nonatomic, assign) BOOL isFirstUnitButtonClicked;
@property (nonatomic, strong) NSString *oldUnitValue;

@property (nonatomic, copy) NSString *unitName;
@property (nonatomic, strong) UIStackView *unitStackView;
@end
@implementation FSDistanceUnitLayout


- (instancetype)initWithUnitName:(NSString *)unitName{
    self = [super initWithPropertyType:FSPropertyTypeDistanceUnit];
    if (self) {
        self.isFirstUnitButtonClicked = YES;
        self.unitName = unitName;
    }
    return self;
}

- (NSArray<FSLayoutDataItem *> *)dataSource{
    if (!_dataSource) {
        NSArray *tmp = @[@"pt",
                         @"inch",
                         @"ft",
                         @"yd",
                         @"p",
                         @"mm",
                         @"cm",
                         @"m"];
        NSMutableArray *dataSource = @[].mutableCopy;
        for (int i = 0  ; i < tmp.count; i ++ ) {
            NSString *unit = tmp[i];
            FSLayoutDataItem *item = [[FSLayoutDataItem alloc] initWithTitle:unit imageName:nil];
            [dataSource addObject:item];
        }
        _dataSource = dataSource.copy;
    }
    return _dataSource;
}

- (void)initBackItem{
    [self.backItem setTitle:FSLocalizedForKey(@"kUnit") forState:UIControlStateNormal];
}

- (void)addContentView{
    [super addContentView];
    [self.contentView mas_updateConstraints:^(MASConstraintMaker *make) {
        make.height.offset(44.f);
    }];
}

- (void)addContentViewSubViews{
    _unitStackView = [UIStackView new];
    _unitStackView.spacing = 12.f;
    _unitStackView.distribution = UIStackViewDistributionFillEqually;
    [self.contentView addSubview:_unitStackView];
    [self.unitStackView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.fs_equalToSuperview().offset(24.f);
        make.right.fs_equalToSuperview().offset(-24.f);
        make.height.offset(28.f);
        make.centerY.fs_equalToSuperview();
    }];
    
    self.unitTextField = [[UITextField alloc] init];
    self.unitTextField.text = @"1";
    self.unitTextField.borderStyle = UITextBorderStyleRoundedRect;
    self.unitTextField.keyboardType = UIKeyboardTypeNumberPad;
    self.unitTextField.delegate = self;
    self.unitTextField.tag = 1000001;
    self.unitTextField.backgroundColor = [UIColor clearColor];
    [self.unitStackView addArrangedSubview:self.unitTextField];
    
    self.distanceUnitBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    [self.distanceUnitBtn setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
    [self.distanceUnitBtn.titleLabel setFont:self.fs_standardGeneralFont_GF2];
//    self.distanceUnitBtn.backgroundColor = [UIColor whiteColor];
    self.distanceUnitBtn.contentMode = UIViewContentModeScaleToFill;
    self.distanceUnitBtn.contentHorizontalAlignment = UIControlContentHorizontalAlignmentCenter;
    self.distanceUnitBtn.contentVerticalAlignment = UIControlContentVerticalAlignmentCenter;
    self.distanceUnitBtn.layer.borderWidth = 1.0f;
    self.distanceUnitBtn.layer.cornerRadius = 3.0f;
    self.distanceUnitBtn.layer.borderColor = [[UIColor colorWithRGB:0x5c5c5c alpha:0.2] CGColor];
    [self.unitStackView addArrangedSubview:self.distanceUnitBtn];
    [self.distanceUnitBtn addTarget:self action:@selector(showUnitLayout) forControlEvents:UIControlEventTouchUpInside];
    
    [self.unitStackView setCustomSpacing:31.f afterView:self.distanceUnitBtn];
    
    UILabel *equalLabel = [[UILabel alloc] init];
    equalLabel.text = @"=";
    [self.unitStackView addSubview:equalLabel];
    [equalLabel mas_makeConstraints:^(MASConstraintMaker *make) {
        make.center.fs_equalToSuperview();
    }];
    
    self.customTextField = [[UITextField alloc] init];
    self.customTextField.text = @"1";
    self.customTextField.borderStyle = UITextBorderStyleRoundedRect;
    self.customTextField.keyboardType = UIKeyboardTypeNumberPad;
    self.customTextField.backgroundColor = [UIColor clearColor];
    [self.unitStackView addArrangedSubview:self.customTextField];

    self.customTextField.delegate = self;
    self.customTextField.tag = 1000002;
    
    self.customUnitBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    [self.customUnitBtn setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
    [self.customUnitBtn.titleLabel setFont:self.fs_standardGeneralFont_GF2];
//    self.customUnitBtn.backgroundColor = [UIColor whiteColor];
    self.customUnitBtn.contentMode = UIViewContentModeScaleToFill;
    self.customUnitBtn.contentHorizontalAlignment = UIControlContentHorizontalAlignmentCenter;
    self.customUnitBtn.contentVerticalAlignment = UIControlContentVerticalAlignmentCenter;
    self.customUnitBtn.layer.borderWidth = 1.0f;
    self.customUnitBtn.layer.cornerRadius = 3.0f;
    self.customUnitBtn.layer.borderColor = [[UIColor colorWithRGB:0x5c5c5c alpha:0.2] CGColor];
    [self.unitStackView addArrangedSubview:self.customUnitBtn];
    [self.customUnitBtn addTarget:self action:@selector(showCustomUnitLayout) forControlEvents:UIControlEventTouchUpInside];
    
    self.unitTextField.userInteractionEnabled = YES;
    self.customTextField.userInteractionEnabled = YES;
    self.distanceUnitBtn.userInteractionEnabled = YES;
    self.customUnitBtn.userInteractionEnabled = YES;
    
    // secrond screen

    [self.unitTextField addTarget:self action:@selector(textChange:) forControlEvents:UIControlEventEditingChanged];
    [self.customTextField addTarget:self action:@selector(textChange:) forControlEvents:UIControlEventEditingChanged];

    [self.unitTextField addTarget:self action:@selector(textEditingEnd:) forControlEvents:UIControlEventEditingDidEnd];
    [self.customTextField addTarget:self action:@selector(textEditingEnd:) forControlEvents:UIControlEventEditingDidEnd];
}

- (void)addAttachedSubView{
    [super addAttachedSubView];
}

- (void)forbidUnit {
    self.unitTextField.userInteractionEnabled = NO;
    self.customTextField.userInteractionEnabled = NO;
    self.distanceUnitBtn.userInteractionEnabled = NO;
    self.customUnitBtn.userInteractionEnabled = NO;
}

- (void)setUnitName:(NSString *)unitName{
    _unitName = unitName;
    NSArray *array = [Utility getDistanceUnitInfo:unitName];
    
    NSString *string0 = @"";
    NSString *string1 = @"";
    NSString *string2 = @"";
    NSString *string3 = @"";
    
    if (array.count > 0) {
        string0 = [array objectAtIndex:0];
    }
    if (array.count > 1) {
        string1 = [array objectAtIndex:1];
    }
    if (array.count > 2) {
        string2 = [array objectAtIndex:2];
    }
    if (array.count > 3) {
        string3 = [array objectAtIndex:3];
    }

    
    self.unitTextField.text = string0;
    self.customTextField.text = string2;
    
    _currentDistanceUnitName = string1;
    _currentDistanceCustomUnitName = string3;
    
    [self.distanceUnitBtn setTitle:_currentDistanceUnitName forState:UIControlStateNormal];
    [self.customUnitBtn setTitle:_currentDistanceCustomUnitName forState:UIControlStateNormal];
    
    for (FSLayoutDataItem *item in self.dataSource) {
        item.isSelect = [item.title isEqualToString:_isFirstUnitButtonClicked ? _currentDistanceUnitName : _currentDistanceCustomUnitName];
    }
}

-(void) showUnitLayout {
    _isFirstUnitButtonClicked = YES;
    [self showTableLayout];
}

-(void) showCustomUnitLayout {
    _isFirstUnitButtonClicked = NO;
    [self showTableLayout];
}

- (void)showTableLayout {
    
    for (FSLayoutDataItem *item in _dataSource) {
        item.isSelect = [item.title isEqualToString:_isFirstUnitButtonClicked ? _currentDistanceUnitName : _currentDistanceCustomUnitName];
    }
    [self.attachedSubview reloadData];
    
    [self.unitTextField resignFirstResponder];
    [self.customTextField resignFirstResponder];
    
    [UIView animateWithDuration:ANIMATE_DURATION_NORMAL animations:^{
        [self addAttachedContentView];
    }];
}

-(void)textEditingEnd:(UITextField *)textField {
    if (textField.text.length == 0) {
        [textField resignFirstResponder];
        textField.text = @"0";
        
        [self writeDataToDistance];
        
        [UIAlertController fs_alert_OK_WithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kDistanceLengthError") actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
            
        }onCurrentSceneResponder:textField];
        return;
    }
}

- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string
{
    BOOL shouldChange = textField.text.length + (string.length - range.length) <= 6;
    if (!shouldChange) {
        textField.text = [textField.text substringToIndex:6];
        [UIAlertController fs_alert_OK_WithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kDistanceLengthError") actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
            
        }onCurrentSceneResponder:textField];
    }
    return shouldChange;
}

-(void)textChange:(UITextField *)textField {
    NSString* number = Knum;
    NSPredicate *numberPre = [NSPredicate predicateWithFormat:@"SELF MATCHES %@",number];
    if(![numberPre evaluateWithObject:textField.text]){
        [UIAlertController fs_alert_OK_WithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kDistanceNumberError") actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
            
        }onCurrentSceneResponder:textField];
        return;
    }
    
    [self writeDataToDistance];
}

-(void)writeDataToDistance {
    NSString *newUnitValue = [NSString stringWithFormat:@"%@ %@ = %@ %@",self.unitTextField.text,_currentDistanceUnitName,self.customTextField.text,_currentDistanceCustomUnitName];
    if (self.delegate && [self.delegate respondsToSelector:@selector(propertyLayout:changedProperty:fromOldValue:toValue:)]) {
        [self.delegate propertyLayout:self changedProperty:self.propertyType fromOldValue:[NSValue valueWithNonretainedObject:_unitName] toValue:[NSValue valueWithNonretainedObject:newUnitValue]];
    }
    _unitName = newUnitValue;
}

#pragma mark - UITextFieldDelegate
- (void)textFieldDidBeginEditing:(UITextField *)textField{
    if(textField.tag == 1000001){
        [self.customTextField resignFirstResponder];
    }else{
        [self.unitTextField resignFirstResponder];
    }
}

#pragma mark -  table view delegate handler

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    NSString *oldUnitName = _currentDistanceUnitName;
    if (!_isFirstUnitButtonClicked) {
        oldUnitName = _currentDistanceCustomUnitName;
    }
    FSLayoutDataItem *item = [self dataForIndexPath:indexPath];
    NSString *newUnitName = item.title;
    if (_isFirstUnitButtonClicked) {
        _currentDistanceUnitName = newUnitName;
        [self.distanceUnitBtn setTitle:_currentDistanceUnitName forState:UIControlStateNormal];
    }else{
        _currentDistanceCustomUnitName = newUnitName;
        [self.customUnitBtn setTitle:_currentDistanceCustomUnitName forState:UIControlStateNormal];
    }
    if (![oldUnitName isEqualToString:newUnitName]) {
        
        for (FSLayoutDataItem *item in self.dataSource) {
            item.isSelect = [item.title isEqualToString:_isFirstUnitButtonClicked ? _currentDistanceUnitName : _currentDistanceCustomUnitName];
        }
        
        [tableView reloadData];
        
        [self writeDataToDistance];
    }
}

@end
