/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSEraseLayout.h"
#import "FSPencilItem.h"

@interface FSEraseLayout ()
@property (nonatomic, assign) FSEraseStyle eraseStyle;
@property (nonatomic, strong) FSPencilItem *wholeItem;
@property (nonatomic, strong) FSPencilItem *partialItem;
@end
@implementation FSEraseLayout

- (instancetype)initWithEraseStyle:(FSEraseStyle)eraseStyle{
    self = [super initWithPropertyType:FSPropertyTypeEarseStyle];
    if (self) {
        self.eraseStyle = eraseStyle;
    }
    return self;
}

- (void)addContentViewSubViews{
    
    UIStackView *stackView = [UIStackView new];
    stackView.distribution = UIStackViewDistributionFillEqually;
    stackView.spacing = 10.f;
    [self.contentView addSubview:stackView];
    
    [stackView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.fs_equalToSuperview().offset(48.f);
        make.right.fs_equalToSuperview().offset(-48.f);
        make.top.fs_equalToSuperview().offset(10.f);
        make.bottom.fs_equalToSuperview().offset(-8.f);
        make.height.offset(48.f);
    }];
    
    FSPencilItem *wholeItem = [[FSPencilItem alloc] initWithRrelation:FSImageTextRelationRight];
    [wholeItem setImageEdgeInsets:UIEdgeInsetsMake(0, 20.f, 0, 0)];
    wholeItem.imageView.contentMode = UIViewContentModeScaleAspectFit;
    wholeItem.titleLabel.textAlignment = NSTextAlignmentCenter;
    wholeItem.font = self.fs_standardGeneralFont_GF2;
    [wholeItem setImage:FSImageNamedAndNormalColor(@"read_erase_whole") forState:UIControlStateNormal | UIControlStateSelected];
    [wholeItem setTitle:FSLocalizedForKey(@"kEraseWhole") forState:UIControlStateNormal | UIControlStateSelected];
    [wholeItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal | UIControlStateSelected];
    [wholeItem fs_setLayerCornerRadius:4.f];
    @weakify(self)
    wholeItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        if (self.eraseStyle != FSEraseStyleWhole) {
            self.eraseStyle = FSEraseStyleWhole;
        }
    };
    self.wholeItem = wholeItem;
    [stackView addArrangedSubview:wholeItem];
    
    FSPencilItem *partialItem = [[FSPencilItem alloc] initWithRrelation:FSImageTextRelationRight];
    [partialItem setImageEdgeInsets:UIEdgeInsetsMake(0, 20.f, 0, 0)];
    partialItem.imageView.contentMode = UIViewContentModeScaleAspectFit;
    partialItem.titleLabel.textAlignment = NSTextAlignmentCenter;
    partialItem.font = self.fs_standardGeneralFont_GF2;
    [partialItem setImage:FSImageNamedAndNormalColor(@"read_erase_partial") forState:UIControlStateNormal | UIControlStateSelected];
    [partialItem setTitle:FSLocalizedForKey(@"kErasePartial") forState:UIControlStateNormal | UIControlStateSelected];
    [partialItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal | UIControlStateSelected];
    [partialItem fs_setLayerCornerRadius:4.f];
    partialItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        if (self.eraseStyle != FSEraseStylePartial) {
            self.eraseStyle = FSEraseStylePartial;
        }
    };
    self.partialItem = partialItem;
    [stackView addArrangedSubview:partialItem];
}

- (void)setEraseStyle:(FSEraseStyle)eraseStyle{
    if (self.delegate && [self.delegate respondsToSelector:@selector(propertyLayout:changedProperty:fromOldValue:toValue:)]) {
        [self.delegate propertyLayout:self changedProperty:self.propertyType fromOldValue:@(_eraseStyle) toValue:@(eraseStyle)];
    }
    _eraseStyle = eraseStyle;
    if (eraseStyle == FSEraseStyleWhole) {
        self.wholeItem.selected = YES;
        self.partialItem.selected = NO;
    }else{
        self.wholeItem.selected = NO;
        self.partialItem.selected = YES;
    }
    
    @weakify(self)
    [BindThemeKeySel(self, setEraseStyle:) fs_updateThemeCompleted:^(id  _Nonnull target, SEL  _Nonnull sel) {
        @strongify(self)
        if (self.wholeItem.selected) {
            [self.wholeItem fs_setLayerBorderWidth:1.f borderColor:self.fs_primaryColor];
            [self.partialItem fs_setLayerBorderWidth:0 borderColor:nil];
        }else{
            [self.partialItem fs_setLayerBorderWidth:1.f borderColor:self.fs_primaryColor];
            [self.wholeItem fs_setLayerBorderWidth:0 borderColor:nil];
        }
    }];
    
    
}
@end
