/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSFontNameLayout.h"

@interface FSFontNameLayout (){
    NSArray *_dataSource;
}
@property (nonatomic, assign) BOOL needCallBack;
@property (nonatomic, copy) NSString *fontName;
@end
@implementation FSFontNameLayout

- (instancetype)initWithFontName:(NSString *)fontName{
    return [self initWithFontNames:nil fontName:fontName];
}

- (instancetype)initWithFontNames:(NSArray<NSString *> *)fontNames fontName:(NSString *)fontName{
    self = [super initWithPropertyType:FSPropertyTypeFontName];
    if (self) {
        if (fontNames.count) {
            NSMutableArray *dataSource = @[].mutableCopy;
            for (int i = 0  ; i < fontNames.count; i ++ ) {
                [dataSource addObject: [[FSLayoutDataItem alloc] initWithTitle:fontNames[i] imageName:nil]];
            }
            _dataSource = dataSource.copy;
        }
        _needCallBack = YES;
        self.fontName = fontName;
        self.unfoldItem.accessibilityIdentifier = @"font_change_id";
    }
    return self;
}

- (void)initBackItem{
    [self.backItem setTitle:FSLocalizedForKey(@"kFontName") forState:UIControlStateNormal];
}

- (NSArray<FSLayoutDataItem *> *)dataSource{
    if (!_dataSource) {
        NSArray *tmp = @[@"Courier",
                         @"Courier-Bold",
                         @"Courier-BoldOblique",
                         @"Courier-Oblique",
                         @"Helvetica",
                         @"Helvetica-Bold",
                         @"Helvetica-BoldOblique",
                         @"Helvetica-Oblique",
                         @"Times-Roman",
                         @"Times-Bold",
                         @"Times-Italic",
                         @"Times-BoldItalic"];
        NSMutableArray *dataSource = @[].mutableCopy;
        for (int i = 0  ; i < tmp.count; i ++ ) {
            [dataSource addObject: [[FSLayoutDataItem alloc] initWithTitle:tmp[i] imageName:nil]];
        }

        _dataSource = dataSource.copy;
    }
    return _dataSource;
}

- (NSString *)fontName{
    return [self dataForIndexPath:self.currentIndexPath].title;
}
- (void)setFontName:(NSString *)fontName{
    fontName = [Utility convert2AppFontString:fontName];
    NSIndexPath *indexPath;
    for (int i = 0; i < self.dataSource.count; i++) {
        FSLayoutDataItem *item = self.dataSource[i];
        if ([item.title isEqualToString:fontName]) {
            indexPath = [NSIndexPath indexPathForRow:i inSection:0];
            break;
        }
    }
    if (!indexPath) {
        NSMutableArray *tmp = self.dataSource.mutableCopy;
        [tmp insertObject:[[FSLayoutDataItem alloc] initWithTitle:fontName imageName:nil] atIndex:0];
        _dataSource = tmp.copy;
        indexPath = [NSIndexPath indexPathForRow:0 inSection:0];
        [self.attachedSubview reloadData];
    }
    id tmp = self.delegate;
    if (!_needCallBack) self.delegate = nil;
    [self tableView:self.attachedSubview didSelectRowAtIndexPath:indexPath];
    if (!_needCallBack) self.delegate = tmp;
}

- (void)setCurrentFontName:(NSString *)fontName{
    _needCallBack = NO;
    self.fontName = fontName;
    _needCallBack = YES;
}

- (void) loadFontNames:(NSArray<NSString *> *)fontNames{
    NSMutableArray *dataSource = @[].mutableCopy;
    for (int i = 0  ; i < fontNames.count; i ++ ) {
        [dataSource addObject: [[FSLayoutDataItem alloc] initWithTitle:fontNames[i] imageName:nil]];
    }
    _dataSource = dataSource.copy;
    [self setFontName:self.fontName];
}

- (void)addContentViewSubViews{
    [super addContentViewSubViews];
    self.leftLab.text = FSLocalizedForKey(@"kFont");
}
- (NSValue *)valueFromLayoutDataItem:(FSLayoutDataItem *)dataItem old:(BOOL)old{
    NSUInteger index = [self.dataSource indexOfObject:dataItem];
    if (index != NSNotFound) {
        return [NSValue valueWithNonretainedObject:dataItem.title];
    }
    return nil;
}

@end
