/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSFontSizeLayout.h"
#import "NSString+FSAdd.h"
#import "FSCornerRadiusStackView.h"

@interface FSFontSizeLayout (){
    NSArray *_dataSource;
}
@property (nonatomic, strong) FSItem *reduceItem;
@property (nonatomic, strong) FSItem *addItem;
@end
@implementation FSFontSizeLayout

- (instancetype)initWithPropertyType:(FSPropertyType)propertyType
{
    if (self = [super initWithPropertyType:propertyType]) {
        self.unfoldItem.accessibilityIdentifier = @"size_value_id";
    }
    return self;
}

- (void)initBackItem{
    [self.backItem setTitle:FSLocalizedForKey(@"kFontSize") forState:UIControlStateNormal];
}

- (NSArray<FSLayoutDataItem *> *)dataSource{
    if (!_dataSource) {
        NSArray *tmp = @[@"6",
                         @"8",
                         @"10",
                         @"12",
                         @"14",
                         @"18",
                         @"24",
                         @"36",
                         @"48",
                         @"64",
                         @"72",
                         @"96",
                         @"144"];
        NSMutableArray *dataSource = @[].mutableCopy;
        if (self.propertyType == FSPropertyTypeFontSizeAuto) {
            [dataSource addObject:[[FSLayoutDataItem alloc] initWithTitle:FSLocalizedForKey(@"kAuto") imageName:nil]];
        }
        for (int i = 0  ; i < tmp.count; i ++ ) {
            NSString *fontSize = tmp[i];
            FSLayoutDataItem *item = [[FSLayoutDataItem alloc] initWithTitle:fontSize imageName:nil];
            [dataSource addObject:item];
        }

        _dataSource = dataSource.copy;
    }
    return _dataSource;
}

- (int)fontSize{
    NSString *title = [self.unfoldItem.currentTitle fs_stringByTrimOccurrences:@"pt"];
    if ([title isEqualToString:FSLocalizedForKey(@"kAuto")]) return 0;
    return [title intValue];
}

- (void)setFontSize:(int)fontSize{
    NSIndexPath *indexPath;
    for (int i = 0; i < self.dataSource.count; i++) {
        FSLayoutDataItem *item = self.dataSource[i];
        if ((self.propertyType == FSPropertyTypeFontSizeAuto && i == 0 && fontSize == 0) || [item.title floatValue] == fontSize) {
            indexPath = [NSIndexPath indexPathForRow:i inSection:0];
            break;
        }
    }
    if (indexPath) {
        [self tableView:self.attachedSubview didSelectRowAtIndexPath:indexPath];
    }else{
        NSNumber *oldValue = @(self.fontSize);
        FSTitleEstimatedTableViewCell *cell = [self.attachedSubview cellForRowAtIndexPath:self.currentIndexPath];
        FSLayoutDataItem *item = [self dataForIndexPath:self.currentIndexPath];
        if (item) {
            self.currentIndexPath = nil;
            item.isSelect = NO;
            cell.rightView.hidden = !item.isSelect;
        }else{
            [self.unfoldItem setTitle:[NSString stringWithFormat:@"%d pt",fontSize] forState:UIControlStateNormal];
        }
        if (self.delegate && [self.delegate respondsToSelector:@selector(propertyLayout:changedProperty:fromOldValue:toValue:)] ) {
            [self.delegate propertyLayout:self changedProperty:self.propertyType fromOldValue:oldValue toValue:@(fontSize)];
        }
        [self verifyFontSizeItem];
    }
}

- (void)verifyFontSizeItem{
    int fontSize = self.fontSize;
    int firstFontSize = [[self.dataSource firstObject].title intValue];
    int lastFontSize = [[self.dataSource lastObject].title intValue];

    if (fontSize == 0) {
        self.reduceItem.enabled = NO;
        self.addItem.enabled = NO;
        return;
    }
    self.reduceItem.enabled = fontSize > firstFontSize;
    self.addItem.enabled = fontSize < lastFontSize;
}

- (void)initUnfoldItem{
    if (self.propertyType == FSPropertyTypeFontSizeFixed) {
        [self.unfoldItem setImage:[UIImage new] forState:UIControlStateNormal];
        [self.unfoldItem setTitleEdgeInsets:UIEdgeInsetsMake(0, 13.f, 0, 13.f)];
    }
}

- (void)addContentViewSubViews{
    [super addContentViewSubViews];
    self.leftLab.text = FSLocalizedForKey(@"kSize");
    
    if (self.propertyType == FSPropertyTypeFontSizeFixed) {
        FSCornerRadiusStackView *itemContetView = [FSCornerRadiusStackView new];
        itemContetView.spacing = 1.f;
        itemContetView.backgroundColor = self.fs_dividerColor_P3;
        [itemContetView fs_setLayerCornerRadius:12.f];
        [self.contentView addSubview:itemContetView];
        [itemContetView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.right.fs_equalToSuperview().offset(-ITEM_MARGIN_NORMAL);
            make.top.fs_equalToSuperview().offset(10.0f);
            make.bottom.fs_equalToSuperview().offset(-10.0f);
        }];

        FSItem *reduceItem = [FSItem new];
        reduceItem.accessibilityIdentifier = @"size_decrease";
        reduceItem.accessibilityLabel = FSLocalizedForKey(@"kDecreaseVoiceOver");
        [reduceItem setImage:FSImageNamedAndNormalColor(@"read_font_size_reduce") forState:UIControlStateNormal];
        
        reduceItem.backgroundColor = self.fs_dividerColor_P2;
        [reduceItem setImageEdgeInsets:UIEdgeInsetsMake(0, 6.f, 0, 6.f)];
        self.reduceItem = reduceItem;
        [itemContetView addArrangedSubview:reduceItem];
        @weakify(self)
        reduceItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            int fontSize = self.fontSize;
            int firstFontSize = [[self.dataSource firstObject].title intValue];
            if (fontSize == firstFontSize) return;
            fontSize--;
            [self.unfoldItem setTitle:[NSString stringWithFormat:@"%d pt",fontSize] forState:UIControlStateNormal];
            [self setFontSize:fontSize];
        };
        
        FSItem *addItem = [FSItem new];
        addItem.accessibilityIdentifier = @"size_increase";
        addItem.accessibilityLabel = FSLocalizedForKey(@"kIncreaseVoiceOver");
        [addItem setImage:FSImageNamedAndNormalColor(@"read_font_size_add") forState:UIControlStateNormal];
        
        addItem.backgroundColor = self.fs_dividerColor_P2;
        [addItem setImageEdgeInsets:UIEdgeInsetsMake(0, 6.f, 0, 6.f)];
        self.addItem = addItem;
        [itemContetView addArrangedSubview:addItem];
        addItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            int fontSize = self.fontSize;
            int lastFontSize = [[self.dataSource lastObject].title intValue];
            if (fontSize == lastFontSize) return;
            fontSize++;
            [self.unfoldItem setTitle:[NSString stringWithFormat:@"%d pt",fontSize] forState:UIControlStateNormal];
            [self setFontSize:fontSize];
        };
        
        [self.unfoldItem mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.right.equalTo(itemContetView.mas_left);
            make.centerY.equalTo(itemContetView);
        }];
    }
}

- (NSValue *)valueFromLayoutDataItem:(FSLayoutDataItem *)dataItem old:(BOOL)old{
    NSUInteger index = [self.dataSource indexOfObject:dataItem];
    if (index != NSNotFound) {
        return @([dataItem.title intValue]);
    }
    return nil;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath{
    FSTitleEstimatedTableViewCell *cell = (FSTitleEstimatedTableViewCell *)[super tableView:tableView cellForRowAtIndexPath:indexPath];
    FSLayoutDataItem *item = [self dataForIndexPath:indexPath];
    if (self.propertyType == FSPropertyTypeFontSizeAuto && indexPath.row == 0) {
        cell.estimatedTitleLabel.text = item.title;
    }else{
        cell.estimatedTitleLabel.text = [NSString stringWithFormat:@"%@ pt",item.title];
    }
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{
    [super tableView:tableView didSelectRowAtIndexPath:indexPath];
    [self verifyFontSizeItem];
    FSLayoutDataItem *item = [self dataForIndexPath:indexPath];
    if (self.propertyType == FSPropertyTypeFontSizeAuto && indexPath.row == 0) {
        [self.unfoldItem setTitle:item.title forState:UIControlStateNormal];
    }else{
        [self.unfoldItem setTitle:[NSString stringWithFormat:@"%@ pt",item.title] forState:UIControlStateNormal];
    }
}

@end
