/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSFormOptionsLayout.h"

typedef void(^FSDidClickedDone)(NSArray *choiceOptions, BOOL isUpdate);

@interface FSFormOptionsItemTableViewCell : FSTitleEstimatedTableViewCell
@end

@interface FSFormOptionsItemTableViewController : FSTableViewController
@property (nonatomic, assign) BOOL multiSelect;
@property (nonatomic, weak) FSChoiceOption *currentOption;
@property (nonatomic, strong) NSIndexPath *currentIndexPath;
@property (nonatomic, assign) BOOL isUpdate;
@property (nonatomic, copy) FSDidClickedDone didClickedDone;
@property (nonatomic, strong) FSItem *addChoiceOption;
@property (nonatomic, weak) UIExtensionsManager *extensionsMgr;
- (void)setCurrentWidget:(FSWidget *)widget;
@end

@interface FSFormOptionsLayout ()
@property (nonatomic, strong) FSWidget *widget;
@property (nonatomic, strong) UILabel *switchTitle;
@property (nonatomic, strong) UISwitch *optionSwitch;
@property (nonatomic, copy) NSArray <FSChoiceOption *> *dataSource;
@property (nonatomic, strong) NSIndexPath *firstSelectedIndexPath;
@end
@implementation FSFormOptionsLayout
@synthesize dataSource = _dataSource;
-(instancetype)initWithWidget:(FSWidget *)widget{
    self = [super initWithPropertyType:FSPropertyTypeFormOptions];
    if (self){
        self.widget = widget;
    }
    return self;
}

- (NSArray<FSChoiceOption *> *)dataSource{
     if (!_dataSource && ![_widget isEmpty] && _widget) {
          FSField *field = [_widget getField];
          NSMutableArray *dataSource = @[].mutableCopy;
          FSChoiceOptionArray *choiceOptions = [field getOptions];
          for (int i = 0; i < [choiceOptions getSize]; i++) {
              FSChoiceOption *choiceOption = [choiceOptions getAt:i];
              [dataSource addObject:choiceOption];
               if (!_firstSelectedIndexPath && choiceOption.selected) {
                    _firstSelectedIndexPath = [NSIndexPath indexPathForRow:i inSection:0];
               }
          }
          _dataSource = dataSource.copy;
     }
     return _dataSource;
}

- (void)initAttachedSubview{
     [self.attachedSubview fs_registerClass:[FSTableViewCell class]];
    [self.attachedSubview fs_setLayerBorderWidth:1.f borderColor:self.fs_dividerColor_P1];
}

- (void)initUnfoldItem{
     self.unfoldItem.imageEdgeInsets = UIEdgeInsetsMake(0, 50.f, 0, 0);
     self.unfoldItem.accessibilityIdentifier = @"combo_box_options";
}

- (void)initBackItem{
     [self.backItem setTitle:FSLocalizedForKey(@"kOptions") forState:UIControlStateNormal];
}

- (void)addContentView{
     [super addContentView];
     self.leftLab.text = FSLocalizedForKey(@"kOptions");
}

- (void)addAttachedSubView{
    UILabel *itemListLab = [UILabel new];
    itemListLab.text = FSLocalizedForKey(@"kItemList");
    itemListLab.font = self.fs_standardGeneralFont_GF2;
    itemListLab.textColor = self.fs_textColor_T4;
    
    [self.attachedContentView addSubview:itemListLab];
    [itemListLab mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.fs_equalToSuperview().offset(ITEM_MARGIN_NORMAL);
        make.top.equalTo(self.backItem.mas_bottom).offset(16.f);
    }];
    
    [self.attachedContentView addSubview:self.attachedSubview];
    [self.attachedSubview mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.fs_equalToSuperview().offset(ITEM_MARGIN_NORMAL);
        make.right.fs_equalToSuperview().offset(-ITEM_MARGIN_NORMAL);
        make.height.offset(115.f).priorityHigh();
        make.height.fs_lessThanOrEqualToSuperview();
        make.top.equalTo(itemListLab.mas_bottom).offset(15.f);
    }];
    
    self.switchTitle = [UILabel new];
    self.switchTitle.font = self.fs_standardGeneralFont_GF2;
    self.switchTitle.textColor = self.fs_textColor_T4;
    [self.attachedContentView addSubview:self.switchTitle];
    [self.switchTitle mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.fs_equalToSuperview().offset(ITEM_MARGIN_NORMAL);
        make.top.equalTo(self.attachedSubview.mas_bottom).offset(15.f);
        make.bottom.fs_equalToSuperview().offset(-18.f);
    }];
    
    self.optionSwitch = [[UISwitch alloc] init];
    BindThemeKeyPath(self.optionSwitch, onTintColor);
    [self.optionSwitch addTarget:self action:@selector(changeOption:) forControlEvents:UIControlEventTouchUpInside];
    [self.attachedContentView addSubview:self.optionSwitch];
    [self.optionSwitch mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.right.fs_equalToSuperview().offset(-ITEM_MARGIN_NORMAL);
        make.centerY.equalTo(self.switchTitle);
        make.size.equalTo(@(CGSizeMake(51.f, 31.f)));
    }];
     
     [self updateOptionConfig];
}

- (void)updateOptionConfig{
     FSField *field = [_widget getField];
     if ([field getType] == FSFieldTypeComboBox) {
         self.optionSwitch.on = ((field.flags & FSFieldFlagComboEdit) == FSFieldFlagComboEdit);
         self.switchTitle.text = FSLocalizedForKey(@"kCustomText");
         self.optionSwitch.accessibilityIdentifier = @"custom_text";
     }else if ([field getType] == FSFieldTypeListBox){
         self.optionSwitch.on = ((field.flags & FSFieldFlagChoiceMultiSelect) == FSFieldFlagChoiceMultiSelect);
         self.switchTitle.text = FSLocalizedForKey(@"kMultipleSelection");
         self.optionSwitch.accessibilityIdentifier = @"mutiSelection_id";
     }
}

- (void)setWidget:(FSWidget *)widget{
    _widget = widget;
    [self updateOptionConfig];
    [self.attachedSubview reloadData];
}

- (void)didMoveToWindow{
     [super didMoveToWindow];
     dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
          if (self.firstSelectedIndexPath) {
               [self.attachedSubview scrollToRowAtIndexPath:self.firstSelectedIndexPath atScrollPosition:UITableViewScrollPositionBottom animated:NO];
          }
     });
}

- (void)changeOption:(UISwitch *)sender{
     if (self.delegate && [self.delegate respondsToSelector:@selector(propertyLayout:changedProperty:fromOldValue:toValue:)] ) {
         [self.delegate propertyLayout:self changedProperty:self.propertyType fromOldValue:@(!sender.on) toValue:@(sender.on)];
     }
    if ([self.switchTitle.text isEqualToString:FSLocalizedForKey(@"kMultipleSelection")]) {
        BOOL isSelect = NO;
        int selectedIndex = -1;
        for (int i = 0; i < self.dataSource.count; i++) {
            FSChoiceOption *choiceOption = self.dataSource[i];
            if (choiceOption.selected) {
                if (selectedIndex < 0) {
                    isSelect = YES;
                    selectedIndex = i;
                }
                choiceOption.selected = NO;
            }
        }
        if (isSelect) {
            FSChoiceOption *choiceOption = self.dataSource[selectedIndex];
            choiceOption.selected = YES;
            FSField *field = [self.widget getField];
            FSChoiceOptionArray *choiceOptions = [field getOptions];
            FSChoiceOptionArray *newChoiceOptions = [[FSChoiceOptionArray alloc] init];
            for (FSChoiceOption *choiceOption in self.dataSource) {
                [newChoiceOptions add:choiceOption];
            }
             if (self.delegate && [self.delegate respondsToSelector:@selector(propertyLayout:changedProperty:fromOldValue:toValue:)] ) {
                 [self.delegate propertyLayout:self changedProperty:self.propertyType fromOldValue:[NSValue valueWithNonretainedObject:choiceOptions] toValue:[NSValue valueWithNonretainedObject:newChoiceOptions]];
             }
        }
    }
    [self.attachedSubview reloadData];
}

- (void)openItemListViewController{
     
     [self.extensionsManager.propertyMainView dismissWithCompletionBlock:^{
          FSFormOptionsItemTableViewController *VC = [[FSFormOptionsItemTableViewController alloc] initWithServices:self.extensionsManager.viewCtrlManager];
          VC.fs_presentByTopViewController = YES;
          VC.extensionsMgr = self.extensionsManager;
          @weakify(self)
          VC.didClickedDone = ^(NSArray *choiceOptionsArr, BOOL isUpdate) {
              @strongify(self)
              if (isUpdate) {
                   self.dataSource = choiceOptionsArr.copy;
                   [self.attachedSubview reloadData];
                   FSField *field = [self.widget getField];
                   FSChoiceOptionArray *choiceOptions = [field getOptions];
                   FSChoiceOptionArray *newChoiceOptions = [[FSChoiceOptionArray alloc] init];
                   for (FSChoiceOption *choiceOption in choiceOptionsArr) {
                       [newChoiceOptions add:choiceOption];
                   }
                   if (self.delegate && [self.delegate respondsToSelector:@selector(propertyLayout:changedProperty:fromOldValue:toValue:)] ) {
                        [self.delegate propertyLayout:self changedProperty:self.propertyType fromOldValue:[NSValue valueWithNonretainedObject:choiceOptions] toValue:[NSValue valueWithNonretainedObject:newChoiceOptions]];
                   }
              }
          };
          [VC setCurrentWidget:self.widget];
          [self.extensionsManager.viewCtrlManager presentViewController:VC animated:YES completion:nil];
     }]; 
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
     FSItem *messageItem = [FSItem new];
     messageItem.accessibilityIdentifier = @"click_to_add_item";
     [messageItem setTitleColor:self.fs_textColor_T2 forState:UIControlStateNormal];
     messageItem.titleLabel.textAlignment = NSTextAlignmentCenter;
     messageItem.titleLabel.font = self.fs_standardGeneralFont_GF3;
     [messageItem sizeToFit];
     @weakify(self)
     messageItem.onTapClick = ^(FSItem * _Nonnull _item) {
         @strongify(self)
         [self openItemListViewController];;
     };
     tableView.backgroundView = messageItem;
     tableView.backgroundColor = [UIColor clearColor];
    if (self.dataSource.count == 0) {
         [messageItem setTitle:FSLocalizedForKey(@"kClickHereFormTip") forState:UIControlStateNormal];
    }else{
         [messageItem setTitleColor:nil forState:UIControlStateNormal];
    }
    return self.dataSource.count;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    return 30;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath{
    FSTableViewCell *cell = [tableView fs_dequeueReusableCellWithClass:[FSTableViewCell class]];
    FSChoiceOption *choiceOption = self.dataSource[indexPath.row];
    cell.textLabel.text = choiceOption.option_label;
     @weakify(cell)
     [BindThemeKeySel(cell.textLabel, setTextColor:) fs_updateThemeCompleted:^(id  _Nonnull target, SEL  _Nonnull sel) {
          @strongify(cell)
          if (choiceOption.selected) {
              cell.textLabel.textColor = cell.fs_primaryColor;
          }else{
              cell.textLabel.textColor = cell.fs_textColor_T4;
          }
     }];
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    [self openItemListViewController];
}
@end

@implementation FSFormOptionsItemTableViewCell
- (void)setHighlighted:(BOOL)highlighted animated:(BOOL)animated{
     [super setHighlighted:highlighted animated:animated];
     if (highlighted) {
          self.backgroundColor = self.fs_dividerColor_P1;
     }else{
          self.backgroundColor = nil;
     }
}
@end

@implementation FSFormOptionsItemTableViewController

- (void)initialize{
     [super initialize];
     self.shouldShowBottomBar = YES;
     self.defaultEstimatedCellStyle = FSEstimatedTableViewCellStyleTitle;
     self.editingImageNormal = FSImageNamedAndNormalColor(@"common_unselect");
     self.editingImageSelected = FSImageNamed(@"common_select");
}

- (FSViewControllerManager *)viewControllerManager{
     return self.services;
}

- (FSItem *)addChoiceOption{
     if(!_addChoiceOption){
          _addChoiceOption = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"read_form_options_add")];
          _addChoiceOption.accessibilityIdentifier = @"add_combo_box_item";
          [_addChoiceOption addTarget:self action:@selector(addChoice:) forControlEvents:UIControlEventTouchUpInside];
     }
     return _addChoiceOption;
}

- (void)setDataSource:(NSArray *)dataSource{
     [super setDataSource:dataSource];
     if (!self.multiSelect) {
          self.currentOption = nil;
          self.currentIndexPath = nil;
          for (int i = 0; i < dataSource.count; i++) {
               FSChoiceOption *choiceOption = dataSource[i];
               if (choiceOption.selected) {
                    self.currentOption = choiceOption;
                    self.currentIndexPath = [NSIndexPath indexPathForRow:i inSection:0];
                    break;
               }
          }
     }
}

- (void)viewDidLoad{
     [super viewDidLoad];
     self.title = FSLocalizedForKey(@"kItemList");
     self.navBarLeftTitle = FSLocalizedForKey(@"kCancel");
     self.navBarRightTitle = FSLocalizedForKey(@"kDone");
     
     @weakify(self)
     self.navBar.rightItem.onTapClick = ^(FSItem * _Nonnull _item) {
          @strongify(self)
          if (self.multiSelect) {
               for (FSChoiceOption *choiceOption in self.selectedDataSource) {
                    choiceOption.selected = YES;
               }
          }
          
          !self.didClickedDone?:self.didClickedDone(self.dataSource, self.isUpdate);
          [self dismissViewControllerAnimated:YES completion:nil];
     };
     self.navBar.rightItem.enabled = NO;
     
     [self.bottomBar addItem:self.addChoiceOption atPosition:FSItemPositionCenter];
     
     self.tableView.editing = YES;
     [self.tableView fs_registerClass:[FSFormOptionsItemTableViewCell class]];
}

- (void)addChoice:(FSItem *)item{
    NSMutableArray *dataSoucre = self.dataSource.mutableCopy;
     NSString *text = [NSString stringWithFormat:@"New Item-%@",[[Utility getUUID] substringToIndex:6]];
     [[[UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kAdd") message:nil textFieldText:text actions:@[FSLocalizedForKey(@"kCancel"),FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action, UITextField * _Nonnull textField) {
         if (index == 1) {
              if (self.multiSelect) {
                  for (FSChoiceOption *choiceOption in dataSoucre) {
                      choiceOption.selected = NO;
                  }
              }
              FSChoiceOption *choiceOption = [[FSChoiceOption alloc] initWithOption_value:textField.text option_label:textField.text selected:YES default_selected:NO];
              [dataSoucre addObject:choiceOption];
              if (!self.multiSelect) {
                  self.currentOption.selected = NO;
              }
              self.dataSource = dataSoucre.copy;
              self.isUpdate = YES;
              NSIndexPath *indexPath = [NSIndexPath indexPathForRow:dataSoucre.count-1 inSection:0];
              if (![self.tableView.indexPathsForVisibleRows containsObject:indexPath]) {
                   [self.tableView scrollToRowAtIndexPath:indexPath atScrollPosition:UITableViewScrollPositionBottom animated:YES];
              }
         }
     }] fs_actionEnables:^BOOL(NSUInteger index, UIAlertAction * _Nonnull action, UITextField * _Nullable textField) {
         if (index == 1) {
              for (FSChoiceOption *option in self.dataSource) {
                  if ([option.option_label isEqualToString:textField.text]) {
                       return NO;
                  }
              }
             return [textField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]].length;
         }
         return YES;
     }] fs_showByViewController:self];
}

- (void)setIsUpdate:(BOOL)isUpdate{
     if (_isUpdate == isUpdate) return;
     _isUpdate = isUpdate;
     self.navBar.rightItem.enabled = YES;
}

- (void)setCurrentWidget:(FSWidget *)widget{
    FSField *field = [widget getField];
    if ([field getType] == FSFieldTypeComboBox) {
        self.multiSelect = NO;
    }else if ([field getType] == FSFieldTypeListBox && (field.flags & FSFieldFlagChoiceMultiSelect) == FSFieldFlagChoiceMultiSelect){
        self.multiSelect = YES;
    }
    NSMutableArray *dataSource = @[].mutableCopy;
    FSChoiceOptionArray *choiceOptions = [field getOptions];
    for (int i = 0; i < [choiceOptions getSize]; i++) {
        FSChoiceOption *choiceOption = [choiceOptions getAt:i];
        [dataSource addObject:choiceOption];
    }
    self.dataSource = dataSource.copy;
}

- (void)tableView:(UITableView *)tableView didDeselectRowAtIndexPath:(NSIndexPath *)indexPath{
     [super tableView:tableView didDeselectRowAtIndexPath:indexPath];
     FSChoiceOption *choiceOption = self.dataSource[indexPath.row];
     if (!self.multiSelect) {
          if ([self.currentOption isEqual:choiceOption]) {
               self.currentOption.selected = NO;
               self.currentIndexPath = nil;
               self.currentOption = nil;
          }
     }else{
          choiceOption.selected = NO;
     }
     self.isUpdate = YES;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [super tableView:tableView didSelectRowAtIndexPath:indexPath];
     FSChoiceOption *choiceOption = self.dataSource[indexPath.row];
     if (!self.multiSelect) {
          if (![self.currentOption isEqual:choiceOption] && self.currentIndexPath) {
               self.currentOption.selected = NO;
               [tableView reloadRowsAtIndexPaths:@[self.currentIndexPath] withRowAnimation:UITableViewRowAnimationNone];
          }
          self.currentIndexPath = indexPath;
          self.currentOption = choiceOption;
          choiceOption.selected = !choiceOption.selected;
          [tableView reloadRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationNone];
     }
     self.isUpdate = YES;
}
    

- (id)dataForAtIndexPath:(NSIndexPath *)indexPath{
     if (self.dataSource.count > indexPath.row) {
          return self.dataSource[indexPath.row];
     }
     return nil;
}
#pragma mark -  table view datasource handler

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView{
     return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.dataSource.count;
}

- (NSString *)dequeueReusableIdentifierWithIndexPath:(NSIndexPath *)indexPath{
    NSString *identifier = NSStringFromClass([FSFormOptionsItemTableViewCell class]);
    return identifier;
}

- (void)configureCell:(FSTitleEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(FSChoiceOption *)choiceOption{
     cell.editingHiddenRightView = NO;
     if (!cell.rightView) {
         FSItem *moreItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"common_more_gray")];
         moreItem.accessibilityIdentifier = @"combo_box_item";
         cell.rightView = moreItem;
     }
     
     cell.contentView.backgroundColor = [UIColor clearColor];
     cell.selectedBackgroundColor = [UIColor clearColor];
     @weakify(self)
     @weakify(cell)
     ((FSItem *)cell.rightView).onTapClick = ^(FSItem * _Nonnull _item) {
         @strongify(self)
         @strongify(cell)
         NSMutableArray *items = @[].mutableCopy;
          [items addObject:[[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kRename") image:FSImageNamedAndNormalColor(@"common_rename") action:^(FSPopoverItem * _Nonnull item, int index) {
               NSString *text = choiceOption.option_label;
               [[[UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kRename") message:FSLocalizedForKey(@"kInputNewItemNameTip") textFieldText:text actions:@[FSLocalizedForKey(@"kCancel"),FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action, UITextField * _Nonnull textField) {
                   if (index == 1) {
                       if ([textField.text compare:text] != NSOrderedSame) {;
                            choiceOption.option_label = textField.text;
                            choiceOption.option_value = textField.text;
                            [self reloadRowsAtIndexPaths:indexPath];
                            self.isUpdate = YES;
                       }
                   }
               }] fs_actionEnables:^BOOL(NSUInteger index, UIAlertAction * _Nonnull action, UITextField * _Nullable textField) {
                   if (index == 1) {
                        for (FSChoiceOption *option in self.dataSource) {
                            if ([option.option_label isEqualToString:textField.text]) {
                                 return NO;
                            }
                        }
                       return [textField.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]].length;
                   }
                   return YES;
               }] fs_showByViewController:self];
          }]];
          
          [items addObject:[[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kDelete") image:FSImageNamedAndNormalColor(@"common_delete_gray") action:^(FSPopoverItem * _Nonnull item, int index) {
               @strongify(self)
               self.isUpdate = YES;
               NSMutableArray *dataSoucre = self.dataSource.mutableCopy;
               [dataSoucre removeObject:choiceOption];
               self.dataSource = dataSoucre.copy;
          }]];

         UIColor *oriColor = cell.backgroundColor;
          if (DEVICE_iPHONE) {
               cell.backgroundColor = self.fs_backgroundColor_B2;
          }
         [[self viewControllerManager] showMoreMenuWithItems:items.copy dismissCompletion:DEVICE_iPHONE  ? ^{
              cell.backgroundColor = oriColor;
         } : nil sourceView:!DEVICE_iPHONE ? ^UIView * _Nonnull(UIPopoverPresentationController * _Nonnull popver) {
             popver.permittedArrowDirections = UIPopoverArrowDirectionRight;
             return _item;
         } : nil];
         
     };
     cell.estimatedTitleLabel.text = choiceOption.option_label;
     if (self.multiSelect) {
          [self selectRow:choiceOption.selected atIndexPath:indexPath];
     }else{
          [self selectRow:choiceOption.selected atIndexPath:indexPath];
     }
}


- (NSString *)titleForEmptyDataSet{
     return FSLocalizedForKey(@"kFormNoOptionsTip");
}

- (UIImage *)imageForEmptyDataSet:(UITableView *)tableView{
     return FSImageNamedAndPrimaryColor(@"common_no_file_found");
}

- (void)dealloc
{
     if (self.extensionsMgr) {
          [self.extensionsMgr showMenuIfNeed];
     }
}
@end
