/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSLineWidthLayout.h"

@interface FSLineWidthLayout ()
@property (nonatomic, assign) BOOL halfValue;
@property (nonatomic, strong) UIView *circleBgView;
@property (nonatomic, strong) UIImageView *circleView;
@property (nonatomic, strong) UILabel *numberView;
@property (nonatomic, strong) UISlider *silder;
@property (nonatomic, strong) UIColor *color;
@property (nonatomic, assign) CGFloat lineWidth;
@end
@implementation FSLineWidthLayout

- (instancetype)initWithColor:(UIColor *)color lineWidth:(CGFloat)lineWidth{
    self = [super initWithPropertyType:FSPropertyTypeLineWidth];
    if (self) {
        _color = color;
        _lineWidth = lineWidth;
        self.lineWidthLayoutValue = FSLineWidthLayoutValueDefault;
        [self setCurrentColor:self.color];
    }
    return self;
}

- (void)addContentView{
    [super addContentView];
    [self.contentView mas_updateConstraints:^(MASConstraintMaker *make) {
        make.height.offset(40.f);
    }];
}

- (void)addContentViewSubViews{
    
    self.circleBgView = [[UIView alloc] init];
    [self.contentView addSubview:self.circleBgView];
    [self.circleBgView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.size.equalTo(@(CGSizeMake(12.f, 12.f)));
        make.centerY.fs_equalToSuperview();
        make.left.fs_equalToSuperview().offset(ITEM_MARGIN_NORMAL);
    }];
    
    self.circleView = [[UIImageView alloc] init];
    self.circleView.clipsToBounds = YES;
    self.circleView.contentMode =  UIViewContentModeScaleAspectFit;
    [self.circleBgView addSubview:self.circleView];
    
    [self.circleView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.size.equalTo(@(CGSizeMake(12.f, 12.f)));
        make.centerY.centerX.fs_equalToSuperview();
    }];
    
    self.numberView = [UILabel new];
    self.numberView.textColor = self.fs_textColor_T4;
    self.numberView.font = self.fs_standardGeneralFont_GF2;
    [self addSubview:self.numberView];
    
    [self.numberView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.fs_equalToSuperview();
        make.right.fs_equalToSuperview().offset(-20.f);
        make.width.offset(50);
    }];
    
    self.silder = [[UISlider alloc] init];
    UIImage *image = [[UIImage fs_imageWithColor:[UIColor whiteColor] size:CGSizeMake(20.f, 20.f)] fs_borderWidth:1.f borderColor:self.fs_dividerColor_P3 cornerRadius:10.f];
    [self.silder setThumbImage:image forState:UIControlStateNormal];
    [self.silder setThumbImage:image forState:UIControlStateHighlighted];
    BindThemeKeyPath(self.silder, tintColor);

    [self.silder addTarget:self action:@selector(sliderChangedValue) forControlEvents:UIControlEventValueChanged];
    
    [self.contentView addSubview:self.silder];
    
    [self.silder mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.circleBgView.mas_right).offset(20);
        make.right.equalTo(self.numberView.mas_left).offset(-20.f);
        make.centerY.fs_equalToSuperview();
    }];
}

- (float)getLineWidthFromSliderValue{
    float lineWidth;
    if (self.halfValue) {
        lineWidth = ((int)self.silder.value)/2.f;
        if (lineWidth - (int)lineWidth > 0.05) {
            self.numberView.text = [NSString stringWithFormat:@"%.1f %@", lineWidth, @"pt"];
        }else{
            self.numberView.text = [NSString stringWithFormat:@"%.0f %@", lineWidth, @"pt"];
        }
        
    }else{
        if (_lineWidthLayoutValue == FSLineWidthLayoutValuePen || _lineWidthLayoutValue == FSLineWidthLayoutValueHighlighter){
            if (self.silder.value < 10) {
                lineWidth = ((int)self.silder.value / 10.0);
                self.numberView.text = [NSString stringWithFormat:@"%.1f %@", lineWidth, @"pt"];
            }else{
                lineWidth = (int)(self.silder.value - 9);
                self.numberView.text = [NSString stringWithFormat:@"%d %@", (int)lineWidth, @"pt"];
            }
        }else{
            lineWidth = (int)self.silder.value;
            self.numberView.text = [NSString stringWithFormat:@"%d %@", (int)lineWidth, @"pt"];
        }
    }
    [self.circleView mas_updateConstraints:^(MASConstraintMaker *make) {
        make.size.equalTo(@(CGSizeMake(lineWidth,lineWidth)));
    }];
    self.circleView.layer.cornerRadius = lineWidth/2;
    return lineWidth;
}

- (void)sliderChangedValue {
    float lineWidth = [self getLineWidthFromSliderValue];
    self.silder.accessibilityValue = [NSString stringWithFormat:@"%.0f %@", lineWidth, @"pt"];
    if (self.delegate && [self.delegate respondsToSelector:@selector(propertyLayout:changedProperty:fromOldValue:toValue:)] ) {
        [self.delegate propertyLayout:self changedProperty:self.propertyType fromOldValue:@(_lineWidth) toValue:@(lineWidth)];
    }
    _lineWidth = lineWidth;
}

- (void)setCurrentColor:(UIColor *)currentColor{
    _color = currentColor;
    
    float lineWidth;
    CGSize size;
    if (self.halfValue) {
        lineWidth = ((int)self.silder.value)/2.f;
        size = CGSizeMake(self.silder.maximumValue/2, self.silder.maximumValue/2);
    }else{
        lineWidth = self.silder.value;
        size = CGSizeMake(self.silder.maximumValue, self.silder.maximumValue);
    }
    if (_color.rgbValue == [UIColor whiteColor].rgbValue) {
        self.circleView.image = [UIImage fs_imageWithColor:_color size:size];
        self.circleView.layer.borderWidth = LAYER_BORDER_WIDTH_05;
        self.circleView.layer.borderColor = self.fs_backgroundColor_B3.CGColor;
    }else{
        self.circleView.image =  [UIImage fs_imageWithColor:_color size:size];
        self.circleView.layer.borderWidth = 0;
    }
    [self.circleView mas_updateConstraints:^(MASConstraintMaker *make) {
        make.size.equalTo(@(CGSizeMake(lineWidth,lineWidth)));
    }];
    self.circleView.layer.cornerRadius = lineWidth/2;
}

- (void)setLineWidthLayoutValue:(FSLineWidthLayoutValue)lineWidthLayoutValue{
    _lineWidthLayoutValue = lineWidthLayoutValue;
    switch (_lineWidthLayoutValue) {
        case FSLineWidthLayoutValuePen:
            self.halfValue = NO;
            self.silder.minimumValue = 1.0f;
            self.silder.maximumValue = 21.f;
            break;
        case FSLineWidthLayoutValueHighlighter:
            self.halfValue = NO;
            self.silder.minimumValue = 1.0f;
            self.silder.maximumValue = 45.f;
            break;
        case FSLineWidthLayoutValueEraser:
            self.halfValue = YES;
            self.silder.minimumValue = 2.f;
            self.silder.maximumValue = 72.f;
            break;
        default:
            self.halfValue = NO;
            self.silder.minimumValue = 1.0f;
            self.silder.maximumValue = 12.0f;
            break;
    }
    CGSize size;
    if (self.halfValue) {
        self.silder.value = _lineWidth * 2;
        size = CGSizeMake(self.silder.maximumValue/2, self.silder.maximumValue/2);
    }else{
        if (_lineWidthLayoutValue == FSLineWidthLayoutValuePen || _lineWidthLayoutValue == FSLineWidthLayoutValueHighlighter){
            if (_lineWidth < 1) {
                self.silder.value = _lineWidth * 10;
            }else{
                self.silder.value = (_lineWidth + 9);
            }
        }else{
           self.silder.value = _lineWidth;
        }
        size = CGSizeMake(self.silder.maximumValue, self.silder.maximumValue);
    }
    [self.circleBgView mas_updateConstraints:^(MASConstraintMaker *make) {
        make.size.equalTo(@(size));
    }];
    [self.contentView mas_updateConstraints:^(MASConstraintMaker *make) {
        make.height.offset(size.height+28.f);
    }];
    [self getLineWidthFromSliderValue];
    self.silder.accessibilityValue = [NSString stringWithFormat:@"%.0f %@", _lineWidth, @"pt"];
}

@end
