/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSOpacityLayout.h"

FSSYNTH_INTERFACE_IMP_CLASS(_FSPropertyOpacityStackView, UIStackView)

@interface FSOpacityLayout ()
@property (nonatomic, strong) _FSPropertyOpacityStackView *opacityStackView;
@property (nonatomic, strong) FSItem *currentItem;
@end
@implementation FSOpacityLayout

- (instancetype)initWithColor:(UIColor *)color opacity:(CGFloat)opacity{
    self = [super initWithPropertyType:FSPropertyTypeOpacity];
    if (self) {
        _opacityStackView = [_FSPropertyOpacityStackView new];
        _opacityStackView.distribution = UIStackViewDistributionEqualSpacing;
        [self addSubview:_opacityStackView];
        
        [_opacityStackView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(self.fs_mas_left).offset(50.f);
            make.right.equalTo(self.fs_mas_right).offset(-50.f);
            make.top.fs_equalToSuperview().offset(13.f);
            make.bottom.fs_equalToSuperview().offset(-13.f);
        }];
        
        for (int i = 1; i < 5; i ++) {
           
            FSItem *opacityItem = [[FSItem alloc] initWithRrelation:FSImageTextRelationBottom];
            opacityItem.accessibilityIdentifier = [NSString stringWithFormat:@"opacity_item_%d",i];
            [opacityItem fs_setLayerCornerRadius:LAYER_BORDER_WIDTH_6];
            opacityItem.tag = i;
            
            @weakify(self)
            opacityItem.onTapClick = ^(FSItem * _Nonnull _item) {
                @strongify(self)
                if (self.currentItem != _item) {
                    NSNumber *opacity = self.currentItem ? @(self.currentItem.tag * 0.25) : nil;
                    [self.currentItem fs_setLayerBorderWidth:0 borderColor:nil];
                    self.currentItem.selected = NO;
                    self.currentItem = _item;
                    self.currentItem.selected = YES;
                    
                    [BindThemeKeySel(self.currentItem, fs_setLayerBorderWidth:borderColor:) fs_updateThemeCompleted:^(FSItem * _Nonnull target, SEL  _Nonnull sel) {
                        if (target.selected) {
                            [target fs_setLayerBorderWidth:1.f borderColor:target.fs_primaryColor];
                        }else{
                            [target fs_setLayerBorderWidth:0 borderColor:nil];
                        }
                    }];
                    
                    if (self.delegate && [self.delegate respondsToSelector:@selector(propertyLayout:changedProperty:fromOldValue:toValue:)]) {
                        [self.delegate propertyLayout:self changedProperty:self.propertyType fromOldValue:opacity toValue:@(opacityItem.tag * 0.25)];
                    }
                    
                }
            };
            
            CGSize size = CGSizeMake(24.f, 24.f);
            UIImage *image = [UIImage fs_imageWithColor:[color colorWithAlphaComponent:0.25 * i] size:size];
            [opacityItem setImage:[FSImageNamed(@"read_property_opacity_bg") fs_addImage:[image fs_cornerRadius:size.width/2]] forState:UIControlStateNormal];
            [opacityItem setImageEdgeInsets:UIEdgeInsetsMake(4.f, 13.f, 6.f, 13.f)];
            [opacityItem setTitleEdgeInsets:UIEdgeInsetsMake(0, 0, 4.f, 0)];
            [opacityItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
            opacityItem.font = self.fs_standardWeakFont_WF2;
            [opacityItem setTitle:[NSString stringWithFormat:@"%d%%",(25 * i)] forState:UIControlStateNormal];
            opacityItem.accessibilityLabel = [NSString stringWithFormat:@"%@ %d%%",FSLocalizedForKey(@"kOpacity"),(25 * i)];
            if (i == opacity/0.25) {
                self.currentItem = opacityItem;
            }
            
            [_opacityStackView addArrangedSubview:opacityItem];
        }
        self.currentItem.selected = YES;
        [BindThemeKeySel(self.currentItem, fs_setLayerBorderWidth:borderColor:) fs_updateThemeCompleted:^(FSItem * _Nonnull target, SEL  _Nonnull sel) {
            if (target.selected) {
                [target fs_setLayerBorderWidth:1.f borderColor:target.fs_primaryColor];
            }else{
                [target fs_setLayerBorderWidth:0 borderColor:nil];
            }
        }];
    }
    return self;
}

- (void)addContentView{};

- (void)setCurrentColor:(UIColor *)currentColor{
    for (FSItem *opacityItem in _opacityStackView.arrangedSubviews) {
        CGSize size = CGSizeMake(24.f, 24.f);
        UIImage *image = [UIImage fs_imageWithColor:[currentColor colorWithAlphaComponent:opacityItem.tag * 0.25] size:size];
        [opacityItem setImage:[FSImageNamed(@"read_property_opacity_bg") fs_addImage:[image fs_cornerRadius:size.width/2]] forState:UIControlStateNormal];
    }
}
@end

