/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSPencilItem.h"

@interface FSPencilItem ()

@property (nonatomic,readwrite, strong) UIImageView *backgroundImageView;
//@property (nullable, nonatomic, readwrite, strong) UILabel     *titleLabel;
//@property (nullable, nonatomic, readwrite, strong) UIImageView *imageView;
@property (nonatomic, assign) FSImageTextRelation relation;

@end

@implementation FSPencilItem
//@synthesize titleLabel = self.titleLabel;
//@synthesize imageView = self.imageView;
/*
// Only override drawRect: if you perform custom drawing.
// An empty implementation adversely affects performance during animation.
- (void)drawRect:(CGRect)rect {
    // Drawing code
}
*/

- (void) makeSubviewsConstraints{
    if ([self.subviews firstObject] != _backgroundImageView) {
        [self sendSubviewToBack:_backgroundImageView];
    }
    if (!self.titleLabel.superview && !self.imageView.superview) return;
    else if (self.titleLabel.superview && !self.imageView.superview) {
        [self.titleLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(self).offset(self.titleEdgeInsets.left);
            make.right.mas_equalTo(self).offset(-self.titleEdgeInsets.right);
            make.top.mas_equalTo(self).offset(self.titleEdgeInsets.top);
            make.bottom.mas_equalTo(self).offset(-self.titleEdgeInsets.bottom);
        }];
    }
    else if (!self.titleLabel.superview && self.imageView.superview) {
        [self.imageView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(self).offset(self.imageEdgeInsets.left);
            make.right.mas_equalTo(self).offset(-self.imageEdgeInsets.right);
            make.top.mas_equalTo(self).offset(self.imageEdgeInsets.top);
            make.bottom.mas_equalTo(self).offset(-self.imageEdgeInsets.bottom);
        }];
    }
    else{
        switch (self.relation) {
            case FSImageTextRelationLeft:{
                    [self.titleLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
                        make.left.mas_equalTo(self).offset(self.titleEdgeInsets.left);
                    }];
                    [self.imageView mas_remakeConstraints:^(MASConstraintMaker *make) {
                        make.left.mas_equalTo(self.titleLabel.mas_right).offset(self.titleEdgeInsets.right + self.imageEdgeInsets.left);
                        make.right.mas_equalTo(self).offset(-self.imageEdgeInsets.right);
                    }];

                    [self.imageView mas_updateConstraints:^(MASConstraintMaker *make) {
                        make.centerY.mas_equalTo(self).priorityHigh();
                        make.centerY.mas_equalTo(self.titleLabel).priorityHigh();
                        make.top.mas_equalTo(self).offset(self.imageEdgeInsets.top);
                        make.bottom.mas_equalTo(self).offset(-self.imageEdgeInsets.bottom);
                    }];
                
            }
                break;
            case FSImageTextRelationRight:{
                    [self.titleLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
                        make.right.mas_equalTo(self).offset(-self.titleEdgeInsets.right);;
                    }];
                    [self.imageView mas_remakeConstraints:^(MASConstraintMaker *make) {
                        make.right.mas_equalTo(self.titleLabel.mas_left).offset(-self.imageEdgeInsets.right - self.titleEdgeInsets.left);
                        make.left.mas_equalTo(self).offset(self.imageEdgeInsets.left);
                        make.width.mas_equalTo(32);
                    }];

                    [self.imageView mas_updateConstraints:^(MASConstraintMaker *make) {
                        make.centerY.mas_equalTo(self).priorityHigh();
                        make.centerY.mas_equalTo(self.titleLabel).priorityHigh();
                        make.top.mas_equalTo(self).offset(self.imageEdgeInsets.top);
                        make.bottom.mas_equalTo(self).offset(-self.imageEdgeInsets.bottom);
                    }];
                
            }
                break;
            case FSImageTextRelationBottom:{
                    [self.titleLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
                        make.bottom.mas_equalTo(self).offset(-self.titleEdgeInsets.bottom);;
                    }];
                    [self.imageView mas_remakeConstraints:^(MASConstraintMaker *make) {
                        make.bottom.mas_equalTo(self.titleLabel.mas_top).offset(-self.imageEdgeInsets.bottom - self.titleEdgeInsets.top);
                        make.top.mas_equalTo(self).offset(self.imageEdgeInsets.top);
                    }];

                    [self.imageView mas_updateConstraints:^(MASConstraintMaker *make) {
                        make.centerX.mas_equalTo(self).priorityHigh();
                        make.centerX.mas_equalTo(self.titleLabel).priorityHigh();
                        make.left.mas_equalTo(self).offset(self.imageEdgeInsets.left);
                        make.right.mas_equalTo(self).offset(-self.imageEdgeInsets.right);
                    }];
                
            }
                break;
            default:{
                    [self.titleLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
                        make.top.mas_equalTo(self).offset(self.titleEdgeInsets.top);
                    }];
                    [self.imageView mas_remakeConstraints:^(MASConstraintMaker *make) {
                        make.top.mas_equalTo(self.titleLabel.mas_bottom).offset(self.imageEdgeInsets.top + self.titleEdgeInsets.bottom);
                        make.bottom.mas_equalTo(self).offset(-self.imageEdgeInsets.bottom);
                    }];

                    [self.imageView mas_updateConstraints:^(MASConstraintMaker *make) {
                        make.centerX.mas_equalTo(self).priorityHigh();
                        make.centerX.mas_equalTo(self.titleLabel).priorityHigh();
                        make.left.mas_equalTo(self).offset(self.imageEdgeInsets.left);
                        make.right.mas_equalTo(self).offset(-self.imageEdgeInsets.right);
                    }];
                
            }
                break;
        }
    }
}

@end
