/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSPencilLayout.h"
#import "FSPencilItem.h"

@interface FSPencilLayout ()
@property (nonatomic, assign) FSPencilStyle pencilStyle;
@property (nonatomic, strong) FSPencilItem *pencilItem;
@property (nonatomic, strong) FSPencilItem *brushItem;
@end
@implementation FSPencilLayout

- (instancetype)initWithPencilStyle:(FSPencilStyle)pencilStyle{
    self = [super initWithPropertyType:FSPropertyTypePen];
    if (self) {
        self.pencilStyle = pencilStyle;
    }
    return self;
}

- (void)addContentViewSubViews{
    
    UIStackView *stackView = [UIStackView new];
    stackView.distribution = UIStackViewDistributionFillEqually;
    stackView.spacing = 10.f;
    [self.contentView addSubview:stackView];
    
    [stackView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.fs_equalToSuperview().offset(48.f);
        make.right.fs_equalToSuperview().offset(-48.f);
        make.top.fs_equalToSuperview().offset(10.f);
        make.bottom.fs_equalToSuperview().offset(-8.f);
        make.height.offset(48.f);
    }];
    
    FSPencilItem *pencilItem = [[FSPencilItem alloc] initWithRrelation:FSImageTextRelationRight];
    [pencilItem setImageEdgeInsets:UIEdgeInsetsMake(0, 20.f, 0, 0)];
    pencilItem.imageView.contentMode = UIViewContentModeScaleAspectFit;
    pencilItem.titleLabel.textAlignment = NSTextAlignmentCenter;
    pencilItem.font = self.fs_standardGeneralFont_GF2;
    [pencilItem setImage:FSImageNamedAndNormalColor(@"read_pen_pencil") forState:UIControlStateNormal | UIControlStateSelected];
    [pencilItem setTitle:FSLocalizedForKey(@"kPen") forState:UIControlStateNormal | UIControlStateSelected];
    [pencilItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal | UIControlStateSelected];
    [pencilItem fs_setLayerCornerRadius:4.f];
    @weakify(self)
    pencilItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        if (self.pencilStyle != FSPenStylePen) {
            self.pencilStyle = FSPenStylePen;
        }
    };
    self.pencilItem = pencilItem;
    [stackView addArrangedSubview:pencilItem];
    
    FSPencilItem *brushItem = [[FSPencilItem alloc] initWithRrelation:FSImageTextRelationRight];
    [brushItem setImageEdgeInsets:UIEdgeInsetsMake(0, 20.f, 0, 0)];
    brushItem.imageView.contentMode = UIViewContentModeScaleAspectFit;
    brushItem.titleLabel.textAlignment = NSTextAlignmentCenter;
    brushItem.font = self.fs_standardGeneralFont_GF2;
    [brushItem setImage:FSImageNamedAndNormalColor(@"read_pen_brush") forState:UIControlStateNormal | UIControlStateSelected];
    [brushItem setTitle:FSLocalizedForKey(@"kBrush") forState:UIControlStateNormal | UIControlStateSelected];
    [brushItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal | UIControlStateSelected];
    [brushItem fs_setLayerCornerRadius:4.f];
    brushItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        if (self.pencilStyle != FSPenStyleBrush) {
            self.pencilStyle = FSPenStyleBrush;
        }
    };
    self.brushItem = brushItem;
    [stackView addArrangedSubview:brushItem];
}

- (void)setPencilStyle:(FSPencilStyle)pencilStyle{
    if (self.delegate && [self.delegate respondsToSelector:@selector(propertyLayout:changedProperty:fromOldValue:toValue:)]) {
        [self.delegate propertyLayout:self changedProperty:self.propertyType fromOldValue:@(_pencilStyle) toValue:@(pencilStyle)];
    }
    _pencilStyle = pencilStyle;
    if (pencilStyle == FSPenStylePen) {
        self.pencilItem.selected = YES;
        self.brushItem.selected = NO;
    }else{
        self.pencilItem.selected = NO;
        self.brushItem.selected = YES;
    }
    
    @weakify(self)
    [BindThemeKeySel(self, setPencilStyle:) fs_updateThemeCompleted:^(id  _Nonnull target, SEL  _Nonnull sel) {
        @strongify(self)
        if (self.pencilItem.selected) {
            [self.pencilItem fs_setLayerBorderWidth:1.f borderColor:self.fs_primaryColor];
            [self.brushItem fs_setLayerBorderWidth:0 borderColor:nil];
        }else{
            [self.brushItem fs_setLayerBorderWidth:1.f borderColor:self.fs_primaryColor];
            [self.pencilItem fs_setLayerBorderWidth:0 borderColor:nil];
        }
    }];
    
    
}
@end
