/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import <UIKit/UIKit.h>
#import "FSPropertyMainView.h"

NS_ASSUME_NONNULL_BEGIN
typedef void (^FSLayoutSubViewOperation) (__kindof UIView *subview);


@interface FSLayoutDataItem : NSObject
@property (nonatomic, copy) NSString *imageName;
@property (nonatomic, copy) NSString *title;
@property (nonatomic, assign) BOOL isSelect;
- (instancetype)initWithTitle:(NSString *)title imageName:(nullable NSString *)imageName;
@end

@class FSPropertyLayout;
@protocol FSPropertyLayoutDelegate <NSObject>

@required
- (void)propertyLayout:(FSPropertyLayout *)propertyLayout changedProperty:(FSPropertyType)property fromOldValue:(NSValue *)oldValue toValue:(NSValue *)toValue;
@end
@interface FSPropertyLayout : UIView<UITableViewDelegate, UITableViewDataSource>

/*
 └──self
     └── contentView
         └── leftLab
         └── unfoldItem
         └── attachedContentView
             ├── backItem
             └── attachedSubview
*/
@property (nonatomic, strong, nullable) NSIndexPath *currentIndexPath;
@property (nonatomic, strong, nullable) FSItem *backItem;
@property (nonatomic, strong, nullable) FSItem *unfoldItem;
@property (nonatomic, strong, nullable) UILabel *leftLab;
@property (nonatomic, strong, nullable) UIView *contentView;
@property (nonatomic, strong, nullable) UIView *attachedContentView;
@property (nonatomic, strong, nullable) __kindof UITableView *attachedSubview;
@property (nonatomic, copy) NSArray<FSLayoutDataItem *> *dataSource;
@property (nonatomic, copy) FSLayoutSubViewOperation addSubview;
@property (nonatomic, copy) FSLayoutSubViewOperation removeSubview;
@property (nonatomic, weak) id<FSPropertyLayoutDelegate>delegate;

- (instancetype)initWithPropertyType:(FSPropertyType)propertyType;
- (FSPropertyType)propertyType;

- (void)initAttachedSubview;
- (void)initBackItem;
- (void)initUnfoldItem;

- (void)addAttachedContentView;
- (void)addAttachedSubView;
- (void)addContentView;
- (void)addContentViewSubViews;

- (FSLayoutDataItem *)dataForIndexPath:(NSIndexPath *)indexPath;
//need override
- (NSValue *)valueFromLayoutDataItem:(FSLayoutDataItem *)dataItem old:(BOOL)old;
@end

NS_ASSUME_NONNULL_END
