/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSPropertyLayout.h"
#import "ColorBox.h"

@interface FSPropertyLayout ()
@property (nonatomic, assign) FSPropertyType propertyType;
@end
@implementation FSPropertyLayout

- (instancetype)initWithPropertyType:(FSPropertyType)propertyType{
    self = [super init];
    if (self) {
        _propertyType = propertyType;
        _currentIndexPath = nil;
        [self addContentView];
        [self addContentViewSubViews];
    }
    return self;
}

- (FSItem *)backItem{
    if (!_backItem) {
        _backItem = [[FSItem alloc] initWithRrelation:FSImageTextRelationRight];
        _backItem.accessibilityIdentifier = @"back_item_id";
        _backItem.accessibilityValue = FSLocalizedForKey(@"kBack");
        @weakify(self)
        _backItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            [self addContentView];
            ! self.removeSubview ? : self.removeSubview(self.attachedContentView);
        };
        _backItem.font = self.fs_standardWeakFont_WF1;
        [_backItem setTitleColor:self.fs_textColor_T2 forState:UIControlStateNormal];
        [_backItem setImage:FSImageNamedAndNormalColor(@"common_back_small") forState:UIControlStateNormal];
        _backItem.imageView.contentMode = UIViewContentModeCenter;
        [_backItem setImageEdgeInsets:UIEdgeInsetsMake(0, 0, 0, 8.f)];
        [self initBackItem];
    }
    return _backItem;
}

- (FSItem *)unfoldItem{
    if (!_unfoldItem) {
        _unfoldItem = [[FSItem alloc] initWithRrelation:FSImageTextRelationLeft];
        _unfoldItem.accessibilityIdentifier = @"unfoldItem_id";
        [_unfoldItem setImage:FSImageNamedAndNormalColor(@"common_accessory_gray") forState:UIControlStateNormal];

        [_unfoldItem setTitleColor:self.fs_textColor_T3 forState:UIControlStateNormal];
        _unfoldItem.font = self.fs_standardGeneralFont_GF2;
        
        @weakify(self)
        _unfoldItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            [UIView animateWithDuration:ANIMATE_DURATION_NORMAL animations:^{
                [self addAttachedContentView];
            }];
        };
        [self initUnfoldItem];
    }
    return _unfoldItem;
}

- (UILabel *)leftLab{
    if(!_leftLab){
        _leftLab = [UILabel new];
        _leftLab.font = self.fs_standardGeneralFont_GF2;
        _leftLab.textColor = self.fs_textColor_T4;
    }
    return _leftLab;
}

- (void)initUnfoldItem{}
- (void)initBackItem{}
- (void)addContentViewSubViews{
    [self.contentView addSubview:self.leftLab];
    
    [self.leftLab mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.fs_equalToSuperview().offset(ITEM_MARGIN_NORMAL);
        make.centerY.fs_equalToSuperview();
    }];
    
    [self.contentView addSubview:self.unfoldItem];
    [self.unfoldItem mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.right.fs_equalToSuperview().offset(-ITEM_TOP_ICON_MARGIN);
        make.top.fs_equalToSuperview().offset(ITEM_TOP_ICON_MARGIN/2);
        make.bottom.fs_equalToSuperview().offset(-ITEM_TOP_ICON_MARGIN/2);
    }];
}
- (void)initAttachedSubview{
    [self.attachedSubview fs_registerClass:[FSTitleEstimatedTableViewCell class]];
}

- (void)addAttachedContentView{
    [self addSubview:self.attachedContentView];
    [self.attachedContentView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.right.top.bottom.fs_equalToSuperview();
    }];
    ! self.addSubview ? : self.addSubview(self.attachedContentView);
}

- (void)addAttachedSubView{
    [_attachedContentView addSubview:self.attachedSubview];
    [self.attachedSubview mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.fs_equalToSuperview();
        make.height.offset(self.dataSource.count * 54.f).priorityMedium();
        make.height.fs_lessThanOrEqualToSuperview().priorityHigh();
        make.top.equalTo(self.backItem.mas_bottom).offset(4.f);
        make.bottom.fs_equalToSuperview();
    }];
}

- (void)addContentView{
    [self addSubview:self.contentView];
    [self.contentView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.right.top.bottom.fs_equalToSuperview();
    }];
}

- (__kindof UIView *)contentView{
    if(!_contentView){
        _contentView = [UIView new];
    }
    return _contentView;
}

- (__kindof UIView *)attachedContentView{
    if(!_attachedContentView){
        _attachedContentView = [UIView new];
        [_attachedContentView addSubview:self.backItem];
        [self.backItem mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.fs_equalToSuperview().offset(ITEM_MEDIUM_SPACE_NORMAL);
            make.top.fs_equalToSuperview();
        }];
        [self addAttachedSubView];
    }
    return _attachedContentView;
}

- (__kindof UITableView *)attachedSubview{
    if(!_attachedSubview){
        _attachedSubview = [UITableView new];
        _attachedSubview.delegate = self;
        _attachedSubview.dataSource = self;
        [_attachedSubview setSeparatorStyle:UITableViewCellSeparatorStyleNone];
        
        [self initAttachedSubview];
    }
    return _attachedSubview;
}

#pragma mark - UITableViewDataSource
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section{
    return self.dataSource.count;
}

- (CGFloat)tableView:(UITableView *)tableView estimatedHeightForRowAtIndexPath:(NSIndexPath *)indexPath{
    return 54.f;
}

#pragma mark - UITableViewDelegate
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath{
    FSTitleEstimatedTableViewCell *cell = [tableView fs_dequeueReusableCellWithClass:[FSTitleEstimatedTableViewCell class]];
    cell.selectedBackgroundColor = self.fs_dividerColor_P1;
    cell.contentView.backgroundColor = nil;
    FSLayoutDataItem *item = [self dataForIndexPath:indexPath];
    if (item.imageName) {
        UIImage *image = FSImageNamedAndNormalColor(item.imageName);
        if (!cell.leftView && image) {
            UIImageView *leftImageView = [UIImageView new];
            cell.leftView = leftImageView;
        }
        ((UIImageView *)cell.leftView).image = image;
    }
    
    cell.estimatedTitleLabel.text = item.title;
    
    if (!cell.rightView) {
        UIImageView *imageView = [UIImageView new];
        [BindThemeKeySel(imageView, setImage:) fs_updateThemeCompleted:^(UIImageView * _Nonnull target, SEL  _Nonnull sel) {
            target.image = FSImageNamedAndPrimaryColor(@"common_select_check");
        }];
        imageView.hidden = YES;
        cell.rightView = imageView;
    }

    cell.rightView.hidden = !item.isSelect;
    if (item.isSelect) {
        cell.accessibilityTraits = UIAccessibilityTraitSelected;
    }else{
        cell.accessibilityTraits = UIAccessibilityTraitNone;
    }
    if (!cell.installed) {
        cell.leftMargin = ITEM_MARGIN_NORMAL;
        [cell installConstraints];
    }
    return cell;
}

- (FSLayoutDataItem *)dataForIndexPath:(NSIndexPath *)indexPath{
    if (!indexPath) return nil;
    return self.dataSource[indexPath.row];
}
- (NSValue *)valueFromLayoutDataItem:(FSLayoutDataItem *)dataItem old:(BOOL)old{
    return nil;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    if (self.currentIndexPath != indexPath) {
        FSLayoutDataItem *oldItem = [self dataForIndexPath:self.currentIndexPath];
        oldItem.isSelect = NO;
        NSValue *oldValue =  [self valueFromLayoutDataItem:oldItem old:YES];
        
        FSTitleEstimatedTableViewCell *cell = [self.attachedSubview cellForRowAtIndexPath:self.currentIndexPath];
        cell.rightView.hidden = !oldItem.isSelect;
        self.currentIndexPath = indexPath;
        if (oldItem.isSelect) {
            cell.accessibilityTraits = UIAccessibilityTraitSelected;
        }else{
            cell.accessibilityTraits = UIAccessibilityTraitNone;
        }
        FSLayoutDataItem *newItem = [self dataForIndexPath:self.currentIndexPath]; 
        newItem.isSelect = YES;
        NSValue *newValue =  [self valueFromLayoutDataItem:newItem old:NO];
        cell = [self.attachedSubview cellForRowAtIndexPath:self.currentIndexPath];
        cell.rightView.hidden = !newItem.isSelect;
        if (newItem.isSelect) {
            cell.accessibilityTraits = UIAccessibilityTraitSelected;
        }else{
            cell.accessibilityTraits = UIAccessibilityTraitNone;
        }
        
        [self.unfoldItem setTitle:newItem.title forState:UIControlStateNormal];
        self.unfoldItem.accessibilityValue = newItem.title;
        if (self.delegate && [self.delegate respondsToSelector:@selector(propertyLayout:changedProperty:fromOldValue:toValue:)] ) {
            [self.delegate propertyLayout:self changedProperty:self.propertyType fromOldValue:oldValue toValue:newValue];
        }
    }
}
- (FSPropertyType)propertyType{
    return _propertyType;
}
@end


@implementation FSLayoutDataItem

- (instancetype)initWithTitle:(NSString *)title imageName:(nullable NSString *)imageName{
    self = [super init];
    if (self) {
        _isSelect = NO;
        _title = title;
        _imageName = imageName;
    }
    return self;
}


@end
