/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import <UIKit/UIKit.h>

NS_ASSUME_NONNULL_BEGIN

typedef NS_ENUM(NSUInteger, FSPropertyType) {
    FSPropertyTypeUnknown = 0,
    FSPropertyTypePropertyColor,
    FSPropertyTypeFillColor,
    FSPropertyTypeBorderColor,
    FSPropertyTypeTextColor,
    FSPropertyTypeBackGroundColor,
    FSPropertyTypeOpacity,
    FSPropertyTypeShapeNote,
    FSPropertyTypeShapeAttachment,
    FSPropertyTypeDistanceUnit,
    FSPropertyTypeFontName,
    FSPropertyTypeFontSizeFixed,
    FSPropertyTypeFontSizeAuto,
    FSPropertyTypeLineWidth,
    FSPropertyTypeBorderWidth,
    FSPropertyTypePen,
    FSPropertyTypeTextRedact,
    FSPropertyTypeTextForm,
    FSPropertyTypeFormOptions,
    FSPropertyTypeRotation,
    FSPropertyTypeFontStyle,
    FSPropertyTypeEarseStyle
};

typedef NS_ENUM(NSUInteger, FSPencilStyle) {
    FSPenStylePen,
    FSPenStyleBrush,
};

typedef NS_ENUM(NSUInteger, FSEraseStyle) {
    FSEraseStylePartial,
    FSEraseStyleWhole,
};

typedef NS_ENUM(NSUInteger, FSLineWidthLayoutValue) {
    FSLineWidthLayoutValueDefault,
    FSLineWidthLayoutValuePen,
    FSLineWidthLayoutValueHighlighter,
    FSLineWidthLayoutValueEraser,
};

static const CGFloat FS_ALPHA_LOCKED = 0.6;
@class FSPropertyMainView;
@class DevicePosition;
@protocol FSPropertyMainViewDelegate <NSObject>

@required
- (void)propertyMainView:(FSPropertyMainView *)propertyMainView changedProperty:(FSPropertyType)property fromOldValue:(NSValue *)oldValue toValue:(NSValue *)newValue;

@optional
- (void)propertyMainViewDidDismiss:(FSPropertyMainView *)propertyMainView;
- (void)propertyMainViewDidShow:(FSPropertyMainView *)propertyMainView;
@end


@class FSPDFViewCtrl;
@protocol IRotationEventListener;

@interface FSPropertyMainView : UIControl <UIPopoverControllerDelegate, IRotationEventListener>

@property (nonatomic, assign, readonly) BOOL isShowing;
@property (nonatomic, assign) unsigned int propertyColor;
@property (nonatomic, assign) unsigned int fillColor;
@property (nonatomic, assign) CGFloat opacity;
@property (nonatomic, assign) CGFloat rotation;
@property (nonatomic, assign) unsigned int icon;
@property (nonatomic, assign) float fontSize;
@property (nonatomic, assign) float lineWidth;
@property (nonatomic, assign) BOOL lock;
@property (nonatomic, assign) FSPencilStyle pencilStyle;
@property (nonatomic, assign) FSEraseStyle eraseStyle;
@property (nonatomic, assign) FSLineWidthLayoutValue lineWidthLayoutValue;
@property (nonatomic, assign) UIPopoverArrowDirection arrowDirection;
@property (nonatomic, copy) NSString *fontName;
@property (nonatomic, copy) NSString *text;
@property (nonatomic, strong) FSWidget *formWidget;
@property (nonatomic, copy) NSString *unitName;
@property (nonatomic, weak) id<FSPropertyMainViewDelegate>delegate;

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager;
- (void)setDistanceLayoutsForbidEdit;
- (void)setColorLayoutColors:(NSArray<UIColor *> *)colors;
- (void)resetAllValues;
- (void)resetProperties:(NSArray <NSNumber *> *)properties;
- (void)showPropertyMainView:(DevicePosition*)devicePos;
- (void)dismiss;
- (void)dismissWithCompletionBlock:(void (^ __nullable)(void))completion;
- (void)unResisterEvent;
@end


NS_ASSUME_NONNULL_END
