/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSPropertyMainView.h"
#import "FSPropertyLayout.h"
#import "FSColorLayout.h"
#import "FSOpacityLayout.h"
#import "FSShapeLayout.h"
#import "FSFontNameLayout.h"
#import "FSFontSizeLayout.h"
#import "FSLineWidthLayout.h"
#import "FSPencilLayout.h"
#import "FSTextLayout.h"
#import "FSFormOptionsLayout.h"
#import "FSDistanceUnitLayout.h"
#import "FSRotationLayout.h"
#import "FSEraseLayout.h"
#import "UIExtensionsManager+Private.h"

FSSYNTH_INTERFACE_IMP_CLASS(_FSPropertyLayoutsContainer, UIView)
FSSYNTH_INTERFACE_IMP_CLASS(_FSPropertyLayoutsScrollView, UIScrollView)
FSSYNTH_INTERFACE_IMP_CLASS(_FSPropertyLayoutsStackView, UIStackView)

static CGFloat kHandleContentViewHeight = 10.f;
@interface FSPropertyMainView () <UIPopoverPresentationControllerDelegate, FSPropertyLayoutDelegate,IWindowFrameChangedListener>
@property (nonatomic, strong) _FSPropertyLayoutsContainer *layoutsContainer;
@property (nonatomic, strong) _FSPropertyLayoutsScrollView *layoutsScrollView;
@property (nonatomic, strong) _FSPropertyLayoutsStackView *layoutsStackView;
@property (nonatomic, strong) UIView *handleContentView;
@property (nonatomic, strong) UIView *handleView;
@property (nonatomic, strong) UIView *disMissView;

@property (nonatomic, strong) NSPointerArray *propertyBarListeners;
@property (nonatomic, strong) NSArray<UIColor *> *colorLayoutColors;
@property (nonatomic, strong) FSViewPopoverController *popViewCtr;

@property (nonatomic, strong) FSColorLayout *propertyColorLayout;
@property (nonatomic, strong) FSColorLayout *fillColorLayout;
@property (nonatomic, strong) FSColorLayout *borderColorLayout;
@property (nonatomic, strong) FSColorLayout *textColorLayout;
@property (nonatomic, strong) FSColorLayout *backgroundColorLayout;
@property (nonatomic, strong) FSOpacityLayout *opacityLayout;
@property (nonatomic, strong) FSShapeLayout *shapeLayout;

@property (nonatomic, strong) FSFontNameLayout *fontNameLayout;
@property (nonatomic, strong) FSFontSizeLayout *fontSizeLayout;
@property (nonatomic, strong) FSLineWidthLayout *lineWidthLayout;
@property (nonatomic, strong) FSLineWidthLayout *borderWidthLayout;
@property (nonatomic, strong) FSPencilLayout *penLayout;
@property (nonatomic, strong) FSEraseLayout *eraseLayout;
@property (nonatomic, strong) FSTextLayout *textLayout;
@property (nonatomic, strong) FSFormOptionsLayout *formOptionsLayout;
@property (nonatomic, strong) FSDistanceUnitLayout *distanceUnitLayout;
@property (nonatomic, strong) FSRotationLayout *rotationLayout;

@property (nonatomic, copy) NSArray<NSNumber *> *currentProperties;

@property (nonatomic, copy) FSLayoutSubViewOperation addSubview;
@property (nonatomic, copy) FSLayoutSubViewOperation removeSubview;

@property (nonatomic, strong) UIPanGestureRecognizer *dismissPan;
@property (nonatomic, strong) DevicePosition *devicePos;
@property (nonatomic, strong) FSReadToolSettings *currentToolSettings;
@end

@implementation FSPropertyMainView {
    __weak UIExtensionsManager *_extensionsManager;
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager{
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        [self initialize];
    }
    return self;
}

- (void)initialize{
    [_extensionsManager registerRotateChangedListener:self];
    [_extensionsManager registerWindowFrameChangedListener:self];
    
    _layoutsContainer = [_FSPropertyLayoutsContainer new];
    _layoutsContainer.backgroundColor = self.fs_backgroundColor_B1;
    [self addLayoutsContainer];
    [self addHandleContentView];
    [self addHandleView];
    if (UIAccessibilityIsVoiceOverRunning()) {
        _disMissView = [[UIView alloc] init];
        _disMissView.isAccessibilityElement = YES;
        UITapGestureRecognizer *backgroundTapRecognizer = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(dismiss)];
        [_disMissView addGestureRecognizer:backgroundTapRecognizer];
        [self addDisMissView];
    }
    
    _layoutsScrollView = [_FSPropertyLayoutsScrollView new];
    _layoutsScrollView.showsHorizontalScrollIndicator = NO;
    [_layoutsContainer addSubview:_layoutsScrollView];
    [_layoutsScrollView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(_handleContentView.mas_bottom);
        make.bottom.equalTo(_layoutsScrollView.superview.fs_mas_bottom);
        make.left.right.fs_equalToSuperview().priorityHigh();
    }];
    
    _layoutsStackView = [_FSPropertyLayoutsStackView new];
    _layoutsStackView.axis = UILayoutConstraintAxisVertical;
    [_layoutsScrollView addSubview:_layoutsStackView];
    [_layoutsStackView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.bottom.left.right.fs_equalToSuperview().priorityHigh();
        make.centerX.width.height.fs_equalToSuperview().priorityHigh();
    }];
    
    self.backgroundColor = [[UIColor blackColor] colorWithAlphaComponent:0.3];
    [self addTarget:self action:@selector(dismiss) forControlEvents:UIControlEventTouchUpInside];
    self.arrowDirection = UIPopoverArrowDirectionUnknown;

}

- (void)unResisterEvent{
    [_extensionsManager unregisterRotateChangedListener:self];
    [_extensionsManager unregisterWindowFrameChangedListener:self];
}

- (void)dealloc
{
}

- (void)addDisMissView{
    [self addSubview:_disMissView];
    [_disMissView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.centerX.top.fs_equalToSuperview().priorityHigh();
        make.bottom.mas_equalTo(_layoutsContainer.mas_top);
        make.width.offset(self.maxWidth).priorityHigh();
    }];
}

- (void)addLayoutsContainer{
    [self addSubview:_layoutsContainer];
    [_layoutsContainer mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.centerX.bottom.fs_equalToSuperview().priorityHigh();
        make.width.offset(self.maxWidth).priorityHigh();
        make.height.lessThanOrEqualTo(self.mas_height).offset(-64.f);
    }];
}

- (void)addHandleContentView{
    _handleContentView  = [UIView new];
    [_layoutsContainer addSubview:_handleContentView];
    [_handleContentView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.right.top.fs_equalToSuperview();
        make.height.offset(kHandleContentViewHeight);
    }];
}

- (void)addHandleView{
    _handleView = [UIView new];
    [_handleView fs_setLayerCornerRadius:LAYER_BORDER_RADIUS_3];
    _handleView.backgroundColor = self.fs_dividerColor_P2;
    [_handleContentView addSubview:_handleView];
    [_handleView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.center.fs_equalToSuperview().priorityHigh();
        make.size.equalTo(@(CGSizeMake(31.f, 4.f))).priorityHigh();
    }];
}

- (CGFloat)maxWidth{
    return MIN(self.pdfViewCtrl.fs_width, self.pdfViewCtrl.fs_height);
}

- (FSPDFViewCtrl *)pdfViewCtrl{
    return _extensionsManager.pdfViewCtrl;
}

#pragma mark -lazy load

- (UIPanGestureRecognizer *)dismissPan{
    if(!_dismissPan){
        _dismissPan = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(handlePanForHide:)];
    }
    return _dismissPan;
}

- (FSColorLayout *)propertyColorLayout{
    if(!_propertyColorLayout){
        _propertyColorLayout = [[FSColorLayout alloc] initWithPropertyType:FSPropertyTypePropertyColor colors:self.colorLayoutColors.count ? self.colorLayoutColors : @[ UIColorHex(#FF0000),UIColorHex(#FFFF00),UIColorHex(#0000FF),UIColorHex(#66CC33),UIColorHex(#993399),UIColorHex(#FFFFFF),UIColorHex(#000000)] currentColor:[UIColor colorWithRGB:self.propertyColor]];
        _propertyColorLayout.delegate = self;
        _propertyColorLayout.addSubview = [self.addSubview copy];
        _propertyColorLayout.removeSubview = [self.removeSubview copy];
    }
    return _propertyColorLayout;
}

- (FSColorLayout *)fillColorLayout{
    if(!_fillColorLayout){
        _fillColorLayout = [[FSColorLayout alloc] initWithPropertyType:FSPropertyTypeFillColor colors:@[ UIColorHex(#FFFFFF00),UIColorHex(#FF0000),UIColorHex(#FFFF00),UIColorHex(#0000FF),UIColorHex(#66CC33),UIColorHex(#993399),UIColorHex(#FFFFFF)] currentColor: (self.fillColor == 0xFFFFFF00 ) ? [UIColor colorWithRGBA:self.fillColor] : [UIColor colorWithRGB:self.fillColor]];
        _fillColorLayout.delegate = self;
        _fillColorLayout.addSubview = [self.addSubview copy];
        _fillColorLayout.removeSubview = [self.removeSubview copy];
    }
    return _fillColorLayout;
}

- (FSColorLayout *)borderColorLayout
{
    if(!_borderColorLayout){
        _borderColorLayout = [[FSColorLayout alloc] initWithPropertyType:FSPropertyTypeBorderColor colors:self.colorLayoutColors.count ? self.colorLayoutColors : @[ UIColorHex(#FF0000),UIColorHex(#FFFF00),UIColorHex(#0000FF),UIColorHex(#66CC33),UIColorHex(#993399),UIColorHex(#FFFFFF),UIColorHex(#000000)] currentColor:[UIColor colorWithRGB:self.propertyColor]];
        _borderColorLayout.delegate = self;
        _borderColorLayout.addSubview = [self.addSubview copy];
        _borderColorLayout.removeSubview = [self.removeSubview copy];
    }
    return _borderColorLayout;
}

- (FSColorLayout *)textColorLayout
{
    if(!_textColorLayout){
        _textColorLayout = [[FSColorLayout alloc] initWithPropertyType:FSPropertyTypeTextColor colors:self.colorLayoutColors.count ? self.colorLayoutColors : @[ UIColorHex(#FF0000),UIColorHex(#FFFF00),UIColorHex(#0000FF),UIColorHex(#66CC33),UIColorHex(#993399),UIColorHex(#FFFFFF),UIColorHex(#000000)] currentColor:[UIColor colorWithRGB:self.propertyColor]];
        _textColorLayout.delegate = self;
        _textColorLayout.addSubview = [self.addSubview copy];
        _textColorLayout.removeSubview = [self.removeSubview copy];
    }
    return _textColorLayout;
}

- (FSColorLayout *)backgroundColorLayout
{
    if(!_backgroundColorLayout){
        _backgroundColorLayout = [[FSColorLayout alloc] initWithPropertyType:FSPropertyTypeBackGroundColor colors:self.colorLayoutColors.count ? self.colorLayoutColors : @[ UIColorHex(#FF0000),UIColorHex(#FFFF00),UIColorHex(#0000FF),UIColorHex(#66CC33),UIColorHex(#993399),UIColorHex(#FFFFFF),UIColorHex(#000000)] currentColor:[UIColor colorWithRGB:self.fillColor]];
        _backgroundColorLayout.delegate = self;
        _backgroundColorLayout.addSubview = [self.addSubview copy];
        _backgroundColorLayout.removeSubview = [self.removeSubview copy];
    }
    return _backgroundColorLayout;
}

- (FSLayoutSubViewOperation)addSubview{
    if(!_addSubview){
        @weakify(self)
        _addSubview = ^(__kindof UIView * _Nonnull subview) {
            @strongify(self)
            [self.layoutsStackView.subviews makeObjectsPerformSelector:@selector(removeFromSuperview)];
            [self.layoutsStackView addArrangedSubview:subview];
            [subview layoutIfNeeded];
            self.popViewCtr.popoverSize = CGSizeMake(IPAD_PREFERENCE_BAR_WIDTH, [self.layoutsContainer systemLayoutSizeFittingSize:UILayoutFittingCompressedSize].height) ;
            [UIView animateWithDuration:ANIMATE_DURATION_NORMAL animations:^{
                [self.layoutsStackView layoutIfNeeded];
            }];
        };
    }
    return _addSubview;
}

- (FSLayoutSubViewOperation)removeSubview{
    if(!_removeSubview){
        @weakify(self)
        _removeSubview = ^(__kindof UIView * _Nonnull subview) {
            @strongify(self)
            [self.layoutsStackView.subviews makeObjectsPerformSelector:@selector(removeFromSuperview)];
            [self resetProperties:self.currentProperties];
            
            if (!DEVICE_iPHONE && ![_extensionsManager fs_currentSceneiPadHorizontalCompact]) {
                self.popViewCtr.popoverSize = CGSizeMake(IPAD_PREFERENCE_BAR_WIDTH, [self.layoutsContainer systemLayoutSizeFittingSize:UILayoutFittingCompressedSize].height);
            }

        };
    }
    return _removeSubview;
}

- (FSFontNameLayout *)fontNameLayout{
    if(!_fontNameLayout){
        _fontNameLayout = [[FSFontNameLayout alloc] initWithFontName:self.fontName];
        _fontNameLayout.addSubview = [self.addSubview copy];
        _fontNameLayout.removeSubview = [self.removeSubview copy];
        _fontNameLayout.delegate = self;
    }
    return _fontNameLayout;
}

- (FSLineWidthLayout *)lineWidthLayout{
    if(!_lineWidthLayout){
        _lineWidthLayout = [[FSLineWidthLayout alloc] initWithColor:[UIColor colorWithRGB:self.propertyColor] lineWidth:self.lineWidth];
        _lineWidthLayout.lineWidthLayoutValue = self.lineWidthLayoutValue;
        _lineWidthLayout.delegate = self;
        _lineWidthLayoutValue = FSLineWidthLayoutValueDefault;
    }
    return _lineWidthLayout;
}

- (FSLineWidthLayout *)borderWidthLayout{
    if(!_borderWidthLayout){
        _borderWidthLayout = [[FSLineWidthLayout alloc] initWithColor:[UIColor colorWithRGB:self.propertyColor] lineWidth:self.lineWidth];
        _borderWidthLayout.lineWidthLayoutValue = self.lineWidthLayoutValue;
        _borderWidthLayout.delegate = self;
        _lineWidthLayoutValue = FSLineWidthLayoutValueDefault;
    }
    return _borderWidthLayout;
}

- (FSPencilLayout *)penLayout{
    if(!_penLayout){
        _penLayout = [[FSPencilLayout alloc] initWithPencilStyle:self.pencilStyle];
        _penLayout.delegate = self;
    }
    return _penLayout;
}

- (FSEraseLayout *)eraseLayout{
    if(!_eraseLayout){
        _eraseLayout = [[FSEraseLayout alloc] initWithEraseStyle:self.eraseStyle];
        _eraseLayout.delegate = self;
    }
    return _eraseLayout;
}

- (FSOpacityLayout *)opacityLayout{
    if(!_opacityLayout){
        _opacityLayout = [[FSOpacityLayout alloc] initWithColor:[UIColor colorWithRGB:self.propertyColor] opacity:self.opacity];
        _opacityLayout.delegate = self;
    }
    return _opacityLayout;
}

- (FSFormOptionsLayout *)formOptionsLayout{
    if(!_formOptionsLayout){
        _formOptionsLayout = [[FSFormOptionsLayout alloc] initWithWidget:self.formWidget];
        _formOptionsLayout.delegate = self;
        _formOptionsLayout.extensionsManager = _extensionsManager;
        _formOptionsLayout.addSubview = [self.addSubview copy];
        _formOptionsLayout.removeSubview = [self.removeSubview copy];
    }
    return _formOptionsLayout;
}

- (FSDistanceUnitLayout *)distanceUnitLayout{
    if(!_distanceUnitLayout){
        _distanceUnitLayout = [[FSDistanceUnitLayout alloc] initWithUnitName:self.unitName];
        _distanceUnitLayout.delegate = self;
        _distanceUnitLayout.addSubview = [self.addSubview copy];
        _distanceUnitLayout.removeSubview = [self.removeSubview copy];
    }
    return _distanceUnitLayout;
}

- (FSRotationLayout *)rotationLayout{
    if(!_rotationLayout){
        _rotationLayout = [[FSRotationLayout alloc] initWithRotation:self.rotation];
        _rotationLayout.delegate = self;
    }
    return _rotationLayout;
}

- (void)setDistanceLayoutsForbidEdit{
    [_distanceUnitLayout forbidUnit];
}

- (void)setColorLayoutColors:(NSArray<UIColor *> *)colors{
    _colorLayoutColors = colors;
    _propertyColorLayout.colors = colors;
}

- (void)resetAllValues{
    _colorLayoutColors = nil;
}

- (FSItem *)createTitleLab:(NSString *)title
{
    FSItem *propertyTitleLab = [FSItem new];
    propertyTitleLab.accessibilityTraits = UIAccessibilityTraitNone;
    propertyTitleLab.customAccessibilityTraits = UIAccessibilityTraitNone;
    [propertyTitleLab setTitle:title forState:FSItemControlStateAll];
    [propertyTitleLab setTitleColor:self.fs_textColor_T2 forState:FSItemControlStateAll];
    propertyTitleLab.font = self.fs_standardWeakFont_WF1;
    [propertyTitleLab setTitleEdgeInsets:UIEdgeInsetsMake(0, ITEM_MARGIN_NORMAL, 0, 0)];
    return propertyTitleLab;
}

- (void)resetProperties:(NSArray <NSNumber *> *)properties{
    if (!self.currentProperties) {
        _colorLayoutColors = nil;
        _propertyColorLayout = nil;
        _fillColorLayout = nil;
        _borderColorLayout = nil;
        _backgroundColorLayout = nil;
        _textColorLayout = nil;
        _opacityLayout = nil;
        _shapeLayout = nil;
        _fontNameLayout = nil;
        _fontSizeLayout = nil;
        _textLayout = nil;
        _formOptionsLayout = nil;
        _distanceUnitLayout = nil;
        _rotationLayout= nil;
        _lineWidthLayout = nil;
        _borderWidthLayout = nil;
        _penLayout = nil;
        _eraseLayout = nil;
    }
    
//    if (!([properties containsObject:@(FSPropertyTypePropertyColor)] && [properties containsObject:@(FSPropertyTypeFillColor)])) {
//        FSItem *propertyLab = [FSItem new];
//        propertyLab.accessibilityTraits = UIAccessibilityTraitNone;
//        propertyLab.customAccessibilityTraits = UIAccessibilityTraitNone;
//        [propertyLab setTitle:FSLocalizedForKey(@"kProperty") forState:FSItemControlStateAll];
//        [propertyLab setTitleColor:self.fs_textColor_T2 forState:FSItemControlStateAll];
//        propertyLab.font = self.fs_standardWeakFont_WF1;
//        [propertyLab setTitleEdgeInsets:UIEdgeInsetsMake(0, ITEM_MARGIN_NORMAL, 0, 0)];
//        [self.layoutsStackView addArrangedSubview:propertyLab];
//    }
    
    for (int i = 0; i < properties.count; i++) {
        NSNumber *property = properties[i];
        FSPropertyType propertyType = [property unsignedIntegerValue];
        
        switch (propertyType) {
            case FSPropertyTypeUnknown: {

                break;
            }
            case FSPropertyTypePropertyColor:
            {
                [self.layoutsStackView addArrangedSubview:[self createTitleLab:FSLocalizedForKey(@"kPropertyColorName")]];
                [self.layoutsStackView addArrangedSubview:self.propertyColorLayout];
                break;
            }
            case FSPropertyTypeFillColor:
            {
                [self.layoutsStackView addArrangedSubview:[self createTitleLab:FSLocalizedForKey(@"kPropertyFillColorName")]];
                [self.layoutsStackView addArrangedSubview:self.fillColorLayout];
                break;
            }
            case FSPropertyTypeBorderColor:
            {
                [self.layoutsStackView addArrangedSubview:[self createTitleLab:FSLocalizedForKey(@"kPropertyBorderColorName")]];
                [self.layoutsStackView addArrangedSubview:self.borderColorLayout];
                break;
            }
            case FSPropertyTypeTextColor:
            {
                [self.layoutsStackView addArrangedSubview:[self createTitleLab:[FSLocalizedForKey(@"kTextColor") uppercaseString]]];
                [self.layoutsStackView addArrangedSubview:self.textColorLayout];
                break;
            }
            case FSPropertyTypeBackGroundColor:
            {
                [self.layoutsStackView addArrangedSubview:[self createTitleLab:[FSLocalizedForKey(@"kBackgroundColor") uppercaseString]]];
                [self.layoutsStackView addArrangedSubview:self.backgroundColorLayout];
                break;
            }
            case FSPropertyTypeOpacity: {
                [self.layoutsStackView addArrangedSubview:[self createTitleLab:FSLocalizedForKey(@"kPropertyOpacityName")]];
                [self.layoutsStackView addArrangedSubview:self.opacityLayout];
                break;
            }
            case FSPropertyTypeShapeNote:
            case FSPropertyTypeShapeAttachment: {
                [self.layoutsStackView addArrangedSubview:[self createTitleLab:FSLocalizedForKey(@"kPropertyShapeSettingsName")]];
                if (!self.shapeLayout) {
                    FSShapeLayout *shapeLayout = [[FSShapeLayout alloc] initWithPropertyType:propertyType];
                    shapeLayout.currentIcon = self.icon;
                    shapeLayout.delegate = self;
                    shapeLayout.addSubview = [self.addSubview copy];
                    shapeLayout.removeSubview = [self.removeSubview copy];
                    self.shapeLayout = shapeLayout;
                }

                [self.layoutsStackView addArrangedSubview:self.shapeLayout];
                break;
            }
            case FSPropertyTypeDistanceUnit: {
                [self.layoutsStackView addArrangedSubview:[self createTitleLab:FSLocalizedForKey(@"kPropertySettingsName")]];
                [self.layoutsStackView addArrangedSubview:self.distanceUnitLayout];
                break;
            }
            case FSPropertyTypeFontName:{
                if (![properties containsObject:@(FSPropertyTypeTextRedact)]) [self.layoutsStackView addArrangedSubview:[self createTitleLab:FSLocalizedForKey(@"kPropertyFontSettingsName")]];
                [self.layoutsStackView addArrangedSubview:self.fontNameLayout];
                break;
            }
            case FSPropertyTypeFontSizeFixed:
            case FSPropertyTypeFontSizeAuto: {
                if (!self.fontSizeLayout) {
                    FSFontSizeLayout *fontSizeLayout = [[FSFontSizeLayout alloc] initWithPropertyType:propertyType];
                    fontSizeLayout.fontSize = self.fontSize;
                    fontSizeLayout.delegate = self;
                    fontSizeLayout.addSubview = [self.addSubview copy];;
                    fontSizeLayout.removeSubview = [self.removeSubview copy];
                    self.fontSizeLayout = fontSizeLayout;
                }
                [self.layoutsStackView addArrangedSubview:self.fontSizeLayout];
                
                break;
            }
            case FSPropertyTypeLineWidth:
            {
                [self.layoutsStackView addArrangedSubview:[self createTitleLab:FSLocalizedForKey(@"kPropertyThicknessName")]];
                [self.layoutsStackView addArrangedSubview:self.lineWidthLayout];
                break;
            }
            case FSPropertyTypeBorderWidth:
            {
                [self.layoutsStackView addArrangedSubview:[self createTitleLab:FSLocalizedForKey(@"kPropertyBorderThicknessName")]];
                [self.layoutsStackView addArrangedSubview:self.borderWidthLayout];
                break;
            }
            case FSPropertyTypeTextRedact:{
                [self.layoutsStackView addArrangedSubview:[self createTitleLab:FSLocalizedForKey(@"kPropertyFontSettingsName")]];
            }
            case FSPropertyTypeTextForm:{
                if (!self.textLayout) {
                    self.textLayout = [[FSTextLayout alloc] initWithPropertyType:propertyType];
                    self.textLayout.extensionsManager = _extensionsManager;
                    self.textLayout.text = self.text;
                    self.textLayout.delegate = self;
                }
                [self.layoutsStackView addArrangedSubview:self.textLayout];
                break;
            }
            case FSPropertyTypePen:{
                [self.layoutsStackView addArrangedSubview:[self createTitleLab:FSLocalizedForKey(@"kPropertySensitivityName")]];
                [self.layoutsStackView addArrangedSubview:self.penLayout];
                break;
            }
            case FSPropertyTypeFormOptions:
                [self.layoutsStackView addArrangedSubview:[self createTitleLab:FSLocalizedForKey(@"kPropertyOptionsName")]];
                [self.layoutsStackView addArrangedSubview:self.formOptionsLayout];
                break;
            case FSPropertyTypeRotation:
                [self.layoutsStackView addArrangedSubview:[self createTitleLab:FSLocalizedForKey(@"kPropertyRotateSettingsName")]];
                [self.layoutsStackView addArrangedSubview:self.rotationLayout];
                break;
            case FSPropertyTypeEarseStyle:{
                [self.layoutsStackView addArrangedSubview:[self createTitleLab:FSLocalizedForKey(@"kPropertyModeName")]];
                [self.layoutsStackView addArrangedSubview:self.eraseLayout];
                break;
            }
        }
        
    }
    self.layoutsStackView.userInteractionEnabled = !self.lock;
    self.layoutsStackView.alpha = !self.lock?:FS_ALPHA_LOCKED;
    _currentProperties = properties.copy;
}

- (void)showPropertyMainView:(DevicePosition*)devicePos{
    
    self.devicePos = devicePos;
    self.currentToolSettings = [_extensionsManager.currentToolSettings copy];
    if (DEVICE_iPHONE || [_extensionsManager fs_currentSceneiPadHorizontalCompact]) {
        [devicePos.inPhoneView addSubview:self];
        self.fs_top = 0;
        self.fs_size = self.pdfViewCtrl.fs_size;
        
        [self mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.right.top.bottom.fs_equalToSuperview();
        }];
        if (self.layoutsContainer) {
            [self.layoutsContainer mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.centerX.fs_equalToSuperview().priorityHigh();
                if (DEVICE_iPHONE) {
                    make.width.mas_equalTo(self.maxWidth).priorityHigh();
                }else{
                    make.width.fs_equalToSuperview().priorityHigh();
                }
                make.height.lessThanOrEqualTo(self.mas_height).offset(-64.f);
                make.top.equalTo(self.layoutsContainer.superview.mas_bottom);
            }];
        }
        [self layoutIfNeeded];
        
        [UIView animateWithDuration:ANIMATE_DURATION_NORMAL animations:^{
            [self.layoutsContainer mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.centerX.fs_equalToSuperview().priorityHigh();
                if (DEVICE_iPHONE) {
                    make.width.mas_equalTo(self.maxWidth).priorityHigh();
                }else{
                    make.width.fs_equalToSuperview().priorityHigh();
                }
                make.height.lessThanOrEqualTo(self.mas_height).offset(-64.f);
                make.bottom.equalTo(self.layoutsContainer.superview.mas_bottom);
            }];
            [self layoutIfNeeded];
        } completion:^(BOOL finished) {
            if (finished) {
                [self scrollToSelectedAnnot];
            }
        }];
        
        [self.layoutsContainer addGestureRecognizer:self.dismissPan];
    }else{
        [self.handleView removeFromSuperview];
        CGFloat height = kHandleContentViewHeight;
        for (UIView *view in self.layoutsStackView.arrangedSubviews) {
            height += [view systemLayoutSizeFittingSize:UILayoutFittingCompressedSize].height;
        }
        if (height < [self.layoutsContainer systemLayoutSizeFittingSize:UILayoutFittingCompressedSize].height) {
            height = [self.layoutsContainer systemLayoutSizeFittingSize:UILayoutFittingCompressedSize].height;
        }
        self.layoutsContainer.frame = CGRectMake(0, 0, IPAD_PREFERENCE_BAR_WIDTH, height);
        self.layoutsContainer.fs_size = CGSizeMake(IPAD_PREFERENCE_BAR_WIDTH, height);
        self.popViewCtr = [_extensionsManager.viewCtrlManager popPopoverCustomView:self.layoutsContainer sourceView:^UIView * _Nonnull(UIPopoverPresentationController * _Nonnull popover) {
            popover.delegate = self;
            
            UIView *carrierView =  nil;
            if (devicePos.carrierViewDelegate && [devicePos.carrierViewDelegate respondsToSelector:@selector(getPopupCarrierView)]) {
                carrierView = [devicePos.carrierViewDelegate getPopupCarrierView];
            }
            if (!carrierView) {
                carrierView = devicePos.inPadView;
            }

            UIPopoverArrowDirection permittedArrowDirections = UIPopoverArrowDirectionAny;
            if (self.arrowDirection != UIPopoverArrowDirectionUnknown) {
                permittedArrowDirections = self.arrowDirection;
            }else{
                if ([devicePos.inPadView isDescendantOfView:[self->_extensionsManager.topToolbar subMenuTopbarContentView]]) {
                    if (self->_extensionsManager.topToolbar.subitemContentPosition == FSTopbarSubitemContentViewPositionLeft) {
                        permittedArrowDirections = UIPopoverArrowDirectionLeft;
                    } else  if (self->_extensionsManager.topToolbar.subitemContentPosition == FSTopbarSubitemContentViewPositionRight) {
                        permittedArrowDirections = UIPopoverArrowDirectionRight;
                    }else{
                        permittedArrowDirections = UIPopoverArrowDirectionUp;
                    }
                }else{
                    UIWindow *window = self->_extensionsManager.pdfViewCtrl.fs_currentSceneKeyWindow;
                    CGRect convertRectToWindowRect = [carrierView convertRect:devicePos.padRect toView:window];
                    if (CGRectGetMaxX(convertRectToWindowRect) > IPAD_PREFERENCE_BAR_WIDTH + 50.f + CGRectGetWidth(devicePos.padRect)/2 && CGRectGetMaxY(convertRectToWindowRect) > window.fs_safeAreaInsets.top + TOOL_BAR_HEIGHT_NORMAL) {
                        permittedArrowDirections = UIPopoverArrowDirectionRight;
                    }else{
                        permittedArrowDirections = UIPopoverArrowDirectionLeft;
                    }
                }
            }
            popover.permittedArrowDirections = permittedArrowDirections;
            return  carrierView;
        } sourceRect:devicePos.padRect];
        self.popViewCtr.autoresizesFollowContnetView = YES;
    }
    _isShowing = YES;

    if (self.delegate  && [self.delegate respondsToSelector:@selector(propertyMainViewDidShow:)]) {
        [self.delegate propertyMainViewDidShow:self];
    }
}

- (void)handlePanForHide:(UIPanGestureRecognizer *)panRecognizer{
    CGPoint point = [panRecognizer translationInView:self.layoutsContainer];
    CGFloat height = CGRectGetHeight(self.layoutsContainer.frame);
    CGFloat supHeight = CGRectGetHeight(self.layoutsContainer.superview.frame);
    CGFloat minY = supHeight - height;
    CGFloat minCenterY = supHeight - height / 2;
    CGPoint center =  CGPointMake(self.layoutsContainer.center.x, self.layoutsContainer.center.y + point.y);
    self.layoutsContainer.center = center;
    if (CGRectGetMinY(self.layoutsContainer.frame) < minY) {
        self.layoutsContainer.frame = CGRectMake(CGRectGetMinX(self.layoutsContainer.frame), minY, CGRectGetWidth(self.layoutsContainer.frame), CGRectGetHeight(self.layoutsContainer.frame));
    }
    if (panRecognizer.state == UIGestureRecognizerStateEnded || panRecognizer.state == UIGestureRecognizerStateCancelled) {
        if (CGRectGetMinY(self.layoutsContainer.frame) < minCenterY) {
            [UIView animateWithDuration:0.2 animations:^{
                self.layoutsContainer.frame = CGRectMake(CGRectGetMinX(self.layoutsContainer.frame), minY, CGRectGetWidth(self.layoutsContainer.frame), CGRectGetHeight(self.layoutsContainer.frame));
            }];
        }else{
            [self dismiss];
        }
    }
    [panRecognizer setTranslation:CGPointMake(0, 0) inView:self.layoutsContainer];
}

- (void)scrollToSelectedAnnot{
    if (!self.isShowing) return;
    CGRect mainViewFrame = self.layoutsContainer.frame;
    FSAnnot *annot = self->_extensionsManager.currentAnnot;
    if (!annot || [annot isEmpty]) {
        return;
    }
    [self.pdfViewCtrl setBottomOffset:0];
    int pageIndex = annot.pageIndex;

    float (^getPositionY)(void) = ^(){

        CGRect pvAnnotRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
        CGRect dvAnnotRect = [self.pdfViewCtrl convertPageViewRectToDisplayViewRect:pvAnnotRect pageIndex:pageIndex];

        float positionY;
        positionY = self.pdfViewCtrl.bounds.size.height - CGRectGetMaxY(dvAnnotRect);
        return positionY;
    };

    float positionY = getPositionY();

    if (positionY < mainViewFrame.size.height) {

        CGRect pvAnnotRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
        CGPoint point = CGPointMake(CGRectGetMidX(pvAnnotRect) - CGRectGetMidX(mainViewFrame), CGRectGetMaxY(pvAnnotRect) - (self.pdfViewCtrl.bounds.size.height - mainViewFrame.size.height) + 20);
        FSPointF *pointF = [self.pdfViewCtrl convertPageViewPtToPdfPt:point pageIndex:pageIndex];
        [self.pdfViewCtrl gotoPage:pageIndex withDocPoint:pointF animated:YES];

        float positionY = getPositionY();

        float dvOffsetY = mainViewFrame.size.height - positionY + 20;

        CGRect offsetRect = CGRectMake(0, 0, 100, dvOffsetY);

        CGRect pvRect = [self.pdfViewCtrl convertDisplayViewRectToPageViewRect:offsetRect pageIndex:annot.pageIndex];
        FSRectF *pdfRect = [self.pdfViewCtrl convertPageViewRectToPdfRect:pvRect pageIndex:annot.pageIndex];
        float pdfOffsetY = [pdfRect getTop] - [pdfRect getBottom];

        if ([self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_SINGLE && ![self.pdfViewCtrl isContinuous]) {
            if (CGRectGetMaxY([self.pdfViewCtrl getPageView:pageIndex].frame) - CGRectGetMaxY(pvAnnotRect) < mainViewFrame.size.height + 20) {
                [self.pdfViewCtrl setBottomOffset:dvOffsetY];
            }
        } else if ([self.pdfViewCtrl isContinuous]) {
            if ([self.pdfViewCtrl getCurrentPage] == [self.pdfViewCtrl.currentDoc getPageCount] - 1) {
                FSRectF *fsRect = [[FSRectF alloc] init];
                [fsRect setLeft:0];
                [fsRect setBottom:pdfOffsetY];
                [fsRect setRight:pdfOffsetY];
                [fsRect setTop:0];
                float tmpPvOffset = [self.pdfViewCtrl convertPdfRectToPageViewRect:fsRect pageIndex:annot.pageIndex].size.width;
                CGRect tmpPvRect = CGRectMake(0, 0, 10, tmpPvOffset);
                CGRect tmpDvRect = [self.pdfViewCtrl convertPageViewRectToDisplayViewRect:tmpPvRect pageIndex:annot.pageIndex];
                [self.pdfViewCtrl setBottomOffset:tmpDvRect.size.height];
            } else {
                double scrollY = [self.pdfViewCtrl getVScrollPos];
                [self.pdfViewCtrl setVScrollPos:scrollY+dvOffsetY animated:YES];
            }
        }
    }
}

#pragma mark <UIPopoverPresentationControllerDelegate>

- (void)popoverPresentationControllerDidDismissPopover:(UIPopoverPresentationController *)popoverPresentationController {
    [self dismiss];
}

- (UIModalPresentationStyle)adaptivePresentationStyleForPresentationController:(UIPresentationController *)controller {
    return UIModalPresentationNone;
}

- (UIModalPresentationStyle)adaptivePresentationStyleForPresentationController:(UIPresentationController *)controller traitCollection:(UITraitCollection *)traitCollection {
    return UIModalPresentationNone;
}

- (void)popoverPresentationController:(UIPopoverPresentationController *)popoverPresentationController willRepositionPopoverToRect:(inout CGRect *)rect inView:(inout UIView  * __nonnull * __nonnull)view{
    FSAnnot *annot = self->_extensionsManager.currentAnnot;
    if (annot && ![annot isEmpty] && self.devicePos.annot && self.devicePos.annot == annot) {
        CGRect pvRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:annot.pageIndex];
        CGRect showRect = [self.pdfViewCtrl convertPageViewRectToDisplayViewRect:pvRect pageIndex:annot.pageIndex];
        if (showRect.origin.x < IPAD_PREFERENCE_BAR_WIDTH && showRect.origin.y < self.layoutsContainer.frame.size.height && self.pdfViewCtrl.bounds.size.width - showRect.size.width - showRect.origin.x < 300 && self.pdfViewCtrl.bounds.size.height - showRect.size.height - showRect.origin.y < self.layoutsContainer.frame.size.height) {
            showRect = CGRectMake(self.pdfViewCtrl.bounds.size.width / 2, self.pdfViewCtrl.bounds.size.height / 2, 10, 10);
        }
        *rect = showRect;
    }
}

- (void)dismiss {
    [self dismissWithCompletionBlock:nil];
}

- (void)dismissWithCompletionBlock:(void (^ __nullable)(void))completion {

    if (DEVICE_iPHONE || [_extensionsManager fs_currentSceneiPadHorizontalCompact]) {
        [self dismissPropertyViewWithPopStyle:NO splitScreen:NO completionBlock:completion];
    }else{
        [self dismissPropertyViewWithPopStyle:YES splitScreen:NO completionBlock:completion];
    }
}

- (void)dismissPropertyViewWithPopStyle:(BOOL)popStyle splitScreen:(BOOL)splitScreen completionBlock:(void (^ __nullable)(void))completion{
    if (!self.isShowing) return;
    if (!popStyle) {

        [UIView animateWithDuration:ANIMATE_DURATION_NORMAL animations:^{
            if (self.layoutsContainer) {
                [self.layoutsContainer mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.centerX.fs_equalToSuperview().priorityHigh();
                    if (DEVICE_iPHONE) {
                        make.width.mas_equalTo(self.maxWidth).priorityHigh();
                    }else{
                        make.width.fs_equalToSuperview().priorityHigh();
                    }
                    make.height.lessThanOrEqualTo(self.mas_height).offset(-64.f);
                    make.top.equalTo(self.layoutsContainer.superview.mas_bottom);
                }];
            }
            [self layoutIfNeeded];
        } completion:^(BOOL finished) {
            if (finished) {
                [self removeFromSuperview];
                if (!splitScreen) {
                    [self.layoutsStackView.subviews makeObjectsPerformSelector:@selector(removeFromSuperview)];
                    self.currentProperties = nil;
                }
                if (completion) {
                    completion();
                }
            }
        }];
        FSAnnot *annot = _extensionsManager.currentAnnot;
        if (annot && ![annot isEmpty]) {
            if ([self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_SINGLE || annot.pageIndex == [self.pdfViewCtrl.currentDoc getPageCount] - 1) {
                [self.pdfViewCtrl setBottomOffset:0];
            }
        }
        [self.layoutsContainer removeGestureRecognizer:self.dismissPan];
    }else{
        [self addLayoutsContainer];
        if (!splitScreen) {
            [self.layoutsStackView.subviews makeObjectsPerformSelector:@selector(removeFromSuperview)];
            self.currentProperties = nil;
        }

        if (self.popViewCtr) {
            [self.popViewCtr dismissViewControllerAnimated:NO completion:^{
                self.popViewCtr = nil;
                if (completion) {
                    completion();
                }
            }];
        }

    }
    
    _isShowing = NO;
    if (!splitScreen) {
        self.arrowDirection = UIPopoverArrowDirectionUnknown;
        if (self.delegate  && [self.delegate respondsToSelector:@selector(propertyMainViewDidDismiss:)]) {
            [self.delegate propertyMainViewDidDismiss:self];
        }
    }
}

#pragma  mark -FSPropertyLayoutDelegate
- (void)propertyLayout:(FSPropertyLayout *)propertyLayout changedProperty:(FSPropertyType)propertyType fromOldValue:(NSValue *)oldValue toValue:(NSValue *)newValue{
    if (propertyType == FSPropertyTypePropertyColor) {
        if ([self.currentProperties containsObject:@(FSPropertyTypeOpacity)]) {
            [self.opacityLayout setCurrentColor:[UIColor colorWithRGB:[(NSNumber *) newValue unsignedIntValue]]];
        }
        if ([self.currentProperties containsObject:@(FSPropertyTypeLineWidth)]) {
            [self.lineWidthLayout setCurrentColor:[UIColor colorWithRGB:[(NSNumber *) newValue unsignedIntValue]]];
        }
    }
    if (propertyType == FSPropertyTypeBorderColor) {
        if ([self.currentProperties containsObject:@(FSPropertyTypeOpacity)]) {
            [self.opacityLayout setCurrentColor:[UIColor colorWithRGB:[(NSNumber *) newValue unsignedIntValue]]];
        }
        if ([self.currentProperties containsObject:@(FSPropertyTypeBorderWidth)]) {
            [self.borderWidthLayout setCurrentColor:[UIColor colorWithRGB:[(NSNumber *) newValue unsignedIntValue]]];
        }
    }
    if (self.delegate && [self.delegate respondsToSelector:@selector(propertyMainView:changedProperty:fromOldValue:toValue:)]) {
        [self.delegate propertyMainView:self changedProperty:propertyType fromOldValue:oldValue toValue:newValue];
    }
}

#pragma mark -IRotationEventListener
- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation {
    if (DEVICE_iPHONE || [_extensionsManager fs_currentSceneiPadHorizontalCompact]) {
        [UIView animateWithDuration:ANIMATE_DURATION_NORMAL animations:^{
        } completion:^(BOOL finished) {
            if (finished) {
                [self scrollToSelectedAnnot];
            }
        }];
    }
}

- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration {

}


#pragma mark -IWindowFrameChangedListener
- (void)onWindowFrameChanged:(CGRect)newFrame horizontalSizeClassChanged:(BOOL)changed{
    if (DEVICE_iPAD && changed) {
        if ([_extensionsManager fs_currentSceneHorizontalCompact]) {
            if (self.popViewCtr && self.isShowing) {
                [self dismissPropertyViewWithPopStyle:YES splitScreen:YES completionBlock:nil];
                [self.popViewCtr dismissViewControllerAnimated:NO completion:^{
                    self.popViewCtr = nil;
                    [self showPropertyMainView:self.devicePos];
                }];
            }
        }else{
            if (self.superview && self.isShowing) {
                [self dismissPropertyViewWithPopStyle:NO splitScreen:YES completionBlock:^{
                    if (self.devicePos.annot) {
                        CGRect rect = [self.pdfViewCtrl convertPdfRectToPageViewRect:self.devicePos.annot.fsrect pageIndex:self.devicePos.annot.pageIndex];
                        CGRect showRect = [self.pdfViewCtrl convertPageViewRectToDisplayViewRect:rect pageIndex:self.devicePos.annot.pageIndex];
                        CGRect pdfRect = self.pdfViewCtrl.bounds;
                        if (DEVICE_iPAD && ![self->_extensionsManager fs_currentSceneiPadHorizontalCompact]) {
                            if (showRect.size.width > pdfRect.size.width/2 && showRect.size.height > pdfRect.size.height/2) {
                                showRect.size.width = pdfRect.size.width/2;
                                showRect.size.height = pdfRect.size.height/2;
                            }
                        }
                        self.devicePos.padRect = showRect;
                    }
                    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                        [self showPropertyMainView:self.devicePos];
                    });
                    
                }];
            }
        }
    }
}

@end
