/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSRotationLayout.h"
FSSYNTH_INTERFACE_IMP_CLASS(_FSPropertyRotationStackView, UIStackView)

@interface FSRotationLayout ()
@property (nonatomic, strong) _FSPropertyRotationStackView *rotationStackView;
@property (nonatomic, strong) FSItem *currentItem;
@property (nonatomic, assign) int rotation;
@end

@implementation FSRotationLayout
- (instancetype)initWithRotation:(int)rotation{
    self = [super initWithPropertyType:FSPropertyTypeRotation];
    if (self) {
        _rotationStackView = [_FSPropertyRotationStackView new];
        _rotationStackView.distribution = UIStackViewDistributionFillEqually;
        _rotationStackView.spacing = 37.f;
        [self addSubview:_rotationStackView];
        
        [_rotationStackView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(self.fs_mas_left).offset(50.f);
            make.right.equalTo(self.fs_mas_right).offset(-50.f);
            make.top.fs_equalToSuperview().offset(13.f);
            make.bottom.fs_equalToSuperview().offset(-13.f);
            make.height.offset(28.f);
        }];
        
        for (int i = 0; i < 4; i ++) {
           
            FSItem *degreeItem = [[FSItem alloc] initWithRrelation:FSImageTextRelationBottom];
            [degreeItem fs_setLayerCornerRadius:LAYER_BORDER_WIDTH_6];
            degreeItem.tag = i;
            
            @weakify(self)
            degreeItem.onTapClick = ^(FSItem * _Nonnull _item) {
                @strongify(self)
                if (self.currentItem != _item) {
                    NSNumber *degree = self.currentItem ? @(degreeItem.tag * 90) : nil;
                    [self.currentItem fs_setLayerBorderWidth:0 borderColor:nil];
                    self.currentItem.selected = NO;
                    self.currentItem = _item;
                    self.currentItem.selected = YES;
                    
                    [BindThemeKeySel(self.currentItem, fs_setLayerBorderWidth:borderColor:) fs_updateThemeCompleted:^(FSItem * _Nonnull target, SEL  _Nonnull sel) {
                        if (target.selected) {
                            [target fs_setLayerBorderWidth:1.f borderColor:target.fs_primaryColor];
                        }else{
                            [target fs_setLayerBorderWidth:1.f borderColor:target.fs_dividerColor_P2];
                        }
                    }];
                    
                    if (self.delegate && [self.delegate respondsToSelector:@selector(propertyLayout:changedProperty:fromOldValue:toValue:)]) {
                        [self.delegate propertyLayout:self changedProperty:self.propertyType fromOldValue:degree toValue:@(degreeItem.tag * 90)];
                    }
                    
                }
            };
            
            degreeItem.titleLabel.textAlignment = NSTextAlignmentCenter;
            [degreeItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
            degreeItem.font = self.fs_standardGeneralFont_GF2;
            [degreeItem setTitle:[NSString stringWithFormat:@"%d°",(90 * i)] forState:UIControlStateNormal];
            
            if (i == rotation/90) {
                self.currentItem = degreeItem;
            }
            
            [_rotationStackView addArrangedSubview:degreeItem];
        }
        self.currentItem.selected = YES;
        [BindThemeKeySel(self.currentItem, fs_setLayerBorderWidth:borderColor:) fs_updateThemeCompleted:^(FSItem * _Nonnull target, SEL  _Nonnull sel) {
            if (target.selected) {
                [target fs_setLayerBorderWidth:1.f borderColor:target.fs_primaryColor];
            }else{
                [target fs_setLayerBorderWidth:0 borderColor:nil];
            }
        }];
    }
    return self;
}

- (void)addContentView{};

- (void)setRotation:(int)rotation{
    for (FSItem *degreeItem in _rotationStackView.arrangedSubviews) {
        if (rotation/90 == degreeItem.tag) {
            degreeItem.onTapClick(degreeItem);
            break;
        }
    }
}

@end
