/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSShapeLayout.h"
#import "FSTitleEstimatedTableViewCell.h"

FSSYNTH_INTERFACE_IMP_CLASS(_FSShapeLayoutSmallContentView, UIView)
FSSYNTH_INTERFACE_IMP_CLASS(_FSShapeLayoutListContentView, UIView)

@interface FSShapeLayout (){
    NSArray<FSLayoutDataItem *> *_dataSource;
}
@end
@implementation FSShapeLayout


- (instancetype)initWithPropertyType:(FSPropertyType)propertyType{
    return [super initWithPropertyType:propertyType];
}

- (NSArray<FSLayoutDataItem *> *)dataSource{
    if (!_dataSource) {
        if (self.propertyType == FSPropertyTypeShapeNote){
            _dataSource = @[
                [[FSLayoutDataItem alloc] initWithTitle:FSLocalizedForKey(@"kIconCheck") imageName:@"read_property_note_checkmark"],
                [[FSLayoutDataItem alloc] initWithTitle:FSLocalizedForKey(@"kIconCircle") imageName:@"read_property_note_oval"],
                [[FSLayoutDataItem alloc] initWithTitle:FSLocalizedForKey(@"kIconComment") imageName:@"read_property_note_comment"],
                [[FSLayoutDataItem alloc] initWithTitle:FSLocalizedForKey(@"kIconCross") imageName:@"read_property_note_cross"],
                [[FSLayoutDataItem alloc] initWithTitle:FSLocalizedForKey(@"kIconHelp") imageName:@"read_property_note_help"],
                [[FSLayoutDataItem alloc] initWithTitle:FSLocalizedForKey(@"kIconInsert") imageName:@"read_property_note_Insert"],
                [[FSLayoutDataItem alloc] initWithTitle:FSLocalizedForKey(@"kIconKey") imageName:@"read_property_note_key"],
                [[FSLayoutDataItem alloc] initWithTitle:FSLocalizedForKey(@"kIconNewParagraph") imageName:@"read_property_note_new_paragraph"],
                [[FSLayoutDataItem alloc] initWithTitle:FSLocalizedForKey(@"kNote") imageName:@"read_property_note_note"],
                [[FSLayoutDataItem alloc] initWithTitle:FSLocalizedForKey(@"kIconParagraph") imageName:@"read_property_note_paragaph"],
                [[FSLayoutDataItem alloc] initWithTitle:FSLocalizedForKey(@"kIconRightArrow") imageName:@"read_property_note_right_arrow"],
                [[FSLayoutDataItem alloc] initWithTitle:FSLocalizedForKey(@"kIconRightPointer") imageName:@"read_property_note_right_pointer"],
                [[FSLayoutDataItem alloc] initWithTitle:FSLocalizedForKey(@"kIconStar") imageName:@"read_property_note_star"],
                [[FSLayoutDataItem alloc] initWithTitle:FSLocalizedForKey(@"kIconUpArrow") imageName:@"read_property_note_up_arrow"],
                [[FSLayoutDataItem alloc] initWithTitle:FSLocalizedForKey(@"kIconUpLeftArrow") imageName:@"read_property_note_upleft_arrow"],
            ];
        }else{
            _dataSource =@[
                [[FSLayoutDataItem alloc] initWithTitle:FSLocalizedForKey(@"kGraph") imageName:@"read_property_attachment_graph"],
                [[FSLayoutDataItem alloc] initWithTitle:FSLocalizedForKey(@"kPushpin") imageName:@"read_property_attachment_pushpin"],
                [[FSLayoutDataItem alloc] initWithTitle:FSLocalizedForKey(@"kPaperclip") imageName:@"read_property_attachment_paperclip"],
                [[FSLayoutDataItem alloc] initWithTitle:FSLocalizedForKey(@"kTag") imageName:@"read_property_attachment_tag"]
            ];
        }
    }
    return _dataSource;
}

- (int)currentIcon{
    return (int)self.currentIndexPath.row;
}
- (void)setCurrentIcon:(int)currentIcon{
    NSIndexPath *indexPath = [NSIndexPath indexPathForRow:currentIcon inSection:0];
    [self tableView:self.attachedSubview didSelectRowAtIndexPath:indexPath];
}

- (void)addContentViewSubViews{
    [super addContentViewSubViews];
    self.leftLab.text = FSLocalizedForKey(@"kShape");
}

- (void)initBackItem{
    [self.backItem setTitle:FSLocalizedForKey(@"kShape") forState:UIControlStateNormal];
}

- (NSValue *)valueFromLayoutDataItem:(FSLayoutDataItem *)dataItem old:(BOOL)old{
    NSUInteger currentIcon = [self.dataSource indexOfObject:dataItem];
    if (currentIcon != NSNotFound) {
        return @(currentIcon);
    }
    return nil;
}

#pragma mark - UITableViewDelegate
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath{
    FSTitleEstimatedTableViewCell* cell = (FSTitleEstimatedTableViewCell*)[super tableView:tableView cellForRowAtIndexPath:indexPath];
    FSLayoutDataItem *item = [self dataForIndexPath:indexPath];
    if (item.imageName && cell.leftView) {
        if ([item.imageName isEqualToString:@"read_property_attachment_tag"]) {
            cell.leftView.transform = CGAffineTransformMakeRotation(3*M_PI_2);
        }else{
            cell.leftView.transform = CGAffineTransformMakeRotation(0);
        }
    }
    return cell;
}

@end

