/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSTextLayout.h"
#import "FSTextViewPopoverViewController.h"


@implementation FSTextLayout

- (instancetype)initWithPropertyType:(FSPropertyType)propertyType
{
    if (self = [super initWithPropertyType:propertyType]) {
        self.unfoldItem.accessibilityIdentifier = @"text_change_id";
    }
    return self;
}

- (void)addContentViewSubViews{
    [super addContentViewSubViews];
    if (self.propertyType == FSPropertyTypeTextForm) {
        self.leftLab.text = FSLocalizedForKey(@"kName");
    }else{
        self.leftLab.text = FSLocalizedForKey(@"kText");
    }
    
    [self.unfoldItem.titleLabel setContentCompressionResistancePriority:UILayoutPriorityFittingSizeLevel forAxis:UILayoutConstraintAxisHorizontal];
    
    [self.unfoldItem mas_updateConstraints:^(MASConstraintMaker *make) {
        make.left.greaterThanOrEqualTo(self.leftLab.mas_right).offset(20.f);
    }];
    @weakify(self)
    self.unfoldItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        
        [self.extensionsManager.propertyMainView dismissWithCompletionBlock:^{
            FSTextViewPopoverViewController *textViewVC =  [self.extensionsManager.viewCtrlManager createViewCtrlType:FSViewCtrlTypeTextViewPopover params:self.extensionsManager.toolbarManager.mainTopbar.toolbar];
            textViewVC.title = self.leftLab.text;
            textViewVC.text = _item.currentTitle;
            textViewVC.extensionsMgr = self.extensionsManager;
            if (self.propertyType == FSPropertyTypeTextForm) {
                textViewVC.okItemEnabled = ^BOOL(NSString *text) {
                    if (![text stringByReplacingOccurrencesOfString:@" " withString:@""].length || ![text stringByReplacingOccurrencesOfString:@"." withString:@""].length || ![text stringByReplacingOccurrencesOfString:@"…" withString:@""].length) {
                       return NO;
                    }else{
                        if ([text hasPrefix:@"."] || [text hasSuffix:@"."] || [text containsString:@".."]) {
                            return NO;
                        }
                         while ([text hasSuffix:@"."]) {
                             text = [text substringToIndex:text.length - 1];
                        }
                    }
                    return !(_item.currentTitle.hash == text.hash);
                };
            }
            [textViewVC setDidClickedOK:^(NSString *text) {
                self.text = text;
            }];
            [[self fs_currentSceneTopMostViewController] presentViewController:textViewVC animated:YES completion:nil];
        }];
    };
}

- (void)setText:(NSString *)text{
    if (self.delegate && [self.delegate respondsToSelector:@selector(propertyLayout:changedProperty:fromOldValue:toValue:)] ) {
        [self.delegate propertyLayout:self changedProperty:self.propertyType fromOldValue:[NSValue valueWithNonretainedObject:_text] toValue:[NSValue valueWithNonretainedObject:text]];
    }
    _text = text;
    self.unfoldItem.titleLabel.lineBreakMode = NSLineBreakByTruncatingMiddle;
    [self.unfoldItem setTitle:text forState:UIControlStateNormal];
}
@end
