/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "ReadingBookmarkModule.h"
#import "PanelController+private.h"
#import "FSPermissionUtilty.h"

@interface ReadingBookmarkModule () {
    UIExtensionsManager *__weak _extensionsManager;
}
@end

@implementation ReadingBookmarkModule

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        [self loadModule];
    }
    return self;
}

- (void)loadModule {
    //Adding reading bookmark button.
    if (_extensionsManager.config.loadReadingBookmark) {
        self.bookmarkItem = [_extensionsManager.toolbarManager addMainItemWithType:FSMainToolbarItemTypeReadingBookmark];
        @weakify(self)
        [BindThemeKeySel(self.bookmarkItem.item, setImage:forState:) fs_updateThemeCompleted:^(FSItem * _Nonnull _item, SEL  _Nonnull sel) {
            @strongify(self)
            if(DEVICE_iPHONE || [self->_extensionsManager fs_currentSceneiPadHorizontalCompact]){
                [_item setImage:FSImageNamedAndPrimaryColor(@"read_bookmark_select") forState:UIControlStateSelected];
                [_item setImage:FSImageNamedAndNormalColor(@"read_bookmark_gray") forState:UIControlStateNormal];
            }else{
                [_item setImage:FSImageNamedAndColor(@"read_bookmark_select",self.fs_iconColor_I7) forState:UIControlStateSelected];
                [_item setImage:FSImageNamedAndColor(@"read_bookmark_gray", self.fs_iconColor_I6) forState:UIControlStateNormal];
            }
        }];
        
        self.bookmarkItem.item.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            if(DEVICE_iPHONE || [self->_extensionsManager fs_currentSceneiPadHorizontalCompact]){
                [self.bookmarkItem.item setImage:FSImageNamedAndPrimaryColor(@"read_bookmark_select") forState:UIControlStateSelected];
            }else{
                [self.bookmarkItem.item setImage:FSImageNamedAndColor(@"read_bookmark_select",self.fs_iconColor_I7) forState:UIControlStateSelected];
            }
            
            if (self->_extensionsManager.isFullScreen) {
                [self->_extensionsManager setFullScreen:NO];
            }
            if ([self->_extensionsManager currentAnnot]) {
                [self->_extensionsManager setCurrentAnnot:nil];
            }
            FSPDFViewCtrl *pdfViewCtrl = self->_extensionsManager.pdfViewCtrl;
            int currentPage = [pdfViewCtrl getCurrentPage];
            if ([pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO || [pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_LEFT || [pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_LEFT || [pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_MIDDLE) {
                currentPage = currentPage / 2 * 2;
            }
            FSReadingBookmark *bookmark = [Utility getReadingBookMarkAtPage:pdfViewCtrl.currentDoc page:currentPage];
            if (!bookmark || [bookmark isEmpty]) {
                @try {
                    [pdfViewCtrl.currentDoc insertReadingBookmark:-1 title:[NSString stringWithFormat: FSLocalizedForKey(@"kPage"), currentPage + 1] dest_page_index:currentPage];
                } @catch (NSException *exception) {
                    return;
                }
                self.bookmarkItem.item.selected = YES;
            } else {
                [pdfViewCtrl.currentDoc removeReadingBookmark:bookmark];
                self.bookmarkItem.item.selected = NO;
            }
            [[NSNotificationCenter defaultCenter] postNotificationName:FSBookmarkUpdateNotification object:nil];
            [self->_extensionsManager supportsMultipleScenesOp:^(UIExtensionsManager * _Nonnull ext) {
                [ext.panelController reloadReadingBookmarkPanel];
            }];
            self->_extensionsManager.isDocModified = YES;
            [self->_extensionsManager onUIInteractElementClicked:FSUIElementReadingBookmark];
        };

        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(bookmarkUpdateNotificationButtonState) name:FSBookmarkUpdateNotification object:nil];
        [_extensionsManager.pdfViewCtrl registerDocEventListener:self];
        [_extensionsManager.pdfViewCtrl registerPageEventListener:self];
        [_extensionsManager.pdfViewCtrl registerLayoutEventListener:self];
    }
}

- (NSString *)getName {
    return @"ReadingBookmark";
}

- (void)bookmarkUpdateNotificationButtonState {
    FSPDFViewCtrl* pdfViewCtrl = _extensionsManager.pdfViewCtrl;
    int currentPage = [pdfViewCtrl getCurrentPage];
    if ([pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO || [pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_LEFT || [pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_LEFT || [pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_MIDDLE) {
        currentPage = currentPage / 2 * 2;
    }
    FSReadingBookmark *bookmark = [Utility getReadingBookMarkAtPage:pdfViewCtrl.currentDoc page:currentPage];
    self.bookmarkItem.item.selected = bookmark ? YES : NO;
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

#pragma mark IDocEventListener
- (void)onDocOpened:(FSPDFDoc *)document error:(int)error {
    if (error != FSErrSuccess) {
        return;
    }
    [self bookmarkUpdateNotificationButtonState];
    dispatch_async(dispatch_get_main_queue(), ^{
        self.bookmarkItem.item.enabled = [FSPermissionUtilty canAssemble:self->_extensionsManager];
    });
}

#pragma mark IPageEventListener
- (void)onPageChanged:(int)oldIndex currentIndex:(int)currentIndex {
    [self bookmarkUpdateNotificationButtonState];
}

#pragma mark ILayeroutEventListener
- (void)onLayoutModeChanged:(PDF_LAYOUT_MODE)oldLayoutMode newLayoutMode:(PDF_LAYOUT_MODE)newLayoutMode {
    [self bookmarkUpdateNotificationButtonState];
}

@synthesize annotType;

@end
