/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "ReflowPageManager.h"


@implementation ReflowPageManager

- (instancetype)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = _extensionsManager.pdfViewCtrl;
        _reflowPages = [[NSMutableDictionary alloc] init];
    }
    return self;
}

- (FSReflowPage *)getPage:(int)pageIndex {
    NSNumber *key = @(pageIndex);
   return [_reflowPages objectForKey:key];
}

- (FSReflowPage *)getParsedPage:(int)pageIndex {
    FSReflowPage *reflowPage;
    NSNumber *key = @(pageIndex);
    reflowPage = [_reflowPages objectForKey:key];
    FSPDFPage *page = [_pdfViewCtrl.currentDoc getPage:pageIndex];
    if (page && ![page isEmpty]){
        if (!page.isParsed){
            [page startParse:FSPDFPageParsePageNormal pause:nil is_reparse:NO];
        }
    }
    if (reflowPage == NULL) {
        reflowPage = [[FSReflowPage alloc] initWithPage:page];
        [_reflowPages setObject:reflowPage forKey:key];
    }
    if(!reflowPage.isParsed){
        float scale = [_pdfViewCtrl getReflowPageScale];
        float ppi = [_pdfViewCtrl getUIDevicePPI];
        float ppiscale = 72.0f / ppi;
        int zoom  = MIN(MAX(100, scale / ppiscale * 100), 1000);
        int width =  [_pdfViewCtrl getPageViewWidth:pageIndex] * [UIScreen mainScreen].scale;
        if (!reflowPage) {
            return nil;
        }
        [reflowPage setScreenSize:(float) width height:FLT_MAX];
        [reflowPage setLineSpace:1.5f];
        [reflowPage setZoom:zoom];
        [reflowPage setImageScale: zoom / 100.f];
        [reflowPage setParseFlags:[_pdfViewCtrl getReflowMode]];
        [reflowPage setTopSpace: 5 * zoom / 100.f];

        FSProgressive *ret = [reflowPage startParse:nil];
        if (ret != nil) {
            FSProgressiveState progress = [ret resume];
            while (progress == FSProgressiveToBeContinued) {
                progress = [ret resume];
            }
            if (progress != FSProgressiveFinished) {
                return nil;
            }
        }

    }
   
    return reflowPage;
}

- (void)removePage:(int)pageIndex {
    NSNumber *key = @(pageIndex);
    [_reflowPages removeObjectForKey:key];
}

- (void)clear {
    [_reflowPages removeAllObjects];
}


@end
