/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "SearchModule.h"
#import "SearchResult.h"
#import "SearchViewController.h"
#import "StringDrawUtil.h"
#import "FSEmptyDataSetView.h"
#import "FSSearchSelectButton.h"
#import "RedactModule+Private.h"
#import "RedactAnnotHandler.h"
#import "PageNavigationModule.h"

#define STYLE_CELL_WIDTH 300
#define STYLE_PAGE_SUMMARY_HEIGHT 24
#define STYLE_PAGE_SUMMARY_LEFT 16
#define STYLE_PAGE_SUMMARY_TOP 6
#define STYLE_PAGE_SUMMARY_WIDTH 200
#define STYLE_PAGE_SUMMARY_WIDTH_IPHONE 260
#define STYLE_PAGE_SUMMARY_LABEL_HEIGHT (STYLE_PAGE_SUMMARY_HEIGHT - STYLE_PAGE_SUMMARY_TOP)
#define STYLE_PAGE_SUMMARY_RIGHT_LEFT 107
#define STYLE_PAGE_SUMMARY_RIGHT_LEFT_IPHONE 47
#define STYLE_PAGE_SUMMARY_FONT [UIFont boldSystemFontOfSize:18.0]
#define STYLE_INFO_LEFT 16
#define STYLE_INFO_TOP 8
#define STYLE_INFO_FONT [UIFont systemFontOfSize:13.0]
#define SEARCH_LABEL_FONT 12.0f
#define TOTALVIEW_MIN_HEIGHT 24
#define TOTALVIEW_MAX_HEIGHT 44

#define SEARCH_BAR_TEXT [self.searchBar.text stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]] //remove prefixed and suffixed whitespace

@interface UIFSSearchCancelCallback : NSObject<FSSearchCancelCallback>
@property (nonatomic, assign) BOOL cancelNow;
-(BOOL)needToCancelNow;
@end

@implementation UIFSSearchCancelCallback
- (instancetype)init {
    self = [super init];
    if (self) {
        self.cancelNow = NO;
    }
    return self;
}

-(BOOL)needToCancelNow
{
    return self.cancelNow;
}
@end


@interface SearchModule () <IRotationEventListener, UIPopoverPresentationControllerDelegate,IWindowFrameChangedListener>
{
    dispatch_semaphore_t _wait;
}

@property (nonatomic, weak) FSPDFViewCtrl *pdfViewCtrl;
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;

@property (nonatomic, strong) FSToolbar *topBar;
@property (nonatomic, strong) FSItem *filterItem;
@property (nonatomic, strong) UISearchBar *searchBar;
@property (nonatomic, strong) FSItem *cancelItem;

@property (nonatomic, strong) FSToolbar *bottomBar;
@property (nonatomic, strong) TbBaseBar *bottomBaseBar;
@property (nonatomic, strong) TbBaseItem *previousItem;
@property (nonatomic, strong) TbBaseItem *nextItem;
@property (nonatomic, strong) TbBaseItem *showListItem;

@property (nonatomic, strong) UIControl *maskView;
@property (nonatomic, strong) UIButton *allSelectBtn;
@property (nonatomic, strong) UILabel *foundLable;
@property (nonatomic, strong) UIView *totalView;
@property (nonatomic, strong) UITableView *tableView;
@property (nonatomic, strong) UIButton *redactionBtn;
@property (nonatomic, strong) UIView *bottomBGView;

@property (nonatomic, assign) BOOL topBaseBarHidden;
@property (nonatomic, assign) BOOL bottomBaseBarHidden;
@property (nonatomic, assign) BOOL tableviewHidden;
@property (nonatomic, assign) BOOL isScroll;

@property (nonatomic, strong) NSMutableArray *arraySearch;
@property (nonatomic, strong) NSMutableArray *arrayTempSearch;

@property (nonatomic, strong) UIViewController *searchbyController;
@property (nonatomic, strong) FSViewPopoverController *searchByPopoverCtrl;
//@property (nonatomic, strong) FSNavigationController *searchByNavCtrl;

@property (nonatomic, strong) NSOperationQueue *searchOPQueue;

@property (nonatomic, strong) NSArray *needDrawRects;
@property (nonatomic, assign) int needPageIndex;

@property (nonatomic, strong) SearchInfo *selectedSearchInfo;
@property (nonatomic, strong) SearchResult *selectedSearchResult;
@property (nonatomic, assign) BOOL isKeyboardShowing;

@property (nonatomic, assign) BOOL onSearchState;

@property (nonatomic, assign) unsigned int textSearchSearchFlags;
@property (nonatomic, strong) NSString *textSearchKeyWords;
@property (nonatomic, strong) UIFSSearchCancelCallback *searchCancelCallBack;
@property (nonatomic, strong) FSEmptyDataSetView *emptyDataSetView;
@property (nonatomic, strong) NSMutableDictionary *selectDict;
@property (nonatomic, assign) int totalCount;
@property (atomic, assign) BOOL isSearching;
@property (nonatomic, strong) UIBarButtonItem *searchBtnitem;
@property (nonatomic, strong) FSItem *searchItem;
@property (nonatomic, strong) NSTimer *timer;
@property (nonatomic, assign) int currentIndex;
@property (nonatomic, assign) int searchCurrentPage;
@property (nonatomic, assign) int width;




@end

@implementation SearchModule {
}

- (NSString *)getName {
    return @"Search";
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        self.pdfViewCtrl = extensionsManager.pdfViewCtrl;
        [extensionsManager registerRotateChangedListener:self];
        [extensionsManager registerWindowFrameChangedListener:self];
        _extensionsManager = extensionsManager;
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(willRotate) name:UIApplicationWillChangeStatusBarOrientationNotification object:nil];
        _textSearchSearchFlags = FSTextSearchSearchNormal;
        self.textSearchKeyWords = nil;
        self.searchCancelCallBack = nil;
    }

    if ([UIBarButtonItem respondsToSelector:@selector(appearanceWhenContainedIn:)]) {
        [[UIBarButtonItem appearanceWhenContainedInInstancesOfClasses:@[ [UISearchBar class] ]] setTintColor:self.fs_backgroundColor_B1];
    }
    [self loadModule];
    self.width = self.pdfViewCtrl.bounds.size.width;
    return self;
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];

    [self.extensionsManager unregisterRotateChangedListener:self];
    [self.extensionsManager unregisterWindowFrameChangedListener:self];
    [self.pdfViewCtrl unregisterDrawEventListener:self];
    [self.pdfViewCtrl unregisterScrollViewEventListener:self];
    [self.pdfViewCtrl unregisterGestureEventListener:self];
    [self.pdfViewCtrl unregisterDocEventListener:self];
}

- (void)loadModule {
    [self.pdfViewCtrl registerDrawEventListener:self];
    [self.pdfViewCtrl registerScrollViewEventListener:self];
    [self.pdfViewCtrl registerGestureEventListener:self];
    [self.pdfViewCtrl registerDocEventListener:self];

    self.arraySearch = [NSMutableArray array];
    self.arrayTempSearch = [NSMutableArray array];
   
    self.searchOPQueue = [[NSOperationQueue alloc] init];
    self.searchOPQueue.maxConcurrentOperationCount = 1;
    _selectedSearchInfo = nil;
    _selectedSearchResult = nil;
    self.selectDict = [NSMutableDictionary dictionary];

    self.maskView = [[UIControl alloc] initWithFrame:CGRectMake(0, 0, CGRectGetWidth(self.pdfViewCtrl.bounds), CGRectGetHeight(self.pdfViewCtrl.bounds))];
    self.maskView.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin | UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleTopMargin | UIViewAutoresizingFlexibleBottomMargin | UIViewAutoresizingFlexibleHeight;

    [self initTopToolBar];
    [self initBottomToolBar];

    UIView *redactionBtnBg = [[UIView alloc] init];
    redactionBtnBg.backgroundColor = [UIColor redColor];
    redactionBtnBg.userInteractionEnabled = YES;

    self.redactionBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    self.redactionBtn.backgroundColor = self.fs_backgroundColor_B2;
    [self.redactionBtn addTarget:self action:@selector(redactPress:) forControlEvents:UIControlEventTouchUpInside];
    [self.redactionBtn setTitle:FSLocalizedForKey(@"kSearchRedaction") forState:UIControlStateNormal];
    self.redactionBtn.titleLabel.font = [UIFont systemFontOfSize:15];
    [BindThemeKeySel(self.redactionBtn, setTitleColor:) fs_updateThemeCompleted:^(UIButton  *_Nonnull target, SEL  _Nonnull sel) {
        [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
    }];
    [self.redactionBtn setTitleColor:self.fs_iconColor_I2 forState:UIControlStateDisabled];
    self.redactionBtn.clipsToBounds = YES;
    self.redactionBtn.titleLabel.numberOfLines = 0;
    self.redactionBtn.titleLabel.textAlignment = NSTextAlignmentCenter;
    if (self.pdfViewCtrl.fs_currentSceneKeyWindow.safeAreaInsets.bottom > 0) {
        [self.redactionBtn setTitleEdgeInsets:UIEdgeInsetsMake(-25, 0, 0, 0)];
    }

    self.tableView = [[UITableView alloc] initWithFrame:CGRectMake(CGRectGetWidth(self.pdfViewCtrl.bounds) - 300, 94, 300, CGRectGetHeight(self.pdfViewCtrl.bounds) - 94) style:UITableViewStylePlain];
    self.tableView.separatorStyle = UITableViewCellSeparatorStyleNone;
    if ([self.tableView respondsToSelector:@selector(setSeparatorInset:)]) {
        [self.tableView setSeparatorInset:UIEdgeInsetsMake(0, 10, 0, 0)];
    }

    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.backgroundColor = self.fs_backgroundColor_B1;

    [self.pdfViewCtrl addSubview:self.tableView];
    [self.pdfViewCtrl addSubview:self.redactionBtn];
    [self.pdfViewCtrl addSubview:redactionBtnBg];
    [self.pdfViewCtrl insertSubview:redactionBtnBg belowSubview:self.redactionBtn];

    [self.redactionBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.equalTo(self.redactionBtn.superview);
        make.left.width.equalTo(self.tableView);
        make.height.mas_equalTo(0);
    }];

    [redactionBtnBg mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.top.right.bottom.equalTo(self.redactionBtn);
    }];
    [self.tableView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.equalTo(self.redactionBtn.mas_top);
        make.left.mas_equalTo(self.tableView.superview.mas_right);
        if (DEVICE_iPAD && ![self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            make.width.mas_equalTo(300);
        }else{
            make.width.equalTo(self.tableView.superview);
        }

    }];

    [self initTotalView];

    self.topBaseBarHidden = YES;
    self.bottomBaseBarHidden = YES;
    self.tableviewHidden = YES;


    //search button.
    FSMainToolbarItem *searchItem  = [self.extensionsManager.toolbarManager addMainItemWithType:FSMainToolbarItemTypeSearch];
    [searchItem.item addTarget:self action:@selector(onClickSearchItem:) forControlEvents:UIControlEventTouchUpInside];

    [BindThemeKeySel(self, reloadEmptyDataSet) fs_updateThemeCompleted:^(SearchModule * _Nonnull target, SEL  _Nonnull sel) {
        dispatch_main_async_safe(^{
            if (target.emptyDataSetView.superview) {
                [target reloadEmptyDataSet];
            }
        });
    }];

    _wait = dispatch_semaphore_create(0);
}

- (void)initTotalView{

    self.allSelectBtn = [UIButton buttonWithType:UIButtonTypeCustom];
    [self.allSelectBtn setImage:FSImageNamedAndNormalColor(@"common_unselect") forState:UIControlStateNormal];
    [BindThemeKeySel(self.allSelectBtn, setImage:forState:) fs_updateThemeCompleted:^(UIButton*  _Nonnull target, SEL  _Nonnull sel) {
        [target setImage:FSImageNamedAndPrimaryColor(@"common_select") forState:UIControlStateSelected];
    }];
    [self.allSelectBtn addTarget:self action:@selector(allSelectPress:) forControlEvents:UIControlEventTouchUpInside];



    self.foundLable = [[UILabel alloc] initWithFrame:CGRectMake(STYLE_PAGE_SUMMARY_LEFT, 0, 300, 24)];
    self.foundLable.textColor = self.fs_textColor_T3;
    self.foundLable.font = [UIFont systemFontOfSize:12.0f];
    self.foundLable.textAlignment = NSTextAlignmentLeft;
    self.foundLable.backgroundColor = [UIColor clearColor];

    self.totalView = [[UIView alloc] initWithFrame:CGRectMake(0, 64, 300, 24)];
    self.totalView.backgroundColor = self.fs_backgroundColor_B2;

    [self.pdfViewCtrl addSubview:self.totalView];
    [self.totalView addSubview:self.foundLable];
    [self.totalView addSubview:self.allSelectBtn];
    [self.totalView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.topBar.mas_bottom);
        make.bottom.equalTo(self.tableView.mas_top);
        make.height.mas_equalTo(@24);
        make.width.left.mas_equalTo(self.tableView);
    }];
    [self.allSelectBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.totalView.fs_mas_left).offset(8);
        make.width.mas_equalTo(0);
        make.height.mas_equalTo(40);
        make.centerY.equalTo(self.allSelectBtn.superview);
    }];
    [self.foundLable mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.allSelectBtn.mas_right).offset(8);
        make.centerY.equalTo(self.foundLable.superview);
        make.height.mas_equalTo(24);
        make.right.equalTo(self.foundLable.superview).offset(-16);
    }];
}

- (void)initTopToolBar{
    self.topBar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleTop];
    self.topBar.tag = TOPBAR_PAGESLIDER_TAG;
    StateBarThemeKeyPath(self.topBar.stateBar, backgroundColor);
    [BindThemeKeySel(self.topBar.stateBar, backgroundView) fs_updateThemeCompleted:^(UIView*  _Nonnull target, SEL  _Nonnull sel) {
        StateBarThemeKeyPath(target, backgroundColor);
    }];
    [_extensionsManager.containerView addSubview:self.topBar];
    [self.topBar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.top.mas_equalTo(self.pdfViewCtrl);
    }];

    __weak typeof(self) weakSelf = self;
    self.filterItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"common_setting_gray")];
    self.filterItem.accessibilityLabel = FSLocalizedForKey(@"kSearchOptionsVoiceover");
    self.filterItem.onTapClick = ^(FSItem *item) {
        if (weakSelf.searchbyController.presentingViewController) {
            [weakSelf.searchbyController.presentingViewController dismissViewControllerAnimated:true completion:nil];
        }else if (weakSelf.searchByPopoverCtrl.presentingViewController) {
            [weakSelf.searchByPopoverCtrl.presentingViewController dismissViewControllerAnimated:true completion:nil];
        } else {

            if (DEVICE_iPAD && ![self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
                CGRect rect = [weakSelf.filterItem convertRect:weakSelf.filterItem.bounds toView:weakSelf.pdfViewCtrl];
                FSNavigationController *searchByNavCtrl = [[FSNavigationController alloc] initWithRootViewController:weakSelf.searchbyController];
                searchByNavCtrl.navigationBarHidden = YES;
                searchByNavCtrl.preferredContentSize = CGSizeMake(250, 50 * 3);
                searchByNavCtrl.modalPresentationStyle = UIModalPresentationPopover;
                searchByNavCtrl.popoverPresentationController.delegate = weakSelf;
                searchByNavCtrl.popoverPresentationController.sourceView = weakSelf.pdfViewCtrl;
                searchByNavCtrl.popoverPresentationController.sourceRect = rect;
                searchByNavCtrl.popoverPresentationController.permittedArrowDirections = UIPopoverArrowDirectionUp;
                [weakSelf.pdfViewCtrl.fs_viewController presentViewController:searchByNavCtrl animated:true completion:nil];
            }else{
                weakSelf.searchbyController.view.frame = CGRectMake(0, 0, MIN(weakSelf.extensionsManager.pdfViewCtrl.fs_width, weakSelf.extensionsManager.pdfViewCtrl.fs_height), 50 * 3 + 10);
                weakSelf.searchByPopoverCtrl = [weakSelf.extensionsManager.viewCtrlManager popPopoverCustomViewCtrl:weakSelf.searchbyController sourceView:nil];
            }
        }
    };
    [self.topBar addSubview:self.filterItem];
    [self.filterItem mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.mas_equalTo(self.filterItem.superview);
        make.left.mas_equalTo(4);
        make.width.mas_equalTo(40);
    }];

    self.cancelItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kCancel")];
    [BindThemeKeySel(self.cancelItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem * _Nonnull target, SEL  _Nonnull sel) {
        [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
    }];
    self.cancelItem.font  = [UIFont systemFontOfSize:15.0f];
    [self.topBar addSubview:self.cancelItem];
    self.cancelItem.onTapClick = ^(FSItem *item) {
        [weakSelf cancelSearchWithExit:NO];
    };
    CGSize cancelSize = self.cancelItem.titleLabel.intrinsicContentSize;
    [self.cancelItem mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.mas_equalTo(self.filterItem);
        make.right.equalTo(self.topBar).offset(-16);
        make.width.mas_equalTo(cancelSize.width);
    }];

    self.searchBar = [[UISearchBar alloc] initWithFrame:CGRectZero];
    self.searchBar.placeholder = FSLocalizedForKey(@"kSearch");
    [self.searchBar setImage:FSImageNamedAndNormalColor(@"search_left_search") forSearchBarIcon:UISearchBarIconSearch state:UIControlStateNormal];
    [self.searchBar setImage:FSImageNamedAndNormalColor(@"search_right_clear") forSearchBarIcon:UISearchBarIconClear state:UIControlStateNormal];
    UITextField *searchTextField = nil;
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
    if (@available(iOS 13.0, *)) {
        searchTextField = self.searchBar.searchTextField;
    }
    else
#endif
    {
        // Fallback on earlier versions
        searchTextField = [self.searchBar valueForKey:@"_searchField"];
    }
    if (searchTextField) {
#if _MAC_CATALYST_
        searchTextField.backgroundColor = self.fs_dividerColor_P2;
#endif
        searchTextField.textColor = self.fs_textColor_T4;
        searchTextField.layer.cornerRadius = 18;
        searchTextField.layer.masksToBounds = YES;
        [searchTextField mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.top.right.bottom.equalTo(self.searchBar);
        }];
    }


    [self.topBar addSubview:self.searchBar];
    [self.searchBar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerY.mas_equalTo(self.filterItem);
        make.left.equalTo(self.filterItem.mas_right);
        make.right.equalTo(self.cancelItem.mas_left).offset(-8);
        make.height.mas_equalTo(36);
    }];
    self.searchBar.delegate = self;
    if ([self.searchBar respondsToSelector:@selector(barTintColor)]) {
        self.searchBar.backgroundImage = [UIImage new];
    } else {
        [[self.searchBar.subviews objectAtIndex:0] removeFromSuperview];
        [self.searchBar setBackgroundColor:[UIColor whiteColor]];
    }
}

-(void)setIsRedactChoose:(BOOL)isRedactChoose{
    _isRedactChoose = isRedactChoose;

    dispatch_async(dispatch_get_main_queue(), ^{

        [self.totalView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(self.topBar.mas_bottom);
            make.bottom.equalTo(self.tableView.mas_top);
            if (isRedactChoose) {
                make.height.mas_equalTo(44);
            }else{
                make.height.mas_equalTo(@24);
            }
            make.width.left.mas_equalTo(self.tableView);
        }];
    });

}

- (void)initBottomToolBar{
    self.bottomBar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleBottom];
    [self.extensionsManager.containerView addSubview:self.bottomBar];
    [self.bottomBar mas_makeConstraints:^(MASConstraintMaker *make) {

        if (DEVICE_iPAD && ![self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            make.centerX.equalTo(self.pdfViewCtrl);
            make.width.mas_equalTo(184);
            make.bottom.equalTo(self.pdfViewCtrl.fs_mas_bottom).offset(-44);
        }else{
            make.left.right.equalTo(self.pdfViewCtrl);
            make.bottom.equalTo(self.pdfViewCtrl);
        }

    }];
    if (DEVICE_iPAD && ![self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
        self.bottomBar.layer.cornerRadius = 6;
    }
    self.bottomBar.clipsToBounds = YES;
    self.bottomBaseBar = [[TbBaseBar alloc] init];
    self.bottomBaseBar.contentView.frame = CGRectMake(0, CGRectGetHeight(self.pdfViewCtrl.bounds), CGRectGetWidth(self.pdfViewCtrl.bounds), 49);

    if (DEVICE_iPHONE || [self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
        self.bottomBaseBar.intervalWidth = 16;
    } else {
        self.bottomBaseBar.intervalWidth = 16;
    }
    [self.bottomBar addSubview:self.bottomBaseBar.contentView];
    [self.bottomBaseBar.contentView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.right.top.bottom.equalTo(self.bottomBaseBar.contentView.superview);
    }];

    self.previousItem = [TbBaseItem createItemWithImage:FSImageNamedAndNormalColor(@"search_previous") imageSelected:FSImageNamedAndNormalColor(@"search_previous") imageDisable:FSImageNamedAndColor(@"search_previous", self.fs_iconColor_I2)];
    self.previousItem.button.accessibilityIdentifier = @"search_previous";
    __weak typeof(self) weakSelf = self;

    self.previousItem.tag = 0;
    [self.bottomBaseBar addItem:self.previousItem displayPosition:Position_CENTER];
    self.previousItem.onTapClick = ^(TbBaseItem *item) {
        [weakSelf previousKeyLocation];
    };


    self.showListItem = [TbBaseItem createItemWithImage:FSImageNamedAndNormalColor(@"search_show_list") imageSelected:FSImageNamedAndNormalColor(@"search_show_list") imageDisable:FSImageNamedAndColor(@"search_show_list", self.fs_iconColor_I2)];
    self.showListItem.button.accessibilityIdentifier = @"search_show_list";
    self.showListItem.onTapClick = ^(TbBaseItem *item) {
        [weakSelf setTableViewHidden:NO];
        [weakSelf setBottomBaseBarHidden:YES];
    };
    [self.bottomBaseBar addItem:self.showListItem displayPosition:Position_CENTER];

//    self.nextItem = [TbBaseItem createItemWithImage:FSImageNamedAndNormalColor(@"search_next") imageSelected:FSImageNamedAndNormalColor(@"search_next") imageDisable:FSImageNamedAndNormalColor(@"search_next_disable")];
    self.nextItem = [TbBaseItem createItemWithImage:FSImageNamedAndNormalColor(@"search_next") imageSelected:FSImageNamedAndNormalColor(@"search_next") imageDisable:FSImageNamedAndColor(@"search_next", self.fs_iconColor_I2)];
    self.nextItem.button.accessibilityIdentifier = @"search_next";
    [self.bottomBaseBar addItem:self.nextItem displayPosition:Position_CENTER];
    self.nextItem.onTapClick = ^(TbBaseItem *item) {
        [weakSelf nextKeyLocation];
    };

    [self.previousItem.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(self.showListItem.contentView.mas_left).offset(-self.bottomBaseBar.intervalWidth);
        make.centerY.mas_equalTo(self.previousItem.contentView.superview);
        make.size.mas_equalTo(self.previousItem.contentView.fs_size);
    }];

    [self.showListItem.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.center.mas_equalTo(self.showListItem.contentView.superview);
        make.size.mas_equalTo(self.showListItem.contentView.fs_size);
    }];

    [self.nextItem.contentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.showListItem.contentView.mas_right).offset(self.bottomBaseBar.intervalWidth);
        make.centerY.mas_equalTo(self.nextItem.contentView.superview);
        make.size.mas_equalTo(self.nextItem.contentView.fs_size);
    }];
}

- (FSEmptyDataSetView *)emptyDataSetView{
    if (!_emptyDataSetView) {
        _emptyDataSetView = [FSEmptyDataSetView new];
    }
    return  _emptyDataSetView;
}

- (void)reloadEmptyDataSet{
    if ([self emptyDataSetShouldDisplay:self.tableView]) {
        if (self.emptyDataSetView.superview != self.tableView) {
            [self.tableView addSubview:self.emptyDataSetView];
            [self.emptyDataSetView mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.center.left.right.bottom.top.fs_equalToSuperview();
            }];
        }
        self.emptyDataSetView.title = [self titleForEmptyDataSet:self.tableView];
        self.emptyDataSetView.image = [self imageForEmptyDataSet:self.tableView];
        self.emptyDataSetView.verticalOffset = -15;
        self.emptyDataSetView.scrollEnabledForSupView = NO;
    }else{
        if (self.emptyDataSetView.superview) {
            [self.emptyDataSetView removeFromSuperview];
        }
    }
}

- (BOOL)emptyDataSetShouldDisplay:(UITableView *)tableView {

    if (self.arraySearch.count <= 0) {
        return YES;
    }else{
        return NO;
    }
}

- (NSString *)titleForEmptyDataSet{
    return FSLocalizedForKey(@"kSearchNoMatches");
}

- (NSAttributedString *)titleForEmptyDataSet:(UITableView *)tableView {
    return [[NSAttributedString alloc] initWithString:[self titleForEmptyDataSet] attributes:@{
        NSFontAttributeName:self.fs_standardGeneralFont_GF2,
        NSForegroundColorAttributeName:self.fs_textColor_T2
        }];
}
- (UIImage *)imageForEmptyDataSet:(UITableView *)tableView {
    return FSImageNamedAndPrimaryColor(@"search_no_matches");
}


- (void)unload {
}

- (void)onClickSearchItem:(FSItem *)item  {
    if (DEVICE_iPAD && ![self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
        self.extensionsManager.panelController.isHidden= YES;
    }
    [self.extensionsManager changeState:FSUIManagerStateSearch];
    [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingSearch];
}

- (void)willRotate {
    if (self.tableView.window) {
        [self.tableView reloadData];
        [self.tableView layoutIfNeeded];
    }

}

- (UIViewController *)searchbyController {
    if (!_searchbyController) {
        SearchbyViewController *searchByCtr = [[SearchbyViewController alloc] initWithStyle:UITableViewStylePlain withSearchFlag:self.textSearchSearchFlags];
        searchByCtr.searchbyClickedHandler = ^(int type, BOOL flag) {
            if (type == 0) {
                if (flag) {
                    self.textSearchSearchFlags |= FSTextSearchSearchMatchCase;
                }else{
                    self.textSearchSearchFlags &= ~FSTextSearchSearchMatchCase;
                }
                [self searchOptionBtnPress];

            } else if (type == 1) {
                if (flag) {
                    self.textSearchSearchFlags |= FSTextSearchSearchMatchWholeWord;
                }else{
                    self.textSearchSearchFlags &= ~FSTextSearchSearchMatchWholeWord;
                }
                [self searchOptionBtnPress];
            }else{
                [self buttonSearchClick:nil];
            }
        };
        _searchbyController = searchByCtr;
    }
    return _searchbyController;
}

- (void)searchOptionBtnPress{
    if (SEARCH_BAR_TEXT.length > 0) {
        [self searchBarSearchButtonClicked:self.searchBar];
    }else{
        if (DEVICE_iPAD && ![self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            [self.searchbyController.presentingViewController dismissViewControllerAnimated:true completion:nil];
        }else{
            [self.searchByPopoverCtrl dismissViewControllerAnimated:true completion:nil];
        }
    }
}

#pragma mark <UIPopoverPresentationControllerDelegate>

- (UIModalPresentationStyle)adaptivePresentationStyleForPresentationController:(UIPresentationController *)controller {
    return UIModalPresentationNone;
}

- (UIModalPresentationStyle)adaptivePresentationStyleForPresentationController:(UIPresentationController *)controller traitCollection:(UITraitCollection *)traitCollection {
    return UIModalPresentationNone;
}

- (void)popoverPresentationController:(UIPopoverPresentationController *)popoverPresentationController willRepositionPopoverToRect:(inout CGRect *)rect inView:(inout UIView  * __nonnull * __nonnull)view{
    CGRect showRect = [self.filterItem convertRect:self.filterItem.bounds toView:self.pdfViewCtrl];
    *rect = showRect;
}

- (void)showSearchBar:(BOOL)show{
    if (show) {
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardWasShown:) name:UIKeyboardDidShowNotification object:nil];
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardWasHidden:) name:UIKeyboardWillHideNotification object:nil];
        self.topBar.hidden = NO;
        [self setTopBaseBarHidden:NO];
        [_extensionsManager setCurrentIsFullScreen:NO];
        [self.searchBar becomeFirstResponder];
        self.onSearchState = YES;
        [_extensionsManager onSearchStarted];
         [(PageNavigationModule *)[self.extensionsManager getModuleByName:@"PageNavigation"] layoutPdfViewContentInsets:NO];
    } else {
        self.topBar.hidden = YES;
        [self setTopBaseBarHidden:YES];
//        [_extensionsManager setCurrentIsFullScreen:YES];
        [self.searchBar resignFirstResponder];
        [self setBottomBaseBarHidden:YES];
        [self setTableViewHidden:YES];
        self.onSearchState = NO;

        [_extensionsManager onSearchCanceled];
        self.textSearchKeyWords = nil;
    }
}

- (void)seachText:(NSString *)seachText seachOption:(unsigned int)textSearchSearchFlags
{
    [_extensionsManager changeState:FSUIManagerStateSearch];
    self.textSearchSearchFlags = textSearchSearchFlags;
    if (self.topBar.hidden){
        [self showSearchBar:YES];
    }
    self.searchBar.text = seachText;
    ((SearchbyViewController *)self.searchbyController).searchFlag = textSearchSearchFlags;
    [self searchOptionBtnPress];
}

- (void)setTopBaseBarHidden:(BOOL)hidden {
    if (_topBaseBarHidden == hidden) {
        return;
    }
    _topBaseBarHidden = hidden;

    if (hidden) {
        [self.topBar mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.right.equalTo(self.pdfViewCtrl);
            make.bottom.equalTo(self.pdfViewCtrl.mas_top);
        }];

    } else {
        [self.topBar mas_remakeConstraints:^(MASConstraintMaker *make) {
           make.top.left.right.equalTo(self.pdfViewCtrl);
        }];

    }

    if (self.pdfViewCtrl.window) {
        [UIView animateWithDuration:0.3f animations:^{
            [self.pdfViewCtrl layoutIfNeeded];
        }];
    }
}

- (void)setBottomBaseBarHidden:(BOOL)hidden{
    if (_bottomBaseBarHidden == hidden) {
        return;
    }
    _bottomBaseBarHidden = hidden;
    if (hidden) {
        [self.bottomBar mas_remakeConstraints:^(MASConstraintMaker *make) {

            make.top.equalTo(self.pdfViewCtrl.mas_bottom);
            if (DEVICE_iPAD && ![self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
                make.centerX.equalTo(self.pdfViewCtrl);
                make.width.mas_equalTo(184);
            }else{
                make.left.right.equalTo(self.pdfViewCtrl);
            }

        }];
    } else {

        [self.bottomBar mas_remakeConstraints:^(MASConstraintMaker *make) {
            if (DEVICE_iPAD && ![self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
                make.centerX.equalTo(self.pdfViewCtrl);
                make.width.mas_equalTo(184);
                make.bottom.equalTo(self.pdfViewCtrl.fs_mas_bottom).offset(-44);
            }else{
                make.left.right.equalTo(self.pdfViewCtrl);
                make.bottom.equalTo(self.pdfViewCtrl);
            }
        }];
    }

    if (self.pdfViewCtrl.window) {
        [UIView animateWithDuration:0.3f animations:^{
            [self.pdfViewCtrl layoutIfNeeded];
        }];
    }
}

- (void)setRedactionBtnContraints{
    [self.redactionBtn mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.bottom.equalTo(self.redactionBtn.superview);
        make.left.width.equalTo(self.tableView);
        if (self.isRedactChoose) {
            if (self.arraySearch.count > 0) {
                make.height.mas_equalTo(48+self.pdfViewCtrl.fs_currentSceneKeyWindow.safeAreaInsets.bottom);
            }else{
                make.height.mas_equalTo(0);
            }

        }else{
            make.height.mas_equalTo(0);
        }
    }];

    [self.allSelectBtn mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.totalView.fs_mas_left).offset(8);
        if (self.isRedactChoose) {
            if (self.arraySearch.count > 0) {
                make.width.mas_equalTo(40);
            }else{
                make.width.mas_equalTo(0);
            }

        }else{
            make.width.mas_equalTo(0);
        }
        make.height.mas_equalTo(40);
        make.centerY.equalTo(self.allSelectBtn.superview);
    }];
}

- (void)setTableViewHidden:(BOOL)hidden {
    if (_tableviewHidden == hidden) {
        return;
    }
    _tableviewHidden = hidden;
    if (hidden) {
        [UIView animateWithDuration:0.3
                         animations:^{
            [self setRedactionBtnContraints];

            [self.tableView mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.bottom.equalTo(self.redactionBtn.mas_top).offset(0);
                make.left.equalTo(self.tableView.superview.mas_right).offset(0);
                if (DEVICE_iPAD && ![self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
                    make.width.mas_equalTo(300);
                }else{
                    make.width.equalTo(self.tableView.superview);
                }
            }];
            [self.pdfViewCtrl layoutIfNeeded];
        }];

        [UIView animateWithDuration:0.4
                         animations:^{
            self.maskView.alpha = 0.1f;
        }
                         completion:^(BOOL finished) {

            [self.maskView removeFromSuperview];
        }];
    } else {
        self.maskView.backgroundColor = [UIColor blackColor];
        self.maskView.alpha = 0.3f;
        self.maskView.tag = 300;
        [self.maskView addTarget:self action:@selector(dissmiss:) forControlEvents:UIControlEventTouchUpInside];
        [_extensionsManager.containerView insertSubview:self.maskView belowSubview:self.topBar];
        [self.maskView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.right.top.bottom.mas_equalTo(self.pdfViewCtrl);
        }];

        [UIView animateWithDuration:0.3
                         animations:^{
            [self setRedactionBtnContraints];
            [self.tableView mas_remakeConstraints:^(MASConstraintMaker *make) {
                if (DEVICE_iPAD && ![self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
                    make.left.equalTo(self.tableView.superview.mas_right).offset(-300);
                    make.width.mas_equalTo(300);
                }else{
                    make.left.equalTo(self.tableView.superview);
                    make.width.equalTo(self.tableView.superview);
                }
                make.bottom.equalTo(self.redactionBtn.mas_top).offset(0);
            }];
            [self.pdfViewCtrl layoutIfNeeded];
        }];
    }
}

- (void)dissmiss:(id)sender {
    UIControl *control = (UIControl *) sender;
    if (control.tag == 300) {
        [self setTableViewHidden:YES];
        [self setBottomBaseBarHidden:NO];
    }
}

#pragma mark Button Aciton

- (void)buttonSearchClick:(id)sender {
    if (DEVICE_iPAD && ![self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
        [self.searchbyController.presentingViewController dismissViewControllerAnimated:true completion:nil];
    }else{
        [self.searchByPopoverCtrl dismissViewControllerAnimated:true completion:nil];
    }

    NSString *keyword = SEARCH_BAR_TEXT;
    NSString *url;
    if (keyword.length > 0) {
        url = [NSString stringWithFormat:@"http://www.google.com/search?q=%@", [keyword stringByAddingPercentEncodingWithAllowedCharacters:[NSCharacterSet URLQueryAllowedCharacterSet]]];
    } else {
        url = @"http://www.google.com";
    }
    [[UIApplication sharedApplication] openURL:[NSURL URLWithString:url] options:@{} completionHandler:nil];
}

- (void)previousKeyLocation {
    if (_selectedSearchResult == nil) {
        if (self.arraySearch.count == 0) {
            return;
        } else {
            self.selectedSearchResult = [self.arraySearch objectAtIndex:0];
        }
    }
    if (_selectedSearchInfo == nil) {
        self.selectedSearchInfo = [self.selectedSearchResult.infos objectAtIndex:self.selectedSearchResult.infos.count - 1];
        [self gotoPage:self.selectedSearchResult.index rects:self.selectedSearchInfo.rects];
    } else {
        NSUInteger infoIndex = [self.selectedSearchResult.infos indexOfObject:self.selectedSearchInfo];
        if (infoIndex == 0) {
            if (self.arraySearch.count < 1)
                return;
            NSUInteger resultIndex = [self.arraySearch indexOfObject:self.selectedSearchResult];
            if (resultIndex == 0) {
                self.selectedSearchResult = [self.arraySearch lastObject];
            } else {
                self.selectedSearchResult = [self.arraySearch objectAtIndex:resultIndex - 1];
            }
            self.selectedSearchInfo = [self.selectedSearchResult.infos lastObject];
        } else {
            self.selectedSearchInfo = [self.selectedSearchResult.infos objectAtIndex:infoIndex - 1];
        }
        [self gotoPage:self.selectedSearchResult.index rects:self.selectedSearchInfo.rects];
    }
}

- (void)nextKeyLocation {
    if (_selectedSearchResult == nil) {
        if (self.arraySearch.count != 0) {
            self.selectedSearchResult = [self.arraySearch objectAtIndex:0];
        } else {
            return;
        }
    }

    if (_selectedSearchInfo == nil) {
        self.selectedSearchInfo = [self.selectedSearchResult.infos objectAtIndex:0];
        [self gotoPage:_selectedSearchResult.index rects:self.selectedSearchInfo.rects];
    } else {
        NSUInteger infoIndex = [self.selectedSearchResult.infos indexOfObject:self.selectedSearchInfo];
        if (infoIndex == self.selectedSearchResult.infos.count - 1) {
            if (self.arraySearch.count < 1)
                return;
            NSUInteger resultIndex = [self.arraySearch indexOfObject:self.selectedSearchResult];
            if (resultIndex == self.arraySearch.count - 1) {
                self.selectedSearchResult = [self.arraySearch objectAtIndex:0];
            } else {
                self.selectedSearchResult = [self.arraySearch objectAtIndex:resultIndex + 1];
            }
            self.selectedSearchInfo = [self.selectedSearchResult.infos objectAtIndex:0];
        } else {
            self.selectedSearchInfo = [self.selectedSearchResult.infos objectAtIndex:infoIndex + 1];
        }
        [self gotoPage:self.selectedSearchResult.index rects:self.selectedSearchInfo.rects];
    }
}


- (void)selectBtnPress:(FSSearchSelectButton*)button{
    if (self.isSearching) {
        return;
    }

    button.selected = !button.selected;

    int section = (int)button.indexPath.section;
    int row = (int)button.indexPath.row - 1;
    NSMutableArray *array = [self.selectDict objectForKey:[NSNumber numberWithInt:section]];


        if (button.selected) {
            button.accessibilityIdentifier = @"common_select_id";
            if (!array ) {
                array = [NSMutableArray array];
                [self.selectDict setObject:array forKey:[NSNumber numberWithInt:section]];
            }
            for (NSNumber *number in array) {
                if ([number isEqualToNumber:[NSNumber numberWithInt:row]]) {
                    break;
                }
            }
            [array addObject:[NSNumber numberWithInt:row]];
        }else{
            button.accessibilityIdentifier = @"common_unselect_id";
            for (NSNumber *number in array) {
                if ([number isEqualToNumber:[NSNumber numberWithInt:row]]) {
                    [array removeObject:number];
                    break;
                }
            }
        }


    int totalCount = 0;
    for (NSArray *array in self.selectDict.allValues) {
        totalCount += array.count;
    }

    if (totalCount >= self.totalCount) {
        self.allSelectBtn.accessibilityIdentifier = @"common_select_id";
        self.allSelectBtn.selected = YES;
    }else{
        self.allSelectBtn.accessibilityIdentifier = @"common_unselect_id";
        self.allSelectBtn.selected = NO;
    }

    if (totalCount == 0) {
        self.redactionBtn.enabled = NO;
    }else{
        self.redactionBtn.enabled = YES;
    }
}

- (void)allSelectPress:(UIButton*)button{
    if (self.isSearching) {
        return;
    }
    self.allSelectBtn.selected = !self.allSelectBtn.selected;
    [self selectAll:button.selected];
    [self.tableView reloadData];

    if (!self.allSelectBtn.selected) {
        self.allSelectBtn.accessibilityIdentifier = @"common_unselect_id";
        self.redactionBtn.enabled = NO;
    }else{
        self.allSelectBtn.accessibilityIdentifier = @"common_select_id";
        self.redactionBtn.enabled = YES;
    }
}

- (void)selectAll:(BOOL)selected{
    if (!self.isRedactChoose) {
        return;
    }
    if (selected) {

        int index = 0;
        for (SearchResult *result in self.arraySearch) {
            NSMutableArray *array = [NSMutableArray array];
            for (int cnt=0; cnt < result.infos.count; cnt++) {
                [array addObject:[NSNumber numberWithInt:cnt]];
            }
            [self.selectDict setObject:array forKey:[NSNumber numberWithInt:index]];
            index++;
        }

    }else{
        [self.selectDict removeAllObjects];
    }
}

- (void)redactPress:(UIButton*)button{

    int sectionCount = (int)self.selectDict.count;

    [[self.pdfViewCtrl fs_currentSceneTopMostViewController].view fs_scheduledMainQueueShowHUDLoading:FSLocalizedForKey(@"kProcessing...") process:^{

        BOOL bDone = NO;
        @try {
            NSMutableArray<FSAnnot*> *annotArray = [NSMutableArray array];
            id<IModule> module = [self.extensionsManager getModuleByName:@"Redact"];
            RedactModule *redactModule = (RedactModule *)module;
            for (int i=0; i< sectionCount; i++) {
                NSNumber *section = [[self.selectDict allKeys] objectAtIndex:i];
                NSArray *array = [self.selectDict objectForKey:section];
                SearchResult *searchResult = nil;
                if (i < self.arraySearch.count) {
                    searchResult = [self.arraySearch objectAtIndex:[section intValue]];
                }
                if (array && searchResult) {
                    for (NSNumber *number in array) {
                        int index = [number intValue];

                        if (index < searchResult.infos.count) {
                            SearchInfo *info = [searchResult.infos objectAtIndex:index];
                            if (info) {
                                for (id obj in info.rects) {
                                    CGRect rect = [obj CGRectValue];
                                    FSRectF *fsrect = [Utility CGRect2FSRectF:rect];

                                    FSAnnot *annote = [redactModule addAnnotToPage:searchResult.index withRect:fsrect needQuads:YES];
                                    if (annote && ![annote isEmpty]) {
                                        [annotArray addObject:annote];
                                    }
                                    bDone = YES;
                                }
                            }

                        }
                    }

                }

            }
            if (annotArray.count > 0){
                id<IAnnotHandler> annotHandler = [self.extensionsManager getAnnotHandlerByType:[annotArray objectAtIndex:0].type];
                if ([annotHandler isKindOfClass:[RedactAnnotHandler class]]) {
                    [(RedactAnnotHandler *)annotHandler addAnnots:annotArray addUndo:YES];
                }

            }

        } @catch (NSException *exception) {

        } @finally {

        }
        if (bDone) {
            self.extensionsManager.isDocModified = YES;
        }
        dispatch_async(dispatch_get_main_queue(), ^{
            [self cancelSearchWithExit:NO];
        });

    }];
}

- (void)gotoPage:(int)index rects:(NSArray *)rects {
    if ([self gotoPage:index animated:YES]) {
        if (rects && rects.count > 0) {
            self.needDrawRects = rects;
            CGRect firstNeedDrawRect = [[rects objectAtIndex:0] CGRectValue];

            self.needPageIndex = index;
            if ([self.pdfViewCtrl isContinuous]) {
                FSPointF *point = [[FSPointF alloc] init];
                [point set:CGRectGetMidX(firstNeedDrawRect) - 80 y:CGRectGetMidY(firstNeedDrawRect) + 150];
                [self.pdfViewCtrl gotoPage:index withDocPoint:point animated:NO];
            } else {
                FSPointF *point = [[FSPointF alloc] init];
                [point set:CGRectGetMidX(firstNeedDrawRect) y:CGRectGetMidY(firstNeedDrawRect)];
                point.x -= 20;
                CGFloat topH = [Utility convertWidth:self.topBar.fs_height fromPageViewToPDF:self.pdfViewCtrl pageIndex:[self.pdfViewCtrl getCurrentPage]];
                [point setY:[point getY] + topH + 20];
                [self.pdfViewCtrl gotoPage:index withDocPoint:point animated:NO];
            }
            
            [self.pdfViewCtrl refreshOverlay];
            [self setTableViewHidden:YES];
            [self setBottomBaseBarHidden:NO];
        }
    }
}

- (BOOL)gotoPage:(int)index animated:(BOOL)animated {
    if (index >= 0 && index < [self.pdfViewCtrl.currentDoc getPageCount]) {
        [self.pdfViewCtrl gotoPage:index animated:animated];
        return YES;
    }
    return NO;
}

- (void)clearSearch {
    self.searchCancelCallBack.cancelNow = YES;
    self.isSearching = NO;
    if(self.timer!=nil){
        [self.timer invalidate];
        self.timer = nil;
    }
    [self.searchBar setText:nil];
    [self.arraySearch removeAllObjects];
    [self.arrayTempSearch removeAllObjects];
    self.currentIndex = 0;
    self.searchCurrentPage = 0;
    [self.searchOPQueue cancelAllOperations];
    self.needDrawRects = nil;
    self.textSearchKeyWords = nil;
    [self.tableView reloadData];
}

- (void)cancelSearchWithExit:(BOOL)exit {
    [self showSearchBar:NO];
    [self clearSearch];

    if (!exit) {
        [self.pdfViewCtrl refreshOverlay];
    }

    [self.searchBar resignFirstResponder];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIKeyboardDidShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] removeObserver:self name:UIKeyboardWillHideNotification object:nil];

    //If there is still in searching, wait for finishing.
    FSTask *task = [[FSTask alloc] init];
    task.run = ^() {

    };
    [self.extensionsManager.taskServer executeSync:task];
    if (self.isRedactChoose) {
        [self.extensionsManager changeState:FSUIManagerStateRedaction];
    }
    self.isRedactChoose = NO;
}

-(BOOL)getResult:(FSTextSearch*)textSearch searchPageIndex:(int)searchPageIndex oldpageIndex:(int)oldpageIndex searchWords:(NSString *)searchWords pageResultDict:(NSMutableDictionary*) pageResultDict completed:(void (^)(SearchResult *ret,int pageIndex))completed{

    [textSearch setStartPage:searchPageIndex];
    BOOL bRet = [textSearch findNext];
    while (bRet) {
        if (!self.isSearching){
            return NO;
        }
        int pageIndex = [textSearch getMatchPageIndex];

        SearchResult *result = [pageResultDict objectForKey:@(pageIndex)];
        if (result == nil) {
            result = [[SearchResult alloc] init];
            result.index = pageIndex;
            result.infos = [NSMutableArray array];

            [pageResultDict setObject:result forKey:@(pageIndex)];
        }

        NSString *sentence = [textSearch getMatchSentence];
        int sentencePos = [textSearch getMatchSentenceStartIndex];
        FSRectFArray *rects = [textSearch getMatchRects];

        SearchInfo *info = [[SearchInfo alloc] init];
        info.snippet = sentence;
        
        info.keywordLocation = sentencePos;
        if (sentence.length < searchWords.length) {
            info.snippet = [searchWords stringByReplacingCharactersInRange:NSMakeRange(0, sentence.length-sentencePos) withString:sentence];
        }
        int width = 300;
        if (DEVICE_iPHONE || [self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            width = self.width;
        }
        if (self.isRedactChoose) {
            width -= 40;
        }

        CGSize textSize = [Utility getTextSize:info.snippet fontSize:SEARCH_LABEL_FONT maxSize:CGSizeMake(width - 2 * STYLE_INFO_LEFT, 1000)];
        info.cellHeight = textSize.height;
        info.rects = [NSMutableArray array];
        for (int i = 0; i < [rects getSize]; i++) {
            FSRectF *fsrect = [rects getAt:i];
            [info.rects addObject:[NSValue valueWithCGRect:[Utility FSRectF2CGRect:fsrect]]];
        }
        [result.infos addObject:info];

        if (pageIndex != oldpageIndex) {
            SearchResult *lastResult = [pageResultDict objectForKey:@(oldpageIndex)];
            completed(lastResult,oldpageIndex);
            oldpageIndex = pageIndex;
        }

        bRet = [textSearch findNext];
    }

    SearchResult *lastResult = [pageResultDict objectForKey:@(oldpageIndex)];
    completed(lastResult,oldpageIndex);
    return YES;
}

- (void)checkScrollViewScrolling:(NSTimer *)timer {
    if (self.tableView.isDragging || self.tableView.isDecelerating) {
        self.isScroll = YES;
    } else {
        self.isScroll = NO;
        if(self.arrayTempSearch.count > 0){
            [self.arraySearch addObjectsFromArray:self.arrayTempSearch];
            [self.arrayTempSearch removeAllObjects];
        }
        if(!self.isSearching && self.totalCount !=0){
            int totalCount = 0;
            for (SearchResult *result in self.arraySearch) {
                totalCount += result.infos.count;
            }
            if(self.totalCount != totalCount){
                self.totalView.backgroundColor = self.fs_backgroundColor_B2;
                self.foundLable.text = [NSString stringWithFormat:@"%@  %d", FSLocalizedForKey(@"kTotalFound"), totalCount];
                self.totalCount = totalCount;
                [self.tableView reloadData];
            }
            if(self.timer!=nil){
                [self.timer invalidate];
                self.timer = nil;
            }
        }
    }
}

-(BOOL)getSearchResult:(NSString *)searchWords withSearchRange:(NSRange)range completed:(void (^)(SearchResult *ret,int pageIndex))completed{
    NSMutableDictionary *pageResultDict = @{}.mutableCopy;
    self.searchCancelCallBack = [[UIFSSearchCancelCallback alloc] init];
    int searchPageIndex = self.searchCurrentPage;
    self.isSearching = YES;
    FSPDFDoc *document = self.pdfViewCtrl.currentDoc;
    int oldpageIndex = self.searchCurrentPage;
    @try {
        FSTextSearch *textSearch = [[FSTextSearch alloc] initWithDocument:document cancel:self.searchCancelCallBack flags: FSTextPageParseTextNormal];
        [textSearch setPattern:searchWords];
        [textSearch setSearchFlags:self.textSearchSearchFlags];
        BOOL isResult = NO;
        if(searchPageIndex == 0){
            isResult =[self getResult:textSearch searchPageIndex:searchPageIndex oldpageIndex:oldpageIndex searchWords:searchWords pageResultDict:pageResultDict completed:completed];
        }else if(searchPageIndex != 0){
            isResult =[self getResult:textSearch searchPageIndex:searchPageIndex oldpageIndex:oldpageIndex searchWords:searchWords pageResultDict:pageResultDict completed:completed];
            oldpageIndex = 0;
            searchPageIndex = 0;
            [textSearch setEndPage:self.searchCurrentPage -1];
            isResult =[self getResult:textSearch searchPageIndex:searchPageIndex oldpageIndex:oldpageIndex searchWords:searchWords pageResultDict:pageResultDict completed:completed];
        }
 
        self.isSearching = NO;

        return isResult;
    } @catch (NSException *e) {
    }
}

- (void)searchTextInPDF:(NSString *)str currentPage:(NSArray *)pages{
    self.textSearchKeyWords = str;
    [self.searchOPQueue cancelAllOperations];
    if (str.length == 0 || str == nil) {
        return;
    }
    if (pages == nil || pages.count == 0) {
        return;
    }
    [self setTableViewHidden:NO];
    [self.emptyDataSetView removeFromSuperview];
    [self.arraySearch removeAllObjects];
    [self.arrayTempSearch removeAllObjects];
    self.currentIndex = 0;
    self.tableView.estimatedRowHeight = 0;
    self.tableView.estimatedSectionHeaderHeight = 0;
    self.tableView.estimatedSectionFooterHeight = 0;

    self.searchCurrentPage = [self.pdfViewCtrl getCurrentPage];
    if(self.timer == nil){
        self.timer = [NSTimer timerWithTimeInterval:0.1 target:self selector:@selector(checkScrollViewScrolling:) userInfo:nil repeats:YES];
        [[NSRunLoop currentRunLoop] addTimer:self.timer forMode:NSRunLoopCommonModes];
    }
    
    __block int totalCount = 0;
    __block NSBlockOperation *op = [NSBlockOperation blockOperationWithBlock:^{

        BOOL isSuccess = [self getSearchResult:str withSearchRange:NSMakeRange(0, 0) completed:^(SearchResult *ret, int pageIndex) {
            totalCount += ret.infos.count;

            dispatch_async(dispatch_get_main_queue(), ^{
                if (ret.infos.count > 0) {
                    [self searchFound:ret pageIndex:pageIndex];
                    if (self.isRedactChoose) {
                        [self setRedactionBtnContraints];
                    }
                }

                if (self.arraySearch.count == 0) {
                    self.totalCount = 0;
                    self.allSelectBtn.hidden = YES;
                    [self.tableView reloadData];
                    self.previousItem.enable = NO;
                    self.nextItem.enable = NO;
                }

            });

        }];

        if (isSuccess) {
            dispatch_async(dispatch_get_main_queue(), ^{
                [self reloadEmptyDataSet];
                [self selectAll:YES];
                self.allSelectBtn.selected = YES;
                self.allSelectBtn.accessibilityIdentifier = @"common_select_id";
                self.redactionBtn.enabled = YES;
                if (self.totalCount <= 0) {
                    self.foundLable.text = @"";
                    self.totalView.backgroundColor = self.fs_backgroundColor_B1;
                }else{
                    self.foundLable.text = [NSString stringWithFormat:@"%@  %d", FSLocalizedForKey(@"kTotalFound"), self.totalCount];
                    self.totalView.backgroundColor = self.fs_backgroundColor_B2;
                }

                dispatch_semaphore_signal(self->_wait);
            });
        }else{
            dispatch_semaphore_signal(self->_wait);
        }

    }];
    [self.searchOPQueue addOperation:op];
}

- (void)searchFound:(SearchResult *)result pageIndex:(int) pageIndex {
    self.currentIndex = pageIndex;
    if(!self.isScroll){
        [self.arraySearch addObject:result];
    }else{
        [self.arrayTempSearch addObject:result];
    }
    int totalCount = 0;
    for (SearchResult *result in self.arraySearch) {
        totalCount += result.infos.count;
    }
    self.totalCount = totalCount;
    if (self.totalCount > 0) {
        self.totalView.backgroundColor = self.fs_backgroundColor_B2;
        self.foundLable.text = [NSString stringWithFormat:@"%@  %d", FSLocalizedForKey(@"kTotalFound"), totalCount];
    }else{
        self.totalView.backgroundColor = self.fs_backgroundColor_B1;
        self.foundLable.text = [NSString stringWithFormat:@""];
    }


    self.allSelectBtn.selected = NO;
    self.allSelectBtn.accessibilityIdentifier = @"common_unselect_id";
    self.allSelectBtn.hidden = NO;
    
    if(!self.isScroll){
        [self.tableView reloadData];
    }

    self.previousItem.enable = YES;
    self.showListItem.enable = YES;
    self.nextItem.enable = YES;

}


- (NSString *)generateRText:(NSString *)content searchKeyword:(NSString *)keyword realLocation:(int)realIndex {
    //Replace multi whitespace with one.
    for (int index = 0; index < keyword.length; index++) {
        if ([keyword characterAtIndex:index] == 32) {
            int j = index;
            for (; [keyword characterAtIndex:j] == 32; j++)
                ;
            NSRange replaceWhitespace = {index, j - index};
            keyword = [keyword stringByReplacingCharactersInRange:replaceWhitespace withString:@" "];
        }
    }

    NSArray *contentArray = [StringDrawUtil seperateString:content bySeparator:keyword];
    NSString *rtText = @"";
    int highlightIndex = 0;
    NSStringCompareOptions options = NSCaseInsensitiveSearch;
    if ((self.textSearchSearchFlags & FSTextSearchSearchMatchCase) == FSTextSearchSearchMatchCase) {
        options = NSLiteralSearch;
    }

    for (NSString *str in contentArray) {
        if ([str compare:keyword options:options] == NSOrderedSame) {
            if (highlightIndex == realIndex) {
                rtText = [rtText stringByAppendingFormat:@"<font face='%@' size=%d color='#dd1100'><b>%@</b></font>", STYLE_INFO_FONT.fontName, (int) STYLE_INFO_FONT.pointSize, str];
            } else {
                rtText = [rtText stringByAppendingFormat:@"<font face='%@' size=%d color='#dd1100'>%@</font>", STYLE_INFO_FONT.fontName, (int) STYLE_INFO_FONT.pointSize, str];
            }
            highlightIndex++;
        } else {
            rtText = [rtText stringByAppendingFormat:@"<font face='%@' size=%d color='#000000'>%@</font>", STYLE_INFO_FONT.fontName, (int) STYLE_INFO_FONT.pointSize, str];
        }
    }
    return rtText;
}

#pragma mark - UISearchBar delegate handler

- (void)searchBar:(UISearchBar *)searchBar textDidChange:(NSString *)searchText{
    if (DEVICE_iPAD && ![self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
        [self.searchbyController.presentingViewController dismissViewControllerAnimated:true completion:nil];
    }else{
        [self.searchByPopoverCtrl dismissViewControllerAnimated:true completion:nil];
    }
}

- (void)searchBarSearchButtonClicked:(UISearchBar *)searchBar {
    if (DEVICE_iPAD && ![self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
        [self.searchbyController.presentingViewController dismissViewControllerAnimated:true completion:nil];
    }else{
        [self.searchByPopoverCtrl dismissViewControllerAnimated:true completion:nil];
    }

    if (self.isSearching) {
        self.isSearching = NO;
        dispatch_semaphore_wait(_wait, DISPATCH_TIME_FOREVER);
    }

    self.selectedSearchResult = nil;
    self.selectedSearchInfo = nil;
    self.redactionBtn.enabled = NO;
    [self.selectDict removeAllObjects];
    [self.arraySearch removeAllObjects];
    [self.arrayTempSearch removeAllObjects];
    self.currentIndex = 0;
    if(self.timer != nil){
        [self.timer invalidate];
        self.timer = nil;
    }
    [self.tableView reloadData];
    if (self.isRedactChoose) {
        [self setRedactionBtnContraints];
    }
    
    if (SEARCH_BAR_TEXT.length > 0) {

        [self.searchBar resignFirstResponder];
        NSString *searchKey = SEARCH_BAR_TEXT;

        NSMutableArray *pages = [NSMutableArray array];
        for (int i = 0; i < [self.pdfViewCtrl.currentDoc getPageCount]; i++) {
            [pages addObject:@(i)];
        }
        self.foundLable.text = FSLocalizedForKey(@"kSearching");
        self.totalView.backgroundColor = self.fs_backgroundColor_B2;

        @try {
            [self searchTextInPDF:searchKey currentPage:pages];
        }
        @catch (NSException *exception) {
            if ([[exception name] isEqualToString:@"OutOfMemory"]) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    [FSPDFViewCtrl recoverForOOM];
                });
                return;
            }
        }

        [self setBottomBaseBarHidden:YES];
    } else {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.searchBar resignFirstResponder];
            self.foundLable.text = @"";
            self.totalView.backgroundColor = self.fs_backgroundColor_B1;
            [self setTableViewHidden:NO];
            [self reloadEmptyDataSet];

        });

    }
}



#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    if (self.arraySearch.count > 0) {
        return self.arraySearch.count; //: arraySearch.count+1/*show total match count*/;
    } else {
        return 0;
    }
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    SearchResult *oneResult = [self.arraySearch objectAtIndex:section];
    return oneResult.infos.count + 1 /*first line is page summary*/;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    CGFloat height = 0;
    SearchResult *oneResult = [self.arraySearch objectAtIndex:indexPath.section];
    if (indexPath.row == 0) //page summary
    {
        height = STYLE_PAGE_SUMMARY_HEIGHT;
    } else {
        SearchInfo *oneInfo = [oneResult.infos objectAtIndex:indexPath.row - 1];

//        NSString *text = oneInfo.snippet;
        int width = 300;
        if (DEVICE_iPHONE || [self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            width = self.pdfViewCtrl.bounds.size.width;
        }
        if (self.isRedactChoose) {
            width -= 40;
        }
        
//        CGSize textSize = [Utility getTextSize:text fontSize:SEARCH_LABEL_FONT maxSize:CGSizeMake(width - 2 * STYLE_INFO_LEFT, 1000)];
        height = MAX(oneInfo.cellHeight + 2 * STYLE_INFO_TOP,self.isRedactChoose? 44:STYLE_PAGE_SUMMARY_HEIGHT);
        if (height < 0 ) {
            height = 0;
        }
    }
    
    return height;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = nil;
    if (indexPath.row == 0) //page summary or show total match count
    {
        static NSString *CellIdentifier = @"PageSummaryCellIdentifier";
        cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
        if (cell == nil) {
            cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleValue2 reuseIdentifier:CellIdentifier];
            cell.backgroundColor = self.fs_backgroundColor_B2;

            UILabel *labelLeft = [[UILabel alloc] initWithFrame:CGRectMake(STYLE_PAGE_SUMMARY_LEFT, STYLE_PAGE_SUMMARY_TOP, STYLE_PAGE_SUMMARY_WIDTH, STYLE_PAGE_SUMMARY_LABEL_HEIGHT)];
            labelLeft.textAlignment = NSTextAlignmentLeft;
            labelLeft.font = [UIFont systemFontOfSize:12]; //STYLE_PAGE_SUMMARY_FONT;
            labelLeft.textColor = self.fs_textColor_T3;
            labelLeft.backgroundColor = [UIColor clearColor];
            [cell.contentView addSubview:labelLeft];
            UILabel *labelRight = [[UILabel alloc] initWithFrame:CGRectMake(((DEVICE_iPHONE || [self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) ? STYLE_PAGE_SUMMARY_RIGHT_LEFT_IPHONE : STYLE_PAGE_SUMMARY_RIGHT_LEFT), STYLE_PAGE_SUMMARY_TOP,
                                                                            ((DEVICE_iPHONE || [self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) ? STYLE_PAGE_SUMMARY_WIDTH_IPHONE : STYLE_PAGE_SUMMARY_WIDTH), STYLE_PAGE_SUMMARY_LABEL_HEIGHT)];
            labelRight.textAlignment = NSTextAlignmentRight;
            labelRight.autoresizingMask = UIViewAutoresizingFlexibleWidth;
            labelRight.font = [UIFont systemFontOfSize:12];
            labelRight.textColor = self.fs_textColor_T3;
            labelRight.backgroundColor = [UIColor clearColor];
            [cell.contentView addSubview:labelRight];
            
            UIView *topLineView = [[UIView alloc] init];
            topLineView.backgroundColor = self.fs_dividerColor_P1;
            topLineView.tag = 1000;
            [cell.contentView addSubview:topLineView];
            [topLineView mas_makeConstraints:^(MASConstraintMaker *make) {
                make.left.right.top.equalTo(cell.contentView);
                make.height.mas_equalTo(0.6);
            }];
            
            UIView *bottomLineView = [[UIView alloc] init];
            bottomLineView.backgroundColor = self.fs_dividerColor_P1;
            [cell.contentView addSubview:bottomLineView];
            [bottomLineView mas_makeConstraints:^(MASConstraintMaker *make) {
                make.left.right.bottom.equalTo(cell.contentView);
                make.height.mas_equalTo(0.6);
            }];
        }
        UILabel *labelLeft = [cell.contentView.subviews objectAtIndex:0];
        UILabel *labelRight = [cell.contentView.subviews objectAtIndex:1];
        UIView *topLineView = [cell.contentView viewWithTag:1000];
        SearchResult *oneResult = [self.arraySearch objectAtIndex:indexPath.section];
        labelLeft.text = [NSString stringWithFormat: FSLocalizedForKey(@"kPage"), oneResult.index + 1 /*page index start from 0*/];
        labelRight.text = [NSString stringWithFormat:@"%lu", (unsigned long) oneResult.infos.count];

        if (topLineView) {
            if (indexPath.section == 0) {
                topLineView.hidden = YES;
            }else{
                topLineView.hidden = NO;
            }
        }

    } else {
        if ([self.arraySearch count] == 0)
            return nil;
        static NSString *CellIdentifier = @"searchCellIdentifier";
        cell = [tableView dequeueReusableCellWithIdentifier:CellIdentifier];
        UILabel *labelContent = nil;
        if (cell == nil) {
            cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:CellIdentifier];
            cell.backgroundColor = self.fs_backgroundColor_B1;
            
            FSSearchSelectButton *button = [FSSearchSelectButton buttonWithType:UIButtonTypeCustom];
            button.accessibilityIdentifier = @"common_unselect_id";
            button.tag = 1000002;
            [button setImage:FSImageNamedAndNormalColor(@"common_unselect") forState:UIControlStateNormal];
            [BindThemeKeySel(button, setImage:forState:) fs_updateThemeCompleted:^(UIButton*  _Nonnull target, SEL  _Nonnull sel) {
                [target setImage:FSImageNamedAndPrimaryColor(@"common_select") forState:UIControlStateSelected];
            }];
            
            [button addTarget:self action:@selector(selectBtnPress:) forControlEvents:UIControlEventTouchUpInside];
            [cell.contentView addSubview:button];

            UILabel *labelContent = [[UILabel alloc] initWithFrame:CGRectZero];
            labelContent.backgroundColor = [UIColor clearColor];
            labelContent.tag = 1000001;
            labelContent.font = [UIFont systemFontOfSize:SEARCH_LABEL_FONT];
            labelContent.numberOfLines = 0;
            labelContent.textColor = self.fs_textColor_T4;
            [cell.contentView addSubview:labelContent];
            [labelContent mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.left.equalTo(button.mas_right).offset(8);
                make.height.equalTo(labelContent.superview);
                make.centerY.equalTo(labelContent.superview);
                make.right.equalTo(labelContent.superview).offset(-STYLE_INFO_LEFT);
                
            }];

        }
        cell.backgroundColor = self.fs_backgroundColor_B1;
        FSSearchSelectButton *button = (FSSearchSelectButton *)[cell viewWithTag:1000002];
        if (button) {
            button.indexPath = indexPath;
            [button mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.left.mas_equalTo(8);
                make.height.mas_equalTo(40);
                make.centerY.equalTo(button.superview);
                if (self.isRedactChoose) {
                    make.width.mas_equalTo(40);
                }else{
                    make.width.mas_equalTo(0);
                }
            }];
            
            NSArray *array = [self.selectDict objectForKey:[NSNumber numberWithInt:(int)indexPath.section]];
            if ([array containsObject:[NSNumber numberWithInt:(int)indexPath.row-1]]) {
                button.selected = YES;
                button.accessibilityIdentifier = @"common_select_id";
            }else{
                button.selected = NO;
                button.accessibilityIdentifier = @"common_unselect_id";
            }
        }
        
        if (!labelContent) {
            labelContent = (UILabel *)[cell viewWithTag:1000001];
        }
        SearchResult *oneResult = [self.arraySearch objectAtIndex:indexPath.section];
        
        SearchInfo *oneInfo = [oneResult.infos objectAtIndex:indexPath.row - 1];

        NSMutableAttributedString *contentStr = [[NSMutableAttributedString alloc]initWithString:oneInfo.snippet];
        NSString *pattern = self.textSearchKeyWords;
        
        if (pattern.length > 0 && contentStr.length > 0 && oneInfo.keywordLocation >= 0) {
            NSRange match = NSMakeRange(oneInfo.keywordLocation, pattern.length);
            NSString *matchStr = [oneInfo.snippet substringWithRange:match];
            if ([matchStr.lowercaseString isEqualToString:pattern.lowercaseString]) {
                [contentStr addAttribute:NSFontAttributeName value:[UIFont systemFontOfSize:12.0] range:match];
                [contentStr addAttribute:NSBackgroundColorAttributeName value:[UIColor colorWithRGB:0xffeea2 alpha:1] range:match];
                [contentStr addAttribute:NSForegroundColorAttributeName value:[UIColor blackColor] range:match];
            }else{
                FSPDFPage *pdfPage = [[self.extensionsManager.pdfViewCtrl currentDoc] getParsedPage:oneResult.index];
                FSTextPage *textPage = [[FSTextPage alloc] initWithPage:pdfPage flags:0];
                
                for (SearchInfo *info in oneResult.infos) {
                    for (id obj in info.rects) {
                        CGRect rect = [obj CGRectValue];
                        FSRectF *docRectIva = [Utility CGRect2FSRectF:rect];
                        NSString *text = [textPage getTextInRect:docRectIva];
                        
                        NSRange match = [oneInfo.snippet rangeOfString:text];
                        [contentStr addAttribute:NSFontAttributeName value:[UIFont systemFontOfSize:12.0] range:match];
                        [contentStr addAttribute:NSBackgroundColorAttributeName value:[UIColor colorWithRGB:0xffeea2 alpha:1] range:match];
                        [contentStr addAttribute:NSForegroundColorAttributeName value:[UIColor blackColor] range:match];
                    }
                }
            }
        }
        
        [labelContent setAttributedText:contentStr];

    }
    
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    if (indexPath.section >= 0 && indexPath.section < [self.arraySearch count]) {
        SearchResult *oneResult = [self.arraySearch objectAtIndex:indexPath.section];
        if (indexPath.row != 0) //page summary
        {
            SearchInfo *oneInfo = [oneResult.infos objectAtIndex:indexPath.row - 1];
            CGRect firstNeedDrawRect = [oneInfo.rects count] ? [[oneInfo.rects objectAtIndex:0] CGRectValue] : CGRectZero;
            self.needDrawRects = oneInfo.rects;
            self.needPageIndex = oneResult.index;

            if ([self.pdfViewCtrl isContinuous]) {
                FSPointF *point = [[FSPointF alloc] init];
                [point set:CGRectGetMidX(firstNeedDrawRect) - 80 y:CGRectGetMidY(firstNeedDrawRect) + 150];
                [self.pdfViewCtrl gotoPage:oneResult.index withDocPoint:point animated:NO];
            } else {
                FSPointF *point = [[FSPointF alloc] init];
                [point set:CGRectGetMidX(firstNeedDrawRect) y:CGRectGetMidY(firstNeedDrawRect)];
                point.x -= 20;
                CGFloat topH = [Utility convertWidth:self.topBar.fs_height fromPageViewToPDF:self.pdfViewCtrl pageIndex:[self.pdfViewCtrl getCurrentPage]];
                [point setY:[point getY] + topH + 20];
                [self.pdfViewCtrl gotoPage:oneResult.index withDocPoint:point animated:NO];
            }
            [self.pdfViewCtrl refreshOverlay];
            [self setTableViewHidden:YES];
            [self setBottomBaseBarHidden:NO];
            self.selectedSearchResult = oneResult;
            self.selectedSearchInfo = oneInfo;
            [self.tableView deselectRowAtIndexPath:indexPath animated:NO];
        }
    }
}

#pragma mark IDrawEventListener

- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context {
    if (self.onSearchState) {

        for (SearchResult *searchResult in self.arraySearch) {
            if (searchResult.index == pageIndex) {
                for (SearchInfo *info in searchResult.infos) {
                    
                    for (id obj in info.rects) {
                        CGRect rect = [obj CGRectValue];
                        FSRectF *docRectIva = [Utility CGRect2FSRectF:rect];
                        CGRect pvRectIna = [self.pdfViewCtrl convertPdfRectToPageViewRect:docRectIva pageIndex:pageIndex];
                        UIColor *color = [UIColor colorWithRGB:0xffeea2 alpha:0.59];
                        CGContextSetFillColorWithColor(context, [color CGColor]);
                        CGContextFillRect(context, pvRectIna);
                    }
                }
            }
        }
        
        if (pageIndex == self.needPageIndex) {
            
            for (id obj in self.needDrawRects) {
                CGRect rect = [obj CGRectValue];
                FSRectF *docRectIva = [Utility CGRect2FSRectF:rect];
                CGRect pvRectIna = [self.pdfViewCtrl convertPdfRectToPageViewRect:docRectIva pageIndex:pageIndex];
                UIColor *color = [UIColor colorWithRGB:0xff8a00 alpha:0.59];
                CGContextSetFillColorWithColor(context, [color CGColor]);
                CGContextFillRect(context, pvRectIna);
            }
        }

    }
}

#pragma mark IGestureEventListener

- (BOOL)onTap:(UITapGestureRecognizer *)recognizer {
    if (self.onSearchState) {
        if (_isKeyboardShowing) {
            [self.searchBar resignFirstResponder];
        }
    }
    return NO;
}

- (BOOL)onLongPress:(UILongPressGestureRecognizer *)recognizer {
    if (self.onSearchState) {
        if (_isKeyboardShowing) {
            [self.searchBar resignFirstResponder];
        }
    }
    return NO;
}

- (void)keyboardWasShown:(NSNotification *)aNotification {
    if (self.onSearchState) {
        _isKeyboardShowing = YES;
    }
}

- (void)keyboardWasHidden:(NSNotification *)aNotification {
    if (self.onSearchState) {
        _isKeyboardShowing = NO;
    }
}

#pragma mark-- IDocEventListener
- (void)onDocWillOpen {
}

- (void)onDocOpened:(FSPDFDoc *)document error:(int)error {
    if ([self.pdfViewCtrl isDynamicXFA]) {
        if (DEVICE_iPHONE || [self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            if (self.searchBtnitem) {
                self.searchBtnitem.enabled = NO;
            }
            
        }else{
            if (self.searchItem) {
                self.searchItem.enabled = NO;
            }
        }
    }else{
        if (DEVICE_iPHONE || [self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            if (self.searchBtnitem) {
                self.searchBtnitem.enabled = YES;
            }
            
        }else{
            if (self.searchItem) {
                self.searchItem.enabled = YES;
            }
        }
    }

}

- (void)onDocWillClose:(FSPDFDoc *)document {
    if (!DEVICE_iPHONE && ![self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact] && self.searchbyController.presentingViewController) {
        [self.searchbyController.presentingViewController dismissViewControllerAnimated:true completion:nil];
    }else{
        [self.searchByPopoverCtrl dismissViewControllerAnimated:true completion:nil];
    }
    if (![self.pdfViewCtrl isDynamicXFA] && self.onSearchState) {
        [self cancelSearchWithExit:YES];
    }
}

- (void)onDocClosed:(FSPDFDoc *)document error:(int)error {
}

- (void)onDocWillSave:(FSPDFDoc *)document {
}

#pragma mark - IRotationEventListener

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation{
    self.maskView.frame = CGRectMake(0, 0, CGRectGetWidth(self.pdfViewCtrl.bounds), CGRectGetHeight(self.pdfViewCtrl.bounds));
}

#pragma mark - IWindowFrameChangedListener
- (void)onWindowFrameChanged:(CGRect)newFrame horizontalSizeClassChanged:(BOOL)changed{
    if (DEVICE_iPAD && changed) {

        if (!self.tableviewHidden) {
            [self.tableView mas_remakeConstraints:^(MASConstraintMaker *make) {
                if (DEVICE_iPAD && ![self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
                    make.left.equalTo(self.tableView.superview.mas_right).offset(-300);
                    make.width.mas_equalTo(300);
                }else{
                    make.left.equalTo(self.tableView.superview);
                    make.width.equalTo(self.tableView.superview);
                }
                make.bottom.equalTo(self.redactionBtn.mas_top).offset(0);
            }];
            [self.tableView reloadData];
        }

        if (!self.bottomBaseBarHidden) {
            [self.bottomBar mas_remakeConstraints:^(MASConstraintMaker *make) {
                if (DEVICE_iPAD && ![self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
                    make.centerX.equalTo(self.pdfViewCtrl);
                    make.width.mas_equalTo(184);
                    make.bottom.equalTo(self.pdfViewCtrl).offset(-24);
                }else{
                    make.left.right.equalTo(self.pdfViewCtrl);
                    make.bottom.equalTo(self.pdfViewCtrl);
                }
            }];
            if (DEVICE_iPAD && ![self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
                self.bottomBar.layer.cornerRadius = 6;
            }else{
                self.bottomBar.layer.cornerRadius = 0;
            }
        }
        
        if (DEVICE_iPAD && ![self.pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            if (self.searchByPopoverCtrl.presentingViewController) {
                [self.searchByPopoverCtrl.presentingViewController dismissViewControllerAnimated:true completion:^{
                    CGRect rect = [self.filterItem convertRect:self.filterItem.bounds toView:self.pdfViewCtrl];
                    FSNavigationController *searchByNavCtrl = [[FSNavigationController alloc] initWithRootViewController:self.searchbyController];
                    searchByNavCtrl.navigationBarHidden = YES;
                    searchByNavCtrl.preferredContentSize = CGSizeMake(250, 50 * 3);
                    searchByNavCtrl.modalPresentationStyle = UIModalPresentationPopover;
                    searchByNavCtrl.popoverPresentationController.delegate = self;
                    searchByNavCtrl.popoverPresentationController.sourceView = self.pdfViewCtrl;
                    searchByNavCtrl.popoverPresentationController.sourceRect = rect;
                    searchByNavCtrl.popoverPresentationController.permittedArrowDirections = UIPopoverArrowDirectionUp;
                    [self.pdfViewCtrl.fs_viewController presentViewController:searchByNavCtrl animated:true completion:nil];
                }];
            }
            
        }else{
            if (self.searchbyController.presentingViewController) {
                [self.searchbyController.presentingViewController dismissViewControllerAnimated:true completion:^{
                    self.searchbyController.view.frame = CGRectMake(0, 0, MIN(self.extensionsManager.pdfViewCtrl.fs_width, self.extensionsManager.pdfViewCtrl.fs_height), 50 * 3 + 10);
                    self.searchByPopoverCtrl = [self.extensionsManager.viewCtrlManager popPopoverCustomViewCtrl:self.searchbyController sourceView:nil];
                }];
            }
        }
    }
}
@end

