/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "SearchViewController.h"
#import "FSBaseViews.h"
#import "FSSearchTableViewCell.h"


@implementation SearchbyViewController

@synthesize searchbyClickedHandler = _searchbyClickedHandler;

- (id)initWithStyle:(UITableViewStyle)style withSearchFlag:(unsigned int)searchFlag{
    self = [super initWithStyle:style];
    if (self) {
        _searchFlag = searchFlag;
        _titles = [[NSArray alloc] initWithObjects:
                                       FSLocalizedForKey(@"kCaseSensitive"),
                                       FSLocalizedForKey(@"kWholeWordsOnly"),
                                       FSLocalizedForKey(@"kSearchInInternet"),
                                       nil];
    }
    return self;
}

- (void)didReceiveMemoryWarning {
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];

    // Release any cached data, images, etc that aren't in use.
}

- (void)setSearchFlag:(unsigned int)searchFlag
{
    _searchFlag = searchFlag;
    if ([self isViewLoaded]){
        [self.tableView reloadData];
    }
}

#pragma mark - View lifecycle

- (void)viewDidLoad {
    [super viewDidLoad];
    self.tableView.scrollEnabled = NO;
    self.navigationItem.title = FSLocalizedForKey(@"kSearch");
    self.tableView.backgroundColor = self.fs_backgroundColor_B1;
    self.tableView.separatorStyle = UITableViewCellSeparatorStyleNone;
}

/*
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}
*/

- (void)searchSwitch:(UISwitch *)sender{
    if (self.searchbyClickedHandler) {
        self.searchbyClickedHandler((int) sender.tag - 100, sender.on);
    }
}

#pragma mark - Table view data source

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    // Return the number of sections.
    return 1;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath{
    return 50;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return _titles.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *cellIdentifier1 = @"searchbyCellWithSwitchIdentifier";
    static NSString *cellIdentifier2 = @"searchbyCellIdentifier";

    FSSearchTableViewCell *cell;
    if (indexPath.row == 0 || indexPath.row == 1){
        cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier1];
    }else{
        cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier2];
    }
    
    cell.backgroundColor = self.tableView.backgroundColor;
    if (cell == nil) {
        if (indexPath.row == 0 || indexPath.row == 1){
            cell = [[FSSearchTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellIdentifier1];
        }else{
            cell = [[FSSearchTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellIdentifier2];
        }
        if (indexPath.row == 0 || indexPath.row == 1) {
            FSSwitch *searchSwitch = [[FSSwitch alloc] initWithFrame:CGRectZero];
            [searchSwitch addTarget:self action:@selector(searchSwitch:) forControlEvents:UIControlEventTouchUpInside];
            searchSwitch.tag = indexPath.row + 100;
            [cell.contentView addSubview:searchSwitch];
            [searchSwitch mas_makeConstraints:^(MASConstraintMaker *make) {
                make.right.equalTo(cell.contentView.mas_right).offset(-15);
                make.centerY.equalTo(cell.contentView);
            }];
            
            BindThemeKeyPath(searchSwitch, onTintColor);

            cell.selectionStyle = UITableViewCellSelectionStyleNone;
        }else{
            cell.accessibilityTraits = UIAccessibilityTraitButton;
        }
    }
    if (indexPath.row == 0 || indexPath.row == 1) {
        for (UIView *view in cell.contentView.subviews) {
            if ([view isKindOfClass:[FSSwitch class]]) {
                FSSwitch *searchSwitch = (FSSwitch *)view;
                if (indexPath.row == 0) searchSwitch.on = (self.searchFlag & FSTextSearchSearchMatchCase);
                if (indexPath.row == 1) searchSwitch.on = (self.searchFlag & FSTextSearchSearchMatchWholeWord);
            }
        }
    }
    cell.backgroundColor = self.fs_backgroundColor_B1;
    cell.titleLabel.text = [_titles objectAtIndex:indexPath.row];

    cell.isAccessibilityElement = YES;
    if (indexPath.row == 0 || indexPath.row == 1) {
        cell.accessibilityLabel = [NSString stringWithFormat:@"%@ %@",cell.titleLabel.text,FSLocalizedForKey(@"koffVoiceOver")];
        cell.accessibilityHint = FSLocalizedForKey(@"kDobClickVoiceOver");
    }else{
        cell.accessibilityLabel = cell.titleLabel.text;
    }
    
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    if (self.searchbyClickedHandler && indexPath.row == 2) {
        self.searchbyClickedHandler(2, YES);
    }
    
    if (UIAccessibilityIsVoiceOverRunning()) {
        if (indexPath.row == 0 || indexPath.row == 1) {
            FSSearchTableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
            for (UIView *view in cell.contentView.subviews) {
                if ([view isKindOfClass:[FSSwitch class]]) {
                    FSSwitch *searchSwitch = (FSSwitch *)view;
                    searchSwitch.on = !searchSwitch.on;
                    cell.accessibilityLabel = [NSString stringWithFormat:@"%@ %@",cell.titleLabel.text,searchSwitch.on ? FSLocalizedForKey(@"konVoiceOver") : FSLocalizedForKey(@"koffVoiceOver")];
                    [self searchSwitch:searchSwitch];
                    break;
                }
            }
        }
    }
}

@end
