/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSSettingBarHorizontalCell.h"

@interface FSSettingBarHorizontalCell()
@property(nonatomic, strong) FSItem* imageItem;
@property(nonatomic, strong) FSItem* selectedImageWithColorCellItem;
@property(nonatomic, strong) FSItem* bgImageWithColorCellItem;
@property(nonatomic, strong) FSItem* titleItem;
@property(nonatomic, strong) UIColor* bgColor;
@property(nonatomic, strong) NSString* bgImageName;
@end

@implementation FSSettingBarHorizontalCell

- (instancetype)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        [self initSubView];
        self.isAccessibilityElement = YES;
    }
    return self;
}

- (void)initSubView
{
    self.itemSelectedImageNameWithColorCell = @"setting_color_selected";
    self.bgImageName = @"setting_color_background";
    self.contentView.backgroundColor = self.fs_backgroundColor_B1;
    self.contentView.frame = self.bounds;
    
    CGFloat imageWidth = 60;
    FSItem* bgImageWithColorCellItem = [FSItem createItemWithImage:FSImageNamed(@"")];
    _bgImageWithColorCellItem = bgImageWithColorCellItem;
    bgImageWithColorCellItem.userInteractionEnabled = NO;
    [self.contentView  addSubview:bgImageWithColorCellItem];
    [_bgImageWithColorCellItem mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(bgImageWithColorCellItem.superview);
        make.top.mas_equalTo(5);
        make.height.width.mas_equalTo(imageWidth);
    }];
    [bgImageWithColorCellItem setHidden:YES];
    
    FSItem* imageItem = [FSItem createItemWithImage:FSImageNamed(@"")];
    _imageItem = imageItem;
    imageItem.userInteractionEnabled = NO;
    [self.contentView addSubview:_imageItem];
    [imageItem mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(bgImageWithColorCellItem.mas_left);
        make.top.equalTo(bgImageWithColorCellItem.mas_top);
        make.height.width.mas_equalTo(imageWidth);
    }];
    
    FSItem* titleItem = [FSItem createItemWithTitle:@""];
    _titleItem = titleItem;
    titleItem.userInteractionEnabled = NO;
    [titleItem.titleLabel setFont:self.fs_standardWeakFont_WF1];
    [_titleItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
    [_titleItem setTitleColor:self.fs_iconColor_I2 forState:UIControlStateDisabled];
    titleItem.titleLabel.textAlignment = NSTextAlignmentCenter;
    [self.contentView addSubview:titleItem];
    _titleItem.titleLabel.numberOfLines = 0;
    [_titleItem mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(-10);
        make.right.mas_equalTo(10);
        make.top.equalTo(imageItem.mas_bottom).offset(10);
    }];
    
    FSItem* selectedImageWithColorCellItem = [FSItem createItemWithImage:FSImageNamed(@"")];
    _selectedImageWithColorCellItem = selectedImageWithColorCellItem;
    selectedImageWithColorCellItem.userInteractionEnabled = NO;
    [self.contentView  addSubview:selectedImageWithColorCellItem];
    [selectedImageWithColorCellItem mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(bgImageWithColorCellItem.mas_left);
        make.top.equalTo(bgImageWithColorCellItem.mas_top);
        make.height.width.mas_equalTo(imageWidth);
    }];
    [selectedImageWithColorCellItem setHidden:YES];
    NSString* itemSelectedImageNameWithColorCell = _itemSelectedImageNameWithColorCell;
    [BindThemeKeySel(_selectedImageWithColorCellItem, setImage:forState:) fs_updateThemeCompleted:^(FSItem*  _Nonnull target, SEL  _Nonnull sel) {
        [target setImage:[FSImageNamed(itemSelectedImageNameWithColorCell) fs_imageWithTintColor:target.fs_primaryColor] forState:UIControlStateSelected];
    }];
}

- (void)setCellInfo:(NSString*)itemTitleName itemImageName:(NSString*)itemImageName sectionType:(FSSettingBarCellSectionType)sectionType  itemType:(SettingItemType)itemType pageColor:(UIColor*)pageColor
{
    _itemImageName = itemImageName;
    _itemTitleName = itemTitleName;
    _sectionType = sectionType;
    _itemType = itemType;
    
    if (_itemTitleName && _itemTitleName.length > 1) {
        self.contentView.accessibilityIdentifier = _itemTitleName;
    }
    
    [_imageItem setImage:FSImageNamed(itemImageName) forState:UIControlStateNormal];
    [_titleItem setTitle:itemTitleName forState:UIControlStateNormal];
    if (_sectionType == SectionTypeHorizontalPage) {
        [BindThemeKeySel(_imageItem, setImage:forState:) fs_updateThemeCompleted:^(FSItem*  _Nonnull target, SEL  _Nonnull sel) {
            [target setImage:[FSImageNamed(itemImageName) fs_imageWithTintColor:target.fs_primaryColor] forState:UIControlStateSelected];
        }];
    }

    if (itemType == PAGECOLOR ) {
        [_bgImageWithColorCellItem setHidden:NO];
        [_bgImageWithColorCellItem setImage:[FSImageNamed(_bgImageName) fs_imageWithTintColor:pageColor] forState:UIControlStateNormal];
    }else{
        [_bgImageWithColorCellItem setHidden:YES];
    }
}
- (void)setCellSelected:(BOOL)cellSelected
{
    if (_sectionType == SectionTypeHorizontalColor) {
        [_selectedImageWithColorCellItem setSelected:cellSelected];
        [_selectedImageWithColorCellItem setHidden:!cellSelected];
        [_imageItem setSelected:NO];
    }
    else
    {
        [_selectedImageWithColorCellItem setHidden:YES];
        [_imageItem setSelected:cellSelected];
    }
}

- (void)setEnabled:(BOOL)enabled
{
    self.imageItem.enabled = enabled;
    self.titleItem.enabled = enabled;
}

@end
