/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSSettingBarUtilities.h"

@implementation FSSettingBarCellInfo
- (instancetype)initWithItemType:(SettingItemType)itemType{
    self = [super init];
    if (self) {
        _itemType = itemType;
        _selected = NO;
        _hidden = NO;
        _enabled = YES;
        _defaultEnabled = YES;
        switch (itemType) {
            case SINGLE:
                _title = FSLocalizedForKey(@"kSinglePage");
                _imageName = @"setting_single_page_normal";
                _sectionType = SectionTypeHorizontalPage;
                _selected = YES;
                break;
            case DOUBLEPAGE:
                _title = FSLocalizedForKey(@"kViewModeNoCover");
                _imageName = @"setting_facing_normal";
                _sectionType = SectionTypeHorizontalPage;
                break;
            case COVERPAGE:
                _title = FSLocalizedForKey(@"kCoverFacing");
                _imageName = @"setting_cover_facing_normal";
                _sectionType = SectionTypeHorizontalPage;
                break;
            case DAYMODE:
                _title = FSLocalizedForKey(@"kDay");
                _imageName = @"setting_color_day_normal";
                _sectionType = SectionTypeHorizontalColor;
                _selected = YES;
                break;
            case PAGECOLOR:
                _title = FSLocalizedForKey(@"kPageColor");
                _imageName = @"setting_color_page_color";
                _sectionType = SectionTypeHorizontalColor;
                break;
            case NIGHTMODE:
                _title = FSLocalizedForKey(@"kNight");
                _imageName = @"setting_color_night_normal";
                _sectionType = SectionTypeHorizontalColor;
                break;
            case CONTINUOUS:
                _title = FSLocalizedForKey(@"kViewModeContinuous");
                _imageName = @"setting_continus_scrolling";
                _sectionType = SectionTypeVertical;
                break;
            case PAGERTL:
                _title = FSLocalizedForKey(@"kRightToLeft");
                _imageName = @"setting_right_to_left";
                _sectionType = SectionTypeVertical;
                break;
            case FITPAGE:
                _title = FSLocalizedForKey(@"kFitPage");
                _imageName = @"setting_fit_page";
                _sectionType = SectionTypeVertical;
                break;
            case FITWIDTH:
                _title = FSLocalizedForKey(@"kFitWidth");
                _imageName = @"setting_fit_width";
                _sectionType = SectionTypeVertical;
                _selected = YES;
                break;
            case REFLOW:
                _title = FSLocalizedForKey(@"kReflow");
                _imageName = @"setting_reflow";
                _sectionType = SectionTypeVertical;
                break;
            case CROPPAGE:
                _title = FSLocalizedForKey(@"kCropMode");
                _imageName = @"setting_crop_mode";
                _sectionType = SectionTypeVertical;
                break;
            case SPEECH:
                _title = FSLocalizedForKey(@"kSpeak");
                _imageName = @"setting_speak";
                _sectionType = SectionTypeVertical;
                break;
            case AUTOFLIP:
                _title = FSLocalizedForKey(@"kAutoFlip");
                _imageName = @"setting_auto_flip";
                _sectionType = SectionTypeVertical;
                break;
            case ROTATE:
                _title = FSLocalizedForKey(@"KRotateView");
                _imageName = @"setting_rotate_view";
                _sectionType = SectionTypeVertical;
                break;
            case PANZOOM:
                _title = FSLocalizedForKey(@"kPanandZoom");
                _imageName = @"setting_pan_zoom";
                _sectionType = SectionTypeVertical;
                break;
        }
    }
    return self;
}

@end
