/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSSettingBarVerticalCell.h"

@interface FSSettingBarVerticalCell()
@property(nonatomic, strong) FSItem* leftImageItem;
@property(nonatomic, strong) FSItem* titleItem;
@property(nonatomic, strong) FSItem* rightImageItem;
@property(nonatomic, assign) SettingItemType cellType;
@end

@implementation FSSettingBarVerticalCell

- (instancetype)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self) {
        [self initSubView];
        self.isAccessibilityElement = YES;
    }
    return self;
}

- (void)initSubView
{
    self.contentView.backgroundColor = self.fs_backgroundColor_B1;
    self.contentView.frame = self.bounds;
    
    FSItem* leftImageItem = [FSItem createItemWithImage:FSImageNamed(@"")];
    _leftImageItem = leftImageItem;
    leftImageItem.userInteractionEnabled = NO;
    [self.contentView addSubview:leftImageItem];
    [leftImageItem mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(16);
        make.centerY.equalTo(leftImageItem.superview);
        make.height.width.mas_equalTo(40);
    }];
    
    FSItem* titleItem = [FSItem createItemWithTitle:@""];
    _titleItem = titleItem;
    titleItem.userInteractionEnabled = NO;
    [titleItem.titleLabel setFont:self.fs_standardGeneralFont_GF2];
    [_titleItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
    [_titleItem setTitleColor:self.fs_iconColor_I2 forState:UIControlStateDisabled];
    titleItem.titleLabel.textAlignment = NSTextAlignmentCenter;
    [self.contentView addSubview:titleItem];
    [titleItem mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(leftImageItem.mas_right).offset(16);
        make.centerY.equalTo(titleItem.superview);
    }];
    [BindThemeKeySel(titleItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem*  _Nonnull target, SEL  _Nonnull sel) {
        [target setTitleColor:target.fs_primaryColor forState:UIControlStateSelected];
    }];
    
    FSItem* rightImageItem = [FSItem createItemWithImage:FSImageNamed(@"common_select_check")];
    rightImageItem.accessibilityIdentifier = @"common_select_check_id";
    _rightImageItem = rightImageItem;
    rightImageItem.userInteractionEnabled = NO;
    [self.contentView addSubview:rightImageItem];
    [rightImageItem mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(-16);
        make.centerY.equalTo(rightImageItem.superview);
        make.height.width.mas_equalTo(40);
    }];
    [BindThemeKeySel(rightImageItem, setImage:forState:) fs_updateThemeCompleted:^(FSItem*  _Nonnull target, SEL  _Nonnull sel) {
        [target setImage:[FSImageNamed(@"common_select_check") fs_imageWithTintColor:target.fs_primaryColor] forState:UIControlStateNormal];
    }];
    [rightImageItem setHidden:YES];
    
    UISwitch *continueSwith = [[UISwitch alloc] init];
    [continueSwith addTarget:self action:@selector(didSwitchBtn) forControlEvents:UIControlEventTouchUpInside];
    _switchBtn = continueSwith;
    continueSwith.onTintColor = UIColor.redColor;
    [BindThemeKeySel(continueSwith, setOnTintColor:) fs_updateThemeCompleted:^(UISwitch*  _Nonnull target, SEL  _Nonnull sel) {
        [target setOnTintColor:target.fs_primaryColor];
    }];
 
    [self.contentView addSubview:continueSwith];
    [continueSwith mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(-16);
        make.centerY.equalTo(continueSwith.superview);
    }];
    [continueSwith setHidden:YES];
}

- (void)didSwitchBtn
{
    if ([self.delegate respondsToSelector:@selector(didClickSwitchBtn:cellType:)]) {
        [self.delegate didClickSwitchBtn:self.switchBtn.on cellType:_cellType];
    }
}

- (void)setCellInfo:(NSString*)itemTitleName itemImageName:(NSString*)itemImageName switchBtnON:(BOOL)switchBtnON cellType:(SettingItemType)cellType
{
    if (itemTitleName && itemTitleName.length > 1) {
        self.contentView.accessibilityIdentifier = itemTitleName;
    }
    _cellType = cellType;
    [_leftImageItem setImage:FSImageNamedAndNormalColor(itemImageName) forState:UIControlStateNormal];
    [_leftImageItem setImage:[FSImageNamed(itemImageName) fs_imageWithTintColor:self.fs_iconColor_I2] forState:UIControlStateDisabled];
    [BindThemeKeySel(_leftImageItem, setImage:forState:) fs_updateThemeCompleted:^(FSItem*  _Nonnull target, SEL  _Nonnull sel) {
        [target setImage:[FSImageNamed(itemImageName) fs_imageWithTintColor:target.fs_primaryColor] forState:UIControlStateSelected];
    }];
    
    [_titleItem setTitle:itemTitleName forState:UIControlStateNormal];
    
    if (cellType == CONTINUOUS || cellType == PAGERTL) {
        [_switchBtn setHidden:NO];
        [_switchBtn setOn:switchBtnON];
        if (switchBtnON){
            _switchBtn.accessibilityIdentifier = @"continue_switch_id_on";
        }else{
            _switchBtn.accessibilityIdentifier = @"continue_switch_id_of";
        }
    }
    else
    {
        [_switchBtn setHidden:YES];
    }
}

- (void)setCellSelected:(BOOL)cellSelected
{
    if (_cellType == FITPAGE || _cellType == FITWIDTH || _cellType == CROPPAGE) {
        [_rightImageItem setHidden:!cellSelected];
        _leftImageItem.selected = cellSelected;
        _titleItem.selected = cellSelected;
    }
    else
    {
        [_rightImageItem setHidden:YES];
        _leftImageItem.selected = NO;
        _titleItem.selected = NO;
    }
}

- (void)setEnabled:(BOOL)enabled
{
    self.leftImageItem.enabled = enabled;
    self.titleItem.enabled = enabled;
}


@end
