/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSSettingContentView.h"
#import "FSSettingBarHorizontalCell.h"
#import "FSSettingBarVerticalCell.h"

static CGFloat HorzontalCellWidth = 75;
static CGFloat HorzontalCellHeight = 85;
static CGFloat HorzontalCellInteritemSpacing = 0;
static CGFloat VerticalCellSideSpace = 0;
static CGFloat VerticalCellHeight = 50;
static CGFloat HorzontalSectionSpacing = 10;
static CGFloat VerticalSectionSpacing = 20;

@interface FSSettingContentView()<UICollectionViewDelegate, UICollectionViewDataSource,FSSettingBarVerticalCellDelegate>
@property (nonatomic, strong) UICollectionView* collectionView;
@property (nonatomic, strong) NSMutableArray<NSMutableArray*>* dataSource;
@property (nonatomic, strong) NSMutableDictionary* dataSourceWithHidden;
@property (nonatomic, assign) NSInteger horzontalPageCellSectionIndex;
@property (nonatomic, assign) NSInteger horzontalColorCellSectionIndex;
@property (nonatomic, assign) NSInteger verticalCellSectionIndex;
@property (nonatomic, assign) CGFloat pdfViewWidth;
@property (nonatomic, strong) FSItem* titleItem;
@end

@implementation FSSettingContentView

- (instancetype)initWithFrame:(CGRect)frame extensionsManager:(UIExtensionsManager *)extensionsManager
{
    self = [super initWithFrame:frame];
    if (self) {
        self.extensionsManager = extensionsManager;
        [self initData];
        [self initCollectionView];
    }
    return self;
}

- (void)initData
{
    _dataSourceWithHidden = [[NSMutableDictionary alloc] init];
    HorzontalCellInteritemSpacing = (self.frame.size.width - HorzontalCellWidth*3)/4;
    _horzontalPageCellSectionIndex = -1;
    _horzontalColorCellSectionIndex = -1;
    _verticalCellSectionIndex = -1;
    _currentShowItemType = -1;
}

- (void)initCollectionView
{
    CGFloat leftOffset = 0;
    if (self.extensionsManager.topToolbar.subitemContentPosition == FSTopbarSubitemContentViewPositionLeft) {
        leftOffset = 12;
    }
    FSItem* title = [FSItem createItemWithTitle:FSLocalizedForKey(@"kViewSettings")];
    title.accessibilityTraits = UIAccessibilityTraitNone;
    title.customAccessibilityTraits = UIAccessibilityTraitNone;
    title.accessibilityLabel = FSLocalizedForKey(@"kViewSettings");
    _titleItem = title;
    title.titleLabel.textColor = self.fs_textColor_T2;
    title.titleLabel.font = self.fs_standardWeakFont_WF1;
    [self addSubview:title];
    [title mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(16 + leftOffset);
        make.top.equalTo(title.superview.fs_mas_top).offset(5);
        make.height.mas_equalTo(20);
    }];
    UICollectionViewFlowLayout *layout = [[UICollectionViewFlowLayout alloc] init];
    layout.scrollDirection = UICollectionViewScrollDirectionVertical;
    _collectionView = [[UICollectionView alloc] initWithFrame:self.bounds collectionViewLayout:layout];
    
    [self addSubview:_collectionView];
    
    _collectionView.backgroundColor = self.fs_backgroundColor_B1;
    
    [_collectionView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(title.mas_bottom).offset(5);
        make.left.right.bottom.mas_equalTo(0);
    }];
    _collectionView.delegate = self;
    _collectionView.dataSource = self;
    _collectionView.allowsMultipleSelection = YES;
    
    [_collectionView registerClass:[FSSettingBarHorizontalCell class] forCellWithReuseIdentifier:NSStringFromClass([FSSettingBarHorizontalCell class])];
    
    [_collectionView registerClass:[FSSettingBarVerticalCell class] forCellWithReuseIdentifier:NSStringFromClass([FSSettingBarVerticalCell class])];
}

- (CGFloat)layoutHeight{
    CGFloat height = 0;
    for (int s = 0; s < [self numberOfSectionsInCollectionView:self.collectionView]; s++) {
        height += [self collectionView:self.collectionView layout:self.collectionView.collectionViewLayout minimumInteritemSpacingForSectionAtIndex:s];
        if (self.dataSource[s] && self.dataSource[s].count > 0) {
            FSSettingBarCellInfo *info = self.dataSource[s][0];
            FSSettingBarCellSectionType sectionType = info.sectionType;
            if (sectionType == _horzontalPageCellSectionIndex || sectionType == _horzontalColorCellSectionIndex) {
                NSIndexPath *indexPath = [NSIndexPath indexPathForItem:0 inSection:s];
                height += [self collectionView:self.collectionView layout:self.collectionView.collectionViewLayout sizeForItemAtIndexPath:indexPath].height;
                continue;
            }
        }
        UIEdgeInsets insets = [self collectionView:self.collectionView layout:self.collectionView.collectionViewLayout insetForSectionAtIndex:s];
        height += insets.top + insets.bottom;
        for (int i = 0; i < [self collectionView:self.collectionView numberOfItemsInSection:s]; i++) {
            NSIndexPath *indexPath = [NSIndexPath indexPathForItem:i inSection:s];
            height += [self collectionView:self.collectionView layout:self.collectionView.collectionViewLayout sizeForItemAtIndexPath:indexPath].height;
        }
    }
    return height;
}

#pragma mark-- setting data

- (void)reloadWithData:(NSMutableArray<NSMutableArray*>*)dataSource
{
    if (self.dataSource == nil) {
        self.dataSource = [NSMutableArray array];
    }
    self.dataSource = [dataSource copy];
}

- (void)reloadData
{
    if (self.collectionView) {
        [self.collectionView reloadData];
    }
}

- (void)reloadDataWithFrameChanged:(CGFloat)newFrameWidth
{
    if (self.collectionView) {
        HorzontalCellInteritemSpacing = (newFrameWidth - HorzontalCellWidth*3)/4;
        [self.collectionView reloadData];
    }
}

- (void)setSectionSelectedWithSectionIndex:(NSInteger)sectionIndex selected:(BOOL)selected
{
    NSInteger j = 0;
    BOOL isReplace = NO;
    for (NSMutableArray* array in self.dataSource) {
        if (j == sectionIndex) {
            for (int i = 0; i < array.count; i++) {
                FSSettingBarCellInfo *info = array[i];
                info.selected = selected;
            }
            isReplace = YES;
            break;
        }
        
        if (isReplace) {
            break;
        }
        j ++;
    }
}

- (void)setExtensionsManager:(UIExtensionsManager *)extensionsManager
{
    _extensionsManager = extensionsManager;
}

#pragma mark--UICollectionViewDelegate

- (CGFloat)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout minimumInteritemSpacingForSectionAtIndex:(NSInteger)section
{
    if (self.dataSource[section] && self.dataSource[section].count > 0) {
        FSSettingBarCellInfo *info = self.dataSource[section][0];
        FSSettingBarCellSectionType sectionType = info.sectionType;
        
        if (sectionType == _horzontalPageCellSectionIndex || sectionType == _horzontalColorCellSectionIndex) {
            return HorzontalCellInteritemSpacing;
        }
        return 0;
    }
    return 0;
}

-(UIEdgeInsets)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout insetForSectionAtIndex:(NSInteger)section
{
    if (self.dataSource[section] && self.dataSource[section].count > 0) {
        FSSettingBarCellInfo *info = self.dataSource[section][0];
        FSSettingBarCellSectionType sectionType = info.sectionType;
        if (sectionType == _horzontalPageCellSectionIndex || sectionType == _horzontalColorCellSectionIndex) {
            return UIEdgeInsetsMake(HorzontalSectionSpacing, HorzontalCellInteritemSpacing, HorzontalSectionSpacing, HorzontalCellInteritemSpacing);
        }
        return UIEdgeInsetsMake(VerticalSectionSpacing, VerticalCellSideSpace, 0, VerticalCellSideSpace);
    }
    return UIEdgeInsetsMake(0,0,0,0);
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section
{
    return self.dataSource[section].count;
}

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout sizeForItemAtIndexPath:(NSIndexPath *)indexPath{
    CGSize cellSize;
    FSSettingBarCellInfo *info = self.dataSource[indexPath.section][indexPath.row];
    FSSettingBarCellSectionType sectionType = info.sectionType;
    
    if (sectionType == _horzontalPageCellSectionIndex || sectionType == _horzontalColorCellSectionIndex) {
        cellSize = CGSizeMake(HorzontalCellWidth, HorzontalCellHeight);
    }
    else
    {
        cellSize = CGSizeMake(self.frame.size.width-VerticalCellSideSpace*2, VerticalCellHeight);
    }
    return  cellSize;
}

- (void)setSectionIndex:(FSSettingBarCellInfo*)info
{
    int iType = info.sectionType;
    if (_horzontalPageCellSectionIndex == -1) {
        _horzontalPageCellSectionIndex = (iType == SectionTypeHorizontalPage ) ? iType : -1;
    }
    if (_horzontalColorCellSectionIndex == -1) {
        _horzontalColorCellSectionIndex = (iType == SectionTypeHorizontalColor) ? iType : -1;
    }
    if (_verticalCellSectionIndex == -1) {
        _verticalCellSectionIndex = (iType == SectionTypeVertical) ? iType : -1;
    }
}

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView
{
    if (self.dataSource.count == 3) {
        _horzontalPageCellSectionIndex = SectionTypeHorizontalPage;
        _horzontalColorCellSectionIndex = SectionTypeHorizontalColor;
        _verticalCellSectionIndex = SectionTypeVertical;
    } else if (self.dataSource.count == 2) {
        [self setSectionIndex:self.dataSource[0][0]];
        [self setSectionIndex:self.dataSource[1][0]];
    } else if (self.dataSource.count == 1) {
        [self setSectionIndex:self.dataSource[0][0]];
    }
    return self.dataSource.count;
}

- (__kindof UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath
{
    FSSettingBarCellInfo *info = self.dataSource[indexPath.section][indexPath.row];
    NSString *title = info.title;
    NSString *imageName =  info.imageName;
    FSSettingBarCellSectionType sectionType = info.sectionType;
    SettingItemType cellType = info.itemType;
    BOOL selected = info.selected;
    BOOL defaultEnabled = info.defaultEnabled;
    BOOL enabled = info.enabled;
    BOOL multiPages = !info.singlePage;
    
    UIColor* pageColor = nil;
    
    if (sectionType == self.horzontalPageCellSectionIndex || sectionType == self.horzontalColorCellSectionIndex) {
        FSSettingBarHorizontalCell* cell  = (FSSettingBarHorizontalCell*)[collectionView dequeueReusableCellWithReuseIdentifier:NSStringFromClass([FSSettingBarHorizontalCell class]) forIndexPath:indexPath];
        
        if (info) {
            if (cellType == PAGECOLOR) {
                pageColor = [UIColor colorWithHexString:info.pageColor];
            }
            [cell setCellInfo:title itemImageName:imageName sectionType:sectionType itemType:cellType pageColor:pageColor];
        }
        cell.cellSelected = selected;
        cell.accessibilityLabel = title;
        if (selected) {
            cell.accessibilityTraits = UIAccessibilityTraitSelected;
        }else{
            cell.accessibilityTraits = UIAccessibilityTraitNone;
        }
        cell.enabled = defaultEnabled && enabled && multiPages;
        cell.userInteractionEnabled = defaultEnabled && enabled && multiPages;
        return cell;
    }
    
    FSSettingBarVerticalCell* cell = (FSSettingBarVerticalCell*)[collectionView dequeueReusableCellWithReuseIdentifier:NSStringFromClass([FSSettingBarVerticalCell class]) forIndexPath:indexPath];
    cell.accessibilityLabel = title;
    cell.delegate = self;
    if (info) {
        BOOL switchBtnON = [info.continusON boolValue];
        if (cellType == CONTINUOUS || cellType == PAGERTL) {
            cell.accessibilityHint = FSLocalizedForKey(@"kDobClickVoiceOver");
            cell.accessibilityValue = cell.switchBtn.on ? FSLocalizedForKey(@"konVoiceOver") :  FSLocalizedForKey(@"koffVoiceOver");
            cell.selectedBackgroundView = nil;
            if (cellType == CONTINUOUS) {
            }else if(cellType == PAGERTL){
                switchBtnON = [info.pageRTLON boolValue];
            }
            cell.switchBtn.on = switchBtnON;
        }else{
            cell.selectedBackgroundView = [UIView new];
            cell.selectedBackgroundView.backgroundColor = self.fs_dividerColor_P1;
            cell.contentView.backgroundColor = nil;
        }
        [cell setCellInfo:title itemImageName:imageName switchBtnON:switchBtnON cellType:cellType];
    }
    cell.cellSelected = selected;
    if (selected && (cellType != CONTINUOUS || cellType != PAGERTL)) {
        cell.accessibilityTraits = UIAccessibilityTraitSelected;
    }else{
        cell.accessibilityTraits = UIAccessibilityTraitNone;
    }
    cell.enabled = defaultEnabled && enabled && multiPages;
    cell.userInteractionEnabled = defaultEnabled && enabled && multiPages;
    return cell;
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath
{
    FSSettingBarCellInfo *info = self.dataSource[indexPath.section][indexPath.row];
    SettingItemType type = info.itemType;
    
    if (indexPath.section == _horzontalPageCellSectionIndex || indexPath.section == _horzontalColorCellSectionIndex)
    {
        [self setSectionSelectedWithSectionIndex:indexPath.section selected:NO];
        if (!(type == PAGECOLOR && self.extensionsManager.pdfViewCtrl.isNightMode)) {
            [_extensionsManager.settingBar.viewSettingManager setItemSelectedWithType:type selected:YES];
        }
        [self.collectionView reloadData];
        if (indexPath.section == _horzontalPageCellSectionIndex) {
            if ([self.extensionsManager getState] == FSUIManagerStateAutoFlip) {
                [self.extensionsManager changeState:FSUIManagerStateNormal];
            }
        }
    }
    else
    {
        [_extensionsManager.settingBar.viewSettingManager setItemSelectedWithType:type selected:YES];
        if (!(type == CONTINUOUS || type == PAGERTL)) [self.collectionView reloadData];
    }
        
    if (type != CONTINUOUS && type != PAGERTL) {
        if (type == PAGECOLOR || type == CROPPAGE) {
            self.currentShowItemType = type;
            [_extensionsManager.settingBar.viewSettingManager showSettingView:NO isReOpen:NO];
        }
        else{
            self.currentShowItemType = -1;
            [_extensionsManager.settingBar didClickItemWithType:type];
        }
    }
    
    if ((type == CONTINUOUS || type == PAGERTL) && UIAccessibilityIsVoiceOverRunning()) {
        BOOL ison = NO;
        if (type == CONTINUOUS) {
            ison = [info.continusON boolValue];
        }else{
            ison = [info.pageRTLON boolValue];
        }
        [self didClickSwitchBtn:!ison cellType:type];
    }
}

#pragma mark --FSSettingBarVerticalCellDelegate
- (void)didClickSwitchBtn:(BOOL)on cellType:(SettingItemType)cellType
{
    if (cellType == CONTINUOUS) {
        [_extensionsManager.settingBar.viewSettingManager setContinuousON:on];
    }else if(cellType == PAGERTL){
        [_extensionsManager.settingBar.viewSettingManager setPageRTLON:on];
    }
    [_extensionsManager.settingBar didClickItemWithType:cellType];
    if (cellType == CONTINUOUS) {
        !_extensionsManager.settingBar.continueItemStateCallback ? : _extensionsManager.settingBar.continueItemStateCallback(on);
    }else if(cellType == PAGERTL){
        !_extensionsManager.settingBar.pageRTLItemStateCallback ? : _extensionsManager.settingBar.pageRTLItemStateCallback(on);
    }
}
@end
