/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
*/


#import <Foundation/Foundation.h>
#import "FSPageColorView.h"

NS_ASSUME_NONNULL_BEGIN

@protocol IFSViewSettingEventListener <NSObject>
@optional
- (void)onShowPageColor:(BOOL)isShow;
- (void)onShowViewSettings:(BOOL)isShow;
@end

@interface FSViewSettingManager : NSObject
@property(nonatomic, assign) BOOL isDayOrNightForLast;
@property (nonatomic, strong) FSItem* singlePageItem;
@property (nonatomic, strong) FSItem* colorModeItem;
@property (nonatomic, assign) BOOL isShowSettingView;
@property (nonatomic, assign) BOOL isShowPageColorView;
@property (nonatomic, strong) FSPageColorView*  _Nullable pageColorView;

- (void)showSettingView:(BOOL)isShow isReOpen:(BOOL)isReOpen;
- (void)showPageColorView:(BOOL)isShow  isReOpen:(BOOL)isReOpen;
- (void)resetPageColorViewSize:(CGFloat)pdfViewWidth;

- (instancetype)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager;
- (void)setItemsInIPAD;

- (void)setIsNightMode:(BOOL)isNightMode;
- (void)setItemPageColor:(UIColor*)color;
- (void)setContinuousON:(BOOL)on;
- (void)setPageRTLON:(BOOL)on;
- (void)setItemTitleWithType:(SettingItemType)type title:(NSString*)title;
- (void)setItemHiddenWithType:(SettingItemType)type hidden:(BOOL)hidden;
- (void)setItemSelectedWithType:(SettingItemType)type selected:(BOOL)selected;
- (void)setItemEnabledWithType:(SettingItemType)type enabled:(BOOL)enabled;
- (void)setItemDefaultEnabledWithType:(SettingItemType)type enabled:(BOOL)enabled;
- (NSString*)getItemPageColor;
- (BOOL)getContinuousON;
- (BOOL)getPageRTLON;
- (NSString*)getItemTitleWithType:(SettingItemType)type;
- (BOOL)getItemHiddenWithType:(SettingItemType)type;
- (BOOL)getItemSelectedWithType:(SettingItemType)type;
- (BOOL)getItemEnabledWithType:(SettingItemType)type;
- (BOOL)getItemDefaultEnabledWithType:(SettingItemType)type;
- (void)registerEventListener:(id<IFSViewSettingEventListener>)listener;
- (void)unregisterEventListener:(id<IFSViewSettingEventListener>)listener;
- (void)moveBottombarWithRecognizer:(UIPanGestureRecognizer *)recognizer;
- (void)setPageColor:(UIColor*)color;
- (void)setPageColorWithNormalState;
- (void)resetPdfViewCtrlSafeArea;
@end

NS_ASSUME_NONNULL_END
