/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
*/

#import "FSViewSettingManager.h"
#import "FSSettingBar+private.h"
#import "ReflowModule.h"

#import "FSSettingContentView.h"

@interface FSViewSettingManager ()<IFSMainTopbarEventListener, IScrollViewEventListener, IPageEventListener>
@property (nonatomic, weak) FSPDFViewCtrl *pdfViewCtrl;
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
@property (nonatomic, strong) FSViewPopoverController* popController;
@property (nonatomic, strong) FSViewPopoverController* pageColorPopVC;
@property (nonatomic, strong) FSItem* dayModeItem;
@property (nonatomic, strong) FSItem* nightModeItem;
@property (nonatomic, strong) FSItem* fitPageItem;
@property (nonatomic, strong) FSItem* fitWidthItem;
@property (nonatomic, strong) FSItem* reflowItem;
@property (nonatomic, strong) FSItem* cropModeItem;
@property (nonatomic, strong) FSItem* speakItem;
@property (nonatomic, strong) FSItem* autoFlipItem;
@property (nonatomic, strong) FSItem* rotateItem;
@property (nonatomic, strong) FSItem* panZoomItem;
@property (nonatomic, strong) FSReadToolbarItem * singleItem;
@property (nonatomic, strong) FSReadToolbarItem * dayReadToolbarItem;
@property (nonatomic, strong) FSReadToolbarItem * colorReadToolbarItem;
@property (nonatomic, strong) FSReadToolbarItem * fitPageReadToolbarItem;
@property (nonatomic, strong) FSReadToolbarItem * fitWidthReadToolbarItem;
@property (nonatomic, strong) FSReadToolbarItem * reflowReadToolbarItem;
@property (nonatomic, strong) FSReadToolbarItem * cropModeReadToolbarItem;
@property (nonatomic, strong) FSReadToolbarItem * speakReadToolbarItem;
@property (nonatomic, strong) FSReadToolbarItem * autoFlipReadToolbarItem;
@property (nonatomic, strong) FSReadToolbarItem * rotateReadToolbarItem;
@property (nonatomic, strong) FSReadToolbarItem * panZoomReadToolbarItem;

@property (nonatomic, strong) NSArray* typeMutexFitArray;
@property (nonatomic, strong) NSArray* typeMutexReflowArray;

@property (nonatomic, strong) NSMutableArray<NSMutableArray*>* dataSource;
@property (nonatomic, strong) NSMutableDictionary* dataSourceWithHidden;
@property (nonatomic, assign) UIImage *backgroundImage;
@property (nonatomic, assign) UIImage *backColorImage;
@property (nonatomic, strong) NSMutableSet<id<IFSViewSettingEventListener>> *listeners;
@property(nonatomic, assign) CGPoint oldPoint;
@end

@implementation FSViewSettingManager

- (instancetype)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager {
    if (self = [super init]) {
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _extensionsManager = extensionsManager;
        self.dataSourceWithHidden = [[NSMutableDictionary alloc] init];
        self.dataSource = [NSMutableArray array];
        [self initSettingContentViewData];

        [self initItemsInIPAD];
        
        self.typeMutexFitArray = @[@(FITPAGE), @(FITWIDTH)];
        self.typeMutexReflowArray = @[@(REFLOW), @(CROPPAGE),@(SPEECH), @(AUTOFLIP), @(PANZOOM)];
        self.listeners = [NSMutableSet set];
        
        [_pdfViewCtrl registerScrollViewEventListener:self];
        [_pdfViewCtrl registerPageEventListener:self];
        [_extensionsManager.toolbarManager registerTopbarEventListener:self];
        self.isDayOrNightForLast = YES;
    }
    return self;
}

- (void)initSettingContentViewData
{
    self.dataSource = [NSMutableArray array];
    NSMutableArray* infoArrayHPage = [NSMutableArray array];
    NSMutableArray* infoArrayHColor = [NSMutableArray array];
    NSMutableArray* infoArrayVertical = [NSMutableArray array];
    for (SettingItemType itemType = SINGLE; itemType <= PANZOOM; itemType++) {
        FSSettingBarCellInfo *cellInfo = [[FSSettingBarCellInfo alloc] initWithItemType:itemType];
        if (cellInfo.sectionType == SectionTypeHorizontalPage ) {
            [infoArrayHPage addObject:cellInfo];
        }else if (cellInfo.sectionType == SectionTypeHorizontalColor) {
            [infoArrayHColor addObject:cellInfo];
        }else if (cellInfo.sectionType == SectionTypeVertical) {
            [infoArrayVertical addObject:cellInfo];
        }
    }
    if (infoArrayHPage.count > 0) {
        [self.dataSource addObject:infoArrayHPage];
    }
    if (infoArrayHColor.count > 0) {
        [self.dataSource addObject:infoArrayHColor];
    }
    if (infoArrayVertical.count > 0) {
        [self.dataSource addObject:infoArrayVertical];
    }
}
    
- (void)initItemsInIPAD
{
    CGFloat spacing = 4;
    UIImage *backgroundImage = [UIImage fs_imageWithColor:[UIColor clearColor]  size:CGSizeMake(40.f, 40.f)];
    UIImage *backColorImage = [[UIImage fs_imageWithColor:self.fs_backgroundColor_B1  size:CGSizeMake(32.f, 32.f)] fs_cornerRadius:3];
    
    if (_singlePageItem == nil) {
        _singlePageItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"setting_page_view")];
        _singlePageItem.titleEdgeInsets = UIEdgeInsetsMake(0, 4, 0, 8);
        _singlePageItem.font = self.fs_standardGeneralFont_GF3;
        _singlePageItem.titleEdgeInsets = UIEdgeInsetsMake(0, 4, 0, 8);
        [_singlePageItem setTitle:FSLocalizedForKey(@"kViewSettings") forState:UIControlStateNormal];
        [_singlePageItem setBackgroudViewColor:[UIColor clearColor] forState:UIControlStateNormal];
        [_singlePageItem setBackgroudViewColor:self.fs_dividerColor_P2 forState:UIControlStateSelected];
        [_singlePageItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
        [_singlePageItem setImageViewBackgroundImage:FSImageNamedAndNormalColor(@"common_unflod_backgroup") forState:UIControlStateNormal];
        [_singlePageItem rebuildImageTextRelation:FSImageTextRelationRight];
        
        _singlePageItem.accessibilityIdentifier = @"setting_page_view_id";
        FSMainToolbarItem* singleMainItem = [[FSMainToolbarItem alloc] initWithCustomView:_singlePageItem];
        singleMainItem.spacing = spacing;
        _singleItem = [[FSReadToolbarItem alloc] initWithCustomItem:singleMainItem];
        _singleItem.title = FSLocalizedForKey(@"kViewSettings");
        _singleItem.image = FSImageNamedAndNormalColor(@"setting_page_view");
        _singleItem.imageName = @"setting_page_view";
        _singleItem.itemContentType = FSItemContentTypeImageAndText;
        [_singleItem setValue:@(FSReadToolSettingsTypeViewSingle) forKey:@"settingsType"];
        
    }
    _singlePageItem.accessibilityLabel = FSLocalizedForKey(@"kViewSettings");
    @weakify(self)
    _singlePageItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [self showSettingView: YES  isReOpen:NO];
        [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingView];
    };
    
    BOOL  isSelectedNightMode = [self getItemSelectedWithType:NIGHTMODE];
    if (_dayModeItem == nil) {
        _dayModeItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(isSelectedNightMode?@"setting_night_ipad":@"setting_day_ipad")];
        _dayModeItem.accessibilityIdentifier = @"setting_night_day_id";
        [_dayModeItem setImage:FSImageNamedAndNormalColor(@"setting_night_ipad") forState:UIControlStateSelected];
        _dayModeItem.font = self.fs_standardGeneralFont_GF3;
        _dayModeItem.titleEdgeInsets = UIEdgeInsetsMake(0, 4, 0, 8);
        [_dayModeItem setTitle:FSLocalizedForKey(@"kDayModeVoiceOver") forState:UIControlStateNormal];
        [_dayModeItem setTitle:FSLocalizedForKey(@"kNightMode") forState:UIControlStateSelected];
        [_dayModeItem setBackgroudViewColor:[UIColor clearColor] forState:UIControlStateNormal];
        [_dayModeItem setBackgroudViewColor:self.fs_dividerColor_P2 forState:UIControlStateSelected];
        [_dayModeItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
        [_dayModeItem rebuildImageTextRelation:FSImageTextRelationRight];
        
        FSMainToolbarItem* dayMainItem = [[FSMainToolbarItem alloc] initWithCustomView:_dayModeItem];
        dayMainItem.spacing = spacing;
        _dayReadToolbarItem = [[FSReadToolbarItem alloc] initWithCustomItem:dayMainItem];

        _dayReadToolbarItem.title = FSLocalizedForKey(@"kDayModeVoiceOver");
        _dayReadToolbarItem.selectTitle = FSLocalizedForKey(@"kNightMode");
        _dayReadToolbarItem.image = FSImageNamedAndNormalColor(isSelectedNightMode?@"setting_night_ipad":@"setting_day_ipad");
        _dayReadToolbarItem.imageName = isSelectedNightMode?@"setting_night_ipad":@"setting_day_ipad";
        _dayReadToolbarItem.itemContentType = FSItemContentTypeImageAndText;
        [_dayReadToolbarItem setValue:@(FSReadToolSettingsTypeViewDay) forKey:@"settingsType"];
        
    }
    _dayModeItem.accessibilityLabel = isSelectedNightMode ? FSLocalizedForKey(@"kNightMode") : FSLocalizedForKey(@"kDayModeVoiceOver");
    _dayModeItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        _item.selected = !_item.isSelected;
        self->_dayReadToolbarItem.image = FSImageNamedAndNormalColor(_item.isSelected?@"setting_night_ipad":@"setting_day_ipad");
        self->_dayReadToolbarItem.imageName = _item.isSelected?@"setting_night_ipad":@"setting_day_ipad";
        [self didClickedItemWithType:_item.isSelected?NIGHTMODE:DAYMODE];
        _item.accessibilityLabel = _item.isSelected ? FSLocalizedForKey(@"kNightMode") : FSLocalizedForKey(@"kDayModeVoiceOver");
    };
    
    if (_colorModeItem == nil) {
        _colorModeItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"setting_pagecolor_ipad")];
        _colorModeItem.accessibilityIdentifier = @"setting_night_day_id";
        _colorModeItem.font = self.fs_standardGeneralFont_GF3;
        _colorModeItem.titleEdgeInsets = UIEdgeInsetsMake(0, 4, 0, 8);
        [_colorModeItem setTitle:FSLocalizedForKey(@"kPageColor") forState:UIControlStateNormal];
        [_colorModeItem setBackgroudViewColor:[UIColor clearColor] forState:UIControlStateNormal];
        [_colorModeItem setBackgroudViewColor:self.fs_dividerColor_P2 forState:UIControlStateSelected];
        [_colorModeItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
        [_colorModeItem setImageViewBackgroundImage:FSImageNamedAndNormalColor(@"common_unflod_backgroup") forState:UIControlStateNormal];
        [_colorModeItem rebuildImageTextRelation:FSImageTextRelationRight];
        
        FSMainToolbarItem* colorMainItem = [[FSMainToolbarItem alloc] initWithCustomView:_colorModeItem];
        colorMainItem.spacing = spacing;
        _colorReadToolbarItem = [[FSReadToolbarItem alloc] initWithCustomItem:colorMainItem];
        _colorReadToolbarItem.title = FSLocalizedForKey(@"kPageColor");
        _colorReadToolbarItem.image = FSImageNamedAndNormalColor(@"setting_pagecolor_ipad");
        _colorReadToolbarItem.imageName = @"setting_pagecolor_ipad";
        _colorReadToolbarItem.itemContentType = FSItemContentTypeImageAndText;
        [_colorReadToolbarItem setValue:@(FSReadToolSettingsTypeViewColor) forKey:@"settingsType"];
    }
    _colorModeItem.accessibilityLabel = FSLocalizedForKey(@"kPageColor");
    _colorModeItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [self didClickedItemWithType:PAGECOLOR];
        
        [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(delaySuspend) object:nil];
        [self performSelector:@selector(delaySuspend) withObject:nil afterDelay:0.2f];
        [self showPageColorView:YES isReOpen:NO];
    };
    
    if (_fitPageItem == nil) {
        _fitPageItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"setting_fit_page")];
        _fitPageItem.accessibilityIdentifier = @"setting_fit_page_id";
        _fitPageItem.font = self.fs_standardGeneralFont_GF3;
        _fitPageItem.titleEdgeInsets = UIEdgeInsetsMake(0, 4, 0, 8);
        [_fitPageItem setTitle:FSLocalizedForKey(@"kFitPage") forState:UIControlStateNormal];
        [_fitPageItem setBackgroudViewColor:[UIColor clearColor] forState:UIControlStateNormal];
        [_fitPageItem setBackgroudViewColor:self.fs_dividerColor_P2 forState:UIControlStateSelected];
        [_fitPageItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
        [_fitPageItem rebuildImageTextRelation:FSImageTextRelationRight];
        
        FSMainToolbarItem* fitPageMainItem = [[FSMainToolbarItem alloc] initWithCustomView:_fitPageItem];
        fitPageMainItem.spacing = spacing;
        _fitPageReadToolbarItem = [[FSReadToolbarItem alloc] initWithCustomItem:fitPageMainItem];
        _fitPageReadToolbarItem.title = FSLocalizedForKey(@"kFitPage");
        _fitPageReadToolbarItem.image = FSImageNamedAndNormalColor(@"setting_fit_page");
        _fitPageReadToolbarItem.imageName = @"setting_fit_page";
        _fitPageReadToolbarItem.itemContentType = FSItemContentTypeImageAndText;
        [_fitPageReadToolbarItem setValue:@(FSReadToolSettingsTypeViewFitPage) forKey:@"settingsType"];
    }
    _fitPageItem.accessibilityLabel = FSLocalizedForKey(@"kFitPage");
    _fitPageItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        _item.selected = YES;

        [self didClickedItemWithType:FITPAGE];
    };
    
    if (_fitWidthItem == nil) {
       
        _fitWidthItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"setting_fit_width")];
        _fitWidthItem.accessibilityIdentifier = @"setting_fit_width_id";
        _fitWidthItem.font = self.fs_standardGeneralFont_GF3;
        _fitWidthItem.titleEdgeInsets = UIEdgeInsetsMake(0, 4, 0, 8);
        [_fitWidthItem setTitle:FSLocalizedForKey(@"kFitWidth") forState:UIControlStateNormal];
        [_fitWidthItem setBackgroudViewColor:[UIColor clearColor] forState:UIControlStateNormal];
        [_fitWidthItem setBackgroudViewColor:self.fs_dividerColor_P2 forState:UIControlStateSelected];
        [_fitWidthItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
        [_fitWidthItem rebuildImageTextRelation:FSImageTextRelationRight];
        
        FSMainToolbarItem* fitWidthMainItem = [[FSMainToolbarItem alloc] initWithCustomView:_fitWidthItem];
        fitWidthMainItem.spacing = spacing;
        _fitWidthReadToolbarItem = [[FSReadToolbarItem alloc] initWithCustomItem:fitWidthMainItem];
        _fitWidthReadToolbarItem.title = FSLocalizedForKey(@"kFitWidth");
        _fitWidthReadToolbarItem.image = FSImageNamedAndNormalColor(@"setting_fit_width");
        _fitWidthReadToolbarItem.imageName = @"setting_fit_width";
        _fitWidthReadToolbarItem.itemContentType = FSItemContentTypeImageAndText;
        [_fitWidthReadToolbarItem setValue:@(FSReadToolSettingsTypeViewFitWidth) forKey:@"settingsType"];
    }
    _fitWidthItem.accessibilityLabel = FSLocalizedForKey(@"kFitWidth");
    _fitWidthItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            _item.selected = YES;
            [self didClickedItemWithType:FITWIDTH];
    };
        
    if (_reflowItem == nil) {
        _reflowItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"setting_reflow")];
        _reflowItem.accessibilityIdentifier = @"setting_reflow_id";
        _reflowItem.font = self.fs_standardGeneralFont_GF3;
        _reflowItem.titleEdgeInsets = UIEdgeInsetsMake(0, 4, 0, 8);
        [_reflowItem setTitle:FSLocalizedForKey(@"kReflow") forState:UIControlStateNormal];
        [_reflowItem setBackgroudViewColor:[UIColor clearColor] forState:UIControlStateNormal];
        [_reflowItem setBackgroudViewColor:self.fs_dividerColor_P2 forState:UIControlStateSelected];
        [_reflowItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
        [_reflowItem rebuildImageTextRelation:FSImageTextRelationRight];
        
        FSMainToolbarItem* reflowMainItem = [[FSMainToolbarItem alloc] initWithCustomView:_reflowItem];
        reflowMainItem.spacing = spacing;
        _reflowReadToolbarItem = [[FSReadToolbarItem alloc] initWithCustomItem:reflowMainItem];
        _reflowReadToolbarItem.title = FSLocalizedForKey(@"kReflow");
        _reflowReadToolbarItem.image = FSImageNamedAndNormalColor(@"setting_reflow");
        _reflowReadToolbarItem.imageName = @"setting_reflow";
        _reflowReadToolbarItem.itemContentType = FSItemContentTypeImageAndText;
        [_reflowReadToolbarItem setValue:@(FSReadToolSettingsTypeViewReflow) forKey:@"settingsType"];
     
    }
    _reflowItem.accessibilityLabel = FSLocalizedForKey(@"kReflow");
    _reflowItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        _item.selected = !_item.isSelected;
        if (_item.isSelected) {
            [self didClickedItemWithType:REFLOW];
        }
        else{
            [self setItemSelectedWithType:REFLOW selected:NO];
            [self.extensionsManager changeState:FSUIManagerStateNormal];
        }
    };
    
    if (_cropModeItem == nil) {
        _cropModeItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"setting_crop_mode")];
        _cropModeItem.accessibilityIdentifier = @"setting_crop_mode_id";
        _cropModeItem.font = self.fs_standardGeneralFont_GF3;
        _cropModeItem.titleEdgeInsets = UIEdgeInsetsMake(0, 4, 0, 8);
        [_cropModeItem setTitle:FSLocalizedForKey(@"kCropMode") forState:UIControlStateNormal];
        [_cropModeItem setBackgroudViewColor:[UIColor clearColor] forState:UIControlStateNormal];
        [_cropModeItem setBackgroudViewColor:self.fs_dividerColor_P2 forState:UIControlStateSelected];
        [_cropModeItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
        [_cropModeItem rebuildImageTextRelation:FSImageTextRelationRight];
        
        FSMainToolbarItem* cropModeMainItem = [[FSMainToolbarItem alloc] initWithCustomView:_cropModeItem];
        cropModeMainItem.spacing = spacing;
        _cropModeReadToolbarItem = [[FSReadToolbarItem alloc] initWithCustomItem:cropModeMainItem];
        _cropModeReadToolbarItem.title = FSLocalizedForKey(@"kCropMode");
        _cropModeReadToolbarItem.image = FSImageNamedAndNormalColor(@"setting_crop_mode");
        _cropModeReadToolbarItem.imageName = @"setting_crop_mode";
        _cropModeReadToolbarItem.itemContentType = FSItemContentTypeImageAndText;
     
        [_cropModeReadToolbarItem setValue:@(FSReadToolSettingsTypeViewCrop) forKey:@"settingsType"];
        
        
    }
    _cropModeItem.accessibilityLabel = FSLocalizedForKey(@"kCropMode");
    _cropModeItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [self.extensionsManager changeState:FSUIManagerStateNormal];
        [self didClickedItemWithType:CROPPAGE];
    };
    
    if (_speakItem == nil) {
        _speakItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"setting_speak")];
        _speakItem.accessibilityIdentifier = @"setting_speak_id";
        _speakItem.font = self.fs_standardGeneralFont_GF3;
        _speakItem.titleEdgeInsets = UIEdgeInsetsMake(0, 4, 0, 8);
        [_speakItem setTitle:FSLocalizedForKey(@"kSpeak") forState:UIControlStateNormal];
        [_speakItem setBackgroudViewColor:[UIColor clearColor] forState:UIControlStateNormal];
        [_speakItem setBackgroudViewColor:self.fs_dividerColor_P2 forState:UIControlStateSelected];
        [_speakItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
        [_speakItem rebuildImageTextRelation:FSImageTextRelationRight];
        
        FSMainToolbarItem* speakItem = [[FSMainToolbarItem alloc] initWithCustomView:_speakItem];
        speakItem.spacing = spacing;
        _speakReadToolbarItem = [[FSReadToolbarItem alloc] initWithCustomItem:speakItem];
        _speakReadToolbarItem.title = FSLocalizedForKey(@"kSpeak");
        _speakReadToolbarItem.image = FSImageNamedAndNormalColor(@"setting_speak");
        _speakReadToolbarItem.imageName = @"setting_speak";
        _speakReadToolbarItem.itemContentType = FSItemContentTypeImageAndText;
        [_speakReadToolbarItem setValue:@(FSReadToolSettingsTypeViewSpeak) forKey:@"settingsType"];
    }
    _speakItem.accessibilityLabel = FSLocalizedForKey(@"kSpeak");
    _speakItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        _item.selected = !_item.isSelected;
        if (_item.isSelected) {
            [self didClickedItemWithType:SPEECH];
        }
        else
        {
            [self setItemSelectedWithType:SPEECH selected:NO];
            [self.extensionsManager changeState:FSUIManagerStateNormal];
        }
    };
    
    if (_autoFlipItem == nil) {
        _autoFlipItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"setting_auto_flip")];
        _autoFlipItem.accessibilityIdentifier = @"setting_auto_flip_id";
        _autoFlipItem.font = self.fs_standardGeneralFont_GF3;
        _autoFlipItem.titleEdgeInsets = UIEdgeInsetsMake(0, 4, 0, 8);
        [_autoFlipItem setTitle:FSLocalizedForKey(@"kAutoFlip") forState:UIControlStateNormal];
        [_autoFlipItem setBackgroudViewColor:[UIColor clearColor] forState:UIControlStateNormal];
        [_autoFlipItem setBackgroudViewColor:self.fs_dividerColor_P2 forState:UIControlStateSelected];
        [_autoFlipItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
        [_autoFlipItem rebuildImageTextRelation:FSImageTextRelationRight];
        
        FSMainToolbarItem* autoFlipItem = [[FSMainToolbarItem alloc] initWithCustomView:_autoFlipItem];
        autoFlipItem.spacing = spacing;
        _autoFlipReadToolbarItem = [[FSReadToolbarItem alloc] initWithCustomItem:autoFlipItem];
        _autoFlipReadToolbarItem.title = FSLocalizedForKey(@"kAutoFlip");
        _autoFlipReadToolbarItem.image = FSImageNamedAndNormalColor(@"setting_auto_flip");
        _autoFlipReadToolbarItem.imageName = @"setting_auto_flip";
        _autoFlipReadToolbarItem.itemContentType = FSItemContentTypeImageAndText;
        [_autoFlipReadToolbarItem setValue:@(FSReadToolSettingsTypeViewAutoFlip) forKey:@"settingsType"];
    }
    _autoFlipItem.accessibilityLabel = FSLocalizedForKey(@"kAutoFlip");
    _autoFlipItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        _item.selected = !_item.isSelected;
        if (_item.isSelected) {
            [self.extensionsManager changeState:FSUIManagerStateNormal];
            [self didClickedItemWithType:AUTOFLIP];
        }
        else
        {
            [self setItemSelectedWithType:AUTOFLIP selected:NO];
            [self.extensionsManager changeState:FSUIManagerStateNormal];
        }
    };
    
    if (_rotateItem == nil) {
        _rotateItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"setting_rotate_view")];
        _rotateItem.accessibilityIdentifier = @"setting_rotate_view_id";
        _rotateItem.font = self.fs_standardGeneralFont_GF3;
        _rotateItem.titleEdgeInsets = UIEdgeInsetsMake(0, 4, 0, 8);
        [_rotateItem setTitle:FSLocalizedForKey(@"kRotateView") forState:UIControlStateNormal];
        [_rotateItem setBackgroudViewColor:[UIColor clearColor] forState:UIControlStateNormal];
        [_rotateItem setBackgroudViewColor:self.fs_dividerColor_P2 forState:UIControlStateSelected];
        [_rotateItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
        [_rotateItem rebuildImageTextRelation:FSImageTextRelationRight];
        
        FSMainToolbarItem* rotateItem = [[FSMainToolbarItem alloc] initWithCustomView:_rotateItem];
        rotateItem.spacing = spacing;
        _rotateReadToolbarItem = [[FSReadToolbarItem alloc] initWithCustomItem:rotateItem];
        _rotateReadToolbarItem.title = FSLocalizedForKey(@"kRotateView");
        _rotateReadToolbarItem.image = FSImageNamedAndNormalColor(@"setting_rotate_view");
        _rotateReadToolbarItem.imageName = @"setting_rotate_view";
        _rotateReadToolbarItem.itemContentType = FSItemContentTypeImageAndText;
        [_rotateReadToolbarItem setValue:@(FSReadToolSettingsTypeViewRotate) forKey:@"settingsType"];
    }
    _rotateItem.accessibilityLabel = FSLocalizedForKey(@"kRotateView");
    _rotateItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        if ([self.extensionsManager getState] == FSUIManagerStateAutoFlip || [self.extensionsManager getState] == FSUIManagerStateSpeech) {
            [self.extensionsManager changeState:FSUIManagerStateNormal];
        }
        
        [self didClickedItemWithType:ROTATE];
    };
    
    if (_panZoomItem == nil) {
        _panZoomItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"setting_pan_zoom")];
        _panZoomItem.accessibilityIdentifier = @"setting_pan_zoom_id";
        _panZoomItem.font = self.fs_standardGeneralFont_GF3;
        _panZoomItem.titleEdgeInsets = UIEdgeInsetsMake(0, 4, 0, 8);
        [_panZoomItem setTitle:FSLocalizedForKey(@"kPanandZoom") forState:UIControlStateNormal];
        [_panZoomItem setBackgroudViewColor:[UIColor clearColor] forState:UIControlStateNormal];
        [_panZoomItem setBackgroudViewColor:self.fs_dividerColor_P2 forState:UIControlStateSelected];
        [_panZoomItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
        [_panZoomItem rebuildImageTextRelation:FSImageTextRelationRight];
        
        FSMainToolbarItem* panZoomItem = [[FSMainToolbarItem alloc] initWithCustomView:_panZoomItem];
        panZoomItem.spacing = spacing;
        _panZoomReadToolbarItem = [[FSReadToolbarItem alloc] initWithCustomItem:panZoomItem];
        _panZoomReadToolbarItem.title = FSLocalizedForKey(@"kPanandZoom");
        _panZoomReadToolbarItem.image = FSImageNamedAndNormalColor(@"setting_pan_zoom");
        _panZoomReadToolbarItem.imageName = @"setting_pan_zoom";
        _panZoomReadToolbarItem.itemContentType = FSItemContentTypeImageAndText;
        [_panZoomReadToolbarItem setValue:@(FSReadToolSettingsTypeViewPanZoom) forKey:@"settingsType"];
    }
    _panZoomItem.accessibilityLabel = FSLocalizedForKey(@"kPanandZoom");
    _panZoomItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        _item.selected = !_item.isSelected;
        if (_item.isSelected) {
            [self didClickedItemWithType:PANZOOM];
        }
        else
        {
            [self setItemSelectedWithType:PANZOOM selected:NO];
            [self.extensionsManager changeState:FSUIManagerStateNormal];
        }
    };
}

- (void)setItemsInIPAD
{
    if ([self getItemInfoWithType:SINGLE] || [self getItemInfoWithType:DOUBLEPAGE] || [self getItemInfoWithType:COVERPAGE] || [self getItemInfoWithType:CONTINUOUS] || [self getItemInfoWithType:PAGERTL]) {
        [self.extensionsManager.toolbarManager addReadItem:_singleItem atToolTag:FSMainTopbarToolTagView];
    }else{
        [self.extensionsManager.toolbarManager removeReadItem:_singleItem atToolTag:FSMainTopbarToolTagView];
    }
   
    if ([self getItemInfoWithType:DAYMODE] && [self getItemInfoWithType:NIGHTMODE]) {
        [_dayModeItem setSelected:[self getItemSelectedWithType:NIGHTMODE]];
        [self.extensionsManager.toolbarManager addReadItem:_dayReadToolbarItem atToolTag:FSMainTopbarToolTagView];
    }else{
        [self.extensionsManager.toolbarManager removeReadItem:_dayReadToolbarItem atToolTag:FSMainTopbarToolTagView];
    }
    
    if ([self getItemInfoWithType:PAGECOLOR]) {
        [_colorModeItem setEnabled:[self getItemEnabledWithType:PAGECOLOR]];
        [self.extensionsManager.toolbarManager addReadItem:_colorReadToolbarItem atToolTag:FSMainTopbarToolTagView];
    }else{
        [self.extensionsManager.toolbarManager removeReadItem:_colorReadToolbarItem atToolTag:FSMainTopbarToolTagView];
    }
    
    if ([self getItemInfoWithType:FITPAGE]) {
        [_fitPageItem setEnabled:[self getItemEnabledWithType:FITPAGE]];
        [_fitPageItem setSelected:[self getItemSelectedWithType:FITPAGE]];
        [self.extensionsManager.toolbarManager addReadItem:_fitPageReadToolbarItem atToolTag:FSMainTopbarToolTagView];
    }else{
        [self.extensionsManager.toolbarManager removeReadItem:_fitPageReadToolbarItem atToolTag:FSMainTopbarToolTagView];
    }
    
    if ([self getItemInfoWithType:FITWIDTH]) {
        [_fitWidthItem setEnabled:[self getItemEnabledWithType:FITWIDTH]];
        [_fitWidthItem setSelected:[self getItemSelectedWithType:FITWIDTH]];
        [self.extensionsManager.toolbarManager addReadItem:_fitWidthReadToolbarItem atToolTag:FSMainTopbarToolTagView];
    }else{
        [self.extensionsManager.toolbarManager removeReadItem:_fitWidthReadToolbarItem atToolTag:FSMainTopbarToolTagView];
    }

    if ([self getItemInfoWithType:REFLOW]) {
        [_reflowItem setEnabled:[self getItemEnabledWithType:REFLOW]];
        [_reflowItem setSelected:[self getItemSelectedWithType:REFLOW]];
        [self.extensionsManager.toolbarManager addReadItem:_reflowReadToolbarItem atToolTag:FSMainTopbarToolTagView];
    }else{
        [self.extensionsManager.toolbarManager removeReadItem:_reflowReadToolbarItem atToolTag:FSMainTopbarToolTagView];
    }
    
    if ([self getItemInfoWithType:CROPPAGE]) {
        [_cropModeItem setEnabled:[self getItemEnabledWithType:CROPPAGE]];
        [_cropModeItem setSelected:[self getItemSelectedWithType:CROPPAGE]];
        [self.extensionsManager.toolbarManager addReadItem:_cropModeReadToolbarItem atToolTag:FSMainTopbarToolTagView];
    }else{
        [self.extensionsManager.toolbarManager removeReadItem:_cropModeReadToolbarItem atToolTag:FSMainTopbarToolTagView];
    }
    
    if ([self getItemInfoWithType:SPEECH]) {
        [_speakItem setEnabled:[self getItemEnabledWithType:SPEECH]];
        [_speakItem setSelected:[self getItemSelectedWithType:SPEECH]];
        [self.extensionsManager.toolbarManager addReadItem:_speakReadToolbarItem atToolTag:FSMainTopbarToolTagView];
    }else{
        [self.extensionsManager.toolbarManager removeReadItem:_speakReadToolbarItem atToolTag:FSMainTopbarToolTagView];
    }
    
    if ([self getItemInfoWithType:AUTOFLIP]) {
        [_autoFlipItem setEnabled:[self getItemEnabledWithType:AUTOFLIP]];
        [_autoFlipItem setSelected:[self getItemSelectedWithType:AUTOFLIP]];
        [self.extensionsManager.toolbarManager addReadItem:_autoFlipReadToolbarItem atToolTag:FSMainTopbarToolTagView];
    }else{
        [self.extensionsManager.toolbarManager removeReadItem:_autoFlipReadToolbarItem atToolTag:FSMainTopbarToolTagView];
    }
    
    if ([self getItemInfoWithType:ROTATE]) {
        [_rotateItem setEnabled:[self getItemEnabledWithType:ROTATE]];
        [self.extensionsManager.toolbarManager addReadItem:_rotateReadToolbarItem atToolTag:FSMainTopbarToolTagView];
    }else{
        [self.extensionsManager.toolbarManager removeReadItem:_rotateReadToolbarItem atToolTag:FSMainTopbarToolTagView];
    }
    
    if ([self getItemInfoWithType:PANZOOM]) {
        [_panZoomItem setEnabled:[self getItemEnabledWithType:PANZOOM]];
        [_panZoomItem setSelected:[self getItemSelectedWithType:PANZOOM]];
        [self.extensionsManager.toolbarManager addReadItem:_panZoomReadToolbarItem atToolTag:FSMainTopbarToolTagView];
    }else{
        [self.extensionsManager.toolbarManager removeReadItem:_panZoomReadToolbarItem atToolTag:FSMainTopbarToolTagView];
    }
    
    [_extensionsManager.toolbarManager resetViewReadItems];
}

//IPAD item click
- (void)didClickedItemWithType:(SettingItemType)itemType
{
    if (itemType != CROPPAGE && (!(itemType == PAGECOLOR && self.extensionsManager.pdfViewCtrl.isNightMode))) {
        [self setItemSelectedWithType:itemType selected:YES];
        if (itemType == PAGECOLOR) {
            [self setItemSelectedWithType:DAYMODE selected:NO];
        }
    }
    self.extensionsManager.settingBar.settingView.currentShowItemType = -1;
    [self.extensionsManager.settingBar didClickItemWithType:itemType];
}

- (NSMutableArray<NSMutableArray*>*)getDataForShowCollectionView
{
    NSMutableArray<NSMutableArray*>* viewData = [NSMutableArray array];
    NSMutableArray* pageArray = [self getItemArrayWithSectionType:SectionTypeHorizontalPage];
    for (FSSettingBarCellInfo *info in pageArray) {
        if (info.itemType == DOUBLEPAGE || info.itemType == COVERPAGE) {
            info.singlePage = [self.pdfViewCtrl getPageCount] <= 1;
        }
    }
    if (pageArray && pageArray.count > 0) {
        [viewData addObject:pageArray];
    }
    
    NSMutableArray *verticalArray = [NSMutableArray array];
    FSSettingBarCellInfo *continuInfo = [self getItemInfoWithType:CONTINUOUS];
    if (continuInfo) {
        if([self.pdfViewCtrl getPageCount] <= 1) {
            continuInfo.continusON = @(NO);
            continuInfo.enabled = NO;
        }else{
            continuInfo.enabled = YES;
        }
        [verticalArray addObject:continuInfo];
    }
    
    FSSettingBarCellInfo *pageRTLInfo = [self getItemInfoWithType:PAGERTL];
    if (pageRTLInfo) {
        if([self.pdfViewCtrl getPageCount] <= 1) {
            pageRTLInfo.pageRTLON = @(NO);
            pageRTLInfo.enabled = NO;
        }else{
            pageRTLInfo.enabled = YES;
        }
        [verticalArray addObject:pageRTLInfo];
    }
    if (verticalArray.count)  [viewData addObject:verticalArray];
   
    if (![self checkIsIphoneDevice]) {
        [self sortArrayBySectionType:viewData];
    }else{
        [self sortArrayBySectionType:self.dataSource];
    }
   
    return ![self checkIsIphoneDevice] ? viewData : self.dataSource;
}

- (void)delaySuspend
{
    [_extensionsManager cancelAutoFullScreen];
}

- (void)showSettingView:(BOOL)isShow  isReOpen:(BOOL)isReOpen
{
    if (_isShowSettingView == isShow) {
        return;
    }
    @weakify(self)
    _isShowSettingView = isShow;
    if (_isShowSettingView) {
        if (![self checkIsIphoneDevice])
        {
            [self.extensionsManager changeState:FSUIManagerStateNormal];
        }
        [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(delaySuspend) object:nil];
        [self performSelector:@selector(delaySuspend) withObject:nil afterDelay:1.f];
        
        [_extensionsManager.settingBar initContentView];

        if ([_pdfViewCtrl isDynamicXFA]) {
            [self setItemEnabledWithType:REFLOW enabled:NO];
            [self setItemEnabledWithType:PANZOOM enabled:NO];
            [self setItemEnabledWithType:CROPPAGE enabled:NO];
        }
        
        if ([self.pdfViewCtrl getPageCount] <= 1 &&
            (![self getItemSelectedWithType:SINGLE] || [self getContinuousON])) {
            [self dealWhenOnlyOnePage];
        }
        [_extensionsManager.settingBar.settingView reloadWithData:[self getDataForShowCollectionView]];
        [_extensionsManager.settingBar updateContentViewLayoutHeight];
        
        _popController = [_extensionsManager.viewCtrlManager popPopoverCustomView:_extensionsManager.settingBar.contentView dismissCompletion:^{
            @strongify(self)
            self.isShowSettingView = NO;
            [self.extensionsManager delayAutoFullScreen];
            self.extensionsManager.settingBar.contentView = nil;
            self.extensionsManager.settingBar.settingView = nil;
            
        } sourceView:[self checkIsIphoneDevice] ? nil :^UIView * _Nonnull(UIPopoverPresentationController * _Nonnull popover) {
            @strongify(self)
          
            if (self.extensionsManager.topToolbar.subitemContentPosition == FSTopbarSubitemContentViewPositionLeft) {
                popover.permittedArrowDirections = UIPopoverArrowDirectionLeft;
            } else  if (self.extensionsManager.topToolbar.subitemContentPosition == FSTopbarSubitemContentViewPositionRight) {
                popover.permittedArrowDirections = UIPopoverArrowDirectionRight;
            }else{
                popover.permittedArrowDirections = UIPopoverArrowDirectionUp;
            }
            
            return self.singlePageItem;
        }];
    }
    else
    {
        if (_popController) {
            [_popController dismissViewControllerAnimated:YES completion:^{
                @strongify(self)
                self.isShowSettingView = NO;
                [self.extensionsManager delayAutoFullScreen];
                if (self.extensionsManager.settingBar.settingView.currentShowItemType == PAGECOLOR || self.extensionsManager.settingBar.settingView.currentShowItemType == CROPPAGE) {
                    
                    [self.extensionsManager.settingBar didClickItemWithType:self.extensionsManager.settingBar.settingView.currentShowItemType];
                    self.extensionsManager.settingBar.settingView.currentShowItemType = -1;
                }
                self.extensionsManager.settingBar.contentView = nil;
                self.extensionsManager.settingBar.settingView = nil;
                if (isReOpen) {
                    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                        [self showSettingView:YES isReOpen:NO];
                    });
                }
            }];
        }
    }
    for (id<IFSViewSettingEventListener> listener in self.listeners) {
        if ([listener respondsToSelector:@selector(onShowViewSettings:)]) {
            [listener onShowViewSettings:isShow];
        }
    }
}

- (void)showPageColorView:(BOOL)isShow  isReOpen:(BOOL)isReOpen
{
    if (_isShowPageColorView == isShow) {
        return;
    }

    _isShowPageColorView = isShow;
    @fs_weakify(self)
    if (_isShowPageColorView) {
        FSPageColorView* pageColorView = [[FSPageColorView alloc] initWithExtensionsManager:_extensionsManager frame:CGRectMake(0, 0, [self checkIsIphoneDevice] ? MIN( _pdfViewCtrl.fs_width,  _pdfViewCtrl.fs_height) : 375, 100)];
        _pageColorView = pageColorView;
 
        _pageColorPopVC = [_extensionsManager.viewCtrlManager popPopoverCustomView:pageColorView dismissCompletion:^{
            @fs_strongify(self)
            self.pageColorView = nil;
            self.isShowPageColorView = NO;
            [self.extensionsManager delayAutoFullScreen];
        } sourceView:![self checkIsIphoneDevice] ? ^UIView * _Nonnull(UIPopoverPresentationController * _Nonnull popover) {
            @fs_strongify(self)
            if (![self checkIsIphoneDevice]) {
                if (self.extensionsManager.topToolbar.subitemContentPosition == FSTopbarSubitemContentViewPositionLeft) {
                    popover.permittedArrowDirections = UIPopoverArrowDirectionLeft;
                } else  if (self.extensionsManager.topToolbar.subitemContentPosition == FSTopbarSubitemContentViewPositionRight) {
                    popover.permittedArrowDirections = UIPopoverArrowDirectionRight;
                }else{
                    popover.permittedArrowDirections = UIPopoverArrowDirectionUp;
                }
            
                return self.colorModeItem;
            }
            popover.permittedArrowDirections = UIPopoverArrowDirectionUp;
            return nil;
        } : nil];
        _pageColorPopVC.autoresizesFollowContnetView = YES;
    
    }else {
        if (_pageColorPopVC) {
            [_pageColorPopVC dismissViewControllerAnimated:YES completion:^{
                @fs_strongify(self)
                self.pageColorView = nil;
                self.isShowPageColorView = NO;
                
                if (isReOpen) {
                    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                        [self.extensionsManager cancelAutoFullScreen];
                        [self showPageColorView:YES isReOpen:NO];
                    });
                }
            }];
        }
    }
}

- (void)resetPageColorViewSize:(CGFloat)pdfViewWidth
{
    if (_pageColorView)
    {
        _pageColorView.frame = CGRectMake(0, 0, [self checkIsIphoneDevice] ? pdfViewWidth : 375, 100);
        [_pageColorView refreshView];
    }
}

#pragma mark -- setter getter

- (void)sortArrayByCellType:(NSMutableArray*)array
{
    [array sortUsingComparator:^NSComparisonResult(FSSettingBarCellInfo * _Nonnull info1, FSSettingBarCellInfo * _Nonnull info2) {
        if (info1.itemType < info2.itemType)
        {
            return (NSComparisonResult)NSOrderedAscending;
        }
        else
        {
            return (NSComparisonResult)NSOrderedDescending;
        }
        return (NSComparisonResult)NSOrderedSame;
    }];
}

- (void)sortArrayBySectionType:(NSMutableArray*)array
{
    [array sortUsingComparator:^NSComparisonResult(id  _Nonnull obj1, id  _Nonnull obj2) {
        NSMutableArray *array1 = (NSMutableArray *)obj1;
        NSMutableArray *array2 = (NSMutableArray *)obj2;
        FSSettingBarCellInfo *info1 = array1[0];
        FSSettingBarCellInfo *info2 = array2[0];
        if (info1.sectionType < info2.sectionType)
        {
            return (NSComparisonResult)NSOrderedAscending;
        }
        else
        {
            return (NSComparisonResult)NSOrderedDescending;
        }
        return (NSComparisonResult)NSOrderedSame;
    }];
}

- (void)setItemValueKeyWithType:(SettingItemType)type value:(id)value key:(NSString*)key
{
    int j = 0;
    for (NSMutableArray* array in self.dataSource) {
        int i = 0;
        BOOL isReplace = NO;
        for (FSSettingBarCellInfo *info in array) {
            if (info.itemType == type) {
                [info setValue:value forKey:key];
                [array replaceObjectAtIndex:i withObject:info];
                isReplace = YES;
                break;
            }
            i ++;
        }
        if (isReplace) {
            break;
        }
        j ++;
    }
}

- (FSSettingBarCellInfo *)removeItemWithType:(SettingItemType)type
{
    int j = 0;
    FSSettingBarCellInfo *removeInfo = nil;
    for (NSMutableArray* array in self.dataSource) {
        int i = 0;
        for (FSSettingBarCellInfo *info in array) {
            if (info.itemType == type) {
                removeInfo = info;
                [array removeObject:info];
                break;
            }
            i ++;
        }
        if (array.count == 0) {
            [self.dataSource removeObject:array];
        }
        if (removeInfo) {
            break;
        }
        j ++;
    }
    return removeInfo;
}

- (void)addItemWithType:(SettingItemType)itemType
{
    FSSettingBarCellInfo *info = [self getItemInfoWithType:itemType];
    if (info != nil) {
        return;
    }
    
    FSSettingBarCellInfo *willShowInfo = [_dataSourceWithHidden valueForKey:[NSString stringWithFormat:@"%lu", itemType]];
    if (willShowInfo) {
        FSSettingBarCellSectionType sectionType = willShowInfo.sectionType;
        NSMutableArray* array = [self getItemArrayWithSectionType:sectionType];
        if (array == nil) {
            array = [NSMutableArray array];
            [self.dataSource addObject:array];
        }
        [array addObject:willShowInfo];
        [self sortArrayByCellType:array];
    }
}

-(void)setItemPageColor:(UIColor*)color
{
    [self setItemValueKeyWithType:PAGECOLOR value:color.hexString key:InfoKeypath(pageColor)];
}

- (void)setContinuousON:(BOOL)on
{
    [self setItemValueKeyWithType:CONTINUOUS value:@(on) key:InfoKeypath(continusON)];
}

- (void)setPageRTLON:(BOOL)on
{
    [self setItemValueKeyWithType:PAGERTL value:@(on) key:InfoKeypath(pageRTLON)];
}

- (void)setItemTitleWithType:(SettingItemType)type title:(NSString*)title
{
    [self setItemValueKeyWithType:type value:title key:InfoKeypath(title)];
}

- (void)setItemUnSelectedWith:(NSArray*)typeArray selectedType:(SettingItemType)selectedType
{
    for (int i = 0; i < typeArray.count; i++) {
        if (((NSNumber*)typeArray[i]).unsignedIntegerValue != selectedType)
        {
            [self setItemSelectedWithType:((NSNumber*)typeArray[i]).unsignedIntegerValue selected:NO];
        }
    }
}

- (void)setItemSelectedWithType:(SettingItemType)type selected:(BOOL)selected
{
    BOOL bDefaultEnabled = [self getItemDefaultEnabledWithType:type];
    if (!bDefaultEnabled) {
        return;
    }
    UIImage *backgroundImage = [UIImage fs_imageDrawByClearColor];
    {
        if (type == REFLOW) {
            BOOL isSelectFitPage = [self getItemSelectedWithType:FITPAGE];
            BOOL isSelectFitWidth = [self getItemSelectedWithType:FITWIDTH];
            [self setItemEnabledWithType:FITPAGE enabled:!selected];
            [self setItemEnabledWithType:FITWIDTH enabled:!selected];

            if (!isSelectFitPage) {
                [self setItemSelectedWithType:FITPAGE selected:isSelectFitPage];
            }
            if (!isSelectFitWidth) {
                [self setItemSelectedWithType:FITWIDTH selected:isSelectFitWidth];
            }
            [self setItemEnabledWithType:ROTATE enabled:!selected];
        }
        else if (type == CROPPAGE){
            [self setItemEnabledWithType:PANZOOM enabled:!selected];
        }
        else if (type == FITPAGE ) {
//            if (selected) {
//                [self.fitPageItem setBackgroundImage:[Utility commonDynamicBackgroundImage] forState:UIControlStateDisabled];
//                [self.fitWidthItem setBackgroundImage:backgroundImage forState:UIControlStateDisabled];
//            }
//            else
//            {
//                [self.fitPageItem setBackgroundImage:backgroundImage forState:UIControlStateDisabled];
//            }
        }
        else if (type == FITWIDTH)
        {
//            if (selected) {
//                [self.fitWidthItem setBackgroundImage:[Utility commonDynamicBackgroundImage] forState:UIControlStateDisabled];
//                [self.fitPageItem setBackgroundImage:backgroundImage forState:UIControlStateDisabled];
//            }
//            else
//            {
//                [self.fitWidthItem setBackgroundImage:backgroundImage forState:UIControlStateDisabled];
//            }
        }
     
        if (selected) {
            if (type == REFLOW ||type == CROPPAGE || type == SPEECH || type == AUTOFLIP || type == PANZOOM) {
                if ([self getItemSelectedWithType:CROPPAGE]) {
                    if (type == REFLOW) {
                        if ([_pdfViewCtrl getCropMode] != PDF_CROP_MODE_NONE) {
                            [_pdfViewCtrl setCropMode:PDF_CROP_MODE_NONE];
                        }
                        [self setItemUnSelectedWith: self.typeMutexReflowArray selectedType:type];
                    }
                }else{
                    [self setItemUnSelectedWith: self.typeMutexReflowArray selectedType:type];
                }   
            }
            else if (type == FITPAGE || type == FITWIDTH) {
                [self setItemUnSelectedWith: self.typeMutexFitArray selectedType:type];
            }
            else if (type == DAYMODE || type == NIGHTMODE || type == PAGECOLOR) {
                dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
                    [self.extensionsManager.thumbnailModule clearThumbnailCachesForPDFAtPath:self.pdfViewCtrl.filePath];
                });
                [self setItemUnSelectedWith: @[@(DAYMODE), @(NIGHTMODE), @(PAGECOLOR)] selectedType:type];
            }
        }
    }
    
    [self setItemValueKeyWithType:type value:@(selected) key:InfoKeypath(selected)];
    switch (type) {
        case REFLOW:
            self.reflowItem.selected = selected;
            break;
        case CROPPAGE:
            self.cropModeItem.selected = selected;
            break;
        case SPEECH:
            self.speakItem.selected = selected;
            break;
        case PANZOOM:
            self.panZoomItem.selected = selected;
            break;
        case AUTOFLIP:
            self.autoFlipItem.selected = selected;
            break;
        case FITPAGE:
            self.fitPageItem.selected = selected;
            break;
        case FITWIDTH:
            self.fitWidthItem.selected = selected;
            break;
        default:
            break;
    }
}

- (void)setItemDefaultEnabledWithType:(SettingItemType)type enabled:(BOOL)enabled
{
    [self setItemValueKeyWithType:type value:@(enabled) key:InfoKeypath(defaultEnabled)];
    [self setSpecailItemEnabledWithType:type enabled:enabled];
}

- (void)setSpecailItemEnabledWithType:(SettingItemType)type enabled:(BOOL)enabled
{
    switch (type) {
        case REFLOW:
            self.reflowItem.enabled = enabled;
            break;
        case CROPPAGE:
            self.cropModeItem.enabled = enabled;
            break;
        case SPEECH:
            self.speakItem.enabled = enabled;
            break;
        case PANZOOM:
            self.panZoomItem.enabled = enabled;
            break;
        case AUTOFLIP:
            self.autoFlipItem.enabled = enabled;
            break;
        case FITPAGE:
            self.fitPageItem.enabled = enabled;
            break;
        case FITWIDTH:
            self.fitWidthItem.enabled = enabled;
            break;
        case ROTATE:
            self.rotateItem.enabled = enabled;
            break;
        default:
            break;
    }
}

- (void)setItemEnabledWithType:(SettingItemType)type enabled:(BOOL)enabled
{
    BOOL bDefaultEnabled = [self getItemDefaultEnabledWithType:type];
    if (!bDefaultEnabled) {
        [self setSpecailItemEnabledWithType:type enabled:enabled];
        return;
    }
    [self setItemValueKeyWithType:type value:@(enabled) key:InfoKeypath(enabled)];
    [self setSpecailItemEnabledWithType:type enabled:enabled];
}

- (void)setItemHiddenWithType:(SettingItemType)type hidden:(BOOL)hidden
{
    if (hidden) {
        FSSettingBarCellInfo *info = [self removeItemWithType:type];
        if (info) {
            [_dataSourceWithHidden setValue:info forKey:[NSString stringWithFormat:@"%lu", type]];
        }
    }
    else
    {
        [self addItemWithType:type];
    }
}

- (FSSettingBarCellInfo *)getItemInfoWithType:(SettingItemType)type
{
    for (NSMutableArray *array in self.dataSource) {
        for (FSSettingBarCellInfo *info in array) {
            if (type == info.itemType) {
                return info;
            }
        }
    }
    return Nil;
}

- (id)getItemValueWithType:(SettingItemType)type key:(NSString*)key
{
    FSSettingBarCellInfo *info = [self getItemInfoWithType:type];
    if (info) {
        return [info valueForKey:key];
    }
    return nil;
}

- (NSMutableArray*)getItemArrayWithSectionType:(FSSettingBarCellSectionType)sectionType
{
    for (NSMutableArray* array in self.dataSource) {
        for (FSSettingBarCellInfo *info in array) {
            if (info.sectionType == sectionType) {
                return array;
            }
        }
    }
    return Nil;
}

-(NSString*)getItemPageColor
{
    return [self getItemValueWithType:PAGECOLOR key:InfoKeypath(pageColor)];
}

- (BOOL)getContinuousON
{
    return [[self getItemValueWithType:CONTINUOUS key:InfoKeypath(continusON)] boolValue];
}

- (BOOL)getPageRTLON
{
    return [[self getItemValueWithType:PAGERTL key:InfoKeypath(pageRTLON)] boolValue];
}

- (NSString*)getItemTitleWithType:(SettingItemType)type
{
    return [self getItemValueWithType:type key:InfoKeypath(title)];
}

- (BOOL)getItemSelectedWithType:(SettingItemType)type
{
    return [[self getItemValueWithType:type key:InfoKeypath(selected)] boolValue];
}

- (BOOL)getItemEnabledWithType:(SettingItemType)type
{
    BOOL bDefaultEnabled = [self getItemDefaultEnabledWithType:type];
    if (!bDefaultEnabled) {
        return NO;
    }
    return [[self getItemValueWithType:type key:InfoKeypath(enabled)] boolValue];
}

- (BOOL)getItemDefaultEnabledWithType:(SettingItemType)type
{
    return [[self getItemValueWithType:type key:InfoKeypath(defaultEnabled)] boolValue];
}

- (BOOL)getItemHiddenWithType:(SettingItemType)type
{
    if ([self getItemInfoWithType:type])
    {
        return [[self getItemValueWithType:type key:InfoKeypath(hidden)] boolValue];
    }
    return YES;
}

- (BOOL)getItemHiddenWithIndexPath:(NSIndexPath*)indexPath
{
    int section = (int)indexPath.section;
    int row = (int)indexPath.row;
    
    for (int i = 0; i < self.dataSource.count; i++) {
        if (i == section) {
            for (int j = 0; j < self.dataSource[i].count; j++) {
                if (j == row) {
                    FSSettingBarCellInfo *info = self.dataSource[i][j];
                    return info.hidden;
                }
            }
            break;
        }
    }
    return NO;
}


- (void)registerEventListener:(id<IFSViewSettingEventListener>)listener {
    
    [self.listeners addObject:listener];
}
- (void)unregisterEventListener:(id<IFSViewSettingEventListener>)listener{
    [self.listeners removeObject:listener];
}


- (void)moveBottombarWithRecognizer:(UIPanGestureRecognizer *)recognizer{
    CGPoint point = [recognizer locationInView:recognizer.view.superview];
    
    if (recognizer.state == UIGestureRecognizerStateBegan) {
        [_extensionsManager cancelAutoFullScreen];
    }else if (recognizer.state == UIGestureRecognizerStateChanged) {
        
        CGSize size = recognizer.view.fs_size;
        CGFloat moveX = point.x - self.oldPoint.x;
        CGFloat moveY = point.y - self.oldPoint.y;
        CGFloat newXWithLeft = recognizer.view.fs_left + moveX;
        if (newXWithLeft < recognizer.view.superview.safeAreaInsets.left) {
            newXWithLeft = recognizer.view.superview.safeAreaInsets.left;
        }
        
        CGFloat newXWithRight = 0;
        BOOL bFlagRight = NO;
        if (newXWithLeft > recognizer.view.superview.fs_right/2) {
            bFlagRight = YES;
            newXWithRight = recognizer.view.fs_right - recognizer.view.superview.fs_right + moveX;
            if (newXWithRight > 0) {
                newXWithRight = 0;
            }
        }
        
        CGFloat newYWithTop = recognizer.view.fs_top + moveY;
        if (newYWithTop < recognizer.view.superview.safeAreaInsets.top) {
            newYWithTop = recognizer.view.superview.safeAreaInsets.top;
        }
        CGFloat newYWithBottom = 0;
        BOOL bFlagBottom = NO;
        if (newYWithTop > recognizer.view.superview.fs_bottom/2) {
            bFlagBottom = YES;
            newYWithBottom = recognizer.view.fs_bottom-recognizer.view.superview.fs_bottom + moveY;
            if (newYWithBottom > 0) {
                newYWithBottom = 0;
            }
            if (newYWithBottom > -1 * recognizer.view.superview.safeAreaInsets.bottom) {
                newYWithBottom = -1 * recognizer.view.superview.safeAreaInsets.bottom;
            }
        }

        [recognizer.view mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.size.mas_equalTo(size);
            if (bFlagRight) {
                make.right.mas_equalTo(newXWithRight);
            }else {
                make.left.mas_equalTo(newXWithLeft);
            }

            if (bFlagBottom) {
                make.bottom.mas_equalTo(newYWithBottom);
            }else{
                make.top.mas_equalTo(newYWithTop);
            }
            
        }];
        
    }else if (recognizer.state == UIGestureRecognizerStateEnded || recognizer.state == UIGestureRecognizerStateCancelled || recognizer.state == UIGestureRecognizerStateFailed) {
        [_extensionsManager delayAutoFullScreen];
    }
    
    self.oldPoint = point;
}

- (void)dealWhenOnlyOnePage{
    if ([self.pdfViewCtrl getPageCount] <= 1) {
        [self setItemSelectedWithType:COVERPAGE selected:NO];
        [self setItemSelectedWithType:DOUBLEPAGE selected:NO];
        [self setItemSelectedWithType:SINGLE selected:YES];
        self.extensionsManager.settingBar.settingView.currentShowItemType = -1;
        [self.extensionsManager.settingBar didClickItemWithType:SINGLE];
        
        [self setContinuousON:NO];
        [self.extensionsManager.settingBar didClickItemWithType:CONTINUOUS];
        [self setPageRTLON:NO];
        [self.extensionsManager.settingBar didClickItemWithType:PAGERTL];
    }
}

#pragma mark <FSMainTopbarEventListener>
- (void)didSetCurrentTagItem:(FSMainTopbarToolTagItem *)curTagItem lastTagItem:(FSMainTopbarToolTagItem *)lastTagItem{
    if (curTagItem.toolTag == FSMainTopbarToolTagView) {
        [self.extensionsManager.settingBar updateLayoutButtonsWithLayout:[self.extensionsManager.pdfViewCtrl getPageLayoutMode]];
    }else if(curTagItem != lastTagItem && lastTagItem.toolTag == FSMainTopbarToolTagView && lastTagItem.toolTag != FSMainTopbarToolTagSign && [self.extensionsManager getState] != FSUIManagerStateAutoFlip && [self.extensionsManager getState] != FSUIManagerStateSpeech && [self.extensionsManager getState] != FSUIManagerStatePanZoom){
        if (![_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            [self.extensionsManager changeState:FSUIManagerStateNormal];
        }
        
    }
}

- (void)setPageColor:(UIColor*)color
{
    if (self.pdfViewCtrl.isNightMode) {
        self.pdfViewCtrl.isNightMode = NO;
    }
    self.isDayOrNightForLast = NO;
    if ([self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_REFLOW) {
        [self.pdfViewCtrl setReflowBackgroundColor:color];
        [self.pdfViewCtrl setMappingModeBackgroundColor:color];
        [self.pdfViewCtrl setPageLayoutMode:PDF_LAYOUT_MODE_REFLOW];
    }
    else
    {
        if (DEVICE_iPHONE) {
            self.isDayOrNightForLast = NO;
        }
        if (self.pdfViewCtrl.colorMode != FSRendererColorModeMapping) {
            if (color.rgbValue != UIColor.whiteColor.rgbValue) {
                [self.pdfViewCtrl setColorMode:FSRendererColorModeMapping];
            }
            [self.pdfViewCtrl setMappingModeBackgroundColor:color];
            [self.pdfViewCtrl setReflowBackgroundColor:color];
        }else{
            if (color.rgbValue == UIColor.whiteColor.rgbValue) {
                [self.pdfViewCtrl setColorMode:FSRendererColorModeNormal];
            }
            [self.pdfViewCtrl setMappingModeBackgroundColor:color];
            [self.pdfViewCtrl setReflowBackgroundColor:color];
            [self.pdfViewCtrl refresh];
        }
    }
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        [self.extensionsManager.thumbnailModule clearThumbnailCachesForPDFAtPath:self.pdfViewCtrl.filePath];
    });
    [_extensionsManager.settingBar.viewSettingManager setItemPageColor:color];
}

- (void)setPageColorWithNormalState
{
    if (self.isDayOrNightForLast) {
        _extensionsManager.pdfViewCtrl.isNightMode = [self getItemSelectedWithType:NIGHTMODE];
    }
    else
    {
        [self setPageColor:_extensionsManager.pdfViewCtrl.reflowBackgroundColor];
    }
}

- (void)setIsNightMode:(BOOL)isNightMode
{
    [self setItemSelectedWithType:isNightMode ? NIGHTMODE: DAYMODE  selected:YES];
    _dayModeItem.selected = isNightMode;
}

- (void)resetPdfViewCtrlSafeArea{
    UIEdgeInsets insets = _extensionsManager.pdfViewCtrl.fs_viewController.additionalSafeAreaInsets;
    if (UIEdgeInsetsEqualToEdgeInsets(insets, UIEdgeInsetsZero)) {
        insets = self.extensionsManager.pdfViewCtrl.fs_currentSceneKeyWindow.safeAreaInsets;
    }
    [_extensionsManager.pdfViewCtrl.fs_viewController setNeedsStatusBarAppearanceUpdate];
    _extensionsManager.pdfViewCtrl.fs_viewController.additionalSafeAreaInsets = insets;
    if (UIEdgeInsetsEqualToEdgeInsets(insets,self.extensionsManager.pdfViewCtrl.fs_currentSceneKeyWindow.safeAreaInsets)) {
        insets = UIEdgeInsetsZero;
    }
    _extensionsManager.pdfViewCtrl.fs_viewController.additionalSafeAreaInsets = insets;
}

- (BOOL)checkIsIphoneDevice
{
    return DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact];
}

#pragma IScrollViewEventListener
-(void)onScrollViewDidScroll:(UIScrollView *)scrollView
{
    if (scrollView.zoomScale != 1) {
        [self setItemSelectedWithType:FITPAGE selected:NO];
        [self setItemSelectedWithType:FITWIDTH selected:NO];
    }

}

#pragma pageEventListener methods

- (void)onPagesRemoved:(NSArray<NSNumber *> *)indexes {
    [self dealWhenOnlyOnePage];
}


@end
