/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "ShapeAnnotHandler.h"
#import "FSPermissionUtilty.h"

@implementation ShapeAnnotHandler

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    if (self = [super initWithUIExtensionsManager:extensionsManager]) {
        self.pageEdgeInsets = UIEdgeInsetsMake(20, 20, 20, 20);
        self.annotSelectionEdgeInsets = UIEdgeInsetsMake(-20, -20, -20, -20);
        self.annotRefreshRectEdgeInsets = UIEdgeInsetsMake(-30, -30, -30, -30);
        self.annotBorderEdgeInsets = UIEdgeInsetsMake(-10, -10, -10, -10);
    }
    return self;
}

- (FSAnnotType)getType {
    return FSAnnotCircle;
}

- (FSMenuOptions)menuOptionsForAnnot:(FSAnnot *)annot {
    FSMenuOptions options = FSMenuOptionOpen;
    if ([self.extensionsManager annotPermissionCanFlatten:annot]) {
        options |= FSMenuOptionFlatten;
    }
    if (![FSPermissionUtilty canAddAnnot:self.extensionsManager]) {
        options = FSMenuOptionOpen;
    }
    if (annot.canModify && [FSPermissionUtilty canAddAnnot:self.extensionsManager]) {
        options |= FSMenuOptionStyle;
        if ([annot canDeleteWithExternsions:self.extensionsManager]) {
            options |= FSMenuOptionDelete;
        }
    }
    if (annot.canReply && [FSPermissionUtilty canAddAnnot:self.extensionsManager]) {
        if ([self.extensionsManager annotsPermissionCanReply:annot]) {
            options |= FSMenuOptionReply;
        }
    }
    return options;
}

- (void)showMenu {
    if (!self.annot) {
        return;
    }
    int pageIndex = self.annot.pageIndex;
    CGRect rect = [self.pdfViewCtrl convertPdfRectToPageViewRect:self.annot.fsrect pageIndex:pageIndex];
    CGRect dvRect = [self.pdfViewCtrl convertPageViewRectToDisplayViewRect:rect pageIndex:pageIndex];
    [self.extensionsManager.menuControl setRect:dvRect margin:20]; // enlarge distance from menu to annot
    [self.extensionsManager.menuControl showMenu];
}

- (void)resetProperties{

    FSAnnot *annot = self.extensionsManager.currentAnnot;
    FSPropertyMainView *propertyMainView = self.extensionsManager.propertyMainView;
    [propertyMainView setPropertyColor:annot.color];
    [propertyMainView setOpacity:annot.opacity];
    [propertyMainView setLineWidth:annot.lineWidth];
    [propertyMainView setFillColor:annot.exfillColor];
    [propertyMainView setLock:![annot canModifyAppearanceWithExternsions:self.extensionsManager]];
    [propertyMainView resetProperties:@[@(FSPropertyTypeBorderColor), @(FSPropertyTypeBorderWidth), @(FSPropertyTypeFillColor), @(FSPropertyTypeOpacity)]];
}


- (CGSize)minSizeForResizingAnnot:(FSAnnot *)annot {
    CGSize minSize = [super minSizeForResizingAnnot:annot];
    if ([annot getType] == FSAnnotSquare) {
        FSSquare *square = [[FSSquare alloc] initWithAnnot:annot];
        FSBorderInfo *borderInfo = [square getBorderInfo];
        if ([borderInfo getStyle] == FSBorderInfoCloudy) {
            int pageIndex = square.pageIndex;
            CGRect rect = [self.pdfViewCtrl convertPdfRectToPageViewRect:[square getRect] pageIndex:pageIndex];
            CGRect innerRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:[square getInnerRect] pageIndex:pageIndex];
            const CGFloat minInnerWidth = 5.0;
            const CGFloat minInnerHeight = 5.0;
            minSize.width = MAX(minSize.width, rect.size.width + minInnerWidth - innerRect.size.width);
            minSize.height = MAX(minSize.height, rect.size.height + minInnerHeight - innerRect.size.height);
        }
    }
    return minSize;
}

- (BOOL)stopCurrentGestureRecognizerShouldBegin:(UIGestureRecognizer *)recognizer annot:(FSAnnot *)annot{
    return [recognizer isKindOfClass:[UILongPressGestureRecognizer class]];
}

@end
