/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "ShapeUtil.h"
#import "FSAnnotExtent.h"

@implementation ShapeUtil

static CGSize _dragDotSize;
+ (CGSize)dragDotSize{
    static UIImage *annotDotImage;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        if (!annotDotImage) annotDotImage = FSImageNamed(@"annotation_drag");
        _dragDotSize = annotDotImage.size;
    });
    return _dragDotSize;
}

+ (NSArray *)getMovePointInRect:(CGRect)rect {
    if (rect.size.width == 0 || rect.size.height == 0) {
        return nil;
    }

    UIImage *dragDot = [UIImage imageNamed:@"annotation_drag.png" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
    CGFloat dragWidth = dragDot.size.width;
    NSMutableArray *array = [NSMutableArray array];
    [array addObject:[NSValue valueWithCGRect:CGRectMake(rect.origin.x - dragWidth / 2.0, rect.origin.y - dragWidth / 2.0, dragWidth, dragWidth)]];
    [array addObject:[NSValue valueWithCGRect:CGRectMake(rect.origin.x - dragWidth / 2.0, rect.origin.y + rect.size.height / 2.0 - dragWidth / 2.0, dragWidth, dragWidth)]];
    [array addObject:[NSValue valueWithCGRect:CGRectMake(rect.origin.x - dragWidth / 2.0, rect.origin.y + rect.size.height - dragWidth / 2.0, dragWidth, dragWidth)]];
    [array addObject:[NSValue valueWithCGRect:CGRectMake(rect.origin.x + rect.size.width / 2.0 - dragWidth / 2.0, rect.origin.y - dragWidth / 2.0, dragWidth, dragWidth)]];
    [array addObject:[NSValue valueWithCGRect:CGRectMake(rect.origin.x + rect.size.width / 2.0 - dragWidth / 2.0, rect.origin.y + rect.size.height - dragWidth / 2.0, dragWidth, dragWidth)]];
    [array addObject:[NSValue valueWithCGRect:CGRectMake(rect.origin.x + rect.size.width - dragWidth / 2.0, rect.origin.y - dragWidth / 2.0, dragWidth, dragWidth)]];
    [array addObject:[NSValue valueWithCGRect:CGRectMake(rect.origin.x + rect.size.width - dragWidth / 2.0, rect.origin.y + rect.size.height / 2.0 - dragWidth / 2.0, dragWidth, dragWidth)]];
    [array addObject:[NSValue valueWithCGRect:CGRectMake(rect.origin.x + rect.size.width - dragWidth / 2.0, rect.origin.y + rect.size.height - dragWidth / 2.0, dragWidth, dragWidth)]];
    return array;
}

+ (NSArray *)getCornerMovePointInRect:(CGRect)rect {
    if (rect.size.width == 0 || rect.size.height == 0) {
        return nil;
    }

    UIImage *dragDot = [UIImage imageNamed:@"annotation_drag.png" inBundle:[NSBundle bundleForClass:[self class]] compatibleWithTraitCollection:nil];
    CGFloat dragWidth = dragDot.size.width;
    NSMutableArray *array = [NSMutableArray array];
    [array addObject:[NSValue valueWithCGRect:CGRectMake(rect.origin.x - dragWidth / 2.0, rect.origin.y - dragWidth / 2.0, dragWidth, dragWidth)]];
    //    [array addObject:[NSValue valueWithCGRect:CGRectMake(rect.origin.x - dragWidth / 2.0, rect.origin.y + rect.size.height / 2.0 - dragWidth / 2.0, dragWidth, dragWidth)]];
    [array addObject:[NSValue valueWithCGRect:CGRectMake(rect.origin.x - dragWidth / 2.0, rect.origin.y + rect.size.height - dragWidth / 2.0, dragWidth, dragWidth)]];
    //    [array addObject:[NSValue valueWithCGRect:CGRectMake(rect.origin.x + rect.size.width / 2.0 - dragWidth / 2.0, rect.origin.y - dragWidth / 2.0, dragWidth, dragWidth)]];
    //    [array addObject:[NSValue valueWithCGRect:CGRectMake(rect.origin.x + rect.size.width / 2.0 - dragWidth / 2.0, rect.origin.y + rect.size.height - dragWidth / 2.0, dragWidth, dragWidth)]];
    [array addObject:[NSValue valueWithCGRect:CGRectMake(rect.origin.x + rect.size.width - dragWidth / 2.0, rect.origin.y - dragWidth / 2.0, dragWidth, dragWidth)]];
    //    [array addObject:[NSValue valueWithCGRect:CGRectMake(rect.origin.x + rect.size.width - dragWidth / 2.0, rect.origin.y + rect.size.height / 2.0 - dragWidth / 2.0, dragWidth, dragWidth)]];
    [array addObject:[NSValue valueWithCGRect:CGRectMake(rect.origin.x + rect.size.width - dragWidth / 2.0, rect.origin.y + rect.size.height - dragWidth / 2.0, dragWidth, dragWidth)]];
    return array;
}

+ (FSAnnotEditType)getEditTypeWithPoint:(CGPoint)point rect:(CGRect)rect defaultEditType:(FSAnnotEditType)defaultEditType {
    CGFloat left = rect.origin.x;
    CGFloat top = rect.origin.y;
    CGFloat w = rect.size.width;
    CGFloat h = rect.size.height;
    NSArray *pointArray = [NSArray arrayWithObjects:
                                       [NSValue valueWithCGPoint:CGPointMake(left, top)],
                                       [NSValue valueWithCGPoint:CGPointMake(left, top + h / 2.0)],
                                       [NSValue valueWithCGPoint:CGPointMake(left, top + h)],
                                       [NSValue valueWithCGPoint:CGPointMake(left + w / 2.0, top)],
                                       [NSValue valueWithCGPoint:CGPointMake(left + w / 2.0, top + h)],
                                       [NSValue valueWithCGPoint:CGPointMake(left + w, top)],
                                       [NSValue valueWithCGPoint:CGPointMake(left + w, top + h / 2.0)],
                                       [NSValue valueWithCGPoint:CGPointMake(left + w, top + h)],
                                       [NSValue valueWithCGPoint:CGPointMake(left + w / 2.0, top + h / 2.0)],
                                       nil];
    __block NSUInteger minIdx;
    __block CGFloat minDiff = FLT_MAX;
    [pointArray enumerateObjectsUsingBlock:^(id _Nonnull obj, NSUInteger idx, BOOL *_Nonnull stop) {
        CGPoint pt = [obj CGPointValue];
        CGFloat diff = ABS(pt.x - point.x) + ABS(pt.y - point.y);
        if (diff < minDiff) {
            minIdx = idx;
            minDiff = diff;
        }
    }];
    const CGFloat tolerance = 30.0f;
    CGPoint closestPoint = [pointArray[minIdx] CGPointValue];
    if ([self isPoint:point closeToPoint:closestPoint tolerance:tolerance]) {
        return (FSAnnotEditType) minIdx;
    }
    return defaultEditType;
}

+ (FSAnnotEditType)getAnnotEditTypeWithPoint:(CGPoint)point rect:(CGRect)rect keepAspectRatio:(BOOL)keepAspectRatio defaultEditType:(FSAnnotEditType)defaultEditType {
    FSAnnotEditType editType = (FSAnnotEditType) [ShapeUtil getEditTypeWithPoint:point rect:rect defaultEditType:(FSAnnotEditType) defaultEditType];
    // if keep aspect ratio, the 4 middle dots are not drawn
    if (keepAspectRatio) {
        if (editType == FSAnnotEditTypeMiddleTop ||
            editType == FSAnnotEditTypeMiddleBottom ||
            editType == FSAnnotEditTypeLeftMiddle ||
            editType == FSAnnotEditTypeRightMiddle) {
            editType = FSAnnotEditTypeFull;
        }
    }
    return editType;
}

+ (NSInteger)getFillSignEditTypeWithPoint:(CGPoint)point rect:(CGRect)rect keepAspectRatio:(BOOL)keepAspectRatio defaultEditType:(NSInteger)defaultEditType {
    NSInteger editType = (NSInteger) [ShapeUtil getEditTypeWithPoint:point rect:rect defaultEditType:(NSInteger) defaultEditType];
    // if keep aspect ratio, the 4 middle dots are not drawn
    if (keepAspectRatio) {
        if (editType == FSAnnotEditTypeMiddleTop ||
            editType == FSAnnotEditTypeMiddleBottom ||
            editType == FSAnnotEditTypeLeftMiddle ||
            editType == FSAnnotEditTypeRightMiddle) {
            editType = FSAnnotEditTypeFull;
        }
    }
    return editType;
}

+ (FSAnnotEditType)getLineEditTypeWithPoint:(CGPoint)point line:(FSLine *_Nonnull)line pdfViewCtrl:(FSPDFViewCtrl *)pdfViewCtrl {
    int pageIndex = line.pageIndex;
    CGPoint startPoint = [pdfViewCtrl convertPdfPtToPageViewPt:[line getStartPoint] pageIndex:pageIndex];
    CGPoint endPoint = [pdfViewCtrl convertPdfPtToPageViewPt:[line getEndPoint] pageIndex:pageIndex];
    BOOL isStartPointCloser = (ABS(startPoint.x - point.x) + ABS(startPoint.y - point.y)) < (ABS(endPoint.x - point.x) + ABS(endPoint.y - point.y));
    if (isStartPointCloser) {
        if ([ShapeUtil isPoint:startPoint closeToPoint:point tolerance:25.0f]) {
            return FSAnnotEditTypeStartPoint;
        }
    } else {
        if ([ShapeUtil isPoint:endPoint closeToPoint:point tolerance:25.0f]) {
            return FSAnnotEditTypeEndPoint;
        }
    }
    return FSAnnotEditTypeFull;
}

+ (CGRect)newRectOfZoomForPanTranslation:(CGPoint)translation oldRect:(CGRect)oldRect minSize:(CGSize)minSize maxSize:(CGSize)maxSize keepAspectRatio:(BOOL)keepAspectRatio pageIndex:(int)pageIndex pdfViewCtrl:(FSPDFViewCtrl *)pdfViewCtrl editType:(FSAnnotEditType)editType {
    return [self newRectOfZoomForPanTranslation:translation oldRect:oldRect minSize:minSize maxSize:maxSize edge:UIEdgeInsetsZero keepAspectRatio:keepAspectRatio pageIndex:pageIndex pdfViewCtrl:pdfViewCtrl editType:editType];
}

+ (CGRect)newRectOfZoomForPanTranslation:(CGPoint)translation oldRect:(CGRect)oldRect minSize:(CGSize)minSize maxSize:(CGSize)maxSize edge:(UIEdgeInsets)edge keepAspectRatio:(BOOL)keepAspectRatio pageIndex:(int)pageIndex pdfViewCtrl:(FSPDFViewCtrl *)pdfViewCtrl editType:(FSAnnotEditType)editType {
    UIView *pageView = [pdfViewCtrl getPageView:pageIndex];
    UIEdgeInsets pageEdgeInsets = UIEdgeInsetsMake(edge.top + self.dragDotSize.height/2, edge.left + self.dragDotSize.width/2, edge.bottom + self.dragDotSize.height/2, edge.right + self.dragDotSize.width/2);
    CGRect pageInsetRect = UIEdgeInsetsInsetRect(pageView.bounds, pageEdgeInsets);
    CGFloat w = CGRectGetWidth(oldRect);
    CGFloat h = CGRectGetHeight(oldRect);
    CGFloat minW = minSize.width;
    CGFloat maxW = maxSize.width;
    CGFloat minH = minSize.height;
    CGFloat maxH = maxSize.height;
    CGFloat dx = 0;
    CGFloat dy = 0;
    CGFloat dw = 0;
    CGFloat dh = 0;
    
    if (editType == FSAnnotEditTypeFull) {
        dx = translation.x;
        dy = translation.y;
        dx = MIN(dx, CGRectGetMaxX(pageInsetRect) - CGRectGetMaxX(oldRect));
        dx = MAX(dx, CGRectGetMinX(pageInsetRect) - CGRectGetMinX(oldRect));
        dy = MIN(dy, CGRectGetMaxY(pageInsetRect) - CGRectGetMaxY(oldRect));
        dy = MAX(dy, CGRectGetMinY(pageInsetRect) - CGRectGetMinY(oldRect));
    } else {
        if (!keepAspectRatio) {
            if (editType == FSAnnotEditTypeLeftTop ||
                editType == FSAnnotEditTypeLeftMiddle ||
                editType == FSAnnotEditTypeLeftBottom) { // left
                dw = -translation.x;
                dw = MIN(dw, maxW - w);
                dw = MAX(dw, minW - w);
                dw = MIN(dw, CGRectGetMinX(oldRect) - CGRectGetMinX(pageInsetRect));
                dx = -dw;
            } else if (editType == FSAnnotEditTypeRightTop ||
                       editType == FSAnnotEditTypeRightMiddle ||
                       editType == FSAnnotEditTypeRightBottom) { // right
                dw = translation.x;
                dw = MIN(dw, maxW - w);
                dw = MAX(dw, minW - w);
                dw = MIN(dw, CGRectGetMaxX(pageInsetRect) - CGRectGetMaxX(oldRect));
            }
            if (editType == FSAnnotEditTypeLeftTop ||
                editType == FSAnnotEditTypeMiddleTop ||
                editType == FSAnnotEditTypeRightTop) { // top
                dh = -translation.y;
                dh = MIN(dh, maxH - h);
                dh = MAX(dh, minH - h);
                dh = MIN(dh, CGRectGetMinY(oldRect) - CGRectGetMinY(pageInsetRect));
                dy = -dh;
            } else if (editType == FSAnnotEditTypeLeftBottom ||
                       editType == FSAnnotEditTypeMiddleBottom ||
                       editType == FSAnnotEditTypeRightBottom) { // bottom
                dh = translation.y;
                dh = MIN(dh, maxH - h);
                dh = MAX(dh, minH - h);
                dh = MIN(dh, CGRectGetMaxY(pageInsetRect) - CGRectGetMaxY(oldRect));
            }
        } else { // keep aspect ratio
            if (editType == FSAnnotEditTypeRightBottom) {
                CGFloat tmp = (w * translation.x + h * translation.y) / (w * w + h * h);
                dw = w * tmp;
                dh = h * tmp;
                maxW = MIN(maxW, CGRectGetMaxX(pageInsetRect) - CGRectGetMinX(oldRect));
                maxH = MIN(maxH, CGRectGetMaxY(pageInsetRect) - CGRectGetMinY(oldRect));
                dw = MIN(MAX(dw, minW - w), maxW - w);
                dh = h / w * dw;
                dh = MIN(MAX(dh, minH - h), maxH - h);
                dw = w / h * dh;
            } else if (editType == FSAnnotEditTypeLeftTop) {
                CGFloat tmp = (w * translation.x + h * translation.y) / (w * w + h * h);
                dw = -w * tmp;
                dh = -h * tmp;
                maxW = MIN(maxW, CGRectGetMaxX(oldRect) - CGRectGetMinX(pageInsetRect));
                maxH = MIN(maxH, CGRectGetMaxY(oldRect) - CGRectGetMinY(pageInsetRect));
                dw = MIN(MAX(dw, minW - w), maxW - w);
                dh = h / w * dw;
                dh = MIN(MAX(dh, minH - h), maxH - h);
                dw = w / h * dh;
                dx = -dw;
                dy = -dh;
            } else if (editType == FSAnnotEditTypeRightTop) {
                CGFloat tmp = (w * translation.x - h * translation.y) / (w * w + h * h);
                dw = w * tmp;
                dh = h * tmp;
                maxW = MIN(maxW, CGRectGetMaxX(pageInsetRect) - CGRectGetMinX(oldRect));
                maxH = MIN(maxH, CGRectGetMaxY(oldRect) - CGRectGetMinY(pageInsetRect));
                dw = MIN(MAX(dw, minW - w), maxW - w);
                dh = h / w * dw;
                dh = MIN(MAX(dh, minH - h), maxH - h);
                dw = w / h * dh;
                dy = -dh;
            } else if (editType == FSAnnotEditTypeLeftBottom) {
                CGFloat tmp = (w * translation.x - h * translation.y) / (w * w + h * h);
                dw = -w * tmp;
                dh = -h * tmp;
                maxW = MIN(maxW, CGRectGetMaxX(oldRect) - CGRectGetMinX(pageInsetRect));
                maxH = MIN(maxH, CGRectGetMaxY(pageInsetRect) - CGRectGetMinY(oldRect));
                dw = MIN(MAX(dw, minW - w), maxW - w);
                dh = h / w * dw;
                dh = MIN(MAX(dh, minH - h), maxH - h);
                dw = w / h * dh;
                dx = -dw;
            }
        }
    }
    CGRect newRect = oldRect;
    newRect.origin.x += dx;
    newRect.origin.y += dy;
    newRect.size.width += dw;
    newRect.size.height += dh;
    return newRect;
}



+ (BOOL)isPoint:(CGPoint)p1 closeToPoint:(CGPoint)p2 tolerance:(CGFloat)tolerance {
    CGFloat dx = fabs(p1.x - p2.x);
    CGFloat dy = fabs(p1.y - p2.y);
    return (dx * dx + dy * dy) < tolerance * tolerance;
}

+ (NSArray *)getCornerPointOfRect:(CGRect)rect {
    if (rect.size.width == 0 || rect.size.height == 0) {
        return nil;
    }
    NSMutableArray *array = [NSMutableArray array];
    [array addObject:[NSValue valueWithCGPoint:CGPointMake(rect.origin.x, rect.origin.y)]];
    [array addObject:[NSValue valueWithCGPoint:CGPointMake(rect.origin.x , rect.origin.y + rect.size.height)]];
    [array addObject:[NSValue valueWithCGPoint:CGPointMake(rect.origin.x + rect.size.width, rect.origin.y)]];
    [array addObject:[NSValue valueWithCGPoint:CGPointMake(rect.origin.x + rect.size.width, rect.origin.y + rect.size.height)]];
    return array;
}

@end
