/**
* Copyright (C) 2003-2025, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
* distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
* is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
* Review legal.txt for additional license and legal information.
*/

#import "SpeechModule.h"
#import <AVFoundation/AVFoundation.h>
#import "StringDrawUtil.h"
#import <MediaPlayer/MediaPlayer.h>
#import "PageNavigationModule.h"
#import "FSSettingBar+private.h"
#import "FSDataItem.h"

@interface FSSpeechMoreSettingGuideView : UIView
@property (nonatomic, strong) UIView *contentView;
@property (nonatomic, strong) UIView *indicatorView;
@property (nonatomic, strong) UILabel *titleLabel;
@property (nonatomic, strong) UILabel *descriptLabel;
@property (nonatomic, strong) UIButton *actionButton;

@property (nonatomic, weak) UIView *alignmentView1;
@property (nonatomic, weak) UIView *alignmentView2;

@property (nonatomic, copy) void(^onGotoClick)();
- (void)showGuideViewWithAlignmentView:(UIView *)alignmentView1 alignmentView2:(UIView *)alignmentView2;
@end

@implementation FSSpeechMoreSettingGuideView

- (void)showGuideViewWithAlignmentView:(UIView *)alignmentView1 alignmentView2:(UIView *)alignmentView2
{
    UIWindow *window = alignmentView1.window;
    if (!window) return;
    _alignmentView1 = alignmentView1;
    _alignmentView2 = alignmentView2;
    [window addSubview:self];
    [self mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.edges.mas_equalTo(self.superview);
    }];
    [self addSubview:self.contentView];
    [self addSubview:self.indicatorView];
    [self resetContentViewFrame];
}

- (void)resetContentViewFrame
{
    if (!self.contentView.superview) return;
    [self.contentView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(self.alignmentView2.fs_mas_right).offset(-30);
        make.bottom.mas_equalTo(self.alignmentView1.fs_mas_top).mas_offset(-50);
        make.width.mas_equalTo(274);
    }];
    
    [self.indicatorView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.centerX.mas_equalTo(self.alignmentView2.mas_centerX);
        make.top.mas_equalTo(self.contentView.fs_mas_bottom);
        make.bottom.mas_equalTo(self.alignmentView1.fs_mas_top);
        make.width.mas_equalTo(7);
    }];
}

- (void)buttonClick
{
    [self removeFromSuperview];
    [Preference setBoolValue:FSPreferenceModuleSpeech type:@"hadShowSetting" value:YES];
    if (self.onGotoClick) self.onGotoClick();
}

- (UIView *)contentView
{
    if (!_contentView) {
        _contentView = [[UIView alloc] init];
        _contentView.backgroundColor = self.fs_backgroundColor_B8;
        [_contentView addSubview:self.titleLabel];
        [self.titleLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(_contentView.fs_mas_left).offset(16);
            make.top.mas_equalTo(_contentView.fs_mas_top).offset(16);
            make.right.mas_equalTo(_contentView.fs_mas_right).offset(-4);
        }];
        [_contentView addSubview:self.descriptLabel];
        [self.descriptLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(_contentView.fs_mas_left).offset(16);
            make.top.mas_equalTo(self.titleLabel.fs_mas_bottom).offset(12);
            make.right.mas_equalTo(_contentView.fs_mas_right).offset(-4);
        }];
        [_contentView addSubview:self.actionButton];
        [self.actionButton mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.bottom.mas_equalTo(_contentView.fs_mas_bottom).offset(-16);
            make.top.mas_equalTo(self.descriptLabel.fs_mas_bottom).offset(12);
            make.right.mas_equalTo(_contentView.fs_mas_right).offset(-16);
            make.height.mas_equalTo(28);
        }];
        _contentView.layer.cornerRadius = 8;
        _contentView.layer.shadowOpacity = 0.25;
        _contentView.layer.shadowOffset = CGSizeMake(10, 10);
    }
    return _contentView;
}

- (UIView *)indicatorView
{
    if (!_indicatorView) {
        _indicatorView = [[UIView alloc] init];
        
        UIView *line = [UIView new];
        line.backgroundColor = self.fs_primaryColor;
        [_indicatorView addSubview:line];
        UIView *circle = [UIView new];
        circle.backgroundColor = self.fs_primaryColor;
        circle.layer.cornerRadius = 3.5;
        [_indicatorView addSubview:circle];
        [line mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.centerX.mas_equalTo(_indicatorView.mas_centerX);
            make.top.mas_equalTo(_indicatorView.mas_top);
            make.bottom.mas_equalTo(circle.mas_top).offset(2);
            make.width.mas_equalTo(2);
        }];
        [circle mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.bottom.mas_equalTo(_indicatorView.mas_bottom);
            make.width.height.mas_equalTo(7);
        }];
    }
    return _indicatorView;
}

- (UILabel *)titleLabel
{
    if (!_titleLabel) {
        _titleLabel = [[UILabel alloc] init];
        _titleLabel.font = self.fs_standardGeneralFont_GF1;
        _titleLabel.textColor = self.fs_textColor_T4;
        _titleLabel.text = FSLocalizedForKey(@"kLanguageVoice");
    }
    return _titleLabel;
}

- (UILabel *)descriptLabel
{
    if (!_descriptLabel) {
        _descriptLabel = [[UILabel alloc] init];
        _descriptLabel.numberOfLines = 0;
        _descriptLabel.font = self.fs_standardGeneralFont_GF3;
        _descriptLabel.textColor = self.fs_textColor_T4;
        _descriptLabel.text = FSLocalizedForKey(@"kVoiceSettingGuideContent");
    }
    return _descriptLabel;
}

- (UIButton *)actionButton
{
    if (!_actionButton) {
        _actionButton = [[UIButton alloc] init];
        _actionButton.layer.cornerRadius = 4;
        _actionButton.contentEdgeInsets = UIEdgeInsetsMake(0, 18, 0, 18);
        _actionButton.backgroundColor = self.fs_primaryColor;
        _actionButton.titleLabel.font = self.fs_standardGeneralFont_GF3;
        [_actionButton setTitleColor:self.fs_textColor_T1 forState:UIControlStateNormal];
        [_actionButton setTitle:FSLocalizedForKey(@"kVoiceSettingBtnTitle") forState:UIControlStateNormal];
        [_actionButton addTarget:self action:@selector(buttonClick) forControlEvents:UIControlEventTouchUpInside];
    }
    return _actionButton;
}

@end

@interface FSSpeechMoreSettingDataItem : FSDataItem

@property (nonatomic, copy) NSString *imageName;
@property (nonatomic, copy) NSString *title;
@property (nonatomic, copy) NSString *settingValue;
@property (nonatomic, copy) void(^onItemSelect)(void);

@end

@implementation FSSpeechMoreSettingDataItem
@end

@interface FSSpeechMoreSettingViewController : FSTableViewController
@property (nonatomic, strong) UIView *roundedRectangleView;
@end

@implementation FSSpeechMoreSettingViewController

- (void)initialize{
    [super initialize];
    self.shouldShowNavigationBar = NO;
    self.defaultEstimatedCellStyle = FSEstimatedTableViewCellStyleTitle;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    [self arrangeRoundView];
    self.subviewTableViewLayoutMargins = UIEdgeInsetsMake(self.fs_currentSceneIsCompact ? 10 : 0, 0, 0, 0);
    self.roundedRectangleView.hidden = !self.fs_currentSceneIsCompact;
    //[self loadSettingData];
}

- (void)traitCollectionDidChange:(UITraitCollection *)previousTraitCollection
{
    if (previousTraitCollection.horizontalSizeClass != self.traitCollection.horizontalSizeClass) {
        self.subviewTableViewLayoutMargins = UIEdgeInsetsMake(self.fs_currentSceneIsCompact ? 10 : 0, 0, 0, 0);
        self.roundedRectangleView.hidden = !self.fs_currentSceneIsCompact;
    }
}

- (UIView *)roundedRectangleView{
    if (!_roundedRectangleView) {
        _roundedRectangleView = [[UIView alloc] init];
        _roundedRectangleView.backgroundColor = self.fs_backgroundColor_B3;
        _roundedRectangleView.layer.cornerRadius = 2;
        _roundedRectangleView.layer.masksToBounds = YES;
    }
    return _roundedRectangleView;
}

- (void)arrangeRoundView{
    [self.view addSubview:self.roundedRectangleView];
    [self.roundedRectangleView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(self.view);
        make.top.equalTo(self.view).offset(3);
        make.size.mas_equalTo(CGSizeMake(31, 4));
    }];
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (void)configureCell:(FSTitleEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(id)dataItem{
    
    if (!cell.installed) {
        cell.leftMargin = COMMON_RIGHT_ICON_MARGINT;
        cell.rightMargin = COMMON_RIGHT_ICON_MARGINT;
        cell.mediumSpace = COMMON_RIGHT_ICON_MARGINT;
    }
    cell.normalBackgroundColor = self.fs_backgroundColor_B15;
    cell.selectedBackgroundColor = self.fs_dividerColor_P1;
    cell.contentView.backgroundColor = self.fs_backgroundColor_B15;
    
    FSSpeechMoreSettingDataItem *dataItem1 = dataItem;
    cell.estimatedTitleLabel.text = dataItem1.title;
    FSItem *leftItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(dataItem1.imageName)];
    leftItem.userInteractionEnabled = NO;
    cell.leftView = leftItem;
    FSItem *rightItem = [FSItem createItemWithTitle:dataItem1.settingValue image:FSImageNamedAndNormalColor(@"common_accessory_gray") imageTextRelation:FSImageTextRelationLeft];
    rightItem.userInteractionEnabled = NO;
    rightItem.titleLabel.font = self.fs_standardGeneralFont_GF2;
    rightItem.titleLabel.textColor = self.fs_textColor_T4;
    cell.rightView = rightItem;
    
    BOOL hadShowSetting = [Preference getBoolValue:FSPreferenceModuleSpeech type:@"hadShowSetting" delaultValue:NO];
    if (!hadShowSetting && indexPath.row == 0) {
        @weakify(self)
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.5 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
            @strongify(self)
            FSSpeechMoreSettingGuideView *guideView = [FSSpeechMoreSettingGuideView new];
            [guideView showGuideViewWithAlignmentView:self.view alignmentView2:cell.rightView];
            guideView.onGotoClick = ^{
                @strongify(self)
                [self dismissViewControllerAnimated:YES completion:nil];
            };
        });
    }
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    FSSpeechMoreSettingDataItem *dataItem = [self dataForAtIndexPath:indexPath];
    [self dismissViewControllerAnimated:YES completion:^{
        if (dataItem.onItemSelect) dataItem.onItemSelect();
    }];
}

@end


@interface FSSpeechVoiceDataItem : FSDataItem

@property (nonatomic, copy) NSString *identifier;
@property (nonatomic, copy) NSString *language;
@property (nonatomic, copy) NSString *languagelocal;
@property (nonatomic, copy) NSString *name;
@property (nonatomic, copy) void(^onItemSelect)(FSSpeechVoiceDataItem *item);

@end

@implementation FSSpeechVoiceDataItem
@end

@interface FSSpeechVoiceLanguageDataItem : FSDataItem

@property (nonatomic, copy) NSString *language;
@property (nonatomic, copy) NSString *languagelocal;
@property (nonatomic, strong) NSMutableArray<FSSpeechVoiceDataItem*> *dataItems;
@property (nonatomic, copy) void(^onItemSelect)(FSSpeechVoiceLanguageDataItem *item,UIViewController *presentViewController);

@end

@implementation FSSpeechVoiceLanguageDataItem
@end

@interface FSSpeechVoiceLanguageViewController : FSTableViewController
@property (nonatomic, strong) UIView *roundedRectangleView;
@end

@implementation FSSpeechVoiceLanguageViewController

- (void)initialize{
    [super initialize];
    self.shouldShowNavigationBar = YES;
    self.defaultEstimatedCellStyle = FSEstimatedTableViewCellStyleTitle;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    if ([NSObject fs_currentDeviceIsPhone]) [self arrangeRoundView];
    self.subviewTableViewLayoutMargins = UIEdgeInsetsMake([NSObject fs_currentDeviceIsPhone] ? 10 : 0, 0, 0, 0);
    self.roundedRectangleView.hidden = ![NSObject fs_currentDeviceIsPhone];
    self.tableView.backgroundColor = self.fs_backgroundColor_B9;
    self.shouldShowNavigationBar = ![NSObject fs_currentDeviceIsPhone];
    self.title = FSLocalizedForKey(@"kVoiceSelection");
    self.navBar.backgroundColor = self.fs_backgroundColor_B15;
    self.tableView.backgroundColor = self.fs_backgroundColor_B9;
    
    FSItem *closeItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kClose")];
    [closeItem setFont:self.fs_standardGeneralFont_GF3];
    [BindThemeKeySel(closeItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem*  _Nonnull target, SEL  _Nonnull sel) {
        [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
    }];
    @weakify(self)
    closeItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [self dismissViewControllerAnimated:YES completion:nil];
    };
    [self.navBar removeItemAtPosition:FSItemPositionLeft];
    [self.navBar addItem:closeItem spacing:16 atPosition:FSItemPositionLeft];
}

- (UIView *)roundedRectangleView{
    if (!_roundedRectangleView) {
        _roundedRectangleView = [[UIView alloc] init];
        _roundedRectangleView.backgroundColor = self.fs_backgroundColor_B3;
        _roundedRectangleView.layer.cornerRadius = 2;
        _roundedRectangleView.layer.masksToBounds = YES;
    }
    return _roundedRectangleView;
}

- (void)arrangeRoundView{
    [self.view addSubview:self.roundedRectangleView];
    [self.roundedRectangleView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(self.view);
        make.top.equalTo(self.view).offset(3);
        make.size.mas_equalTo(CGSizeMake(31, 4));
    }];
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (void)configureCell:(FSTitleEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(id)dataItem{
    if (!cell.rightView) {
        FSItem *infoItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"common_accessory_gray")];
        cell.rightView = infoItem;
    }
    cell.normalBackgroundColor = self.fs_backgroundColor_B15;
    cell.selectedBackgroundColor = self.fs_dividerColor_P1;
    cell.contentView.backgroundColor = self.fs_backgroundColor_B15;
    
    FSSpeechVoiceLanguageDataItem *dataItem1 = dataItem;
    cell.estimatedTitleLabel.text = dataItem1.languagelocal;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    FSSpeechVoiceLanguageDataItem *dataItem = [self dataForAtIndexPath:indexPath];
    if (dataItem.onItemSelect) dataItem.onItemSelect(dataItem,self);
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section
{
    return DEVICE_iPHONE ? 0 : 12;
}
@end

@interface FSSpeechVoiceViewController : FSTableViewController
@property (nonatomic, strong) UIView *roundedRectangleView;
@end

@implementation FSSpeechVoiceViewController

- (void)initialize{
    [super initialize];
    self.shouldShowNavigationBar = YES;
    self.defaultEstimatedCellStyle = FSEstimatedTableViewCellStyleTitle;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    if ([NSObject fs_currentDeviceIsPhone]) [self arrangeRoundView];
    self.subviewTableViewLayoutMargins = UIEdgeInsetsMake([NSObject fs_currentDeviceIsPhone] ? 10 : 0, 0, 0, 0);
    self.roundedRectangleView.hidden = ![NSObject fs_currentDeviceIsPhone];
    self.shouldShowNavigationBar = ![NSObject fs_currentDeviceIsPhone];
    self.title = FSLocalizedForKey(@"kVoiceSelection");
    self.navBar.backgroundColor = self.fs_backgroundColor_B15;
    self.tableView.backgroundColor = self.fs_backgroundColor_B9;
    
    FSItem *closeItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"common_back_gray")];
    @weakify(self)
    closeItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [self.navigationController popViewControllerAnimated:YES];
    };
    [self.navBar removeItemAtPosition:FSItemPositionLeft];
    [self.navBar addItem:closeItem spacing:4 atPosition:FSItemPositionLeft];
}

- (UIView *)roundedRectangleView{
    if (!_roundedRectangleView) {
        _roundedRectangleView = [[UIView alloc] init];
        _roundedRectangleView.backgroundColor = self.fs_backgroundColor_B3;
        _roundedRectangleView.layer.cornerRadius = 2;
        _roundedRectangleView.layer.masksToBounds = YES;
    }
    return _roundedRectangleView;
}

- (void)arrangeRoundView{
    [self.view addSubview:self.roundedRectangleView];
    [self.roundedRectangleView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(self.view);
        make.top.equalTo(self.view).offset(3);
        make.size.mas_equalTo(CGSizeMake(31, 4));
    }];
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (void)configureCell:(FSTitleEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(id)dataItem{
    if (!cell.rightView) {
        FSItem *infoItem = [FSItem createItemWithImage:FSImageNamedAndPrimaryColor(@"common_select_check")];
        cell.rightView = infoItem;
    }
    cell.normalBackgroundColor = self.fs_backgroundColor_B15;
    cell.selectedBackgroundColor = self.fs_dividerColor_P1;
    cell.contentView.backgroundColor = self.fs_backgroundColor_B15;
    
    FSSpeechVoiceDataItem *dataItem1 = dataItem;
    cell.rightView.hidden = !dataItem1.selected;
    cell.estimatedTitleLabel.text = [NSString stringWithFormat:@"%@-%@",dataItem1.languagelocal,dataItem1.name];
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
    for (FSSpeechVoiceDataItem *dataItem in self.dataSource) {
        dataItem.selected = NO;
    }
    FSSpeechVoiceDataItem *dataItem = [self dataForAtIndexPath:indexPath];
    dataItem.selected = YES;
    [self.tableView reloadData];
    if (dataItem.onItemSelect) dataItem.onItemSelect(dataItem);
}

- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section
{
    if ([NSObject fs_currentDeviceIsPhone]) {
        UIView *view = [UIView new];
        view.backgroundColor = self.fs_backgroundColor_B8;
        FSItem *closeItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"common_back_gray")];
        @weakify(self)
        closeItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            [self.navigationController popViewControllerAnimated:YES];
        };
        [view addSubview:closeItem];
        UILabel *title = [[UILabel alloc] init];
        title.font = self.fs_standardGeneralFont_GF1;
        title.textColor = self.fs_textColor_T4;
        title.text = self.sectionIndexTitles1[section];
        [view addSubview:title];
        [closeItem mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(view.mas_safeAreaLayoutGuideLeft).offset(4);
            make.centerY.mas_equalTo(view.mas_centerY);
        }];
        [title mas_makeConstraints:^(MASConstraintMaker *make) {
            make.centerY.mas_equalTo(view.mas_centerY);
            make.centerX.mas_equalTo(view.mas_centerX);
        }];
        return view;
    }else {
        UIView *view = [super tableView:tableView viewForHeaderInSection:section];
        view.backgroundColor = self.fs_backgroundColor_B9;
        return view;
    }
}

@end

@interface RateItemCell : UITableViewCell
@property (nonatomic, assign) BOOL itemOnselected;
@property (nonatomic, strong) UIImageView *selectImageItem;
@property (nonatomic, strong) FSItem *titleItem;
@end

@implementation RateItemCell

- (instancetype)initWithStyle:(UITableViewCellStyle)style reuseIdentifier:(NSString *)reuseIdentifier isSelect:(BOOL)isSelect
{
    if (self = [super initWithStyle:style reuseIdentifier:reuseIdentifier]) {
        
        self.backgroundColor = self.fs_backgroundColor_B15;
        self.itemOnselected = isSelect;
        self.textLabel.font = self.fs_standardGeneralFont_GF2;
        self.textLabel.textColor = self.fs_textColor_T4;
      
        self.selectImageItem = [[UIImageView alloc] init];
        self.selectImageItem.bounds = CGRectMake(0, 0, 40, 40);
        self.selectImageItem.image = FSImageNamedAndNormalColor(@"common_select_check");
        [BindThemeKeySel(self.selectImageItem, setImage:forState:) fs_updateThemeCompleted:^(UIImageView*  _Nonnull target, SEL  _Nonnull sel) {
            [target setImage:FSImageNamedAndPrimaryColor(@"common_select_check")];
        }];
        
        self.selectImageItem.layer.anchorPoint = CGPointMake(1.0, 0.5);
        [self.contentView addSubview:self.selectImageItem];
        self.selectImageItem.hidden = YES;
        self.selectImageItem.contentMode = UIViewContentModeScaleAspectFill;
        
        self.imageView.layer.anchorPoint = CGPointMake(0, 0.5);
        
        self.titleItem = [[FSItem alloc] init];
        self.titleItem.accessibilityTraits = UIAccessibilityTraitHeader;
        self.titleItem.customAccessibilityTraits = UIAccessibilityTraitHeader;
        [self.contentView addSubview:self.titleItem];
        [self.titleItem mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(16);
            make.centerY.equalTo(self.contentView);
        }];
        self.titleItem.titleLabel.font = self.fs_standardGeneralFont_GF2;
        [self.titleItem setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
        [BindThemeKeySel(self.titleItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem*  _Nonnull target, SEL  _Nonnull sel) {
            [target setTitleColor:target.fs_primaryColor forState:UIControlStateSelected];
        }];
        
    }
    return self;
}

- (void)setItemOnselected:(BOOL)itemOnselected
{
    _itemOnselected = itemOnselected;
    self.selectImageItem.hidden = !itemOnselected;
    self.titleItem.selected = itemOnselected;
}

- (void)layoutSubviews
{
    [super layoutSubviews];
    self.selectImageItem.center = CGPointMake( self.contentView.frame.size.width-10, self.frame.size.height * 0.5);
}

@end

typedef enum : NSUInteger {
    RateItemType0,
    RateItemType1,
    RateItemType2,
    RateItemType3,
    RateItemType4,
    RateItemType5,
} RateItemType;

@interface RateItemCtr : FSTableViewController
@property (nonatomic, copy) void (^ctrHidden)();
@property (nonatomic, copy) void (^onRateItemClick)(RateItemType type);
@property (nonatomic, assign) RateItemType curRateType;
@property (nonatomic, strong) UIView *roundedRectangleView;
@end

@implementation RateItemCtr

- (void)initialize{
    [super initialize];
    self.shouldShowNavigationBar = YES;
}

- (instancetype)init
{
    if (self = [super init]) {
        _curRateType = RateItemType2;
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];
    if ([NSObject fs_currentDeviceIsPhone]) [self arrangeRoundView];
    self.subviewTableViewLayoutMargins = UIEdgeInsetsMake([NSObject fs_currentDeviceIsPhone] ? 10 : 0, 0, 0, 0);
    self.roundedRectangleView.hidden = ![NSObject fs_currentDeviceIsPhone];
    self.shouldShowNavigationBar = ![NSObject fs_currentDeviceIsPhone];
    self.title = FSLocalizedForKey(@"kSpeechRate");
    self.navBar.backgroundColor = self.fs_backgroundColor_B15;
    self.tableView.backgroundColor = self.fs_backgroundColor_B9;
    
    FSItem *closeItem = [FSItem createItemWithTitle:FSLocalizedForKey(@"kClose")];
    [closeItem setFont:self.fs_standardGeneralFont_GF3];
    [BindThemeKeySel(closeItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem*  _Nonnull target, SEL  _Nonnull sel) {
        [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
    }];
    @weakify(self)
    closeItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [self dismissViewControllerAnimated:YES completion:nil];
    };
    [self.navBar removeItemAtPosition:FSItemPositionLeft];
    [self.navBar addItem:closeItem spacing:16 atPosition:FSItemPositionLeft];
}

- (BOOL)emptyDataSetShouldDisplay:(UITableView *)tableView
{
    return NO;
}

- (UIView *)roundedRectangleView{
    if (!_roundedRectangleView) {
        _roundedRectangleView = [[UIView alloc] init];
        _roundedRectangleView.backgroundColor = self.fs_backgroundColor_B3;
        _roundedRectangleView.layer.cornerRadius = 2;
        _roundedRectangleView.layer.masksToBounds = YES;
    }
    return _roundedRectangleView;
}

- (void)arrangeRoundView{
    [self.view addSubview:self.roundedRectangleView];
    [self.roundedRectangleView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(self.view);
        make.top.equalTo(self.view).offset(3);
        make.size.mas_equalTo(CGSizeMake(31, 4));
    }];
}

#pragma mark - Table view data source

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section
{
    return DEVICE_iPHONE ? 0 : 12;
}

//- (UIView*)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section
//{
//    UIView *view = [[UIView alloc] init];
//    view.backgroundColor = self.fs_backgroundColor_B1;
//    
//    UILabel *title = [[UILabel alloc] init];
//    title.textColor = self.fs_textColor_T2;
//    title.font = self.fs_standardWeakFont_WF1;
//    title.backgroundColor = self.fs_backgroundColor_B1;
//    title.text = FSLocalizedForKey(@"kSpeechRateTitle");
//    view.bounds = CGRectMake(0, 0, tableView.frame.size.width, 44);
//    [view addSubview:title];
//    [title mas_remakeConstraints:^(MASConstraintMaker *make) {
//        make.left.equalTo(title.superview.mas_left).offset(16);
//        make.top.equalTo(title.superview.mas_top).offset(15);
//    }];
//    if (DEVICE_iPHONE || [self fs_currentSceneiPadHorizontalCompact]) {
//        UILabel* dragLabel = [[UILabel alloc] init];
//        [dragLabel setBackgroundColor:self.fs_backgroundColor_B3];
//        [view addSubview:dragLabel];
//        [dragLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
//            make.centerX.equalTo(dragLabel.superview.mas_centerX);
//            make.top.mas_equalTo(3);
//            make.size.mas_equalTo(CGSizeMake(31, 4));
//        }];
//        dragLabel.layer.cornerRadius = 3;
//        dragLabel.layer.masksToBounds = YES;
//    }
//    return view;
//}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return 6;
}


- (RateItemCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *cellIdentifier = @"cell";
    RateItemCell *cell = [[RateItemCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellIdentifier isSelect:NO];
    cell.isAccessibilityElement = YES;
    NSString *title = @"";
    RateItemType type = (RateItemType)indexPath.row;
    switch (type) {
        case RateItemType0:
            title = @"0.5X";
            break;
        case RateItemType1:
            title = @"0.75X";
            break;
        case RateItemType2:
            title = @"1X";
            break;
        case RateItemType3:
            title = @"1.25X";
            break;
        case RateItemType4:
            title = @"1.5X";
            break;
        case RateItemType5:
            title = @"2X";
            break;
        default:
            title = @"1X";
            break;
    }
    [cell.titleItem setTitle:title forState:UIControlStateNormal];
    NSString *accessbilityLabel = [title stringByReplacingOccurrencesOfString:@"X" withString:@""];
    accessbilityLabel = [accessbilityLabel stringByAppendingFormat:@" %@",FSLocalizedForKey(@"kTimesVoiceover")];
    cell.accessibilityLabel = accessbilityLabel;
    if (type == _curRateType) {
        cell.itemOnselected = YES;
        cell.accessibilityTraits = UIAccessibilityTraitSelected;
    }else{
        cell.accessibilityTraits = UIAccessibilityTraitNone;
    }
    cell.selectionStyle = UITableViewCellSelectionStyleDefault;
    UIView *backgroundView = [[UIView alloc] initWithFrame:cell.contentView.frame];
    backgroundView.backgroundColor = self.fs_dividerColor_P1;
    cell.selectedBackgroundView = backgroundView;
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    RateItemType type = (RateItemType)indexPath.row;
    if (self.onRateItemClick) {
        _curRateType = type;
        self.onRateItemClick(type);
        [tableView reloadData];
    }
}

- (void)dismissViewControllerAnimated:(BOOL)flag completion:(void (^)(void))completion
{
    if (self.ctrHidden) {
        self.ctrHidden();
    }
    [super dismissViewControllerAnimated:flag completion:completion];
}

@end

typedef enum
{
	READING_STATUS_NONE = 0,
	READING_STATUS_PAGE,
	READING_STATUS_SELECTED,
    READING_STATUS_TEXT
} READING_STATUS;

@interface SpeechModule () <AVSpeechSynthesizerDelegate, IDocEventListener, IStateChangeListener, IDrawEventListener, UIPopoverPresentationControllerDelegate, IGestureEventListener, IFSViewSettingEventListener,IFullScreenListener,IPageEventListener,IThumbnailEventListener,IPanelChangedListener,IWindowFrameChangedListener>

#define MIN_READING_SPEED 0.05
#define MAX_READING_SPEED 1.0
#define DEFAULT_READING_SPEED 0.40
#define CHANGE_READING_SPEED 0.05

@property (strong, nonatomic) AVSpeechSynthesizer *speechSynthesizer;
@property (copy, nonatomic) NSString *readingContent;
@property (copy, nonatomic) NSArray *arrayBounds;
@property (assign, nonatomic) int boundIndex;
@property (assign, nonatomic) READING_STATUS currentStatus;
@property (assign, nonatomic) int currentPageindex;
@property (assign, nonatomic) int currentReadingIndex;
@property (assign, nonatomic) int readingOffset;
@property (assign, nonatomic) int readStartCharPos;
@property (assign, nonatomic) int readEndCharPos;

@property (strong, nonatomic)  FSToolbar *toolbarBottom;

@property (strong, nonatomic)  UIButton *buttonPreviousPage;
@property (strong, nonatomic)  UIButton *buttonPlay;
@property (strong, nonatomic)  UIButton *buttonNextPage;
@property (strong, nonatomic)  UIButton *buttonCycle;
@property (strong, nonatomic)  UIButton *buttonRate;

//@property (nonatomic, strong, nullable) FSMainToolbar *topToolBar;
@property (nonatomic, strong, nullable) FSToolbar *topToolBar;
@property (nonatomic, assign) BOOL isInitedData;
@property (nonatomic, assign) BOOL isInitedBarViewData;

@property (nonatomic, assign) FSUIManagerState oldState;
@property (nonatomic, assign) BOOL isSpeechState;
@property (nonatomic, strong) UIView *toolbarBottomSpeech;

@property (nonatomic, weak) FSPDFViewCtrl *pdfViewCtrl;
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
@property (nonatomic, strong) UIPanGestureRecognizer *pan;
@property (nonatomic, strong) FSItem* exitItem;

@property (nonatomic, copy) NSString* selectVoiceIdentifier;
@property (nonatomic, copy) NSString* matchedVoiceIdentifier;
@end
@implementation SpeechModule
@synthesize selectVoiceIdentifier = _selectVoiceIdentifier;
static NSString *speechSpeed = @"speechSpeed";
static NSString *speechAutoNextPage = @"speechAutoNextPage";

- (instancetype)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager {
    if (self = [super init]) {
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _extensionsManager = extensionsManager;
        _isSpeechState = NO;
        _oldState = [_extensionsManager getState];
        [self loadModule];
    }
    return self;
}

- (FSToolbar *)topToolBar{
    if (!_topToolBar) {
        _topToolBar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleTop];
        _topToolBar.tag = TOPBAR_PAGESLIDER_TAG;
        _topToolBar.hidden = YES;
        StateBarThemeKeyPath(_topToolBar.stateBar, backgroundColor);
        [BindThemeKeySel(_topToolBar.stateBar, backgroundView) fs_updateThemeCompleted:^(UIView*  _Nonnull target, SEL  _Nonnull sel) {
            StateBarThemeKeyPath(target, backgroundColor);
        }];
        [_extensionsManager.containerView addSubview:_topToolBar];
        
        [_topToolBar mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.right.top.mas_equalTo(self.pdfViewCtrl);
        }];
        
        UIView *viewTopSpeech = [UIView new];
        [_topToolBar addSubview:viewTopSpeech];
        [viewTopSpeech mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.right.top.bottom.mas_equalTo(viewTopSpeech.superview);
        }];
        
        UILabel  *labelSpeech = [[UILabel alloc] init];
        labelSpeech.accessibilityTraits = UIAccessibilityTraitHeader;
        labelSpeech.text = FSLocalizedForKey(@"kSpeak");
        labelSpeech.font = self.fs_standardGeneralFont_GF1;
        labelSpeech.textColor = self.fs_textColor_T4;
        [viewTopSpeech addSubview:labelSpeech];
        [labelSpeech mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.center.mas_equalTo(labelSpeech.superview);
        }];
        
        UIButton *buttonCancel = [[UIButton alloc] init];

        [buttonCancel setTitle:FSLocalizedForKey(@"kClose") forState:UIControlStateNormal];
        buttonCancel.titleLabel.font = self.fs_standardGeneralFont_GF2;
        [BindThemeKeySel(buttonCancel, setTitleColor:forState:) fs_updateThemeCompleted:^(UIButton*  _Nonnull target, SEL  _Nonnull sel) {
            [target setTitleColor:target.fs_primaryColor forState:UIControlStateNormal];
            [target setTitleColor:[target.fs_primaryColor colorWithAlphaComponent:0.5] forState:UIControlStateHighlighted];
        }];
        [buttonCancel addTarget:self action:@selector(buttonCancelClick) forControlEvents:UIControlEventTouchUpInside];

        [viewTopSpeech addSubview:buttonCancel];
        [buttonCancel mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(buttonCancel.superview.mas_left).offset(12);
            make.centerY.mas_equalTo(labelSpeech);
            make.height.mas_equalTo(40);
        }];
    }
    return _topToolBar;
}


- (FSToolbar *)toolbarBottom{
    if (!_toolbarBottom) {
        _toolbarBottom = [[FSToolbar alloc] initWithStyle:FSToolbarStyleBottom];
        _toolbarBottom.hidden = YES;
        _toolbarBottom.hasDivide = NO;
        [_extensionsManager.containerView addSubview:_toolbarBottom];
        //estimate
        _toolbarBottom.frame = CGRectMake(0, 0, _extensionsManager.containerView.fs_width, 44);
        _toolbarBottom.fs_bottom = _extensionsManager.containerView.fs_height;
        
        UIView* toolbarBottomSpeech = [UIView new];
        _toolbarBottomSpeech = toolbarBottomSpeech;
        [_toolbarBottom addSubview:toolbarBottomSpeech];
        [toolbarBottomSpeech mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.top.bottom.centerX.mas_equalTo(toolbarBottomSpeech.superview);
        }];
        
        self.buttonPreviousPage = [[UIButton alloc] init];
        self.buttonPreviousPage.accessibilityLabel = FSLocalizedForKey(@"kPreReadingVoiceOver");
        [toolbarBottomSpeech addSubview:_buttonPreviousPage];
        [_buttonPreviousPage setImage:FSImageNamedAndNormalColor(@"speech_previous") forState:UIControlStateNormal];
        [_buttonPreviousPage setImage:[Utility unableDynamicImage:FSImageNamedAndNormalColor(@"speech_previous")] forState:UIControlStateHighlighted];
        [_buttonPreviousPage addTarget:self action:@selector(barButtonPreviousPageClicked:) forControlEvents:UIControlEventTouchUpInside];

        self.buttonNextPage = [[UIButton alloc] init];
        self.buttonNextPage.accessibilityLabel = FSLocalizedForKey(@"kNextPageBtnVoiceOver");
        [toolbarBottomSpeech addSubview:_buttonNextPage];
        [_buttonNextPage setImage:FSImageNamedAndNormalColor(@"speech_next") forState:UIControlStateNormal];
        [_buttonNextPage setImage:[Utility unableDynamicImage:FSImageNamedAndNormalColor(@"speech_next")] forState:UIControlStateHighlighted];
        [_buttonNextPage addTarget:self action:@selector(barButtonNextPageClicked:) forControlEvents:UIControlEventTouchUpInside];

        self.buttonPlay = [[UIButton alloc] init];
        self.buttonPlay.accessibilityLabel = FSLocalizedForKey(@"kPlayBtnContinueVoiceOver");
        [toolbarBottomSpeech addSubview:_buttonPlay];
        [_buttonPlay setImage:FSImageNamedAndNormalColor(@"speech_pause") forState:UIControlStateNormal];
        [_buttonPlay setImage:[Utility unableDynamicImage:FSImageNamedAndNormalColor(@"speech_pause")] forState:UIControlStateHighlighted];
        [_buttonPlay addTarget:self action:@selector(barButtonPlayClicked:) forControlEvents:UIControlEventTouchUpInside];

        self.buttonCycle = [[UIButton alloc] init];
        self.buttonCycle.accessibilityLabel = FSLocalizedForKey(@"kRepeatbtnVoiceOver");
        [toolbarBottomSpeech addSubview:_buttonCycle];
        [self.buttonCycle setImage:FSImageNamedAndNormalColor(@"speech_circulation") forState:UIControlStateNormal];
        [self.buttonCycle setBackgroundImage:[Utility commonDynamicBackgroundImage] forState:UIControlStateSelected];
        [_buttonCycle addTarget:self action:@selector(buttonCycleClicked:) forControlEvents:UIControlEventTouchUpInside];
        self.buttonCycle.selected = !_autoNextPage;
        
        self.buttonRate = [[UIButton alloc] init];
        self.buttonRate.accessibilityIdentifier = @"moreSetting";
        [toolbarBottomSpeech addSubview:_buttonRate];
        [self.buttonRate setImage:FSImageNamedAndNormalColor(@"speech_moreSetting") forState:UIControlStateNormal];
        [self.buttonRate setBackgroundImage:[Utility commonDynamicBackgroundImage] forState:UIControlStateSelected];
//        self.buttonRate.titleLabel.font =  self.fs_standardImportantFont_IF3;
//        NSString *speedStr = [self translaterRateItemTypeToStr:[self translaterFloatSpeedToRateType:_readingSpeed]];
//        [self.buttonRate setTitle:speedStr forState:UIControlStateNormal];
//        [self.buttonRate setTitleColor:self.fs_iconColor_I3 forState:UIControlStateNormal];
//        [self.buttonRate setTitleColor:[self.fs_iconColor_I3 colorWithAlphaComponent:0.5] forState:UIControlStateHighlighted];
//        NSString *accessbilityLabel = [speedStr stringByReplacingOccurrencesOfString:@"X" withString:@""];
//        accessbilityLabel = [accessbilityLabel stringByAppendingFormat:@" %@",FSLocalizedForKey(@"kTimesVoiceover")];
        self.buttonRate.accessibilityLabel = @"more Setting";
        
        [_buttonRate addTarget:self action:@selector(buttonRateClicked:) forControlEvents:UIControlEventTouchUpInside];

        [_buttonCycle mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(_buttonCycle.superview);
            make.centerY.mas_equalTo(toolbarBottomSpeech);
            make.width.height.mas_equalTo(40);
        }];
        
        [_buttonPreviousPage mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(_buttonCycle.mas_right).offset(ITEM_MARGIN_NORMAL);
            make.centerY.mas_equalTo(toolbarBottomSpeech);
            make.width.height.mas_equalTo(40);
        }];
        
        [_buttonPlay mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(_buttonPreviousPage.mas_right).offset(ITEM_MARGIN_NORMAL);
            make.centerY.mas_equalTo(toolbarBottomSpeech);
            make.width.height.mas_equalTo(40);
        }];
        
        [_buttonNextPage mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(_buttonPlay.mas_right).offset(ITEM_MARGIN_NORMAL);
            make.centerY.mas_equalTo(toolbarBottomSpeech);
            make.width.height.mas_equalTo(40);
        }];
        [self reLayoutBottomToolbar];
    }
    return _toolbarBottom;
}


#pragma mark -- IFullScreenListener
- (void)onFullScreen:(BOOL)isFullScreen {
    if ([_extensionsManager getState] == FSUIManagerStateSpeech && !(isFullScreen && _extensionsManager.currentToolHandler != nil)) {
        [self showSpeechBar:!isFullScreen animation:YES];
    }
}

- (void)speak
{  
    [self showSpeechBar:YES animation:YES];
    [self refreshInterface];
    
    if (UIAccessibilityIsVoiceOverRunning()) {
        [self barButtonPlayClicked:self.buttonPlay];
    }
}

- (void)showSpeechBar:(BOOL)isShow animation:(BOOL)animation{
    
    if (self.toolbarBottom.hidden != isShow) {
        return;
    }
	
    if (!isShow) {
        if (![self checkIsIphoneDevice]) {
            [_extensionsManager setHiddenTopToolbar:YES];
        }
        self.topToolBar.hidden = YES;
        self.toolbarBottom.hidden  = YES;
    } else {
        if (![self checkIsIphoneDevice]) {
            self.topToolBar.hidden = YES;
            [_extensionsManager setHiddenTopToolbar:NO];
            [self.toolbarBottom fs_setShadowColor:self.fs_dividerColor_P3];
        }
        else
        {
            [_extensionsManager setHiddenTopToolbar:YES];
            [_extensionsManager setHiddenBottomToolbar:YES];
            self.topToolBar.hidden = NO;
            [self.toolbarBottom fs_setShadowColor:nil];
        }
        self.toolbarBottom.hidden  = NO;
        [self.toolbarBottom.superview bringSubviewToFront:self.toolbarBottom];
        BOOL hadShowSetting = [Preference getBoolValue:FSPreferenceModuleSpeech type:@"hadShowSetting" delaultValue:NO];
        if (!hadShowSetting) {
            [self buttonRateClicked:self.buttonRate];
        }
    }
    
	if (animation)
	{
		[Utility addAnimation:self.topToolBar.layer type:self.topToolBar.hidden?kCATransitionReveal:kCATransitionMoveIn subType:self.topToolBar.hidden?kCATransitionFromTop: kCATransitionFromBottom timeFunction:kCAMediaTimingFunctionEaseInEaseOut duration:0.3];
		[Utility addAnimation:self.toolbarBottom.layer type:self.toolbarBottom.hidden?kCATransitionReveal:kCATransitionMoveIn subType:self.toolbarBottom.hidden?kCATransitionFromBottom: kCATransitionFromTop timeFunction:kCAMediaTimingFunctionEaseInEaseOut duration:0.3];
	}
    [(PageNavigationModule*)[_extensionsManager getModuleByName:@"PageNavigation"] showPageNum: isShow];
    [_extensionsManager setStatusBarHidden:_extensionsManager.currentIsFullScreen && _topToolBar.hidden];
    [_extensionsManager.pdfViewCtrl.fs_viewController setNeedsStatusBarAppearanceUpdate];
    [_extensionsManager.settingBar.viewSettingManager resetPdfViewCtrlSafeArea];
    
    [(PageNavigationModule *)[_extensionsManager getModuleByName:@"PageNavigation"] layoutPdfViewContentInsets:NO];
}

	
- (void)dealloc {
	[[NSNotificationCenter defaultCenter] removeObserver:self name:UIApplicationDidEnterBackgroundNotification object:nil];
	[[UIApplication sharedApplication] endReceivingRemoteControlEvents];
}

#pragma mark buttonClick methods
- (void)buttonCancelClick {
    [self.topToolBar setHidden:YES];
    [_extensionsManager changeState:_oldState];
    [_extensionsManager setFullScreen:NO ];
    [self removeRemoteControlEventHandler];
}

- (void)barButtonPreviousPageClicked:(id)sender {
    if ([self.extensionsManager canAutoHideTopBottomToolBar]) {
        [self.extensionsManager delayHideTopBottomToolBar];
    }
	if (_currentStatus == READING_STATUS_PAGE && (_currentPageindex-1) >= 0)
    {
        [self stopOnly];
        _currentPageindex--;
        [_pdfViewCtrl gotoPage:_currentPageindex animated:YES];
        self.currentStatus = READING_STATUS_PAGE;
        if (![self speakContentWithIndex:_currentPageindex]) {
            [self speakStop];
            return;
        }
    }
}

- (void)barButtonPlayClicked:(id)sender {
    if ([self.extensionsManager canAutoHideTopBottomToolBar]) {
        [self.extensionsManager delayHideTopBottomToolBar];
    }
	if (_currentStatus == READING_STATUS_NONE)
	{
//		Reading current page
		[self refreshInterface];
        self.currentStatus = READING_STATUS_PAGE;
		[self speakContentWithIndex:_currentPageindex];
        [_pdfViewCtrl gotoPage:_currentPageindex animated:YES];
	}
	else if (_currentStatus == READING_STATUS_PAGE || _currentStatus == READING_STATUS_SELECTED || _currentStatus == READING_STATUS_TEXT)
	{
		if (self.speechSynthesizer.paused)
		{
			[self.speechSynthesizer continueSpeaking];
		}
		else
		{
			[self.speechSynthesizer pauseSpeakingAtBoundary:AVSpeechBoundaryImmediate];
		}
	}
}

- (void)speakStop {
    if ([self.extensionsManager canAutoHideTopBottomToolBar]) {
        [self.extensionsManager delayHideTopBottomToolBar];
    }
	[self stop];
}

- (void)barButtonNextPageClicked:(id)sender {
    if ([self.extensionsManager canAutoHideTopBottomToolBar]) {
        [self.extensionsManager delayHideTopBottomToolBar];
    }
    dispatch_async(dispatch_get_main_queue(), ^{
        if (self.currentStatus == READING_STATUS_PAGE &&  (self.currentPageindex+1) < [[self.pdfViewCtrl currentDoc] getPageCount]){
            
            [self stopOnly];
            self.currentPageindex++;
            [self.pdfViewCtrl gotoPage:self.currentPageindex animated:YES];
            self.currentStatus = READING_STATUS_PAGE;
            if (![self speakContentWithIndex:self.currentPageindex]) {
                [self speakStop];
                return;
            }
        }
    });
}

- (void)buttonCycleClicked:(UIButton *)sender {
    if ([self.extensionsManager canAutoHideTopBottomToolBar]) {
        [self.extensionsManager delayHideTopBottomToolBar];
    }
    sender.selected = !sender.selected;
    _autoNextPage = !sender.selected;
}

- (void)buttonRateClicked:(id)sender {
    if ([self.extensionsManager canAutoHideTopBottomToolBar]) {
        [self.extensionsManager delayHideTopBottomToolBar];
    }
    //[self presentRateItemCtr];
    [self presentMoreSettingCtr];
}

- (NSArray *)moreSettingDataSource
{
    @weakify(self)
    NSMutableArray *dataArray = [NSMutableArray array];
    AVSpeechSynthesisVoice *voice = [AVSpeechSynthesisVoice voiceWithIdentifier:self.selectVoiceIdentifier];
    
    
    FSSpeechMoreSettingDataItem *item1 = [FSSpeechMoreSettingDataItem new];
    item1.imageName = @"speech_voice";
    item1.title = FSLocalizedForKey(@"kVoiceSelection");
    if (voice) item1.settingValue = [[NSLocale localeWithLocaleIdentifier:[NSBundle mainBundle].preferredLocalizations.firstObject] localizedStringForLocaleIdentifier:voice.language];
    item1.onItemSelect = ^{
        @strongify(self)
        [self presentVoiceLanguageSelectCtr];
    };
    FSSpeechMoreSettingDataItem *item2 = [FSSpeechMoreSettingDataItem new];
    item2.imageName = @"speech_rate";
    item2.title = FSLocalizedForKey(@"kSpeechRate");
    item2.settingValue = [self translaterRateItemTypeToStr:[self translaterFloatSpeedToRateType:self.readingSpeed]];
    item2.onItemSelect = ^{
        @strongify(self)
        [self presentRateItemCtr];
    };
    [dataArray addObject:item1];
    [dataArray addObject:item2];
    return [NSArray arrayWithArray:dataArray];
}

- (void)presentMoreSettingCtr
{
    [self.extensionsManager cancelAutoFullScreen];
    if ([self checkIsIphoneDevice] ) {
        CGFloat height = 118;
        FSSpeechMoreSettingViewController *ctr = [[FSSpeechMoreSettingViewController alloc] init];
        ctr.dataSource = self.moreSettingDataSource;
        @fs_weakify(self)
        [ctr fs_makeTransitionAnimator:^(FSPresentedViewControllerAnimator * _Nonnull animator) {
            @fs_strongify(self)
            animator.size = CGSizeMake(MIN(self.pdfViewCtrl.fs_width, self.pdfViewCtrl.fs_height), height);
            animator.presentationStyle = FSModalPresentationBottom;
            animator.maxVerticalHeigh = height;
        }];
        [self.pdfViewCtrl.fs_viewController presentViewController:ctr animated:YES completion:nil];
    }else{
        FSSpeechMoreSettingViewController *ctr = [[FSSpeechMoreSettingViewController alloc] init];
        ctr.dataSource = self.moreSettingDataSource;
        ctr.modalPresentationStyle = UIModalPresentationPopover;
        ctr.preferredContentSize = CGSizeMake(316, 118);
        UIPopoverPresentationController * popover = [ctr popoverPresentationController];
        popover.permittedArrowDirections = UIPopoverArrowDirectionDown;
        popover.backgroundColor = self.fs_backgroundColor_B1;
        popover.delegate = self;
        popover.sourceView = self.buttonRate;
        popover.sourceRect = self.buttonRate.bounds;
        [self.pdfViewCtrl.fs_viewController presentViewController:ctr animated:YES completion:nil];
    }
}

- (NSString *)selectVoiceIdentifier
{
    if (!_selectVoiceIdentifier) {
        NSString *voiceIdentifier = [Preference getStringValue:FSPreferenceModuleSpeech type:@"voiceIdentifier" delaultValue:nil];
        
        NSString *languageCode = [[NSLocale autoupdatingCurrentLocale] objectForKey:NSLocaleLanguageCode];
        NSString *identifier = [[NSLocale autoupdatingCurrentLocale] objectForKey:NSLocaleIdentifier];
        NSArray *allVoices = [AVSpeechSynthesisVoice speechVoices];
        __block BOOL find = NO;
        __block NSString *matchedLangIdentifier = nil;
        __block NSString *bestIdentifier = nil;
        [allVoices enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
            AVSpeechSynthesisVoice *voiceItem = obj;
            if ([voiceIdentifier isEqualToString:voiceItem.identifier]) {
                find = YES;
            }
            if (!matchedLangIdentifier && [[voiceItem.language lowercaseString] hasPrefix:[languageCode lowercaseString]]) {
                matchedLangIdentifier = voiceItem.identifier;
            }
            if (!bestIdentifier && [[voiceItem.language lowercaseString] isEqualToString:[identifier lowercaseString]]) {
                bestIdentifier = voiceItem.identifier;
            }
        }];
        if (!bestIdentifier) bestIdentifier = matchedLangIdentifier;
        if (!find) {
            [Preference setStringValue:FSPreferenceModuleSpeech type:@"voiceIdentifier" value:bestIdentifier];
            voiceIdentifier = bestIdentifier;
        }
        _selectVoiceIdentifier = voiceIdentifier;
    }
    return _selectVoiceIdentifier;
}

- (void)setSelectVoiceIdentifier:(NSString *)currentVoiceIdentifier
{
    _selectVoiceIdentifier = currentVoiceIdentifier;
    [Preference setStringValue:FSPreferenceModuleSpeech type:@"voiceIdentifier" value:currentVoiceIdentifier];
}

- (NSArray *)voiceDataSource
{
    @weakify(self)
    NSMutableArray *dataArray = [NSMutableArray array];
    NSArray *allVoices = [AVSpeechSynthesisVoice speechVoices];
    [allVoices enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        @strongify(self)
        AVSpeechSynthesisVoice *voiceItem = obj;
        FSSpeechVoiceLanguageDataItem *languageitem;
        for (FSSpeechVoiceLanguageDataItem *tmp in dataArray) {
            if ([[[tmp.language componentsSeparatedByString:@"-"] firstObject] isEqualToString:[[voiceItem.language componentsSeparatedByString:@"-"] firstObject]]) {
                languageitem = tmp;
                break;
            }
        }
        
        if (!languageitem) {
            languageitem = [FSSpeechVoiceLanguageDataItem new];
            languageitem.language = voiceItem.language;
            languageitem.languagelocal = [[NSLocale localeWithLocaleIdentifier:[NSBundle mainBundle].preferredLocalizations.firstObject] localizedStringForLanguageCode:voiceItem.language];
            languageitem.dataItems = [NSMutableArray array];
            languageitem.onItemSelect = ^(FSSpeechVoiceLanguageDataItem *item, UIViewController *presentViewController) {
                @strongify(self)
                [self presentVoiceSelectCtr:item.dataItems navigationController:presentViewController.navigationController];
            };
            [dataArray addObject:languageitem];
        }
        
        FSSpeechVoiceDataItem *item = [FSSpeechVoiceDataItem new];
        item.identifier = voiceItem.identifier;
        item.language = voiceItem.language;
        item.languagelocal = [[NSLocale localeWithLocaleIdentifier:[NSBundle mainBundle].preferredLocalizations.firstObject] localizedStringForLocaleIdentifier:voiceItem.language];
        item.name = voiceItem.name;
        item.onItemSelect = ^(FSSpeechVoiceDataItem *item) {
            @strongify(self)
            self.selectVoiceIdentifier = item.identifier;
            [self speakAfterSpeedChange];
        };
        item.selected = [item.identifier isEqualToString:self.selectVoiceIdentifier];
        [languageitem.dataItems addObject:item];
    }];
    [dataArray sortUsingComparator:^NSComparisonResult(FSSpeechVoiceLanguageDataItem *obj1, FSSpeechVoiceLanguageDataItem *obj2) {
        return [obj1.languagelocal localizedCompare:obj2.languagelocal];
    }];
    return [NSArray arrayWithArray:dataArray];
}

- (void)presentVoiceSelectCtr:(NSMutableArray *)dataSourece navigationController:(UINavigationController *)navigationController
{
    [self.extensionsManager cancelAutoFullScreen];
    [dataSourece sortUsingComparator:^NSComparisonResult(FSSpeechVoiceDataItem *obj1, FSSpeechVoiceDataItem *obj2) {
        return [obj1.languagelocal localizedCompare:obj2.languagelocal];
    }];
    for (FSSpeechVoiceDataItem *item in dataSourece) {
        item.selected = [item.identifier isEqualToString:self.selectVoiceIdentifier];
    }
    FSSpeechVoiceViewController *ctr = [[FSSpeechVoiceViewController alloc] init];
    ctr.dataSource = dataSourece;
    FSSpeechVoiceDataItem *item = [dataSourece firstObject];
    if (item) {
        ctr.sectionStyle = FSTableViewSectionStyleTitle;
        ctr.sectionIndexTitles1 = @[[[NSLocale localeWithLocaleIdentifier:[NSBundle mainBundle].preferredLocalizations.firstObject] localizedStringForLanguageCode:item.language]];
        ctr.sectionHeaderHeight = DEVICE_iPHONE ? 40 : 32;
    }
    
    [navigationController pushViewController:ctr animated:YES];
}

- (void)presentVoiceLanguageSelectCtr
{
    [self.extensionsManager cancelAutoFullScreen];
    if (DEVICE_iPHONE) {
        CGFloat height = 308;
        FSSpeechVoiceLanguageViewController *ctr = [[FSSpeechVoiceLanguageViewController alloc] init];
        FSNavigationController *navCtr = [[FSNavigationController alloc] initWithRootViewController:ctr];
        ctr.dataSource = self.voiceDataSource;
        @fs_weakify(self)
        [navCtr fs_makeTransitionAnimator:^(FSPresentedViewControllerAnimator * _Nonnull animator) {
            @fs_strongify(self)
            animator.size = CGSizeMake(MIN(self.pdfViewCtrl.fs_width, self.pdfViewCtrl.fs_height), height);
            animator.presentationStyle = FSModalPresentationBottom;
            animator.maxVerticalHeigh = height;
        }];
        [self.pdfViewCtrl.fs_viewController presentViewController:navCtr animated:YES completion:nil];
    }else{
        FSSpeechVoiceLanguageViewController *ctr = [[FSSpeechVoiceLanguageViewController alloc] init];
        FSNavigationController *navCtr = [[FSNavigationController alloc] initWithRootViewController:ctr];
        ctr.dataSource = self.voiceDataSource;
        navCtr.modalPresentationStyle = UIModalPresentationFormSheet;
        [self.pdfViewCtrl.fs_viewController presentViewController:navCtr animated:YES completion:nil];
    }
}

- (void)presentRateItemCtr
{
    [self.extensionsManager cancelAutoFullScreen];
    if (DEVICE_iPHONE) {
        CGFloat height = 308;
        RateItemCtr *ctr = [[RateItemCtr alloc] init];
        ctr.curRateType = [self translaterFloatSpeedToRateType:_readingSpeed];
        ctr.onRateItemClick = ^(RateItemType type) {
            [self resetReadRate:type];
        };
        @fs_weakify(self)
        ctr.ctrHidden = ^{
            @fs_strongify(self)
            [self.extensionsManager delayAutoFullScreen];
        };
        [ctr fs_makeTransitionAnimator:^(FSPresentedViewControllerAnimator * _Nonnull animator) {
            @fs_strongify(self)
            animator.size = CGSizeMake(MIN(self.pdfViewCtrl.fs_width, self.pdfViewCtrl.fs_height), height);
            animator.presentationStyle = FSModalPresentationBottom;
            animator.maxVerticalHeigh = height;
        }];
        [self.pdfViewCtrl.fs_viewController presentViewController:ctr animated:YES completion:nil];
    }else{
        RateItemCtr *ctr = [[RateItemCtr alloc] init];
        ctr.curRateType = [self translaterFloatSpeedToRateType:_readingSpeed];
        ctr.onRateItemClick = ^(RateItemType type) {
            [self resetReadRate:type];
        };
        @weakify(self)
        ctr.ctrHidden = ^{
            @strongify(self)
            [self.extensionsManager delayAutoFullScreen];
        };
        ctr.modalPresentationStyle = UIModalPresentationFormSheet;
        ctr.presentationController.delegate = self;
//        ctr.preferredContentSize = CGSizeMake(316, 306);
//        UIPopoverPresentationController * popover = [ctr popoverPresentationController];
//        popover.permittedArrowDirections = UIPopoverArrowDirectionDown;
//        popover.backgroundColor = self.fs_backgroundColor_B1;
//        popover.delegate = self;
//        popover.sourceView = self.buttonRate;
//        popover.sourceRect = self.buttonRate.bounds;
        [self.pdfViewCtrl.fs_viewController presentViewController:ctr animated:YES completion:nil];
    }
}

#pragma mark -UIAdaptivePresentationControllerDelegate
- (UIModalPresentationStyle)adaptivePresentationStyleForPresentationController:(UIPresentationController *)controller{
    if ([controller isKindOfClass:[UIPopoverPresentationController class]]) return UIModalPresentationPopover;
    if (controller.presentedViewController.traitCollection.horizontalSizeClass == UIUserInterfaceSizeClassRegular && [[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad) {
        return UIModalPresentationFormSheet;
    }else{
        return UIModalPresentationFullScreen;
    }
}

-(UIModalPresentationStyle)adaptivePresentationStyleForPresentationController:(UIPresentationController *)controller traitCollection:(nonnull UITraitCollection *)traitCollection
{
    if ([controller isKindOfClass:[UIPopoverPresentationController class]]) return UIModalPresentationPopover;
    if (traitCollection.horizontalSizeClass == UIUserInterfaceSizeClassRegular && [[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPad) {
        return UIModalPresentationFormSheet;
    }else{
        return UIModalPresentationFullScreen;
    }
}

- (void)resetReadRate:(RateItemType)type
{
    switch (type) {
        case RateItemType0:
        {
            self.readingSpeed = 0.2;
        }
            break;
        case RateItemType1:
        {
            self.readingSpeed = 0.3;
        }
            break;
        case RateItemType2:
        {
            self.readingSpeed = 0.4;
        }
            break;
        case RateItemType3:
        {
            self.readingSpeed = 0.5;
        }
            break;
        case RateItemType4:
        {
            self.readingSpeed = 0.6;
        }
            break;
        case RateItemType5:
        {
            self.readingSpeed = 0.8;
        }
            break;
        default:
            break;
    }
//    NSString *speedStr = [self translaterRateItemTypeToStr:type];
//    [self.buttonRate setTitle:speedStr forState:UIControlStateNormal];
//    NSString *accessbilityLabel = [speedStr stringByReplacingOccurrencesOfString:@"X" withString:@""];
//    accessbilityLabel = [accessbilityLabel stringByAppendingFormat:@" %@",FSLocalizedForKey(@"kTimesVoiceover")];
//    self.buttonRate.accessibilityLabel = accessbilityLabel;
    if (self.currentStatus != READING_STATUS_NONE) {
        [self speakAfterSpeedChange];
    }
}

- (RateItemType)translaterFloatSpeedToRateType:(float)readSpeed
{

    if (readSpeed >= 0.25 && readSpeed < 0.35) {
        return RateItemType1;
    }
    
    if (readSpeed >= 0.35 && readSpeed < 0.45) {
        return RateItemType2;
    }
    
    if (readSpeed >= 0.45 && readSpeed < 0.55) {
        return RateItemType3;
    }
    
    if (readSpeed >= 0.55 && readSpeed < 0.7) {
        return RateItemType4;
    }
    
    if (readSpeed >= 0.7) {
        return RateItemType5;
    }
    return RateItemType0;
}

- (NSString *)translaterRateItemTypeToStr:(RateItemType)type
{
    NSString *speedStr = @"1X";
    switch (type) {
        case RateItemType0:
        {
            speedStr = @"0.5X";
        }
            break;
        case RateItemType1:
        {
            speedStr = @"0.75X";
        }
            break;
        case RateItemType2:
        {
            speedStr = @"1X";
        }
            break;
        case RateItemType3:
        {
            speedStr = @"1.25X";
        }
            break;
        case RateItemType4:
        {
            speedStr = @"1.5X";
        }
            break;
        case RateItemType5:
        {
            speedStr = @"2X";
        }
            break;
        default:
        {
            speedStr = @"1X";
        }
            break;
    }
    return speedStr;
}

#pragma mark - AVSpeechSynthesizerDelegate

- (void)speechSynthesizer:(AVSpeechSynthesizer *)synthesizer didCancelSpeechUtterance:(AVSpeechUtterance *)utterance
{
	[self stop];
}

- (void)speechSynthesizer:(AVSpeechSynthesizer *)synthesizer didFinishSpeechUtterance:(AVSpeechUtterance *)utterance
{
    dispatch_async(dispatch_get_main_queue(), ^{
        /*if (self.boundIndex < (self.arrayBounds.count -1) ) {
            self.boundIndex++;
            NSString *text = self.arrayBounds[self.boundIndex][2];
            [self updateReadingLanguage:text];
            if (self.currentStatus != READING_STATUS_SELECTED) [self resetOverlayView];
            self.readingContent = text;
            [self speak:text];
            return;
        }else */if (self.currentStatus == READING_STATUS_PAGE)
        {
            if (!self.autoNextPage) {

                //Multi page mode cannot reset the value of currentPageindex.
                [self stopOnly];
                [self speakContentWithIndex:self.currentPageindex];
                return;
            }else if (self.autoNextPage && (self.currentPageindex+1) < [[self.pdfViewCtrl currentDoc] getPageCount])
            {
                [self resetOverlayView];
                
                int currentPageindex = [self->_pdfViewCtrl getCurrentPage];
                BOOL turnPage = NO;
                if (currentPageindex <= self.currentPageindex) {
                    turnPage = YES;
                }
                self.currentPageindex++;
                int pageCount = [self.pdfViewCtrl.currentDoc getPageCount];
                
                while (self.currentPageindex<pageCount) {
                    FSTextPage *textPage = [Utility getTextSelect:self.pdfViewCtrl.currentDoc pageIndex:self.currentPageindex];
                    if (!textPage.getCharCount) {
                        self.currentPageindex++;
                    }else{
                        break;
                    }
                }
                if (self.currentPageindex>=pageCount) {
                    [self stop];
                    return;
                }
                [self speakContentWithIndex:self.currentPageindex];
                [self refreshInterface];
                if (turnPage) {
                    [self.pdfViewCtrl gotoPage:self.currentPageindex animated:YES];
                }
                return;
            }
            

        }else  if (self.currentStatus == READING_STATUS_SELECTED){
            if (self.buttonCycle.selected) {
                [self speakSelectPageIndex:self.currentPageindex startCharPos:self.readStartCharPos endCharPos:self.readEndCharPos];
                return;
            }
        }else  if (self.currentStatus == READING_STATUS_TEXT){
            if (self.buttonCycle.selected) {
                [self speakText:self.readingContent];
                return;
            }
        }
        [self stop];
    });
}

- (void)speechSynthesizer:(AVSpeechSynthesizer *)synthesizer willSpeakRangeOfSpeechString:(NSRange)characterRange utterance:(AVSpeechUtterance *)utterance
{
    dispatch_async(dispatch_get_main_queue(), ^{
        self.currentReadingIndex = (int)characterRange.location + self.readingOffset;
        if (self.currentStatus == READING_STATUS_PAGE)
        {
            __block BOOL changed = NO;
            [self.arrayBounds enumerateObjectsWithOptions:NSEnumerationReverse usingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
                NSArray *array = obj;
                int index = [array[3] intValue];
                if (self.currentReadingIndex >= index)
                {
                    int lastBoundIndex = self.boundIndex;
                    self.boundIndex = (int)idx;
                    if (lastBoundIndex != self.boundIndex)
                    {
                        changed = YES;
                    }
                    *stop = YES;
                }
            }];
            
            if (changed)
            {
                [self refreshOverlayView];
                [self refreshInterface];
            }
        }
    });
}

- (void)speechSynthesizer:(AVSpeechSynthesizer *)synthesizer didStartSpeechUtterance:(AVSpeechUtterance *)utterance
{
    if (self.currentStatus != READING_STATUS_SELECTED || (self.currentStatus == READING_STATUS_SELECTED && _boundIndex == 0)) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self refreshOverlayView];
            [self refreshInterface];
            [self setNowPlayingInfo];
        });
    }else{
        dispatch_async(dispatch_get_main_queue(), ^{
            [self setNowPlayingInfo];
        });
    }
}

- (void)speechSynthesizer:(AVSpeechSynthesizer *)synthesizer didPauseSpeechUtterance:(AVSpeechUtterance *)utterance
{
    dispatch_async(dispatch_get_main_queue(), ^{
        [self resetOverlayView];
        [self refreshInterface];
        [self resetNowPlayingInfo];
    });
}

- (void)speechSynthesizer:(AVSpeechSynthesizer *)synthesizer didContinueSpeechUtterance:(AVSpeechUtterance *)utterance
{
    dispatch_async(dispatch_get_main_queue(), ^{
        [self refreshOverlayView];
        [self refreshInterface];
        [self setNowPlayingInfo];
    });
}

#pragma mark - Public method

- (void)speakFromTextMarkup:(FSTextMarkup *)textMarkup {
    FSTextPage *textPage = [Utility getTextSelect:self.pdfViewCtrl.currentDoc pageIndex:textMarkup.pageIndex];
    NSMutableString *readingText = [NSMutableString string];
    for (int i = 0; i < textMarkup.quadPoints.getSize; i++) {
        FSQuadPoints *quadPoints = [textMarkup.quadPoints getAt:i];
        
        NSString *text = nil;
        NSMutableArray *tmp = @[ [[FSRectF alloc] initWithLeft1:quadPoints.third.x bottom1:quadPoints.third.y right1:quadPoints.second.x top1:quadPoints.second.y], [[FSRectF alloc] initWithLeft1:quadPoints.first.x bottom1:quadPoints.first.y right1:quadPoints.fourth.x top1:quadPoints.fourth.y], [[FSRectF alloc] initWithLeft1:quadPoints.second.x bottom1:quadPoints.second.y right1:quadPoints.third.x top1:quadPoints.third.y] ].mutableCopy;
        while (tmp.count) {
            FSRectF *dibRect = tmp.firstObject;
            @try {
                text = [textPage getTextInRect:dibRect];
                break;
            } @catch (NSException *exception) {
                [tmp removeObject:dibRect];
            }
        }
        [readingText appendString:[NSString stringWithFormat:@"%@ ",text]];
    }
    [self speakText:readingText];
}


- (void)speakText:(NSString *)text{
    if (!_isInitedBarViewData) {
        _isInitedBarViewData = YES;
    }
    
    [self stopOnly];
    [self refreshInterface];
    self.currentStatus = READING_STATUS_TEXT;
    if ([_extensionsManager getState] != FSUIManagerStateSpeech) {
        [_extensionsManager changeState:FSUIManagerStateSpeech];
        _isSpeechState = YES;
        [self showSpeechBar:YES animation:YES];
    }
    _boundIndex = -1;
    [self speakContentWithText:text];
}

- (void)speakFromPageIndex:(int)pageIndex{
    if (self.currentStatus == READING_STATUS_TEXT || self.currentStatus == READING_STATUS_SELECTED || self.currentStatus == READING_STATUS_PAGE) return;
    [self speakFromPageIndex:pageIndex startCharPos:0];
    
    
}

- (void)speakFromPageIndex:(int)pageIndex startCharPos:(int)startPos{
    if (!_isInitedBarViewData) {
        _isInitedBarViewData = YES;
    }
    [self stopOnly];
    [self refreshInterface];
    self.currentStatus = READING_STATUS_PAGE;
    if ([_extensionsManager getState] != FSUIManagerStateSpeech) {
        [_extensionsManager changeState:FSUIManagerStateSpeech];
        _isSpeechState = YES;
        [self showSpeechBar:YES animation:YES];
    }
    _currentPageindex = pageIndex;
    [self speakContentWithIndex:pageIndex startCharPos:startPos endCharPos:-1];
}

- (void)speakSelectPageIndex:(int)pageIndex startCharPos:(int)startPos endCharPos:(int)pos{
    self.readStartCharPos = startPos;
    self.readEndCharPos = pos;
    if (!_isInitedBarViewData) {
        _isInitedBarViewData = YES;
    }
    
    [self stopOnly];
    [self refreshInterface];
    self.currentStatus = READING_STATUS_SELECTED;
    if ([_extensionsManager getState] != FSUIManagerStateSpeech) {
        [_extensionsManager changeState:FSUIManagerStateSpeech];
        _isSpeechState = YES;
        [self showSpeechBar:YES animation:YES];
    }
    _currentPageindex = pageIndex;
    [self speakContentWithIndex:pageIndex startCharPos:startPos endCharPos:pos];
}


- (void)speakContentWithIndex:(int)pageIndex startCharPos:(int)startPos endCharPos:(int)pos;
{
    if (self.pdfViewCtrl.isDynamicXFA) return;

    NSMutableString *readingText = [NSMutableString string];
    
    FSTextPage *textPage = [Utility getTextSelect:self.pdfViewCtrl.currentDoc pageIndex:pageIndex];
    
    int count = (pos == -1 ? [textPage getCharCount] - startPos  : (pos - startPos) +1);
//    if (startPos == pos && pos == 0) {
//        count = 1;
//    }
    UIEdgeInsets edgeInsets = [self.pdfViewCtrl getCropInsets:pageIndex];
    FSPDFPage *page = [self.pdfViewCtrl.currentDoc getPage:pageIndex];
    if (!page || [page isEmpty]) {
        [self stop];
        return;
    }
    float pageWidth = [page getWidth];
    float pageHeight = [page getHeight];
    FSRectF *limitRect = [[FSRectF alloc] initWithLeft1:edgeInsets.left bottom1:edgeInsets.bottom right1:(pageWidth - edgeInsets.right) top1:(pageHeight - edgeInsets.top)];
    
    NSMutableArray *arrayBounds =  [Utility getTextRects:textPage start:startPos count:count limitRect:limitRect].mutableCopy;
    _boundIndex = -1;
    [arrayBounds enumerateObjectsUsingBlock:^(NSArray *arr, NSUInteger idx, BOOL *stop) {
        NSMutableArray *arrayOne = arr.mutableCopy;
        [arrayOne addObject:[NSNumber numberWithInt:(int)readingText.length]];
        NSString *text = arrayOne[2];
        [readingText appendString:[NSString stringWithFormat:@"%@ ",text]];
        [arrayBounds replaceObjectAtIndex:idx withObject:arrayOne.copy];
    }];
    self.arrayBounds = arrayBounds.copy;
//    if (arrayBounds.count > 0) {
//        _boundIndex = 0;
//        NSString *text = self.arrayBounds[_boundIndex][2];
//        [self speakContentWithText:text];
//    }else{
//        [self speakContentWithText:readingText];
//    }
    [self speakContentWithText:readingText];
}

- (void)stop
{
	[self stopOnly];
    [self resetNowPlayingInfo];
	self.currentStatus = READING_STATUS_NONE;
}

- (void)refreshInterface
{
    int titleIndex = _currentPageindex;
    NSString *content = @"";
    
    if (_currentStatus == READING_STATUS_NONE)
    {
        titleIndex = [_pdfViewCtrl getCurrentPage];
        _currentPageindex = titleIndex;
    
        self.buttonPreviousPage.enabled = NO;
        self.buttonNextPage.enabled = NO;
        [self.buttonPlay setImage:FSImageNamedAndNormalColor(@"speech_start") forState:UIControlStateNormal];
        [self.buttonPlay setImage:[Utility unableDynamicImage:FSImageNamedAndNormalColor(@"speech_start")] forState:UIControlStateHighlighted];
    }
    else if (_currentStatus == READING_STATUS_PAGE)
    {
        if (self.arrayBounds.count > 0 && _boundIndex >= 0 &&  _boundIndex < self.arrayBounds.count)
        {
            content = self.arrayBounds[_boundIndex][2];
        }
        
        if ((_currentPageindex-1) < 0)
        {
            self.buttonPreviousPage.enabled = NO;
        }
        else
        {
            self.buttonPreviousPage.enabled = YES;
        }
        if ((_currentPageindex+1) >= [[_pdfViewCtrl currentDoc] getPageCount])
        {
            self.buttonNextPage.enabled = NO;
        }
        else
        {
            self.buttonNextPage.enabled = YES;
        }
        if (self.speechSynthesizer.paused)
        {
            [self.buttonPlay setImage:FSImageNamedAndNormalColor(@"speech_start") forState:UIControlStateNormal];
            [self.buttonPlay setImage:[Utility unableDynamicImage:FSImageNamedAndNormalColor(@"speech_start")] forState:UIControlStateHighlighted];
            self.buttonPlay.accessibilityLabel = FSLocalizedForKey(@"kPlayBtnContinueVoiceOver");
        }
        else
        {
            [self.buttonPlay setImage:FSImageNamedAndNormalColor(@"speech_pause") forState:UIControlStateNormal];
            [self.buttonPlay setImage:[Utility unableDynamicImage:FSImageNamedAndNormalColor(@"speech_pause")] forState:UIControlStateHighlighted];
            self.buttonPlay.accessibilityLabel = FSLocalizedForKey(@"kPlayBtnPauseVoiceOver");
        }
        
    }
    else if (_currentStatus == READING_STATUS_SELECTED)
    {
        content = self.readingContent;
        
        self.buttonPreviousPage.enabled = NO;
        self.buttonNextPage.enabled = NO;
        if (self.speechSynthesizer.paused)
        {
            [self.buttonPlay setImage:FSImageNamedAndNormalColor(@"speech_start") forState:UIControlStateNormal];
            [self.buttonPlay setImage:[Utility unableDynamicImage:FSImageNamedAndNormalColor(@"speech_start")] forState:UIControlStateHighlighted];
            self.buttonPlay.accessibilityLabel = FSLocalizedForKey(@"kPlayBtnContinueVoiceOver");
        }
        else
        {
            [self.buttonPlay setImage:FSImageNamedAndNormalColor(@"speech_pause") forState:UIControlStateNormal];
            [self.buttonPlay setImage:[Utility unableDynamicImage:FSImageNamedAndNormalColor(@"speech_pause")] forState:UIControlStateHighlighted];
            self.buttonPlay.accessibilityLabel = FSLocalizedForKey(@"kPlayBtnPauseVoiceOver");
        }
        
    }
    else if (_currentStatus == READING_STATUS_TEXT)
    {
        content = self.readingContent;
        
        self.buttonPreviousPage.enabled = NO;
        self.buttonNextPage.enabled = NO;
        if (self.speechSynthesizer.paused)
        {
            [self.buttonPlay setImage:FSImageNamedAndNormalColor(@"speech_start") forState:UIControlStateNormal];
            [self.buttonPlay setImage:[Utility unableDynamicImage:FSImageNamedAndNormalColor(@"speech_start")] forState:UIControlStateHighlighted];
            self.buttonPlay.accessibilityLabel = FSLocalizedForKey(@"kPlayBtnContinueVoiceOver");
        }
        else
        {
            [self.buttonPlay setImage:FSImageNamedAndNormalColor(@"speech_pause") forState:UIControlStateNormal];
            [self.buttonPlay setImage:[Utility unableDynamicImage:FSImageNamedAndNormalColor(@"speech_pause")] forState:UIControlStateHighlighted];
            self.buttonPlay.accessibilityLabel = FSLocalizedForKey(@"kPlayBtnPauseVoiceOver");
        }
        
    }
}

#pragma IGestureEventListener

- (BOOL)onTap:(UITapGestureRecognizer *)recognizer {
    if ([_extensionsManager getState] == FSUIManagerStateSpeech) {
        [_extensionsManager setFullScreen:!self.toolbarBottom.hidden];
        return YES;
    }
    return NO;
}

- (BOOL)onLongPress:(UILongPressGestureRecognizer *)recognizer {
    return NO;
}

#pragma IStateChangeListener
-(void)onStateChanged:(FSUIManagerState)state lastState:(FSUIManagerState)lastState
{
    if (state != FSUIManagerStateSpeech && state != FSUIManagerStateAnnotTool && _isSpeechState) {
        _oldState = state;
        _isSpeechState = NO;
        if (_currentStatus != READING_STATUS_NONE)
        {
            if (!self.speechSynthesizer.paused)
            {
                [self barButtonPlayClicked:self.buttonPlay];
            }
            _currentStatus = READING_STATUS_NONE;
        }
        [self.toolbarBottom setHidden:YES];
        [self.topToolBar setHidden:YES];
        [self resetOverlayView];
        [self stop];
        [_extensionsManager.settingBar.viewSettingManager setItemSelectedWithType:SPEECH selected:NO];
    }else if (state == FSUIManagerStateSpeech) {
        
        _isSpeechState = YES;
        
        if ((lastState != FSUIManagerStateAnnotTool && lastState != FSUIManagerStateSpeech) || _extensionsManager.fullScreenToolbarMgr.floatyToolbar.hidden) {
            if (_extensionsManager.currentAnnot) {
                _extensionsManager.currentAnnot = nil;
            }
            
            [_pdfViewCtrl gotoPage:_currentPageindex animated:YES];
            AVAudioSession *audioSession = [AVAudioSession sharedInstance];
            NSError *setCategoryError = nil;
            [audioSession setCategory:AVAudioSessionCategoryPlayback error:&setCategoryError];
            
            if (!_isInitedBarViewData) {
                _isInitedBarViewData = YES;
            }
            if ([self checkIsIphoneDevice]) {
                [_toolbarBottom mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.left.right.bottom.mas_equalTo(self.pdfViewCtrl);
                }];
            }
            else
            {
                [_toolbarBottom mas_remakeConstraints:^(MASConstraintMaker *make) {
                    make.bottom.mas_equalTo(self.toolbarBottom.superview.fs_mas_bottom).offset(-40);
                    make.centerX.mas_equalTo(self.toolbarBottom.superview);
                    make.width.mas_equalTo(354);
                }];
            }
            [self speak];
            [self.extensionsManager.settingBar.viewSettingManager setItemSelectedWithType:SPEECH selected:YES];
            [self reLayoutBottomToolbar];
        }
    }
}

- (void)appEnterBackground
{
//    if (_currentStatus == READING_STATUS_PAGE || _currentStatus == READING_STATUS_SELECTED)
//    {
//        if (self.speechSynthesizer.isSpeaking) {
//            [self.speechSynthesizer pauseSpeakingAtBoundary:AVSpeechBoundaryImmediate];
//        }
//    }
}

- (void)appEnterForeground
{
//    [self.speechSynthesizer continueSpeaking];
}


- (void)resetOverlayView
{
	_readingIndex = -1;
    dispatch_main_async_safe(^{
        [self.pdfViewCtrl refresh:self.currentPageindex needRender:NO];
    });
    
}

- (void)refreshOverlayView
{
    if (self.arrayBounds.count > 0 && (self.currentStatus == READING_STATUS_PAGE) &&  _boundIndex < self.arrayBounds.count)
    {
        _readingRect = [Utility CGRect2FSRectF:[self.arrayBounds[_boundIndex][0] CGRectValue]];
        _readingDirection = [self.arrayBounds[_boundIndex][1] intValue];
    }
    _readingIndex = _currentPageindex;
    [_pdfViewCtrl refresh:_currentPageindex needRender:NO];
}

- (void)resetNowPlayingInfo
{
    if ([MPNowPlayingInfoCenter class])
    {
        [MPNowPlayingInfoCenter defaultCenter].nowPlayingInfo = nil;
    }
}

- (void)setNowPlayingInfo
{
    NSMutableDictionary *currentlyPlayingTrackInfo = [NSMutableDictionary dictionary];
    NSString *page = [NSString stringWithFormat:FSLocalizedForKey(@"kSpeechPage"), _currentPageindex+1];
    NSString *name = self.pdfViewCtrl.filePath;
    if (!name.length) return;
    NSArray *array = [name componentsSeparatedByString:@"/"];
    NSString *fileName = [[[array lastObject] componentsSeparatedByString:@"."] firstObject];
    [currentlyPlayingTrackInfo setObject:fileName forKey:MPMediaItemPropertyTitle];
    [currentlyPlayingTrackInfo setObject:@"FoxitSpeech" forKey:MPNowPlayingInfoPropertyExternalUserProfileIdentifier];
    if (_currentStatus == READING_STATUS_PAGE)
    {
        [currentlyPlayingTrackInfo setObject:page forKey:MPMediaItemPropertyAlbumTitle];
    }
    [MPNowPlayingInfoCenter defaultCenter].nowPlayingInfo = currentlyPlayingTrackInfo;
    
    [self remoteControlEventHandler];
}

- (void)stopOnly
{
	self.speechSynthesizer.delegate = nil;
	[self.speechSynthesizer stopSpeakingAtBoundary:AVSpeechBoundaryImmediate];
	self.speechSynthesizer = nil;
	self.readingContent = nil;
	self.arrayBounds = nil;
    self.readingRect = nil;
	_readingOffset = 0;
	[self resetOverlayView];
}

- (void)setCurrentStatus:(READING_STATUS)currentStatus
{
	_currentStatus = currentStatus;
	[self refreshInterface];
}

- (BOOL)speakContentWithIndex:(int)pageIndex
{
	NSMutableString *readingText = [NSMutableString string];
    FSTextPage *textPage = [Utility getTextSelect:self.pdfViewCtrl.currentDoc pageIndex:pageIndex];
    
    UIEdgeInsets edgeInsets = [self.pdfViewCtrl getCropInsets:pageIndex];
    FSPDFPage *page = [self.pdfViewCtrl.currentDoc getPage:pageIndex];
    if (!page || [page isEmpty]) {
        [self stop];
        return NO;
    }
    float pageWidth = [page getWidth];
    float pageHeight = [page getHeight];
    FSRectF *limitRect = [[FSRectF alloc] initWithLeft1:edgeInsets.left bottom1:edgeInsets.bottom right1:(pageWidth - edgeInsets.right) top1:(pageHeight - edgeInsets.top)];
    NSMutableArray *arrayBounds =  [Utility getTextRects:textPage start:0 count:[textPage getCharCount] limitRect:limitRect].mutableCopy;
	_boundIndex = -1;
	[arrayBounds enumerateObjectsUsingBlock:^(NSArray *arr, NSUInteger idx, BOOL *stop) {
		NSMutableArray *arrayOne = arr.mutableCopy;
		[arrayOne addObject:[NSNumber numberWithInt:(int)readingText.length]];
		NSString *text = arrayOne[2];
		[readingText appendString:[NSString stringWithFormat:@"%@ ",text]];
        [arrayBounds replaceObjectAtIndex:idx withObject:arrayOne.copy];
	}];
    self.arrayBounds = arrayBounds.copy;
//    if (arrayBounds.count > 0) {
//        _boundIndex = 0;
//        NSString *text = self.arrayBounds[_boundIndex][2];
//        [self speakContentWithText:text];
//    }else{
//        [self speakContentWithText:readingText];
//    }
    [self speakContentWithText:readingText];
    return [textPage getCharCount];
}

- (void)speakContentWithText:(NSString*)content
{
	_readingOffset = 0;
	self.readingContent = content;
	[self speak:self.readingContent];
}

- (void)speakAfterSpeedChange
{
	_readingOffset = _currentReadingIndex;
	if (self.readingContent && self.readingContent.length > 0 && _currentReadingIndex >=0 && _currentReadingIndex < self.readingContent.length)
	{
		NSString *text = [self.readingContent substringFromIndex:_currentReadingIndex];
		[self speak:text];
	}
	[self refreshInterface];
}

- (void)updateReadingLanguage:(NSString*)text
{
    AVSpeechSynthesisVoice *voice = [AVSpeechSynthesisVoice voiceWithIdentifier:self.selectVoiceIdentifier];
    if (voice){
        self.matchedVoiceIdentifier = self.selectVoiceIdentifier;
        return;
    }

	NSString *lang = [StringDrawUtil languageForString:text];
    NSString *langPre;
    if (lang) langPre = [[lang componentsSeparatedByString:@"-"] firstObject];
    
	NSArray *allVoices = [AVSpeechSynthesisVoice speechVoices];
    __block NSString *machedIdentifier = nil;
	__block NSString *bestIdentifier = nil;
	[allVoices enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        AVSpeechSynthesisVoice *voiceItem = obj;
        if ([[voiceItem.language lowercaseString] hasPrefix:[langPre lowercaseString]]) {
            if (!machedIdentifier) machedIdentifier = voiceItem.identifier;
            if ([[voiceItem.language lowercaseString] hasPrefix:[lang lowercaseString]]){
                bestIdentifier = voiceItem.identifier;
            }
        }
	}];
	
    if (!bestIdentifier) bestIdentifier = machedIdentifier;
	self.matchedVoiceIdentifier = bestIdentifier;
}

- (void)speak:(NSString*)text
{
    [self updateReadingLanguage:text];
	[self.speechSynthesizer stopSpeakingAtBoundary:AVSpeechBoundaryImmediate];
	self.speechSynthesizer.delegate = nil;
	self.speechSynthesizer = nil;
	
	AVSpeechUtterance *utterance = [AVSpeechUtterance speechUtteranceWithString:text];
	[utterance setRate:_readingSpeed];
    AVSpeechSynthesisVoice *voice = [AVSpeechSynthesisVoice voiceWithIdentifier:self.matchedVoiceIdentifier];
    if (!voice) voice = [AVSpeechSynthesisVoice voiceWithLanguage:[AVSpeechSynthesisVoice currentLanguageCode]];
    [utterance setVoice:voice];//[AVSpeechSynthesisVoice voiceWithLanguage:self.readingLanguage]];
	self.speechSynthesizer = [[AVSpeechSynthesizer alloc] init];
	self.speechSynthesizer.delegate = self;
	[self.speechSynthesizer speakUtterance:utterance];
}

#pragma IDocEventListener methods

- (void)onDocOpened:(FSPDFDoc *)document error:(int)error{
    if (_isSpeechState) {
        [[UIApplication sharedApplication] beginReceivingRemoteControlEvents];
    }
}

- (void)onDocWillClose:(FSPDFDoc *)document{
    if (_isSpeechState) {
        [[UIApplication sharedApplication] endReceivingRemoteControlEvents];
    }
}


#pragma IDvDrawEventListener

- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context{
    if (([_extensionsManager getState] == FSUIManagerStateSpeech || [_extensionsManager getState] == FSUIManagerStateAnnotTool)&& self.readingIndex == pageIndex) {
        if (self.currentStatus == READING_STATUS_PAGE) {
            if (![self.readingRect isEmpty] && self.readingRect) {
                int readIndex = self.readingIndex;

                FSRectF *readingRect = self.readingRect;
                
                CGRect pageRect = [_pdfViewCtrl convertPdfRectToPageViewRect:readingRect pageIndex:readIndex];
                CGRect parentRect = [_pdfViewCtrl convertPageViewRectToDisplayViewRect:pageRect pageIndex:readIndex];

                CGRect selfRect = [[_pdfViewCtrl getPageView:readIndex] convertRect:parentRect fromView:[_pdfViewCtrl getDisplayView]];

                CGContextSaveGState(context);


                CGContextSetRGBFillColor(context, 0, 0, 1, 0.3);
                CGContextFillRect(context, selfRect);
                CGContextRestoreGState(context);
            }
        }else if (self.currentStatus == READING_STATUS_SELECTED) {
            [self.arrayBounds enumerateObjectsUsingBlock:^(NSArray  *_Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
                int readIndex = self.readingIndex;

                FSRectF *readingRect = [Utility CGRect2FSRectF:[obj[0] CGRectValue]];;
                
                CGRect pageRect = [_pdfViewCtrl convertPdfRectToPageViewRect:readingRect pageIndex:readIndex];
                CGRect parentRect = [_pdfViewCtrl convertPageViewRectToDisplayViewRect:pageRect pageIndex:readIndex];

                CGRect selfRect = [[_pdfViewCtrl getPageView:readIndex] convertRect:parentRect fromView:[_pdfViewCtrl getDisplayView]];

                CGContextSaveGState(context);


                CGContextSetRGBFillColor(context, 0, 0, 1, 0.3);
                CGContextFillRect(context, selfRect);
                CGContextRestoreGState(context);
            }];
        }
    }
}


#pragma IModule methods
-(NSString*)getName
{
	return @"Speech";
}
-(void)loadModule
{
    _isInitedData = YES;
    _isInitedBarViewData = NO;
    _autoNextPage = YES;
    _readingSpeed = DEFAULT_READING_SPEED;
    
    [_pdfViewCtrl registerGestureEventListener:self];
    [_extensionsManager registerStateChangeListener:self];
	[_pdfViewCtrl registerDocEventListener:self];
    [_pdfViewCtrl registerDrawEventListener:self];
    [_extensionsManager registerFullScreenListener:self];
    [_extensionsManager.settingBar.viewSettingManager registerEventListener:self];
    [_pdfViewCtrl registerPageEventListener:self];
    [_extensionsManager.thumbnailModule registerThumbnailEventListener:self];
    [_extensionsManager.panelController registerPanelChangedListener:self];
    [_extensionsManager registerWindowFrameChangedListener:self];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(appEnterBackground) name:UIApplicationDidEnterBackgroundNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(appEnterForeground) name:UIApplicationWillEnterForegroundNotification object:nil];
}

#pragma mark--pageEventListener methods

- (void)onPageChanged:(int)oldIndex currentIndex:(int)currentIndex {
    if ([_extensionsManager getState] == FSUIManagerStateSpeech)
    {
        //[self speakFromPageIndex:currentIndex];
    }
}

- (void)onPagesRemoved:(NSArray<NSNumber *> *)indexes
{
    if ([_extensionsManager getState] == FSUIManagerStateSpeech)
    {
        BOOL deleteContain = NO;
        int lessIndex = 0;
        for (NSNumber *obj in indexes) {
            if ([obj integerValue] == _currentPageindex) {
                deleteContain = YES;
                break;
            }
            
            if ([obj integerValue] < _currentPageindex) {
                lessIndex++;
            }
        }
        
        if (deleteContain) {
            [self stop];
        }else{
            _currentPageindex = _currentPageindex - lessIndex;
        }
    }
}

- (void)onPagesMoved:(NSArray<NSNumber *> *)indexes dstIndex:(int)dstIndex
{
    if ([_extensionsManager getState] == FSUIManagerStateSpeech)
    {
        BOOL moveContain = NO;
        int containIndex = 0;
        int lessIndex = 0;
        int lessCurIndex = 0;
        for (int i = 0; i < indexes.count; i++) {
            NSNumber *obj = indexes[i];
            if ([obj integerValue] == _currentPageindex) {
                moveContain = YES;
                containIndex = i;
            }
            
            if ([obj integerValue] < dstIndex) {
                lessIndex++;
            }
            
            if ([obj integerValue] < _currentPageindex) {
                lessCurIndex++;
            }
        }
        
        if (moveContain) {
            _currentPageindex = dstIndex + containIndex - lessIndex;
        }else{
            if (dstIndex <= _currentPageindex) {
                _currentPageindex = _currentPageindex + indexes.count - lessCurIndex;
            }else{
                _currentPageindex = _currentPageindex - lessCurIndex;
            }
        }
    }
}

- (void)onPagesInsertedAtRange:(NSRange)range
{
    if ([_extensionsManager getState] == FSUIManagerStateSpeech)
    {
        if (range.location <= _currentPageindex) {
            _currentPageindex = _currentPageindex + range.length;
        }
    }
}


#pragma mark -- IThumbnailEventListener
- (void)onThumbnailChanged:(BOOL)isHidden
{
    if ([self.extensionsManager getState] == FSUIManagerStateSpeech) {
        self.toolbarBottom.hidden = !isHidden;
        if (isHidden) {
            [self.extensionsManager setFullScreen:NO];
        }
    }
}

#pragma mark --IPanelChangedListener
- (void)onPanelChanged:(BOOL)isHidden
{
    if ([_extensionsManager getState] == FSUIManagerStateSpeech) {
        if (![self checkIsIphoneDevice] && !isHidden) {
            [self.toolbarBottom.superview bringSubviewToFront:self.toolbarBottom];
        }
    }
}

-(void)reLayoutBottomToolbar
{
    if ([self checkIsIphoneDevice]) {
        [_toolbarBottom mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.right.bottom.mas_equalTo(_toolbarBottom.superview);
        }];
        
        [_buttonRate mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(_buttonNextPage.mas_right).offset(ITEM_MARGIN_NORMAL);
            make.right.mas_equalTo(_buttonRate.superview);
            make.centerY.mas_equalTo(_toolbarBottomSpeech);
        }];
        
        if (_exitItem && _exitItem.superview) {
            [_exitItem removeFromSuperview];
        }
        if (_pan && [_toolbarBottom.gestureRecognizers containsObject:_pan]) {
            [_toolbarBottom removeGestureRecognizer:_pan];
        }
        
    }else {
        _toolbarBottom.layer.cornerRadius = 6;
        [_toolbarBottom mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.bottom.mas_equalTo(self.toolbarBottom.superview.fs_mas_bottom).offset(-40);
            make.centerX.mas_equalTo(self.toolbarBottom.superview);
            make.width.mas_equalTo(IPAD_PREFERENCE_BAR_WIDTH);
        }];
        
        if (!_pan) {
            _pan = [[UIPanGestureRecognizer alloc] initWithTarget:_extensionsManager.settingBar.viewSettingManager action:@selector(moveBottombarWithRecognizer:)];
        }
        if (![_toolbarBottom.gestureRecognizers containsObject:_pan]) {
            [_toolbarBottom addGestureRecognizer:_pan];
        }
        
        [_buttonRate mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(_buttonNextPage.mas_right).offset(ITEM_MARGIN_NORMAL);
            make.centerY.mas_equalTo(_toolbarBottomSpeech);
        }];
        
        if (!_exitItem) {
            _exitItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"setting_exit")];
            _exitItem.accessibilityLabel = FSLocalizedForKey(@"kExitVoiceOver");
            @weakify(self)
            _exitItem.onTapClick = ^(FSItem * _Nonnull _item) {
                @strongify(self)
                [self->_extensionsManager changeState:FSUIManagerStateNormal];
            };
        }
        
        if (!_exitItem.superview) {
            [_toolbarBottomSpeech addSubview:_exitItem];
            [_exitItem mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.left.mas_equalTo(_buttonRate.mas_right).offset(ITEM_MARGIN_NORMAL);
                make.right.mas_equalTo(_exitItem.superview);
                make.centerY.mas_equalTo(_toolbarBottomSpeech);
            }];
        }
    }
}

#pragma mark-- IWindowFrameChangedListener
- (void)onWindowFrameChanged:(CGRect)newFrame horizontalSizeClassChanged:(BOOL)changed
{
    if (DEVICE_iPAD && changed) {
        if ([_extensionsManager getState] == FSUIManagerStateSpeech) {
            [self reLayoutBottomToolbar];
            self.toolbarBottom.hidden = YES;
            [self.extensionsManager setCurrentIsFullScreen:NO];
            [self showSpeechBar:YES animation:NO];
        }else if ([_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]){
            [self speakStop];
            [self showSpeechBar:NO animation:NO];
        }
    }
}

- (BOOL)checkIsIphoneDevice
{
    return DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact];
}
- (void)remoteControlEventHandler
{
    MPRemoteCommandCenter *commandCenter = [MPRemoteCommandCenter sharedCommandCenter];
    commandCenter.playCommand.enabled = YES;
    [commandCenter.playCommand addTarget:self action:@selector(playAction)];

    [commandCenter.pauseCommand addTarget:self action:@selector(playAction)];
    [commandCenter.previousTrackCommand addTarget:self action:@selector(prePageAction)];
    [commandCenter.nextTrackCommand addTarget:self action:@selector(nextPageAction)];

    commandCenter.togglePlayPauseCommand.enabled = YES;
    [commandCenter.togglePlayPauseCommand addTarget:self action:@selector(playAction)];
}

- (void)removeRemoteControlEventHandler
{
    MPRemoteCommandCenter *commandCenter = [MPRemoteCommandCenter sharedCommandCenter];
    [commandCenter.playCommand removeTarget:self];
    [commandCenter.pauseCommand removeTarget:self];
    [commandCenter.previousTrackCommand removeTarget:self];
    [commandCenter.nextTrackCommand removeTarget:self];
    [commandCenter.togglePlayPauseCommand removeTarget:self];
}

- (MPRemoteCommandHandlerStatus)playAction
{
    [self barButtonPlayClicked:nil];
    return MPRemoteCommandHandlerStatusSuccess;
}

- (MPRemoteCommandHandlerStatus)prePageAction
{
    [self barButtonPreviousPageClicked:nil];
    return MPRemoteCommandHandlerStatusSuccess;
}

- (MPRemoteCommandHandlerStatus)nextPageAction
{
    [self barButtonNextPageClicked:nil];
    return MPRemoteCommandHandlerStatusSuccess;
}
@end
