/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSAddTextStampViewController.h"
#import "FSBaseViews.h"
#import "FSCustomStampAttributes.h"

typedef void(^FSTAddTextStampTextAction)(NSString *text);
@interface FSTAddTextStampTextFieldCell : FSTableViewCell<UITextFieldDelegate>
@property (nonatomic, copy) FSTAddTextStampTextAction action;
@property (nonatomic, strong) UITextField *textField;
@end

@implementation FSTAddTextStampTextFieldCell

- (void)initialize{
    [super initialize];
    
    UILabel *textLab = [UILabel new];
    textLab.font = self.fs_standardGeneralFont_GF2;
    textLab.textColor = self.fs_textColor_T4;
    textLab.text = FSLocalizedForKey(@"kText");
    
    [self.contentView addSubview:textLab];
    [textLab mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(textLab.superview).offset(ITEM_MARGIN_NORMAL);
        make.centerY.equalTo(textLab.superview);
    }];
    
    UITextField *textField = [UITextField new];
    textField.delegate = self;
    textField.font = self.fs_standardGeneralFont_GF2;
    textField.textColor = self.fs_textColor_T4;
    textField.textAlignment = NSTextAlignmentLeft;
    textField.borderStyle = UITextBorderStyleNone;
    textField.clearButtonMode = UITextFieldViewModeAlways;
    textField.autocorrectionType = UITextAutocorrectionTypeNo;
    [textField addTarget:self action:@selector(didChangedTextField:) forControlEvents:UIControlEventEditingChanged];

    NSMutableDictionary *attrs = [NSMutableDictionary dictionary];
    attrs[NSFontAttributeName] = self.fs_standardGeneralFont_GF2;
    attrs[NSForegroundColorAttributeName] = self.fs_textColor_T3;
    NSAttributedString *attStr = [[NSAttributedString alloc] initWithString:FSLocalizedForKey(@"kStampText") attributes:attrs];
    textField.attributedPlaceholder = attStr;
    
    [self.contentView addSubview:textField];
    [textField mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(textLab.superview).offset(-ITEM_MARGIN_NORMAL);
        make.left.equalTo(textLab.mas_right).offset(ITEM_MEDIUM_SPACE_NORMAL);
        make.centerY.equalTo(textLab.superview);
    }];
    self.textField = textField;
}

- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string
{
    return textField.text.length + (string.length - range.length) <= 37;
}

- (void)didChangedTextField:(UITextField *)textField{
    textField.text = [Utility regularFilterStrEmoji:textField.text];
    !_action ? : _action(textField.text.length ? textField.text :  textField.placeholder);
}

@end

typedef void(^FSTAddTextStampSwitchAction)(BOOL on);
@interface FSTAddTextStampSwitchCell : FSTableViewCell
@property (nonatomic, strong) UILabel *textLab;
@property (nonatomic, copy) FSTAddTextStampSwitchAction action;
@property (nonatomic, strong) FSSwitch *switchBtn;
@end

@implementation FSTAddTextStampSwitchCell

- (instancetype)initWithText:(NSString *)text{
    self = [super init];
    if (self) {
        self.textLab.text = text;
    }
    return self;
}

- (void)initialize{
    [super initialize];
    
    UILabel *textLab = [UILabel new];
    textLab.font = self.fs_standardGeneralFont_GF2;
    textLab.textColor = self.fs_textColor_T4;
    
    [self.contentView addSubview:textLab];
    [textLab mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(textLab.superview).offset(ITEM_MARGIN_NORMAL);
        make.centerY.equalTo(textLab.superview);
    }];
    
    self.textLab = textLab;
    
    FSSwitch *switchBtn = [[FSSwitch alloc] initWithFrame:CGRectZero];
    [switchBtn addTarget:self action:@selector(switchBtnOn:) forControlEvents:UIControlEventTouchUpInside];
    [self.contentView addSubview:switchBtn];
    [switchBtn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.equalTo(switchBtn.superview.mas_right).offset(-ITEM_MARGIN_NORMAL);
        make.centerY.equalTo(switchBtn.superview);
    }];
    
    BindThemeKeyPath(switchBtn, onTintColor);
    self.switchBtn = switchBtn;
}

- (void)switchBtnOn:(FSSwitch *)sender{
    !_action ? : _action(sender.on);
}

@end

typedef void(^FSTAddTextStampStyleAction)(NSInteger style);
@interface FSTAddTextStampStyleFooterView : UIView
{
    NSArray *_colorsHex;
}
@property (nonatomic, strong) UIStackView *styleOptionsView;
@property (nonatomic, strong) FSItem *currentStyleItem;
@property (nonatomic, copy) FSTAddTextStampStyleAction action;
@property (nonatomic, copy) NSString *colorHex;
@property (nonatomic, strong) UIImageView *pdfThumbnail;
@end

@implementation FSTAddTextStampStyleFooterView

- (instancetype)initWithFrame:(CGRect)frame colorsHex:(NSArray<NSString *> *)colorsHex{
    self = [super initWithFrame:frame];
    if (self) {
        _colorsHex = colorsHex;
        [self initialize];
    }
    return self;
}

- (void)initialize{
    
    UIStackView *styleOptionsView = [UIStackView new];
    styleOptionsView.distribution = UIStackViewDistributionFillEqually;
    
    CGSize itemSize = CGSizeMake(70.f, 70.f);
    NSArray *colorsHex = _colorsHex;
    styleOptionsView.spacing = (MIN_FOREGROUND_KEY_WINDOW_WIDTH  - itemSize.width * colorsHex.count) / (colorsHex.count - 1);
    for (int i = 0; i < colorsHex.count; i++) {
        FSItem *item = [FSItem createItemWithTitle:@"Aa"];
        item.customAccessibilityTraits = UIAccessibilityTraitNone;
        item.accessibilityLabel = [NSString stringWithFormat:@"%@ %d",FSLocalizedForKey(@"kStyle"), i+1];
        item.tag = i;
        [item setTitleColor:[UIColor fs_whiteColor] forState:UIControlStateNormal];
        item.titleLabel.font = self.fs_standardImportantFont_IF2;
        item.titleLabel.textAlignment = NSTextAlignmentCenter;
        UIColor *color = [UIColor colorWithHexString:colorsHex[i]];
        [BindThemeKeySel(item, setBackgroundImage:forState:) fs_updateThemeCompleted:^(FSItem *  _Nonnull target, SEL  _Nonnull sel) {
            UIImage *normalImage = [[UIImage fs_imageWithColor:color size:CGSizeMake(itemSize.width - 8, itemSize.height - 8)] fs_borderWidth:4 borderColor:[UIColor fs_clearColor] cornerRadius:31.f];
            UIImage *selectedImage = [[UIImage fs_imageDrawRingWithRingW:2 borderColor:target.fs_primaryColor size:itemSize]  fs_addImage:normalImage];
            [item setBackgroundImage:selectedImage forState:UIControlStateSelected];
            [item setBackgroundImage:normalImage forState:UIControlStateNormal];
        }];
        @fs_weakify(self)
        item.onTapClick = ^(FSItem * _Nonnull _item) {
            @fs_strongify(self)
            if (self.currentStyleItem != _item) {
                self.currentStyleItem.selected = NO;
                _item.selected = YES;
                self.currentStyleItem = _item;
                !self.action ? :self.action (_item.tag);
            }
        };
        
        [styleOptionsView addArrangedSubview:item];
    }
    
    [self addSubview:styleOptionsView];
    [styleOptionsView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.fs_equalToSuperview().offset(ITEM_MARGIN_NORMAL);
        make.centerX.fs_equalToSuperview();
        make.width.offset(MIN_FOREGROUND_KEY_WINDOW_WIDTH);
        make.height.offset(70.f);
    }];
    
    self.styleOptionsView = styleOptionsView;
    
    UIImageView *pdfThumbnail = [UIImageView new];
    [self addSubview:pdfThumbnail];
    [pdfThumbnail mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(styleOptionsView.mas_bottom).offset(37.f);
        make.centerX.fs_equalToSuperview();
        make.width.offset(180.f);
        make.height.offset(55.f);
    }];
    _pdfThumbnail = pdfThumbnail;
}

- (void)layoutSubviews{
    [super layoutSubviews];
    CGSize itemSize = CGSizeMake(70.f, 70.f);
    self.styleOptionsView.spacing = (MIN(self.fs_width, IPAD_PREFERENCE_BAR_WIDTH)  - itemSize.width * _colorsHex.count) / (_colorsHex.count - 1);
    [self.styleOptionsView mas_updateConstraints:^(MASConstraintMaker *make) {
        make.width.offset(MIN(self.fs_width, IPAD_PREFERENCE_BAR_WIDTH));
    }];
}

- (void)setColorHex:(NSString *)colorHex{
    _colorHex = colorHex;
    for (int i = 0; i < _colorsHex.count; i ++) {
        NSString *colorHex = _colorsHex[i];
        if ([colorHex isEqualToString:_colorHex]) {
            FSItem *item = self.styleOptionsView.arrangedSubviews[i];
            item.onTapClick(item);
            break;
        }
    }
}

@end

@interface FSAddTextStampViewController ()
@property (nonatomic, strong) FSTAddTextStampTextFieldCell *textFieldCell;
@property (nonatomic, strong) FSTAddTextStampSwitchCell *authorCell;
@property (nonatomic, strong) FSTAddTextStampSwitchCell *dateCell;
@property (nonatomic, strong) FSTAddTextStampSwitchCell *titmeCell;
@property (nonatomic, strong) FSTAddTextStampStyleFooterView *stampStyleFooter;
@property (nonatomic, strong) FSCustomTextStampAttributes *stampAttributes;
@end

@implementation FSAddTextStampViewController

- (void)initialize{
    [super initialize];
    self.title = FSLocalizedForKey(@"kAddTextStamp");
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    self.navBarLeftTitle = FSLocalizedForKey(@"kCancel");
    self.navBarRightTitle = FSLocalizedForKey(@"kDone");
    @fs_weakify(self)
    self.navBar.rightItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @fs_strongify(self)
        [self dismissViewControllerAnimated:YES completion:^{}];
        !self.didClickedDone ? : self.didClickedDone(self.stampAttributes);
    };
    self.stampAttributes = self.params;
    if (!self.stampAttributes) {
        self.stampAttributes = [FSCustomTextStampAttributes new];
        self.stampAttributes.creatAuthor = ((FSViewControllerManager *)[self services]).extensionsManager.annotAuthor;
        self.stampAttributes.text = self.textFieldCell.textField.text.length ? self.textFieldCell.textField.text :  self.textFieldCell.textField.placeholder;
        self.navBar.rightItem.enabled = NO;
    }

    self.tableView.tableFooterView = self.stampStyleFooter;
    [self drawPdfThumbnail];
}

- (void)drawPdfThumbnail{
    FSPDFPage *page = [[self.stampAttributes currentDoc] getPage:0];
    UIImage *image = [Utility drawPage:page targetSize:CGSizeMake(180, 55) shouldDrawAnnotation:YES needPause:nil rotation:0 enableForPrint:NO pdfViewCtrl:nil];
    self.stampStyleFooter.pdfThumbnail.image = image;
}

- (BOOL)emptyDataSetShouldDisplay:(UITableView *)tableView{
    return NO;
}

- (__kindof UITableViewCell *)tableView:(UITableView *)tableView dequeueReusableCellWithIdentifier:(NSString *)identifier forIndexPath:(NSIndexPath *)indexPath withDataItem:(id)dataItem{
    if (indexPath.row == 0) {
        return self.textFieldCell;
    }else if (indexPath.row == 1) {
        return self.authorCell;
    }else if (indexPath.row == 2) {
        return self.dateCell;
    }
    return self.titmeCell;
}

#pragma mark - UITableViewDataSource

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section{
    return 4;
}


/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/


- (FSTAddTextStampTextFieldCell *)textFieldCell{
    if(!_textFieldCell){
        _textFieldCell = [[FSTAddTextStampTextFieldCell alloc] init];
        _textFieldCell.textField.text = self.stampAttributes.text;
        @fs_weakify(self)
        _textFieldCell.action = ^(NSString *text) {
            @fs_strongify(self)
            if (self.stampAttributes.text.hash == text.hash)  return;
            self.navBar.rightItem.enabled = self.textFieldCell.textField.text.length;
            self.stampAttributes.text = text;
            [self drawPdfThumbnail];
        };
    }
    return _textFieldCell;
}

- (FSTAddTextStampSwitchCell *)authorCell{
    if(!_authorCell){
        _authorCell = [[FSTAddTextStampSwitchCell alloc] initWithText:FSLocalizedForKey(@"kAuthor")];
        _authorCell.switchBtn.on = self.stampAttributes.hasAuthor;
        @fs_weakify(self)
        _authorCell.action = ^(BOOL on) {
            @fs_strongify(self)
            self.stampAttributes.hasAuthor = on;
            [self drawPdfThumbnail];
        };
    }
    return _authorCell;
}

- (FSTAddTextStampSwitchCell *)dateCell{
    if(!_dateCell){
        _dateCell = [[FSTAddTextStampSwitchCell alloc] initWithText:FSLocalizedForKey(@"kDate")];
        _dateCell.switchBtn.on = self.stampAttributes.hasDate;
        @fs_weakify(self)
        _dateCell.action = ^(BOOL on) {
            @fs_strongify(self)
            self.stampAttributes.hasDate = on;
            [self drawPdfThumbnail];
        };
    }
    return _dateCell;
}

- (FSTAddTextStampSwitchCell *)titmeCell{
    if(!_titmeCell){
        _titmeCell = [[FSTAddTextStampSwitchCell alloc] initWithText:FSLocalizedForKey(@"kTime")];
        _titmeCell.switchBtn.on = self.stampAttributes.hasTime;
        @fs_weakify(self)
        _titmeCell.action = ^(BOOL on) {
            @fs_strongify(self)
            self.stampAttributes.hasTime = on;
            [self drawPdfThumbnail];
        };
    }
    return _titmeCell;
}

- (FSTAddTextStampStyleFooterView *)stampStyleFooter{
    if(!_stampStyleFooter){
        _stampStyleFooter = [[FSTAddTextStampStyleFooterView alloc] initWithFrame:CGRectMake(0, 0, 0, 308) colorsHex:self.stampAttributes.colorsHex];
        _stampStyleFooter.colorHex = self.stampAttributes.colorHex;
        @fs_weakify(self)
        _stampStyleFooter.action = ^(NSInteger style) {
            @fs_strongify(self)
            self.stampAttributes.colorHex = [self.stampAttributes colorsHex][style];
            [self drawPdfThumbnail];
        };
    }
    return _stampStyleFooter;
}
@end
