/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSCustomStampAttributes.h"
#import "StampUtil.h"
#import "YYKitMacro.h"


@implementation FSCustomStampAttributes
- (FSPDFDoc *)currentDoc{return nil;}

+ (NSDictionary *)modelCustomPropertyMapper {
    return @{};
}
@end


@implementation FSCustomTextStampAttributes
{
    NSArray *_colorsHex;
    FSPDFDoc *_currentDoc;
    FSRectF *_stampTextTopRectF;
    FSRectF *_stampTextCenterRectF;
}
- (instancetype)init{
    self = [super init];
    if (self) {
        _stampTextTopRectF =  [[FSRectF alloc] initWithLeft1:5.509960 bottom1:14.063001 right1:94.489998 top1:26.366999];
        _stampTextCenterRectF = [[FSRectF alloc] initWithLeft1:3.98593 bottom1:3.632810 right1:95.897003 top1:26.366999];
        _hasAuthor = YES;
        _hasDate = YES;
        _hasTime = YES;
        _colorHex = [self colorsHex][0];
    }
    return self;
}

- (void)setText:(NSString *)text{
    _text = text;
    if (_currentDoc) {
        FSForm *form = [[FSForm alloc] initWithDocument:_currentDoc];
        FSField *field = [form getField:0 filter:@"Stamp_Text"];
        [field setValue:text];
        FSControl *control = [field getControl:0];
        [[control getWidget] resetAppearanceStream];
    }
}

- (void)setHasAuthor:(BOOL)hasAuthor{
    _hasAuthor = hasAuthor;
    [self adjustStampTextToDynamic:NO];
}

- (void)setHasDate:(BOOL)hasDate{
    _hasDate = hasDate;
    [self adjustStampTextToDynamic:NO];
}

- (void)setHasTime:(BOOL)hasTime{
    _hasTime = hasTime;
    [self adjustStampTextToDynamic:NO];
}

- (NSString *)currentAuthor{
    if (!_currentAuthor) {
        return _creatAuthor;
    }
    return _currentAuthor;
}

- (void)setColorHex:(NSString *)colorHex{
    _colorHex = colorHex;
    if (_currentDoc) {
        FSPDFPage *page = [_currentDoc getPage:0];
        for (int i = 0; i < page.getAnnotCount; i++) {
            FSAnnot *annot = [page getAnnot:i];
            if (annot.type == FSAnnotSquare) {
                FSSquare *square = [[FSSquare alloc] initWithAnnot:annot];
                if (i == 0) {
                    square.borderColor = [UIColor colorWithHexString:_colorHex].rgbValue;
                    [square resetAppearanceStream];
                }else if (i == 1){
                    square.fillColor = [UIColor colorWithHexString:_colorHex].rgbValue;
                    [square resetAppearanceStream];
                }
            }
        }
    }
}

- (void)adjustStampTextToDynamic:(BOOL)dynamic{
    if (_currentDoc) {
        FSForm *form = [[FSForm alloc] initWithDocument:_currentDoc];
        FSField *field = [form getField:0 filter:@"Stamp_Text"];
        FSWidget *widget = [[field getControl:0] getWidget];
        if (self.hasAuthor || self.hasDate || self.hasTime) {
            widget.fsrect = _stampTextTopRectF;
        }else{
            widget.fsrect = _stampTextCenterRectF;
        }
        [widget resetAppearanceStream];
        
        field = [form getField:0 filter:@"Author_Date_Time_Text"];
        if (self.hasAuthor || self.hasDate || self.hasTime) {
            if (![field isEmpty]) {
                widget = [[field getControl:0] getWidget];
            }else{
                FSDefaultAppearance *ap = [[FSDefaultAppearance alloc] init];
                ap.flags = FSDefaultAppearanceFlagFont | FSDefaultAppearanceFlagTextColor | FSDefaultAppearanceFlagFontSize;
                ap.font = [[FSFont alloc] initWithFont_id:FSFontStdIDHelveticaB];
                ap.text_size = 0;
                ap.text_color = 0xffffff;
                
                FSRectF *rectF = [[FSRectF alloc] initWithLeft1:3.96341 bottom1:3.57868 right1:95.731995 top1:15.99];
                FSPDFPage *page = [_currentDoc getParsedPage:0];
                FSControl *control = [form addControl:page field_name:@"Author_Date_Time_Text" field_type:FSFieldTypeTextField rect:rectF];
                widget = [control getWidget];
                [control setDefaultAppearance:ap];
            }
            [self adjustAuthorDateTimeWidget:widget needDynamic:dynamic];
        }else{
            if (![field isEmpty]) {
                [form removeField:field];
            }
        }
    }
}

- (void)adjustAuthorDateTimeWidget:(FSWidget *)widget needDynamic:(BOOL)needDynamic{
    if (needDynamic) {
        [self adjustWidgetAActionJavaScript:widget];
        [[widget getField] reset];
    }else{
        NSMutableString *text = [NSMutableString string];
        if (self.hasAuthor || self.hasDate || self.hasTime) {
            if (self.hasAuthor) {
                [text appendString:self.currentAuthor ? self.currentAuthor : @""];
            }
            if (self.hasDate || self.hasTime) {
                NSString *dateFormatStr = @"HH:mm:ss,MM/dd/yyyy";
                if (!self.hasDate) {
                    dateFormatStr = [dateFormatStr stringByReplacingOccurrencesOfString:@",MM/dd/yyyy" withString:@""];
                }
                if (!self.hasTime) {
                    dateFormatStr = [dateFormatStr stringByReplacingOccurrencesOfString:@"HH:mm:ss," withString:@""];
                }
                NSDate *date;
                if (!self.creatTime) {
                    date = [NSDate date];
                    self.creatTime = date.timeIntervalSince1970;
                }else{
                    date = [NSDate dateWithTimeIntervalSince1970:self.creatTime];
                }
                NSString *dateStr = [date fs_stringWithFormat:dateFormatStr];
                if (self.hasAuthor) {
                    [text appendFormat:@" , %@",dateStr];
                }else{
                    [text appendString:dateStr];
                }
            }
        }else{
            [text setString:@""];
        }
        [[widget getField] setValue:text];
        [widget resetAppearanceStream];
    }
}

- (void)adjustWidgetAActionJavaScript:(FSWidget *)widget{
    NSMutableString *javaScriptStr = [NSMutableString stringWithString:@"event.value = "];
    if (self.hasAuthor || self.hasDate || self.hasTime) {
        if (self.hasAuthor) {
            [javaScriptStr appendString:@"((!identity.name || identity.loginName != (event.source.source || this).Collab.user) ? (event.source.source || this).Collab.user : identity.name)"];
        }
        if (self.hasDate || self.hasTime) {
            NSString *dateFormat = @"HH:MM:ss,mm/dd/yyyy";
            if (!self.hasDate) {
                dateFormat = [dateFormat stringByReplacingOccurrencesOfString:@",mm/dd/yyyy" withString:@""];
            }
            if (!self.hasTime) {
                dateFormat = [dateFormat stringByReplacingOccurrencesOfString:@"HH:MM:ss," withString:@""];
            }
            [javaScriptStr insertString:[NSString stringWithFormat:@"event.value = (new Date()).toString();AFDate_FormatEx(\"%@\");", dateFormat] atIndex:0];
            if (self.hasAuthor) {
                [javaScriptStr appendString:@"+ \" , \" + event.value"];
            }else{
                [javaScriptStr appendString:@"event.value"];
            }
        }
        [javaScriptStr appendString:@";"];

    }else{
        [javaScriptStr setString:@""];
    }
    [widget addAActionJavaScript:javaScriptStr actionTriggerEvent:FSAdditionalActionTriggerFieldRecalculateValue];
}

- (FSPDFDoc *)creatDynamicStampDoc{
    FSPDFDoc *doc = self.currentDoc;
    [self adjustStampTextToDynamic:YES];
    return doc;
}

- (FSPDFDoc *)currentDoc{
    if(!_currentDoc){
        CGSize size = CGSizeMake(100, 30);
        FSPDFDoc *doc = [FSPDFDoc new];
        FSPDFPage *page = [doc insertPage:0 width:size.width height:size.height];
        
        FSRectF *rectF = [[FSRectF alloc] initWithLeft1:0 bottom1:0 right1:100 top1:30];
        FSAnnot *annot = [page addAnnot:FSAnnotSquare rect:rectF];
        FSSquare *square = [[FSSquare alloc] initWithAnnot:annot];
        square.borderColor = [UIColor colorWithHexString:self.colorHex].rgbValue;
        square.lineWidth = 2;
        [square resetAppearanceStream];
        
        rectF = [[FSRectF alloc] initWithLeft1:2 bottom1:2 right1:98 top1:28];
        annot = [page addAnnot:FSAnnotSquare rect:rectF];
        square = [[FSSquare alloc] initWithAnnot:annot];
        square.borderColor = [UIColor fs_whiteColor].rgbValue;
        square.fillColor = [UIColor colorWithHexString:self.colorHex].rgbValue;
        square.lineWidth = 1.2;
        [square resetAppearanceStream];

        @try {
            FSForm *form = [[FSForm alloc] initWithDocument:doc];
            FSDefaultAppearance *ap = [[FSDefaultAppearance alloc] init];
            ap.flags = FSDefaultAppearanceFlagFont | FSDefaultAppearanceFlagTextColor | FSDefaultAppearanceFlagFontSize;
            ap.font = [[FSFont alloc] initWithFont_id:FSFontStdIDHelveticaB];
            ap.text_size = 0;
            ap.text_color = 0xffffff;
            
            if (self.hasAuthor || self.hasDate || self.hasTime) {
                rectF = [[FSRectF alloc] initWithLeft1:3.96341 bottom1:3.57868 right1:95.731995 top1:15.99];
                FSControl *control = [form addControl:page field_name:@"Author_Date_Time_Text" field_type:FSFieldTypeTextField rect:rectF];
                FSWidget *widget = [control getWidget];
                [self adjustAuthorDateTimeWidget:widget needDynamic:NO];
                [control setDefaultAppearance:ap];
                [widget resetAppearanceStream];
                
                rectF = _stampTextTopRectF;
            }else{
                rectF = _stampTextCenterRectF;
            }
            FSControl *control = [form addControl:page field_name:@"Stamp_Text" field_type:FSFieldTypeTextField rect:rectF];
            FSWidget *widget = [control getWidget];
            FSField *field = [widget getField];
            [field setValue:self.text];
            ap.font = [[FSFont alloc] initWithFont_id:FSFontStdIDHelvetica];
            [control setDefaultAppearance:ap];
            [widget resetAppearanceStream];
        } @catch (NSException *exception) {
        }
        _currentDoc = doc;
    }
    return _currentDoc;
}

- (NSArray<NSString *> *)colorsHex{
    if (!_colorsHex) {
        _colorsHex = @[@"#CA4F2E", @"#4EA882", @"#216AB9", @"#923094"];
    }
    return _colorsHex;
}

+ (nullable NSArray<NSString *> *)modelPropertyBlacklist{
    return @[ @keypath([FSCustomTextStampAttributes new], currentAuthor)];
}

+ (NSDictionary *)modelCustomPropertyMapper {
    return [super modelCustomPropertyMapper];
}
@end

@implementation FSCustomImageStampAttributes

@synthesize image = _image;
- (instancetype)init{
    self = [super init];
    if (self) {
        self.creatTime = [NSDate date].timeIntervalSince1970;
        _opacity = 1.f;
    }
    return self;
}

- (CGSize)stampScaleByImage{
    CGFloat w = 100;
    CGFloat h = 30;
    CGSize imageSize = self.image.size;
    CGFloat scalePage = w / h;
    CGFloat scaleImage = imageSize.width / imageSize.height;
    if (scalePage > scaleImage) {
        w = (h / imageSize.height) * imageSize.width;
    }else{
        h = (w / imageSize.width) * imageSize.height;
    }
    return  CGSizeMake(w, h);
}

- (CGSize)imageConverToPDFSize{
    
    NSString *imagePath;
    if (self.imageName.length) {
        imagePath = [NSString stringWithFormat:@"%@/%@",CUSTOM_IMAGE_STAMP_IMAGE_CACHE,self.imageName];
    }else if (self.image){
        imagePath = [NSString stringWithFormat:@"%@.png",[Utility temporaryPath]];
        if (![UIImagePNGRepresentation(self.image) writeToFile:imagePath atomically:YES]) {
            return CGSizeZero;
        }
    }
    
    @try {
        NSString *pdfPath = [NSString stringWithFormat:@"%@.pdf",[Utility temporaryPath]];
        [FSConvert fromImage:imagePath saved_pdf_path:pdfPath];
        FSPDFDoc *pdfDoc = [[FSPDFDoc alloc] initWithPath:pdfPath];

        if ([pdfDoc load:nil] == FSErrSuccess) {
            FSPDFPage *pdfPage = [pdfDoc getPage:0];
            return CGSizeMake([pdfPage getWidth], [pdfPage getHeight]);
        }
    } @catch (NSException *exception) {

    }
    return CGSizeZero;
}

- (FSPDFDoc *)creatImageStampDoc{
    NSString *imagePath;
    if (self.imageName.length) {
        imagePath = [NSString stringWithFormat:@"%@/%@",CUSTOM_IMAGE_STAMP_IMAGE_CACHE,self.imageName];
    }else if (self.image){
        imagePath = [NSString stringWithFormat:@"%@.png",[Utility temporaryPath]];
        if (![UIImagePNGRepresentation(self.image) writeToFile:imagePath atomically:YES]) {
            return nil;
        }
    }
    
    @try {
        NSString *pdfPath = [NSString stringWithFormat:@"%@.pdf",[Utility temporaryPath]];
        [FSConvert fromImage:imagePath saved_pdf_path:pdfPath];
        FSPDFDoc *pdfDoc = [[FSPDFDoc alloc] initWithPath:pdfPath];
        
        if ([pdfDoc load:nil] == FSErrSuccess) {
            return pdfDoc;
        }
    } @catch (NSException *exception) {
        CGSize size = [self stampScaleByImage];
        if (CGSizeEqualToSize(size, CGSizeZero)) {
            return nil;
        }
        CGFloat w = size.width;
        CGFloat h = size.height;
        
        FSPDFDoc *doc = [FSPDFDoc new];
        FSPDFPage *page = [doc insertPage:0 width:w height:h];
        
        FSRectF *rectF = [[FSRectF alloc] initWithLeft1:0 bottom1:0 right1:w top1:h];
        
        FSScreen *screen = [[FSScreen alloc] initWithAnnot:[page addAnnot:FSAnnotScreen rect:rectF]];
        if (![screen getCptr]) {
            return nil;
        }
        screen.opacity = self.opacity;
        FSImage *fs_image = [Utility uiimageToFSImage:self.image];
        if (fs_image) {
            [screen setImage:fs_image frame_index:0 compress:0];
        }else{
            return nil;
        }
        [screen resetAppearanceStream];
        return doc;
    }
    return nil;
}

- (NSString *)imageName{
    if (!_imageName) {
        if (!_image) {
            return nil;
        }
        NSString *imageName = [NSString stringWithFormat:@"%@.png",[Utility getUUID]];
        NSString *imagePath = [NSString stringWithFormat:@"%@/%@",CUSTOM_IMAGE_STAMP_IMAGE_CACHE,imageName];
        if ([UIImagePNGRepresentation(_image) writeToFile:imagePath atomically:YES]) {
            _imageName = imageName;
        }
    }
    return _imageName;
}

- (UIImage *)image{
    if (!_image) {
        if (!_imageName) {
            return nil;
        }
        NSString *imagePath = [NSString stringWithFormat:@"%@/%@",CUSTOM_IMAGE_STAMP_IMAGE_CACHE,_imageName];
        _image = [UIImage imageWithContentsOfFile:imagePath];
    }
    return _image;
}

- (void)setImage:(UIImage *)image{
    if (_image == image) {
        return;
    }
    _image = image;
    if (_imageName) {
        NSString *imagePath = [NSString stringWithFormat:@"%@/%@",CUSTOM_IMAGE_STAMP_IMAGE_CACHE,_imageName];
        [UIImagePNGRepresentation(image) writeToFile:imagePath atomically:YES];
    }
}

- (void)copyImageFileWithPath:(NSString *)path{
    if ([[NSFileManager defaultManager] fileExistsAtPath:path]) {
        NSString *imageName = [NSString stringWithFormat:@"%@.png",[Utility getUUID]];
        NSString *imagePath = [NSString stringWithFormat:@"%@/%@",CUSTOM_IMAGE_STAMP_IMAGE_CACHE,imageName];
        if ([[NSFileManager defaultManager] copyItemAtPath:path toPath:imagePath error:nil]) {
            _imageName = imageName;
        }
    }
}

- (void)clearImage{
    _image = nil;
    NSString *imagePath = [NSString stringWithFormat:@"%@/%@",CUSTOM_IMAGE_STAMP_IMAGE_CACHE,_imageName];
    if ([[NSFileManager defaultManager] fileExistsAtPath:imagePath]) {
        [[NSFileManager defaultManager] removeItemAtPath:imagePath error:nil];
    }
    _imageName = nil;
}

+ (nullable NSArray<NSString *> *)modelPropertyBlacklist{
    return @[ @keypath([FSCustomImageStampAttributes new], image)];
}

+ (NSDictionary *)modelCustomPropertyMapper {
    return [super modelCustomPropertyMapper];
}
@end

