/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSImageStampOpacityViewController.h"
#import "FSCustomStampAttributes.h"
#import "FSSlider.h"

@interface FSImageStampOpacityViewController ()
@property (nonatomic, strong) UIImageView *imageView;
@property (nonatomic, strong) UIButton *unionStyleBtn;
@property (nonatomic, strong) FSSlider *slider;
@property (nonatomic, strong) UIView *layoutView;
@property (nonatomic, strong) FSCustomImageStampAttributes *imageStampAttributes;
@end

@implementation FSImageStampOpacityViewController

- (void)initialize{
    [super initialize];
    self.shouldBindTemeForStatusBar = NO;
    self.shouldShowBottomBar = YES;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    
    if (self.params) {
        self.imageStampAttributes = self.params;
    }
    
    [self configNavBar];
    
    [self addImageView];
    
    [self configBottomBar];
    
    [self addUnionStyle];
    
    self.view.backgroundColor = self.fs_backgroundColor_B5;

}

- (void)configNavBar{
    self.navBarLeftTitle = FSLocalizedForKey(@"kCancel");
    [self.navBar.leftItem setTitleColor:[UIColor fs_whiteColor] forState:UIControlStateNormal];
    self.navBarRightTitle = FSLocalizedForKey(@"kDone");
    [self.navBar.rightItem setTitleColor:[UIColor fs_whiteColor] forState:UIControlStateNormal];
    @fs_weakify(self)
    self.navBar.rightItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @fs_strongify(self)
        [self dismissViewControllerAnimated:YES completion:self.dismissCompletion];
        self.imageStampAttributes.opacity = self.slider.value;
        !self.didClickedDone ? : self.didClickedDone(self.imageStampAttributes);
    };
    self.title = FSLocalizedForKey(@"kOpacity");
    [self.navBar.titleItem setTitleColor:[UIColor fs_whiteColor] forState:UIControlStateNormal];
    
    self.navBar.hasDivide = NO;
    self.navBar.barTintColor = [UIColor fs_clearColor];
    self.navBar.stateBar.backgroundColor = [UIColor fs_clearColor];
}

- (void)addImageView{
    _imageView = [UIImageView new];
    _imageView.image = self.imageStampAttributes.image;
    _imageView.alpha = self.imageStampAttributes.opacity;
    _imageView.contentMode = UIViewContentModeScaleAspectFit;
    [self.view addSubview:_imageView];
    
    [_imageView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.fs_equalToSuperview().offset(ITEM_MARGIN_NORMAL);
        make.right.fs_equalToSuperview().offset(-ITEM_MARGIN_NORMAL);
        make.top.equalTo(self.navBar.mas_bottom).offset(8.f);
        make.bottom.equalTo(self.bottomBar.mas_top).offset(-40.f);
    }];
}

- (void)configBottomBar{
    self.bottomBar.backgroundColor = [[UIColor fs_blackColor] colorWithAlphaComponent:0.4];
    self.bottomBar.hasDivide = NO;
    
    FSSlider *slider = [FSSlider new];
    [slider setIsAccessibilityElement:YES];
    slider.accessibilityLabel = [NSString stringWithFormat:@"%@ %d%%",FSLocalizedForKey(@"kOpacity"), (int)(slider.value * 100)];
    [slider setThumbImage:[[UIImage fs_imageWithColor:[UIColor fs_whiteColor] size:CGSizeMake(18.f, 18.f)] fs_borderWidth:1.f borderColor:UIColorHex(#A9A9A9) cornerRadius:9.f] forState:UIControlStateNormal];
    slider.fs_allowTapAction = YES;
    slider.value = self.imageStampAttributes.opacity;
    BindThemeKeyPath(slider, tintColor);
    [slider addTarget:self action:@selector(changeOpacity:) forControlEvents:UIControlEventValueChanged];
    [self.bottomBar addSubview:slider];
    [slider mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.fs_equalToSuperview().offset(56.f);
        make.right.fs_equalToSuperview().offset(-56.f);
        make.centerY.fs_equalToSuperview();
    }];
    self.slider = slider;
}

- (void)addUnionStyle{
    UIView *layoutView = [UIView new];
    layoutView.backgroundColor = [UIColor fs_clearColor];
    [self.view addSubview:layoutView];
    self.layoutView = layoutView;
    
    UIButton *btn = [UIButton new];
    btn.titleEdgeInsets = UIEdgeInsetsMake(0, 0, 6, 0);
    [btn setBackgroundImage:FSImageNamed(@"common_union") forState:UIControlStateNormal];
    [btn setTitleColor:UIColorHex(#2E2E2E) forState:UIControlStateNormal];
    btn.titleLabel.font = self.fs_standardWeakFont_WF1;
    [btn setTitle:[NSString stringWithFormat:@"%d%%",(int)(self.imageStampAttributes.opacity * 100)] forState:UIControlStateNormal];
    
    [self.view addSubview:btn];
    [btn mas_makeConstraints:^(MASConstraintMaker *make) {
        make.centerX.equalTo(layoutView.mas_right);
        make.bottom.equalTo(self.bottomBar.mas_top).offset(6);
    }];
    self.unionStyleBtn = btn;
}

- (void)viewDidLayoutSubviews{
    [super viewDidLayoutSubviews];
    [self updateLayoutViewConstraints];
}

- (void)changeOpacity:(UISlider *)slider{
    slider.accessibilityValue = [NSString stringWithFormat:@"%@ %d%%",FSLocalizedForKey(@"kOpacity"), (int)(slider.value * 100)];
    self.imageView.alpha = slider.value;
    [self.unionStyleBtn setTitle:[NSString stringWithFormat:@"%d%%",(int)(self.slider.value * 100)] forState:UIControlStateNormal];
    [self updateLayoutViewConstraints];
}

- (void)updateLayoutViewConstraints{
    CGRect thumbRectForBounds = [self.slider thumbRectForBounds:self.slider.bounds trackRect:[self.slider trackRectForBounds:self.slider.bounds] value:self.slider.value];
    [self.layoutView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.slider);
        make.top.equalTo(self.bottomBar.mas_top);
        make.width.offset(CGRectGetMidX(thumbRectForBounds));
        make.height.offset(1.f);
    }];
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
