/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSStampIdentityViewController.h"
#import "StampIdentityModel.h"
#import "Utility.h"
#import "StampUtil.h"

@interface FSStampIdentityViewController ()
@property (nonatomic, strong) NSMutableArray *itemArray;

@end

@implementation FSStampIdentityViewController

-(void)initialize{
    [super initialize];
        
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    if (self = [super init]) {
        self.extensionsManager = extensionsManager;
    }
    return self;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    self.tableView.backgroundColor = self.fs_backgroundColor_B9;
    self.title = FSLocalizedForKey(@"kStampIdentity");
    self.navBar.hasDivide = YES;
    [self.navBar.leftItem setTitle:FSLocalizedForKey(@"kCancel") forState:UIControlStateNormal];
    [self.navBar.leftItem mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.navBar.leftItem.superview.fs_mas_left).offset(12);
        make.centerY.mas_equalTo(self.navBar.leftItem.superview);
    }];
    [self.navBar.leftItem setImage:nil forState:UIControlStateNormal];
    @weakify(self)
    self.navBar.leftItem.onTapClick = ^(FSItem *_item) {
        @strongify(self)
        [self dismissViewControllerAnimated:NO completion:^{
            nil;
        }];
    };
    [self.navBar.rightItem setTitle:FSLocalizedForKey(@"kOK") forState:UIControlStateNormal];
    self.navBar.rightItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [self dismissViewControllerAnimated:YES completion:^{
            @strongify(self)
            NSString *jsonString = [[NSUserDefaults standardUserDefaults] objectForKey:Key_Identity];
            NSMutableDictionary *identity = [[NSMutableDictionary alloc] init];
            if(jsonString && ![jsonString isEqualToString:@""]){
                identity = [Utility dictionaryWithJsonString:jsonString];
            }
            
            for (int i = 1  ;i< self.itemArray.count;i++) {
                StampIdentityModel *model = self.itemArray[i];
                    identity[model.name] = model.value;
            }
            [[NSUserDefaults standardUserDefaults] setObject:[Utility convertToJSONData:identity] forKey:Key_Identity];
            [[NSUserDefaults standardUserDefaults] synchronize];
            if(self.delegate){
                [self.delegate onOkClick];
            }
        }];
    };
    [self loadData];

}

-(void)loadData{
    self.itemArray = [[NSMutableArray alloc] init];
    NSString *jsonString = [[NSUserDefaults standardUserDefaults] objectForKey:Key_Identity];
    NSMutableDictionary *identity = [[NSMutableDictionary alloc] init];
    if(jsonString && ![jsonString isEqualToString:@""]){
        identity = [Utility dictionaryWithJsonString:jsonString];
    }
    
    StampIdentityModel *model = [[StampIdentityModel alloc] init];
    model.name = FSLocalizedForKey(@"kStampLoginName");
    
    model.value = _extensionsManager.annotAuthor;
    if([self.extensionsManager.appInfoProvider getUserName] != nil && ![@"" isEqualToString:[self.extensionsManager.appInfoProvider getUserName]]){
        model.value = [self.extensionsManager.appInfoProvider getUserName];
    }
    [self.itemArray addObject:model];
    StampIdentityModel *model2 = [[StampIdentityModel alloc] init];
    model2.name = FSLocalizedForKey(@"kStampFirstName");
    model2.value = [identity objectForKey:FSLocalizedForKey(@"kStampFirstName")];
    [self.itemArray addObject:model2];
    StampIdentityModel *model3 = [[StampIdentityModel alloc] init];
    model3.name = FSLocalizedForKey(@"kStampLastName");
    model3.value = [identity objectForKey:FSLocalizedForKey(@"kStampLastName")];
    [self.itemArray addObject:model3];
    StampIdentityModel *model4 = [[StampIdentityModel alloc] init];
    model4.name = FSLocalizedForKey(@"kStampTitle");
    model4.value = [identity objectForKey:FSLocalizedForKey(@"kStampTitle")];
    [self.itemArray addObject:model4];
    StampIdentityModel *model5 = [[StampIdentityModel alloc] init];
    model5.name = FSLocalizedForKey(@"kStampOrganizationName");
    model5.value = [identity objectForKey:FSLocalizedForKey(@"kStampOrganizationName")];
    [self.itemArray addObject:model5];
    StampIdentityModel *model6 = [[StampIdentityModel alloc] init];
    model6.name = FSLocalizedForKey(@"kStampOrganizationUnit");
    model6.value = [identity objectForKey:FSLocalizedForKey(@"kStampOrganizationUnit")];
    [self.itemArray addObject:model6];
    StampIdentityModel *model7 = [[StampIdentityModel alloc] init];
    model7.name = FSLocalizedForKey(@"kStampEmail");
    model7.value = [identity objectForKey:FSLocalizedForKey(@"kStampEmail")];
    [self.itemArray addObject:model7];
    self.dataSource = self.itemArray;
}


#pragma mark - table view delegate handler
- (StampIdentityModel *)dataForAtIndexPath:(NSIndexPath *)indexPath{
    return self.itemArray[indexPath.row];
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (void)configureCell:(FSTitleEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(StampIdentityModel *)dataItem{
    cell.backgroundColor = self.fs_backgroundColor_B8;
    cell.estimatedTitleLabel.text = dataItem.name;
    
    if (!cell.rightView) {
        UITextField *textField = [[UITextField alloc] init];
        cell.rightView = textField;
        textField.textColor = self.fs_textColor_T3;
        textField.textAlignment = NSTextAlignmentRight;
        [textField setFont:[UIFont systemFontOfSize:14.0f]];
        textField.tag = indexPath.row;
        [textField addTarget:self action:@selector(textFieldTextChanged:) forControlEvents:UIControlEventEditingChanged];
        cell.rightMargin = 16;
        if(indexPath.row == 0){
            textField.enabled = NO;
        }else{
            textField.enabled = YES;

        }
        
    }
    if([cell.rightView isKindOfClass:UITextField.class]){
        UITextField *textField =  ((UITextField *)cell.rightView);
        if(!dataItem.value || [dataItem.value isEqualToString:@""]){
            if(indexPath.row == 1 || indexPath.row == 2){
                textField.placeholder = FSLocalizedForKey(@"kStampEntername");
            }else if(indexPath.row == 3){
                textField.placeholder = FSLocalizedForKey(@"kStampEnterTitle");
            }else if(indexPath.row == 4){
                textField.placeholder = FSLocalizedForKey(@"kStampEnterOrganizationName");
            }else if(indexPath.row == 5){
                textField.placeholder = FSLocalizedForKey(@"kStampEnterOrganizationUnit");
            }else if(indexPath.row == 6){
                textField.placeholder = FSLocalizedForKey(@"kStampEnterYourEmail");
            }
        }else{
            textField.text = dataItem.value;
        }
    }
}

- (void)textFieldTextChanged:(UITextField *)textField {
    StampIdentityModel *model = self.itemArray[textField.tag];
    model.value = textField.text;
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section{
    return 24;
}

- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section{
    UIView * headerView = [[UIView alloc] init];
    headerView.backgroundColor = self.fs_backgroundColor_B9;
    UILabel *lbText = [[UILabel alloc] init];
    [headerView addSubview:lbText];
    lbText.text = FSLocalizedForKey(@"kStampIdentity");
    lbText.font = self.fs_standardWeakFont_WF1;
    lbText.textColor = self.fs_textColor_T4;
    [lbText mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(headerView.fs_mas_left).offset(16);
        make.centerY.mas_equalTo(headerView);
        
    }];
    return headerView;
}

@end

