/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "StampIconController.h"
#import "FSCornerRadiusStackView.h"
#import "FSPagerView.h"
#import "FSCollectionViewCell.h"
#import "FSCollectionView.h"
#import "FSAddTextStampViewController.h"
#import "FSCustomStampAttributes.h"
#import "StampUtil.h"
#import "FSMediaFileToPDFViewController.h"
#import "FSImageStampOpacityViewController.h"

@class FSStampTabItem;
@interface StampIconController ()<FSPagerViewDataSource, FSPagerViewDelegate, FSTabPagerBarDelegate, FSTabPagerBarDataSource>
{
    FSPDFViewCtrl * __weak _pdfViewCtrl;
}
@property (nonatomic, weak) UIExtensionsManager *extensionManager;
@property (nonatomic, weak) FSReadToolSettings *toolSettings;
@property (nonatomic, assign) BOOL isiPhoneLandscape;

@property (nonatomic, strong) FSToolbar *topBar;

@property (nonatomic, strong) FSTabPagerBar *tabPagerBar;
@property (nonatomic, strong) FSPagerView *pagerContentView;
@property (nonatomic, weak) FSStampTabItem *currentTabItem;
@property (nonatomic, copy) NSArray <FSStampTabItem *>* tabItems;
@end

@implementation StampButton

- (void)setStampIcon:(NSInteger)stampIcon
{
    _stampIcon = stampIcon;
    [self setVoiceOverStringFromIcon:stampIcon];
}

- (void)setVoiceOverStringFromIcon:(int)stampIcon
{
    NSArray *stampStr = @[@"approved",@"completed",@"confidential",@"draft",@"emergency",@"expired",@"final",@"received",@"reviewed",@"revised",@"verified",@"void",@"accepted",@"initial",@"rejected",@"sign here",@"witness",@"approved",@"confidential",@"received",@"received",@"revised"];
    if (stampIcon < stampStr.count && stampIcon > -1) {
        self.accessibilityLabel = stampStr[stampIcon];
        if (stampIcon < 12) {
            self.accessibilityValue = FSLocalizedForKey(@"kStandardStamps");
        }else if(stampIcon < 17){
            self.accessibilityValue = FSLocalizedForKey(@"kSignHere");
        }else{
            self.accessibilityValue = FSLocalizedForKey(@"kDynamicStamps");
        }
    }
}

- (void)setLayerBorderWidth:(CGFloat)borderWidth{
    [self fs_setLayerCornerRadius:5.0f];
    [BindThemeKeySel(self, fs_setLayerBorderWidth:borderColor:) fs_updateThemeCompleted:^(StampButton * _Nonnull target, SEL  _Nonnull sel) {
        [target fs_setLayerBorderWidth:borderWidth borderColor:target.fs_primaryColor];
    }];
}

@end


@interface FSStampCollectionViewCell : FSCollectionViewCell
@end

@implementation FSStampCollectionViewCell
- (instancetype)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:frame];
    if (self) {
        self.checkPosition = FSCollectionCellCheckItemPositionTopRight;
        StampButton *stampBtn = [[StampButton alloc] initWithFrame:CGRectMake(0, 0, 160, 61)];
        stampBtn.contentEdgeInsets = UIEdgeInsetsMake(10, 10, 10, 10);
        stampBtn.tag = 100;
        [self.contentView addSubview:stampBtn];
        [stampBtn mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.right.top.bottom.fs_equalToSuperview();
        }];
        
        UIImageView *stampImageView = [UIImageView new];
        stampImageView.opaque = NO;
        stampImageView.tag = 200;
        [self.contentView addSubview:stampImageView];
        [stampImageView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.fs_equalToSuperview().offset(10.f);
            make.right.fs_equalToSuperview().offset(-10.f);
            make.top.fs_equalToSuperview().offset(10.f);
            make.bottom.fs_equalToSuperview().offset(-10.f);
        }];
    }
    return self;
}

@end

#define STAMP_TYPE_STANDER 1
#define STAMP_TYPE_SIGNHERE 2
#define STAMP_TYPE_DYNAMIC 3
#define STAMP_TYPE_CUSTOM 4

typedef NS_ENUM(NSUInteger, FS_STAMP_TYPE) {
    FS_STAMP_TYPE_STANDER,
    FS_STAMP_TYPE_SIGNHERE,
    FS_STAMP_TYPE_DYNAMIC,
    FS_STAMP_TYPE_CUSTOM,
};

@class  FSStampTabContentViewController;
@interface FSStampTabItem : NSObject
@property (nonatomic, copy) NSString *title;
@property (nonatomic, strong) FSItem *topItem;
@property (nonatomic, strong) FSStampTabContentViewController *contentViewController;
@property (nonatomic, assign) FS_STAMP_TYPE stampType;
- (instancetype)initWithStampType:(FS_STAMP_TYPE)type;
@end

@interface FSStampTabContentViewController : FSCollectionViewController
{
    FSCollectionView *_collectionView;
}
@property (nonatomic, weak) FSStampTabItem *tabItem;
@property (nonatomic, weak) StampIconController *parentVC;
@property (nonatomic, assign) FS_STAMP_TYPE stampType;
@property (nonatomic, assign) BOOL isiPhoneLandscape;
@property (nonatomic, assign) CGPDFDocumentRef pdfDocumentRef;
@property (nonatomic, assign) NSInteger currentIcon;
@property (nonatomic, copy) SelectIconHandler selectHandler;
@property (nonatomic, copy) CloseHandle closeHandler;
@property (nonatomic, strong) NSMutableDictionary *stampImages;
- (instancetype)initWithServices:(nullable id<FSViewControllerServices>)services type:(FS_STAMP_TYPE)type;
- (void)postReloadCustomStampsNotification;
@end

@implementation FSStampTabContentViewController
- (instancetype)initWithServices:(nullable id<FSViewControllerServices>)services type:(FS_STAMP_TYPE)type {
    _stampType = type;
    self = [super initWithServices:services];
    if (self) {
        self.stampImages = @{}.mutableCopy;
        NSURL *pdfURL = [[NSBundle bundleForClass:[self class]] URLForResource:@"icon" withExtension:@"pdf"];
        self.pdfDocumentRef = CGPDFDocumentCreateWithURL((CFURLRef) pdfURL);
    }
    return self;
}

- (void)initialize{
    [super initialize];
    self.shouldShowNavigationBar = NO;
}

- (void)dealloc {
    if (self.pdfDocumentRef) {
        CGPDFDocumentRelease(self.pdfDocumentRef);
    }
}

- (void)postReloadCustomStampsNotification{}

- (FSCollectionView *)collectionView{
    if (!_collectionView) {
        UICollectionViewFlowLayout *layout = [[UICollectionViewFlowLayout alloc] init];
        layout.sectionInsetReference = UICollectionViewFlowLayoutSectionInsetFromContentInset;
        layout.itemSize = CGSizeMake(160, 61);
        _collectionView = [[FSCollectionView alloc] initWithFrame:CGRectZero collectionViewLayout:layout] ;
        [_collectionView registerClass:[FSStampCollectionViewCell class] forCellWithReuseIdentifier:@"FSStampCollectionViewCell"];
        _collectionView.backgroundColor = self.fs_backgroundColor_B1;
        
        _collectionView.delegate = self;
        _collectionView.dataSource = self;
        _collectionView.allowsMultipleSelection = YES;
    }
    return _collectionView;
}

- (BOOL)emptyDataSetShouldDisplay:(UICollectionView *)collectionView{
    return NO;
}

- (id)dataForAtIndexPath:(NSIndexPath *)indexPath{
    return self.stampImages[@(indexPath.item)];
}

- (__kindof UICollectionViewCell *)collectionView:(UICollectionView *)collectionView dequeueReusableCellWithReuseIdentifier:(NSString *)identifier forIndexPath:(NSIndexPath *)indexPath withDataItem:(id)dataItem{
    return [collectionView dequeueReusableCellWithReuseIdentifier:@"FSStampCollectionViewCell" forIndexPath:indexPath];
}

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView{
    return 1;
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section{
    if (_stampType == FS_STAMP_TYPE_STANDER) {
        return 12;
    } else if (_stampType == FS_STAMP_TYPE_SIGNHERE) {
        return 5;
    } else if (_stampType == FS_STAMP_TYPE_DYNAMIC) {
        return 5;
    }
    return 0;
}

- (void)configureCell:(UICollectionViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(id)dataItem{
    cell.backgroundColor = [UIColor fs_clearColor];
    StampButton *stampBtn = (StampButton *) [cell.contentView viewWithTag:100];

    int index = 19;
    if (_stampType == FS_STAMP_TYPE_STANDER) {
        index = 19;
        stampBtn.stampIcon = indexPath.item;
    } else if (_stampType == FS_STAMP_TYPE_SIGNHERE) {
        index = 31;
        stampBtn.stampIcon = 12 + indexPath.item;
    } else if (_stampType == FS_STAMP_TYPE_DYNAMIC) {
        index = 36;
        stampBtn.stampIcon = 17 + indexPath.item;
    }
    
    index += indexPath.item;
    
    UIImage *image = dataItem;
    if (!image) {
        UIGraphicsBeginImageContextWithOptions(CGSizeMake(280, 82), YES, [UIScreen mainScreen].scale);
        CGContextRef context = UIGraphicsGetCurrentContext();
        CGContextSaveGState(context);
        CGContextTranslateCTM(context, 0.0, 82);
        CGContextScaleCTM(context, 1.0, -1.0);
        CGContextScaleCTM(context, 2.8, 2.73);
        CGPDFPageRef page = CGPDFDocumentGetPage(_pdfDocumentRef, index + 1);
        CGContextDrawPDFPage(context, page);
        CGContextRestoreGState(context);
        image = UIGraphicsGetImageFromCurrentImageContext();
        UIGraphicsEndImageContext();
        self.stampImages[@(indexPath.item)] = image;
    }

    [stampBtn setImage:image forState:UIControlStateNormal];
    [stampBtn addTarget:self action:@selector(onClicked:) forControlEvents:UIControlEventTouchUpInside];
    if (_stampType == FS_STAMP_TYPE_DYNAMIC) {
        UILabel *label = [stampBtn.imageView viewWithTag:11];
        if (!label) {
            label = [UILabel new];
            label.textColor = [UIColor whiteColor];
            label.font = [UIFont boldSystemFontOfSize:15.f];
            label.adjustsFontSizeToFitWidth = YES;
            label.minimumScaleFactor = 0.1;
            label.tag = 11;
        }
        label.text = [NSString stringWithFormat:@"%@, %@",self.parentVC.extensionManager.annotAuthor ? : @"", [[NSDate date] fs_stringWithFormat:@"HH:mm:ss, MM/dd/yyyy"]];
        [stampBtn.imageView addSubview:label];
        [label mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.fs_equalToSuperview().offset(8);
            make.right.fs_equalToSuperview().offset(-8);
            make.bottom.fs_equalToSuperview();
            make.top.equalTo(label.superview.mas_centerY).offset(-5);
        }];
    }

    if (index - 19 == self.currentIcon) {
        stampBtn.layer.borderWidth = 2.0f;
        [BindThemeKeySel(stampBtn.layer, setBorderColor:) fs_updateThemeCompleted:^(CALayer * _Nonnull target, SEL  _Nonnull sel) {
            target.borderColor = [target.fs_primaryColor CGColor];
        }];
        stampBtn.layer.cornerRadius = 5.0f;
        stampBtn.backgroundColor = [UIColor clearColor];
    } else {
        stampBtn.layer.borderWidth = 0.0f;
    }
}

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout sizeForItemAtIndexPath:(NSIndexPath *)indexPath{
    return CGSizeMake(160, 61);
}

- (UIEdgeInsets)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout insetForSectionAtIndex:(NSInteger)section{
    return UIEdgeInsetsMake(10, 10, 10, 10);
}

- (void)onClicked:(id)sender {
    StampButton *button = (StampButton *) sender;
    self.currentIcon = button.stampIcon;
    if (self.selectHandler) {
        self.selectHandler((int)self.currentIcon);
    }
    if (self.closeHandler) {
        self.closeHandler();
    }
    [self postReloadCustomStampsNotification];
}

@end

@interface FSStampTabCustomContentViewController : FSStampTabContentViewController<FSMediaFileToPDFViewControllerDelegate>
@property (nonatomic, strong) NSMutableArray<FSCustomStamps *> *customStamps;
@property (nonatomic, weak) FSCustomStampAttributes *currentCustomStamp;
@property (nonatomic, strong) NSOperationQueue *operationQueue;
@property (nonatomic, strong) NSMutableDictionary<NSIndexPath *, NSBlockOperation *> *operations;
@end
static NSMutableArray<FSCustomStamps *> *_commonCustomStamps;
static NSOperationQueue *_operationQueue;
static NSMutableDictionary *_operations;
static NSMutableDictionary *_stampImages;
@implementation FSStampTabCustomContentViewController

+ (NSMutableDictionary *)stampImages{
    if (!_stampImages) {
        _stampImages = @{}.mutableCopy;
    }
    return _stampImages;
}

+ (NSMutableArray<FSCustomStamps *> *)commonCustomStamps{
    if (!_commonCustomStamps) {
        _commonCustomStamps = FSCustomTextAndImageStamps.customStamps;
    }
    return _commonCustomStamps;
}

+ (NSOperationQueue *)operationQueue{
    if (!_operationQueue) {
        _operationQueue = ({
            NSOperationQueue *queue = [[NSOperationQueue alloc] init];
            queue.name = @"com.foxit.rdk.stamp.cache.image";
            queue.maxConcurrentOperationCount = 1;
            queue;
        });
    }
    return _operationQueue;
}

+ (NSMutableDictionary<NSIndexPath *, NSBlockOperation *> *)operations{
    if (!_operations) {
        _operations = [NSMutableDictionary<NSIndexPath *, NSBlockOperation *> dictionary];
    }
    return _operations;
}

- (NSMutableDictionary *)stampImages{
    return [FSStampTabCustomContentViewController stampImages];
}

- (void)initialize{
    [super initialize];
    self.shouldShowBottomBar = YES;
    self.shouldLoadDataOnViewDidLoad = YES;
    self.editingImageNormal = FSImageNamedAndNormalColor(@"common_unselect");
    self.editingImageSelected = FSImageNamed(@"common_select");
    
    self.stampImages = [FSStampTabCustomContentViewController stampImages];
    self.operationQueue = [FSStampTabCustomContentViewController operationQueue];
    self.operations = [FSStampTabCustomContentViewController operations];
}

- (void)viewDidLoad{
    [super viewDidLoad];
    
    [self.editLeftItem setTitle:FSLocalizedForKey(@"kSelectAll") forState:UIControlStateNormal];
    [self.editLeftItem setTitle:FSLocalizedForKey(@"kDeselectAll") forState:UIControlStateSelected];
    [self.editRightItem setTitle:@"" forState:UIControlStateNormal];
    [self.editRightItem setImage:FSImageNamedAndNormalColor(@"common_delete_gray") forState:UIControlStateNormal];
    self.editRightItem.accessibilityLabel = FSLocalizedForKey(@"kDelete");
    
    FSItem *addTextStampItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"comment_stamp_add_text")];
    addTextStampItem.accessibilityLabel = FSLocalizedForKey(@"kCreateTextStampVoiceover");
    @fs_weakify(self)
    addTextStampItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @fs_strongify(self)
        FSAddTextStampViewController *vc = [[FSAddTextStampViewController alloc] initWithServices:self.services];
        vc.didClickedDone = ^(FSCustomTextStampAttributes *textStampAttributes) {
            [self updateTextStamp:textStampAttributes];
        };
        [self.parentVC fs_presentViewController:vc animated:YES completion:nil];
    };
    [self.bottomBar addItem:addTextStampItem atPosition:FSItemPositionCenter];
    
    FSItem *addImageStampItem = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"comment_stamp_add_image")];
    addImageStampItem.accessibilityLabel = FSLocalizedForKey(@"kCreateImageStampVoiceover");
    addImageStampItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @fs_strongify(self)
        NSMutableArray *items = @[].mutableCopy;
        [items addObject:[[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kFromCamera") image:FSImageNamedAndNormalColor(@"common_camera") action:^(FSPopoverItem * _Nonnull item, int index) {
            FSMediaFileToPDFViewController *mediaFileToPDFViewController = [[FSMediaFileToPDFViewController alloc] init];
            mediaFileToPDFViewController.delegate = self;
            [self.parentVC presentViewController:mediaFileToPDFViewController animated:NO completion:nil];
            [mediaFileToPDFViewController openCamera];
        }]];
        [items addObject:[[FSPopoverItem alloc] initWithTitle:FSLocalizedForKey(@"kFromAlbum") image:FSImageNamedAndNormalColor(@"common_image") action:^(FSPopoverItem * _Nonnull item, int index) {
            FSMediaFileToPDFViewController *mediaFileToPDFViewController = [[FSMediaFileToPDFViewController alloc] init];
            mediaFileToPDFViewController.delegate = self;
            if (DEVICE_iPHONE || [self fs_currentSceneiPadHorizontalCompact]) {
                mediaFileToPDFViewController.modalPresentationStyle = UIModalPresentationFullScreen;
            }else{
                mediaFileToPDFViewController.modalPresentationStyle = UIModalPresentationFormSheet;
            }
            [self.parentVC presentViewController:mediaFileToPDFViewController animated:NO completion:nil];
            [mediaFileToPDFViewController openAlbum];
        }]];
        
        [(FSViewControllerManager *)(self.services) showMoreMenuWithItems:items.copy sourceView:![self fs_currentSceneIsCompact] ? ^UIView * _Nonnull(UIPopoverPresentationController * _Nonnull popver) {
            popver.permittedArrowDirections = UIPopoverArrowDirectionRight;
            return _item;
        } : nil];
    };
    [self.bottomBar addItem:addImageStampItem spacing:ITEM_MARGIN_MEDIUM atPosition:FSItemPositionCenter];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(reloadData) name:FSNotificationNameSomeReloadCustomStamps object:nil];
}

- (void)dealloc{
    [self.operationQueue cancelAllOperations];
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)postReloadCustomStampsNotification{
    [[NSNotificationCenter defaultCenter] postNotificationName:FSNotificationNameSomeReloadCustomStamps object:self];
}

- (void)setCustomEditing:(BOOL)editing{
    [super setCustomEditing:editing];
    for (UICollectionViewCell *cell in self.collectionView.visibleCells) {
        StampButton *stampBtn = (StampButton *) [cell.contentView viewWithTag:100];
        NSUInteger index = stampBtn.stampIcon;
        [stampBtn setLayerBorderWidth:((self.customEditing && cell.selected) || (index == self.currentIcon && !self.customEditing)) ? 2.0f : 0.0f];
    }
}

- (void)updateTextStamp:(FSCustomTextStampAttributes *)textStampAttributes{
    FSCustomTextStamps *textStamp;
    for (FSCustomStamps *stamp in self.customStamps) {
        if ([stamp isKindOfClass:[FSCustomTextStamps class]]) {
            textStamp = (FSCustomTextStamps *)stamp;
            break;
        }
    }
    if (!textStamp) {
        textStamp = FSCustomTextAndImageStamps.textStamps;
        [self.customStamps insertObject:textStamp atIndex:0];
    }
    if (![textStamp.textStampes containsObject:textStampAttributes]) {
        [textStamp.textStampes addObject:textStampAttributes];
    }
    [textStamp.modelToJSONData writeToFile:CUSTOM_TEXT_STAMP_FILE atomically:YES];
    [self postReloadCustomStampsNotification];
}

- (void)updateImageStamp:(FSCustomImageStampAttributes *)imageStampAttributes{
    FSCustomImageStamps *imageStamp;
    for (FSCustomStamps *stamp in self.customStamps) {
        if ([stamp isKindOfClass:[FSCustomImageStamps class]]) {
            imageStamp = (FSCustomImageStamps *)stamp;
            break;
        }
    }
    if (!imageStamp) {
        imageStamp = FSCustomTextAndImageStamps.imageStamps;
        [self.customStamps addObject:imageStamp];
    }
    if (![imageStamp.imageStampes containsObject:imageStampAttributes]) {
        [imageStamp.imageStampes addObject:imageStampAttributes];
    }
    [imageStamp.modelToJSONData writeToFile:CUSTOM_IMAGE_STAMP_FILE atomically:YES];
    [self postReloadCustomStampsNotification];
}

- (FSCustomStampAttributes *)getStampAttributesForIcon:(int)icon{
    for (FSCustomStamps *custom in self.customStamps) {
        if (icon >= 1000 && icon < 2000) {
            if ([custom isKindOfClass:[FSCustomTextStamps class]]) {
                if (custom.stamps.count > (icon - 1000)) {
                    return custom.stamps[icon - 1000];
                }
            }
        }else{
            if ([custom isKindOfClass:[FSCustomImageStamps class]]) {
                if (custom.stamps.count > (icon - 2000)) {
                    return custom.stamps[icon - 2000];
                }
            }
        }
    }
    return nil;
}

- (void)checkToolSettingsAttributes:(FSCustomStampAttributes *)deleteA isDelete:(BOOL)isDelete{
    void(^block)(FSReadToolSettings *) = ^(FSReadToolSettings *toolSettings){
        FSCustomStampAttributes *currentToolStamp = [self getStampAttributesForIcon:toolSettings.stampIcon];
        if (currentToolStamp == deleteA){
            toolSettings.stampIcon = 0;
        }else{
            NSMutableArray *tmp;
            if ([currentToolStamp isKindOfClass:[FSCustomTextStampAttributes class]]) {
                tmp = FSCustomTextAndImageStamps.textStamps.stamps.mutableCopy;
            }else{
                tmp = FSCustomTextAndImageStamps.imageStamps.stamps.mutableCopy;
            }
            if ([tmp containsObject:currentToolStamp]) {
                [tmp removeObject:deleteA];
                toolSettings.stampIcon = [currentToolStamp isKindOfClass:[FSCustomTextStampAttributes class]] ? 1000 : 2000 + (int)[tmp indexOfObject:currentToolStamp];
            }
        }
    };
    FSReadToolSettings *defToolSettings = [self.parentVC.extensionManager getReadToolSettingsForType:FSReadToolSettingsTypeStamp];
    FSReadToolSettings *addToolsSettings = self.parentVC.extensionManager.addToolsSettings[@(FSReadToolSettingsTypeStamp)];
    if (isDelete) {
        block(defToolSettings);
        block(addToolsSettings);
    }else{
        defToolSettings.stampIcon = (int)self.currentIcon;
        addToolsSettings.stampIcon = (int)self.currentIcon;
    }

    NSArray<FSMainTopbarToolTagItem *> *toolTagItemArray = self.parentVC.extensionManager.topToolbar.tagItems;
    for (int i = 0; i < toolTagItemArray.count; i++) {
        FSMainTopbarToolTagItem *item = toolTagItemArray[i];
        NSArray<FSReadToolbarItem *> *readtoolItems = item.readToolbarItems;
        if (item.toolTag == FSMainTopbarToolTagHome || item.toolTag == FSMainTopbarToolTagDrawing) {
            for (FSReadToolbarItem *toolbarItem in readtoolItems) {
                if (toolbarItem.settingsType == FSReadToolSettingsTypeStamp) {
                    if (isDelete) {
                        block(toolbarItem.toolSettings);
                        [toolbarItem.toolbarItem.item setImage:[Utility getStampImageWithIcon:toolbarItem.toolSettings.stampIcon] forState:UIControlStateNormal];
                    }
                    if (item.toolTag == FSMainTopbarToolTagDrawing) {
                        break;
                    }
                }
            }
            for (FSReadToolbarItem *toolbarItem in item.preAddToolItems) {
                if (toolbarItem.settingsType == FSReadToolSettingsTypeStamp) {
                    if (isDelete) {
                        block(toolbarItem.toolSettings);
                        [toolbarItem.toolbarItem.preAddToolItem setImage:[Utility getStampImageWithIcon:toolbarItem.toolSettings.stampIcon] forState:UIControlStateNormal];
                    }
                }
            }
        }
    }
}

- (void)loadData{
    self.customStamps = [FSStampTabCustomContentViewController commonCustomStamps];
    [self reloadData];
}

- (NSString *)titles1ForAtSection:(NSUInteger)section{
    FSCustomStamps *custom = self.customStamps[section];
    if ([custom isKindOfClass:[FSCustomTextStamps class]]) {
        return FSLocalizedForKey(@"kTextStampes");
    }
    return FSLocalizedForKey(@"kImageStampes");
}

- (id)dataForAtIndexPath:(NSIndexPath *)indexPath{
    FSCustomStampAttributes *attributes = [self.customStamps[indexPath.section] stamps][indexPath.row];
    if ([attributes isKindOfClass:[FSCustomImageStampAttributes class]]) {
        return self.stampImages[@(2000 + indexPath.item)];
    }
    return self.stampImages[@(1000 + indexPath.item)];
}

- (void)collectionView:(UICollectionView *)collectionView willDisplayCell:(UICollectionViewCell *)cell forItemAtIndexPath:(NSIndexPath *)indexPath{
    
    BOOL isImage = NO;
    FSCustomStampAttributes *attributes = [self.customStamps[indexPath.section] stamps][indexPath.row];
    if ([attributes isKindOfClass:[FSCustomImageStampAttributes class]]) {
        isImage = YES;
    }
    
    UIImage *image = [self dataForAtIndexPath:indexPath];
    if (!image) {
        UIImageView *imageView = (UIImageView *) [cell.contentView viewWithTag:200];
        imageView.image = nil;
        NSBlockOperation *op;
        __weak __block NSBlockOperation *weakOp;
        @fs_weakify(self)
        if (isImage) {
            imageView.image = FSImageNamedAndColor(@"common_placeholder_picture", self.fs_iconColor_I3);
            weakOp = op = [NSBlockOperation blockOperationWithBlock:^{
                if (weakOp.isCancelled) {
                    return;
                }
                @fs_strongify(self)
                FSCustomImageStampAttributes *imageStampAttributes = (FSCustomImageStampAttributes *)attributes;
                UIImage *image = imageStampAttributes.image;
                image = [image fs_compressWithMaxLength:1024 * 32];
                if (image) {
                    self.stampImages[@(2000 + indexPath.item)] = image;
                    dispatch_async(dispatch_get_main_queue(), ^{
                        NSIndexPath *indexPathForCell = [collectionView indexPathForCell:cell];
                        if (!weakOp.isCancelled && [indexPathForCell isEqual:indexPath]) {
                            UIImageView *imageView = (UIImageView *) [cell.contentView viewWithTag:200];
                            imageView.image = image;
                            imageView.alpha = imageStampAttributes.opacity;
                        }else{
                            [self collectionView:collectionView willDisplayCell:cell forItemAtIndexPath:indexPathForCell];
                        }
                    });
                }
            }];
        }else{
            weakOp = op = [NSBlockOperation blockOperationWithBlock:^{
                if (weakOp.isCancelled) {
                    return;
                }
                @fs_strongify(self)
                ((FSCustomTextStampAttributes *)attributes).currentAuthor = ((FSViewControllerManager *)[self services]).extensionsManager.annotAuthor;
                FSPDFPage *page = [[attributes currentDoc] getPage:0];
                UIImage *image = [Utility drawPage:page targetSize:CGSizeMake(140, 41) shouldDrawAnnotation:YES needPause:nil rotation:0 enableForPrint:NO pdfViewCtrl:nil];
                self.stampImages[@(1000 + indexPath.item)] = image;
                dispatch_async(dispatch_get_main_queue(), ^{
                    NSIndexPath *indexPathForCell = [collectionView indexPathForCell:cell];
                    if (!weakOp.isCancelled && [indexPathForCell isEqual:indexPath]) {
                        UIImageView *imageView = (UIImageView *) [cell.contentView viewWithTag:200];
                        imageView.image = image;
                        imageView.alpha = 1;
                    }else{
                        [self collectionView:collectionView willDisplayCell:cell forItemAtIndexPath:indexPathForCell];
                    }
                });
            }];
        }
        
        [self.operationQueue addOperation:op];
        if (self.operations[indexPath]) {
            [self.operations[indexPath] cancel];
        }
        self.operations[indexPath] = op;
    }else{
        UIImageView *imageView = (UIImageView *) [cell.contentView viewWithTag:200];
        imageView.image = image;
        if (isImage) {
            FSCustomImageStampAttributes *imageStampAttributes = (FSCustomImageStampAttributes *)attributes;
            imageView.alpha = imageStampAttributes.opacity;
        }else{
            imageView.alpha = 1;
        }
    }
}

- (void)configureCell:(FSCollectionViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(id)dataItem{
    @fs_weakify(self)
    [cell fs_setLayerCornerRadius:LAYER_BORDER_WIDTH_3];
    cell.backgroundColor = [UIColor fs_clearColor];
    UIImageView *imageView = (UIImageView *) [cell.contentView viewWithTag:200];
    StampButton *stampBtn = (StampButton *) [cell.contentView viewWithTag:100];
    stampBtn.accessibilityLabel = self.tabItem.title;
    stampBtn.accessibilityTraits = UIAccessibilityTraitNone;
    cell.editItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @fs_strongify(self)
        [self selectItem:_item.selected atIndexPath:indexPath];
        if (_item.selected) {
            [self collectionView:self.collectionView didSelectItemAtIndexPath:indexPath];
        }else{
            [self collectionView:self.collectionView didDeselectItemAtIndexPath:indexPath];
        }
        if (self.customEditing) {
            [stampBtn setLayerBorderWidth:_item.selected ? 2.0f : 0.0f];
        }
    };

    BOOL isImage = NO;
    int index = 1000;
    FSCustomStampAttributes *attributes = [self.customStamps[indexPath.section] stamps][indexPath.row];
    if ([attributes isKindOfClass:[FSCustomImageStampAttributes class]]) {
        isImage = YES;
        index = 2000;
    }
    
    if (isImage) {
        imageView.contentMode = UIViewContentModeScaleAspectFit;
        cell.backgroundColor = self.fs_backgroundColor_B10;
    }else{
        imageView.contentMode = UIViewContentModeScaleToFill;
    }
    
    index += indexPath.item;
    stampBtn.stampIcon = index;
    [stampBtn addTarget:self action:@selector(onClicked:) forControlEvents:UIControlEventTouchUpInside];

    if (index == self.currentIcon) {
        self.currentCustomStamp = attributes;
    }
    [stampBtn setLayerBorderWidth:((self.customEditing && cell.selected) || (index == self.currentIcon && !self.customEditing)) ? 2.0f : 0.0f];
}

- (void)collectionView:(UICollectionView *)collectionView didEndDisplayingCell:(UICollectionViewCell *)cell forItemAtIndexPath:(NSIndexPath *)indexPath {
    if (![self.collectionView.indexPathsForVisibleItems containsObject:indexPath]) {
        NSBlockOperation *operation = self.operations[indexPath];
        [operation cancel];
        self.operations[indexPath] = nil;
    }
}

- (BOOL)emptyDataSetShouldDisplay:(UICollectionView *)collectionView{
    BOOL shouldDisplay = !self.customStamps.count;
    if (shouldDisplay) {
        self.parentVC.navBar.rightItem.selected = NO;
        if (self.customEditing) {
            self.customEditing = NO;
        }
    }
    self.parentVC.navBar.rightItem.enabled = !shouldDisplay;
    return !self.customStamps.count;
}

- (NSString *)titleForEmptyDataSet{
    return FSLocalizedForKey(@"kNoCustomStamps");
}

- (UIImage *)imageForEmptyDataSet:(UICollectionView *)collectionView{
    return FSImageNamedAndPrimaryColor(@"comment_custom_stamp_no_data");
}

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView{
    return self.customStamps.count;
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section{
    return [self.customStamps[section] stamps].count;
}

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout sizeForItemAtIndexPath:(NSIndexPath *)indexPath{
    FSCustomStamps *custom = self.customStamps[indexPath.section];
    if ([custom isKindOfClass:[FSCustomTextStamps class]]) {
        return CGSizeMake(160, 61);
    }
    return CGSizeMake(160, 76);
}

- (void)onClicked:(StampButton *)sender {
    if (self.customEditing) {
        FSCustomStampAttributes *stamp;
        for (FSCustomStamps *stamps in self.customStamps) {
            if (sender.stampIcon >= 2000) {
                if ([stamps isKindOfClass:[FSCustomImageStamps class]]) {
                    stamp = ((FSCustomImageStamps *)stamps).imageStampes[sender.stampIcon - 2000];
                    @fs_weakify(self)
                    FSImageStampOpacityViewController *vc = [[FSImageStampOpacityViewController alloc] initWithServices:self.services params:stamp];
                    vc.didClickedDone = ^(FSCustomImageStampAttributes *imageStampAttributes) {
                        @fs_strongify(self)
                        self.stampImages[@(sender.stampIcon)] = nil;
                        [self updateImageStamp:imageStampAttributes];
                    };
                    [self.parentVC fs_presentViewController:vc animated:YES completion:nil];
                    break;
                }
            }else if (sender.stampIcon >= 1000) {
                if ([stamps isKindOfClass:[FSCustomTextStamps class]]) {
                    stamp = ((FSCustomTextStamps *)stamps).textStampes[sender.stampIcon - 1000];
                    @fs_weakify(self)
                    FSAddTextStampViewController *vc = [[FSAddTextStampViewController alloc] initWithServices:self.services params:stamp];
                    vc.title = FSLocalizedForKey(@"kEditTextStamp");
                    vc.didClickedDone = ^(FSCustomTextStampAttributes *textStampAttributes) {
                        @fs_strongify(self)
                        self.stampImages[@(sender.stampIcon)] = nil;
                        [self updateTextStamp:textStampAttributes];
                    };
                    [self.parentVC fs_presentViewController:vc animated:YES completion:nil];
                    break;
                }
            }
        }
        return;
    }
    [super onClicked:sender];
}

- (void)setCloseHandler:(CloseHandle)closeHandler{
    if (closeHandler) {
        @fs_weakify(self)
        CloseHandle _closeHandler = ^{
            @fs_strongify(self)
            self.currentCustomStamp = nil;
            closeHandler();
        };
        [super setCloseHandler:_closeHandler];
    }else{
        [super setCloseHandler:nil];
    }
}

- (void)setSelectedIndexPaths:(NSArray<NSIndexPath *> *)selectedIndexPaths{
    [super setSelectedIndexPaths:selectedIndexPaths];
    int allCount = 0;
    for (FSCustomImageStamps *stamps in self.customStamps) {
        allCount +=stamps.stamps.count;
    }
    self.editLeftItem.selected = self.selectedIndexPaths.count == allCount;
}

- (void)clickedEditLeftItem{
    self.editLeftItem.selected = !self.editLeftItem.selected;
    if (self.editLeftItem.selected) {
        [self selectAll];
    }else{
        [self deselectAll];
    }
    for (UICollectionViewCell *cell in self.collectionView.visibleCells) {
        StampButton *stampBtn = (StampButton *) [cell.contentView viewWithTag:100];
        NSUInteger index = stampBtn.stampIcon;
        [stampBtn setLayerBorderWidth:((self.customEditing && cell.selected) || (index == self.currentIcon && !self.customEditing)) ? 2.0f : 0.0f];
    }
}

- (void)clickedEditRightItem{
    
    if ([self selectedIndexPaths].count > 0 && [self selectedDataSource].count > 0) {
        void(^block)(void) = ^{
            [self.view fs_scheduledMainQueueShowHUDLoading:FSLocalizedForKey(@"kDeleting") process:^{
                [self deleteIndexPaths:self.selectedIndexPaths];
                [super clickedEditRightItem];
            }];
        };
        if (!self.editLeftItem.selected) {
            block();
        }else{
            [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kConfirm") message:FSLocalizedForKey(@"kDeleteCustomStampTip") actions:@[ FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kOK")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                if (index == 1) {
                    block();
                }
            } onCtrl:[self fs_currentSceneTopMostViewController]];
        }
    }
}

- (void)deleteIndexPaths:(NSArray<NSIndexPath *> *)indexPaths{
    NSMutableArray *copyStamps = @[].mutableCopy;
    for (FSCustomStamps *stamps in self.customStamps) {
        [copyStamps addObject:stamps.stamps.mutableCopy];
    }
    NSMutableArray *needDeleteStamps = @[].mutableCopy;
    
    FSCustomStampAttributes *currentCustomStamp = self.currentCustomStamp;
    BOOL currentCustomStampDeleted = NO;
    for (NSIndexPath *indexPath in indexPaths) {
        FSCustomStamps *stamps = self.customStamps[indexPath.section];
        FSCustomStampAttributes *copyStampA = copyStamps[indexPath.section][indexPath.item];
        if ([copyStampA isKindOfClass:[FSCustomImageStampAttributes class]]) {
            [(FSCustomImageStampAttributes *)copyStampA clearImage];
        }
        [self checkToolSettingsAttributes:copyStampA isDelete:YES];
        [stamps.stamps removeObject:copyStampA];
        if (!stamps.stamps.count) {
            [needDeleteStamps addObject:stamps];
        }
        if (copyStampA == self.currentCustomStamp) {
            currentCustomStampDeleted = YES;
            for (FSStampTabItem *tabItem in self.parentVC.tabItems) {
                tabItem.contentViewController.currentIcon = 0;
                if (tabItem == self.tabItem) {
                    continue;
                }
                [tabItem.contentViewController postReloadCustomStampsNotification];
            }
            if (self.selectHandler) {
                self.selectHandler(0);
            }
        }
    }
    
    if (!currentCustomStampDeleted && currentCustomStamp) {
        NSUInteger currentIcon = self.currentIcon;
        if ([currentCustomStamp isKindOfClass:[FSCustomImageStampAttributes class]]) {
            self.currentIcon = 2000 + [FSCustomTextAndImageStamps.imageStamps.stamps indexOfObject:currentCustomStamp];
        }else{
            self.currentIcon = 1000 + [FSCustomTextAndImageStamps.textStamps.stamps indexOfObject:currentCustomStamp];
        }
        if (currentIcon != self.currentIcon) {
            if (self.selectHandler) {
                self.selectHandler((int)self.currentIcon);
            }
            [self checkToolSettingsAttributes:nil isDelete:NO];
        }
        self.currentCustomStamp = currentCustomStamp;
    }
    
    [[FSStampTabCustomContentViewController stampImages] removeAllObjects];
    
    [self deleteItemsAtIndexPaths:self.selectedIndexPaths completion:^(BOOL finished) {
        if (finished) {
            if (needDeleteStamps.count){
                [self.customStamps removeObjectsInArray:needDeleteStamps];
            }
            [self postReloadCustomStampsNotification];
            FSCustomTextStamps *textStamps;
            FSCustomImageStamps *imageStamps;
            for (FSCustomStamps *stamps in self.customStamps) {
                if ([stamps isKindOfClass:[FSCustomTextStamps class]]) {
                    textStamps = (FSCustomTextStamps *)stamps;
                }else  if ([stamps isKindOfClass:[FSCustomImageStamps class]]) {
                    imageStamps = (FSCustomImageStamps *)stamps;
                }
            }
            if (!textStamps) {
                if ([[NSFileManager defaultManager] fileExistsAtPath:CUSTOM_TEXT_STAMP_FILE]) {
                    [[NSFileManager defaultManager] removeItemAtPath:CUSTOM_TEXT_STAMP_FILE error:nil];
                }
            }else{
                [textStamps.modelToJSONData writeToFile:CUSTOM_TEXT_STAMP_FILE atomically:YES];
            }
            if (!imageStamps) {
                if ([[NSFileManager defaultManager] fileExistsAtPath:CUSTOM_IMAGE_STAMP_PATH]) {
                    [[NSFileManager defaultManager] removeItemAtPath:CUSTOM_IMAGE_STAMP_PATH error:nil];
                }
            }else{
                [imageStamps.modelToJSONData writeToFile:CUSTOM_IMAGE_STAMP_FILE atomically:YES];
            }

        }
    }];
}

#pragma mark <FSMediaFileToPDFViewControllerDelegate>
- (void)mediaFileToPDFViewController:(FSMediaFileToPDFViewController *)mediaFileToPDFViewController didSelectedImage:(UIImage *)image filePath:(NSString *)path{
    if ([[NSFileManager defaultManager] fileExistsAtPath:path] || image) {
        FSCustomImageStampAttributes *imageStampAttributes = [FSCustomImageStampAttributes new];
        imageStampAttributes.image = image;
        [imageStampAttributes copyImageFileWithPath:path];
        imageStampAttributes.opacity = 1;
        FSImageStampOpacityViewController *opacityViewController = [[FSImageStampOpacityViewController alloc] initWithServices:self.services params:imageStampAttributes];
        opacityViewController.didClickedDone = ^(FSCustomImageStampAttributes *imageStampAttributes) {
            [self updateImageStamp:imageStampAttributes];
        };
        [self.parentVC fs_presentViewController:opacityViewController animated:YES completion:nil];
    }
}



- (BOOL)mediaFileToPDFViewControllerShouldDismissViewControllerAnimated:(FSMediaFileToPDFViewController *)mediaFileToPDFViewController{
    return NO;
}

@end

@implementation FSStampTabItem
- (instancetype)initWithStampType:(FS_STAMP_TYPE)type{
    self = [super init];
    if (self) {
        _stampType = type;
        [self initialize];
    }
    return self;
}

- (void)initialize{
    
    NSString *imageName = @"";
    switch (_stampType) {
        case FS_STAMP_TYPE_STANDER:
            imageName = @"read_segment_stamp1";
            self.title = FSLocalizedForKey(@"kStandardStamps");
            break;
        case FS_STAMP_TYPE_SIGNHERE:
            imageName = @"read_segment_stamp2";
            self.title = FSLocalizedForKey(@"kSignHere");
            break;
        case FS_STAMP_TYPE_DYNAMIC:
            imageName = @"read_segment_stamp3";
            self.title = FSLocalizedForKey(@"kDynamicStamps");
            break;
        case FS_STAMP_TYPE_CUSTOM:
            imageName = @"read_segment_stamp4";
            self.title = FSLocalizedForKey(@"kCustomStamps");
            break;
    }
    
    
    FSItem *topItem = [FSItem createItemWithImage:[FSImageNamed(imageName) fs_imageWithTintColor:self.fs_iconColor_I3]];
    topItem.accessibilityLabel = self.title;
    topItem.imageView.contentMode = UIViewContentModeCenter;
    topItem.tag = _stampType+1;
    [BindThemeKeySel(topItem, setImage:forState:)
     fs_updateThemeCompleted:^(FSItem * _Nonnull target, SEL  _Nonnull sel) {
        [target setImage:FSImageNamedAndPrimaryColor(imageName) forState:UIControlStateSelected];
    }];
    
    _topItem = topItem;
}

@end

@implementation StampIconController

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager toolSettings:(FSReadToolSettings *)toolSettings {
    self = [super init];
    if (self) {
        _extensionManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        _toolSettings = toolSettings;
        self.currentIcon = toolSettings.stampIcon;

        if (DEVICE_iPAD && ![_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            self.shouldBindTemeForStatusBar = NO;
        }
        self.isiPhoneLandscape = NO;
    }
    return self;
}

- (id<FSViewControllerServices>)services{
    return _extensionManager.viewCtrlManager;
}

- (void)viewDidLoad{
    [super viewDidLoad];
    self.view.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;

    self.navBar.contentBarHeight = TOOL_BAR_HEIGHT_NORMAL * 2;
    
    self.navBarLeftTitle = FSLocalizedForKey(@"kClose");
    self.title = FSLocalizedForKey(@"kStamp");
    [self.navBar.leftItem addTarget:self action:@selector(backClicked) forControlEvents:UIControlEventTouchUpInside];
    if (DEVICE_iPAD && ![_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
        self.navBar.leftItem.hidden = YES;
    }
    [self.navBar.rightItem setTitle:FSLocalizedForKey(@"kEdit") forState:UIControlStateNormal];
    [self.navBar.rightItem setTitle:FSLocalizedForKey(@"kDone") forState:UIControlStateSelected];
    @fs_weakify(self)
    self.navBar.rightItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @fs_strongify(self)
        _item.selected =  !_item.selected;
        self.currentTabItem.contentViewController.customEditing = _item.selected;
    };
    
    self.topBar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleDefault];
    [self.topBar addItem:self.navBar.leftItem atPosition:FSItemPositionLeft];
    [self.topBar addItem:self.navBar.rightItem atPosition:FSItemPositionRight];
    [self.topBar addItem:self.navBar.titleItem atPosition:FSItemPositionCenter];
    [self.navBar addSubview:self.topBar];
    
    [self.topBar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.top.right.fs_equalToSuperview();
    }];
    
    [self createTabItems];
    [self adddPagerView];
    [self resetTitleItem];
    
    self.isiPhoneLandscape = NO;
    if ((DEVICE_iPHONE && UIInterfaceOrientationIsLandscape([UIApplication sharedApplication].statusBarOrientation)) ||
        ([_pdfViewCtrl fs_currentSceneiPadHorizontalCompact] && _extensionManager.pdfViewCtrl.fs_width > 420)) {
        self.isiPhoneLandscape = YES;
    }
}

- (void)setIsiPhoneLandscape:(BOOL)isiPhoneLandscape{
    _isiPhoneLandscape = isiPhoneLandscape;
    for (FSStampTabItem *tabItem in self.tabItems) {
        tabItem.contentViewController.isiPhoneLandscape = isiPhoneLandscape;
        [tabItem.contentViewController postReloadCustomStampsNotification];
    }
}

- (void)createTabItems{
    NSMutableArray *tabItems = @[].mutableCopy;
    for (FS_STAMP_TYPE type = FS_STAMP_TYPE_STANDER; type <= FS_STAMP_TYPE_CUSTOM; type++) {
        FSStampTabItem *tabItem = [[FSStampTabItem alloc] initWithStampType:type];
        FSStampTabContentViewController *contentVC;
        if (type == FS_STAMP_TYPE_CUSTOM) {
            contentVC = [[FSStampTabCustomContentViewController alloc] initWithServices:self->_extensionManager.viewCtrlManager type:type];
        }else{
            contentVC = [[FSStampTabContentViewController alloc] initWithServices:self->_extensionManager.viewCtrlManager type:type];
        }
        contentVC.tabItem = tabItem;
        contentVC.parentVC = self;
        contentVC.currentIcon = self.currentIcon;
        @fs_weakify(self)
        contentVC.selectHandler = ^(int icon) {
            @fs_strongify(self)
            self.currentIcon = icon;
            if (self.selectHandler) {
                self.selectHandler(self.currentIcon);
            }
        };
        contentVC.closeHandler = ^{
            @fs_strongify(self)
            [self dismissViewControllerAnimated:YES completion:self.dismissCompletion];
            self.closeHandler();
        };
        tabItem.contentViewController = contentVC;
        [tabItems addObject:tabItem];
    }
    
    self.tabItems = tabItems.copy;
        
        FSStampTabItem *tabItem;
        int index = 0;
        if (self.currentIcon >= 0 && self.currentIcon <= 11) {
            index = 0;
        } else if (self.currentIcon >= 12 && self.currentIcon <= 16) {
            index = 1;
        } else if (self.currentIcon >= 17 && self.currentIcon <= 21) {
            index = 2;
        } else {
            index = 3;
        }
    tabItem = self.tabItems[index];
    self.currentTabItem = tabItem;
}

- (void)setCurrentTabItem:(FSStampTabItem *)currentTabItem{
    if (_currentTabItem != currentTabItem) {
        _currentTabItem  = currentTabItem;
        self.title = currentTabItem.title;
        self.navBar.rightItem.hidden = currentTabItem.stampType != FS_STAMP_TYPE_CUSTOM;
    }
}

- (void)adddPagerView{
    _tabPagerBar = [FSTabPagerBar new];
    _tabPagerBar.delegate = self;
    _tabPagerBar.dataSource = self;
    _tabPagerBar.itemEdging = 0;
    _tabPagerBar.itemContentMode = FSTabPagerBarItemContentModeScaleAspectFit;
    BindThemeKeyPath(_tabPagerBar, progressColor);
    [self.navBar addSubview:_tabPagerBar];
    [_tabPagerBar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.topBar.mas_bottom);
        make.height.equalTo(self.topBar);
        make.left.bottom.right.fs_equalToSuperview();
    }];
    
    _pagerContentView = [FSPagerView new];
    _pagerContentView.delegate = self;
    _pagerContentView.dataSource = self;
    [self.view addSubview:_pagerContentView];
    [_pagerContentView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.equalTo(self.navBar.mas_bottom);
        make.left.right.bottom.equalTo(self.view);
    }];
    
    [self reloadTabPager];
    [_tabPagerBar scrollToItemAtIndex:self.currentTabItem.stampType animated:NO];
    [_pagerContentView scrollToViewAtIndex:self.currentTabItem.stampType animate:NO];
}

- (void)reloadTabPager{
    [_tabPagerBar reloadData];
    [_pagerContentView reloadData];
}

- (void)resetTitleItem
{
    if (self.navBar.titleItem) {
        CGSize closeItemSize = [Utility getTextSize:FSLocalizedForKey(@"kClose") fontSize:self.navBar.leftItem.titleLabel.font.pointSize maxSize:CGSizeMake(150, 30)];
        CGFloat titleItemWidth = (self.view.fs_width/2 - 16*2 - closeItemSize.width)*2 - self.view.safeAreaInsets.left - self.view.safeAreaInsets.right;
        [self.navBar.titleItem mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.center.mas_equalTo(self.navBar.titleItem.superview);
            make.size.mas_equalTo(CGSizeMake(titleItemWidth, 20));
        }];
    }
}

- (void)viewDidLayoutSubviews{
    [super viewDidLayoutSubviews];
    [self resetTitleItem];
}

- (void)backClicked {
    [self dismissViewControllerAnimated:YES
                             completion:^{
        if (self.closeHandler) {
            self.closeHandler();
        }
    }];
}

- (void)dealloc {
    self.selectHandler = nil;
}

- (void)viewWillTransitionToSize:(CGSize)size withTransitionCoordinator:(id<UIViewControllerTransitionCoordinator>)coordinator {
    [super viewWillTransitionToSize:size withTransitionCoordinator:coordinator];

    self.isiPhoneLandscape = NO;
    if (DEVICE_iPHONE && size.width > size.height) {
        self.isiPhoneLandscape = YES;
    } else {
        if (size.width > 420 && [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            self.isiPhoneLandscape = YES;
        }else{
            self.isiPhoneLandscape = NO;
        }
    }
}

#pragma mark -FSPagerViewDataSource
- (NSInteger)numberOfViewsInPagerView:(FSPagerView *)pagerView;{
    return self.tabItems.count;
}

- (UIView *)pagerView:(FSPagerView *)pagerView viewForIndex:(NSInteger)index{
    FSStampTabItem *item = self.tabItems[index];
    UIEdgeInsets layoutMargins = pagerView.safeAreaInsets;
    item.contentViewController.collectionView.contentInset = layoutMargins;
    item.contentViewController.needMakeUpSectionHeaderInset = YES;
    item.contentViewController.editBar.subviewContentViewLayoutMargins = layoutMargins;
    item.contentViewController.bottomBar.subviewContentViewLayoutMargins = layoutMargins;
    return item.contentViewController.view;
}

#pragma mark -FSPagerViewDelegate

- (void)pagerView:(FSPagerView *)pagerView transitionFromIndex:(NSInteger)fromIndex toIndex:(NSInteger)toIndex progress:(CGFloat)progress{
    FSStampTabItem *tabItem = self.tabItems[toIndex];
    self.currentTabItem = tabItem;
    [_tabPagerBar scrollToItemFromIndex:fromIndex toIndex:toIndex progress:progress];
}

#pragma mark -FSTabPagerBarDelegate

- (NSInteger)numberOfItemsInPagerTabBar:(FSTabPagerBar *)pagerTabBar{
    return self.tabItems.count;
}

- (__kindof UIView *)pagerTabBar:(FSTabPagerBar *)pagerTabBar cellForItemAtIndex:(NSInteger)index{
    if (!self.tabItems || self.tabItems.count < index + 1) {
        return nil;
    }
    FSStampTabItem *tabItam = self.tabItems[index];
    FSItem *topItem = tabItam.topItem;
    return  topItem;
}

#pragma mark -FSTabPagerBarDelegate

- (CGFloat)pagerTabBar:(FSTabPagerBar *)pagerTabBar widthForItemAtIndex:(NSInteger)index{
    return pagerTabBar.fs_width / self.tabItems.count;
}

- (void)pagerTabBar:(FSTabPagerBar *)pagerTabBar didSelectItemAtIndex:(NSInteger)index{
    FSStampTabItem *tabItem = self.tabItems[index];
    self.currentTabItem = tabItem;
    [_pagerContentView scrollToViewAtIndex:index animate:YES];
}

- (void)pagerTabBar:(FSTabPagerBar *)pagerTabBar transitionFromItem:(__kindof UIView *)fromItem toItem:(__kindof UIView *)toItem animated:(BOOL)animated{}

@end
