/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "StampModule.h"
#import "StampAnnotHandler.h"
#import "StampToolHandler.h"
#import "Utility.h"
#import <FoxitRDK/FSPDFViewControl.h>

@interface StampModule ()

@property (nonatomic, weak) FSMainToolbarItem *propertyItem;
@property (nonatomic, weak) FSReadToolbarItem *readToolbarItem;

@end

@implementation StampModule {
    FSPDFViewCtrl *__weak _pdfViewCtrl;
    UIExtensionsManager *__weak _extensionsManager;
}

- (NSString *)getName {
    return @"Stamp";
}

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _pdfViewCtrl = extensionsManager.pdfViewCtrl;
        [self loadModule];
        StampToolHandler* toolHandler = [[StampToolHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerToolHandler:toolHandler];
        StampAnnotHandler* annotHandler = [[StampAnnotHandler alloc] initWithUIExtensionsManager:extensionsManager];
        [_extensionsManager registerAnnotHandler:annotHandler];
    }
    return self;
}

- (void)loadModule {
    FSReadToolbarItem *stampItem = [[FSReadToolbarItem alloc] initWithToolSettings:[_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeStamp]];
    [_extensionsManager.toolbarManager addReadItem:stampItem atToolTag:FSMainTopbarToolTagComment];
}

- (void)didClickedReadToolItem:(FSReadToolbarItem *)readToolbarItem {
    [_extensionsManager changeState:FSUIManagerStateAnnotTool];
    id<IToolHandler> toolHandler = [_extensionsManager getToolHandlerByName:Tool_Stamp];
    toolHandler.toolSettings = readToolbarItem.toolSettings;
    [_extensionsManager setCurrentToolHandler:toolHandler];

    self.readToolbarItem = readToolbarItem;
    FSMainToolbarItem *propertyItem =  [_extensionsManager.toolbarManager getMainItemWithType:FSMainToolbarItemTypeProperty];
    propertyItem.item.onTapClick = ^(FSItem * _Nonnull _item) {
        if (DEVICE_iPHONE) {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:CGRectZero inView:self->_extensionsManager.pdfViewCtrl];
        } else {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:_item.bounds inView:_item];
        }
    };
    self.propertyItem = propertyItem;
    
    [_extensionsManager registerAnnotPropertyListener:self];
    [_extensionsManager onUIInteractElementClicked:FSUIElementReadingCommentBarStamp];
}
#pragma mark - IAnnotPropertyListener

- (void)onToolSettingsChanged:(FSReadToolSettings *)toolSettings {
    if (self.readToolbarItem && self.readToolbarItem.toolSettings == toolSettings) {
        [self.propertyItem.item setImage:[Utility getStampImageWithIcon:toolSettings.stampIcon] forState:UIControlStateNormal];
        
        if ( _extensionsManager.topToolbar.currentTagItem.toolTag == FSMainTopbarToolTagHome) {
            if (_extensionsManager.topToolbar.currentTagItem.currentReadToolbarItem == self.readToolbarItem) {
                [self.readToolbarItem.toolbarItem.item setImage:[Utility getStampImageWithIcon:toolSettings.stampIcon] forState:UIControlStateNormal];
            }
        }
    }
}

@end
