/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "TextboxModule.h"
#import "FtAnnotHandler.h"
#import "SelectToolHandler.h"
#import "TextboxToolHandler.h"

@interface TextboxModule ()

@property (nonatomic, weak) UIExtensionsManager *extensionsManager;

@property (nonatomic, weak) FSMainToolbarItem *propertyItem;
@property (nonatomic, weak) FSReadToolbarItem *readToolbarItem;
@property (nonatomic, strong) FSReadToolbarItem *textboxItem;
@end

@implementation TextboxModule

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager {
    self = [super init];
    if (self) {
        self.extensionsManager = extensionsManager;
        [_extensionsManager registerAnnotPropertyListener:self];
        [self loadModule];
    }
    return self;
}

- (NSString *)getName {
    return @"Textbox";
}

- (void)loadModule {
    TextboxToolHandler *toolHandler = [[TextboxToolHandler alloc] initWithUIExtensionsManager:_extensionsManager];
    [_extensionsManager registerUndoEventListener:toolHandler];
    [_extensionsManager registerToolHandler:toolHandler];

    FtAnnotHandler *annotHandler = [[FtAnnotHandler alloc] initWithUIExtensionsManager:_extensionsManager];
    [_extensionsManager registerAnnotHandler:annotHandler];
    
    _textboxItem = [[FSReadToolbarItem alloc] initWithToolSettings:[_extensionsManager getReadToolSettingsForType:FSReadToolSettingsTypeTextbox]];
    [_extensionsManager.toolbarManager addReadItem:_textboxItem atToolTag:FSMainTopbarToolTagComment];
}

- (void)didClickedReadToolItem:(FSReadToolbarItem *)readToolbarItem {
    SelectToolHandler *select = (SelectToolHandler *) [_extensionsManager getToolHandlerByName:Tool_Select];
    [select clearSelection];
    [_extensionsManager changeState:FSUIManagerStateAnnotTool];
    id<IToolHandler> toolHandler = [_extensionsManager getToolHandlerByName:Tool_Textbox];
    toolHandler.toolSettings = readToolbarItem.toolSettings;
    [_extensionsManager setCurrentToolHandler:toolHandler];
    
    self.readToolbarItem = readToolbarItem;
    FSMainToolbarItem *propertyItem =  [_extensionsManager.toolbarManager getMainItemWithType:FSMainToolbarItemTypeProperty];
    propertyItem.item.onTapClick = ^(FSItem * _Nonnull _item) {
        [[UIApplication sharedApplication] sendAction:@selector(resignFirstResponder) to:nil from:nil forEvent:nil];
        
        CGRect rect = [_item convertRect:_item.bounds toView:self->_extensionsManager.pdfViewCtrl];
        if (DEVICE_iPHONE) {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:rect inView:self->_extensionsManager.pdfViewCtrl];
        } else {
            [self->_extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings rect:_item.bounds inView:_item];
        }
    };
    self.propertyItem = propertyItem;

    [_extensionsManager onUIInteractElementClicked:FSUIElementReadingCommentBarTextbox];
}

#pragma mark <IAnnotPropertyListener>

- (void)onToolSettingsChanged:(FSReadToolSettings *)toolSettings {
    if (self.readToolbarItem && self.readToolbarItem.toolSettings == toolSettings) {
        self.readToolbarItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }else if (_textboxItem.toolSettings == toolSettings){
        _textboxItem.propertyColor = [UIColor colorWithRGB:toolSettings.settingsColor];
    }
}

@synthesize annotType;

@end
