/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSThumbnailCell.h"
#import "PrivateDefine.h"

@implementation FSReorderableCollectionViewPlaceholderCell
@end

@interface FSThumbnailCell ()

@end

@implementation FSThumbnailCell

- (id)init {
    return [self initWithFrame:CGRectZero];
}

- (id)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        self.isAccessibilityElement = YES;
        // Initialization code
        self.isEditing = NO;
        self.alwaysHideCheckBox = NO;

        self.contentView.autoresizingMask = UIViewAutoresizingFlexibleTopMargin | UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin | UIViewAutoresizingFlexibleBottomMargin | UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;

        [self addSubviews];
        [self addButtons];
    }
    return self;
}

- (void)addSubviews {
    self.contentView.autoresizingMask = UIViewAutoresizingFlexibleTopMargin | UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin | UIViewAutoresizingFlexibleBottomMargin | UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
    self.imageView = [[UIImageView alloc] init];
    self.imageView.frame = self.contentView.bounds;
    self.imageView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
    self.imageView.contentMode = UIViewContentModeScaleAspectFit;
    self.imageView.accessibilityIdentifier = @"page_image_id";
    [self.contentView addSubview:self.imageView];

    self.labelNumber = [FSItem new];
    [self.labelNumber fs_setLayerCornerRadius:LAYER_BORDER_WIDTH_6];
    self.labelNumber.enabled = NO;
    self.labelNumber.titleEdgeInsets = UIEdgeInsetsMake(3.f, ITEM_MEDIUM_SPACE_NORMAL, 3.f, ITEM_MEDIUM_SPACE_NORMAL);
    self.labelNumber.font = self.fs_standardGeneralFont_GF2;
    [self.labelNumber setTitleColor:self.fs_textColor_T1 forState:UIControlStateDisabled];
    self.labelNumber.backgroundColor = [[UIColor fs_blackColor] colorWithAlphaComponent:0.5];
    [self.contentView addSubview:self.labelNumber];
    [self.labelNumber mas_makeConstraints:^(MASConstraintMaker *make) {
        make.bottom.fs_equalToSuperview().offset(-ITEM_MEDIUM_SPACE_NORMAL);
        make.centerX.fs_equalToSuperview();
    }];
    
    [self.contentView addSubview:self.bookmark];
    [self.bookmark mas_makeConstraints:^(MASConstraintMaker *make) {
        make.top.fs_equalToSuperview();
        make.right.fs_equalToSuperview().offset(-ITEM_MARGIN_NORMAL);
    }];
}

- (void)addButtons {
    [self.contentView addSubview:self.checkBtn];
}

- (void)updateButtonFramesWithThumbnailWidth:(CGFloat)thumbnailWidth {
    CGRect frame = self.checkBtn.frame;
    frame.origin.x = 10.f;
    self.checkBtn.frame = frame;
}

#pragma mark - Properties getter/setter

- (FSItem *)bookmark{
    if(!_bookmark){
        _bookmark = [FSItem new];
        _bookmark.accessibilityIdentifier = @"page_bookmark_id";
        [BindThemeKeySel(_bookmark, setImage:forState:) fs_updateThemeCompleted:^(FSItem *  _Nonnull target, SEL  _Nonnull sel) {
            [target setImage:FSImageNamedAndPrimaryColor(@"thumbnail_bookmark") forState:UIControlStateNormal];
        }];
    }
    return _bookmark;
}

- (UIButton *)checkBtn {
    if (_checkBtn == nil) {
        UIButton *button = [[UIButton alloc] init];
        button.frame = CGRectMake(10, 10, 40.f, 40.f);
        [button setBackgroundImage:FSImageNamed(@"thumbnail_checkboxi_white_backgroup") forState:UIControlStateNormal];
        button.userInteractionEnabled = NO;
        button.alpha = 0.f;
        [button setImage:FSImageNamedAndNormalColor(@"common_unselect") forState:UIControlStateNormal];
        [BindThemeKeySel(button, setImage:forState:) fs_updateThemeCompleted:^(UIButton * _Nonnull target, SEL  _Nonnull sel) {
            [target setImage:FSImageNamedAndPrimaryColor(@"common_select") forState:UIControlStateSelected];
        }];
        _checkBtn = button;
    }
    return _checkBtn;
}

- (void)setIsEditing:(BOOL)isEditing {
    [self setIsEditing:isEditing animated:NO];
}

- (void)setIsEditing:(BOOL)isEditing animated:(BOOL)animated {
    if (_isEditing == isEditing) {
        return;
    }
    _isEditing = isEditing;
    if (animated) {
        [UIView animateWithDuration:0.3
            delay:0
            options:UIViewAnimationOptionCurveEaseInOut | UIViewAnimationOptionBeginFromCurrentState
            animations:^{
                self.checkBtn.alpha = isEditing && (!self.alwaysHideCheckBox ? 1 : 0);
            }
            completion:^(BOOL finished) {
                self.checkBtn.alpha = isEditing && (!self.alwaysHideCheckBox ? 1 : 0);
                [self.checkBtn.superview bringSubviewToFront:self.checkBtn];
            }];
    } else {
        self.checkBtn.alpha = isEditing && (!self.alwaysHideCheckBox ? 1 : 0);
    }

    if (!isEditing) {
        self.checkBtn.selected = NO;
    }
}

- (void)setSelected:(BOOL)selected {
    [super setSelected:selected];
    [self updateVisualAppearanceForSelected:selected];
}

#pragma mark - public methods

- (void)prepareForReuse {
    [super prepareForReuse];
    self.delegate = nil;
    self.tag = 0;
    self.isEditing = NO;
    self.alwaysHideCheckBox = NO;
    self.alpha = 1.0f;
    self.contentView.alpha = 1.0f;
    self.backgroundColor = [UIColor clearColor];
    self.imageView.image = nil;
    [self.contentView.subviews enumerateObjectsUsingBlock:^(__kindof UIView *_Nonnull obj, NSUInteger idx, BOOL *_Nonnull stop) {
        if ([obj isKindOfClass:[UIActivityIndicatorView class]]) {
            [obj removeFromSuperview];
            *stop = YES;
        }
    }];
}

#pragma mark - private methods

- (void)updateVisualAppearanceForSelected:(BOOL)selected {
    if (self.checkBtn.selected != selected) {
        self.checkBtn.selected = selected;
    }
    [self fs_setLayerCornerRadius:LAYER_BORDER_RADIUS_3];
}

@end

@interface FSPastePlaceholderCollectionViewCell ()
@property (nonatomic, strong) CAShapeLayer *border;
@end

@implementation FSPastePlaceholderCollectionViewCell

- (id)initWithFrame:(CGRect)frame {
    self = [super initWithFrame:frame];
    if (self) {
        // Initialization code
        
        [BindThemeKeySel(self, backgroundView) fs_updateThemeCompleted:^(FSReorderableCollectionViewPlaceholderCell * _Nonnull target, SEL  _Nonnull sel) {
            target.backgroundColor = [target.fs_primaryColor colorWithAlphaComponent:0.2];
        }];

        self.border = [CAShapeLayer layer];
        [BindThemeKeySel(self.border, strokeColor) fs_updateThemeCompleted:^(CAShapeLayer * _Nonnull target, SEL  _Nonnull sel) {
            target.strokeColor = target.fs_primaryColor.CGColor;
        }];
        self.border.fillColor = [UIColor clearColor].CGColor;
        self.border.path = [UIBezierPath bezierPathWithRect:self.bounds].CGPath;
        self.border.frame = self.bounds;
        self.border.lineWidth = 2.f;
        self.border.lineDashPattern = @[@4, @2];
        [self.layer addSublayer:self.border];
        
        UILabel *tipLabel = [UILabel new];
        tipLabel.textAlignment = NSTextAlignmentCenter;
        tipLabel.textColor = self.fs_textColor_T4;
        tipLabel.font = self.fs_standardWeakFont_WF1;
        tipLabel.text = FSLocalizedForKey(@"kThumnailPasteTip");
        tipLabel.numberOfLines = 0;
        [self.contentView addSubview:tipLabel];
        [tipLabel mas_makeConstraints:^(MASConstraintMaker *make) {
            make.center.fs_equalToSuperview();
            make.left.fs_equalToSuperview().offset(ITEM_MARGIN_LITTLE_NORMAL);
            make.right.fs_equalToSuperview().offset(-ITEM_MARGIN_LITTLE_NORMAL);
        }];
        
        FSItem *closeItem = [FSItem new];
        closeItem.accessibilityLabel = FSLocalizedForKey(@"kDelete");
        @weakify(self)
        closeItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            if (self.onTapClickClose) {
                self.onTapClickClose();
            }
        };
        [BindThemeKeySel(closeItem, setImage:forState:) fs_updateThemeCompleted:^(FSItem * _Nonnull target, SEL  _Nonnull sel) {
            [target setImage:FSImageNamedAndPrimaryColor(@"thumbnail_close") forState:UIControlStateNormal];
        }];
        
        [self.contentView addSubview:closeItem];
        [closeItem mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.right.fs_equalToSuperview();
        }];
    }
    return self;
}

- (void)layoutSubviews{
    [super layoutSubviews];
    if (self.border) {
        
        [self.border removeFromSuperlayer];
        self.border = [CAShapeLayer layer];
        [BindThemeKeySel(self.border, strokeColor) fs_updateThemeCompleted:^(CAShapeLayer * _Nonnull target, SEL  _Nonnull sel) {
            target.strokeColor = target.fs_primaryColor.CGColor;
        }];
        self.border.fillColor = [UIColor clearColor].CGColor;
        self.border.path = [UIBezierPath bezierPathWithRect:self.bounds].CGPath;
        self.border.frame = self.bounds;
        self.border.lineWidth = 2.f;
        self.border.lineDashPattern = @[@4, @2];
        [self.layer addSublayer:self.border];
    }
}

@end
