/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSViewController.h"
#import "FSReorderableCollectionView.h"

@class FSThumbnailViewController;
@protocol FSPageOrganizerDelegate;
@protocol FSThumbnailViewControllerDelegate;
@protocol IRotationEventListener;

@class FSThumbnailChildViewController;
NS_ASSUME_NONNULL_BEGIN

@interface FSThumbnailViewController : FSViewController <IRotationEventListener>

@property (nonatomic, weak, nullable) id<FSThumbnailViewControllerDelegate> delegate;
@property (nonatomic, weak, nullable) id<FSPageOrganizerDelegate> pageManipulationDelegate;
@property (nonatomic, strong, nonnull) FSPDFDoc *document;
@property (nonatomic) BOOL isEditing;
@property (nonatomic, assign, readonly) BOOL hasPasteItemInChildViewController;
@property (nonatomic, strong) FSThumbnailChildViewController *currentThumbnailChildViewController;
@property (nonatomic, strong, readonly) FSItem *segmentCurrentItem;
@property (nonatomic, strong, readonly) NSArray<FSItem *> *segmentItems;
@property (nonatomic, weak) FSPDFViewCtrl *pdfViewCtrl;
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;

- (void)movePageIndexs:(NSArray <NSNumber *> *)pageIndexs toIndex:(NSUInteger)index beforeOrAfter:(BOOL)before;
- (void)duplicatePages:(void (^)(NSString *errorInfo))completionBlock;
- (void)onCellSelectedOrDeselected;
- (void)showInsertMenu:(UIView *)view;
- (FSPDFDoc *)getCurrnetChildViewOfDocForSelectedPageIndexs;
- (void)insertPages:(NSString *)path success:(void (^)(NSString *result))success error:(void (^)(NSString *errorInfo))error;
@end

@protocol FSThumbnailViewControllerDelegate

- (void)exitThumbnailViewController:(FSThumbnailViewController *)thumbnailViewController;
- (void)thumbnailViewController:(FSThumbnailViewController *)thumbnailViewController openPage:(int)page;
- (void)thumbnailViewController:(FSThumbnailViewController *)thumbnailViewController getThumbnailForPageAtIndex:(NSUInteger)index thumbnailSize:(CGSize)thumbnailSize needPause:(BOOL (^__nullable)(void))needPause callback:(void (^)(UIImage *))callback;

@end

NS_ASSUME_NONNULL_END
