/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSAddToolsViewController.h"
#import "FSDataItem.h"
#import "FSReadToolbarItem+Private.h"

FSSYNTH_INTERFACE_IMP_CLASS(_FSAddToolEntryDivider, UIView)

CGFloat const kTop_Bottom_Margin = 12.f;
CGFloat const kVerticalSpace_Margin = 14.f;

typedef void (^FSAddedToolsViewMoveItemCallBack)(NSUInteger sourceIndex, NSUInteger destIndex);
typedef void (^FSAddedToolsViewRemoveItemCallBack)(NSUInteger index);
@interface _FSAddedToolsView : UIView

@property(nonatomic, copy)FSAddedToolsViewMoveItemCallBack moveItemCallBack;
@property(nonatomic, copy)FSAddedToolsViewRemoveItemCallBack removeItemCallBack;

- (void)addToolItem:(FSItem *)item;
- (void)deleteToolItem:(FSItem *)item;
@end

@interface FSAddToolEntryTableViewCell : FSTitleEstimatedTableViewCell
@property (nonatomic, strong) FSItem *propertyItem;
@property (nonatomic, assign, readwrite) BOOL installed;

@end
@implementation FSAddToolEntryTableViewCell
@synthesize installed = _installed;

- (void)installConstraints{
    if (_installed) return;
    !self.willInstallConstraints ? : self.willInstallConstraints(self);
    if (self.leftView) {
        [self.leftView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.fs_equalToSuperview().offset(self.leftMargin).priorityHigh();
            make.centerY.fs_equalToSuperview();
            if (!CGSizeEqualToSize(CGSizeZero, self.leftViewSzie)) {
                make.size.mas_lessThanOrEqualTo(self.leftViewSzie);
            }else if(!CGSizeEqualToSize(CGSizeZero, self.leftView.fs_size)){
                make.width.mas_lessThanOrEqualTo(self.leftView.fs_width);
                make.height.mas_lessThanOrEqualTo(self.leftView.fs_height);
            }else{
                make.size.mas_equalTo(self.leftView).priority(MASLayoutPriorityFittingSizeLevel);
            }
        }];
    }
    
    if (self.rightView) {
        [self.rightView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.right.fs_equalToSuperview().offset(-self.rightMargin).priorityHigh();
            make.centerY.fs_equalToSuperview();
            make.size.mas_equalTo(self.rightView).priority(MASLayoutPriorityFittingSizeLevel);
        }];
    }
    _installed = YES;
    !self.didInstallConstraints ? : self.didInstallConstraints(self);
}
@end

@interface FSAddToolTitleTableViewCell : FSTitleEstimatedTableViewCell @end
@implementation FSAddToolTitleTableViewCell @end

@interface FSAddToolsViewController ()<IAnnotPropertyListener, FSPopupCarrierViewDelegate>

@property (nonatomic, strong) _FSAddedToolsView *addedToolsView;
@property (nonatomic, strong) UILabel *toolListLab;

@property (nonatomic, strong) NSMutableArray<FSReadToolbarItem *> *readToolItems;
@property (nonatomic, weak) NSIndexPath *currentPropertyIndexPath;
@end

@implementation FSAddToolsViewController

- (void)initialize{
    [super initialize];
    self.shouldLoadDataOnViewDidLoad = YES;
}

- (FSViewControllerManager *)viewControllerManager{
    return [super services];
}

- (FSMainTopbarToolTagItem *)currentTagItem{
    return [super params];
}

- (void)loadData{
    NSMutableArray *toolTypes = @[ FSLocalizedForKey(@"kEdit"), FSLocalizedForKey(@"kComment") , FSLocalizedForKey(@"kDrawing") , FSLocalizedForKey(@"kForm") ].mutableCopy;
    if (DEVICE_iPAD) {
        [toolTypes addObject: FSLocalizedForKey(@"kProtect")];
    }

    NSMutableArray *dataSource = @[].mutableCopy;
    for (int i = 0; i < toolTypes.count ; i++ ) {
        NSMutableArray *tmp = @[].mutableCopy;
        UIExtensionsManager *extensionsManager  = [self viewControllerManager].extensionsManager;
        switch (i) {
            case 0:
            {
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Audio]) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeAudio extensionsManager:extensionsManager addTool:YES]];
                }
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Video]) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeVideo extensionsManager:extensionsManager addTool:YES]];
                }
                if (!self.viewControllerManager.extensionsManager.config.disableLink) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeLink extensionsManager:extensionsManager addTool:YES]];
                }
            }
                break;
            case 1:
            {
                FSReadToolSettingsType toolSettingType = FSReadToolSettingsTypeNote;
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Note]) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeNote extensionsManager:extensionsManager addTool:YES]];
                }
                if (self.viewControllerManager.extensionsManager.config.loadAttachment) {
                    toolSettingType = FSReadToolSettingsTypeAttachments;
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeAttachments extensionsManager:extensionsManager addTool:YES]];
                }
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Stamp]) {
                    toolSettingType = FSReadToolSettingsTypeStamp;
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeStamp extensionsManager:extensionsManager addTool:YES]];
                }
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Freetext]) {
                    toolSettingType = FSReadToolSettingsTypeTypewriter;
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeTypewriter extensionsManager:extensionsManager addTool:YES]];
                }
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Textbox]) {
                    toolSettingType = FSReadToolSettingsTypeTextbox;
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeTextbox extensionsManager:extensionsManager addTool:YES]];
                }
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Callout]) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeCallout extensionsManager:extensionsManager addTool:YES]];
                }
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Highlight]) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeHighlight extensionsManager:extensionsManager addTool:YES]];
                }
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Underline]) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeUnderline extensionsManager:extensionsManager addTool:YES]];
                }
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Squiggly]) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeSquiggly extensionsManager:extensionsManager addTool:YES]];
                }
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Strikeout]) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeStrikeout extensionsManager:extensionsManager addTool:YES]];
                }
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Replace]) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeReplaceText extensionsManager:extensionsManager addTool:YES]];
                }
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Insert]) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeInsertText extensionsManager:extensionsManager addTool:YES]];
                }

                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Highlight]) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeAreaHighlight extensionsManager:extensionsManager addTool:YES]];
                }
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Distance]) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeMeasure extensionsManager:extensionsManager addTool:YES]];
                }
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Image]) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeImageAnnotation extensionsManager:extensionsManager addTool:YES]];
                }
            }
                break;
            case 2:
            {
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Pencil]) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypePencil extensionsManager:extensionsManager addTool:YES]];
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeHighlighter extensionsManager:extensionsManager addTool:YES]];
                }
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Eraser]) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeEraser extensionsManager:extensionsManager addTool:YES]];
                }
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Rectangle]) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeRectangle extensionsManager:extensionsManager addTool:YES]];
                }
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Oval]) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeOval extensionsManager:extensionsManager addTool:YES]];
                }
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Line]) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeLine extensionsManager:extensionsManager addTool:YES]];
                }
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Arrow]) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeArrow extensionsManager:extensionsManager addTool:YES]];
                }
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Polygon]) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypePolygon extensionsManager:extensionsManager addTool:YES]];
                }
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_PolyLine]) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypePolyLine extensionsManager:extensionsManager addTool:YES]];
                }
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Cloud]) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeCloud extensionsManager:extensionsManager addTool:YES]];
                }
            }
                break;
            case 3:
            {
                if (self.viewControllerManager.extensionsManager.config.loadForm) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeTextField extensionsManager:extensionsManager addTool:YES]];
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeCheckBox extensionsManager:extensionsManager addTool:YES]];
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeRadioButton extensionsManager:extensionsManager addTool:YES]];
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeComboBox extensionsManager:extensionsManager addTool:YES]];
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeListBox extensionsManager:extensionsManager addTool:YES]];
                    if (self.viewControllerManager.extensionsManager.config.loadSignature) {
                        [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeSignatureField extensionsManager:extensionsManager addTool:YES]];
                    }
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeImageField extensionsManager:extensionsManager addTool:YES]];
                }
            }
                break;
            case 4:
            {
                if ([self.viewControllerManager.extensionsManager.config.tools containsObject:Tool_Redaction]) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeRedact extensionsManager:extensionsManager addTool:YES]];
                }
                if (self.viewControllerManager.extensionsManager.config.loadEncryption) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypePassWord extensionsManager:extensionsManager addTool:YES]];
                }
                if (self.viewControllerManager.extensionsManager.config.loadSignature) {
                    [tmp addObject:[[FSReadToolbarItem alloc] initWithToolSettingType:FSReadToolSettingsTypeTrutedCertificates extensionsManager:extensionsManager addTool:YES]];
                }
            }
                break;
        }
        if (tmp.count) {
            [tmp insertObject:[[FSAddToolsTitleItem alloc] initWithTitle:toolTypes[i] subReadItems:tmp] atIndex:0];
            [dataSource addObject:tmp];
        }
    }
    
    self.dataSource = dataSource;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view.
    self.title = FSLocalizedForKey(@"kAddTools");
    self.navBarLeftTitle = FSLocalizedForKey(@"kClose");
    
    [self.tableView fs_registerClass:[FSAddToolTitleTableViewCell class]];
    [self.tableView fs_registerClass:[FSAddToolEntryTableViewCell class]];
    
    _addedToolsView = [_FSAddedToolsView new];
    [self.view addSubview:_addedToolsView];
    @fs_weakify(self)
    _addedToolsView.moveItemCallBack = ^(NSUInteger sourceIndex, NSUInteger destIndex) {
        @fs_strongify(self)
        if (sourceIndex < self.readToolItems.count &&
            destIndex < self.readToolItems.count) {
            id readToolItem = [self.readToolItems objectAtIndex:sourceIndex];
            [self.readToolItems removeObjectAtIndex:sourceIndex];
            [self.readToolItems insertObject:readToolItem atIndex:destIndex];
        }
    };
    
    _addedToolsView.removeItemCallBack = ^(NSUInteger index) {
        @fs_strongify(self)
        if (index < self.readToolItems.count) {
            [self.readToolItems removeObjectAtIndex:index];
        }
    };
    
    _toolListLab = [UILabel new];

    _toolListLab.text = FSLocalizedForKey(@"kToolList");
    _toolListLab.font = self.fs_standardGeneralFont_GF2;
    _toolListLab.textColor = self.fs_textColor_T4;
    [self.view addSubview:_toolListLab];
    
    self.readToolItems = @[].mutableCopy;
    self.currentTagItem.preAddToolItems = self.readToolItems;
    
    for (FSReadToolbarItem *readItem in self.currentTagItem.readToolbarItems) {
        [self addToolItemToToolsViewWithListItem:readItem];
    }
    
    [[self viewControllerManager].extensionsManager registerAnnotPropertyListener:self];
}

#define AddedToolsViewHeight 40.f + (40.f + kVerticalSpace_Margin)*(self.traitCollection.verticalSizeClass == UIUserInterfaceSizeClassCompact ? 2 : 3) - kVerticalSpace_Margin + kTop_Bottom_Margin * 2
- (void)makeConstraints{
    [super makeConstraints];
    
    [self.addedToolsView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.fs_equalToSuperview();
        make.top.equalTo(self.shouldShowNavigationBar ? self.navBar.mas_bottom : self.addedToolsView.superview);
        make.height.offset(AddedToolsViewHeight);
    }];
    
    [self.toolListLab mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.view.fs_mas_left).offset(ITEM_MARGIN_NORMAL);
        make.top.equalTo(self.addedToolsView.mas_bottom).offset(14.f);
    }];
    
    [self.tableView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.view.fs_mas_left).offset(ITEM_MARGIN_NORMAL);
        make.right.equalTo(self.view.fs_mas_right).offset(-ITEM_MARGIN_NORMAL);
        make.top.equalTo(self.toolListLab.mas_bottom);
        make.bottom.equalTo(self.view.fs_mas_bottom).offset(-ITEM_MARGIN_NORMAL);
    }];
}

- (void)traitCollectionDidChange:(UITraitCollection *)previousTraitCollection{
    [super traitCollectionDidChange:previousTraitCollection];
    [self.addedToolsView mas_updateConstraints:^(MASConstraintMaker *make) {
        make.height.offset(AddedToolsViewHeight);
    }];
    
    [self reloadData];
}

- (void)viewWillLayoutSubviews{
    [super viewWillLayoutSubviews];
    [self reloadData];
}

- (__kindof UITableViewCell *)tableView:(UITableView *)tableView dequeueReusableCellWithIdentifier:(NSString *)identifier forIndexPath:(NSIndexPath *)indexPath withDataItem:(FSReadToolbarItem *)dataItem{
    if (indexPath.row == 0) {
        identifier = NSStringFromClass([FSAddToolTitleTableViewCell class]);
    }else{
        identifier = NSStringFromClass([FSAddToolEntryTableViewCell class]);
    }
    return [super tableView:tableView dequeueReusableCellWithIdentifier:identifier forIndexPath:indexPath withDataItem:dataItem];
}

- (void)tableView:(UITableView *)tableView willDisplayCell:(UITableViewCell *)cell forRowAtIndexPath:(NSIndexPath *)indexPath{

    CGFloat radius = 4.f;
    cell.backgroundColor = UIColor.clearColor;
    CAShapeLayer *normalLayer = [[CAShapeLayer alloc] init];
    CAShapeLayer *selectLayer = [[CAShapeLayer alloc] init];
    CGRect bounds = CGRectInset(cell.bounds, 0, 0);
    NSInteger rowNum = [tableView numberOfRowsInSection:indexPath.section];
    UIBezierPath *bezierPath = nil;
    
    if (rowNum == 1) {
        bezierPath = [UIBezierPath bezierPathWithRoundedRect:bounds
                                           byRoundingCorners:UIRectCornerAllCorners
                                                 cornerRadii:CGSizeMake(radius, radius)];
    } else {
        if (indexPath.row == 0) {
            bezierPath = [UIBezierPath bezierPathWithRoundedRect:bounds
                                               byRoundingCorners:(UIRectCornerTopLeft|UIRectCornerTopRight)
                                                     cornerRadii:CGSizeMake(radius, radius)];
            
        } else if (indexPath.row == rowNum - 1) {
            bezierPath = [UIBezierPath bezierPathWithRoundedRect:bounds
                                               byRoundingCorners:(UIRectCornerBottomLeft|UIRectCornerBottomRight)
                                                     cornerRadii:CGSizeMake(radius, radius)];
        } else {
            bezierPath = [UIBezierPath bezierPathWithRect:bounds];
        }
    }
    
    normalLayer.path = bezierPath.CGPath;
    selectLayer.path = bezierPath.CGPath;
    
    
    UIView *nomarBgView = [[UIView alloc] initWithFrame:bounds];
    normalLayer.fillColor = self.fs_backgroundColor_B2.CGColor;
    [nomarBgView.layer insertSublayer:normalLayer atIndex:0];
    nomarBgView.backgroundColor = UIColor.clearColor;
    cell.backgroundView = nomarBgView;
    
    UIView *selectBgView = [[UIView alloc] initWithFrame:bounds];
    selectLayer.fillColor = self.fs_backgroundColor_B2.CGColor;
    [selectBgView.layer insertSublayer:selectLayer atIndex:0];
    selectBgView.backgroundColor = UIColor.clearColor;
    cell.selectedBackgroundView = selectBgView;
}

- (void)configureCell:(FSTitleEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(id)dataItem{
    
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    if (indexPath.row == 0) {
        cell.contentView.backgroundColor = [UIColor clearColor];
        if (!cell.rightView) {
            cell.rightView = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"read_tool_up_angle")];
        }
        
        if ([dataItem isKindOfClass:[FSAddToolsTitleItem class]]) {
            FSAddToolsTitleItem *addToolsTitleItem = dataItem;
            FSItem *angleItem = (FSItem *)cell.rightView;
            [angleItem setImage:addToolsTitleItem.unfold ? FSImageNamedAndNormalColor(@"read_tool_down_angle") : FSImageNamedAndNormalColor(@"read_tool_up_angle") forState:UIControlStateNormal];
            angleItem.accessibilityLabel = addToolsTitleItem.unfold ? FSLocalizedForKey(@"kUnfoldVoiceOver") : FSLocalizedForKey(@"kFoldVoiceOver");
            
            @weakify(self)
            angleItem.onTapClick = ^(FSItem * _Nonnull _item) {
                @strongify(self)
                addToolsTitleItem.unfold = !addToolsTitleItem.unfold;
                
                _item.accessibilityLabel = addToolsTitleItem.unfold ? FSLocalizedForKey(@"kUnfoldVoiceOver") : FSLocalizedForKey(@"kFoldVoiceOver");
                NSMutableArray *items = self.dataSource[indexPath.section];
                NSMutableArray *indexPaths = @[].mutableCopy;
                for (int i = 0; i < addToolsTitleItem.subReadItems.count ; i++) {
                    [indexPaths addObject:[NSIndexPath indexPathForRow:i+1 inSection:indexPath.section]];
                }
                if (addToolsTitleItem.unfold) {
                    [items addObjectsFromArray:addToolsTitleItem.subReadItems];
                    [self insertRowsAtIndexPaths:indexPaths];
                }else{
                    [items removeObjectsInArray:addToolsTitleItem.subReadItems];
                    [self deleteRowsAtIndexPaths:indexPaths];
                }

                dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)((addToolsTitleItem.unfold?0:0.3) * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                    [self.tableView reloadRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationNone];
                });
                
                [_item setImage:addToolsTitleItem.unfold ? FSImageNamedAndNormalColor(@"read_tool_down_angle") : FSImageNamedAndNormalColor(@"read_tool_up_angle") forState:UIControlStateNormal];
            };
            
            cell.estimatedTitleLabel.text = addToolsTitleItem.title;
            cell.estimatedTitleLabel.textColor = self.fs_textColor_T3;
        }
        
    }else{
        cell.contentView.backgroundColor = [UIColor clearColor];
        if (!cell.leftView) {
            cell.leftView = [UIImageView new];
        }
        
        if ([cell isKindOfClass:[FSAddToolEntryTableViewCell class]] && [dataItem isKindOfClass:[FSReadToolbarItem class]]) {
            FSReadToolbarItem *readToolbarItem = dataItem;
            FSAddToolEntryTableViewCell *entryCell = (FSAddToolEntryTableViewCell *)cell;
            FSItem *propertyItem = entryCell.propertyItem;
            if (!propertyItem) {
                propertyItem = [FSItem new];
                [propertyItem setImageEdgeInsets:UIEdgeInsetsMake(0, 0, 0, 9.f)];
                [entryCell.contentView addSubview:propertyItem];
                entryCell.propertyItem = propertyItem;
                
                _FSAddToolEntryDivider *line = [_FSAddToolEntryDivider new];
                line.backgroundColor = self.fs_dividerColor_P3;
                [propertyItem addSubview:line];
                
                [line mas_makeConstraints:^(MASConstraintMaker *make) {
                    make.centerY.fs_equalToSuperview();
                    make.width.offset(DIVIDE_VIEW_THICKNES);
                    make.height.offset(20.f);
                    make.right.fs_equalToSuperview().offset(-4.f);
                }];
            }
            
            if (readToolbarItem.toolSettings.settingsType == FSReadToolSettingsTypeStamp) {
                propertyItem.hidden = NO;
                [propertyItem setImage:[Utility getStampImageWithIcon:readToolbarItem.toolSettings.stampIcon] forState:UIControlStateNormal];
            }else{
                [propertyItem setImage:FSImageNamedAndCustomColor(@"read_property_color_ring",readToolbarItem.propertyColor) forState:UIControlStateNormal];
                propertyItem.hidden = !readToolbarItem.propertyColor;
            }
            if (readToolbarItem.settingsType == FSReadToolSettingsTypeRedact) {
                propertyItem.hidden = YES;
            }
            propertyItem.accessibilityLabel = FSLocalizedForKey(@"kProperty");
            @weakify(self)
            propertyItem.onTapClick = ^(FSItem * _Nonnull _item) {
                @strongify(self)
                self.currentPropertyIndexPath = indexPath;
                CGRect rect = [_item convertRect:_item.bounds toView:self.view];
                DevicePosition *devicePos = [[DevicePosition alloc] init];
                devicePos.phoneRect = rect;
                devicePos.inPhoneView = self.view;
                devicePos.padRect = _item.bounds;
                devicePos.inPadView = _item;
                devicePos.carrierViewDelegate = self;
                [self.viewControllerManager.extensionsManager showPropertyWithToolSettings:readToolbarItem.toolSettings devicePos:devicePos];
                self.viewControllerManager.extensionsManager.stampIconController.popoverPresentationController.permittedArrowDirections = UIPopoverArrowDirectionRight;
                
            };
            
            @weakify(entryCell)
            entryCell.didInstallConstraints = ^(id _entryCell) {
                @strongify(entryCell)
                [entryCell.propertyItem mas_makeConstraints:^(MASConstraintMaker *make) {
                    make.right.equalTo(entryCell.rightView.mas_left);
                    make.centerY.fs_equalToSuperview();
                }];
            };
            
            ((UIImageView *)cell.leftView).image = readToolbarItem.image;
            
            if (!cell.rightView) {
                cell.rightView = [FSItem createItemWithImage:FSImageNamedAndNormalColor(@"read_tool_add_unring")];
            }
            FSItem *addItem = (FSItem *)cell.rightView;
            addItem.accessibilityLabel = FSLocalizedForKey(@"kAdd");
            addItem.onTapClick = ^(FSItem * _Nonnull _item) {
                @strongify(self)
                FSReadToolbarItem *copyReadToolbarItem = [[FSReadToolbarItem alloc] initWithToolSettings:readToolbarItem.toolSettings.copy];
                copyReadToolbarItem.extensionsManager = [self viewControllerManager].extensionsManager;
                [self addToolItemToToolsViewWithListItem:copyReadToolbarItem];
            };
            
            cell.estimatedTitleLabel.text = readToolbarItem.title;
            cell.estimatedTitleLabel.numberOfLines = 1;
            [cell.estimatedTitleLabel mas_remakeConstraints:^(MASConstraintMaker *make) {
                if (cell.leftView) {
                    make.left.mas_equalTo(cell.leftView.mas_right).offset(cell.mediumSpace).priorityHigh();
                }else{
                    make.left.fs_equalToSuperview().offset(cell.leftMargin).priorityHigh();
                }
                make.right.lessThanOrEqualTo(propertyItem.mas_left);
                make.top.fs_equalToSuperview().offset(cell.topMargin).priorityHigh();
                make.bottom.fs_equalToSuperview().offset(-cell.bottomMargin).priorityHigh();
                make.height.mas_greaterThanOrEqualTo(TITLE_LABEL_MIN_HEIGHT).priorityHigh();
            }];
        }
    }
}

- (void)navigationBarBackTouch{
    [super navigationBarBackTouch];
    self.currentTagItem.readToolbarItems = self.readToolItems;
    [[self viewControllerManager].extensionsManager unregisterAnnotPropertyListener:self];
    [[self viewControllerManager].extensionsManager delayAutoFullScreen];
}

- (void)addToolItemToToolsViewWithListItem:(FSReadToolbarItem *)listItem{
    FSItem *item = nil;
    if (listItem.readToolbarItemType == FSReadToolbarItemTypeCustom) {
        item = [FSItem createItemWithImage:listItem.toolbarItem.customView.fs_snapshoot];
        item.accessibilityLabel = FSLocalizedForKey(@"kCustom");
    }else{
        if (listItem.toolbarItem.itemType == FSMainToolbarItemTypeAddTool ||
            listItem.toolbarItem.itemType == FSMainToolbarItemTypeMultiSelect) return;
        item = [FSReadToolbarItem createItemFromReadToolbarItem:listItem needBackground:NO];
        if (listItem.settingsType == FSReadToolSettingsTypeStamp) {
            [item setImage:[Utility getStampImageWithIcon:listItem.toolSettings.stampIcon] forState:UIControlStateNormal];
        }
        item.accessibilityLabel = listItem.title;
    }
    item.accessibilityHint = FSLocalizedForKey(@"kDobClickRemoveVoiceOver");

    UIImageView *imageView = [[UIImageView alloc] initWithImage:FSImageNamed(@"read_tool_delete_item")];
    [item addSubview:imageView];
    [imageView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.top.bottom.equalTo(imageView.superview);
    }];
    
    [self.readToolItems addObject:listItem];
    listItem.toolbarItem.preAddToolItem = item;
    [_addedToolsView addToolItem:item];
}

#pragma mark- IAnnotPropertyListener
- (void)onToolSettingsChanged:(FSReadToolSettings *)toolSettings{
    [self.tableView reloadRowsAtIndexPaths:@[self.currentPropertyIndexPath] withRowAnimation:UITableViewRowAnimationNone];
}

#pragma mark - UITableViewDelegate

- (UIView *)tableView:(UITableView *)tableView viewForHeaderInSection:(NSInteger)section{
    UIView *view = [UIView new];
    view.backgroundColor = self.view.backgroundColor;
    return view;
}

- (CGFloat)tableView:(UITableView *)tableView heightForHeaderInSection:(NSInteger)section{
    if (section == 0) {
        return 12.f;
    }
    return 14.f;
}

#pragma mark --FSPopupCarrierViewDelegate
- (UIView *)getPopupCarrierView{

    FSAddToolEntryTableViewCell *entryCell = [self.tableView cellForRowAtIndexPath:self.currentPropertyIndexPath];
    if ([entryCell isKindOfClass:[FSAddToolEntryTableViewCell class]]) {
        FSItem *propertyItem = entryCell.propertyItem;
        if (propertyItem != nil) {
            return propertyItem;
        }
    }
    return nil;
}


@end


@interface FSAddedToolsViewCell :UICollectionViewCell
-(void)starAnimation:(BOOL)animaiton;
@end
@implementation FSAddedToolsViewCell
-(void)starAnimation:(BOOL)animaiton
{
    if (animaiton) {
        CABasicAnimation *basicAnimation = [CABasicAnimation animationWithKeyPath:@"transform.rotation.z"];
        basicAnimation.fromValue = @(-M_PI_4/8);
        basicAnimation.toValue = @(M_PI_4/8);
        basicAnimation.duration = 0.08;
        basicAnimation.repeatCount = MAXFLOAT;
        basicAnimation.autoreverses = YES;
        [self.layer addAnimation:basicAnimation forKey:@"basicAnimation"];
    }else{
        [self.layer removeAllAnimations];
    }
}
@end

@interface _FSAddedToolsView () <UICollectionViewDelegate, UICollectionViewDataSource>

@property (nonatomic, strong) UILabel *tipLab;
@property (nonatomic, strong) UILabel *titleLab;
@property (nonatomic, strong) NSMutableArray *addedTools;
@property (nonatomic, strong) UICollectionView *collectionView;
@property (nonatomic, strong) FSAddedToolsViewCell *longPressCell;
@end
static NSString *const reuseIDForAddToolsCell = @"addToolsCell";
@implementation _FSAddedToolsView

- (instancetype)init{
    self = [super init];
    if (self) {
        [self initialize];
    }
    return self;
}

- (void)initialize{
    _addedTools = @[].mutableCopy;
    
    _titleLab = [UILabel new];
    _titleLab.text = FSLocalizedForKey(@"kAddedTools");
    _titleLab.font = self.fs_standardGeneralFont_GF2;
    _titleLab.textColor = self.fs_textColor_T4;
    [_titleLab sizeToFit];;
    [self addSubview:_titleLab];
    
    [_titleLab mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.fs_mas_left).offset(ITEM_MARGIN_NORMAL);
        make.top.fs_equalToSuperview().offset(ITEM_MARGIN_NORMAL - 1);
    }];
    
    UICollectionViewFlowLayout *layout = [[UICollectionViewFlowLayout alloc] init];
    layout.itemSize = CGSizeMake(40.0f, 40.0f);
    _collectionView = [[UICollectionView alloc] initWithFrame:CGRectZero collectionViewLayout:layout];
    _collectionView.backgroundColor = self.fs_backgroundColor_B2;
    _collectionView.contentInset = UIEdgeInsetsMake(12, ITEM_MARGIN_NORMAL, 12, ITEM_MARGIN_NORMAL);
    [_collectionView registerClass:[FSAddedToolsViewCell class] forCellWithReuseIdentifier:reuseIDForAddToolsCell];
    _collectionView.delegate = self;
    _collectionView.dataSource = self;
    _collectionView.allowsMultipleSelection = NO;
    [_collectionView fs_setLayerCornerRadius:4.f];
    [self addSubview:_collectionView];
    
    UILongPressGestureRecognizer *longPressGesture = [[UILongPressGestureRecognizer alloc] initWithTarget:self action:@selector(moveAction:)];
    _collectionView.userInteractionEnabled = YES;
    [_collectionView addGestureRecognizer:longPressGesture];
    
    [_collectionView mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.fs_mas_left).offset(ITEM_MARGIN_NORMAL);
        make.right.equalTo(self.fs_mas_right).offset(-ITEM_MARGIN_NORMAL);
        make.top.fs_equalToSuperview().offset(TOOL_BAR_HEIGHT_NORMAL);
        make.bottom.fs_equalToSuperview();
    }];
    
    _tipLab = [UILabel new];
    _tipLab.text = FSLocalizedForKey(@"kAddToolsTip");
    _tipLab.numberOfLines = 0;
    _tipLab.textAlignment = NSTextAlignmentCenter;
    _tipLab.font = self.fs_standardGeneralFont_GF3;
    _tipLab.textColor = self.fs_textColor_T2;
    [self addSubview:_tipLab];
    
    [_tipLab mas_makeConstraints:^(MASConstraintMaker *make) {
        make.width.equalTo(_collectionView.mas_width).offset(-(38.f * 2));
        make.center.equalTo(_collectionView);
    }];
}

- (void)moveAction:(UILongPressGestureRecognizer *)longGes {
    if (longGes.state == UIGestureRecognizerStateBegan) {
        NSIndexPath *selectPath = [self.collectionView indexPathForItemAtPoint:[longGes locationInView:longGes.view]];
        self.longPressCell = (FSAddedToolsViewCell*)[_collectionView cellForItemAtIndexPath:selectPath];
        if (self.longPressCell) {
            [self.longPressCell starAnimation:YES];
        }
        [self.collectionView beginInteractiveMovementForItemAtIndexPath:selectPath];
    }else if (longGes.state == UIGestureRecognizerStateChanged) {
        [self.collectionView updateInteractiveMovementTargetPosition:[longGes locationInView:longGes.view]];
    }else if (longGes.state == UIGestureRecognizerStateEnded) {
        [self.collectionView endInteractiveMovement];
        if (self.longPressCell) {
            [self.longPressCell starAnimation:NO];
        }
    }else {
        [self.collectionView cancelInteractiveMovement];
        if (self.longPressCell) {
            [self.longPressCell starAnimation:NO];
        }
    }
}

- (void)addToolItem:(FSItem *)item{
    item.userInteractionEnabled = NO;
    _tipLab.hidden = YES;
    [self.addedTools addObject:item];
    
    [_collectionView reloadData];
    [_collectionView scrollToItemAtIndexPath:[NSIndexPath indexPathForRow:self.addedTools.count-1 inSection:0] atScrollPosition:UICollectionViewScrollPositionBottom animated:YES];
}

- (void)deleteToolItem:(FSItem *)item{
    
    [self.addedTools removeObject:item];
    [item removeFromSuperview];
    [self.collectionView reloadData];
    if (!self.addedTools.count) _tipLab.hidden = NO;
}

- (void)layoutSubviews{
    [super layoutSubviews];
}


#pragma mark <UICollectionViewDataSource>

- (NSInteger)numberOfSectionsInCollectionView:(UICollectionView *)collectionView{
    return 1;
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return self.addedTools.count;
}

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {

    FSAddedToolsViewCell *cell = [collectionView dequeueReusableCellWithReuseIdentifier:reuseIDForAddToolsCell forIndexPath:indexPath];
    
    [cell.contentView.subviews makeObjectsPerformSelector:@selector(removeFromSuperview)];
    FSItem *item = [self.addedTools objectAtIndex:indexPath.row];
    [cell.contentView addSubview:item];
    [item mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.right.top.bottom.equalTo(cell.contentView);
    }];
    return cell;
}

- (void)traitCollectionDidChange:(UITraitCollection *)previousTraitCollection{
    [super traitCollectionDidChange:previousTraitCollection];

}

#pragma mark <UICollectionViewDelegate>

- (void)collectionView:(UICollectionView *)collectionView moveItemAtIndexPath:(NSIndexPath *)sourceIndexPath toIndexPath:(NSIndexPath *)destinationIndexPath {

    if (sourceIndexPath.row < self.addedTools.count &&
        destinationIndexPath.row < self.addedTools.count ) {
        
        if (sourceIndexPath.row == destinationIndexPath.row) {
            return;
        }
        
        id item = [self.addedTools objectAtIndex:sourceIndexPath.row];
        [self.addedTools removeObjectAtIndex:sourceIndexPath.row];
        [self.addedTools insertObject:item atIndex:destinationIndexPath.row];
        
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.5 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
            [collectionView reloadData];
            if (self.moveItemCallBack) {
                self.moveItemCallBack(sourceIndexPath.row, destinationIndexPath.row);
            }
        });
    }
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath{
    if (indexPath.row < self.addedTools.count) {
        FSItem* item = [self.addedTools objectAtIndex:indexPath.row];
        [self deleteToolItem:item];
        if (self.removeItemCallBack) {
            self.removeItemCallBack(indexPath.row);
        }
    }

}

#pragma mark <UICollectionViewDelegateFlowLayout>

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout*)collectionViewLayout sizeForItemAtIndexPath:(NSIndexPath *)indexPath{
    return CGSizeMake(40.0f, 40.0f);
}


- (CGFloat)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout minimumLineSpacingForSectionAtIndex:(NSInteger)section {
    
    return 14.0f;
}

- (CGFloat)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout minimumInteritemSpacingForSectionAtIndex:(NSInteger)section {
    return 14.0f;
}

@end
