/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#ifndef FSMainToolbar_Private_h
#define FSMainToolbar_Private_h

#import "FSMainToolbar.h"
#import "FSScrollToolbar.h"
#import "FSReadToolbarItem+Private.h"


@interface FSMainToolbarItem ()
@property (nonatomic, assign) FSMainToolbarItemType itemType;
@property (nonatomic, strong) __kindof UIView *customView;
@property (nonatomic, weak, readonly) FSItem *item;
@property (nonatomic, weak) FSItem *preAddToolItem;
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
- (instancetype)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager itemType:(FSMainToolbarItemType)itemType;
- (instancetype)initWithCustomView:(UIView *)customView spacing:(CGFloat)spacing;
@end

@interface FSMainToolbar ()<UIViewAdapter>
{
    @protected
    NSMutableArray<FSMainToolbarItem *> *_itemsForPositionLeft;
    NSMutableArray<FSMainToolbarItem *> *_itemsForPositionCenter;
    NSMutableArray<FSMainToolbarItem *> *_itemsForPositionRight;
}
@property (nonatomic, strong) __kindof UIStackView *contentView;
@property (nonatomic, strong) FSToolbar *toolbar;
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
@property (nonatomic, assign) BOOL hidden;
@property (nonatomic, assign, readonly) BOOL addedToSupview;

- (instancetype)initWithToolbar:(FSToolbar *)toolbar;
- (instancetype)initWithStyle:(FSToolbarStyle)style;
- (instancetype)initWithStyle:(FSToolbarStyle)style extensionsManager:(UIExtensionsManager *)extensionsManager;
- (void)addItem:(__kindof UIView *)item spacing:(CGFloat)spacing atPosition:(FSItemPosition)position;
- (FSMainToolbarItem *)getToolbarItemForItemType:(FSMainToolbarItemType)itemType;
- (void)setHidden:(BOOL)hidden animated:(BOOL)animated;
- (void)removeItem:(FSMainToolbarItem *)item;
- (void)adjustContentBarHeight;
@end


@interface FSMainTopbarToolTagItem ()
@property (nonatomic, assign) BOOL moduleLoad;
@property (nonatomic, weak) FSMainTopbar *ownerTopbar;
@property (nonatomic, strong) FSItem *barItem;
@property (nonatomic, strong) FSScrollToolbar *subTagToolbar;
@property (nonatomic, strong) FSReadToolbarItem *currentReadToolbarItem;
@property (nonatomic, assign) BOOL isOnToolbar;
@property (nonatomic, assign) BOOL isAutoAdjustOnToolbar;
@property (nonatomic, assign) BOOL isSourceOperation;
@property (nonatomic, assign) BOOL isIPADCompact;
@property (nonatomic, assign) BOOL needUndoMainItem;
@property (nonatomic, assign) BOOL addedMainItem;
@property (nonatomic, weak) NSMutableArray<FSReadToolbarItem *> *preAddToolItems;
- (void)scrollToCurrentReadToolbarItem;
@end

@protocol FSMainTopbarDelegate <IFSMainTopbarEventListener>

@end

@interface FSMainTopbar ()
@property (nonatomic, strong) FSToolbar *mainMenuTopbar;
@property (nonatomic, strong) __kindof UIView *subMenuTopbarContentView;
@property (nonatomic, strong, readwrite) FSMainTopbarToolTagItem *currentTagItem;
@property (nonatomic, strong) FSMainTopbarToolTagItem *lastTagItem;
@property (nonatomic, assign) UILayoutConstraintAxis axis;
@property (nonatomic, assign) BOOL didLayoutSubviews;
@property (nonatomic, assign) BOOL isSubPositionChanged;
@property (nonatomic, weak) id<FSMainTopbarDelegate> delegate;

- (void)resetHomeTagItem;
- (FSMainToolbarItem *)getCurrentSelectedItem;

- (void)resetViewTagItem;
- (void)switchToTag:(FSMainTopbarToolTag)tag;
- (FSMainTopbarToolTagItem*)getTagItemWithToolTag:(FSMainTopbarToolTag)toolTag;
@end

#endif /* FSMainToolbar_Private_h */
