/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSReadToolbarItem+Private.h"
#import "FSMainToolbar+Private.h"

@interface FSMainToolbarItem ()

@property (nonatomic, weak) FSMainToolbar *suptoolbar;
@end

@implementation FSMainToolbarItem

- (instancetype)initWithExtensionsManager:(UIExtensionsManager *)extensionsManager itemType:(FSMainToolbarItemType)itemType{
    self = [self initWithCustomView:[self creatItemWithExtensionsManager:extensionsManager type:itemType]];
    if (self) {
        if (itemType == FSMainToolbarItemTypePan) {
            _spacing = ITEM_TOP_ICON_MARGIN;
        }
        _itemType = itemType;
    }
    return self;
}

- (instancetype)initWithCustomView:(UIView *)customView{
    return [self initWithCustomView:customView spacing:0];
}

- (instancetype)initWithCustomView:(UIView *)customView spacing:(CGFloat)spacing{
    self = [super init];
    if (self) {
        _customView = customView;
        _spacing = spacing;
    }
    return self;
}

- (FSItem *)item{
    if ([self.customView isKindOfClass:[FSItem class]]) {
        return self.customView;
    }
    return nil;
}

- (__kindof UIView *)creatItemWithExtensionsManager:(UIExtensionsManager *)extensionsManager type:(FSMainToolbarItemType)type{
    FSItem *item = [FSItem new];
    switch (type) {
        case FSMainToolbarItemTypeBack:
            [item setImage:[self.fs_navBarBackIcon fs_imageWithTintColor:self.fs_iconColor_I6] forState:UIControlStateNormal];
            item.accessibilityIdentifier = @"read_exit_item";
            item.accessibilityLabel = FSLocalizedForKey(@"kBack");
            break;
        case FSMainToolbarItemTypeMore:
            [item setImage:FSImageNamedAndColor(@"common_more_gray", self.fs_iconColor_I6) forState:UIControlStateNormal];
            item.accessibilityIdentifier = @"read_common_more_item";
            item.accessibilityLabel = FSLocalizedForKey(@"kMore");
            break;
        case FSMainToolbarItemTypeSearch:
            [item setImage:FSImageNamedAndColor(@"read_search_gray", self.fs_iconColor_I6) forState:UIControlStateNormal];
            item.accessibilityIdentifier = @"read_search_item";
            item.accessibilityLabel = FSLocalizedForKey(@"kSearchContentItemVoiceOver");
            break;
        case FSMainToolbarItemTypePan:
            [item setImage:FSImageNamedAndColor(@"read_pan_h_hander", self.fs_iconColor_I5) forState:UIControlStateNormal];
            item.accessibilityIdentifier = @"read_pan_hander_item";
            item.accessibilityLabel = FSLocalizedForKey(@"kPanHanderItemVoiceOver");
            break;
        case FSMainToolbarItemTypeAddTool:
            item = [[FSItem alloc] initWithRrelation:FSImageTextRelationRight];
            [item setImage:FSImageNamedAndNormalColor(@"read_add_tool") forState:UIControlStateNormal];
            item.accessibilityIdentifier = @"read_add_tool_item";
            item.accessibilityLabel = FSLocalizedForKey(@"kAddTools");
            break;
        case FSMainToolbarItemTypeRedo:
            [item setImage:FSImageNamedAndNormalColor(@"read_redo") forState:UIControlStateNormal];
            item.accessibilityIdentifier = @"read_redo_item";
            item.accessibilityLabel = FSLocalizedForKey(@"kRedoItemVoiceOver");
            break;
        case FSMainToolbarItemTypeUndo:
            [item setImage:FSImageNamedAndNormalColor(@"read_undo") forState:UIControlStateNormal];
            item.accessibilityIdentifier = @"read_undo_item";
            item.accessibilityLabel = FSLocalizedForKey(@"kUndoItemVoiceOver");
            break;
        case FSMainToolbarItemTypeProperty: {
            [item setBackgroundImage:FSImageNamedAndNormalColor(@"common_unflod_backgroup") forState:FSItemControlStateAll];
            [item setBackgroundImage:[FSImageNamedAndNormalColor(@"common_unflod_backgroup") fs_applyingAlpha:IMAGE_DISABLED_ALPHA] forState:UIControlStateDisabled];
            UIImage *ringImage = nil;
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
            if (@available(iOS 13.0, *)) {
                ringImage = FSImageNamed(@"read_property_color_ring");
                UITraitCollection *lightTraitCollection = [UITraitCollection traitCollectionWithUserInterfaceStyle:UIUserInterfaceStyleLight];
                UIImage *lightRingImage = [ringImage fs_imageWithTintColor:[self.fs_iconColor_I3 resolvedColorWithTraitCollection:lightTraitCollection]];
                UITraitCollection *darkTraitCollection = [UITraitCollection traitCollectionWithUserInterfaceStyle:UIUserInterfaceStyleDark];
                UIImage *darkRingImage = [ringImage fs_imageWithTintColor:[self.fs_iconColor_I3 resolvedColorWithTraitCollection:darkTraitCollection]];
                lightRingImage = [lightRingImage fs_applyingAlpha:IMAGE_DISABLED_ALPHA];
                darkRingImage = [darkRingImage fs_applyingAlpha:IMAGE_DISABLED_ALPHA];
                ringImage = [UIImage fs_dynamicImageWithNormalImage:lightRingImage darkImage:darkRingImage];
            } else
#endif
            {
                ringImage = [FSImageNamedAndNormalColor(@"read_property_color_ring") fs_applyingAlpha:IMAGE_DISABLED_ALPHA];
            }
            [item setImage:ringImage forState:UIControlStateDisabled];
            item.enabled = NO;
            item.accessibilityIdentifier = @"read_property_color_item";
            item.accessibilityLabel = FSLocalizedForKey(@"kProperty");
            break;
        }
        case FSMainToolbarItemTypeMultiSelect:
        {
            UIImage *backgroundImage = [UIImage fs_imageWithColor:[UIColor clearColor]  size:CGSizeMake(40.f, 40.f)];
            [item setBackgroundImage:backgroundImage forState:UIControlStateNormal];
            [item setBackgroundImage:[Utility commonDynamicBackgroundImage] forState:UIControlStateSelected];
            [item setImage:FSImageNamedAndNormalColor(@"read_tool_multi_select") forState:UIControlStateNormal];
            item.accessibilityIdentifier = @"read_tool_multi_select_item";
            item.accessibilityLabel = FSLocalizedForKey(@"kSelect");
        }
            break;
        case FSMainToolbarItemTypeDivide:
            item.isAccessibilityElement = NO;
            [item setImage:[UIImage fs_imageWithColor:self.fs_dividerColor_P3 size:CGSizeMake(DIVIDE_VIEW_THICKNES, 20.f)] forState:UIControlStateNormal];
            [item setImageEdgeInsets:UIEdgeInsetsMake(0, ITEM_TOP_ICON_MARGIN, 0, ITEM_TOP_ICON_MARGIN)];
            break;
        case FSMainToolbarItemTypePanel:
            if ([extensionsManager fs_currentSceneIsCompact]) {
                [item setImage:FSImageNamedAndNormalColor(@"read_left_pannel_gray") forState:UIControlStateNormal];
                [item setImage:FSImageNamedAndNormalColor(@"read_left_pannel_gray") forState:UIControlStateSelected];
                [item setBackgroundImage:nil forState:UIControlStateNormal];
                [item setBackgroundImage:nil forState:UIControlStateSelected];
            }else{
                item = [FSItem createItemWithImage:FSImageNamedAndColor(@"read_panel_gray", self.fs_iconColor_I6) background:FSImageNamedAndColor(@"common_select_backgroup_gray", self.fs_backgroundColor_B13)];
                [item setImage:FSImageNamedAndColor(@"read_panel_gray",self.fs_textColor_T10) forState:UIControlStateSelected];
                [item setBackgroundImage:[UIImage new] forState:UIControlStateNormal];
                [item setBackgroundImage:[UIImage new] forState:UIControlStateDisabled];
                [item setBackgroundImage:[UIImage new] forState:UIControlStateHighlighted];
            }
            item.accessibilityIdentifier = @"read_pannel_item";
            item.accessibilityLabel = FSLocalizedForKey(@"kPanelListVoiceover");
            break;
        case FSMainToolbarItemTypeViewSettings:
            [item setImage:FSImageNamedAndNormalColor(@"read_view_gray") forState:UIControlStateNormal];
            item.accessibilityIdentifier = @"read_view_item";
            item.accessibilityLabel = FSLocalizedForKey(@"kViewSettings");
            break;
        case FSMainToolbarItemTypeThumbnail:
            if ([extensionsManager fs_currentSceneIsCompact]) {
                [item setImage:FSImageNamedAndNormalColor(@"read_thumbnail_gray") forState:UIControlStateNormal];
            }else{
                [item setImage:FSImageNamedAndColor(@"read_thumbnail_gray", self.fs_iconColor_I6) forState:UIControlStateNormal];
            }
            item.accessibilityIdentifier = @"read_thumbnail_item";
            item.accessibilityLabel = FSLocalizedForKey(@"kThumbnail");
            break;
        case FSMainToolbarItemTypeReadingBookmark:
            if ([extensionsManager fs_currentSceneIsCompact]) {
                [item setImage:FSImageNamedAndPrimaryColor(@"read_bookmark_select") forState:UIControlStateSelected];
                [item setImage:FSImageNamedAndNormalColor(@"read_bookmark_gray") forState:UIControlStateNormal];
            }else{

                [item setImage:FSImageNamedAndColor(@"read_bookmark_select",self.fs_iconColor_I7) forState:UIControlStateSelected];
                [item setImage:FSImageNamedAndColor(@"read_bookmark_gray", self.fs_iconColor_I6) forState:UIControlStateNormal];
            }
            
            item.accessibilityIdentifier = @"read_bookmark_item";
            item.accessibilityLabel = FSLocalizedForKey(@"kAddBookMarkItemVoiceOver");
            break;
    }
    return item;
}

- (void)setWidth:(CGFloat)width{
    _width = width;
    [self.customView mas_updateConstraints:^(MASConstraintMaker *make) {
        make.width.offset(width).priority(25.f);
    }];
}

- (void)setSpacing:(CGFloat)spacing{
    _spacing = spacing;
    if ([self.suptoolbar.itemsForPositionLeft containsObject:self]) {
        [self.customView mas_updateConstraints:^(MASConstraintMaker *make) {
            make.left.offset(spacing);
        }];
    }
    if ([self.suptoolbar.itemsForPositionCenter containsObject:self] && [self.suptoolbar.itemsForPositionCenter firstObject] == self) {
        [self.customView mas_updateConstraints:^(MASConstraintMaker *make) {
            make.left.offset(spacing);
        }];
    }
    if ([self.suptoolbar.itemsForPositionCenter containsObject:self]) {
        [self.customView mas_updateConstraints:^(MASConstraintMaker *make) {
            make.right.offset(-spacing);
        }];
    }
}

@end

@interface FSMainTopbarToolTagItem ()<IToolEventListener>

@end
@implementation FSMainTopbarToolTagItem

+ (instancetype)ItemWithToolTag:(FSMainTopbarToolTag)toolTag readToolbarItems:(nullable NSArray<FSReadToolbarItem *> *)readToolbarItems{
    FSMainTopbarToolTagItem *item = [[self alloc] init];
    if (item) {
        item.phoneTitleEdgeInsets = UIEdgeInsetsMake(0, 12, 0, 0);
        item.phoneImageEdgeInsets = UIEdgeInsetsMake(5, 0, 5, 4);
        item.padTitleEdgeInsets = UIEdgeInsetsMake(5, 8, 5, 8);
        item.padImageEdgeInsets = UIEdgeInsetsMake(0, 0, 0, 0);
    
        item->_toolTag = toolTag;
        item->_readToolbarItems = readToolbarItems.copy;
        item.isOnToolbar = YES;
        
        item.needUndoMainItem = YES;
        if (toolTag == FSMainTopbarToolTagView || toolTag == FSMainTopbarToolTagProtect) {
            item.needUndoMainItem = YES;
        }
    }
    return item;
}

- (void)setIsOnToolbar:(BOOL)isOnToolbar{
    _isOnToolbar = isOnToolbar;
    _isAutoAdjustOnToolbar = isOnToolbar;
}

- (void)setReadToolbarItems:(NSArray<FSReadToolbarItem *> *)readToolbarItems{

    FSReadToolbarItem *addReadItem = nil;
    FSReadToolbarItem *addSortItem = nil;
    FSReadToolbarItem *multiSelectReadItem = nil;
    for (FSReadToolbarItem *readItem in _readToolbarItems) {
        if (readItem.toolbarItem.itemType == FSMainToolbarItemTypeAddTool) {
            addReadItem = readItem;
        }else if (readItem.toolbarItem.itemType == FSMainToolbarItemTypeMultiSelect) {
            multiSelectReadItem = readItem;
        }else if (readItem.toolbarItem.itemType == FSMainToolbarItemTypeSortTool) {
            addSortItem = readItem;
        }
        if (addReadItem && multiSelectReadItem) {
            break;
        }
    }
    if (addReadItem) {
        NSMutableArray *tmp = readToolbarItems.mutableCopy;
        if (![readToolbarItems containsObject:addReadItem]) {
            [tmp addObject:addReadItem];
        }
        
        NSInteger totalCount = readToolbarItems.count;
        for (FSReadToolbarItem *readItem in readToolbarItems) {
            if (readItem.toolbarItem.itemType == FSMainToolbarItemTypeAddTool ||
                readItem.toolbarItem.itemType == FSMainToolbarItemTypeMultiSelect) {
                totalCount--;
            }
        }
        if (totalCount <= 0 && self.ownerTopbar.axis == UILayoutConstraintAxisHorizontal) {
            [addReadItem.toolbarItem.item setTitle:FSLocalizedForKey(@"kAddTools") forState:UIControlStateNormal];
        }else{
            [addReadItem.toolbarItem.item setTitle:nil forState:UIControlStateNormal];
        }
        readToolbarItems = tmp;
    }
    
    if(addSortItem){
        NSMutableArray *tmp = readToolbarItems.mutableCopy;
        if (![readToolbarItems containsObject:addSortItem]) {
            [tmp addObject:addSortItem];
        }
        readToolbarItems = tmp;
    }
    
    if (multiSelectReadItem && [self.ownerTopbar.extensionsManager fs_currentSceneIsCompact]) {
        NSMutableArray *tmp = readToolbarItems.mutableCopy;
        if (![readToolbarItems containsObject:multiSelectReadItem]) {
            [tmp insertObject:multiSelectReadItem atIndex:0];
        }
        readToolbarItems = tmp;
    }
    for (FSReadToolbarItem *readItem in readToolbarItems) {
        if (readItem.itemContentType & FSItemContentTypeText){
            if (self.ownerTopbar.axis == UILayoutConstraintAxisHorizontal && readItem.extensionsManager.toolbarManager.isShowReadBarItemDescript){
                [readItem.toolbarItem.item setTitle:readItem.title forState:UIControlStateNormal];
                [readItem.toolbarItem.item setTitle:readItem.selectTitle forState:UIControlStateSelected];
            }else{
                [readItem.toolbarItem.item setTitle:nil forState:UIControlStateNormal];
                [readItem.toolbarItem.item setTitle:nil forState:UIControlStateSelected];
            }
        }
    }
    
    NSMutableArray *changeItems = _readToolbarItems.mutableCopy;
    [changeItems removeObjectsInArray:readToolbarItems];
    for (FSReadToolbarItem *readItem in changeItems) {
        if (self.currentReadToolbarItem == readItem) {
            self.currentReadToolbarItem = nil;
            break;
        }
    }
    _readToolbarItems = readToolbarItems.copy;
    if (self.ownerTopbar.currentTagItem == self) {
        [self.ownerTopbar resetCurrentTagItem];
    }
}

- (void)setMainToolbarItems:(NSArray<FSMainToolbarItem *> *)mainToolbarItems{
    _mainToolbarItems = mainToolbarItems.copy;
    if (self.ownerTopbar.currentTagItem == self) {
        [self.ownerTopbar resetCurrentTagItem];
    }
}

- (void)setCurrentReadToolbarItem:(FSReadToolbarItem *)currentReadToolbarItem{
    if (!currentReadToolbarItem) {
        for (FSMainToolbarItem *mainItem in _mainToolbarItems) {
            if (mainItem.itemType == FSMainToolbarItemTypeProperty) {
                mainItem.item.onTapClick = nil;
                mainItem.item.enabled = NO;
                break;
            }
        }
        if ([self.ownerTopbar.extensionsManager.currentToolHandler respondsToSelector:@selector(toolSettings)]) {
            if (self.ownerTopbar.extensionsManager.currentToolHandler.toolSettings.settingsType == _currentReadToolbarItem.toolSettings.settingsType &&
                [self.ownerTopbar.extensionsManager getState] != FSUIManagerStateFillSign) {
                self.ownerTopbar.extensionsManager.currentToolHandler = nil;
            }
        }
        _currentReadToolbarItem.toolbarItem.item.selected = NO;
    }
    _currentReadToolbarItem = currentReadToolbarItem;
}

- (void)setNeedUndoMainItem:(BOOL)needUndoMainItem{
    if(_needUndoMainItem == needUndoMainItem) return;
    _needUndoMainItem = needUndoMainItem;
}

- (void)scrollToCurrentReadToolbarItem{
    [self scrollToItem:self.currentReadToolbarItem];
}

- (void)scrollToItem:(FSReadToolbarItem *)item{
    if (!item || ![self.readToolbarItems containsObject:item]) return;
    [self.subTagToolbar scrollTo:item.toolbarItem.item];
}
#pragma mark IHandlerEventListener

- (void)onToolChanged:(NSString *)lastToolName CurrentToolName:(NSString *)toolName {
    if (!toolName.length && self.currentReadToolbarItem) {
        self.currentReadToolbarItem = nil;
        if (self.ownerTopbar.extensionsManager.toolbarManager.mainTopbar.currentTagItem.toolTag != FSMainTopbarToolTagSign &&
            self.ownerTopbar.extensionsManager.currentState != FSUIManagerStateRedaction &&
            self.ownerTopbar.extensionsManager.currentState != FSUIManagerStateSpeech &&
            self.ownerTopbar.extensionsManager.currentState != FSUIManagerStateAutoFlip &&
            self.ownerTopbar.extensionsManager.currentState != FSUIManagerStatePanZoom ) {
            [self.ownerTopbar.extensionsManager changeState:self.ownerTopbar.extensionsManager.lastManagerStateWhenAnnotTool];
        }
    }
}

@end

FSSYNTH_INTERFACE_IMP_CLASS(_FSTopToolbarStackView, UIStackView)

@implementation FSMainToolbar

- (instancetype)initWithToolbar:(FSToolbar *)toolbar{
    self = [super init];
    if (self) {
        self.toolbar = toolbar;
        _FSTopToolbarStackView *contentView =  [_FSTopToolbarStackView new];
        contentView.axis = UILayoutConstraintAxisVertical;
        [contentView addArrangedSubview:self.toolbar];
        self.contentView = contentView;
        self.contentView.backgroundColor = self.toolbar.backgroundColor;
    
        _itemsForPositionLeft = @[].mutableCopy;
        _itemsForPositionCenter = @[].mutableCopy;
        _itemsForPositionRight = @[].mutableCopy;
        
        for (FSToolbarItem *item in toolbar.leftItems.items) {
            FSMainToolbarItem *mainItem = [[FSMainToolbarItem alloc] initWithCustomView:item.item spacing:item.margin];
            [_itemsForPositionLeft addObject:mainItem];
        }
        for (FSToolbarItem *item in toolbar.centerItems.items) {
            FSMainToolbarItem *mainItem = [[FSMainToolbarItem alloc] initWithCustomView:item.item spacing:item.margin];
            [_itemsForPositionCenter addObject:mainItem];
        }
        for (FSToolbarItem *item in toolbar.rightItems.items) {
            FSMainToolbarItem *mainItem = [[FSMainToolbarItem alloc] initWithCustomView:item.item spacing:item.margin];
            [_itemsForPositionRight addObject:mainItem];
        }
        _contentHeight = toolbar.contentBarHeight;
    }
    return self;
}

- (instancetype)initWithStyle:(FSToolbarStyle)style{
    FSToolbar *toolbar = [[FSToolbar alloc] initWithStyle:style];
    self = [self initWithToolbar:toolbar];
    if (self) {
        toolbar.adapter = self;
    }
    return self;
}

- (instancetype)initWithStyle:(FSToolbarStyle)style extensionsManager:(UIExtensionsManager *)extensionsManager{
    self = [self initWithStyle:style];
    if (self) {
        _extensionsManager = extensionsManager;
    }
    return self;
}

- (__kindof UIView *)getToolbarContentView{
    return self.contentView;
}

- (FSToolbar *)mainToolbar{
    return self.toolbar;
}

- (NSArray<FSMainToolbarItem *> *)itemsForPositionLeft{
    return _itemsForPositionLeft.copy;
}

- (NSArray<FSMainToolbarItem *> *)itemsForPositionCenter{
    return _itemsForPositionCenter.copy;
}

- (NSArray<FSMainToolbarItem *> *)itemsForPositionRight{
    return _itemsForPositionRight.copy;
}

- (void)setItemsForPositionLeft:(NSArray<FSMainToolbarItem *> *)itemsForPositionLeft{
    for (FSMainToolbarItem *item in _itemsForPositionLeft.copy) {
        [self removeItem:item atPosition:FSMainToolbarItemPositionLeft];
    }
    _itemsForPositionLeft = itemsForPositionLeft.mutableCopy;
    for (FSMainToolbarItem *item in _itemsForPositionLeft.copy) {
        [self addItem:item atPosition:FSMainToolbarItemPositionLeft];
    }
}

- (void)setItemsForPositionCenter:(NSArray<FSMainToolbarItem *> *)itemsForPositionCenter{
    for (FSMainToolbarItem *item in _itemsForPositionCenter.copy) {
        [self removeItem:item atPosition:FSMainToolbarItemPositionCenter];
    }
    _itemsForPositionCenter = itemsForPositionCenter.mutableCopy;
    for (FSMainToolbarItem *item in _itemsForPositionCenter.copy) {
        [self addItem:item atPosition:FSMainToolbarItemPositionCenter];
    }
}

- (void)setItemsForPositionRight:(NSArray<FSMainToolbarItem *> *)itemsForPositionRight{
    for (FSMainToolbarItem *item in _itemsForPositionRight.copy) {
        [self removeItem:item atPosition:FSMainToolbarItemPositionRight];
    }
    _itemsForPositionRight = itemsForPositionRight.mutableCopy;
    for (FSMainToolbarItem *item in _itemsForPositionRight.copy) {
        [self addItem:item atPosition:FSMainToolbarItemPositionRight];
    }
}

- (void)addItem:(FSMainToolbarItem *)item atPosition:(FSMainToolbarItemPosition)position{
    [self.mainToolbar addItem:item.customView spacing:item.spacing atPosition:(FSItemPosition)position];
    item.suptoolbar = self;
    [item.customView mas_updateConstraints:^(MASConstraintMaker *make) {
        make.top.bottom.fs_equalToSuperview().priority(25.f);
        make.width.offset(item.width).priority(25.f);
    }];
    
    switch (position) {
        case FSMainToolbarItemPositionLeft:
            if (![_itemsForPositionLeft containsObject:item]) [_itemsForPositionLeft addObject:item];
            break;
        case FSMainToolbarItemPositionCenter:
            if (![_itemsForPositionCenter containsObject:item]) [_itemsForPositionCenter addObject:item];
            break;
        case FSMainToolbarItemPositionRight:
            if (![_itemsForPositionRight containsObject:item]) [_itemsForPositionRight addObject:item];
            break;
    }
}

- (void)removeItemAtPosition:(FSMainToolbarItemPosition)position index:(NSUInteger)index{
    FSMainToolbarItem *item;
    switch (position) {
        case FSMainToolbarItemPositionLeft:
            if (index > _itemsForPositionLeft.count - 1) item = nil;
            item = _itemsForPositionLeft[index];
            break;
        case FSMainToolbarItemPositionCenter:
            if (index > _itemsForPositionCenter.count - 1) item = nil;
            item = _itemsForPositionCenter[index];
            break;
        case FSMainToolbarItemPositionRight:
            if (index > _itemsForPositionRight.count - 1) item = nil;
            item = _itemsForPositionRight[index];
            break;

    }
    if (item) [self removeItem:item atPosition:position];
}

- (void)removeItem:(FSMainToolbarItem *)item atPosition:(FSMainToolbarItemPosition)position{
    [self.mainToolbar removeItem:item.customView atPosition:(FSItemPosition)position];
    item.suptoolbar = nil;
    
    switch (position) {
        case FSMainToolbarItemPositionLeft:
            [_itemsForPositionLeft removeObject:item];
            break;
        case FSMainToolbarItemPositionCenter:
            [_itemsForPositionCenter removeObject:item];
            break;
        case FSMainToolbarItemPositionRight:
            [_itemsForPositionRight removeObject:item];
            break;
    }
}

- (void)removeItem:(FSMainToolbarItem *)item{
    
    FSItemPosition position = FSItemPositionLeft;
    if ([_itemsForPositionLeft containsObject:item]) {
        position = FSItemPositionLeft;
    }else if ([_itemsForPositionCenter containsObject:item]){
        position = FSItemPositionCenter;
    }else if ([_itemsForPositionRight containsObject:item]){
        position = FSItemPositionRight;
    }
    
    [self.mainToolbar removeItem:item.customView atPosition:(FSItemPosition)position];
    item.suptoolbar = nil;
    
    switch (position) {
        case FSMainToolbarItemPositionLeft:
            [_itemsForPositionLeft removeObject:item];
            break;
        case FSMainToolbarItemPositionCenter:
            [_itemsForPositionCenter removeObject:item];
            break;
        case FSMainToolbarItemPositionRight:
            [_itemsForPositionRight removeObject:item];
            break;
    }
}

- (void)addItem:(__kindof UIView *)item spacing:(CGFloat)spacing atPosition:(FSItemPosition)position{
    FSMainToolbarItem *mainToolbarItem = [[FSMainToolbarItem alloc] initWithCustomView:item spacing:spacing];
    [self addItem:mainToolbarItem atPosition:(FSMainToolbarItemPosition)position];
}

- (void)setBackgroundColor:(UIColor *)color{
    self.toolbar.backgroundColor = color;
}

- (FSMainToolbarItem *)getToolbarItemForItemType:(FSMainToolbarItemType)itemType{
    for (FSMainToolbarItem *item in self.itemsForPositionLeft) {
        if (item.itemType == itemType) {
            return item;
        }
    }
    for (FSMainToolbarItem *item in self.itemsForPositionCenter) {
        if (item.itemType == itemType) {
            return item;
        }
    }
    for (FSMainToolbarItem *item in self.itemsForPositionRight) {
        if (item.itemType == itemType) {
            return item;
        }
    }
    return nil;
}

- (void)setContentHeight:(CGFloat)contentHeight{
    _contentHeight = contentHeight;
    [self adjustContentBarHeight];
}

- (BOOL)hidden{
    return self.contentView.hidden;
}

- (void)setHidden:(BOOL)hidden{
    [self setHidden:hidden animated:NO];
}

- (BOOL)addedToSupview{
    return self.contentView.superview ? YES : NO;
}

- (void)setHidden:(BOOL)hidden animated:(BOOL)animated{
    self.contentView.hidden = hidden;
    
    if (animated) {
        if (self.toolbar.style == FSToolbarStyleTop) {
            [Utility addAnimation:self.contentView.layer type:self.contentView.hidden ? kCATransitionReveal : kCATransitionMoveIn subType:self.contentView.hidden ? kCATransitionFromTop : kCATransitionFromBottom timeFunction:kCAMediaTimingFunctionEaseInEaseOut duration:0.3];
        }
        if (self.toolbar.style == FSToolbarStyleBottom) {
            [Utility addAnimation:self.contentView.layer type:self.contentView.hidden ? kCATransitionReveal : kCATransitionMoveIn subType:self.contentView.hidden ? kCATransitionFromBottom : kCATransitionFromTop timeFunction:kCAMediaTimingFunctionEaseInEaseOut duration:0.3];
            [Utility addAnimation:self.contentView.layer type:self.contentView.hidden ? kCATransitionReveal : kCATransitionMoveIn subType:self.contentView.hidden ? kCATransitionFromBottom : kCATransitionFromTop timeFunction:kCAMediaTimingFunctionEaseInEaseOut duration:0.3];
        }
    }
}

- (void)adjustContentBarHeight{
    [_toolbar setContentBarHeight:_contentHeight];
}

#pragma mark - UIViewAdapter
- (void)traitCollectionDidChange:(UITraitCollection *)previousTraitCollection fromView:(__kindof UIView *)view{}
- (void)layoutSubviewsFromView:(__kindof UIView *)view{}
@end

FSSYNTH_INTERFACE_IMP_CLASS(_FSSubMenuTopbarContentView, FSAdapterUIView)

@interface FSMainTopbar () <IWindowFrameChangedListener>
{
    @protected
    CGFloat _contentHeight;
}
@property (nonatomic, strong) FSWeakPointerArray *listeners;
@property (nonatomic, weak) FSToolsPopoverViewController *toolsPop;
@property (nonatomic, weak) FSToolbarSettingsPopover *toolSettingsPop;
@property (nonatomic, assign) CGSize mainMenuTopbarSize;
@property (nonatomic, strong) FSItem *toolSettingsItem;
@property (nonatomic, strong) UIColor *tagItemBgColor;
@property (nonatomic, strong) UIColor *backgroundColor;

@end

@implementation FSMainTopbar
@dynamic itemsForPositionCenter;

- (instancetype)initWithStyle:(FSToolbarStyle)style extensionsManager:(UIExtensionsManager *)extensionsManager{
    self = [super initWithStyle:FSToolbarStyleTop extensionsManager:extensionsManager];
    if (self) {
        
        [BindThemeKeySel(self.toolbar, stateBar) fs_updateThemeCompleted:^(UIView *  _Nonnull target, SEL  _Nonnull sel) {
            target.backgroundColor = target.fs_backgroundColor_B12;
        }];
        
        FSToolbar *mainMenuTopbar = [[FSToolbar alloc] initWithStyle:FSToolbarStyleDefault];
        mainMenuTopbar.adapter = self;
        mainMenuTopbar.backgroundColor = self.fs_backgroundColor_B12;
        [BindThemeKeySel(mainMenuTopbar, backgroundView) fs_updateThemeCompleted:^(UIImageView*  _Nonnull target, SEL  _Nonnull sel) {
            target.backgroundColor = self.fs_backgroundColor_B12;
            
        }];
        [self.toolbar addSubview:mainMenuTopbar];
        self.mainMenuTopbar = mainMenuTopbar;
        
        _FSSubMenuTopbarContentView *subMenuTopbarContentView = [_FSSubMenuTopbarContentView new];
        subMenuTopbarContentView.adapter = self;
        [self.toolbar addSubview:subMenuTopbarContentView];
        self.subMenuTopbarContentView = subMenuTopbarContentView;
        
        [mainMenuTopbar mas_makeConstraints:^(MASConstraintMaker *make) {
            make.top.left.right.fs_equalToSuperview();
            make.height.mas_equalTo(TOOL_BAR_HEIGHT_NORMAL);
        }];
        
        //temp constraints
        [subMenuTopbarContentView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.bottom.left.right.fs_equalToSuperview().priorityMedium();
            make.top.mas_equalTo(mainMenuTopbar.mas_bottom).priorityMedium();
        }];
        
        [self.extensionsManager registerWindowFrameChangedListener:self];
        
        self.horizontalContentHeight = TOOL_BAR_HEIGHT_NORMAL * 2;
        self.verticalContentWidth = TOOL_BAR_HEIGHT_NORMAL;
        
        self.tagItemBgColor = self.fs_backgroundColor_B13;
    }
    return self;
}

- (void)dealloc
{
    [self.extensionsManager unregisterWindowFrameChangedListener:self];
}

- (FSToolbar *)mainToolbar{
    return self.mainMenuTopbar;
}

- (FSItem *)toolSettingsItem{
    if(!_toolSettingsItem){
        _toolSettingsItem = [FSItem createItemWithImage:FSImageNamedAndColor(@"read_tool_settings_popover_normal", self.fs_iconColor_I6) imageSelected:FSImageNamedAndColor(@"read_tool_settings_popover_selected", self.fs_iconColor_I6)];
        _toolSettingsItem.customAccessibilityTraits = UIAccessibilityTraitNone;
        _toolSettingsItem.accessibilityLabel = FSLocalizedForKey(@"kToolbarModeOver");
        @fs_weakify(self)
        _toolSettingsItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @fs_strongify(self)
            _item.selected = !_item.selected;
            if (_item.selected) {
                _item.accessibilityLabel = FSLocalizedForKey(@"kExitVoiceOver");
            }else{
                _item.accessibilityLabel = FSLocalizedForKey(@"kToolbarModeOver");
            }
            FSToolbarSettingsPopover *toolSettingsPop = (FSToolbarSettingsPopover *)[self.extensionsManager.viewCtrlManager popPopoverCtrlClass:[FSToolbarSettingsPopover class] groups:nil sourceView:!DEVICE_iPHONE ? ^UIView * _Nonnull(UIPopoverPresentationController * _Nonnull popover) {
                popover.permittedArrowDirections = UIPopoverArrowDirectionUp;
                return _item;
            } : nil autoAdapt:NO];
            toolSettingsPop.toolTagItems = self.tagItems;
            toolSettingsPop.settingsChanged = ^(FSToolbarSettingsPopover * _Nonnull popover) {
                [self rearrangeTagItems:popover.toolTagItems];
            };
            
            [self.extensionsManager cancelAutoFullScreen];
            
            toolSettingsPop.dismissCompletion = ^{
                _item.selected = NO;
                _item.accessibilityLabel = FSLocalizedForKey(@"kToolbarModeOver");
                [self.extensionsManager delayAutoFullScreen];
                self.toolSettingsPop = nil;
                if (self.extensionsManager.shouldShowMenu) {
                    [self.extensionsManager showMenu];
                }
            };
            
            [toolSettingsPop setAction:^(FSPopoverItem * _Nonnull popoverItem, int index) {
                FSMainTopbarToolTagItem *tagItem = [self.tagItems objectAtIndex:index];
                tagItem.barItem.onTapClick(tagItem.barItem);
            }];
            toolSettingsPop.selectedItem = self.currentTagItem;
            
            self.toolSettingsPop = toolSettingsPop;
        };
    }
    return _toolSettingsItem;
}

- (FSWeakPointerArray *)listeners{
    if(!_listeners){
        _listeners = [FSWeakPointerArray array];
    }
    return _listeners;
}

- (void)setAttachView:(__kindof UIView *)attachView{
    if (_attachView) [self.contentView removeArrangedSubview:_attachView];
    _attachView = attachView;
    if (_attachView) [self.contentView addArrangedSubview:attachView];
}

- (void)setItemsForPositionLeft:(NSArray<FSMainToolbarItem *> *)itemsForPositionLeft{
    [super setItemsForPositionLeft:itemsForPositionLeft];
    [self updateTagItemsUI];
}

- (void)setItemsForPositionRight:(NSArray<FSMainToolbarItem *> *)itemsForPositionRight{
    [super setItemsForPositionRight:itemsForPositionRight];
    [self updateTagItemsUI];
}

- (void)setTagItems:(NSArray<FSMainTopbarToolTagItem *> *)tagItems{
    if (_tagItems) {
        for (FSMainTopbarToolTagItem *tagItem in _tagItems) {
            [self.mainMenuTopbar removeItem:tagItem.barItem atPosition:FSItemPositionCenter];
            tagItem.ownerTopbar = nil;
            [self.extensionsManager unregisterToolEventListener:tagItem];
        }
    }
    _tagItems = tagItems.copy;
    if (![_tagItems containsObject:self.currentTagItem]) {
        self.currentTagItem = nil;
    }
    if (![_tagItems containsObject:self.lastTagItem]) {
        self.lastTagItem = nil;
    }
    for (int i = 0; i < _tagItems.count; i ++ ) {
        FSMainTopbarToolTagItem *tagItem = _tagItems[i];
        if (!tagItem.addedMainItem && self.extensionsManager.toolbarManager.rangeItemsed) [self.extensionsManager.toolbarManager tagItemAddMainItem:tagItem];
        if ((tagItem.toolTag == FSMainTopbarToolTagComment && !self.extensionsManager.config.defaultSettings.annotations) ||
            (tagItem.toolTag == FSMainTopbarToolTagDrawing && !self.extensionsManager.config.defaultSettings.annotations) ||
            (tagItem.toolTag == FSMainTopbarToolTagForm && !self.extensionsManager.config.loadForm)) {
            tagItem.moduleLoad = NO;
        }else{
            tagItem.moduleLoad = YES;
        }
        [self.extensionsManager registerToolEventListener:tagItem];
        
        if (!tagItem.barItem) {
            NSString *title = tagItem.title;
            FSItem *barItem = [FSItem new];
          
            barItem.accessibilityIdentifier = [NSString stringWithFormat:@"MainTopbarToolTag_%ld",tagItem.toolTag];
            barItem.accessibilityValue = FSLocalizedForKey(@"kTabItemVoiceOver");
            barItem.accessibilityTraits = UIAccessibilityTraitNone;
            barItem.customAccessibilityTraits = UIAccessibilityTraitNone;
            barItem.layer.masksToBounds = YES;
            barItem.layer.cornerRadius = 6;
            [barItem setTitle:title forState:UIControlStateNormal];
            barItem.titleLabel.lineBreakMode = NSLineBreakByTruncatingMiddle;
            [barItem.titleLabel setContentCompressionResistancePriority:UILayoutPriorityDefaultHigh - 1 forAxis:UILayoutConstraintAxisHorizontal];
            [barItem setTitleColor:self.fs_textColor_T11 forState:UIControlStateNormal];
            barItem.font = self.fs_standardGeneralFont_GF2;
            barItem.owner = tagItem;
            
            @fs_weakify(self)
            [BindThemeKeySel(barItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem * _Nonnull target, SEL  _Nonnull sel) {
                @fs_strongify(self)
                [target setTitleColor:target.fs_textColor_T10 forState:UIControlStateSelected];
                [target setImage:[target.currentImage fs_imageWithTintColor:target.fs_textColor_T10] forState:UIControlStateSelected];
                [target setBackgroundImage:[UIImage fs_imageWithColor:self.tagItemBgColor] forState:UIControlStateSelected];
                [target setBackgroundImage:nil forState:UIControlStateNormal];
            }];
            tagItem.barItem = barItem;

        }
        tagItem.ownerTopbar = self;
    }
    if (self.currentTagItem == nil || ![_tagItems containsObject:self.currentTagItem]) {
        FSMainTopbarToolTagItem *currentTagItem = [_tagItems firstObject];
        if (_lastTagItem) {
            currentTagItem = _lastTagItem;
        }
        currentTagItem.isIPADCompact = DEVICE_iPAD && [self.extensionsManager fs_currentSceneiPadHorizontalCompact];
        self.currentTagItem = currentTagItem;
    }
    
    if (self.currentTagItem && !self.currentTagItem.barItem.superview) {
        if (DEVICE_iPHONE || [self.extensionsManager fs_currentSceneiPadHorizontalCompact]) {
            [self satisfyShortViewDisplay];
        }else{
            [self rearrangeTagItemsSatisfyViewDisplay];
        }
    }
}

- (void)rearrangeTagItems:(NSArray<FSMainTopbarToolTagItem *> *)tagItems{
    if (_tagItems) {
        for (FSMainTopbarToolTagItem *tagItem in _tagItems) {
            if (!tagItem.isOnToolbar) {
                [self.mainMenuTopbar removeItem:tagItem.barItem atPosition:FSItemPositionCenter];
                tagItem.ownerTopbar = nil;
                [self.extensionsManager unregisterToolEventListener:tagItem];
            }
        }
    }
    _tagItems = tagItems.copy;
    
    [self rearrangeTagItemsSatisfyViewDisplay];
}

- (void)rearrangeTagItemsSatisfyViewDisplay{
    if (!self.currentTagItem.isOnToolbar) {
        self.currentTagItem = nil;
    }
    FSItem *currentBarItem = self.currentTagItem.barItem;
    currentBarItem.selected = YES;
    currentBarItem.imageView.hidden = NO;
    [currentBarItem setImage:nil forState:UIControlStateSelected];
   
    [self.mainMenuTopbar removeItemAtPosition:FSItemPositionCenter];
    
    CGFloat minSpace = 40.f;
    CGFloat menuViewHalfW = self.mainMenuTopbar.fs_width / 2;
    CGFloat barLeftW = [self.mainMenuTopbar getWidthWithPosition:FSItemPositionLeft];
    CGFloat barRightW = [self.mainMenuTopbar getWidthWithPosition:FSItemPositionRight];
    BOOL satisBarCenterHalfW = NO;
    
    BOOL (^addTagItem)(FSMainTopbarToolTagItem *) = ^BOOL(FSMainTopbarToolTagItem *tagItem){
        tagItem.barItem.font = self.fs_standardGeneralFont_GF2;
        tagItem.barItem.imageView.hidden = (self.currentTagItem != tagItem);

//        [tagItem.barItem setImage:[UIImage fs_imageWithColor:self.fs_textColor_T10 size:CGSizeMake(tagItem.barItem.titleLabel.intrinsicContentSize.width - 5, 2.f)] forState:UIControlStateSelected];
        [tagItem.barItem rebuildImageTextRelation:FSImageTextRelationTop];
        @weakify(self)
        tagItem.barItem.onTapClick = ^(FSItem * _Nonnull _item) {
            @strongify(self)
            if (self.currentTagItem != _item.owner) {
                if (((FSMainTopbarToolTagItem *)_item.owner).toolTag == FSMainTopbarToolTagEdit || ((FSMainTopbarToolTagItem *)_item.owner).toolTag == FSMainTopbarToolTagForm) {
                    @weakify(self)
                    [self.extensionsManager.permissionProvider checkPermission:FSFunctionEdit checkPermissionState:^(FSPermissionState state) {
                        @strongify(self)
                        if (state == FSPermissionStateShow) {
                            self.currentTagItem.barItem.imageView.hidden = YES;
                            self.currentTagItem.barItem.selected = NO;
                            self.currentTagItem = _item.owner;
                            _item.selected = YES;
                            _item.imageView.hidden = NO;
                        }
                    }];
                }else{
                    self.currentTagItem.barItem.imageView.hidden = YES;
                    self.currentTagItem.barItem.selected = NO;
                    self.currentTagItem = _item.owner;
                    _item.selected = YES;
                    _item.imageView.hidden = NO;
                }
            }
        };
        [self.mainMenuTopbar addItem:tagItem.barItem spacing:[self.mainMenuTopbar getItemCountWithPosition:FSItemPositionCenter] == 0 ? 0 : 32.f  atPosition:FSItemPositionCenter];
        CGFloat barCenterHalfW = ([self.mainMenuTopbar getWidthWithPosition:FSItemPositionCenter layoutIfNeeded:YES] + 16.f + 40)  / 2;
        return (menuViewHalfW - barLeftW -  barCenterHalfW) < minSpace || (menuViewHalfW - barRightW -  barCenterHalfW) < minSpace;
    };
    for (int i = 0; i < self.tagItems.count; i++) {
        if (i == 0 && self.currentTagItem) {
            satisBarCenterHalfW = addTagItem(self.currentTagItem);
        }
        FSMainTopbarToolTagItem *tagItem = self.tagItems[i];
        if (tagItem == self.currentTagItem) {
            [self.mainMenuTopbar removeItem:self.currentTagItem.barItem];
            if (satisBarCenterHalfW && tagItem.isOnToolbar) {
                addTagItem(tagItem);
                continue;
            }
        }
        if (tagItem.isOnToolbar) tagItem.isAutoAdjustOnToolbar = !satisBarCenterHalfW;
        if (!tagItem.isOnToolbar || !tagItem.isAutoAdjustOnToolbar) continue;
        satisBarCenterHalfW = addTagItem(tagItem);
        if (satisBarCenterHalfW) {
            [self.mainMenuTopbar removeItem:tagItem.barItem];
            if (tagItem.isOnToolbar) tagItem.isAutoAdjustOnToolbar = !satisBarCenterHalfW;
            if (tagItem.isSourceOperation) {
                tagItem.isAutoAdjustOnToolbar = YES;
                addTagItem(tagItem);
                BOOL stop = NO;
                while (!stop) {
                    for (NSUInteger i = [self.mainMenuTopbar getItemCountWithPosition:FSItemPositionCenter] - 1; i >= 0 ; i--) {
                        FSItem *barItem = [self.mainMenuTopbar getItemAtPosition:FSItemPositionCenter index:i];
                        if (barItem == self.currentTagItem.barItem || barItem == tagItem.barItem) {
                            if (!(self.currentTagItem && [self.mainMenuTopbar getItemCountWithPosition:FSItemPositionCenter] == 2)) {
                                continue;
                            }
                        }
                        [self.mainMenuTopbar removeItem:barItem];
                        ((FSMainTopbarToolTagItem *)barItem.owner).isOnToolbar = NO;
                        CGFloat barCenterHalfW = ([self.mainMenuTopbar getWidthWithPosition:FSItemPositionCenter layoutIfNeeded:YES] + 16.f + 40)  / 2;
                        if (!((menuViewHalfW - barLeftW -  barCenterHalfW) < minSpace || (menuViewHalfW - barRightW -  barCenterHalfW) < minSpace)) {
                            stop = YES;
                        }
                        break;
                    }
                }
            }
        }
    }
    [self.mainMenuTopbar addItem:self.toolSettingsItem spacing:8.f  atPosition:FSItemPositionCenter];
    self.toolSettingsItem.hidden = !self.tagItems.count;
    _toolSettingsPop.toolTagItems = _tagItems;
}

- (void)setCurrentTagItem:(FSMainTopbarToolTagItem *)currentTagItem{
    if (_currentTagItem == currentTagItem) {
        return;
    }
    
    switch (currentTagItem.toolTag) {
        case FSMainTopbarToolTagHome:
            [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingToolsTabHome];
            break;
        case FSMainTopbarToolTagEdit:
            [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingToolsTabEdit];
            break;
        case FSMainTopbarToolTagComment:
            [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingToolsTabComment];
            break;
        case FSMainTopbarToolTagDrawing:
            [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingToolsTabDrawing];
            break;
        case FSMainTopbarToolTagView:
            [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingToolsTabView];
            break;
        case FSMainTopbarToolTagForm:
            [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingToolsTabForm];
            break;
        case FSMainTopbarToolTagSign:
            [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingToolsTabFillSign];
            break;
        default:
            break;
    }

    _lastTagItem = _currentTagItem;
    FSMainTopbarToolTagItem *lastTagItem = _lastTagItem;

    if (!currentTagItem) {
        self.toolbar.contentBarHeight = TOOL_BAR_HEIGHT_NORMAL;
        if (!_didLayoutSubviews) {
            self.horizontalContentHeight = TOOL_BAR_HEIGHT_NORMAL;
        }
    }else{
        if ([self.subMenuTopbarContentView isDescendantOfView:[self getToolbarContentView]]){
            [self adjustContentBarHeight];
            if (!_didLayoutSubviews) {
                self.horizontalContentHeight = TOOL_BAR_HEIGHT_NORMAL * 2;
            }
        }
    }
    
    [_currentTagItem.subTagToolbar removeFromSuperview];
    _currentTagItem.barItem.selected = NO;
    _currentTagItem.barItem.imageView.hidden = YES;
    _currentTagItem = currentTagItem;
    if (!_currentTagItem.isIPADCompact) {
        _currentTagItem.isOnToolbar = YES;
    }
    
    if (_currentTagItem) {
        if(_currentTagItem.ownerTopbar == nil){
            _currentTagItem.ownerTopbar = self;
            [self.extensionsManager unregisterToolEventListener:_currentTagItem];
            [self.extensionsManager registerToolEventListener:_currentTagItem];
        }
        FSScrollToolbar *subTagToolbar = _currentTagItem.subTagToolbar;
        if (!subTagToolbar) {
            subTagToolbar = [[FSScrollToolbar alloc] initWithStyle:FSToolbarStyleDefault];
            subTagToolbar.verticalScrollBarEdgeInsets = UIEdgeInsetsMake(3, 0, 0, 0);
            subTagToolbar.axis = self.axis;
            if (self.backgroundColor){
                subTagToolbar.backgroundColor = self.backgroundColor;
            }
            _currentTagItem.subTagToolbar = subTagToolbar;
        }
        if (!subTagToolbar.superview){
            [self.subMenuTopbarContentView addSubview:subTagToolbar];
            
            [subTagToolbar mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.left.right.top.bottom.fs_equalToSuperview();
            }];
        }
    }
    
    [self resetCurrentTagItem];
    [self checkCurrentTagItem];
    [self.subMenuTopbarContentView bringSubviewToFront:_currentTagItem.subTagToolbar];
    
    for(FSScrollToolbar *bar in self.subMenuTopbarContentView.subviews) {
        if ([bar isKindOfClass:[FSScrollToolbar class]]) {
            if (bar == _currentTagItem.subTagToolbar) {
                bar.hidden = NO;
            }else{
                bar.hidden = YES;
            }
        }
    }
    
    if (self.delegate && [self.delegate respondsToSelector:@selector(didSetCurrentTagItem:lastTagItem:)]) {
        [self.delegate didSetCurrentTagItem:_currentTagItem lastTagItem:lastTagItem];
    }
    
    for (id<IFSMainTopbarEventListener> listener in self.listeners) {
        if ([listener respondsToSelector:@selector(didSetCurrentTagItem:lastTagItem:)]) {
            [listener didSetCurrentTagItem:_currentTagItem lastTagItem:lastTagItem];
        }
    }
}

- (void)setSubitemContentPosition:(FSTopbarSubitemContentViewPosition)subContentPosition{
    _subitemContentPosition = subContentPosition;
    if (_didLayoutSubviews) {
        [self adjustContentBarHeight];
        [self.extensionsManager.toolbarManager moveToolbarToPosition:subContentPosition];
    }
    _isSubPositionChanged = YES;
}

- (void)checkCurrentTagItem{
    for (FSMainToolbarItem *mainItem in _currentTagItem.mainToolbarItems) {
        if (mainItem.itemType == FSMainToolbarItemTypePan || mainItem.itemType == FSMainToolbarItemTypeDivide) {
            if (_currentTagItem.subTagToolbar.axis == UILayoutConstraintAxisHorizontal) {
                mainItem.item.transform = CGAffineTransformIdentity;
            }else{
                mainItem.item.transform = CGAffineTransformMakeRotation(M_PI_2);
            }
        }
    }
}

- (void)setContentHeight:(CGFloat)contentHeight{
    _contentHeight = contentHeight;
    _horizontalContentHeight = contentHeight;
    _verticalContentWidth = contentHeight;
    [self adjustContentBarHeight];
}


- (void)setHorizontalContentHeight:(CGFloat)horizontalContentHeight{
    _horizontalContentHeight = horizontalContentHeight;
    [self adjustContentBarHeight];
}

- (void)setVerticalContentWidth:(CGFloat)verticalContentWidth{
    _verticalContentWidth = verticalContentWidth;
    [self adjustContentBarHeight];
}

- (void)adjustContentBarHeight{
    if (self.subitemContentPosition == FSTopbarSubitemContentViewPositionTop) {
        _contentHeight = _horizontalContentHeight;
        [self.toolbar setContentBarHeight:_horizontalContentHeight];
    }else{
        _contentHeight = _verticalContentWidth;
        [self.toolbar setContentBarHeight:TOOL_BAR_HEIGHT_NORMAL];
    }
}

- (void)setBackgroundColor:(UIColor *)color{
    _backgroundColor = color;
    for (int i = 0; i < _tagItems.count; i ++ ) {
        FSMainTopbarToolTagItem *tagItem = _tagItems[i];
        tagItem.subTagToolbar.backgroundColor = color;
    }
}

- (void)resetCurrentTagItem{

    [_currentTagItem.subTagToolbar removeItemAtPosition:FSScrollToolbarItemPositionRight];
    [_currentTagItem.subTagToolbar removeItemAtPosition:FSScrollToolbarItemPositionScroll];
    
    [self checkCurrentTagItem];
    for (FSMainToolbarItem *mainItem in _currentTagItem.mainToolbarItems) {
        if ([mainItem.customView isDescendantOfView:_currentTagItem.subTagToolbar]) {
            continue;
        }
        [_currentTagItem.subTagToolbar addItem:mainItem.customView spacing:mainItem.spacing atPosition:FSScrollToolbarItemPositionRight];
    }
    
    for (FSReadToolbarItem *readItem in _currentTagItem.readToolbarItems) {
        if (readItem.readToolbarItemType == FSReadToolbarItemTypeExist) {
            if (!readItem.extensionsManager) {
                readItem.extensionsManager = self.extensionsManager;
            }
            
            if (readItem.toolbarItem) {
                if ([readItem.toolbarItem.customView isDescendantOfView:_currentTagItem.subTagToolbar]) {
                    continue;
                }
                if (readItem.itemContentType & FSItemContentTypeText){
                    if (self.axis == UILayoutConstraintAxisHorizontal && self.extensionsManager.toolbarManager.isShowReadBarItemDescript){
                        [readItem.toolbarItem.item setTitle:readItem.title forState:UIControlStateNormal];
                        [readItem.toolbarItem.item setTitle:readItem.selectTitle forState:UIControlStateSelected];
                    }else{
                        [readItem.toolbarItem.item setTitle:nil forState:UIControlStateNormal];
                        [readItem.toolbarItem.item setTitle:nil forState:UIControlStateSelected];
                    }
                }
                if (_currentTagItem.toolTag == FSMainTopbarToolTagHome) {
                    if (readItem.settingsType == FSReadToolSettingsTypeStamp) {
                        [readItem.toolbarItem.item setImage:[Utility getStampImageWithIcon:readItem.toolSettings.stampIcon] forState:UIControlStateNormal];
                    }
                }
                if (readItem.settingsType == FSReadToolSettingsTypeImageAnnotation) {
                    if([self.extensionsManager.permissionProvider checkPermission:FSFunctionImageAnnoteItem] == FSPermissionStateHide) continue;
                }
                [_currentTagItem.subTagToolbar addItem:readItem.toolbarItem.customView spacing:readItem.toolbarItem.spacing atPosition:FSScrollToolbarItemPositionScroll];
            }
        }else{
            if ([readItem.toolbarItem.customView isDescendantOfView:_currentTagItem.subTagToolbar] || !readItem.toolbarItem.customView) {
                continue;
            }
            [_currentTagItem.subTagToolbar addItem:readItem.toolbarItem.customView spacing:readItem.toolbarItem.spacing atPosition:FSScrollToolbarItemPositionScroll];
        }
    }
}

- (void)setStateBarColor:(nullable UIColor *)color{
    [BindThemeKeySel(self.mainMenuTopbar, backgroundView) fs_updateThemeCompleted:^(UIImageView*  _Nonnull target, SEL  _Nonnull sel) {
        target.backgroundColor = color;
    }];
    [BindThemeKeySel(self.toolbar, stateBar) fs_updateThemeCompleted:^(UIView *  _Nonnull target, SEL  _Nonnull sel) {
        target.backgroundColor = color;
    }];
}

- (void)setTagItemBackgroudColor:(nullable UIColor *)color{
    self.tagItemBgColor = color;
    for (int i = 0; i < _tagItems.count; i ++ ) {
        FSMainTopbarToolTagItem *tagItem = _tagItems[i];
        FSItem *barItem = tagItem.barItem;
        @fs_weakify(self)
        [BindThemeKeySel(barItem, setTitleColor:forState:) fs_updateThemeCompleted:^(FSItem * _Nonnull target, SEL  _Nonnull sel) {
            @fs_strongify(self)
            [target setTitleColor:target.fs_textColor_T10 forState:UIControlStateSelected];
            [target setImage:[target.currentImage fs_imageWithTintColor:target.fs_textColor_T10] forState:UIControlStateSelected];
            [target setBackgroundImage:[UIImage fs_imageWithColor:self.tagItemBgColor] forState:UIControlStateSelected];
            [target setBackgroundImage:nil forState:UIControlStateNormal];
        }];
    }
}

- (void)registerEventListener:(id<IFSMainTopbarEventListener>)listener{
    if (![self.listeners containsObject:listener]) [self.listeners addObject:listener];
}

- (void)unregisterEventListener:(id<IFSMainTopbarEventListener>)listener{
    [self.listeners removeObject:listener];
}

- (void)resetHomeTagItem{
    
    FSMainTopbarToolTagItem *homeTagItem = nil;
    for (FSMainTopbarToolTagItem *tagItem in self.tagItems) {
        if (tagItem.toolTag == FSMainTopbarToolTagHome) {
            homeTagItem = tagItem;
            break;
        }
    }
    if (!homeTagItem) {
        return;
    }
    [self resetTagItemWithItem:homeTagItem];
}

- (void)resetViewTagItem{
    
    FSMainTopbarToolTagItem *viewTagItem = nil;
    for (FSMainTopbarToolTagItem *tagItem in self.tagItems) {
        if (tagItem.toolTag == FSMainTopbarToolTagView) {
            viewTagItem = tagItem;
            break;
        }
    }
    if (!viewTagItem) {
        return;
    }
    [self resetTagItemWithItem:viewTagItem];
}

- (void)switchToTag:(FSMainTopbarToolTag)tag{
    FSMainTopbarToolTagItem *currentItem = self.currentTagItem;
    if (currentItem.toolTag != tag) {
        for (FSMainTopbarToolTagItem *tagItem in self.tagItems) {
            if (tagItem.toolTag == tag) {
                [self setCurrentTagItem:tagItem];
                [self resetCurrentTagItem];
                if (DEVICE_iPHONE || [self.extensionsManager fs_currentSceneiPadHorizontalCompact]) {
                    [self satisfyShortViewDisplay];
                }else{
                    [self rearrangeTagItemsSatisfyViewDisplay];
                }
                break;
            }
        }
    }
}

- (FSMainTopbarToolTagItem*)getTagItemWithToolTag:(FSMainTopbarToolTag)toolTag{
    FSMainTopbarToolTagItem *retTagItem = nil;
    for (FSMainTopbarToolTagItem *tagItem in self.tagItems) {
        if (tagItem.toolTag == toolTag) {
            retTagItem = tagItem;
            break;
        }
    }
    return retTagItem;
}

- (void)checkTagItemWithItem:(FSMainTopbarToolTagItem *)tagItem{
    for (FSMainToolbarItem *mainItem in tagItem.mainToolbarItems) {
        if (mainItem.itemType == FSMainToolbarItemTypePan || mainItem.itemType == FSMainToolbarItemTypeDivide) {
            if (tagItem.subTagToolbar.axis == UILayoutConstraintAxisHorizontal) {
                mainItem.item.transform = CGAffineTransformIdentity;
            }else{
                mainItem.item.transform = CGAffineTransformMakeRotation(M_PI_2);
            }
        }
    }
}

- (void)resetTagItemWithItem:(FSMainTopbarToolTagItem *)tagItem{
    [tagItem.subTagToolbar removeItemAtPosition:FSScrollToolbarItemPositionRight];
    [tagItem.subTagToolbar removeItemAtPosition:FSScrollToolbarItemPositionScroll];
    
    [self checkTagItemWithItem:tagItem];
    for (FSMainToolbarItem *mainItem in tagItem.mainToolbarItems) {
        if ([mainItem.customView isDescendantOfView:tagItem.subTagToolbar]) {
            continue;
        }
        [tagItem.subTagToolbar addItem:mainItem.customView spacing:mainItem.spacing atPosition:FSScrollToolbarItemPositionRight];
    }
    
    for (FSReadToolbarItem *readItem in tagItem.readToolbarItems) {
        if (readItem.readToolbarItemType == FSReadToolbarItemTypeExist) {
            if (!readItem.extensionsManager) {
                readItem.extensionsManager = self.extensionsManager;
            }
            
            if (readItem.toolbarItem) {
                if ([readItem.toolbarItem.customView isDescendantOfView:tagItem.subTagToolbar]) {
                    continue;
                }
                if (tagItem.toolTag == FSMainTopbarToolTagHome) {
                    if (readItem.settingsType == FSReadToolSettingsTypeStamp) {
                        [readItem.toolbarItem.item setImage:[Utility getStampImageWithIcon:readItem.toolSettings.stampIcon] forState:UIControlStateNormal];
                    }
                }
                [tagItem.subTagToolbar addItem:readItem.toolbarItem.customView spacing:readItem.toolbarItem.spacing atPosition:FSScrollToolbarItemPositionScroll];
            }
        }else{
            if ([readItem.toolbarItem.customView isDescendantOfView:tagItem.subTagToolbar] || !readItem.toolbarItem.customView) {
                continue;
            }
            [tagItem.subTagToolbar addItem:readItem.toolbarItem.customView spacing:readItem.toolbarItem.spacing atPosition:FSScrollToolbarItemPositionScroll];
        }
    }
}

- (void)setAxis:(UILayoutConstraintAxis)axis{
    _axis = axis;
    for (FSMainTopbarToolTagItem *tagItem in self.tagItems) {
        if (tagItem.subTagToolbar) {
            tagItem.subTagToolbar.axis = axis;
        }
    }
}

- (void)satisfyShortViewDisplay{
    FSItem *currentBarItem = self.currentTagItem.barItem;
    currentBarItem.selected = YES;
    currentBarItem.imageView.hidden = NO;
    [currentBarItem rebuildImageTextRelation:FSImageTextRelationLeft];
    currentBarItem.font = self.fs_standardGeneralFont_GF2;
    [currentBarItem setImage:FSImageNamedAndColor(@"read_shangxia_purple",self.fs_textColor_T10) forState:UIControlStateSelected];

    [self.mainMenuTopbar removeItemAtPosition:FSItemPositionCenter];
    if (!currentBarItem) return;
    @weakify(self)
    currentBarItem.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        NSMutableArray *tmp = @[].mutableCopy;
        for (FSMainTopbarToolTagItem *tagItem in self.tagItems) {
//            if (!tagItem.moduleLoad) continue;
            FSToolsPopoverItem *popItem = [[FSToolsPopoverItem alloc] initWithTitle:tagItem.title image:tagItem.image];
            if ([self.currentNewTagItems containsObject:tagItem]) popItem.isNewItem = YES;
            [tmp addObject:popItem];
        }
        
        [self.extensionsManager cancelAutoFullScreen];
        FSToolsPopoverViewController *toolsPop = (FSToolsPopoverViewController *)[self.extensionsManager.viewCtrlManager popPopoverCtrlClass:[FSToolsPopoverViewController class] groups:@[[[FSPopoverItemGroup alloc] initWithTitle:FSLocalizedForKey(@"kTools") items:tmp.copy]] sourceView:!DEVICE_iPHONE ? ^UIView * _Nonnull(UIPopoverPresentationController * _Nonnull popover) {
            popover.permittedArrowDirections = UIPopoverArrowDirectionUp;
            return _item;
        } : nil autoAdapt:NO];
        self.toolsPop = toolsPop;
        
        if (self.delegate && [self.delegate respondsToSelector:@selector(didShowTagTabPopuper)]) {
            [self.delegate didShowTagTabPopuper];
        }
        
        for (id<IFSMainTopbarEventListener> listener in self.listeners) {
            if ([listener respondsToSelector:@selector(didShowTagTabPopuper)]) {
                [listener didShowTagTabPopuper];
            }
        }
        
        toolsPop.dismissCompletion = ^{

            [self.extensionsManager delayAutoFullScreen];
            self.toolsPop = nil;
            if (self.extensionsManager.shouldShowMenu) {
                [self.extensionsManager showMenu];
            }

            if (self.delegate && [self.delegate respondsToSelector:@selector(didDismissTagTabPopuper)]) {
                [self.delegate didDismissTagTabPopuper];
            }
            
            for (id<IFSMainTopbarEventListener> listener in self.listeners) {
                if ([listener respondsToSelector:@selector(didDismissTagTabPopuper)]) {
                    [listener didDismissTagTabPopuper];
                }
            }
        };
        [toolsPop setAction:^(FSPopoverItem * _Nonnull popoverItem, int index) {
            FSMainTopbarToolTagItem *tagItem = [self.tagItems objectAtIndex:index];
            if (self.currentTagItem != tagItem) {
                if (tagItem.toolTag == FSMainTopbarToolTagEdit || tagItem.toolTag == FSMainTopbarToolTagForm) {
                    [self.extensionsManager.permissionProvider checkPermission:FSFunctionEdit checkPermissionState:^(FSPermissionState state) {
                        if (state == FSPermissionStateShow) {
                            self.currentTagItem.barItem.imageView.hidden = YES;
                            self.currentTagItem.barItem.selected = NO;
                            [self.mainMenuTopbar removeItem:_item];

                            self.currentTagItem = tagItem;

                            self.currentTagItem.barItem.font = self.fs_standardGeneralFont_GF2;
                            [self.currentTagItem.barItem  setImage:FSImageNamedAndColor(@"read_shangxia_purple",self.fs_textColor_T10) forState:UIControlStateSelected];
                            [self.currentTagItem.barItem rebuildImageTextRelation:FSImageTextRelationLeft];
                            self.currentTagItem.barItem.onTapClick = _item.onTapClick;
                            self.currentTagItem.barItem.selected = YES;
                            self.currentTagItem.barItem.imageView.hidden = NO;
                            [self.mainMenuTopbar addItem:self.currentTagItem.barItem atPosition:FSItemPositionCenter];
                        }
                    }];
                }else{
                    self.currentTagItem.barItem.imageView.hidden = YES;
                    self.currentTagItem.barItem.selected = NO;
                    [self.mainMenuTopbar removeItem:_item];

                    self.currentTagItem = tagItem;

                    self.currentTagItem.barItem.font = self.fs_standardGeneralFont_GF2;
                    [self.currentTagItem.barItem  setImage:FSImageNamedAndColor(@"read_shangxia_purple",self.fs_textColor_T10) forState:UIControlStateSelected];
                    [self.currentTagItem.barItem rebuildImageTextRelation:FSImageTextRelationLeft];
                    self.currentTagItem.barItem.onTapClick = _item.onTapClick;
                    self.currentTagItem.barItem.selected = YES;
                    self.currentTagItem.barItem.imageView.hidden = NO;
                    [self.mainMenuTopbar addItem:self.currentTagItem.barItem atPosition:FSItemPositionCenter];
                }
            }
        }];
        toolsPop.selectedIndex = [self.tagItems indexOfObject:self.currentTagItem];
    };
    [self.mainMenuTopbar addItem:currentBarItem atPosition:FSItemPositionCenter];
}


- (void)setHidden:(BOOL)hidden animated:(BOOL)animated{
    [super setHidden:hidden animated:animated];

    if (self.toolbar.style == FSToolbarStyleTop) {
        self.subMenuTopbarContentView.hidden = hidden;
        if (![self.subMenuTopbarContentView isDescendantOfView:[self getToolbarContentView]]) {
            if (self.subitemContentPosition == FSTopbarSubitemContentViewPositionLeft) {
                [Utility addAnimation:self.subMenuTopbarContentView.layer type:self.subMenuTopbarContentView.hidden ? kCATransitionReveal : kCATransitionMoveIn subType:self.subMenuTopbarContentView.hidden ? kCATransitionFromRight : kCATransitionFromLeft  timeFunction:kCAMediaTimingFunctionEaseInEaseOut duration:0.3];
            }
            else if (self.subitemContentPosition == FSTopbarSubitemContentViewPositionRight) {
                [Utility addAnimation:self.subMenuTopbarContentView.layer type:self.subMenuTopbarContentView.hidden ? kCATransitionReveal : kCATransitionMoveIn subType:self.subMenuTopbarContentView.hidden ? kCATransitionFromLeft : kCATransitionFromRight  timeFunction:kCAMediaTimingFunctionEaseInEaseOut duration:0.3];
            }
        }
    }
}

- (FSMainToolbarItem *)getCurrentSelectedItem{
    if (self.currentTagItem.currentReadToolbarItem.toolbarItem) {
        return self.currentTagItem.currentReadToolbarItem.toolbarItem;
    }
    for (FSMainToolbarItem *item in self.currentTagItem.mainToolbarItems) {
        if (item.itemType == FSMainToolbarItemTypeMultiSelect) {
            return item;
        }
    }
    return nil;
}

- (void)updateTagItemsUI{
    if (DEVICE_iPHONE || [self.extensionsManager fs_currentSceneiPadHorizontalCompact]) {

        for (FSMainTopbarToolTagItem *tagItem in _tagItems) {
            tagItem.barItem.titleEdgeInsets = tagItem.phoneTitleEdgeInsets;
            tagItem.barItem.imageEdgeInsets = tagItem.phoneImageEdgeInsets;
        }
        [self satisfyShortViewDisplay];
    }else{
        for (FSMainTopbarToolTagItem *tagItem in _tagItems) {
            tagItem.barItem.titleEdgeInsets = tagItem.padTitleEdgeInsets;
            tagItem.barItem.imageEdgeInsets = tagItem.padImageEdgeInsets;
        }
        [self rearrangeTagItemsSatisfyViewDisplay];
    }
}

#pragma mark - UIViewAdapter
- (void)traitCollectionDidChange:(UITraitCollection *)previousTraitCollection fromView:(__kindof UIView *)view{
    if (view == self.toolbar) {
        [super traitCollectionDidChange:previousTraitCollection fromView:view];
    }
}

- (void)layoutSubviewsFromView:(__kindof UIView *)view{
    if (view == self.toolbar) {
        [super layoutSubviewsFromView:view];
    }else if (view == self.mainMenuTopbar){
        if (!_didLayoutSubviews && !CGRectEqualToRect(view.bounds, CGRectZero)) {
            _didLayoutSubviews = YES;
            self.subitemContentPosition = _subitemContentPosition;
        }
     
    }else{
     
        if (!CGSizeEqualToSize(_mainMenuTopbarSize, view.bounds.size)) {
            [self updateTagItemsUI];
            
#if _MAC_CATALYST_
            if (self.toolSettingsPop) {
                [self.toolSettingsPop dismissViewControllerAnimated:NO completion:self.toolSettingsPop.dismissCompletion];
            }
#endif
            _mainMenuTopbarSize = view.bounds.size;
        }
    }
    if (_isSubPositionChanged) {
        _isSubPositionChanged = NO;
        if (self.delegate && [self.delegate respondsToSelector:@selector(onSubitemViewChanged:)]) {
            [self.delegate onSubitemViewChanged:_subitemContentPosition];
        }
        
        for (id<IFSMainTopbarEventListener> listener in self.listeners) {
            if ([listener respondsToSelector:@selector(onSubitemViewChanged:)]) {
                [listener onSubitemViewChanged:_subitemContentPosition];
            }
        }
    }
}

#pragma mark - IWindowFrameChangedListener

-(void)onWindowFrameChanged:(CGRect)newFrame horizontalSizeClassChanged:(BOOL)changed{
    if (DEVICE_iPAD && changed) {
        if (self.toolsPop) {
            [self.toolsPop dismissViewControllerAnimated:NO completion:^{
                self.toolsPop = nil;
                FSItem *currentBarItem = self.currentTagItem.barItem;
                !currentBarItem.onTapClick ? : currentBarItem.onTapClick(currentBarItem);
            }];
        }
        if (self.toolSettingsPop) {
            [self.toolSettingsPop dismissViewControllerAnimated:NO completion:self.toolSettingsPop.dismissCompletion];
        }
    }
}

@end

@implementation FSMainBottombar
@synthesize average = _average;
@dynamic itemsForPositionLeft;
@dynamic itemsForPositionRight;

- (instancetype)initWithStyle:(FSToolbarStyle)style extensionsManager:(UIExtensionsManager *)extensionsManager{
    self = [super initWithStyle:FSToolbarStyleBottom extensionsManager:extensionsManager];
    if (self) {
        self.toolbar.adapter = self;
        _average = YES;
    }
    return self;
}

- (void)setItemsForPositionCenter:(NSArray<FSMainToolbarItem *> *)itemsForPositionCenter{
    _itemsForPositionCenter = itemsForPositionCenter.mutableCopy;
    [self setNeedLayout];
}

- (BOOL)isAverage{
    return _average;
}

- (void)setAverage:(BOOL)average{
    _average = average;
    [self itemsDisplay];
}

- (void)setNeedLayout{
    [self itemsDisplay];
}

- (void)itemsDisplay{
    [self.toolbar removeItemAtPosition:FSItemPositionCenter];
    if (_average) {
        CGFloat spacing =  (IPAD_PREFERENCE_BAR_WIDTH - 50 * 2 - 40.f * self.itemsForPositionCenter.count)/( self.itemsForPositionCenter.count - 1);
        for (FSMainToolbarItem *mainItem in  self.itemsForPositionCenter) {
            [self.toolbar addItem:mainItem.customView spacing:spacing atPosition:FSItemPositionCenter];
        }
    }else{
        for (FSMainToolbarItem *mainItem in  self.itemsForPositionCenter) {
            [self.toolbar addItem:mainItem.customView spacing:mainItem.spacing atPosition:FSItemPositionCenter];
        }
    }
}

#pragma mark - UIViewAdapter

- (void)layoutSubviewsFromView:(__kindof UIView *)view{
    [super layoutSubviewsFromView:view];
    if (view == self.toolbar) {
        [self itemsDisplay];
    }
}

@end
