/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSReadToolbarItem+Private.h"
#import "AudioVideoModule.h"
#import "FillSignModule.h"
#import "FSPermissionUtilty.h"
#import <objc/runtime.h>

@implementation FSReadToolSettings

-(void)encodeWithCoder:(NSCoder *)aCoder{
    [super encodeWithCoder:aCoder];
    unsigned int count = 0;
    objc_property_t *propertes = class_copyPropertyList([FSReadToolSettings class], &count);
    for (int i=0; i<count; i++) {
        const char *propertyName = property_getName(propertes[i]);
        NSString *name = [NSString stringWithUTF8String:propertyName];
        id value = [self valueForKey:name];
        if (value) [aCoder encodeObject:value forKey:name];
    }
    free(propertes);
}
 
-(id)initWithCoder:(NSCoder *)aDecoder{
    if (self = [super initWithCoder:aDecoder]) {
        unsigned int count =0;
        objc_property_t *propertes = class_copyPropertyList([FSReadToolSettings class], &count);
        for (int i = 0; i < count; i++) {
            const char *propertyName = property_getName(propertes[i]);
            NSString *name = [NSString stringWithUTF8String:propertyName];
            if ([name isEqualToString:@"settingsColor"]) continue;
            id value = [aDecoder decodeObjectForKey:name];
            if (value) [self setValue:value forKey:name];
        }
        free(propertes);
    }
    return self;
}

- (id)copyWithZone:(NSZone *)zone{
    FSReadToolSettings *settingItem = [[FSReadToolSettings allocWithZone:zone] initWithToolSettingType:self.settingsType];
    [settingItem copySettingFromConfigSetting:self];
    return settingItem;
}

- (instancetype)initWithToolSettingType:(FSReadToolSettingsType)type{
    return [self initWithToolSettingType:type extensionsManager:nil];
}

- (instancetype)initWithToolSettingType:(FSReadToolSettingsType)type extensionsManager:(UIExtensionsManager *)extensionsManager{
    self = [super init];
    if (self) {
        _noteIcon = -1;
        _settingsType = type;
        switch (type) {
            case FSReadToolSettingsTypeText:
                
                break;
            case FSReadToolSettingsTypeImage:
                
                break;
            case FSReadToolSettingsTypeAudio:
                [self copySettingFromConfigSetting:[SettingObj new]];
                break;
            case FSReadToolSettingsTypeVideo:
                [self copySettingFromConfigSetting:[SettingObj new]];
                break;
            case FSReadToolSettingsTypeLink:
                [self copySettingFromConfigSetting:[SettingObj new]];
                break;
            case FSReadToolSettingsTypeNote:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.note];
                break;
                
            case FSReadToolSettingsTypeAttachments:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.attachment];
                break;
            case FSReadToolSettingsTypeStamp:
                break;
            case FSReadToolSettingsTypeTypewriter:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.typewriter];
                break;
            case FSReadToolSettingsTypeTextbox:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.textbox];
                break;
            case FSReadToolSettingsTypeCallout:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.callout];
                break;
            case FSReadToolSettingsTypeHighlight:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.highlight];
                break;
            case FSReadToolSettingsTypeUnderline:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.underline];
                break;
            case FSReadToolSettingsTypeSquiggly:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.squiggly];
                break;
            case FSReadToolSettingsTypeStrikeout:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.strikeout];
                break;
            case FSReadToolSettingsTypeReplaceText:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.replace];
                break;
            case FSReadToolSettingsTypeInsertText:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.insert];
                break;
            case FSReadToolSettingsTypeAreaHighlight:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.areaHighlight];
                break;
            case FSReadToolSettingsTypeMeasure:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.measure];
                break;
            case FSReadToolSettingsTypeImageAnnotation:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.image];
                break;
            case FSReadToolSettingsTypePencil:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.pencil];
                break;
            case FSReadToolSettingsTypeHighlighter:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.highlighter];
                break;
            case FSReadToolSettingsTypeEraser:
                [self copySettingFromConfigSetting:[SettingObj new]];
                break;
            case FSReadToolSettingsTypeRectangle:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.rectangle];
                break;
            case FSReadToolSettingsTypeOval:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.oval];
                break;
            case FSReadToolSettingsTypeLine:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.line];
                break;
            case FSReadToolSettingsTypeArrow:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.arrow];
                break;
            case FSReadToolSettingsTypePolygon:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.polygon];
                break;
            case FSReadToolSettingsTypePolyLine:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.polyline];
                break;
            case FSReadToolSettingsTypeCloud:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.cloud];
                break;
            case FSReadToolSettingsTypeTextField:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.form.textField];
                break;
            case FSReadToolSettingsTypeCheckBox:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.form.checkBox];
                break;
            case FSReadToolSettingsTypeRadioButton:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.form.radioButton];
                break;
            case FSReadToolSettingsTypeComboBox:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.form.comboBox];
                break;
            case FSReadToolSettingsTypeListBox:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.form.listBox];
                break;
            case FSReadToolSettingsTypeSignatureField:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.signature];
                break;
            case FSReadToolSettingsTypeImageField:
                [self copySettingFromConfigSetting:[SettingObj new]];
                break;
            case FSReadToolSettingsTypeRedact:
                [self copySettingFromConfigSetting:extensionsManager.config.defaultSettings.annotations.redaction];
                break;
            default:
                break;
        }
        
    }
    return self;
}

- (void)copySettingFromConfigSetting:(SettingObj *)settingObj{
    self.icon = [settingObj.icon copy];
    self.textFace = [settingObj.textFace copy];
    self.scaleFromUnit = [settingObj.scaleFromUnit copy];
    self.scaleToUnit = [settingObj.scaleToUnit copy];

    self.textSize = settingObj.textSize;
    self.opacity = settingObj.opacity == -1 ? 1.f : settingObj.opacity;
    self.thickness = settingObj.thickness;
    self.rotation = settingObj.rotation;
    self.scaleFromValue = settingObj.scaleFromValue;
    self.scaleToValue = settingObj.scaleToValue;
    self.multipleSelection = settingObj.multipleSelection;
    self.customText = settingObj.customText;

    self.color = [settingObj.color fs_colorFromCopy];
    self.fillColor = [settingObj.fillColor fs_colorFromCopy];
    self.textColor = [settingObj.textColor fs_colorFromCopy];
    
    if (_settingsType == FSReadToolSettingsTypeNote) {
        if ([settingObj isKindOfClass:[FSReadToolSettings class]]) {
            self.noteIcon = ((FSReadToolSettings *)settingObj).noteIcon;
        }else{
            self.noteIcon = [Utility getIconTypeWithIconName:self.icon annotType:FSAnnotNote];
        }
    }
    
    if (_settingsType == FSReadToolSettingsTypeEraser) {
        if ([settingObj isKindOfClass:[FSReadToolSettings class]]) {
            self.eraserLineWidth = ((FSReadToolSettings *)settingObj).eraserLineWidth;
            self.eraseStyle = ((FSReadToolSettings *)settingObj).eraseStyle;
        }else{
            self.eraseStyle = 0;
            self.eraserLineWidth = 18.f;
        }
    }
    
    if (_settingsType == FSReadToolSettingsTypeAttachments) {
        if ([settingObj isKindOfClass:[FSReadToolSettings class]]) {
            self.attachmentIcon = ((FSReadToolSettings *)settingObj).attachmentIcon;
        }else{
            self.attachmentIcon = [Utility getIconTypeWithIconName:self.icon annotType:FSAnnotFileAttachment];
        }
    }
    
    if (_settingsType == FSReadToolSettingsTypeMeasure) {
        if ([settingObj isKindOfClass:[FSReadToolSettings class]]) {
            self.distanceUnit = ((FSReadToolSettings *)settingObj).distanceUnit;
        }else{
            self.distanceUnit = ({
                [NSString stringWithFormat:@"%d %@ = %d %@",self.scaleFromValue,self.scaleFromUnit,self.scaleToValue,self.scaleToUnit];
            });
        }
    }
    
    if (_settingsType == FSReadToolSettingsTypeStamp) {
        if ([settingObj isKindOfClass:[FSReadToolSettings class]]) {
            self.stampIcon = ((FSReadToolSettings *)settingObj).stampIcon;
        }
    }
    
    if (_settingsType == FSReadToolSettingsTypeHighlighter) {
        if ([settingObj isKindOfClass:[FSReadToolSettings class]]) {
            self.thickness = ((FSReadToolSettings *)settingObj).thickness;
            self.color = ((FSReadToolSettings *)settingObj).color;
        }
    }
    
    if (_settingsType == FSReadToolSettingsTypeImageAnnotation) {
        if ([settingObj isKindOfClass:[FSReadToolSettings class]]) {
            self.screenAnnotRotation = ((FSReadToolSettings *)settingObj).screenAnnotRotation;
        }else{
            self.screenAnnotRotation = [Utility rotationForValue:@(self.rotation)];
        }
    }
    
    if (_settingsType == FSReadToolSettingsTypePencil) {
        if ([settingObj isKindOfClass:[FSReadToolSettings class]]) {
            self.pencilStyle = ((FSReadToolSettings *)settingObj).pencilStyle;
        }else{
            self.pencilStyle = 0;
        }
    }
}

- (void)setPropertyColor:(UIColor * _Nullable)propertyColor{
    switch (_settingsType) {
        case FSReadToolSettingsTypeText:
            break;
        case FSReadToolSettingsTypeImage:
            break;
        case FSReadToolSettingsTypeAudio:
            break;
        case FSReadToolSettingsTypeVideo:
            break;
        case FSReadToolSettingsTypeLink:
            break;
            
        case FSReadToolSettingsTypeStamp:
            break;
            
        case FSReadToolSettingsTypeTypewriter:
        case FSReadToolSettingsTypeTextbox:
        case FSReadToolSettingsTypeCallout:
            self.textColor = propertyColor;
            break;
            
        case FSReadToolSettingsTypeImageAnnotation:
            break;
            
            
        case FSReadToolSettingsTypeTextField:
            break;
        case FSReadToolSettingsTypeCheckBox:
            break;
        case FSReadToolSettingsTypeRadioButton:
            break;
        case FSReadToolSettingsTypeComboBox:
            break;
        case FSReadToolSettingsTypeListBox:
            break;
        case FSReadToolSettingsTypeSignatureField:
            break;
        case FSReadToolSettingsTypeImageField:
            break;
        case FSReadToolSettingsTypeRedact:
            self.textColor = propertyColor;
            break;
        default:
            self.color = propertyColor;
            break;
    }
}

- (UIColor *)propertyColor{
    switch (_settingsType) {
        case FSReadToolSettingsTypeText:
            break;
        case FSReadToolSettingsTypeImage:
            break;
        case FSReadToolSettingsTypeAudio:
            break;
        case FSReadToolSettingsTypeVideo:
            break;
        case FSReadToolSettingsTypeLink:
            break;
            
        case FSReadToolSettingsTypeStamp:
            break;
            
        case FSReadToolSettingsTypeTypewriter:
            return self.textColor;
        case FSReadToolSettingsTypeTextbox:
            return self.textColor;
        case FSReadToolSettingsTypeCallout:
            return self.textColor;
            
        case FSReadToolSettingsTypeImageAnnotation:
            return self.fs_primaryColor;
            
        case FSReadToolSettingsTypeEraser:
            return self.fs_dividerColor_P3;
            
        case FSReadToolSettingsTypeTextField:
            break;
        case FSReadToolSettingsTypeCheckBox:
            break;
        case FSReadToolSettingsTypeRadioButton:
            break;
        case FSReadToolSettingsTypeComboBox:
            break;
        case FSReadToolSettingsTypeListBox:
            break;
        case FSReadToolSettingsTypeSignatureField:
            break;
        case FSReadToolSettingsTypeImageField:
            break;
        case FSReadToolSettingsTypeRedact:
            return self.fillColor;
        default:
            return self.color;
            
    }
    return nil;
}

- (unsigned int)settingsColor{
    switch (_settingsType) {
        case FSReadToolSettingsTypeText:
            break;
        case FSReadToolSettingsTypeImage:
            break;
        case FSReadToolSettingsTypeAudio:
            break;
        case FSReadToolSettingsTypeVideo:
            break;
        case FSReadToolSettingsTypeLink:
            break;
            
        case FSReadToolSettingsTypeStamp:
            break;
            
        case FSReadToolSettingsTypeTypewriter:
            return self.textColor.rgbValue;
        case FSReadToolSettingsTypeTextbox:
            return self.textColor.rgbValue;
        case FSReadToolSettingsTypeCallout:
            return self.textColor.rgbValue;
            
        case FSReadToolSettingsTypeImageAnnotation:
            break;
            
        case FSReadToolSettingsTypeEraser:
            return self.fs_dividerColor_P3.rgbValue;
            
        case FSReadToolSettingsTypeTextField:
            return self.textColor.rgbValue;
        case FSReadToolSettingsTypeCheckBox:
            return self.textColor.rgbValue;
        case FSReadToolSettingsTypeRadioButton:
            return self.textColor.rgbValue;
        case FSReadToolSettingsTypeComboBox:
            return self.textColor.rgbValue;
        case FSReadToolSettingsTypeListBox:
            return self.textColor.rgbValue;
        case FSReadToolSettingsTypeSignatureField:
            break;
        case FSReadToolSettingsTypeImageField:
            break;
        case FSReadToolSettingsTypeRedact:
            return self.fillColor.rgbValue;
        default:
            return self.color.rgbValue;
            
    }
    return 0;
}

@end

@implementation FSReadToolbarItem

- (instancetype)initWithCustomItem:(FSMainToolbarItem *)customItem{
    self = [super init];
    if (self) {
        _customItem = customItem;
        if (customItem.itemType >= FSMainToolbarItemTypePan || customItem.itemType <= FSMainToolbarItemTypeDivide) {
            _readToolbarItemType = FSReadToolbarItemTypeExist;
        }else{
            _readToolbarItemType = FSReadToolbarItemTypeCustom;
        }
    }
    return self;
}

- (instancetype)initWithToolSettingType:(FSReadToolSettingsType)settingType extensionsManager:(UIExtensionsManager *)extensionsManager{
    return [self initWithToolSettingType:settingType extensionsManager:extensionsManager addTool:NO];
}

- (instancetype)initWithToolSettingType:(FSReadToolSettingsType)settingType extensionsManager:(UIExtensionsManager *)extensionsManager addTool:(BOOL)addTool{
    NSAssert(extensionsManager, @"extensionsManager is not nil");
    self = [super init];
    if (self) {
        FSReadToolSettings *settingItem = [[FSReadToolSettings alloc] initWithToolSettingType:settingType extensionsManager:extensionsManager];
        if (addTool) {
            if (!extensionsManager.addToolsSettings) extensionsManager.addToolsSettings = @{}.mutableCopy;
            FSReadToolSettings *tmp =  [extensionsManager.addToolsSettings objectForKey:@(settingType)];
            if (tmp) {
                settingItem = tmp;
            }else{
                extensionsManager.addToolsSettings[@(settingType)] = settingItem;
            }
        }
        _extensionsManager = extensionsManager;
        _toolSettings = settingItem;
        _readToolbarItemType = FSReadToolbarItemTypeExist;
        _settingsType = settingType;
        [self initialize];
    }
    return self;
}

- (instancetype)initWithToolSettingType:(FSReadToolSettingsType)settingType{
    return [self initWithToolSettingType:settingType withItemContentType:FSItemContentTypeImage];
}

- (instancetype)initWithToolSettingType:(FSReadToolSettingsType)settingType withItemContentType:(FSItemContentType)itemContentType{
    return [self initWithToolSettingType:settingType toolSettings:nil withItemContentType:itemContentType];
}

- (instancetype)initWithToolSettings:(FSReadToolSettings *)toolSettings{
    return [self initWithToolSettings:toolSettings  withItemContentType:FSItemContentTypeImage];
}

- (instancetype)initWithToolSettings:(FSReadToolSettings *)toolSettings withItemContentType:(FSItemContentType)itemContentType
{
    return [self initWithToolSettingType:toolSettings ? toolSettings.settingsType : 0 toolSettings:toolSettings withItemContentType:itemContentType];
}

- (instancetype)initWithToolSettingType:(FSReadToolSettingsType)settingType toolSettings:(FSReadToolSettings *)toolSettings withItemContentType:(FSItemContentType)itemContentType{
    self = [super init];
    if (self) {
        _itemContentType = itemContentType;
        _toolSettings = toolSettings;
        _readToolbarItemType = FSReadToolbarItemTypeExist;
        _settingsType = settingType;
        [self initialize];
    }
    return self;
}

- (void)initialize{
    [self setTitleAndImage];
    _image = FSImageNamedAndNormalColor(_imageName);
    _enabled = YES;
    
    if (self.settingsType == FSReadToolSettingsTypeTextField ||
        self.settingsType ==  FSReadToolSettingsTypeCheckBox ||
        self.settingsType ==  FSReadToolSettingsTypeRadioButton ||
        self.settingsType ==  FSReadToolSettingsTypeComboBox ||
        self.settingsType ==  FSReadToolSettingsTypeListBox ||
        self.settingsType ==  FSReadToolSettingsTypeSignatureField ||
        self.settingsType == FSReadToolSettingsTypeImageField ||
        self.settingsType ==  FSReadToolSettingsTypeAudio ||
        self.settingsType ==  FSReadToolSettingsTypeVideo ||
        self.settingsType ==  FSReadToolSettingsTypeLink) {
        self.enableProperty = NO;
    }else{
        self.enableProperty = YES;
    }
}

- (BOOL)replaceSettings:(FSReadToolSettings *)settings{
    if (_toolbarItem) {
        return NO;
    }
    _toolSettings = settings;
    _settingsType = settings.settingsType;
    [self initialize];
    return YES;
}

- (void)setEnabled:(BOOL)enabled{
    _enabled = enabled;
    _toolbarItem.item.enabled = _enabled;
}

- (void)setTitle:(NSString *)title
{
    _title = title;
    if (!_selectTitle || _selectTitle.length < 1) _selectTitle = title;
}

- (void)setExtensionsManager:(UIExtensionsManager *)extensionsManager{
    _extensionsManager = extensionsManager;
    BOOL canAddAnnot = [self.extensionsManager annotsPermissionCanAdd];
    
    if (self.settingsType == FSReadToolSettingsTypeTextField ||
        self.settingsType ==  FSReadToolSettingsTypeCheckBox ||
        self.settingsType ==  FSReadToolSettingsTypeRadioButton ||
        self.settingsType ==  FSReadToolSettingsTypeComboBox ||
        self.settingsType ==  FSReadToolSettingsTypeListBox ||
        self.settingsType ==  FSReadToolSettingsTypeSignatureField ||
        self.settingsType ==  FSReadToolSettingsTypeImageField) {
        _enabled = ![FSPermissionUtilty canNotEnterForm:_extensionsManager];
        if (![_extensionsManager allowFormPermission]) {
            _enabled = NO;
        }
    }else if (self.settingsType >= FSReadToolSettingsTypeFillSignText &&
               self.settingsType <=  FSReadToolSettingsTypeFillSignRoundRectangle){
        FillSignModule *fillSignModule = (FillSignModule *)[self.extensionsManager getModuleByName:Module_Fill_Sign];
        _enabled = [fillSignModule canAddFillSign];
        
    }else if (self.settingsType ==  FSReadToolSettingsTypeAudio ||
              self.settingsType ==  FSReadToolSettingsTypeVideo ||
              self.settingsType ==  FSReadToolSettingsTypeLink){
        _enabled = (canAddAnnot && [FSPermissionUtilty canAddEditAnnot:_extensionsManager]);
        if (self.settingsType ==  FSReadToolSettingsTypeLink){
            _enabled = !self.extensionsManager.config.disableLink && _enabled;
        }
    }else if (self.settingsType == FSReadToolSettingsTypeDigitalSignature){
        _enabled = canAddAnnot;
        if ([FSPermissionUtilty canAddSign:extensionsManager]) {
            if (extensionsManager.config.loadSignature &&
                [FSPermissionUtilty canAddSignToDocument:extensionsManager] &&
                [FSPermissionUtilty canAddSign:extensionsManager] &&
                ![extensionsManager.pdfViewCtrl isDynamicXFA] &&
                [FSPermissionUtilty canFillForm:extensionsManager]){
                if ([extensionsManager.permissionProvider checkPermission:FSFunctionPDFSign] != FSPermissionStateHide) {
                    _enabled = YES;
                }
            }
        }
    }else if (self.settingsType ==  FSReadToolSettingsTypePassWord ||
              self.settingsType ==  FSReadToolSettingsTypeTrutedCertificates){
        if (self.settingsType ==  FSReadToolSettingsTypePassWord) {
            FSPDFDocPasswordType psswordType = [_extensionsManager.pdfViewCtrl.currentDoc getPasswordType];
            if (psswordType == FSPDFDocPwdUser ||
                psswordType == FSPDFDocPwdOwner) {
                self.title = FSLocalizedForKey(@"kDecryption");
                _imageName = @"more_password_remove_password";
            }
            if ([_extensionsManager.pdfViewCtrl isDynamicXFA] || [_extensionsManager.pdfViewCtrl isRMSProtected] || !_extensionsManager.enableModifyDocument || _extensionsManager.pdfViewCtrl.currentDoc.isProtected) {
                _enabled = NO;
            }
        }
    }else{
        if (self.settingsType !=  FSReadToolSettingsTypeEraser) {
            _enabled = canAddAnnot;
        }else{
            _enabled = [Utility canAddAnnot:self.extensionsManager.pdfViewCtrl] && [self.extensionsManager annotsPermissionCanModify];
        }
    }
}

- (FSReadToolSettings *)toolSettings{
    if (!_toolSettings && self.extensionsManager) {
        _toolSettings = [[FSReadToolSettings alloc] initWithToolSettingType:_settingsType extensionsManager:self.extensionsManager];
    }
    return _toolSettings;
}

- (FSMainToolbarItem *)toolbarItem{
    if (_customItem) return _customItem;
    if (!_toolbarItem && self.toolSettings) {
        FSItem *item = nil;
        NSDictionary *typeMapToolName = @{
            @(FSReadToolSettingsTypeAudio):Tool_Audio,
            @(FSReadToolSettingsTypeVideo):Tool_Video,
            @(FSReadToolSettingsTypeLink):Tool_Link,
            @(FSReadToolSettingsTypeNote):Tool_Note,
            @(FSReadToolSettingsTypeAttachments):Tool_Attachment,
            @(FSReadToolSettingsTypeStamp):Tool_Stamp,
            @(FSReadToolSettingsTypeTypewriter):Tool_Freetext,
            @(FSReadToolSettingsTypeTextbox):Tool_Textbox,
            @(FSReadToolSettingsTypeCallout):Tool_Callout,
            @(FSReadToolSettingsTypeHighlight):Tool_Highlight,
            @(FSReadToolSettingsTypeAreaHighlight):Tool_Highlight,
            @(FSReadToolSettingsTypeUnderline):Tool_Underline,
            @(FSReadToolSettingsTypeSquiggly):Tool_Squiggly,
            @(FSReadToolSettingsTypeStrikeout):Tool_Strikeout,
            @(FSReadToolSettingsTypeReplaceText):Tool_Replace,
            @(FSReadToolSettingsTypeInsertText):Tool_Insert,
            @(FSReadToolSettingsTypeMeasure):Tool_Distance,
            @(FSReadToolSettingsTypeImageAnnotation):Tool_Image,
            @(FSReadToolSettingsTypePencil):Tool_Pencil,
            @(FSReadToolSettingsTypeHighlighter):Tool_Pencil,
            @(FSReadToolSettingsTypeEraser):Tool_Eraser,
            @(FSReadToolSettingsTypeRectangle):Tool_Rectangle,
            @(FSReadToolSettingsTypeOval):Tool_Oval,
            @(FSReadToolSettingsTypeLine):Tool_Line,
            @(FSReadToolSettingsTypeArrow):Tool_Arrow,
            @(FSReadToolSettingsTypePolygon):Tool_Polygon,
            @(FSReadToolSettingsTypePolyLine):Tool_PolyLine,
            @(FSReadToolSettingsTypeCloud):Tool_Polygon,
            @(FSReadToolSettingsTypeTextField):Tool_Form,
            @(FSReadToolSettingsTypeCheckBox):Tool_Form,
            @(FSReadToolSettingsTypeRadioButton):Tool_Form,
            @(FSReadToolSettingsTypeComboBox):Tool_Form,
            @(FSReadToolSettingsTypeListBox):Tool_Form,
            @(FSReadToolSettingsTypeSignatureField):Tool_Form,
            @(FSReadToolSettingsTypeImageField):Tool_Form,
            @(FSReadToolSettingsTypeFillSignText):Module_Fill_Sign,
            @(FSReadToolSettingsTypeFillSignSegText):Module_Fill_Sign,
            @(FSReadToolSettingsTypeFillSignProfile):Module_Fill_Sign,
            @(FSReadToolSettingsTypeFillSignCheckMark):Module_Fill_Sign,
            @(FSReadToolSettingsTypeFillSignCrossMark):Module_Fill_Sign,
            @(FSReadToolSettingsTypeFillSignDot):Module_Fill_Sign,
            @(FSReadToolSettingsTypeFillSignLine):Module_Fill_Sign,
            @(FSReadToolSettingsTypeFillSignRoundRectangle):Module_Fill_Sign,
            @(FSReadToolSettingsTypeDigitalSignature):Module_Signature,
            @(FSReadToolSettingsTypeRedact):Tool_Password,
            @(FSReadToolSettingsTypePassWord):Tool_Redaction,
            @(FSReadToolSettingsTypeTrutedCertificates):Module_Signature
        };
        NSDictionary *typeMapModuleName = @{
            @(FSReadToolSettingsTypeAudio):Module_Audio_Video,
            @(FSReadToolSettingsTypeVideo):Module_Audio_Video,
            @(FSReadToolSettingsTypeLink):Tool_Link,
            @(FSReadToolSettingsTypeNote):Tool_Note,
            @(FSReadToolSettingsTypeAttachments):Tool_Attachment,
            @(FSReadToolSettingsTypeStamp):Tool_Stamp,
            @(FSReadToolSettingsTypeTypewriter):Tool_Freetext,
            @(FSReadToolSettingsTypeTextbox):Tool_Textbox,
            @(FSReadToolSettingsTypeCallout):Tool_Callout,
            @(FSReadToolSettingsTypeHighlight):Tool_Markup,
            @(FSReadToolSettingsTypeAreaHighlight):Tool_Markup,
            @(FSReadToolSettingsTypeUnderline):Tool_Markup,
            @(FSReadToolSettingsTypeSquiggly):Tool_Markup,
            @(FSReadToolSettingsTypeStrikeout):Tool_Markup,
            @(FSReadToolSettingsTypeReplaceText):Tool_Replace,
            @(FSReadToolSettingsTypeInsertText):Tool_Insert,
            @(FSReadToolSettingsTypeMeasure):Tool_Distance,
            @(FSReadToolSettingsTypeImageAnnotation):Tool_Image,
            @(FSReadToolSettingsTypePencil):Tool_Pencil,
            @(FSReadToolSettingsTypeHighlighter):Tool_Pencil,
            @(FSReadToolSettingsTypeEraser):Tool_Eraser,
            @(FSReadToolSettingsTypeRectangle):@"Shape",
            @(FSReadToolSettingsTypeOval):@"Shape",
            @(FSReadToolSettingsTypeLine):@"Line",
            @(FSReadToolSettingsTypeArrow):@"Line",
            @(FSReadToolSettingsTypePolygon):Tool_Polygon,
            @(FSReadToolSettingsTypePolyLine):Tool_PolyLine,
            @(FSReadToolSettingsTypeCloud):Tool_Polygon,
            @(FSReadToolSettingsTypeTextField):Tool_Form,
            @(FSReadToolSettingsTypeCheckBox):Tool_Form,
            @(FSReadToolSettingsTypeRadioButton):Tool_Form,
            @(FSReadToolSettingsTypeComboBox):Tool_Form,
            @(FSReadToolSettingsTypeListBox):Tool_Form,
            @(FSReadToolSettingsTypeSignatureField):Tool_Form,
            @(FSReadToolSettingsTypeImageField):Tool_Form,
            @(FSReadToolSettingsTypeFillSignText):Tool_Fill_Sign,
            @(FSReadToolSettingsTypeFillSignSegText):Tool_Fill_Sign,
            @(FSReadToolSettingsTypeFillSignProfile):Tool_Fill_Sign,
            @(FSReadToolSettingsTypeFillSignCheckMark):Tool_Fill_Sign,
            @(FSReadToolSettingsTypeFillSignCrossMark):Tool_Fill_Sign,
            @(FSReadToolSettingsTypeFillSignDot):Tool_Fill_Sign,
            @(FSReadToolSettingsTypeFillSignLine):Tool_Fill_Sign,
            @(FSReadToolSettingsTypeFillSignRoundRectangle):Tool_Fill_Sign,
            @(FSReadToolSettingsTypeDigitalSignature):Tool_Signature,
            @(FSReadToolSettingsTypeRedact):Module_Redaction,
            @(FSReadToolSettingsTypePassWord):Module_Password,
            @(FSReadToolSettingsTypeTrutedCertificates):Tool_Signature
        };
        
        switch (self.settingsType) {
            case FSReadToolSettingsTypeText:
                break;
            case FSReadToolSettingsTypeImage:
                break;
            case FSReadToolSettingsTypeAudio:
            case FSReadToolSettingsTypeVideo:
            case FSReadToolSettingsTypeLink:
            case FSReadToolSettingsTypeNote:
            case FSReadToolSettingsTypeAttachments:
            case FSReadToolSettingsTypeStamp:
            case FSReadToolSettingsTypeTypewriter:
            case FSReadToolSettingsTypeTextbox:
            case FSReadToolSettingsTypeCallout:
            case FSReadToolSettingsTypeHighlight:
            case FSReadToolSettingsTypeAreaHighlight:
            case FSReadToolSettingsTypeUnderline:
            case FSReadToolSettingsTypeSquiggly:
            case FSReadToolSettingsTypeStrikeout:
            case FSReadToolSettingsTypeReplaceText:
            case FSReadToolSettingsTypeInsertText:
            case FSReadToolSettingsTypeMeasure:
            case FSReadToolSettingsTypeImageAnnotation:
            case FSReadToolSettingsTypePencil:
            case FSReadToolSettingsTypeHighlighter:
            case FSReadToolSettingsTypeEraser:
            case FSReadToolSettingsTypeRectangle:
            case FSReadToolSettingsTypeOval:
            case FSReadToolSettingsTypeLine:
            case FSReadToolSettingsTypeArrow:
            case FSReadToolSettingsTypePolygon:
            case FSReadToolSettingsTypePolyLine:
            case FSReadToolSettingsTypeCloud:
            case FSReadToolSettingsTypeTextField:
            case FSReadToolSettingsTypeCheckBox:
            case FSReadToolSettingsTypeRadioButton:
            case FSReadToolSettingsTypeComboBox:
            case FSReadToolSettingsTypeListBox:
            case FSReadToolSettingsTypeSignatureField:
            case FSReadToolSettingsTypeImageField:
            case FSReadToolSettingsTypeFillSignText:
            case FSReadToolSettingsTypeFillSignSegText:
            case FSReadToolSettingsTypeFillSignProfile:
            case FSReadToolSettingsTypeFillSignCheckMark:
            case FSReadToolSettingsTypeFillSignCrossMark:
            case FSReadToolSettingsTypeFillSignDot:
            case FSReadToolSettingsTypeFillSignLine:
            case FSReadToolSettingsTypeFillSignRoundRectangle:
            case FSReadToolSettingsTypeDigitalSignature:
            case FSReadToolSettingsTypeRedact:
            case FSReadToolSettingsTypePassWord:
            case FSReadToolSettingsTypeTrutedCertificates:
            {
                NSString *toolName = typeMapToolName[@(self.settingsType)];
                NSString *moduleName = typeMapModuleName[@(self.settingsType)];
                id<IModule> module = [_extensionsManager getModuleByName:moduleName];
                if ([self.extensionsManager.config.tools containsObject:toolName] ||
                    ([toolName isEqualToString:Tool_Attachment] && self.extensionsManager.config.loadAttachment) ||
                    (([toolName isEqualToString:Tool_Fill_Sign] || [toolName isEqualToString:Tool_Signature]) && module) ||
                    [toolName isEqualToString:Tool_Link] ||
                    (module && self.extensionsManager.config.loadForm)) {
                    item = [FSReadToolbarItem createItemFromReadToolbarItem:self needBackground:YES];
                    item.enabled = _enabled;
                    @weakify(self)
                    item.onTapClick = ^(FSItem * _Nonnull _item) {
                        @strongify(self)
                        [self didClickedItem:_item moduleName:moduleName];
                    };
                }
            }
            break;
        }
        _toolbarItem = item ? [[FSMainToolbarItem alloc] initWithCustomView:item spacing:ITEM_BOTH_SPACE_NORMAL] : nil ;
        if (_toolbarItem) {
            item.owner = _toolbarItem;
        }
    }
    return _toolbarItem;
}

- (void)didClickedItem:(FSItem *)item moduleName:(NSString *)moduleName{
    FSMainTopbarToolTagItem *currentTagItem =  self.extensionsManager.toolbarManager.mainTopbar.currentTagItem;
    FSMainToolbarItem *propertyItem = [self.extensionsManager.toolbarManager getMainItemWithType:FSMainToolbarItemTypeProperty];
    propertyItem.item.onTapClick = nil;
    if (currentTagItem.currentReadToolbarItem == self) {
        item.selected = NO;
    }else{
        if ([self.extensionsManager.currentToolHandler respondsToSelector:@selector(toolSettings)]) {
            FSReadToolSettings *toolSettings = self.extensionsManager.currentToolHandler.toolSettings;
            if (toolSettings) {
                FSMainTopbarToolTagItem *oldTagItem;
                for (FSMainTopbarToolTagItem *tagItem in self.extensionsManager.toolbarManager.mainTopbar.tagItems) {
                    if (oldTagItem) break;
                    for (FSReadToolbarItem *readItem in tagItem.readToolbarItems) {
                        if (readItem.toolSettings == toolSettings) {
                            oldTagItem = tagItem;
                            break;
                        }
                    }
                }
                if (oldTagItem.currentReadToolbarItem) {
                    oldTagItem.currentReadToolbarItem = nil;
                }
            }
        }

        FSItem *currentItem = currentTagItem.currentReadToolbarItem.toolbarItem.item;
        currentItem.selected = NO;
        currentTagItem.currentReadToolbarItem = self;
        currentItem = self.toolbarItem.item;
        currentItem.selected = YES;
    }
    
    if (self.settingsType ==  FSReadToolSettingsTypeRedact ||
        self.settingsType ==  FSReadToolSettingsTypePassWord ||
        self.settingsType ==  FSReadToolSettingsTypeTrutedCertificates ||
        self.settingsType ==  FSReadToolSettingsTypeRMS ||
        self.settingsType ==  FSReadToolSettingsTypeAIP) {
        item.selected = NO;
        id<IReadToolSettingsModule> module = [self.extensionsManager getModuleByName:moduleName];
        [module didClickedReadToolItem:self];
        return;
    }
    
    if (item.selected) {
        propertyItem.item.enabled = self.enableProperty;
        if (self.settingsType == FSReadToolSettingsTypeStamp) {
            [propertyItem.item setImage:[Utility getStampImageWithIcon:self.toolSettings.stampIcon] forState:UIControlStateNormal];
        }else{
            [propertyItem.item setImage:FSImageNamedAndCustomColor(@"read_property_color_ring",self.propertyColor) forState:UIControlStateNormal];
        }
        if (self.settingsType == FSReadToolSettingsTypeAudio || self.settingsType == FSReadToolSettingsTypeVideo) {
            AudioVideoModule *audioVideoModule = [self.extensionsManager getModuleByName:moduleName];
            if (self.settingsType == FSReadToolSettingsTypeAudio) {
                audioVideoModule.mediaFileType = FSMediaFileTypeAudio;
            }else{
                audioVideoModule.mediaFileType = FSMediaFileTypeVideo;
            }
            [audioVideoModule didClickedReadToolItem:self];
        } else{
            id<IReadToolSettingsModule> module = [self.extensionsManager getModuleByName:moduleName];
            
            if (self.settingsType == FSReadToolSettingsTypeHighlight ||
                self.settingsType == FSReadToolSettingsTypeAreaHighlight) {
                module.annotType = FSAnnotHighlight;
            }
            if (self.settingsType == FSReadToolSettingsTypeUnderline) {
                module.annotType = FSAnnotUnderline;
            }
            if (self.settingsType == FSReadToolSettingsTypeSquiggly) {
                module.annotType = FSAnnotSquiggly;
            }
            if (self.settingsType == FSReadToolSettingsTypeStrikeout) {
                module.annotType = FSAnnotStrikeOut;
            }
            
            if (self.settingsType == FSReadToolSettingsTypeRectangle) {
                module.annotType = FSAnnotSquare;
            }
            if (self.settingsType == FSReadToolSettingsTypeOval) {
                module.annotType = FSAnnotCircle;
            }
            if (self.settingsType == FSReadToolSettingsTypePolygon) {
                module.annotType = FSAnnotPolygon;
            }
            if (self.settingsType == FSReadToolSettingsTypeCloud) {
                module.annotType = FSAnnotCloud;
            }
            [module didClickedReadToolItem:self];
        }
    }else{
        currentTagItem.currentReadToolbarItem = nil;
        propertyItem.item.enabled = NO;
        [self.extensionsManager setCurrentToolHandler:nil];
        if (currentTagItem.toolTag != FSMainTopbarToolTagSign) {
            [self.extensionsManager changeState:self.extensionsManager.lastManagerStateWhenAnnotTool];
        }
    }
}

- (FSReadToolSettings *)creatReadToolSettingItemForToolSettingType:(FSReadToolSettingsType)type extensionsManager:(UIExtensionsManager *)extensionsManager{
    FSReadToolSettings *item = [[FSReadToolSettings alloc] initWithToolSettingType:type extensionsManager:extensionsManager];
    return item;
}

- (void)setTitleAndImage{
    FSReadToolSettingsType type = self.toolSettings ? self.toolSettings.settingsType : _settingsType;
    switch (type) {
        case FSReadToolSettingsTypeText:
            break;
        case FSReadToolSettingsTypeImage:
            break;
        case FSReadToolSettingsTypeAudio:
            self.title = FSLocalizedForKey(@"kAudio");
            _imageName = @"read_tool_audio";
            break;
        case FSReadToolSettingsTypeVideo:
            self.title = FSLocalizedForKey(@"kVideo");
            _imageName = @"read_tool_video";
            break;
        case FSReadToolSettingsTypeLink:
            self.title = FSLocalizedForKey(@"kLink");
            _imageName = @"read_tool_link";
            break;
            
        case FSReadToolSettingsTypeNote:
            self.title = FSLocalizedForKey(@"kNote");
            _imageName = @"comment_note";
            break;
        case FSReadToolSettingsTypeAttachments:
            self.title = FSLocalizedForKey(@"kAttachment");
            _imageName = @"comment_attachment";
            break;
        case FSReadToolSettingsTypeStamp:
            self.title = FSLocalizedForKey(@"kStamp");
            _imageName = @"comment_stamp";
            break;
        case FSReadToolSettingsTypeTypewriter:
            self.title = FSLocalizedForKey(@"kTypewriter");
            _imageName = @"comment_typewriter";
            break;
        case FSReadToolSettingsTypeTextbox:
            self.title = FSLocalizedForKey(@"kTextbox");
            _imageName = @"comment_textbox";
            break;
        case FSReadToolSettingsTypeCallout:
            self.title = FSLocalizedForKey(@"kCallout");
            _imageName = @"comment_callout";
            break;
        case FSReadToolSettingsTypeHighlight:
            self.title = FSLocalizedForKey(@"kHighlight");
            _imageName = @"comment_highlight";
            break;
        case FSReadToolSettingsTypeUnderline:
            self.title = FSLocalizedForKey(@"kUnderline");
            _imageName = @"comment_underline";
            break;
        case FSReadToolSettingsTypeSquiggly:
            self.title = FSLocalizedForKey(@"kSquiggly");
            _imageName = @"comment_squiggly";
            break;
        case FSReadToolSettingsTypeStrikeout:
            self.title = FSLocalizedForKey(@"kStrikeout");
            _imageName = @"comment_strikeout";
            break;
        case FSReadToolSettingsTypeReplaceText:
            self.title = FSLocalizedForKey(@"kReplaceText");
            _imageName = @"comment_replace_text";
            break;
        case FSReadToolSettingsTypeInsertText:
            self.title = FSLocalizedForKey(@"kInsertText");
            _imageName = @"comment_insert_text";
            break;
        case FSReadToolSettingsTypeAreaHighlight:
            self.title = FSLocalizedForKey(@"kAreaHighlight");
            _imageName = @"comment_area_highlight";
            break;
        case FSReadToolSettingsTypeMeasure:
            self.title = FSLocalizedForKey(@"kMeasure");
            _imageName = @"comment_measure";
            break;
        case FSReadToolSettingsTypeImageAnnotation:
            self.title = FSLocalizedForKey(@"kImageAnnotation");
            _imageName = @"comment_image";
            break;
            
        case FSReadToolSettingsTypePencil:
            self.title = FSLocalizedForKey(@"kPencil");
            _imageName = @"comment_pencil";
            break;
            
        case FSReadToolSettingsTypeHighlighter:
            self.title = FSLocalizedForKey(@"kHighlighter");
            _imageName = @"comment_highlighter";
            break;
        case FSReadToolSettingsTypeEraser:
            self.title = FSLocalizedForKey(@"kEraser");
            _imageName = @"read_tool_eraser";
            break;
        case FSReadToolSettingsTypeRectangle:
            self.title = FSLocalizedForKey(@"kRectangle");
            _imageName = @"comment_rectangle";
            break;
        case FSReadToolSettingsTypeOval:
            self.title = FSLocalizedForKey(@"kCircle");
            _imageName = @"comment_oval";
            break;
        case FSReadToolSettingsTypeLine:
            self.title = FSLocalizedForKey(@"kLine");
            _imageName = @"comment_line";
            break;
        case FSReadToolSettingsTypeArrow:
            self.title = FSLocalizedForKey(@"kArrow");
            _imageName = @"comment_arrow";
            break;
        case FSReadToolSettingsTypePolygon:
            self.title = FSLocalizedForKey(@"kPolygon");
            _imageName = @"comment_polygon";
            break;
        case FSReadToolSettingsTypePolyLine:
            self.title = FSLocalizedForKey(@"kPolyLine");
            _imageName = @"comment_polyLine";
            break;
        case FSReadToolSettingsTypeCloud:
            self.title = FSLocalizedForKey(@"kCloud");
            _imageName = @"comment_cloud";
            break;
            
        case FSReadToolSettingsTypeTextField:
            self.title = FSLocalizedForKey(@"kTextField");
            _imageName = @"read_tool_text_field";
            break;
        case FSReadToolSettingsTypeCheckBox:
            self.title = FSLocalizedForKey(@"kCheckBox");
            _imageName = @"read_tool_check_box";
            break;
        case FSReadToolSettingsTypeRadioButton:
            self.title = FSLocalizedForKey(@"kRadioButton");
            _imageName = @"read_tool_radio_button";
            break;
        case FSReadToolSettingsTypeComboBox:
            self.title = FSLocalizedForKey(@"kComboBox");
            _imageName = @"read_tool_combo_box";
            break;
        case FSReadToolSettingsTypeListBox:
            self.title = FSLocalizedForKey(@"kListBox");
            _imageName = @"read_tool_list_box";
            break;
        case FSReadToolSettingsTypeSignatureField:
            self.title = FSLocalizedForKey(@"kSignatureField");
            _imageName = @"read_tool_signature_field";
            break;
        case FSReadToolSettingsTypeImageField:
            self.title = FSLocalizedForKey(@"kImageField");
            _imageName = @"read_tool_image_field";
            break;
        case FSReadToolSettingsTypeFillSignText:
            self.title = FSLocalizedForKey(@"kAddText");
            _imageName = @"fill_sign_text";
            break;
        case FSReadToolSettingsTypeFillSignSegText:
            self.title = FSLocalizedForKey(@"kAddCombField");
            _imageName = @"fill_sign_segtext";
            break;
        case FSReadToolSettingsTypeFillSignProfile:
            self.title = FSLocalizedForKey(@"kProfiletitle");
            _imageName = @"fill_sign_profile";
            break;
        case FSReadToolSettingsTypeFillSignCheckMark:
            self.title = FSLocalizedForKey(@"kAddCheck");
            _imageName = @"fill_sign_check";
            break;
        case FSReadToolSettingsTypeFillSignCrossMark:
            self.title = FSLocalizedForKey(@"kAddX");
            _imageName = @"fill_sign_X";
            break;
        case FSReadToolSettingsTypeFillSignDot:
            self.title = FSLocalizedForKey(@"kAddDot");
            _imageName = @"fill_sign_dot";
            break;
        case FSReadToolSettingsTypeFillSignLine:
            self.title = FSLocalizedForKey(@"kAddLine");
            _imageName = @"fill_sign_line";
            break;
        case FSReadToolSettingsTypeFillSignRoundRectangle:
            self.title = FSLocalizedForKey(@"kAddRoundedRectangle");
            _imageName = @"fill_sign_roundrect";
            break;
        case FSReadToolSettingsTypeDigitalSignature:
            self.title = FSLocalizedForKey(@"kSignatureTitle");
            _imageName = @"fill_sign_sign";
            break;
        case FSReadToolSettingsTypeRedact:
            self.title = FSLocalizedForKey(@"kRedaction");
            _imageName = @"more_redact_redaction";
            break;
        case FSReadToolSettingsTypePassWord:
            self.title = FSLocalizedForKey(@"kEncryption");
            _imageName = @"more_password_file_encryption";
            FSPDFDocPasswordType psswordType = [_extensionsManager.pdfViewCtrl.currentDoc getPasswordType];
            if (psswordType == FSPDFDocPwdUser ||
                psswordType == FSPDFDocPwdOwner) {
                self.title = FSLocalizedForKey(@"kDecryption");
                _imageName = @"more_password_remove_password";
            }
            break;
        case FSReadToolSettingsTypeTrutedCertificates:
            self.title = FSLocalizedForKey(@"kTrustedCert");
            _imageName = @"more_protect_cert_protect";
            break;
    }
}

- (void)setPropertyColor:(UIColor * _Nullable)propertyColor{
    self.toolSettings.propertyColor = propertyColor;
    [_toolbarItem.item setImage:FSImageNamedAndCustomColor(self.imageName, self.propertyColor) forState:UIControlStateNormal];
    FSMainTopbarToolTagItem *currentTagItem =  self.extensionsManager.toolbarManager.mainTopbar.currentTagItem;
    if (currentTagItem.currentReadToolbarItem == self) {
        FSMainToolbarItem *propertyItem = [self.extensionsManager.toolbarManager getMainItemWithType:FSMainToolbarItemTypeProperty];
        propertyItem.item.enabled = YES;
        [propertyItem.item setImage:FSImageNamedAndCustomColor(@"read_property_color_ring", self.propertyColor) forState:UIControlStateNormal];
    }
}

- (UIColor *)propertyColor{
    return self.toolSettings.propertyColor;
}

- (void)selectFormSuperBar{
    !self.toolbarItem.item.onTapClick ? : self.toolbarItem.item.onTapClick(self.toolbarItem.item);
}

+ (FSItem *)createItemFromReadToolbarItem:(FSReadToolbarItem *)readToolbarItem needBackground:(BOOL)needBackground{
    FSItem *item = [FSItem new];
    item.font = self.fs_standardGeneralFont_GF3;
    [item setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
    item.titleEdgeInsets = UIEdgeInsetsMake(0, 4, 0, 8);
    if (needBackground) {
//        UIImage *backgroundImage = [UIImage fs_imageWithColor:[UIColor clearColor]  size:CGSizeMake(40.f, 40.f)];
//        [item setBackgroundImage:backgroundImage forState:UIControlStateNormal];
//        [item setBackgroundImage:[Utility commonDynamicBackgroundImage] forState:UIControlStateSelected];
        [item setBackgroudViewColor:[UIColor clearColor] forState:UIControlStateNormal];
        [item setBackgroudViewColor:self.fs_dividerColor_P2 forState:UIControlStateSelected];
    }
    if ((readToolbarItem.itemContentType & FSItemContentTypeText) && readToolbarItem.extensionsManager.toolbarManager.isShowReadBarItemDescript){
        [item setTitle:readToolbarItem.title forState:UIControlStateNormal];
    }
    if (readToolbarItem.itemContentType & FSItemContentTypeImage){
        [item setImage:FSImageNamedAndCustomColor(readToolbarItem.imageName, readToolbarItem.propertyColor) forState:UIControlStateNormal];
    }
    if (readToolbarItem.itemContentType == FSItemContentTypeImageAndText) [item rebuildImageTextRelation:FSImageTextRelationRight];
    item.accessibilityIdentifier = readToolbarItem.imageName;
    item.accessibilityLabel = readToolbarItem.title;
    switch (readToolbarItem.toolSettings.settingsType) {
        case FSReadToolSettingsTypeText:
            break;
        case FSReadToolSettingsTypeImage:
            break;
        case FSReadToolSettingsTypeAudio:
            break;
        case FSReadToolSettingsTypeVideo:
            break;
        case FSReadToolSettingsTypeLink:
            break;
            
        case FSReadToolSettingsTypeAttachments:
            break;
        case FSReadToolSettingsTypeStamp:
            break;
        case FSReadToolSettingsTypeTypewriter:

            break;
        case FSReadToolSettingsTypeTextbox:
            break;
        case FSReadToolSettingsTypeCallout:
            break;
        case FSReadToolSettingsTypeHighlight:
            break;
        case FSReadToolSettingsTypeUnderline:
            break;
        case FSReadToolSettingsTypeSquiggly:
            break;
        case FSReadToolSettingsTypeStrikeout:
            break;
        case FSReadToolSettingsTypeReplaceText:
            break;
        case FSReadToolSettingsTypeInsertText:
            break;
        case FSReadToolSettingsTypeAreaHighlight:
            break;
        case FSReadToolSettingsTypeMeasure:
            break;
        case FSReadToolSettingsTypeImageAnnotation:
            break;
            
        case FSReadToolSettingsTypePencil:
            break;
        case FSReadToolSettingsTypeHighlighter:
            break;
        case FSReadToolSettingsTypeEraser:
            break;
        case FSReadToolSettingsTypeRectangle:
            break;
        case FSReadToolSettingsTypeOval:
            break;
        case FSReadToolSettingsTypeLine:
            break;
        case FSReadToolSettingsTypeArrow:
            break;
        case FSReadToolSettingsTypePolygon:
            break;
        case FSReadToolSettingsTypePolyLine:
            break;
        case FSReadToolSettingsTypeCloud:
            break;
            
        case FSReadToolSettingsTypeTextField:
            break;
        case FSReadToolSettingsTypeCheckBox:
            break;
        case FSReadToolSettingsTypeRadioButton:
            break;
        case FSReadToolSettingsTypeComboBox:
            break;
        case FSReadToolSettingsTypeListBox:
            break;
        case FSReadToolSettingsTypeSignatureField:
            break;
        case FSReadToolSettingsTypeImageField:
            break;
        default:
            break;
    }

    return item;
}

@end

@implementation FSAddToolsTitleItem

- (instancetype)initWithTitle:(NSString *)title subReadItems:(NSArray <FSReadToolbarItem *> *)subReadItems{
    self = [super init];
    if (self) {
        self.title = title;
        _subReadItems = subReadItems.copy;
        _unfold = YES;
    }
    return self;
}
@end
