/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSScrollToolbar.h"

@interface FSGradientRoundView : UIImageView @end

@implementation FSGradientRoundView

- (instancetype)initWithFrame:(CGRect)frame{
    self = [super initWithFrame:frame];
    if (self) {
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
        UIImage *image = FSImageNamed(@"read_tool_bar_mask");
#else
        UIImage *image = FSImageNamed(@"read_tool_bar_mask_less_xcode9");
#endif
        self.contentMode = UIViewContentModeCenter;
        self.image = image;
    }
    return self;
}

@end

@interface FSToolbarItemsContentView : FSAdapterUIScrollView @end
@implementation FSToolbarItemsContentView
- (instancetype)init{
    self = [super init];
    if (self) {
//        self.bounces = NO;
        self.showsVerticalScrollIndicator = NO;
        self.showsHorizontalScrollIndicator = NO;
    }
    return self;
}
@end

@interface FSScrollToolbar ()<UIScrollViewDelegate, UIViewAdapter>
@property (nonatomic, strong) FSGradientRoundView *leftGradientView;
@property (nonatomic, strong) FSGradientRoundView *rightGradientView;
@property (nonatomic, strong) FSToolbarItemsContentView *itemsContentView;

@end

@implementation FSScrollToolbar
@synthesize layoutGuideHiddenBlankViewCenter = _layoutGuideHiddenBlankViewCenter;

- (FSLayoutGuideHiddenBlankView *)layoutGuideHiddenBlankViewCenter{
    if (!_layoutGuideHiddenBlankViewCenter) {
        _layoutGuideHiddenBlankViewCenter = [FSLayoutGuideHiddenBlankView new];
        _layoutGuideHiddenBlankViewCenter.hidden = YES;
        [self insertSubview:_layoutGuideHiddenBlankViewCenter atIndex:0];
        [_layoutGuideHiddenBlankViewCenter mas_makeConstraints:^(MASConstraintMaker *make) {
            if (self.axis == UILayoutConstraintAxisHorizontal) {
                make.centerX.fs_equalToSuperview().priorityMedium();
                if (self.layoutGuideHiddenBlankViewRight) make.right.lessThanOrEqualTo(self.layoutGuideHiddenBlankViewRight.mas_left).priorityHigh();
                make.top.bottom.fs_equalToSuperview();
            }else{
                make.centerY.fs_equalToSuperview().priorityMedium();
                make.left.right.fs_equalToSuperview();
                if (self.layoutGuideHiddenBlankViewRight) make.bottom.lessThanOrEqualTo(self.layoutGuideHiddenBlankViewRight.mas_top).priorityHigh();
            }
        }];
        _layoutGuideHiddenBlankViewCenter.adapter = self;
    }
    return _layoutGuideHiddenBlankViewCenter;
}

- (void)setAxis:(UILayoutConstraintAxis)axis{
    [super setAxis:axis];
    
    if (self.axis == UILayoutConstraintAxisVertical) {
        self.itemsContentView.contentInset = self.verticalScrollBarEdgeInsets;
    }else{
        self.itemsContentView.contentInset = self.horizontalScrollBarEdgeInsets;
    }
    [self makeGradientViewConstraints];
}

- (void)makeGradientViewConstraints{
    if (self.axis == UILayoutConstraintAxisVertical) {
        [self.leftGradientView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.top.equalTo(self.mas_top).offset(-15);
            make.centerX.fs_equalToSuperview();
        }];
        
        self.leftGradientView.transform = CGAffineTransformMakeRotation(M_PI_2*3);
        
        if (self.layoutGuideHiddenBlankViewRight) {
            [self.rightGradientView mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.bottom.equalTo(self.layoutGuideHiddenBlankViewRight.mas_top).offset(15);
                make.centerX.fs_equalToSuperview();
            }];
        }
        self.rightGradientView.transform = CGAffineTransformMakeRotation(M_PI_2);
    }else{
        
        [self.leftGradientView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.equalTo(self.mas_left).offset(-5);
            make.centerY.fs_equalToSuperview();
        }];
        
        self.leftGradientView.transform = CGAffineTransformMakeRotation(M_PI);
        
        if (self.layoutGuideHiddenBlankViewRight) {
            [self.rightGradientView mas_remakeConstraints:^(MASConstraintMaker *make) {
                make.right.equalTo(self.layoutGuideHiddenBlankViewRight.mas_left).offset(5);
                make.centerY.fs_equalToSuperview();
            }];
        }
        self.rightGradientView.transform = CGAffineTransformIdentity;
    }
    [self checkGradientViewsHidden];
}

- (void)checkGradientViewsHidden{
    self.leftGradientView.hidden = YES;
    for (FSToolbarItem *item in self.centerItems.items) {
        UIView *view = item.item;
        if (CGRectEqualToRect(view.bounds, CGRectZero)) continue;
        CGRect itemRectToSelfRect = [view convertRect:view.bounds toView:self];
        CGRect intersectsRect = self.leftGradientView.frame;
        if (self.axis == UILayoutConstraintAxisHorizontal) {
            intersectsRect.origin = CGPointMake(-self.leftGradientView.fs_width, 0);
        }else{
            intersectsRect.origin = CGPointMake(0, -self.leftGradientView.fs_height);
        }
        if (CGRectIntersectsRect(itemRectToSelfRect, intersectsRect)) {
            self.leftGradientView.hidden = NO;
            break;
        }
    }
    self.rightGradientView.hidden = YES;
    for (FSToolbarItem *item in self.centerItems.items) {
        UIView *view = item.item;
        if (CGRectEqualToRect(view.bounds, CGRectZero)) continue;
        CGRect itemRectToSelfRect = [view convertRect:view.bounds toView:self];
        CGRect intersectsRect = self.rightGradientView.frame;
        if (self.axis == UILayoutConstraintAxisHorizontal) {
            intersectsRect.origin = CGPointMake(self.itemsContentView.fs_width, 0);
        }else{
            intersectsRect.origin = CGPointMake(0, self.itemsContentView.fs_height);
        }
        if (CGRectIntersectsRect(itemRectToSelfRect, intersectsRect)) {
            self.rightGradientView.hidden = NO;
            break;
        }
    }
}

- (void)addItem:(__kindof UIView *)item spacing:(CGFloat)spacing atPosition:(FSScrollToolbarItemPosition)position{
    if (position == FSScrollToolbarItemPositionScroll) {
        [super addItem:item spacing:spacing atPosition:FSItemPositionCenter];
    }else if (position == FSScrollToolbarItemPositionRight) {
        [super addItem:item spacing:spacing atPosition:FSItemPositionRight];
    } else{
        [super addItem:item spacing:spacing atPosition:position];
    }
}

- (void)removeItemAtPosition:(FSScrollToolbarItemPosition)position{
    if (position == FSScrollToolbarItemPositionScroll) {
        [super removeItemAtPosition:FSItemPositionCenter];
    }else if (position == FSScrollToolbarItemPositionRight) {
        [super removeItemAtPosition:FSItemPositionRight];
    }else{
        [super removeItemAtPosition:position];
    }
}

- (void)scrollTo:(UIView *)item{
    if (![item isDescendantOfView:self]) return;
    [self.itemsContentView scrollRectToVisible:item.frame animated:YES];
}

- (void)addItem:(FSItem *)item margin:(CGFloat)margin spacing:(CGFloat)spacing atPosition:(FSItemPosition)position{
    if (position == FSItemPositionCenter) {
        if (!self.itemsContentView) {
            self.itemsContentView = [FSToolbarItemsContentView new];
            self.itemsContentView.delegate = self;
            self.itemsContentView.adapter = self;
            if (self.axis == UILayoutConstraintAxisVertical) {
                self.itemsContentView.contentInset = self.verticalScrollBarEdgeInsets;
            }else{
                self.itemsContentView.contentInset = self.horizontalScrollBarEdgeInsets;
            }
            [self addSubview:self.itemsContentView];
            [self.itemsContentView mas_makeConstraints:^(MASConstraintMaker *make) {
                if (self.axis == UILayoutConstraintAxisHorizontal) {
                    make.left.top.bottom.fs_equalToSuperview();
                    if (self.layoutGuideHiddenBlankViewRight) make.right.equalTo(self.layoutGuideHiddenBlankViewRight.mas_left);
                    else make.right.fs_equalToSuperview().priorityLow();
                }else{
                    make.top.left.right.fs_equalToSuperview();
                    if (self.layoutGuideHiddenBlankViewRight) make.bottom.equalTo(self.layoutGuideHiddenBlankViewRight.mas_top);
                    else make.bottom.fs_equalToSuperview().priorityLow();
                }
            }];
        }
        [self.itemsContentView addSubview:item];
        margin = spacing;
    }
    [super addItem:item margin:margin spacing:spacing atPosition:position];
    if (position == FSItemPositionRight) {
        [self.itemsContentView mas_updateConstraints:^(MASConstraintMaker *make) {
            if (self.axis == UILayoutConstraintAxisHorizontal) {
                make.right.equalTo(self.layoutGuideHiddenBlankViewRight.mas_left);
            }else{
                make.bottom.equalTo(self.layoutGuideHiddenBlankViewRight.mas_top);
            }
        }];
        
        [self.layoutGuideHiddenBlankViewCenter mas_updateConstraints:^(MASConstraintMaker *make) {
            if (self.axis == UILayoutConstraintAxisHorizontal) {
                if (self.layoutGuideHiddenBlankViewRight) make.right.lessThanOrEqualTo(self.layoutGuideHiddenBlankViewRight.mas_left).priorityHigh();
            }else{
                if (self.layoutGuideHiddenBlankViewRight) make.bottom.lessThanOrEqualTo(self.layoutGuideHiddenBlankViewRight.mas_top).priorityHigh();
            }
        }];
    }
    
    [self addGradientViews];
}

- (void)cleanAuxiliaryViewForPosition:(FSItemPosition)position{
    [super cleanAuxiliaryViewForPosition:position];
    if (position == FSItemPositionCenter) {
        [self.itemsContentView removeFromSuperview];
        self.itemsContentView = nil;
    }
}

- (void)addGradientViews{
    if (!self.leftGradientView) {
        FSGradientRoundView *gradientView = [[FSGradientRoundView alloc] init];
        [self addSubview:gradientView];
        self.leftGradientView = gradientView;
    }
    [self bringSubviewToFront:self.leftGradientView];
    if (self.layoutGuideHiddenBlankViewRight) {
        if (!self.rightGradientView) {
            FSGradientRoundView *gradientView = [[FSGradientRoundView alloc] init];
            [self addSubview:gradientView];
            self.rightGradientView = gradientView;
        }
        [self bringSubviewToFront:self.rightGradientView];
    }
    
    [self makeGradientViewConstraints];
}

- (void)resetContentBarContraints{}

- (void)adjustLayoutGuideHiddenBlankViewCenter{
    if (self.axis == UILayoutConstraintAxisHorizontal && self.itemsContentView.contentSize.width > self.itemsContentView.fs_width) {
        [_layoutGuideHiddenBlankViewCenter mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.fs_equalToSuperview().offset(-self.itemsContentView.fs_offsetX).priorityMedium();
            make.top.bottom.fs_equalToSuperview();
        }];
    } else if (self.axis == UILayoutConstraintAxisVertical && self.itemsContentView.contentSize.height > self.itemsContentView.fs_height) {
        [_layoutGuideHiddenBlankViewCenter mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.top.fs_equalToSuperview().offset(-self.itemsContentView.fs_offsetY).priorityMedium();
            make.left.right.fs_equalToSuperview();
        }];
    }
}

- (void)layoutSubviews{
    [super layoutSubviews];
    if (self.axis == UILayoutConstraintAxisVertical && self.itemsContentView.fs_height > self.layoutGuideHiddenBlankViewCenter.fs_height) {
        [self.layoutGuideHiddenBlankViewCenter mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.centerY.mas_equalTo(self.itemsContentView).priorityMedium();
            make.left.right.fs_equalToSuperview();
            if (self.layoutGuideHiddenBlankViewRight) make.bottom.lessThanOrEqualTo(self.layoutGuideHiddenBlankViewRight.mas_top).priorityHigh();
        }];
    } else if (self.axis == UILayoutConstraintAxisHorizontal && self.itemsContentView.fs_width > self.layoutGuideHiddenBlankViewCenter.fs_width) {
        [self.layoutGuideHiddenBlankViewCenter mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.centerX.fs_equalToSuperview().priorityMedium();
            if (self.layoutGuideHiddenBlankViewRight) make.right.lessThanOrEqualTo(self.layoutGuideHiddenBlankViewRight.mas_left).priorityHigh();
            make.top.bottom.fs_equalToSuperview();
        }];
    }
    [self adjustLayoutGuideHiddenBlankViewCenter];
}

#pragma mark -UIScrollViewDelegate
- (void)scrollViewDidScroll:(UIScrollView *)scrollView{
    [self adjustLayoutGuideHiddenBlankViewCenter];
}

#pragma mark -UIViewAdapter
- (void)layoutSubviewsFromView:(__kindof UIView *)view{
    if (self.layoutGuideHiddenBlankViewCenter == view) {
        if (!CGSizeEqualToSize(self.itemsContentView.contentSize, self.layoutGuideHiddenBlankViewCenter.fs_size)) {
            self.itemsContentView.contentSize = self.layoutGuideHiddenBlankViewCenter.fs_size;
            [self adjustLayoutGuideHiddenBlankViewCenter];
        }
    }else if (self.itemsContentView == view){
        [self checkGradientViewsHidden];
    }
}
@end
