/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import <Foundation/Foundation.h>
#import "FSMainToolbar+Private.h"

NS_ASSUME_NONNULL_BEGIN

@interface FSToolbarManager : NSObject<IPanelChangedListener>
- (instancetype)init NS_UNAVAILABLE;

@property (nonatomic, strong) FSMainTopbar *mainTopbar;
@property (nonatomic, strong, nullable) FSMainBottombar *mainBottombar;
@property (nonatomic, weak) UIExtensionsManager *extensionsManager;
@property (nonatomic, assign) BOOL isViewTagBeforeChanged;
@property (nonatomic, assign) BOOL rangeItemsed;
@property (nonatomic, assign) BOOL isShowReadBarItemDescript;
@property (nonatomic, strong) NSMutableSet *listeners;
- (void)registerTopbarEventListener:(id<IFSMainTopbarEventListener>)listener;
- (void)unregisterTopbarEventListener:(id<IFSMainTopbarEventListener>)listener;
- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager NS_DESIGNATED_INITIALIZER;

- (void)buildTopbar;
- (void)installTopbar;
- (void)buildBottombar;
- (void)installBottombar;
- (void)addMainItems;

- (void)removeTopbar;
- (void)removeBottombar;

- (FSMainToolbarItem *)addMainItemWithType:(FSMainToolbarItemType)itemType;
- (FSMainToolbarItem *)getMainItemWithType:(FSMainToolbarItemType)itemType;

- (void)addReadItem:(FSReadToolbarItem *)item atToolTag:(FSMainTopbarToolTag)toolTag;
- (void)removeReadItem:(FSReadToolbarItem *)item atToolTag:(FSMainTopbarToolTag)toolTag;
- (void)tagItemAddMainItem:(FSMainTopbarToolTagItem *)item;
- (void)rangeItems;
- (void)resetViewReadItems;
- (void)rangeViewTagItems;
- (void)rangeProtectTagItems;

- (void)moveToolbarToPosition:(FSTopbarSubitemContentViewPosition)position;
-(void)setToolbarItemHiddenWithTag:(FS_TOOLBAR_ITEM_TAG)hiddenItemTag hidden:(BOOL)isHidden;
- (void)registerEventListens;

@end

NS_ASSUME_NONNULL_END
