/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSToolbarManager.h"
#import "FSPagerView.h"
#import "MultipleSelectionToolHandler.h"
#import "UndoModule.h"
#import "MultipleSelectionModule.h"
#import "FSSettingBar+private.h"
#import "PageNavigationModule.h"
#import "FSUtilities.h"
#import "FSPermissionUtilty.h"

FSSYNTH_INTERFACE_IMP_CLASS(_FSPlaceholderToolbar, UIView)

@interface _FSPanToolbarMaskView : UIView
@property (nonatomic, weak) _FSPlaceholderToolbar *topBar;
@property (nonatomic, weak) _FSPlaceholderToolbar *leftBar;
@property (nonatomic, weak) _FSPlaceholderToolbar *rightBar;
@property (nonatomic, assign) FSTopbarSubitemContentViewPosition moveOnPosition;
- (instancetype)initWithToolbar:(FSMainTopbar *)toolbar;
- (void)moveToPosition:(FSTopbarSubitemContentViewPosition)position;
@end

@interface FSToolbarManager ()<IDocEventListener, IFSMainTopbarEventListener,IWindowFrameChangedListener>
@property (nonatomic, weak) _FSPanToolbarMaskView *panToolbarMaskView;
@property (nonatomic, strong) FSMainToolbarItem *panelListItem;

@property (nonatomic, strong) FSMainToolbarItem *addItem;
@property (nonatomic, strong) FSMainToolbarItem *sortItem;

@property (nonatomic, strong) NSMutableArray *homeReadItems;
@property (nonatomic, strong) NSMutableArray *editReadItems;
@property (nonatomic, strong) NSMutableArray *commentReadItems;
@property (nonatomic, strong) NSMutableArray *drawingReadItems;
@property (nonatomic, strong) NSMutableArray *viewReadItems;
@property (nonatomic, strong) NSMutableArray *protectItems;
@property (nonatomic, strong) NSMutableArray *formReadItems;
@property (nonatomic, strong) NSMutableArray <FSReadToolbarItem *>*fillSignReadItems;

@property (nonatomic, strong) NSMutableArray *homeMainItems;
@property (nonatomic, strong) NSMutableArray *formMainItems;
@property (nonatomic, strong) NSMutableDictionary *removedToolbarItems;
@property (nonatomic, strong) NSArray *topToolbarRightItems;
@property (nonatomic, strong) NSArray *iphoneBottombarcenterItems;
@property (nonatomic, strong) FSMainToolbarItem *thumbnailItem;
@property (nonatomic, strong) FSMainToolbarItem *bookmarkItem;
@property (nonatomic, strong) FSMainToolbarItem *viewSettingsItem;
@property (nonatomic, strong) FSMainToolbarItem *panItem;
@property (nonatomic, strong) FSMainTopbarToolTagItem *viewTagItem;
@property (nonatomic, strong) FSMainTopbarToolTagItem *protectTagItem;

@end

@implementation FSToolbarManager

- (instancetype)initWithUIExtensionsManager:(UIExtensionsManager *)extensionsManager{
    self = [super init];
    if (self) {
        _extensionsManager = extensionsManager;
        _homeReadItems = @[].mutableCopy;
        _editReadItems = @[].mutableCopy;
        _commentReadItems = @[].mutableCopy;
        _drawingReadItems = @[].mutableCopy;
        _viewReadItems = @[].mutableCopy;
        _formReadItems = @[].mutableCopy;
        _fillSignReadItems = @[].mutableCopy;
        
        _homeMainItems = @[].mutableCopy;
        _formMainItems = @[].mutableCopy;
        _protectItems = @[].mutableCopy;
        
        [_extensionsManager.pdfViewCtrl registerDocEventListener:self];
        
        _topToolbarRightItems = @[@(FSMainToolbarItemTypeMore),@(FSMainToolbarItemTypeSearch),@(FSMainToolbarItemTypeReadingBookmark),@(FSMainToolbarItemTypeThumbnail)];
        _iphoneBottombarcenterItems = @[@(FSMainToolbarItemTypePanel),@(FSMainToolbarItemTypeViewSettings),@(FSMainToolbarItemTypeThumbnail),@(FSMainToolbarItemTypeReadingBookmark)];
    }
    return self;
}

- (void)registerEventListens{
    [_extensionsManager registerWindowFrameChangedListener:self];
}

- (void)dealloc{
    [_extensionsManager.panelController unregisterPanelChangedListener:self];
    [_extensionsManager unregisterWindowFrameChangedListener:self];
}

- (FSToolbar *)mainMenuTopbar{
    return self.mainTopbar.mainMenuTopbar;
}

- (UIView *)subMenuTopbarContentView{
    return self.mainTopbar.subMenuTopbarContentView;
}

- (NSMutableSet *)listeners{
    if(!_listeners){
        _listeners = [NSMutableSet set];
    }
    return _listeners;
}

- (void)registerTopbarEventListener:(id<IFSMainTopbarEventListener>)listener{
    [self.listeners addObject:listener];
}

- (void)unregisterTopbarEventListener:(id<IFSMainTopbarEventListener>)listener{
    [self.listeners removeObject:listener];
}

#pragma mark - build main body and conmmon

- (void)buildTopbar{
    self.mainTopbar = [[FSMainTopbar alloc] initWithStyle:FSToolbarStyleTop extensionsManager:self.extensionsManager];
    self.mainTopbar.delegate = self;
    [self addMainTopBarTagItems];
    
    [self installTopbar];
}

- (void)installTopbar{
    [_extensionsManager.containerView addSubview:[self.mainTopbar getToolbarContentView]];

    [[self.mainTopbar getToolbarContentView] mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.top.right.fs_equalToSuperview();
    }];
}

- (void)buildBottombar{
    if (DEVICE_iPHONE || [_extensionsManager fs_currentSceneiPadHorizontalCompact]) {
        self.mainBottombar = [[FSMainBottombar alloc] initWithStyle:FSToolbarStyleBottom extensionsManager:self.extensionsManager];
        [self installBottombar];
        if ([_extensionsManager fs_currentSceneiPadHorizontalCompact]) {
            [self.mainBottombar setHidden:self.extensionsManager.hiddenBottomToolbar animated:NO];
        }
    }else{
        [self removeBottombar];
        self.mainBottombar = nil;
    }
}

- (void)installBottombar{
    if (DEVICE_iPHONE || [_extensionsManager fs_currentSceneiPadHorizontalCompact]) {
        if ([self.mainTopbar getToolbarContentView]) {
            [_extensionsManager.containerView insertSubview:[self.mainBottombar getToolbarContentView] aboveSubview:[self.mainTopbar getToolbarContentView]];
        }else{
            [_extensionsManager.containerView addSubview:[self.mainBottombar getToolbarContentView]];
        }
        
       
        [[self.mainBottombar getToolbarContentView] mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.bottom.right.fs_equalToSuperview();
        }];
    }
}

- (void)addMainItems{
    
    @weakify(self)
    FSMainToolbarItem *backItem  = [self addMainItemWithType:FSMainToolbarItemTypeBack];
    backItem.item.onTapClick = ^(FSItem * _Nonnull item) {
        @strongify(self)
        [self.extensionsManager onClickBackButton:item];
    };
    
    if (self.extensionsManager.config.loadReadingBookmark || self.extensionsManager.config.loadOutline || [self.extensionsManager.config.tools fs_containsAnyObjectNotInArray:@[ Tool_Select, Tool_Eraser ]] || self.extensionsManager.config.loadAttachment) {
        
        FSMainToolbarItem *panelListItem  = [self addMainItemWithType:FSMainToolbarItemTypePanel];
        [BindThemeKeySel(panelListItem.item, setImage:forState:) fs_updateThemeCompleted:^(FSItem * _Nonnull _item, SEL  _Nonnull sel) {
            @strongify(self)
            if(self.extensionsManager.fs_currentSceneIsCompact){
                [_item setImage:FSImageNamedAndNormalColor(@"read_left_pannel_gray") forState:UIControlStateNormal];
                [_item setImage:FSImageNamedAndNormalColor(@"read_left_pannel_gray") forState:UIControlStateSelected];

                [_item setBackgroundImage:nil forState:UIControlStateNormal];
                [_item setBackgroundImage:nil forState:UIControlStateSelected];
            }else{
                [_item setImage:FSImageNamedAndColor(@"read_panel_gray",self.fs_iconColor_I6) forState:UIControlStateNormal];
                [_item setImage:FSImageNamedAndColor(@"read_panel_gray",self.fs_textColor_T10) forState:UIControlStateSelected];
                [_item setBackgroundImage:FSImageNamedAndColor(@"common_select_backgroup_gray", self.fs_iconColor_I7) forState:UIControlStateSelected];
                [_item setBackgroundImage:nil forState:UIControlStateNormal];
            }

        }];

        [panelListItem.item addTarget:self action:@selector(didClickedMainItem:) forControlEvents:UIControlEventTouchUpInside];
        _panelListItem = panelListItem;
    }
    FSMainToolbarItem *viewSettingsItem = [self addMainItemWithType:FSMainToolbarItemTypeViewSettings];
    [viewSettingsItem.item addTarget:self action:@selector(didClickedMainItem:) forControlEvents:UIControlEventTouchUpInside];
    self.viewSettingsItem = viewSettingsItem;
}

- (void)removeTopbar{
    [[self.mainTopbar getToolbarContentView] removeFromSuperview];
}

- (void)removeBottombar{
    [[self.mainBottombar getToolbarContentView] removeFromSuperview];
}

- (void)didClickedMainItem:(FSItem *)mainItem {
    if (self.extensionsManager.currentAnnot) {
        [self.extensionsManager setCurrentAnnot:nil];
    }
    if ([self.extensionsManager.currentToolHandler isKindOfClass:[MultipleSelectionToolHandler class]]) {
        [self.extensionsManager setCurrentToolHandler:nil];
    }
    if (mainItem.tag == FSMainToolbarItemTypePanel){
        self.extensionsManager.panelController.isHidden = !self.extensionsManager.panelController.isHidden;
    }else if (mainItem.tag == FSMainToolbarItemTypeViewSettings){
        self.extensionsManager.hiddenSettingBar = NO;
    }
}

#pragma mark - tool bar common

- (void)addMainTopBarTagItems{
    [self addHomeTagItem];
    [self addEiditTagItem];
    [self addCommentTagItem];
    [self addDrawingTagItem];
    if (DEVICE_iPAD && ![_extensionsManager fs_currentSceneiPadHorizontalCompact]) {
        [self addViewTagItem];
    }
    [self addFormTagItem];
    [self addFillSignTagItem];
    if (DEVICE_iPAD) {
        [self addProtectTagItem];
    }
}

- (void)addHomeTagItem{
    NSMutableArray *readToolbarItems = @[].mutableCopy;
    
    NSMutableArray *tmp = self.mainTopbar.tagItems ? self.mainTopbar.tagItems.mutableCopy : @[].mutableCopy;
    FSMainTopbarToolTagItem *tagItem  = [FSMainTopbarToolTagItem ItemWithToolTag:FSMainTopbarToolTagHome readToolbarItems:readToolbarItems];
    tagItem.title = FSLocalizedForKey(@"kHome");
    tagItem.image = FSImageNamedAndNormalColor(@"read_home_gray");
    [tmp addObject:tagItem];
    self.mainTopbar.tagItems = tmp;
}

- (void)addEiditTagItem{
    
    NSMutableArray *readToolbarItems = @[].mutableCopy;
    
    NSMutableArray *tmp = self.mainTopbar.tagItems ? self.mainTopbar.tagItems.mutableCopy : @[].mutableCopy;
    FSMainTopbarToolTagItem *tagItem  = [FSMainTopbarToolTagItem ItemWithToolTag:FSMainTopbarToolTagEdit readToolbarItems:readToolbarItems];
    tagItem.title = FSLocalizedForKey(@"kEdit");
    tagItem.image = FSImageNamedAndNormalColor(@"read_edit_gary");
    [tmp addObject:tagItem];
    self.mainTopbar.tagItems = tmp;
}

- (void)addCommentTagItem{
    NSMutableArray *readToolbarItems = @[].mutableCopy;
    NSMutableArray *tmp = self.mainTopbar.tagItems ? self.mainTopbar.tagItems.mutableCopy : @[].mutableCopy;
    FSMainTopbarToolTagItem *tagItem  = [FSMainTopbarToolTagItem ItemWithToolTag:FSMainTopbarToolTagComment readToolbarItems:readToolbarItems];
    tagItem.title = FSLocalizedForKey(@"kComment");
    tagItem.image = FSImageNamedAndNormalColor(@"read_comment_gray");
    [tmp addObject:tagItem];
    self.mainTopbar.tagItems = tmp;
}

- (void)addDrawingTagItem{
    NSMutableArray *readToolbarItems = @[].mutableCopy;
    NSMutableArray *tmp = self.mainTopbar.tagItems ? self.mainTopbar.tagItems.mutableCopy : @[].mutableCopy;
    FSMainTopbarToolTagItem *tagItem  = [FSMainTopbarToolTagItem ItemWithToolTag:FSMainTopbarToolTagDrawing readToolbarItems:readToolbarItems];
    tagItem.title = FSLocalizedForKey(@"kDrawing");
    tagItem.image = FSImageNamedAndNormalColor(@"read_drawing_gray");
    [tmp addObject:tagItem];
    self.mainTopbar.tagItems = tmp;
}

- (void)addViewTagItem{
    NSInteger index = self.mainTopbar.tagItems.count;

    NSMutableArray *readToolbarItems = @[].mutableCopy;
    NSMutableArray *tagItems = self.mainTopbar.tagItems ? self.mainTopbar.tagItems.mutableCopy : @[].mutableCopy;
    if (!self.viewTagItem) {
        FSMainTopbarToolTagItem *tagItem  = [FSMainTopbarToolTagItem ItemWithToolTag:FSMainTopbarToolTagView readToolbarItems:readToolbarItems];
        tagItem.title = FSLocalizedForKey(@"kView");
        tagItem.image = FSImageNamedAndNormalColor(@"read_view_gray");
        self.viewTagItem = tagItem;
    }
    [tagItems insertObject:self.viewTagItem atIndex:index];
    self.mainTopbar.tagItems = tagItems;
}

- (void)addFormTagItem{
    NSMutableArray *readToolbarItems = @[].mutableCopy;
    NSMutableArray *tmp = self.mainTopbar.tagItems ? self.mainTopbar.tagItems.mutableCopy : @[].mutableCopy;
    FSMainTopbarToolTagItem *tagItem  = [FSMainTopbarToolTagItem ItemWithToolTag:FSMainTopbarToolTagForm readToolbarItems:readToolbarItems];;
    tagItem.title = FSLocalizedForKey(@"kForm");
    tagItem.image = FSImageNamedAndNormalColor(@"read_form_gray");
    [tmp addObject:tagItem];
    self.mainTopbar.tagItems = tmp;
}

- (void)addFillSignTagItem{
    NSMutableArray *readToolbarItems = @[].mutableCopy;
    NSMutableArray *tmp = self.mainTopbar.tagItems ? self.mainTopbar.tagItems.mutableCopy : @[].mutableCopy;
    FSMainTopbarToolTagItem *tagItem  = [FSMainTopbarToolTagItem ItemWithToolTag:FSMainTopbarToolTagSign readToolbarItems:readToolbarItems];
    tagItem.title = FSLocalizedForKey(@"kFill&Sign");
    tagItem.image = FSImageNamedAndNormalColor(@"read_sign_gray");
    [tmp addObject:tagItem];
    self.mainTopbar.tagItems = tmp;
}

- (void)addProtectTagItem{
    NSInteger index = self.mainTopbar.tagItems.count;

    NSMutableArray *readToolbarItems = @[].mutableCopy;
    NSMutableArray *tagItems = self.mainTopbar.tagItems ? self.mainTopbar.tagItems.mutableCopy : @[].mutableCopy;
    if (!self.protectTagItem) {
        FSMainTopbarToolTagItem *tagItem  = [FSMainTopbarToolTagItem ItemWithToolTag:FSMainTopbarToolTagProtect readToolbarItems:readToolbarItems];
        tagItem.title = FSLocalizedForKey(@"kProtect");
        tagItem.image = FSImageNamedAndNormalColor(@"more_password_file_encryption");
        self.protectTagItem = tagItem;
    }
    [tagItems insertObject:self.protectTagItem atIndex:index];
    self.mainTopbar.tagItems = tagItems;
}

- (NSArray <FSReadToolbarItem *> *)defaultOrderReadToolItemsForToolTag:(FSMainTopbarToolTag)toolTag{
    NSArray *orderReadToolTagTypes;
    NSMutableArray *sortItmes = @[].mutableCopy;
    NSMutableArray *tmp;
    switch (toolTag) {
        case FSMainTopbarToolTagHome:
            orderReadToolTagTypes = @[@(FSReadToolSettingsTypeNote), @(FSReadToolSettingsTypeHighlight), @(FSReadToolSettingsTypeTypewriter), @(FSReadToolSettingsTypePencil), @(FSReadToolSettingsTypeEraser), @(FSReadToolSettingsTypeText), @(FSReadToolSettingsTypeImage)];
            tmp = _homeReadItems.mutableCopy;
            break;
        case FSMainTopbarToolTagEdit:
            orderReadToolTagTypes =  @[@(FSReadToolSettingsTypeAudio), @(FSReadToolSettingsTypeVideo), @(FSReadToolSettingsTypeLink)];
            tmp = _editReadItems.mutableCopy;
            break;
        case FSMainTopbarToolTagComment:
            orderReadToolTagTypes =  @[@(FSReadToolSettingsTypeNote), @(FSReadToolSettingsTypeAttachments), @(FSReadToolSettingsTypeStamp), @(FSReadToolSettingsTypeTypewriter), @(FSReadToolSettingsTypeCallout),@(FSReadToolSettingsTypeTextbox), @(FSReadToolSettingsTypeHighlight), @(FSReadToolSettingsTypeUnderline), @(FSReadToolSettingsTypeSquiggly), @(FSReadToolSettingsTypeStrikeout), @(FSReadToolSettingsTypeReplaceText), @(FSReadToolSettingsTypeInsertText), @(FSReadToolSettingsTypeAreaHighlight), @(FSReadToolSettingsTypeMeasure), @(FSReadToolSettingsTypeImageAnnotation)];
            tmp = _commentReadItems.mutableCopy;
            break;
        case FSMainTopbarToolTagDrawing:
            orderReadToolTagTypes = @[@(FSReadToolSettingsTypePencil), @(FSReadToolSettingsTypeHighlighter), @(FSReadToolSettingsTypeEraser), @(FSReadToolSettingsTypeRectangle), @(FSReadToolSettingsTypeOval), @(FSReadToolSettingsTypeLine), @(FSReadToolSettingsTypeArrow), @(FSReadToolSettingsTypePolyLine), @(FSReadToolSettingsTypePolygon), @(FSReadToolSettingsTypeCloud)];
            tmp = _drawingReadItems.mutableCopy;
            break;
        case FSMainTopbarToolTagView:
            tmp = _viewReadItems.mutableCopy;
            break;
        case FSMainTopbarToolTagForm:
            tmp = _formReadItems.mutableCopy;
            break;
        case FSMainTopbarToolTagSign:
        {
            FSReadToolbarItem *firstItem = [_fillSignReadItems firstObject];
            if (firstItem.settingsType == FSReadToolSettingsTypeDigitalSignature) {
                [_fillSignReadItems removeObject:firstItem];
                [_fillSignReadItems addObject:firstItem];
            }
            tmp = _fillSignReadItems.mutableCopy;
        }
            break;
        case FSMainTopbarToolTagProtect:
            orderReadToolTagTypes = @[@(FSReadToolSettingsTypeRedact), @(FSReadToolSettingsTypePassWord), @(FSReadToolSettingsTypeTrutedCertificates)];
            tmp = _protectItems.mutableCopy;
            break;
        default:
            break;
    }
    for (NSNumber *itemType in orderReadToolTagTypes) {
        for (FSReadToolbarItem *item in tmp) {
            if (itemType.integerValue == item.settingsType) {
                [sortItmes addObject:item];
                [tmp removeObject:item];
                break;
            }
        }
    }
    if (tmp.count) {
        [sortItmes addObjectsFromArray:tmp];
    }
    return sortItmes.count ? sortItmes : nil;
}

- (void)panSubMenuTopbarWithRecognizer:(UIPanGestureRecognizer *)recognizer{
    CGPoint point = [recognizer locationInView:_panToolbarMaskView];
    if (recognizer.state == UIGestureRecognizerStateBegan) {
        [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingToolbarDrag];
        
        _FSPanToolbarMaskView *panToolbarMaskView = [[_FSPanToolbarMaskView alloc] initWithToolbar:self.mainTopbar];
        [panToolbarMaskView moveToPosition:self.mainTopbar.subitemContentPosition];
        [_extensionsManager.containerView addSubview:panToolbarMaskView];
        [panToolbarMaskView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.right.top.bottom.fs_equalToSuperview();
        }];
        [panToolbarMaskView layoutIfNeeded];
        self.panToolbarMaskView = panToolbarMaskView;
        
        CGRect frame = [self.subMenuTopbarContentView convertRect:self.subMenuTopbarContentView.bounds toView:_extensionsManager.containerView];

        self.subMenuTopbarContentView.frame = frame;
        [panToolbarMaskView addSubview:self.subMenuTopbarContentView];
        [recognizer setTranslation:CGPointZero inView:panToolbarMaskView];
        
        [self.subMenuTopbarContentView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.left.fs_equalToSuperview().offset(frame.origin.x);
            make.top.fs_equalToSuperview().offset(frame.origin.y);
            make.size.mas_equalTo(frame.size);
        }];
        
        [_extensionsManager cancelAutoFullScreen];
        
    }else if (recognizer.state == UIGestureRecognizerStateChanged) {
        if (self.mainTopbar.subitemContentPosition == FSTopbarSubitemContentViewPositionTop) {
            self.subMenuTopbarContentView.fs_top = point.y - recognizer.view.fs_height / 2;
            self.subMenuTopbarContentView.fs_right = point.x + recognizer.view.fs_width / 2;
        }else{
            self.subMenuTopbarContentView.fs_bottom = point.y + recognizer.view.fs_height / 2;
            self.subMenuTopbarContentView.fs_right = point.x + recognizer.view.fs_width / 2;
        }
        
        if (point.y <= self.mainTopbar.toolbar.fs_height ||
            CGRectIntersectsRect(self.subMenuTopbarContentView.frame, _panToolbarMaskView.topBar.frame)) {
            [_panToolbarMaskView moveToPosition:FSTopbarSubitemContentViewPositionTop];
        }else if (point.x > _panToolbarMaskView.fs_width/2){
            [_panToolbarMaskView moveToPosition:FSTopbarSubitemContentViewPositionRight];
        }else{
            [_panToolbarMaskView moveToPosition:FSTopbarSubitemContentViewPositionLeft];
        }
        
    }else{
        self.mainTopbar.subitemContentPosition = _panToolbarMaskView.moveOnPosition;
        [_panToolbarMaskView removeFromSuperview];
        
        [_extensionsManager delayAutoFullScreen];
    }
}

- (void)moveToolbarToPosition:(FSTopbarSubitemContentViewPosition)position{
    if (!self.extensionsManager.panelController.isHidden && self.mainTopbar.subitemContentPosition == FSTopbarSubitemContentViewPositionLeft) {
        position = FSTopbarSubitemContentViewPositionRight;
    }
    if (position == FSTopbarSubitemContentViewPositionTop) {
        self.subMenuTopbarContentView.layer.cornerRadius = 0;
        self.subMenuTopbarContentView.clipsToBounds = NO;
        [self.mainTopbar.toolbar addSubview:self.subMenuTopbarContentView];
        
        for (FSMainTopbarToolTagItem *tagItem in self.mainTopbar.tagItems) {
            if (tagItem.subTagToolbar) {
                for (FSToolbarItem *item in tagItem.subTagToolbar.centerItems.items) {
                    item.margin = ITEM_BOTH_SPACE_NORMAL;   
                }
            }
            for (FSReadToolbarItem *item in tagItem.readToolbarItems) {
                if ((item.itemContentType & FSItemContentTypeText) && self.extensionsManager.toolbarManager.isShowReadBarItemDescript){
                    [item.toolbarItem.item setTitle:item.title forState:UIControlStateNormal];
                    [item.toolbarItem.item setTitle:item.selectTitle forState:UIControlStateSelected];
                }
            }
        }

        self.mainTopbar.axis = UILayoutConstraintAxisHorizontal;
        
        if (self.mainTopbar.currentTagItem.toolTag == FSMainTopbarToolTagHome) {
            if (self.mainTopbar.currentTagItem.readToolbarItems.count <= 1) {
                [self.addItem.item setTitle:FSLocalizedForKey(@"kAddTools") forState:UIControlStateNormal];
            }else{
                [self.addItem.item setTitle:nil forState:UIControlStateNormal];
            }
        }
        
        [self.subMenuTopbarContentView mas_remakeConstraints:^(MASConstraintMaker *make) {
            make.bottom.left.right.fs_equalToSuperview();
            make.top.mas_equalTo(self.mainMenuTopbar.mas_bottom);
        }];
        
        int flag = 0;
        for (FSMainToolbarItem *mainItem in self.mainTopbar.currentTagItem.mainToolbarItems) {
            if (flag > 1) break;
            if (mainItem.itemType == FSMainToolbarItemTypePan || mainItem.itemType == FSMainToolbarItemTypeDivide) {
                mainItem.item.transform = CGAffineTransformIdentity;
                flag++;
            }
        }
        
    }else{
        self.subMenuTopbarContentView.layer.cornerRadius = 4;
        self.subMenuTopbarContentView.clipsToBounds = YES;
        [_extensionsManager.containerView addSubview:self.subMenuTopbarContentView];
        
        for (FSMainTopbarToolTagItem *tagItem in self.mainTopbar.tagItems) {
            if (tagItem.subTagToolbar) {
                for (FSToolbarItem *item in tagItem.subTagToolbar.centerItems.items) {
                    item.margin = 0;
                }
            }
            for (FSReadToolbarItem *item in tagItem.readToolbarItems) {
                if (item.itemContentType & FSItemContentTypeText){
                    [item.toolbarItem.item setTitle:nil forState:UIControlStateNormal];
                    [item.toolbarItem.item setTitle:nil forState:UIControlStateSelected];
                }
            }
        }
        self.mainTopbar.axis = UILayoutConstraintAxisVertical;
        
        int flag = 0;
        for (FSMainToolbarItem *mainItem in self.mainTopbar.currentTagItem.mainToolbarItems) {
            if (flag > 1) break;
            if (mainItem.itemType == FSMainToolbarItemTypePan || mainItem.itemType == FSMainToolbarItemTypeDivide) {
                mainItem.item.transform = CGAffineTransformMakeRotation(M_PI_2);
                flag++;
            }
        }

        if (![self.mainTopbar.currentTagItem.mainToolbarItems containsObject:self.addItem]) [self.addItem.item setTitle:nil forState:UIControlStateNormal];
        
        [self.subMenuTopbarContentView mas_remakeConstraints:^(MASConstraintMaker *make) {
            if (position == FSTopbarSubitemContentViewPositionLeft) {
                make.left.mas_equalTo(self.subMenuTopbarContentView.superview.fs_mas_left).offset(ITEM_LEFT_MARGIN_NORMAL);
            }else{
                make.right.mas_equalTo(self.subMenuTopbarContentView.superview.fs_mas_right).offset(-ITEM_LEFT_MARGIN_NORMAL);
            }
            if (DEVICE_iPAD) {
                make.centerY.fs_equalToSuperview().priorityHigh();
            }else{
                make.top.mas_equalTo(self.mainMenuTopbar.mas_bottom).offset(50.f).priorityMedium();
                make.top.greaterThanOrEqualTo(self.mainMenuTopbar.mas_bottom).offset(50.f).priorityHigh();
            }

            make.width.offset(self.mainTopbar.verticalContentWidth);
            make.height.mas_equalTo(self.subMenuTopbarContentView.superview.mas_height).multipliedBy(0.64).priorityMedium();
            make.height.mas_greaterThanOrEqualTo(self.subMenuTopbarContentView.superview.mas_height).multipliedBy(0.55);
            make.bottom.mas_lessThanOrEqualTo(self.subMenuTopbarContentView.superview.fs_mas_bottom).offset(-88);
        }];
    }
    
    if (DEVICE_iPAD) {
        for (FSMainToolbarItem *mainItem in self.mainTopbar.currentTagItem.mainToolbarItems) {
            if (mainItem.itemType == FSMainToolbarItemTypeProperty) {
                if (position == FSTopbarSubitemContentViewPositionRight) {
                    [mainItem.item setBackgroundImage:FSImageNamedAndNormalColor(@"common_unflod_backgroup_left") forState:UIControlStateNormal];
                    [mainItem.item setBackgroundImage:[FSImageNamedAndNormalColor(@"common_unflod_backgroup_left") fs_applyingAlpha:IMAGE_DISABLED_ALPHA] forState:UIControlStateDisabled];
                }else{
                    [mainItem.item setBackgroundImage:FSImageNamedAndNormalColor(@"common_unflod_backgroup") forState:UIControlStateNormal];
                    [mainItem.item setBackgroundImage:[FSImageNamedAndNormalColor(@"common_unflod_backgroup") fs_applyingAlpha:IMAGE_DISABLED_ALPHA] forState:UIControlStateDisabled];
                }
            }
        }
        
        if (self.mainTopbar.currentTagItem.toolTag == FSMainTopbarToolTagView) {
            UIImage *backColorImage = [UIImage fs_imageWithColor:self.fs_dividerColor_P2  size:CGSizeMake(32.f, 32.f)];
            if (position == FSTopbarSubitemContentViewPositionRight) {
                [self.extensionsManager.settingBar.viewSettingManager.singlePageItem setImageViewBackgroundImage:FSImageNamedAndNormalColor(@"common_unflod_backgroup_left") forState:UIControlStateNormal];
                [self.extensionsManager.settingBar.viewSettingManager.colorModeItem setImageViewBackgroundImage:FSImageNamedAndNormalColor(@"common_unflod_backgroup_left") forState:UIControlStateNormal];
            }else{
                [self.extensionsManager.settingBar.viewSettingManager.singlePageItem setImageViewBackgroundImage:FSImageNamedAndNormalColor(@"common_unflod_backgroup") forState:UIControlStateNormal];
                [self.extensionsManager.settingBar.viewSettingManager.colorModeItem setImageViewBackgroundImage:FSImageNamedAndNormalColor(@"common_unflod_backgroup") forState:UIControlStateNormal];
            }
        }
    }
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.1 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
        [(PageNavigationModule *)[self.extensionsManager getModuleByName:@"PageNavigation"] layoutPdfViewContentInsets:NO];
    });
    
}

- (FSMainToolbarItem *)addMainItemWithType:(FSMainToolbarItemType)itemType{
    FSMainToolbarItem *mainItem = [[FSMainToolbarItem alloc] initWithExtensionsManager:self.extensionsManager itemType:itemType];
    mainItem.item.tag = itemType;
    if (((DEVICE_iPHONE || [_extensionsManager fs_currentSceneiPadHorizontalCompact])&& (itemType == FSMainToolbarItemTypePanel || itemType == FSMainToolbarItemTypeThumbnail || itemType == FSMainToolbarItemTypeReadingBookmark)) || itemType == FSMainToolbarItemTypeViewSettings) {
        NSMutableArray *itemsForPositionCenter = self.mainBottombar.itemsForPositionCenter.mutableCopy;
        if (itemType == FSMainToolbarItemTypeThumbnail && itemsForPositionCenter.count >=3) {
            [itemsForPositionCenter insertObject:mainItem atIndex:2];
        }else{
            [itemsForPositionCenter addObject:mainItem];
        }
        
        self.mainBottombar.itemsForPositionCenter = itemsForPositionCenter;
    }else{
        if (itemType == FSMainToolbarItemTypeBack) {
            [self.mainTopbar addItem:mainItem atPosition:FSMainToolbarItemPositionLeft];
        }else if (itemType == FSMainToolbarItemTypePanel){
            [self.mainTopbar addItem:mainItem atPosition:FSMainToolbarItemPositionLeft];
        }else{
            [self.mainTopbar addItem:mainItem atPosition:FSMainToolbarItemPositionRight];
        }
    }
    
    if (itemType == FSMainToolbarItemTypeThumbnail) {
        self.thumbnailItem = mainItem;
    }else if (itemType == FSMainToolbarItemTypeReadingBookmark) {
        self.bookmarkItem = mainItem;
    }
    
    return mainItem;
}

///MARK:It's only sorted the first time
- (void)addReadItem:(FSReadToolbarItem *)item atToolTag:(FSMainTopbarToolTag)toolTag{
    if (item) {
        if (toolTag == FSMainTopbarToolTagHome) {
            if (![_homeReadItems containsObject:item]) {
                [_homeReadItems addObject:item];
            }
        }else if (toolTag == FSMainTopbarToolTagComment){
            if (![_commentReadItems containsObject:item]) {
                [_commentReadItems addObject:item];
            }
        }else if (toolTag == FSMainTopbarToolTagDrawing){
            if (![_drawingReadItems containsObject:item]) {
                [_drawingReadItems addObject:item];
            }
        }else if (toolTag == FSMainTopbarToolTagForm){
            if (![_formReadItems containsObject:item]) {
                [_formReadItems addObject:item];
            }
        }else if (toolTag == FSMainTopbarToolTagSign){
            if (![_fillSignReadItems containsObject:item]) {
                [_fillSignReadItems addObject:item];
            }
        }else if (toolTag == FSMainTopbarToolTagView){
            if (![_viewReadItems containsObject:item]) {
                [_viewReadItems addObject:item];
            }
        }else if (toolTag == FSMainTopbarToolTagEdit){
            if (![_editReadItems containsObject:item]) {
                [_editReadItems addObject:item];
            }
        }else if (toolTag == FSMainTopbarToolTagProtect){
            if (![_protectItems containsObject:item]) {
                [_protectItems addObject:item];
            }
        }
    }
}

- (void)removeReadItem:(FSReadToolbarItem *)item atToolTag:(FSMainTopbarToolTag)toolTag{
    if (item) {
        if (toolTag == FSMainTopbarToolTagHome) {
            if ([_homeReadItems containsObject:item]) {
                [_homeReadItems removeObject:item];
            }
        }else if (toolTag == FSMainTopbarToolTagComment){
            if ([_commentReadItems containsObject:item]) {
                [_commentReadItems removeObject:item];
            }
        }else if (toolTag == FSMainTopbarToolTagDrawing){
            if ([_drawingReadItems containsObject:item]) {
                [_drawingReadItems removeObject:item];
            }
        }else if (toolTag == FSMainTopbarToolTagForm){
            if ([_formReadItems containsObject:item]) {
                [_formReadItems removeObject:item];
            }
        }else if (toolTag == FSMainTopbarToolTagSign){
            if ([_fillSignReadItems containsObject:item]) {
                [_fillSignReadItems removeObject:item];
            }
        }else if (toolTag == FSMainTopbarToolTagView){
            if ([_viewReadItems containsObject:item]) {
                [_viewReadItems removeObject:item];
            }
        }else if (toolTag == FSMainTopbarToolTagEdit){
            if ([_editReadItems containsObject:item]) {
                [_editReadItems removeObject:item];
            }
        }else if (toolTag == FSMainTopbarToolTagProtect){
            if ([_protectItems containsObject:item]) {
                [_protectItems removeObject:item];
            }
        }
    }
}

- (void)tagItemAddMainItem:(FSMainTopbarToolTagItem *)item{
    if (item.toolTag < FSMainTopbarToolTagCustom ){
        item.readToolbarItems = [self defaultOrderReadToolItemsForToolTag:item.toolTag].copy;
    }
    
    if ((self.extensionsManager.config.defaultSettings.enableTopbarDraggable == 1 && !(DEVICE_iPAD && ![_extensionsManager fs_currentSceneiPadHorizontalCompact])) ||
        (self.extensionsManager.config.defaultSettings.enableTopbarDraggable == 2 && (DEVICE_iPAD && ![_extensionsManager fs_currentSceneiPadHorizontalCompact])) ||
        self.extensionsManager.config.defaultSettings.enableTopbarDraggable == 3) {
        [self addPanItemToTagItem:item];
    }
    
    if (item.toolTag == FSMainTopbarToolTagHome){
        [self addAddItemToTagItem:item];
    }else{
        [self addSortItemToTagItem:item];
    }
    
    if (item.needUndoMainItem) {
        [self addUndoRedoItemToTagItem:item];
    }

    if (item.toolTag == FSMainTopbarToolTagHome ||
        item.toolTag == FSMainTopbarToolTagComment ||
        item.toolTag == FSMainTopbarToolTagDrawing) {
        [self addPropertyToTagItem:item];

        if ([self.extensionsManager.config.tools containsObject:Tool_Multiple_Selection]) {
            [self addMultiSelectToTagItem:item];
        }
    }
    
    if (item.toolTag != FSMainTopbarToolTagView && item.toolTag != FSMainTopbarToolTagProtect){
        [self addDividerLineToTagItem:item];
    }
    item.addedMainItem = YES;
}

- (void)tagItemAddMainItems{
    for (FSMainTopbarToolTagItem *item in self.mainTopbar.tagItems) {
        [self tagItemAddMainItem:item];
    }
}

- (void)rangeItems{
    [self tagItemAddMainItems];
    [self.mainTopbar resetCurrentTagItem];
    
    NSArray *rightItemSeq;
    if (DEVICE_iPAD && ![_extensionsManager fs_currentSceneiPadHorizontalCompact]) {
        rightItemSeq = @[@(FSMainToolbarItemTypeMore), @(FSMainToolbarItemTypeSearch), @(FSMainToolbarItemTypeReadingBookmark), @(FSMainToolbarItemTypeThumbnail)];
    }else{
        rightItemSeq = @[@(FSMainToolbarItemTypeMore), @(FSMainToolbarItemTypeSearch)];
    }
    NSMutableArray *itemsForPositionRight = @[].mutableCopy;
    NSMutableArray *tmp = self.mainTopbar.itemsForPositionRight.mutableCopy;
    for (NSNumber *itemType in rightItemSeq) {
        for (FSMainToolbarItem *rightMainItem in tmp) {
            if (itemType.integerValue == rightMainItem.itemType) {
                [tmp removeObject:rightMainItem];
                [itemsForPositionRight addObject:rightMainItem];
                break;
            }
        }
    }
    if (tmp.count) {
        [itemsForPositionRight addObjectsFromArray:tmp];
    }
    self.mainTopbar.itemsForPositionRight = itemsForPositionRight;
    self.rangeItemsed = YES;
}

- (void)resetViewReadItems
{
    NSMutableArray *tempReadToolbarItems = @[].mutableCopy;
    for (FSMainTopbarToolTagItem *item in self.mainTopbar.tagItems) {
        if (item.toolTag == FSMainTopbarToolTagView) {
            if(item.readToolbarItems.count > 1 && item.readToolbarItems.count-1 == _viewReadItems.count){
                for (int i=0;i < item.readToolbarItems.count- 1;i++){
                    for (FSReadToolbarItem *viewItem in _viewReadItems) {
                        if(item.readToolbarItems[i].settingsType == viewItem.settingsType){
                            [tempReadToolbarItems addObject:viewItem];
                            break;
                        }
                    }
                }
            }else{
                tempReadToolbarItems = _viewReadItems;
            }
            item.readToolbarItems = tempReadToolbarItems;
            break;
        }
    }
   
}

//- (void)rangeViewTagItems{
//    for (FSMainTopbarToolTagItem *item in self.mainTopbar.tagItems) {
//        if (item.toolTag == FSMainTopbarToolTagView) {
//            item.readToolbarItems = [self defaultOrderReadToolItemsForToolTag:item.toolTag].copy;
//            break;
//        }
//
//    }
//}

- (void)rangeProtectTagItems
{
    NSMutableArray *tempReadToolbarItems = @[].mutableCopy;
    NSMutableArray *defaultReadToolbarItems = @[].mutableCopy;
//    _protectItems
    for (FSMainTopbarToolTagItem *item in self.mainTopbar.tagItems) {
        if (item.toolTag == FSMainTopbarToolTagProtect) {
            defaultReadToolbarItems = [self defaultOrderReadToolItemsForToolTag:item.toolTag].copy;
            if(item.readToolbarItems.count > 1 && item.readToolbarItems.count-1 == defaultReadToolbarItems.count){
                for (int i=0;i < item.readToolbarItems.count- 1;i++){
                    for (FSReadToolbarItem *protectItem in defaultReadToolbarItems) {
                        if(item.readToolbarItems[i].settingsType == protectItem.settingsType){
                            [tempReadToolbarItems addObject:protectItem];
                            break;
                        }
                    }
                }
            }else{
                tempReadToolbarItems = defaultReadToolbarItems;
            }
            item.readToolbarItems = tempReadToolbarItems;
            break;
        }

    }
}

- (void)addPanItemToTagItem:(FSMainTopbarToolTagItem *)tagItem{
    [self insertPanItemToTagItem:tagItem];
}

- (void)insertPanItemToTagItem:(FSMainTopbarToolTagItem *)tagItem{
    NSMutableArray *tmp = tagItem.mainToolbarItems ? tagItem.mainToolbarItems.mutableCopy : @[].mutableCopy;
    FSMainToolbarItem *panItem = [[FSMainToolbarItem alloc] initWithExtensionsManager:self.extensionsManager itemType:FSMainToolbarItemTypePan];
    [tmp insertObject:panItem atIndex:0];
    self.panItem = panItem;
    tagItem.mainToolbarItems = tmp;
    
    UIPanGestureRecognizer *pan = [[UIPanGestureRecognizer alloc] initWithTarget:self action:@selector(panSubMenuTopbarWithRecognizer:)];
    [panItem.item addGestureRecognizer:pan];
}

- (void)addUndoRedoItemToTagItem:(FSMainTopbarToolTagItem *)tagItem{
    NSInteger index = tagItem.mainToolbarItems.count;
    [self insertUndoRedoItemToTagItem:tagItem atIndex:index];
}

- (void)insertUndoRedoItemToTagItem:(FSMainTopbarToolTagItem *)tagItem atIndex:(NSUInteger)index{
    UndoModule *undoModule = nil;
    for (id module in self.extensionsManager.modules) {
        if ([module isKindOfClass:[UndoModule class]]) {
            undoModule = module;
            break;
        }
    }
    
    if (undoModule) {
        NSMutableArray *tmp = tagItem.mainToolbarItems ? tagItem.mainToolbarItems.mutableCopy : @[].mutableCopy;
        if (DEVICE_iPAD && ![_extensionsManager fs_currentSceneiPadHorizontalCompact]) {
            BOOL hasRedoItem = NO;
            for (FSMainToolbarItem *item in tmp) {
                if (item.itemType == FSMainToolbarItemTypeRedo) {
                    hasRedoItem = YES;
                    break;
                }
            }
            if (!hasRedoItem) {
                FSMainToolbarItem *redoItem = [[FSMainToolbarItem alloc] initWithExtensionsManager:self.extensionsManager itemType:FSMainToolbarItemTypeRedo];
                [tmp insertObject:redoItem atIndex:index];
                [undoModule addRedoItem:redoItem];
                index++;
            }
        }
        
        BOOL hasUndoItem = NO;
        for (FSMainToolbarItem *item in tmp) {
            if (item.itemType == FSMainToolbarItemTypeUndo) {
                hasUndoItem = YES;
                break;
            }
        }
        if (!hasUndoItem) {
            FSMainToolbarItem *undoItem = [[FSMainToolbarItem alloc] initWithExtensionsManager:self.extensionsManager itemType:FSMainToolbarItemTypeUndo];
            if (DEVICE_iPHONE || [_extensionsManager fs_currentSceneiPadHorizontalCompact]) {
                [undoItem.item setBackgroundImage:FSImageNamedAndNormalColor(@"common_unflod_backgroup") forState:FSItemControlStateAll];
                [undoItem.item setBackgroundImage:[FSImageNamedAndNormalColor(@"common_unflod_backgroup") fs_applyingAlpha:IMAGE_DISABLED_ALPHA] forState:UIControlStateDisabled];
            }
            [tmp insertObject:undoItem atIndex:index];
            [undoModule addUndoItem:undoItem];
        }
        tagItem.mainToolbarItems = tmp;
    }
}

- (void)addPropertyToTagItem:(FSMainTopbarToolTagItem *)tagItem{
    NSMutableArray *tmp = tagItem.mainToolbarItems ? tagItem.mainToolbarItems.mutableCopy : @[].mutableCopy;
    FSMainToolbarItem *propertyItem = [[FSMainToolbarItem alloc] initWithExtensionsManager:self.extensionsManager itemType:FSMainToolbarItemTypeProperty];
    [tmp addObject:propertyItem];
    tagItem.mainToolbarItems = tmp;
}

- (void)addMultiSelectToTagItem:(FSMainTopbarToolTagItem *)tagItem{
    MultipleSelectionModule *multipleSelectionModule = [self.extensionsManager getModuleByName:Module_Multiple_Selection];
    if (multipleSelectionModule) {
        if ([_extensionsManager fs_currentSceneiPadHorizontalCompact] || DEVICE_iPHONE) {
            NSMutableArray *tmp = tagItem.readToolbarItems ? tagItem.readToolbarItems.mutableCopy : @[].mutableCopy;
            FSMainToolbarItem *multipleSelectionItem = [[FSMainToolbarItem alloc] initWithExtensionsManager:self.extensionsManager itemType:FSMainToolbarItemTypeMultiSelect];
            [multipleSelectionModule addMultipleSelectionItem:multipleSelectionItem];
            [tmp insertObject:[[FSReadToolbarItem alloc] initWithCustomItem:multipleSelectionItem] atIndex:0];
            tagItem.readToolbarItems = tmp;
        }else{
            NSMutableArray *tmp = tagItem.mainToolbarItems ? tagItem.mainToolbarItems.mutableCopy : @[].mutableCopy;
            FSMainToolbarItem *multipleSelectionItem = [[FSMainToolbarItem alloc] initWithExtensionsManager:self.extensionsManager itemType:FSMainToolbarItemTypeMultiSelect];
            [multipleSelectionModule addMultipleSelectionItem:multipleSelectionItem];
            [tmp addObject:multipleSelectionItem];
            tagItem.mainToolbarItems = tmp;
        }
    }
}

- (void)addDividerLineToTagItem:(FSMainTopbarToolTagItem *)tagItem{
    NSMutableArray *tmp = tagItem.mainToolbarItems ? tagItem.mainToolbarItems.mutableCopy : @[].mutableCopy;
    FSMainToolbarItem *divideItem = [[FSMainToolbarItem alloc] initWithExtensionsManager:self.extensionsManager itemType:FSMainToolbarItemTypeDivide];
    [tmp addObject:divideItem];
    tagItem.mainToolbarItems = tmp;
}

- (FSMainToolbarItem *)getMainItemWithType:(FSMainToolbarItemType)itemType{
    NSArray *targetArray = nil;
    switch (itemType) {
        case FSMainToolbarItemTypeProperty:
            if (!self.mainTopbar.hidden) {
                targetArray = self.mainTopbar.currentTagItem.mainToolbarItems;
            }else if (!self.extensionsManager.fullScreenToolbarMgr.floatyToolbar.hidden){
                FSMainTopbarToolTagItem *homeTagItem = nil;
                 for (FSMainTopbarToolTagItem *tagItem in self.mainTopbar.tagItems) {
                     if (tagItem.toolTag == FSMainTopbarToolTagHome) {
                         homeTagItem = tagItem;
                         break;
                     }
                 }
                 targetArray = homeTagItem.mainToolbarItems;
            }
            break;
        case FSMainToolbarItemTypeBack:
            targetArray = self.mainTopbar.itemsForPositionLeft;
            break;
        case FSMainToolbarItemTypePanel:
            if (DEVICE_iPHONE || [_extensionsManager fs_currentSceneiPadHorizontalCompact]) {
                targetArray = self.mainBottombar.itemsForPositionCenter;
            }else{
                targetArray = self.mainTopbar.itemsForPositionLeft;
            }
            break;
        case FSMainToolbarItemTypeSearch:
        case FSMainToolbarItemTypeMore:
            targetArray = self.mainTopbar.itemsForPositionRight;
            break;
        case FSMainToolbarItemTypeThumbnail:
        case FSMainToolbarItemTypeReadingBookmark:
        case FSMainToolbarItemTypeViewSettings:
            if (DEVICE_iPHONE || [_extensionsManager fs_currentSceneiPadHorizontalCompact]) {
                targetArray = self.mainBottombar.itemsForPositionCenter;
            }else{
                targetArray = self.mainTopbar.itemsForPositionRight;
            }
            break;
        default:
            break;
    }
    for (FSMainToolbarItem *mainItem in targetArray) {
        if (mainItem.itemType == itemType) {
            return mainItem;
        }
    }
    return nil;
}

-(void)setToolbarItemHiddenWithTag:(FS_TOOLBAR_ITEM_TAG)hiddenItemTag hidden:(BOOL)isHidden{
    
    if (hiddenItemTag >= FS_TOOLBAR_ITEM_TAG_BACK && hiddenItemTag <= FS_TOOLBAR_ITEM_TAG_READING_BOOKMARK) {
        FSMainToolbarItemType itemType = FSMainToolbarItemTypeUnknown;
        if (hiddenItemTag == FS_TOOLBAR_ITEM_TAG_BACK) {
            itemType = FSMainToolbarItemTypeBack;
        }else if (hiddenItemTag == FS_TOOLBAR_ITEM_TAG_MORE) {
            itemType = FSMainToolbarItemTypeMore;
        }else if (hiddenItemTag == FS_TOOLBAR_ITEM_TAG_SEARCH) {
            itemType = FSMainToolbarItemTypeSearch;
        }else if (hiddenItemTag == FS_TOOLBAR_ITEM_TAG_PANEL) {
            itemType = FSMainToolbarItemTypePanel;
        }else if (hiddenItemTag == FS_TOOLBAR_ITEM_TAG_VIEW_SETTINGS) {
            itemType = FSMainToolbarItemTypeViewSettings;
        }else if (hiddenItemTag == FS_TOOLBAR_ITEM_TAG_THUMBNAIL) {
            itemType = FSMainToolbarItemTypeThumbnail;
        }else if (hiddenItemTag == FS_TOOLBAR_ITEM_TAG_READING_BOOKMARK) {
            itemType = FSMainToolbarItemTypeReadingBookmark;
        }
        
        [self setMainToolbarItemHiddenWithTag:itemType toolbarItemTag:hiddenItemTag hidden:isHidden];
        if (hiddenItemTag == FS_TOOLBAR_ITEM_TAG_VIEW_SETTINGS) {
            [self setMainTopbarToolTagItemHiddenWithTag:FSMainTopbarToolTagView toolbarItemTag:FS_TOOLBAR_ITEM_TAG_VIEW hidden:isHidden];
        }
        if (hiddenItemTag == FS_TOOLBAR_ITEM_TAG_MORE){
            self.extensionsManager.smallTopToolbar.moreBtn.hidden = isHidden;
        }

    }else{

        FSMainTopbarToolTag toolTag = FSMainTopbarToolTagHome;
        
        if (hiddenItemTag == FS_TOOLBAR_ITEM_TAG_HOME) {
            toolTag = FSMainTopbarToolTagHome;
        }else if (hiddenItemTag == FS_TOOLBAR_ITEM_TAG_EDIT) {
            toolTag = FSMainTopbarToolTagEdit;
        }else if (hiddenItemTag == FS_TOOLBAR_ITEM_TAG_COMMENT) {
            toolTag = FSMainTopbarToolTagComment;
        }else if (hiddenItemTag == FS_TOOLBAR_ITEM_TAG_DRAWING) {
            toolTag = FSMainTopbarToolTagDrawing;
        }else if (hiddenItemTag == FS_TOOLBAR_ITEM_TAG_VIEW) {
            toolTag = FSMainTopbarToolTagView;
        }else if (hiddenItemTag == FS_TOOLBAR_ITEM_TAG_FORM) {
            toolTag = FSMainTopbarToolTagForm;
        }else if (hiddenItemTag == FS_TOOLBAR_ITEM_TAG_SIGN) {
            toolTag = FSMainTopbarToolTagSign;
        }else if (hiddenItemTag == FS_TOOLBAR_ITEM_TAG_PROTECT) {
            toolTag = FSMainTopbarToolTagProtect;
        }
        
        [self setMainTopbarToolTagItemHiddenWithTag:toolTag toolbarItemTag:hiddenItemTag hidden:isHidden];
        if (hiddenItemTag == FS_TOOLBAR_ITEM_TAG_VIEW) {
            [self setMainToolbarItemHiddenWithTag:FSMainToolbarItemTypeViewSettings toolbarItemTag:FS_TOOLBAR_ITEM_TAG_VIEW_SETTINGS hidden:isHidden];
        }
    }
}

-(BOOL)isToolbarItemHiddenWithTag:(FS_TOOLBAR_ITEM_TAG)hiddenItemTag{
    if ([self.removedToolbarItems objectForKey:[NSNumber numberWithInt:hiddenItemTag]]) {
        return YES;
    }else{
        return NO;
    }
}

-(void)setMainToolbarItemHiddenWithTag:(FSMainToolbarItemType)itemType toolbarItemTag:(FS_TOOLBAR_ITEM_TAG)hiddenItemTag hidden:(BOOL)isHidden{
    if (isHidden) {
        if (!self.removedToolbarItems) {
            self.removedToolbarItems = [NSMutableDictionary dictionary];
        }

        FSMainToolbarItem *toolbarItem = [self.extensionsManager.topToolbar getToolbarItemForItemType:itemType];
        if (toolbarItem) {
            [self.removedToolbarItems setObject:toolbarItem forKey:[NSNumber numberWithInteger:hiddenItemTag]];
            [self.extensionsManager.topToolbar removeItem:toolbarItem];
        }
        if (!toolbarItem) {
            toolbarItem = [self.extensionsManager.bottomToolbar getToolbarItemForItemType:itemType];
            if (toolbarItem) {
                [self.removedToolbarItems setObject:toolbarItem forKey:[NSNumber numberWithInteger:hiddenItemTag]];
                [self.extensionsManager.bottomToolbar removeItem:toolbarItem];
            }
        }

    }else{
        FSMainToolbarItem *toolbarItem = [self.removedToolbarItems objectForKey:[NSNumber numberWithInteger:hiddenItemTag]];
        if (!toolbarItem) {
            return;
        }
        [self.removedToolbarItems removeObjectForKey:[NSNumber numberWithInteger:hiddenItemTag]];
        NSMutableArray<FSMainToolbarItem *> *leftArray = [self.extensionsManager.topToolbar.itemsForPositionLeft mutableCopy];
        if (!leftArray) {
            leftArray = [NSMutableArray array];
        }
        NSMutableArray<FSMainToolbarItem *> *rightArray = [self.extensionsManager.topToolbar.itemsForPositionRight mutableCopy];
        if (!rightArray) {
            rightArray = [NSMutableArray array];
        }
        NSMutableArray<FSMainToolbarItem *> *bottomCenterArray = [self.extensionsManager.bottomToolbar.itemsForPositionCenter mutableCopy];
        if (!bottomCenterArray) {
            bottomCenterArray = [NSMutableArray array];
        }
        BOOL isTopRight = NO;
        BOOL isBottomCenter = NO;
        
        switch (itemType) {
            case FSMainToolbarItemTypeBack:
            {
                [leftArray insertObject:toolbarItem atIndex:0];
                self.extensionsManager.topToolbar.itemsForPositionLeft = leftArray;
                break;
            }
            case FSMainToolbarItemTypeMore:
            case FSMainToolbarItemTypeSearch:
                [rightArray addObject:toolbarItem];
                isTopRight = YES;
                break;
            case FSMainToolbarItemTypePanel:
                if (DEVICE_iPAD) {
                    [leftArray addObject:toolbarItem];
                    self.extensionsManager.topToolbar.itemsForPositionLeft = leftArray;
                }else{
                    [bottomCenterArray addObject:toolbarItem];
                    isBottomCenter = YES;
                }
                break;
            case FSMainToolbarItemTypeViewSettings:
                [bottomCenterArray addObject:toolbarItem];
                isBottomCenter = YES;
                break;
            case FSMainToolbarItemTypeThumbnail:
                if (DEVICE_iPAD) {
                    [rightArray addObject:toolbarItem];
                    isTopRight = YES;
                }else{
                    [bottomCenterArray addObject:toolbarItem];
                    isBottomCenter = YES;
                }
                break;
            case FSMainToolbarItemTypeReadingBookmark:
                if (DEVICE_iPAD) {
                    [rightArray addObject:toolbarItem];
                    isTopRight = YES;
                }else{
                    [bottomCenterArray addObject:toolbarItem];
                    isBottomCenter = YES;
                }
                break;
                
            default:
                break;
        }
        
        if (isTopRight) {
            NSMutableArray *tmpRightArray = [NSMutableArray array];
            for (NSNumber *number in self.topToolbarRightItems) {
                for (FSMainToolbarItem *item in rightArray) {
                    if (item.itemType == [number integerValue]) {
                        [tmpRightArray addObject:item];
                        break;
                    }
                }
            }
            [rightArray removeObjectsInArray:tmpRightArray];
            [tmpRightArray addObjectsFromArray:rightArray];
            self.extensionsManager.topToolbar.itemsForPositionRight = tmpRightArray;
        }
        
        if (isBottomCenter) {
            NSMutableArray *tmpCenterArray = [NSMutableArray array];
            for (NSNumber *number in self.iphoneBottombarcenterItems) {
                for (FSMainToolbarItem *item in bottomCenterArray) {
                    if (item.itemType == [number integerValue]) {
                        [tmpCenterArray addObject:item];
                        break;
                    }
                }
            }
            [bottomCenterArray removeObjectsInArray:tmpCenterArray];
            [tmpCenterArray addObjectsFromArray:bottomCenterArray];
            self.extensionsManager.bottomToolbar.itemsForPositionCenter = tmpCenterArray;
        }
    }

}

-(void)setMainTopbarToolTagItemHiddenWithTag:(FSMainTopbarToolTag)toolTag toolbarItemTag:(FS_TOOLBAR_ITEM_TAG)hiddenItemTag hidden:(BOOL)isHidden{
    
    if (isHidden) {
        if (!self.removedToolbarItems) {
            self.removedToolbarItems = [NSMutableDictionary dictionary];
        }
        FSMainTopbarToolTagItem *hiddenTagItem = nil;
        NSMutableArray<FSMainTopbarToolTagItem *> *tagItems = [self.extensionsManager.topToolbar.tagItems mutableCopy];
        for (FSMainTopbarToolTagItem *tagItem in tagItems) {
            if (tagItem.toolTag == toolTag) {
                hiddenTagItem = tagItem;
                break;
            }
        }
        if (hiddenTagItem) {
            [self.removedToolbarItems setObject:hiddenTagItem forKey:[NSNumber numberWithInteger:hiddenItemTag]];
            [tagItems removeObject:hiddenTagItem];
        }
        self.extensionsManager.topToolbar.tagItems = tagItems;
    }else{
        FSMainTopbarToolTagItem *hiddenTagItem = [self.removedToolbarItems objectForKey:[NSNumber numberWithInteger:hiddenItemTag]];
        if (!hiddenTagItem) {
            return;
        }
        [self.removedToolbarItems removeObjectForKey:[NSNumber numberWithInteger:hiddenItemTag]];
        NSMutableArray<FSMainTopbarToolTagItem *> *tagItems = [self.extensionsManager.topToolbar.tagItems mutableCopy];
        if (!tagItems) {
            tagItems = [NSMutableArray<FSMainTopbarToolTagItem *> array];
        }
        [tagItems addObject:hiddenTagItem];
        self.extensionsManager.topToolbar.tagItems = tagItems;
    }
}

#pragma mark - get Preference data

- (BOOL)isShowReadBarItemDescript
{
    return [Preference getBoolValue:FSPreferenceModuleToolbarSettings type:@"readBarItemDescriptShow" delaultValue:YES];
}

#pragma mark - home

- (void)addAddItemToTagItem:(FSMainTopbarToolTagItem *)tagItem{
    
    NSMutableArray *tmp = tagItem.readToolbarItems ? tagItem.readToolbarItems.mutableCopy : @[].mutableCopy;
    FSMainToolbarItem *addItem = [[FSMainToolbarItem alloc] initWithExtensionsManager:self.extensionsManager itemType:FSMainToolbarItemTypeAddTool];
    addItem.spacing = ITEM_BOTH_SPACE_NORMAL;
    self.addItem = addItem;
    [self.addItem.item setImage:FSImageNamedAndNormalColor(@"read_add_tool") forState:UIControlStateNormal];
    if (tmp.count <= 0) {
        [self.addItem.item setTitle:FSLocalizedForKey(@"kAddTools") forState:UIControlStateNormal];
    }else{
        [self.addItem.item setTitle:nil forState:UIControlStateNormal];
    }
    [self.addItem.item setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
    self.addItem.item.font = self.fs_standardGeneralFont_GF2;
    [self.addItem.item setImageEdgeInsets:UIEdgeInsetsMake(0, 0, 0, ITEM_TOP_ICON_MARGIN)];
    [tmp addObject:[[FSReadToolbarItem alloc] initWithCustomItem:addItem]];
    tagItem.readToolbarItems = tmp;
    
    @weakify(self)
    self.addItem.item.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [[self.extensionsManager.viewCtrlManager presentViewCtrlType:FSViewCtrlTypeAddTools params:self.mainTopbar.currentTagItem] setDismissCompletion:^{
            [self.extensionsManager showMenuIfNeed];
        }];
        [self.extensionsManager cancelAutoFullScreen];
        [self.extensionsManager onUIInteractElementClicked:FSUIElementReadingHomeBarAddTools];
    };
}

#pragma mark exclude home
- (void)addSortItemToTagItem:(FSMainTopbarToolTagItem *)tagItem{
    
    NSMutableArray *tmp = tagItem.readToolbarItems ? tagItem.readToolbarItems.mutableCopy : @[].mutableCopy;
    FSMainToolbarItem *sortItem = [[FSMainToolbarItem alloc] initWithExtensionsManager:self.extensionsManager itemType:FSMainToolbarItemTypeSortTool];
    sortItem.spacing = ITEM_BOTH_SPACE_NORMAL;
    self.sortItem = sortItem;
    [self.sortItem.item setImage:FSImageNamedAndNormalColor(@"read_sort_tool") forState:UIControlStateNormal];
  
    [self.sortItem.item setTitleColor:self.fs_textColor_T4 forState:UIControlStateNormal];
    self.sortItem.item.font = self.fs_standardGeneralFont_GF2;
    [self.sortItem.item setImageEdgeInsets:UIEdgeInsetsMake(0, 0, 0, ITEM_TOP_ICON_MARGIN)];
    [tmp addObject:[[FSReadToolbarItem alloc] initWithCustomItem:sortItem]];
    tagItem.readToolbarItems = tmp;
    
    @weakify(self)
    self.sortItem.item.onTapClick = ^(FSItem * _Nonnull _item) {
        @strongify(self)
        [[self.extensionsManager.viewCtrlManager presentViewCtrlType:FSViewCtrlTypeSortTools params:self.mainTopbar.currentTagItem] setDismissCompletion:^{
            [self.extensionsManager showMenuIfNeed];
        }];
        [self.extensionsManager cancelAutoFullScreen];
    };
}

#pragma mark - FSMainTopbarDelegate
- (void)didShowTagTabPopuper{
    for (id<IFSMainTopbarEventListener> listener in self.listeners) {
        if ([listener respondsToSelector:@selector(didShowTagTabPopuper)]) {
            [listener didShowTagTabPopuper];
        }
    }
}
- (void)didDismissTagTabPopuper{
    for (id<IFSMainTopbarEventListener> listener in self.listeners) {
        if ([listener respondsToSelector:@selector(didDismissTagTabPopuper)]) {
            [listener didDismissTagTabPopuper];
        }
    }
}
- (void)didSetCurrentTagItem:(FSMainTopbarToolTagItem *)curTagItem lastTagItem:(FSMainTopbarToolTagItem *)lastTagItem{
    if (self.extensionsManager.currentToolHandler) {
        self.extensionsManager.currentToolHandler = nil;
    }
    for (id<IFSMainTopbarEventListener> listener in self.listeners) {
        if ([listener respondsToSelector:@selector(didSetCurrentTagItem:lastTagItem:)]) {
            [listener didSetCurrentTagItem:curTagItem lastTagItem:lastTagItem];
        }
    }
}
- (void)onSubitemViewChanged:(FSTopbarSubitemContentViewPosition)position{
    for (id<IFSMainTopbarEventListener> listener in self.listeners) {
        if ([listener respondsToSelector:@selector(onSubitemViewChanged:)]) {
            [listener onSubitemViewChanged:position];
        }
    }
}

#pragma mark - IPanelChangedListener
- (void)onPanelChanged:(BOOL)isHidden{
    self.panelListItem.item.selected = !isHidden;
    if (!isHidden) {
        [_extensionsManager cancelAutoFullScreen];
        if (self.mainTopbar.subitemContentPosition == FSTopbarSubitemContentViewPositionLeft) {
            [self moveToolbarToPosition:FSTopbarSubitemContentViewPositionRight];
        }
    }else{
        [_extensionsManager delayAutoFullScreen];
        if (self.mainTopbar.subitemContentPosition == FSTopbarSubitemContentViewPositionLeft) {
            [self moveToolbarToPosition:FSTopbarSubitemContentViewPositionLeft];
        }
    }
}
#pragma mark - IDocEventListener
- (void)onDocWillOpen{}
- (void)onDocOpened:(FSPDFDoc * _Nullable)document error:(int)error{
    if (document) {
        if(self.thumbnailItem){
            if([self.extensionsManager fs_currentSceneIsCompact]){
                [self.thumbnailItem.item setImage:FSImageNamedAndNormalColor(@"read_thumbnail_gray") forState:UIControlStateNormal];
            }else{
                [self.thumbnailItem.item setImage:FSImageNamedAndColor(@"read_thumbnail_gray", self.fs_iconColor_I6) forState:UIControlStateNormal];
            }
        }
        if(self.bookmarkItem){
            if ([self.extensionsManager fs_currentSceneIsCompact]) {
                [self.bookmarkItem.item setImage:FSImageNamedAndPrimaryColor(@"read_bookmark_select") forState:UIControlStateSelected];
                [self.bookmarkItem.item setImage:FSImageNamedAndNormalColor(@"read_bookmark_gray") forState:UIControlStateNormal];
            }else{

                [self.bookmarkItem.item setImage:FSImageNamedAndColor(@"read_bookmark_select",self.fs_iconColor_I7) forState:UIControlStateSelected];
                [self.bookmarkItem.item setImage:FSImageNamedAndColor(@"read_bookmark_gray", self.fs_iconColor_I6) forState:UIControlStateNormal];
            }
        }
        BOOL canAddAnnot = [self.extensionsManager annotsPermissionCanAdd];
        for (FSMainTopbarToolTagItem *tagItem in self.mainTopbar.tagItems) {
            if (tagItem.toolTag == FSMainTopbarToolTagHome ||
                tagItem.toolTag == FSMainTopbarToolTagComment ||
                tagItem.toolTag == FSMainTopbarToolTagDrawing ||
                tagItem.toolTag == FSMainTopbarToolTagEdit ||
                tagItem.toolTag == FSMainTopbarToolTagForm) {
                for (FSReadToolbarItem *readItem in tagItem.readToolbarItems) {
                    if (readItem.readToolbarItemType == FSReadToolbarItemTypeExist) {
                        if (tagItem.toolTag == FSMainTopbarToolTagHome ||
                            tagItem.toolTag == FSMainTopbarToolTagComment ||
                            tagItem.toolTag == FSMainTopbarToolTagDrawing ||
                            tagItem.toolTag == FSMainTopbarToolTagEdit) {
                            if (readItem.settingsType !=  FSReadToolSettingsTypeEraser) {
                                readItem.enabled = canAddAnnot;
                            }else{
                                readItem.enabled = [Utility canAddAnnot:self.extensionsManager.pdfViewCtrl] && [self.extensionsManager annotsPermissionCanModify];
                            }
                            if (readItem.settingsType ==  FSReadToolSettingsTypeAudio ||
                                readItem.settingsType ==  FSReadToolSettingsTypeVideo ||
                                readItem.settingsType ==  FSReadToolSettingsTypeLink) {
                                readItem.enabled = canAddAnnot;
                                if (readItem.settingsType ==  FSReadToolSettingsTypeLink){
                                    readItem.enabled = !self.extensionsManager.config.disableLink && readItem.enabled ;
                                }
                            }
                        }
                    }
                }
            }else{
                continue;
            }
        }
    }
}
- (void)onDocWillClose:(FSPDFDoc *)document{}
- (void)onDocClosed:(FSPDFDoc * _Nullable)document error:(int)error{}
- (void)onDocWillSave:(FSPDFDoc *)document{}
- (void)onDocSaved:(FSPDFDoc *)document error:(int)error{}

#pragma mark - IWindowFrameChangedListener
- (void)onWindowFrameChanged:(CGRect)newFrame horizontalSizeClassChanged:(BOOL)changed{
    
    if (DEVICE_iPAD && changed) {
        for (FSMainTopbarToolTagItem *tagItem in self.mainTopbar.tagItems) {
            if (self.extensionsManager.config.defaultSettings.enableTopbarDraggable == 3 || (self.extensionsManager.config.defaultSettings.enableTopbarDraggable == 2 && ![_extensionsManager fs_currentSceneiPadHorizontalCompact]) ||
                (self.extensionsManager.config.defaultSettings.enableTopbarDraggable == 1 && [_extensionsManager fs_currentSceneiPadHorizontalCompact])) {
                BOOL hasPanItem = NO;
                for (FSMainToolbarItem *toolbarItem in tagItem.mainToolbarItems) {
                    if (toolbarItem.itemType == FSMainToolbarItemTypePan) {
                        hasPanItem = YES;
                    }
                }
                if (!hasPanItem) {
                    [self insertPanItemToTagItem:tagItem];
                }
            } else if((self.extensionsManager.config.defaultSettings.enableTopbarDraggable == 1 && ![_extensionsManager fs_currentSceneiPadHorizontalCompact]) ||
                      (self.extensionsManager.config.defaultSettings.enableTopbarDraggable == 2 && [_extensionsManager fs_currentSceneiPadHorizontalCompact])){
                NSMutableArray<FSMainToolbarItem *> *tmp = tagItem.mainToolbarItems.mutableCopy;
                for (FSMainToolbarItem *item in tmp) {
                    if (item.itemType == FSMainToolbarItemTypePan) {
                        [tmp removeObject:item];
                        break;
                    }
                }
                tagItem.mainToolbarItems = tmp;
            }
        }

        if([self->_extensionsManager fs_currentSceneIsCompact]){
            [self.thumbnailItem.item setImage:FSImageNamedAndNormalColor(@"read_thumbnail_gray") forState:UIControlStateNormal];

            [self.bookmarkItem.item setImage:FSImageNamedAndPrimaryColor(@"read_bookmark_select") forState:UIControlStateSelected];
            [self.bookmarkItem.item setImage:FSImageNamedAndNormalColor(@"read_bookmark_gray") forState:UIControlStateNormal];
        }else{
            [self.thumbnailItem.item setImage:FSImageNamedAndColor(@"read_thumbnail_gray", self.fs_iconColor_I6) forState:UIControlStateNormal];

            [self.bookmarkItem.item setImage:FSImageNamedAndColor(@"read_bookmark_select",self.fs_iconColor_I7) forState:UIControlStateSelected];
            [self.bookmarkItem.item setImage:FSImageNamedAndColor(@"read_bookmark_gray", self.fs_iconColor_I6) forState:UIControlStateNormal];
        }
        self.extensionsManager.lastManagerStateWhenAnnotTool = FSUIManagerStateNormal;
        if ([_extensionsManager fs_currentSceneiPadHorizontalCompact]) {
            if (self.mainTopbar.itemsForPositionRight.count > 0) {
                NSMutableArray<FSMainToolbarItem*> *tmp = self.mainTopbar.itemsForPositionRight.mutableCopy;
                NSMutableArray<FSMainToolbarItem*> *removeArray = [NSMutableArray array];
                for (FSMainToolbarItem *item in tmp) {
                    if (item.itemType == FSMainToolbarItemTypeThumbnail ||
                        item.itemType == FSMainToolbarItemTypeReadingBookmark) {
                        [removeArray addObject:item];
                    }
                }
                [tmp removeObjectsInArray:removeArray];
                self.mainTopbar.itemsForPositionRight = tmp;
                
            }
            
            if (self.mainTopbar.itemsForPositionLeft.count > 0) {
                NSMutableArray *tmp = self.mainTopbar.itemsForPositionLeft.mutableCopy;
                NSMutableArray<FSMainToolbarItem*> *removeArray = [NSMutableArray array];
                for (FSMainToolbarItem *item in tmp) {
                    if (item.itemType == FSMainToolbarItemTypePanel) {
                        [removeArray addObject:item];
                        break;
                    }
                }
                [tmp removeObjectsInArray:removeArray];
                self.mainTopbar.itemsForPositionLeft = tmp;
            }
         
            [self removeBottombar];
            [self buildBottombar];
            self.extensionsManager.hiddenBottomToolbar = self.extensionsManager.hiddenTopToolbar;
            if (self.mainBottombar.itemsForPositionCenter) {
                NSMutableArray *tmp = self.mainBottombar.itemsForPositionCenter.mutableCopy;
                if (self.panelListItem && ![self isToolbarItemHiddenWithTag:FS_TOOLBAR_ITEM_TAG_PANEL]) {
                    [self.panelListItem.item setImage:FSImageNamedAndNormalColor(@"read_left_pannel_gray") forState:UIControlStateNormal];
                    [self.panelListItem.item setImage:FSImageNamedAndNormalColor(@"read_left_pannel_gray") forState:UIControlStateSelected];
                    [self.panelListItem.item setBackgroundImage:nil forState:UIControlStateNormal];
                    [self.panelListItem.item setBackgroundImage:nil forState:UIControlStateSelected];
                    [tmp addObject:self.panelListItem];
                }
                if (self.viewSettingsItem &&
                    ![self isToolbarItemHiddenWithTag:FS_TOOLBAR_ITEM_TAG_VIEW_SETTINGS] &&
                     ![self isToolbarItemHiddenWithTag:FS_TOOLBAR_ITEM_TAG_VIEW]) {
                    [tmp addObject:self.viewSettingsItem];
                }
                if (self.thumbnailItem && ![self isToolbarItemHiddenWithTag:FS_TOOLBAR_ITEM_TAG_THUMBNAIL]) {
                    [tmp addObject:self.thumbnailItem];
                }
                if (self.bookmarkItem && ![self isToolbarItemHiddenWithTag:FS_TOOLBAR_ITEM_TAG_READING_BOOKMARK]) {
                    [tmp addObject:self.bookmarkItem];
                }

                self.mainBottombar.itemsForPositionCenter = tmp;
            }

            
            for (FSMainTopbarToolTagItem *tagItem in self.mainTopbar.tagItems) {
                NSMutableArray<FSMainToolbarItem *> *tmp = tagItem.mainToolbarItems.mutableCopy;
                NSMutableArray<FSMainToolbarItem *> *removeArray = [NSMutableArray array];
                FSMainToolbarItem *multiSelectItem = nil;
                for (FSMainToolbarItem *barItem in tmp) {
                    if (tagItem.toolTag == FSMainTopbarToolTagView) {
                        continue;
                    }
                    if (barItem.itemType == FSMainToolbarItemTypeRedo ||
                        barItem.itemType == FSMainToolbarItemTypeMultiSelect) {
                        if (barItem.itemType == FSMainToolbarItemTypeMultiSelect) {
                            multiSelectItem = barItem;
                        }
                        [removeArray addObject:barItem];
                    }else if (barItem.itemType == FSMainToolbarItemTypeUndo){
                        [barItem.item setBackgroundImage:FSImageNamedAndNormalColor(@"common_unflod_backgroup") forState:FSItemControlStateAll];
                        barItem.item.enabled = [_extensionsManager canUndo] || [_extensionsManager canRedo];
                    }
                }
                [tmp removeObjectsInArray:removeArray];
                tagItem.mainToolbarItems = tmp;
                
                if (multiSelectItem) {
                    NSMutableArray *tmpReadItems = tagItem.readToolbarItems ? tagItem.readToolbarItems.mutableCopy : @[].mutableCopy;
                    [tmpReadItems insertObject:[[FSReadToolbarItem alloc] initWithCustomItem:multiSelectItem] atIndex:0];
                    tagItem.readToolbarItems = tmpReadItems;
                }
            }
            
            FSMainTopbarToolTag curToolTag = self.mainTopbar.currentTagItem.toolTag;
            NSMutableArray<FSMainTopbarToolTagItem*> *tagItems = self.mainTopbar.tagItems.mutableCopy;
            [tagItems removeObject:self.viewTagItem];
            self.mainTopbar.tagItems = tagItems;
            if (curToolTag == FSMainTopbarToolTagView) {
                _isViewTagBeforeChanged = YES;
            }

            if (_extensionsManager.settingBar.viewSettingManager.isShowSettingView) {
                [_extensionsManager.settingBar.viewSettingManager showSettingView:NO isReOpen:YES];
            }
            if (_extensionsManager.settingBar.viewSettingManager.isShowPageColorView) {
                FSUIManagerState state = [_extensionsManager getState];
                if (state == FSUIManagerStateReflow || state == FSUIManagerStateSpeech || state == FSUIManagerStatePanZoom || state == FSUIManagerStateAutoFlip){
                    [_extensionsManager.settingBar.viewSettingManager showPageColorView:NO isReOpen:NO];
                }else{
                    [_extensionsManager.settingBar.viewSettingManager showPageColorView:NO isReOpen:YES];
                }
                
            }
            
        }else{
            
            if (self.mainTopbar.itemsForPositionRight.count <= 2 && self.thumbnailItem && self.bookmarkItem) {
                NSMutableArray *tmp = self.mainTopbar.itemsForPositionRight.mutableCopy;
                if (![self isToolbarItemHiddenWithTag:FS_TOOLBAR_ITEM_TAG_READING_BOOKMARK]) {
                    [self.bookmarkItem.item setImage:FSImageNamedAndColor(@"read_bookmark_select",self.fs_iconColor_I7) forState:UIControlStateSelected];
                    [self.bookmarkItem.item setImage:FSImageNamedAndColor(@"read_bookmark_gray", self.fs_iconColor_I6) forState:UIControlStateNormal];
                    [tmp addObject:self.bookmarkItem];
                }
                if (![self isToolbarItemHiddenWithTag:FS_TOOLBAR_ITEM_TAG_THUMBNAIL]) {
                    [self.thumbnailItem.item setImage:FSImageNamedAndColor(@"read_thumbnail_gray", self.fs_iconColor_I6) forState:UIControlStateNormal];
                    [tmp addObject:self.thumbnailItem];
                }
                self.mainTopbar.itemsForPositionRight = tmp;
            }
            //if (self.mainTopbar.itemsForPositionLeft.count <= 1) {

            if (![self isToolbarItemHiddenWithTag:FS_TOOLBAR_ITEM_TAG_PANEL]) {
                NSMutableArray *tmp = self.mainTopbar.itemsForPositionLeft.mutableCopy;
                [self.panelListItem.item setImage:FSImageNamedAndColor(@"read_panel_gray",self.fs_iconColor_I6) forState:UIControlStateNormal];
                [self.panelListItem.item setImage:FSImageNamedAndColor(@"read_panel_gray",self.fs_textColor_T10) forState:UIControlStateSelected];
                [self.panelListItem.item setBackgroundImage:FSImageNamedAndColor(@"common_select_backgroup_gray", self.fs_iconColor_I7) forState:UIControlStateSelected];
                [self.panelListItem.item setBackgroundImage:nil forState:UIControlStateNormal];
                [tmp addObject:self.panelListItem];
                self.mainTopbar.itemsForPositionLeft = tmp;
            }
            //}
            [self removeBottombar];
            
            if (![self isToolbarItemHiddenWithTag:FS_TOOLBAR_ITEM_TAG_VIEW] &&
                ![self isToolbarItemHiddenWithTag:FS_TOOLBAR_ITEM_TAG_VIEW_SETTINGS])
            {
                [self addViewTagItem];
//                [self rangeViewTagItems];
                
                
                if (_isViewTagBeforeChanged || _extensionsManager.settingBar.viewSettingManager.isShowSettingView) {
                    _isViewTagBeforeChanged = NO;
                    [self.mainTopbar setCurrentTagItem:self.viewTagItem];
                    if (_extensionsManager.settingBar.viewSettingManager.isShowSettingView) {
                        [_extensionsManager.settingBar.viewSettingManager showSettingView:NO isReOpen:YES];
                    }
                }else{
                    FSUIManagerState state = [_extensionsManager getState];
                    if (state == FSUIManagerStateReflow || state == FSUIManagerStateSpeech || state == FSUIManagerStatePanZoom || state == FSUIManagerStateAutoFlip) {
                        [self.mainTopbar setCurrentTagItem:self.viewTagItem];
                    }
                }
            }else{
                if (_isViewTagBeforeChanged || _extensionsManager.settingBar.viewSettingManager.isShowSettingView) {
                    _isViewTagBeforeChanged = NO;
                    if (_extensionsManager.settingBar.viewSettingManager.isShowSettingView) {
                        [_extensionsManager.settingBar.viewSettingManager showSettingView:NO isReOpen:NO];
                    }
                }
            }
            
            for (FSMainTopbarToolTagItem *tagItem in self.mainTopbar.tagItems) {
                FSReadToolbarItem *multiSelectItem = nil;
                NSMutableArray *tmpReadItems = tagItem.readToolbarItems ? tagItem.readToolbarItems.mutableCopy : @[].mutableCopy;
                for (FSReadToolbarItem *readItem in tmpReadItems) {
                    if (readItem.toolbarItem.itemType == FSMainToolbarItemTypeMultiSelect) {
                        multiSelectItem = readItem;
                        [tmpReadItems removeObject:readItem];
                        tagItem.readToolbarItems = tmpReadItems;
                        break;
                    }
                }

                NSMutableArray<FSMainToolbarItem *> *tmp = tagItem.mainToolbarItems.mutableCopy;
                NSMutableArray<FSMainToolbarItem *> *removeArray = [NSMutableArray array];
                NSInteger index = -1;
                NSInteger curIndex = 0;
                for (FSMainToolbarItem *barItem in tmp) {
                    
                    if (barItem.itemType == FSMainToolbarItemTypeUndo) {
                        index = curIndex;
                        [removeArray addObject:barItem];
                        break;
                    }
                    curIndex ++;
                }
                [tmp removeObjectsInArray:removeArray];
                if (multiSelectItem) {
                    [tmp insertObject:multiSelectItem.toolbarItem atIndex:tmp.count-1];
                }
                tagItem.mainToolbarItems = tmp;
                if (index > -1) {
                    [self insertUndoRedoItemToTagItem:tagItem atIndex:index];
                }
            }

            if (_extensionsManager.settingBar.viewSettingManager.isShowPageColorView) {
                [self.mainTopbar setCurrentTagItem:self.viewTagItem];
                [_extensionsManager.settingBar.viewSettingManager showPageColorView:NO isReOpen:YES];
            }
        }
    }
}

@end

@interface _FSPanToolbarMaskView ()

@property (nonatomic, assign) BOOL didLayoutSubviews;
@property (nonatomic, weak) FSMainTopbar *toolbar;

@end
@implementation _FSPanToolbarMaskView

- (instancetype)initWithToolbar:(FSMainTopbar *)toolbar{
    self = [super init];
    if (self) {
        _toolbar = toolbar;
        [self initialize];

    }
    return self;
}

- (void)initialize{
    _FSPlaceholderToolbar *topBar = [_FSPlaceholderToolbar new];
    [self addSubview:topBar];
    [topBar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.right.fs_equalToSuperview();
        make.top.equalTo(self.fs_mas_top).offset(TOOL_BAR_HEIGHT_NORMAL);
        make.height.offset(_toolbar.horizontalContentHeight - TOOL_BAR_HEIGHT_NORMAL);
    }];
    _topBar = topBar;
    
    _FSPlaceholderToolbar *leftBar = [_FSPlaceholderToolbar new];
    [self addSubview:leftBar];
    
    [leftBar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.mas_equalTo(self.fs_mas_left).offset(ITEM_LEFT_MARGIN_NORMAL);
        if (DEVICE_iPAD) {
            make.centerY.fs_equalToSuperview().priorityHigh();
        }else{
            make.top.mas_equalTo(topBar.mas_top).offset(50.f).priorityMedium();
            make.top.greaterThanOrEqualTo(topBar.mas_top).offset(50.f).priorityHigh();
        }
        make.width.offset(_toolbar.verticalContentWidth);
        make.height.mas_equalTo(self.mas_height).multipliedBy(0.64).priorityMedium();
        make.height.mas_greaterThanOrEqualTo(self.mas_height).multipliedBy(0.55);
        make.bottom.mas_lessThanOrEqualTo(self.fs_mas_bottom).offset(-88);
    }];
    _leftBar = leftBar;
    
    _FSPlaceholderToolbar *rightBar = [_FSPlaceholderToolbar new];
    [self addSubview:rightBar];
    
    
    [rightBar mas_makeConstraints:^(MASConstraintMaker *make) {
        make.right.mas_equalTo(self.fs_mas_right).offset(-ITEM_LEFT_MARGIN_NORMAL);
        make.top.width.height.equalTo(leftBar);
    }];
    _rightBar = rightBar;
    
    [BindThemeKeySel(self, initialize) fs_updateThemeCompleted:^(_FSPanToolbarMaskView * _Nonnull target, SEL  _Nonnull sel) {
        topBar.backgroundColor = [target.fs_primaryColor colorWithAlphaComponent:0.1];
        leftBar.backgroundColor = [target.fs_primaryColor colorWithAlphaComponent:0.1];
        rightBar.backgroundColor = [target.fs_primaryColor colorWithAlphaComponent:0.1];
        if (target.didLayoutSubviews) [target moveToPosition:target.moveOnPosition];
    }];
}

- (void)moveToPosition:(FSTopbarSubitemContentViewPosition)position{
    if (_didLayoutSubviews && _moveOnPosition == position) return;
    _moveOnPosition = position;
    
    NSArray *lineDashPattern = @[ @3 , @3 ];
    switch (position) {
        case FSTopbarSubitemContentViewPositionTop:
            [_topBar fs_setLayerBorderWidth:DIVIDE_VIEW_THICKNES borderColor:self.fs_primaryColor];
            [_leftBar fs_setLayerBorderWidth:DIVIDE_VIEW_THICKNES borderColor:self.fs_primaryColor lineDashPattern:lineDashPattern];
            [_rightBar fs_setLayerBorderWidth:DIVIDE_VIEW_THICKNES borderColor:self.fs_primaryColor lineDashPattern:lineDashPattern];
            break;
        case FSTopbarSubitemContentViewPositionLeft:
            [_topBar fs_setLayerBorderWidth:DIVIDE_VIEW_THICKNES borderColor:self.fs_primaryColor lineDashPattern:lineDashPattern];
            [_leftBar fs_setLayerBorderWidth:DIVIDE_VIEW_THICKNES borderColor:self.fs_primaryColor];
            [_rightBar fs_setLayerBorderWidth:DIVIDE_VIEW_THICKNES borderColor:self.fs_primaryColor lineDashPattern:lineDashPattern];
            
            break;
        case FSTopbarSubitemContentViewPositionRight:
            [_topBar fs_setLayerBorderWidth:DIVIDE_VIEW_THICKNES borderColor:self.fs_primaryColor lineDashPattern:lineDashPattern];
            [_leftBar fs_setLayerBorderWidth:DIVIDE_VIEW_THICKNES borderColor:self.fs_primaryColor lineDashPattern:lineDashPattern];
            [_rightBar fs_setLayerBorderWidth:DIVIDE_VIEW_THICKNES borderColor:self.fs_primaryColor];
            break;
    }
}

- (void)layoutSubviews{
    [super layoutSubviews];
    [self moveToPosition:self.moveOnPosition];
}
@end
