/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSToolbarSettingsPopover.h"
#import "Preference.h"
#import "FSToolsPopoverViewController.h"

@interface FSToolbarSettingsPopoverPlaceholderCell : FSEstimatedTableViewCell<UIViewAdapter>
@property (nonatomic, strong) UILabel *placeholderTipLab;
@property (nonatomic, strong) FSAdapterUIView *placeholderTipContentView;
@property (nonatomic, assign) BOOL placeholderHighlighted;
- (void)setPlaceholderText:(NSString *)text;
@end

@implementation FSToolbarSettingsPopoverPlaceholderCell

- (void)initialize{
    [super initialize];
}

- (FSAdapterUIView *)placeholderTipContentView{
    if(!_placeholderTipContentView){
        _placeholderTipContentView = [FSAdapterUIView new];
        _placeholderTipContentView.adapter = self;
        _placeholderTipContentView.backgroundColor = [UIColor fs_clearColor];
        [self adjustPlaceholderTipContentViewBorder];
        [_placeholderTipContentView addSubview:self.placeholderTipLab];
        [_placeholderTipLab mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.fs_equalToSuperview().offset(ITEM_BOTH_SPACE_NORMAL);
            make.right.fs_equalToSuperview().offset(-ITEM_BOTH_SPACE_NORMAL);
            make.top.bottom.fs_equalToSuperview();
        }];
    }
    return _placeholderTipContentView;
}

- (UILabel *)placeholderTipLab{
    if(!_placeholderTipLab){
        _placeholderTipLab = [UILabel new];
        _placeholderTipLab.textColor = self.fs_textColor_T9;
        _placeholderTipLab.font = self.fs_standardWeakFont_WF1;
        _placeholderTipLab.textAlignment = NSTextAlignmentCenter;
    }
    return _placeholderTipLab;
}

- (void)prepareForReuse{
    [super prepareForReuse];
    self.placeholderHighlighted = NO;
}

- (void)setPlaceholderText:(NSString *)text{
    self.placeholderTipLab.text = text;
    if (!self.placeholderTipContentView.superview) {
        [self.contentView addSubview:self.placeholderTipContentView];
        [self.placeholderTipContentView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.fs_equalToSuperview().offset(ITEM_MARGIN_NORMAL);
            make.right.fs_equalToSuperview().offset(-ITEM_MARGIN_NORMAL);
            make.top.fs_equalToSuperview().offset(5.f);
            make.bottom.fs_equalToSuperview().offset(-5.f);
            make.height.offset(TOOL_BAR_HEIGHT_NORMAL);
        }];
    }
}

- (void)setPlaceholderHighlighted:(BOOL)placeholderHighlighted{
    if (_placeholderHighlighted == placeholderHighlighted) return;
    _placeholderHighlighted = placeholderHighlighted;
    [self adjustPlaceholderTipContentViewBorder];
}

- (void)adjustPlaceholderTipContentViewBorder{
    [_placeholderTipContentView fs_setLayerBorderWidth:1 borderColor:_placeholderHighlighted ? self.fs_primaryColor : self.fs_dividerColor_P3 lineDashPattern:@[ @3 , @3 ]];
}

- (void)layoutSubviewsFromView:(__kindof UIView *)view{
    if (view == _placeholderTipContentView) {
        [self adjustPlaceholderTipContentViewBorder];
    }
}

@end

@interface FSToolbarSettingsItem : FSPopoverItem
@property (nonatomic, weak) FSMainTopbarToolTagItem *tagItem;
- (instancetype)initWithTagItem:(FSMainTopbarToolTagItem *)tagItem;
@end

@implementation FSToolbarSettingsItem

- (instancetype)initWithTagItem:(FSMainTopbarToolTagItem *)tagItem{
    self = [super initWithTitle:tagItem.title image:tagItem.image];
    if (self) {
        self.tagItem = tagItem;
    }
    return self;
}

@end
@interface FSToolbarSettingsPopover ()<UITableViewDragDelegate, UITableViewDropDelegate>
@property (nonatomic, strong) FSToolbarSettingsPopoverFooterView *footView;
@end

@implementation FSToolbarSettingsPopover
- (void)viewDidLoad{
    [super viewDidLoad];
    [self.tableView fs_registerClass:[FSToolbarSettingsPopoverPlaceholderCell class]];
    self.tableView.dragDelegate = self;
    self.tableView.dropDelegate = self;
}

- (void)setToolTagItems:(NSArray<FSMainTopbarToolTagItem *> *)toolTagItems{
    _toolTagItems = toolTagItems;
    NSMutableArray *groups = @[].mutableCopy;
    NSMutableArray *onToolbar = @[].mutableCopy;
    NSMutableArray *moreTools = @[].mutableCopy;
    for (FSMainTopbarToolTagItem *tagItem in toolTagItems) {
        tagItem.isSourceOperation = NO;
        FSToolbarSettingsItem *popItem = [[FSToolbarSettingsItem alloc] initWithTagItem:tagItem];
        if (tagItem.isAutoAdjustOnToolbar) {
            [onToolbar addObject:popItem];
        }else{
            [moreTools addObject:popItem];
        }
    }
    
    [groups addObject:[[FSPopoverItemGroup alloc] initWithTitle:FSLocalizedForKey(@"kOnToolbar").uppercaseString items:onToolbar.copy]];
    [groups addObject:[[FSPopoverItemGroup alloc] initWithTitle:FSLocalizedForKey(@"kMoreTools").uppercaseString items:moreTools.copy]];
    self.groups = groups;
    [self.tableView reloadData];
}

- (void)configureModalPresentationStyle{
    if (self.popoverShowStyle == FSTableViewPopoverShowStyleDefault) {
        UIExtensionsManager *ex = ((FSViewControllerManager *)self.services).extensionsManager;
        if (ex.pdfViewCtrl.fs_width > ex.pdfViewCtrl.fs_height) {
            self.popoverSize = CGSizeMake(ex.pdfViewCtrl.fs_height, 0);
        }else{
            self.popoverSize = CGSizeMake(ex.pdfViewCtrl.fs_width, 0);
        }
    }else{
        self.popoverSize = CGSizeMake(342, 0);
    }
    [super configureModalPresentationStyle];
}

- (BOOL)emptyDataSetShouldDisplay:(UITableView *)tableView{
    return NO;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section{
    if (![self dataForAtIndexPath:[NSIndexPath indexPathForRow:0 inSection:section]]) {
        return 1;
    }
    return [super tableView:tableView numberOfRowsInSection:section];
}

- (CGFloat)tableView:(UITableView *)tableView heightForFooterInSection:(NSInteger)section {
    if (section == (self.groups.count - 1)){
        return 52;
    }else{
        return [super tableView:tableView heightForFooterInSection:section];
    }
}

- (UIView *)tableView:(UITableView *)tableView viewForFooterInSection:(NSInteger)section {
    if (section == (self.groups.count - 1)){
        return self.footView;
    }else{
        return [super tableView:tableView viewForFooterInSection:section];
    }
}

- (__kindof UITableViewCell *)tableView:(UITableView *)tableView dequeueReusableCellWithIdentifier:(NSString *)identifier forIndexPath:(NSIndexPath *)indexPath withDataItem:(id)dataItem{
    if (!dataItem) {
        identifier = NSStringFromClass([FSToolbarSettingsPopoverPlaceholderCell class]);
    }
    return [super tableView:tableView dequeueReusableCellWithIdentifier:identifier forIndexPath:indexPath withDataItem:dataItem];
}

- (void)configureCell:(FSTitleEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(FSToolbarSettingsItem *)item{
    if (![self dataForAtIndexPath:indexPath]) {
        FSToolbarSettingsPopoverPlaceholderCell *placeholderCell = (FSToolbarSettingsPopoverPlaceholderCell *)cell;
        if (indexPath.section == 0) {
            [placeholderCell setPlaceholderText:FSLocalizedForKey(@"kOnToolbarNoToolTip")];
        }else{
            [placeholderCell setPlaceholderText:FSLocalizedForKey(@"kMoreToolsNoToolTip")];
        }
    }else{
        [super configureCell:cell atIndexPath:indexPath withDataItem:item];
        cell.isAccessibilityElement = NO;
        cell.estimatedTitleLabel.textColor = self.fs_textColor_T4;
        
        if (!cell.rightView) {
            FSItem *dragGripItem = [FSItem createItemWithImage:FSImageNamedAndColor(@"common_drag_grip", self.fs_iconColor_I5)];
            dragGripItem.customAccessibilityTraits = UIAccessibilityTraitNone;
            dragGripItem.isAccessibilityElement = YES;
            cell.rightView = dragGripItem;
        }

        cell.rightView.accessibilityLabel = [NSString stringWithFormat:FSLocalizedForKey(@"kTragMainTopTagVoiceOver"), item.tagItem.title];
        
        if (self.selectedItem == item.tagItem) {
            BindThemeKeyPath(cell.estimatedTitleLabel, textColor);
            [BindThemeKeySel(cell.leftView, setImage:) fs_updateThemeCompleted:^(UIImageView * _Nonnull target, SEL  _Nonnull sel) {
                [target setImage:[target.image fs_imageWithTintColor:target.fs_primaryColor]];
            }];
        }
    }
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath{
    FSEstimatedTableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
    if ([cell isKindOfClass:[FSToolbarSettingsPopoverPlaceholderCell class]]) {
        [tableView deselectRowAtIndexPath:indexPath animated:NO];
        return;
    }
//    !self.selectedItemChanged ?: self.selectedItemChanged(self);
    NSMutableArray *onToolbar = [[self.groups firstObject] items].mutableCopy;
    NSMutableArray *moreTools = [[self.groups lastObject] items].mutableCopy;
    if (indexPath.section == 1) {
        if (!moreTools.count) return;
        [((FSViewControllerManager *)self.services).extensionsManager.topToolbar setCurrentTagItem:nil];
        FSToolbarSettingsItem *item = [moreTools objectAtIndex:indexPath.row];
        item.tagItem.isSourceOperation = YES;
        
        [moreTools removeObject:item];
        [onToolbar addObject:item];
        
        NSMutableArray *tagItems = @[].mutableCopy;
        for (FSToolbarSettingsItem *item in onToolbar) {
            item.tagItem.isOnToolbar = YES;
            [tagItems addObject:item.tagItem];
        }
        for (FSToolbarSettingsItem *item in moreTools) {
            item.tagItem.isOnToolbar = NO;
            [tagItems addObject:item.tagItem];
        }
        
        _toolTagItems = tagItems;
        
        !self.settingsChanged ?: self.settingsChanged(self);
        
        NSUInteger index = [_toolTagItems indexOfObject:item.tagItem];
        !item.action ? : item.action(item, (int)index);
        !self.action ? : self.action(item, (int)index);
        [self dismissViewControllerAnimated:YES completion:^{
            !self.dismissCompletion ? : self.dismissCompletion();
            !self.splitScrrenDismissCompletion ?: self.splitScrrenDismissCompletion();
        }];
    }else{
        FSToolbarSettingsItem *item = [onToolbar objectAtIndex:indexPath.row];
        NSUInteger index = [_toolTagItems indexOfObject:item.tagItem];
        [self dismissViewControllerAnimated:YES completion:^{
            !item.action ? : item.action(item, (int)index);
            !self.action ? : self.action(item, (int)index);
            !self.dismissCompletion ? : self.dismissCompletion();
            !self.splitScrrenDismissCompletion ?: self.splitScrrenDismissCompletion();
        }];
    }
    [tableView deselectRowAtIndexPath:indexPath animated:YES];
}

#pragma mark - UITableViewDragDelegate
- (nonnull NSArray<UIDragItem *> *)tableView:(nonnull UITableView *)tableView itemsForBeginningDragSession:(nonnull id<UIDragSession>)session atIndexPath:(nonnull NSIndexPath *)indexPath {
    FSToolbarSettingsItem *popoverItem = [self dataForAtIndexPath:indexPath];
    if (popoverItem) {
        popoverItem.tagItem.isSourceOperation = YES;
        NSItemProvider *provider = [[NSItemProvider alloc] initWithItem:popoverItem.title typeIdentifier:@"UIImage"];
        UIDragItem *dragItem = [[UIDragItem alloc] initWithItemProvider:provider];
        return @[dragItem];
    }
    return nil;
}

#pragma mark - UITableViewDropDelegate
- (void)tableView:(nonnull UITableView *)tableView performDropWithCoordinator:(nonnull id<UITableViewDropCoordinator>)coordinator {
    NSMutableArray *onToolbar = [[self.groups firstObject] items].mutableCopy;
    NSMutableArray *moreTools = [[self.groups lastObject] items].mutableCopy;
    
    NSIndexPath *sourceIndexPath = [[coordinator.items firstObject] sourceIndexPath];
    NSIndexPath *destinationIndexPath = coordinator.destinationIndexPath;

    if (sourceIndexPath.section == 0 && destinationIndexPath.section == 0) {
        FSPopoverItem *moveItem = [onToolbar objectAtIndex:sourceIndexPath.row];
        [onToolbar removeObject:moveItem];
        [onToolbar fs_insertObject:moveItem atIndex:destinationIndexPath.row];
    }else if (sourceIndexPath.section == 1 && destinationIndexPath.section == 1) {
        FSPopoverItem *moveItem = [moreTools objectAtIndex:sourceIndexPath.row];
        [moreTools removeObject:moveItem];
        [moreTools fs_insertObject:moveItem atIndex:destinationIndexPath.row];
    }else{
        FSToolbarSettingsItem *moveItem;
        if (sourceIndexPath.section == 0) {
            moveItem = [onToolbar objectAtIndex:sourceIndexPath.row];
            [onToolbar removeObject:moveItem];
            [moreTools fs_insertObject:moveItem atIndex:destinationIndexPath.row];
            
            if (moveItem.tagItem == self.selectedItem) {
                self.selectedItem = nil;
            }
        }else{
            moveItem = [moreTools objectAtIndex:sourceIndexPath.row];
            [moreTools removeObject:moveItem];
            [onToolbar fs_insertObject:moveItem atIndex:destinationIndexPath.row];
        }
    }
    
    [self.groups firstObject].items = onToolbar;
    [self.groups lastObject].items = moreTools;
    [self autoCalculatePopoverHeight];
    [tableView reloadData];
    
    NSMutableArray *tagItems = @[].mutableCopy;
    for (FSToolbarSettingsItem *item in onToolbar) {
        item.tagItem.isOnToolbar = YES;
        [tagItems addObject:item.tagItem];
    }
    for (FSToolbarSettingsItem *item in moreTools) {
        item.tagItem.isOnToolbar = NO;
        [tagItems addObject:item.tagItem];
    }
    
    _toolTagItems = tagItems;
    !self.settingsChanged ?: self.settingsChanged(self);
    
    
    onToolbar = [[self.groups firstObject] items].mutableCopy;
    moreTools = [[self.groups lastObject] items].mutableCopy;
    for (FSToolbarSettingsItem *item in onToolbar) {
        item.tagItem.isOnToolbar = YES;
    }
    for (FSToolbarSettingsItem *item in moreTools) {
        item.tagItem.isOnToolbar = NO;
    }
}

- (UITableViewDropProposal *)tableView:(UITableView *)tableView dropSessionDidUpdate:(id<UIDropSession>)session withDestinationIndexPath:(nullable NSIndexPath *)destinationIndexPath{
    NSIndexPath *placeholderIndexPath1 = [NSIndexPath indexPathForRow:0 inSection:0];
    FSEstimatedTableViewCell *cell1 = [tableView cellForRowAtIndexPath:placeholderIndexPath1];
    if ([cell1 isKindOfClass:[FSToolbarSettingsPopoverPlaceholderCell class]]) {
        [(FSToolbarSettingsPopoverPlaceholderCell *)cell1 setPlaceholderHighlighted:placeholderIndexPath1.section == destinationIndexPath.section];
    }
    NSIndexPath *placeholderIndexPath2 = [NSIndexPath indexPathForRow:0 inSection:1];
    FSEstimatedTableViewCell *cell2 = [tableView cellForRowAtIndexPath:placeholderIndexPath2];
    if ([cell2 isKindOfClass:[FSToolbarSettingsPopoverPlaceholderCell class]]) {
        [(FSToolbarSettingsPopoverPlaceholderCell *)cell2 setPlaceholderHighlighted:placeholderIndexPath2.section == destinationIndexPath.section];
    }
    
    UITableViewDropIntent intent = UITableViewDropIntentAutomatic;
    if (!destinationIndexPath ||
        ([cell1 isKindOfClass:[FSToolbarSettingsPopoverPlaceholderCell class]] && destinationIndexPath.section == 0) ||
        ([cell2 isKindOfClass:[FSToolbarSettingsPopoverPlaceholderCell class]] && destinationIndexPath.section == 1)) {
        intent = UITableViewDropIntentInsertIntoDestinationIndexPath;
    }
    UITableViewDropProposal *dropProposal = [[UITableViewDropProposal alloc] initWithDropOperation:UIDropOperationMove intent:intent];
    return dropProposal;
}

- (void)tableView:(UITableView *)tableView dropSessionDidEnd:(id<UIDropSession>)session{
    NSIndexPath *placeholderIndexPath1 = [NSIndexPath indexPathForRow:0 inSection:0];
    FSEstimatedTableViewCell *cell1 = [tableView cellForRowAtIndexPath:placeholderIndexPath1];
    if ([cell1 isKindOfClass:[FSToolbarSettingsPopoverPlaceholderCell class]]) {
        [(FSToolbarSettingsPopoverPlaceholderCell *)cell1 setPlaceholderHighlighted:NO];
    }
    NSIndexPath *placeholderIndexPath2 = [NSIndexPath indexPathForRow:0 inSection:1];
    FSEstimatedTableViewCell *cell2 = [tableView cellForRowAtIndexPath:placeholderIndexPath2];
    if ([cell2 isKindOfClass:[FSToolbarSettingsPopoverPlaceholderCell class]]) {
        [(FSToolbarSettingsPopoverPlaceholderCell *)cell2 setPlaceholderHighlighted:NO];
    }
}

- (FSToolbarSettingsPopoverFooterView *)footView {
    if (!_footView) {
        _footView = [[FSToolbarSettingsPopoverFooterView alloc] init];
        @weakify(self)
        _footView.onValueChange = ^{
            @strongify(self)
            [((FSViewControllerManager *)self.services).extensionsManager.topToolbar resetCurrentTagItem];
        };
    }
    return _footView;
}
@end
