/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSToolsPopoverViewController.h"

@interface FSToolbarSettingsPopoverFooterView()
@property (nonatomic, strong) UIView *separateView;
@property (nonatomic, strong) UIView *footerContentView;
@property (nonatomic, strong) UILabel *footerLabel;
@property (nonatomic, strong) UISwitch *footerSwitch;
@end

@implementation FSToolbarSettingsPopoverFooterView

- (instancetype)init
{
    if (self = [super init]){
        [self addSubview:self.separateView];
        [self.separateView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.top.right.mas_equalTo(self);
            make.height.mas_equalTo(DIVIDE_SECTION_HEADER_HEIGHT);
        }];
        [self addSubview:self.footerContentView];
        [self.footerContentView mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.bottom.right.mas_equalTo(self);
            make.top.mas_equalTo(self.separateView.mas_bottom);
        }];
    }
    return self;
}

- (void)onSwitchValueChange:(UISwitch *)switchItem
{
    [Preference setBoolValue:FSPreferenceModuleToolbarSettings type:@"readBarItemDescriptShow" value:switchItem.on];
    if (self.onValueChange) self.onValueChange();
}

- (UIView *)separateView {
    if (!_separateView) {
        _separateView = [[UIView alloc] init];
        _separateView.backgroundColor = self.fs_backgroundColor_B2;
    }
    return _separateView;
}

- (UILabel *)footerLabel {
    if (!_footerLabel) {
        _footerLabel = [[UILabel alloc] init];
        _footerLabel.font = self.fs_standardGeneralFont_GF2;
        _footerLabel.textColor = self.fs_textColor_T4;
        _footerLabel.text = FSLocalizedForKey(@"kReadBarItemIsShowTextTitle");
    }
    return _footerLabel;
}

- (UISwitch *)footerSwitch {
    if (!_footerSwitch) {
        _footerSwitch = [[UISwitch alloc] init];
        _footerSwitch.on = [Preference getBoolValue:FSPreferenceModuleToolbarSettings type:@"readBarItemDescriptShow" delaultValue:YES];
        _footerSwitch.onTintColor = self.fs_primaryColor;
        [_footerSwitch addTarget:self action:@selector(onSwitchValueChange:) forControlEvents:UIControlEventValueChanged];
    }
    return _footerSwitch;
}

- (UIView *)footerContentView {
    if (!_footerContentView) {
        _footerContentView = [[UIView alloc] init];
        _footerContentView.backgroundColor = self.fs_backgroundColor_B1;
        [_footerContentView addSubview:self.footerLabel];
        [_footerContentView addSubview:self.footerSwitch];
        [self.footerSwitch mas_makeConstraints:^(MASConstraintMaker *make) {
            make.right.mas_equalTo(_footerContentView.fs_right).offset(-ITEM_MARGIN_NORMAL);
            make.centerY.mas_equalTo(_footerContentView.mas_centerY);
        }];
        [self.footerLabel mas_makeConstraints:^(MASConstraintMaker *make) {
            make.left.mas_equalTo(_footerContentView.fs_left).offset(ITEM_MARGIN_NORMAL);
            make.right.mas_equalTo(self.footerSwitch.mas_left).offset(-2);
            make.centerY.mas_equalTo(_footerContentView.mas_centerY);
        }];
    }
    return _footerContentView;
}
@end

@interface FSToolsPopoverViewController ()
@property (nonatomic, strong) FSToolbarSettingsPopoverFooterView *footView;
@end

@implementation FSToolsPopoverViewController

- (void)configureModalPresentationStyle{

    if (self.popoverShowStyle == FSTableViewPopoverShowStyleDefault) {
        UIExtensionsManager *ex = ((FSViewControllerManager *)self.services).extensionsManager;
        if (ex.pdfViewCtrl.fs_width > ex.pdfViewCtrl.fs_height) {
            self.popoverSize = CGSizeMake(ex.pdfViewCtrl.fs_height, 0);
        }else{
            self.popoverSize = CGSizeMake(ex.pdfViewCtrl.fs_width, 0);
        }
    }else{
        self.popoverSize = CGSizeMake(342, 0);
    }
    [super configureModalPresentationStyle];
}

- (CGFloat)tableView:(UITableView *)tableView heightForFooterInSection:(NSInteger)section {
    if (section == (self.groups.count - 1)){
        return 52;
    }else{
        return [super tableView:tableView heightForFooterInSection:section];
    }
}

- (UIView *)tableView:(UITableView *)tableView viewForFooterInSection:(NSInteger)section {
    if (section == (self.groups.count - 1)){
        return self.footView;
    }else{
        return [super tableView:tableView viewForFooterInSection:section];
    }
}

- (void)configureCell:(FSTitleEstimatedTableViewCell *)cell atIndexPath:(NSIndexPath *)indexPath withDataItem:(FSToolsPopoverItem *)item{
    [super configureCell:cell atIndexPath:indexPath withDataItem:item];
    cell.estimatedTitleLabel.textColor = self.fs_textColor_T4;
    
    if (!cell.installed) {
        cell.rightMargin = 16.f;
    }
    if (!cell.rightView) {
        UIImageView *imageView = [UIImageView new];
        imageView.image = FSImageNamedAndPrimaryColor(@"common_select_check");
        [BindThemeKeySel(imageView, setImage:) fs_updateThemeCompleted:^(UIImageView * _Nonnull target, SEL  _Nonnull sel) {
            target.image  = [target.image fs_imageWithTintColor:target.fs_primaryColor];
        }];
        cell.rightView = imageView;
    }
    
    ((UIImageView *)cell.rightView).image = FSImageNamedAndPrimaryColor(@"common_select_check");
    
    if (self.selectedIndex == indexPath.row) {
        BindThemeKeyPath(cell.estimatedTitleLabel, textColor);
        [BindThemeKeySel(cell.leftView, setImage:) fs_updateThemeCompleted:^(UIImageView * _Nonnull target, SEL  _Nonnull sel) {
            [target setImage:[target.image fs_imageWithTintColor:target.fs_primaryColor]];
        }];
        cell.rightView.hidden = NO;
    }else{
        if (item.isNewItem) {
            ((UIImageView *)cell.rightView).image = FSImageNamedAndPrimaryColor(@"common_new_tag");
            cell.rightView.hidden = NO;
        }else{
            cell.rightView.hidden = YES;
        }
    }
}

- (FSToolbarSettingsPopoverFooterView *)footView {
    if (!_footView) {
        _footView = [[FSToolbarSettingsPopoverFooterView alloc] init];
        @weakify(self)
        _footView.onValueChange = ^{
            @strongify(self)
            [((FSViewControllerManager *)self.services).extensionsManager.topToolbar resetCurrentTagItem];
        };
    }
    return _footView;
}

@end

@implementation FSToolsPopoverItem @end
