/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSToolsSortMenuViewController.h"

@interface FSToolsSortMenuViewController ()  <UITableViewDataSource, UITableViewDelegate>

@property (nonatomic, strong) NSMutableArray<FSReadToolbarItem *> *readToolbarItems;

@property (nonatomic, strong) NSMutableArray<FSMainToolbarItem *> *mainToolbarItems;

@property (nonatomic, strong) UITableView *tableView;

@property (nonatomic, assign) BOOL isMove;

@end

@implementation FSToolsSortMenuViewController

- (FSViewControllerManager *)viewControllerManager{
    return [super services];
}

- (FSMainTopbarToolTagItem *)currentTagItem{
    return [super params];
}


- (void)viewDidLoad{
    [super viewDidLoad];
    self.tableView = [[UITableView alloc] initWithFrame:CGRectZero style:UITableViewStylePlain];
    self.tableView.delegate = self;
    self.tableView.dataSource = self;
    self.tableView.backgroundColor = self.fs_backgroundColor_B9;
    self.tableView.separatorStyle = UITableViewCellSeparatorStyleNone;
    [self.view addSubview:self.tableView];
    [self.tableView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.left.equalTo(self.view.mas_left);
        make.right.equalTo(self.view.mas_right);
        make.top.equalTo(self.navBar.mas_bottom);
        make.bottom.equalTo(self.view.mas_bottom);
    }];
    self.navBarLeftTitle = FSLocalizedForKey(@"kClose");
    self.title = [self currentTagItem].title;
    self.navBar.hasDivide = YES;
    @weakify(self)
    self.navBar.leftItem.onTapClick = ^(FSItem *_item) {
        @strongify(self)
        [self dismissViewControllerAnimated:NO completion:^{
            if (self.isMove) {
                [self currentTagItem].readToolbarItems = self.readToolbarItems;
            }
        }];

    };
    [self loadData];
}

- (void)loadData{
    self.readToolbarItems = [NSMutableArray array];
    self.tableView.editing = YES;
    self.isMove = NO;
    for (FSReadToolbarItem * item in [self currentTagItem].readToolbarItems) {
        if(item.customItem != nil &&
           (item.customItem.itemType == FSMainToolbarItemTypeMultiSelect|| item.customItem.itemType  == FSMainToolbarItemTypeSortTool)){
            continue;
        }
        [self.readToolbarItems addObject:item];
    }
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView{
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    return self.readToolbarItems.count;
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    return PREFERENCE_CELL_ROW_HEIGHT_NORMAL;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{

    static NSString *cellIdentifier = @"sorttCellIdentifier";
    FSTitleEstimatedTableViewCell *cell = [self.tableView dequeueReusableCellWithIdentifier:cellIdentifier];

    if (!cell) {
        cell = [[FSTitleEstimatedTableViewCell alloc] initWithStyle:UITableViewCellStyleDefault reuseIdentifier:cellIdentifier];

    }
    cell.contentView.backgroundColor = self.fs_backgroundColor_B8;
    cell.backgroundColor = self.fs_backgroundColor_B8;
    FSReadToolbarItem *item = [self.readToolbarItems objectAtIndex:indexPath.row];
    cell.isAccessibilityElement = NO;
    cell.estimatedTitleLabel.textColor = self.fs_textColor_T4;
    if(item.toolbarItem.item.selected && item.selectTitle){
        cell.estimatedTitleLabel.text = item.selectTitle;
    }else{
        cell.estimatedTitleLabel.text = item.title;
    }

    if (!cell.leftView) {
        UIImageView *imageView = [UIImageView new];
        imageView.fs_size = CGSizeMake(40.f, 40.f);
        imageView.contentMode = UIViewContentModeScaleAspectFit;
        cell.leftView = imageView;
    }
    if(item.imageName){
        ((UIImageView *)cell.leftView).image = FSImageNamedAndNormalColor(item.imageName);
        if(item.settingsType == FSReadToolSettingsTypeViewDay && item.toolbarItem.item.selected){
            ((UIImageView *)cell.leftView).image = FSImageNamedAndNormalColor(@"setting_night_ipad");
        }
    }

    [cell installConstraints];
    return cell;
}

- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
    return YES;
}

- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)sourceIndexPath toIndexPath:(NSIndexPath *)destinationIndexPath
{
    if (sourceIndexPath.row == destinationIndexPath.row) {
        return;
    }
    FSReadToolbarItem *moveItem = [_readToolbarItems objectAtIndex:sourceIndexPath.row];
    [self.readToolbarItems removeObject: moveItem];
    [self.readToolbarItems insertObject:moveItem atIndex:destinationIndexPath.row];
    self.isMove = YES;
}


- (UITableViewCellEditingStyle)tableView:(UITableView *)tableView editingStyleForRowAtIndexPath:(NSIndexPath *)indexPath{
    return UITableViewCellEditingStyleNone;
}

- (BOOL)tableView:(UITableView *)tableview shouldIndentWhileEditingRowAtIndexPath:(NSIndexPath *)indexPath {
    return NO;
}

@end
