/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */
#import "UIExtensionsManager.h"
#import "UIExtensionsConfig+private.h"
#import "AnnotationPanel.h"
#import "OutlinePanel.h"
#import "CropViewController.h"
#import "PanAndZoomViewController.h"
#import "FSAnnotExtent.h"
#import "MenuControl.h"
#import "MenuItem.h"
#import "FSSettingBar+private.h"
#import "UIButton+Extensions.h"
#import "FSUndo.h"
#import "FSThumbnailCache.h"
#import "FSThumbnailViewController.h"
#import "PanelController.h"
#import "FSSettingBar.h"
#import "DynamicXFAHandler.h"
#import "AudioPlayerControl.h"

#import "AttachmentModule.h"
#import "EraseModule.h"
#import "FormModule.h"
#import "FreetextModule.h"
#import "ImageModule.h"
#import "InsertModule.h"
#import "LineModule.h"
#import "LinkModule.h"
#import "MarkupModule.h"
#import "MoreModule.h"
#import "NoteModule.h"
#import "PageNavigationModule.h"
#import "PasswordModule.h"
#import "PencilModule.h"
#import "PolygonModule.h"
#import "PolyLineModule.h"
#import "ReadingBookmarkModule.h"
#import "ReflowModule.h"
#import "ReplaceModule.h"
#import "AudioVideoModule.h"
#import "Search/SearchModule.h"
#import "SelectionModule.h"
#import "ShapeModule.h"
#import "SignToolHandler.h"
#import "FillSignModule.h"
#import "StampModule.h"
#import "TextboxModule.h"
#import "CalloutModule.h"
#import "UndoModule.h"
#import "DistanceModule.h"
#import "MultipleSelectionModule.h"
#import "RedactModule+Private.h"
#import "CommentModule.h"
#import "VoiceModule.h"
#import "SignatureModule.h"
#import "FSURIModule.h"

#import "AttachmentAnnotHandler.h"
#import "AttachmentToolHandler.h"
#import "CaretAnnotHandler.h"
#import "DigitalSignatureAnnotHandler.h"
#import "EraseToolHandler.h"
#import "FormAnnotHandler.h"
#import "FtAnnotHandler.h"
#import "FtToolHandler.h"
#import "InsertToolHandler.h"
#import "LineAnnotHandler.h"
#import "LineToolHandler.h"
#import "LinkAnnotHandler.h"
#import "NoteAnnotHandler.h"
#import "NoteToolHandler.h"
#import "PencilAnnotHandler.h"
#import "PencilToolHandler.h"
#import "PolygonAnnotHandler.h"
#import "PolygonToolHandler.h"
#import "PolyLineAnnotHandler.h"
#import "PolyLineToolHandler.h"
#import "ReplaceToolHandler.h"
#import "SelectToolHandler.h"
#import "ShapeAnnotHandler.h"
#import "ShapeToolHandler.h"
#import "SignToolHandler.h"
#import "StampAnnotHandler.h"
#import "StampIconController.h"
#import "StampToolHandler.h"
#import "ImageAnnotHandler.h"
#import "AudioVideoAnnotHandler.h"
#import "TextMKAnnotHandler.h"
#import "TextMKToolHandler.h"
#import "RedactToolHandler.h"
#import "FSURIAnnotHandler.h"
#import "MultipleSelectionToolHandler.h"
#import "FSPagerView.h"
#import "FSPageColorView.h"
#import "FSCropSelectAlertController.h"
#import "MenuView+Private.h"
#import "FSUndoGroup.h"
#import "AnnotationSignature.h"
#import "FSPermissionUtilty.h"
#import "FSHistoryDataModule+Private.h"
#import "FSAppInfoProvider.h"
#import "FSPermissionProvider+Private.h"

FSSYNTH_INTERFACE_IMP_CLASS(_FSPDFViewCtrlContainerView, UIView)

@interface UIExtensionsManager ()
@property (nonatomic, strong) FSReadToolSettings *lastPropertyToolSettings;
@property (nonatomic, strong) DevicePosition *devicePos;
@end

@implementation UIExtensionsManager {
    UIControl *maskView;
}

@synthesize prefersStatusBarHidden = _prefersStatusBarHidden;

+ (UIStatusBarStyle)preferredStatusBarStyle{
    return [FSPreferences sharePreferences].preferredStatusBarStyle;
}

+ (void)setPreferredStatusBarStyle:(UIStatusBarStyle)preferredStatusBarStyle{
    [FSPreferences sharePreferences].preferredStatusBarStyle = preferredStatusBarStyle;
}

+ (UIColor *)primaryColor{
    return [FSPreferences sharePreferences].primaryColor;
}

+ (void)setPrimaryColor:(UIColor *)primaryColor{
    [FSPreferences sharePreferences].backgroundColor_B12 =  UIColor_DarkMode( primaryColor.fs_lightColor,[FSPreferences sharePreferences].backgroundColor_B12.fs_darkColor);
    [FSPreferences sharePreferences].iconColor_I7 =  UIColor_DarkMode([FSPreferences sharePreferences].iconColor_I7.fs_lightColor, primaryColor.fs_darkColor);
    [FSPreferences sharePreferences].backgroundColor_B13 =  UIColor_DarkMode([FSPreferences sharePreferences].backgroundColor_B13.fs_lightColor, primaryColor.fs_darkColor);
    [FSPreferences sharePreferences].textColor_T10 =  UIColor_DarkMode(primaryColor.fs_lightColor, [FSPreferences sharePreferences].textColor_T10.fs_darkColor);
    if([FSPreferences sharePreferences].stateBarColor){
        [FSPreferences sharePreferences].stateBarColor = UIColor_DarkMode(primaryColor.fs_lightColor, [FSPreferences sharePreferences].stateBarColor.fs_darkColor);
    }
    [FSPreferences sharePreferences].primaryColor = primaryColor;
}

+ (UIColor *)stateBarColor{
    return [FSPreferences sharePreferences].stateBarColor;
}

+ (void)setStateBarColor:(UIColor *)stateBarColor{
    [FSPreferences sharePreferences].stateBarColor = stateBarColor;
}

#pragma mark - Life cycle
- (id)initWithPDFViewControl:(FSPDFViewCtrl *)viewctrl {
    return [self initWithPDFViewControl:viewctrl configuration:nil];
}

- (id)initWithPDFViewControl:(FSPDFViewCtrl *)viewctrl configuration:(NSData *_Nullable)jsonConfigData {
    UIExtensionsConfig *config;
    if (jsonConfigData) {
        config = [[UIExtensionsConfig alloc] initWithJSONData:jsonConfigData];
        if (!config) {
            UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kInvalidJSON") message:FSLocalizedForKey(@"kFailedLoadJSON") preferredStyle:UIAlertControllerStyleAlert];
            UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                                   style:UIAlertActionStyleCancel
                                                                 handler:nil];
            [alertController addAction:cancelAction];
            [[viewctrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
            return nil;
        }
    } else {
        config = [[UIExtensionsConfig alloc] init];
    }
    return [self initWithPDFViewControl:viewctrl configurationObject:config];
}

- (id)initWithPDFViewControl:(FSPDFViewCtrl *)viewctrl configurationObject:(UIExtensionsConfig *_Nonnull)configuration {
    self = [super init];
    
#if DEBUG
    [FSFPSMonitor setHidden:NO];
#endif
    
    if ([[FSPDFViewCtrl openedPdfViewCtrls] containsObject:viewctrl]) {
        [UIExtensionsManager addExtensionsManager:self];
    }
    
    _pdfViewCtrl = viewctrl;
    FSItem.appearanceDisableTitleColor = self.fs_textColor_T2;
    self.fs_navBarBackIcon = FSImageNamedAndNormalColor(@"common_back_gray");
    [[IQKeyboardManager sharedManager] setEnable:YES];
    [[IQKeyboardManager sharedManager] setShouldResignOnTouchOutside:YES];
    [[IQKeyboardManager sharedManager] setEnableAutoToolbar:NO];
    
    
    _viewCtrlManager = [[FSViewControllerManager alloc] initWithUIExtensionsManager:self];
    _toolbarManager = [[FSToolbarManager alloc] initWithUIExtensionsManager:self];
    _fullScreenToolbarMgr = [[FSFullScreenToolbarMgr alloc] initWithUIExtensionsManager:self];
    _menuViewManager = [[FSMenuViewManager alloc] initWithUIExtensionsManager:self];
    _containerView = [[FSPDFViewCtrlContainerView alloc] initWithExtensionsManager:self];
    _cropManager = [[FSCropManager alloc] initWithExtensionsManager:self];
    
    self.config = configuration;
    self.currentPageIndex = -1;
    self.lastWindowHorizontalSizeClass = self.pdfViewCtrl.fs_currentSceneKeyWindow.traitCollection.horizontalSizeClass;
    
    self.currentState = FSUIManagerStateNormal;
    self.lastManagerStateWhenAnnotTool = self.currentState;

    self.toolHandlers = [NSMutableArray array];
    self.annotHandlers = [NSMutableArray array];
    self.annotListeners = [NSMutableArray array];
    self.toolListeners = [NSMutableArray array];
    self.searchListeners = [NSMutableArray array];
    self.currentToolHandler = nil;
    self.rotateListeners = [[NSMutableArray alloc] init];
    self.annotPropertyListeners = [NSMutableArray<IAnnotPropertyListener> array];
    self.windowFrameListeners = [NSMutableArray<IWindowFrameChangedListener> array];
    self.propertyViewListeners = [NSMutableArray<IPropertyViewEventListener> array];
    //_pdfViewCtrl.nightColorMode = FSNightColorModeDefault;
    
    self.highlightFormColor = self.config.defaultSettings.highlightFormColor;
    self.enableLinks =  !self.config.disableLink;
    self.enableHighlightLinks = self.config.defaultSettings.highlightLink;
    self.noteIcon = [Utility getIconTypeWithIconName:self.config.defaultSettings.annotations.note.icon annotType:FSAnnotNote];
    self.attachmentIcon = [Utility getIconTypeWithIconName:self.config.defaultSettings.annotations.attachment.icon annotType:FSAnnotFileAttachment];
    self.linksHighlightColor = self.config.defaultSettings.highlightLinkColor;
    self.selectionHighlightColor = [UIColor colorWithRed:0 green:0 blue:1 alpha:0.3];
    self.securityHandlers = [NSMutableArray array];
    self.isDocModified = NO;
    self.docSaveFlag = FSPDFDocSaveFlagIncremental;
    self.taskServer = [[FSTaskServer alloc] init];

    self.undoItems = [NSMutableArray<UndoItem *> array];
    self.redoItems = [NSMutableArray<UndoItem *> array];
    self.undoListeners = [NSMutableArray<IFSUndoEventListener> array];

    self.shouldShowMenu = NO;
    self.shouldShowPropertyBar = NO;
    self.isRotating = NO;
    self.isScrollViewDragging = NO;
    self.isScrollViewDecelerating = NO;
    self.isScrollViewZooming = NO;
    _isFullScreen = self.config.defaultSettings.fullscreen;
    _enableTopToolbar = YES;
    _enableBottomToolbar = YES;
    _isSuspendAutoFullScreen = NO;
    self.continueAddAnnot = self.config.defaultSettings.annotations.continuouslyAdd;
    _annotAuthor = [[UIDevice currentDevice].name stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
    _enableHighlightForm = self.config.defaultSettings.highlightForm;
    _canUpdateAnnotDefaultProperties = YES;
    _enableModifyDocument = YES;
    _disableFingerWhenLinkApplePencil = YES;
    
    [self buildToolbars];
    [self buildItems];

    [_pdfViewCtrl registerLayoutEventListener:self];
    [_pdfViewCtrl registerDrawEventListener:self];
    [_pdfViewCtrl registerGestureEventListener:self];
    [_pdfViewCtrl registerRecoveryEventListener:self];
    [_pdfViewCtrl registerDocEventListener:self];
    [_pdfViewCtrl registerPageEventListener:self];
    [_pdfViewCtrl registerScrollViewEventListener:self];
    [_toolbarManager registerEventListens];
    [_viewCtrlManager registerEventListens];
    [_fullScreenToolbarMgr registerEventListens];
    
    
    [self.pdfViewCtrl.fs_currentSceneKeyWindow addObserver:self forKeyPath:@"frame" options:NSKeyValueObservingOptionNew|NSKeyValueObservingOptionOld context:nil];
    
    self.panelController = [[FSPanelController alloc] initWithExtensionsManager:self];
    [self.panelController registerPanelChangedListener:self.toolbarManager];
    [_pdfViewCtrl.fs_viewController addChildViewController:self.panelController];
    self.panelController.isHidden = YES;
    
    self.propertyMainView = [[FSPropertyMainView alloc] initWithUIExtensionsManager:self];
    self.propertyMainView.delegate = self;

    NSMutableArray *modules = [NSMutableArray array];
    if (self.config.loadPageNavigation)
        [modules addObject:[[PageNavigationModule alloc] initWithUIExtensionsManager:self]];
    if (self.config.loadSearch)
        [modules addObject:[[SearchModule alloc] initWithUIExtensionsManager:self]];
    if (self.config.loadReadingBookmark)
        [modules addObject:[[ReadingBookmarkModule alloc] initWithUIExtensionsManager:self]];
    [modules addObject:[[MoreModule alloc] initWithUIExtensionsManager:self]];
    if (self.config.loadForm)
        [modules addObject:[[FormModule alloc] initWithUIExtensionsManager:self]];
    
    NSSet *tools = self.config.tools;
    if ([tools fs_containsAnyObjectNotInArray:@[ Tool_Multiple_Selection ]])
        [modules addObject:[[MultipleSelectionModule alloc] initWithUIExtensionsManager:self]];
    
    if ([tools fs_containsAnyObjectInSet:[UIExtensionsConfig annotTools]] || [tools fs_containsAnyObjectInArray:@[Tool_Fill_Sign]]) {
        [modules addObject:[[UndoModule alloc] initWithUIExtensionsManager:self]];
    }
    // annotations
    if ([tools fs_containsAnyObjectInArray:@[ Tool_Highlight, Tool_Underline, Tool_Squiggly, Tool_Strikeout ]]) {
        [modules addObject:[[MarkupModule alloc] initWithUIExtensionsManager:self]];
    }
    if ([tools containsObject:Tool_Note]) {
        [modules addObject:[[NoteModule alloc] initWithUIExtensionsManager:self]];
    }
    if ([tools fs_containsAnyObjectInArray:@[ Tool_Oval, Tool_Rectangle ]]) {
        [modules addObject:[[ShapeModule alloc] initWithUIExtensionsManager:self]];
    }
    if ([tools containsObject:Tool_Freetext]) {
        [modules addObject:[[FreetextModule alloc] initWithUIExtensionsManager:self]];
    }
    if ([tools containsObject:Tool_Textbox]) {
        [modules addObject:[[TextboxModule alloc] initWithUIExtensionsManager:self]];
    }
    if ([tools containsObject:Tool_Callout]) {
        [modules addObject:[[CalloutModule alloc] initWithUIExtensionsManager:self]];
    }
    if ([tools containsObject:Tool_Pencil]) {
        [modules addObject:[[PencilModule alloc] initWithUIExtensionsManager:self]];
    }
    if ([tools containsObject:Tool_Eraser]) {
        [modules addObject:[[EraseModule alloc] initWithUIExtensionsManager:self]];
    }
    if ([tools fs_containsAnyObjectInArray:@[ Tool_Line, Tool_Arrow ]]) {
        [modules addObject:[[LineModule alloc] initWithUIExtensionsManager:self]];
    }
    if ([tools fs_containsAnyObjectInArray:@[ Tool_Distance ]]) {
        [modules addObject:[[DistanceModule alloc] initWithUIExtensionsManager:self]];
    }
    if ([tools containsObject:Tool_Stamp]) {
        [modules addObject:[[StampModule alloc] initWithUIExtensionsManager:self]];
    }
    if ([tools containsObject:Tool_Insert]) {
        [modules addObject:[[InsertModule alloc] initWithUIExtensionsManager:self]];
    }
    if ([tools containsObject:Tool_Replace]) {
        [modules addObject:[[ReplaceModule alloc] initWithUIExtensionsManager:self]];
    }
    if ([tools containsObject:Tool_Image]) {
        [modules addObject:[[ImageModule alloc] initWithUIExtensionsManager:self]];
    }
    if ([tools fs_containsAnyObjectInArray:@[ Tool_Polygon, Tool_Cloud ]]) {
        [modules addObject:[[PolygonModule alloc] initWithUIExtensionsManager:self]];
    }
    if ([tools containsObject:Tool_PolyLine]) {
        [modules addObject:[[PolyLineModule alloc] initWithUIExtensionsManager:self]];
    }
    /// rdk 6.2 add
    if ([tools fs_containsAnyObjectInArray:@[ Tool_Audio, Tool_Video ]]) {
        [modules addObject:[[AudioVideoModule alloc] initWithUIExtensionsManager:self]];
        /// rdk 7.1 add
        if ([tools containsObject:Tool_Audio]) {
            [modules addObject:[[VoiceModule alloc] initWithUIExtensionsManager:self]];
        }
    }
    /// rdk 7.0 add
    if ([tools containsObject:Tool_Redaction]) {
        [modules addObject:[[RedactModule alloc] initWithUIExtensionsManager:self]];
    }
    if (self.config.loadAttachment) {
        [modules addObject:[[AttachmentModule alloc] initWithUIExtensionsManager:self]];
    }
    if (self.config.loadThumbnail) {
        self.thumbnailModule =  [[ThumbnailModule alloc] initWithExtensionsManager:self];
        [modules addObject:self.thumbnailModule];
    }

    [modules addObject:[[ReflowModule alloc] initWithUIExtensionsManager:self]];
    
    if (self.config.loadSignature)
         [modules addObject:[[SignatureModule alloc] initWithExtensionsManager:self]];
    if (self.config.fillSign){
        self.fillSignModule = [[FillSignModule alloc] initWithUIExtensionsManager:self];
        [modules addObject:self.fillSignModule];
    }
    if ([self.config.tools containsObject:Tool_Select])
        [modules addObject:[[SelectionModule alloc] initWithUIExtensionsManager:self]];
    
    [modules addObject:[[LinkModule alloc] initWithUIExtensionsManager:self]];
    
    self.passwordModule = [[PasswordModule alloc] initWithExtensionsManager:self];
    [modules addObject:self.passwordModule];
    
    [modules addObject:[[CommentModule alloc] initWithExtensionsManager:self]];
    
    self.speechModule =  [[SpeechModule alloc] initWithExtensionsManager:self];
    [modules addObject:self.speechModule];
    
    [modules addObject:[[FSURIModule alloc] initWithUIExtensionsManager:self]];
    
    [modules addObject:[[FSHistoryDataModule alloc] initWithExtensionsManager:self]];
    
    self.autoFlipModule =  [[FSAutoFlipModule alloc] initWithExtensionsManager:self];
    
    self.panZoomView = [[PanZoomView alloc] initWithUIExtensionsManager:self];
    
    self.modules = modules;
    self.menuControl = [[MenuControl alloc] initWithUIExtensionsManager:self];
    self.menuControl.delegate = self;
    
    [self.toolbarManager rangeItems];

    DynamicXFAHandler* xfaAnnotHandler = [[DynamicXFAHandler alloc] initWithUIExtensionsManager:self];
    [self registerAnnotHandler:xfaAnnotHandler];
    [_pdfViewCtrl registerDocEventListener:xfaAnnotHandler];

    FSCallbackSubstitute *annotIconProviderCallbackSubstitute = [FSCallbackSubstitute annotIconProviderCallbackSubstitute];
    _iconProvider = [[ExAnnotIconProviderCallback alloc] init];
    annotIconProviderCallbackSubstitute.currentTarget = _iconProvider;
    [FSLibrary setAnnotIconProviderCallback:annotIconProviderCallbackSubstitute];
    
    FSCallbackSubstitute *actionHandlerSubstitute = [FSCallbackSubstitute actionHandlerSubstitute];
    ExActionHandler *actionHandler = [[ExActionHandler alloc] initWithExtensionsManager:self];
    _actionHandler = actionHandler;
    actionHandlerSubstitute.currentTarget = _actionHandler;
    [FSLibrary setActionCallback:actionHandlerSubstitute];
    
    self.permissionProvider = [[FSPermissionProvider alloc] init];
    self.appInfoProvider = [[FSAppInfoProvider alloc] init];
    self.presentedVCAnimator = [[FSPresentedViewControllerAnimator alloc] initWithViewCtrl:_pdfViewCtrl];

    [FSLibrary enableJavaScript:configuration.runJavaScript];

    UISettingsModel *seting = self.config.defaultSettings;
    
    self.pdfViewCtrl.reflowBackgroundColor = seting.reflowBackgroundColor;
    
    if ([seting.zoomMode isEqualToString:@"FitWidth"]) {
        [self.settingBar.viewSettingManager setItemSelectedWithType:FITWIDTH selected:YES];
        [self.pdfViewCtrl setZoomMode:PDF_DISPLAY_ZOOMMODE_FITWIDTH];
    }else if ([seting.zoomMode isEqualToString:@"FitPage"]){
        [self.settingBar.viewSettingManager setItemSelectedWithType:FITPAGE selected:YES];
        [self.pdfViewCtrl setZoomMode:PDF_DISPLAY_ZOOMMODE_FITPAGE];
    }

    if ([seting.colorMode isEqualToString:@"Normal"]) {
        [self.settingBar.viewSettingManager setItemSelectedWithType:DAYMODE selected:YES];
        [self.pdfViewCtrl setColorMode:FSRendererColorModeNormal];
    }else if ([seting.colorMode isEqualToString:@"Night"]){
        [self.settingBar.viewSettingManager setItemSelectedWithType:NIGHTMODE selected:YES];
        [self.settingBar didClickItemWithType:NIGHTMODE];
    }else if ([seting.colorMode isEqualToString:@"Map"]){
        if (self.pdfViewCtrl.isNightMode) {
            self.pdfViewCtrl.isNightMode = NO;
        }
        self.pdfViewCtrl.mappingModeBackgroundColor = seting.mapBackgroundColor;
        self.pdfViewCtrl.mappingModeForegroundColor = seting.mapForegroundColor;
        [self.pdfViewCtrl setColorMode:FSRendererColorModeMapping];
        [self.settingBar.viewSettingManager setItemSelectedWithType:PAGECOLOR selected:YES];
        [self.settingBar.viewSettingManager setItemPageColor:seting.mapBackgroundColor];
    }else if ([seting.colorMode isEqualToString:@"MapGray"]){
        if (self.pdfViewCtrl.isNightMode) {
            self.pdfViewCtrl.isNightMode = NO;
        }
        self.pdfViewCtrl.mappingModeBackgroundColor = seting.mapBackgroundColor;
        self.pdfViewCtrl.reflowBackgroundColor = seting.mapBackgroundColor;
        [self.pdfViewCtrl setColorMode:FSRendererColorModeMappingGray];
        [self.settingBar.viewSettingManager setItemSelectedWithType:PAGECOLOR selected:YES];
        [self.settingBar.viewSettingManager setItemPageColor:seting.mapBackgroundColor];
    }
    
    if ([seting.pageMode isEqualToString:@"Single"]) {
        [self.settingBar didClickItemWithType:SINGLE];
        [self.settingBar.viewSettingManager setItemSelectedWithType:SINGLE selected:YES];
    }else if ([seting.pageMode isEqualToString:@"Reflow"]){
        [self.settingBar didClickItemWithType:REFLOW];
        [self.settingBar.viewSettingManager setItemSelectedWithType:REFLOW selected:YES];
    }
    
    if ([seting.pageMode isEqualToString:@"Facing"]) {
        [self.settingBar didClickItemWithType:DOUBLEPAGE];
        [self.settingBar.viewSettingManager setItemSelectedWithType:DOUBLEPAGE selected:YES];
    }else if ([seting.pageMode isEqualToString:@"CoverLeft"]){
        [self.pdfViewCtrl setPageLayoutMode:PDF_LAYOUT_MODE_TWO_LEFT];
    }else if ([seting.pageMode isEqualToString:@"CoverMiddle"]){
        [self.pdfViewCtrl setPageLayoutMode:PDF_LAYOUT_MODE_TWO_MIDDLE];
    }else if ([seting.pageMode isEqualToString:@"CoverRight"]){
        [self.settingBar didClickItemWithType:COVERPAGE];
        [self.settingBar.viewSettingManager setItemSelectedWithType:COVERPAGE selected:YES];
    }
    
    if (seting.continuous == YES && ![seting.pageMode isEqualToString:@"Reflow"]) {
        [self.settingBar.viewSettingManager setContinuousON:YES];
        [self.settingBar didClickItemWithType:CONTINUOUS];
    }else if(seting.continuous == NO && ![seting.pageMode isEqualToString:@"Reflow"]) {
        [self.settingBar.viewSettingManager setContinuousON:NO];
        [self.settingBar didClickItemWithType:CONTINUOUS];
    }
    
    return self;
}

- (void)dealloc
{
    [self.panelController removeFromParentViewController];
    [_pdfViewCtrl unregisterLayoutEventListener:self];
    [_pdfViewCtrl unregisterDrawEventListener:self];
    [_pdfViewCtrl unregisterGestureEventListener:self];
    [_pdfViewCtrl unregisteRecoveryEventListener:self];
    [_pdfViewCtrl unregisterDocEventListener:self];
    [_pdfViewCtrl unregisterPageEventListener:self];
    [_pdfViewCtrl unregisterScrollViewEventListener:self];
    
    @try {
        [self.pdfViewCtrl.fs_currentSceneKeyWindow removeObserver:self forKeyPath:@"frame"];
    } @catch (NSException *exception) {
        
    }
}

#pragma mark Lazy load

- (NSMutableDictionary *)defaultReadToolSettingItems{
    if(!_defaultReadToolSettingItems){
        _defaultReadToolSettingItems = @{}.mutableCopy;
    }
    return _defaultReadToolSettingItems;
}

- (NSMutableArray *)stateChangeListeners{
    if(!_stateChangeListeners){
        _stateChangeListeners = @[].mutableCopy;
    }
    return _stateChangeListeners;
}

- (NSMutableArray *)fullScreenListeners{
    if(!_fullScreenListeners){
        _fullScreenListeners = @[].mutableCopy;
    }
    return _fullScreenListeners;
}

- (NSMutableArray *)docModifiedListeners{
    if(!_docModifiedListeners){
        _docModifiedListeners = @[].mutableCopy;
    }
    return _docModifiedListeners;
}

- (NSMutableArray *)pageNumberListeners{
    if(!_pageNumberListeners){
        _pageNumberListeners = @[].mutableCopy;
    }
    return _pageNumberListeners;
}

- (DynamicXFASignatureAnnotHandler *)dynamicXFASignatureAnnotHandler{
    if(!_dynamicXFASignatureAnnotHandler){
        _dynamicXFASignatureAnnotHandler = [[DynamicXFASignatureAnnotHandler alloc] initWithUIExtensionsManager:self];
        [self.pdfViewCtrl registerScrollViewEventListener:_dynamicXFASignatureAnnotHandler];
        [self registerRotateChangedListener:_dynamicXFASignatureAnnotHandler];
        [self registerAnnotHandler:_dynamicXFASignatureAnnotHandler];
    }
    return _dynamicXFASignatureAnnotHandler;
}

#pragma mark Get selector
- (FSMainTopbar *)topToolbar{
    return self.toolbarManager.mainTopbar.addedToSupview ? self.toolbarManager.mainTopbar : nil;
}

- (FSMainBottombar *)bottomToolbar{
    return self.toolbarManager.mainBottombar.addedToSupview ? self.toolbarManager.mainBottombar : nil;
}

- (FSFullScreenTopbar *)smallTopToolbar{
    return self.fullScreenToolbarMgr.smallTopToolbar.superview ? self.fullScreenToolbarMgr.smallTopToolbar : nil;
}

- (FSFullScreenBottombar *)smallBottomToolbar{
    return self.fullScreenToolbarMgr.smallBottomToolbar.superview ? self.fullScreenToolbarMgr.smallBottomToolbar : nil;
}

#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
- (UIWindowScene *)windowScene{
    return self.pdfViewCtrl.windowScene;
}
#endif

#pragma mark <SettingBarDelegate>

- (void)settingBarSinglePageLayout:(FSSettingBar *)settingBar {
    if ([self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_SINGLE) {
        return;
    }
    [self.pdfViewCtrl setPageLayoutMode:PDF_LAYOUT_MODE_SINGLE];
    
    for (id<IPageNumberListener> listener in self.pageNumberListeners) {
        if ([listener respondsToSelector:@selector(updatePageNumber)]) {
            [listener updatePageNumber];
        }
    }
}

- (void)settingBarContinuousLayout:(FSSettingBar *)settingBar {
    
    for (id<IPageNumberListener> listener in self.pageNumberListeners) {
        if ([listener respondsToSelector:@selector(updatePageNumber)]) {
            [listener updatePageNumber];
        }
    }
}

- (void)settingBarDoublePageLayout:(FSSettingBar *)settingBar {
    if ([self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO) {
        return;
    }
    [self.pdfViewCtrl setPageLayoutMode:PDF_LAYOUT_MODE_TWO];

    for (id<IPageNumberListener> listener in self.pageNumberListeners) {
        if ([listener respondsToSelector:@selector(updatePageNumber)]) {
            [listener updatePageNumber];
        }
    }
}

- (void)settingBarCoverPageLayout:(FSSettingBar *)settingBar {
    if ([self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_TWO_RIGHT) {
        return;
    }
    [self.pdfViewCtrl setPageLayoutMode:PDF_LAYOUT_MODE_TWO_RIGHT];
    
    for (id<IPageNumberListener> listener in self.pageNumberListeners) {
        if ([listener respondsToSelector:@selector(updatePageNumber)]) {
            [listener updatePageNumber];
        }
    }
}

- (void)settingBarThumbnail:(FSSettingBar *)settingBar {;
    [self.thumbnailModule showThumbnailView];
    self.hiddenSettingBar = YES;
}

- (void)settingBarReflow:(FSSettingBar *)settingBar {
    [self setCurrentToolHandler:nil];
    [self changeState:FSUIManagerStateReflow];
}

- (void)enterReflowMode:(BOOL)flag{
    BOOL isReflow = [self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_REFLOW;
    if (isReflow == flag) {
        return;
    }
    if (self.pdfViewCtrl.colorMode == FSRendererColorModeNormal) {
        UISettingsModel *seting = self.config.defaultSettings;
        self.pdfViewCtrl.reflowBackgroundColor = seting.reflowBackgroundColor;
    }
    else{
        self.pdfViewCtrl.reflowBackgroundColor = self.pdfViewCtrl.mappingModeBackgroundColor;
    }
    
    ReflowModule *reflowModule = (ReflowModule *)[self getModuleByName:@"Reflow"];
    [reflowModule enterReflowMode:flag];
    
    self.hiddenSettingBar = YES;
}

- (void)enterRedactionMode:(BOOL)flag{

    if (flag) {
        [self setCurrentToolHandler:nil];
    }
    RedactModule *redactModule = (RedactModule *)[self getModuleByName:@"Redact"];
    [redactModule enterRedactionMode:flag];
    
    self.hiddenSettingBar = YES;
}

- (void)quitGotoMode{
    [(PageNavigationModule *)[self getModuleByName:@"PageNavigation"] quitGotoMode];
}

- (void)settingBarCrop:(FSSettingBar *)settingBar {
    [self setCropMode];
}

- (void)settingBarSpeech:(FSSettingBar *)settingBar {
    self.hiddenSettingBar = YES;
    [self changeState:FSUIManagerStateSpeech];
}

- (void)settingBarPanAndZoom:(FSSettingBar *)settingBar {
    [self changeState:FSUIManagerStatePanZoom];
}

- (void)settingBar:(FSSettingBar *)settingBar isLockScreen:(BOOL)isLockScreen {
    _isScreenLocked = isLockScreen;
    self.hiddenSettingBar = YES;
}

- (void)settingBarPageColor:(FSSettingBar *)settingBar
{
    if (!(DEVICE_iPHONE || [_pdfViewCtrl fs_currentSceneiPadHorizontalCompact])) {
        return;
    }
    [self cancelAutoFullScreen];
    [self.settingBar.viewSettingManager showPageColorView:YES isReOpen:NO];
    return;
}

- (void)settingBar:(FSSettingBar *)settingBar isNightMode:(BOOL)isNightMode {

    self.settingBar.viewSettingManager.isDayOrNightForLast = YES;
    if ([self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_REFLOW) {
        UIColor* color = isNightMode ? UIColor.blackColor : UIColor.whiteColor;
        [self.pdfViewCtrl setReflowBackgroundColor:color];
        [self.pdfViewCtrl setPageLayoutMode:PDF_LAYOUT_MODE_REFLOW];
        self.pdfViewCtrl.isNightMode = isNightMode;
    }else{
        if (!(self.pdfViewCtrl.isNightMode == isNightMode && self.pdfViewCtrl.colorMode != FSRendererColorModeMappingGray)) {
            self.pdfViewCtrl.isNightMode = isNightMode;
            if (self.pdfViewCtrl.nightColorMode == FSNightColorModeMappingGray && isNightMode) {
                self.pdfViewCtrl.colorMode = FSRendererColorModeMappingGray;
            }
        }else if (!isNightMode){
            if (self.pdfViewCtrl.isNightMode != isNightMode){
                self.pdfViewCtrl.isNightMode = isNightMode;
            }
            if (self.pdfViewCtrl.colorMode != FSRendererColorModeNormal){
                [self.pdfViewCtrl setColorMode:FSRendererColorModeNormal];
            }
        }
    }
}

- (void)settingBarFitPage:(FSSettingBar *)settingBar {
    [self.pdfViewCtrl setZoomMode:PDF_DISPLAY_ZOOMMODE_FITPAGE];
}

- (void)settingBarFitWidth:(FSSettingBar *)settingBar {
    [self.pdfViewCtrl setZoomMode:PDF_DISPLAY_ZOOMMODE_FITWIDTH];
}

- (void)settingBarRotate:(FSSettingBar *)settingBar {
    int viewRotation = [self.pdfViewCtrl getViewRotation];
    viewRotation = (viewRotation + 1 ) % 4 ;
    [self.pdfViewCtrl rotateView:viewRotation];
    [self.pdfViewCtrl setPageLayoutMode:[self.pdfViewCtrl getPageLayoutMode]];
}

- (void)settingBarAutoFlip:(FSSettingBar *)settingBar
{
    self.hiddenSettingBar = YES;
    [self changeState:FSUIManagerStateAutoFlip];
}

- (void)settingBarDidChangeSize:(FSSettingBar *)settingBar {
 
}

#pragma mark - IPageEventListener

- (void)onPagesRemoved:(NSArray<NSNumber *> *)indexes {
    self.isDocModified = YES;
    self.docSaveFlag = FSPDFDocSaveFlagXRefStream;
    // remove all undo/redo items in removed pages
    NSPredicate *predicate = [NSPredicate predicateWithBlock:^BOOL(UndoItem *undoItem, NSDictionary *bindings) {
        if ([undoItem isKindOfClass:[FSUndoGroup class]]) {
            FSUndoGroup *undoGroup = (FSUndoGroup*)undoItem;
            NSArray<NSNumber*> *indexArray = [undoGroup getIndexArray];
            [undoGroup removePageIndex:indexes];
            return indexArray.count;
        }else{
            return ![indexes containsObject:[NSNumber numberWithInt:undoItem.pageIndex]];
        }
        
    }];
    [self.undoItems filterUsingPredicate:predicate];
    [self.redoItems filterUsingPredicate:predicate];

    void (^updatePageIndex)(UndoItem *item) = ^(UndoItem *item) {
        
        if ([item isKindOfClass:[FSUndoGroup class]]) {
            FSUndoGroup *itemGroup = (FSUndoGroup *)item;
            NSMutableArray *indexArray = itemGroup.pageArray;
            NSMutableArray *newIndexArray = [NSMutableArray array];
            for (NSNumber *number in indexArray) {
                
                int subcount = 0;
                for (NSNumber *x in indexes) {
                    if ([number intValue] > [x intValue])
                        subcount++;
                    
                }
                NSNumber* newNumber = [NSNumber numberWithInt:[number intValue] - subcount];
                [newIndexArray addObject:newNumber];
            }
            itemGroup.pageArray = newIndexArray;
            
        }else{
            int subcount = 0;
            for (NSNumber *x in indexes) {
                if (item.pageIndex > [x intValue])
                    subcount++;
            }
            item.pageIndex -= subcount;
        }
        

    };

    [self.undoItems enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        updatePageIndex((UndoItem *) obj);
    }];

    [self.redoItems enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        updatePageIndex((UndoItem *) obj);
    }];

    [self undoRedoStateChanged];
}

- (void)onPagesMoved:(NSArray<NSNumber *> *)indexes dstIndex:(int)dstIndex {
    //todo wei update undo/redo
    self.isDocModified = YES;
}

- (void)onPagesRotated:(NSArray<NSNumber *> *)indexes rotation:(int)rotation {
    self.isDocModified = YES;
}

- (void)onPagesInsertedAtRange:(NSRange)range {
    self.isDocModified = YES;
    //Load Form if there it is.
    FormAnnotHandler *handler = (FormAnnotHandler *) [self getAnnotHandlerByType:FSAnnotWidget];
    if (!handler)
        return;
    [handler onDocOpened:self.pdfViewCtrl.currentDoc error:0];

    //Reset the cropMode.
    [_pdfViewCtrl setCropMode:PDF_CROP_MODE_NONE];
    [self.settingBar.viewSettingManager setItemSelectedWithType:CROPPAGE selected:NO];
}

- (void)onPageChanged:(int)oldIndex currentIndex:(int)currentIndex {
    if ([self canAutoHideTopBottomToolBar] && ![[self.pdfViewCtrl fs_currentSceneTopMostViewController] isKindOfClass:NSClassFromString(@"FSViewPopoverController")]) {
        [self delayHideTopBottomToolBar];
    }
    [self hideMenu];
}

- (void)setCropMode {
    [_cropManager showCropMode];
}

- (void)setCurrentToolHandler:(id<IToolHandler>)toolHandler {
    id<IToolHandler> lastToolHandler = _currentToolHandler;
    if (lastToolHandler != nil) {
        [lastToolHandler onDeactivate];
    }
    if (toolHandler != nil) {
        if ([self currentAnnot] != nil)
            [self setCurrentAnnot:nil];
    }

    _currentToolHandler = toolHandler;

    if (_currentToolHandler != nil) {
        [_currentToolHandler onActivate];
        
        NSString* name = [_currentToolHandler getName];
        if ([name isEqualToString:Tool_Markup] || [name isEqualToString:Tool_Shape] || [name  isEqualToString:Tool_Line] ||
            [name  isEqualToString:Tool_Distance]) {
            _pdfViewCtrl.minimumPressDuration = 0.1;
        }
        else
        {
            _pdfViewCtrl.minimumPressDuration = 0.5;
        }
    }else{
        _pdfViewCtrl.minimumPressDuration = 0.5;
    }

    for (id<IToolEventListener> listener in self.toolListeners) {
        if ([listener respondsToSelector:@selector(onToolChanged:CurrentToolName:)]) {
            [listener onToolChanged:[lastToolHandler getName] CurrentToolName:[_currentToolHandler getName]];
        }
    }
}

- (id<FSUndo>)currentUndoToolHandler{
    if (self.currentToolHandler && [self.currentToolHandler conformsToProtocol:@protocol(FSUndo)]) {
        return (id<FSUndo>)self.currentToolHandler;
    }
    return nil;
}

- (id<IToolHandler>)getToolHandlerByName:(NSString *)name {
    // tools share one tool handler
    if ([@[ Tool_Highlight, Tool_Squiggly, Tool_Strikeout, Tool_Underline ] containsObject:name]) {
        name = Tool_Markup;
    } else if ([@[ Tool_Rectangle, Tool_Oval ] containsObject:name]) {
        name = Tool_Shape;
    } else if ([name isEqualToString:Tool_Arrow]) {
        name = Tool_Line;
    }else if ([name isEqualToString:Tool_Audio]) {
        name = Tool_Video;
    }
    for (id<IToolHandler> toolHandler in self.toolHandlers) {
        if ([toolHandler respondsToSelector:@selector(getName)]) {
            if ([[toolHandler getName] isEqualToString:name]) {
                return toolHandler;
            }
        }
    }
    return nil;
}

- (id<IModule>)getModuleByName:(NSString *)name{
    for (id module in self.modules) {
        if ([module respondsToSelector:@selector(getName)] && [[module getName] isEqualToString:name]) {
            return module;
        }
    }
    return nil;
}

- (void)registerToolHandler:(id<IToolHandler>)toolHandler {
    if (self.toolHandlers && ![self.toolHandlers containsObject:toolHandler]) {
        [self.toolHandlers addObject:toolHandler];
    }
}

- (void)unregisterToolHandler:(id<IToolHandler>)toolHandler {
    if ([self.toolHandlers containsObject:toolHandler]) {
        [self.toolHandlers removeObject:toolHandler];
    }
}
- (void)registerAnnotHandler:(id<IAnnotHandler>)annotHandler {
    if (self.annotHandlers && ![self.annotHandlers containsObject:annotHandler]) {
        [self.annotHandlers addObject:annotHandler];
    }
}
- (void)unregisterAnnotHandler:(id<IAnnotHandler>)annotHandler {
    if ([self.annotHandlers containsObject:annotHandler]) {
        [self.annotHandlers removeObject:annotHandler];
    }
}

- (id<IAnnotHandler>)getAnnotHandlerByType:(FSAnnotType)type {
    if ([_pdfViewCtrl isDynamicXFA]) {
        return nil;
    }
    if (type == FSAnnotSquiggly || type == FSAnnotStrikeOut || type == FSAnnotUnderline) {
        type = FSAnnotHighlight;
    }
    if (type == FSAnnotSquare) {
        type = FSAnnotCircle;
    }
    for (id<IAnnotHandler> annotHandler in self.annotHandlers) {
        if ([annotHandler respondsToSelector:@selector(getType)]) {
            if ([annotHandler getType] == type) {
                return annotHandler;
            }else if ((type == FSAnnotAudio || type == FSAnnotVideo) &&
                      [annotHandler isKindOfClass:[AudioVideoAnnotHandler class]]){
                return annotHandler;
            }
        }
    }
    return nil;
}

- (id<IAnnotHandler>)getAnnotHandlerByAnnot:(FSAnnot *)annot {
    
    if (!annot.nonempty) return nil;
    FSAnnotType type = [annot getType];
    if (type == FSAnnotSquiggly || type == FSAnnotStrikeOut || type == FSAnnotUnderline) {
        type = FSAnnotHighlight;
    }
    if (type == FSAnnotSquare) {
        type = FSAnnotCircle;
    }
    if (type == FSAnnotPSInk) {
        type = FSAnnotInk;
    }
    for (id<IAnnotHandler> annotHandler in self.annotHandlers) {
        if ([annotHandler respondsToSelector:@selector(getType)]) {
            if ([annotHandler getType] == type ||
                ([annotHandler getType] == FSAnnotScreen && type == FSAnnotRichMedia)) {
                if (FSAnnotWidget == type) {
                    FSWidget *widget = [[FSWidget alloc] initWithAnnot:annot];
                    FSField *field = [widget getField];
                    FSFieldType fieldType = [field isEmpty] ? FSFieldTypeUnknown : [field getType];
                    if (FSFieldTypeSignature == fieldType && [annotHandler isKindOfClass:[DigitalSignatureAnnotHandler class]]) {
                        return annotHandler;
                    }
                    if (FSFieldTypeSignature != fieldType && [annotHandler isKindOfClass:[FormAnnotHandler class]]) {
                        return annotHandler;
                    }
                }
                else {
                    if (type == FSAnnotScreen) {
                        FSScreen *screen = [[FSScreen alloc] initWithAnnot:annot];
                        FSScreenActionType screenActiontype = screen.screenActiontype;
                        if ((screenActiontype == FSScreenActionTypeUnknow || screenActiontype == FSScreenActionTypeImage) && [annotHandler isKindOfClass:[ImageAnnotHandler class]]) return annotHandler;
                        if (screenActiontype == FSScreenActionTypeAudio && [annotHandler isKindOfClass:[AudioVideoAnnotHandler class]]) return annotHandler;
                        if (screenActiontype == FSScreenActionTypeVideo && [annotHandler isKindOfClass:[AudioVideoAnnotHandler class]]) return annotHandler;
                        if (screenActiontype == FSScreenActionTypeAdditionalActionURI && [annotHandler isKindOfClass:[FSURIAnnotHandler class]]) return annotHandler;
                        continue;
                    }else if (type == FSAnnotRichMedia){
                        if ([annotHandler isKindOfClass:[AudioVideoAnnotHandler class]]) return annotHandler;
                        continue;
                    }
                     return annotHandler;
                }
            }
        }
    }
    return nil;
}

- (void)registerDocModifiedEventListener:(id<IDocModifiedEventListener>)listener {
    if (self.docModifiedListeners) {
        [self.docModifiedListeners addObject:listener];
    }
}
- (void)unregisterDocModifiedEventListener:(id<IDocModifiedEventListener>)listener {
    if ([self.docModifiedListeners containsObject:listener]) {
        [self.docModifiedListeners removeObject:listener];
    }
}

- (void)registerAnnotEventListener:(id<IAnnotEventListener>)listener {
    if (self.annotListeners) {
        [self.annotListeners addObject:listener];
    }
}
- (void)unregisterAnnotEventListener:(id<IAnnotEventListener>)listener {
    if ([self.annotListeners containsObject:listener]) {
        [self.annotListeners removeObject:listener];
    }
}

- (void)registerToolEventListener:(id<IToolEventListener>)listener {
    if (self.toolListeners) {
        [self.toolListeners addObject:listener];
    }
}
- (void)unregisterToolEventListener:(id<IToolEventListener>)listener {
    if ([self.toolListeners containsObject:listener]) {
        [self.toolListeners removeObject:listener];
    }
}

- (void)registerUndoEventListener:(id<IFSUndoEventListener>)listener {
    if (self.undoListeners) {
        [self.undoListeners addObject:listener];
    }
}

- (void)unregisterUndoEventListener:(id<IFSUndoEventListener>)listener {
    if ([self.undoListeners containsObject:listener]) {
        [self.undoListeners removeObject:listener];
    }
}

- (void)registerSearchEventListener:(id<ISearchEventListener>)listener {
    if (self.searchListeners) {
        [self.searchListeners addObject:listener];
    }
}

- (void)unregisterSearchEventListener:(id<ISearchEventListener>)listener {
    if ([self.searchListeners containsObject:listener]) {
        [self.searchListeners removeObject:listener];
    }
}

#pragma mark <MenuControlDelegate>

- (void)menuControlDidHide:(MenuControl *)menuControl {
    if (self.isRotating || self.isScrollViewDragging || self.isScrollViewDecelerating || self.isScrollViewZooming) {
        return;
    }
    self.shouldShowMenu = NO;
}

- (void)menuControlDidShow:(MenuControl *)menuControl {
    if (self.isRotating || self.isScrollViewDragging || self.isScrollViewDecelerating || self.isScrollViewZooming) {
        return;
    }
    self.shouldShowMenu = YES;
}

- (void)hideMenu {
    if (self.menuControl.isMenuVisible) {
        [self.menuControl hideMenu];
    }
    
    FillSignMenu *fillSignMenu = self.fillSignModule.fillSignMenu;
    if (fillSignMenu.isMenuVisible) {
        [fillSignMenu setMenuVisible:NO animated:NO];
    }
}

- (void)showMenu {
    if (self.currentToolHandler && [self.currentToolHandler respondsToSelector:@selector(showMenu)]) {
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.3 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
            if (self.currentToolHandler && [self.currentToolHandler respondsToSelector:@selector(showMenu)]) {
                [(id<IMenuEventToolHandler>)self.currentToolHandler showMenu];
            }
        });
        return;
    }

    if (self.currentAnnot == nil || [self.currentAnnot isEmpty]) {
        return;
    }
    id<IAnnotHandler> annotHandler = [self getAnnotHandlerByAnnot:self.currentAnnot];
    if (annotHandler == nil) {
        return;
    }
    
    CGRect dvRect = CGRectNull;
    if (self.currentAnnot) {
        if (![self.pdfViewCtrl isPageVisible:self.currentAnnot.pageIndex]) {
            return;
        }
        int pageIndex = self.currentAnnot.pageIndex;
        FSAnnot *annot = self.currentAnnot;
        CGRect pvRect = [_pdfViewCtrl convertPdfRectToPageViewRect:annot.fsrect pageIndex:pageIndex];
        if ([annot getFlags] & FSAnnotFlagNoRotate || [annot getFlags] & FSAnnotFlagNoZoom || [annot getType] == FSAnnotNote) {
            pvRect = [Utility getAnnotRect:annot pdfViewCtrl:self.pdfViewCtrl];
        }
        dvRect = [_pdfViewCtrl convertPageViewRectToDisplayViewRect:pvRect pageIndex:pageIndex];
    } else {
        int pageIndex = self.currentPageIndex;
        CGPoint pvPoint = self.currentPoint;
        CGRect pvRect = CGRectMake(pvPoint.x, pvPoint.y, 2, 2);
        dvRect = [_pdfViewCtrl convertPageViewRectToDisplayViewRect:pvRect pageIndex:pageIndex];
    }
    if (CGRectIsEmpty(dvRect) || CGRectIsNull(CGRectIntersection(dvRect, self.pdfViewCtrl.bounds))) {
        return;
    }
    [self.menuControl setRect:dvRect];
    [self.menuControl showMenu];
}

- (void)showMenuIfNeed{
    if (!self.shouldShowPropertyBar) {
        if (self.shouldShowMenu) {
            [self showMenu];
        }
    }
}

#pragma mark <FSPropertyMainViewDelegate>

- (void)propertyMainViewDidShow:(FSPropertyMainView *)propertyMainView{
    self.shouldShowPropertyBar = YES;
    
    if (self.propertyViewListeners.count > 0) {
        for (id<IPropertyViewEventListener> listener in self.propertyViewListeners) {
            if ([listener respondsToSelector:@selector(onPropertyViewDidShow)]) {
                [listener onPropertyViewDidShow];
            }
        }
    }
}

- (void)propertyMainViewDidDismiss:(FSPropertyMainView *)propertyMainView{
    self.currentToolSettings = nil;
    self.shouldShowPropertyBar = NO;
    if (self.shouldShowMenu) {
        [self showMenu];
    }
    
    if (self.propertyViewListeners.count > 0) {
        for (id<IPropertyViewEventListener> listener in self.propertyViewListeners) {
            if ([listener respondsToSelector:@selector(onPropertyViewDidDismiss)]) {
                [listener onPropertyViewDidDismiss];
            }
        }
    }
}

- (void)hidePropertyBar {
    if (self.propertyMainView.isShowing) {
        [self.propertyMainView dismiss];
    }
}

- (void)showPropertyBar {
    if (self.currentAnnot && self.currentAnnot.pageIndex == [self.pdfViewCtrl getCurrentPage]) {
        int pageIndex = self.currentAnnot.pageIndex;
        CGRect pvRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:self.currentAnnot.fsrect pageIndex:pageIndex];
        CGRect dvRect = [self.pdfViewCtrl convertPageViewRectToDisplayViewRect:pvRect pageIndex:pageIndex];
        DevicePosition *devicePos = [[DevicePosition alloc] init];
        devicePos.phoneRect = dvRect;
        devicePos.inPhoneView = self.pdfViewCtrl;
        devicePos.padRect = dvRect;
        devicePos.inPadView = self.pdfViewCtrl;
        [self.propertyMainView showPropertyMainView:devicePos];
    }
}

#pragma mark - IGestureEventListener

- (void)registerGestureEventListener:(id<IGestureEventListener>)listener {
    if (!self.guestureEventListeners) {
        self.guestureEventListeners = [NSMutableArray<IGestureEventListener> array];
    }
    if (![self.guestureEventListeners containsObject:listener]) {
        [self.guestureEventListeners addObject:listener];
    }
}

- (void)unregisterGestureEventListener:(id<IGestureEventListener>)listener {
    [self.guestureEventListeners removeObject:listener];
}

- (void)comment {
    [self onUIInteractElementClicked:FSUIElementReadingLongPressComment];
    
    float pageWidth = [_pdfViewCtrl getPageViewWidth:self.currentPageIndex];
    float pageHeight = [_pdfViewCtrl getPageViewHeight:self.currentPageIndex];

//    float scale = pageWidth / 1000.0;
    
    CGPoint pvPoint = self.currentPoint;

    if (pvPoint.x > pageWidth - NOTE_ANNOTATION_WIDTH )
        pvPoint.x = pageWidth - NOTE_ANNOTATION_WIDTH ;
    if (pvPoint.y > pageHeight - NOTE_ANNOTATION_WIDTH )
        pvPoint.y = pageHeight - NOTE_ANNOTATION_WIDTH ;

//    CGRect rect = CGRectMake(pvPoint.x , pvPoint.y , NOTE_ANNOTATION_WIDTH , NOTE_ANNOTATION_WIDTH );
    float scale = [_pdfViewCtrl getPageViewWidth:self.currentPageIndex] / 1000.0;
    
    FSRectF *iconfsrect = [[FSRectF alloc] initWithLeft1:0 bottom1:0 right1:NOTE_ANNOTATION_WIDTH top1:NOTE_ANNOTATION_WIDTH];
    CGRect iconrect = [_pdfViewCtrl convertPdfRectToPageViewRect:iconfsrect pageIndex:self.currentPageIndex];
    float iconwidth = iconrect.size.width;
    CGRect rect = CGRectMake(pvPoint.x - NOTE_ANNOTATION_WIDTH * scale / 2, pvPoint.y - NOTE_ANNOTATION_WIDTH * scale / 2, iconwidth, iconwidth);
    
    FSRectF *dibRect = [_pdfViewCtrl convertPageViewRectToPdfRect:rect pageIndex:self.currentPageIndex];

    FSPointF *tempPoint = [_pdfViewCtrl convertPageViewPtToPdfPt:pvPoint pageIndex:self.currentPageIndex];
    FSRectF *annotTempRect = [[FSRectF alloc] init];
    annotTempRect.left = tempPoint.x;
    annotTempRect.bottom = tempPoint.y - NOTE_ANNOTATION_WIDTH;
    annotTempRect.right = annotTempRect.left + NOTE_ANNOTATION_WIDTH;
    annotTempRect.top = annotTempRect.bottom + NOTE_ANNOTATION_WIDTH;
    dibRect = annotTempRect;
    
    FSTextViewPopoverViewController *noteDialog = [self.viewCtrlManager presentViewCtrlType:FSViewCtrlTypeTextViewPopover params:self.topToolbar.getToolbarContentView];
    noteDialog.okItemEnabled = ^BOOL(NSString *text) {
        return YES;
    };
    noteDialog.title = FSLocalizedForKey(@"kNote");
    noteDialog.didClickedOK = ^(NSString *text) {
        FSPDFPage *page = [self->_pdfViewCtrl.currentDoc getPage:self.currentPageIndex];
        if (!page || [page isEmpty])
            return;

        FSReadToolSettings* toolSettings = [self getReadToolSettingsForType:FSReadToolSettingsTypeNote];
        unsigned int color = [self getReadToolSettingsForType:FSReadToolSettingsTypeNote].settingsColor;
        float opacity = [self getReadToolSettingsForType:FSReadToolSettingsTypeNote].opacity;
        color = toolSettings ? (unsigned int)toolSettings.color.rgbValue : color;
        opacity = toolSettings ? toolSettings.opacity : opacity;
        FSNote *note = [[FSNote alloc] initWithAnnot:[page addAnnot:FSAnnotNote rect:dibRect]];
        note.color = color;
        note.opacity = opacity;
        note.icon = [self getReadToolSettingsForType:FSReadToolSettingsTypeNote].noteIcon;
        if (toolSettings) {
            note.icon = toolSettings.noteIcon;
        }
        note.author = self.annotAuthor;
        note.contents = text;
        note.NM = [Utility getUUID];
        note.lineWidth = 2;
        note.modifiedDate = [NSDate date];
        note.createDate = [NSDate date];
        note.subject = @"Note";
        note.flags = FSAnnotFlagPrint | FSAnnotFlagNoZoom | FSAnnotFlagNoRotate;
        [self.pdfViewCtrl lockRefresh];
        [note resetAppearanceStream];
        [self.pdfViewCtrl unlockRefresh];
        id<IAnnotHandler> annotHandler = [self getAnnotHandlerByAnnot:note];
        [annotHandler addAnnot:note];
    };
    [self setCurrentToolHandler:nil];
}

- (void)typeWriter {
    FtToolHandler *toolHandler = (FtToolHandler *) [self getToolHandlerByName:Tool_Freetext];
    toolHandler.toolSettings = [self getReadToolSettingsForType:FSReadToolSettingsTypeTypewriter];
    [self setCurrentToolHandler:toolHandler];
    toolHandler.freeTextStartPoint = self.currentPoint;
    [toolHandler onPageViewTap:self.currentPageIndex recognizer:nil];
    toolHandler.isTypewriterToolbarActive = NO;
    if (self.currentToolHandler == (id<IToolHandler>)self) { // todo
        [self setCurrentToolHandler:nil];
    }
}

- (void)signature {
    [self onUIInteractElementClicked:FSUIElementReadingLongPressSign];
    
    SignToolHandler *toolHandler = (SignToolHandler *) [self getToolHandlerByName:Tool_Signature];
    [self setCurrentToolHandler:toolHandler];
    toolHandler.isAdded = NO;
    toolHandler.signatureStartPoint = self.currentPoint;
    [toolHandler signList];
    int pageIndex = self.currentPageIndex;
    @weakify(toolHandler)
    toolHandler.didSelectedSign = ^{
        @strongify(toolHandler)
        [toolHandler onPageViewTap:pageIndex recognizer:nil];
        toolHandler.didSelectedSign = nil;
    };
}

- (void)redact{
    [self onUIInteractElementClicked:FSUIElementReadingLongPressRedaction];
    
    if (!self.isSupportRedaction) {
        FSAlertViewCtrlShowOnCtrl(FSLocalizedForKey(@"kWarning"), FSLocalizedForKey(@"kFailedLoadModuleForLicenseTip"), self.pdfViewCtrl);
        return;
    }
//    RedactToolHandler *toolHandler = (RedactToolHandler *) [self getToolHandlerByName:Tool_Redaction];
//    [self setCurrentToolHandler:toolHandler];
    @weakify(self)
    [self.permissionProvider checkPermission:FSFunctionRedaction checkPermissionState:^(FSPermissionState state) {
        @strongify(self)
        if (state == FSPermissionStateShow) {
            [self changeState:FSUIManagerStateRedaction];
        }
    }];
}

- (void)pencil{
    
    if (self.getState != FSUIManagerStateNormal) {
        [self changeState:FSUIManagerStateNormal];
    }
    
    BOOL isExistPencil = NO;
    if (!self.fullScreenToolbarMgr.floatyToolbar.hidden) {
        for (FSMainTopbarToolTagItem *tagItem in self.toolbarManager.mainTopbar.tagItems) {
            if (tagItem.toolTag == FSMainTopbarToolTagHome) {
                for (FSReadToolbarItem *readItem in tagItem.readToolbarItems) {
                    if (readItem.settingsType == FSReadToolSettingsTypePencil) {
                        [readItem selectFormSuperBar];
                        [self.toolbarManager.mainTopbar.currentTagItem scrollToCurrentReadToolbarItem];
                        isExistPencil = YES;
                        break;
                    }
                }
                break;
            }
        }
    }
    
    if (!isExistPencil) {
        if (self.currentIsFullScreen) {
            [self setFullScreen:NO];
        }
        
        isExistPencil = NO;
        FSMainTopbarToolTagItem *currentItem = self.toolbarManager.mainTopbar.currentTagItem;
        for (FSReadToolbarItem *readItem in currentItem.readToolbarItems) {
            if (readItem.settingsType == FSReadToolSettingsTypePencil) {
                [readItem selectFormSuperBar];
                [self.toolbarManager.mainTopbar.currentTagItem scrollToCurrentReadToolbarItem];
                isExistPencil = YES;
                break;
            }
        }
        
        if (!isExistPencil) {
            [self.toolbarManager.mainTopbar switchToTag:FSMainTopbarToolTagDrawing];
            FSMainTopbarToolTagItem *currentItem = self.toolbarManager.mainTopbar.currentTagItem;
            for (FSReadToolbarItem *readItem in currentItem.readToolbarItems) {
                if (readItem.settingsType == FSReadToolSettingsTypePencil) {
                    [readItem selectFormSuperBar];
                    [self.toolbarManager.mainTopbar.currentTagItem scrollToCurrentReadToolbarItem];
                    break;
                }
            }
        }
    }
}

- (void)showBlankMenu:(int)pageIndex point:(CGPoint)point {
    if([self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_REFLOW){
        return;
    }
    if (![self shouldEditAnnotPoint:point pageIndex:pageIndex]) return;
    
    self.currentPoint = point;
    self.currentPageIndex = pageIndex;
    NSMutableArray *array = [NSMutableArray array];
    MenuItem *commentItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kNote") object:self action:@selector(comment)];
    MenuItem *typeWriterItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kTypewriter") object:self action:@selector(typeWriter)];
    MenuItem *signatureItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kSignAction") object:self action:@selector(signature)];
    MenuItem *redactItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kRedaction") object:self action:@selector(redact)];
    MenuItem *peniclItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kPencil") object:self action:@selector(pencil)];

    if ([self annotsPermissionCanAdd]) {
        if ([self.config.tools containsObject:Tool_Note]) {
            [array addObject:commentItem];
        }
        if ([self.config.tools containsObject:Tool_Freetext]) {
            [array addObject:typeWriterItem];
        }
        
        if ([self.config.tools containsObject:Tool_Pencil] && self.enableTopToolbar) {
            [array addObject:peniclItem];
        }
        
        if (self.currentState != FSUIManagerStateRedaction) {
            if ([self.config.tools containsObject:Tool_Redaction]) {
                if ([self.permissionProvider checkPermission:FSFunctionRedaction] != FSPermissionStateHide) [array addObject:redactItem];
            }
        }
    }
    if ([FSPermissionUtilty canAddSign:self]) {
        if (self.config.loadSignature &&
            [FSPermissionUtilty canAddSignToDocument:self] &&
            [FSPermissionUtilty canAddSign:self] &&
            ![_pdfViewCtrl isDynamicXFA] &&
            [FSPermissionUtilty canFillForm:self]){
            if ([self.permissionProvider checkPermission:FSFunctionPDFSign] != FSPermissionStateHide) [array addObject:signatureItem];
        }
    }

    array = [self didForMenuControlWillShow:FSMenuControlSceneBlankSpace array:array];
    
    if (array.count > 0) {
        CGRect dvRect = CGRectMake(point.x, point.y, 2, 2);
        dvRect = [_pdfViewCtrl convertPageViewRectToDisplayViewRect:dvRect pageIndex:pageIndex];
        MenuControl *annotMenu = self.menuControl;
        annotMenu.menuItems = array;
        [annotMenu setRect:dvRect];
        [annotMenu showMenu];
    }
}

- (BOOL)shouldEditAnnotPoint:(CGPoint)point pageIndex:(int)pageIndex{
    if ([_pdfViewCtrl isDynamicXFA]) {
        return YES;
    }
    
    BOOL ret = YES;
    if (pageIndex < 0) ret = NO;
    if (ret) {
        FSPDFPage *currentPage = [_pdfViewCtrl.currentDoc getPage:pageIndex];
        CGFloat pageH = currentPage.getHeight;
        CGFloat pageW = currentPage.getWidth;
        
        FSRotation rotation = currentPage.rotation;
    //    FSRotation viewRoation = (FSRotation)[_pdfViewCtrl getViewRotation];
    //    rotation = (rotation + viewRoation) % 4;
        if (rotation == FSRotation90 || rotation == FSRotation270) {
            CGFloat tmpPageH = pageH;
            pageH = pageW;
            pageW = tmpPageH;
        }
        
        CGRect pointRect = CGRectMake(point.x, point.y, 5, 5);
        FSRectF *rect = [_pdfViewCtrl convertPageViewRectToPdfRect:pointRect pageIndex:pageIndex];
        FSRectF *realRect = [Utility getPageRealRect:currentPage];
        
        if (rect.right > realRect.right ||
            rect.top > realRect.top ||
            rect.left < realRect.left ||
            rect.bottom < realRect.bottom) {
            ret = NO;
        }
    }

    if (!ret) {
        [UIAlertController fs_alert_OK_WithTitle:@"kError" message:@"kEditOutsideDocument" actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
            if (self.shouldShowMenu) {
                [self showMenu];
            }
        } onCurrentSceneResponder:self.pdfViewCtrl];
    }
    return ret;
}

- (BOOL)onLongPress:(UILongPressGestureRecognizer *)gestureRecognizer {
    
    if ([self canAutoHideTopBottomToolBar]) {
        [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(autoFullScreen) object:nil];
    }
    if (gestureRecognizer.state == UIGestureRecognizerStateEnded) {
        if ([self canAutoHideTopBottomToolBar]) {
            [self delayHideTopBottomToolBar];
        }
    }
   

    if ([self getState] == FSUIManagerStateComparision) {
        return NO;
    }
//    if([_pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_REFLOW){
//    }
    
    CGPoint point = [gestureRecognizer locationInView:[_pdfViewCtrl getDisplayView]];
    int pageIndex = [_pdfViewCtrl getPageIndex:point];
    //    if (pageIndex < 0)
    //        return NO;
    //    UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
    //    CGPoint cgPoint = [gestureRecognizer locationInView:pageView];
    //    CGRect rect1 = [pageView frame];
    //    CGSize size = rect1.size;
    //    if (cgPoint.x > size.width || cgPoint.y > size.height || cgPoint.x < 0 || cgPoint.y < 0)
    //        return NO;
    
    if (self.currentState == FSUIManagerStateFillSign) {
        FillSignToolHandler *fillSignToolHandler = self.fillSignModule.fillSignToolHandler;
        if (fillSignToolHandler && !self.currentToolHandler) {
            return [fillSignToolHandler onPageViewLongPress:pageIndex recognizer:gestureRecognizer];
        }
    }
    
    id<IToolHandler> originToolHandler = self.currentToolHandler;
    if (self.currentToolHandler != nil) {
        if (gestureRecognizer.state == UIGestureRecognizerStateBegan){
            CGPoint point = [gestureRecognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];
            FSAnnot* annot = [self getAnnotAtPoint:point pageIndex:pageIndex];
            if (![self checkIsCanSelectedAnnotForLongPress:annot])
            {
                [self setCurrentAnnot:nil];
                if ([self.currentToolHandler onPageViewLongPress:pageIndex recognizer:gestureRecognizer]) {
                    return YES;
                }
            }
        }else {
            if ([self.currentToolHandler onPageViewLongPress:pageIndex recognizer:gestureRecognizer]) {
                return YES;
            }
        }
        
    }
    id<IToolHandler> selectTool = [self getToolHandlerByName:Tool_Select];
    if (self.currentToolHandler == selectTool)
        [self setCurrentToolHandler:nil];

    if ([self checkIsCanCreateAnnotForLongPress]) {
        //annot handler
        id<IAnnotHandler> annotHandler = nil;
        point = [gestureRecognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];
        FSAnnot *annot = nil;
        if (gestureRecognizer.state == UIGestureRecognizerStateBegan){
            annot = [self getAnnotAtPoint:point pageIndex:pageIndex];
            if (annot != nil && ![annot isEmpty]) {
                if (![annot isKindOfClass:[FSTextMarkup class]]) //for adding text markups one more at same text
                {
                    annotHandler = [self getAnnotHandlerByAnnot:annot];
                    if (annotHandler != nil) {
                        if ([self isCleanLongPressSelectedAnnot:annot]) {
                            [self setCurrentAnnot:nil];
                        }
                        return [annotHandler onPageViewLongPress:pageIndex recognizer:gestureRecognizer annot:annot];
                    }
                }
            }
        }else {
            annot = self.currentAnnot;
            if (annot != nil && ![annot isEmpty]) {
                annotHandler = [self getAnnotHandlerByAnnot:annot];
                return [annotHandler onPageViewLongPress:pageIndex recognizer:gestureRecognizer annot:annot];
            }
        }

        id<IAnnotHandler> linkAnnotHandler = [self getAnnotHandlerByType:FSAnnotLink];
        if (linkAnnotHandler && [linkAnnotHandler onPageViewLongPress:pageIndex recognizer:gestureRecognizer annot:nil]) {
            return YES;
        }
        
        if (![_pdfViewCtrl isDynamicXFA]) {
            if (originToolHandler != selectTool && [selectTool onPageViewLongPress:pageIndex recognizer:gestureRecognizer]) {
                if (self.currentToolHandler == nil) {
                    [self setCurrentToolHandler:selectTool];
                }
                return YES;
            }
        }
        
        id<IModule> module = [self getModuleByName:@"Redact"];
        RedactModule *redactModule = (RedactModule *)module;
        if (self.currentState == FSUIManagerStateRedaction && redactModule.bCanRedact) {
            return NO;
        }
        
        if (gestureRecognizer.state == UIGestureRecognizerStateBegan) {
            [self showBlankMenu:pageIndex point:point];
            #if _MAC_CATALYST_
                [Utility cancelRecognizer:gestureRecognizer];
            #endif
            return YES;
        }
    }

    for (id<IGestureEventListener> listener in self.guestureEventListeners) {
        if ([listener respondsToSelector:@selector(onLongPress:)]) {
            if ([listener onLongPress:gestureRecognizer])
                return YES;
        }
    }

    return NO;
}

#pragma mark ILinkEventListener

- (void)registerLinkEventListener:(id<ILinkEventListener>)listener {
    if (!self.linkEventListeners) {
        self.linkEventListeners = [NSMutableArray<ILinkEventListener> array];
    }
    [self.linkEventListeners addObject:listener];
}

- (void)unregisterLinkEventListener:(id<ILinkEventListener>)listener {
    [self.linkEventListeners removeObject:listener];
}

- (BOOL)onLinkOpen:(id)link LocationInfo:(CGPoint)pointParam {
    for (id<ILinkEventListener> listener in self.linkEventListeners) {
        if ([listener respondsToSelector:@selector(onLinkOpen:LocationInfo:)]) {
            if ([listener onLinkOpen:link LocationInfo:pointParam])
                return YES;
        }
    }
    return NO;
}

#pragma mark ISignatureEventListener
- (void)registerSignatureEventListener:(id<ISignatureEventListener>)listener{
    if (!self.signatureEventListeners) {
        self.signatureEventListeners = [NSMutableArray<ISignatureEventListener> array];
    }
    [self.signatureEventListeners addObject:listener];
}

- (void)unregisterSignatureEventListener:(id<ISignatureEventListener>)listener{
    [self.signatureEventListeners removeObject:listener];
}

- (void)onDigitalSignatureSigned:(BOOL)success originalDoc:(FSPDFDoc *)originalDoc outputPath:(NSString *)outputPath{
    for (id<ISignatureEventListener> listener in self.signatureEventListeners) {
        if ([listener respondsToSelector:@selector(onDigitalSignatureSigned:originalDoc:outputPath:)]) {
            [listener onDigitalSignatureSigned:success originalDoc:originalDoc outputPath:outputPath];
        }
        if ([listener respondsToSelector:@selector(onDigitalSignatureSigned:)]) {
            [listener onDigitalSignatureSigned:success];
        }
    }
}

- (BOOL)canReplaceDigitalSignatureOriginalFile:(NSString *)originalFile{
    for (id<ISignatureEventListener> listener in self.signatureEventListeners) {
        if ([listener respondsToSelector:@selector(canReplaceDigitalSignatureOriginalFile:)]) {
            if (![listener canReplaceDigitalSignatureOriginalFile:originalFile]) {
                return NO;
            }
        }
    }
    return YES;
}

#pragma mark IUIInteractionEventListener
- (void)registerUIInteractionEventListener:(id<IUIInteractionEventListener>)listener {
    if (!self.uiInteractionEventListeners) {
        self.uiInteractionEventListeners = [NSMutableArray<IUIInteractionEventListener> array];
    }
    if (self.uiInteractionEventListeners && ![self.uiInteractionEventListeners containsObject:listener]) {
        [self.uiInteractionEventListeners addObject:listener];
    }
}

- (void)unregisterUIInteractionEventListener:(id<IUIInteractionEventListener>)listener {
    [self.uiInteractionEventListeners removeObject:listener];
}

- (void)onUIInteractElementClicked:(FSUIElementType)element {
    
    for (id<IUIInteractionEventListener> listener in self.uiInteractionEventListeners) {
        if ([listener respondsToSelector:@selector(onUIInteractElementClicked:)]) {
            [listener onUIInteractElementClicked:element];
        }
    }
}

- (BOOL)onTap:(UITapGestureRecognizer *)gestureRecognizer {
    if (self.currentState == FSUIManagerStateComparision) {
        if ([self canAutoHideTopBottomToolBar]) {
            [self delayHideTopBottomToolBar];
        }
        return NO;
    }
    if ([self canAutoHideTopBottomToolBar]) {
        [self delayHideTopBottomToolBar];
    }
    
    CGPoint point = [gestureRecognizer locationInView:[_pdfViewCtrl getDisplayView]];
    int pageIndex = [_pdfViewCtrl getPageIndex:point];
    if (pageIndex < 0)
        return NO;
    UIView *pageView = [_pdfViewCtrl getPageView:pageIndex];
    PDF_LAYOUT_MODE pageLayoutMode = [self.pdfViewCtrl getPageLayoutMode];
    BOOL (^fullScreenBlock)(void) = ^(){
        if ((self.currentState == FSUIManagerStatePanZoom ||  self.currentState == FSUIManagerStateAutoFlip || self.currentState == FSUIManagerStateSpeech || self.currentState == FSUIManagerStateReflow)) {
            return NO;
        }

        BOOL flag = self.currentIsFullScreen;
        CGFloat width = self.pdfViewCtrl.fs_width;
        CGPoint cgPoint = [pageView convertPoint:[gestureRecognizer locationInView:pageView] toView:self.pdfViewCtrl];
        if ((width * 0.2 < cgPoint.x && cgPoint.x < width * 0.8) || !self.pdfViewCtrl.enablePageFlippingByTouchBorder) {
            BOOL topbarSelected = [self.topToolbar getCurrentSelectedItem].item.selected && self.fullScreenToolbarMgr.floatyToolbar.hidden;
            [self setFullScreen:!flag && !topbarSelected];
            return YES;
        }
        return NO;
    };
    
//    if (cgPoint.x > size.width || cgPoint.y > size.height || cgPoint.x < 0 || cgPoint.y < 0)
//        return NO;

//    if (pageLayoutMode == PDF_LAYOUT_MODE_REFLOW) {
//        return NO;
//    }
    
    BOOL (^fillSignTapBlock)(void) = ^(){
        if (self.currentState == FSUIManagerStateFillSign) {
            FillSignToolHandler *fillSignToolHandler = self.fillSignModule.fillSignToolHandler;
            if (fillSignToolHandler && !self.currentToolHandler) {
                if (![fillSignToolHandler onPageViewTap:pageIndex recognizer:gestureRecognizer]) {
                    return fullScreenBlock();
                }
            }
            return YES;
        }
        return NO;
    };

    id<IToolHandler> originToolHandler = self.currentToolHandler;
    if (self.currentToolHandler != nil && [self.pdfViewCtrl getPageLayoutMode] != PDF_LAYOUT_MODE_REFLOW) {
        CGPoint newpoint = [_pdfViewCtrl convertDisplayViewPtToPageViewPt:point pageIndex:pageIndex];
        if (![self shouldEditAnnotPoint:newpoint pageIndex:pageIndex]) return YES;
        
        if ([self checkIsCanCreateAnnotForSingleTap]) {
            CGPoint point = [gestureRecognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];
            FSAnnot* annot = [self getAnnotAtPoint:point pageIndex:pageIndex];
            if (!(annot != nil && ![annot isEmpty]) || [self checkIsCanCreateAnnotOnAnnot]) {
                [self setCurrentAnnot:nil];
                if ([self.currentToolHandler onPageViewTap:pageIndex recognizer:gestureRecognizer]) {
                    return YES;
                }
            }
        }else{
            [self setCurrentAnnot:nil];
            if ([self.currentToolHandler onPageViewTap:pageIndex recognizer:gestureRecognizer]) {
                return YES;
            }
        }
    }
    id<IToolHandler> selectTool = [self getToolHandlerByName:Tool_Select];
    if (self.currentToolHandler == selectTool)
        [self setCurrentToolHandler:nil];

    if ([self checkIsCanSelectedAnnotForSingleTap]) {
        //annot handler
        FSAnnot *annot = nil;
        id<IAnnotHandler> annotHandler = nil;
        annot = self.currentAnnot;
        if (annot != nil && ![annot isEmpty]) {
            annotHandler = [self getAnnotHandlerByAnnot:annot];
            return [annotHandler onPageViewTap:pageIndex recognizer:gestureRecognizer annot:annot];
        }
        point = [gestureRecognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];
        annot = [self getAnnotAtPoint:point pageIndex:pageIndex];

        if (![_pdfViewCtrl isDynamicXFA] && originToolHandler != selectTool && [selectTool onPageViewTap:pageIndex recognizer:gestureRecognizer]) {
            return YES;
        }
        if ([_pdfViewCtrl isDynamicXFA]) {
            FSXFAWidget *widget = nil;
            widget = self.currentWidget;
            if (widget != nil && ![widget isEmpty]) {
                id<IAnnotHandler> dynamicXFAHandler = [self getDynamicAnnotHandler];
                if (dynamicXFAHandler != nil) {
                    return [dynamicXFAHandler onPageViewTap:pageIndex recognizer:gestureRecognizer widget:widget];
                }
            }
            
            FSXFAWidget *xfaWidget = [self getXFAWidgetAtPoint:point pageIndex:pageIndex];
            if (xfaWidget != nil && ![xfaWidget isEmpty]) {
                id<IAnnotHandler> dynamicXFAHandler = [self getDynamicAnnotHandler];
                if (dynamicXFAHandler != nil) {
                    return [dynamicXFAHandler onPageViewTap:pageIndex recognizer:gestureRecognizer widget:xfaWidget];
                }
            }
        }
        
        if (annot != nil && ![annot isEmpty]) {
            if (annot.isMarkup && annot.getType != FSAnnotCaret && self.currentState != FSUIManagerStateFillSign) {
                FSMarkup *markup = [[FSMarkup alloc] initWithAnnot:annot];
                if (markup.isGrouped && [self.config.tools containsObject:Tool_Multiple_Selection] && [FSPermissionUtilty canAddAnnot:self]) {
                    id<IToolHandler> toolHandler = [self getToolHandlerByName:Tool_Multiple_Selection];
                    if ([toolHandler isKindOfClass:[MultipleSelectionToolHandler class]]) {
                        [self setCurrentToolHandler:toolHandler];
                        self.shouldShowMenu = YES;
                        if ([(MultipleSelectionToolHandler *)toolHandler selectedGroupAnnot:annot])  return YES;
                    }
                }
            }
            annotHandler = [self getAnnotHandlerByAnnot:annot];
            if (self.currentState == FSUIManagerStateFillSign && ![annotHandler isKindOfClass:[DigitalSignatureAnnotHandler class]] && fillSignTapBlock())  return YES;
            if (annotHandler != nil) {
                return [annotHandler onPageViewTap:pageIndex recognizer:gestureRecognizer annot:annot];
            }
        }else if (self.currentState == FSUIManagerStateFillSign && fillSignTapBlock()){
            return YES;
        }
        
        id<IAnnotHandler> linkAnnotHandler = [self getAnnotHandlerByType:FSAnnotLink];
        if (linkAnnotHandler && [linkAnnotHandler onPageViewTap:pageIndex recognizer:gestureRecognizer annot:nil]) {
            return YES;
        }
    }

    for (id<IGestureEventListener> listener in self.guestureEventListeners) {
        if ([listener respondsToSelector:@selector(onTap:)]) {
            if ([listener onTap:gestureRecognizer])
                return YES;
        }
    }
    
    if (self.currentState == FSUIManagerStateRedaction) {
        return NO;
    }
    
    return fullScreenBlock();
}

- (BOOL)onDoubleTap:(UITapGestureRecognizer *)gestureRecognizer {

    if ([self canAutoHideTopBottomToolBar]) {
        [self delayHideTopBottomToolBar];
    }
    return NO;
}

- (BOOL)onPan:(UIPanGestureRecognizer *)recognizer {

    CGPoint point = [recognizer locationInView:[_pdfViewCtrl getDisplayView]];
    int pageIndex = [_pdfViewCtrl getPageIndex:point];
    if (recognizer.state == UIGestureRecognizerStateBegan) {
        self.panBeginPageIndex = pageIndex;
    }else{
        if (pageIndex < 0 && self.panBeginPageIndex >= 0) {
            pageIndex = self.panBeginPageIndex;
        }
    }
    if (pageIndex < 0){
        id<IToolHandler> originToolHandler = self.currentToolHandler;
        if (originToolHandler != nil) {
            [recognizer setValue:@(UIGestureRecognizerStateEnded) forKey:@"state"];
            if ([self.currentToolHandler onPageViewPan:pageIndex recognizer:recognizer]) {
                return YES;
            }
        }
        return NO;
    }
  
    if (self.currentState == FSUIManagerStateFillSign) {
        FillSignToolHandler *fillSignToolHandler = self.fillSignModule.fillSignToolHandler;
        if (fillSignToolHandler && !self.currentToolHandler) {
            if (fillSignToolHandler.editFillHandler) {
                return [fillSignToolHandler onPageViewPan:pageIndex recognizer:recognizer];
            }
        }
    }
    
    id<IToolHandler> originToolHandler = self.currentToolHandler;
    if (self.currentToolHandler != nil) {
        id<IToolHandler> selectTool = [self getToolHandlerByName:Tool_Select];
        if (![self checkIsCanSlidingForSingleTouch] || self.currentToolHandler == selectTool) {
            if (self.currentAnnot) {
            [self setCurrentAnnot:nil];
            }
            if ([self.currentToolHandler onPageViewPan:pageIndex recognizer:recognizer]) {
                return YES;
            }
        }
    }

    if (recognizer.state == UIGestureRecognizerStateChanged && [self getToolHandlerByName:Tool_Shape] == originToolHandler) {
        ShapeToolHandler *shapeToolHandler = (ShapeToolHandler *)originToolHandler;
        if (shapeToolHandler.annot != nil && shapeToolHandler.annot.pageIndex != pageIndex) {
            [recognizer setValue:@(UIGestureRecognizerStateEnded) forKey:@"state"];
            return YES;
        }
        UIView *pageview = [_pdfViewCtrl getPageView:pageIndex];
        CGPoint p = [recognizer locationInView:pageview];
        BOOL isOut = ![pageview pointInside:p withEvent:nil];

        if (isOut) {
            [recognizer setValue:@(UIGestureRecognizerStateEnded) forKey:@"state"];
            return YES;
        }
    }
    
    id<IToolHandler> selectTool = [self getToolHandlerByName:Tool_Select];
    if (self.currentToolHandler == selectTool)
        [self setCurrentToolHandler:nil];

    if ([self checkIsCanSlidingForSingleTouch]) {
        //annot handler
        FSAnnot *annot = nil;
        id<IAnnotHandler> annotHandler = nil;
        annot = self.currentAnnot;
        if (annot != nil && ![annot isEmpty]) {
            if ([[annot getPage] getIndex] == pageIndex ||
                recognizer.state == UIGestureRecognizerStateEnded ||
                recognizer.state == UIGestureRecognizerStateCancelled) {
                annotHandler = [self getAnnotHandlerByAnnot:annot];
                return [annotHandler onPageViewPan:pageIndex recognizer:recognizer annot:annot];
            }
        }
        

        point = [recognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];
        annot = [self getAnnotAtPoint:point pageIndex:pageIndex];
        if (annot != nil && ![annot isEmpty]) {
            annotHandler = [self getAnnotHandlerByAnnot:annot];
            if (annotHandler != nil) {
                return [annotHandler onPageViewPan:pageIndex recognizer:recognizer annot:annot];
            }
        }
        id<IToolHandler> selectTool = [self getToolHandlerByName:Tool_Select];
        if (originToolHandler != selectTool && [selectTool onPageViewPan:pageIndex recognizer:recognizer]) {
            return YES;
        }
    }
    for (id<IGestureEventListener> listener in self.guestureEventListeners) {
        if ([listener respondsToSelector:@selector(onPan:)]) {
            if ([listener onPan:recognizer])
                return YES;
        }
    }
    return NO;
}


- (id<IAnnotHandler>)getDynamicAnnotHandler {
    for (id<IAnnotHandler> annotHandler in self.annotHandlers) {
        if ([annotHandler respondsToSelector:@selector(getName)]) {
            if ([[annotHandler getName] isEqualToString: @"FSXFAWidget"]) {
                return annotHandler;
            }
        }
    }
    return nil;
}

-(FSXFAWidget *)getXFAWidgetAtPoint:(CGPoint)pvPoint pageIndex:(int)pageIndex {
    FSXFAPage *page = [[_pdfViewCtrl getXFADoc] getPage:pageIndex];
    
    FSMatrix2D *matrix = [_pdfViewCtrl getDisplayMatrix:pageIndex];
    FSPointF *devicePoint = [[FSPointF alloc] init];
    [devicePoint set:pvPoint.x y:pvPoint.y];
    FSXFAWidget *widget = [page getWidgetAtDevicePoint:matrix device_point:devicePoint tolerance:5];
    
    if (!widget || [widget isEmpty]) {
        return nil;
    }
    return widget;
}
- (FSAnnot *)getAnnotAtPoint:(CGPoint)pvPoint pageIndex:(int)pageIndex {
    if ([_pdfViewCtrl isDynamicXFA] || pageIndex < 0) {
        return nil;
    }
    FSPDFPage *page = [_pdfViewCtrl.currentDoc getPage:pageIndex];
    if (!page || [page isEmpty]) {
        return nil;
    }
    FSMatrix2D *matrix = [_pdfViewCtrl getDisplayMatrix:pageIndex];
    FSPointF *devicePoint = [[FSPointF alloc] init];
    [devicePoint set:pvPoint.x y:pvPoint.y];
    FSAnnot *annot = [self getAnnotAtDevicePoint:page devicePoint:devicePoint matrix:matrix];
  
    if (!annot || [annot isEmpty]) {
        return nil;
    }

    if (![self.config canInteractWithAnnot:annot]) {
        return nil;
    }
    
    id<IAnnotHandler> annotHandler = [self getAnnotHandlerByAnnot:annot];
    if([_pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_REFLOW) {
        FSReflowPage * reflowPage = [[ReflowModule getReflowManager] getParsedPage:pageIndex];
        if ([annotHandler isHitAnnot:annot point:[_pdfViewCtrl convertPageViewPtToReflowPagePt:pvPoint reflowPage:reflowPage pageIndex:pageIndex]]) {
            return annot;
        }
    }else {
        if ([annotHandler isHitAnnot:annot point:[_pdfViewCtrl convertPageViewPtToPdfPt:pvPoint pageIndex:pageIndex]]) {
            return annot;
        }
    }
  
    if (annot.type == FSAnnotStrikeOut) {
        FSMarkup *markup = [[FSMarkup alloc] initWithAnnot:annot];
        FSMarkupArray *groupAnnots = [markup getGroupElements];
        for (int i = 0; i < [groupAnnots getSize]; i++) {
            FSAnnot *groupAnnot = [groupAnnots getAt:i];
            if (groupAnnot.type == FSAnnotCaret) {
                //                return groupAnnot;
                // can't just return groupAnnot since it will be destroyed in MarkupArray dtor
                return [[FSAnnot alloc] initWithAnnot:groupAnnot];
            }
        }
        return annot;
    }
    return nil;
}

- (FSAnnot *)getAnnotAtDevicePoint:(FSPDFPage *)page devicePoint:(FSPointF*)devicePoint matrix:(FSMatrix2D*)matrix{
    
    //In redact mode, returns the order of annots:  redact > link > note> other annot
    //In normal mode，returns the order of annots: link > redact > note> other annot
    
    FSAnnot *retAnnot = nil;
    FSAnnotArray *annotArray = [page getAnnotsAtDevicePoint:devicePoint tolerance:5 matrix:matrix];
    u_long count = [annotArray getSize];
    if (count > 0) {
        retAnnot = [annotArray getAt:0];
        FSAnnot *redactAnnot = nil;
        FSAnnot *linkAnnot = nil;
        FSAnnot *noteAnnot = nil;
        for (int i = 0; i < count; i++) {
            FSAnnot *annot = [annotArray getAt:i];
            if (!redactAnnot && [annot getType] == FSAnnotRedact) {
                redactAnnot = annot;
            }else if (!linkAnnot && [annot getType] == FSAnnotLink){
                linkAnnot = annot;
            }else if (!noteAnnot && [annot getType] == FSAnnotNote){
                noteAnnot = annot;
            }
        }
        
        if (self.currentState == FSUIManagerStateRedaction) {
            if (redactAnnot) {
                retAnnot = redactAnnot;
            }else if (linkAnnot){
                retAnnot = linkAnnot;
            }else if (noteAnnot){
                retAnnot = noteAnnot;
            }
        }else if (!self.currentToolHandler){
            if (linkAnnot) {
                retAnnot = linkAnnot;
            }else if (redactAnnot){
                retAnnot = redactAnnot;
            }else if (noteAnnot){
                retAnnot = noteAnnot;
            }
        }
    }
    
    return retAnnot;
}

- (void)enableModification:(BOOL)isEnabled{
    self.enableModifyDocument = isEnabled;
}

- (BOOL)checkIsCanCreateAnnotForSingleTap
{
    NSString* currentToolName = [self.currentToolHandler getName];
    if (self.currentToolHandler == nil || [currentToolName isEqualToString:Tool_Select] || [currentToolName isEqualToString:Tool_Note] || [currentToolName isEqualToString:Tool_Freetext] || [currentToolName isEqualToString:Tool_Textbox] || [currentToolName isEqualToString:Tool_Callout] || [currentToolName isEqualToString:Tool_Stamp] || [currentToolName isEqualToString:Tool_Insert] || [currentToolName isEqualToString:Tool_Replace] || [currentToolName isEqualToString:Tool_Line] || [currentToolName isEqualToString:Tool_Arrow] || [currentToolName isEqualToString:Tool_Attachment] || [currentToolName isEqualToString:Tool_Shape] || [currentToolName isEqualToString:Tool_Markup] || [currentToolName isEqualToString:Tool_Distance] || [currentToolName isEqualToString:Tool_Image] || [currentToolName isEqualToString:Tool_Highlight] || [currentToolName isEqualToString:Tool_Polygon] || [currentToolName isEqualToString:Tool_Cloud] || [currentToolName isEqualToString:Tool_PolyLine] || [currentToolName isEqualToString:Tool_Pencil])
    {
        return YES;
    }
    return NO;
}

- (BOOL)checkIsCanCreateAnnotOnAnnot
{
    NSString* currentToolName = [self.currentToolHandler getName];
    if (self.currentToolHandler == nil || [currentToolName isEqualToString:Tool_Polygon] || [currentToolName isEqualToString:Tool_Cloud] || [currentToolName isEqualToString:Tool_PolyLine] )
    {
        return YES;
    }
    return NO;
}

- (BOOL)checkIsCanSelectedAnnotForSingleTap
{
    NSString* currentToolName = [self.currentToolHandler getName];
    if (self.currentToolHandler == nil || [currentToolName isEqualToString:Tool_Select] || [currentToolName isEqualToString:Tool_Note] || [currentToolName isEqualToString:Tool_Freetext] || [currentToolName isEqualToString:Tool_Textbox] || [currentToolName isEqualToString:Tool_Callout] || [currentToolName isEqualToString:Tool_Stamp] || [currentToolName isEqualToString:Tool_Insert] || [currentToolName isEqualToString:Tool_Replace] || [currentToolName isEqualToString:Tool_Line] || [currentToolName isEqualToString:Tool_Arrow] || [currentToolName isEqualToString:Tool_Attachment] || [currentToolName isEqualToString:Tool_Shape] || [currentToolName isEqualToString:Tool_Markup] || [currentToolName isEqualToString:Tool_Distance] || [currentToolName isEqualToString:Tool_Image] || [currentToolName isEqualToString:Tool_Highlight])
    {
        return YES;
    }
    return NO;
}

- (BOOL)isCleanLongPressSelectedAnnot:(FSAnnot *)annot{
    if (self.currentAnnot && ![self.currentAnnot isEmpty]) {
        if (![self.currentAnnot isEqualToAnnot:annot]) {
            FSAnnotType annotType = annot.getType;
            if (annotType != FSAnnotWidget) {
                return YES;
            }
        }
    }
    return NO;
}

- (BOOL)checkIsCanCreateAnnotForLongPress
{
    NSString* currentToolName = [self.currentToolHandler getName];
    if (self.currentToolHandler == nil || [currentToolName isEqualToString:Tool_Select] /*|| [currentToolName isEqualToString:Tool_Note] */|| [currentToolName isEqualToString:Tool_Freetext] || [currentToolName isEqualToString:Tool_Textbox] || [currentToolName isEqualToString:Tool_Callout] || [currentToolName isEqualToString:Tool_Stamp] || [currentToolName isEqualToString:Tool_Insert] || [currentToolName isEqualToString:Tool_Replace] || [currentToolName isEqualToString:Tool_Line] || [currentToolName isEqualToString:Tool_Arrow] /*|| [currentToolName isEqualToString:Tool_Attachment] */|| [currentToolName isEqualToString:Tool_Shape] || [currentToolName isEqualToString:Tool_Markup] || [currentToolName isEqualToString:Tool_Distance] || [currentToolName isEqualToString:Tool_Image] || [currentToolName isEqualToString:Tool_Highlight] || [currentToolName isEqualToString:Tool_Polygon] || [currentToolName isEqualToString:Tool_Cloud] || [currentToolName isEqualToString:Tool_PolyLine] || [currentToolName isEqualToString:Tool_Pencil])
    {
        return YES;
    }
    return NO;
}

- (BOOL)checkIsCanSelectedAnnotForLongPress:(FSAnnot*)annot
{
    if (annot != nil && ![annot isEmpty] && (annot.type == FSAnnotNote || annot.type == FSAnnotFileAttachment))
    {
        return YES;
    }
    return NO;
}

- (BOOL)checkIsCanSlidingForSingleTouch
{
    NSString* currentToolName = [self.currentToolHandler getName];
    if (self.currentToolHandler == nil || [currentToolName isEqualToString:Tool_Select] || [currentToolName isEqualToString:Tool_Note] || [currentToolName isEqualToString:Tool_Freetext] || [currentToolName isEqualToString:Tool_Textbox] || [currentToolName isEqualToString:Tool_Callout] || [currentToolName isEqualToString:Tool_Stamp] || [currentToolName isEqualToString:Tool_Insert] || [currentToolName isEqualToString:Tool_Replace] || [currentToolName isEqualToString:Tool_Line] || [currentToolName isEqualToString:Tool_Arrow] || [currentToolName isEqualToString:Tool_Attachment] || [currentToolName isEqualToString:Tool_Shape] || [currentToolName isEqualToString:Tool_Markup] || [currentToolName isEqualToString:Tool_Distance] || [currentToolName isEqualToString:Tool_Image] || [currentToolName isEqualToString:Tool_Highlight] || [currentToolName isEqualToString:Tool_Polygon] || [currentToolName isEqualToString:Tool_Cloud] || [currentToolName isEqualToString:Tool_PolyLine] || [currentToolName isEqualToString:Tool_Link])
    {
        return YES;
    }
    return NO;
}

- (BOOL)onShouldBegin:(UIGestureRecognizer *)recognizer {
    [self adapterMultipleScenes];
    
    CGPoint point = [recognizer locationInView:[_pdfViewCtrl getDisplayView]];
    int pageIndex = [_pdfViewCtrl getPageIndex:point];
    if (pageIndex < 0)
        return NO;

    
    if (_currentToolHandler != nil) {
        if ([_currentToolHandler respondsToSelector:@selector(onPageViewShouldBegin:recognizer:)] && [_currentToolHandler onPageViewShouldBegin:pageIndex recognizer:recognizer]) {
            return YES;
        }
    }

    if (self.currentState == FSUIManagerStateFillSign) {
        FillSignToolHandler *fillSignToolHandler = self.fillSignModule.fillSignToolHandler;
        if (fillSignToolHandler && !self.currentToolHandler) {
            if (fillSignToolHandler.editFillHandler) {
                return [fillSignToolHandler onPageViewShouldBegin:pageIndex recognizer:recognizer];
            }
        }
    }

    if ([self checkIsCanSlidingForSingleTouch]) {
        //annot handler
        FSAnnot *annot = nil;
        id<IAnnotHandler> annotHandler = nil;
        annot = self.currentAnnot;
        if (annot != nil && ![annot isEmpty]) {
            annotHandler = [self getAnnotHandlerByAnnot:annot];
            return [annotHandler onPageViewShouldBegin:pageIndex recognizer:recognizer annot:annot];
        }

        if ([recognizer isKindOfClass:[UITapGestureRecognizer class]]) {
            if (((UITapGestureRecognizer *) recognizer).numberOfTapsRequired == 2) {
                if (self.currentToolHandler) {
                    return NO;
                }else{
                    return YES;
                }
            }
        }

        if ([recognizer isKindOfClass:[UITapGestureRecognizer class]] ||
            [recognizer isKindOfClass:[UILongPressGestureRecognizer class]] ||
            [recognizer isKindOfClass:[UIPanGestureRecognizer class]]) {
            CGPoint point = [recognizer locationInView:[_pdfViewCtrl getPageView:pageIndex]];

            if ([_pdfViewCtrl isDynamicXFA]) {
                FSXFAWidget *widget = self.currentWidget;
                if (widget != nil && ![widget isEmpty]) {
                    annotHandler = [self getDynamicAnnotHandler];
                    return [annotHandler onPageViewShouldBegin:pageIndex recognizer:recognizer widget:widget];
                }
                
                FSXFAWidget *xfaWidget = [self getXFAWidgetAtPoint:point pageIndex:pageIndex];
                if (xfaWidget != nil && ![xfaWidget isEmpty]) {
                    id<IAnnotHandler> dynamicXFAHandler = [self getDynamicAnnotHandler];
                    if (dynamicXFAHandler != nil && ![recognizer isKindOfClass:[UIPanGestureRecognizer class]]) {
                        return [dynamicXFAHandler onPageViewShouldBegin:pageIndex recognizer:recognizer widget:xfaWidget];
                    }
                }
                return NO;
            }else{
                annot = [self getAnnotAtPoint:point pageIndex:pageIndex];
                if (annot != nil && ![annot isEmpty]) {
                    annotHandler = [self getAnnotHandlerByAnnot:annot];
                    if (annotHandler != nil && ![recognizer isKindOfClass:[UIPanGestureRecognizer class]]) {
                        return [annotHandler onPageViewShouldBegin:pageIndex recognizer:recognizer annot:annot];
                    }
                }
                
                id<IAnnotHandler> linkAnnotHandler = [self getAnnotHandlerByType:FSAnnotLink];
                if (linkAnnotHandler && [linkAnnotHandler onPageViewShouldBegin:pageIndex recognizer:recognizer annot:nil] && ![recognizer isKindOfClass:[UIPanGestureRecognizer class]]) {
                    return YES;
                }
                
                id<IToolHandler> selectTool = [self getToolHandlerByName:Tool_Select];
                if (self.currentToolHandler != selectTool && [selectTool onPageViewShouldBegin:pageIndex recognizer:recognizer]) {
                    return YES;
                }
            }
        }
        return NO;
    }
    
    
    if (self.currentState == FSUIManagerStateRedaction) {
        return NO;
    }
    
    // return no here,
    // make the pan gesture recognized by the page container
    if (self.currentToolHandler != nil) {
        NSString *name = [self.currentToolHandler getName];
        if (name != nil) {
            if ([name isEqualToString:Tool_Fill_Sign]) {
                return [self.currentToolHandler onPageViewShouldBegin:pageIndex recognizer:recognizer];
            }
            if ([name isEqualToString:Tool_Pencil]) {
                return [self.currentToolHandler onPageViewShouldBegin:pageIndex recognizer:recognizer];
            }
            return YES;
        }
    }

    for (id<IGestureEventListener> listener in self.guestureEventListeners) {
        if ([listener respondsToSelector:@selector(onShouldBegin:)]) {
            if ([listener onShouldBegin:recognizer])
                return YES;
        }
    }

    return NO;
}

- (BOOL)onTouchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {

    UITouch *touch = [touches anyObject];
    CGPoint point = [touch locationInView:[_pdfViewCtrl getDisplayView]];
    int pageIndex = [_pdfViewCtrl getPageIndex:point];
    if (pageIndex < 0)
        return NO;
    
    self.currentPoint = point;
    self.currentPageIndex = pageIndex;
    if (self.currentState == FSUIManagerStateRedaction || [_pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_REFLOW) {
        return NO;
    }
    if (_currentToolHandler != nil) {
        if ([_currentToolHandler onPageViewTouchesBegan:pageIndex touches:touches withEvent:event]) {
            return YES;
        }
    } else {
        //annot handler
        FSAnnot *annot = nil;
        id<IAnnotHandler> annotHandler = nil;
        annot = self.currentAnnot;
        if (annot != nil && ![annot isEmpty]) {
            annotHandler = [self getAnnotHandlerByAnnot:annot];
            return [annotHandler onPageViewTouchesBegan:pageIndex touches:touches withEvent:event annot:annot];
        }
        point = [[touches anyObject] locationInView:[_pdfViewCtrl getPageView:pageIndex]];
        
        if ([_pdfViewCtrl isDynamicXFA]) {
            FSXFAWidget *widget = nil;
            widget = self.currentWidget;
            if (widget != nil && ![widget isEmpty]) {
                annotHandler = [self getDynamicAnnotHandler];
                return [annotHandler onPageViewTouchesBegan:pageIndex touches:touches withEvent:event widget:widget];
            }
            
            FSXFAWidget *xfaWidget = [self getXFAWidgetAtPoint:point pageIndex:pageIndex];
            if (xfaWidget != nil && ![xfaWidget isEmpty]) {
                id<IAnnotHandler> dynamicXFAHandler = [self getDynamicAnnotHandler];
                if (dynamicXFAHandler != nil) {
                    return [dynamicXFAHandler onPageViewTouchesBegan:pageIndex touches:touches withEvent:event widget:xfaWidget];
                }
            }
        }
        
        annot = [self getAnnotAtPoint:point pageIndex:pageIndex];
        if (annot != nil && ![annot isEmpty]) {
            annotHandler = [self getAnnotHandlerByAnnot:annot];
            if (annotHandler != nil) {
                return [annotHandler onPageViewTouchesBegan:pageIndex touches:touches withEvent:event annot:annot];
            }
        }
        return NO;
    }
    return NO;
}
- (BOOL)onTouchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {

    UITouch *touch = [touches anyObject];
    CGPoint point = [touch locationInView:[_pdfViewCtrl getDisplayView]];
    int pageIndex = [_pdfViewCtrl getPageIndex:point];
    if (pageIndex < 0)
        return NO;

    if ([_pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_REFLOW || self.currentState == FSUIManagerStateRedaction) {
        return NO;
    }
    if (_currentToolHandler != nil) {
        if ([_currentToolHandler onPageViewTouchesMoved:pageIndex touches:touches withEvent:event]) {
            return YES;
        }
    } else {
        //annot handler
        FSAnnot *annot = nil;
        id<IAnnotHandler> annotHandler = nil;
        annot = self.currentAnnot;
        if (annot != nil && ![annot isEmpty]) {
            annotHandler = [self getAnnotHandlerByAnnot:annot];
            return [annotHandler onPageViewTouchesMoved:pageIndex touches:touches withEvent:event annot:annot];
        }
        point = [[touches anyObject] locationInView:[_pdfViewCtrl getPageView:pageIndex]];
        annot = [self getAnnotAtPoint:point pageIndex:pageIndex];
        if (annot != nil && ![annot isEmpty]) {
            annotHandler = [self getAnnotHandlerByAnnot:annot];
            if (annotHandler != nil) {
                return [annotHandler onPageViewTouchesMoved:pageIndex touches:touches withEvent:event annot:annot];
            }
        }
        return NO;
    }
    return NO;
}

- (BOOL)onTouchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
    UITouch *touch = [touches anyObject];
    CGPoint point = [touch locationInView:[_pdfViewCtrl getDisplayView]];
    int pageIndex = [_pdfViewCtrl getPageIndex:point];
    if (pageIndex < 0){
        if (_currentToolHandler != nil &&[[_currentToolHandler getName] isEqualToString:Tool_Pencil]) {
            if ([_currentToolHandler onPageViewTouchesEnded:pageIndex touches:touches withEvent:event]) {
                return YES;
            }
        }
        return NO;
    }

    if ([_pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_REFLOW || self.currentState == FSUIManagerStateRedaction) {
        return NO;
    }
    /** The current tool will handle the tourches first if it's actived. */
    if (_currentToolHandler != nil) {
        if ([_currentToolHandler onPageViewTouchesEnded:pageIndex touches:touches withEvent:event]) {
            return YES;
        }
    } else {
        //annot handler
        FSAnnot *annot = nil;
        id<IAnnotHandler> annotHandler = nil;
        annot = self.currentAnnot;
        if (annot != nil && ![annot isEmpty]) {
            annotHandler = [self getAnnotHandlerByAnnot:annot];
            return [annotHandler onPageViewTouchesEnded:pageIndex touches:touches withEvent:event annot:annot];
        }
        
        if ([_pdfViewCtrl isDynamicXFA]) {
            FSXFAWidget *widget = nil;
            widget = self.currentWidget;
            if (widget != nil && ![widget isEmpty]) {
                annotHandler = [self getDynamicAnnotHandler];
                return [annotHandler onPageViewTouchesEnded:pageIndex touches:touches withEvent:event widget:widget];
            }
            
            FSXFAWidget *xfaWidget = [self getXFAWidgetAtPoint:point pageIndex:pageIndex];
            if (xfaWidget != nil && ![xfaWidget isEmpty]) {
                id<IAnnotHandler> dynamicXFAHandler = [self getDynamicAnnotHandler];
                if (dynamicXFAHandler != nil) {
                    return [dynamicXFAHandler onPageViewTouchesEnded:pageIndex touches:touches withEvent:event widget:xfaWidget];
                }
            }
        }
        
        point = [[touches anyObject] locationInView:[_pdfViewCtrl getPageView:pageIndex]];
        annot = [self getAnnotAtPoint:point pageIndex:pageIndex];
        if (annot != nil && ![annot isEmpty]) {
            annotHandler = [self getAnnotHandlerByAnnot:annot];
            if (annotHandler != nil) {
                return [annotHandler onPageViewTouchesEnded:pageIndex touches:touches withEvent:event annot:annot];
            }
        }
        return NO;
    }
    return NO;
}

- (BOOL)onTouchesCancelled:(NSSet *)touches withEvent:(UIEvent *)event {
    UITouch *touch = [touches anyObject];
    CGPoint point = [touch locationInView:[_pdfViewCtrl getDisplayView]];
    int pageIndex = [_pdfViewCtrl getPageIndex:point];
    if (pageIndex < 0)
        return NO;

    if ([_pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_REFLOW || self.currentState == FSUIManagerStateRedaction) {
        return NO;
    }

    if (_currentToolHandler != nil) {
        if ([_currentToolHandler onPageViewTouchesCancelled:pageIndex touches:touches withEvent:event]) {
            return YES;
        }
    } else {
        //annot handler
        FSAnnot *annot = nil;
        id<IAnnotHandler> annotHandler = nil;
        annot = self.currentAnnot;
        if (annot != nil && ![annot isEmpty]) {
            annotHandler = [self getAnnotHandlerByAnnot:annot];
            return [annotHandler onPageViewTouchesCancelled:pageIndex touches:touches withEvent:event annot:annot];
        }
        point = [[touches anyObject] locationInView:[_pdfViewCtrl getPageView:pageIndex]];
        annot = [self getAnnotAtPoint:point pageIndex:pageIndex];
        if (annot != nil && ![annot isEmpty]) {
            annotHandler = [self getAnnotHandlerByAnnot:annot];
            if (annotHandler != nil) {
                return [annotHandler onPageViewTouchesCancelled:pageIndex touches:touches withEvent:event annot:annot];
            }
        }
        return NO;
    }
    return NO;
}

#pragma mark IDrawEventListener
- (void)onDraw:(int)pageIndex inContext:(CGContextRef)context {
    id<IToolHandler> currentToolHandler = self.currentToolHandler;
    for (id<IToolHandler> handler in self.toolHandlers) {
        if (currentToolHandler == handler) continue;
        if ([handler respondsToSelector:@selector(onDraw:inContext:)]) {
            [handler onDraw:pageIndex inContext:context];
        }
    }
    
    id<IAnnotHandler> annotHandler = [self getAnnotHandlerByType:FSAnnotLink];
    if (annotHandler) {
        if ([annotHandler respondsToSelector:@selector(onDraw:inContext:annot:)]) {
            [annotHandler onDraw:pageIndex inContext:context annot:nil];
        }
    }

    if (self.currentAnnot) {
        id<IAnnotHandler> annotHandler = [self getAnnotHandlerByAnnot:self.currentAnnot];
        if ([annotHandler respondsToSelector:@selector(onDraw:inContext:annot:)]) {
            [annotHandler onDraw:pageIndex inContext:context annot:self.currentAnnot];
        }
    }
    
    if (self.currentWidget) {
        id<IAnnotHandler> annotHandler = [self getDynamicAnnotHandler];
        if (annotHandler) {
            [annotHandler onDraw:pageIndex inContext:context widget:self.currentWidget];
        }
    }

    if (currentToolHandler) {
        if ([currentToolHandler respondsToSelector:@selector(onDraw:inContext:)]) {
            [currentToolHandler onDraw:pageIndex inContext:context];
        }
    }
}

- (void)setDocModified:(BOOL)isDocModified{
    [self supportsMultipleScenesOp:^(UIExtensionsManager * _Nonnull self) {
        [self willChangeValueForKey:@"isDocModified"];
        self->_isDocModified = isDocModified;
        if (isDocModified) {
            for (id<IDocModifiedEventListener> listener in self.docModifiedListeners) {
                if ([listener respondsToSelector:@selector(onDocModified:)]) {
                    [listener onDocModified:self.pdfViewCtrl.currentDoc];
                }
            }
        }
        [self didChangeValueForKey:@"isDocModified"];
    }];
}

- (void)onAnnotAdded:(FSPDFPage *)page annot:(FSAnnot *)annot {
    [self supportsMultipleScenesOp:^(UIExtensionsManager * _Nonnull self) {
        for (id<IAnnotEventListener> listener in self.annotListeners) {
            if ([listener respondsToSelector:@selector(onAnnotAdded:annot:)]) {
                [listener onAnnotAdded:page annot:annot];
            }
        }
        self.isDocModified = YES;
        if ([self getState] != FSUIManagerStateAnnotTool)
            return;
        if (!self.continueAddAnnot && ![@[ Tool_Pencil, Tool_Callout, Tool_Polygon, Tool_PolyLine ] containsObject:[self.currentToolHandler getName]]) {
            [self setCurrentToolHandler:nil];
        }
    }];
}

- (void)onAnnotsAdded:(NSArray<FSAnnot *> *)annots{
    [self supportsMultipleScenesOp:^(UIExtensionsManager * _Nonnull self) {
        for (id<IAnnotEventListener> listener in self.annotListeners) {
            if ([listener respondsToSelector:@selector(onAnnotsAdded:)]) {
                [listener onAnnotsAdded:annots];
            }
        }
        self.isDocModified = YES;
        if ([self getState] != FSUIManagerStateAnnotTool)
            return;
        if (!self.continueAddAnnot && ![@[ Tool_Pencil, Tool_Callout, Tool_Polygon, Tool_PolyLine ] containsObject:[self.currentToolHandler getName]]) {
            [self setCurrentToolHandler:nil];
        }
    }];
}

- (void)onAnnotWillDelete:(FSPDFPage *)page annot:(FSAnnot *)annot {
    [self supportsMultipleScenesOp:^(UIExtensionsManager * _Nonnull self) {
        if (annot == self.currentAnnot) {
            self.currentAnnot = nil;
        }

        for (id<IAnnotEventListener> listener in self.annotListeners) {
            if ([listener respondsToSelector:@selector(onAnnotWillDelete:annot:)]) {
                [listener onAnnotWillDelete:page annot:annot];
            }
        }
    }];
}

- (void)onAnnotDeleted:(FSPDFPage *)page annot:(FSAnnot *)annot {
    for (id<IAnnotEventListener> listener in self.annotListeners) {
        if ([listener respondsToSelector:@selector(onAnnotDeleted:annot:)]) {
            [listener onAnnotDeleted:page annot:annot];
        }
    }
    self.isDocModified = YES;
}

- (void)onAnnotsWillDelete:(NSArray<FSAnnot *> *)annots{

    [self supportsMultipleScenesOp:^(UIExtensionsManager * _Nonnull self) {
        for (FSAnnot *annot in annots) {
            if (annot == self.currentAnnot){
                self.currentAnnot = nil;
                break;
            }
        }

        for (id<IAnnotEventListener> listener in self.annotListeners) {
            if ([listener respondsToSelector:@selector(onAnnotsWillDelete:)]) {
                [listener onAnnotsWillDelete:annots];
            }
        }
    }];
}

- (void)onAnnotModified:(FSPDFPage *)page annot:(FSAnnot *)annot {
    for (id<IAnnotEventListener> listener in self.annotListeners) {
        if ([listener respondsToSelector:@selector(onAnnotModified:annot:)]) {
            [listener onAnnotModified:page annot:annot];
        }
    }

    self.isDocModified = YES;
}
- (void)onAnnotSelected:(FSPDFPage *)page annot:(FSAnnot *)annot {
    for (id<IAnnotEventListener> listener in self.annotListeners) {
        if ([listener respondsToSelector:@selector(onAnnotSelected:annot:)]) {
            [listener onAnnotSelected:page annot:annot];
        }
    }
}

- (void)onAnnotDeselected:(FSPDFPage *)page annot:(FSAnnot *)annot {
    for (id<IAnnotEventListener> listener in self.annotListeners) {
        if ([listener respondsToSelector:@selector(onAnnotDeselected:annot:)]) {
            [listener onAnnotDeselected:page annot:annot];
        }
    }
}

- (void)onAnnotWillFlatten:(FSPDFPage *)page annot:(FSAnnot *)annot{
    [self supportsMultipleScenesOp:^(UIExtensionsManager * _Nonnull self) {
        if (annot == self.currentAnnot) {
            self.currentAnnot = nil;
        }

        for (id<IAnnotEventListener> listener in self.annotListeners) {
            if ([listener respondsToSelector:@selector(onAnnotWillFlatten:annot:)]) {
                [listener onAnnotWillFlatten:page annot:annot];
            }
        }
    }];
}

- (void)onAnnotFlattened:(FSPDFPage *)page annot:(FSAnnot *)annot {
    for (id<IAnnotEventListener> listener in self.annotListeners) {
        if ([listener respondsToSelector:@selector(onAnnotFlattened:annot:)]) {
            [listener onAnnotFlattened:page annot:annot];
        }
    }
    self.isDocModified = YES;
}

#pragma mark <IScrollViewEventListener>

- (void)onScrollViewWillBeginDragging:(UIScrollView *)scrollView {
    self.isScrollViewDragging = YES;
    if (self.shouldShowPropertyBar) {
        [self hidePropertyBar];
    } else if (self.shouldShowMenu) {
        [self hideMenu];
    }
}

- (void)onScrollViewDidEndDragging:(UIScrollView *)scrollView willDecelerate:(BOOL)decelerate {
    self.isScrollViewDragging = NO;
    if (!decelerate) { // if will decelerate, show menu or propert bar when end decelerating
        if (self.shouldShowPropertyBar) {
            [self hidePropertyBar];
        }
        if (self.shouldShowMenu) {
            [self showMenu];
        }
    }
}

- (void)onScrollViewWillBeginDecelerating:(UIScrollView *)scrollView {
    self.isScrollViewDecelerating = YES;
}

- (void)onScrollViewDidEndDecelerating:(UIScrollView *)scrollView {
    self.isScrollViewDecelerating = NO;
    if (self.shouldShowPropertyBar) {
        [self hidePropertyBar];
    }
    if (self.shouldShowMenu) {
        [self showMenu];
    }
}

- (void)onScrollViewWillBeginZooming:(UIScrollView *)scrollView {
    self.isScrollViewZooming = YES;
    if (self.shouldShowPropertyBar) {
        [self hidePropertyBar];
    } else if (self.shouldShowMenu) {
        [self hideMenu];
    }
}

- (void)onScrollViewDidEndZooming:(UIScrollView *)scrollView {
    self.isScrollViewZooming = NO;
    if (self.shouldShowPropertyBar) {
        [self hidePropertyBar];
    }
    if (self.shouldShowMenu) {
        [self showMenu];
    }
}

- (void)setCurrentAnnot:(FSAnnot *)annot {
    if ([Utility isAnnot:annot uniqueToAnnot:self.currentAnnot]) {
        return;
    }

    if (self.currentAnnot != nil && ![self.currentAnnot isEmpty]) {
        id<IAnnotHandler> annotHandler = [self getAnnotHandlerByAnnot:self.currentAnnot];
        if (annotHandler) {
            [annotHandler onAnnotDeselected:self.currentAnnot];
            [self onAnnotDeselected:[annot getPage] annot:self.currentAnnot];
        }
    }

    _currentAnnot = annot;

    if (annot == nil || [annot isEmpty]) {
        self.shouldShowMenu = NO;
        self.shouldShowPropertyBar = NO;
        self.menuControl.menuItems = nil;
        [self hideMenu];
        [self hidePropertyBar];
        if (self.currentToolHandler.type == FSAnnotInk) {
            [self.currentToolHandler onDeactivate];
        }
    } else {
        
        void (^block)(void) = ^{
            id<IAnnotHandler> annotHandler = [self getAnnotHandlerByAnnot:annot];
            if (annotHandler) {
                [annotHandler onAnnotSelected:annot];
                [self onAnnotSelected:[annot getPage] annot:annot];
            }
        };
        if([self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_REFLOW){
            dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.3 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                block();
            });
            return;
        }
        
        FSRectF *fsrect = annot.type == FSAnnotCaret ? [Utility getCaretAnnotRect:[[FSCaret alloc] initWithAnnot:annot]] : annot.fsrect;
        
        if ([self isVisibleRect:fsrect pageIndex:annot.pageIndex needAdjust:YES] ) {
            block();
        }else{
            dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.3 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                block();
            });
        }
    }
}

- (void)setCurrentWidget:(FSXFAWidget *)currentWidget {
    
    if (self.currentWidget != nil && ![self.currentWidget isEmpty]) {
        id<IAnnotHandler> annotHandler = [self getDynamicAnnotHandler];
        if (annotHandler) {
            [annotHandler onXFAWidgetDeselected:currentWidget];
        }
    }
    
    _currentWidget = currentWidget;
    
    if (currentWidget == nil) {
        self.shouldShowMenu = NO;
        self.shouldShowPropertyBar = NO;
        [self hideMenu];
        [self hidePropertyBar];
    } else {
        
        void (^block)(void) = ^{
            id<IAnnotHandler> annotHandler = [self getDynamicAnnotHandler];
            if (annotHandler) {
                [annotHandler onXFAWidgetSelected:currentWidget];
            }
        };
        
        FSRectF *fsrect = [currentWidget getRect];
        if ([self isVisibleRect:fsrect pageIndex:currentWidget.pageIndex needAdjust:YES]) {
            block();
        }else{
            dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.3 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                block();
            });
        }
    }
}

- (BOOL)isVisibleRect:(FSRectF *)rect pageIndex:(int)pageIndex needAdjust:(BOOL)needAdjust{
    CGRect pvRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:rect pageIndex:pageIndex];
    pvRect = CGRectIntersection([_pdfViewCtrl getPageView:pageIndex].bounds, pvRect);
    CGRect dvRect = [_pdfViewCtrl convertPageViewRectToDisplayViewRect:pvRect pageIndex:pageIndex];
    CGRect visibleRect = _pdfViewCtrl.bounds;
    float topHeight = [self topHeight];
    visibleRect.origin.y += topHeight;
    visibleRect.size.height -= topHeight;
    BOOL isVisible = CGRectContainsRect(visibleRect, dvRect);
    if (!isVisible && needAdjust) {
        CGRect pvRect = [self.pdfViewCtrl convertPdfRectToPageViewRect:rect pageIndex:pageIndex];

       __block BOOL canGoTo = YES;
        FSPointF * (^leftPt)(CGRect) = ^(CGRect pvRect){
            canGoTo = NO;
            FSPointF *fspt = [[FSPointF alloc] init];
            CGRect visibleRectToPageRect = [self.pdfViewCtrl convertRect:self.pdfViewCtrl.bounds toView:[self.pdfViewCtrl getPageView:pageIndex]];
            if (CGRectGetMaxY(pvRect) > CGRectGetMaxY(visibleRectToPageRect) || CGRectGetMinY(pvRect) < CGRectGetMinY(visibleRectToPageRect)) {
                pvRect.origin.y -= topHeight;
                canGoTo = YES;
            }
            FSRectF *fsrect = [self.pdfViewCtrl convertPageViewRectToPdfRect:pvRect pageIndex:pageIndex];
            [fspt set:fsrect.left y:fsrect.top];
            return fspt;
        };

        FSPointF *fspt = [[FSPointF alloc] init];
        BOOL isIntersect = CGRectIntersectsRect(visibleRect, dvRect);
        if (isIntersect && visibleRect.size.width > (dvRect.size.width + 10) && visibleRect.size.height > (dvRect.size.height + 10)) {
            CGRect pvRectToPdfDisplayRect = [[_pdfViewCtrl getPageView:pageIndex] convertRect:pvRect toView:[_pdfViewCtrl getDisplayView]];
            if (!CGRectContainsPoint(visibleRect, CGPointMake(CGRectGetMaxX(dvRect), CGRectGetMinY(dvRect))) && !CGRectContainsPoint(visibleRect, CGPointMake(CGRectGetMaxX(dvRect), CGRectGetMaxY(dvRect)))) { //right

                float x = pvRect.origin.x - CGRectGetWidth(visibleRect) + CGRectGetWidth(pvRect) + 10.f;
                CGPoint pt = CGPointMake(x, pvRect.origin.y - pvRectToPdfDisplayRect.origin.y);
                fspt = [_pdfViewCtrl convertPageViewPtToPdfPt:pt pageIndex:pageIndex];
            }else if (!CGRectContainsPoint(visibleRect, CGPointMake(CGRectGetMinX(dvRect), CGRectGetMinY(dvRect))) && !CGRectContainsPoint(visibleRect, CGPointMake(CGRectGetMinX(dvRect), CGRectGetMaxY(dvRect)))) { //left
                CGPoint pt = CGPointMake(pvRect.origin.x - 10, pvRect.origin.y - pvRectToPdfDisplayRect.origin.y);
                fspt = [_pdfViewCtrl convertPageViewPtToPdfPt:pt pageIndex:pageIndex];
            }else if (!CGRectContainsPoint(visibleRect, CGPointMake(CGRectGetMinX(dvRect), CGRectGetMaxY(dvRect))) && !CGRectContainsPoint(visibleRect, CGPointMake(CGRectGetMaxX(dvRect), CGRectGetMaxY(dvRect)))) { //bottom

                float y = pvRect.origin.y - CGRectGetHeight(visibleRect) + CGRectGetHeight(pvRect) + 10.f;
                CGPoint pt = CGPointMake(pvRect.origin.x - pvRectToPdfDisplayRect.origin.x, y);
                fspt = [_pdfViewCtrl convertPageViewPtToPdfPt:pt pageIndex:pageIndex];
            }else{
                fspt = leftPt(pvRect);
            }
        }else{
            fspt = leftPt(pvRect);
        }
        if (canGoTo) {
            [_pdfViewCtrl gotoPage:pageIndex withDocPoint:fspt animated:YES];
        }
    }
    return isVisible;
}

#pragma mark - annot property

- (void)showPropertyWithToolSettings:(FSReadToolSettings *)toolSettings rect:(CGRect)rect inView:(UIView *)view{
    DevicePosition *devicePos = [[DevicePosition alloc] init];
    devicePos.phoneRect = rect;
    devicePos.inPhoneView = self.pdfViewCtrl;
    devicePos.inPadView = view;
    devicePos.padRect = rect;
    [self showPropertyWithToolSettings:toolSettings devicePos:devicePos];
}

- (void)showPropertyWithToolSettings:(FSReadToolSettings *)toolSettings devicePos:(DevicePosition*)devicePos{
    
    if (!toolSettings || !devicePos) {
        return;
    }
    self.lastPropertyToolSettings = toolSettings;
    self.devicePos = devicePos;
    // stamp
    unsigned int settingsColor = toolSettings.settingsColor;
    unsigned int fillColor = (toolSettings.fillColor.rgbaValue == 0xFFFFFF00 ? 0xFFFFFF00 : toolSettings.fillColor.rgbValue);
    CGFloat opacity = toolSettings.opacity;
    int icon = toolSettings.settingsType == FSReadToolSettingsTypeNote ? toolSettings.noteIcon : toolSettings.attachmentIcon;
    NSString *fontName = toolSettings.textFace;
    float fontSize = toolSettings.textSize;
    float lineWidth = toolSettings.settingsType == FSReadToolSettingsTypeEraser ? toolSettings.eraserLineWidth : toolSettings.thickness;
    NSString *distanceUnit = toolSettings.distanceUnit;
    int rotation = toolSettings.rotation;
    
    FSReadToolSettingsType settingsType = toolSettings.settingsType;
    if (settingsType == FSReadToolSettingsTypeStamp) {
        {
            __weak UIExtensionsManager *weakSelf = self;
            self.stampIconController = [[StampIconController alloc] initWithUIExtensionsManager:self toolSettings:toolSettings];
            self.stampIconController.currentIcon = toolSettings.stampIcon;
            self.stampIconController.modalPresentationStyle = UIModalPresentationFullScreen;
            self.stampIconController.selectHandler = ^(int icon) {
                toolSettings.stampIcon = icon;
                for (id<IAnnotPropertyListener> listener in weakSelf.annotPropertyListeners) {
                    if ([listener respondsToSelector:@selector(onToolSettingsChanged:)]) {
                        [listener onToolSettingsChanged:toolSettings];
                    }
                }
            };
            self.stampIconController.closeHandler = ^{
                weakSelf.stampIconController = nil;
            };
        }

        if (DEVICE_iPAD && ![_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            self.stampIconController.modalPresentationStyle = UIModalPresentationPopover;
            self.stampIconController.preferredContentSize = CGSizeMake(380, 420);
            self.stampIconController.popoverPresentationController.delegate = self;
            self.stampIconController.popoverPresentationController.sourceRect = devicePos.padRect;
            self.stampIconController.popoverPresentationController.sourceView = devicePos.inPadView;
            UIPopoverArrowDirection permittedArrowDirections;
            if ([devicePos.inPadView isDescendantOfView:[self.topToolbar subMenuTopbarContentView]]) {
                if (self.topToolbar.subitemContentPosition == FSTopbarSubitemContentViewPositionLeft) {
                    permittedArrowDirections = UIPopoverArrowDirectionLeft;
                } else  if (self.topToolbar.subitemContentPosition == FSTopbarSubitemContentViewPositionRight) {
                    permittedArrowDirections = UIPopoverArrowDirectionRight;
                }else{
                    permittedArrowDirections = UIPopoverArrowDirectionUp;
                }
            }else{
                permittedArrowDirections = UIPopoverArrowDirectionRight;
            }
            self.stampIconController.popoverPresentationController.permittedArrowDirections = permittedArrowDirections;
        }

        [[self.viewCtrlManager navigationControllerStack].topNavigationController presentViewController:self.stampIconController animated:YES completion:nil];

        return;
    }
    self.currentToolSettings = toolSettings;
    [self.propertyMainView setPropertyColor:settingsColor];
    [self.propertyMainView setOpacity:opacity];
    [self.propertyMainView setFontName:fontName];
    [self.propertyMainView setFontSize:fontSize];
    [self.propertyMainView setIcon:icon];
    [self.propertyMainView setLineWidth:lineWidth];
    [self.propertyMainView setLock:NO];
    [self.propertyMainView setPencilStyle:FSPenStylePen];
    [self.propertyMainView setUnitName:distanceUnit];
    [self.propertyMainView setRotation:rotation];

    // image
    if (settingsType == FSReadToolSettingsTypeImageAnnotation) {
        [self.propertyMainView setPropertyColor:self.fs_primaryColor.rgbValue];
        [self.propertyMainView resetProperties:@[@(FSPropertyTypeRotation), @(FSPropertyTypeOpacity)]];
    } else if (settingsType == FSReadToolSettingsTypeLine  ||
        settingsType == FSReadToolSettingsTypePolyLine ||
        settingsType == FSReadToolSettingsTypeLine ||
        settingsType == FSReadToolSettingsTypeArrow ||
        settingsType == FSReadToolSettingsTypeHighlighter) {
        if (settingsType == FSReadToolSettingsTypeHighlighter) [self.propertyMainView setLineWidthLayoutValue:FSLineWidthLayoutValueHighlighter];
        [self.propertyMainView resetProperties:@[@(FSPropertyTypePropertyColor), @(FSPropertyTypeLineWidth), @(FSPropertyTypeOpacity)]];
    } else if (settingsType == FSReadToolSettingsTypeTypewriter || settingsType == FSReadToolSettingsTypeCallout || settingsType == FSReadToolSettingsTypeTextbox) {
        [self.propertyMainView resetProperties:@[@(FSPropertyTypePropertyColor), @(FSPropertyTypeOpacity), @(FSPropertyTypeFontName), @(FSPropertyTypeFontSizeFixed)]];
    } else if (settingsType == FSReadToolSettingsTypeEraser) {
        self.propertyMainView.lineWidthLayoutValue = FSLineWidthLayoutValueEraser;
        [self.propertyMainView setEraseStyle:toolSettings.eraseStyle];
        [self.propertyMainView resetProperties:@[@(FSPropertyTypeEarseStyle), @(FSPropertyTypeLineWidth)]];
        
    }else if (settingsType == FSReadToolSettingsTypePencil) {
        [self.propertyMainView setLineWidthLayoutValue:FSLineWidthLayoutValuePen];
        [self.propertyMainView setPencilStyle:toolSettings.pencilStyle];
        [self.propertyMainView resetProperties:@[@(FSPropertyTypePen), @(FSPropertyTypePropertyColor), @(FSPropertyTypeLineWidth) , @(FSPropertyTypeOpacity)]];
    }else if (settingsType == FSReadToolSettingsTypeAttachments) {
        [self.propertyMainView resetProperties:@[@(FSPropertyTypePropertyColor), @(FSPropertyTypeOpacity), @(FSPropertyTypeShapeAttachment)]];
    }
    // distance
    else if (settingsType == FSReadToolSettingsTypeMeasure) {
        [self.propertyMainView resetProperties:@[@(FSPropertyTypeDistanceUnit), @(FSPropertyTypePropertyColor), @(FSPropertyTypeLineWidth) ,@(FSPropertyTypeOpacity)]];
    }else if (settingsType == FSReadToolSettingsTypeNote) {
        [self.propertyMainView resetProperties:@[@(FSPropertyTypePropertyColor), @(FSPropertyTypeOpacity), @(FSPropertyTypeShapeNote)]];
    }else if (settingsType == FSReadToolSettingsTypePolygon || settingsType == FSReadToolSettingsTypeOval || settingsType == FSReadToolSettingsTypeRectangle || settingsType == FSReadToolSettingsTypeCloud) {
        [self.propertyMainView setFillColor:fillColor];
        [self.propertyMainView resetProperties:@[@(FSPropertyTypeBorderColor), @(FSPropertyTypeBorderWidth), @(FSPropertyTypeFillColor), @(FSPropertyTypeOpacity)]];
    }else {
        [self.propertyMainView resetProperties:@[@(FSPropertyTypePropertyColor), @(FSPropertyTypeOpacity)]];
    }
    
    self.propertyMainView.delegate = self;
    [self.propertyMainView showPropertyMainView:devicePos];
}

/** @brief Customized annotation type on application level. */

- (FSAnnotType)filterAnnotType:(FSAnnotType)annotType {
    NSString *toolHandlerName = [self.currentToolHandler getName];
    if (FSAnnotLine == annotType) {
        LineToolHandler *toolHandler = [self getToolHandlerByName:Tool_Line];
        if (toolHandler.isArrowLine)
            return FSAnnotArrowLine;

        if (toolHandler.isDistanceTool) {
            return FSAnnotDistance;
        }
    } else if (FSAnnotCaret == annotType) {
        if ([toolHandlerName isEqualToString:Tool_Insert]) {
            return FSAnnotInsert;
        } else if ([toolHandlerName isEqualToString:Tool_Replace]) {
            return FSAnnotReplace;
        } else if (self.currentAnnot && ![self.currentAnnot.intent isEqualToString:@"Replace"]) {
            return FSAnnotInsert;
        }
    } else if (FSAnnotFreeText == annotType) {
        if ([toolHandlerName isEqualToString:Tool_Textbox]) {
            return FSAnnotTextbox;
        } else if ([toolHandlerName isEqualToString:Tool_Callout]) {
            return FSAnnotCallout;
        } else if (self.currentAnnot && self.currentAnnot.intent.length == 0) {
            return FSAnnotTextbox;
        } else if (self.currentAnnot && [self.currentAnnot.intent isEqualToString:@"FreeTextCallout"]) {
            return FSAnnotCallout;
        }
    } else if (FSAnnotPolygon == annotType) {
        BOOL isPolygon = YES;
        if ([[self.currentToolHandler getName] isEqualToString:Tool_Polygon]) {
            isPolygon = ((PolygonToolHandler *) self.currentToolHandler).isPolygon;
        } else if (self.currentAnnot && self.currentAnnot.type == FSAnnotPolygon) {
            isPolygon = !([[([[FSPolygon alloc] initWithAnnot:self.currentAnnot]) getBorderInfo] getStyle] == FSBorderInfoCloudy);
        }
        if (!isPolygon) {
            return FSAnnotCloud;
        }
    }else if (annotType == FSAnnotScreen) {
        if ([[self.currentToolHandler getName] isEqualToString:Tool_Image] ||
            (self.currentAnnot && self.currentAnnot.screenActiontype == FSScreenActionTypeImage)) {
            return FSAnnotImage;
        }
    }

    return (int)annotType;
}

- (FSReadToolSettings *)getReadToolSettingsForType:(FSReadToolSettingsType)settingsType{
    FSReadToolSettings *toolSettings = self.defaultReadToolSettingItems[@(settingsType)];
    if (toolSettings) return toolSettings;
    toolSettings = [[FSReadToolSettings alloc] initWithToolSettingType:settingsType extensionsManager:self];
    self.defaultReadToolSettingItems[@(settingsType)] = toolSettings;
    return toolSettings;
}


-(BOOL)checkValidAnnotType:(FSAnnotType)annotType{
    NSArray *unsupportAnnotTypes = @[@(FSAnnotPrinterMark), @(FSAnnotTrapNet), @(FSAnnotWatermark), @(FSAnnot3D), @(FSAnnotPopup), @(FSAnnotUnknownType), @(FSAnnotLink), @(FSAnnotPSInk), @(FSAnnotSound), @(FSAnnotMovie), @(FSAnnotWidget)];
    if ([unsupportAnnotTypes containsObject:@(annotType)]) {
        return NO;
    }
    
    NSMutableArray *annotTypesArr = @[].mutableCopy;
    for (int i =0 ; i < 28 ; i++) {
        [annotTypesArr addObject:@(i)];
    }
    
    NSArray *customUIFSAnnotType = @[@(FSAnnotArrowLine), @(FSAnnotInsert), @(FSAnnotReplace), @(FSAnnotTextbox), @(FSAnnotCloud), @(FSAnnotCallout), @(FSAnnotDistance)];
    if (![annotTypesArr containsObject:@(annotType)] && ![customUIFSAnnotType containsObject:@(annotType)] ) {
        return NO;
    }
    return  YES;
}

- (void)registerAnnotPropertyListener:(id<IAnnotPropertyListener>)listener {
    [self.annotPropertyListeners addObject:listener];
}

- (void)unregisterAnnotPropertyListener:(id<IAnnotPropertyListener>)listener {
    if (listener) {
        [self.annotPropertyListeners removeObject:listener];
    }
}

- (void)showSearchBar:(BOOL)show {
    __block SearchModule *search = nil;
    [self.modules enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop) {
        id<IModule> module = obj;
        if ([module respondsToSelector:@selector(getName)] &&
            [[module getName] isEqualToString:@"Search"]) {
            search = (SearchModule *) module;
            *stop = YES;
        }
    }];
        [search showSearchBar:show];
}

- (NSString *)getCurrentSelectedText {
    id<IToolHandler> toolHandler = [self getToolHandlerByName:Tool_Select];
    if (!toolHandler)
        return nil;
    SelectToolHandler *selHandler = (SelectToolHandler *) toolHandler;
    return [selHandler copyText];
}

#pragma mark - IRecoveryEventListener

- (void)onWillRecover {
    self.currentAnnot = nil;
    self.currentToolHandler = nil;
}

- (void)onRecovered {
    [FSLibrary setAnnotIconProviderCallback:[FSCallbackSubstitute annotIconProviderCallbackSubstitute]];
    [FSLibrary setActionCallback:[FSCallbackSubstitute actionHandlerSubstitute]];
}

- (BOOL)shouldDrawAnnot:(FSAnnot *)annot {
    if (annot && ![annot isEmpty]) {
        id<IAnnotHandler> annotHandler = [self getAnnotHandlerByAnnot:annot];
        if ([annotHandler respondsToSelector:@selector(shouldDrawAnnot:inPDFViewCtrl:)]) {
            BOOL results = [annotHandler shouldDrawAnnot:annot inPDFViewCtrl:self.pdfViewCtrl];
            if (results) {
                UIExtensionsManager *currentExtensionsManager = UIExtensionsManager.currentExtManager;
                if (self.shouldSyncOtherScenes) {
                    id<IAnnotHandler> currentAnnotHandler = [currentExtensionsManager getAnnotHandlerByAnnot:annot];
                    if ([currentAnnotHandler isKindOfClass:[annotHandler class]]) {
                        if (![currentAnnotHandler shouldDrawAnnot:annot inPDFViewCtrl:currentExtensionsManager.pdfViewCtrl]) {
                            results = NO;
                        }
                    }
                }
            }
            return results;
        }
    }
    return YES;
}

#pragma mark <UIPopoverPresentationControllerDelegate>

- (void)popoverPresentationControllerDidDismissPopover:(UIPopoverPresentationController *)popoverPresentationController {
    if (_settingbarPopoverCtr) {
        [self setHiddenSettingBar:YES];
        _settingbarPopoverCtr = nil;
    }
    if (self.isSaveAsAlertControllerShow) {
        self.isSaveAsAlertControllerShow = NO;
        [self delayAutoFullScreen];
    }
    if (self.stampIconController) {
        self.stampIconController = nil;
    }
}

- (UIModalPresentationStyle)adaptivePresentationStyleForPresentationController:(UIPresentationController *)controller {
    return UIModalPresentationNone;
}

- (void)buildToolbars {
    [self.toolbarManager buildTopbar];
    [self.toolbarManager buildBottombar];
    
    self.settingBar = [[FSSettingBar alloc] initWithUIExtensionsManager:self];
    self.settingBar.delegate = self;
    [self setHiddenSettingBar:YES animated:NO];
    if (self.config.defaultSettings.fullscreen) {
        [self.fullScreenToolbarMgr buildFullScreenToolbar];
    }
}

- (void)buildItems{
    [self.toolbarManager addMainItems];
}

- (void)saveAndCloseCurrentDoc:(void (^_Nullable)(BOOL success))completion {
    NSString *filePath = self.pdfViewCtrl.filePath;
    
    if (self.preventOverrideFilePath && self.preventOverrideFilePath.length > 0 && ![filePath isEqualToString:self.preventOverrideFilePath]) {
        BOOL isOK = YES;
        isOK = [self.pdfViewCtrl saveDoc:self.preventOverrideFilePath flag:self.docSaveFlag];
        
        [self.pdfViewCtrl closeDoc:^() {
            if (completion) {
                completion(isOK);
            }
        }];
        
        return;
    }
    
    NSString *intermediateFilePath = nil;
    
    NSFileManager *fileManager = [NSFileManager defaultManager];
    BOOL isAsync = ![fileManager fileExistsAtPath:filePath];
    if (filePath) {
        if (!isAsync) {
            NSString *tempDir = NSTemporaryDirectory();
            intermediateFilePath = [tempDir stringByAppendingPathComponent:[filePath lastPathComponent]];
            [self.pdfViewCtrl saveDoc:intermediateFilePath flag:self.docSaveFlag];
        } else {
            NSString *fileName = [filePath lastPathComponent];
            NSString *path = [DOCUMENT_PATH stringByAppendingPathComponent:fileName];
            if ([fileManager fileExistsAtPath:path]) {
                [fileManager removeItemAtPath:path error:nil];
            }
            NSLog(@"saving async doc to %@...", path);
            // todo wei, show alert view with cancel button
            [self.pdfViewCtrl saveDoc:path flag:self.docSaveFlag];
        }
    }
    filePath = self.pdfViewCtrl.filePath;
    [self.pdfViewCtrl closeDoc:^() {
        BOOL isOK = YES;
        if (filePath && !isAsync) {
            NSError *error = nil;
            if ([fileManager fileExistsAtPath:intermediateFilePath]) {
                [fileManager removeItemAtPath:filePath error:nil];
                isOK = [fileManager moveItemAtPath:intermediateFilePath toPath:filePath error:&error];
            }
        }
        if (completion) {
            completion(isOK);
        }
    }];
}

- (void)saveAndCloseCurrentDoc:(NSString *)filePath completion:(void (^_Nullable)(BOOL success))completion {
    
    NSString *intermediateFilePath = nil;
    
    NSFileManager *fileManager = [NSFileManager defaultManager];
    BOOL isExists = [fileManager fileExistsAtPath:filePath];
    if (filePath) {
        if (isExists) {
            NSString *tempDir = NSTemporaryDirectory();
            intermediateFilePath = [tempDir stringByAppendingPathComponent:[filePath lastPathComponent]];
            [self.pdfViewCtrl saveDoc:intermediateFilePath flag:self.docSaveFlag];
        } else {
            [self.pdfViewCtrl saveDoc:filePath flag:self.docSaveFlag];
        }
    }
    
    [self.pdfViewCtrl closeDoc:^() {
        BOOL isOK = YES;
        if (filePath && isExists) {
            NSError *error = nil;
            if ([fileManager fileExistsAtPath:intermediateFilePath]) {
                [fileManager removeItemAtPath:filePath error:nil];
                isOK = [fileManager moveItemAtPath:intermediateFilePath toPath:filePath error:&error];
            }
        }
        if (completion) {
            completion(isOK);
        }
    }];
}

- (void)buttonSaveClick {
    self.isFileEdited = YES;
    
    if (!self.enableModifyDocument) {
        if (self.goBack) {
            self.goBack();
        }
    }
    
    [self saveAndCloseCurrentDoc:^(BOOL success) {
        if (self.goBack && success)
            self.goBack();
    }];
}

- (void)buttonSaveAsClick {
    self.isFileEdited = YES;
    
    [self documentSaveAS:nil error:nil];
}

-(void)documentSaveAS:(void (^_Nullable)(void))successed error:(void (^_Nullable)(void))error {
    
    FSFileSelectDestinationViewController *selectDestination = [[FSFileSelectDestinationViewController alloc] init];
    selectDestination.isRootFileDirectory = YES;
    selectDestination.fileSelectDestinationType = FSFileSelectDestinationTypeSelectFolder;
    selectDestination.expectFileType = @[@"*"];
    [selectDestination loadFilesWithPath:DOCUMENT_PATH];
    selectDestination.operatingHandler = ^(FSFileSelectDestinationViewController *controller, NSArray *destinationFolder) {
        [controller dismissViewControllerAnimated:YES completion:^{
            if (destinationFolder.count == 0)
                return;
            
            __block void (^inputFileName)(NSString *path) = ^(NSString *path) {
                BOOL isDir = NO;
                NSFileManager *fileManager = [NSFileManager defaultManager];
                if (![fileManager fileExistsAtPath:path isDirectory:&isDir]) {
                    return;
                }
                
                
                if (!isDir) {
                    dispatch_async(dispatch_get_main_queue(), ^{
                        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kFileAlreadyExists") preferredStyle:UIAlertControllerStyleAlert];
                        UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleCancel handler:^(UIAlertAction *action) {
                                                                                 dispatch_async(dispatch_get_main_queue(), ^{
                                                                                     inputFileName([path stringByDeletingLastPathComponent]);
                                                                                 });
                                                                             }];
                        [alertController addAction:cancelAction];
                        [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
                    });
                    return;
                }
                
                UIAlertController *alertController = [UIAlertController alertControllerWithTitle:nil message:FSLocalizedForKey(@"kInputNewFileName") preferredStyle:UIAlertControllerStyleAlert];
                [alertController addTextFieldWithConfigurationHandler:^(UITextField *_Nonnull textField) {
                    textField.text = @"";
                }];
                UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel") style:UIAlertActionStyleCancel handler:nil];
                UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
                    
                    UITextField *textField = alertController.textFields[0];
                    NSString *newName = textField.text;
                    
                    NSString *newFilePath = [path stringByAppendingString:[NSString stringWithFormat:@"/%@.%@", newName,[self->_pdfViewCtrl.filePath pathExtension]]];
                    NSString *oldFilePath = self->_pdfViewCtrl.filePath;
                                             
                    if (newName.length < 1) {
                        UIAlertController *alertController = [UIAlertController alertControllerWithTitle:nil message:FSLocalizedForKey(@"kInputNewFileName") preferredStyle:UIAlertControllerStyleAlert];
                        UIAlertAction *okAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK") style:UIAlertActionStyleCancel handler:^(UIAlertAction *action) {
                            dispatch_async(dispatch_get_main_queue(), ^{
                                inputFileName(path);
                            });
                        }];
                        
                        [alertController addAction:okAction];
                        [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
                        
                    }
                    else if ([fileManager fileExistsAtPath:newFilePath]) {
                        if ([newFilePath isEqualToString:oldFilePath]) {
                            [self saveAndCloseCurrentDoc:newFilePath completion:^(BOOL success) {
                                if (self.goBack)
                                    self.goBack();
                                if (successed != nil) {
                                    successed();
                                }
                            }];
                            
                            inputFileName = nil;
                        }else{
                            dispatch_async(dispatch_get_main_queue(), ^{
                                UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kFileAlreadyExists") preferredStyle:UIAlertControllerStyleAlert];
                                UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel")
                                                                                       style:UIAlertActionStyleCancel
                                                                                     handler:^(UIAlertAction *action) {
                                                                                         dispatch_async(dispatch_get_main_queue(), ^{
                                                                                             inputFileName(path);
                                                                                         });
                                                                                     }];
                                [alertController addAction:cancelAction];
                                
                                UIAlertAction *action = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kReplace")
                                                                                 style:UIAlertActionStyleDefault
                                                                               handler:^(UIAlertAction *action) {
                                                                                   [fileManager removeItemAtPath:newFilePath error:nil];
                                                                                   [self saveAndCloseCurrentDoc:newFilePath completion:^(BOOL success) {
                                                                                       if (self.goBack)
                                                                                           self.goBack();
                                                                                       if (successed != nil) {
                                                                                           successed();
                                                                                       }
                                                                                   }];
                                                                                   
                                                                                   inputFileName = nil;
                                                                               }];
                                [alertController addAction:action];
                                [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
                            });
                        }
                    }
                    else {
                        [self saveAndCloseCurrentDoc:newFilePath completion:^(BOOL success) {
                            if (self.goBack)
                                self.goBack();
                            if (successed != nil) {
                                successed();
                            }
                        }];
                        
                        inputFileName = nil;
                    }
                }];
                [alertController addAction:cancelAction];
                [alertController addAction:action];
                [self.pdfViewCtrl.fs_viewController presentViewController:alertController animated:YES completion:nil];
            };
            
            inputFileName(destinationFolder[0]);
        }];
        
    };
    selectDestination.cancelHandler = ^(FSFileSelectDestinationViewController *controller) {
        [controller dismissViewControllerAnimated:YES completion:nil];
        if (error) {
            error();
        }
    };
    FSNavigationController *selectDestinationNavController = [[FSNavigationController alloc] initWithRootViewController:selectDestination];
    [selectDestinationNavController fs_setCustomTransitionAnimator:self.presentedVCAnimator];
    UIViewController *rootViewController = self.pdfViewCtrl.fs_viewController;
    [rootViewController presentViewController:selectDestinationNavController animated:YES completion:nil];
}

- (void)setMoreItemClikedCallback:(void (^)(UIView* moreItem))clickedCallback
{
    _moreItemClickedCallback = clickedCallback;
}

- (void)buttonDiscardChangeClick {
    self.isFileEdited = NO;
    [self.pdfViewCtrl closeDoc:nil];
    if (self.goBack)
        self.goBack();
}

- (void)setHiddenBottomToolbar:(BOOL)isHiddenBottomToolbar {
    [self.fullScreenToolbarMgr setHiddenBottomToolbar:!isHiddenBottomToolbar];
    if (!self.bottomToolbar) {
        _hiddenBottomToolbar = YES;
        return;
    }
    if (_hiddenBottomToolbar == isHiddenBottomToolbar) {
        return;
    }
    _hiddenBottomToolbar = isHiddenBottomToolbar;
    [self.bottomToolbar setHidden:isHiddenBottomToolbar animated:YES];
    if ([self.delegate respondsToSelector:@selector(uiextensionsManager:onToolBar:hidden:)]) {
        [self.delegate uiextensionsManager:self onToolBar:FSBottomBar hidden:isHiddenBottomToolbar];
    }
}

- (void)setHiddenTopToolbar:(BOOL)hiddenTopToolbar {
    [self.fullScreenToolbarMgr setHiddenTopToolbar:!hiddenTopToolbar];
    if (!self.topToolbar) {
        _hiddenTopToolbar = YES;
        return;
    }
    if (_hiddenTopToolbar == hiddenTopToolbar) {
        return;
    }
    _hiddenTopToolbar = hiddenTopToolbar;
    [self.topToolbar setHidden:hiddenTopToolbar animated:YES];
    if ([self.delegate respondsToSelector:@selector(uiextensionsManager:onToolBar:hidden:)]) {
        [self.delegate uiextensionsManager:self onToolBar:FSTopBar hidden:hiddenTopToolbar];
    }
}

- (void)setStatusBarHidden:(BOOL)hidden{
    _prefersStatusBarHidden = hidden;
}

- (BOOL)prefersStatusBarHidden{
    BOOL isLandscape = UIInterfaceOrientationIsLandscape([UIApplication sharedApplication].statusBarOrientation);
    if (DEVICE_iPHONE && isLandscape) _prefersStatusBarHidden = YES;
    if (!self.pdfViewCtrl.currentDoc) _prefersStatusBarHidden = NO;
    return _prefersStatusBarHidden;
}

- (void)setHiddenSettingBar:(BOOL)hiddenSettingBar {
    if ([self canAutoHideTopBottomToolBar]) {
        if (hiddenSettingBar) {
            [self delayHideTopBottomToolBar];
        }else{
            [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(autoFullScreen) object:nil];
        }
    }
    [self setHiddenSettingBar:hiddenSettingBar animated:YES];
}

- (void)setHiddenSettingBar:(BOOL)hiddenSettingBar animated:(BOOL)animated {
    [self.settingBar.viewSettingManager showSettingView:!hiddenSettingBar isReOpen:NO];
}

- (UIViewController *)settingbarPopoverCtr {
    if (!_settingbarPopoverCtr) {
        _settingbarPopoverCtr = [[UIViewController alloc] init];
    }
    return _settingbarPopoverCtr;
}

- (void)hideSettingBar {
    self.hiddenSettingBar = YES;
}

- (BOOL)hiddenPanel {
    return self.panelController.isHidden;
}

- (void)setHiddenPanel:(BOOL)hiddenPanel {
    self.panelController.isHidden = hiddenPanel;
}

- (BOOL)canAutoHideTopBottomToolBar{
    if (!self.panelController.isHidden ) {
        [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(autoFullScreen) object:nil];
        return NO;
    }
    
    if (self.thumbnailModule && !self.thumbnailModule.isThumbnailViewHidden) {
        [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(autoFullScreen) object:nil];
        return NO;
    }
    
    if (!self.enableTopToolbar && self.enableBottomToolbar) {
        return self.isFullScreen && !self.hiddenBottomToolbar;
    }
    else if (self.enableTopToolbar && !self.enableBottomToolbar) {
        return self.isFullScreen && !self.hiddenTopToolbar;
    }
    else if (!self.enableTopToolbar && !self.enableBottomToolbar) {
        return (self.isFullScreen && !self.hiddenTopToolbar) || (self.isFullScreen && !self.hiddenBottomToolbar);
    }
    return self.isFullScreen;
}

- (CGFloat)topHeight{
    float topHeight = 0;
    if ([self.topToolbar contentView] && ![self.topToolbar contentView].hidden) {
        topHeight += [self.topToolbar contentView].fs_height;
    }
    if (!self.smallTopToolbar.hidden) {
        topHeight += [self.smallTopToolbar getDisplayHeight];
    }
    return topHeight;
}

- (void)dissmiss:(id)sender {
    UIControl *control = (UIControl *) sender;
    if (control.tag == 200) {

    } else if (control.tag == 201) {
        self.more.hidden = YES;
    }
}

#pragma -mark rewrite configuration
- (void)setEnableHighlightForm:(BOOL)enableHighlightForm{
    if (_enableHighlightForm == enableHighlightForm) return;
    _enableHighlightForm = enableHighlightForm;
    FormAnnotHandler *handler = (FormAnnotHandler *) [self getAnnotHandlerByType:FSAnnotWidget];
    if (!handler) return;
    [handler setEnableHighlightForm:enableHighlightForm];
}

- (void)setHighlightFormColor:(UIColor *)highlightFormColor{
    if (CGColorEqualToColor(_highlightFormColor.CGColor, highlightFormColor.CGColor)) return;
    _highlightFormColor = highlightFormColor;
    FormAnnotHandler *handler = (FormAnnotHandler *) [self getAnnotHandlerByType:FSAnnotWidget];
    if (!handler) return;
    [handler setHighlightFormColor:highlightFormColor];
}

- (void)setActionHandler:(id<FSActionCallback>)actionHandler{
    if (_actionHandler && [FSCallbackSubstitute actionHandlerSubstitute].currentTarget == _actionHandler){
        [FSCallbackSubstitute actionHandlerSubstitute].currentTarget = actionHandler;
    }
    _actionHandler = actionHandler;
}

#pragma mark <ILayoutEventListener>

- (void)onLayoutModeChanged:(PDF_LAYOUT_MODE)oldLayoutMode newLayoutMode:(PDF_LAYOUT_MODE)newLayoutMode {
    if(oldLayoutMode == PDF_LAYOUT_MODE_UNKNOWN) return;
    [self changeState:FSUIManagerStateNormal];
}

#pragma mark - handle fullScreen event

- (void)setIsFullScreen:(BOOL)isFullScreen {
    if (_isFullScreen == isFullScreen) {
        return;
    }
    _isFullScreen = isFullScreen;
    [self setFullScreen:isFullScreen];
}

- (void)setFullScreen:(BOOL)fullScreen{
    if (!self.isFullScreen) {
        _currentIsFullScreen = NO;
        return ;
    }
    _currentIsFullScreen = fullScreen;
    BOOL topbarSelected = [self.topToolbar getCurrentSelectedItem].item.selected && self.fullScreenToolbarMgr.floatyToolbar.hidden;
    BOOL canFullScreenState = (self.currentState == FSUIManagerStateNormal  || self.currentState == FSUIManagerStateFillSign || self.currentState == FSUIManagerStateCreateForm);
    if (canFullScreenState && !topbarSelected && self.panelController.isHidden) {
        if (fullScreen) {
            self.hiddenTopToolbar = YES;
            self.hiddenBottomToolbar = YES;
        } else {
            self.hiddenTopToolbar = NO;
            self.hiddenBottomToolbar = NO;
            if ([self canAutoHideTopBottomToolBar]) {
                [self delayHideTopBottomToolBar];
            }
        }
    }

    for (id<IFullScreenListener> listener in self.fullScreenListeners) {
        if ([listener respondsToSelector:@selector(onFullScreen:)]) {
            [listener onFullScreen:fullScreen];
        }
    }
    
    if (!self.shouldShowPropertyBar && self.menuControl.isFirstResponder) {
        if (self.shouldShowMenu) {
            [self showMenu];
        }
    }
}

- (void)suspendAutoFullScreen {
    _isSuspendAutoFullScreen = YES;
    [self cancelAutoFullScreen];
}

- (void)resumeAutoFullScreen {
    _isSuspendAutoFullScreen = NO;
    [self delayAutoFullScreen];
}

- (void)cancelAutoFullScreen{
    if([self canAutoHideTopBottomToolBar]) {
        [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(autoFullScreen) object:nil];
    }
}

- (void)delayAutoFullScreen{
    if (_isSuspendAutoFullScreen) return;
    if([self canAutoHideTopBottomToolBar]) {
        [self performSelector:@selector(autoFullScreen) withObject:nil afterDelay:5.f];
    }
}

-(void)delayHideTopBottomToolBar {
    if (_isSuspendAutoFullScreen)  return;
    [self cancelAutoFullScreen];
    [self delayAutoFullScreen];
}

- (void)autoFullScreen{
    if(self.menuControl.isMenuVisible && [self.pdfViewCtrl getPageLayoutMode] == PDF_LAYOUT_MODE_REFLOW) {
        return;
    }
    [self setFullScreen:YES];
}

- (void)registerFullScreenListener:(id<IFullScreenListener>)listener {
    if (self.fullScreenListeners) {
        [self.fullScreenListeners addObject:listener];
    }
}

- (void)unregisterFullScreenListener:(id<IFullScreenListener>)listener {
    if ([self.fullScreenListeners containsObject:listener]) {
        [self.fullScreenListeners removeObject:listener];
    }
}

#pragma mark - IPageNumberListener
- (void)registerPageNumberListener:(id<IPageNumberListener>)listener {
    if (self.pageNumberListeners) {
        [self.pageNumberListeners addObject:listener];
    }
}

- (void)unregisterPageNumerListener:(id<IPageNumberListener>)listener {
    if ([self.pageNumberListeners containsObject:listener]) {
        [self.pageNumberListeners removeObject:listener];
    }
}

#pragma mark - IRotationEventListener

- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration {
    self.isRotating = YES;
    if (self.shouldShowMenu) {
        [self hideMenu];
    }

    for (id<IRotationEventListener> listener in self.rotateListeners) {
        if ([listener respondsToSelector:@selector(willRotateToInterfaceOrientation:duration:)]) {
            [listener willRotateToInterfaceOrientation:toInterfaceOrientation duration:duration];
        }
    }

    [self.pdfViewCtrl willRotateToInterfaceOrientation:toInterfaceOrientation duration:duration];
}

- (void)willAnimateRotationToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration {
    
    for (id<IRotationEventListener> listener in self.rotateListeners) {
        if ([listener respondsToSelector:@selector(willAnimateRotationToInterfaceOrientation:duration:)]) {
            [listener willAnimateRotationToInterfaceOrientation:toInterfaceOrientation duration:duration];
        }
    }
    
    [self.pdfViewCtrl willAnimateRotationToInterfaceOrientation:toInterfaceOrientation duration:duration];
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation {
    self.isRotating = NO;
    if (!self.shouldShowPropertyBar) {
        if (self.shouldShowMenu) {
            [self showMenu];
        }
    }

    for (id<IRotationEventListener> listener in self.rotateListeners) {
        if ([listener respondsToSelector:@selector(didRotateFromInterfaceOrientation:)]) {
            [listener didRotateFromInterfaceOrientation:fromInterfaceOrientation];
        }
    }

    [self.pdfViewCtrl didRotateFromInterfaceOrientation:fromInterfaceOrientation];

}

- (void)registerRotateChangedListener:(id<IRotationEventListener>)listener {
    if (self.rotateListeners && ![self.rotateListeners containsObject:listener]) {
        [self.rotateListeners addObject:listener];
    }
}

- (void)unregisterRotateChangedListener:(id<IRotationEventListener>)listener {
    if ([self.rotateListeners containsObject:listener]) {
        [self.rotateListeners removeObject:listener];
    }
}

- (void)registerStateChangeListener:(id<IStateChangeListener>)listener {
    if (self.stateChangeListeners) {
        [self.stateChangeListeners addObject:listener];
    }
}

- (void)unregisterStateChangeListener:(id<IStateChangeListener>)listener {
    if ([self.stateChangeListeners containsObject:listener]) {
        [self.stateChangeListeners removeObject:listener];
    }
}

- (void)changeState:(FSUIManagerState)state {
    FSUIManagerState oldState = self.currentState;
    NSArray *supportStates = @[@(FSUIManagerStateNormal), @(FSUIManagerStateReflow), @(FSUIManagerStateSpeech), @(FSUIManagerStateAnnotTool), @(FSUIManagerStatePanZoom),@(FSUIManagerStateCreateForm), @(FSUIManagerStateSpeech), @(FSUIManagerStateFillSign),@(FSUIManagerStateRedaction),@(FSUIManagerStateSearch),@(FSUIManagerStateAutoFlip),@(FSUIManagerStateComparision)];
    if (state != FSUIManagerStateNormal && state != FSUIManagerStateFillSign && state != FSUIManagerStateAutoFlip) {
        if ([_pdfViewCtrl isDynamicXFA]) {
            return;
        }
    }
    if (![supportStates containsObject:@(state)]) {
        self.currentState = 0;
        return ;
    }
    
    if (state == FSUIManagerStateSearch && !self.config.loadSearch) {
        return;
    }
    
    if (state == FSUIManagerStateComparision && !self.isComparisonViewCtrlVisible) {
        return;
    }

    if (state == FSUIManagerStateFillSign) {
        FillSignModule *module = (FillSignModule *)[self getModuleByName:Module_Fill_Sign];
        if (!module) {
            return;
        }
        if (self.currentAnnot) {
            [self setCurrentAnnot:nil];
        }
    }
    
    if (state == FSUIManagerStateNormal) {
        [self setCurrentAnnot:nil];
        [self setCurrentWidget:nil];
    }
    if (state == FSUIManagerStateAnnotTool && oldState != FSUIManagerStateAnnotTool) {
        self.lastManagerStateWhenAnnotTool = self.currentState;
    }
    
    self.currentState = state;
    
    if (oldState == FSUIManagerStateRedaction && oldState != state) {
        [self enterRedactionMode:NO];
    }
    if (oldState == FSUIManagerStateSearch && oldState != state) {
        [self showSearchBar:NO];
    }

    [self delayAutoFullScreen];
    if (oldState != state) {
        self.hiddenSettingBar = YES;
    }
    
    switch (state) {
        case FSUIManagerStateNormal:
            if (oldState != FSUIManagerStateAnnotTool && oldState != FSUIManagerStateNormal) {
                self.hiddenTopToolbar = NO;
                self.hiddenBottomToolbar = NO;
            }
            break;
        case FSUIManagerStateAnnotTool:
//            self.hiddenTopToolbar = NO;
//            self.hiddenBottomToolbar = NO;
            [self cancelAutoFullScreen];
            break;
        case FSUIManagerStateSearch:
            self.hiddenTopToolbar = YES;
            self.hiddenBottomToolbar = YES;
            if (self.currentAnnot) [self setCurrentAnnot:nil];
            [self showSearchBar:YES];
            break;
        case FSUIManagerStatePanZoom:
            break;
            
        case FSUIManagerStateCreateForm:
            break;
            
        case FSUIManagerStateSpeech:   
            [self.speechModule speakFromPageIndex:[self.pdfViewCtrl getCurrentPage]];
            break;
        case FSUIManagerStateAutoFlip:
            [self.autoFlipModule enterAutoFlipMode:YES];
            break;
        case FSUIManagerStateReflow:
            [self enterReflowMode:YES];
            break;
        case FSUIManagerStateFillSign:
            break;
        default:
            self.hiddenTopToolbar = YES;
            self.hiddenBottomToolbar = YES;
            break;
    }
    
    if (state == FSUIManagerStateReflow ) {
        [self enterReflowMode:YES];
    }
    
    if (state == FSUIManagerStateRedaction ) {
        [self enterRedactionMode:YES];
    }
    
    if (self.stateChangeListeners) {
        for (id<IStateChangeListener> listener in self.stateChangeListeners) {
            if ([listener respondsToSelector:@selector(onStateChanged:)]) {
                [listener onStateChanged:state];
            }
            if ([listener respondsToSelector:@selector(onStateChanged:lastState:)]) {
                [listener onStateChanged:state lastState:oldState];
            }
        }
    }
}

- (void)enableTopToolbar:(BOOL)isEnabled {
    _enableTopToolbar = isEnabled;
    if (isEnabled) {
        if (self.topToolbar)
            return;

        [self.toolbarManager installTopbar];
    } else {
        if (!self.topToolbar)
            return;
        [self.toolbarManager removeTopbar];
    }
}

- (void)enableBottomToolbar:(BOOL)isEnabled {
    _enableBottomToolbar = isEnabled;
    if (isEnabled) {
        if (self.bottomToolbar)
            return;

        [self.toolbarManager installBottombar];
    } else {
        if (!self.bottomToolbar)
            return;
        [self.toolbarManager removeBottombar];
    }
}

- (void)enableSmallTopToolbar:(BOOL)isEnabled{
    _enableSmallTopToolbar = isEnabled;
    if (isEnabled) {
        if (self.smallTopToolbar)
            return;

        [self.fullScreenToolbarMgr installTopbar];
    } else {
        if (!self.smallTopToolbar)
            return;
        [self.fullScreenToolbarMgr removeTopbar];
    }
}

- (void)enableSmallBottomToolbar:(BOOL)isEnabled{
    _enableSmallBottomToolbar = isEnabled;
    if (isEnabled) {
        if (self.smallBottomToolbar)
            return;

        [self.fullScreenToolbarMgr installBottombar];
    } else {
        if (!self.smallBottomToolbar)
            return;
        [self.fullScreenToolbarMgr removeBottombar];
    }
}

- (FSUIManagerState)getState {
    return self.currentState;
}

#pragma mark - IDocEventListener

- (void)onDocWillOpen {
}

- (void)onDocOpened:(FSPDFDoc *)document error:(int)error {

    [document enableUpdatePageLabel:YES];
    
    [self onDocOpenedSupportsMultiplehook];
    
    if ([self canAutoHideTopBottomToolBar]) {
        [self setFullScreen:NO];
        [self delayHideTopBottomToolBar];
    }
    
    self.backButton.enabled = YES;
    
    BOOL enabled = [FSPermissionUtilty canAssemble:self];
    [self.settingBar.viewSettingManager setItemEnabledWithType:CROPPAGE enabled:enabled];
    [self.toolbarManager getMainItemWithType:FSMainToolbarItemTypeReadingBookmark].item.enabled = enabled;

    enabled = ![_pdfViewCtrl isDynamicXFA];
    [self.settingBar.viewSettingManager setItemEnabledWithType:REFLOW enabled:enabled];
    [self.settingBar.viewSettingManager setItemEnabledWithType:PANZOOM enabled:enabled];
    [self.settingBar.viewSettingManager setItemEnabledWithType:CROPPAGE enabled:enabled];
    
    [self.toolbarManager getMainItemWithType:FSMainToolbarItemTypeSearch].item.enabled = enabled;
}

- (void)onDocWillClose:(FSPDFDoc *)document {
    [self onDocWillCloseSupportsMultiplehook];
    
    id<IToolHandler> toolHandler = [self getToolHandlerByName:Tool_Freetext];
    if (toolHandler && [toolHandler isKindOfClass:[FtToolHandler class]]) {
        FtToolHandler *ftToolhandler = (FtToolHandler *) toolHandler;
        [ftToolhandler exitWithoutSave];
    }
    
    NSString *filePath = self.pdfViewCtrl.filePath;
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        [self.thumbnailModule clearThumbnailCachesForPDFAtPath:filePath];
    });
    
    [self changeState:FSUIManagerStateNormal];
    
    [[AudioPlayerControl sharedControl] dismiss];
}

- (void)onDocClosed:(FSPDFDoc *)document error:(int)error {
    
    [self onDocClosedSupportsMultiplehook];
    
    _currentAnnot = nil;
    _currentToolHandler = nil;
    [self clearUndoRedo];
    self.menuControl.menuItems = nil;
    
    if(_isDocModified) _isDocModified = NO;
}

- (void)onDocWillSave:(FSPDFDoc *)document {
    
}

#pragma mark - IToolEventListener

- (void)onToolChanged:(NSString *)lastToolName CurrentToolName:(NSString *)toolName {
    if (toolName == nil) {
        //Dismiss annotation type.
        FSPDFViewCtrl *pdfViewCtrl = self.pdfViewCtrl;
        for (UIView *view in pdfViewCtrl.subviews) {
            if (view.tag == 2113) {
                [view removeFromSuperview];
            }
        }
    }
    if (toolName && ![toolName isEqualToString:Tool_Select] && ![toolName isEqualToString:Tool_Signature]) {
        if ([self getToolHandlerByName:toolName] != nil) {
            [self changeState:FSUIManagerStateAnnotTool];
        }
    }
}

- (void)onCurrentAnnotChanged:(FSAnnot *)lastAnnot currentAnnot:(FSAnnot *)currentAnnot {
}

#pragma mark <ISearchEventListener>

- (void)onSearchStarted {
    for (id<ISearchEventListener> listener in self.searchListeners) {
        if ([listener respondsToSelector:@selector(onSearchStarted)]) {
            [listener onSearchStarted];
        }
    }
}

- (void)onSearchCanceled {
    if (self.currentState == FSUIManagerStateSearch) {
        [self changeState:FSUIManagerStateNormal];
    }
    for (id<ISearchEventListener> listener in self.searchListeners) {
        if ([listener respondsToSelector:@selector(onSearchCanceled)]) {
            [listener onSearchCanceled];
        }
    }
}

#pragma mark - IPageEventListener
- (void)onPageVisible:(int)index {
}

- (void)onPageInvisible:(int)index {
}

#pragma mark - IWindowFrameChangedListener
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
    if ([keyPath isEqualToString:@"frame"]) {
        UIUserInterfaceSizeClass curHorizontalSizeClass = [self fs_currentSceneSizeClass];
        CGRect oldFrame = [[change objectForKey:@"old"] CGRectValue];
        CGRect newFrame = [[change objectForKey:@"new"] CGRectValue];
        if (!DEVICE_iPHONE && !(oldFrame.size.width == newFrame.size.height && oldFrame.size.height == newFrame.size.width)) {
            [self onWindowFrameChanged:newFrame horizontalSizeClassChanged:curHorizontalSizeClass != self.lastWindowHorizontalSizeClass];
        }
        self.lastWindowHorizontalSizeClass = curHorizontalSizeClass;
    }
}

- (void)registerWindowFrameChangedListener:(id<IWindowFrameChangedListener>)listener
{
    if (self.windowFrameListeners && ![self.windowFrameListeners containsObject:listener]) {
        [self.windowFrameListeners addObject:listener];
    }
}

- (void)unregisterWindowFrameChangedListener:(id<IWindowFrameChangedListener>)listener
{
    if ([self.windowFrameListeners containsObject:listener]) {
        [self.windowFrameListeners removeObject:listener];
    }
}

- (void)onWindowFrameChanged:(CGRect)newFrame horizontalSizeClassChanged:(BOOL)changed
{
    FSMessageDebouncePerformDefault(^{
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.3 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
            if (!self.shouldShowPropertyBar) {
                if (self.shouldShowMenu) {
                    [self showMenu];
                }
            }
        });
    });

    if (self.windowFrameListeners.count > 0) {
        for (id<IWindowFrameChangedListener> listener in self.windowFrameListeners) {
            if ([listener respondsToSelector:@selector(onWindowFrameChanged:horizontalSizeClassChanged:)]) {
                [listener onWindowFrameChanged:newFrame horizontalSizeClassChanged:changed];
            }
        }
    }
    
    if (DEVICE_iPAD) {
        if ([_pdfViewCtrl fs_currentSceneiPadHorizontalCompact]) {
            if (self.stampIconController && self.stampIconController.modalPresentationStyle == UIModalPresentationPopover){
                [self.stampIconController dismissViewControllerAnimated:YES completion:^{
                    self.stampIconController = nil;
                    [self showPropertyWithToolSettings:self.lastPropertyToolSettings devicePos:self.devicePos];
                }];
            }
        }else{
            if (self.stampIconController && self.stampIconController.modalPresentationStyle == UIModalPresentationFullScreen){
                [self.stampIconController dismissViewControllerAnimated:YES completion:^{
                    self.stampIconController = nil;
                    [self showPropertyWithToolSettings:self.lastPropertyToolSettings devicePos:self.devicePos];
                }];
            }
        }
    }
}

#pragma mark - IPropertyViewEventListener
- (void)registerPropertyViewEvnetListener:(id<IPropertyViewEventListener>)listener
{
    if (self.propertyViewListeners) {
        [self.propertyViewListeners addObject:listener];
    }
}

- (void)unregisterPropertyViewEvnetListener:(id<IPropertyViewEventListener>)listener
{
    if ([self.propertyViewListeners containsObject:listener]) {
        [self.propertyViewListeners removeObject:listener];
    }
}

#pragma mark - top and bottom toolbar events

- (void)onClickBackButton:(UIControl *)control {
    if (self.currentAnnot) {
        [self setCurrentAnnot:nil];
    }
    
    if (self.currentWidget) {
        [self setCurrentWidget:nil];
    }
    
    if (self.currentToolHandler) {
        [self setCurrentToolHandler:nil];
    }
    
    [self hideMenu];
    
    if ([self.delegate respondsToSelector:@selector(quitUIExtensionsManager:control:)]) {
        [self.delegate quitUIExtensionsManager:self control:control];
        return;
    }
    
    BOOL isAsync = (self.pdfViewCtrl.filePath != nil) && ![[NSFileManager defaultManager] fileExistsAtPath:self.pdfViewCtrl.filePath];
    if (!isAsync && /* ![self.pdfViewCtrl.currentDoc isModified] &&  todo maybe by DocEventCallback */ (!_isDocModified || self.someDocFromOtherScenes.count)) {
        [self.pdfViewCtrl closeDoc:nil];
        if (self.goBack)
            self.goBack();
        return;
    }

//    BOOL canSave = [self.pdfViewCtrl isDynamicXFA] ? !self.isViewSignedDocument && [self.pdfViewCtrl isDynamicXFA] : !self.isViewSignedDocument && [Utility canModifyContents:self.pdfViewCtrl];

    if (self.isAutoSaveDoc /*&& canSave*/) {
        [self buttonSaveClick];
        return;
    }
    
    UIAlertController * alertController = [UIAlertController alertControllerWithTitle:nil message:nil preferredStyle:UIAlertControllerStyleActionSheet];
    UIAlertAction* saveAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kSave") style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        [self buttonSaveClick];
    }];
    UIAlertAction* saveAsAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kSaveAs") style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        [self buttonSaveAsClick];
    }];
    UIAlertAction* discardAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(isAsync ? @"kDonotSave" : @"kDiscardChange") style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        [self buttonDiscardChangeClick];
    }];
    UIAlertAction* cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel") style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {
        [alertController dismissViewControllerAnimated:YES completion:nil];
    }];
    
    [alertController addAction:saveAction];
    [alertController addAction:saveAsAction];
    [alertController addAction:discardAction];
    if (DEVICE_iPHONE) {
        [alertController addAction:cancelAction];
    }
    
    [alertController setModalPresentationStyle:UIModalPresentationPopover];
    UIPopoverPresentationController *popPresenter = [alertController popoverPresentationController];
    popPresenter.sourceView = control;
    popPresenter.sourceRect = control.bounds;
    popPresenter.delegate = self;
    
    [[self.pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:^{
        [self cancelAutoFullScreen];
        self.isSaveAsAlertControllerShow = YES;
    }];
}

#pragma mark <FSUndo>
- (void)addUndoItem:(UndoItem *)undoItem {
    if (!undoItem)
        return;
    [self.undoItems addObject:undoItem];
    [self.redoItems removeAllObjects];
    [self undoRedoStateChanged];
}

- (void)removeUndoItem:(UndoItem *)undoItem {
    if (!undoItem)
        return;
    void (^removeUndoItem) (NSMutableArray *items) = ^(NSMutableArray *items){
        if ([items containsObject:undoItem]) {
            [items removeObject:undoItem];
        }else{
            [items enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
                if ([obj isKindOfClass:[NSMutableArray class]]) {
                    NSMutableArray *undoItems = obj;
                    if (!undoItems.count) {
                        [items removeObject:undoItems];
                         *stop = YES;
                    }
                    if ([undoItems containsObject:undoItem]) {
                        [undoItems removeObject:undoItem];
                        if (!undoItems.count) [items removeObject:undoItems];
                        *stop = YES;
                    }
                }
            }];
        }
    };
    removeUndoItem(self.undoItems);
    removeUndoItem(self.redoItems);
    [self undoRedoStateChanged];
}

- (void)addUndoItems:(NSMutableArray *)undoItems {
    if (!undoItems || undoItems.count == 0)
        return;
    [self.undoItems addObject:undoItems];
    [self.redoItems removeAllObjects];
    [self undoRedoStateChanged];
}

-(void)removeUndoItems {
    NSMutableArray *tempUndoArray = [NSMutableArray arrayWithArray: self.undoItems];
    for (int i = 0 ; i < tempUndoArray.count; i++) {
        id undoObj = [tempUndoArray objectAtIndex:i];
        if ([undoObj isKindOfClass:[NSMutableArray class]]) {
            [self.undoItems removeObject:undoObj];
        }
    }
    NSMutableArray *tempredoArray = [NSMutableArray arrayWithArray: self.redoItems];
    for (int i = 0 ; i < tempredoArray.count; i++) {
        id undoObj = [tempredoArray objectAtIndex:i];
        if ([undoObj isKindOfClass:[NSMutableArray class]]) {
            [self.redoItems removeObject:undoObj];
        }
    }

    [self undoRedoStateChanged];
}

-(void)removeUndoItems:(NSArray<UndoItem *> *)items {
    [self.undoItems removeObjectsInArray:items];
    [self.redoItems removeObjectsInArray:items];
}

- (void)addUndoItems:(NSMutableArray *)undoItems redoItems:(NSMutableArray *)redoItems{
    if (undoItems && undoItems.count > 0)
    {
        [self.undoItems addObjectsFromArray:undoItems];
        [self.redoItems removeAllObjects];
    }
    
    if (redoItems && redoItems.count > 0){
        NSRange range = NSMakeRange(0, redoItems.count);
        NSIndexSet *indexSet = [NSIndexSet indexSetWithIndexesInRange:range];
        [self.redoItems insertObjects:redoItems atIndexes:indexSet];
    }
    
    [self undoRedoStateChanged];
}

- (BOOL)canUndo {
    return self.undoItems.count > 0 ||
    (self.currentUndoToolHandler && [self.currentUndoToolHandler canUndo]);
}

- (BOOL)canRedo {
    return self.redoItems.count > 0 ||
    (self.currentUndoToolHandler && [self.currentUndoToolHandler canRedo]);
}

- (void)undo {
    if (self.currentAnnot) {
        [self setCurrentAnnot:nil];
    }
    
    if (self.currentUndoToolHandler && [self.currentUndoToolHandler canUndo]) {
        [self.currentUndoToolHandler undo];
        return;
    }
    
    if (self.undoItems.count == 0)
        return;
    @synchronized(self) {
        for (id<IFSUndoEventListener> listener in self.undoListeners) {
            if ([listener respondsToSelector:@selector(onWillUndo)]) {
                [listener onWillUndo];
            }
            
            if ([listener respondsToSelector:@selector(onStopUndo)]) {
                if ([listener onStopUndo]){
                    [self undoRedoStateChanged];
                    return;
                }
            }
        }
        
        if (!self.undoItems.count) {
            [self undoRedoStateChanged];
            return;
        }
        id undoObj = [self.undoItems objectAtIndex:self.undoItems.count - 1];
        
        if ([undoObj isKindOfClass:[NSMutableArray class]]) {
            NSMutableArray *undoItems = (NSMutableArray *)undoObj;
            for (int i = 0 ; i < undoItems.count; i++) {
                UndoItem *item = [undoItems objectAtIndex:i];
                if (item.undo) {
                    item.undo(item);
                }
            }
            
            [self.redoItems addObject:undoObj];
            [self.undoItems removeObject:undoObj];
        }else{
            
            UndoItem *item = (UndoItem *)undoObj;
            NSString *firstGroupid = item.groupID;
            NSString *groupID;
            
            do {
                if (item.undo) {
                    item.undo(item);
                }
                [self.redoItems addObject:item];
                [self.undoItems removeObject:item];
                
                if (self.undoItems.count == 0) break;
                item = [self.undoItems objectAtIndex:self.undoItems.count - 1];
                if ([item isKindOfClass:[UndoItem class]]) groupID = item.groupID;
                else groupID = nil;
            } while (firstGroupid != nil && [groupID isEqualToString:firstGroupid]);

        }
        
        [self undoRedoStateChanged];
        
        for (id<IFSUndoEventListener> listener in self.undoListeners) {
            if ([listener respondsToSelector:@selector(onDidUndo)]) {
                [listener onDidUndo];
            }
        }
    }
}

- (void)redo {
    if (self.currentAnnot) {
        [self setCurrentAnnot:nil];
    }

    if (self.redoItems.count == 0)
    {
        if (self.currentUndoToolHandler && [self.currentUndoToolHandler canRedo]) {
            [self.currentUndoToolHandler redo];
        }
        return;
    }
    @synchronized(self) {
        for (id<IFSUndoEventListener> listener in self.undoListeners) {
            if ([listener respondsToSelector:@selector(onWillRedo)]) {
                [listener onWillRedo];
            }
            if ([listener respondsToSelector:@selector(onStopRedo)]) {
                if ([listener onStopRedo]){
                    [self undoRedoStateChanged];
                    return;
                }
            }
        }
        
        if (!self.redoItems.count) {
            [self undoRedoStateChanged];
            return;
        }

        id undoObj = [self.redoItems objectAtIndex:self.redoItems.count - 1];
        
        if ([undoObj isKindOfClass:[NSMutableArray class]]) {
            NSMutableArray *undoItems = (NSMutableArray *)undoObj;
            for (int i = 0 ; i < undoItems.count; i++) {
                UndoItem *item = [undoItems objectAtIndex:i];
                if (item.redo) {
                    item.redo(item);
                }
            }
            
            [self.undoItems addObject:undoObj];
            [self.redoItems removeObject:undoObj];
        }else{
            UndoItem *item = (UndoItem *)undoObj;
            NSString *firstGroupid = item.groupID;
            NSString *groupID;
            
            do {
                if (item.redo) {
                    item.redo(item);
                }
                [self.undoItems addObject:item];
                [self.redoItems removeObject:item];
                
                if (self.redoItems.count == 0) break;
                item = [self.redoItems objectAtIndex:self.redoItems.count - 1];
                if ([item isKindOfClass:[UndoItem class]]) groupID = item.groupID;
                else groupID = nil;
            } while (firstGroupid != nil && [groupID isEqualToString:firstGroupid]);
        }

        [self undoRedoStateChanged];
        
        for (id<IFSUndoEventListener> listener in self.undoListeners) {
            if ([listener respondsToSelector:@selector(onDidRedo)]) {
                [listener onDidRedo];
            }
        }
    }
}

- (void)clearUndoRedo {
    [self.undoItems removeAllObjects];
    [self.redoItems removeAllObjects];
    [self undoRedoStateChanged];
}

- (void)undoRedoStateChanged{
    [self supportsMultipleScenesOp:^(UIExtensionsManager * _Nonnull self) {
        for (id<IFSUndoEventListener> listener in self.undoListeners) {
            if ([listener respondsToSelector:@selector(onUndoChanged)]) {
                [listener onUndoChanged];
            }
        }
    }];
}

#pragma mark print methods

+ (void)printDoc:(FSPDFDoc *)doc animated:(BOOL)animated inWindow:(UIWindow *)window jobName:(nullable NSString *)jobName delegate:(nullable id<UIPrintInteractionControllerDelegate>)delegate completionHandler:(nullable UIPrintInteractionCompletionHandler)completion {
    [Utility printDoc:doc animated:animated inWindow:window jobName:jobName delegate:delegate completionHandler:completion];
}

+ (void)printDoc:(FSPDFDoc *)doc fromRect:(CGRect)rect inView:(UIView *)view animated:(BOOL)animated inWindow:(UIWindow *)window jobName:(nullable NSString *)jobName delegate:(nullable id<UIPrintInteractionControllerDelegate>)delegate completionHandler:(nullable UIPrintInteractionCompletionHandler)completion {
    [Utility printDoc:doc fromRect:rect inView:view animated:animated inWindow:window jobName:jobName delegate:delegate completionHandler:completion];
}

- (NSMutableArray *)didForMenuControlWillShow:(FSMenuControlScene)menuControlScene array:(NSMutableArray *)array
{
    if ([self.menuControlDelegate respondsToSelector:@selector(menuControlWillShow:menuControlScene:)])
    {
        FSMenuControl* menuControl = [[FSMenuControl alloc] init];
        menuControl.pageIndex = self.currentPageIndex;
        menuControl.displayPoint = self.currentPoint;
        NSMutableArray* itemArray = [[NSMutableArray alloc] init];
        for (MenuItem* menuItem in array) {
            FSMenuItem* fsMenuItem = [[FSMenuItem alloc] initWithMenuItem:menuItem];
            [itemArray addObject:fsMenuItem];
        }
        menuControl.menuItems = [itemArray copy];
        [self.menuControlDelegate menuControlWillShow:menuControl menuControlScene:menuControlScene];
        NSMutableArray* menuItemArray = [[NSMutableArray alloc] init];
        for (FSMenuItem* menuItem in menuControl.menuItems) {
            MenuItem* item = [[MenuItem alloc] initWithMenuItem:menuItem];
            [menuItemArray addObject:item];
        }
        return [menuItemArray mutableCopy];
    }
    return array;
}

#pragma mark - Main top and bottom bar
-(NSMutableDictionary *)getToolbarItemHiddenStatus {

    NSMutableDictionary *toolbarButtonInfo = [@{
                                               @"Back":@"YES",
                                               @"More":@"YES",
                                               @"Search":@"YES",
                                               @"Panel":@"YES",
                                               @"ViewSettings":@"YES",
                                               @"Thumbnail":@"YES",
                                               @"ReadingBookmark":@"YES"
                                               } mutableCopy];
    
    NSMutableDictionary *toolbarButtonTagDic = [@{
        @"Back":[NSNumber numberWithInt:FSMainToolbarItemTypeBack],
        @"More":[NSNumber numberWithInt:FSMainToolbarItemTypeMore],
        @"Search":[NSNumber numberWithInt:FSMainToolbarItemTypeSearch],
        @"Panel":[NSNumber numberWithInt:FSMainToolbarItemTypePanel],
        @"ViewSettings":[NSNumber numberWithInt:FSMainToolbarItemTypeViewSettings],
        @"Thumbnail":[NSNumber numberWithInt:FSMainToolbarItemTypeThumbnail],
        @"ReadingBookmark":[NSNumber numberWithInt:FSMainToolbarItemTypeReadingBookmark]
                                              } mutableCopy];
    
    for (NSString *key in toolbarButtonTagDic.allKeys) {
        int tag = [[toolbarButtonTagDic objectForKey:key] intValue];
        FSMainToolbarItem *toolbarItem = [self.topToolbar getToolbarItemForItemType:tag];
        if (!toolbarItem) {
            toolbarItem = [self.bottomToolbar getToolbarItemForItemType:tag];
        }
        if (toolbarItem) {
            [toolbarButtonInfo setObject:@"NO" forKey:key];
        }
    }
    
    NSMutableDictionary *toolbarSegmentInfo = [@{
        @"Home":@"YES",
        @"Edit":@"YES",
        @"Comment":@"YES",
        @"Drawing":@"YES",
        @"ViewPad":@"YES",
        @"Form":@"YES",
        @"Sign":@"YES",
        @"Protect":@"YES",
    } mutableCopy];
    NSMutableDictionary *toolbarSegmentTagDic = [@{
        @"Home":[NSNumber numberWithInt:FSMainTopbarToolTagHome],
        @"Edit":[NSNumber numberWithInt:FSMainTopbarToolTagEdit],
        @"Comment":[NSNumber numberWithInt:FSMainTopbarToolTagComment],
        @"Drawing":[NSNumber numberWithInt:FSMainTopbarToolTagDrawing],
        @"ViewPad":[NSNumber numberWithInt:FSMainTopbarToolTagView],
        @"Form":[NSNumber numberWithInt:FSMainTopbarToolTagForm],
        @"Sign":[NSNumber numberWithInt:FSMainTopbarToolTagSign],
        @"Protect":[NSNumber numberWithInt:FSMainTopbarToolTagProtect]
    } mutableCopy];

    for (NSString *key in toolbarSegmentTagDic.allKeys) {
        int tag = [[toolbarSegmentTagDic objectForKey:key] intValue];

        for (FSMainTopbarToolTagItem *tagItem in self.topToolbar.tagItems) {
            if (tagItem.toolTag == tag) {
                [toolbarSegmentInfo setObject:@"NO" forKey:key];
                break;
            }
        }
    }
    
    [toolbarButtonInfo addEntriesFromDictionary:toolbarSegmentInfo];
    return toolbarButtonInfo;
}

#pragma mark - set toolbar items hide/show
-(void)setToolbarItemHiddenWithTag:(FS_TOOLBAR_ITEM_TAG)itemTag hidden:(BOOL)isHidden{

    [self.toolbarManager setToolbarItemHiddenWithTag:itemTag hidden:isHidden];
}

@end

