/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSCloudSyncStatusView.h"


@interface FSCloudSyncStatusArcView : UIView

@end

@implementation FSCloudSyncStatusArcView

- (instancetype)init
{
    if (self = [super init]){
        self.backgroundColor = [UIColor clearColor];
        [self addAnimation];
    }
    return self;
}

- (CGSize)intrinsicContentSize
{
    return CGSizeMake(20, 20);
}

- (void)drawRect:(CGRect)rect
{
    UIBezierPath *backgroundPath = [[UIBezierPath alloc] init];
    backgroundPath.lineWidth = 2;
    [self.fs_primaryColor set];
    backgroundPath.lineCapStyle = kCGLineCapRound;
    backgroundPath.lineJoinStyle = kCGLineJoinRound;
    CGFloat radius = 8;
    [backgroundPath addArcWithCenter:(CGPoint){rect.size.width * 0.5, rect.size.height * 0.5} radius:radius startAngle:M_PI * 1.5 endAngle:M_PI * 2.0  clockwise:YES];
    [backgroundPath stroke];
}

- (void)addAnimation
{
    CABasicAnimation *rotationAnimation = [CABasicAnimation animationWithKeyPath:@"transform.rotation.z"];
    rotationAnimation.toValue = [NSNumber numberWithFloat: M_PI * 2.0 ];
    rotationAnimation.duration = 1.0;
    rotationAnimation.cumulative = YES;
    rotationAnimation.repeatCount = MAXFLOAT;
    rotationAnimation.removedOnCompletion = NO;
    [self.layer addAnimation:rotationAnimation forKey:@"rotationAnimation"];
}

@end


@interface FSCloudSyncStatusView ()
@property (nonatomic, strong) UIImageView *statusImageView;
@property (nonatomic, strong) FSCloudSyncStatusArcView *arcView;
@end

@implementation FSCloudSyncStatusView

- (instancetype)init
{
    if (self = [super init]){
        self.backgroundColor = [UIColor clearColor];
    }
    return self;
}

- (CGSize)intrinsicContentSize
{
    return CGSizeMake(20, 20);
}

- (void)setStatusType:(FSCloudSyncFileStatusType)statusType
{
    _statusType = statusType;
    if (self.arcView.superview) {
        [self.arcView removeFromSuperview];
    }
    [self addSubview:self.arcView];
    [self.arcView mas_remakeConstraints:^(MASConstraintMaker *make) {
        make.center.mas_equalTo(self);
    }];
    self.accessibilityIdentifier = @"cloud_sync_syning";
    [self setNeedsDisplay];
}

- (void)setProgress:(CGFloat)progress
{
    _progress = progress;
    [self setNeedsDisplay];
}

- (void)drawRect:(CGRect)rect
{
    if (self.statusType == FSCloudSyncFileStatusTypeSyncing){
        UIBezierPath *backgroundPath = [[UIBezierPath alloc] init];
        CGFloat radius = 8;
        [backgroundPath addArcWithCenter:(CGPoint){rect.size.width * 0.5, rect.size.height * 0.5} radius:radius startAngle:M_PI * 1.5 endAngle:M_PI * 1.5 + M_PI * 2  clockwise:YES];
        [self.fs_dividerColor_P2 setFill];
        backgroundPath.lineCapStyle = kCGLineCapRound;
        backgroundPath.lineJoinStyle = kCGLineJoinRound;
        [backgroundPath fill];
        UIBezierPath *progressPath = [[UIBezierPath alloc] init];
        progressPath.lineWidth = 2;
        [self.fs_primaryColor set];
        progressPath.lineCapStyle = kCGLineCapRound;
        progressPath.lineJoinStyle = kCGLineJoinRound;
        [progressPath addArcWithCenter:(CGPoint){rect.size.width * 0.5, rect.size.height * 0.5} radius:radius startAngle:M_PI * 1.5 endAngle:M_PI * 1.5 + M_PI * 2 * _progress clockwise:YES];
        [progressPath stroke];
    }
}

- (UIImageView *)statusImageView
{
    if (!_statusImageView){
        _statusImageView = [[UIImageView alloc] init];
    }
    return _statusImageView;
}

- (FSCloudSyncStatusArcView *)arcView
{
    if (!_arcView){
        _arcView = [[FSCloudSyncStatusArcView alloc] init];
    }
    return _arcView;
}

@end
