/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "FSFileAndImagePicker.h"
#import "PrivateDefine.h"
#import "FileSelectDestinationViewController.h"
#import "PhotoToPDF/PhotoToPDFViewController.h"
#import "FSMediaFileToPDFViewController.h"
#import "FSAudioRecorderController.h"
#import <AVFoundation/AVFoundation.h>
#import "AudioPlayerControl.h"

@interface FSFileAndImagePicker () <PhotoToPDFViewControllerDelegate, FSMediaFileToPDFViewControllerDelegate, FSAVAudioRecorderViewControllerDelegate>

@property (nonatomic, weak) UIViewController *rootViewController;
@property (nonatomic, copy) NSArray<NSString *> *imageTypes;
@property (nonatomic, copy) NSArray<NSString *> *audioTypes;
@property (nonatomic, copy) NSArray<NSString *> *videoTypes;
@end

@implementation FSFileAndImagePicker

- (id)init {
    if (self = [super init]) {
        _imageTypes = @[ @"jbig2", @"jpx", @"tif", @"gif", @"png", @"jpg", @"bmp", @"jpeg" ]; // image types by default
        _audioTypes = @[ @"mp3", @"caf", @"aac", @"wav", @"wma", @"cda", @"flac", @"m4a", @"mid", @"mka", @"mp2", @"mpa", @"mpc", @"ape", @"ofr", @"ogg", @"ra", @"wv", @"tta", @"ac3", @"dts" ]; // audio
        _videoTypes = @[ @"avi", @"ivf", @"wmp", @"wm", @"wvx", @"wmx", @"asf", @"asx", @"mp2", @"swf", @"mpeg", @"mov", @"mp4" ]; // video
        self.fileType = FSMediaFileTypeImage;
    }
    return self;
}

- (void)setFileType:(FSMediaFileType)fileType{
    _fileType = fileType;
    switch (fileType) {
        case FSMediaFileTypeAudio:
            _expectedFileTypes = _audioTypes;
            break;
            
        case FSMediaFileTypeVideo:
            _expectedFileTypes = _videoTypes;
            break;
        case FSMediaFileTypeImage:
            _expectedFileTypes = _imageTypes;
            break;
        default:
            _expectedFileTypes = nil;
            break;
    }
}

- (void)presentInRootViewController:(UIViewController *)rootViewController fromView:(UIView *_Nullable)view {
    
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:nil message:nil preferredStyle:DEVICE_iPHONE ? UIAlertControllerStyleActionSheet : UIAlertControllerStyleAlert];
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel")
                                                           style:UIAlertActionStyleCancel
                                                         handler:^(UIAlertAction *action) {
                                                             [self.delegate fileAndImagePickerDidCancel:self];
                                                         }];
    
    UIAlertAction *pageAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kBlankPage")
                                                          style:UIAlertActionStyleDefault
                                                        handler:^(UIAlertAction *action) {
                                                            UIStoryboard *storyboard = [UIStoryboard storyboardWithName:@"InsertPage" bundle:[NSBundle bundleForClass:[self class]]];
                                                            UINavigationController *nav = [storyboard instantiateInitialViewController];
                                                            nav.modalPresentationStyle = UIModalPresentationFullScreen;
//                                                            FSInsertPageConfigVCtrl *VC = (FSInsertPageConfigVCtrl *)nav.topViewController;
//                                                            [VC setUIExtensionsManager:self.extensionsManager];
//                                                            VC.delegate = self.insertPageDelegate;
//                                                            [rootViewController presentViewController:nav animated:YES completion:nil];
                                                        }];
    
    
    UIAlertAction *fileAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kFromDocument")
                                                         style:UIAlertActionStyleDefault
                                                       handler:^(UIAlertAction *action) {
                                                           [alert dismissViewControllerAnimated:NO completion:nil];
                                                           
                                                           FileSelectDestinationViewController *selectDestination = [[FileSelectDestinationViewController alloc] init];
                                                           selectDestination.isRootFileDirectory = YES;
                                                           selectDestination.fileOperatingMode = FileListMode_Import;
                                                           selectDestination.expectFileType = self.expectedFileTypes;
                                                           [selectDestination loadFilesWithPath:DOCUMENT_PATH];
                                                           selectDestination.operatingHandler = ^(FileSelectDestinationViewController *controller, NSArray *destinationFolder) {
                                                               [controller dismissViewControllerAnimated:YES completion:^{
                                                                   if (destinationFolder.count > 0) {
                                                                       NSString *srcPath = destinationFolder[0];
                                                                       if ([self.imageTypes containsObject:srcPath.pathExtension.lowercaseString]) {
                                                                           UIImage *image = [UIImage imageWithContentsOfFile:srcPath];
                                                                           if (image) {
                                                                               [self.delegate fileAndImagePicker:self didPickImage:image imagePath:srcPath];
                                                                           }
                                                                       } else {
                                                                           [self.delegate fileAndImagePicker:self didPickFileAtPath:srcPath];
                                                                       }
                                                                   }
                                                               }];
                                                           };
                                                           selectDestination.cancelHandler = ^(FileSelectDestinationViewController *controller) {
                                                               [controller dismissViewControllerAnimated:YES completion:^{
                                                                   [self.delegate fileAndImagePickerDidCancel:self];
                                                               }];
                                                           };
                                                           dispatch_async(dispatch_get_main_queue(), ^{
                                                               FSNavigationController *selectDestinationNavController = [[FSNavigationController alloc] initWithRootViewController:selectDestination];
                                                               [selectDestinationNavController fs_setCustomTransitionAnimator:self->_extensionsManager.presentedVCAnimator];
                                                               [rootViewController presentViewController:selectDestinationNavController animated:YES completion:nil];
                                                           });
                                                       }];
    
    UIAlertAction *photoAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kFromAlbum")
                                                          style:UIAlertActionStyleDefault
                                                        handler:^(UIAlertAction *action) {
                                                            PhotoToPDFViewController *photoController = [[PhotoToPDFViewController alloc] initWithButton:(UIButton*)view];
                                                            photoController.delegate = self;
                                                            [rootViewController presentViewController:photoController animated:NO completion:nil];
                                                            [photoController openAlbum];
                                                        }];
    
    UIAlertAction *cameraAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kFromCamera")
                                                           style:UIAlertActionStyleDefault
                                                         handler:^(UIAlertAction *action) {
                                                             PhotoToPDFViewController *photoController = [[PhotoToPDFViewController alloc] initWithButton:(UIButton*)view];
                                                             photoController.delegate = self;
                                                             [rootViewController presentViewController:photoController animated:NO completion:nil];
                                                             [photoController openCamera];
                                                         }];
    
    [alert addAction:pageAction];
    [alert addAction:cancelAction];
    [alert addAction:fileAction];
    [alert addAction:photoAction];
    [alert addAction:cameraAction];
    
    //    if (view) {
    //        alert.popoverPresentationController.sourceView = view;
    //        alert.popoverPresentationController.sourceRect = view.bounds;
    //    }
    [rootViewController presentViewController:alert animated:YES completion:nil];
}

- (void)presentInRootViewController:(UIViewController *)rootViewController{
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:nil message:nil preferredStyle:DEVICE_iPHONE ? UIAlertControllerStyleActionSheet : UIAlertControllerStyleAlert];
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel")
                                                           style:UIAlertActionStyleCancel
                                                         handler:^(UIAlertAction *action) {
                                                             [self.delegate fileAndImagePickerDidCancel:self];
                                                         }];
    UIAlertAction *fileAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kFromDocument")
                                                         style:UIAlertActionStyleDefault
                                                       handler:^(UIAlertAction *action) {
                                                           [alert dismissViewControllerAnimated:NO completion:nil];
                                                           
                                                           FileSelectDestinationViewController *selectDestination = [[FileSelectDestinationViewController alloc] init];
                                                           selectDestination.isRootFileDirectory = YES;
                                                           selectDestination.fileOperatingMode = FileListMode_Import;
                                                           selectDestination.expectFileType = self.expectedFileTypes;
                                                           [selectDestination loadFilesWithPath:DOCUMENT_PATH];
                                                           selectDestination.operatingHandler = ^(FileSelectDestinationViewController *controller, NSArray *destinationFolder) {
                                                               [controller dismissViewControllerAnimated:YES completion:^{
                                                                   if (destinationFolder.count > 0) {
                                                                       NSString *srcPath = destinationFolder[0];
                                                                       if ([self.imageTypes containsObject:srcPath.pathExtension.lowercaseString]) {
                                                                           UIImage *image = [UIImage imageWithContentsOfFile:srcPath];
                                                                           if (image) {
                                                                               [self.delegate fileAndImagePicker:self didPickImage:image imagePath:srcPath];
                                                                           }
                                                                       } else {
                                                                           [self.delegate fileAndImagePicker:self didPickFileAtPath:srcPath];
                                                                       }
                                                                   }
                                                               }];
                                                           };
                                                           selectDestination.cancelHandler = ^(FileSelectDestinationViewController *controller) {
                                                               [controller dismissViewControllerAnimated:YES completion:^{
                                                                   [self.delegate fileAndImagePickerDidCancel:self];
                                                               }];
                                                           };
                                                           FSNavigationController *selectDestinationNavController = [[FSNavigationController alloc] initWithRootViewController:selectDestination];
                                                           [selectDestinationNavController fs_setCustomTransitionAnimator:self->_extensionsManager.presentedVCAnimator];
                                                           [rootViewController presentViewController:selectDestinationNavController animated:YES completion:nil];
                                                       }];
    
    [alert addAction:cancelAction];
    [alert addAction:fileAction];
    
    _rootViewController = rootViewController;
    if (self.fileType == FSMediaFileTypeAudio) {
        [self audioFileAddWithAlert:alert];
    }else if (self.fileType == FSMediaFileTypeVideo) {
        [self videoFileAddWithAlert:alert];
    }else if (self.fileType == FSMediaFileTypeImage){
        [self imageFileAddWithAlert:alert];
    }
    else{
        [self defaultFileAddWithAlert:alert];
    }
}

- (void)audioFileAddWithAlert:(UIAlertController *)alert{
    
    UIAlertAction *recordAudioAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kRecordAudio")
                                                           style:UIAlertActionStyleDefault
                                                         handler:^(UIAlertAction *action) {
                                                             [[AudioPlayerControl sharedControl] dismiss];
                                                             self.rootViewController.definesPresentationContext = YES;
                                                             FSAudioRecorderController *presentViewController = [[FSAudioRecorderController alloc] init];
                                                             presentViewController.modalPresentationStyle = UIModalPresentationOverCurrentContext;
                                                             presentViewController.view.backgroundColor = [UIColor clearColor];
                                                             presentViewController.delegate = self;
                                                             [self.rootViewController presentViewController:presentViewController animated:NO completion:nil];
                                                         }];
    
    [alert addAction:recordAudioAction];

    [_rootViewController presentViewController:alert animated:YES completion:nil];
}

- (void)videoFileAddWithAlert:(UIAlertController *)alert{
    UIAlertAction *photoAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kFromAlbum")
                                                          style:UIAlertActionStyleDefault
                                                        handler:^(UIAlertAction *action) {
                                                            FSMediaFileToPDFViewController *mediaFileToPDFViewController = self.mediaFileToPDFViewController;
                                                            [self.rootViewController presentViewController:mediaFileToPDFViewController animated:NO completion:nil];
                                                            [mediaFileToPDFViewController openAlbum];
                                                        }];
    
    UIAlertAction *cameraAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kRecordVideo")
                                                           style:UIAlertActionStyleDefault
                                                         handler:^(UIAlertAction *action) {
                                                             FSMediaFileToPDFViewController *mediaFileToPDFViewController = self.mediaFileToPDFViewController;
                                                             [self.extensionsManager.viewCtrlManager.navigationControllerStack.topNavigationController presentViewController:mediaFileToPDFViewController animated:NO completion:nil];
                                                             [mediaFileToPDFViewController openCamera];
                                                         }];
    
    [alert addAction:photoAction];
    [alert addAction:cameraAction];
    
    [_rootViewController presentViewController:alert animated:YES completion:nil];
}

- (void)imageFileAddWithAlert:(UIAlertController *)alert{
    UIAlertAction *photoAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kFromAlbum")
                                                          style:UIAlertActionStyleDefault
                                                        handler:^(UIAlertAction *action) {
                                                            PhotoToPDFViewController *photoController = [[PhotoToPDFViewController alloc] initWithButton:nil];
                                                            photoController.delegate = self;
                                                            [self.rootViewController presentViewController:photoController animated:NO completion:nil];
                                                            [photoController openAlbum];
                                                        }];
    
    UIAlertAction *cameraAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kFromCamera")
                                                           style:UIAlertActionStyleDefault
                                                         handler:^(UIAlertAction *action) {
                                                             PhotoToPDFViewController *photoController = [[PhotoToPDFViewController alloc] initWithButton:nil];
                                                             photoController.delegate = self;
                                                             [self.rootViewController presentViewController:photoController animated:NO completion:nil];
                                                             [photoController openCamera];
                                                         }];
    [alert addAction:photoAction];
    [alert addAction:cameraAction];
    
    [_rootViewController presentViewController:alert animated:YES completion:nil];
}

- (void)defaultFileAddWithAlert:(UIAlertController *)alert{
    UIAlertAction *photoAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kFromAlbum")
                                                          style:UIAlertActionStyleDefault
                                                        handler:^(UIAlertAction *action) {
                                                            FSMediaFileToPDFViewController *mediaFileToPDFViewController = self.mediaFileToPDFViewController;
                                                            [self.rootViewController presentViewController:mediaFileToPDFViewController animated:NO completion:nil];
                                                            [mediaFileToPDFViewController openAlbum];
                                                        }];
    
    UIAlertAction *cameraAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kFromCamera")
                                                           style:UIAlertActionStyleDefault
                                                         handler:^(UIAlertAction *action) {
                                                             FSMediaFileToPDFViewController *mediaFileToPDFViewController = self.mediaFileToPDFViewController;
                                                             [self.rootViewController presentViewController:mediaFileToPDFViewController animated:NO completion:nil];
                                                             [mediaFileToPDFViewController openCamera];
                                                         }];
    
    [alert addAction:photoAction];
    [alert addAction:cameraAction];
    
    //    if (view) {
    //        alert.popoverPresentationController.sourceView = view;
    //        alert.popoverPresentationController.sourceRect = view.bounds;
    //    }
    [_rootViewController presentViewController:alert animated:YES completion:nil];
}

- (FSMediaFileToPDFViewController *)mediaFileToPDFViewController{
    FSMediaFileToPDFViewController *mediaFileToPDFViewController = [[FSMediaFileToPDFViewController alloc] init];
    mediaFileToPDFViewController.fileType = self.fileType;
    mediaFileToPDFViewController.delegate = self;
    return mediaFileToPDFViewController;
}

#pragma mark --show menu--
- (void)showMenu:(CGRect)dvRect {

    NSArray<MenuItem *> *menuItems = nil;

    if (self.fileType == FSMediaFileTypeAudio) {
        menuItems = [self getAudioMenuItems];
    }else if (self.fileType == FSMediaFileTypeVideo){
        menuItems = [self getVideoMenuItems];
    }else if (self.fileType == FSMediaFileTypeImage){
        menuItems = [self getImageMenuItems];
    }
    if (_adjustMenusAction) {
        menuItems = _adjustMenusAction(menuItems.mutableCopy);
    }
    
    if (menuItems && menuItems.count > 0) {
        _extensionsManager.menuControl.menuItems = menuItems;
        if (_extensionsManager.menuControl.menuItems.count > 0) {
            [_extensionsManager.menuControl setRect:dvRect];
            [_extensionsManager.menuControl showMenu];
        }
        
    }
}
    
- (NSArray<MenuItem *> *)getAudioMenuItems {
    NSMutableArray<MenuItem *> *items = @[].mutableCopy;

    MenuItem *fromDocumentItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kChooseAudio") object:self action:@selector(FromDocument)];
    [items addObject:fromDocumentItem];

    MenuItem *recordAudioItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kRecordAudio") object:self action:@selector(recodeAudio)];
    [items addObject:recordAudioItem];
    
    return items;
}

- (NSArray<MenuItem *> *)getVideoMenuItems {
    NSMutableArray<MenuItem *> *items = @[].mutableCopy;

    MenuItem *fromDocumentItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kFromDocument") object:self action:@selector(FromDocument)];
    [items addObject:fromDocumentItem];

    MenuItem *fromAlbumItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kFromAlbum") object:self action:@selector(fromAlbum)];
    [items addObject:fromAlbumItem];

    MenuItem *recordVideoItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kRecordVideo") object:self action:@selector(recodeVideo)];
    [items addObject:recordVideoItem];
    
    return items;
}

- (NSArray<MenuItem *> *)getImageMenuItems {
    NSMutableArray<MenuItem *> *items = @[].mutableCopy;

    MenuItem *fromDocumentItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kFromDocument") object:self action:@selector(FromDocument)];
    [items addObject:fromDocumentItem];

    MenuItem *fromAlbumItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kFromAlbum") object:self action:@selector(fromImageAlbum)];
    [items addObject:fromAlbumItem];

    MenuItem *recordVideoItem = [[MenuItem alloc] initWithTitle:FSLocalizedForKey(@"kFromCamera") object:self action:@selector(fromCamera)];
    [items addObject:recordVideoItem];
    
    return items;
}

#pragma mark --menu action--
- (void)FromDocument{
        
    FSFileSelectDestinationViewController *selectDestination = [[FSFileSelectDestinationViewController alloc] init];
    selectDestination.isRootFileDirectory = YES;
    selectDestination.fileSelectSpecialActionType = FSFileSelectSpecialActionTypeImport;
    selectDestination.expectFileType = self.expectedFileTypes;
    [selectDestination loadFilesWithPath:DOCUMENT_PATH];
    selectDestination.operatingHandler = ^(FSFileSelectDestinationViewController *controller, NSArray *destinationFolder) {
        [controller dismissViewControllerAnimated:YES completion:^{
            if (destinationFolder.count > 0) {
                NSString *srcPath = destinationFolder[0];
                if ([self.imageTypes containsObject:srcPath.pathExtension.lowercaseString]) {
                    UIImage *image = [UIImage imageWithContentsOfFile:srcPath];
                    if (image) {
                        [self.delegate fileAndImagePicker:self didPickImage:image imagePath:srcPath];
                    }
                } else {
                    [self.delegate fileAndImagePicker:self didPickFileAtPath:srcPath];
                }
            }
        }];
    };

    selectDestination.cancelHandler = ^(FSFileSelectDestinationViewController *controller) {
        [controller dismissViewControllerAnimated:YES completion:^{
            [self.delegate fileAndImagePickerDidCancel:self];
        }];
    };
    FSNavigationController *selectDestinationNavController = [[FSNavigationController alloc] initWithRootViewController:selectDestination];
    if (DEVICE_iPHONE) {
        selectDestinationNavController.modalPresentationStyle = UIModalPresentationFullScreen;
    }else{
        selectDestinationNavController.modalPresentationStyle = UIModalPresentationFormSheet;
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= __IPHONE_13_0
        if (@available(iOS 13.0, *)) selectDestinationNavController.modalInPresentation = YES;
#endif
    }
    [[self.extensionsManager.pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:selectDestinationNavController animated:YES completion:nil];
    
}

- (void)recodeAudio{
    
    [[AudioPlayerControl sharedControl] dismiss];
    
    AVAuthorizationStatus authStatus = [AVCaptureDevice authorizationStatusForMediaType:AVMediaTypeAudio];
    if (authStatus == AVAuthorizationStatusRestricted || authStatus == AVAuthorizationStatusDenied)
    {
        [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kCannotRecordAudio") message:FSLocalizedForKey(@"kEnableAudioPrivacy") actions:@[FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kGoSetting")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
            if (index == 0) {
                return;
            }else if (index == 1){
                NSURL*url =[NSURL URLWithString:UIApplicationOpenSettingsURLString];
                if([[UIApplication sharedApplication] canOpenURL:url]) {
                    [[UIApplication sharedApplication] openURL:url options:@{} completionHandler:nil];
                }
            }
        } onCtrl:[self.extensionsManager.pdfViewCtrl fs_currentSceneTopMostViewController]];
        return;
    }else if (authStatus == AVAuthorizationStatusNotDetermined){
        [AVCaptureDevice requestAccessForMediaType:AVMediaTypeAudio completionHandler:^(BOOL granted) {
            dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.5 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                if (granted) {
                    FSAudioRecorderController *presentViewController = [[FSAudioRecorderController alloc] init];
                    presentViewController.modalPresentationStyle = UIModalPresentationOverCurrentContext;
                    presentViewController.view.backgroundColor = [UIColor clearColor];
                    presentViewController.delegate = self;
                    UIViewController *rootVC = [self.extensionsManager.pdfViewCtrl fs_currentSceneTopMostViewController];
                    rootVC.definesPresentationContext = YES;
                    [rootVC presentViewController:presentViewController animated:NO completion:nil];
                }else{
                    
                    [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kCannotRecordAudio") message:FSLocalizedForKey(@"kEnableAudioPrivacy") actions:@[FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kGoSetting")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                        if (index == 0) {
                            return;
                        }else if (index == 1){
                            NSURL*url =[NSURL URLWithString:UIApplicationOpenSettingsURLString];
                            if([[UIApplication sharedApplication] canOpenURL:url]) {
                                [[UIApplication sharedApplication] openURL:url options:@{} completionHandler:nil];
                            }
                        }
                    } onCtrl:[self.extensionsManager.pdfViewCtrl fs_currentSceneTopMostViewController]];
                    return;
                }
            });
        }];
    }else{
        FSAudioRecorderController *presentViewController = [[FSAudioRecorderController alloc] init];
        presentViewController.modalPresentationStyle = UIModalPresentationOverCurrentContext;
        presentViewController.view.backgroundColor = [UIColor clearColor];
        presentViewController.delegate = self;
        UIViewController *rootVC = [self.extensionsManager.pdfViewCtrl fs_currentSceneTopMostViewController];
        rootVC.definesPresentationContext = YES;
        [rootVC presentViewController:presentViewController animated:NO completion:nil];
    }
}

-(void) gotoSettings:(NSString*)title message:(NSString*)message
{
    UIAlertController * alert = [UIAlertController alertControllerWithTitle:title message:message preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction * cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel") style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {

    }];
    UIAlertAction * setAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kGoSetting") style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        dispatch_async(dispatch_get_main_queue(), ^{
            NSURL * url = [NSURL URLWithString:UIApplicationOpenSettingsURLString];
            [UIApplication.sharedApplication openURL:url options:@{} completionHandler:^(BOOL success) {

            }];
        });
    }];
    [alert addAction:cancelAction];
    [alert addAction:setAction];
    [[self.extensionsManager.pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:alert animated:YES completion:nil];
}

- (void)fromAlbum{
    FSMediaFileToPDFViewController *mediaFileToPDFViewController = self.mediaFileToPDFViewController;
    UIViewController *rootVC = [self.extensionsManager.pdfViewCtrl fs_currentSceneTopMostViewController];
    [rootVC presentViewController:mediaFileToPDFViewController animated:YES completion:nil];
    [mediaFileToPDFViewController openAlbum];
}

- (void)fromImageAlbum{
    FSMediaFileToPDFViewController *mediaFileToPDFViewController = [[FSMediaFileToPDFViewController alloc] init];
    mediaFileToPDFViewController.delegate = self;
    [[self.extensionsManager.pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:mediaFileToPDFViewController animated:NO completion:^{
        [mediaFileToPDFViewController openAlbum];
    }];
}

- (void)fromCamera{
    FSMediaFileToPDFViewController *mediaFileToPDFViewController = [[FSMediaFileToPDFViewController alloc] init];
    mediaFileToPDFViewController.delegate = self;
    [[self.extensionsManager.pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:mediaFileToPDFViewController animated:NO completion:^{
        [mediaFileToPDFViewController openCamera];
    }];
}

- (void)recodeVideo{

    AVAuthorizationStatus authStatus = [AVCaptureDevice authorizationStatusForMediaType:AVMediaTypeVideo];
    if (authStatus == AVAuthorizationStatusRestricted || authStatus ==AVAuthorizationStatusDenied){
        [self gotoSettings:FSLocalizedForKey(@"kCannotRecordVideo") message:FSLocalizedForKey(@"kEnableVideoPrivacy")];
        return;
    }
    
    FSMediaFileToPDFViewController *mediaFileToPDFViewController = self.mediaFileToPDFViewController;
    [[self.extensionsManager.pdfViewCtrl fs_currentSceneTopMostViewController] presentViewController:mediaFileToPDFViewController animated:NO completion:nil];
    [mediaFileToPDFViewController openCamera];
}

#pragma mark <PhotoToPDFViewControllerDelegate>

- (void)photoToPDFViewController:(PhotoToPDFViewController *)photoToPDFViewController didSelectImage:(UIImage *)image {
    [self.delegate fileAndImagePicker:self didPickImage:image imagePath:nil];
}

- (void)photoToPDFViewControllerDidCancel:(PhotoToPDFViewController *)photoToPDFViewController {
    [self.delegate fileAndImagePickerDidCancel:self];
}

#pragma mark <FSMediaFileToPDFViewControllerDelegate>

- (void)mediaFileToPDFViewController:(FSMediaFileToPDFViewController *)mediaFileToPDFViewController didPickFileAtPath:(NSString *)filePath{
    if (self.delegate) {
        [self.delegate fileAndImagePicker:self didPickFileAtPath:filePath];
    }
}

- (void)mediaFileToPDFViewController:(FSMediaFileToPDFViewController *)mediaFileToPDFViewController didSelectVideo:(AVPlayer *)video{
    if (self.delegate) {
        [self.delegate fileAndImagePicker:self didSelectVideo:video];
    }
}

- (void)mediaFileToPDFViewController:(FSMediaFileToPDFViewController *)mediaFileToPDFViewController didSelectedImage:(UIImage *)image{
     [self.delegate fileAndImagePicker:self didPickImage:image imagePath:nil];
}

- (void)mediaFileToPDFViewControllerDidCancel:(FSMediaFileToPDFViewController *)mediaFileToPDFViewController{
     [self.delegate fileAndImagePickerDidCancel:self];
}

#pragma mark <FSAVAudioRecorderViewControllerDelegate>

- (void)audioRecorderViewController:(FSAudioRecorderController *)audioRecorderViewController didRecordFileAtPath:(NSString *)filePath{
    if (self.delegate) {
        [self.delegate fileAndImagePicker:self didPickFileAtPath:filePath];
    }
}
- (void)audioRecorderViewControllerDidCancel:(FSAudioRecorderController *)audioRecorderViewController{
    [self.delegate fileAndImagePickerDidCancel:self];
}

@end
