/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#import "FSMediaFileToPDFViewController.h"
#import <MobileCoreServices/MobileCoreServices.h>
#import "Utility.h"
#import <Photos/Photos.h>

@interface FSMediaFileToPDFViewController ()

@property (nonatomic, strong) UIImagePickerController *mediaController;
@property (nonatomic) BOOL canPick;

@end

@implementation FSMediaFileToPDFViewController

- (instancetype)init{
    self = [super init];
    if (self) {
        self.mediaController = [[UIImagePickerController alloc] init];
        self.modalPresentationStyle = UIModalPresentationFullScreen;
        self.mediaController.delegate = self;
    }
    return self;
}

- (UIInterfaceOrientationMask)supportedInterfaceOrientations {
    return  UIInterfaceOrientationMaskPortrait;
}

- (UIInterfaceOrientation)preferredInterfaceOrientationForPresentation{
    return UIInterfaceOrientationPortrait;
}


- (void)addMediaControllerForSourceType:(UIImagePickerControllerSourceType)sourceType{
    self.mediaController.sourceType = sourceType;
    [self.view addSubview:self.mediaController.view];
    [self.mediaController.view mas_makeConstraints:^(MASConstraintMaker *make) {
        make.left.top.bottom.right.fs_equalToSuperview();
    }];
    [self addChildViewController:self.mediaController];
    [self.mediaController didMoveToParentViewController:self];
}

- (void)openAlbum {
    PHAuthorizationStatus statusA = [PHPhotoLibrary authorizationStatus];
    if (statusA == PHAuthorizationStatusRestricted || statusA == PHAuthorizationStatusDenied) {
        [self dismissViewControllerAnimated:NO completion:^{
            [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kALBUM") actions:@[FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kGoSetting")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                if (index == 0) {
                    return;
                }else if (index == 1){
                    NSURL*url =[NSURL URLWithString:UIApplicationOpenSettingsURLString];
                    if([[UIApplication sharedApplication] canOpenURL:url]) {
                        [[UIApplication sharedApplication] openURL:url options:@{} completionHandler:nil];
                    }
                }
            } onCtrl:[self fs_currentSceneTopMostViewController]];
        }];
        return;
    }else if (statusA == PHAuthorizationStatusNotDetermined){
        [PHPhotoLibrary  requestAuthorization:^(PHAuthorizationStatus status) {
            if (status == PHAuthorizationStatusRestricted || status == PHAuthorizationStatusDenied) {
                dispatch_async(dispatch_get_main_queue(), ^{
                    [self dismissViewControllerAnimated:YES completion:nil];
                });
                return;
            }
            
            dispatch_async(dispatch_get_main_queue(), ^{
                [self addMediaControllerForSourceType:UIImagePickerControllerSourceTypePhotoLibrary];
                if (self.fileType == FSMediaFileTypeVideo) {
                    self.mediaController.mediaTypes=@[(NSString *)kUTTypeMovie];
                }
                self.canPick = true;
            });
        }];
    }else{
        if ([UIImagePickerController isSourceTypeAvailable:UIImagePickerControllerSourceTypePhotoLibrary]) {
            [self addMediaControllerForSourceType:UIImagePickerControllerSourceTypePhotoLibrary];
            if (self.fileType == FSMediaFileTypeVideo) {
                self.mediaController.mediaTypes=@[(NSString *)kUTTypeMovie];
            }
            
            self.canPick = true;
        } else {
            [self dismissViewControllerAnimated:NO completion:^{
                UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kFailed") message:FSLocalizedForKey(@"kFailedOpenAlbum") preferredStyle:UIAlertControllerStyleAlert];
                UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                                       style:UIAlertActionStyleCancel
                                                                     handler:nil];
                [alertController addAction:cancelAction];
                [[self fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
            }];
        }
    }
}

- (void)openCamera {
    NSString *mediaType = AVMediaTypeVideo;
    AVAuthorizationStatus authStatus = [AVCaptureDevice authorizationStatusForMediaType:mediaType];
    if (authStatus == AVAuthorizationStatusRestricted || authStatus == AVAuthorizationStatusDenied)
    {
        [self dismissViewControllerAnimated:NO completion:^{
            [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kNotAllowTakePhoto") message:FSLocalizedForKey(@"kCAMERADenied") actions:@[FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kGoSetting")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                if (index == 0) {
                    return;
                }else if (index == 1){
                    NSURL*url =[NSURL URLWithString:UIApplicationOpenSettingsURLString];
                    if([[UIApplication sharedApplication] canOpenURL:url]) {
                        [[UIApplication sharedApplication] openURL:url options:@{} completionHandler:nil];
                    }
                }
            } onCtrl:[self fs_currentSceneTopMostViewController]];
            if (self.delegate && [self.delegate respondsToSelector:@selector(mediaFileToPDFViewControllerDidDismissCompletion:)]) {
                [self.delegate mediaFileToPDFViewControllerDidDismissCompletion:self];
            }
        }];
        return;
    }else if (authStatus == AVAuthorizationStatusNotDetermined){
        [AVCaptureDevice requestAccessForMediaType:AVMediaTypeVideo completionHandler:^(BOOL granted) {
            dispatch_async(dispatch_get_main_queue(), ^{
                if (granted) {
                    [self addMediaControllerForSourceType:UIImagePickerControllerSourceTypeCamera];
                    self.mediaController.showsCameraControls = YES;
                    if (self.fileType == FSMediaFileTypeVideo) {
                        self.mediaController.mediaTypes=@[(NSString *)kUTTypeMovie];
                        self.mediaController.videoQuality=UIImagePickerControllerQualityTypeIFrame1280x720;
                        self.mediaController.cameraCaptureMode=UIImagePickerControllerCameraCaptureModeVideo;
                    }
                    self.canPick = true;
                }else{
                    [self dismissViewControllerAnimated:NO completion:^{
                        
                        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(0.5 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                            [UIAlertController fs_alertWithTitle:FSLocalizedForKey(@"kNotAllowTakePhoto") message:FSLocalizedForKey(@"kCAMERADenied") actions:@[FSLocalizedForKey(@"kCancel"), FSLocalizedForKey(@"kGoSetting")] actionBack:^(NSUInteger index, UIAlertAction * _Nonnull action) {
                                if (index == 0) {
                                    return;
                                }else if (index == 1){
                                    NSURL*url =[NSURL URLWithString:UIApplicationOpenSettingsURLString];
                                    if([[UIApplication sharedApplication] canOpenURL:url]) {
                                        [[UIApplication sharedApplication] openURL:url options:@{} completionHandler:nil];
                                    }
                                }
                            } onCtrl:[self fs_currentSceneTopMostViewController]];

                        });
                        if (self.delegate && [self.delegate respondsToSelector:@selector(mediaFileToPDFViewControllerDidDismissCompletion:)]) {
                            [self.delegate mediaFileToPDFViewControllerDidDismissCompletion:self];
                        }
                    }];
                    return;
                }
            });
        }];
    }else{
        if ([UIImagePickerController isSourceTypeAvailable:UIImagePickerControllerSourceTypeCamera]) {
            [self addMediaControllerForSourceType:UIImagePickerControllerSourceTypeCamera];
            self.mediaController.showsCameraControls = YES;
            if (self.fileType == FSMediaFileTypeVideo) {
                self.mediaController.mediaTypes=@[(NSString *)kUTTypeMovie];
                self.mediaController.videoQuality=UIImagePickerControllerQualityTypeIFrame1280x720;
                self.mediaController.cameraCaptureMode=UIImagePickerControllerCameraCaptureModeVideo;
            }

            self.canPick = true;
        } else {
            [self dismissViewControllerAnimated:NO completion:^{
                UIAlertController *alertController = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kFailed") message:FSLocalizedForKey(@"kFailedOpenCamera") preferredStyle:UIAlertControllerStyleAlert];
                UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kOK")
                                                                       style:UIAlertActionStyleCancel
                                                                     handler:nil];
                [alertController addAction:cancelAction];
                [[self fs_currentSceneTopMostViewController] presentViewController:alertController animated:YES completion:nil];
                if (self.delegate && [(NSObject *)self.delegate respondsToSelector:@selector(mediaFileToPDFViewControllerDidDismissCompletion:)]) {
                    [self.delegate mediaFileToPDFViewControllerDidDismissCompletion:self];
                }
            }];
        }
    }
}

- (void)imagePickerController:(UIImagePickerController *)picker didFinishPickingMediaWithInfo:(NSDictionary<NSString *, id> *)info {
    BOOL needDismiss = YES;
    BOOL animated = YES;
    if (self.delegate && [self.delegate respondsToSelector:@selector(mediaFileToPDFViewControllerShouldDismissWhenDidFinishPickingMedia:)]) {
        needDismiss = [self.delegate mediaFileToPDFViewControllerShouldDismissWhenDidFinishPickingMedia:self];
    }
    if (self.delegate && [self.delegate respondsToSelector:@selector(mediaFileToPDFViewControllerShouldDismissViewControllerAnimated:)]) {
        animated = [self.delegate mediaFileToPDFViewControllerShouldDismissViewControllerAnimated:self];
    }
    
    void(^respondDelegate)() = ^{
        BOOL canPick = self.canPick;
        self.canPick = false;
        if (canPick) {
            if (self.delegate) {
                if (self.fileType == FSMediaFileTypeVideo) {
                    if ([self.delegate respondsToSelector:@selector(mediaFileToPDFViewController:didPickFileAtPath:)]){
                        NSURL *url = (NSURL *) [info objectForKey:UIImagePickerControllerMediaURL];;
                        NSString *urlStr = [url path];
                        if (UIVideoAtPathIsCompatibleWithSavedPhotosAlbum(urlStr)) {
                            [self.delegate mediaFileToPDFViewController:self didPickFileAtPath:url.path];
                        }
                    }
                }else{
                    if ([self.delegate respondsToSelector:@selector(mediaFileToPDFViewController:didSelectedImage:filePath:)]){
                        NSURL *url = (NSURL *) [info objectForKey:UIImagePickerControllerImageURL];
                        NSString *urlStr = [url path];
                        UIImage *image = (UIImage *) [info objectForKey:UIImagePickerControllerOriginalImage];
                        if (!image){
                            image = (UIImage *) [info objectForKey:UIImagePickerControllerMediaMetadata];
                        }
                        image = [image fs_fixOrientation];
                        [self.delegate mediaFileToPDFViewController:self didSelectedImage:image filePath:urlStr];
                    }
                    
                    if ([self.delegate respondsToSelector:@selector(mediaFileToPDFViewController:didSelectedImage:)]) {
                        UIImage *image = (UIImage *) [info objectForKey:UIImagePickerControllerOriginalImage];
                        image = [image fs_fixOrientation];
                        [self.delegate mediaFileToPDFViewController:self didSelectedImage:image];
                    }
                }
            }
        }
    };
    
    if (needDismiss) {
        [self dismissViewControllerAnimated:animated completion:^{
            respondDelegate();
            if (self.delegate && [self.delegate respondsToSelector:@selector(mediaFileToPDFViewControllerDidDismissCompletion:)]) {
                [self.delegate mediaFileToPDFViewControllerDidDismissCompletion:self];
            }
        }];
    }else{
        respondDelegate();
    }
}

- (void)imagePickerControllerDidCancel:(UIImagePickerController *)picker {
    [self dismissViewControllerAnimated:NO completion:^{
        if (self.delegate && [self.delegate respondsToSelector:@selector(mediaFileToPDFViewControllerDidDismissCompletion:)]) {
            [self.delegate mediaFileToPDFViewControllerDidDismissCompletion:self];
        }
    }];
    self.canPick = false;
    if (self.delegate && [self.delegate respondsToSelector:@selector(mediaFileToPDFViewControllerDidCancel:)]) {
        [self.delegate mediaFileToPDFViewControllerDidCancel:self];
    }
}
@end
