/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "ViewController.h"
#import "AppDelegate.h"
#import <UIKit/UIKit.h>
#import <AVFoundation/AVFoundation.h>


@interface ViewController () <FSFileSelectDelegate, IDocEventListener, UIExtensionsManagerDelegate>

@property (nonatomic, strong) UIViewController *pdfViewController;
@property (nonatomic, strong) IBOutlet UIButton *openScanBtn;
@end

@implementation ViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view, typically from a nib.
    //See 'FSLocalization' for details
//    [FSLocalization setCurrentLanguageID:FSLocalizationLanguageOptionalEnglish];
    
    self.view.backgroundColor = [UIColor whiteColor];
    self.view.autoresizingMask = UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleTopMargin | UIViewAutoresizingFlexibleRightMargin | UIViewAutoresizingFlexibleBottomMargin | UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleWidth;

    self.pdfViewControl = [[FSPDFViewCtrl alloc] initWithFrame:[[UIScreen mainScreen] bounds]];
    [self.pdfViewControl setRMSAppClientId:@"972b6681-fa03-4b6b-817b-c8c10d38bd20" redirectURI:@"com.foxitsoftware.com.mobilepdf-for-ios://authorize"];
    [self.pdfViewControl registerDocEventListener:self];
   
    FSFileListViewController *filelistViewController;
#if __IPHONE_OS_VERSION_MAX_ALLOWED >= 130000
    if (@available(iOS 13.0, *)) {
        self.pdfViewControl.windowScene = self.windowScene;
        filelistViewController = [[FSFileListViewController alloc] initWithWindowScene:self.windowScene];
    } else
#endif
    {
        // Fallback on earlier versions
        filelistViewController = [[FSFileListViewController alloc] init];
    }

    filelistViewController.delegate = self;
    [self.pdfViewControl registerDocEventListener:filelistViewController];
    self.rootViewController = [[UINavigationController alloc] initWithRootViewController:filelistViewController];
    self.rootViewController.navigationBarHidden = YES;
    self.rootViewController.interactivePopGestureRecognizer.enabled = NO;
    [self addChildViewController:self.rootViewController];
    [self.view addSubview:self.rootViewController.view];
    [self.rootViewController.view setTranslatesAutoresizingMaskIntoConstraints:NO];
    NSMutableArray *contraints = @[].mutableCopy;
    [contraints addObject:[self.view.leftAnchor constraintEqualToAnchor:self.rootViewController.view.leftAnchor]];
    [contraints addObject:[self.view.rightAnchor constraintEqualToAnchor:self.rootViewController.view.rightAnchor]];
    [contraints addObject:[self.view.topAnchor constraintEqualToAnchor:self.rootViewController.view.topAnchor]];
    [contraints addObject:[self.view.bottomAnchor constraintEqualToAnchor:self.rootViewController.view.bottomAnchor]];
    [self.view addConstraints:contraints];
    
    self.openScanBtn.hidden = YES;
#if !_MAC_CATALYST_
    self.openScanBtn.hidden = NO;
    [filelistViewController.view addSubview:self.openScanBtn];
    
    [self.openScanBtn.rightAnchor constraintEqualToAnchor:filelistViewController.view.rightAnchor constant:-15].active = true;
    if (@available(iOS 11.0, *)) {
        [self.openScanBtn.bottomAnchor constraintEqualToAnchor:filelistViewController.view.safeAreaLayoutGuide.bottomAnchor constant:-15].active = true;
    } else {
        [self.openScanBtn.bottomAnchor constraintEqualToAnchor:filelistViewController.view.bottomAnchor constant:-15].active = true;
    }
    
    UIImage *image = [UIImage imageNamed:@"scan"];
    [self.openScanBtn setBackgroundImage:[image imageWithRenderingMode:UIImageRenderingModeAlwaysTemplate] forState:UIControlStateNormal];
    [self.openScanBtn setTintColor:UIExtensionsManager.primaryColor];
    self.openScanBtn.accessibilityLabel = FSLocalizedForKey(@"kScanVoiceover");
    [PDFScanManager initializeScanner:0 serial2:0];
    [PDFScanManager initializeCompression:0 serial2:0];

#endif
}

#if !_MAC_CATALYST_
- (void)showAlertWithOpenScanError
{
    UIAlertController *alert = [UIAlertController alertControllerWithTitle:FSLocalizedForKey(@"kNotAllowTakePhoto") message:FSLocalizedForKey(@"kCAMERADenied") preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *conform = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kGoSetting") style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
            NSURL*url =[NSURL URLWithString:UIApplicationOpenSettingsURLString];
            if([[UIApplication sharedApplication] canOpenURL:url]) {
                [[UIApplication sharedApplication] openURL:url];
            }
    }];

    UIAlertAction *cancel = [UIAlertAction actionWithTitle:FSLocalizedForKey(@"kCancel") style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {
            
    }];
    [alert addAction:conform];
    [alert addAction:cancel];
    [self presentViewController:alert animated:YES completion:nil];
}

- (void)showPDFScanViewController
{
    UIViewController* vc =  [[PDFScanManager shareManager] getPDFScanView];
    if (vc)
    {
        [self presentViewController:vc animated:YES completion:nil];
        
        [PDFScanManager setSaveAsCallBack:^(NSError * _Nonnull error, NSString * _Nonnull savePath) {
              if (savePath) {
                  if (vc.presentingViewController) {
                      [vc.presentingViewController dismissViewControllerAnimated:NO completion:nil];
                  }
                  [vc dismissViewControllerAnimated:NO completion:nil];
                  [self didFileSelected:savePath];
            }
        }];
    }
}
#endif

- (IBAction)openScan:(UIButton *)sender{
    #if !_MAC_CATALYST_
    
    NSString *mediaType = AVMediaTypeVideo;
    AVAuthorizationStatus authStatus = [AVCaptureDevice authorizationStatusForMediaType:mediaType];
    if (authStatus == AVAuthorizationStatusRestricted || authStatus == AVAuthorizationStatusDenied)
    {
        [self showAlertWithOpenScanError];
        
    }else if (authStatus == AVAuthorizationStatusNotDetermined){
        [AVCaptureDevice requestAccessForMediaType:AVMediaTypeVideo completionHandler:^(BOOL granted) {
            dispatch_async(dispatch_get_main_queue(), ^{
                if (granted) {
                    [self showPDFScanViewController];
                }else{
                    [self showAlertWithOpenScanError];
                }
            });
        }];
    }else{
        [self showPDFScanViewController];
    }

    #endif
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - rotate event

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    return !self.extensionsMgr.isScreenLocked;
}

- (BOOL)shouldAutorotate {
    return !self.extensionsMgr.isScreenLocked;
}

- (UIInterfaceOrientationMask)supportedInterfaceOrientations {
    return UIInterfaceOrientationMaskAll;
}

- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration {
    [self.extensionsMgr willRotateToInterfaceOrientation:toInterfaceOrientation duration:duration];
}

- (void)willAnimateRotationToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration {
    [self.extensionsMgr willAnimateRotationToInterfaceOrientation:toInterfaceOrientation duration:duration];
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation {
    [self.extensionsMgr didRotateFromInterfaceOrientation:fromInterfaceOrientation];
}

- (BOOL)prefersStatusBarHidden {
    return self.extensionsMgr.prefersStatusBarHidden;
}

#pragma mark <IDocEventListener>

- (void)onDocOpened:(FSPDFDoc *)document error:(int)error {
    // Called when a document is opened.
}

- (void)onDocClosed:(FSPDFDoc *)document error:(int)error {
    // Called when a document is closed.
}

- (void)onDocLoading:(FSPDFDoc *)document progress:(int)progress{
    // Called when a document is loading.
    NSLog(@"%d",progress);
}

#pragma mark <FSFileSelectDelegate>

- (void)didFileSelected:(NSString *)filePath {
    
    __weak typeof(self) weakSelf = self;
    
    if (!self.pdfViewController) {
        self.pdfViewController = [[UIViewController alloc] init];
        self.pdfViewController.view = self.pdfViewControl;
        self.pdfViewController.automaticallyAdjustsScrollViewInsets = NO;
    }
    
    if (self.extensionsMgr == nil) {
        NSString *configPath = [[NSBundle mainBundle] pathForResource:@"uiextensions_config" ofType:@"json"];
        self.extensionsMgr = [[UIExtensionsManager alloc] initWithPDFViewControl:self.pdfViewControl configuration:[NSData dataWithContentsOfFile:configPath]];
        
        if (nil == self.extensionsMgr) {
            return;
        }
        
        self.extensionsMgr.goBack = ^() {
            [weakSelf.rootViewController popViewControllerAnimated:YES];
        };
        self.extensionsMgr.delegate = self;
        self.pdfViewControl.extensionsManager = self.extensionsMgr;
    }
    
    if (self.pdfViewControl.filePath != nil) {
        return;
    }
    
    [self.pdfViewControl openDoc:filePath
                        password:nil
                      completion:^(FSErrorCode error) {
                          if (error == FSErrSuccess) {
                              [weakSelf showPDFView];
                          } else if (error == FSErrDeviceLimitation) {
                              [self alertWithTitle:FSLocalizedForKey(@"kError") message:FSLocalizedForKey(@"kDeviceLimitatio")];
                          } else if (error == FSErrCanNotGetUserToken) {
                              [self alertWithTitle:FSLocalizedForKey(@"kError") message:FSLocalizedForKey(@"kCanNotGetUserToken")];
                          } else if (error == FSErrInvalidLicense) {
                              [self alertWithTitle:FSLocalizedForKey(@"kWarning") message:FSLocalizedForKey(@"kInvalidLicense")];
                          } else {
                              [self alertWithTitle:FSLocalizedForKey(@"kError") message:FSLocalizedForKey(@"kFailedToOpen")];
                          }
                      }];
}

- (void)showPDFView {
    BOOL isShowing = (self.rootViewController.topViewController.view == self.pdfViewControl);
    if (isShowing) {
        return;
    }
    
    dispatch_async(dispatch_get_main_queue(), ^{
        if(![self.rootViewController.topViewController isKindOfClass:[UIViewController class]] || [self.rootViewController.topViewController isKindOfClass:[FSFileListViewController class]]) {
            [self.rootViewController pushViewController:self.pdfViewController animated:YES];
        }
    });
}

- (void)alertWithTitle:(NSString *)title message:(NSString *)message {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:title message:message preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:nil];
    [alertController addAction:cancelAction];
    [self presentViewController:alertController animated:YES completion:nil];
}

@end

