/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "ExportAnnotData.h"
#import <FoxitRDK/FSPDFObjC.h>

@implementation ExportAnnotData

+ (NSString *)getColorHexByDecimal:(NSInteger)decimal {
    NSString *hex =@"";
    NSString *letter;
    NSInteger number;
    for (int i = 0; i < 9; i ++) {
        
        number = decimal % 16;
        decimal = decimal / 16;
        switch (number) {
            case 10:
                letter =@"A"; break;
            case 11:
                letter =@"B"; break;
            case 12:
                letter =@"C"; break;
            case 13:
                letter =@"D"; break;
            case 14:
                letter =@"E"; break;
            case 15:
                letter =@"F"; break;
            default:
                letter = [NSString stringWithFormat:@"%ld", number];
        }
        hex = [letter stringByAppendingString:hex];
        if (decimal == 0) {
            while (hex.length < 6) {
                hex = [@"0" stringByAppendingString:hex];
            }
            break;
        }
    }
    return [NSString stringWithFormat:@"#%@",hex];
}

+ (NSString *)annotTypeEnumToString:(FSAnnotType)type{
    switch (type) {
        case FSAnnotNote:
            return @"Text";
        case FSAnnotLink:
            return @"Link";
        case FSAnnotFreeText:
            return @"FreeText";
        case FSAnnotLine:
            return @"Line";
        case FSAnnotSquare:
            return @"Square";
        case FSAnnotCircle:
            return @"Circle";
        case FSAnnotPolygon:
            return @"Polygon";
        case FSAnnotPolyLine:
            return @"PolyLine";
        case FSAnnotHighlight:
            return @"Highlight";
        case FSAnnotUnderline:
            return @"Underline";
        case FSAnnotSquiggly:
            return @"Squiggly";
        case FSAnnotStrikeOut:
            return @"StrikeOut";
        case FSAnnotStamp:
            return @"Stamp";
        case FSAnnotCaret:
            return @"Caret";
        case FSAnnotInk:
            return @"Ink";
        case FSAnnotPSInk:
            return @"PSInk";
        case FSAnnotFileAttachment:
            return @"FileAttachment";
        case FSAnnotSound:
            return @"Sound";
        case FSAnnotMovie:
            return @"Movie";
        case FSAnnotWidget:
            return @"Widget";
        case FSAnnotScreen:
            return @"Screen";
        case FSAnnotPrinterMark:
            return @"PrinterMark";
        case FSAnnotTrapNet:
            return @"TrapNet";
        case FSAnnotWatermark:
            return @"Watermark";
        case FSAnnot3D:
            return @"3D";
        case FSAnnotPopup:
            return @"Popup";
        case FSAnnotRedact:
            return @"Redact";
        case FSAnnotRichMedia:
            return @"RichMedia";
        default:
            return @"UnknownType";
    }
}

+ (NSString *)annotFlagsEnumToString:(FSAnnotFlags)flags{
    NSMutableArray *flagsArr = @[].mutableCopy;
    if (flags & FSAnnotFlagInvisible) {
        [flagsArr addObject:@"Invisible"];
    }
    if (flags & FSAnnotFlagHidden) {
        [flagsArr addObject:@"Hidden"];
    }
    if (flags & FSAnnotFlagPrint) {
        [flagsArr addObject:@"Print"];
    }
    if (flags & FSAnnotFlagNoZoom) {
        [flagsArr addObject:@"NoZoom"];
    }
    if (flags & FSAnnotFlagNoRotate) {
        [flagsArr addObject:@"NoRotate"];
    }
    if (flags & FSAnnotFlagNoView) {
        [flagsArr addObject:@"NoView"];
    }
    if (flags & FSAnnotFlagReadOnly) {
        [flagsArr addObject:@"ReadOnly"];
    }
    if (flags & FSAnnotFlagLocked) {
        [flagsArr addObject:@"Locked"];
    }
    if (flags & FSAnnotFlagToggleNoView) {
        [flagsArr addObject:@"ToggleNoView"];
    }
    if (flags & FSAnnotFlagLockedContents) {
        [flagsArr addObject:@"LockedContents"];
    }
    return [flagsArr componentsJoinedByString:@","];
}

+ (NSString *)convertFSDateTime2DataString:(FSDateTime *)time {
    return [NSString stringWithFormat:@"D:%d%02d%02d%02d%02d%02d+%02d'%02d'",time.year,time.month,time.day,time.hour,time.minute,time.second,time.utc_hour_offset,time.utc_minute_offset];
}

+ (NSString *)annotBorderStyle:(FSBorderInfoStyle)style {
    switch (style) {
        case FSBorderInfoSolid:
            return @"Solid";
        case FSBorderInfoDashed:
            return @"Dashed";
        case FSBorderInfoUnderLine:
            return @"UnderLine";
        case FSBorderInfoBeveled:
            return @"Beveled";
        case FSBorderInfoInset:
            return @"Inset";
        case FSBorderInfoCloudy:
            return @"Cloudy";
    }
}

+ (NSString *)makeupEndingStyle:(FSMarkupEndingStyle)style {
    switch (style) {
        case FSMarkupEndingStyleNone:
            return @"None";
        case FSMarkupEndingStyleSquare:
            return @"Square";
        case FSMarkupEndingStyleCircle:
            return @"Circle";
        case FSMarkupEndingStyleDiamond:
            return @"Diamond";
        case FSMarkupEndingStyleOpenArrow:
            return @"OpenArrow";
        case FSMarkupEndingStyleClosedArrow:
            return @"ClosedArrow";
        case FSMarkupEndingStyleButt:
            return @"Butt";
        case FSMarkupEndingStyleROpenArrow:
            return @"ROpenArrow";
        case FSMarkupEndingStyleRClosedArrow:
            return @"RClosedArrow";
        case FSMarkupEndingStyleSlash:
            return @"Slash";
    }
}

+ (NSString *)makeupState:(FSMarkupState)state {
    switch (state) {
        case FSMarkupStateNone:
            return @"None";
        case FSMarkupStateMarked:
            return @"Marked";
        case FSMarkupStateUnmarked:
            return @"Unmarked";
        case FSMarkupStateAccepted:
            return @"Accepted";
        case FSMarkupStateRejected:
            return @"Rejected";
        case FSMarkupStateCancelled:
            return @"Cancelled";
        case FSMarkupStateCompleted:
            return @"Completed";
        case FSMarkupStateDeferred:
            return @"Deferred";
        case FSMarkupStateFuture:
            return @"Future";
    }
}

+ (id)retriveDict:(NSMutableDictionary *)json key:(NSString *)key root:(FSPDFObject *)root{
    id ret;
    FSPDFDictionary *dict;
    char hexCode[]  = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
    switch (root.getType) {
        case FSPDFObjectBoolean:
            return @([root getBoolean]);
            
        case FSPDFObjectNumber:
            if (ABS([root getInteger] - [root getFloat]) < 0.001) {
                return @([root getInteger]);
            }
            return @([root getFloat]);
        case FSPDFObjectString:{
            NSData *cs = [root getString];
            uint8_t *buffer = (uint8_t *)cs.bytes;
            ret = [NSMutableString stringWithString:@"<"];
            for (int i = 0; i < cs.length; i++) {
                uint8_t ch = *(buffer+i);
                [ret appendFormat:@"%c",hexCode[(ch & 0xf0) >> 4]];
                [ret appendFormat:@"%c",hexCode[(ch & 0x0f)]];
            }
            [ret appendString:@">"];
            return ret;
        }
        case FSPDFObjectName:
            return [NSString stringWithFormat:@"/%@",[root getName]];
        case FSPDFObjectArray:
        {
            ret = @[].mutableCopy;
            FSPDFArray *pdfArr = [root getArray];
            for (int i = 0 ; i < pdfArr.getElementCount; i++) {
                [ret addObject:[self retriveDict:json key:key root:[pdfArr getElement:i]]];
            }
            return ret;
        }

        case FSPDFObjectStream:
        {
            ret = @{}.mutableCopy;
            FSPDFStream *stream = [root getStream];
            dict = stream.getDictionary;
            unsigned long long size = [stream getDataSize:YES];
            if (size) {
                NSData *temp = [stream getData:YES];
                NSMutableString *rawData = [NSMutableString string];
                
                uint8_t *buffer = (uint8_t *)temp.bytes;
                for (int i = 0; i < temp.length; i++) {
                    uint8_t ch = *(buffer+i);
                    [rawData appendFormat:@"%c",hexCode[(ch & 0xf0) >> 4]];
                    [rawData appendFormat:@"%c",hexCode[(ch & 0x0f)]];
                }
                [ret setValue:rawData forKey:@"stream"];
            }
        }
        case FSPDFObjectDictionary:
        {
            if (!ret) ret = @{}.mutableCopy;
            if (!dict) dict = root.getDict;
            
            FSPDFObject *type = [dict getElement:@"Type"];
            if (type && [[type getWideString] isEqualToString:@"OCG"]) {
                return @{};
            }
            long pos = [dict moveNext:0];
            while (pos) {
                NSString *key2 = [dict getKey:pos];
                
                FSPDFObject *obj = [dict getElement:key2];
                ret[key2] = [self retriveDict:json key:key2 root:obj];
                pos = [dict moveNext:pos];
                
            }
            return ret;
        }
        case FSPDFObjectNull:
        {
            return [NSNull null];
        }
        case FSPDFObjectReference:
        {
            ret = [NSMutableString string];
            unsigned int objNum = [root getDirectObject].getObjNum;
            [ret appendFormat:@"%dR",objNum];
            
            json[@(objNum).stringValue] = [self retriveDict:json key:key root:[root getDirectObject]];
            
            return ret;
        }
        default:
            break;
    }
    return ret;
}

+ (void)exportAnnotDataToJson:(FSPDFDoc *)doc toPath:(NSString *)path{
    
    NSMutableArray *exportAnnots = @[].mutableCopy;
    for (int i = 0; i < [doc getPageCount]; i++) {
        FSPDFPage *page = [doc getPage:i];
        for (int j = 0 ; j < [page getAnnotCount]; j++) {
            FSAnnot *annot = [page getAnnot:j];
            [exportAnnots addObject:[self getAnnotDataWithAnnot:annot]];
        }
    }
    NSData *jsonData = [NSJSONSerialization dataWithJSONObject:exportAnnots options:NSJSONWritingPrettyPrinted
                                                         error:nil];
    if (jsonData) {
        NSString *jsonStr = [[NSString alloc] initWithData:jsonData encoding:NSUTF8StringEncoding];
        jsonStr = [jsonStr stringByReplacingOccurrencesOfString:@"\\/" withString:@"/"];
        jsonData = [jsonStr dataUsingEncoding:NSUTF8StringEncoding];
        [jsonData writeToFile:path atomically:YES];
    }
}

+ (NSMutableDictionary *)annotExportToJSON:(FSAnnot *)annot{
    NSMutableDictionary *annotDataDic = @{}.mutableCopy;
    FSAnnotType annotType = [annot getType];
    [annotDataDic setValue:[[self annotFlagsEnumToString:[annot getFlags]] lowercaseString] forKey:@"flags"];
    [annotDataDic setValue:[self convertFSDateTime2DataString:[annot getModifiedDateTime]] forKey:@"date"];
    [annotDataDic setValue:[[self annotTypeEnumToString:annotType] lowercaseString] forKey:@"type"];
    [annotDataDic setValue:@(annot.getPage.getIndex) forKey:@"page"];
    [annotDataDic setValue:[NSString stringWithFormat:@"%f,%f,%f,%f",annot.getRect.left,annot.getRect.bottom,annot.getRect.right,annot.getRect.top] forKey:@"rect"];
    [annotDataDic setValue:[annot getUniqueID] forKey:@"name"];
    if (annotType == FSAnnotPopup) {
        FSPopup *popup = [[FSPopup alloc] initWithAnnot:annot];
        [annotDataDic setValue:[popup getOpenStatus] ? @"yes" : @"no" forKey:@"open"];
        return annotDataDic;
    }
    
    [annotDataDic setValue:[self getColorHexByDecimal:annot.borderColor&0x00ffffff] forKey:@"color"];
    [annotDataDic setValue:[NSString stringWithFormat:@"%@",annot.content] forKey:@"contents"];
    [annotDataDic setValue:@(annot.borderInfo.width) forKey:@"width"];
    
    [annotDataDic setValue:@{} forKey:@"customEntries"];
    if ([annot.getDict hasKey:@"customEntries"]) {
        NSString *jsonStr = [[annot.getDict getElement:@"customEntries"] getWideString];
        
        NSData *jsonData = [jsonStr dataUsingEncoding:NSUTF8StringEncoding];
        NSError *err;
        NSDictionary *customEntries = [NSJSONSerialization JSONObjectWithData:jsonData
                                                                options:NSJSONReadingMutableContainers
                                                                  error:&err];
        if (!err) {
            [annotDataDic setValue:customEntries forKey:@"customEntries"];
        }
    }
    [annotDataDic setValue:[[self annotBorderStyle:[annot.borderInfo getStyle]] lowercaseString] forKey:@"style"];
    
    if ([annot.borderInfo getStyle] == FSBorderInfoDashed) {
        [annotDataDic setValue:@(annot.borderInfo.dash_phase) forKey:@"dashPhase"];
        NSMutableArray *tmp = @[].mutableCopy;
        for (int i = 0; i < [annot.borderInfo.getDashes getSize]; i++) {
            CGFloat dashes = [annot.borderInfo.getDashes getAt:i];
            [tmp addObject:@(dashes)];
        }
        [annotDataDic setValue:[tmp componentsJoinedByString:@","] forKey:@"dashes"];
    }else if ([annot.borderInfo getStyle] == FSBorderInfoCloudy) {
        [annotDataDic setValue:@(annot.borderInfo.cloud_intensity) forKey:@"intensity"];
    }
    return annotDataDic;
}

+ (NSMutableDictionary *)markupAnnotExportToJSON:(FSAnnot *)annot{
    NSMutableDictionary *annotDataDic = @{}.mutableCopy;
    FSAnnotType annotType = [annot getType];
    if ([annot isMarkup]) {
        FSMarkup *markupAnnot = [[FSMarkup alloc] initWithAnnot:annot];
        FSPDFDictionary *dict = [markupAnnot getDict];
        FSPDFObject *RTObj = [dict getElement:@"RI"];
        
        NSString *RT;
        int IRT = 0;
        FSPDFObject *IRTDict = [dict getElement:@"IRT"];
        if (IRTDict) {
            IRTDict = [IRTDict getDict];
            IRT = IRTDict.getObjNum;
        }
        
        if ([dict hasKey:@"StateModel"] && RTObj) {
            RT = [RTObj getName];
            RT = RT ? RT:@"R";
        }else{
            RT = @"R";
        }
        NSString *replyType = RT;
        if (replyType.length && IRT) {
            NSString *inreplyto = [[[[markupAnnot.getPage.getDocument getIndirectObject:IRT] getDict] getElement:@"NM"] getWideString];
            [annotDataDic setValue:inreplyto forKey:@"inreplyto"];
            [annotDataDic setValue:[replyType lowercaseString] forKey:@"replyType"];
        }
        
        if (markupAnnot.intent.length) {
            [annotDataDic setValue:markupAnnot.intent forKey:@"IT"];
            [annotDataDic setValue:markupAnnot.intent forKey:@"intent"];
        }
        [annotDataDic setValue:@(markupAnnot.opacity) forKey:@"opacity"];
        [annotDataDic setValue:markupAnnot.subject forKey:@"subject"];
        [annotDataDic setValue:markupAnnot.title forKey:@"title"];
        [annotDataDic setValue:[self convertFSDateTime2DataString:[markupAnnot getCreationDateTime]] forKey:@"creationdate"];
        
        if (annotType == FSAnnotFreeText || annotType == FSAnnotStamp || annotType == FSAnnotScreen) {
            FSRotation rotation;
            if (annotType == FSAnnotFreeText) {
                FSFreeText *freeText = [[FSFreeText alloc] initWithAnnot:annot];
                rotation = freeText.rotation;
            }else if (annotType == FSAnnotStamp) {
                FSStamp *stamp = [[FSStamp alloc] initWithAnnot:annot];
                rotation = stamp.rotation;
            }else {
                FSScreen *screen = [[FSScreen alloc] initWithAnnot:annot];
                rotation = screen.rotation;
            }
            [annotDataDic setValue:@(360 - rotation * 90) forKey:@"rotation"];
        }
        if (annotType == FSAnnotLine || annotType == FSAnnotPolygon || annotType == FSAnnotPolyLine || annotType == FSAnnotSquare || annotType == FSAnnotCircle) {
            
            NSString *measureRatio;
            float measureConversionFactor;
            NSString *measureUnit;
            if (annotType == FSAnnotLine) {
                FSLine *line = [[FSLine alloc] initWithAnnot:annot];
                measureRatio = line.measureRatio;
                measureConversionFactor = [line getMeasureConversionFactor:FSMarkupMeasureTypeX];
                measureUnit = [line getMeasureUnit:FSMarkupMeasureTypeX];
            }else if (annotType == FSAnnotPolygon) {
                FSPolygon *polygon = [[FSPolygon alloc] initWithAnnot:annot];
                measureRatio = polygon.measureRatio;
                measureConversionFactor = [polygon getMeasureConversionFactor:FSMarkupMeasureTypeX];
                measureUnit = [polygon getMeasureUnit:FSMarkupMeasureTypeX];
            }else if (annotType == FSAnnotPolyLine) {
                FSPolyLine *polyline = [[FSPolyLine alloc] initWithAnnot:annot];
                measureRatio = polyline.measureRatio;
                measureConversionFactor = [polyline getMeasureConversionFactor:FSMarkupMeasureTypeX];
                measureUnit = [polyline getMeasureUnit:FSMarkupMeasureTypeX];
            }else if (annotType == FSAnnotSquare) {
                FSSquare *square = [[FSSquare alloc] initWithAnnot:annot];
                measureRatio = square.measureRatio;
                measureConversionFactor = [square getMeasureConversionFactor:FSMarkupMeasureTypeX];
                measureUnit = [square getMeasureUnit:FSMarkupMeasureTypeX];
            }else{
                FSCircle *circle = [[FSCircle alloc] initWithAnnot:annot];
                measureRatio = circle.measureRatio;
                measureConversionFactor = [circle getMeasureConversionFactor:FSMarkupMeasureTypeX];
                measureUnit = [circle getMeasureUnit:FSMarkupMeasureTypeX];
            }
            if (measureRatio.length) {
                [annotDataDic setValue:measureRatio forKey:@"measureRatio"];
                [annotDataDic setValue:@(measureConversionFactor) forKey:@"measureConversionFactor"];
                [annotDataDic setValue:measureUnit forKey:@"measureUnit"];
            }
        }

        if (![markupAnnot.popup isEmpty]) {
            [annotDataDic setValue:[self getAnnotDataWithAnnot:markupAnnot.popup] forKey:@"popup"];
        }
    }
    return annotDataDic;
}

+ (NSMutableDictionary *)caretAnnotExportToJSON:(FSAnnot *)annot{
    NSMutableDictionary *annotDataDic = @{}.mutableCopy;
    FSCaret *caret = [[FSCaret alloc] initWithAnnot:annot];
    NSString *fringe = [NSString stringWithFormat:@"%f,%f,%f,%f",caret.innerRect.left - annot.getRect.left, caret.innerRect.bottom - annot.getRect.bottom, annot.getRect.right - caret.innerRect.right,annot.getRect.top - caret.innerRect.top];
    [annotDataDic setValue:fringe forKey:@"fringe"];
    [annotDataDic removeObjectForKey:@"style"];
    [annotDataDic removeObjectForKey:@"width"];
    [annotDataDic removeObjectForKey:@"dashPhase"];
    [annotDataDic removeObjectForKey:@"intensity"];
    return annotDataDic;
}

+ (NSMutableDictionary *)circleAnnotExportToJSON:(FSAnnot *)annot{
    NSMutableDictionary *annotDataDic = @{}.mutableCopy;
    FSCircle *circle = [[FSCircle alloc] initWithAnnot:annot];
    NSString *fringe = [NSString stringWithFormat:@"%f,%f,%f,%f",circle.innerRect.left - annot.getRect.left, circle.innerRect.bottom - annot.getRect.bottom, annot.getRect.right - circle.innerRect.right,annot.getRect.top - circle.innerRect.top];
    [annotDataDic setValue:fringe forKey:@"fringe"];
    if (circle.fillColor) {
        NSString *color = [self getColorHexByDecimal:circle.fillColor&0x00ffffff];
        [annotDataDic setValue:color forKey:@"interior-color"];
        [annotDataDic setValue:color forKey:@"interiorColor"];
    }
    if ([circle.intent isEqualToString:@"CircleDimension"]) {
        [annotDataDic setValue:circle.measureRatio forKey:@"measureRatio"];
        [annotDataDic setValue:@([circle getMeasureConversionFactor:FSMarkupMeasureTypeX]) forKey:@"measureConversionFactor"];
        [annotDataDic setValue:[circle getMeasureUnit:FSMarkupMeasureTypeX] forKey:@"measureUnit"];
    }
    return annotDataDic;
}

+ (NSMutableDictionary *)freeTextAnnotExportToJSON:(FSAnnot *)annot{
    NSMutableDictionary *annotDataDic = @{}.mutableCopy;
    FSFreeText *freeText = [[FSFreeText alloc] initWithAnnot:annot];
    FSAlignment alignment = freeText.alignment;
    if (!((freeText.intent.length == 0) || [freeText.intent isEqualToString:@"FreeTextCallout"])) {
        alignment = FSAlignmentLeft;
        freeText.alignment = FSAlignmentLeft;
    }
    [annotDataDic setValue:@(alignment) forKey:@"justification"];
    
    NSString *DA = @"";
    if ([freeText getDefaultAppearance].font.getName.length) {
        DA = [NSString stringWithFormat:@"/%@ ", [freeText getDefaultAppearance].font.getName];
    }
    if ([freeText getDefaultAppearance].text_size) {
        DA = [NSString stringWithFormat:@"%@%f Tf ",DA, [freeText getDefaultAppearance].text_size];
    }
    
    DA = [NSString stringWithFormat:@"%@%.2f %.2f %.2f rg",DA, ((annot.getBorderColor & 0xFF0000) >> 16) / 255.0f, ((annot.getBorderColor & 0xFF00) >> 16) / 255.0f,((annot.getBorderColor & 0xFF) >> 16) / 255.0f];
    
    [annotDataDic setValue:[self getColorHexByDecimal:freeText.getDefaultAppearance.getText_color&0x00ffffff] forKey:@"fontColor"];
    [annotDataDic setValue:DA forKey:@"defaultappearance"];
   
    if (freeText.fillColor) {
        NSString *color = [self getColorHexByDecimal:freeText.fillColor&0x00ffffff];
        [annotDataDic setValue:color forKey:@"interior-color"];
        [annotDataDic setValue:color forKey:@"interiorColor"];
    }
    
    if ([freeText.intent isEqualToString:@"FreeTextCallout"]) {
        NSString *fringe = [NSString stringWithFormat:@"%f,%f,%f,%f",freeText.innerRect.left - annot.getRect.left, freeText.innerRect.bottom - annot.getRect.bottom, annot.getRect.right - freeText.innerRect.right,annot.getRect.top - freeText.innerRect.top];
        [annotDataDic setValue:fringe forKey:@"fringe"];
        
        NSMutableString *callout = [NSMutableString string];
        FSPointFArray *points = [freeText getCalloutLinePoints];
        [callout appendFormat:@"%f,%f,%f,%f,%f,%f",[points getAt:0].x,[points getAt:0].y,[points getAt:1].x,[points getAt:1].y,[points getAt:2].x,[points getAt:2].y];
        [annotDataDic setValue:callout forKey:@"callout"];
        
        [annotDataDic setValue:[self makeupEndingStyle:[freeText getCalloutLineEndingStyle]] forKey:@"head"];
    }
    return annotDataDic;
}

+ (NSMutableDictionary *)inkAnnotExportToJSON:(FSAnnot *)annot{
    NSMutableDictionary *annotDataDic = @{}.mutableCopy;
    FSInk *ink = [[FSInk alloc] initWithAnnot:annot];
    FSPath *inkListInfo = ink.inkList;
    NSMutableString *inklist = [NSMutableString stringWithString:@"["];
    for (int index = 0; index < inkListInfo.getPointCount; index++) {
        if (index!=0 && [inkListInfo getPointType:index] == 1){
            inklist = [NSMutableString stringWithString:[inklist substringToIndex:inklist.length-1]];
            [inklist appendString:@"],["];
        }
        [inklist appendFormat:@"%f,%f;",[inkListInfo getPoint:index].x, [inkListInfo getPoint:index].y];
    }
    [inklist replaceCharactersInRange:NSMakeRange(inklist.length-1, 1) withString:@"]"];
    [annotDataDic setValue:inklist forKey:@"inklist"];
    return annotDataDic;
}

+ (NSMutableDictionary *)lineAnnotExportToJSON:(FSAnnot *)annot{
    NSMutableDictionary *annotDataDic = @{}.mutableCopy;
    FSLine *line = [[FSLine alloc] initWithAnnot:annot];
    
    if (line.styleFillColor) {
        NSString *color = [self getColorHexByDecimal:line.styleFillColor&0x00ffffff];
        [annotDataDic setValue:color forKey:@"interior-color"];
        [annotDataDic setValue:color forKey:@"interiorColor"];
    }
    
    [annotDataDic setValue:[NSString stringWithFormat:@"%f,%f",[line getStartPoint].x, [line getStartPoint].y] forKey:@"start"];
    [annotDataDic setValue:[NSString stringWithFormat:@"%f,%f",[line getEndPoint].x,[line getEndPoint].y] forKey:@"end"];
    [annotDataDic setValue:[self makeupEndingStyle:[line getLineStartStyle]] forKey:@"head"];
    [annotDataDic setValue:[self makeupEndingStyle:[line getLineEndStyle]] forKey:@"tail"];
    [annotDataDic setValue:@(line.leaderLineLength) forKey:@"leaderLength"];
    [annotDataDic setValue:@(line.leaderLineExtensionLength) forKey:@"leaderExtend"];
    [annotDataDic setValue:[line hasCaption] ? @"yes" : @"no"  forKey:@"caption"];
    return annotDataDic;
}

+ (NSMutableDictionary *)noteAnnotExportToJSON:(FSAnnot *)annot{
    NSMutableDictionary *annotDataDic = @{}.mutableCopy;
    FSNote *note = [[FSNote alloc] initWithAnnot:annot];
    
    [annotDataDic setValue:note.iconName  forKey:@"icon"];
    if (note.isStateAnnot) {
        [annotDataDic setValue:note.getStateModel == 1 ? @"Marked" : @"Review"  forKey:@"statemodel"];
        [annotDataDic setValue:[self makeupState:note.state]  forKey:@"state"];
        [annotDataDic removeObjectForKey:@"content"];
        [annotDataDic removeObjectForKey:@"color"];
    }
    return annotDataDic;
}

+ (NSMutableDictionary *)polygonAnnotExportToJSON:(FSAnnot *)annot{
    NSMutableDictionary *annotDataDic = @{}.mutableCopy;
    FSPolygon *polygon = [[FSPolygon alloc] initWithAnnot:annot];
    
    NSMutableString *pointList = [NSMutableString string];
    for (int i = 0; i < [polygon.vertexes getSize]; i++) {
        [pointList appendFormat:@"%f,%f;",[polygon.vertexes getAt:i].x,[polygon.vertexes getAt:i].y];
    }
    pointList = [NSMutableString stringWithString:[pointList substringToIndex:pointList.length-1]];
    
    if (polygon.fillColor) {
        NSString *color = [self getColorHexByDecimal:polygon.fillColor&0x00ffffff];
        [annotDataDic setValue:color forKey:@"interior-color"];
        [annotDataDic setValue:color forKey:@"interiorColor"];
    }
    if ([polygon.intent isEqualToString:@"PolygonDimension"]) {
        [annotDataDic setValue:polygon.measureRatio forKey:@"measureRatio"];
        [annotDataDic setValue:@([polygon getMeasureConversionFactor:FSMarkupMeasureTypeX]) forKey:@"measureConversionFactor"];
        [annotDataDic setValue:[polygon getMeasureUnit:FSMarkupMeasureTypeX] forKey:@"measureUnit"];
    }
    [annotDataDic setValue:pointList forKey:@"vertices"];
    return annotDataDic;
}


+ (NSMutableDictionary *)polyLineAnnotExportToJSON:(FSAnnot *)annot{
    NSMutableDictionary *annotDataDic = @{}.mutableCopy;
    FSPolyLine *polyline = [[FSPolyLine alloc] initWithAnnot:annot];
    
    NSMutableString *pointList = [NSMutableString string];
    for (int i = 0; i < [polyline.vertexes getSize]; i++) {
        [pointList appendFormat:@"%f,%f;",[polyline.vertexes getAt:i].x,[polyline.vertexes getAt:i].y];
    }
    pointList = [NSMutableString stringWithString:[pointList substringToIndex:pointList.length-1]];
    
    if (polyline.styleFillColor) {
        NSString *color = [self getColorHexByDecimal:polyline.styleFillColor&0x00ffffff];
        [annotDataDic setValue:color forKey:@"interior-color"];
        [annotDataDic setValue:color forKey:@"interiorColor"];
    }
    [annotDataDic setValue:[self makeupEndingStyle:[polyline getLineStartStyle]] forKey:@"head"];
    [annotDataDic setValue:[self makeupEndingStyle:[polyline getLineEndStyle]] forKey:@"tail"];
    [annotDataDic setValue:pointList forKey:@"vertices"];
    return annotDataDic;
}

+ (NSMutableDictionary *)redactAnnotExportToJSON:(FSAnnot *)annot{
    NSMutableDictionary *annotDataDic = @{}.mutableCopy;
    FSRedact *redact = [[FSRedact alloc] initWithAnnot:annot];
    
    if (redact.applyFillColor) {
        NSString *color = [self getColorHexByDecimal:redact.applyFillColor&0x00ffffff];
        [annotDataDic setValue:color forKey:@"interior-color"];
        [annotDataDic setValue:color forKey:@"interiorColor"];
    };
    
    NSMutableString *jsonQP = [NSMutableString string];
    for (int i = 0; i < [redact.quadPoints getSize]; i++) {
        FSQuadPoints *quadPoint = [redact.quadPoints getAt:i];
        [jsonQP appendFormat:@"%f,%f,",[quadPoint getFirst].x,[quadPoint getFirst].y];
        [jsonQP appendFormat:@"%f,%f,",[quadPoint getSecond].x,[quadPoint getSecond].y];
        [jsonQP appendFormat:@"%f,%f,",[quadPoint getThird].x,[quadPoint getThird].y];
        [jsonQP appendFormat:@"%f,%f,",[quadPoint getFourth].x,[quadPoint getFourth].y];
    }
    [annotDataDic setValue:[jsonQP substringToIndex:jsonQP.length-1] forKey:@"coords"];
    [annotDataDic setValue:redact.overlayText forKey:@"overlay-text"];
    [annotDataDic setValue:redact.overlayText forKey:@"overlayText"];
    
    NSString *DA = @"";
    if ([redact getDefaultAppearance].font.getName.length) {
        DA = [NSString stringWithFormat:@"/%@ ", [redact getDefaultAppearance].font.getName];
    }
    if ([redact getDefaultAppearance].text_size) {
        DA = [NSString stringWithFormat:@"%@%f Tf ",DA, [redact getDefaultAppearance].text_size];
    }
    
    DA = [NSString stringWithFormat:@"%@%.2f %.2f %.2f rg",DA, ((annot.getBorderColor & 0xFF0000) >> 16) / 255.0f, ((annot.getBorderColor & 0xFF00) >> 16) / 255.0f,((annot.getBorderColor & 0xFF) >> 16) / 255.0f];
    [annotDataDic setValue:DA forKey:@"defaultappearance"];
    [annotDataDic setValue:[self getColorHexByDecimal:[redact getDefaultAppearance].text_color&0x00ffffff] forKey:@"fontColor"];
    [annotDataDic setValue:@(redact.overlayTextAlignment) forKey:@"justification"];
    return annotDataDic;
}

+ (NSMutableDictionary *)soundAnnotExportToJSON:(FSAnnot *)annot{
    NSMutableDictionary *annotDataDic = @{}.mutableCopy;
    FSSound *sound = [[FSSound alloc] initWithAnnot:annot];
    [annotDataDic setValue:@(sound.getSamplingRate) forKey:@"rate"];
    NSMutableDictionary *rawDict = @{}.mutableCopy;
    FSPDFDictionary *dict = sound.getDict;
    FSPDFObject *sd = [dict getElement:@"Sound"];
    if (sd) {
        [rawDict setValue:[self retriveDict:rawDict key:@"" root:[sd getDirectObject]] forKey:@"Sound"];
        [annotDataDic setValue:rawDict forKey:@"raw"];
    }
    return annotDataDic;
}

+ (NSMutableDictionary *)squareAnnotExportToJSON:(FSAnnot *)annot{
    NSMutableDictionary *annotDataDic = @{}.mutableCopy;
    FSSquare *square = [[FSSquare alloc] initWithAnnot:annot];
    NSString *fringe = [NSString stringWithFormat:@"%f,%f,%f,%f",square.innerRect.left - annot.getRect.left, square.innerRect.bottom - annot.getRect.bottom, annot.getRect.right - square.innerRect.right,annot.getRect.top - square.innerRect.top];
    [annotDataDic setValue:fringe forKey:@"fringe"];
    if (square.fillColor) {
        NSString *color = [self getColorHexByDecimal:square.fillColor&0x00ffffff];
        [annotDataDic setValue:color forKey:@"interior-color"];
        [annotDataDic setValue:color forKey:@"interiorColor"];
    }
    if ([square.intent isEqualToString:@"CustomDimension"]) {
        [annotDataDic setValue:square.measureRatio forKey:@"measureRatio"];
        [annotDataDic setValue:@([square getMeasureConversionFactor:FSMarkupMeasureTypeX]) forKey:@"measureConversionFactor"];
        [annotDataDic setValue:[square getMeasureUnit:FSMarkupMeasureTypeX] forKey:@"measureUnit"];
    }
    return annotDataDic;
}

+ (NSMutableDictionary *)stampAnnotExportToJSON:(FSAnnot *)annot{
    NSMutableDictionary *annotDataDic = @{}.mutableCopy;
    FSStamp *stamp = [[FSStamp alloc] initWithAnnot:annot];
    [annotDataDic setValue:stamp.iconName  forKey:@"icon"];

    NSMutableDictionary *rawDict = @{}.mutableCopy;
    FSPDFDictionary *dict = stamp.getDict;
    FSPDFObject *ap = [dict getElement:@"AP"];
    if (ap) {
        [rawDict setValue: [self retriveDict:rawDict key:@"" root:[[[[ap getDirectObject] getDict] getElement:@"N"] getDirectObject]] forKey:@"N"];
        [annotDataDic setValue:rawDict forKey:@"raw"];
    }
    return annotDataDic;
}

+ (NSMutableDictionary *)textMarkupAnnotExportToJSON:(FSAnnot *)annot{
    NSMutableDictionary *annotDataDic = @{}.mutableCopy;
    FSTextMarkup *textMarkup = [[FSTextMarkup alloc] initWithAnnot:annot];
    
    NSMutableString *jsonQP = [NSMutableString string];
    for (int i = 0; i < [textMarkup.quadPoints getSize]; i++) {
        FSQuadPoints *quadPoint = [textMarkup.quadPoints getAt:i];
        [jsonQP appendFormat:@"%f,%f,",[quadPoint getFirst].x,[quadPoint getFirst].y];
        [jsonQP appendFormat:@"%f,%f,",[quadPoint getSecond].x,[quadPoint getSecond].y];
        [jsonQP appendFormat:@"%f,%f,",[quadPoint getThird].x,[quadPoint getThird].y];
        [jsonQP appendFormat:@"%f,%f,",[quadPoint getFourth].x,[quadPoint getFourth].y];
    }
    [annotDataDic setValue:[jsonQP substringToIndex:jsonQP.length-1] forKey:@"coords"];
    return annotDataDic;
}

+ (NSMutableDictionary *)fileAttachmentAnnotExportToJSON:(FSAnnot *)annot{
    NSMutableDictionary *annotDataDic = @{}.mutableCopy;
    FSFileAttachment *fileAttachment = [[FSFileAttachment alloc] initWithAnnot:annot];
    
    [annotDataDic setValue:fileAttachment.iconName  forKey:@"icon"];
    NSMutableDictionary *rawDict = @{}.mutableCopy;
    FSPDFDictionary *dict = annot.getDict;
    FSPDFObject *fs = [dict getElement:@"FS"];
    if (fs) {
        [rawDict setValue:[self retriveDict:rawDict key:@"" root:[fs getDirectObject]] forKey:@"FS"];
        [annotDataDic setValue:rawDict forKey:@"raw"];
    }
    return annotDataDic;
}

+ (NSMutableDictionary *)getAnnotDataWithAnnot:(FSAnnot *)annot{
    NSMutableDictionary *annotDataDic = @{}.mutableCopy;
    FSAnnotType annotType = [annot getType];
    [annotDataDic addEntriesFromDictionary:[self annotExportToJSON:annot]];
    [annotDataDic addEntriesFromDictionary:[self markupAnnotExportToJSON:annot]];
    
    if (annotType == FSAnnotCaret) {
        [annotDataDic addEntriesFromDictionary:[self caretAnnotExportToJSON:annot]];
    }else if (annotType == FSAnnotCircle) {
        [annotDataDic addEntriesFromDictionary:[self circleAnnotExportToJSON:annot]];
    }else if (annotType == FSAnnotFreeText) {
        [annotDataDic addEntriesFromDictionary:[self freeTextAnnotExportToJSON:annot]];
    }else if (annotType == FSAnnotInk) {
        [annotDataDic addEntriesFromDictionary:[self inkAnnotExportToJSON:annot]];
    }else if (annotType == FSAnnotLine) {
        [annotDataDic addEntriesFromDictionary:[self lineAnnotExportToJSON:annot]];
    }else if (annotType == FSAnnotNote) {
        [annotDataDic addEntriesFromDictionary:[self noteAnnotExportToJSON:annot]];
    }else if (annotType == FSAnnotPolygon) {
        [annotDataDic addEntriesFromDictionary:[self polygonAnnotExportToJSON:annot]];
    }else if (annotType == FSAnnotPolyLine) {
        [annotDataDic addEntriesFromDictionary:[self polyLineAnnotExportToJSON:annot]];
    }else if (annotType == FSAnnotRedact) {
        [annotDataDic addEntriesFromDictionary:[self redactAnnotExportToJSON:annot]];
    }else if (annotType == FSAnnotSound) {
        [annotDataDic addEntriesFromDictionary:[self soundAnnotExportToJSON:annot]];
    }else if (annotType == FSAnnotSquare) {
        [annotDataDic addEntriesFromDictionary:[self squareAnnotExportToJSON:annot]];
    }else if (annotType == FSAnnotStamp) {
        [annotDataDic addEntriesFromDictionary:[self stampAnnotExportToJSON:annot]];
    }else if (annotType == FSAnnotFileAttachment){
        [annotDataDic addEntriesFromDictionary:[self fileAttachmentAnnotExportToJSON:annot]];
    }else if (annotType == FSAnnotHighlight || annotType == FSAnnotUnderline || annotType == FSAnnotStrikeOut || annotType == FSAnnotSquiggly) {
        [annotDataDic addEntriesFromDictionary:[self textMarkupAnnotExportToJSON:annot]];
    }
    
    return annotDataDic;
}
@end
