/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import <Foundation/Foundation.h>
#import <FoxitRDK/FSPDFObjC.h>

@interface PlatformAPIDemo : NSObject

@property (nonatomic, strong) FSPDFDoc *doc;

/** @brief	Load document from path by using the specified password. */
+ (instancetype)loadDocFromPath:(NSString *)path withPasswd:(NSString *)passwd;
/** @brief	Extract text from a PDF document to a TXT file. */
- (BOOL)extractTextOfDoc:(FSPDFDoc *)doc toPath:(NSString *)path;
/** @brief	Edit bookmark appearances and titles, and save these changes to a PDF file. */
- (BOOL)modifyBookmarkOfDoc:(FSPDFDoc *)doc saveAsNewDocAtPath:(NSString *)path;
/** @brief	Add annotations, and save these changes to a PDF file. */
- (BOOL)addAnnotToPage:(FSPDFPage *)page saveAsNewDocAtPath:(NSString *)path;
/** @brief	Export document info to a TXT file. */
- (BOOL)exportDocInfoOfDoc:(FSPDFDoc *)doc toPath:(NSString *)path;
/** @brief	Render a specified page to Bitmap, and save it to a PNG image. */
- (BOOL)renderPage:(FSPDFPage *)page atPath:(NSString *)path;
/** @briefAdd signature to a PDF document, and verify it.*/
- (BOOL)addSignatureToPage:(FSPDFPage *)page saveAsNewDocAtPath:(NSString *)path;
/** @briefAdd watermark to a PDF document, and verify it.*/
- (BOOL)addWatermark:(NSString *)input_file saveAsNewDocAtPath:(NSString *)path;
/** @briefSearch character*/
- (BOOL)searchWithSaveResultsPath:(NSString *)path;
/** @brief*/
- (BOOL)exampleGraphicsObjectsInputPath:(NSString *)inputPath saveAsNewDocAtPath:(NSString *)path;
/** @briefAdd signature to a PDF document, and verify it.*/
+ (BOOL)image2pdf:(NSString *)input_file saveAsNewDocAtPath:(NSString *)path;
@end
