/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */


#import "PlatformAPIDemoCallBack.h"

@implementation MyIconProvider
-(id)init {
    use_dynamic_stamp_ = NO;
    icon_file_folder_ = @"";
    return self;
}

-(id)initWithPath:(NSString*)icon_file_folder{
    icon_file_folder_ = icon_file_folder;
    use_dynamic_stamp_ = NO;
    return self;
}

// If one icon provider offers different style icon for one icon name of a kind of annotaion,
// please use different provider ID or version in order to distinguish different style for Foxit PDF SDK.
// Otherwise, only the first style icon for the same icon name of same kind of annotation will have effect.
-(NSString*)getProviderID {
    if (use_dynamic_stamp_)
        return @"Simple Demo Dynamic MyIconProvider";
    else
        return @"Simple Demo MyIconProvider";
}

-(NSString*)getProviderVersion {
    return @"1.0.0";
}

-(BOOL)hasIcon:(FSAnnotType)annot_type iconName:(NSString *)icon_name {
    if (annot_type != FSAnnotStamp) return NO;
    NSString* icon_file_path = @"";
    if (use_dynamic_stamp_)
        icon_file_path = [icon_file_folder_ stringByAppendingPathComponent:@"DynamicStamps"];
    else
        icon_file_path = [icon_file_folder_ stringByAppendingPathComponent:@"StaticStamps"];
    icon_file_path = [icon_file_path stringByAppendingPathComponent:icon_name];
    icon_file_path = [icon_file_path stringByAppendingString:@".pdf"];
    FSPDFDoc* doc = [[FSPDFDoc alloc] initWithPath:icon_file_path];
    FSErrorCode error_code = [doc load:nil];
    if (error_code != FSErrSuccess) {
        NSLog(@"Fail to load icon file:%@, with error value: %ld", icon_file_path, error_code);
        return NO;
    }
    return ![doc isEmpty];
}

-(BOOL)canChangeColor:(FSAnnotType)annot_type iconName:(NSString *)icon_name {
    return NO;
}

-(FSPDFPage*)getIcon:(FSAnnotType)annot_type iconName:(NSString *)iconName color:(unsigned int)color {
    NSString* icon_file_path = @"";
    if (use_dynamic_stamp_)
        icon_file_path = [icon_file_folder_ stringByAppendingPathComponent:@"DynamicStamps"];
    else
        icon_file_path = [icon_file_folder_ stringByAppendingPathComponent:@"StaticStamps"];
    icon_file_path = [icon_file_path stringByAppendingPathComponent:iconName];
    icon_file_path = [icon_file_path stringByAppendingString:@".pdf"];

    FSPDFDoc* doc = [[FSPDFDoc alloc] initWithPath:icon_file_path];
    FSErrorCode error_code = [doc load:nil];
    if (error_code != FSErrSuccess) {
        NSLog(@"Fail to load icon file:%@, with error value: %ld", icon_file_path, error_code);
        return nil;
    }
    return [doc getPage:0];
}

-(NSNumber*)getDisplayWidth:(FSAnnotType)annot_type iconName:(NSString *)icon_name {
    return [NSNumber numberWithFloat:0.0f];
}

-(NSNumber*)getDisplayHeight:(FSAnnotType)annot_type iconName:(NSString *)icon_name {
    return [NSNumber numberWithFloat:0.0f];
}

- (FSShadingColor *)getShadingColor:(FSAnnotType)annotType iconName:(NSString *)iconName refColor:(unsigned int)refColor shadingIndex:(int)shadingIndex ShadingColor:(FSShadingColor *)out_shading_color {
    return nil;
}


-(void)setUseDynamicStamp:(BOOL)use_dynamic_stamp {
    use_dynamic_stamp_ = use_dynamic_stamp;
}

@end

@implementation MyActionEventHandler

-(int)getCurrentPage:(FSPDFDoc *)document {
    return -1;
}

-(void)setCurrentPage:(FSPDFDoc *)document pageIndex:(int)page_index {}


-(BOOL)getDocChangeMark:(FSPDFDoc *)document {
    return NO;
}

-(int)alert:(NSString *)msg title:(NSString *)title type:(int)type icon:(int)icon {
    return 0;
}

-(FSIdentityProperties*)getIdentityProperties {
    FSIdentityProperties *identityProperties = [[FSIdentityProperties alloc] init];
    [identityProperties set:@"foxitsoftware" email:@"simple_demo@foxitsoftware.cn" login_name:@"simple demo" name:@"Simple" first_name:nil last_name:nil title:nil department:nil];
    return identityProperties;
}

- (BOOL)setPageRotation:(FSPDFDoc *)pdfDoc pageIndex:(int)pageIndex rotation:(FSRotation)rotation {
    return NO;
}

- (unsigned int)verifySignature:(FSPDFDoc *)document signature:(FSSignature *)signature {
    return FSSignatureStateVerifyErrorData;
}

- (FSRotation)getPageRotation:(FSPDFDoc *)pdfDoc pageIndex:(int)pageIndex {
    return FSRotation0;
}

- (BOOL)setDocChangeMark:(FSPDFDoc *)document change_mark:(BOOL)change_mark {
    return YES;
}

- (NSString *)browseFile:(BOOL)is_open_dialog file_format:(NSString *)file_format file_filter:(NSString *)file_filter {
    return @"";
}

- (NSString *)getAppInfo:(FSActionCallbackAppInfoType)type {
    return @"";
}

- (void)setCurrentPage:(FSPDFDoc *)pdfDoc destination:(FSDestination *)destination {
}

- (BOOL)submitForm:(FSPDFDoc *)document formData:(NSData *)formData url:(NSString *)url file_format_type:(FSFileFormatType)file_format_type {
    return NO;
}

- (BOOL)invalidateRect:(FSPDFDoc *)document page_index:(int)page_index pdf_rect:(FSRectF *)pdf_rect {
    return NO;
}
@end

