/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "ViewController.h"
#import "CustomCell.h"
#import "PlatformAPIDemo.h"
#import <FoxitRDK/FSPDFObjC.h>

static NSString *const kDescription = @"description";
static NSString *const kItem = @"item";
@interface ViewController ()

@property (nonatomic, strong) NSArray *listFunction;

@end

@implementation ViewController


- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view, typically from a nib.
    self.listFunction = [self dataSource];
    self.tableView.tableFooterView = nil;
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [self.listFunction count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *testID = @"testID";
    CustomCell *cell = [tableView dequeueReusableCellWithIdentifier:testID];

    NSUInteger row = [indexPath row];

    cell.funcDescription.text = [[self.listFunction objectAtIndex:row] objectForKey:kDescription];
    cell.runBtn.tag = row;
    [cell.runBtn setTitle:[[self.listFunction objectAtIndex:row] objectForKey:kItem] forState:UIControlStateNormal];
    cell.runBtn.titleLabel.numberOfLines = 0;
    cell.runBtn.titleLabel.textAlignment = NSTextAlignmentCenter;
    cell.accessoryType = UITableViewCellAccessoryNone;
    [cell.runBtn addTarget:self action:@selector(onTouchBtnInCell:) forControlEvents:(UIControlEventTouchUpInside)];

    return cell;
}

- (void)onTouchBtnInCell:(UIButton *)sender {
    NSUInteger row = sender.tag;
    NSString *msg = nil;
    
    NSString *docDir = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    NSString *outputPrefixPath = [docDir stringByAppendingPathComponent:sender.titleLabel.text];
    NSString *test_files = [[[NSBundle mainBundle] resourcePath] stringByAppendingPathComponent:@"test_files"];
    
    NSString *testFilePath = [test_files stringByAppendingPathComponent:@"FoxitBigPreview.pdf"];
    PlatformAPIDemo *demo = nil;
    
    NSFileManager *manager = [[NSFileManager alloc] init];
    [manager createDirectoryAtPath:[docDir stringByAppendingPathComponent:sender.titleLabel.text] withIntermediateDirectories:NO attributes:nil error:nil];
    
    BOOL isSuccess = NO;
    NSString *savedPath = nil;
    switch (row) {
        case 0: {
            demo = [PlatformAPIDemo loadDocFromPath:testFilePath withPasswd:nil];
            savedPath = [docDir stringByAppendingString:@"/pdf2txt/FoxitBigPreview.txt"];
            isSuccess = [demo extractTextOfDoc:demo.doc toPath:savedPath];
            break;
        }
            
        case 1: {
            testFilePath = [test_files stringByAppendingPathComponent:@"Outline.pdf"];
            demo = [PlatformAPIDemo loadDocFromPath:testFilePath withPasswd:nil];
            savedPath = [docDir stringByAppendingString:@"/outline/Outline_edit.pdf"];
            isSuccess = [demo modifyBookmarkOfDoc:demo.doc saveAsNewDocAtPath:savedPath];
            break;
        }
            
        case 2: {
            testFilePath = [test_files stringByAppendingPathComponent:@"annotation_input.pdf"];
            demo = [PlatformAPIDemo loadDocFromPath:testFilePath withPasswd:nil];
            int iPage = 0;
            FSPDFPage *pPage = [demo.doc getPage:iPage];
            savedPath = [docDir stringByAppendingString:@"/annotation/annotation.pdf"];
            isSuccess = [demo addAnnotToPage:pPage saveAsNewDocAtPath:savedPath];
            break;
        }
            
        case 3: {
            demo = [PlatformAPIDemo loadDocFromPath:testFilePath withPasswd:nil];
            savedPath = [docDir stringByAppendingString:@"/docinfo/FoxitBigPreview_docinfo.txt"];
            isSuccess = [demo exportDocInfoOfDoc:demo.doc toPath:savedPath];
            break;
        }
            
        case 4: {
            demo = [PlatformAPIDemo loadDocFromPath:testFilePath withPasswd:nil];
            int iPage = 0;
            FSPDFPage *pPage = [demo.doc getPage:iPage];
            savedPath = [docDir stringByAppendingString:@"/render/FoxitBigPreview.png"];
            isSuccess = [demo renderPage:pPage atPath:savedPath];
            break;
        }
            
        case 5: {
            testFilePath = [test_files stringByAppendingPathComponent:@"Sample.pdf"];
            demo = [PlatformAPIDemo loadDocFromPath:testFilePath withPasswd:nil];
            int iPage = 0;
            FSPDFPage *pPage = [demo.doc getPage:iPage];
            savedPath = [docDir stringByAppendingString:@"/signature/Sample_signature.pdf"];
            isSuccess = [demo addSignatureToPage:pPage saveAsNewDocAtPath:savedPath];
            break;
        }
            
        case 6: {
            savedPath = outputPrefixPath;
            isSuccess = [PlatformAPIDemo image2pdf:test_files saveAsNewDocAtPath:savedPath];
            break;
        }
            
        case 7: {
            demo = [PlatformAPIDemo loadDocFromPath:[test_files stringByAppendingPathComponent:@"AboutFoxit.pdf"] withPasswd:nil];
            savedPath = [outputPrefixPath stringByAppendingPathComponent:@"watermark_add.pdf"];
            isSuccess = [demo addWatermark:test_files saveAsNewDocAtPath:savedPath];
            break;
        }
        case 8: {
            demo = [PlatformAPIDemo loadDocFromPath:[test_files stringByAppendingPathComponent:@"AboutFoxit.pdf"] withPasswd:nil];
            savedPath = [outputPrefixPath stringByAppendingPathComponent:@"search.txt"];
            isSuccess = [demo searchWithSaveResultsPath:savedPath];
            break;
        }
            
        case 9: {
            demo = [PlatformAPIDemo loadDocFromPath:[test_files stringByAppendingPathComponent:@"graphics_objects.pdf"] withPasswd:nil];
            savedPath = [outputPrefixPath stringByAppendingPathComponent:@"graphics_objects.pdf"];
            isSuccess = [demo exampleGraphicsObjectsInputPath:test_files saveAsNewDocAtPath:savedPath];
            break;
        }
            
        default:
            isSuccess = YES;
            break;
    }
    NSLog(@"%@", savedPath);
    
    msg = [NSString stringWithFormat:@"The generated file was saved to %@", savedPath];
    NSString *title = isSuccess ? @"Successfully!" : @"Failed!";
    
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:title message:msg preferredStyle:UIAlertControllerStyleAlert];
    UIAlertAction *cancelAction = [UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:nil];
    [alertController addAction:cancelAction];
    [self presentViewController:alertController animated:YES completion:nil];
}

- (void)presentAlertController:(UIViewController *)alertControllerToPresent animated:(BOOL)flag completion:(void (^)(void))completion {
    [self presentViewController:alertControllerToPresent animated:flag completion:completion];
}

- (NSArray *)dataSource{
    return @[
        @{
            kDescription:@"Extract text from a PDF document to a TXT file.",
            kItem:@"pdf2txt",
        },
        @{
            kDescription:@"Edit outline (aka bookmark) appearances and titles.",
            kItem:@"outline",
        },
        @{
            kDescription:@"Add annotations and export annotations to a JSON file.",
            kItem:@"annotation",
        },
        @{
            kDescription:@"Export document info to a TXT file.",
            kItem:@"docinfo",
        },
        @{
            kDescription:@"Render a specified page to Bitmap.",
            kItem:@"render",
        },        @
        {
            kDescription:@"Add a signature to PDF, sign PDF and verify the signature.",
            kItem:@"signature",
        },
        @{
            kDescription:@"Convert images to PDF files.",
            kItem:@"image2pdf",
        },
        @{
            kDescription:@"Add text, image and PDF-page watermarks to PDF files.",
            kItem:@"watermark",
        },
        @{
            kDescription:@"Search PDF files.",
            kItem:@"search",
        },
        @{
            kDescription:@"Create a PDF Doc with graphics objects.",
            kItem:@"graphics_objects",
        },
    ];
}


@end
