/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

import UIKit

protocol TabDelegate {
    func closeTab(_ tab: Tab)
}

class Tab: UICollectionViewCell {
    
    let closeButton = UIButton(type: .custom)
    var label = UILabel()
    var delegate: TabDelegate?
    weak var info: TabInfo? {
        didSet {
            if let path = info?.path {
                let fileName = URL(fileURLWithPath: path).lastPathComponent
                label.text = fileName
            }
        }
    }
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        
        self.selectedBackgroundView = UIView(frame: self.bounds)
        self.selectedBackgroundView!.backgroundColor = UIColor.white.withAlphaComponent(0.8)
        closeButton.setImage(UIImage(named: "closeTab"), for: .normal)
        closeButton.sizeToFit()
        closeButton.contentEdgeInsets = UIEdgeInsets(top: 10, left: 6, bottom: 10, right: 6)
        closeButton.addTarget(self, action: #selector(onClose(_:)), for: .touchUpInside)
        contentView.addSubview(closeButton)
        
        label.contentMode = .center
        label.textAlignment = .center
        contentView.addSubview(label)
        
        closeButton.translatesAutoresizingMaskIntoConstraints = false
        label.translatesAutoresizingMaskIntoConstraints = false
        let allViews = ["closeButton": closeButton, "label": label] as [String : Any]
        contentView.addConstraints(NSLayoutConstraint.constraints(withVisualFormat: "H:|-0-[closeButton]-0-[label]-0-|", options: [.alignAllCenterY], metrics: nil, views: allViews))
        contentView.addConstraints(NSLayoutConstraint.constraints(withVisualFormat: "V:|-0-[label]-0-|", options: [.alignAllCenterY], metrics: nil, views: allViews))
        #if swift(>=4.0)
            closeButton.setContentHuggingPriority(UILayoutPriority(rawValue: label.contentHuggingPriority(for: .vertical).rawValue + 1.0), for: .horizontal)
            closeButton.setContentCompressionResistancePriority(UILayoutPriority(rawValue: label.contentCompressionResistancePriority(for: .vertical).rawValue + 1.0), for: .horizontal)
        #else
            closeButton.setContentHuggingPriority(label.contentHuggingPriority(for: .horizontal) + 1.0, for: .horizontal)
            closeButton.setContentCompressionResistancePriority(label.contentCompressionResistancePriority(for: .horizontal) + 1.0, for: .horizontal)
        #endif

//        closeButton.setContentHuggingPriority(.defaultHigh, for: .horizontal)
//        closeButton.setContentCompressionResistancePriority(.defaultHigh, for: .horizontal)
//        label.setContentHuggingPriority(.defaultLow, for: .horizontal)
//        label.setContentCompressionResistancePriority(.defaultLow, for: .horizontal)
    }
    
    required init?(coder aDecoder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    @IBAction func onClose(_ button: UIButton) {
        self.delegate?.closeTab(self)
    }
}
