/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

import AVFoundation
import UIKit

class ViewController: UIViewController {
    @IBOutlet weak var openScanBtn: UIButton!
    var rootViewController: UINavigationController?
    lazy var fileList = FSFileListViewController.init()
    lazy var tabsViewController: TabsViewController = {
        let vc = TabsViewController()
        vc.delegate = self
        return vc
    }()
    var allowTabsButton: UIBarButtonItem? = nil
    var isOpeningFile = false
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        let DOCUMENT_PATH = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true)[0]
        print(DOCUMENT_PATH)
        copyToDocument(PDFs: ["Sample", "complete_pdf_viewer_guide_ios"], overwrite: false)
        
        fileList.delegate = self
        allowTabsButton = UIBarButtonItem(image: UIImage(named: "disallowTabs"), style: .plain, target: self, action: #selector(onClickAllowTabs(_:)))
        allowTabsButton?.accessibilityLabel = self.localizedString(translation_key: "kSingleModeReaderVoiceover")
        allowTabsButton?.tintColor = UIColor.white
        var items = fileList.topToolbar.items ?? []
        items.append(allowTabsButton!)
        fileList.topToolbar.items = items
        
        rootViewController = UINavigationController(rootViewController: fileList)
        rootViewController!.isNavigationBarHidden = true
        rootViewController!.delegate = self
        addChildViewController(rootViewController!)
        view.addSubview(rootViewController!.view)
        rootViewController!.view.translatesAutoresizingMaskIntoConstraints = false
        
        let contraints = NSMutableArray.init();
        contraints.add(view.leftAnchor .constraint(equalTo: rootViewController!.view.leftAnchor))
        contraints.add(view.rightAnchor .constraint(equalTo: rootViewController!.view.rightAnchor))
        contraints.add(view.topAnchor .constraint(equalTo: rootViewController!.view.topAnchor))
        contraints.add(view.bottomAnchor .constraint(equalTo: rootViewController!.view.bottomAnchor))
        view.addConstraints(contraints as! [NSLayoutConstraint]);
        
        self.openScanBtn.isHidden = true;
#if !targetEnvironment(macCatalyst)
        self.openScanBtn.isHidden = false;
        self.openScanBtn.accessibilityLabel = self.localizedString(translation_key: "kScanVoiceover")
        fileList.view.addSubview(self.openScanBtn);
        
        self.openScanBtn.rightAnchor.constraint(equalTo: fileList.view.rightAnchor, constant: -15).isActive = true;
        if #available(iOS 11.0, *) {
            self.openScanBtn.bottomAnchor.constraint(equalTo: fileList.view.safeAreaLayoutGuide.bottomAnchor, constant: -15).isActive = true
        } else {
            self.openScanBtn.bottomAnchor.constraint(equalTo: fileList.view.bottomAnchor, constant: -15).isActive = true;
        };
        
        let image = UIImage(named: "scan")
        self.openScanBtn.setBackgroundImage(image?.withRenderingMode(UIImageRenderingMode.alwaysTemplate), for: UIControlState.normal)
        self.openScanBtn.tintColor = UIExtensionsManager.primaryColor;
        
        PDFScanManager.initializeScanner(0, serial2: 0);
        PDFScanManager.initializeCompression(0, serial2: 0);
#endif
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    
    @IBAction func onClickAllowTabs(_ item: UIBarButtonItem) {
        let message = self.localizedString(translation_key: "kWantToSwitchTo").appending((tabsViewController.allowTabs ? self.localizedString(translation_key: "kSingle") : self.localizedString(translation_key: "kTabs"))).appending(self.localizedString(translation_key: "kReaderMode"))
        let alertController = UIAlertController(title: self.localizedString(translation_key: "kSwitchReaderMode"), message: message, preferredStyle: .alert)
        let cancelAction = UIAlertAction(title: self.localizedString(translation_key: "kCancel"), style: .cancel)
        let okAction = UIAlertAction(title: self.localizedString(translation_key: "kOK"), style: .default) { [unowned self] (_) in
            self.switchAllowTabs()
        }
        alertController.addAction(cancelAction)
        alertController.addAction(okAction)
        present(alertController, animated: true, completion: nil)
    }
    
#if !targetEnvironment(macCatalyst)
    func showAlertWithOpenScanError()
    {
        let message = self.localizedString(translation_key: "kCAMERADenied")
        let alertController = UIAlertController(title: self.localizedString(translation_key: "kNotAllowTakePhoto"), message: message, preferredStyle: .alert)
        let cancelAction = UIAlertAction(title: self.localizedString(translation_key: "kCancel"), style: .cancel)
        let okAction = UIAlertAction(title: self.localizedString(translation_key: "kGoSetting"), style: .default) { [unowned self] (_) in
            let url = NSURL.init(fileURLWithPath:UIApplicationOpenSettingsURLString);

            if (UIApplication.shared.canOpenURL( url as URL))
            {
                UIApplication.shared.openURL( url as URL)
            }
                
           
        }
        alertController.addAction(cancelAction)
        alertController.addAction(okAction)
        present(alertController, animated: true, completion: nil)
    
    }

    func showPDFScanViewController()
    {
        let VC =  PDFScanManager.share().getPDFScanView();
        self.present(VC, animated: true, completion: nil);
        
        PDFScanManager.saveAsCallBack = { (error, savePath) -> () in
            if ((savePath) != nil) {
                if ((VC.presentingViewController) != nil) {
                    VC.presentingViewController?.dismiss(animated: false, completion: nil)
                }
                VC.dismiss(animated: false, completion: nil)

                    self.didFileSelected(savePath!)
                }
        }
    }
#endif
    @IBAction func openScan(_ sender: UIButton) {
#if !targetEnvironment(macCatalyst)

let mediaType = AVMediaType.video;
let authStatus = AVCaptureDevice .authorizationStatus(for: mediaType);
if (authStatus == AVAuthorizationStatus.restricted || authStatus == AVAuthorizationStatus.denied)
    {
        self.showAlertWithOpenScanError();
        
}else if (authStatus == AVAuthorizationStatus.notDetermined){
    AVCaptureDevice.requestAccess(for: AVMediaType.video) { (Bool) in
        DispatchQueue.main.async {
            if (Bool) {
                self.showPDFScanViewController();
            }else{
                self.showAlertWithOpenScanError();
            }
        }
    }
}else{
    self.showPDFScanViewController();
}
#endif
    }
    
    //MARK: - rotate event
    override var shouldAutorotate: Bool {
        if let extensionsManager = tabsViewController.currentPDFView?.extensionsManager as? UIExtensionsManager {
            return !extensionsManager.isScreenLocked
        } else {
            return true
        }
    }
    
    override var supportedInterfaceOrientations: UIInterfaceOrientationMask {
        return .all
    }
    
    override func willRotate(to toInterfaceOrientation: UIInterfaceOrientation, duration: TimeInterval) {
        (tabsViewController.currentPDFView?.extensionsManager as? UIExtensionsManager)?.willRotate(to: toInterfaceOrientation, duration: duration)
    }
    
    override func willAnimateRotation(to toInterfaceOrientation: UIInterfaceOrientation, duration: TimeInterval) {
        (tabsViewController.currentPDFView?.extensionsManager as? UIExtensionsManager)?.willAnimateRotation(to: toInterfaceOrientation, duration: duration)
    }
    
    override func didRotate(from fromInterfaceOrientation: UIInterfaceOrientation) {
        (tabsViewController.currentPDFView?.extensionsManager as? UIExtensionsManager)?.didRotate(from: fromInterfaceOrientation)
    }
    
    override var prefersStatusBarHidden: Bool{
        if let extensionsManager = tabsViewController.currentPDFView?.extensionsManager as? UIExtensionsManager {
            return extensionsManager.prefersStatusBarHidden
        } else {
            return false
        }
    }
}

extension ViewController : FSFileSelectDelegate {
    
    func didFileSelected(_ filePath: String) {
        if isOpeningFile {
            return
        }
        isOpeningFile = true
        tabsViewController.openPDF(atPath: filePath) { [weak self] (errorCode) in
            if errorCode == .errSuccess {
                self?.pushViewController(self!.tabsViewController, transitionFromLeft: false)
            } else if errorCode == .errDeviceLimitation {
                self?._alertWithTitle(title: self?.localizedString(translation_key: "kError"), message:self?.localizedString(translation_key: "kDeviceLimitatio"))
            } else if errorCode == .errCanNotGetUserToken {
                self?._alertWithTitle(title: self?.localizedString(translation_key: "kError"), message: self?.localizedString(translation_key: "kCanNotGetUserToken"))
            } else if errorCode == .errInvalidLicense {
                self?._alertWithTitle(title: self?.localizedString(translation_key: "kWarning"), message: self?.localizedString(translation_key: "kInvalidLicense"))
            } else if errorCode == .errFile || errorCode == .errUnknown || errorCode.rawValue == -24 {
                self?.isOpeningFile = false
            }
            else {
                self?._alertWithTitle(title: self?.localizedString(translation_key: "kError"), message: self?.localizedString(translation_key: "kFailedToOpen"))
            }
        }
    }
    
}

extension ViewController: TabsViewControllerDelegate {
    
    func tabsViewController(showFileList tabsViewController: TabsViewController) {
        isOpeningFile = false
        pushViewController(fileList, transitionFromLeft: true)
    }
    
    func tabsViewController(exit tabsViewController: TabsViewController) {
        isOpeningFile = false
        if rootViewController?.viewControllers.last == tabsViewController {
            tabsViewController.suspendAutoFullScreen()
            rootViewController!.popViewController(animated: true)
        }
    }
    
}

extension ViewController: UINavigationControllerDelegate {
    
    func navigationController(_ navigationController: UINavigationController, didShow viewController: UIViewController, animated: Bool) {
        isOpeningFile = false
    }
    
}

fileprivate extension ViewController {
    
    func copyToDocument(PDFs filenames: [String], overwrite: Bool) {
        let fileManager = FileManager.default
        for filename in filenames {
            guard let fromPath = Bundle.main.path(forResource: filename, ofType: "pdf") else {
                print("\(filename).pdf not existed in bundle")
                continue
            }
            let DOCUMENT_PATH = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true)[0]
            let toPath = DOCUMENT_PATH + "/\(filename).pdf"
            
            if fileManager.fileExists(atPath: toPath) {
                if !overwrite {
                    continue
                }
                do {
                    try fileManager.removeItem(atPath: toPath)
                }
                catch {
                    continue
                }
            }
            do {
                try fileManager.copyItem(atPath: fromPath, toPath: toPath)
            }
            catch {
                print("Fail to copy \(filename). \(error.localizedDescription)")
            }
        }
    }
    
    func pushViewController(_ viewController: UIViewController, transitionFromLeft: Bool) {
        var vcs = rootViewController!.viewControllers
        if vcs.last == viewController {
            return
        }
        if let index = vcs.index(where: { $0 == viewController}) {
            vcs.remove(at: index)
            rootViewController!.viewControllers = vcs
        }
  
        if(transitionFromLeft) {
            let transition = CATransition()
            transition.type = kCATransitionPush
            transition.subtype = kCATransitionFromLeft
            rootViewController?.view.layer.add(transition, forKey: "pushViewController")
            rootViewController!.pushViewController(viewController, animated: true)
            rootViewController?.view.layer.removeAnimation(forKey: "pushViewController")
        } else {
            rootViewController!.pushViewController(viewController, animated: true)
        }
    }
    
    func switchAllowTabs() {
        let disallowTabsImg =  UIImage(named: "disallowTabs")
        let allowTabsImg =  UIImage(named: "allowTabs")
        if allowTabsButton!.image == disallowTabsImg {
            allowTabsButton!.image = allowTabsImg
            allowTabsButton!.accessibilityLabel = localizedString(translation_key: "kTabsModeReaderVoiceover")
        } else {
            allowTabsButton!.image = disallowTabsImg
            allowTabsButton!.accessibilityLabel = localizedString(translation_key: "kSingleModeReaderVoiceover")
        }
        tabsViewController.allowTabs = !tabsViewController.allowTabs;
    }
    
    func _alertWithTitle(title:String?, message:String?){
        var alertController: UIAlertController
        alertController = UIAlertController(title: title, message: message, preferredStyle: .alert)
        let action = UIAlertAction(title: "OK", style: .cancel, handler: { [unowned self] (_) in
            self.isOpeningFile = false
        })
        alertController.addAction(action)
        self.present(alertController, animated: true)
    }
    
    
    func localizedString(translation_key:String?) ->String!{
        guard let path = Bundle.main.path(forResource: FSLocalization.languageAbbr(), ofType: "lproj") else { return "" }
        let languageBundle = Bundle.init(path: path)
        return languageBundle?.localizedString(forKey: translation_key!, value: "", table: "FoxitLocalizable")
    }
}
