/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to 
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement 
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.

 */

import UIKit

class CustomCell: UITableViewCell {
    @IBOutlet var funcDescription: UILabel!
    @IBOutlet var runBtn: UIButton!
    var delegate: AnyObject!

    @IBAction func runFunctionTest(_ sender: AnyObject) {
        let row = runBtn.tag
        var msg: String? = nil
        let docDir = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true)[0]
        var demo: PlatformAPIDemo? = nil
        let manager = FileManager()
        let paths = ["/pdf2txt", "/outline", "/annotation", "/docinfo", "/render", "/signature", "/image2pdf", "/watermark", "/search", "/graphics_objects"]
        for i in 0..<paths.count {
            do {
                try manager.createDirectory(atPath: docDir + paths[i], withIntermediateDirectories: false, attributes: nil)
            } catch _ {
            }
        }
      
        var isSuccess = false
        var savedPath: String? = nil
        var pPage: FSPDFPage!
        var testFilePath = Bundle.main.path(forResource: "test_files/FoxitBigPreview", ofType: "pdf")!
        demo = PlatformAPIDemo(testFilePath, passwd: nil)
        switch row {
            case 0:
                savedPath = docDir + "/pdf2txt/FoxitBigPreview.txt"
                isSuccess = demo!.extractTextOfDoc(demo!.doc, toPath: savedPath!)
                break
            case 1:
                testFilePath = Bundle.main.path(forResource: "test_files/Outline", ofType: "pdf")!
                demo = PlatformAPIDemo(testFilePath, passwd: nil)
                savedPath = docDir + "/outline/Outline_edit.pdf"
                isSuccess = demo!.modifyBookmarkOfDoc(demo!.doc, saveAsNewDocAtPath: savedPath!)
                break
            case 2:
                testFilePath = Bundle.main.path(forResource: "test_files/annotation_input", ofType: "pdf")!
                demo = PlatformAPIDemo(testFilePath, passwd: nil)
                let iPage = 0
                pPage = demo!.doc.getPage(Int32(iPage))
                savedPath = docDir + "/annotation/annotation.pdf"
                isSuccess = demo!.addAnnotToPage(pPage, saveAsNewDocAtPath: savedPath!)
                break
            case 3:
                savedPath = docDir + "/docinfo/FoxitBigPreview_docinfo.txt"
                isSuccess = demo!.exportDocInfoOfDoc(demo!.doc, toPath: savedPath!)
                break
            case 4:
                let iPage = 0
                pPage = demo!.doc.getPage(Int32(iPage))
                savedPath = docDir + "/render/FoxitBigPreview.png"
                isSuccess = demo!.renderPage(pPage, atPath: savedPath!)
                break
            case 5:
                testFilePath = Bundle.main.path(forResource: "test_files/Sample", ofType: "pdf")!
                demo = PlatformAPIDemo(testFilePath, passwd: nil)
                let page = demo!.doc.getPage(0)
                savedPath = docDir + "/signature/Sample_signature.pdf"
                isSuccess = demo!.addSignature(to: page!, path: savedPath!)
                break
           case 6:
                let test_files = Bundle.main.path(forResource: "test_files", ofType: nil)!
                savedPath = docDir + "/image2pdf"
                isSuccess = (demo?.image2pdf(input_file: test_files, saveAsNewDocAtPath: savedPath!))!
                break;
           case 7:
                testFilePath = Bundle.main.path(forResource: "test_files/AboutFoxit", ofType: "pdf")!
                demo = PlatformAPIDemo(testFilePath, passwd: nil)
                savedPath = docDir + "/watermark/watermark_add.pdf"
                isSuccess = demo!.addWatermark(input_file: testFilePath, saveAsNewDocAtPath: savedPath!)
                break
            case 8:
                testFilePath = Bundle.main.path(forResource: "test_files/AboutFoxit", ofType: "pdf")!
                demo = PlatformAPIDemo(testFilePath, passwd: nil)
                savedPath = docDir + "/search/search.txt";
                isSuccess = (demo?.searchWithSaveResultsPath(path: savedPath!))!
                break
            case 9:
                testFilePath = Bundle.main.path(forResource: "test_files/graphics_objects", ofType: "pdf")!
                demo = PlatformAPIDemo(testFilePath, passwd: nil)
                savedPath = docDir + "/graphics_objects/graphics_objects.pdf";
                isSuccess = (demo?.addGraphicsObjects(inputPath: Bundle.main.path(forResource: "test_files", ofType: nil)!, saveAsNewDocAtPath: savedPath!))!
                break;
            default:
                isSuccess = true
                break
        }

        msg = "The generated file was saved to " + savedPath!
        let title = isSuccess ? "Successfully!" : "Failed!"
        let alert = UIAlertView(title: title, message: msg, delegate: nil, cancelButtonTitle: "OK")
        alert.show()
    }
}

