/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

import Foundation

class ExportAnnotData {

    class func getColorHexByDecimal(dec: Int) -> String {
        var hex: String = ""
        var letter: String
        var number: Int
        var decimal = dec
        for _ in 0..<9 {
            number = decimal%16
            decimal = decimal/16
            switch number {
                case 10:
                
                letter = "A"
                break
                case 11:
                
                letter = "B"
                break
                case 12:
                
                letter = "C"
                break
                case 13:
                
                letter = "D"
                break
                case 14:
                
                letter = "E"
                break
                case 15:
                
                letter = "F"
                break
                default:
                
                letter = String(number)
            }
            hex = letter + hex 
            if decimal == 0 {
                while hex.count < 6 {
                    hex = "0" + hex
                }
                break
            }
        }
        return "#"+hex
    }
    
    
    class func annotTypeEnumToString(type: FSAnnotType) -> String {
        switch type {
        case FSAnnotType.annotNote:
            
            return "Text"
        case FSAnnotType.annotLink:
            
            return "Link"
        case FSAnnotType.annotFreeText:
            
            return "FreeText"
        case FSAnnotType.annotLine:
            
            return "Line"
        case FSAnnotType.annotSquare:
            
            return "Square"
        case FSAnnotType.annotCircle:
            
            return "Circle"
        case FSAnnotType.annotPolygon:
            
            return "Polygon"
        case FSAnnotType.annotPolyLine:
            
            return "PolyLine"
        case FSAnnotType.annotHighlight:
            
            return "Highlight"
        case FSAnnotType.annotUnderline:
            
            return "Underline"
        case FSAnnotType.annotSquiggly:
            
            return "Squiggly"
        case FSAnnotType.annotStrikeOut:
            
            return "StrikeOut"
        case FSAnnotType.annotStamp:
            
            return "Stamp"
        case FSAnnotType.annotCaret:
            
            return "Caret"
        case FSAnnotType.annotInk:
            
            return "Ink"
        case FSAnnotType.annotPSInk:
            
            return "PSInk"
        case FSAnnotType.annotFileAttachment:
            
            return "FileAttachment"
        case FSAnnotType.annotSound:
            
            return "Sound"
        case FSAnnotType.annotMovie:
            
            return "Movie"
        case FSAnnotType.annotWidget:
            
            return "Widget"
        case FSAnnotType.annotScreen:
            
            return "Screen"
        case FSAnnotType.annotPrinterMark:
            
            return "PrinterMark"
        case FSAnnotType.annotTrapNet:
            
            return "TrapNet"
        case FSAnnotType.annotWatermark:
            
            return "Watermark"
        case FSAnnotType.annot3D:
            
            return "3D"
        case FSAnnotType.annotPopup:
            
            return "Popup"
        case FSAnnotType.annotRedact:
            
            return "Redact"
        case FSAnnotType.annotRichMedia:
            
            return "RichMedia"
        default:
            return "UnknownType"
        }
    }
    
    class func annotFlagsEnumToString(flags: UInt32) -> String {
        let flagsArr: NSMutableArray = NSMutableArray.init()
    
        if  flags & UInt32(FSAnnotFlags.invisible.rawValue) != 0 {
            flagsArr.add("Invisible")
        }
        if flags & UInt32(FSAnnotFlags.hidden.rawValue) != 0 {
            flagsArr.add("Hidden")
        }
        if flags & UInt32(FSAnnotFlags.print.rawValue) != 0 {
            flagsArr.add("Print")
        }
        if flags & UInt32(FSAnnotFlags.noZoom.rawValue) != 0 {
            flagsArr.add("NoZoom")
        }
        if flags & UInt32(FSAnnotFlags.noRotate.rawValue) != 0 {
            flagsArr.add("NoRotate")
        }
        if flags & UInt32(FSAnnotFlags.noView.rawValue) != 0 {
            flagsArr.add("NoView")
        }
        if flags & UInt32(FSAnnotFlags.readOnly.rawValue) != 0 {
            flagsArr.add("ReadOnly")
        }
        if flags & UInt32(FSAnnotFlags.locked.rawValue) != 0 {
            flagsArr.add("Locked")
        }
        if flags & UInt32(FSAnnotFlags.toggleNoView.rawValue) != 0 {
            flagsArr.add("ToggleNoView")
        }
        if flags & UInt32(FSAnnotFlags.lockedContents.rawValue) != 0 {
            flagsArr.add("LockedContents")
        }
        return flagsArr.componentsJoined(by: ",")
    }
    
    class func convertFSDateTime2DataString(time: FSDateTime) -> String {
        return String(format: "D:%d%02d%02d%02d%02d%02d+%02d'%02d'", time.year,time.month,time.day,time.hour,time.minute,time.second,time.utc_hour_offset,time.utc_minute_offset)
    }
    
    class func annotBorderStyle(style: FSBorderInfoStyle) -> String {
        switch style {
        case FSBorderInfoStyle.solid:
            
            return "Solid"
        case FSBorderInfoStyle.dashed:
            
            return "Dashed"
        case FSBorderInfoStyle.underLine:
            
            return "UnderLine"
        case FSBorderInfoStyle.beveled:
            
            return "Beveled"
        case FSBorderInfoStyle.inset:
            
            return "Inset"
        case FSBorderInfoStyle.cloudy:
            
            return "Cloudy"
        }
    }
    
    class func makeupEndingStyle(style: FSMarkupEndingStyle) -> String {
        switch style {
        case FSMarkupEndingStyle.none:
            
            return "None"
        case FSMarkupEndingStyle.square:
            
            return "Square"
        case FSMarkupEndingStyle.circle:
            
            return "Circle"
        case FSMarkupEndingStyle.diamond:
            
            return "Diamond"
        case FSMarkupEndingStyle.openArrow:
            
            return "OpenArrow"
        case FSMarkupEndingStyle.closedArrow:
            
            return "ClosedArrow"
        case FSMarkupEndingStyle.butt:
            
            return "Butt"
        case FSMarkupEndingStyle.rOpenArrow:
            
            return "ROpenArrow"
        case FSMarkupEndingStyle.rClosedArrow:
            
            return "RClosedArrow"
        case FSMarkupEndingStyle.slash:
            
            return "Slash"
        }
    }
    
    class func makeupState(state: FSMarkupState) -> String {
        switch state {
        case .none:
            return "None"
        case .marked:
            return "Marked"
        case .unmarked:
            return "Unmarked"
        case .accepted:
            return "Accepted"
        case .rejected:
            return "Rejected"
        case .cancelled:
            return "Cancelled"
        case .completed:
            return "Completed"
        case .deferred:
            return "Deferred"
        case .future:
            return "Future"
        }
    }
    
    class  func retriveDict(json: NSMutableDictionary, key: String, root: FSPDFObject) -> AnyObject {
        var ret: AnyObject = "" as AnyObject
        var dict : FSPDFDictionary? = nil
        let hexCode: [NSString] = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "A", "B", "C", "D", "E", "F"]
        
        switch root.getType() {
        case .boolean:
            return NSNumber.init(value: root.getBoolean())
        case .number:
            if abs(Float(root.getInteger()) - root.getFloat()) < 0.001 {
                return NSNumber.init(value: root.getInteger())
            }
            return NSNumber.init(value: root.getFloat())
        case .string:
            
            let cs: NSData = (root.getString() ?? Data.init()) as NSData
            let buffer = [UInt8](cs as Data)
            ret = NSMutableString(string: "<")
            let tmp: NSMutableString = ret as! NSMutableString;
            for ch in buffer {
                tmp.appendFormat("%@",hexCode[(Int(ch)&0xf0)>>4])
                tmp.appendFormat("%@",hexCode[(Int(ch)&0x0f)])
            }
            ret.append(">")
            return ret
            
        case .name:
            
            return NSString.init(format: "/%@", root.getName())
        case .array:
            
            ret = NSMutableArray.init()
            let pdfArr: FSPDFArray = root.getArray()
            
                for i in 0..<pdfArr.getElementCount() {
                    ret.add(retriveDict(json: json, key: key, root: pdfArr.getElement(i)))
                }
                return ret
            
        case .stream:
            
            ret = NSMutableDictionary.init()
            let stream: FSPDFStream = root.getStream()
            dict = stream.getDictionary()
            let size: UInt64 = stream.getDataSize(true)
            if (size != 0) {
                let temp: NSData = stream.getData(true)! as NSData
                let rawData: NSMutableString = NSMutableString()
                let buffer = [UInt8](temp as Data)
                for ch in buffer {
                    rawData.appendFormat("%@",hexCode[(Int(ch)&0xf0)>>4])
                    rawData.appendFormat("%@",hexCode[(Int(ch)&0x0f)])
                }
                ret.setValue(rawData, forKey: "stream")
            }
            fallthrough
            
        case .dictionary:
            
            if !ret.isKind(of: NSMutableDictionary.self) {
                ret = NSMutableDictionary.init()
            }
            if (dict == nil) {
                dict = root.getDict()
            }
            let type: FSPDFObject? = dict!.getElement("Type")
            if type != nil && type?.getWideString() == "OCG" {
                return NSMutableDictionary.init()
            }
            var pos: CLong = dict!.moveNext(0)
            while (pos != 0) {
                let key2: String = dict!.getKey(pos)
                let obj: FSPDFObject = dict!.getElement(key2)
                ret.setValue(retriveDict(json: json, key: key2, root: obj), forKey: key2)
                pos = dict!.moveNext(pos)
            }
            return ret
            
        case .null:
            return NSNull.init()
        case .reference:
            
            ret = NSMutableString.init()
            let objNum: UInt32 = root.getDirect().getObjNum()
            
            let tmp: NSMutableString = ret as! NSMutableString;
            tmp.appendFormat("%dR",objNum)
            json.setValue(retriveDict(json: json, key: key, root: root.getDirect()), forKey: NSNumber.init(value: objNum).stringValue)
            return ret
            
        default:
            
            break
        }
        return ret
    }
    
    class func exportAnnotDataToJson(doc: FSPDFDoc, toPath path: String) {
        let exportAnnots: NSMutableArray = NSMutableArray.init()
        for i in 0..<doc.getPageCount() {
            let page: FSPDFPage = doc.getPage(i)
            for j in 0..<page.getAnnotCount() {
                let annot: FSAnnot = page.getAnnot(j)
                exportAnnots.add(self.getAnnotDataWithAnnot(annot: annot))
            }
        }
        
        var jsonData: Data = try! JSONSerialization.data(withJSONObject: exportAnnots, options: JSONSerialization.WritingOptions.prettyPrinted)
        if !jsonData.isEmpty {
            var jsonStr:NSString = NSString.init(data: jsonData, encoding: String.Encoding.utf8.rawValue)!;
            jsonStr = jsonStr.replacingOccurrences(of: "\\/", with: "/") as NSString
            jsonData = jsonStr.data(using: String.Encoding.utf8.rawValue)!;
            (jsonData as NSData).write(toFile: path, atomically: true)
        }
    }
    
    class func annotExportToJSON(annot: FSAnnot) -> NSMutableDictionary {
        let annotDataDic: NSMutableDictionary = NSMutableDictionary.init()
        let annotType: FSAnnotType = annot.getType()
        annotDataDic.setValue(annotFlagsEnumToString(flags: annot.flags).lowercased(), forKey: "flags")
        annotDataDic.setValue(convertFSDateTime2DataString(time: annot.modifiedDateTime), forKey: "date")
        annotDataDic.setValue(annotTypeEnumToString(type: annotType).lowercased(), forKey: "type")
        annotDataDic.setValue(NSNumber.init(value:annot.getPage().getIndex()), forKey: "page")
        annotDataDic.setValue(NSString.init(format: "%f,%f,%f,%f", annot.getRect().left,annot.getRect().bottom,annot.getRect().right,annot.getRect().top), forKey: "rect")
        annotDataDic.setValue(annot.uniqueID, forKey: "name")
        if annotType == FSAnnotType.annotPopup {
            let popup: FSPopup = FSPopup(annot: annot)
            annotDataDic.setValue(popup.getOpenStatus ? "yes" : "no", forKey: "open")
            return annotDataDic
        }
        annotDataDic.setValue(getColorHexByDecimal(dec: Int(annot.borderColor)&0x00ffffff), forKey: "color")
        annotDataDic.setValue(NSString.init(format: "%@", annot.content), forKey: "contents")
        annotDataDic.setValue(NSNumber.init(value:annot.borderInfo.width), forKey: "width")
        
        annotDataDic.setValue(NSDictionary.init(), forKey: "customEntries")
        if annot.getDict().hasKey("customEntries") {
            let jsonStr = annot.getDict().getElement("customEntries").getWideString()!

            let jsonData = jsonStr.data(using: String.Encoding.utf8)!
            
            let customEntries:AnyObject = try! JSONSerialization.jsonObject(with: jsonData, options: JSONSerialization.ReadingOptions.mutableContainers) as AnyObject
            if !customEntries.isEqual(NSNull()) {
                annotDataDic.setValue(customEntries, forKey: "customEntries")
            }
        }

        annotDataDic.setValue(annotBorderStyle(style: annot.borderInfo.style).lowercased(), forKey: "style")
        if annot.borderInfo.style == FSBorderInfoStyle.dashed {
            annotDataDic.setValue(annot.borderInfo.dash_phase, forKey: "dashPhase")
            let tmp: NSMutableArray = NSMutableArray.init()
            for i in 0..<annot.borderInfo.dashes.getSize() {
                let dashes: Float = annot.borderInfo.dashes.getAt(i)
                tmp.add(NSNumber.init(value: dashes))
            }
            annotDataDic.setValue(tmp.componentsJoined(by:","), forKey: "dashes")
        } else if annot.borderInfo.style == FSBorderInfoStyle.cloudy {
            annotDataDic.setValue(annot.borderInfo.cloud_intensity, forKey: "intensity")
        }
        return annotDataDic
    }
        
        class func markupAnnotExportToJSON(annot: FSAnnot) -> NSMutableDictionary {
            let annotDataDic: NSMutableDictionary = NSMutableDictionary.init()
            let annotType: FSAnnotType = annot.getType()
            if annot.isMarkup() {
                let markupAnnot: FSMarkup = FSMarkup(annot: annot)
                let dict: FSPDFDictionary = markupAnnot.getDict()
                let RTObj: FSPDFObject? = dict.getElement("RI")
                var RT: String
                var IRT: UInt32 = 0
                var IRTDict: FSPDFObject? = dict.getElement("IRT")
                if !(IRTDict?.isEqual(NSNull()) ?? false) {
                    IRTDict = IRTDict?.getDict()
                    IRT = IRTDict?.getObjNum() ?? 0
                }
                if dict.hasKey("StateModel") && !(RTObj?.isEqual(NSNull()) ?? false) {
                    RT = RTObj?.getName() ?? "R"
                } else {
                    RT = "R"
                
                }
                let replyType: String = RT
                if (replyType.count != 0) && IRT != 0 {
                    let inreplyto: String = markupAnnot.getPage().getDocument().getIndirectObject(IRT).getDict().getElement("NM").getWideString();
                    annotDataDic.setValue(inreplyto, forKey: "inreplyto")
                    annotDataDic.setValue(replyType.lowercased(), forKey: "replyType")
                }
                if (markupAnnot.intent.count != 0) {
                    annotDataDic.setValue(markupAnnot.intent, forKey: "IT")
                    annotDataDic.setValue(markupAnnot.intent, forKey: "intent")
                }
                annotDataDic.setValue(markupAnnot.opacity, forKey: "opacity")
                annotDataDic.setValue(markupAnnot.subject, forKey: "subject")
                annotDataDic.setValue(markupAnnot.title, forKey: "title")
                annotDataDic.setValue(convertFSDateTime2DataString(time: markupAnnot.creationDateTime), forKey: "creationdate")
                if annotType == FSAnnotType.annotFreeText || annotType == FSAnnotType.annotStamp || annotType == FSAnnotType.annotScreen {
                    var rotation: Int32
                    if annotType == FSAnnotType.annotFreeText {
                        let freeText: FSFreeText = FSFreeText(annot: annot)
                        rotation = Int32(freeText.rotation.rawValue)
                    } else if annotType == FSAnnotType.annotStamp {
                        let stamp: FSStamp = FSStamp(annot: annot)
                        rotation = stamp.rotation
                    } else {
                        let screen: FSScreen = FSScreen(annot: annot)
                        rotation = Int32(screen.rotation.rawValue)
                    
                    }
                    annotDataDic.setValue(NSNumber.init(value:360-rotation*90), forKey: "rotation")
                }
                if annotType == FSAnnotType.annotLine || annotType == FSAnnotType.annotPolygon || annotType == FSAnnotType.annotPolyLine || annotType == FSAnnotType.annotSquare || annotType == FSAnnotType.annotCircle {
                    var measureRatio: String
                    var measureConversionFactor: CFloat
                    var measureUnit: String
                    if annotType == FSAnnotType.annotLine {
                        let line: FSLine = FSLine(annot: annot)
                        measureRatio = line.measureRatio
                        measureConversionFactor = line.getMeasureConversionFactor(FSMarkupMeasureType.X)
                        measureUnit = line.getMeasureUnit(FSMarkupMeasureType.X)
                    } else if annotType == FSAnnotType.annotPolygon {
                        let polygon: FSPolygon = FSPolygon(annot: annot)
                        measureRatio = polygon.measureRatio
                        measureConversionFactor = polygon.getMeasureConversionFactor(FSMarkupMeasureType.X)
                        measureUnit = polygon.getMeasureUnit(FSMarkupMeasureType.X)
                    } else if annotType == FSAnnotType.annotPolyLine {
                        let polyline: FSPolyLine = FSPolyLine(annot: annot)
                        measureRatio = polyline.measureRatio
                        measureConversionFactor = polyline.getMeasureConversionFactor(FSMarkupMeasureType.X)
                        measureUnit = polyline.getMeasureUnit(FSMarkupMeasureType.X)
                    } else if annotType == FSAnnotType.annotSquare {
                        let square: FSSquare = FSSquare(annot: annot)
                        measureRatio = square.measureRatio
                        measureConversionFactor = square.getMeasureConversionFactor(FSMarkupMeasureType.X)
                        measureUnit = square.getMeasureUnit(FSMarkupMeasureType.X)
                    } else {
                        let circle: FSCircle = FSCircle(annot: annot)
                        measureRatio = circle.measureRatio
                        measureConversionFactor = circle.getMeasureConversionFactor(FSMarkupMeasureType.X)
                        measureUnit = circle.getMeasureUnit(FSMarkupMeasureType.X)
                    
                    }
                    if (measureRatio.count != 0) {
                        annotDataDic.setValue(measureRatio, forKey: "measureRatio")
                        annotDataDic.setValue(measureConversionFactor, forKey: "measureConversionFactor")
                        annotDataDic.setValue(measureUnit, forKey: "measureUnit")
                    }
                }
                if !markupAnnot.popup.isEmpty() {
                    annotDataDic.setValue(self.getAnnotDataWithAnnot(annot: markupAnnot.popup), forKey: "popup")
                }
            }
            return annotDataDic
        }
    class func caretAnnotExportToJSON(annot: FSAnnot) -> NSMutableDictionary {
        let annotDataDic: NSMutableDictionary = NSMutableDictionary.init()
        let caret: FSCaret = FSCaret(annot: annot)
        let fringe: NSString = NSString.init(format: "%f,%f,%f,%f", caret.innerRect.left-annot.getRect().left,caret.innerRect.bottom-annot.getRect().bottom,annot.getRect().right-caret.innerRect.right,annot.getRect().top-caret.innerRect.top)
        annotDataDic.setValue(fringe, forKey: "fringe")
        annotDataDic.removeObject(forKey: "style")
        annotDataDic.removeObject(forKey: "width")
        annotDataDic.removeObject(forKey: "dashPhase")
        annotDataDic.removeObject(forKey: "intensity")
        return annotDataDic
    }
        
    class func circleAnnotExportToJSON(annot: FSAnnot) -> NSMutableDictionary {
        let annotDataDic: NSMutableDictionary = NSMutableDictionary.init()
        let circle: FSCircle = FSCircle(annot: annot)
        let fringe: NSString = NSString.init(format: "%f,%f,%f,%f", circle.innerRect.left-annot.getRect().left,circle.innerRect.bottom-annot.getRect().bottom,annot.getRect().right-circle.innerRect.right,annot.getRect().top-circle.innerRect.top)
        annotDataDic.setValue(fringe, forKey: "fringe")
        if (circle.fillColor != 0) {
            let color: String = self.getColorHexByDecimal(dec: Int(circle.fillColor)&0x00ffffff)
            annotDataDic.setValue(color, forKey: "interior-color")
            annotDataDic.setValue(color, forKey: "interiorColor")
        }
        if circle.intent == "CircleDimension" {
            annotDataDic.setValue(circle.measureRatio, forKey: "measureRatio")
            annotDataDic.setValue(circle.getMeasureConversionFactor(FSMarkupMeasureType.X), forKey: "measureConversionFactor")
            annotDataDic.setValue(circle.getMeasureUnit(FSMarkupMeasureType.X), forKey: "measureUnit")
        }
        return annotDataDic
    }
        
    class func freeTextAnnotExportToJSON(annot: FSAnnot) -> NSMutableDictionary {
        let annotDataDic: NSMutableDictionary = NSMutableDictionary.init()
        let freeText: FSFreeText = FSFreeText(annot: annot)
        var alignment: FSAlignment = freeText.alignment
        if !((freeText.intent.count == 0) || freeText.intent == "FreeTextCallout") {
            alignment = FSAlignment.left
            freeText.alignment = FSAlignment.left
        }
        annotDataDic.setValue(NSNumber.init(value:alignment.rawValue), forKey: "justification")
        var DA: String = ""
        if freeText.getDefaultAppearance().font.getName().count != 0 {
            DA = "/" + freeText.getDefaultAppearance().font.getName();
        }
        if freeText.getDefaultAppearance().text_size != 0 {
            DA += " " + String(freeText.getDefaultAppearance().text_size) + " Tf "
        }
        
        DA += String(format: "%.2f %.2f %.2f rg",Double(((annot.borderColor&0xFF0000)>>16))/255.0,Double(((annot.borderColor&0xFF00)>>16))/255.0,Double(((annot.borderColor&0xFF)>>16))/255.0)
        annotDataDic.setValue(self.getColorHexByDecimal(dec: Int(freeText.getDefaultAppearance().text_color)&0x00ffffff), forKey: "fontColor")
        annotDataDic.setValue(DA, forKey: "defaultappearance")
        if (freeText.fillColor != 0) {
            let color: String = self.getColorHexByDecimal(dec: Int(freeText.fillColor)&0x00ffffff)
            annotDataDic.setValue(color, forKey: "interior-color")
            annotDataDic.setValue(color, forKey: "interiorColor")
        }
        if freeText.intent == "FreeTextCallout" {
            let fringe: NSString = NSString.init(format: "%f,%f,%f,%f", freeText.innerRect.left-annot.getRect().left,freeText.innerRect.bottom-annot.getRect().bottom,annot.getRect().right-freeText.innerRect.right,annot.getRect().top-freeText.innerRect.top)
            annotDataDic.setValue(fringe, forKey: "fringe")
            let callout: NSMutableString = NSMutableString()
            let points: FSPointFArray = freeText.calloutLinePoints
            callout.appendFormat("%f,%f,%f,%f,%f,%f",points.getAt(0).x,points.getAt(0).y,points.getAt(1).x,points.getAt(1).y,points.getAt(2).x,points.getAt(2).y)
            annotDataDic.setValue(callout, forKey: "callout")
            annotDataDic.setValue(self.makeupEndingStyle(style: freeText.calloutLineEndingStyle), forKey: "head")
        }
        return annotDataDic
    }
    
    class func inkAnnotExportToJSON(annot: FSAnnot) -> NSMutableDictionary {
        let annotDataDic: NSMutableDictionary = NSMutableDictionary.init()
        let ink: FSInk = FSInk(annot: annot)
        let inkListInfo: FSPath = ink.inkList
        var inklist: NSMutableString = NSMutableString(string: "[")
        for index in 0..<inkListInfo.getPointCount() {
            if index != 0 && inkListInfo.getPointType(index).rawValue == 1 {
                inklist = NSMutableString(string: inklist.substring(to: inklist.length-1))
                inklist.append("],[")
            }
            inklist.appendFormat("%f,%f;",inkListInfo.getPoint(index).x,inkListInfo.getPoint(index).y)
        }
        inklist.replaceCharacters(in: NSMakeRange(inklist.length-1, 1), with: "]")
        annotDataDic.setValue(inklist, forKey: "inklist")
        return annotDataDic
    }
    
    class func lineAnnotExportToJSON(annot: FSAnnot) -> NSMutableDictionary {
        let annotDataDic: NSMutableDictionary = NSMutableDictionary.init()
        let line: FSLine = FSLine(annot: annot)
        if (line.styleFillColor != 0) {
            let color: String = self.getColorHexByDecimal(dec: Int(line.styleFillColor)&0x00ffffff)
            annotDataDic.setValue(color, forKey: "interior-color")
            annotDataDic.setValue(color, forKey: "interiorColor")
        }
        annotDataDic.setValue(String(line.startPoint.x) + "," + String(line.startPoint.y), forKey: "start")
        annotDataDic.setValue(String(line.endPoint.x) + "," + String(line.endPoint.y), forKey: "end")
        annotDataDic.setValue(self.makeupEndingStyle(style: line.lineStartStyle), forKey: "head")
        annotDataDic.setValue(self.makeupEndingStyle(style: line.lineEndStyle), forKey: "tail")
        annotDataDic.setValue(line.leaderLineLength, forKey: "leaderLength")
        annotDataDic.setValue(line.leaderLineExtensionLength, forKey: "leaderExtend")
        annotDataDic.setValue(line.hasCaption() ? "yes" : "no", forKey: "caption")
        return annotDataDic
    }
    
    class func noteAnnotExportToJSON(annot: FSAnnot) -> NSMutableDictionary {
        let annotDataDic: NSMutableDictionary = NSMutableDictionary.init()
        let note: FSNote = FSNote(annot: annot)
        annotDataDic.setValue(note.iconName, forKey: "icon")
        if note.isStateAnnot() {
            if note.getStateModel() == FSMarkupStateModel.marked {
                annotDataDic.setValue("Marked", forKey: "statemodel")
            }else{
                annotDataDic.setValue("Review", forKey: "statemodel")
            }
            annotDataDic.setValue(makeupState(state:note.state), forKey: "state")
            annotDataDic.removeObject(forKey: "content")
            annotDataDic.removeObject(forKey: "color")
        }
        return annotDataDic
    }
    
    class func polygonAnnotExportToJSON(annot: FSAnnot) -> NSMutableDictionary {
        let annotDataDic: NSMutableDictionary = NSMutableDictionary.init()
        let polygon: FSPolygon = FSPolygon(annot: annot)
        var pointList: NSMutableString = NSMutableString()
        for i in 0..<polygon.vertexes.getSize() {
            pointList.appendFormat("%f,%f;",polygon.vertexes.getAt(i).x,polygon.vertexes.getAt(i).y)
        }
        pointList = NSMutableString(string: pointList.substring(to: pointList.length-1))
        if polygon.fillColor != 0 {
            let color: String = self.getColorHexByDecimal(dec: Int(polygon.fillColor)&0x00ffffff)
            annotDataDic.setValue(color, forKey: "interior-color")
            annotDataDic.setValue(color, forKey: "interiorColor")
        }
        if polygon.intent == "PolygonDimension" {
            annotDataDic.setValue(polygon.measureRatio, forKey: "measureRatio")
            annotDataDic.setValue(polygon.getMeasureConversionFactor(FSMarkupMeasureType.X), forKey: "measureConversionFactor")
            annotDataDic.setValue(polygon.getMeasureUnit(FSMarkupMeasureType.X), forKey: "measureUnit")
        }
        annotDataDic.setValue(pointList, forKey: "vertices")
        return annotDataDic
    }
    
    class func polyLineAnnotExportToJSON(annot: FSAnnot) -> NSMutableDictionary {
        let annotDataDic: NSMutableDictionary = NSMutableDictionary.init()
        let polyline: FSPolyLine = FSPolyLine(annot: annot)
        var pointList: NSMutableString = NSMutableString()
        for i in 0..<polyline.vertexes.getSize() {
            pointList.appendFormat("%f,%f;",polyline.vertexes.getAt(i).x,polyline.vertexes.getAt(i).y)
        }
        pointList = NSMutableString(string: pointList.substring(to: pointList.length-1))
        if (polyline.styleFillColor != 0) {
            let color: String = self.getColorHexByDecimal(dec: Int(polyline.styleFillColor)&0x00ffffff)
            annotDataDic.setValue(color, forKey: "interior-color")
            annotDataDic.setValue(color, forKey: "interiorColor")
        }
        annotDataDic.setValue(self.makeupEndingStyle(style: polyline.lineStartStyle), forKey: "head")
        annotDataDic.setValue(self.makeupEndingStyle(style: polyline.lineEndStyle), forKey: "tail")
        annotDataDic.setValue(pointList, forKey: "vertices")
        return annotDataDic
    }
    
    class func redactAnnotExportToJSON(annot: FSAnnot) -> NSMutableDictionary {
        let annotDataDic: NSMutableDictionary = NSMutableDictionary.init()
        let redact: FSRedact = FSRedact(annot: annot)
        if redact.applyFillColor != 0 {
            let color: String = self.getColorHexByDecimal(dec: Int(redact.applyFillColor)&0x00ffffff)
            annotDataDic.setValue(color, forKey: "interior-color")
            annotDataDic.setValue(color, forKey: "interiorColor")
        }
        
        let jsonQP: NSMutableString = NSMutableString()
        for i in 0..<redact.quadPoints.getSize() {
            let quadPoint: FSQuadPoints = redact.quadPoints.getAt(i)
            jsonQP.appendFormat("%f,%f,",quadPoint.first.x,quadPoint.first.y)
            jsonQP.appendFormat("%f,%f,",quadPoint.second.x,quadPoint.second.y)
            jsonQP.appendFormat("%f,%f,",quadPoint.third.x,quadPoint.third.y)
            jsonQP.appendFormat("%f,%f,",quadPoint.fourth.x,quadPoint.fourth.y)
        }
        annotDataDic.setValue(jsonQP.substring(to: jsonQP.length-1), forKey: "coords")
        annotDataDic.setValue(redact.overlayText, forKey: "overlay-text")
        annotDataDic.setValue(redact.overlayText, forKey: "overlayText")
        var DA: String = ""
        if redact.getDefaultAppearance().font.getName().count != 0 {
            DA = "/" + redact.getDefaultAppearance().font.getName();
        }
        if redact.getDefaultAppearance().text_size != 0 {
            DA += " " + String(redact.getDefaultAppearance().text_size) + " Tf "
        }
        DA += String(format: "%.2f %.2f %.2f rg",Double(((annot.borderColor&0xFF0000)>>16))/255.0,Double(((annot.borderColor&0xFF00)>>16))/255.0,Double(((annot.borderColor&0xFF)>>16))/255.0)
        annotDataDic.setValue(DA, forKey: "defaultappearance")
        annotDataDic.setValue(self.getColorHexByDecimal(dec: Int(redact.getDefaultAppearance().text_color)&0x00ffffff), forKey: "fontColor")
        annotDataDic.setValue(NSNumber.init(value:redact.overlayTextAlignment.rawValue), forKey: "justification")
        return annotDataDic
    }
    
    class func soundAnnotExportToJSON(annot: FSAnnot) -> NSMutableDictionary {
        let annotDataDic: NSMutableDictionary = NSMutableDictionary.init()
        let sound: FSSound = FSSound(annot: annot)
        annotDataDic.setValue(sound.getSamplingRate, forKey: "rate")
        let rawDict: NSMutableDictionary = NSMutableDictionary.init()
        let dict: FSPDFDictionary = sound.getDict()
        let sd: FSPDFObject = dict.getElement("Sound")
        if !sd.isEqual(NSNull()) {
            rawDict.setValue(self.retriveDict(json: rawDict, key: "", root: sd.getDirect()), forKey: "Sound")
            annotDataDic.setValue(rawDict, forKey: "raw")
        }
        return annotDataDic
    }
    
    class func squareAnnotExportToJSON(annot: FSAnnot) -> NSMutableDictionary {
        let annotDataDic: NSMutableDictionary = NSMutableDictionary.init()
        let square: FSSquare = FSSquare(annot: annot)
        let fringe: NSString = NSString.init(format: "%f,%f,%f,%f", square.innerRect.left-annot.getRect().left,square.innerRect.bottom-annot.getRect().bottom,annot.getRect().right-square.innerRect.right,annot.getRect().top-square.innerRect.top)
        annotDataDic.setValue(fringe, forKey: "fringe")
        if square.fillColor != 0 {
            let color: String = self.getColorHexByDecimal(dec: Int(square.fillColor)&0x00ffffff)
            annotDataDic.setValue(color, forKey: "interior-color")
            annotDataDic.setValue(color, forKey: "interiorColor")
        }
        if square.intent == "CustomDimension" {
            annotDataDic.setValue(square.measureRatio, forKey: "measureRatio")
            annotDataDic.setValue(square.getMeasureConversionFactor(FSMarkupMeasureType.X), forKey: "measureConversionFactor")
            annotDataDic.setValue(square.getMeasureUnit(FSMarkupMeasureType.X), forKey: "measureUnit")
        }
        return annotDataDic
    }
    
    class func stampAnnotExportToJSON(annot: FSAnnot) -> NSMutableDictionary {
        let annotDataDic: NSMutableDictionary = NSMutableDictionary.init()
        let stamp: FSStamp = FSStamp(annot: annot)
        annotDataDic.setValue(stamp.iconName, forKey: "icon")
        let rawDict: NSMutableDictionary = NSMutableDictionary.init()
        let dict: FSPDFDictionary = stamp.getDict()
        let ap: FSPDFObject = dict.getElement("AP")
        if !ap.isEqual(NSNull()) {
            rawDict.setValue(self.retriveDict(json: rawDict, key: "", root: ap.getDirect().getDict().getElement("N").getDirect()), forKey: "N")
            annotDataDic.setValue(rawDict, forKey: "raw")
        }
        return annotDataDic
    }
    
    class func textMarkupAnnotExportToJSON(annot: FSAnnot) -> NSMutableDictionary {
        let annotDataDic: NSMutableDictionary = NSMutableDictionary.init()
        let textMarkup: FSTextMarkup = FSTextMarkup(annot: annot)
        let jsonQP: NSMutableString = NSMutableString()
        for i in 0..<textMarkup.quadPoints.getSize() {
            let quadPoint: FSQuadPoints = textMarkup.quadPoints.getAt(i)
            jsonQP.appendFormat("%f,%f,",quadPoint.first.x,quadPoint.first.y)
            jsonQP.appendFormat("%f,%f,",quadPoint.second.x,quadPoint.second.y)
            jsonQP.appendFormat("%f,%f,",quadPoint.third.x,quadPoint.third.y)
            jsonQP.appendFormat("%f,%f,",quadPoint.fourth.x,quadPoint.fourth.y)
        }
        annotDataDic.setValue(jsonQP.substring(to: jsonQP.length-1), forKey: "coords")
        return annotDataDic
    }
    
    class func fileAttachmentAnnotExportToJSON(annot: FSAnnot) -> NSMutableDictionary {
        let annotDataDic: NSMutableDictionary = NSMutableDictionary.init()
        let fileAttachment: FSFileAttachment = FSFileAttachment(annot: annot)
        annotDataDic.setValue(fileAttachment.iconName, forKey: "icon")
        
        let rawDict: NSMutableDictionary = NSMutableDictionary.init()
        let dict: FSPDFDictionary = annot.getDict()
        let fs: FSPDFObject = dict.getElement("FS")
        if !fs.isEqual(NSNull()) {
            rawDict.setValue(retriveDict(json: rawDict, key: "", root: fs.getDirect()), forKey: "FS")
            annotDataDic.setValue(rawDict, forKey: "raw")
        }
        return annotDataDic
    }
        
    class func getAnnotDataWithAnnot(annot: FSAnnot) -> NSMutableDictionary {
        let annotDataDic: NSMutableDictionary = NSMutableDictionary.init()
        let annotType: FSAnnotType = annot.getType()
        annotDataDic.addEntries(from: self.annotExportToJSON(annot: annot) as! [AnyHashable : Any])
        annotDataDic.addEntries(from: self.markupAnnotExportToJSON(annot: annot) as! [AnyHashable : Any])
        if annotType == FSAnnotType.annotCaret {
            annotDataDic.addEntries(from: self.caretAnnotExportToJSON(annot: annot) as! [AnyHashable : Any])
        } else if annotType == FSAnnotType.annotCircle {
            annotDataDic.addEntries(from: self.circleAnnotExportToJSON(annot: annot) as! [AnyHashable : Any])
        } else if annotType == FSAnnotType.annotFreeText {
            annotDataDic.addEntries(from: self.freeTextAnnotExportToJSON(annot: annot) as! [AnyHashable : Any])
        } else if annotType == FSAnnotType.annotInk {
            annotDataDic.addEntries(from: self.inkAnnotExportToJSON(annot: annot) as! [AnyHashable : Any])
        } else if annotType == FSAnnotType.annotLine {
            annotDataDic.addEntries(from: self.lineAnnotExportToJSON(annot: annot) as! [AnyHashable : Any])
        } else if annotType == FSAnnotType.annotNote {
            annotDataDic.addEntries(from: self.noteAnnotExportToJSON(annot: annot) as! [AnyHashable : Any])
        } else if annotType == FSAnnotType.annotPolygon {
            annotDataDic.addEntries(from: self.polygonAnnotExportToJSON(annot: annot) as! [AnyHashable : Any])
        } else if annotType == FSAnnotType.annotPolyLine {
            annotDataDic.addEntries(from: self.polyLineAnnotExportToJSON(annot: annot) as! [AnyHashable : Any])
        } else if annotType == FSAnnotType.annotRedact {
            annotDataDic.addEntries(from: self.redactAnnotExportToJSON(annot: annot) as! [AnyHashable : Any])
        } else if annotType == FSAnnotType.annotSound {
            annotDataDic.addEntries(from: self.soundAnnotExportToJSON(annot: annot) as! [AnyHashable : Any])
        } else if annotType == FSAnnotType.annotSquare {
            annotDataDic.addEntries(from: self.squareAnnotExportToJSON(annot: annot) as! [AnyHashable : Any])
        } else if annotType == FSAnnotType.annotStamp {
            annotDataDic.addEntries(from: self.stampAnnotExportToJSON(annot: annot) as! [AnyHashable : Any])
        } else if annotType == FSAnnotType.annotFileAttachment {
            annotDataDic.addEntries(from: self.fileAttachmentAnnotExportToJSON(annot: annot) as! [AnyHashable : Any])
        }else if annotType == FSAnnotType.annotHighlight || annotType == FSAnnotType.annotUnderline || annotType == FSAnnotType.annotStrikeOut || annotType == FSAnnotType.annotSquiggly {
            annotDataDic.addEntries(from: self.textMarkupAnnotExportToJSON(annot: annot) as! [AnyHashable : Any])
        }
        return annotDataDic
    }

}

