/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.

 */
import Foundation
class MyIconProvider: NSObject {

    var use_dynamic_stamp_ = false;
    var icon_file_folder_ = ""
    override init() {
        use_dynamic_stamp_ = false
        icon_file_folder_ = ""
    }
    
    init(folder: String, use_dynamic_stamp_: Bool) {
        icon_file_folder_ = folder
        self.use_dynamic_stamp_ = use_dynamic_stamp_
    }
}

extension MyIconProvider: FSIconProviderCallback {
    
    
    func getProviderID() -> String! {
        if (use_dynamic_stamp_) {
            return "Simple Demo Dynamic MyIconProvider"
        } else {
            return "Simple Demo MyIconProvider"
        }
    }
    
    func getProviderVersion() -> String! {
        return "1.0.0";
    }
    
    func hasIcon(_ annotType: FSAnnotType, iconName: String!) -> Bool {
        if (annotType != FSAnnotType.annotStamp){
            return false
        }
        var icon_file_path = "";
        if (use_dynamic_stamp_) {
            icon_file_path = icon_file_folder_.appending("/DynamicStamps/")
        } else {
            icon_file_path = icon_file_folder_.appending("/StaticStamps/")
        }
        icon_file_path = icon_file_path.appending(iconName)
        icon_file_path = icon_file_path.appending(".pdf")
        let doc = FSPDFDoc(path: icon_file_path)
        let error_code = doc?.load(nil)
        if (error_code != FSErrorCode.errSuccess) {
//            NSLog("Fail to load icon file:%@, with error value: %ld", icon_file_path, error_code ?? FSErrorCode.errUnknown)
            return false;
        }
        return !(doc?.isEmpty())!
    }
    
    func canChangeColor(_ annotType: FSAnnotType, iconName: String!) -> Bool {
        return false
    }
    
    func getIcon(_ annotType: FSAnnotType, iconName: String!, color: UInt32) -> FSPDFPage! {
        var icon_file_path = "";
        if (use_dynamic_stamp_) {
            icon_file_path = icon_file_folder_.appending("/DynamicStamps/")
        } else {
            icon_file_path = icon_file_folder_.appending("/StaticStamps/")
        }
        icon_file_path = icon_file_path.appending(iconName)
        icon_file_path = icon_file_path.appending(".pdf")
        let doc = FSPDFDoc(path: icon_file_path)
        let error_code = doc?.load(nil)
        if (error_code != FSErrorCode.errSuccess) {
//            NSLog("Fail to load icon file:%@, with error value: %ld", icon_file_path, error_code ?? FSErrorCode.errUnknown)
            return nil
        }
        return doc?.getPage(0);
    }
    
    func getShadingColor(_ annotType: FSAnnotType, iconName: String!, refColor: UInt32, shading shadingIndex: Int32, shadingColor out_shading_color: FSShadingColor!) -> FSShadingColor! {
        return nil
    }
    
    func getDisplayWidth(_ annotType: FSAnnotType, iconName: String!) -> NSNumber! {
        return NSNumber(value: 0)
    }
    
    func getDisplayHeight(_ annotType: FSAnnotType, iconName: String!) -> NSNumber! {
        return NSNumber(value: 0)
    }
    
    
}


class MyActionEventHandler : NSObject {
    
}

extension MyActionEventHandler: FSActionCallback {
    func getJapaneseEras(_ year: Int32, month: Int32, day: Int32) -> String! {
        return ""
    }
    
    func query(_ keywords: String!, search_scope: FSActionCallbackSearchScope, search_option: FSSearchOption!, di_path: String!) {
    }
    
    func addSearchIndex(_ di_path: String!, is_selected: Bool) -> FSSearchIndexConfig! {
        return FSSearchIndexConfig.init()
    }
    
    func removeSearchIndex(_ searchIndexConfig: FSSearchIndexConfig!) -> Bool {
        return false
    }
    
    func getSignatureAPStyleNameList() -> [String]! {
        return []
    }
    
    func invalidateRect(_ document: FSPDFDoc!, page_index: Int32, pdf_rect: FSRectF!) -> Bool {
        return false
    }
    
    func setCurrentPage(_ pdfDoc: FSPDFDoc!, destination: FSDestination!) {
    }
    
    func getAppInfo(_ type: FSActionCallbackAppInfoType) -> String! {
        return ""
    }
    
    func browseFile(_ is_open_dialog: Bool, file_format: String!, file_filter: String!) -> String! {
        return ""
    }
    
    func submitForm(_ document: FSPDFDoc!, form formData: Data!, url: String!, file_format_type: FSFileFormatType) -> Bool {
        return false
    }
    
    func getCurrentPage(_ pdfDoc: FSPDFDoc!) -> Int32 {
        return -1
    }
    
    func setCurrentPage(_ pdfDoc: FSPDFDoc!, pageIndex: Int32) {
        
    }
    
    func getPageRotation(_ pdfDoc: FSPDFDoc!, pageIndex: Int32) -> FSRotation {
        return FSRotation.rotation0
    }
    
    func setPageRotation(_ pdfDoc: FSPDFDoc!, pageIndex: Int32, rotation: FSRotation) -> Bool {
        return false
    }
    
    func alert(_ msg: String!, title: String!, type: Int32, icon: Int32) -> Int32 {
        return 0
    }
    
    func getIdentityProperties() -> FSIdentityProperties! {
        let identityProperties = FSIdentityProperties.init()
        identityProperties?.set("foxitsoftware", email: "simple_demo@foxitsoftware.cn", login_name: "simple demo", name: "Simple", first_name: nil, last_name: nil, title: nil, department: nil);
        return identityProperties
    }
    
    func setDocChangeMark(_ document: FSPDFDoc!, change_mark: Bool) -> Bool {
        return true
    }
    
    func getDocChangeMark(_ document: FSPDFDoc!) -> Bool {
        return true
    }
    
    func verifySignature(_ document: FSPDFDoc!, signature: FSSignature!) -> UInt32 {
        return UInt32(FSSignatureStates.verifyErrorData.rawValue)
    }
    
    
}
