/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to 
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement 
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.

 */

import UIKit

class ViewController: UIViewController, UISearchBarDelegate, ISearchEventListener {
    
    @IBOutlet weak var topBar: UIToolbar!
    @IBOutlet weak var pdfViewCtrl: FSPDFViewCtrl!
    
    var extensionsManager: UIExtensionsManager!
    var panelController: FSPanelController?
    var searchButton: UIButton!
    var bookmarkButton: UIButton!
    var popoverController: UIPopoverController!
    
    var space = 0.0
    var buttonInset: CGFloat = 0.0
    var willPopBookmarkViewController = false
        
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
        UIDevice.current.beginGeneratingDeviceOrientationNotifications()
        willPopBookmarkViewController = false
        
        self.buildTopToolBar()
        let pdfPath = Bundle.main.path(forResource: "Sample", ofType: "pdf")!
        let doc = FSPDFDoc(path: pdfPath)
        if (doc?.isEmpty())! {
            return
        }
        if doc!.load(nil) == .errSuccess {
            self.pdfViewCtrl.setDoc(doc!)
        }
        self.extensionsManager = UIExtensionsManager(pdfViewControl: self.pdfViewCtrl)
        //Disable the default toolbars.
        self.extensionsManager.enableTopToolbar(false);
        self.extensionsManager.enableBottomToolbar(false);
        self.extensionsManager.enableSmallTopToolbar(false);
        self.extensionsManager.enableSmallBottomToolbar(false)
        self.pdfViewCtrl.extensionsManager = self.extensionsManager
        self.extensionsManager.register(self)
        
        self.panelController = self.extensionsManager.panelController
        self.panelController?.isHidden = true
        
        let paths = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true)
        let cachesDir = paths[0]
        
        print(cachesDir);
    }
    
    func buildTopToolBar() {
        self.searchButton = UIButton(type: .custom)
        let searchImage = UIImage(named: "search.png", in: Bundle(for: UIExtensionsManager.self), compatibleWith: nil) ?? UIImage()
        let highLightedSearchImage = self.applyingAlpha(image: searchImage, alpha: 0.5)
        searchButton.setImage(searchImage, for: .normal)
        searchButton.setImage(highLightedSearchImage, for: .highlighted)
        searchButton.addTarget(self, action: #selector(self.showSearchBar), for: .touchUpInside)
        self.searchButton.contentEdgeInsets = UIEdgeInsetsMake(0, buttonInset, 0, buttonInset)
        searchButton.sizeToFit()
        searchButton.autoresizingMask = .flexibleLeftMargin;

        self.bookmarkButton = UIButton(type: .custom)
        let bookmarkImage = UIImage(named: "document_list_state_iphone.png", in: Bundle(for: UIExtensionsManager.self), compatibleWith: nil) ?? UIImage()
        let highLightedBookmarkImage = self.applyingAlpha(image: bookmarkImage, alpha: 0.5)
        bookmarkButton.setImage(bookmarkImage, for: .normal)
        bookmarkButton.setImage(highLightedBookmarkImage, for: .highlighted)
        bookmarkButton.addTarget(self, action: #selector(togglePanel), for: .touchUpInside)
        self.bookmarkButton.contentEdgeInsets = UIEdgeInsetsMake(0, buttonInset, 0, buttonInset)
        bookmarkButton.sizeToFit()
        bookmarkButton.autoresizingMask = .flexibleLeftMargin;
        
        let spaceItem = UIBarButtonItem(barButtonSystemItem:UIBarButtonSystemItem.flexibleSpace, target: nil, action: nil);
        
        self.topBar.items =  [spaceItem, UIBarButtonItem(customView: self.searchButton), UIBarButtonItem(customView: self.bookmarkButton)]
        
    }
    
    // MARK: - Bookmark
    /** @brief Show bookmarks of the current PDF document when click the outline button. */
    func showBookmarkViewController() {
        self.panelController?.isHidden = false
    }
    
    @objc func togglePanel() {
        self.panelController?.isHidden = !(self.panelController?.isHidden)!
        extensionsManager.showSearchBar(false)
        pdfViewCtrl.endEditing(true)
    }
    
    // MARK: DemoBookmarkViewControllerDelegate
    /** @brief Trigged when the bookmark select has been selected. */
    func onSelect(_ bookmark: FSBookmark) {
        let dest = bookmark.destination!
        let point = FSPointF()!
        point.set(dest.getLeft(), y: dest.getTop())
        self.pdfViewCtrl.gotoPage(dest.getPageIndex(self.pdfViewCtrl.currentDoc), withDocPoint: point, animated: false)
        self.dismissBookmarkViewController()
    }
    
    func dismissBookmarkViewController() {
        if UIDevice.current.userInterfaceIdiom == .phone {
            self.dismiss(animated: true, completion: nil)
        }
        else {
            if popoverController.isPopoverVisible {
                popoverController.dismiss(animated: true)
            }
            self.popoverController = nil
        }
    }
   
    // MARK: - Search
    /** @brief Show search bar when click the search button. */
    @objc func showSearchBar() {
        if extensionsManager.currentAnnot != nil{
            extensionsManager.currentAnnot = nil
        }
        if !(self.panelController?.isHidden)! {
            self.panelController?.isHidden = true
        }
        extensionsManager.change(FSUIManagerState.search)
    }
    
    // MARK: Rotation
    func didRotate() {
        self.searchButton.isHidden = false
        self.bookmarkButton.isHidden = false
        if willPopBookmarkViewController {
            self.showBookmarkViewController()
        }
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    // MARK: -  UIImage Catagory
    func applyingAlpha(image:UIImage, alpha:CGFloat) -> UIImage? {
        
        UIGraphicsBeginImageContextWithOptions(image.size, false, image.scale)

        let ctx:CGContext? = UIGraphicsGetCurrentContext()
        if (ctx == nil){
            return nil
        }
        let area:CGRect = CGRect(x:0, y:0, width:image.size.width, height:image.size.height)
        image.draw(in: area, blendMode:.normal, alpha: alpha)
        
        let newImage:UIImage? = UIGraphicsGetImageFromCurrentImageContext();
        UIGraphicsEndImageContext();
        return newImage;
    }
    
    // MARK: - IRotationEventListener
    
    override func willRotate(to toInterfaceOrientation: UIInterfaceOrientation, duration: TimeInterval) {
        extensionsManager.willRotate(to: toInterfaceOrientation, duration: duration)
        if willPopBookmarkViewController {
            self.dismissBookmarkViewController()
        }
        self.searchButton.isHidden = true
        self.bookmarkButton.isHidden = true
    }
    
    override func willAnimateRotation(to toInterfaceOrientation: UIInterfaceOrientation, duration: TimeInterval) {
        extensionsManager.willAnimateRotation(to: toInterfaceOrientation, duration: duration)
    }
    
    override func didRotate(from fromInterfaceOrientation: UIInterfaceOrientation) {
        extensionsManager.didRotate(from: fromInterfaceOrientation)
        self.didRotate()
    }

}

