/**
 * Copyright (C) 2003-2025, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and is the proprietary of Foxit Software Inc.. It is not allowed to
 * distribute any parts of Foxit PDF SDK for iOS to third party or public without permission unless an agreement
 * is signed between Foxit Software Inc. and customers to explicitly grant customers permissions.
 * Review legal.txt for additional license and legal information.
 */

#import "ViewController.h"
#import <uiextensionsDynamic/uiextensionsDynamic.h>
#import <FoxitRDK/FSPDFObjC.h>

@interface ViewController () <ISearchEventListener,UIExtensionsManagerDelegate>

@property (strong, nonatomic) IBOutlet FSPDFViewCtrl *pdfViewCtrl;
@property (nonatomic, strong) UIExtensionsManager *extensionsManager;
@property (nonatomic, strong) FSPanelController *panelController;

@property (nonatomic, strong) UIButton *searchButton;
@property (nonatomic, strong) UIButton *bookmarkButton;

@property (nonatomic, strong) UIPopoverController *popoverController;
@end

@implementation ViewController {
    //    float statusBarHeight;
    //    float space;
    BOOL willPopBookmarkViewController;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    // Do any additional setup after loading the view, typically from a nib.
    
    [[UIDevice currentDevice] beginGeneratingDeviceOrientationNotifications];
    willPopBookmarkViewController = NO;
    
    [self buildTopToolBar];

    NSString *pdfPath = [[NSBundle mainBundle] pathForResource:@"Sample" ofType:@"pdf"];
    FSPDFDoc *doc = [[FSPDFDoc alloc] initWithPath:pdfPath];
    if (FSErrSuccess == [doc load:nil]) {
        [self.pdfViewCtrl openDoc:pdfPath
                         password:nil
                       completion:^(FSErrorCode error) {
                           if (error == FSErrSuccess) {
                           }
                       }];
    }
    
    self.extensionsManager = [[UIExtensionsManager alloc] initWithPDFViewControl:self.pdfViewCtrl];
    [self.extensionsManager enableTopToolbar:NO];
    [self.extensionsManager enableBottomToolbar:NO];
    [self.extensionsManager enableSmallTopToolbar:NO];
    [self.extensionsManager enableSmallBottomToolbar:NO];
    self.pdfViewCtrl.extensionsManager = self.extensionsManager;
    [self.extensionsManager registerSearchEventListener:self];
    self.extensionsManager.delegate = self;
    self.panelController = self.extensionsManager.panelController;
    self.panelController.isHidden = YES;

    NSString *documentPath = [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
    NSLog(@"%@", documentPath);
}


- (void)buildTopToolBar {
    _searchButton = [UIButton buttonWithType:UIButtonTypeCustom];
    UIImage *searchImage = [UIImage imageNamed:@"search.png" inBundle:[NSBundle bundleForClass:[UIExtensionsManager class]] compatibleWithTraitCollection:nil];
    UIImage *highLightedSearchImage = [self applyingAlpha:searchImage alpha:0.5];
    [_searchButton setImage:searchImage forState:UIControlStateNormal];
    [_searchButton setImage:highLightedSearchImage forState:UIControlStateHighlighted];
    [_searchButton addTarget:self action:@selector(showSearchBar) forControlEvents:UIControlEventTouchUpInside];
    _searchButton.contentEdgeInsets = UIEdgeInsetsMake(0, 8, 0, 8);
    [_searchButton sizeToFit];

    UIImage *bookmarkImage = [UIImage imageNamed:@"document_list_state_iphone.png" inBundle:[NSBundle bundleForClass:[UIExtensionsManager class]] compatibleWithTraitCollection:nil];
    UIImage *highLightedBookmarkImage = [self applyingAlpha:bookmarkImage alpha:0.5];
    _bookmarkButton = [UIButton buttonWithType:UIButtonTypeCustom];
    [_bookmarkButton setImage:bookmarkImage forState:UIControlStateNormal];
    [_bookmarkButton setImage:highLightedBookmarkImage forState:UIControlStateHighlighted];
    [_bookmarkButton addTarget:self action:@selector(togglePanel) forControlEvents:UIControlEventTouchUpInside];
    _bookmarkButton.contentEdgeInsets = UIEdgeInsetsMake(0, 8, 0, 8);
    [_bookmarkButton sizeToFit];

    UIBarButtonItem *spaceItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil];

    self.topToolBar.items = @[ spaceItem, [[UIBarButtonItem alloc] initWithCustomView:_searchButton], [[UIBarButtonItem alloc] initWithCustomView:_bookmarkButton] ];
}

#pragma mark - UIExtensionsManagerDelegate
-(BOOL)uiextensionsManager:(UIExtensionsManager *)uiextensionsManager openNewDocAtPath:(NSString *)path shouldCloseCurrentDoc:(BOOL)closeCurrentDoc completionCallBack:(nonnull void (^)(void))completionCallBack{
    FSPDFDoc *doc = [[FSPDFDoc alloc] initWithPath:path];
    if (FSErrSuccess == [doc load:nil]) {
        [self.pdfViewCtrl openDoc:path
                            password:nil
                          completion:^(FSErrorCode error) {
                              if (error == FSErrSuccess) {
                              }
                          }];
        return YES;
    }else{
        return NO;
    }
}

#pragma mark - Bookmark
/** @brief Show bookmarks of the current PDF document when click the outline button. */
- (void)showBookmarkViewController {
    self.panelController.isHidden = NO;
}
- (void)togglePanel {
    self.panelController.isHidden = !self.panelController.isHidden;
    [_extensionsManager showSearchBar:NO];
    [self.pdfViewCtrl endEditing:YES];
}

/** @brief Trigged when the bookmark select has been selected. */
- (void)onSelectBookmark:(FSBookmark *)bookmark {
    FSDestination *dest = [bookmark getDestination];
    FSPointF *point = [[FSPointF alloc] init];
    [point set:[dest getLeft] y:[dest getTop]];
    [self.pdfViewCtrl gotoPage:[dest getPageIndex:self.pdfViewCtrl.currentDoc] withDocPoint:point animated:NO];
    [self dismissBookmarkViewController];

}

- (void)dismissBookmarkViewController {
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
        [self dismissViewControllerAnimated:YES completion:nil];
    } else {
        if ([_popoverController isPopoverVisible]) {
            [_popoverController dismissPopoverAnimated:YES];
        }
        _popoverController = nil;
    }
}

#pragma mark - Search
/** @brief Show search bar when click the search button. */
- (void)showSearchBar {
    if (_extensionsManager.currentAnnot) {
        [_extensionsManager setCurrentAnnot:nil];
    }

    if (!self.panelController.isHidden) {
        self.panelController.isHidden = YES;
    }
    [_extensionsManager changeState:FSUIManagerStateSearch];
}

#pragma mark ISearchEventListener

- (void)onSearchStarted {
}

- (void)onSearchCanceled {
}

#pragma mark -

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

#pragma mark - IRotationEventListener
- (void)willRotateToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration {
    [_extensionsManager willRotateToInterfaceOrientation:toInterfaceOrientation duration:duration];
    willPopBookmarkViewController = [_popoverController isPopoverVisible];
    if (willPopBookmarkViewController) {
        [self dismissBookmarkViewController];
    }
}

- (void)willAnimateRotationToInterfaceOrientation:(UIInterfaceOrientation)toInterfaceOrientation duration:(NSTimeInterval)duration {
    [_extensionsManager willAnimateRotationToInterfaceOrientation:toInterfaceOrientation duration:duration];
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation {
    [_extensionsManager didRotateFromInterfaceOrientation:fromInterfaceOrientation];
    if (willPopBookmarkViewController) {
        [self showBookmarkViewController];
    }
}

#pragma mark - UIImage Catagory
- (UIImage *)applyingAlpha:(UIImage*)image alpha:(CGFloat)alpha {
    if (image.size.width <= 0 || image.size.height <= 0) return nil;
    UIGraphicsBeginImageContextWithOptions(image.size, NO, image.scale);
    CGContextRef ctx = UIGraphicsGetCurrentContext();
    if (!ctx)
        return nil;
    CGRect area = CGRectMake(0, 0, image.size.width, image.size.height);
    [image drawInRect:area blendMode:kCGBlendModeNormal alpha:alpha];
    
    UIImage *newImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return newImage;
}

@end
